/****************************************************************************
NAME:  axx_align_attitude

PURPOSE:
To align all attitude inputs with each other and with ephem time.
Output a table of sensor attitude data as a function of time
from start of DCE.

DEVELOPMENT HISTORY:
$Log: axx_align_attitude.c,v $
Revision 1.5  2001/12/13 19:17:54  choate
Added checks to limit length of EPA and IRU data

Revision 1.4  2001/05/31 19:46:53  choate
Changed calls to exx_init_iru and exx_sc_attitude

Revision 1.3  2000/12/07 14:42:45  storey
Reduced the number of IRU and quaternion points by 1 to get rid of unwanted
time jumps.

Revision 1.2  2000/11/27 16:26:36  storey
Fixed bad ALIPCD structure quaternion field names.


RETURN VALUE:
Type = int
Value    Description
-----    -----------
SUCCESS  Successful completion 
ERROR    Operation failed

NOTES:
We are using gyro rate, ephemeris, and quaternion data available in the EO1
housekeeping data after it has been identified as covering the DCE.


ALGORITHM REFERENCES:
*****************************************************************************
                        Property of the U.S. Government
                            USGS EROS Data Center
*****************************************************************************/
#include <string.h>
#include <stdio.h>
#include "xxx_ALIPCD.h"		/* defines the PCD structure */
#include "xxx_Types.h"
#include "exx_init_model.h"
#include "axx_alimodel.h"	/* model structure definition */
#include "axx_init_model.h"
#include "xxx_const.h"  	/* defines several constants used */
#include "xxx_math.h"	/* prototype definitions */
#include "axx_forward_model.h"  /* definition GYRO_SAMP_RATE */
#include "xxx_LogStatus.h"

int axx_align_attitude(
    ALIPCD *pcd,		/* I: EO1 Payload Correction Data */
    ALIMODEL *amodel,		/* IO: ALI model structure */
    CPF *cpf    		/* I: calibration param file structure */
    )

{
VECTOR	iru[NPCD];		/* the gyro data */
VECTOR 	epa[NPCD];	/* the epa (euler parms) expressed as roll, pitch and yaw */
double	first_roll,		/* the first roll in our scene */
	first_pitch,		/* the first pitch in our scene */
	first_yaw;		/* the first yaw in our scene */
double	irutime[NPCD],		/* Time codes for IRU data points */
	epatime[NPCD];		/* Time codes for quaternion data points */
double	quaternion[4];		/* Temporary quaternion vector */
double	start_time,		/* ACS telemetry start time code */
	stop_time;		/* ACS telemetry stop time code */

int     ii,jj;  		/* looping variable			     */	

int  	niru;		/* number of gyro points */
int  	start_iru;	/* the element to start pulling the gyro */
int  	stop_iru;	/* the element to stop pulling the gyro */
int	nepa;		/* number of quaternions */
int	start_epa;	/* the element to start pulling the quaternions */
int	stop_epa;	/* the element to stop pulling the quaternions */
int	status;		/* function return code */

char ErrMsg[STRLEN];            /* error message used by xxx_LogError */

/*
  Find the relevant IRU rate and quaternion data in the PCD
*/
start_time = pcd->DCE_List[pcd->active_DCE].acs_start_tc;
stop_time = pcd->DCE_List[pcd->active_DCE].acs_stop_tc;
start_iru = xxx_find_index( start_time, pcd->ACS_Group->imu_xrate );
stop_iru = xxx_find_index( stop_time, pcd->ACS_Group->imu_xrate ) - 1;
if ( stop_iru-start_iru+1 > NPCD )
{
   xxx_LogStatus( "AXXLIB", __FILE__, __LINE__,
                  "Too much IRU data found. Limiting length." );
   stop_iru = start_iru + NPCD - 1;
   stop_time = pcd->GPS_Group->acsutctime.time[stop_iru];
}
start_epa = xxx_find_index( start_time, pcd->ACS_Group->quat1 );
stop_epa = xxx_find_index( stop_time, pcd->ACS_Group->quat1 ) - 1;
if ( stop_epa-start_epa+1 > NPCD )
{
   xxx_LogStatus( "AXXLIB", __FILE__, __LINE__,
                  "Too much EPA data found. Limiting length." );
   stop_epa = start_epa + NPCD - 1;
}

/*
  Load the IRU data into the local iru vector array
*/
start_time = exx_gpstime( amodel->acs_time );
niru = stop_iru - start_iru + 1;
for ( jj=0,ii=start_iru; jj<niru; jj++,ii++ )
{
    iru[jj].x = pcd->ACS_Group->imu_xrate.value.d[ii];
    iru[jj].y = pcd->ACS_Group->imu_yrate.value.d[ii];
    iru[jj].z = pcd->ACS_Group->imu_zrate.value.d[ii];
    irutime[jj] = exx_timecodetogps( pcd->ACS_Group->imu_xrate.time[ii] ) - start_time;
}

/*
  Rotate the gyro data to the attitude control system coordinates
*/
status = exx_iru2acs(niru,iru,cpf);
if (status != SUCCESS) 
{
    xxx_LogStatus("EXXLIB", __FILE__, __LINE__, 
	"Error returned from exx_iru2acs");
    return(ERROR);
}
/*
  Remove the effects due to orbital motion 
*/
status = exx_init_iru(amodel->acs_time, amodel->ephem_time, amodel->nephem,
    &amodel->eci_satpos[0][0], &amodel->eci_satpos[1][0], &amodel->eci_satpos[2][0],
    &amodel->eci_satvel[0][0], &amodel->eci_satvel[1][0], &amodel->eci_satvel[2][0],
    niru, irutime, iru);
if (status != SUCCESS) 
{
    xxx_LogStatus("EXXLIB", __FILE__, __LINE__, 
        "Error returned from exx_init_iru");
    return(ERROR);
}
/* 
  Load the quaternion data into the local epa vector array
*/
nepa = stop_epa - start_epa + 1;
for ( jj=0,ii=start_epa; jj<nepa; jj++,ii++ )
{
    quaternion[0] = pcd->ACS_Group->quat1.value.d[ii];
    quaternion[1] = pcd->ACS_Group->quat2.value.d[ii];
    quaternion[2] = pcd->ACS_Group->quat3.value.d[ii];
    quaternion[3] = pcd->ACS_Group->quat4.value.d[ii];
    epatime[jj] = exx_timecodetogps( pcd->ACS_Group->quat1.time[ii] ) - start_time;
    status = exx_sc_attitude( amodel->acs_time, amodel->ephem_time, amodel->nephem,
                  &amodel->eci_satpos[0][0], &amodel->eci_satpos[1][0], &amodel->eci_satpos[2][0],
                  &amodel->eci_satvel[0][0], &amodel->eci_satvel[1][0], &amodel->eci_satvel[2][0],
                  epatime[jj], quaternion, &epa[jj].x, &epa[jj].y, &epa[jj].z);
    if (status != SUCCESS) 
    {
        xxx_LogStatus("EXXLIB", __FILE__, __LINE__,
	    "Error returned from exx_sc_attitude");
        return(ERROR);
    }/* end if (status ... */
}/* end for (ii ... */

/************************************************************************
          KALMAN FILTER
  Combine IRU and epa (attitude) data
  Return integrated attitude at one second
  intervals from amodel->acs_time in the
  iru vector array. Update number of samples
  in the niru variable.
***************************************************************************/
status = exx_kalman_smooth_iru(&nepa, epatime, epa, 
	                       &niru, irutime, iru);
if (status != SUCCESS)
{
    xxx_LogStatus("EXXLIB", __FILE__, __LINE__, 
	"Error returned from exx_kalman_smooth_iru");
    return(ERROR);
}

/*********************************** 
  pull the gyro from the smoothed data
***********************************/ 

/* set aside the first gyro point for our scene */
first_roll = iru[0].x;
first_pitch = iru[0].y;
first_yaw = iru[0].z;

/* 
  Retrieve the initial space craft r,p,y.
  Load the first merged attitude value into the alimodel structure
  as the spacecraft roll, pitch, and yaw.
*/
amodel->sc_roll = first_roll;
amodel->sc_pitch = first_pitch;
amodel->sc_yaw = first_yaw;
amodel->nrpy = niru;
for (ii=0; ii<niru; ii++)
{
    amodel->rpy[ii][0] = iru[ii].x - first_roll;
    amodel->rpy[ii][1] = iru[ii].y - first_pitch;
    amodel->rpy[ii][2] = iru[ii].z - first_yaw;
}/* end for ii */

return(SUCCESS);
}
