/******************************************************************************
NAME:           axx_calc_position

PURPOSE:
Finds the satellite position, velocity, and time using the forward model.

DEVELOPMENT HISTORY:
$Log: axx_calc_position.c,v $
Revision 1.4  2001/05/31 19:45:26  choate
Changed call to exx_movesat

Revision 1.3  2001/01/03 17:05:43  storey
Changed output to be full time as Year, DOY, SOD instead of Year, DOY,
and seconds from DCE start.

Revision 1.2  2000/12/07 22:58:04  storey
Fixed call to gxx_movesat.

Revision 1.1  2000/11/27 13:29:20  storey
Original version


RETURN VALUE:
Type = int
Value    	Description
-----    	-----------
ERROR		Error finding attitude correction
SUCCESS		Successfully found position

NOTES:

ALGORITHM REFERENCES:
******************************************************************************
                        Property of the U.S. Government
                            USGS EROS Data Center
******************************************************************************/
#include "axx_alimodel.h"
#include "xxx_LogStatus.h"

int axx_calc_position(
	double line,		/*I: Line of interest   (target)	*/
	double sample,		/*I: Sample of interest (target)	*/
	BandNumber band,	/*I: Band the line/sample are in.	*/
	DetType dettype,	/*I: Detector type			*/
	const CPF *cpf,		/*I: Cal parm file constants		*/
	ALIMODEL *amodel,	/*I: ALI model structure information	*/
	VECTOR *satpos,		/*O: satellite position (meters) in Earth
				     Centered Inertial (ECI) coordinates*/
	VECTOR *satvel,		/*O: sat velocity (meters/millisec) in ECI*/
	double *Time,		/*O: time of the target image point	*/
	int  *year,		/*O: year of image point		*/
	int  *day)		/*O: day of image point			*/
{
double	delta_time,		/* time from ephemeris/ads to first scan*/
	ref_delta,		/* reference delta time from start of 
				   image to the ephemeris/ads time of 
				   interest				*/
	img_time[3];		/* Image point year, doy, sod		*/

/*
 * Initalize the time.  Time will be referenced from first line/sample in
 * the image to the target line/sample.
 */
*Time = 0.0;

/*
 * Find the time from start of image
 */
axx_findtime(line, sample, band, amodel, cpf, dettype, Time);

/*
 * Calculate the delta time (seconds) from the start of the ephemeris
 * information to the scan line of interest
 */
delta_time = exx_gpstime( amodel->dce_time ) - exx_gpstime( amodel->ephem_time );
ref_delta  = *Time + delta_time;

/*
 * Move the satellite to the ephemeris time
 */
exx_movesat(amodel->nephem, &amodel->ecr_satpos[0][0], &amodel->ecr_satpos[1][0], 
            &amodel->ecr_satpos[2][0], &amodel->ecr_satvel[0][0], &amodel->ecr_satvel[1][0],
            &amodel->ecr_satvel[2][0],  ref_delta, satpos, satvel);

/*
 * Load the time values
 */
ref_delta = *Time + exx_gpstime( amodel->dce_time );
exx_gpstoyds( ref_delta, img_time );
*year = (int)(img_time[0]+0.5);
*day = (int)(img_time[1]+0.5);
*Time = img_time[2];

return(SUCCESS);
}
