/******************************************************************************
NAME:           axx_findlos

PURPOSE:
Find the line of sight vector in sensor coordinates

DEVELOPMENT HISTORY:
$Log: axx_findlos.c,v $
Revision 1.4  2001/03/28 22:43:24  storey
Added check for L0R software version to see if L1R data is in detector or
ground pixel order and to set up the LOS model accordingly.

Revision 1.3  2001/01/08 13:12:58  storey
Fixed scaling error in normalized detector coordinate computation. Was being
divided by number of detectors (pixel edges) instead of number of detectors
minus 1 (pixel centers).

Revision 1.2  2000/12/04 20:35:05  storey
Make SCA logic zero-based to conform to detector logic.

Revision 1.1  2000/11/27 13:29:23  storey
Original version



RETURN VALUE:
    ERROR
    SUCCESS

NOTES:

ALGORITHM REFERENCES:
******************************************************************************
                        Property of the U.S. Government
                            USGS EROS Data Center
******************************************************************************/
#include "xxx_Types.h"
#include "axx_alimodel.h"

int axx_findlos(
    BandNumber band,	/*I: Input point band number		*/
    int sca,		/*I: Input point SCA number (0-based)	*/
    double detector,	/*I: Input point detector number	*/
    DetType dettype,	/*I: Type of detector model to use	*/
    ALIMODEL *amodel,	/*I: ALI model structure 		*/
    const CPF *cpf,	/*I: CPF data structure			*/
    VECTOR *losv) 	/*O: Line of sight vector               */
{
double norm_det;	/* Legendre normalized detector number  */
double x,		/* Look vector X coordinate		*/
       y,		/* Look vector Y coordinate		*/
       maglos;		/* LOS vector magnitude			*/
double ifov,		/* ALI IFOV (band dependent)		*/
       x_off,		/* Along-track detector offset		*/
       y_off;		/* Across-track detector offset		*/
VECTOR losvec;		/* Local LOS vector			*/
int ndet,		/* Nearest integer detector		*/
    even_odd;		/* Even/odd detector flag		*/

/* Compute the normalized detector number */
/* See if the L0R was created with software version 1.1.2 or higher  */
if ( (amodel->l0r_sw_version[0] > 1) ||
     (amodel->l0r_sw_version[0] == 1 && amodel->l0r_sw_version[1] > 1) ||
     (amodel->l0r_sw_version[0] == 1 && amodel->l0r_sw_version[1] == 1 && amodel->l0r_sw_version[2] > 1) )
  detector = amodel->sca_num_det[band] - 1 - detector;
norm_det = 2.0 * detector / ((double)amodel->sca_num_det[band]-1) - 1.0;

/* Evaluate the Legendre polynomials from the ALI model */
x = amodel->sca_coef_x[sca][band][0]
  + amodel->sca_coef_x[sca][band][1] * norm_det
  + amodel->sca_coef_x[sca][band][2] * (1.5*norm_det*norm_det - 0.5);
y = amodel->sca_coef_y[sca][band][0]
  + amodel->sca_coef_y[sca][band][1] * norm_det
  + amodel->sca_coef_y[sca][band][2] * (1.5*norm_det*norm_det - 0.5);

/* Do extra processing for ACTUAL and EXACT detectors */
if ( dettype != NOMINAL )
{
    ndet = (int)floor(detector+0.5);
    even_odd = ndet % 2;
    /* Make sure we're inside the SCA for CPF lookup purposes */
    if ( ndet < 0 )
        ndet = even_odd;
    if ( ndet > amodel->sca_num_det[band]-1 )
        ndet = amodel->sca_num_det[band]-2+even_odd;
    
    /* Get stuff from the CPF */
    if ( band == pan )
    {
        ifov = cpf->ALI_Parameters.IFOV_PAN_along;
        x_off = cpf->Focal_Plane_Parameters.SCA_Parameters[sca].Along_Detector_Offset_PAN[ndet];
        y_off = cpf->Focal_Plane_Parameters.SCA_Parameters[sca].Across_Detector_Offset_PAN[ndet];
    }
    else
    {
        ifov = cpf->ALI_Parameters.IFOV_MS_along;
        x_off = cpf->Focal_Plane_Parameters.SCA_Parameters[sca].Along_Detector_Offset_MS[band-1][ndet];
        y_off = cpf->Focal_Plane_Parameters.SCA_Parameters[sca].Across_Detector_Offset_MS[band-1][ndet];
    }
    
    /* Apply the even/odd offsets */
    if ( even_odd == 0 )	/* Even detector */
    {
        x += ifov*cpf->Focal_Plane_Parameters.Odd_Even_Offsets.Even_Detector_Shift[band];
    }
    else			/* Odd detector */
    {
        x += ifov*cpf->Focal_Plane_Parameters.Odd_Even_Offsets.Odd_Detector_Shift[band];
    }

    /* If the EXACT option is selected, apply the detector-specific offsets */
    if ( dettype == EXACT )
    {
        x -= ifov*x_off;
        y -= ifov*y_off;
    }
}

/* Construct the NOMINAL LOS vector */
losvec.x = x;
losvec.y = y;
losvec.z = 1.0;


/* Normalize for output */
maglos = xxx_norm( &losvec );
losv->x = losvec.x/maglos;
losv->y = losvec.y/maglos;
losv->z = losvec.z/maglos;

return(SUCCESS);
}
