/******************************************************************************
NAME:           axx_findtime

PURPOSE:
Find the time into the scene given the line, sample, and band

DEVELOPMENT HISTORY:
$Log: axx_findtime.c,v $
Revision 1.2  2000/12/04 20:24:42  storey
Changed logic to DIVIDE by frame_rate instead of MULTIPLY.

Revision 1.1  2000/11/27 13:29:25  storey
Original version



RETURN VALUE:
   ERROR
   SUCCESS

NOTES:
   The input sample number is 0-relative and relative to the SCA

   The accounting for the odd/even detector offsets is based on the value
   of the dettype variable

ALGORITHM REFERENCES:
******************************************************************************
                        Property of the U.S. Government
                           USGS EROS Data Center
******************************************************************************/
#include "xxx_LogStatus.h"
#include "xxx_const.h"
#include "axx_alimodel.h"

int axx_findtime (
	double line,		/*I: Line from start of the image (0-rel)*/
	double sample,		/*I: Sample from start of the line (0-rel)*/
	BandNumber band,	/*I: Band in the image			*/
	ALIMODEL *amodel,	/*I: ALI model structure		*/
	const CPF *cpf,		/*I: CPF constants			*/
	DetType dettype,	/*I: Detector option NOMINAL or ACTUAL  */
	double *Time)		/*O: Time from start of image (seconds)	*/
{
int	detector,		/* Detector number			*/
	even_odd,		/* Flag for even/odd detector 
				     0 = even, 1 = odd			*/
	nom_shift,		/* CPF line shift for nominal detector	*/
	even_odd_shift[2];	/* Even/Odd detector shifts from CPF	*/

/* Get stuff from the CPF */
nom_shift = cpf->Focal_Plane_Parameters.Odd_Even_Offsets.Nom_Detector_Shift[band];
even_odd_shift[0] = cpf->Focal_Plane_Parameters.Odd_Even_Offsets.Even_Detector_Shift[band];
even_odd_shift[1] = cpf->Focal_Plane_Parameters.Odd_Even_Offsets.Odd_Detector_Shift[band];

/* Compute the detector number */
detector = (int)floor(sample+0.5);
if ( detector < 0 )
     detector = 0;
if ( detector > amodel->sca_num_det[band]-1 )
     detector = amodel->sca_num_det[band] - 1;
if ( (amodel->l0r_sw_version[0] > 1) ||
     (amodel->l0r_sw_version[0] == 1 && amodel->l0r_sw_version[1] > 1) ||
     (amodel->l0r_sw_version[0] == 1 && amodel->l0r_sw_version[1] == 1 && amodel->l0r_sw_version[2] > 1) )
  detector = amodel->sca_num_det[band] - 1 - detector;
even_odd = detector % 2;

/* if were are using the pan band */
if (band == pan)
{
	/* compute the time from the line number and pan frame rate */
	*Time = (line-nom_shift) / amodel->pan_frame_rate;
	if ( dettype != NOMINAL )
	    *Time -= even_odd_shift[even_odd] / amodel->pan_frame_rate;
}
else
{
	/* compute the time from the line number and ms frame rate */
	*Time = (line-nom_shift) / amodel->ms_frame_rate;
	if ( dettype != NOMINAL )
	    *Time -= even_odd_shift[even_odd] / amodel->ms_frame_rate;
}

return(SUCCESS);
}
