/******************************************************************************
NAME:           axx_forward_etm

PURPOSE:
For a given line, sample and band, propagate the forward ETM+ satellite model
to determine a latitude and longitude for the specified point.

DEVELOPMENT HISTORY:
$Log: axx_forward_ali.c,v $
Revision 1.1  2000/11/27 13:29:26  storey
Original version



RETURN VALUE:
Type = int
Value    	Description
-----    	-----------
ERROR		error propagating image location 
SUCCESS		image location successfully propagated forward

NOTES:
The axx_getlos function does the majority of the forward model processing.

ALGORITHM REFERENCES:
******************************************************************************
                        Property of the U.S. Government
                            USGS EROS Data Center
******************************************************************************/
#include "xxx_LogStatus.h"
#include "axx_alimodel.h"
#include "exx_geometric.h"

int axx_forward_ali(
	double line,		/*I: Input line	number			     */
	double sample,		/*I: Input sample number		     */
	BandNumber band,	/*I: Input band number			     */
	int sca,		/*I: ALI SCA number			     */
	double tarelev,		/*I: Target elevation (at line/sample)	     */
	const CPF *cpf,		/*I: Cal parm file constants                 */
	ALIMODEL *amodel,	/*I/O: ALI model information		     */
	DetType dettype,	/*I: Detector type to project
				     NOMINAL, ACTUAL, or EXACT		     */
	double *tarlatd,	/*O: Target latitude			     */
	double *tarlong)	/*O: Target longitude			     */
{
double	tarlatc,		/* geocentric latitude of the target	     */
        tarrc,			/* radius of the target			     */
        ht;			/* height of the target in geodetic coords   */
VECTOR satpos,                  /* satellite position vector                 */
       los;                     /* line of sight in OCI coordinates          */

/*
 * Call getlos which does the main calculations for forward_ali.
 */
if  (axx_getlos(line, sample, band, sca, tarelev, cpf, amodel, dettype, 
		&tarlatc, tarlong, &tarrc, &satpos, &los) != SUCCESS)
{ 
    xxx_LogStatus("AXXLIB", __FILE__, __LINE__,
	"Error calculating line of sight intersection with the Earth");
    return(ERROR);
}

/*
 * Calculate the geodetic latitude and height of the target
 */
if (exx_centh2det(tarlatc, tarrc, cpf, tarlatd, &ht) != SUCCESS)
{ 
    xxx_LogStatus("AXXLIB", __FILE__, __LINE__,
	"Error calculating geodetic latitude and height");
    return(ERROR);
}

return(SUCCESS);
}
