/******************************************************************************
NAME:           axx_getlos

PURPOSE:
Find the position were the line of sight vector intersects the Earth's surface

DEVELOPMENT HISTORY:
$Log: axx_getlos.c,v $
Revision 1.3  2001/05/31 19:48:16  choate
Changed call to exx_movesat

Revision 1.2  2000/11/28 22:32:24  storey
Changed incorrect call to exx_findatt to axx_findatt.

Revision 1.1  2000/11/27 13:29:27  storey
Original version


RETURN VALUE:
Type = int
Value    	Description
-----    	-----------
ERROR		Error finding attitude correction
SUCCESS		Successfully found position

NOTES:
     For single point calculations (e.g., calculate a lat/long for a corner 
     point) the delay flag should be set to TRUE, and the terrain can be 
     done if the elevation of the input point is known.

     The band that the line and sample of interest is in must be known so
     that band offset, detector delay, and lines per scan can be determined
     when calculating the time that the point was acquired.

ALGORITHM REFERENCES:
******************************************************************************
                        Property of the U.S. Government
                            USGS EROS Data Center
******************************************************************************/
#include "axx_alimodel.h"
#include "xxx_LogStatus.h"

int axx_getlos(
	double line,		/*I: Line of interest   (target)	*/
	double sample,		/*I: Sample of interest (target)	*/
	BandNumber band,	/*I: Band the line/sample are in.	*/
	int sca,		/*I: ALI SCA number			*/
	double tarelev,		/*I: Elevation of target above the 
				     ellipsoid				*/
	const CPF *cpf,		/*I: Cal parm file constants		*/
	ALIMODEL *amodel,	/*I: ALI model structure information	*/
	DetType dettype,	/*I: Detector type:
				     NOMINAL-nominal detector array
				     ACTUAL-with even/odd offsets
                                     EXACT-with detector-specific delay	*/
	double *tarlatc,	/*O: Target geocentric latitude (radians)*/
	double *tarlong,	/*O: Target longitude (radians (radians)*/
	double *tarrc,		/*O: Radius of the earth at the target	*/
        VECTOR *satpos,         /*O: satellite position (meters) in Earth
                                     Centered Inertial (ECI) coordinates */
        VECTOR *pertlos)        /*O: perturbed LOS (OCS-orbital coord)  */
{
int 	status;			/* status of return from function	*/

double 	Time,			/* time of the target from image start	*/
	delta_time,		/* time from ephem_time to image start	*/
	ref_delta,		/* reference delta time from start of 
				   ephemeris data to the image time	*/
	roll,			/* roll correction at time of interest	*/
	pitch,			/* pitch correction at time of interest	*/
	yaw;			/* yaw correction at time of interest	*/

VECTOR  losvec,			/* line of sight to target vector	*/
        newlos,                 /* new line of sight (LOS) vector in
                                   perturbed spacecraft coordinates */
	satvel;			/* sat velocity (meters/millisec) in ECI*/

/*
 * Initalize the time.  Time will be referenced from first line/sample in
 * the image to the target line/sample.
 */
Time = 0.0;

/*
 * Find the current detector and time from start of image
 */
status = axx_findtime(line, sample, band, amodel, cpf, dettype, &Time);
if (status != SUCCESS)
{
    xxx_LogStatus("AXXLIB", __FILE__, __LINE__, "Error returned from axx_findtime");
    return(ERROR);
}

/*
 * Find the ALI look vector
 */
status = axx_findlos(band, sca, sample, dettype, amodel, cpf, &losvec);
if (status != SUCCESS)
{
    xxx_LogStatus("AXXLIB", __FILE__, __LINE__, 
        "Error returned from axx_findlos");
    return(ERROR);
}

/*
 * Find the roll, pitch, and yaw correction for the above time
 */
status = axx_findatt(amodel, Time, &roll, &pitch, &yaw);
if (status != SUCCESS)
{
    xxx_LogStatus("AXXLIB",__FILE__, __LINE__, 
        "Error returned from axx_findatt");
    return(ERROR);
}

/*
 * Calculate the delta time (seconds) from the start of the ephemeris
 * information to the scan line of interest
 */
delta_time = exx_gpstime( amodel->dce_time ) - exx_gpstime( amodel->ephem_time );
ref_delta  = Time + delta_time;

/*
 * Move the satellite to the ephemeris time
 */
exx_movesat(amodel->nephem, &amodel->ecr_satpos[0][0], &amodel->ecr_satpos[1][0], 
            &amodel->ecr_satpos[2][0], &amodel->ecr_satvel[0][0], &amodel->ecr_satvel[1][0], 
            &amodel->ecr_satvel[2][0],  ref_delta, satpos, &satvel);

/*
 * Find the new line of sight vector from the perturbed spacecraft coordinates
 */
status = axx_attitude(satpos, &satvel, &losvec, roll, pitch, yaw, cpf,
             pertlos, &newlos);
if (status != SUCCESS)
{
    xxx_LogStatus("AXXLIB",__FILE__, __LINE__, 
        "Error returned from axx_attitude");
    return(ERROR);
}

/*
 * Find the position were the line of sight vector intersects the Earth's
 * surface
 */
status = axx_findtarpos(satpos, &newlos, tarelev, cpf, tarlatc, tarlong,tarrc);
if (status != SUCCESS)
{
    xxx_LogStatus("AXXLIB",__FILE__, __LINE__, 
        "Error returned from axx_findtarpos");
    return(ERROR);
}

return(SUCCESS);
}
