#include <stdio.h>
#include <math.h>
#include <unistd.h>
#include <sys/types.h>
#include <stddef.h>
#include "xxx_Types.h"        /* SUCCESS/ERROR definitions */
#include "xxx_LogStatus.h"    /* xxx_LogStatus prototype */
#include "xxx_const.h"
#include "axx_grid.h"
#include "xxx_eval_poly.h"

/******************************************************************************
NAME:           axx_ols2ils

PURPOSE:        
The axx_ols2ils routine maps an output space line/sample back into its 
corresponding input space line/sample.

RETURN VALUE:
The input space line, sample, grid row, and grid column.

NOTES:
none

******************************************************************************/

int axx_ols2ils
( 
    const axx_GRID_BAND_TYPE *grid_band_ptr, /* I: pointer to grid band */
    int  *row_ptr,     /* O: pointer to input space grid row */
    int  *col_ptr,     /* O: pointer to input space grid column */
    long input_nl,
    long input_ns,
    double oline,      /* I: output space line to map to input space */
    double osamp,      /* I: output space sample to map to input space */
    double *iline_ptr, /* O: pointer to input space line */
    double *isamp_ptr, /* O: pointer to input space sample */
    int  iplane        /* I: elevation plane */
)
{
    int  new_row;    /* new cell row found to contain the output pixel */
    int  new_col;    /* new cell column found to contain the output pixel*/
    int  cell_index; /* cell index of new row/column */
    int  ncols = grid_band_ptr->ngrid_samps - 1; /* columns in grid */
    int  nrows = grid_band_ptr->ngrid_lines - 1; /* columns in grid */
    int  offset;
    double new_line; /* new input line */
    double new_samp; /* new input sample */
    double *poly_samps_coeff;
    double *poly_lines_coeff;
    struct COEFFICIENTS *coeff_ptr; /* pointer to projtosat coefficients for
                                       the current grid cell */
    double lms;      /* line multiplied by sample cached value */
    char error_msg[STRLEN]; 

    /* Make first guess of input pixel for an output pixel using 
       the "rough polynomial". */
    offset = iplane * ( grid_band_ptr->degree +1 ) * ( grid_band_ptr->degree + 1 );
    poly_samps_coeff = &grid_band_ptr->poly_samps[offset];
    poly_lines_coeff = &grid_band_ptr->poly_lines[offset];
    new_samp = xxx_eval_poly(grid_band_ptr->degree, poly_samps_coeff, osamp, oline);
    new_line = xxx_eval_poly(grid_band_ptr->degree, poly_lines_coeff, osamp, oline);

    new_row = (int)(new_line / grid_band_ptr->grid_cell_size.line);
    new_col = (int)(new_samp / grid_band_ptr->grid_cell_size.samp);

    /* make sure the new_row, new_col, new_samp, new_line are legal. I step in one pixel around
       the borders just to make sure with the "rough polynomial" and to avoid and inclusive,
       exclusive situations (i.e. pixels are really mapped by center of pixel leaving one
       pixel "not mapped") */
    if ((new_samp >= 1) && (new_samp < (input_ns-2)) && (new_line >= 1) && (new_line < (input_nl-2)) )
    {
       if ((new_row >= 0) && (new_row < nrows) && (new_col >= 0) && (new_col < ncols))
       {
           /* After rough guess call routine to find the correct grid 
              cell associated with the output pixel */
           if (axx_findgridcell(grid_band_ptr, &new_row, &new_col, oline, osamp, iplane) ==ERROR )
           {
               char error_msg[STRLEN]; /* this is declared here to prevent the
                                          possiblity that it may take a significant
                                          amount of time to allocate the space */
   
               printf("Error locating grid cell at output line,sample (%f,%f)\n", oline, osamp);
               sprintf(error_msg,"Error locating grid cell at output line,sample\n");
                        
               /* log an error message since axx_findgridcell failed */
               xxx_LogStatus("AXXLIB",__FILE__,__LINE__, error_msg);
   
               /* error, so zero out data */
               *row_ptr = 0;
               *col_ptr = 0;
               *isamp_ptr = 0.0;
               *iline_ptr = 0.0;
   
               return (ERROR);
           }
           else
           {
               /* Find input pixel location for output pixel location 
                  using the inverse mapping coefficients of the proper 
                  grid cell */
               *row_ptr = new_row;
               *col_ptr = new_col;
   
               lms = oline * osamp;
   
               cell_index = nrows * ncols * iplane + new_row * ncols + new_col;
   
               coeff_ptr = &grid_band_ptr->projtosat[cell_index];
               *isamp_ptr = coeff_ptr->a[0] + coeff_ptr->a[1] * osamp + 
                            coeff_ptr->a[2] * oline + coeff_ptr->a[3] * lms;
               *iline_ptr = coeff_ptr->b[0] + coeff_ptr->b[1] * osamp + 
                            coeff_ptr->b[2] * oline + coeff_ptr->b[3] * lms;
   
               return (SUCCESS);
           }
       }
    }

    /* error, so zero out data */
    *row_ptr = 0;
    *col_ptr = 0;
    *isamp_ptr = 0.0;
    *iline_ptr = 0.0;

    /* do not generate a message since this error is normal when in the 
       fill area of the output image */
    return (ERROR);
}

/******************************************************************************
NAME:           axx_smart_ols2ils

RETURN VALUE:
The input space line, sample, grid row, and grid column.

NOTES:
This routine differs from the normal axx_ols2ils routine because it uses a 
little intelligence to find the input space line/sample.  Instead of doing a 
call to axx_findgridcell every time it is called, it assumes that the current 
row/column passed in are a good guess.  It computes the input line/sample and 
checks the result to see if it is in the current cell.  If it is, the routine
returns.  If it is not, the normal method is used.

******************************************************************************/

int axx_smart_ols2ils 
( 
    const axx_GRID_BAND_TYPE *grid_band_ptr, /* I: pointer to grid band */
    int  *row_ptr,     /* I/O: pointer to input space grid row */
    int  *col_ptr,     /* I/O: pointer to input space grid column */
    double oline,      /* I: output space line to map to input space */
    double osamp,      /* I: output space sample to map to input space */
    double *iline_ptr, /* O: pointer to input space line */
    double *isamp_ptr, /* O: pointer to input space sample */
    int  iplane        /* I: zplane */
)
{
    int  new_row;    /* new cell row found to contain the output pixel */
    int  new_col;    /* new cell column found to contain the output pixel*/
    int  cell_index; /* cell index of new row/column */
    int  ncols = grid_band_ptr->ngrid_samps - 1; /* columns in grid */
    int  nrows = grid_band_ptr->ngrid_lines - 1; /* rows in grid */
    double new_line; /* new input line */
    double new_samp; /* new input sample */
    double *poly_samps_coeff;
    double *poly_lines_coeff;
    struct COEFFICIENTS *coeff_ptr; /* pointer to projtosat coefficients for
                                       the current grid cell */
    double lms;      /* line multiplied by sample cached value */
    int offset;

    /* make the assumption that the last row/column (passed in) is 
       most likely the same as the current one.  If the output to
       input equations find the input point within the bounds of the
       last cell, call it good */
    lms = oline * osamp;

    cell_index = nrows * ncols * iplane + *row_ptr * ncols + *col_ptr;

    coeff_ptr = &grid_band_ptr->projtosat[cell_index];
    *isamp_ptr = coeff_ptr->a[0] + coeff_ptr->a[1] * osamp + 
                 coeff_ptr->a[2] * oline + coeff_ptr->a[3] * lms;
    *iline_ptr = coeff_ptr->b[0] + coeff_ptr->b[1] * osamp + 
                 coeff_ptr->b[2] * oline + coeff_ptr->b[3] * lms;

    if ((*isamp_ptr >= grid_band_ptr->in_samps[*col_ptr]) && 
        (*isamp_ptr < grid_band_ptr->in_samps[*col_ptr+1]) &&
        (*iline_ptr >= grid_band_ptr->in_lines[*row_ptr*2]) &&
        (*iline_ptr < grid_band_ptr->in_lines[*row_ptr*2+1]))
    {
        return (SUCCESS);
    }

    /* Make first guess of input pixel for an output pixel using 
       the "rough polynomial". */
    offset = iplane * ( grid_band_ptr->degree +1 ) * ( grid_band_ptr->degree + 1 );
    poly_samps_coeff = &grid_band_ptr->poly_samps[offset];
    poly_lines_coeff = &grid_band_ptr->poly_lines[offset];
    new_samp = xxx_eval_poly(grid_band_ptr->degree, poly_samps_coeff, osamp, oline);
    new_line = xxx_eval_poly(grid_band_ptr->degree, poly_lines_coeff, osamp, oline);

    new_row = (int)(new_line / grid_band_ptr->grid_cell_size.line);
    new_col = (int)(new_samp / grid_band_ptr->grid_cell_size.samp);

    /* make sure the new_row and new_col are legal */
    if ((new_row >= 0) && (new_row < grid_band_ptr->ngrid_lines/2) &&
        (new_col >= 0) && (new_col < ncols))
    {
        /* After rough guess call routine to find the correct grid 
           cell associated with the output pixel */
        if (axx_findgridcell(grid_band_ptr, &new_row, &new_col, oline, 
                             osamp, iplane) ==ERROR )
        {
            char error_msg[STRLEN]; /* this is declared here to prevent the
                                       possiblity that it may take a significant
                                       amount of time to allocate the space */

            sprintf(error_msg,
                    "Error locating grid cell at output line,sample (%lf,%lf)",
                    oline, osamp);
                     
            /* log an error message since axx_findgridcell failed */
            xxx_LogStatus("AXXLIB",__FILE__,__LINE__, error_msg);

            /* error, so zero out data */
            *row_ptr = 0;
            *col_ptr = 0;
            *isamp_ptr = 0.0;
            *iline_ptr = 0.0;

            return (ERROR);
        }
        else
        {
            /* Find input pixel location for output pixel location 
               using the inverse mapping coefficients of the proper 
               grid cell */
            *row_ptr = new_row;
            *col_ptr = new_col;

            /* don't need to do this again 
            lms = oline * osamp;*/

            cell_index = nrows * ncols * iplane + new_row * ncols + new_col;

            coeff_ptr = &grid_band_ptr->projtosat[cell_index];
            *isamp_ptr = coeff_ptr->a[0] + coeff_ptr->a[1] * osamp + 
                         coeff_ptr->a[2] * oline + coeff_ptr->a[3] * lms;
            *iline_ptr = coeff_ptr->b[0] + coeff_ptr->b[1] * osamp + 
                         coeff_ptr->b[2] * oline + coeff_ptr->b[3] * lms;

            return (SUCCESS);
        }
    }

    /* error, so zero out results */
    *row_ptr = 0;
    *col_ptr = 0;
    *isamp_ptr = 0.0;
    *iline_ptr = 0.0;

    /* do not generate a message since this error is normal when in the 
       fill area of the output image */
    return (ERROR);
}

/******************************************************************************
NAME:           axx_all_ols2ils

PURPOSE:        
The axx_all_ols2ils routine maps an output space line/sample back into its 
corresponding input space line/sample. It's a more forgiving version of
the axx_ols2ils routine.

RETURN VALUE:
The input space line, sample, grid row, and grid column.

NOTES:
none

******************************************************************************/
int axx_all_ols2ils
( 
    const axx_GRID_BAND_TYPE *grid_band_ptr, /* I: pointer to grid band */
    int  *row_ptr,     /* O: pointer to input space grid row */
    int  *col_ptr,     /* O: pointer to input space grid column */
    long input_nl,
    long input_ns,
    double oline,      /* I: output space line to map to input space */
    double osamp,      /* I: output space sample to map to input space */
    double *iline_ptr, /* O: pointer to input space line */
    double *isamp_ptr, /* O: pointer to input space sample */
    int  iplane        /* I: elevation plane */
)
{
    int  new_row;    /* new cell row found to contain the output pixel */
    int  new_col;    /* new cell column found to contain the output pixel*/
    int  cell_index; /* cell index of new row/column */
    int  ncols = grid_band_ptr->ngrid_samps - 1; /* columns in grid */
    int  nrows = grid_band_ptr->ngrid_lines - 1; /* columns in grid */
    int  offset;
    double new_line; /* new input line */
    double new_samp; /* new input sample */
    double *poly_samps_coeff;
    double *poly_lines_coeff;
    struct COEFFICIENTS *coeff_ptr; /* pointer to projtosat coefficients for
                                       the current grid cell */
    double lms;      /* line multiplied by sample cached value */
    char error_msg[STRLEN]; 

    /* Make first guess of input pixel for an output pixel using 
       the "rough polynomial". */
    offset = iplane * ( grid_band_ptr->degree +1 ) * ( grid_band_ptr->degree + 1 );
    poly_samps_coeff = &grid_band_ptr->poly_samps[offset];
    poly_lines_coeff = &grid_band_ptr->poly_lines[offset];
    new_samp = xxx_eval_poly(grid_band_ptr->degree, poly_samps_coeff, osamp, oline);
    new_line = xxx_eval_poly(grid_band_ptr->degree, poly_lines_coeff, osamp, oline);

    new_row = (int)(new_line / grid_band_ptr->grid_cell_size.line);
    new_col = (int)(new_samp / grid_band_ptr->grid_cell_size.samp);

    /* Here's the difference with the axx_ols2ils library routine. We want to find the closest 
       grid cell even if we're outside the grid. This can happen when trying to map even/odd
       detector offsets around the edge of the grid. */
    if ((new_row >= 0) && (new_row < nrows) && (new_col >= 0) && (new_col < ncols))
    {
        /* After rough guess call routine to find the correct grid 
           cell associated with the output pixel */
        if (axx_findgridcell(grid_band_ptr, &new_row, &new_col, oline, osamp, iplane) ==ERROR )
        {
           /* error, so zero out data */
           *row_ptr = 0;
           *col_ptr = 0;
           *isamp_ptr = 0.0;
           *iline_ptr = 0.0;
   
           return (ERROR);
        }
        else
        {
           /* Find input pixel location for output pixel location 
              using the inverse mapping coefficients of the proper 
              grid cell */
           *row_ptr = new_row;
           *col_ptr = new_col;
   
           lms = oline * osamp;
   
           cell_index = nrows * ncols * iplane + new_row * ncols + new_col;
   
           coeff_ptr = &grid_band_ptr->projtosat[cell_index];
           *isamp_ptr = coeff_ptr->a[0] + coeff_ptr->a[1] * osamp + 
                        coeff_ptr->a[2] * oline + coeff_ptr->a[3] * lms;
           *iline_ptr = coeff_ptr->b[0] + coeff_ptr->b[1] * osamp + 
                        coeff_ptr->b[2] * oline + coeff_ptr->b[3] * lms;
   
           return (SUCCESS);
        }
    }
    /* Use the edge grid cells */
    if ( new_row < 0 ) new_row = 0;
    if ( new_col < 0 ) new_col = 0;
    if ( new_row >= nrows ) new_row = nrows-1;
    if ( new_col >= ncols ) new_col = ncols-1;
    /* Find input pixel location for output pixel location 
       using the inverse mapping coefficients of the proper 
       grid cell */
    *row_ptr = new_row;
    *col_ptr = new_col;
   
    lms = oline * osamp;
   
    cell_index = nrows * ncols * iplane + new_row * ncols + new_col;
   
    coeff_ptr = &grid_band_ptr->projtosat[cell_index];
    *isamp_ptr = coeff_ptr->a[0] + coeff_ptr->a[1] * osamp + 
                 coeff_ptr->a[2] * oline + coeff_ptr->a[3] * lms;
    *iline_ptr = coeff_ptr->b[0] + coeff_ptr->b[1] * osamp + 
                 coeff_ptr->b[2] * oline + coeff_ptr->b[3] * lms;
   
    return (SUCCESS);

}

/******************************************************************************
NAME:           axx_3d_ols2ils

PURPOSE:        
The axx_3d_ols2ils routine maps an output space line/sample/elevation to the
corresponding input space line/sample. It uses the more forgiving version of
the axx_ols2ils routine, axx_all_ols2ils, on each bounding zplane.

RETURN VALUE:
The input space line and sample.

NOTES:
none

******************************************************************************/
int axx_3d_ols2ils
( 
    const axx_GRID_BAND_TYPE *grid_band_ptr, /* I: pointer to grid band */
    long input_nl,	/* I: number of lines in input image */
    long input_ns,	/* I: number of samples in input image */
    double oline,      	/* I: output space line to map to input space */
    double osamp,      	/* I: output space sample to map to input space */
    double *iline_ptr, 	/* O: pointer to input space line */
    double *isamp_ptr, 	/* O: pointer to input space sample */
    double elev        	/* I: point elevation */
)
{
    int grid_row,	/* Grid cell row number */
        grid_col;	/* Grid cell column number */
    int zplane;		/* Grid Z-plane index number */
    double iline,	/* Input line */
           isamp,	/* Input samp */
           iline0,	/* Input line in first Z-plane */
           isamp0,	/* Input samp in first Z-plane */
           iline1,	/* Input line in second Z-plane */
           isamp1;	/* Input samp in second Z-plane */
    double elev0,	/* First Z-plane elevation */
           elev1;	/* Second Z-plane elevation */
    double w0, w1;	/* Z-plane interpolation weights */
    int status0,	/* First Z-plane ols2ils return status */
        status1;        /* Second Z-plane ols2ils return status */

    /* Determine what "Z planes" to use for mapping */
    zplane = (int)floor( elev / grid_band_ptr->zspacing ) + grid_band_ptr->zeroplane;
    if( zplane+1 >= grid_band_ptr->nzplanes )
    {
        xxx_LogStatus("AXXLIB",__FILE__,__LINE__, "Error finding zplane");
        return(ERROR);
    }

    /* Find input line/sample for input line/sample for Z plane 1 */
    status0 = axx_all_ols2ils(grid_band_ptr, &grid_row, &grid_col, input_nl, input_ns,
                              oline, osamp, &iline0, &isamp0, zplane);

    /* Find input line/sample for input line/sample for Z plane 2 */
    status1 = axx_all_ols2ils(grid_band_ptr, &grid_row, &grid_col, input_nl, input_ns,
                              oline, osamp, &iline1, &isamp1, (zplane+1));

    if( status0==SUCCESS && status1==SUCCESS )
    {
        /* Find elevation that corresponds to planes 1 and 2 */
        elev0 = grid_band_ptr->zspacing * (zplane - grid_band_ptr->zeroplane);
        elev1 = elev0 + grid_band_ptr->zspacing;

        /* Find interpolation weights for point between Z planes 1 and 2 */
        w0 = ( elev1 - elev ) / ( elev1 - elev0 );
        w1 = ( elev - elev0 ) / ( elev1 - elev0 );

        /* Interpolated input line/sample from between Z planes */
        isamp = isamp0 * w0 + isamp1 * w1;
        iline = iline0 * w0 + iline1 * w1;

        /* if the input line/sample are within the input image bounds */
        if ((iline >= 1) && (iline <  (input_nl-2)) &&
            (isamp >= 1) && (isamp <  (input_ns-2)))
        {
            *iline_ptr = iline;
            *isamp_ptr = isamp;
            return (SUCCESS);
        }
        else
        {
            *iline_ptr = 0.0;
            *isamp_ptr = 0.0;
            return (ERROR);
        }
    }
    else
    {
        *iline_ptr = 0.0;
        *isamp_ptr = 0.0;
        return (ERROR);
    }
}



