/* ********************************************************************
NAME: axx_out_of_pcd

PURPOSE:
fill the model structure with the data from the ALI PCD

DEVELOPMENT HISTORY:
$Log: axx_out_of_pcd.c,v $
Revision 1.3  2001/01/03 19:32:29  storey
Added log message containing DCE duration as computed from housekeeping data.

Revision 1.2  2000/11/28 20:18:18  storey
Fixed logic error in search for closest DCE to image start time.

Revision 1.1  2000/11/27 13:29:29  storey
Original version


RETURN VALUE:
Type = int
Value    Description
-----    -----------
SUCCESS  Successful completion 
ERROR    Operation failed

NOTES:

ALGORITHM REFERENCES:
******************************************************************************
		        Property of the U.S. Government
                             USGS EROS Data Center
******************************************************************************/
#include <stdio.h>
#include <string.h>
#include "xxx_Types.h"  	/* defines the ERROR variables */
#include "xxx_ALIPCD.h"		/* defines the PCD structure */
#include "xxx_LogStatus.h"	/* prototype for xxx_LogStatus.c */
#include "xxx_const.h"  	/* defines several constants used */
#include "axx_alimodel.h"	/* model structure definition */
#include "xxx_math.h"	/* prototype definitions */


int axx_out_of_pcd(
    ALIPCD *pcd,		/* I: Payload Correction Data from the satellite */
    ALIMODEL *amodel)		/* IO: The ALI model information */

{
int 	ii, jj, kk, qq;     	/* Loop variables                            */

double  dtime;			/* time offset				     */
double  gpstime;		/* GPS reference time                        */
double  timecode;		/* Time code corresponding to gpstime        */
double  timecode_rate;		/* Spacecraft clock rate (seconds per second)*/
char    Message[STRLEN];	/* Output message text		    	     */


/* ***************************
  Setting the satellite number
  **************************** */
amodel->satnum = 1;

/********************************************************************
  Find the DCE that most closely matches the DCE time that was loaded
  into the amodel structure based on the time in the image file.
******************************************************************** */
sprintf(Message,"Initial DCE Time:  %04d %03d %lf",
        (int)amodel->dce_time[0], (int)amodel->dce_time[1], amodel->dce_time[2]);
xxx_LogStatus("AXXLIB", __FILE__, __LINE__,Message);

pcd->active_DCE = 0;
gpstime = exx_gpstime( amodel->dce_time );
dtime = fabs( gpstime - pcd->DCE_List[pcd->active_DCE].start_time );
for (ii=1; ii<pcd->num_DCE; ii++)
{
  if ( dtime > fabs( gpstime - pcd->DCE_List[ii].start_time ) )
  {
    pcd->active_DCE = ii;
    dtime = fabs( gpstime - pcd->DCE_List[pcd->active_DCE].start_time );
  }
}

sprintf(Message,"Selected DCE %d",pcd->active_DCE);
xxx_LogStatus("AXXLIB", __FILE__, __LINE__,Message);

/* Convert the actual DCE time to year, doy, sod */
exx_gpstoyds( pcd->DCE_List[pcd->active_DCE].start_time, amodel->dce_time );

sprintf(Message,"Refined DCE Time:  %04d %03d %lf",
        (int)amodel->dce_time[0], (int)amodel->dce_time[1], amodel->dce_time[2]);
xxx_LogStatus("AXXLIB", __FILE__, __LINE__,Message);

sprintf(Message,"DCE Duration:  %lf",
        pcd->DCE_List[pcd->active_DCE].stop_time-pcd->DCE_List[pcd->active_DCE].start_time);
xxx_LogStatus("AXXLIB", __FILE__, __LINE__,Message);

/*******************************************************************
  Get the frame rate and integration time parameters from the ALI
  housekeeping data. First pick a representative telemetry record
  index in the middle of the DCE (these parameters shouldn't change
  within a DCE).
********************************************************************/
jj = (pcd->DCE_List[pcd->active_DCE].ali_start_index + 
      pcd->DCE_List[pcd->active_DCE].ali_stop_index) / 2;
amodel->ms_frame_rate = pcd->ALI_Group->frame_rate.value.d[jj];

/* Test Code to alter apparently inaccurate frame rates */
if ( fabs(amodel->ms_frame_rate - 212.121180) < 0.0001 ) amodel->ms_frame_rate = 212.023285;
if ( fabs(amodel->ms_frame_rate - 182.377713) < 0.0001 ) amodel->ms_frame_rate = 182.509135;

amodel->pan_frame_rate = 3 * amodel->ms_frame_rate;
amodel->ms_int_time = pcd->ALI_Group->ms_int_time.value.d[jj];
amodel->pan_int_time = pcd->ALI_Group->pan_int_time.value.d[jj];

/* find the ACS time */
/* The ACS time reference is in the GPS telemetry. It gives a sequence
   of GPS times (time relative to the GPS epoch) that are time stamped
   with the spacecraft time codes. This provides a basis for building
   the relationship between the spacecraft clock (time codes) and
   GPS time. What we're going to do here is figure out what this
   (presumably linear) relationship is for this DCE and initialize
   the exx_init_timecode routine so that we can subsequently use the
   exx_timecodetogps and exx_gpstotimecode routines to convert back
   and forth (e.g., when mucking with the ephemeris and attitude data). */
kk = pcd->DCE_List[pcd->active_DCE].gps_start_index;
qq = pcd->DCE_List[pcd->active_DCE].gps_stop_index;
if ( qq <= kk )
  {
    qq = kk + 1;
    if ( qq >= pcd->GPS_Group->acsutctime.count )
      {
        kk--;
        qq--;
      }
  }
gpstime = pcd->GPS_Group->acsutctime.value.d[kk];
timecode = pcd->GPS_Group->acsutctime.time[kk];

/* IMPROVEMENT_OPPORTUNITY
   We'll do the rate computation from the two end points for now.
   Could also do a regression with all the points. */

timecode_rate = (pcd->GPS_Group->acsutctime.value.d[qq] - gpstime)
              / (pcd->GPS_Group->acsutctime.time[qq] - timecode);

exx_init_timecode( gpstime, timecode, timecode_rate );

/* Now we'll use this timing information to find the start time for the
   quaternion data */

timecode = pcd->DCE_List[pcd->active_DCE].acs_start_tc;
kk = xxx_find_index( timecode, pcd->ACS_Group->quat1 );
timecode = pcd->ACS_Group->quat1.time[kk];
gpstime = exx_timecodetogps( timecode );

/* Finally, we'll load the acs_time array */
exx_gpstoyds( gpstime, amodel->acs_time );

return(SUCCESS);
}
