/******************************************************************************
NAME: axx_process_ali_model

PURPOSE:
Load the line-of-sight Legendre polynomial coefficients, convert the
ephemeris data to both ECR and ECI, merge attitude quaternions and IRU
data, generate roll, pitch, and yaw correction information and calculate
the initial spacecraft roll, pitch and yaw. Fill the model structure with
this information.

DEVELOPMENT HISTORY:
Original revision of forward routines.
$Log: axx_process_ali_model.c,v $
Revision 1.2  2000/11/28 14:33:24  storey
Deleted unused time_of_start variable.

Revision 1.1  2000/11/27 13:29:30  storey
Original version


RETURN VALUE:
Type = int
Value    Description
-----    -----------
SUCCESS  Successful completion
ERROR    Operation failed

NOTES:
The SCA line-of-sight focal plane parameters are read from the
calibration parameter file and stored in the ali model structure for future
calls to the model.


ALGORITHM REFERENCES:

******************************************************************************
		        Property of the U.S. Government
                            USGS EROS Data Center
******************************************************************************/
#include <string.h>
#include "xxx_Types.h"  /* This include defines the ERROR vairaibles	     */
#include "xxx_ALIPCD.h"
#include "xxx_LogStatus.h"
#include "xxx_const.h"  /* This include defines several constants used       */
#include "xxx_math.h"
#include "axx_alimodel.h"

int axx_process_ali_model(
    struct EPHEM *ephem,/* I: Ephemeris points */
    double  xp,         /* I: True pole postion in the mean pole coords system,
                           x-axis */
    double yp,          /* I: True pole position in the mean pole coords
                           system, y-axis */
    double ut1_utc,     /* I: Earth rotation angle difference ut1 - utc */
    CPF *cpf,		/* I: calibration parameter file data	     */
    ALIPCD *pcd,	/* I: Payload Correction Data from the
			      satellite				     */
    ALIMODEL *amodel)	/* IO: The ALI model information	     */
{
int 	status;		/* Status of function return		     */

/*
  Load the SCA line-of-sight vector coefficients
*/
status = axx_los_coeffs(amodel, cpf);
if (status != SUCCESS) 
    {
    xxx_LogStatus("AXXLIB", __FILE__, __LINE__,
	"Error returned from axx_los_coeffs");
    return(ERROR);
    }

/*
  Convert the ephemeris information to Earth Centered Earth Fixed system 
  and store both ECEF and Earth Centered Inertial system coordinate
  frames in the amodel structure.
  Here ephem time is the time of the first ephemeris point
*/
status = axx_convert_ephem(ephem, xp, yp, ut1_utc, amodel);
if (status != SUCCESS) 
    {
    xxx_LogStatus("AXXLIB", __FILE__, __LINE__,
	"Error returned from axx_convert_ephem");
    return(ERROR);
    }

/*
    Align the attitude information and generate the roll, pitch, and yaw
    correction information

   IRU and quaternion data (in PCD), try to sync and
   output = r, p, and y (amodel->rpy)

   Also calculates the initial space craft roll, pitch and yaw
   (at acs_time - before img data)
*/
status = axx_align_attitude( pcd, amodel, cpf );
if (status != SUCCESS) 
    {
    xxx_LogStatus("AXXLIB", __FILE__, __LINE__,
	"Error returned from axx_align_attitude");
    return(ERROR);
    }

return(SUCCESS);
}
