/******************************************************************************
FUNCTION:	axx_run_alimodel

PURPOSE:	Get information needed to process model data and run model.
                Call axx_out_of_pcd to get timing information. This includes
                using the dce_time parameter in the ALI model (initially set
                based on the image time) to determine which PCD DCE we are in,
                updating the dce_time to match the values in the PCD structure,
                and selecting the ephemeris data source (ACS or GPS).

DEVELOPMENT HISTORY:
$Log: axx_run_alimodel.c,v $
Revision 1.3  2001/01/03 19:34:35  storey
Added #include "exx_geometric.h".

Revision 1.2  2000/11/28 12:53:38  storey
Fixed bad call to ALI PCD read routine xxx_read_ALIPCD.


RETURN VALUE:
Type = int
Value    Description
-----    -----------
ERROR    Error processing the model data
SUCCESS  Model data was updated  

NOTES:

It is assumed that memory for the model structure has been allocated in
the calling program.

There are two sources of ephemeris information in the PCD - from the ACS
records (predicted uplink data) and from the GPS records (on-board GPS data).
This routine will select one based on the setting of the amodel->ephem_type
parameter which is set in the calling routine based on a user-entered parameter.


ALGORITHM REFERENCES:
	See axx_alimodel.h for description of amodel structure.	
*******************************************************************************/
#include "xxx_LogStatus.h"
#include "xxx_ALIPCD.h"
#include "exx_geometric.h"
#include "axx_alimodel.h"

int axx_run_alimodel
(
   char *hk_dir,			/* I: Housekeeping data directory */
   char *hk_root,			/* I: Housekeeping data file root name */
   char *cpf_name,			/* I: Path of the cpf (optional) */
   ALIMODEL *amodel,		        /* I/O: ALI model structure */
   CPF **cpf				/* O: CPF structure */
)

{
   ALIPCD pcd;				/* ALI PCD data structure */
   struct EPHEM ephem[NPCD];		/* ephemeris data from the PCD */
   double xp;				/* x shift pole wander in arc sec. */ 
   double yp;				/* y shift pole wander in arc sec. */ 
   double ut1_utc;			/* UT1 - UTC time difference in seconds */
   double mjd;				/* modified Julian date */
   char	ErrMsg[STRLEN];			/* error message used by xxx_LogError */


/*----------------------------------------------------------------------
  Read information needed to process model data. First get the PCD 
  information.
----------------------------------------------------------------------*/

   if ((xxx_read_ALIPCD (&pcd, hk_dir, hk_root )) == FAIL)
   {
      sprintf(ErrMsg,"Error loading ALI PCD from housekeeping file");
      xxx_LogStatus ("AXXLIB", __FILE__, __LINE__, ErrMsg);
      return (ERROR);
   }

/*----------------------------------------------------------------------
   Get initial timing data from the ALIPCD structure.
   This includes using the amodel->dce_time values initially populated
   from the image start time to determine which DCE we're in, and
   updating the dce_time array with more accurate data.
----------------------------------------------------------------------*/
   if (axx_out_of_pcd( &pcd, amodel ) != SUCCESS) 
   {
      xxx_LogStatus("AXXLIB", __FILE__, __LINE__,
                    "Error returned from axx_out_of_pcd");
      xxx_free_ALIPCD( &pcd );
      return(ERROR);
   }

/*----------------------------------------------------------------------
   Get the CPF information.
----------------------------------------------------------------------*/

   *cpf = exx_read_cpf (cpf_name);
   if (*cpf == NULL)
   {
      xxx_LogStatus ("AXXLIB", __FILE__, __LINE__, 
                     "Error reading CPF in exx_read_cpf");
      xxx_free_ALIPCD( &pcd );
      return (ERROR);
   }

/*----------------------------------------------------------------------
   Make sure our data is in the effective time period for the CPF
----------------------------------------------------------------------*/
if (exx_check_date(amodel->dce_time, *cpf) != SUCCESS)
    {
    xxx_LogStatus ("AXXLIB", __FILE__, __LINE__, 
                   "Error returned from exx_check_date.");
    xxx_free_ALIPCD( &pcd );
    return (ERROR);
    }

/*----------------------------------------------------------------------
   Select the valid ephemeris points.
----------------------------------------------------------------------*/
   if (exx_valid_ephem (&pcd, amodel->ephem_type, *cpf, &amodel->nephem, ephem) == ERROR)
   {
      xxx_LogStatus ("AXXLIB", __FILE__, __LINE__, 
                     "Error obtaining valid ephemeris points exx_valid_ephem");
      xxx_free_ALIPCD( &pcd );
      return (ERROR);
   }

/*----------------------------------------------------------------------
   Get x and y shift pole wander and UT1-UTC time difference
----------------------------------------------------------------------*/

   if (exx_getmjdcoords (ephem->time, &mjd, &xp, &yp, &ut1_utc, *cpf) == ERROR)
   {
      xxx_LogStatus ("AXXLIB", __FILE__, __LINE__, 
                     "Error returned from exx_getmjdcoords");
      xxx_free_ALIPCD( &pcd );
      return (ERROR);
   }
  
/*----------------------------------------------------------------------
   Generate updated model information.
----------------------------------------------------------------------*/
   
   if (axx_process_ali_model (ephem, xp, yp, ut1_utc,
		*cpf, &pcd, amodel) == ERROR)
   {
      xxx_LogStatus ("AXXLIB", __FILE__, __LINE__, 
                     "Error updating the ALI model");
      xxx_free_ALIPCD( &pcd );
      return (ERROR);
   }

/* ---------------------
   free the pcd
   --------------------- */
   xxx_free_ALIPCD( &pcd );

   return(SUCCESS); 
}
