/******************************************************************************
FUNCTION:	axx_update_amodel

PURPOSE:	Uses precision data to update the ALI model data
		and write to a separate model output file.

DEVELOPMENT HISTORY:
$Log: axx_update_amodel.c,v $
Revision 1.4  2001/05/21 17:49:00  storey
Added logic to compute the reference spacecraft roll, pitch, and yaw at the
precision update reference time. This is used in the enhancement to the ALI
alignment calibration logic.

Revision 1.3  2001/01/03 17:08:07  storey
Changed calling sequence to receive full ref_time array (Year,DOY,SOD) instead
of just seconds. Compute reference time as offset from DCE start.

Revision 1.2  2000/12/07 22:57:41  storey
Changed module name to axx_update_amodel to match include file.

Revision 1.1  2000/11/27 13:29:32  storey
Original version


RETURN VALUE:
Type = int
Value    Description
-----    -----------
ERROR    Error updating the model file
SUCCESS  Model file was updated  

NOTES:

ALGORITHM REFERENCES:
	See axx_alimodel.h for description of amodel structure.	
*******************************************************************************/
#include <stdio.h>
#include <string.h>
#include "hdf.h"
#include "xxx_LogStatus.h"
#include "axx_alimodel.h"
#include "axx_forward_model.h"

int axx_update_amodel
(
   char *alimodel_name,		/* I: name of original ALI model file             */
   char *prec_alimodel_name,	/* I: name of precision corrected ALI model file  */
   char *hk_dir,		/* I: name of housekeeping data directory         */
   char *hk_root,		/* I: housekeeping data file root name            */
   char *cpf_name,   		/* I: Path of the cpf (optional) 	          */
   double *reftime,   		/* I: precision reference time (year, doy, sod)   */
   VECTOR satpos,    		/* I: precision satellite position                */
   VECTOR satvel,    		/* I: precision satellite velocity                */
   VECTOR rpyangle,  		/* I: precision roll, pitch, yaw angle            */
   VECTOR rpyrate,   		/* I: precision roll, pitch, yaw rate             */
   double *refatt		/* O: roll, pitch, and yaw at reference time      */
)

{
   ALIMODEL *amodel; 			/* points to ALI model structure */
   char	ErrMsg[STRLEN];			/* error message used by xxx_LogError */
   CPF *cpf=NULL;			/* CPF structure */

/*----------------------------------------------------------------------
  Allocate space for the model structure.
----------------------------------------------------------------------*/

   if ((amodel = malloc(sizeof(ALIMODEL)))== NULL)
   {
      xxx_LogStatus ("AXXLIB", __FILE__,  __LINE__, 
                     "Error - cannot allocate model structure.");
      return (ERROR);
   }

/*----------------------------------------------------------------------
  Open the original model file and fill the model structure.
----------------------------------------------------------------------*/

   if (axx_get_alimodel (alimodel_name, amodel) == FAIL)
   {
      xxx_LogStatus ("AXXLIB", __FILE__, __LINE__, 
                     "Error returned from axx_get_alimodel.");
      free(amodel);
      return (ERROR);
   }

/*----------------------------------------------------------------------
  Update the precision values in the model structure.        
----------------------------------------------------------------------*/

   amodel->p_roll = rpyangle.x;
   amodel->p_rrate = rpyrate.x;
   amodel->p_pitch = rpyangle.y;
   amodel->p_prate = rpyrate.y;
   amodel->p_yaw = rpyangle.z;
   amodel->p_yrate = rpyrate.z;
   amodel->p_x = satpos.x;
   amodel->p_vx = satvel.x;
   amodel->p_y = satpos.y;
   amodel->p_vy = satvel.y;
   amodel->p_z = satpos.z;
   amodel->p_vz = satvel.z;
   amodel->p_ref_time = (reftime[2] - amodel->dce_time[2])
                      + (reftime[1] - amodel->dce_time[1])*SEC_PER_DAY;

/*----------------------------------------------------------------------
   Find the spacecraft attitude at the reference time
----------------------------------------------------------------------*/
   if (axx_findatt(amodel, amodel->p_ref_time, 
                   &refatt[0], &refatt[1], &refatt[2]) == ERROR)
   {
      xxx_LogStatus ("AXXLIB", __FILE__, __LINE__, 
                     "Error finding reference attitude in axx_findatt");
      free(amodel);
      return (ERROR);
   }

/*----------------------------------------------------------------------
   Set this to let run_alimodel know that it is precision updated.
----------------------------------------------------------------------*/

   amodel->correction_type = PRECISION;

/*----------------------------------------------------------------------
   Process the model data using the precision information.
----------------------------------------------------------------------*/
   
   if (axx_run_alimodel(hk_dir, hk_root, cpf_name, amodel, &cpf) == ERROR)
   {
      xxx_LogStatus ("AXXLIB", __FILE__, __LINE__, 
                     "Error processing model data in axx_run_alimodel");
      free(amodel);
      return (ERROR);
   }

/*----------------------------------------------------------------------
   Write out the ALI model with the modified information to an HDF file.
----------------------------------------------------------------------*/

   if (axx_put_alimodel(prec_alimodel_name, amodel) == ERROR)
   {
      sprintf (ErrMsg, "Error writing new ALI model file %s", prec_alimodel_name);
      xxx_LogStatus ("AXXLIB", __FILE__, __LINE__, ErrMsg);
      free(amodel);
      exx_free_cpf(cpf);
      
      return (ERROR);
   }

   free(amodel);

   return(SUCCESS); 
}
