#ifndef _AXX_RPS_H
#define _AXX_RPS_H

#include "xxx_structures.h"
#include "xxx_ali_level0r.h"
#include "xxx_ali_level1r.h"



/* ---------
   Constants

   --------- */
#define NUM_CAL_SOURCES  2   /* Right now... IC lamp and scene histograms */
#define NUM_LAMP_STATES  4

/* Constants defining the overall number of detectors (useful when there isn't
   access to a LEVEL0R or CAL_INFO structure).  */
#define NUM_DETS_MS     320
#define NUM_DETS_PAN    960

/* Constants defining the type of data source contained in Level0R scenes */
#define PRECAL           0
#define IMAGE            1
#define DARK             2
#define LAMP             3

/* Constants defining results of detector checks */
#define GOOD             1
#define MAX_ERROR        2
#define MIN_ERROR        3
#define BOTH_ERROR       4

/* Constants defining the ALI's dynamic range (assumed to be unsigned, 12-bit
   numbers between 0 and 4095) */
#define MAX_ALI_RANGE 4095   
#define MIN_ALI_RANGE 0

/* Constant defining fill data conditions (Most-significant byte all 1's) */
#define FILL_DATA   0xf000   /* Can be changed whenever necessary */
#define FILL_FRAMES      6   /* Number of potential fill frames at edges of
				scene data */

/* Constants defining the type of calibration performed */
#define CAL_REL_CPF      0
#define CAL_REL_IC       1
#define CAL_REL_HIST     2
#define CAL_ABS_SOLAR    3
#define CAL_ABS_LUNAR    4
#define CAL_ABS_GLOOK    5

/* Constants defining lamp state/bias data.  The database will contain
   lamp states 1 (the brightest) to 4 (the darkest).  Add two more states to
   describe the bias.  State 5 (PRECAL_OFFSET) represents the bias data
   BEFORE the scene is imaged (with the shutter door closed).  State 6
   (DARK_OFFSET) represents the bias data AFTER the scene is imaged (again,
   with the shutter door closed).*/
#define PRECAL_OFFSET    5
#define DARK_OFFSET      6

/* Miscellaneous constants */
#define ADJ_BINS         2        /* Used as default for detector saturation
				     detection */
#define MAX_EST_GAIN   1.20       /* "Kludge" for situations (particularly
				     IC-based gain) where the estimated
				     gain values are WELL ABOVE what might
				     reasonably be expected.  This parameter
				     can be altered if necessary. */

#define NO_VALUE       -1

#define LOW             0
#define HIGH            1


/* Constants defining expected fill ranges in Level0R products (these
   frames will be ignored when doing a fill characterization).  These can
   be changed if the processing procedures change. */



 /* ---------------------------------
 * Calibration information structure
 * --------------------------------- */
typedef struct SCENE_HIST_STATS
{
  int16         band_maxDN;         /* maximum response for entire band */
  int16         band_minDN;         /* minimum response for entire band */
  float64       band_meanDN;        /* mean response for entire band */
  float64       band_stddev;        /* std. dev. of response for entire band */
  int32         band_valid_frames;  /* total number of valid frames per band */
  int32         *valid_frames;      /* number of valid histogram counts for
				       each detector */
  float64       *meanDN;            /* mean detector response */
  float64       *stddev;            /* std.dev of detector response */
  int16         *maxDN;             /* maximum detector response */
  int16         *minDN;             /* minimum detector response */
} SCENE_HIST_STATS;



typedef struct SCENE_ARTIFACT_STATS
{
  char         *type;      /* type of artifact.  Currently accounted
			      for artifacts include 'IN' (Impulse
			      Noise), 'CNMagnitude'/'CNFrequency'
			      (Coherent Noise), 'DropLines',
			      'FillFrame'/'FillSegment' (amount of
			      data filling that had to be done),
			      'SatHigh'/'SatLow' (Detector Saturation
			      levels).  */

  float64      *value;      /* Used for those artifacts that have a
			       specific numeric value.  Used only
			       with IN, CNMagnitude/CNFrequency
			       artifacts. */

  int32        *number;     /* Again, used for those artifacts that
		               have a specific numeric value.  */
} SCENE_ARTIFACT_STATS;




typedef struct IC_LAMP_STATS
{
  float64	*det_mean;   /* mean lamp state value.  There will be
			         1280 elements (MS) or 3840 elements (pan)
			         in this array. */
  float64       *det_stddev;  /* std. dev. of lamp state value.  There
				 will be 1280 elements (MS) or 3840
				 elements (pan) in this array.*/

  int32         *valid_frames;
  
  float64       *band_average;       /* this array holds the mean
				        lamp state response over ALL
					detectors in a given band. */
  int32         band_valid_frames;
} IC_LAMP_STATS;


typedef struct GAIN_STATS
{
  float64      *p1;               /* relative detector gains for known
				     calibration sources */
  float64      *p2;               /* relative detector biases for known
				     calibration sources */
  char         *cal_method;       /* calibration description (IC,
				     histogram, solar, lunar, ground-look)
  int32        *valid_frames;     /* number of valid frames used for each
				     detector for a given calibration source
				     */
  float64      *p1_stderr;
  float64      *p2_stderr;
} GAIN_STATS;


typedef struct OFFSET_STATS
{
  float64       *mean;
  float64       *stddev;
  int32         *valid_frames;
} OFFSET_STATS;


typedef struct CAL_INFO
{
  IC_LAMP_STATS        ic_lamp;
  SCENE_HIST_STATS     hist;
  GAIN_STATS           gain;
  OFFSET_STATS         precal;
  OFFSET_STATS         dark;
  int32		       band;
  int32		       ndets;
} CAL_INFO;


/****************************************************************************/
/* -----------------------------------
   Level 0 R Prototypes
   ----------------------------------- */

int axx_allocate_cal_info(
CAL_INFO *cal_info,      /* 0: Structure to contain calibration data */
int32    ndets,          /* I: Number of detectors */
int32    band,           /* I: Current 0-indexed band number */
int16    skip_length);   /* I: Number of frames to skip to reach consecutive
			    data (used only to access bias data) */

int axx_calc_bias(
char      basename[],    /* I: "Name" of EO1 scene */
LEVEL0R   *level0r,      /* I: Structure containing level 0R data */
CAL_INFO  *cal_info,     /* O: Structure to contain bias information */
int32     sca,           /* I: Current SCA number */
int32     band_num,      /* I: 1-referenced index to band number */
int16     skip_length,   /* I: number of frames to skip for consecutive data
			       (1 for MS bands, 3 for pan band) */
int       trend_on,      /* I: "Flag" indicating whether gain data should
                               be written to the database  */
char      database[]);   /* I: Name of trending database  */

int axx_calc_hist(
char    basename[],      /* I: Name of EO1 scene */
LEVEL0R *level0r,        /* I: Structure containing Level0 data */
SCENE_HIST_STATS *hs,    /* O: Structure to contain histogram data */
int32   sca,             /* I: Current SCA */
int32   band,            /* I: Current 0-indexed band number */
int32   num_dets,        /* I: Number of detectors */
const int src,           /* I: Data source to histogram */
int     trend_on,        /* I: Flag to indicate whether data is to
			       be written to database */
char    database[]);     /* I: Name of trending database */

int axx_calc_ICRDG(
char     basename[],     /* I: Name of EO1 scene */
LEVEL0R  *level0r,       /* I: Structure containing Level 0 data */
CAL_INFO *cal_info,      /* O: Structure to contain gain data */
int32    sca,            /* I: Current SCA */
int32    band,           /* I: Current 0-indexed band number */
int16    skip_length,    /* I: Number of frames to skip to reach
			       consecutive data (bias data only) */
int      trend_on,       /* I: Flag to indicate whether data is to
			       be written to database */
char     database[]);    /* I: Name of trending database */

int axx_calibrate(
LEVEL0R *level0r,        /* I: Structure containing level 0R data */
CAL_INFO *cal_info,      /* I: Structure containing gain & bias information */
int32 band_num,          /* I: 0-referenced index to band number */
int16 skip_length,       /* I: Number of frames to skip over */
int32 scale,             /* I: Scale factor of output level 1R data*/
LEVEL1R *level1r);       /* O: Structure to contian level 1R data */

int axx_char_det_fill(
LEVEL0R    *level0r,     /* I: Structure containing data to check for fill */
char       basename[],   /* I: Name of EO1 scene  */
int32      sca,          /* I: Current SCA */
int32      band,         /* I: Current 0-indexed band number */
const int src)           /* I: Particular data source to check for fill */;

int axx_char_det_sat(
char       basename[],   /* I: Name of EO1 scene */
LEVEL0R    *level0r,     /* I: Structure containing data to check for
			       saturation */
CAL_INFO   *cal_info,    /* I: Structure containing image histogram data */
int32      sca,          /* I: Current SCA */
int32      band,         /* I: Current 0-indexed band number */
const int  bias_src,     /* I: Particular source for bias data (precal or
			       dark) */
SCENE_ARTIFACT_STATS *ar,  /* O: Structure to contain saturation data */
int        trend_on,       /* I: Flag to indicate whether data is to be
			         written to database */
char       database[]);    /* I: Name of trending database */

int axx_convolve_detector_data(
int16  *pLineBuff,         /* I: Array of detector data to convolve */
int32  dLineLength,        /* I: Length of detector data array */
int32  dFilterLength,      /* I: Length of convolution filter */
int16  *pOutBuff)          /* O: Array of convolved detector data */;

int axx_create_l1r_attrs(
LEVEL0R l0r,                    /* I: Structure containing level 0R data */
LEVEL1R *l1r);                  /* O: Structure to contain level 1R data */

int axx_create_l1r_header(
L0R_HEADER l0header,            /* I: Structure containing level 0R header */
L1R_HEADER *l1header);          /* O: Structure to contain level 1R header */

int axx_do_lamp_gains(
CAL_INFO   *cal_info,           /* I/O: Structure to contain lampstate data */
int32      band);               /* I: Current 0-indexed band number */

int axx_do_lamp_stats(
OFFSET_STATS    *os,            /* I: Structure containing mean bias data */
int16           *pLine,         /* I: Array containing detector data */
int32           *pLimits,       /* I: Array containing starting/stopping frame
				      information */
int32           det,            /* I: Current detector */
int32           num_dets,       /* I: Number of detectors */
int32           state,          /* I: Current lamp state */
int16           skip_length,    /* I: Number of frames to skip to reach
				      consecutive data (bias data only) */
float64         *pMean,         /* O: Mean detector lamp response */
float64         *pStd,          /* O: Std. deviation of detector response */
int32           *pCount);       /* O: Number of valid frames of lamp data */

int axx_find_minmax(
int32    det,                   /* I: Current detector */
int32    num_dets,              /* I: Number of detectors */
int32    num_frames,            /* I: Number of frames */
int16    *pData,                /* I: Pointer to data */
int16    *pMax,                 /* O: Maximum DN value */
int16    *pMin);                /* O: Minimum DN value */

int axx_find_sat(
const int   level,              /* I: Indicates which saturation level to
				      determine (high or low) */
int32       num_dets,           /* I: Number of detectors */
int32       num_frames,         /* I: Number of frames of data per detector */
SCENE_HIST_STATS *hs,           /* I: Structure containing image histogram
				      data */
int16       *pData,             /* I: Pointer to image data */
int32       *det_sats);         /* O: Array to contain detector saturation
				      levels */

int axx_find_state_bounds(
int32    band,                  /* I: Current 0-indexed band */
int32    det,                   /* I: Current detector */
int32    dTotalLength,          /* I: Convolved detector data array length */
int16    *pConv,                /* I: Convolved detector data */
int16    *pRank,                /* I: Rank data array */
float64  t1,                    /* I: Threshold value */
int32    pBound[],              /* O: Array containing starting/stopping frame
				      locations (these define states) */
int16    *pNumPeaks);           /* O: Number of detected convolution peaks
				      (should be 3, since there are 4 lamp
				      states) */

int axx_free_cal_info(
CAL_INFO *cal_info);            /* O: Structure to be cleaned up */

int axx_generate_1r(
LEVEL0R    level0r,             /* I: Structure containing Level 0 data */
LEVEL1R    *level1r);           /* O: Structure to contain Level 1R data */

int axx_MakeALIBandString(
int32  band,                    /* I: 0-indexed band number */
char   bName[]);                /* O: Equivalent ALI band name */

int axx_set_ranks(
int32    band,                  /* I: Current 0-indexed band */
int32    totalLength,           /* I: Convolved detector data array length */
int16    *pConv,                /* I: Convolved detector data */
int16    *pRank,                /* O: Rank data array */
float64  *pT1);                 /* O: Peak threshold value (below this,
				      convolved data is essentially "noise") */
int axx_vert_moment(
int16 *p_array,                 /* I: 2D array to determine detector mean &
				      stddev */
int32 xdim,                     /* I: number of detectors */
int32 ydim,                     /* I: number of frames */
int16 skip_length,              /* I: number of frames to skip to get to
				      consecutive data (1 for MS bands, 3
				      for pan band)  */
float64 *p_mean,                /* O: array of means */
float64 *p_stddev,              /* O: array of standard deviations */
int32 *p_num_valid_points);     /* O: number of valid points in mean and
				      stddev */



#endif
