#include <stdio.h>
#include <unistd.h>
#include <sys/types.h>
#include <stddef.h>
#include "xxx_const.h"
#include "axx_grid.h"

/******************************************************************************
NAME:           axx_malloc_grid

PURPOSE:
Malloc space for all the *pointers* in the grid.  The actual grid structure
should already exist.

RETURN VALUE:   Type = int
Value    Description
-----    -----------
SUCCESS  Successful completion
ERROR    Operation failed

NOTES:
This routine will loop through the specified bands, mallocing space for the
pointers within the grid.  If no bands were specified, then all the bands in
the grid will be malloced.  In this case, the bands array will be filled in
with the bands in the grid and nbands will be set to the correct value.  If
bands 6L and 6H are both present, then their pointers within the grid will
have the same address.  Thus we only need to malloc space for the pointer for
one of the bands.  This routine will choose band 6L.

ALGORITHM REFERENCES:
See axx_grid.h for a description of the grid structure.
******************************************************************************/

int axx_malloc_grid (
  BandNumber *bands,         /* I/O: Array of bands to free from the grid */
  int *nbands,               /* I/O: Number of bands in the bands array   */
  axx_GRID_TYPE *grid)       /* I/O: Grid structure to be malloced       */
{
   int  i, j, k, n;           /* Loop variables                           */
   int  currref;              /* Current reference band value             */
   char ErrMsg[STRLEN];       /* Error message                            */

  /* If no bands are specified then malloc all the bands in the grid.
   */
   if (*nbands == 0)
   {
      *nbands = grid->nbands;
      memcpy(bands, grid->bands, *nbands * sizeof (BandNumber));
   }

  /* Loop through the bands mallocing the correct reference bands.
   */
   for (i = 0; i < *nbands; i++)
   {
      currref = bands[i];
      for( k=0 ; k<NSCA ; k++ )
      {
       /* Get the reference value of the current band.  */
   
         grid->gridbands[k][currref].in_lines = 
               calloc (grid->gridbands[k][currref].ngrid_lines, sizeof (int ));
         if (grid->gridbands[k][currref].in_lines == NULL)
         { /* Free previously malloced pointers. */
            strcpy (ErrMsg, "Error mallocing space for the pointers.");
            xxx_LogStatus ("AXXLIB", __FILE__, __LINE__, ErrMsg);
            for (n = 0; n < k; n++)
            {
               free (grid->gridbands[n][currref].in_lines);
               free (grid->gridbands[n][currref].in_samps);
               free (grid->gridbands[n][currref].out_lines);
               free (grid->gridbands[n][currref].out_samps);
               free (grid->gridbands[n][currref].poly_lines);
               free (grid->gridbands[n][currref].poly_samps);
               free (grid->gridbands[n][currref].sattoproj);
               free (grid->gridbands[n][currref].projtosat);
               free (grid->gridbands[n][currref].delta_line_odd);
               free (grid->gridbands[n][currref].delta_samp_odd);
               free (grid->gridbands[n][currref].delta_line_even);
               free (grid->gridbands[n][currref].delta_samp_even);
            }
            for (j = 0; j < i; j++)
            {   
               currref = bands[j];
               for( n=0 ; n<NSCA ; n++ )
               {
                  free (grid->gridbands[n][currref].in_lines);
                  free (grid->gridbands[n][currref].in_samps);
                  free (grid->gridbands[n][currref].out_lines);
                  free (grid->gridbands[n][currref].out_samps);
                  free (grid->gridbands[n][currref].poly_lines);
                  free (grid->gridbands[n][currref].poly_samps);
                  free (grid->gridbands[n][currref].sattoproj);
                  free (grid->gridbands[n][currref].projtosat);
                  free (grid->gridbands[n][currref].delta_line_odd);
                  free (grid->gridbands[n][currref].delta_samp_odd);
                  free (grid->gridbands[n][currref].delta_line_even);
                  free (grid->gridbands[n][currref].delta_samp_even);
               }
            }
            return (ERROR);
         }
   
         grid->gridbands[k][currref].in_samps = 
               calloc (grid->gridbands[k][currref].ngrid_samps, sizeof (int ));
         if (grid->gridbands[k][currref].in_samps == NULL)
         { /* Free previously malloced pointers. */
            strcpy (ErrMsg, "Error mallocing space for the pointers.");
            xxx_LogStatus ("AXXLIB", __FILE__, __LINE__, ErrMsg);
            free (grid->gridbands[k][currref].in_lines);
            for (n = 0; n < k; n++)
            {   
               free (grid->gridbands[n][currref].in_lines);
               free (grid->gridbands[n][currref].in_samps);
               free (grid->gridbands[n][currref].out_lines);
               free (grid->gridbands[n][currref].out_samps);
               free (grid->gridbands[n][currref].poly_lines);
               free (grid->gridbands[n][currref].poly_samps);
               free (grid->gridbands[n][currref].sattoproj);
               free (grid->gridbands[n][currref].projtosat);
               free (grid->gridbands[n][currref].delta_line_odd);
               free (grid->gridbands[n][currref].delta_samp_odd);
               free (grid->gridbands[n][currref].delta_line_even);
               free (grid->gridbands[n][currref].delta_samp_even);
            }   
            for (j = 0; j < i; j++)
            { 
               currref = bands[j];
               for (n = 0; n < NSCA; n++)
               {
                  free (grid->gridbands[n][currref].in_lines);
                  free (grid->gridbands[n][currref].in_samps);
                  free (grid->gridbands[n][currref].out_lines);
                  free (grid->gridbands[n][currref].out_samps);
                  free (grid->gridbands[n][currref].poly_lines);
                  free (grid->gridbands[n][currref].poly_samps);
                  free (grid->gridbands[n][currref].sattoproj);
                  free (grid->gridbands[n][currref].projtosat);
                  free (grid->gridbands[n][currref].delta_line_odd);
                  free (grid->gridbands[n][currref].delta_samp_odd);
                  free (grid->gridbands[n][currref].delta_line_even);
                  free (grid->gridbands[n][currref].delta_samp_even);
               }
            }
            return (ERROR);
         }
   
         grid->gridbands[k][currref].out_lines = malloc (grid->gridbands[k][currref].nzplanes *
                   grid->gridbands[k][currref].ngrid_lines *
                   grid->gridbands[k][currref].ngrid_samps * sizeof (double));
         if (grid->gridbands[k][currref].out_lines == NULL)
         { /* Free previously malloced pointers. */
            strcpy (ErrMsg, "Error mallocing space for the pointers.");
            xxx_LogStatus ("AXXLIB", __FILE__, __LINE__, ErrMsg);
            free (grid->gridbands[k][currref].in_lines);
            free (grid->gridbands[k][currref].in_samps);
            for (n = 0; n < k; n++)
            {  
               free (grid->gridbands[n][currref].in_lines);
               free (grid->gridbands[n][currref].in_samps);
               free (grid->gridbands[n][currref].out_lines);
               free (grid->gridbands[n][currref].out_samps);
               free (grid->gridbands[n][currref].poly_lines);
               free (grid->gridbands[n][currref].poly_samps);
               free (grid->gridbands[n][currref].sattoproj);
               free (grid->gridbands[n][currref].projtosat);
               free (grid->gridbands[n][currref].delta_line_odd);
               free (grid->gridbands[n][currref].delta_samp_odd);
               free (grid->gridbands[n][currref].delta_line_even);
               free (grid->gridbands[n][currref].delta_samp_even);
            }   
            for (j = 0; j < i; j++)
            { 
               currref = bands[j];
               for( n=0 ; n<NSCA ; n++ )
               {
                  free (grid->gridbands[n][currref].in_lines);
                  free (grid->gridbands[n][currref].in_samps);
                  free (grid->gridbands[n][currref].out_lines);
                  free (grid->gridbands[n][currref].out_samps);
                  free (grid->gridbands[n][currref].poly_lines);
                  free (grid->gridbands[n][currref].poly_samps);
                  free (grid->gridbands[n][currref].sattoproj);
                  free (grid->gridbands[n][currref].projtosat);
                  free (grid->gridbands[n][currref].delta_line_odd);
                  free (grid->gridbands[n][currref].delta_samp_odd);
                  free (grid->gridbands[n][currref].delta_line_even);
                  free (grid->gridbands[n][currref].delta_samp_even);
               }
            }
            return (ERROR);
         }
   
         grid->gridbands[k][currref].out_samps = malloc (grid->gridbands[k][currref].nzplanes *
                   grid->gridbands[k][currref].ngrid_lines *
                   grid->gridbands[k][currref].ngrid_samps * sizeof (double));
         if (grid->gridbands[k][currref].out_samps == NULL)
         { /* Free previously malloced pointers. */
            strcpy (ErrMsg, "Error mallocing space for the pointers.");
            xxx_LogStatus ("AXXLIB", __FILE__, __LINE__, ErrMsg);
            free (grid->gridbands[k][currref].in_lines);
            free (grid->gridbands[k][currref].in_samps);
            free (grid->gridbands[k][currref].out_lines);
            for (n = 0; n < k; n++)
            {  
               free (grid->gridbands[n][currref].in_lines);
               free (grid->gridbands[n][currref].in_samps);
               free (grid->gridbands[n][currref].out_lines);
               free (grid->gridbands[n][currref].out_samps);
               free (grid->gridbands[n][currref].poly_lines);
               free (grid->gridbands[n][currref].poly_samps);
               free (grid->gridbands[n][currref].sattoproj);
               free (grid->gridbands[n][currref].projtosat);
               free (grid->gridbands[n][currref].delta_line_odd);
               free (grid->gridbands[n][currref].delta_samp_odd);
               free (grid->gridbands[n][currref].delta_line_even);
               free (grid->gridbands[n][currref].delta_samp_even);
            }   
            for (j = 0; j < i; j++)
            { 
               currref = bands[j];
               for(n = 0; n < NSCA; n++ )
               {
                  free (grid->gridbands[n][currref].in_lines);
                  free (grid->gridbands[n][currref].in_samps);
                  free (grid->gridbands[n][currref].out_lines);
                  free (grid->gridbands[n][currref].out_samps);
                  free (grid->gridbands[n][currref].poly_lines);
                  free (grid->gridbands[n][currref].poly_samps);
                  free (grid->gridbands[n][currref].sattoproj);
                  free (grid->gridbands[n][currref].projtosat);
                  free (grid->gridbands[n][currref].delta_line_odd);
                  free (grid->gridbands[n][currref].delta_samp_odd);
                  free (grid->gridbands[n][currref].delta_line_even);
                  free (grid->gridbands[n][currref].delta_samp_even);
               }
            }
            return (ERROR);
         }
   
         grid->gridbands[k][currref].delta_line_odd = malloc(2 * grid->gridbands[k][currref].ngrid_lines *
                   grid->gridbands[k][currref].ngrid_samps * sizeof (double));
         if (grid->gridbands[k][currref].delta_line_odd == NULL)
         { /* Free previously malloced pointers. */
            strcpy (ErrMsg, "Error mallocing space for the pointers.");
            xxx_LogStatus ("AXXLIB", __FILE__, __LINE__, ErrMsg);
            free (grid->gridbands[k][currref].in_lines);
            free (grid->gridbands[k][currref].in_samps);
            free (grid->gridbands[k][currref].out_lines);
            free (grid->gridbands[k][currref].out_samps);
            for (n = 0; n < k; n++)
            {
               free (grid->gridbands[n][currref].in_lines);
               free (grid->gridbands[n][currref].in_samps);
               free (grid->gridbands[n][currref].out_lines);
               free (grid->gridbands[n][currref].out_samps);
               free (grid->gridbands[n][currref].poly_lines);
               free (grid->gridbands[n][currref].poly_samps);
               free (grid->gridbands[n][currref].sattoproj);
               free (grid->gridbands[n][currref].projtosat);
               free (grid->gridbands[n][currref].delta_line_odd);
               free (grid->gridbands[n][currref].delta_samp_odd);
               free (grid->gridbands[n][currref].delta_line_even);
               free (grid->gridbands[n][currref].delta_samp_even);
            }
            for (j = 0; j < i; j++)
            {
               currref = bands[j];
               for(n = 0; n < NSCA; n++ )
               {
                  free (grid->gridbands[n][currref].in_lines);
                  free (grid->gridbands[n][currref].in_samps);
                  free (grid->gridbands[n][currref].out_lines);
                  free (grid->gridbands[n][currref].out_samps);
                  free (grid->gridbands[n][currref].poly_lines);
                  free (grid->gridbands[n][currref].poly_samps);
                  free (grid->gridbands[n][currref].sattoproj);
                  free (grid->gridbands[n][currref].projtosat);
                  free (grid->gridbands[n][currref].delta_line_odd);
                  free (grid->gridbands[n][currref].delta_samp_odd);
                  free (grid->gridbands[n][currref].delta_line_even);
                  free (grid->gridbands[n][currref].delta_samp_even);
               }
            }
            return (ERROR);
         }

         grid->gridbands[k][currref].delta_samp_odd = malloc(2 * grid->gridbands[k][currref].ngrid_lines *
                   grid->gridbands[k][currref].ngrid_samps * sizeof (double));
         if (grid->gridbands[k][currref].delta_samp_odd == NULL)
         { /* Free previously malloced pointers. */
            strcpy (ErrMsg, "Error mallocing space for the pointers.");
            xxx_LogStatus ("AXXLIB", __FILE__, __LINE__, ErrMsg);
            free (grid->gridbands[k][currref].in_lines);
            free (grid->gridbands[k][currref].in_samps);
            free (grid->gridbands[k][currref].out_lines);
            free (grid->gridbands[k][currref].out_samps);
            free (grid->gridbands[k][currref].delta_line_odd);
            for (n = 0; n < k; n++)
            {
               free (grid->gridbands[n][currref].in_lines);
               free (grid->gridbands[n][currref].in_samps);
               free (grid->gridbands[n][currref].out_lines);
               free (grid->gridbands[n][currref].out_samps);
               free (grid->gridbands[n][currref].poly_lines);
               free (grid->gridbands[n][currref].poly_samps);
               free (grid->gridbands[n][currref].sattoproj);
               free (grid->gridbands[n][currref].projtosat);
               free (grid->gridbands[n][currref].delta_line_odd);
               free (grid->gridbands[n][currref].delta_samp_odd);
               free (grid->gridbands[n][currref].delta_line_even);
               free (grid->gridbands[n][currref].delta_samp_even);
            }
            for (j = 0; j < i; j++)
            {
               currref = bands[j];
               for(n = 0; n < NSCA; n++ )
               {
                  free (grid->gridbands[n][currref].in_lines);
                  free (grid->gridbands[n][currref].in_samps);
                  free (grid->gridbands[n][currref].out_lines);
                  free (grid->gridbands[n][currref].out_samps);
                  free (grid->gridbands[n][currref].poly_lines);
                  free (grid->gridbands[n][currref].poly_samps);
                  free (grid->gridbands[n][currref].sattoproj);
                  free (grid->gridbands[n][currref].projtosat);
                  free (grid->gridbands[n][currref].delta_line_odd);
                  free (grid->gridbands[n][currref].delta_samp_odd);
                  free (grid->gridbands[n][currref].delta_line_even);
                  free (grid->gridbands[n][currref].delta_samp_even);
               }
            }
            return (ERROR);
         }

         grid->gridbands[k][currref].delta_line_even = malloc(2 * grid->gridbands[k][currref].ngrid_lines *
                   grid->gridbands[k][currref].ngrid_samps * sizeof (double));
         if (grid->gridbands[k][currref].delta_line_even == NULL)
         { /* Free previously malloced pointers. */
            strcpy (ErrMsg, "Error mallocing space for the pointers.");
            xxx_LogStatus ("AXXLIB", __FILE__, __LINE__, ErrMsg);
            free (grid->gridbands[k][currref].in_lines);
            free (grid->gridbands[k][currref].in_samps);
            free (grid->gridbands[k][currref].out_lines);
            free (grid->gridbands[k][currref].out_samps);
            free (grid->gridbands[k][currref].delta_line_odd);
            free (grid->gridbands[k][currref].delta_samp_odd);
            for (n = 0; n < k; n++)
            {
               free (grid->gridbands[n][currref].in_lines);
               free (grid->gridbands[n][currref].in_samps);
               free (grid->gridbands[n][currref].out_lines);
               free (grid->gridbands[n][currref].out_samps);
               free (grid->gridbands[n][currref].poly_lines);
               free (grid->gridbands[n][currref].poly_samps);
               free (grid->gridbands[n][currref].sattoproj);
               free (grid->gridbands[n][currref].projtosat);
               free (grid->gridbands[n][currref].delta_line_odd);
               free (grid->gridbands[n][currref].delta_samp_odd);
               free (grid->gridbands[n][currref].delta_line_even);
               free (grid->gridbands[n][currref].delta_samp_even);
            }
            for (j = 0; j < i; j++)
            {
               currref = bands[j];
               for(n = 0; n < NSCA; n++ )
               {
                  free (grid->gridbands[n][currref].in_lines);
                  free (grid->gridbands[n][currref].in_samps);
                  free (grid->gridbands[n][currref].out_lines);
                  free (grid->gridbands[n][currref].out_samps);
                  free (grid->gridbands[n][currref].poly_lines);
                  free (grid->gridbands[n][currref].poly_samps);
                  free (grid->gridbands[n][currref].sattoproj);
                  free (grid->gridbands[n][currref].projtosat);
                  free (grid->gridbands[n][currref].delta_line_odd);
                  free (grid->gridbands[n][currref].delta_samp_odd);
                  free (grid->gridbands[n][currref].delta_line_even);
                  free (grid->gridbands[n][currref].delta_samp_even);
               }
            }
            return (ERROR);
         }

         grid->gridbands[k][currref].delta_samp_even = malloc(2 * grid->gridbands[k][currref].ngrid_lines *
                   grid->gridbands[k][currref].ngrid_samps * sizeof (double));
         if (grid->gridbands[k][currref].delta_samp_odd == NULL)
         { /* Free previously malloced pointers. */
            strcpy (ErrMsg, "Error mallocing space for the pointers.");
            xxx_LogStatus ("AXXLIB", __FILE__, __LINE__, ErrMsg);
            free (grid->gridbands[k][currref].in_lines);
            free (grid->gridbands[k][currref].in_samps);
            free (grid->gridbands[k][currref].out_lines);
            free (grid->gridbands[k][currref].out_samps);
            free (grid->gridbands[k][currref].delta_line_odd);
            free (grid->gridbands[k][currref].delta_samp_odd);
            free (grid->gridbands[k][currref].delta_line_even);
            for (n = 0; n < k; n++)
            {
               free (grid->gridbands[n][currref].in_lines);
               free (grid->gridbands[n][currref].in_samps);
               free (grid->gridbands[n][currref].out_lines);
               free (grid->gridbands[n][currref].out_samps);
               free (grid->gridbands[n][currref].poly_lines);
               free (grid->gridbands[n][currref].poly_samps);
               free (grid->gridbands[n][currref].sattoproj);
               free (grid->gridbands[n][currref].projtosat);
               free (grid->gridbands[n][currref].delta_line_odd);
               free (grid->gridbands[n][currref].delta_samp_odd);
               free (grid->gridbands[n][currref].delta_line_even);
               free (grid->gridbands[n][currref].delta_samp_even);
            }
            for (j = 0; j < i; j++)
            {
               currref = bands[j];
               for(n = 0; n < NSCA; n++ )
               {
                  free (grid->gridbands[n][currref].in_lines);
                  free (grid->gridbands[n][currref].in_samps);
                  free (grid->gridbands[n][currref].out_lines);
                  free (grid->gridbands[n][currref].out_samps);
                  free (grid->gridbands[n][currref].poly_lines);
                  free (grid->gridbands[n][currref].poly_samps);
                  free (grid->gridbands[n][currref].sattoproj);
                  free (grid->gridbands[n][currref].projtosat);
                  free (grid->gridbands[n][currref].delta_line_odd);
                  free (grid->gridbands[n][currref].delta_samp_odd);
                  free (grid->gridbands[n][currref].delta_line_even);
                  free (grid->gridbands[n][currref].delta_samp_even);
               }
            }
            return (ERROR);
         }

         grid->gridbands[k][currref].poly_lines = malloc (grid->gridbands[k][currref].nzplanes * 
                   (grid->gridbands[k][currref].degree+1) *
                   (grid->gridbands[k][currref].degree+1) * sizeof (double));
         if (grid->gridbands[k][currref].poly_lines == NULL)
         { /* Free previously malloced pointers. */
            strcpy (ErrMsg, "Error mallocing space for the pointers.");
            xxx_LogStatus ("AXXLIB", __FILE__, __LINE__, ErrMsg);
            free (grid->gridbands[k][currref].in_lines);
            free (grid->gridbands[k][currref].in_samps);
            free (grid->gridbands[k][currref].out_lines);
            free (grid->gridbands[k][currref].out_samps);
            free (grid->gridbands[k][currref].delta_line_odd);
            free (grid->gridbands[k][currref].delta_samp_odd);
            free (grid->gridbands[k][currref].delta_line_even);
            free (grid->gridbands[k][currref].delta_samp_even);
            for (n = 0; n < k; n++)
            {
               free (grid->gridbands[n][currref].in_lines);
               free (grid->gridbands[n][currref].in_samps);
               free (grid->gridbands[n][currref].out_lines);
               free (grid->gridbands[n][currref].out_samps);
               free (grid->gridbands[n][currref].poly_lines);
               free (grid->gridbands[n][currref].poly_samps);
               free (grid->gridbands[n][currref].sattoproj);
               free (grid->gridbands[n][currref].projtosat);
               free (grid->gridbands[n][currref].delta_line_odd);
               free (grid->gridbands[n][currref].delta_samp_odd);
               free (grid->gridbands[n][currref].delta_line_even);
               free (grid->gridbands[n][currref].delta_samp_even);
            }
            for (j = 0; j < i; j++)
            {
               currref = bands[j];
               for(n = 0; n < NSCA; n++ )
               {
                  free (grid->gridbands[n][currref].in_lines);
                  free (grid->gridbands[n][currref].in_samps);
                  free (grid->gridbands[n][currref].out_lines);
                  free (grid->gridbands[n][currref].out_samps);
                  free (grid->gridbands[n][currref].poly_lines);
                  free (grid->gridbands[n][currref].poly_samps);
                  free (grid->gridbands[n][currref].sattoproj);
                  free (grid->gridbands[n][currref].projtosat);
                  free (grid->gridbands[n][currref].delta_line_odd);
                  free (grid->gridbands[n][currref].delta_samp_odd);
                  free (grid->gridbands[n][currref].delta_line_even);
                  free (grid->gridbands[n][currref].delta_samp_even);
               }
            }
            return (ERROR);
         }
   
         grid->gridbands[k][currref].poly_samps = malloc (grid->gridbands[k][currref].nzplanes *
                  (grid->gridbands[k][currref].degree+1) *
                  (grid->gridbands[k][currref].degree+1) * sizeof (double));
         if (grid->gridbands[k][currref].poly_samps == NULL)
         { /* Free previously malloced pointers. */
            strcpy (ErrMsg, "Error mallocing space for the pointers.");
            xxx_LogStatus ("AXXLIB", __FILE__, __LINE__, ErrMsg);
            free (grid->gridbands[k][currref].in_lines);
            free (grid->gridbands[k][currref].in_samps);
            free (grid->gridbands[k][currref].out_lines);
            free (grid->gridbands[k][currref].out_samps);
            free (grid->gridbands[k][currref].poly_lines);
            free (grid->gridbands[k][currref].delta_line_odd);
            free (grid->gridbands[k][currref].delta_samp_odd);
            free (grid->gridbands[k][currref].delta_line_even);
            free (grid->gridbands[k][currref].delta_samp_even);
            for (n = 0; n < k; n++)
            {
               free (grid->gridbands[n][currref].in_lines);
               free (grid->gridbands[n][currref].in_samps);
               free (grid->gridbands[n][currref].out_lines);
               free (grid->gridbands[n][currref].out_samps);
               free (grid->gridbands[n][currref].poly_lines);
               free (grid->gridbands[n][currref].poly_samps);
               free (grid->gridbands[n][currref].sattoproj);
               free (grid->gridbands[n][currref].projtosat);
               free (grid->gridbands[n][currref].delta_line_odd);
               free (grid->gridbands[n][currref].delta_samp_odd);
               free (grid->gridbands[n][currref].delta_line_even);
               free (grid->gridbands[n][currref].delta_samp_even);
            }
            for (j = 0; j < i; j++)
            { 
               currref = bands[j];
               for(n = 0; n < NSCA; n++ )
               {
                  free (grid->gridbands[n][currref].in_lines);
                  free (grid->gridbands[n][currref].in_samps);
                  free (grid->gridbands[n][currref].out_lines);
                  free (grid->gridbands[n][currref].out_samps);
                  free (grid->gridbands[n][currref].poly_lines);
                  free (grid->gridbands[n][currref].poly_samps);
                  free (grid->gridbands[n][currref].sattoproj);
                  free (grid->gridbands[n][currref].projtosat);
                  free (grid->gridbands[n][currref].delta_line_odd);
                  free (grid->gridbands[n][currref].delta_samp_odd);
                  free (grid->gridbands[n][currref].delta_line_even);
                  free (grid->gridbands[n][currref].delta_samp_even);
               }
            }
            return (ERROR);
         }

         grid->gridbands[k][currref].sattoproj = malloc (grid->gridbands[k][currref].nzplanes *
                   (grid->gridbands[k][currref].ngrid_lines-1) *
                   (grid->gridbands[k][currref].ngrid_samps-1) * sizeof (struct COEFFICIENTS));
         if (grid->gridbands[k][currref].sattoproj == NULL)
         { /* Free previously malloced pointers. */
            strcpy (ErrMsg, "Error mallocing space for pointers.");
            xxx_LogStatus ("AXXLIB", __FILE__, __LINE__, ErrMsg);
            free (grid->gridbands[k][currref].in_lines);
            free (grid->gridbands[k][currref].in_samps);
            free (grid->gridbands[k][currref].out_lines);
            free (grid->gridbands[k][currref].out_samps);
            free (grid->gridbands[k][currref].poly_lines);
            free (grid->gridbands[k][currref].poly_samps);
            free (grid->gridbands[k][currref].delta_line_odd);
            free (grid->gridbands[k][currref].delta_samp_odd);
            free (grid->gridbands[k][currref].delta_line_even);
            free (grid->gridbands[k][currref].delta_samp_even);
            for (n = 0; n < k; n++)
            {
               free (grid->gridbands[n][currref].in_lines);
               free (grid->gridbands[n][currref].in_samps);
               free (grid->gridbands[n][currref].out_lines);
               free (grid->gridbands[n][currref].out_samps);
               free (grid->gridbands[n][currref].poly_lines);
               free (grid->gridbands[n][currref].poly_samps);
               free (grid->gridbands[n][currref].sattoproj);
               free (grid->gridbands[n][currref].projtosat);
               free (grid->gridbands[n][currref].delta_line_odd);
               free (grid->gridbands[n][currref].delta_samp_odd);
               free (grid->gridbands[n][currref].delta_line_even);
               free (grid->gridbands[n][currref].delta_samp_even);
            }
            for (j = 0; j < i; j++)
            { 
               currref = bands[j];
               for(n = 0; n < NSCA; n++ )
               {
                  free (grid->gridbands[n][currref].in_lines);
                  free (grid->gridbands[n][currref].in_samps);
                  free (grid->gridbands[n][currref].out_lines);
                  free (grid->gridbands[n][currref].out_samps);
                  free (grid->gridbands[n][currref].poly_lines);
                  free (grid->gridbands[n][currref].poly_samps);
                  free (grid->gridbands[n][currref].sattoproj);
                  free (grid->gridbands[n][currref].projtosat);
                  free (grid->gridbands[n][currref].delta_line_odd);
                  free (grid->gridbands[n][currref].delta_samp_odd);
                  free (grid->gridbands[n][currref].delta_line_even);
                  free (grid->gridbands[n][currref].delta_samp_even);
               }
            }
            return (ERROR);
         }
   
         grid->gridbands[k][currref].projtosat = malloc (grid->gridbands[k][currref].nzplanes *
                   (grid->gridbands[k][currref].ngrid_lines-1) *
                   (grid->gridbands[k][currref].ngrid_samps-1) * sizeof (struct COEFFICIENTS));
         if (grid->gridbands[k][currref].projtosat == NULL)
         { /* Free previously malloced pointers. */
            strcpy (ErrMsg, "Error mallocing space for pointers.");
            xxx_LogStatus ("AXXLIB", __FILE__, __LINE__, ErrMsg);
            free (grid->gridbands[k][currref].in_lines);
            free (grid->gridbands[k][currref].in_samps);
            free (grid->gridbands[k][currref].out_lines);
            free (grid->gridbands[k][currref].out_samps);
            free (grid->gridbands[k][currref].poly_lines);
            free (grid->gridbands[k][currref].poly_samps);
            free (grid->gridbands[k][currref].sattoproj);
            free (grid->gridbands[k][currref].delta_line_odd);
            free (grid->gridbands[k][currref].delta_samp_odd);
            free (grid->gridbands[k][currref].delta_line_even);
            free (grid->gridbands[k][currref].delta_samp_even);
            for (n = 0; n < k; n++)
            {
               free (grid->gridbands[n][currref].in_lines);
               free (grid->gridbands[n][currref].in_samps);
               free (grid->gridbands[n][currref].out_lines);
               free (grid->gridbands[n][currref].out_samps);
               free (grid->gridbands[n][currref].poly_lines);
               free (grid->gridbands[n][currref].poly_samps);
               free (grid->gridbands[n][currref].sattoproj);
               free (grid->gridbands[n][currref].projtosat);
               free (grid->gridbands[n][currref].delta_line_odd);
               free (grid->gridbands[n][currref].delta_samp_odd);
               free (grid->gridbands[n][currref].delta_line_even);
               free (grid->gridbands[n][currref].delta_samp_even);
            }
            for (j = 0; j < i; j++)
            { 
               currref = bands[j];
               for(n = 0; n < NSCA; n++ )
               {
                  free (grid->gridbands[n][currref].in_lines);
                  free (grid->gridbands[n][currref].in_samps);
                  free (grid->gridbands[n][currref].out_lines);
                  free (grid->gridbands[n][currref].out_samps);
                  free (grid->gridbands[n][currref].poly_lines);
                  free (grid->gridbands[n][currref].poly_samps);
                  free (grid->gridbands[n][currref].sattoproj);
                  free (grid->gridbands[n][currref].projtosat);
                  free (grid->gridbands[n][currref].delta_line_odd);
                  free (grid->gridbands[n][currref].delta_samp_odd);
                  free (grid->gridbands[n][currref].delta_line_even);
                  free (grid->gridbands[n][currref].delta_samp_even);
               }
            }
            return (ERROR);
         }
      }
   }

   return (SUCCESS);
}
