/******************************************************************************
NAME: axx_open_l1r_sca_band 

PURPOSE: Opens a Level 1R ALI SCA file in preparation to read a particular band.

RETURNS:
    NULL on error, otherwise a pointer to the L1R_SCA_BAND structure for the
    image opened.
*******************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include "xxx_Types.h"
#include "xxx_LogStatus.h"
#include "xxx_ali_level1r.h"
#include "axx_hdf_io.h"     /* axx_get_user_band prototype */
#include "axx_l1r_image.h"

L1R_SCA_BAND* axx_open_l1r_sca_band
(
    const char *l1r_file_name,  /* I: Level 1R file name for the SCA */
    BandNumber band             /* I: band to open */
)
{
    char emsg[STRLEN];          /* Error message buffer */
    char progname[STRLEN];      /* Program name */
    int32 sds_id;               /* SDS file ID */
    int32 attr_index;           /* Index of attributes */
    int32 sds_index;            /* index of the current SDS */
    char8 sds_name[9];          /* SDS name */
    int32 sd_id;                /* SD file id for the file opened */
    int found = 0;              /* flag that the correct SDS has been found */
    L1R_SCA_BAND image;         /* local storage for the image info */
    L1R_SCA_BAND *image_ptr;    /* allocated structure to return to the calling
                                   routine */
    int frame_shift_index;      /* index in attributes of the needed frame
                                   shift value */

    sprintf(progname,"READ_L1R_IMAGE");

    /* Get SD file ID */
    if ((sd_id = SDstart(l1r_file_name, DFACC_RDONLY)) == FAIL)
    {
        sprintf(emsg, "Could not open SDS file %s\n", l1r_file_name);
        xxx_LogStatus(progname,__FILE__,__LINE__, emsg);
        return NULL;
    }

    /* search for the correct imagery SDS.  There are two SDS's in the file
       with the same name that hold imagery data.  The one with 1 band is the
       pan band and the one with 9 bands is the multispectral data. */
    sds_index = 0;
    while (!found)
    {
        /* select the current SDS for reading */
        sds_id = SDselect(sd_id, sds_index);
        if (sds_id == FAIL)
        {
            sprintf(emsg, "Could not find the correct image SDS in file %s\n", 
                    l1r_file_name);
            xxx_LogStatus(progname,__FILE__,__LINE__, emsg);
            SDend(sd_id);
            return NULL;
        }

        /* get the SDS name for the current SDS */
        if (SDgetinfo(sds_id, sds_name, &image.number_of_dims, image.dim_sizes,
                      &image.HDF_type, &image.number_of_attributes) == FAIL)
        {
            sprintf(emsg, "Could not find the correct image SDS in file %s\n", 
                    l1r_file_name);
            xxx_LogStatus(progname,__FILE__,__LINE__, emsg);
            SDendaccess(sds_id);
            SDend(sd_id);
            return NULL;
        }

        /* look at each of the image SDS records to see if it contains the band
           requested */
        if ((strncmp(sds_name,"Level 1R",8) == 0) ||
            (strncmp(sds_name,"LEVEL1R",7) == 0))
        {
            if (band == pan)
            {
                /* the requested band is the pan band, so assume that if a 
                   single band image SDS is encountered that it is the pan 
                   band (doesn't appear to be any other way to tell easily) */
                if (image.dim_sizes[0] == 1)
                    found = 1;
            }
            else
            {
                /* a multispectral band has been requested, so assume if the
                   number of bands in the SDS is 9, that it is the 
                   multispectral SDS */
                if (image.dim_sizes[0] == 9)
                    found = 1;
            }
        }

        /* if the desired SDS hasn't been found, end access to the current 
           one.  Otherwise, leave it open so that it can be read later. */
        if (!found)
        {
            if (SDendaccess(sds_id) == FAIL)
            {
                sprintf(emsg, "Error ending access to an SDS in file %s\n", 
                        l1r_file_name);
                xxx_LogStatus(progname,__FILE__,__LINE__, emsg);
                SDend(sd_id);
                return NULL;
            }
        }

        sds_index++;
    }

    /* if the correct SDS wasn't found, return NULL to indicate the band
       could not be found */
    if (!found)
    {
        sprintf(emsg, "Band %d not found in file %s\n", band, l1r_file_name);
        xxx_LogStatus(progname,__FILE__,__LINE__, emsg);
        SDend(sd_id);
        return NULL;
    }

    /* the desired SDS was found, so read the attributes */
    image.attributes.valid_map[0] = INVALID;
    if ((attr_index = SDfindattr(sds_id, "Dataset Type")) != FAIL)
    {
        if (SDreadattr(sds_id, attr_index, image.attributes.dataset_type) 
            == FAIL)
        {
            sprintf(emsg,"Could not read Dataset Type attribute from %s.",
                    l1r_file_name);
            xxx_LogStatus(progname,__FILE__,__LINE__,emsg);
            SDendaccess(sds_id);
            SDend(sd_id);
            return NULL;
        }
        image.attributes.valid_map[0] = VALID;
    }

    image.attributes.valid_map[1] = INVALID;
    if ((attr_index = SDfindattr(sds_id, "ALI Sensor")) != FAIL)
    {
        if (SDreadattr(sds_id, attr_index, image.attributes.ali_sensor) == FAIL)
        {
            sprintf(emsg,"Could not read ALI Sensor attribute from %s.",
                    l1r_file_name);
            xxx_LogStatus(progname,__FILE__,__LINE__,emsg);
            SDendaccess(sds_id);
            SDend(sd_id);
            return NULL;
        }
        image.attributes.valid_map[1] = VALID;
    }

    image.attributes.valid_map[2] = INVALID;
    if ((attr_index = SDfindattr(sds_id, "Data Start Time")) != FAIL)
    {
        if (SDreadattr(sds_id, attr_index, image.attributes.data_start_time) 
            == FAIL)
        {
            sprintf(emsg,"Could not read Data Start Time attribute from %s.",
                    l1r_file_name);
            xxx_LogStatus(progname,__FILE__,__LINE__,emsg);
            SDendaccess(sds_id);
            SDend(sd_id);
            return NULL;
        }
        image.attributes.valid_map[2] = VALID;
    }

    image.attributes.valid_map[3] = INVALID;
    if ((attr_index = SDfindattr(sds_id, "Data Duration")) != FAIL)
    {
        if (SDreadattr(sds_id, attr_index, &image.attributes.data_duration) 
            == FAIL)
        {
            sprintf(emsg,"Could not read Data Duration attribute from %s.",
                    l1r_file_name);
            xxx_LogStatus(progname,__FILE__,__LINE__,emsg);
            SDendaccess(sds_id);
            SDend(sd_id);
            return NULL;
        }
        image.attributes.valid_map[3] = VALID;
    }

    image.attributes.valid_map[4] = INVALID;
    if ((attr_index = SDfindattr(sds_id, "Integration Time")) != FAIL)
    {
        if (SDreadattr(sds_id, attr_index, &image.attributes.integration_time) 
            == FAIL)
        {
            sprintf(emsg,"Could not read Integration Time attribute from %s.",
                    l1r_file_name);
            xxx_LogStatus(progname,__FILE__,__LINE__,emsg);
            SDendaccess(sds_id);
            SDend(sd_id);
            return NULL;
        }
        image.attributes.valid_map[4] = VALID;
    }

    image.attributes.valid_map[5] = INVALID;
    if ((attr_index = SDfindattr(sds_id, "Frame Rate")) != FAIL)
    {
        if (SDreadattr(sds_id, attr_index, &image.attributes.frame_rate) 
            == FAIL)
        {
            sprintf(emsg,"Could not read Frame Rate attribute from %s.",
                    l1r_file_name);
            xxx_LogStatus(progname,__FILE__,__LINE__,emsg);
            SDendaccess(sds_id);
            SDend(sd_id);
            return NULL;
        }
        image.attributes.valid_map[5] = VALID;
    }

    image.attributes.valid_map[6] = INVALID;
    if ((attr_index = SDfindattr(sds_id, "Data Gate Open Time")) != FAIL)
    {
        if (SDreadattr(sds_id, attr_index, 
                       &image.attributes.data_gate_open_time) == FAIL)
        {
            sprintf(emsg,"Could not read Data Gate Open Time attribute "
                    "from %s.", l1r_file_name);
            xxx_LogStatus(progname,__FILE__,__LINE__,emsg);
            SDendaccess(sds_id);
            SDend(sd_id);
            return NULL;
        }
        image.attributes.valid_map[6] = VALID;
    }

    image.attributes.valid_map[7] = INVALID;
    if ((attr_index = SDfindattr(sds_id, "Data Gate Close Time")) != FAIL)
    {
        if (SDreadattr(sds_id, attr_index, 
                       &image.attributes.data_gate_close_time) == FAIL)
        {
            sprintf(emsg,"Could not read Data Gate Close Time attribute "
                    "from %s.", l1r_file_name);
            xxx_LogStatus(progname,__FILE__,__LINE__,emsg);
            SDendaccess(sds_id);
            SDend(sd_id);
            return NULL;
        }
        image.attributes.valid_map[7] = VALID;
    }

    image.attributes.valid_map[8] = INVALID;
    if ((attr_index = SDfindattr(sds_id, "Bytes prior to SYNC")) != FAIL)
    {
        if (SDreadattr(sds_id, attr_index, &image.attributes.bytes_prior_sync) 
            == FAIL)
        {
            sprintf(emsg,"Could not read Bytes prior to SYNC attribute "
                    "from %s.", l1r_file_name);
            xxx_LogStatus(progname,__FILE__,__LINE__,emsg);
            SDendaccess(sds_id);
            SDend(sd_id);
            return NULL;
        }
        image.attributes.valid_map[8] = VALID;
    }

    image.attributes.valid_map[9] = INVALID;
    if ((attr_index = SDfindattr(sds_id, "Number of cross track pixels")) 
        != FAIL)
    {
        if (SDreadattr(sds_id, attr_index, 
                       &image.attributes.number_cross_track_pixels) == FAIL)
        {
            sprintf(emsg,"Could not read Number of cross track pixels "
                    "attribute from %s.", l1r_file_name);
            xxx_LogStatus(progname,__FILE__,__LINE__,emsg);
            SDendaccess(sds_id);
            SDend(sd_id);
            return NULL;
        }
        image.attributes.valid_map[9] = VALID;
    }

    image.attributes.valid_map[10] = INVALID;
    if ((attr_index = SDfindattr(sds_id, "Number of along track pixels")) 
        != FAIL)
    {
        if (SDreadattr(sds_id, attr_index, 
                       &image.attributes.number_along_track_pixels) == FAIL)
        {
            sprintf(emsg,"Could not read Number of along track pixels "
                    "attribute from %s.", l1r_file_name);
            xxx_LogStatus(progname,__FILE__,__LINE__,emsg);
            SDendaccess(sds_id);
            SDend(sd_id);
            return NULL;
        }
        image.attributes.valid_map[10] = VALID;
    }

    image.attributes.valid_map[11] = INVALID;
    if ((attr_index = SDfindattr(sds_id, "Number of bands")) != FAIL)
    {
        if (SDreadattr(sds_id, attr_index, &image.attributes.number_of_bands) 
            == FAIL)
        {
            sprintf(emsg,"Could not read Number of bands attribute from %s.",
                    l1r_file_name);
            xxx_LogStatus(progname,__FILE__,__LINE__,emsg);
            SDendaccess(sds_id);
            SDend(sd_id);
            return NULL;
        }
        image.attributes.valid_map[11] = VALID;
    }

    image.attributes.valid_map[12] = INVALID;
    if ((attr_index = SDfindattr(sds_id, "Percent Missing Data")) != FAIL)
    {
        if (SDreadattr(sds_id, attr_index, 
                       &image.attributes.percent_missing_data) == FAIL)
        {
            sprintf(emsg,"Could not read Percent Missing Data attribute "
                    "from %s.", l1r_file_name);
            xxx_LogStatus(progname,__FILE__,__LINE__,emsg);
            SDendaccess(sds_id);
            SDend(sd_id);
            return NULL;
        }
        image.attributes.valid_map[12] = VALID;
    }

    image.attributes.valid_map[13] = INVALID;
    if ((attr_index = SDfindattr(sds_id, "Frame shift PAN")) != FAIL)
    {
        if (SDreadattr(sds_id, attr_index, &image.attributes.frame_shift[0]) 
            == FAIL)
        {
            sprintf(emsg,"Could not read Frame shift PAN attribute from %s.",
                    l1r_file_name);
            xxx_LogStatus(progname,__FILE__,__LINE__,emsg);
            SDendaccess(sds_id);
            SDend(sd_id);
            return NULL;
        }
        image.attributes.valid_map[13] = VALID;
    }

    image.attributes.valid_map[14] = INVALID;
    if ((attr_index = SDfindattr(sds_id, "Frame shift 1p")) != FAIL)
    {
        if (SDreadattr(sds_id, attr_index, &image.attributes.frame_shift[1]) 
            == FAIL)
        {
            sprintf(emsg,"Could not read Frame shift 1p attribute from %s.",
                    l1r_file_name);
            xxx_LogStatus(progname,__FILE__,__LINE__,emsg);
            SDendaccess(sds_id);
            SDend(sd_id);
            return NULL;
        }
        image.attributes.valid_map[14] = VALID;
    }

    image.attributes.valid_map[15] = INVALID;
    if ((attr_index = SDfindattr(sds_id, "Frame shift 1")) != FAIL)
    {
        if (SDreadattr(sds_id, attr_index, &image.attributes.frame_shift[2]) 
            == FAIL)
        {
            sprintf(emsg,"Could not read Frame shift 1 attribute from %s.",
                    l1r_file_name);
            xxx_LogStatus(progname,__FILE__,__LINE__,emsg);
            SDendaccess(sds_id);
            SDend(sd_id);
            return NULL;
        }
        image.attributes.valid_map[15] = VALID;
    }

    image.attributes.valid_map[16] = INVALID;
    if ((attr_index = SDfindattr(sds_id, "Frame shift 2")) != FAIL)
    {
        if (SDreadattr(sds_id, attr_index, &image.attributes.frame_shift[3]) 
            == FAIL)
        {
            sprintf(emsg,"Could not read Frame shift 2 attribute from %s.",
                    l1r_file_name);
            xxx_LogStatus(progname,__FILE__,__LINE__,emsg);
            SDendaccess(sds_id);
            SDend(sd_id);
            return NULL;
        }
        image.attributes.valid_map[16] = VALID;
    }

    image.attributes.valid_map[17] = INVALID;
    if ((attr_index = SDfindattr(sds_id, "Frame shift 3")) != FAIL)
    {
        if (SDreadattr(sds_id, attr_index, &image.attributes.frame_shift[4]) 
            == FAIL)
        {
            sprintf(emsg,"Could not read Frame shift 3 attribute from %s.",
                    l1r_file_name);
            xxx_LogStatus(progname,__FILE__,__LINE__,emsg);
            SDendaccess(sds_id);
            SDend(sd_id);
            return NULL;
        }
        image.attributes.valid_map[17] = VALID;
    }

    image.attributes.valid_map[18] = INVALID;
    if ((attr_index = SDfindattr(sds_id, "Frame shift 4")) != FAIL)
    {
        if (SDreadattr(sds_id, attr_index, &image.attributes.frame_shift[5]) 
            == FAIL)
        {
            sprintf(emsg,"Could not read Frame shift 4 attribute from %s.",
                    l1r_file_name);
            xxx_LogStatus(progname,__FILE__,__LINE__,emsg);
            SDendaccess(sds_id);
            SDend(sd_id);
            return NULL;
        }
        image.attributes.valid_map[18] = VALID;
    }

    image.attributes.valid_map[19] = INVALID;
    if ((attr_index = SDfindattr(sds_id, "Frame shift 4p")) != FAIL)
    {
        if (SDreadattr(sds_id, attr_index, &image.attributes.frame_shift[6]) 
            == FAIL)
        {
            sprintf(emsg,"Could not read Frame shift 4p attribute from %s.",
                    l1r_file_name);
            xxx_LogStatus(progname,__FILE__,__LINE__,emsg);
            SDendaccess(sds_id);
            SDend(sd_id);
            return NULL;
        }
        image.attributes.valid_map[19] = VALID;
    }
    image.attributes.valid_map[20] = INVALID;
    if ((attr_index = SDfindattr(sds_id, "Frame shift 5p")) != FAIL)
    {
        if (SDreadattr(sds_id, attr_index, &image.attributes.frame_shift[7]) 
            == FAIL)
        {
            sprintf(emsg,"Could not read Frame shift 5p attribute from %s.",
                    l1r_file_name);
            sprintf(emsg,"Could not read attribute.");
            xxx_LogStatus(progname,__FILE__,__LINE__,emsg);
            SDendaccess(sds_id);
            SDend(sd_id);
            return NULL;
        }
        image.attributes.valid_map[20] = VALID;
    }

    image.attributes.valid_map[21] = INVALID;
    if ((attr_index = SDfindattr(sds_id, "Frame shift 5")) != FAIL)
    {
        if (SDreadattr(sds_id, attr_index, &image.attributes.frame_shift[8]) 
            == FAIL)
        {
            sprintf(emsg,"Could not read Frame shift 5 attribute from %s.",
                    l1r_file_name);
            xxx_LogStatus(progname,__FILE__,__LINE__,emsg);
            SDendaccess(sds_id);
            SDend(sd_id);
            return NULL;
        }
        image.attributes.valid_map[21] = VALID;
    }

    image.attributes.valid_map[22] = INVALID;
    if ((attr_index = SDfindattr(sds_id, "Frame shift 7")) != FAIL)
    {
        if (SDreadattr(sds_id, attr_index, &image.attributes.frame_shift[9]) 
            == FAIL)
        {
            sprintf(emsg,"Could not read Frame shift 7 attribute from %s.",
                    l1r_file_name);
            xxx_LogStatus(progname,__FILE__,__LINE__,emsg);
            SDendaccess(sds_id);
            SDend(sd_id);
            return NULL;
        }
        image.attributes.valid_map[22] = VALID;
    }

    /* verify that the number of along track pixels and frame shift attributes
       have been read since they are required to read the image data */
    if (band == pan)
        frame_shift_index = 13;
    else
        frame_shift_index = 14 + axx_get_user_band(band) - 2;
    if ((image.attributes.valid_map[10] != VALID) 
        || (image.attributes.valid_map[frame_shift_index] != VALID))
    {
        if (image.attributes.valid_map[10] != VALID) 
        {
            sprintf(emsg, "Error: along track pixel attribute missing from %s",
                    l1r_file_name);
        }
        else
            sprintf(emsg, "Error: frame shift missing from %s", l1r_file_name);

        xxx_LogStatus(progname,__FILE__,__LINE__,emsg);
        SDendaccess(sds_id);
        SDend(sd_id);
        return NULL;
    }

    /* now that all the attributes have been successfully read from the SDS, 
       allocate memory for the structure to return and copy the local 
       structure to it */
    image_ptr = malloc(sizeof(*image_ptr));
    if (!image_ptr)
    {
        xxx_LogStatus(progname,__FILE__,__LINE__,"Error allocating memory");
        SDendaccess(sds_id);
        SDend(sd_id);
        return NULL;
    }
    memcpy(image_ptr,&image,sizeof(*image_ptr));

    /* set the file name in the structure (so it can be used when printing
       out error messages later if needed) */
    strncpy(image_ptr->filename, l1r_file_name, sizeof(image_ptr->filename));
    image_ptr->filename[sizeof(image_ptr->filename) - 1] = '\0';

    /* set the HDF entries into the structure */
    image_ptr->sd_id = sd_id;
    image_ptr->sds_id = sds_id;
    image_ptr->ns = image_ptr->dim_sizes[2];
    image_ptr->nl = image_ptr->attributes.number_along_track_pixels;
    image_ptr->band = band;
    if (band == pan)
    {
        /* for pan band, reading the first band in the SDS */
        image_ptr->band_index = 0;
        image_ptr->frame_shift = image_ptr->attributes.frame_shift[0];
    }
    else
    {
        /* convert the passed in band to an index ("- 2" since the first 
           multispectral band is number 2 and we need them 0-relative for 
           calculating the offset in the file */
        int band_index = axx_get_user_band(band) - 2;

        image_ptr->band_index = band_index;
        image_ptr->frame_shift = 
                image_ptr->attributes.frame_shift[band_index+1];
    }

    return image_ptr;
}
