/******************************************************************************
NAME:		axx_write_1g_image

PURPOSE:	Write to the specified 1G image.

RETURN VALUE: 
Type = int
Value           Description
-----           -----------
SUCCESS     	Successfully wrote image
ERROR     	Error occurred; image not written

NOTES:		The 1G image is a VGroup containing a SDS for each band.  The 
		data for each SDS are stored in an external element.
                The subsampling factor if set to 0 or 1 will result in writing
                the full resolution data.  Set to n will result in writing to
		every n'th data point in both dimensions, starting with sl, ss.
		(The data are not subsampled from the input array "data".)
		**This routine assumes the band is a 2-dimensional SDS.  If it 
		is of higher dimension, results will be undefined.


******************************************************************************/
#include <stdio.h>
#include "xxx_Types.h"
#include "xxx_LogStatus.h"
#include "axx_hdf_io.h"

int axx_write_1g_image (
	const HDFIO *hdfid,	/* I-HDF IO structure			*/
	int sca,
        int nband,
        BandNumber band,	/* I-ETM+ (enumerated) band number to write */
        int  sl,                /* I-Line to start writing (0-rel)	*/
        int  ss,                /* I-Sample to start writing (0-rel)	*/
        int  nl,                /* I-Number of lines to write   	*/
        int  ns,                /* I-Number of samples to write 	*/
        int  subsamp,           /* I-Subsampling factor         	*/
        void *data)             /* O-Data buffer			*/
{
intn	hdf_status;		/* HDF function return status	*/
int32	start[2],		/* starting data location	*/
	stride[2],		/* subsampling factor		*/
	edge[2];		/* size of data buffer		*/
char	msg[STRLEN];		/* Error message string		*/
int	max_bands = NBANDS;

/*
 * Set up the structure
 */
if (hdfid == NULL)
    {
    sprintf(msg,"Error writing to 1G image");
    xxx_LogStatus("AXXLIB", __FILE__, __LINE__, msg);
    return(ERROR);
    }

/*
 * Set up the specification for the block to write
 */
start[0] = sl; /* start is 0-relative */
start[1] = ss;
edge[0] = nl;
edge[1] = ns;

if ((subsamp == 0) || (subsamp == 1))
    {
    hdf_status = SDwritedata(hdfid->sds_id[sca*max_bands+(int)band], start, NULL, edge, 
		 (VOIDP)data);
    }
else
    {
    stride[0] = subsamp;
    stride[1] = subsamp;

    hdf_status = SDwritedata(hdfid->sds_id[sca*max_bands+(int)band], start, stride, edge, 
		 (VOIDP)data);
    }

if (hdf_status == FAIL)
   {
   sprintf(msg, "Error writing to SDS %s",hdfid->sds_name[sca*max_bands+(int)band]);
   xxx_LogStatus("AXXLIB", __FILE__, __LINE__, msg);
   return(ERROR);
   }

return(SUCCESS);
}
