/*  axx_allocate_cal_info():  routine to dynamically allocate necessary
                              storage for all members in a band's CAL_INFO
			      structure.  */

#include <stdio.h>
#include <stdlib.h>

#include "axx_rps.h"                  /* Generic ALI RPS header */
#include "xxx_Types.h"                /* SUCCESS & ERROR defs */

int axx_allocate_cal_info(
CAL_INFO *cal_info,                   /* O:  Structure to contain radiometric
					 data  */
int32 ndets,                          /* I:  Number of detectors in a band */
int32 band,                           /* I:  Current 0-indexed band number */
int16 skip_length)                    /* I:  Number of elements to skip to
					 get to consecutive data values
					 (1 for MS bands, 3 for pan band)  */
{
  int32 num_elements;            /* local variable to hold total number of
				     elements for each structure */
  

  /* Copy the necessary size information into cal_info.
   ---------------------------------------------------------*/
  cal_info->band = band;
  cal_info->ndets = ndets;

 

  /* Before we can allocate necessary storage, we have to account for the fact
     that the MS bands and pan band don't store data the same way.  Consecutive
     data values are spaced 1 frame apart for the MS bands, and 3 frames
     apart for the pan band.  To that end, a parameter 'skip_length'
     has been passed in, whose value was set prior to entering this routine;
     the value depends on which band we're dealing with.  For MS bands, it's
     been set to 1, and for the pan band, it's been set to 3.  */

  /* Allocate the IC lamp buffers first--it's generally got the most
     memory needs.  There will only be one set of lamp state measurements
     per detector--we don't need to consider skip length yet.
  -------------------------------------------------------------------*/
  num_elements = NUM_LAMP_STATES*cal_info->ndets;

  cal_info->ic_lamp.det_mean = (float64*)calloc(num_elements, sizeof(float64));
  cal_info->ic_lamp.det_stddev = (float64*)calloc(num_elements, 
						  sizeof(float64));
  cal_info->ic_lamp.valid_frames = (int32*)calloc(num_elements, sizeof(int32));
  cal_info->ic_lamp.band_average = (float64*)calloc(NUM_LAMP_STATES,
						    sizeof(float64));
   
  if((cal_info->ic_lamp.det_mean == NULL) ||
     (cal_info->ic_lamp.det_stddev == NULL) ||
     (cal_info->ic_lamp.valid_frames == NULL) ||
     (cal_info->ic_lamp.band_average == NULL))
    {
      
      return ERROR;
    }

  

  /* Allocate the bias buffers.  Here we need to consider the skip length.
   -----------------------------------------------------------------------*/
  num_elements = skip_length*cal_info->ndets;

  cal_info->precal.mean = (float64*)calloc(num_elements, sizeof(float64));
  cal_info->precal.stddev = (float64*)calloc(num_elements, sizeof(float64));
  cal_info->precal.valid_frames = (int32*)calloc(num_elements, sizeof(int32));
  cal_info->dark.mean   = (float64*)calloc(num_elements, sizeof(float64));
  cal_info->dark.stddev = (float64*)calloc(num_elements, sizeof(float64));
  cal_info->dark.valid_frames   = (int32*)calloc(num_elements, sizeof(int32));

  if((cal_info->precal.mean == NULL) ||(cal_info->precal.stddev == NULL) ||
     (cal_info->dark.mean == NULL) || (cal_info->dark.stddev == NULL) ||
     (cal_info->precal.valid_frames == NULL) ||
     (cal_info->dark.valid_frames == NULL))   
    {
      free(cal_info->ic_lamp.det_mean);
      free(cal_info->ic_lamp.det_stddev);
      free(cal_info->ic_lamp.valid_frames);
      free(cal_info->ic_lamp.band_average);

      return ERROR;
    }
     
 
  /* ########################################################################*/

  /* Allocate the relative gain buffers.  We don't need to deal with
     skip length for these--there will be only one gain value per detector,
     even in the pan band. 
  ------------------------------------------------------------------*/
  cal_info->gain.p1 = (float64*)calloc(cal_info->ndets, sizeof(float64));
  cal_info->gain.p2 = (float64*)calloc(cal_info->ndets, sizeof(float64));
  cal_info->gain.p1_stderr = (float64*)calloc(cal_info->ndets,
					      sizeof(float64));
  cal_info->gain.p2_stderr = (float64*)calloc(cal_info->ndets,
					      sizeof(float64));

  cal_info->gain.cal_method = (char*)malloc(20*sizeof(char));

  if((cal_info->gain.p1 == NULL) || (cal_info->gain.p2 == NULL) ||
     (cal_info->gain.p1_stderr == NULL) || (cal_info->gain.p2_stderr == NULL)
     || (cal_info->gain.cal_method == NULL))
    {
      free(cal_info->ic_lamp.det_mean);
      free(cal_info->ic_lamp.det_stddev);
      free(cal_info->ic_lamp.valid_frames);
      free(cal_info->ic_lamp.band_average);
      free(cal_info->precal.mean);
      free(cal_info->precal.stddev);
      free(cal_info->precal.valid_frames);
      free(cal_info->dark.mean);
      free(cal_info->dark.stddev);
      free(cal_info->dark.valid_frames);

      return ERROR;
    }
  
 

  /*#########################################################################*/
   	
  /* Allocate the histogram buffers.  Again, there will only be one value
     of max, min, mean, std.dev and number of frames per detector.
   -------------------------------------------------------------------------*/
  cal_info->hist.meanDN   = (float64*)calloc(cal_info->ndets, sizeof(float64));
  cal_info->hist.stddev = (float64*)calloc(cal_info->ndets, sizeof(float64));
  cal_info->hist.valid_frames = (int32*)calloc(cal_info->ndets, sizeof(int32));
  cal_info->hist.maxDN    = (int16*)calloc(cal_info->ndets, sizeof(int16));
  cal_info->hist.minDN    = (int16*)calloc(cal_info->ndets, sizeof(int16));
  
  if((cal_info->hist.meanDN == NULL) || (cal_info->hist.stddev == NULL) ||
     (cal_info->hist.valid_frames == NULL) || (cal_info->hist.maxDN == NULL) ||
     (cal_info->hist.minDN == NULL))
    {
      free(cal_info->ic_lamp.det_mean);
      free(cal_info->ic_lamp.det_stddev);
      free(cal_info->ic_lamp.valid_frames);
      free(cal_info->ic_lamp.band_average);
      free(cal_info->precal.mean);
      free(cal_info->precal.stddev);
      free(cal_info->precal.valid_frames);
      free(cal_info->dark.mean);
      free(cal_info->dark.stddev);
      free(cal_info->dark.valid_frames);
      free(cal_info->gain.p1);
      free(cal_info->gain.p2);
      free(cal_info->gain.p1_stderr);
      free(cal_info->gain.p2_stderr);
      free(cal_info->gain.cal_method);

      return ERROR;
    }


  /*#######################################################################*/

  /* YEAH!!  We got everything allocated for a band... Go on from here */
  return SUCCESS;
	
}



