/* axx_calc_ICRDG():  routine to compute the relative gain
                      (mean and standard deviation) for each detector,
		      using IC lamp state data.  Differences in the
		      way the MS bands and pan bands store data has
		      been explicitly taken into account.

   The computation is performed as follows...

   1.  A line of lamp state data is "convolved" with a decreasing step filter
       of length P and unit magnitude, to detect the lamp state intensity
       changes:

       h = [... 1 1 0 -1 -1 ...]

       Since the filter coefficients are ones, the convolution process
       reduces to a simple series of additions and subtractions.


   2.  Using the convolved data, a threshold value is determined according
       to 
           T1 = m1 + k*s1, where m1 is the mean of the convolved data,
	                   s1 is the standard deviation, and k is a constant.

       The convolved data is then ranked according to whether it exceeds T1
       (rank=1), or does not (rank=0).  This is done to isolate the
       convolution peaks.


   3.  The ranks are searched to locate the peaks, in order to determine
       appropriate starting and stopping frames over which to compute the
       lamp state statistics.


   4.  Compute the lamp state statistics for EACH detector.

   5.  Compute a band-averaged gain for EACH lamp state.

   6.  Compute the gain and standard error (for the slope coefficient) of
       the linear model y = ax + b.  In this situation, y is the
       band-averaged lamp state response , and x is the mean lamp state
       response for an individual detector (corrected for bias).  If this
       were plotted, there would be
      
       cal_info[band].gain.ndets plots of four points

       Those four points are the band-averaged response of each
       lamp state.  As our understanding of the response characterization
       improves, we may need to switch to a quadratic (or higher-order)
       model.  Of course, going to a higher-order model makes the
       computations much messier!!  */
  

#include <stdio.h>
#include <stdlib.h>

#include "xxx_ali_level0r.h"
#include "xxx_Types.h"
#include "xxx_LogStatus.h"
#include "axx_rps.h"
#include "mysql.h"

#define   FILTER_LENGTH 11  /* Length of convolution filter */

int axx_calc_ICRDG(
char      basename[],    /* I:  "Name" of EO1 scene */
LEVEL0R   *level0r,      /* I:  Structure containing lamp data */
CAL_INFO  *cal_info,     /* O:  Structure to contain calibration data */
int32     sca,           /* I:  Current SCA number */
int32     band,          /* I:  0-referenced index to band number */
int16     skip_length,   /* I:  Number of frames to skip to reach consecutive
			        data frames in the bias data (1 for MS bands,
				3 for pan band)   */
int       trend_on,      /* I:  "Flag" indicating whether gain data should
			        be written to the database  */
char      database[])    /* I:  Name of trending database  */

{

  MYSQL     *mysql;             /* MYSQL data structure */
  int       id;                /* Scene ID number */
  char      sql_query[1024];   /* Query string */
  char      bandString[6];     /* String containing ALI band name */

  int16  *pLineBuff;     /* 1D array to hold detector lamp data */
  
  int16  *pOutBuff;      /* 1D array to hold convolution output for a
			    detector */
  int16  *pRankBuff;     /* 1D array to hold "ranks" */
  
  int16  *pLampData;     /* array of all lamp data for a given band */
  
  int32  pBounds[2*NUM_LAMP_STATES];  /* array to hold starting and stopping
					 frame locations */
  char   msg[STRLEN];
  float64 t1;            /* peak threshold value */

  int32  filterLength;   /* length of convolution "filter" */
  int32  length;         /* length */
  int32  frame;          /* counters */
  int32  det;
  int32  loc;            /* current pixel location in pLampData
			    (row, col) */
  int32  index;          /* index into ic_lamp member arrays */
  int32  state;          /* current lamp state */
  int32  num_frames;     /* total number of frames */
  
  int16  numPeaks;       /* number of detected "convolution" peaks */
  int16  skip;           /* current frame that's been skipped to+ */


  if(level0r->lamp[band].number_of_dims != 3)
   {
     sprintf(msg, "Lamp data doesn't exist in Level 0R dataset, not performing Calc_ICRDG.");
     xxx_LogStatus("rps", __FILE__, __LINE__, msg);
     return(ERROR);
   }

    
  /* Preliminary initializations. 
     -----------------------------------*/
  num_frames = level0r->lamp[band].dim_sizes[N_FRAMES_IDX];
  sprintf(cal_info->gain.cal_method, "IC");


  /* Make the ALI band name from the current band number
     ---------------------------------------------------------*/
  axx_MakeALIBandString(band, bandString);



  /* Allocate the working buffers. 
     ---------------------------------------*/ 
  pLineBuff = (int16*)calloc(num_frames, sizeof(int16));
  pRankBuff = (int16*)calloc(num_frames, sizeof(int16));
  pOutBuff  = (int16*)calloc(num_frames, sizeof(int16));
  
  if((pLineBuff == NULL) || (pRankBuff == NULL) || (pOutBuff == NULL))
    {
      sprintf(msg, "Error allocating working buffers, aborting.");
      xxx_LogStatus("rps", __FILE__, __LINE__, msg);
      exit(ERROR);
    }
  
  
  /* Set the base address for the 2D array of lamp data.  Since this is a
     pointer, it might be that it has to be "referenced" by an '->'
     rather than a '.'.  A segmentation fault/core dump should confirm
     this possibility. 
     -----------------------------------------------------------------------*/
  /* NOTE:  THE DOT WORKS!!!
     ----------------------------*/
  pLampData = level0r->lamp[band].data;



  /* Copy a detector's data to the line buffer.  In this same interval,
     reinitialize the rank buffer. 
     ----------------------------------------------------------------------*/
  for(det = 0; det < cal_info->ndets; det++)
    {
      for(frame = 0; frame < num_frames; frame++)
	{
	  loc = det + frame*cal_info->ndets;
	  pLineBuff[frame] = pLampData[loc];

	  if(pRankBuff[frame] != 0)
	    pRankBuff[frame] = 0;
	}

      /* Convolve the filter and the lamp detector data.
	 ----------------------------------------------------*/
      filterLength = FILTER_LENGTH;
      axx_convolve_detector_data(pLineBuff, num_frames,
				 filterLength, pOutBuff);

      /* Compute threshold T1 and  set up the rank buffer.
	 -----------------------------------------------------*/
      length = num_frames - filterLength - 10;
      axx_set_ranks(band, length, pOutBuff, pRankBuff, &t1);

       
      /* Determine appropriate starting and stopping frames for
	 computing lamp state statistics.   Technically, the parameter
	 'det' isn't necessary to do this--it's included simply to
	 provide information whenever we can't find a set of
	 lamp state transitions. 
	 ---------------------------------------------------------------*/
      if(axx_find_state_bounds(band, det, length, pOutBuff, pRankBuff,
			       t1, pBounds, &numPeaks) != SUCCESS)
	{
	  sprintf(msg, "Error determining lamp state bounds");
	  sprintf(msg, "%s for sca %d band %s detector %d.", msg, sca,
		  bandString, det+1);
	  xxx_LogStatus("rps", __FILE__, __LINE__, msg);
	}

      /* Determine the mean detector response for each lamp state. First,
	 however, run checks to make sure that the total number of lamp
	 state transitions that were detected is exactly 3 (corresponding
	 to the fact there are 4 lamp states)
	 ---------------------------------------------------------------*/
      if(numPeaks == 3)
	{
	  for(state = 0; state < NUM_LAMP_STATES; state++)
	    {
	      index = det + state*cal_info->ndets;
	      axx_do_lamp_stats(&cal_info->dark, pLineBuff, pBounds, det,
				   cal_info->ndets, state, skip_length,
				   &(cal_info->ic_lamp.det_mean[index]),
				   &(cal_info->ic_lamp.det_stddev[index]),
				   &(cal_info->ic_lamp.valid_frames[index]));
	    }
	}
      else
	{
	  sprintf(msg, "Invalid number of detected peaks for band %s,",
		  bandString);
	  sprintf(msg, "%s detector %d.", msg, det+1);
	  xxx_LogStatus("rps", __FILE__, __LINE__, msg);
	}
    }


  /* Compute band-averaged lamp states, then compute the gain
     ---------------------------------------------------------------*/
  if(axx_do_lamp_gains(cal_info, band) != SUCCESS)
    {
      sprintf(msg, "Error computing relative detector gain for band %s",
	      bandString);
      sprintf(msg, "%s detector %d.", msg, det+1);
      xxx_LogStatus("rps", __FILE__, __LINE__, msg);
      return ERROR;
    }
  
  /* We don't need these buffers anymore, so free up the memory
     --------------------------------------------------------------------*/
  free(pLineBuff);
  free(pOutBuff);
  free(pRankBuff);


  /**************************************************************************/


  /* If we're trending, write the gain and lampstate data to the database.
     ----------------------------------------------------------------*/
  if(trend_on)
    {

      mysql = mysql_init(NULL);
      /* Step 1:  Get the scene ID given the EO1 scene name.
	 -------------------------------------------------------*/
      if(xxx_GetSceneID(database, basename, mysql, &id) != SUCCESS)
	{
	  sprintf(msg, "Error retrieving ID of scene %s.", basename);
	  xxx_LogStatus("rps", __FILE__, __LINE__, msg);
	  return ERROR;
	}
      
      /* Step 2: Write the data to the database.
         ---------------------------------------------------*/
      for(det = 0; det < cal_info->ndets; det++)
	{        
	  sprintf(sql_query, "insert into rad_gain_det values");
	  sprintf(sql_query, "%s(%d, %d, '%s', %d, '%s', %f, %f)",
		  sql_query, id, sca, bandString, det+1,
		  cal_info->gain.cal_method, cal_info->gain.p1[det],
		  cal_info->gain.p1_stderr[det]);
		  
	  if(mysql_real_query(mysql, sql_query, strlen(sql_query)))
	    {
	      sprintf(msg, "IC-based gain record insert failed");
	      sprintf(msg, "%s for detector %d, Error: %s",
		      msg, det+1, mysql_error(mysql));
	      xxx_LogStatus("rps", __FILE__, __LINE__, msg);
	    }
	  
	  skip = 0;
	  for(state = 0; state < NUM_LAMP_STATES; state++)
	    {
	      index = det + state*cal_info->ndets;
	      sprintf(sql_query, "insert into rad_lampstate_det values");
	      sprintf(sql_query, "%s(%d, %d, '%s', %d, %d, %d, %d, %f, %f)",
		      sql_query, id, sca, bandString, skip, det+1,
		      state+1, cal_info->ic_lamp.valid_frames[index],
		      cal_info->ic_lamp.det_mean[index],
			  cal_info->ic_lamp.det_stddev[index]);
		  
	      if(mysql_real_query(mysql, sql_query, strlen(sql_query)))
		{
		  sprintf(msg, "Lamp state record insert failed for");
		  sprintf(msg, "%s detector %d, Error: %s",
			  msg, det+1, mysql_error(mysql));
		  xxx_LogStatus("rps", __FILE__, __LINE__, msg);
		}
	    }
	}
      
      /* Step 4:  Close the database. 
         ------------------------------------*/
      mysql_close(mysql);
    }
  
  
  return SUCCESS;
}



