/* axx_calc_bias():      routine to compute mean offsets and standard 
                         deviations for each detector using both the pre-cal
                         and dark images.  In both cases, the shutter door
                         is closed.  */

#include <stdio.h>

#include "xxx_ali_level0r.h"
#include "xxx_Types.h"
#include "xxx_LogStatus.h"
#include "axx_rps.h"
#include "mysql.h"

int axx_calc_bias(
char      basename[],           /* I:  "Name" of EO1 scene */
LEVEL0R   *level0r,             /* I: Structure containing level 0R data */
CAL_INFO  *cal_info,            /* O: Structure to contain bias information */
int32     sca,                  /* I: Current SCA number */
int32     band,                 /* I: 0-indexed current band */
int16     skip_length,          /* I: Number of frames to skip to reach
                                      consecutive data frames  (1 for MS bands,
                                      3 for pan bands)*/
int       trend_on,             /* I: "Flag" indicating whether gain data should
                                      be written to the database  */
char      database[])           /* I: Name of trending database  */

{
  
  int16	skip;             /* current frame that's been skipped to+ */
  int32	det;              /* current detector number (0-index) */
  int32	index;            /* index into ic_lamp member arrays */
  char	sql_query[1024];  /* Query string */
  int	id;               /* Scene ID number */
  MYSQL	*mysql;           /* MYSQL data structure */
  char	bandString[6];    /* String containing ALI band name */
  char	msg[STRLEN];      /* Error message string */

  /* Note to self:
   * ...dim_sizes[0] -- number of bands
   * ...dim_sizes[1] -- number of frames
   * ...dim_sizes[2] -- number of detectors

 
     0 -> N_BANDS_IDX
     1 -> N_FRAMES_IDX
     2 -> N_DETS_IDX
  */


  /* Compute the mean pre-cal bias level before the scene has been acquired.
     Check to make sure we're not dealing with the pan band, though.  The
     input parameter 'skip_length' makes sure we've accounted for the pan
     band having consecutive data 3 frames apart and the MS bands having
     consecutive data 1 frame apart.
  ----------------------------------------*/
 
  axx_vert_moment(level0r->precal[band].data,
		  level0r->precal[band].dim_sizes[N_DETS_IDX],
		  level0r->precal[band].dim_sizes[N_FRAMES_IDX],
		  skip_length,
		  cal_info->precal.mean,
		  cal_info->precal.stddev,
		  cal_info->precal.valid_frames);

  /* Compute the mean bias level after the scene has been acquired.
   ------------------------------------------------------------------*/
  axx_vert_moment(level0r->dark[band].data,
		  level0r->dark[band].dim_sizes[N_DETS_IDX],
		  level0r->dark[band].dim_sizes[N_FRAMES_IDX],
		  skip_length,
		  cal_info->dark.mean,
		  cal_info->dark.stddev,
		  cal_info->dark.valid_frames);

  /* If we're trending, write the gain and lampstate data to the database.
     ----------------------------------------------------------------*/
  if(trend_on)
    {

      mysql = mysql_init(NULL);
      /* Step 1:  Get the scene ID given the EO1 scene name.
         -------------------------------------------------------*/
      if(xxx_GetSceneID(database, basename, mysql, &id) != SUCCESS)
        {
          sprintf(msg, "Error retrieving ID of scene %s.", basename);
          xxx_LogStatus("rps", __FILE__, __LINE__, msg);
          return ERROR;
        }

      /* Make the ALI band name from the current band number
         ---------------------------------------------------------*/
      axx_MakeALIBandString(band, bandString);

      /* Step 2: Write the data to the database.
         ---------------------------------------------------*/
      for(det = 0; det < cal_info->ndets; det++)
        {
          for(skip = 0; skip < skip_length; skip++)
            {
              index = det + skip*cal_info->ndets;

              /* Put in the precal offset
                 ---------------------------------*/
              sprintf(sql_query, "insert into rad_lampstate_det values");
              sprintf(sql_query, "%s(%d, %d, '%s', %d, %d, %d, %d, %f, %f)",
                      sql_query, id, sca, bandString, skip, det+1,
                      PRECAL_OFFSET,
                      cal_info->precal.valid_frames[index],
                      cal_info->precal.mean[index],
                      cal_info->precal.stddev[index]);
              
              if(mysql_real_query(mysql, sql_query, strlen(sql_query)))
                {
                  sprintf(msg, "Precal bias record insert failed");
                  sprintf(msg, "%s for detector %d, Error: %s",
                          msg, det+1, mysql_error(mysql));
                  xxx_LogStatus("rps", __FILE__, __LINE__, msg);
                }
              
              /* Put in the dark offset
               ------------------------------------*/
              sprintf(sql_query, "insert into rad_lampstate_det values");
              sprintf(sql_query, "%s(%d, %d, '%s', %d, %d, %d, %d, %f, %f)",
                      sql_query, id, sca, bandString, skip, det+1,
                      DARK_OFFSET,
                      cal_info->dark.valid_frames[index],
                      cal_info->dark.mean[index],
                      cal_info->dark.stddev[index]);
              
              if(mysql_real_query(mysql, sql_query, strlen(sql_query)))
                {
                  sprintf(msg, "Dark bias record insert failed");
                  sprintf(msg, "%s for detector %d, Error: %s",
                          msg, det+1, mysql_error(mysql));
                  xxx_LogStatus("rps", __FILE__, __LINE__, msg);
                }
            }
        }
    }

  /* We're finished.
   -----------------*/
  sprintf(msg, "Mean precal and dark biases calculated.");
  xxx_LogStatus("rps", __FILE__, __LINE__, msg);
  return SUCCESS;
}

