/* axx_calc_hist():    routine to obtain histogram-related information
                       from an ALI scene for each detector, as well as
		       a scene-specific band average histogram.  */

#include <stdio.h>
#include <math.h>

#include "xxx_Types.h"
#include "xxx_LogStatus.h"
#include "axx_rps.h"
#include "mysql.h"

int axx_calc_hist(
char              basename[],     /* I:  Name of EO1 scene */
LEVEL0R           *level0r,       /* I: Structure containing image data */
SCENE_HIST_STATS  *hs,            /* 0: Structure to contain the histogram
				     data */
int32             sca,            /* I: Current SCA number */
int32             band,           /* I: Current 0-indexed band number */
int32             num_dets,       /* I: Total number of detectors */
const int         src,            /* I: Current data source */
int               trend_on,       /* I: Flag indicating whether we're
				        trending data or not */
char              database[])     /* I: Name of trending database */
{

  MYSQL      *mysql;               /* MYSQL data structure */
  char       sql_query[1024];     /* Query string */
  int        id;                  /* Returned scene ID number */

  int16    *pData;         /* Pointer to array of source data */
  int32    loc;            /* Current location within image data array */
  int32    det;            /* Current detector */
  int32    frame;          /* Current frame */
  int32    num_frames;     /* Total number of frames */
  int32    hist_index;     /* Index into histogram data arrays */
  char     msg[STRLEN];
  char     bandString[6];  /* Name of ALI band */

  int16    skip;           /* Current number of frames to skip when
			      obtaining consecutive data */


  /* Set things up according to the data source we want to histogram.
   ---------------------------------------------------------------------*/
  switch(src)
    {
      /* Right now, this version of the routine can find histograms
	 for the 4 data sources contained in a Level 0R scene. */

    case PRECAL:
      {
	pData = level0r->precal[band].data;
	num_frames = level0r->precal[band].dim_sizes[N_FRAMES_IDX];
	break;
      }
    case IMAGE:
      {
	pData = level0r->image[band].data;
	num_frames = level0r->image[band].dim_sizes[N_FRAMES_IDX];
	break;
      }
    case DARK:
      {
	pData = level0r->dark[band].data;
	num_frames = level0r->dark[band].dim_sizes[N_FRAMES_IDX];
	break;
      }
    case LAMP:
      {
	pData = level0r->lamp[band].data;
	num_frames = level0r->lamp[band].dim_sizes[N_FRAMES_IDX];
	break;
      }
    }

  
  /* Do some quick initializations, just in case they don't get initialized
     to zero when allocated
  -------------------------------*/
  hs->band_valid_frames = 0;
  hs->band_meanDN = 0.0;
  hs->band_stddev = 0.0;


  for(det = 0; det < num_dets; det++)
    {

      /* Find the minimum and maximum DN for each detector.
	 ---------------------------------------------------------------*/
      axx_find_minmax(det, num_dets, num_frames, pData, &hs->maxDN[det],
		      &hs->minDN[det]);


      /* Now compute the mean DN value, standard deviation, and number
	 of valid frames for each detector.
	 -----------------------------------------------------------------*/
      for(frame = 0; frame < num_frames; frame++)
	{
	  /* Determine the proper index into the scene data array
	     ---------------------------------------------------------*/
	  loc = det + frame*num_dets;
	  
	  /* Compute the mean DN value, standard deviation, and valid
	     number of frames in for each detector.
	     -----------------------------------------------------------*/
	  if((pData[loc] & FILL_DATA) == 0)
	    {
	      hs->meanDN[det] += (float64)pData[loc];
	      hs->stddev[det] += (float64)(pData[loc]*pData[loc]);
	      (hs->valid_frames[det])++;
	    }
	}

	  /* Now update the band-averaged stuff from each detector.
	   -------------------------------------------------------------*/
	  hs->band_valid_frames += hs->valid_frames[det];
	  hs->band_meanDN       += hs->meanDN[det];
	  hs->band_stddev       += hs->stddev[det];

	  /* Now update the detector-specific data.
	   --------------------------------------------*/
	  hs->meanDN[det] /= hs->valid_frames[det];
	  hs->stddev[det] -= 
	    (hs->valid_frames[det])*(hs->meanDN[det]*hs->meanDN[det]);
	  hs->stddev[det] /= (hs->valid_frames[det] - 1);
	  hs->stddev[det] = sqrt(hs->stddev[det]);
    }
  
  /* Compute the mean and standard deviation in response for the band.
     However, do this only for the image data at this time.
   ----------------------------------------------------------------------*/
  if(src == IMAGE)
    {
      hs->band_meanDN /= hs->band_valid_frames;
      hs->band_stddev -= (hs->band_valid_frames)*(hs->band_meanDN*hs->band_meanDN);
      hs->band_stddev /= (hs->band_valid_frames - 1);
      hs->band_stddev = sqrt(hs->band_stddev);
    }


  /**************************************************************************/


  /* If we're trending, write the histogram data to the database.
     ----------------------------------------------------------------*/
  if(trend_on)
    {
      /* Make the ALI band name from the current band number
       ---------------------------------------------------------*/
      axx_MakeALIBandString(band, bandString);
      
      
      mysql = mysql_init(NULL);
      /* Step 1:  Get the scene ID given the EO1 scene name.
	 -------------------------------------------------------*/
      if(xxx_GetSceneID(database, basename, mysql, &id) != SUCCESS)
	{
	  sprintf(msg, "Error retrieving ID of scene %s.", basename);
	  xxx_LogStatus("rps", __FILE__, __LINE__, msg);
	  return ERROR;
	}
      

      /* Step 2: Write the data to the database.
         ---------------------------------------------------*/
      for(det = 0; det < num_dets; det++)
	{     
	  sprintf(sql_query, "insert into rad_hist_det values");
	  sprintf(sql_query, "%s(%d, %d, '%s', %d, %d, %d, %d, %f, %f)",
		  sql_query, id, sca, bandString, det+1,
		  hs->maxDN[det],
		  hs->minDN[det],
		  hs->valid_frames[det],
		  hs->meanDN[det],
		  hs->stddev[det]);

	  if(mysql_real_query(mysql, sql_query, strlen(sql_query)))
	    {
	      sprintf(msg, "Histogram record insert failed");
	      sprintf(msg, "%s for detector %d, Error: %s",
		      msg, det+1, mysql_error(mysql));
	      xxx_LogStatus("rps", __FILE__, __LINE__, msg);
	    }
	}


      /* Step 3:  Close the database. 
         ------------------------------------*/
      mysql_close(mysql);

    }

  return SUCCESS;

}





