/* axx_char_det_sat():        routine to estimate detector saturation levels
                              and test scenes to find saturated pixels.
			      The saturation type, affected detector and
			      frame numbers, the actual DN value and the
			      estimated saturation value are recorded in a
			      text report that will probably be deleted when
			      processing is completed.

                              Most of the code in this routine is
                              "housekeeping" in nature, ie, setting up
                              buffers, deciding which data is being worked
                              with, etc.*/


#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>

#include "xxx_Types.h"
#include "xxx_LogStatus.h"
#include "axx_rps.h"
#include "mysql.h"

int axx_char_det_sat(
char                  basename[],        /* I: Name of EO1 scene */
LEVEL0R               *level0r,          /* I: Structure containing scene
					       data */
CAL_INFO              *cal_info,         /* I: Structure containing scene
					       histogram/bias data  */
int32                 sca,               /* I: Current SCA */
int32                 band,              /* I: Current 1-indexed band */
const int             bias_src,          /* I: Kind of bias data to work
					       with (precal or dark)  */
SCENE_ARTIFACT_STATS  *image_ar,         /* O: Structure to contain saturation
					       artifact data */
int                   trend_on,          /* I: Flag to indicate whether to
					       write saturation data to the
					       database  */
char                  database[])        /* I: Name of database we're using
					       for trending  */

{

  SCENE_HIST_STATS    bias_hs;           /* Structure to contain bias
					    histogram data */
  FILE   *report;
  char   reportName[1024];

  int32  *high_det_sat;
  int32  *low_det_sat;
  char   msg[STRLEN];                    /* message buffer */
  char   bandString[6];                  /* String to represent "true" band
					    designation*/
  int32  image_frames;                   /* number of frames of image data
					    per detector */
  int32  bias_frames;                    /* number of frames of bias data
					    per detector */
  int32  det;                            /* current detector */
  int32  frame;                          /* current frame */
  int32  data_index;                     /* index into image data arrays */
  int16  *pImageData;                    /* pointer to image data */
  int16  *pBiasData;                     /* pointer to bias data */

  MYSQL      *mysql;                      /* MYSQL data structure */
  char       sql_query[1024];            /* Query string */
  int        id;                         /* Scene ID number */

  /* Preliminary initializations.
   -------------------------------------------------------*/
  image_frames = level0r->image[band].dim_sizes[N_FRAMES_IDX];
  pImageData   = level0r->image[band].data;
  axx_MakeALIBandString(band, bandString);


  /* Create the report file name, then create the file.
     ------------------------------------------------------*/
  sprintf(reportName, "%s_SCA%d%s_saturation.txt", basename, sca,
	  bandString);
  if((report = fopen(reportName, "wt")) == NULL)
    {
      sprintf(msg, "Error creating report file %s.", reportName);
      xxx_LogStatus("rps", __FILE__, __LINE__, msg);
      exit(ERROR);
    }

  

  /* Allocate the bias histogram structure.
   --------------------------------------------*/
  bias_hs.meanDN = (float64*)calloc(cal_info->ndets, sizeof(float64));
  bias_hs.stddev = (float64*)calloc(cal_info->ndets, sizeof(float64));
  bias_hs.valid_frames = (int32*)calloc(cal_info->ndets, sizeof(int32));
  bias_hs.maxDN  = (int16*)calloc(cal_info->ndets, sizeof(int16));
  bias_hs.minDN  = (int16*)calloc(cal_info->ndets, sizeof(int16));

  if((bias_hs.meanDN == NULL) || (bias_hs.stddev == NULL) ||
     (bias_hs.valid_frames == NULL) || (bias_hs.maxDN == NULL) ||
     (bias_hs.minDN == NULL))
    {
      sprintf(msg, "Error allocating bias histogram structure, aborting.");
      xxx_LogStatus("rps", __FILE__, __LINE__, msg);
      fclose(report);
      unlink(reportName);
      exit(ERROR);
    }

  /* Allocate the image artifact structure. 
     --------------------------------------------*/
  image_ar->value = (float64*)calloc(cal_info->ndets, sizeof(float64));
  image_ar->number = (int32*)calloc(cal_info->ndets, sizeof(int32));
  
  image_ar->type = (char *)malloc(20*sizeof(char));
  if((image_ar->value == NULL) || (image_ar->number == NULL) ||
     (image_ar->number == NULL))
	
    {
      sprintf(msg, "Error allocating image artifact structure, aborting.");
      xxx_LogStatus("rps", __FILE__, __LINE__, msg);
      free(bias_hs.meanDN);
      free(bias_hs.stddev);
      free(bias_hs.valid_frames);
      free(bias_hs.maxDN);
      free(bias_hs.minDN);
      fclose(report);
      unlink(reportName);
      exit(ERROR);
    }
  

  /* Allocate the detector saturation level arrays.
   -----------------------------------------------------*/
  high_det_sat = (int32*)calloc(cal_info->ndets, sizeof(int32));
  low_det_sat  = (int32*)calloc(cal_info->ndets, sizeof(int32));
  if((high_det_sat == NULL) || (low_det_sat == NULL))
    {
      sprintf(msg, "Error allocating saturation level buffers, aborting.");
      xxx_LogStatus("rps", __FILE__, __LINE__, msg);
      free(bias_hs.meanDN);
      free(bias_hs.stddev);
      free(bias_hs.valid_frames);
      free(bias_hs.maxDN);
      free(bias_hs.minDN);
      free(image_ar->type);
      free(image_ar->value);
      free(image_ar->number);
      fclose(report);
      unlink(reportName);
      exit(ERROR);
    }
 
  /* First, determine whether we're using 'precal' bias data or 'dark' bias
     data. We'll need this information later. 
  ---------------------------------------------------*/
  switch(bias_src)
    {
    case PRECAL:
      {
	pBiasData = level0r->precal[band].data;
	bias_frames = level0r->precal[band].dim_sizes[N_FRAMES_IDX];
	break;
      }
    case DARK:
      {
	pBiasData = level0r->dark[band].data;
	bias_frames = level0r->dark[band].dim_sizes[N_FRAMES_IDX];
	break;
      }
    default:
      {
	/* Consider this a fatal error.  The user should go back and
	   make sure he/she's got the right specifier in the calling
	   routine. */
	sprintf(msg, "Bias specifier must be either PRECAL or DARK.");
	xxx_LogStatus("rps", __FILE__, __LINE__, msg);
	free(bias_hs.meanDN);
	free(bias_hs.stddev);
	free(bias_hs.valid_frames);
	free(bias_hs.maxDN);
	free(bias_hs.minDN);
	free(image_ar->type);
	free(image_ar->value);
	free(image_ar->number);
	fclose(report);
	unlink(reportName);
	exit(ERROR);
      }
    }

  /* Generate the bias histogram.  If we can't do it, it's considered for
     our purposes as a fatal error.  The parameter '0' in the call to
     axx_calc_hist() insures that we will NOT trend the bias histogram
     data.
  --------------------------------------------------------------------------*/
  if(axx_calc_hist(basename, level0r, &bias_hs, sca, band, cal_info->ndets,
		   bias_src, 0, database) != SUCCESS)
    {
      sprintf(msg, "Error generating bias histogram data.");
      xxx_LogStatus("rps", __FILE__, __LINE__, msg);
      free(bias_hs.meanDN);
      free(bias_hs.stddev);
      free(bias_hs.valid_frames);
      free(bias_hs.maxDN);
      free(bias_hs.minDN);
      free(image_ar->type);
      free(image_ar->value);
      free(image_ar->number);
      fclose(report);
      unlink(reportName);
      exit(ERROR);
    }


  /* Initialize the value array in the artifact structure to the constant
     value NO_VALUE (-1).  Change this value in each member of the array
     ONLY IF a reasonable saturation value is found.  The idea is to insure
     that only those detectors where saturated pixels have been detected
     will be written to the report file and/or the database.
     ----------------------------------------------------------------------*/
  for(det = 0; det < cal_info->ndets; det++)
    image_ar->value[det] = (float64)NO_VALUE;


  /* Find the saturation levels if there's anything to find.
   --------------------------------------------------------------*/
  axx_find_sat(HIGH, cal_info->ndets, image_frames, &cal_info->hist,
	       pImageData, high_det_sat);

  axx_find_sat(LOW, cal_info->ndets, bias_frames, &bias_hs, pBiasData,
	       low_det_sat);

  
  /* Write the header for the report file here.
   ----------------------------------------------------------------*/
  fprintf(report, "\n\nSaturation Level Analysis Results\n\n");
  fprintf(report, "%s:  SCA %d Band %s\n\n", basename, sca, bandString);
  fprintf(report, "Det.    Frame    Level        DN Value     Sat. Value\n");
  fprintf(report, "__________________________________________________________");
  fprintf(report, "\n");

  /* Now check the image data to see if any pixels are saturated.   If
     any are found, write them in the report. 
  ------------------------------------------------------------------------*/
  sprintf(image_ar->type, "Saturation");
  for(det = 0; det < cal_info->ndets; det++)
    {
      for(frame = 0; frame < image_frames; frame++)
	{
	  data_index = det + frame*cal_info->ndets;
	  if((pImageData[data_index] & FILL_DATA) == 0)
	    {
	      
	      /* Check to see that a pixel is not unexpectedly pegging out
		 at either end of the ALI's dynamic range (4095 high,
		 0 low)
		 --------------------------------------------------------*/
	      if(pImageData[data_index] >= MAX_ALI_RANGE)
		{
		  fprintf(report, "%d\t%d\tHighLimit\t%d\t%d\n", det+1,
			  frame+1, pImageData[data_index], MAX_ALI_RANGE);
		  (image_ar->number[det])++;
		  image_ar->value[det] = (float64)MAX_ALI_RANGE;
		}
	      else if(pImageData[data_index] <= MIN_ALI_RANGE)
		{
		  fprintf(report, "%d\t%d\tLowLimit\t%d\t%d\n", det+1,
			  frame+1, pImageData[data_index], MIN_ALI_RANGE);
		  (image_ar->number[det])++;
		  image_ar->value[det] = (float64)MIN_ALI_RANGE;
		}
	      else
		{
		  /* We don't have saturation at either end of the
		     dynamic range, but we still might have some analog
		     saturation floating around.  We'll check for it
		     here.
		     -------------------------------------------------------*/
		  if(pImageData[data_index] > high_det_sat[det])
		    {
		      if(high_det_sat[det] != 0)
			{
			  fprintf(report, "%d\t%d\tHigh%d\t%d\n", det+1,
				  frame+1, pImageData[data_index],
				  high_det_sat[det]);
			  (image_ar->number[det])++;
			  image_ar->value[det] = (float64)high_det_sat[det];
			}	  
		    }

		  if(pImageData[data_index] < low_det_sat[det])
		    {
		      if(low_det_sat[det] != 0)
			{
			  fprintf(report, "%d\t%d\tLow\t%d\t%d\n", det+1,
				  frame+1, pImageData[data_index],
				  low_det_sat[det]);
			  (image_ar->number[det])++;
			  image_ar->value[det] = (float64)low_det_sat[det];
			}
		    }
		}
	    }
	}
    }

  fclose(report);

 
  /* Clean up.
   ----------------*/
  free(bias_hs.meanDN);
  free(bias_hs.stddev);
  free(bias_hs.valid_frames);
  free(bias_hs.maxDN);
  free(bias_hs.minDN);
  free(low_det_sat);
  free(high_det_sat);

  
  /**************************************************************************/


  /* If we're trending, write the saturation data to the database.
     ----------------------------------------------------------------*/
  if(trend_on)
    {
      /* Make the ALI band name of the current band number
       ---------------------------------------------------------*/
      axx_MakeALIBandString(band, bandString);


      mysql = mysql_init(NULL);
      /* Step 1:  Get the scene ID given the EO1 scene name.
	 -------------------------------------------------------*/
      if(xxx_GetSceneID(database, basename, mysql, &id) != SUCCESS)
	{
	  sprintf(msg, "Error retrieving ID of scene %s.", basename);
	  xxx_LogStatus("rps", __FILE__, __LINE__, msg);
	  return ERROR;
	}
      

      /* Step 2: Write the data to the database.
         ---------------------------------------------------*/
      for(det = 0; det < cal_info->ndets; det++)
	{
	  if(image_ar->value[det] != (float64)NO_VALUE)
	    {
	      sprintf(sql_query, "insert into rad_artifact_det values");
	      sprintf(sql_query, "%s(%d, %d, '%s', %d, '%s', %f, %d)",
		      sql_query, id, sca, bandString, det+1,
		      image_ar->type,
		      image_ar->value[det],
		      image_ar->number[det]);
	      
	      if(mysql_real_query(mysql, sql_query, strlen(sql_query)))
		{
		  sprintf(msg, "Artifact record insert failed");
		  sprintf(msg, "%s for detector %d, Error: %s",
			  msg, det+1, mysql_error(mysql));
		  xxx_LogStatus("rps", __FILE__, __LINE__, msg);
		}
	    }   
	}

      /* Step 4:  Close the database. 
         ------------------------------------*/
      mysql_close(mysql);
    }
  

  /* Free up the artifact storage and set the pointers to NULL.  The idea is
     "leave the area in the condition it was originally found in...".  In
     addition, it will help locate indavertent accesses to these arrays before
     they're recreated for the next artifact...
  -----------------------------------------------------------------------*/
  free(image_ar->type);
  image_ar->type = NULL;
  free(image_ar->value);
  image_ar->value = NULL;
  free(image_ar->number);
  image_ar->number = NULL;

  return SUCCESS;
}



