/* axx_convolve_detector_data()

This routine takes one detector's line of data and convolves it
with a step filter of the form

                  [... 1 1 1 1 1 0 -1 -1 -1 -1 -1 ...],
whose length can be changed by changing a #define statement in
axx_calc_RDG.c

Normally, the convolution process consists of multiplication and addition
steps.  Since the filter coefficients are of unit magnitude, only the addition
steps need to be performed, allowing a significant increase in processing
speed.  Another speed improvement consists of initializing two sums: the sum of
the data "covered" by the +1's, and the sum of the data "covered" by the -1's.
As the filter is moved across the data, only 4 additions/subtractions
need to be performed:

1.  Subtract out the data covered by the first +1 filter coefficient.

2.  Add in the data covered by the 0 filter coefficient.

3.  Add in the data covered by the first -1 filter coefficient.

4.  Subtract out the data covered by the last -1 filter coefficient.


*/

#include "xxx_ali_level0r.h"
#include "axx_rps.h"
#include "xxx_Types.h"


int axx_convolve_detector_data(
int16  *pLine,	      /* I: 1D Array of lamp data */
int32  dLineLength,   /* I: Length of data buffer */
int32  dFilterLength, /* I: Length of filter buffer */
int16  *pOutput)      /* O: Convolution output buffer */

{
    int32 frame;               /* current frame in line of detector data */
    int32 pos_index;           /* offset values for adding data to convolution
			        buffer */
    int32 neg_index;
    int16 sum1;                /* running totals for positive and negative
			          "halves" of convolution filter */
    int16 sum2;

    sum1 = 0;               
    sum2 = 0;


    /* Initialize the two sum values sum1 and sum2.  Here, sum1 represents
       the sum of values (implicitly) multiplied by the +1 coefficients in
       the filter.  Similarly, sum2 represents the sum of values (implicitly)
       multiplied by -1.
    */
    
    for(frame = 0; frame < (dFilterLength/2); frame++)
      {
	sum1 += pLine[frame];
      }

    for(frame = ((dFilterLength/2)+1); frame < dFilterLength; frame++)
      {
	sum2 -= pLine[frame];
      }
    pOutput[0] = sum1 + sum2;


    /* Now go through and do the convolution, sliding the filter one frame
       at a time in the data buffer. Here, subtract the earliest value-
       add the latest value to the positive sum, and add the earliest
       value-subtract the latest value to the negative sum.  */

    for(frame = 1; frame < dLineLength-dFilterLength; frame++)
      {
	
	pos_index = frame + (dFilterLength/2) - 1;
        neg_index = frame + (dFilterLength - 1);

        sum1 = sum1 - pLine[frame-1] + pLine[pos_index];
	sum2 = sum2 + pLine[frame+(dFilterLength/2)] - pLine[neg_index];

	pOutput[frame] = sum1 + sum2;

      }
    return SUCCESS;
}

	
