/*  axx_do_lamp_gains():  Routine to compute detector-specific and band
                          averaged lamp state means and standard deviations,
			  along with an estimate of the relative detector
			  gain (ratio of detector-specific mean to
			  band averaged mean), and its standard error.  */


#include <stdio.h>
#include <stdlib.h>
#include <math.h>

#include "hdf.h"
#include "mfhdf.h"
#include "xxx_Types.h"
#include "axx_rps.h"
#include "mysql.h"

int axx_do_lamp_gains(
CAL_INFO *cal_info,       /* O:  Structure to put gain numbers into */
int32    band)            /* I:  Current 0-based band number */
{
  float64  *sy;           /* Buffers to perform statistical calculations */
  float64  *sx;
  float64  *sxx;
  float64  *sxy;
  float64  *syy;
  
  float64  factor;        /* Multiplication factor for calculation of
			     intercept standard error */
  float64  sigma2;
  
  int32    state;         /* Current lamp state */
  int32    index;         /* Index into 'ic_lamp' data arrays */
  int32    det;           /* Current detector   */


  /* Allocate the statistics buffers.
     -----------------------------------------*/
  sy  = (float64*)calloc(cal_info->ndets, sizeof(float64));
  sxy = (float64*)calloc(cal_info->ndets, sizeof(float64));
  sx  = (float64*)calloc(cal_info->ndets, sizeof(float64));
  sxx = (float64*)calloc(cal_info->ndets, sizeof(float64));
  syy = (float64*)calloc(cal_info->ndets, sizeof(float64));
  
  if((sy == NULL) || (sxy == NULL) || (sx == NULL) || (sxx == NULL) ||
     (syy == NULL))
    {
      if(sy != NULL)
	free(sy);
      if(sxy != NULL)
	free(sxy);
      if(sx != NULL)
	free(sx);
      if(sxx != NULL)
	free(sxx);
      if(syy != NULL)
	free(syy);
      return ERROR;
    }
  
  
  /* Now compute a band-averaged response for each lamp state.  We don't need
     to worry about pan band bias here because it was removed before we got
     to this point.
     ------------------------------------------------------------------------*/
  for(state = 0; state < NUM_LAMP_STATES; state++)
    {
      for(det = 0; det < cal_info->ndets; det++)
	{
	  index = det + state*cal_info->ndets;
	  
	  if(cal_info->ic_lamp.det_mean[index] != (float64)0)
	    {
	      cal_info->ic_lamp.band_average[state] +=
		cal_info->ic_lamp.det_mean[index];
	    }
	}
      
      cal_info->ic_lamp.band_average[state] /= cal_info->ndets;
    }
   

  /* Now we can compute a relative gain factor and standard error for each
     detector...  For now, we're assuming a linear gain model for
     simplicity.  We'll let:
     
     x represent the mean detector response
     y represent the band-averaged lamp state response
     
     It's quite possible that we'll have to go to a quadratic
     (or higher-order) model to do a reasonable job.
     
     slope:  (sxy - ((sx*sy)/n)) /(sxx - ((sx*sx)/n))
     slope std_error:  sqrt(sigma2 / sxx),
     intercept:  (sxx*sy - sx*sxy) / (sxx - ((sx*sx)/n))
     intercept std_error:  sqrt(sigma2*factor)
     
     where sigma2 = (syy - slope*sxy) / (n-2) and
     factor = (n*sxx + (sum(x_i))^2) /(n^2*sxx)  (a simplification
     of (1/n + x_bar^2/sxx)      */
  

  for(det = 0; det < cal_info->ndets; det++)
    {
      for(state = 0; state < NUM_LAMP_STATES; state++)
	{
	  index = det + state*cal_info->ndets;
	  if(cal_info->ic_lamp.det_mean[index] != (float64)0)
	    {
	      sxy[det] += cal_info->ic_lamp.det_mean[index]*cal_info->ic_lamp.band_average[state];
	      sx[det] += cal_info->ic_lamp.det_mean[index];
	      sy[det] += cal_info->ic_lamp.band_average[state];
	      sxx[det] += cal_info->ic_lamp.det_mean[index]*cal_info->ic_lamp.det_mean[index];
	      syy[det] += cal_info->ic_lamp.band_average[state]*cal_info->ic_lamp.band_average[state];
	      
	    }
	}
	   
      if(sx[det] == (float64)0)
	{
	  cal_info->gain.p1[det] = 1.0;
	  cal_info->gain.p2[det] = 0.0;
	  continue;
	}
      else
	{
	  sxx[det] -= (sx[det]*sx[det])/cal_info->ndets;
	  sxy[det] -= (sx[det]*sy[det])/cal_info->ndets;
	  syy[det] -= (sy[det]*sy[det])/cal_info->ndets;
	  
	  cal_info->gain.p1[det] = sxy[det]/sxx[det];
	  
	  cal_info->gain.p2[det] = (sy[det]/cal_info->ndets) - cal_info->gain.p1[det]*(sx[det]/cal_info->ndets);
	  
	  sigma2 = syy[det];
	  sigma2 -= cal_info->gain.p1[det]*sxy[det];
	  sigma2 /= (cal_info->ndets - 2);
	  
	  cal_info->gain.p1_stderr[det] = sqrt(sigma2 / sxx[det]);
	  factor = cal_info->ndets*sxx[det];
	  factor += (sx[det]*sx[det]);
	  factor /= ((cal_info->ndets*cal_info->ndets)*sxx[det]);
	  cal_info->gain.p2_stderr[det] = sqrt(sigma2*factor);
	}
    }
  



   /* Free the new buffers
    --------------------------*/
   free(sx);
   free(sy);
   free(sxx);
   free(sxy);
   free(syy);

   return SUCCESS; 
}

