/* axx_do_lamp_stats():    Routine to compute the mean DN value/std dev.
                           for each lamp state, given the starting and
			   stopping frames.  */

#include <math.h>

#include "xxx_ali_level0r.h"
#include "xxx_Types.h"
#include "xxx_LogStatus.h"
#include "axx_rps.h"

int axx_do_lamp_stats(
OFFSET_STATS  *os,       /* I:  Structure containing biases*/
int16     *pLine,        /* I:  Array containing level 0R lamp data for one
			        detector */
int32     *pLimits,      /* I:  Array containing starting/stopping frames */
int32     det,
int32     num_dets,
int32     state,         /* I:  Current lamp state */
int16     skip_length,
float64   *pMean,        /* O:  Mean lamp state */
float64   *pStd,         /* 0:  Std. deviation */
int32     *pCount)       /* 0:  Valid frames per state */ 
{
  char    msg[STRLEN];   /* message string */
  float64 *means;        /* means for various skip values.  These will be
			    averaged at the end to obtain an overall lamp
			    state mean.  Needed primarily for the pan band
			    to account for the bias */        
  float64 *stds;         /* std. devs for various skip values */
  int32   *counts;       /* number of valid frames for various skip values */
  int32   frame;         /* current frame */
  int32   index;         /* index into bias arrays */
  int16   skip;
  
  
  means = (float64*)calloc(skip_length, sizeof(float64));
  stds  = (float64*)calloc(skip_length, sizeof(float64));
  counts = (int32*)calloc(skip_length, sizeof(int32));
  if((means == NULL) || (stds == NULL) || (counts == NULL))
    {
      sprintf(msg, "Error allocating temp. buffers, aborting.");
      xxx_LogStatus("rps", __FILE__, __LINE__, msg);
      exit(ERROR);
    }

  
  
  /* Compute the lamp stats for the current lamp state.
     ------------------------------------------------------*/

  /* First, sum up the lamp data for a given skip state (0, 1, 2).  Subtract
     out the appropriate bias here, depending on the skip state.
   -------------------------------------------------------------------*/
  for(frame = pLimits[2*state]; frame <= pLimits[1 + 2*state]; frame++)
    {
      if((pLine[frame] != 0) && ((pLine[frame] & FILL_DATA) == 0))
	{
	  skip = frame % skip_length;
	  index = det + skip*num_dets;
	  means[skip] += (pLine[frame] - os->mean[index]);
	  (counts[skip])++;
	  stds[skip] += (pLine[frame] - os->mean[index])*(pLine[frame] - os->mean[index]);
	}
    } 
  
  /* Next, for the given skip state, figure out the mean and variance.
     Use these to compute the total mean (it will be a weighted mean,
     since the number of samples is different) and variance (using a rather
     ugly formula):

     mean = sum(Ni*mean_i)
            ---------------
	          N

     var = sum((Ni-1)*var_i + sum(Ni*mean_i*mean_i) - N*(mean*mean)
           --------------------------------------------------------
	                 (N - 1)

     where var_i  is the variance of the data for skip state i (0, 1, 2)
           Ni     is the number of valid frames in skip state i
	   mean_i is the mean of the data for skip state i
	   N      is the total number of valid frames of data
	   mean   is the weighted mean of the data

   --------------------------------------------------------------------------*/
  for(skip = 0; skip < skip_length; skip++)
    {
      means[skip] /= counts[skip];
      stds[skip] -= (counts[skip]*means[skip]*means[skip]);
      stds[skip] /= (counts[skip] - 1);
      (*pMean) += counts[skip]*means[skip];
      (*pStd)  += (counts[skip] - 1)*stds[skip];
      (*pStd)  += counts[skip]*means[skip]*means[skip];
      (*pCount) += counts[skip];
    }
  
  /* Compute the final statistics.  
   -------------------------------------*/
  *pMean /= *pCount;
  *pStd -= (*pCount)*(*pMean)*(*pMean);
  *pStd /= (*pCount - 1);
  *pStd = sqrt(*pStd);
  

  /* Clean up.
   ------------*/
  free(means);
  free(stds);
  free(counts);

  return SUCCESS;
}




