/* axx_find_sat():            routine to estimate high and low detector
                              saturation levels for ALI scenes.  The routine
			      itself is "generic"; the calling routine
			      "axx_char_det_sat" passes in the information
			      that specifies which saturation level is
			      to be determined (HIGH or LOW).   */
			      
#include <stdio.h>
#include <stdlib.h>

#include "xxx_Types.h"
#include "xxx_LogStatus.h"
#include "axx_rps.h"

int axx_find_sat(
const int    level,           /* I:  Saturation level */
int32        num_dets,        /* I:  Total number of detectors */
int32        num_frames,      /* I:  Total number of frames in image data */
SCENE_HIST_STATS   *hs,       /* I:  Histogram for image (high-level)data */
int16        *pData,          /* I:  Pointer to image data  */
int32        *det_sats)       /* O:  Array containing detector saturation
				     values */
{
  char   msg[STRLEN];                 /* message string */

  int32  SatThresholdCount;           /* number of counts a DN level has to
					 be for it to be considered saturated
				         */
  int32  AdjThresholdCount;           
  int32  AdjBinSum;                   /* number of counts in bins adjacent to
					 the potential saturated bin */
  int32  max_bin;                     /* maximum DN value */
  int32  min_bin;                     /* minimum DN value */
  int32  bin;                         /* current DN value */
  int32  high_limit;                  /* smallest bin given distance away from
					 potential high-level saturated bin */
  int32  low_limit;                   /* largest bin given distance away from
					 potential low-level saturated bin */
  int32  det;                         /* current detector */
  int32  frame;                       /* current frame */
  int32  data_index;                  /* index into scene data array */
  int32  ii;                          /* loop counters */
  int32  *det_hist;                   /* temporary storage for histogram */



  /* Allocate a detector-based histogram buffer.  First, however, estimate
     SatThresholdCount and AdjThresholdCount for each detector.

     For LANDSAT 7 (LPGS), a default count of 1000 is used for
     SatThresholdCount, and a default count of 10 is used for
     AdjThresholdCount.  SatThresholdCount is apparently estimated to be
     about 15% of the total number of minor frames (6600), and
     AdjThresholdCount is 1% of that.  For ALI, we'll use a range of default
     percentages, as follows...

     1.  If the number of valid frames in a data source is 1000 or less
         (typical for L0R bias data), we'll use a somewhat conservative
	 value of 25% of the number of frames for a saturation threshold.
	 Otherwise, the threshold is set at 15% of the number of frames.

	 Accordingly, the threshold for counts in adjacent bins is set at
	 10% if the threshold value SatThresholdCount is under 100, and
	 1% otherwise.

     2.  The parameters high_limit and low_limit are estimated as follows...

         high_limit is the maximum bin value - 15% of the
	 standard deviation; low_limit is the minimum bin value + 15% of
	 the standard deviation.


         These are very "arbitrary" cut-offs at this point; they can be
         changed if necessary.  This was done to maximize the possibility
         of having enough histogram data to look at.
  -------------------------------------------------------------------------*/
  AdjBinSum = 0;

  for(det = 0; det < num_dets; det++)
    {
      /* Set a threshold level, SatThresholdCount, for the minimum number of
	 counts necessary for a DN value to be considered "saturated."  Base
	 this value on 25% of the valid frames for bias data, or 15% of the
	 scene data.  At this point, this should be considered a "kludge."
      ----------------------------------------------------------------------*/
      if(hs->valid_frames[det] < 1000)
	SatThresholdCount = (int32)(0.25*(float64)hs->valid_frames[det]+0.5);
      else
	SatThresholdCount = (int32)(0.15*(float64)hs->valid_frames[det]+0.5);

      
      /* Set a threshold level, AdjThresholdCount, for the minimum number of
	 counts contained in the two bins adjacent to the potentially
	 saturated bin.  Again, this should be considered a "kludge."
      -----------------------------------------------------------------------*/
      if(SatThresholdCount < 100)
	AdjThresholdCount = (int32)(((float64)SatThresholdCount/10.)+0.5);
      else
	AdjThresholdCount = (int32)(((float64)SatThresholdCount/100.)+0.5);

      
      /* Fetch the maximum and minimum DN values from the histogram data.
       -------------------------------------------------------------------*/
      max_bin = hs->maxDN[det];
      min_bin = hs->minDN[det];

      /* Normally, an efficient number of bins to use can be determined
	 according to
	 
	 maxDN[det] - minDN[det] + 1;
	 
	 To keep things simple and avoid offset math complications,
	 we'll use 0 in place of min[det].  We know that min[det] is
	 not likely to be 0 (unless something really goofy is happening
	 with the detectors, such as a detector permanently stuck on
	 the minimum end of its dynamic range). 
	 -------------------------------------------------*/
      det_hist = (int32*)calloc(max_bin+1, sizeof(int32));
      if(det_hist == NULL)
	{
	  sprintf(msg, "Buffer allocation failed, aborting.");
	  xxx_LogStatus("rps", __FILE__, __LINE__, msg);
	  return ERROR;
	}
      
      /* Set bin limits to define the range of bins.  high_limit is the
	 smallest bin below the maximum bin value, low_limit is the largest
	 bin above the minimum bin value.
      ---------------------------------------------------------------------*/
      high_limit = (int32)((float64)hs->maxDN[det]-0.15*hs->stddev[det]+0.5);
      low_limit = (int32)((float64)hs->minDN[det]+0.15*hs->stddev[det]+0.5);


      /* Count every non-fill pixel value to generate a full detector
	 histogram.
	 --------------------------------------------------------------*/
      for(frame = 0; frame < num_frames; frame++)
	{
	  data_index = det + frame*num_dets;
	  if((pData[data_index] & FILL_DATA) == 0)
	    det_hist[pData[data_index]]++;
	}

      /* Starting with the maximum bin value, look for a bin with at
	 least SatThresholdCount counts.  Search up to high_limit.
	 -----------------------------------------------------------------*/
      if(level == HIGH)
	{
	  /*  If max_bin and high_limit numbers are the same, just go
	      ahead and check the next lower two bins
	      -----------------------------------------------*/
	  if(max_bin == high_limit)
	    {
	      for(ii = 1; ii <= ADJ_BINS; ii++)
		AdjBinSum += det_hist[max_bin-ii];
	      
	      if(AdjBinSum <= AdjThresholdCount)
		{
		  det_sats[det] = max_bin-ii-1;;
		}
	    }
	  else
	    {
	      for(bin = max_bin; bin > high_limit; bin--)
		{
		  if(det_hist[bin] >= SatThresholdCount)
		    {
		      /* If there aren't enough adjacent bins to the bin of
			 interest, just set that bin as the high-end
			 saturation level.
			 ---------------------------------------------------*/
		      if(bin > max_bin-ADJ_BINS)
			{
			  det_sats[det] = bin;
			  break;
			}
		      else
			{
			  for(ii = 1; ii <= ADJ_BINS; ii++)
			    AdjBinSum += det_hist[bin-ii];
			  
			  /* Check the sum of counts in the next adjacent bins.
			     If they're  less than the adjacent threshold
			     count, set the bin as the high-end saturation
			     bin.
			     ------------------------------------------------*/
			  if(AdjBinSum <= AdjThresholdCount)
			    {
			      det_sats[det] = bin;
			      break;
			    }  
			}
		    }
		}
	    }
	}

      /* Starting with the minimum bin value, look for a bin with at
	 least SatThreshold counts.  Search up to low_limit. 
      -------------------------------------------------------------------*/
      else
	{
	  
	  /* If min_bin and low_limit numbers are the same, just go
	     ahead and check the next higher two bins.
	     ----------------------------------------------------------*/
	  if(min_bin == low_limit)
	    {
	      if(det_hist[min_bin] >= SatThresholdCount)
		{
		  for(ii = 1; ii <= ADJ_BINS; ii++)
		    AdjBinSum += det_hist[min_bin+ii];
		  
		  if(AdjBinSum <= AdjThresholdCount)
		    {
		      det_sats[det] = min_bin+ii-1;
		    }
		}
	    }
	  else
	    {
	      for(bin = min_bin; bin < low_limit; bin++)
		{
		  if(det_hist[bin] >= SatThresholdCount)
		    {
		      /* If there aren't enough adjacent bins to the bin of
			 interest, just set that bin as the low-end
			 saturation level.
			 ---------------------------------------------------*/
		      if(bin < min_bin+ADJ_BINS)
			{
			  det_sats[det] = bin;
			  break;
			}
		      else
			{
			  for(ii = 1; ii <= ADJ_BINS; ii++)
			    AdjBinSum += det_hist[bin+ii];
			  
			  /* Check the sum of counts in the next adjacent bins.
			     If they're less than the adjacent threshold count,
			     set the bin as the high-end saturation bin.
			     ----------------------------------------------*/
			  if(AdjBinSum <= AdjThresholdCount)
			    {
			      det_sats[det] = bin;
			      break;
			    }  
			}
		    }
		}
	    }
	}

      /* We don't need the buffer for this detector anymore, so get rid of
	 it. 
      --------------------------------------------------------------------*/
      free(det_hist);
    }

  return SUCCESS;
}

