/* axx_find_state_bounds():  Routine to find reasonable starting and
                             stopping frames for computing lamp state
			     statistics.  */


#include "xxx_ali_level0r.h"
#include "xxx_LogStatus.h"
#include "xxx_Types.h"
#include "axx_rps.h"

#define  MIN_PEAK_WIDTH         5  /* Minimum width of frames with convolved
				      output above the threshold */

#define  MAX_BACKGROUND_MS     10  /* Maximum value of swing (+-) in
				      convolved output -- used in noise
				      testing */
#define  MAX_BACKGROUND_PAN    20


int axx_find_state_bounds(
int32 band,           /* I:  current band number */
int32 det,            /* I:  current detector */
int32 totalLength,    /* I:  number of frames to consider */
int16 *pConv,         /* I:  array holding convolved output values */
int16 *pRank,         /* I:  array holding ranks for current
			     convolved output */
float64  t1,          /* I:  peak threshold */
int32 pBound[],       /* O:  array to hold start and stop frames */
int16 *pNumPeaks)     /* O:  number of valid convolution peaks (representing
			     lamp state transitions were detected) */
{
  char msg[STRLEN];     /* message string */
  int32 background;     /* background "noise" in convolution output */
  int32 state;          /* state number (0, 1, 2, 3) */
  int32 index;          /* index number (0=start, 1=stop) */
  int32 loc;            /* location in boundary array */
  int32 frame;             /* counters */
  int32 zz;
  int32 stop_frame;     /* tentative stopping frame for current lamp state */
  int16 num_peaks;      /* number of transitions in convolution output */
  int16 peak_width;     /* width (in frames) of a convolution peak */
  int16 max_value;      /* current output maximum value */
  int16 loop_count;     /* number of times through processing loop */



  /* Set the initial starting frame for lamp state 0, and the initial
     stopping frame for lamp state 3. */

  pBound[0] = 9;          /* start 10 frames from beginning in lamp
                              state 0 */
   
  pBound[2*NUM_LAMP_STATES - 1] = totalLength;  
  /* stop 19 frames from the end in lamp state 3 */

  /* Reinitialize the other start/stop locations to 0. */
  for(frame = 1; frame <= 6; frame++)
    {
      if(pBound[frame] != 0)
	pBound[frame] = 0;
    }

  /* First, do some basic "sanity checks" on the threshold value.  */
  *pNumPeaks = 0;

  /* a.  Make sure the maximum convolved output is greater than the
         the threshold value T1.  */
  max_value = pConv[9];
  for(frame = 10; frame < totalLength; frame++)
    {
      if(pConv[frame] >= max_value)
	max_value = pConv[frame];
    }
  if((float64)max_value <= t1)
    {
      sprintf(msg, "Maximum peak value %d less than threshold value %lf",
	      max_value, t1);
      sprintf(msg, "%s for detector %d.", msg, det+1);
      xxx_LogStatus("rps", __FILE__, __LINE__, msg);
    }


  /* b.  Another noise check.  For MS band images, the convolution output
         tends to vary between 10 and -10, and between about 20 and -20 for
         pan band images.  If the maximum convolution output is 10 (20) or
	 less, the output is too noisy to do reliable peak detection, so
	 quit... */
  if(band != PAN_BAND)
    background = MAX_BACKGROUND_MS;
  else
    background = MAX_BACKGROUND_PAN;

  if(max_value <= background)
    {
      sprintf(msg, "Maximum peak value %d at or below background",
	      max_value);
      sprintf(msg, "%s value %d for detector %d.", msg, background, det+1);
      xxx_LogStatus("rps", __FILE__, __LINE__, msg);
    }
    

  /* We've passed some basic sanity checks, but there could still be some
     hidden problems.  The biggest problem is that we won't find exactly
     three lamp state transitions.  As we're checking the ranks to find
     suitable start/stop frames for our statistics, we'll count the number
     of "valid" transitions we find.

     What is a "valid" transition?  A valid transition, most simply, is one
     that lasts a minimum number of frames or longer.  Generally, in the
     earlier detectors, peak regions in the convolution output appear to be
     on the order of 20-30 frames wide (from threshold crossing to threshold
     crossing).  In the later detectors, the width seems to narrow due to
     more noise being present, down to about 10 frames or less.  Therefore,
     a "valid" transition will be defined for our purposes here as one
     that lasts (from threshold crossing to threshold crossing) about
     5 frames or longer (perhaps a little risky, but might be necessary
     for really noisy detectors--this can be adjusted if necessary).  */


  frame = 9;
  state = 0;
  
  while((frame >= 9) && (frame <= totalLength))
    {
      /* 1. check for zero rank or we're at the limits of what we're
	 interested in checking. */
      if((pRank[frame] == 0) || (frame == 9) || (frame == totalLength))
	{
	  frame++;
	  continue;
	}

       
      if((pRank[frame] != 0) && (pRank[frame-1] == 0))
	{
	  if(pRank[frame+1] == 0)         /* check for an isolated "spike" */
	    {
	      frame += 2;
	      continue;
	    }
	  else
	    {
	      stop_frame = frame-20;      /* tentative peak in conv. output */
	      peak_width = 0;
	      zz = frame;                 /* mark beginning of peak */
	      (*pNumPeaks)++;          /* count this as a peak for now */

	      /* Determine the width of this tentative peak. */
	      while(pRank[zz] != 0)
		{
		  peak_width++;
		  zz++;
		}

	      /* if the width is greater than a defined mininum width
		 (about 5 pixels wide) call it a peak and deal with it
		 accordingly...
		 a.  If the current lamp state isn't the last one (3),
		     treat the peak as representing a valid lamp state
		     transition.  Record the stopping frame for the
		     current lamp state as 20 frames earlier, and record
		     the starting frame for the next lamp state as 20
		     frames beyond the peak width. */
	      if(peak_width >= MIN_PEAK_WIDTH)
		{
		  if(state < 3)
		    {
		      index = 1;
		      pBound[index+2*state] = stop_frame;
		      state++;
		      pBound[2*state] = frame+peak_width+20;
		      frame += peak_width+20;
		    }
		  else
		    {
		      frame += peak_width; 
		    }
		  continue;
		}
	      else
		{
		  frame += peak_width;
		  (*pNumPeaks)--;
		}
	      continue;
	    }
	}
      else
	{
	  frame++;
	  continue;
	}
    }

  return SUCCESS;
}

