/* axx_set_ranks():  Routine to take a convolved line of detector data,
                     compute one threshold value, and determine which
                     data exceeded the threshold value  The goal is to
		     isolate the frames where valid lamp state transition
		     peaks have occurred.

The idea for this approach is based on work by Yoon-Pheng Law characterizing
LANDSAT 7 coherent noise.  For this work, Law computed two thresholds and used
them to detect coherent noise peaks: 

1.  The first threshold he computed is similar to the threshold deterimined
    in this routine.  It looks at all the data values.
2.  The second threshold he computed was based on data values less than
    the first threshold.

Two threshold values were necessary for Law's work because the peaks he was
attempting to detect were relatively small in magnitude.  Observations of
the convolved output for test ALI imagery tend to show a magnitude sufficiently
large enough to need only one threshold value.  */


#include <math.h>    /* for sqrt() function */

#include "axx_rps.h"
#include "xxx_Types.h"

int axx_set_ranks(
int32  band,               /* I: Current band number */
int32  totalLength,        /* I: Number of frames to consider */
int16* pConv,              /* I: Array containing convolution output */
int16* pRank,              /* O: Array to hold rank values */
float64 *pT1)              /* O: threshold value for current line of detector
			         data */
{
   int32 frame;      /* counter */
   float64 m1;       /* mean for all convolved output */
   float64 s1;       /* std. deviation for all convolved output */
   int32 sum1;       /* running sums of convolved output/output^2 */
   int32 sum2;
   int32 count;      /* count of non-zero frames */

   /* Initialize sums and count so that T1 can be computed from the
      convolved data. */
   sum1 = 0;
   sum2 = 0;
   count = 0;
     
   /* Determine T1 according to the following formulas:

      T1 = m1 + k1*s1, where k1 is a constant.
     
      k will differ between the Pan band and the MS bands, so... for
      a first cut, set k to 4.0 for the Pan band and 3.0 for the MS bands.

      NOTE:  Discard the first 10 outputs or so at the front end of the line,
             and the last  outputs at the back end of the
	     line.  The extreme outputs there will throw off all the results.
   */

   for(frame = 9; frame < totalLength; frame++)
     {
       if(pConv[frame] != 0)
	 {
	   sum1 += pConv[frame];
	   sum2 += pConv[frame]*pConv[frame];
	   count++;
	 }
     }
   m1 = (float64)sum1 / (float64)count;
   s1 = sqrt(((float64)sum2 / (float64)count) - m1*m1);
   if(band == 9)
     *pT1 = m1 + 4.0*s1;
   else
     *pT1 = m1 + 3.0*s1;

  
   /* Finally, set the rank of the convolved output.  The ranks are as
      follows:

      Rank = 0:  Convolved output did not exceed T1
      Rank = 1:  Convolved output exceeded T1
    */

   for(frame = 9; frame < totalLength; frame++)
     {
       if((float64)pConv[frame] >= *pT1)
	 pRank[frame] = 1;
     }
   
   return SUCCESS;
}















