/*
 * axx_vert_moment(p_array, xdim, ydim, skip_length, p_mean, p_stddev,
 *                 p_num_valid_points)
 * 
 *      This function finds the mean and standard deviation of a two
 *      diminsional array (p_array) in the vertical direction.  It accounts
 *      for the fact that consecutive data points may not be spaced 1 
 *      frame apart (which is the case, for example, with precal/dark bias
 *      data from the ALI sensor onboard EO1) through the 'skip_length'
 *      parameter.
 */


#include <stdio.h>
#include <stdlib.h>
#include <math.h>

#include "hdf.h"
#include "mfhdf.h"
#include "xxx_Types.h"
#include "axx_rps.h"

int axx_vert_moment(
int16 *p_array,               /* I: 2D array to determine detector mean &
				    stddev */
int32 xdim,                   /* I: number of detectors */
int32 ydim,                   /* I: number of frames */
int16 skip_length,            /* I: number of frames to skip to reach
				    consecutive data frames (1 for MS
				    bands, 3 for pan band) */
float64 *p_mean,              /* O: 2D array of means */
float64 *p_stddev,            /* O: 2D array of standard deviations */
int32   *p_num_valid_points)  /* O: 2D array of number of valid points in
				    mean & stddev */
{
  int32 skip;                 /* index representing number of frames to skip
				 to get to next data value */
  int32 yy;                   /* current minor frame (wrt to skip value) */
  int32 xx;                   /* current detector */
  int32 loc;                  /* index into data array */
  int32 index;                /* index into offset arrays */

  for(skip = 0; skip < skip_length; skip++)
    {
      for(yy = skip; yy < ydim; yy += skip_length)
	{
	  for(xx = 0; xx < xdim; xx++)
	    {
	      loc = xx + yy*xdim;

	      /* Make sure we're not using fill data
	       ----------------------------------------*/
	      index = xx + skip*xdim;
	      if((p_array[loc] & FILL_DATA) == 0)
		{
		  p_mean[index] += p_array[loc];
		  p_stddev[index] += p_array[loc]*p_array[loc];
		  (p_num_valid_points[index])++;
		}
	    }
	}

      /* Finish computing the mean and standard deviation. 
       ------------------------------------------------------*/
      for(xx = 0; xx < xdim; xx++)
	{
	  index = xx + skip*xdim;
	  p_mean[index] /= p_num_valid_points[index];
	  p_stddev[index] -= 
	    p_num_valid_points[index]*p_mean[index]*p_mean[index];
	  p_stddev[index] /= (p_num_valid_points[index] - 1);
	  p_stddev[index] = sqrt(p_stddev[index]);
	}
    }

  return SUCCESS ;
}



