/****************************************************************************
NAME: exx_iru2acs

PURPOSE:
Rotate the gyro data to the attitude control system coordinates
and converts from arc seconds to radians.

DEVELOPMENT HISTORY:
$Log: exx_iru2acs.c,v $
Revision 1.1  2000/11/22 21:52:47  storey
Original version - rotate IRU data to ACS frame


RETURN VALUE:
Type = int
Value    Description
-----    -----------
SUCCESS  Successful completion 
ERROR    Operation failed

NOTES:

ALGORITHM REFERENCES:
******************************************************************************
                        Property of the U.S. Government
                           USGS EROS Data Center
*****************************************************************************/
#include <math.h>
#include "xxx_Types.h"
#include "xxx_LogStatus.h"
#include "xxx_const.h"
#include "xxx_math.h"
#include "exx_cpf.h"

int exx_iru2acs(
    int niru,		/* I: number of IRU points			*/
    VECTOR *iru,	/* I/O:  IRU data				*/
    const CPF *cpf)	/* I: info from CPF				*/ 
{
int	i;		/* Looping variables				*/
int    status;		/* status of return from function		*/

double	matrix1[3][3],	/* temporary matricies */	
	newmatrix[3][3],
        att_to_iru[3][3], /* the attitude to IRU conversion matrix */
	roll[3][3],	/* the original roll, pitch and yaw components of
		 	   the gyro matrix */
	pitch[3][3],	   
	yaw[3][3];
double	sectorad;	/* Conversion from arcseconds to radians */


/* Initialize the units conversion */
sectorad = atan(1.0)/45.0/3600.0;

/*
  calculate the inverse of the IRU to attitude matrix from the CPF
*/
status = xxx_invmatrix(cpf->Attitude_Parameters.IRU_To_Attitude_Matrix,att_to_iru);
if (status != SUCCESS)
{
    xxx_LogStatus("EXXLIB", __FILE__, __LINE__, "Error inverting matrix");
    return(ERROR);
}

/*
  Initialize the roll matrix
*/
roll[0][0] = 1.0;
roll[0][1] = 0.0;
roll[0][2] = 0.0;
roll[1][0] = 0.0;
roll[2][0] = 0.0;

/*
  Initialize the pitch matirix
*/
pitch[0][1] = 0.0;
pitch[1][0] = 0.0;
pitch[1][1] = 1.0;
pitch[1][2] = 0.0;
pitch[2][1] = 0.0;

/*
  Initialize the yaw matrix
*/
yaw[0][2] = 0.0;
yaw[1][2] = 0.0;
yaw[2][0] = 0.0;
yaw[2][1] = 0.0;
yaw[2][2] = 1.0;


for (i = 0; i < niru; i++)
{
    /*
      Initialize the roll matrix
    */
    roll[1][1] = cos(iru[i].x*sectorad);
    roll[1][2] = sin(iru[i].x*sectorad);
    roll[2][1] = -roll[1][2];
    roll[2][2] = roll[1][1];
     
    /*
      Initialize the pitch matirix
    */
    pitch[0][0] = cos(iru[i].y*sectorad);
    pitch[0][2] = -sin(iru[i].y*sectorad);
    pitch[2][0] = -pitch[0][2];
    pitch[2][2] = pitch[0][0];
    
    /*
      Initialize the yaw matrix
    */
    yaw[0][0] = cos(iru[i].z*sectorad);
    yaw[0][1] = sin(iru[i].z*sectorad);
    yaw[1][0] = -yaw[0][1];
    yaw[1][1] = yaw[0][0];
 
    xxx_multmatrix((const double(*)[3])roll,(const double(*)[3])att_to_iru,newmatrix);
    xxx_multmatrix((const double(*)[3])pitch,(const double(*)[3])newmatrix,matrix1);
    xxx_multmatrix((const double(*)[3])yaw,(const double(*)[3])matrix1,newmatrix);
    xxx_multmatrix(cpf->Attitude_Parameters.IRU_To_Attitude_Matrix,
	           (const double(*)[3])newmatrix,matrix1);
    
    iru[i].x = -atan2(matrix1[2][1], matrix1[2][2]);	/* roll coord */
    iru[i].y = asin(matrix1[2][0]);	       		/* pitch coord */
    iru[i].z = -atan2(matrix1[1][0], matrix1[0][0]);	/* yaw coord */
}
return(SUCCESS);
}
