/******************************************************************************
NAME: exx_kalman_smooth_gps

PURPOSE:
to run the kalman filter on the GPS data
and then smooth output from the kalman filter.

DEVELOPMENT HISTORY:

RETURN VALUE:
Type = int
Value    Description
-----    -----------
SUCCESS  Successful completion
ERROR    Operation failed

NOTES:
  This file contains the matrix operations needed for Kalman 
  filtering.  For the notes that preceed each function the 
  following definitions are used.

  [X]  -> State matrix
  [S]  -> State transition matrix
  [P]  -> Error covariance matrix  
  [Q]  -> Covariance matrix of state vector noise
  [R]  -> Covariance matrix of state vector noise
  [H]  -> Matrix relating state matrix to measurement vector
  [K]  -> Kalman gain matrix
  [Z]  -> Measurement matrix
  [I]  -> Identity matrix
  [_]  -> Predicted 
  [ ]' -> Transpose of matrix
  [ ]^ -> Inverse of matrix

ALGORITHM REFERENCES:
Introduction to Random Signal Analysis and Kalman Filtering, 
by Robert Grover Brown (p. 195)
******************************************************************************
		        Property of the U.S. Government
                            USGS EROS Data Center
******************************************************************************/
#include <stdio.h>
#include <math.h>
#include <string.h>
#include "xxx_LogStatus.h"
#include "xxx_const.h"
#include "xxx_structures.h"
#include "axx_alimodel.h"
#include "exx_cpf.h"
#include "exx_potential.h"

/* defines to show the size of matricies being used */
#define		M_SIZE		6
#define		N_SIZE		6

/* define nominal time step for GPS data */
#define	GPS_TIME	1.0
/* Define number of steps used to propagate data */
#define NUM_STEPS	10

/* Defines for filter noise parameters (could be put in system table) */
#define PNOISE_POS_SIGMA	/* 15.0e-2 */ 5.0e-1
#define PNOISE_VEL_SIGMA	/* 15.0e-2 */ 5.0e-1
#define	ONOISE_POS_SIGMA 	25.0 
#define	ONOISE_VEL_SIGMA	0.02
#define	SNOISE_POS_SIGMA	25.0
#define	SNOISE_VEL_SIGMA	8.0
				
/* lpgslite: moved out of function, too much local data? */
static double Pn[M_SIZE*M_SIZE];	/* Predicted error cov matrix at time k	*/
static double K[M_SIZE*N_SIZE];		/* Kalman gain matrix			*/
static double H[M_SIZE*N_SIZE];		/* M_SIZE relating state to measurement	*/
static double S[M_SIZE*M_SIZE];		/* State transition matrix		*/
static double A[M_SIZE*M_SIZE];		/* Smoothing gain matrix		*/
static double Q[M_SIZE*M_SIZE];		/* Process noise			*/
static double R[N_SIZE*N_SIZE];		/* Measurment noise			*/
static double Xk[M_SIZE];		/* State matrix at time k		*/
static double X[NPCD][M_SIZE];		/* Matrix containing states at all k	*/
static double Xk1[NPCD][M_SIZE];	/* Matrix containing states at all k+1	*/
static double P[NPCD][M_SIZE*M_SIZE];	/* Filtered error covar matrix		*/
static double Pn1[NPCD][M_SIZE*M_SIZE]; /* Predicted error covar matrix at k+1	*/
static double XN[M_SIZE];		/* Estimate of state [X] up to N	*/
static double XN1[M_SIZE];		/* Estimate of state [X] up to N+1	*/
static double z[N_SIZE];		/* Measured value matrix		*/
static double pre[N_SIZE];		/* Predicted measurement(from predicted state)*/

int exx_kalman_smooth_gps(
    double rawpos[3][NPCD],         /* ephemeris position telemetry data */
    double rawvel[3][NPCD],         /* ephemeris velocity telemetry data */
    int nephem,			    /* Number of ephemeris points	 */
    double etime[NPCD],             /* ephemeris time telemetry 	 */
    const CPF *cpf,                 /* CPF structure */
    double smpos[3][NPCD],	    /* Smoothed GPS position		 */
    double smvel[3][NPCD])	    /* Smoothed GPS velocity		 */
{
double dt;			/* Time between GPS samples		*/
double att_buff[3][NPCD];	/* Temporary attitude data buffer */
int kcount,gps_count,i,j,k;	/* Counters				*/
int m_size, n_size;		/* size of the arrays			*/
int acnt;			/* Counter for attitude data		*/
int scnt;			/* Counter for smoothed data		*/
int status;			/* status of return from a function	*/
double *ptr_x;			/* Pointer to smoothed X position	*/
double *ptr_y;			/* Pointer to smoothed Y position	*/
double *ptr_z;			/* Pointer to smoothed Z position	*/
double *ptr_xv;			/* Pointer to smoothed X velocity	*/
double *ptr_yv;			/* Pointer to smoothed Y velocity	*/
double *ptr_zv;			/* Pointer to smoothed Z velocity	*/
double tmpx[NPCD];		/* Temporary data buffer		*/
double tmpy[NPCD];		/* Temporary data buffer		*/
double tmpz[NPCD];		/* Temporary data buffer		*/
double tmpxv[NPCD];		/* Temporary data buffer		*/
double tmpyv[NPCD];		/* Temporary data buffer		*/
double tmpzv[NPCD];		/* Temporary data buffer		*/
double acc_x;			/* Gravity model acceleration in X 	*/
double acc_y;			/* Gravity model acceleration in Y 	*/
double acc_z;			/* Gravity model acceleration in Z 	*/
double sigma_x;                 /* Standard dev. in X for process noise */
double sigma_y;                 /* Standard dev. in Y for process noise */
double sigma_z;                 /* Standard dev. in Z for process noise */
double sigma_xv;		/* Standard dev. in XV for process noise */
double sigma_yv;		/* Standard dev. in YV for process noise */
double sigma_zv;		/* Standard dev. in ZV for process noise */
double coef[4];			/* Polynomial coefficients 		 */
double step_X[M_SIZE];		/* Array used to shorten length of prediction calc */
double step_Xk1[M_SIZE];	/* Array used to shorten length of prediction calc */
double time_step;		/* Time step used for prediction calc 	 */
double time;			/* Total accumulation of time in prediction calc */
double frac;			/* Fraction of step of prediction calc 	 */
double resx[NPCD];		/* Residual left after removal of trend  */
double resy[NPCD];		/* Residual left after removal of trend  */
double resz[NPCD];		/* Residual left after removal of trend  */

/* zero out the matricies */
for (i = 0; i < M_SIZE*M_SIZE; i++)
    {
    Pn[i] = 0.0;
    K[i] = 0.0;
    H[i] = 0.0;
    S[i] = 0.0;
    A[i] = 0.0;
    Q[i] = 0.0;
    R[i] = 0.0;
    for(j = 0; j < NPCD; j++)
	{
	P[j][i] = 0.0;
	Pn1[j][i] = 0.0;
	}
    }

/*
 * Set up state transition matrix.
 */
S[0]  = 1.0; S[1]  = 0.0; S[2]  = 0.0; S[3]  = dt;  S[4]  = 0.0; S[5]  = 0.0;
S[6]  = 0.0; S[7]  = 1.0; S[8]  = 0.0; S[9]  = 0.0; S[10] = dt;  S[11] = 0.0;
S[12] = 0.0; S[13] = 0.0; S[14] = 1.0; S[15] = 0.0; S[16] = 0.0; S[17] = dt;
S[18] = 0.0; S[19] = 0.0; S[20] = 0.0; S[21] = 1.0; S[22] = 0.0; S[23] = 0.0;
S[24] = 0.0; S[25] = 0.0; S[26] = 0.0; S[27] = 0.0; S[28] = 1.0; S[29] = 0.0;
S[30] = 0.0; S[31] = 0.0; S[32] = 0.0; S[33] = 0.0; S[34] = 0.0; S[35] = 1.0;

for( i = 0; i < 3; i++)
   Pn[i*M_SIZE+i] = SNOISE_POS_SIGMA;
for( i = 3; i < M_SIZE; i++)
   Pn[i*M_SIZE+i] = SNOISE_VEL_SIGMA;
for(i = 0; i < M_SIZE*M_SIZE; i++)
   Pn[i] = Pn[i] * Pn[i]; 
/**********************************
  Set Pn1 in case smoother is set
  to go from nephem-1 to 0
**********************************/
for(i = 0; i < M_SIZE*M_SIZE; i++)
   Pn1[0][i] = Pn[i]; 

for( i = 0; i < 3; i++)
   R[i*M_SIZE+i] = ONOISE_POS_SIGMA;
for( i = 3; i < M_SIZE; i++)
   R[i*M_SIZE+i] = ONOISE_VEL_SIGMA;
for(i = 0; i < M_SIZE*M_SIZE; i++)
   R[i] = R[i] * R[i]; 

for( i = 0; i < M_SIZE; i++)
   H[i*M_SIZE+i] = 1.0;

/*
 * Initialize state
 */
/**********************************
  Set Xk1 in case smoother is set
  to go from nephem-1 to 0
**********************************/
dt = etime[1] - etime[0];
Xk1[0][0] = Xk[0] = rawpos[0][0];
Xk1[0][1] = Xk[1] = rawpos[1][0];
Xk1[0][2] = Xk[2] = rawpos[2][0];
Xk1[0][3] = Xk[3] = rawvel[0][0] * 1000.0;  
Xk1[0][4] = Xk[4] = rawvel[1][0] * 1000.0;  
Xk1[0][5] = Xk[5] = rawvel[2][0] * 1000.0;  

m_size = M_SIZE;
n_size = N_SIZE;

sigma_x  = PNOISE_POS_SIGMA;
sigma_y  = PNOISE_POS_SIGMA;
sigma_z  = PNOISE_POS_SIGMA;
sigma_xv = PNOISE_VEL_SIGMA;
sigma_yv = PNOISE_VEL_SIGMA;
sigma_zv = PNOISE_VEL_SIGMA;

/* nephem = nephem - 1; */
for( gps_count=0, kcount=0 ; gps_count<nephem ; kcount++, gps_count++ )
{

   /*
    * Set up measurement matrix
    */
   z[0] = rawpos[0][gps_count];
   z[1] = rawpos[1][gps_count];
   z[2] = rawpos[2][gps_count];
   z[3] = rawvel[0][gps_count] * 1000.0;
   z[4] = rawvel[1][gps_count] * 1000.0;
   z[5] = rawvel[2][gps_count] * 1000.0;

   if( gps_count<nephem-1 )
      dt = etime[gps_count+1] - etime[gps_count];
   else 
      dt = GPS_TIME;

   /*
    * Set up process noise.
    */
   Q[0]  = (dt * dt * sigma_x * sigma_x) + ((dt * dt * dt * dt * sigma_xv * sigma_xv) / 4.0);
   Q[3]  = (dt * dt * dt * sigma_xv * sigma_xv) / 2.0;
   Q[7]  = (dt * dt * sigma_y * sigma_y) + ((dt * dt * dt * dt * sigma_yv * sigma_yv) / 4.0);
   Q[10] = (dt * dt * dt * sigma_yv * sigma_yv) / 2.0;
   Q[14] = (dt * dt * sigma_z * sigma_z) + ((dt * dt * dt * dt * sigma_zv * sigma_zv) / 4.0);
   Q[17] = (dt * dt * dt * sigma_zv * sigma_zv) / 2.0;
   Q[18] = (dt * dt * dt * sigma_xv * sigma_xv) / 2.0;
   Q[21] = dt * dt * sigma_xv * sigma_xv;
   Q[25] = (dt * dt * dt * sigma_yv * sigma_yv) / 2.0;
   Q[28] = dt * dt * sigma_yv * sigma_yv;
   Q[32] = (dt * dt * dt * sigma_zv * sigma_zv) / 2.0;
   Q[35] = dt * dt * sigma_zv * sigma_zv;

   /*
    * Filter data.
    */
   status = xxx_kalman_gain( Pn, H, R, K, m_size, n_size );
   if (status != SUCCESS)
   {
      xxx_LogStatus("XXXLIB", __FILE__, __LINE__, "Error returned from xxx_kalman_gain");
      return(ERROR);
   }

   status = xxx_filter_state( Xk, &X[kcount][0], K, z, H, m_size, n_size );
   if (status != SUCCESS)
   {
      xxx_LogStatus("XXXLIB", __FILE__, __LINE__, "Error returned from xxx_filter_state");
      return(ERROR);
   }

   status = xxx_filter_error_covar( K, H, Pn, &P[kcount][0], m_size, n_size );
   if (status != SUCCESS)
   {
      xxx_LogStatus("XXXLIB", __FILE__, __LINE__, "Error returned from xxx_filter_error_covar");
      return(ERROR);
   }

   S[3]  = dt;
   S[10] = dt;
   S[17] = dt;

   /*
    * Predict error covariance and state matrix for next iteration.
    */
   status = xxx_predict_error_covar(S, &P[kcount][0], &Pn1[kcount][0], Q, m_size);
   if (status != SUCCESS)
   {
       xxx_LogStatus("XXXLIB", __FILE__, __LINE__, "Error returned from xxx_predict_error_covar");
       return(ERROR);
   }

   for( k=0 ; k<M_SIZE ; k++ )
      step_X[k] = X[kcount][k];

   time_step = 0.0;
   time      = 0.0;
   /*
    * Propagate state in steps if necessary.
    */
   for( k=0 ; k<NUM_STEPS ; k++ )
   {
      frac = ((double)k + 1) / NUM_STEPS;
      time_step = frac * dt - time;

      /*
       * Setup up state matrix
       */
      S[3]  = time_step;
      S[10] = time_step;
      S[17] = time_step;

      status = xxx_predict_state(S, step_X, step_Xk1, m_size);
      if (status != SUCCESS)
      {
         xxx_LogStatus("XXXLIB", __FILE__, __LINE__, "Error returned from xxx_predict_state");
         return(ERROR);
      }

      /*
       * Find force (acceleration) due to Earth's potential.
       */
      acc_x = p2x(cpf, step_X[0], step_X[1], step_X[2], step_X[3], step_X[4], step_X[5]);
      acc_y = p2y(cpf, step_X[0], step_X[1], step_X[2], step_X[3], step_X[4], step_X[5]);
      acc_z = p2z(cpf, step_X[0], step_X[1], step_X[2], step_X[3], step_X[4], step_X[5]);

      step_X[0] = step_Xk1[0];
      step_X[1] = step_Xk1[1];
      step_X[2] = step_Xk1[2];
      step_X[3] = step_Xk1[3] + acc_x * time_step;
      step_X[4] = step_Xk1[4] + acc_y * time_step;
      step_X[5] = step_Xk1[5] + acc_z * time_step;

      time = time + time_step;
   }
   for( k=0 ; k<M_SIZE ; k++ )
      Xk1[kcount][k] = step_X[k];

   status = xxx_matrix_multiply(H, &Xk1[kcount][0], pre, n_size, m_size, m_size, 1);
   if (status != SUCCESS)
   {
       xxx_LogStatus("XXXLIB", __FILE__, __LINE__, "Error returned from xxx_matrix_multiply");
       return(ERROR);
   }

   for( j=0 ; j<m_size ; j++ )
       Xk[j] = Xk1[kcount][j];
   for( j=0 ; j<m_size*m_size ; j++ )
       Pn[j] = Pn1[kcount][j];
}

ptr_x  = tmpx;
ptr_y  = tmpy;
ptr_z  = tmpz;
ptr_xv = tmpxv;
ptr_yv = tmpyv;
ptr_zv = tmpzv;

for( j=0 ; j<m_size ; j++ )
   XN[j] = Xk1[kcount-1][j];

/*
 * Smooth from nephem to 0 since the first measurement
 * was used for initial conditions.
 * Kalman went from k=0 (corresponding to second ephemris point)
 * to nephem-1.
 */
dt = GPS_TIME;
for(scnt=kcount-1 ; scnt>=0 ; scnt--)
{
   /*
    * Setup up state matrix
    */
   S[3]  = dt;
   S[10] = dt;
   S[17] = dt;

   /*
    * Smooth data.
    */
   status = xxx_smooth_gain( &P[scnt][0], &Pn1[scnt][0], S, A, m_size );
   if (status != SUCCESS)
   {
       xxx_LogStatus("XXXLIB", __FILE__, __LINE__, "Error returned from xxx_smooth_gain");
       return(ERROR);
   }

   status = xxx_smooth_state(&X[scnt][0], &Xk1[scnt][0], XN, A, XN1, m_size);
   if (status != SUCCESS)
   {
      xxx_LogStatus("XXXLIB", __FILE__, __LINE__, "Error returned from xxx_smooth_state");
      return(ERROR);
   }

   *ptr_x = XN1[0];
   ptr_x++;
   *ptr_y = XN1[1];
   ptr_y++;
   *ptr_z = XN1[2];
   ptr_z++;
   *ptr_xv = XN1[3];
   ptr_xv++;
   *ptr_yv = XN1[4];
   ptr_yv++;
   *ptr_zv = XN1[5];
   ptr_zv++;

   for( j=0 ; j<m_size ; j++ )
       XN[j] = XN1[j];

   dt = etime[scnt+1] - etime[scnt];
}

/*
 * Reverse the order of the smoothed data set.  The smoothing algorithm
 * starts with the last data sample and works forward.  
 */
/* 
 * Ignoring first (last point smoothed) smoothed point, this is throwing
 * off the fit_smooth routine.
 */
for( i=kcount-1, k=0 ; i>=0 ; i--, k++ )
{
   smpos[0][k] = tmpx[i];
   smpos[1][k] = tmpy[i];
   smpos[2][k] = tmpz[i];
   smvel[0][k] = tmpxv[i];
   smvel[1][k] = tmpyv[i];
   smvel[2][k] = tmpzv[i];
}

return(SUCCESS);
}
