/******************************************************************************
NAME: exx_kalman_smooth_iru

PURPOSE:
to run the kalman filter on the IRU and the epa (attitude) data
and then smooth output from the kalman filter.

DEVELOPMENT HISTORY:
$Log: exx_kalman_smooth_iru.c,v $
Revision 1.3  2000/11/28 14:34:03  storey
Fixed two filter initialization bugs.

Revision 1.2  2000/11/27 16:19:18  storey
Added missing temporary array declaration.


RETURN VALUE:
Type = int
Value    Description
-----    -----------
SUCCESS  Successful completion
ERROR    Operation failed

NOTES:
  This file contains the matrix operations needed for Kalman 
  filtering.  For the notes that preceed each function the 
  following definitions are used.

  [X]  -> State matrix
  [S]  -> State transition matrix
  [P]  -> Error covariance matrix  
  [Q]  -> Covariance matrix of state vector noise
  [R]  -> Covariance matrix of state vector noise
  [H]  -> Matrix relating state matrix to measurement vector
  [K]  -> Kalman gain matrix
  [Z]  -> Measurement matrix
  [I]  -> Identity matrix
  [_]  -> Predicted 
  [ ]' -> Transpose of matrix
  [ ]^ -> Inverse of matrix

ALGORITHM REFERENCES:
Introduction to Random Signal Analysis and Kalman Filtering, 
by Robert Grover Brown (p. 195)
******************************************************************************
		        Property of the U.S. Government
                            USGS EROS Data Center
******************************************************************************/

#include <stdio.h>
#include <math.h>
#include <string.h>
#include "xxx_LogStatus.h"
#include "xxx_const.h"
#include "axx_alimodel.h"
#include "exx_init_model.h"

#ifdef DEBUG2 /* allows for easy debugging */
    /* defines to allow printing, so we can trace errors if necessary */
    #define		PAXIS		0
#endif

/* defines to show the size of matricies being used */
#define		M_SIZE		3
#define		N_SIZE		3

/* Define for worst case gyro data sizing */
#define MAX_GYRO	NPCD
#define	LAGRANGE_PTS	4

/* Defines for filter noise parameters (could be put in system table) */
#define PNOISE_ATTRATE_SIGMA	1.0e-2	/* Attitude rate process noise in
					   radians per second per second  */
#define PNOISE_DRIFT_SIGMA	1.0e-5	/* Gyro drift rate process noise in
					   radians per second per second  */
#define	ONOISE_EPA_SIGMA	5.0	/* Quaternion observation noise in
					   arcsec  */
#define	ONOISE_GYRO_SIGMA	1.00 	/* IRU rate observation noise 
					   in arcsec per second */
#define	SNOISE_ATT_SIGMA	45.0	/* Initial attitude state standard
					   deviation in arcsec */
#define	SNOISE_ATTRATE_SIGMA	45.0	/* Initial attitude rate state standard
					   deviation in arcsec per second */
#define	SNOISE_DRIFT_SIGMA	5.0	/* Initial drift state standard
					   deviation in arcsec per second  */

/* lpgslite: moved out of function, too much local data? */
static double Pn[M_SIZE*M_SIZE];	/* Predicted error cov matrix at time k	*/
static double K[M_SIZE*N_SIZE];		/* Kalman gain matrix			*/
static double H[M_SIZE*N_SIZE];		/* M_SIZE relating state to measurement	*/
static double S[M_SIZE*M_SIZE];		/* State transition matrix		*/
static double A[M_SIZE*M_SIZE];		/* Smoothing gain matrix		*/
static double Q[M_SIZE*M_SIZE];		/* Process noise				*/
static double R[N_SIZE*N_SIZE];		/* Measurment noise			*/
static double Xk[M_SIZE];		/* State matrix at time k		*/
static double X[MAX_GYRO][M_SIZE];	/* Matrix containing states at all k	*/
static double Xk1[MAX_GYRO][M_SIZE];	/* Matrix containing states at all k+1	*/
static double P[MAX_GYRO][M_SIZE*M_SIZE];	/* Filtered error covar matrix	*/
static double Pn1[MAX_GYRO][M_SIZE*M_SIZE]; /* Predicted error covar matrix at k+1*/
static double XN[M_SIZE];		/* Estimate of state [X] up to N	*/
static double XN1[M_SIZE];		/* Estimate of state [X] up to N+1	*/
static double z[N_SIZE];		/* Measured value matrix		*/
static double pre[N_SIZE];		/* Predicted measurement(from predicted state)*/
static double iru_rate[NAXIS][MAX_GYRO];/* Input gyro rate data			*/
static double tmp[MAX_GYRO];		/* Temporary data buffer		*/

int exx_kalman_smooth_iru(
    int *nepa,			/* I: Number of atttitude measurements 		*/
    double *epatime,		/* I: Array of epa sample times (from acs_time) */
    VECTOR *epa,		/* I: Satellite epa attitude data 		*/
    int *niru,			/* I/O: Number of gyro measurements 		*/
    double *irutime,		/* I/O: Array of IRU sample times (from acs_time) */
    VECTOR *iru  		/* I/O: Input IRU and output "smoothed" data 	*/
)

{
double dt = IRU_TIME;		/* Time between IRU samples		*/
double *ptr;			/* Pointer				*/
double sigma2;			/* Process noise associated with roll dot dot*/
double sigma3;			/* Process noise associated with drift dot*/
double att_buff[3][NPCD];	/* Temporary attitude data buffer */
int state[MAX_GYRO];		/* Flag stating what set of matrixes to use*/
int axis,i,j,k,cnt;		/* Counters				*/
int m_size, n_size;		/* size of the arrays			*/
int mf, pt;			/* the major frame in the pcd, and the gyro
				   point in the major frame 		*/
int mf2, pt2;			/* the major frame in the pcd, and the gyro
				   point in the major frame		*/
int natt;			/* Number of output attitude points	*/
int acnt;			/* Counter for attitude data		*/
int scnt;			/* Counter for smoothed data		*/
int status;			/* status of return from a function	*/

/* zero out the matricies */
for (i = 0; i < M_SIZE*M_SIZE; i++)
    {
    Pn[i] = 0.0;
    K[i] = 0.0;
    H[i] = 0.0;
    S[i] = 0.0;
    A[i] = 0.0;
    Q[i] = 0.0;
    R[i] = 0.0;
    for(j = 0; j < MAX_GYRO; j++)
	{
	P[j][i] = 0.0;
	Pn1[j][i] = 0.0;
	}
    }

/*------------------------------------------------------------------------
  Time synchronize the input EPA and IRU data. Use Lagrange
  interpolation to synchronize the samples to one second intervals from
  the ALI model acs_time origin.
-------------------------------------------------------------------------*/
/*------------------------------------------------------------------------
  Compute the number of output attitude data points
-------------------------------------------------------------------------*/
natt = MIN( (int)floor(epatime[(*nepa)-1]/dt), (int)floor(irutime[(*niru)-1]/dt) );

/*------------------------------------------------------------------------
  Synchronize the EPA data
-------------------------------------------------------------------------*/
for( i=0 ; i<(*nepa); i++ )
{
    att_buff[0][i] = epa[i].x;
    att_buff[1][i] = epa[i].y;
    att_buff[2][i] = epa[i].z;
}
for( i=0; i<natt; i++ )
{
    mf = (int)floor((double)i*dt-epatime[0]);
    mf2 = mf - LAGRANGE_PTS/2;
    if ( mf2 < 0 ) mf2 = 0;
    if ( mf2 > (*nepa)-LAGRANGE_PTS ) mf2 = (*nepa) - LAGRANGE_PTS;
    epa[i].x = xxx_Lagrange( &att_buff[0][mf2], &epatime[mf2], LAGRANGE_PTS, (double)i );
    epa[i].y = xxx_Lagrange( &att_buff[1][mf2], &epatime[mf2], LAGRANGE_PTS, (double)i );
    epa[i].z = xxx_Lagrange( &att_buff[2][mf2], &epatime[mf2], LAGRANGE_PTS, (double)i );
}

/*------------------------------------------------------------------------
  Synchronize the IRU data
-------------------------------------------------------------------------*/
for( i=0 ; i<(*niru); i++ )
{
    att_buff[0][i] = iru[i].x;
    att_buff[1][i] = iru[i].y;
    att_buff[2][i] = iru[i].z;
}
for( i=0; i<natt; i++ )
{
    mf = (int)floor((double)i-irutime[0]);
    mf2 = mf - LAGRANGE_PTS/2;
    if ( mf2 < 0 ) mf2 = 0;
    if ( mf2 > (*niru)-LAGRANGE_PTS ) mf2 = (*niru) - LAGRANGE_PTS;
    iru[i].x = xxx_Lagrange( &att_buff[0][mf2], &irutime[mf2], LAGRANGE_PTS, (double)i );
    iru[i].y = xxx_Lagrange( &att_buff[1][mf2], &irutime[mf2], LAGRANGE_PTS, (double)i );
    iru[i].z = xxx_Lagrange( &att_buff[2][mf2], &irutime[mf2], LAGRANGE_PTS, (double)i );
}

/*---------------------------------------------------------------------
  Set up state transition matrix.
  --------------------------------------------------------------------*/
S[0] = 1.0;	S[1] = dt;	S[2] = 0.0;
S[3] = 0.0;	S[4] = 1.0;	S[5] = 0.0;
S[6] = 0.0;	S[7] = 0.0;	S[8] = 1.0;

/*----------------------------------------------------------------------
   The following state noise matrix is for two white noise processes
   driving the network.
 ----------------------------------------------------------------------*/
sigma2 = PNOISE_ATTRATE_SIGMA;
sigma3 = PNOISE_DRIFT_SIGMA;
Q[0] = (dt * dt * dt * dt * sigma2 * sigma2) / 4.0;
Q[1] = (dt * dt * dt * sigma2 * sigma2) / 2.0;
Q[2] = 0.0; 

Q[3] = (dt * dt * dt * sigma2 * sigma2) / 2.0;
Q[4] = (dt * dt * sigma2 * sigma2);
Q[5] = 0.0; 

Q[6] = 0.0;
Q[7] = 0.0;
Q[8] = (dt * dt * sigma3 * sigma3);
/*---------------------------------------------------------------------
  I use this initialization when I have two noise processes driving the
  process.
 ---------------------------------------------------------------------*/
Pn[0] = SNOISE_ATT_SIGMA*A2R;	Pn[1] = 0.0;				Pn[2] = 0.0;
Pn[3] = 0.0;			Pn[4] = SNOISE_ATTRATE_SIGMA*A2R;	Pn[5] = 0.0;
Pn[6] = 0.0;			Pn[7] = 0.0;				Pn[8] = SNOISE_DRIFT_SIGMA*A2R;
Pn[0] *= Pn[0];		/* Square to convert standard deviation to variance */
Pn[4] *= Pn[4];		/* Square to convert standard deviation to variance */
Pn[8] *= Pn[8];		/* Square to convert standard deviation to variance */

/*----------------------------------------------------------------------
   The following state noise matrix is for one white noise process
   driving the network.
   sigma2 = 1.0e+4;
   sigma3 = 1.0e-0;
   Q[0] = 0.0;
   Q[1] = 0.0;
   Q[2] = 0.0;

   Q[3] = 0.0;
   Q[4] = (dt * dt * sigma2 * sigma2);
   Q[5] = 0.0;

   Q[6] = 0.0;
   Q[7] = 0.0;
   Q[8] = 0.0;

   Pn[0] = 5.0e-2;	Pn[1] = 0.0;	Pn[2] = 0.0;
   Pn[3] = 0.0;		Pn[4] = 5.0e+5;	Pn[5] = 0.0;
   Pn[6] = 0.0;		Pn[7] = 0.0;	Pn[8] = 5.0e-4;
 ----------------------------------------------------------------------*/

for( axis=0 ; axis<NAXIS ; axis++ )
{
    /*  Check to see if Euler parameters are OK for first PCD value time synced
	with gyro. */
    /* Initialize attitude state with the first valid quaternion
       measurement.  Loop throught the L0R report PCD structure,
       starting with the first value, to find the first "valid" value.
       Set initial state to this value */

    if (axis == 0)
        Xk[0] = epa[0].x;/* Initialize to first EPA point */
    if (axis == 1)
        Xk[0] = epa[0].y;/* Initialize to first EPA point */
    if (axis == 2)
        Xk[0] = epa[0].z;/* Initialize to first EPA point */

    if (axis == 0)
        Xk[1] = iru[0].x;/* Initialize to first IRU point */
    if (axis == 1)
        Xk[1] = iru[0].y;/* Initialize to first IRU point */
    if (axis == 2)
        Xk[1] = iru[0].z;/* Initialize to first IRU point */

    Xk[2] = 0.0;		/* drift value is initially zero */

    /*-----------------------------------------------------------------
     Set up matrices for case where gyro, and quaternion are
     available.  The sign on H[5] is dependent on whether the
     sign on the drift has been changed or not.
     -----------------------------------------------------------------*/
    m_size = 3;
    n_size = 2;

    H[0] =  1.0;	H[1] =  0.0;	H[2] =  0.0;
    H[3] =  0.0;	H[4] =  1.0;	H[5] = -1.0;

    R[0] = ONOISE_EPA_SIGMA*A2R;
    R[1] = 0.0;
    R[2] = 0.0;
    R[3] = ONOISE_GYRO_SIGMA*A2R;

    R[0] *= R[0];	/* Square to convert std dev to variance */
    R[3] *= R[3];	/* Square to convert std dev to variance */

    for ( cnt=0,i=0; i<natt; cnt++,i++ )
    {
        state[cnt] = 1;
        if (axis == 0)
        {
            z[0] = epa[i].x;
            z[1] = iru[i].x;
        }
        if (axis == 1)
        {
            z[0] = epa[i].y;
            z[1] = iru[i].y;
        }
        if (axis == 2)
        {
            z[0] = epa[i].z;
            z[1] = iru[i].z;
        }

/*----------------------------------------------------------------------
 Filter data.
 ---------------------------------------------------------------------*/
	status = xxx_kalman_gain( Pn, H, R, K, m_size, n_size );
	if (status != SUCCESS)
    	{
    	    xxx_LogStatus("XXXLIB", __FILE__, __LINE__, 
		"Error returned from xxx_kalman_gain");
    	    return(ERROR);
    	}
	
	status = xxx_filter_state( Xk, &X[cnt][0], K, z, H, m_size, n_size );
	if (status != SUCCESS)
    	{
    	    xxx_LogStatus("XXXLIB", __FILE__, __LINE__, 
		"Error returned from xxx_filter_state");
    	    return(ERROR);
    	}
        status = xxx_filter_error_covar( K, H, Pn, &P[cnt][0], m_size, n_size );
	if (status != SUCCESS)
    	{
    	    xxx_LogStatus("XXXLIB", __FILE__, __LINE__, 
		"Error returned from xxx_filter_error_covar");
    	    return(ERROR);
    	}

/*----------------------------------------------------------------------
 Predict error covariance and state matrix for next iteration.
 ---------------------------------------------------------------------*/
	status = xxx_predict_error_covar(S,&P[cnt][0], &Pn1[cnt][0], Q, m_size);
	if (status != SUCCESS)
    	{
    	    xxx_LogStatus("XXXLIB", __FILE__, __LINE__, 
		"Error returned from xxx_predict_error_covar");
    	    return(ERROR);
    	}

	status = xxx_predict_state( S, &X[cnt][0], &Xk1[cnt][0], m_size );
	if (status != SUCCESS)
    	{
    	    xxx_LogStatus("XXXLIB", __FILE__, __LINE__, 
		"Error returned from xxx_predict_state");
    	    return(ERROR);
    	}

        status = xxx_matrix_multiply( H, &Xk1[cnt][0], pre, n_size, m_size, m_size, 1 );
	if (status != SUCCESS)
    	{
    	    xxx_LogStatus("XXXLIB", __FILE__, __LINE__, 
		"Error returned from xxx_matrix_multiply");
    	    return(ERROR);
    	}

        for( j=0 ; j<m_size ; j++ )
            Xk[j] = Xk1[cnt][j];
        for( j=0 ; j<m_size*m_size ; j++ )
            Pn[j] = Pn1[cnt][j];

    }/* end for i */

/*----------------------------------------------------------------------
  NOTE: If refering to the book "Introduction to Random Signal Analysis
        and Kalman filtering" the predicted covariance matrix at k+1
        is stored in Pn1[cnt] in the code while the updated covariance
        matrix at k is stored in P[cnt].  The same holds for the 
        state matrix, predicted state at k+1 is stored in Xk1[cnt]
        the updated estimate at k in X[cnt].
  ----------------------------------------------------------------------*/
    ptr = tmp;
    m_size = 3;
    for( j=0 ; j<m_size ; j++ )
	XN[j] = Xk1[cnt-1][j];

    for( i=natt-1, scnt=cnt-1, k=0 ; i>=0 ; i--, scnt--, k++ ) 
    {
	/*---------------------------------------------------------------------
  	Smooth data.
  	----------------------------------------------------------------------*/

	/* --------------------------------------------------------------------
	
   	If the matrix multiply associated with the H matrix below is not
   	used (it shouldn't be needed for the application, it was just put
   	in as a check for mw) then following code doesn't need to be updated.
	----------------------------------------------------------------------*/
         m_size = 3;
         n_size = 2;

        status = xxx_smooth_gain( &P[scnt][0], &Pn1[scnt][0], S, A, m_size );
	if (status != SUCCESS)
    	{
    	    xxx_LogStatus("XXXLIB", __FILE__, __LINE__, 
		"Error returned from xxx_smooth_gain");
    	    return(ERROR);
    	}

        status = xxx_smooth_state(&X[scnt][0],&Xk1[scnt][0],XN, A, XN1, m_size);
	if (status != SUCCESS)
    	{
    	    xxx_LogStatus("XXXLIB", __FILE__, __LINE__, 
		"Error returned from xxx_smooth_state");
    	    return(ERROR);
    	}

        *ptr = XN1[0];
        ptr++;

        for( j=0 ; j<m_size ; j++ )
            XN[j] = XN1[j];
    }

/*-----------------------------------------------------------------------
  Reverse the order of the smoothed data set.  (The smoothing algorithm
  starts with the last data sample and works forward)
  ----------------------------------------------------------------------*/
    for( i=cnt-1, k=0 ; i>=0 ; i--, k++ )
    {
        if (axis==0)
            iru[k].x = tmp[i];
        else if (axis==1)
            iru[k].y = tmp[i];
        else
            iru[k].z = tmp[i];
    }
} /* for (axis .. */
*niru = natt;
     
return(SUCCESS);
}
