/******************************************************************************
NAME:           exx_movesat

PURPOSE:
Compute the satellite position and velocity at dtime from ephemeris reference
time, by computing each component from Lagrange interpolation. 

RETURN VALUE:
    No return value

NOTES:

ALGORITHM REFERENCES:
******************************************************************************
                        Property of the U.S. Government
                            USGS EROS Data Center
******************************************************************************/
#include "xxx_const.h"
#include "xxx_math.h"

void exx_movesat(
    int nephem,		/*I: Number of ephemeris points */
    double *satpos_x,	/*I: Satellite position X */ 
    double *satpos_y, 	/*I: Satellite position Y */
    double *satpos_z, 	/*I: Satellite position Z */
    double *satvel_x, 	/*I: Satellite velocity X */
    double *satvel_y, 	/*I: Satellite velocity Y */
    double *satvel_z, 	/*I: Satellite velocity Z */
    double dtime,	/*I: Delta time from the reference time */
    VECTOR *satpos,	/*O: New satellite position at "dtime" */
    VECTOR *satvel )	/*O: New satellite velocity at "dtime" */
{
double etime[NPCD];		/* Array of Ephemeris Times */
double *xpos, *ypos, *zpos;	/* Position array pointers */
double *xvel, *yvel, *zvel;	/* Velocity array pointers */
int start_index;		/* Interpolation start index */
int i;				/* Loop index */

/*
 *  Construct the time array
 */
etime[0] = 0.0;
for ( i=1; i<nephem; i++ )
{
    etime[i] = etime[i-1] + EPHEM_TIME;
}

/*
 *  Compute the starting Lagrange index
 */
start_index = (int)floor( dtime/EPHEM_TIME - LAGRANGE_PTS/2 );
if ( start_index < 0 ) start_index = 0;
if ( start_index > nephem-LAGRANGE_PTS )
     start_index = nephem - LAGRANGE_PTS;

/*
 *  Calculate the satellite position
 */
satpos->x = xxx_Lagrange( &satpos_x[start_index], &etime[start_index],
                          LAGRANGE_PTS, dtime );
satpos->y = xxx_Lagrange( &satpos_y[start_index], &etime[start_index],
                          LAGRANGE_PTS, dtime );
satpos->z = xxx_Lagrange( &satpos_z[start_index], &etime[start_index],
                          LAGRANGE_PTS, dtime );

/*
**  Calculate the satellite velocity
*/
satvel->x = xxx_Lagrange( &satvel_x[start_index], &etime[start_index],
                          LAGRANGE_PTS, dtime );
satvel->y = xxx_Lagrange( &satvel_y[start_index], &etime[start_index],
                          LAGRANGE_PTS, dtime );
satvel->z = xxx_Lagrange( &satvel_z[start_index], &etime[start_index],
                          LAGRANGE_PTS, dtime );

return;
}
