/****************************************************************************
NAME: exx_sc_attitude

PURPOSE:
Find the spacecrafts attitude at some reference time.

DEVELOPMENT HISTORY:


RETURN VALUE:
Type = int
Value    Description
-----    -----------
SUCCESS  Successful completion 
ERROR    Operation failed

NOTES:

ALGORITHM REFERENCES:

	NOTE:	The EPA's must be from the same major frame as the
		reference attitude data and the reference ephemeris
		data (the first time of good attitude and ephemeris).
		Or we may have to interpolate sc_roll, sc_pitch, and
		sc_yaw, so that sc_attitude is time sync.ed referenced
		to tm_attitude.  The sc_attitude values are constant
		for entire image.




    -     -     -      -   -     -     -    -   -     -   -     -   -     -
   | Vector|   |Attitude| | Vector|   |      | |       | |       | | Vector|
   |   in  |   |Perturb | |   in  |   | Yaw  | | Pitch | | Roll  | |   in  |
   | Space | = |Matrix  | | Space | = | Rot. | | Rot.  | | Rot.  | | Space |
   |craft's|   |        | |craft's|   | Marix| | Matrix| | Matrix| |craft's|
   |  Body |   |        | |Orbit  |   |      | |       | |       | |Orbit  |
    -     -     -      -   -     -     -    -   -     -   -     -   -     -


                                                               -1
      -     -     -       -    -     -     -       -    -      -    -     -
     |       |   | Space   |  | Vector|   | Space   |  |Attitude|  | Vector|
     |Vector |   | craft's |  |  in   |   | craft's |  |Perturb |  |   in  |
     |  in   | = | Body    |  | Space | = | Orbit   |  |Matrix  |  | Space |
     |ECITOD |   |  to     |  |craft's|   |  to     |  |        |  |craft's|
     |       |   | ECITOD  |  | Body  |   | ECITOD  |  |        |  | Body  |
      -     -     -       -    -     -     -       -    -      -    -     -




*****************************************************************************/
#include <math.h>
#include "xxx_const.h"
#include "xxx_structures.h"
#include "xxx_Types.h"
#include "xxx_LogStatus.h"
#include "xxx_math.h"
#include "exx_init_model.h"

int exx_sc_attitude(
    double *acs_time,	/* I: Attitude control system ref time */
    double *ephem_time, /* I: Ephemeris ref time */
    int nephem,		/* I: Number of ephemeris points */
    double *satpos_x,	/* I: Satellite position X */
    double *satpos_y,	/* I: Satellite position Y */
    double *satpos_z,	/* I: Satellite position Z */
    double *satvel_x,	/* I: Satellite velocity X */
    double *satvel_y,	/* I: Satellite velocity Y */
    double *satvel_z,	/* I: Satellite velocity Z */
    double epatime,	/* I: quaternion time relative to acs_time */
    double *quat,	/* I: quaternion vector */
    double *epa_roll,   /* O: roll calculated for epa point */
    double *epa_pitch,  /* IO: pitch calculated for epa point */
    double *epa_yaw     /* O: yaw calculated for epa point */
    )

{
int status;			/* status of return from funtions	     */
VECTOR	satvel,			/* Spacecraft velocity vector	 	     */
	scox,			/* Spacecraft's orbital x-axis unit vector   */
	scoy,			/* Spacecraft's orbital y-axis unit vector   */
        scoz;           	/* Spacecraft's orbital z-axis unit vector   */

double	scbtoeci[3][3],		/* Spacecraft's body to ECITOD matrix (EPAs) */
	scotoeci[3][3],         /* Spacecraft's orbital to ECITOD matrix     */
	invscotoeci[3][3],      /* Inverse of orbital to ECITOD matrix       */
	invperturb[3][3],       /* Inverse of attitude perturb matrix        */
	magx,			/* Magnitude of the spacecraft x	     */
	magy,			/* Magnitude of the spacecraft y	     */
	magz,			/* Magnitude of the satellite position       */
	epa[4];			/* euler parameters for a point		     */

double	etime;			/* quaternion time relative to ephem_time    */

/* Compute etime */
etime = epatime + exx_gpstime( acs_time ) - exx_gpstime( ephem_time );

/*
  Set up the direction cosines matrix from the attitude control system (ACS)
  reference axes to the ECITOD
*/

  /* set up the epa matrix, so we don't have to type so much later */
epa[0] = quat[0];
epa[1] = quat[1];
epa[2] = quat[2];
epa[3] = quat[3];

/* set up matrix for spacecraft's body to ECI */
scbtoeci[0][0] = epa[0]*epa[0] - epa[1]*epa[1]
               - epa[2]*epa[2] + epa[3]*epa[3];
scbtoeci[0][1] = 2.0*(epa[0]*epa[1] - epa[2]*epa[3]);
scbtoeci[0][2] = 2.0*(epa[0]*epa[2] + epa[1]*epa[3]);

scbtoeci[1][0] = 2.0*(epa[0]*epa[1] + epa[2]*epa[3]);
scbtoeci[1][1] = epa[1]*epa[1] - epa[2]*epa[2]
               + epa[3]*epa[3] - epa[0]*epa[0];
scbtoeci[1][2] = 2.0*(epa[1]*epa[2] - epa[0]*epa[3]);

scbtoeci[2][0] = 2.0*(epa[0]*epa[2] - epa[1]*epa[3]);
scbtoeci[2][1] = 2.0*(epa[1]*epa[2] + epa[0]*epa[3]);
scbtoeci[2][2] = epa[2]*epa[2] + epa[3]*epa[3]
               - epa[0]*epa[0] - epa[1]*epa[1];

/*
  Calculate the Spacecraft's orbital reference system's axis unit vectors
*/
/*
  Calculate the cross products of the satellite ECI position and satellite
  velocity
*/
exx_movesat( nephem, satpos_x, satpos_y, satpos_z, satvel_x,
             satvel_y, satvel_z, etime, &scox, &satvel );
scoz.x = -scox.x;
scoz.y = -scox.y;
scoz.z = -scox.z;

xxx_cross(&scoz,&satvel,&scoy);
xxx_cross(&scoy,&scoz,&scox);

/*
  Calculate the magnitude of the vectors
*/
magx = xxx_norm(&scox);
magy = xxx_norm(&scoy);
magz = xxx_norm(&scoz);

/*
  Set up the second direction cosine matrix
*/
if (magx == 0.0)
{
    xxx_LogStatus("EXXLIB", __FILE__, __LINE__, "Divide by zero with magx");
    return(ERROR);
}
if (magy == 0.0)
{
    xxx_LogStatus("EXXLIB", __FILE__, __LINE__, "Divide by zero with magy");
    return(ERROR);
}
if (magz == 0.0)
{
    xxx_LogStatus("EXXLIB", __FILE__, __LINE__, "Divide by zero with magz");
    return(ERROR);
}
scotoeci[0][0] = scox.x / magx;
scotoeci[0][1] = scoy.x / magy;
scotoeci[0][2] = scoz.x / magz;

scotoeci[1][0] = scox.y / magx;
scotoeci[1][1] = scoy.y / magy;
scotoeci[1][2] = scoz.y / magz;

scotoeci[2][0] = scox.z / magx;
scotoeci[2][1] = scoy.z / magy;
scotoeci[2][2] = scoz.z / magz;

/*
  Inverse the second direction cosine matrix
*/
status = xxx_invmatrix(scotoeci,invscotoeci);
if (status != SUCCESS) 
{
    xxx_LogStatus("EXXLIB", __FILE__, __LINE__, 
        "Error inverting scotoeci matrix");
    return(ERROR);
}

/*
  Multiply the inverse and direction cosine matrix
  Inverse perturbation matrix is the same as Body to Orbit (scb2sco)
*/
xxx_multmatrix((const double(*)[3])invscotoeci,(const double(*)[3])scbtoeci,invperturb);

/*
  Calculate the attitude roll, pitch, and yaw
*/
*epa_roll  = -atan2(invperturb[2][1],invperturb[2][2]);
*epa_pitch =  asin(invperturb[2][0]);
*epa_yaw   = -atan2(invperturb[1][0],invperturb[0][0]);

#ifdef DEBUG
printf("Quaternion Attitude:  %lf %lf %lf %lf\n",
        epatime, *epa_roll, *epa_pitch, *epa_yaw);
#endif
 
return(SUCCESS);
}
