/****************************************************************************
NAME: exx_calc_nutang

PURPOSE:
Provides fast evaluation of the nutation components according to
      the 1980 IAU Theory of Nutation.

DEVELOPMENT HISTORY:
$LOG$

RETURN VALUE:
No value returned.

NOTES:
      Code is from Navel Observatory Vector Astronomy Subroutines (NOVAS)
      C Version 1.0
      June 1996

      U. S. Navel Observatory
      Astronomical Applications Dept.
      3450 Massachusetts Ave., NW
      Washington, DC  20392-5420

ALGORITHM REFERENCES:
      Seidelmann, P.K., (1981) 1980 IAU Theory of Nutaion: The Final 
	Report of the IAU Working Group on Nutation, pp. 79-106.
      Explanatory Supplement to the Astronomical Almanac, 1992
      Kaplan, G. H. et. al. (1989). Astron. Journ. Vol. 97, 
         pp. 1197-1210, and references therein.
      Kaplan, G. H. "NOVAS: Naval Observatory Vector Astrometry
         Subroutines"; USNO internal document dated 20 Oct 1988;
         revised 15 Mar 1990.
      Miller, B. R. (1989). Proceedings of the ACM-SIGSAM International
         Symposium on Symbolic and Algebraic Computation; pp. 199-206.
****************************************************************************
                Landsat 7 Image Assessment System (IAS) Software
                        Property of the U.S. Government
                                NASA/GSFC/EDC
*****************************************************************************/
#include <math.h>
#include "exx_geometric.h" /* prototype for this function */

/* Mean anomaly of the moon for epoch J2000 */
#define L_MOON_2000 8328.691422883896  /* radians/Julian Century
				   	p.94&96 Seidelmann */
/* Mean anomaly of the sun for epoch J2000 */
#define L_SUN_2000  628.3019560241842  /* radians/Julain Century 
					p.94&96 Seidelmann */
/* Mean longitude of moon's node for epoch J2000 */
#define OMEGA_MOON_2000 -33.75704593375351 /* radians/Julian Century 
					p.94&97 Seidelmann */
/* Mean longitude of the moon - mean longitude of moon's node */
#define F_2000 8433.466158318453 /* radians/Julain Century 
					p.94&97 Seidelmann */
/* Mean longitude of the moon - mean longitude of the sun */
#define D_2000 7771.377146170642 /* rad/Julian Century
					p.94&97 Seidelmann */



void exx_calc_nutang (
    double tdbtime,  	/*TDB (Barycentric Dynamic Time) time in Julian 
			centuries since J2000.0 */

    /* the two components of nutation */
    double *longnutation,/* Nutation in longitude in seconds of arc. (psi) */
    double *obliqnutation/*  Nutation in obliquity in seconds of arc.(epsilon) */
)

{
   /* NOTE: when referencing the letters in "" they come from the table in
      Seidelmann p.81 (also in Expl. Supp)  That will look like this...

	   Argument multiple of				Coefficient for
	 ________________________		Longitude	    Obliquity
        {                        }		
   No.  l    l'   F    D    Omega                  Si                   Ci
   i    ai   bi   ci   di   ei   (days)      Sa          Sb        Ca       Cb 
   ____________________________________________________________________________
   1    0    0    0    0    1    6798.4    -171996    -174.2T    92025    +8.9T
   ...
   */

   /* NOTE: if comparing the constants  below to the references of Seidelmann or the 
      Explanatory Supplement to the Astrnomical Alamanac - the elements of the
      arrays are in a different order :( */

   /* "Sa": Longitude of Nutation constants (p. 81 Seidelmann) */ 
   double clng[106] = {1.0,   1.0,  -1.0, -1.0,   1.0,  -1.0,  -1.0,
                      -1.0,  -1.0,  -1.0, -1.0,   1.0,  -1.0,   1.0,
                      -1.0,   1.0,   1.0, -1.0,  -1.0,   1.0,   1.0,
                      -1.0,   1.0,  -1.0,  1.0,  -1.0,  -1.0,  -1.0,
                       1.0,  -1.0,  -1.0,  1.0,  -1.0,   1.0,   2.0,
                       2.0,   2.0,   2.0,  2.0,  -2.0,   2.0,   2.0,
                       2.0,   3.0,  -3.0, -3.0,   3.0,  -3.0,   3.0,
                      -3.0,   3.0,   4.0,  4.0,  -4.0,  -4.0,   4.0,
                      -4.0,   5.0,   5.0,  5.0,  -5.0,   6.0,   6.0,
                       6.0,  -6.0,   6.0, -7.0,   7.0,   7.0,  -7.0,
                      -8.0,  10.0,  11.0, 12.0, -13.0, -15.0, -16.0,
                     -16.0,  17.0, -21.0,-22.0,  26.0,  29.0,  29.0,
                     -31.0, -38.0, -46.0, 48.0, -51.0,  58.0,  59.0,
                      63.0,  63.0,-123.0,129.0,-158.0,-217.0,-301.0,
                    -386.0,-517.0, 712.0,1426.0,2062.0,-2274.0,
                  -13187.0,-171996.0},

	/* "Sb": Longitude of nutation constants in time (p. 81, Seidelmann) */
      	clngx[14]={ 0.1,-0.1,0.1,0.1,0.1,0.1,0.2,-0.2,-0.4,0.5,1.2,
                 -1.6,-3.4,-174.2},

	/* "Ca": Obliquity of nutation constants (p. 81 Seidelmann) */
       cobl[64]={    1.0,    1.0,    1.0,   -1.0,   -1.0,   -1.0,
                     1.0,    1.0,    1.0,    1.0,    1.0,   -1.0,
                     1.0,   -1.0,    1.0,   -1.0,   -1.0,   -1.0,
                     1.0,   -1.0,    1.0,    1.0,   -1.0,   -2.0,
                    -2.0,   -2.0,    3.0,    3.0,   -3.0,    3.0,
                     3.0,   -3.0,    3.0,    3.0,   -3.0,    3.0,
                     3.0,    5.0,    6.0,    7.0,   -7.0,    7.0,
                    -8.0,    9.0,  -10.0,  -12.0,   13.0,   16.0,
                   -24.0,   26.0,   27.0,   32.0,  -33.0,  -53.0,
                    54.0,  -70.0,  -95.0,  129.0,  200.0,  224.0,
                  -895.0,  977.0, 5736.0,92025.0},

	/* "Cb": Obliquity of nutation constants in time (p.81 Seidelmann) */
       coblx[8]={ -0.1, -0.1,  0.3,  0.5, -0.5, -0.6, -3.1,  8.9};

   short int i,ii,i1,i2,iop;
   short int nav1[10]={0,0,1,0,2,1,3,0,4,0}, /* which element of a to use */
       nav2[10]={ 0, 0, 0, 5, 1, 1, 3, 3, 4, 4},
       nav[183]={ 2, 0, 1, 1, 5, 2, 2, 0, 2, 1, 0, 3, 2, 5, 8, 1,17, 8,
                  1,18, 0, 2, 0, 8, 0, 1, 3, 2, 1, 8, 0,17, 1, 1,15, 1,
                  2,21, 1, 1, 2, 8, 2, 0,29, 1,21, 2, 2, 1,29, 2, 0, 9,
                  2, 5, 4, 2, 0, 4, 0, 1, 9, 2, 1, 4, 0, 2, 9, 2, 2, 4,
                  1,14,44, 2, 0,45, 2, 5,44, 2,50, 0, 1,36, 2, 2, 5,45,
                  1,37, 2, 2, 1,45, 2, 1,44, 2,53, 1, 2, 8, 4, 1,40, 3,
                  2,17, 4, 2, 0,64, 1,39, 8, 2,27, 4, 1,50,18, 1,21,47,
                  2,44, 3, 2,44, 8, 2,45, 8, 1,46, 8, 0,67, 2, 1, 5,74,
                  1, 0,74, 2,50, 8, 1, 5,78, 2,17,53, 2,53, 8, 2, 0,80,
                  2, 0,81, 0, 7,79, 1, 7,81, 2, 1,81, 2,24,44, 1, 1,79,
                  2,27,44},

	/* tells what sin, "s", element to use with the longitudinal
	   time constants "Sb" */ 
      llng[106]={ 57, 25, 82, 34, 41, 66, 33, 36, 19, 88, 18,104, 93,
                  84, 47, 28, 83, 86, 69, 75, 89, 30, 58, 73, 46, 77,
                  23, 32, 59, 72, 31, 16, 74, 22, 98, 38, 62, 96, 37,
                  35,  6, 76, 85, 51, 26, 10, 13, 63,105, 52,102, 67,
                  99, 15, 24, 14,  3,100, 65, 11, 55, 68, 20, 87, 64,
                  95, 27, 60, 61, 80, 91, 94, 12, 43, 71, 42, 97, 70,
                   7, 49, 29,  2,  5, 92, 50, 78, 56, 17, 48, 40, 90,
                   8, 39, 54, 81, 21,103, 53, 45,101,  0,  1,  9, 44,
                  79,  4},
	/* tells what sin, "s", element to use with the longitudinal
	   time constants "Sb" */ 
      llngx[14]={ 81, 7, 97, 0, 39, 40, 9, 44, 45,103,101, 79, 1, 4},

      lobl[64]={  51, 98, 17, 21,  5,  2, 63,105, 38, 52,102, 62, 96,
                  37, 35, 76, 36, 88, 85,104, 93, 84, 83, 67, 99,  8,
                  68,100, 60, 61, 91, 87, 64, 80, 95, 65, 55, 94, 43,
                  97,  0, 71, 70, 42, 49, 92, 50, 78, 56, 90, 48, 40,
                  39, 54,  1, 81,103, 53, 45,101,  9, 44, 79,  4},
      loblx[8] ={ 53,  1,103,  9, 44,101, 79,  4};

   double a[5],angle,
	cc, /* cos(angle)*cos(angle) */
	ss1,/* sin(angle)*sin(angle) */
	cs, /* cos(angle)*sin(angle) */
	sc, /* sin(angle)*cos(angle) */
	c[106], /* cos terms */
	s[106], /* sin terms */
	lng,lngx,obl,oblx;

   /* the array "a" is the set of fundamental arguments in the 
	FK5 reference system */
   /* "l": mean longitude of the Moon minusthe mean longitude of the moon's perigee
      (p.98 Seidelmenn) */
   a[0] = 2.3555483935439407 + tdbtime * (L_MOON_2000) + 
				tdbtime * tdbtime * (1.517951635553957e-4 +
				3.1028075591010306e-7 * tdbtime);

   /* "l'": mean longitude of the Sun minus the mean longitude of the Sun's perigee
      (p.98 Seidelmenn) */
   a[1] = 6.240035939326023 + tdbtime * (L_SUN_2000) +
                            tdbtime * tdbtime * (-2.7973749400020225e-6 - 
                            5.817764173314431e-8 * tdbtime);

   /* "F": Mean longitude of the sun minus the mean longitude of the moons's node
      (p.98 Seidelmenn) */
   a[2] = 1.6279019339719611 + tdbtime * (F_2000)  + 
                             tdbtime * tdbtime * (-6.427174970469119e-5 + 
                             5.332950492204896e-8 * tdbtime);

   /* "D": Mean elongation of the Moon from the Sun (p.98 Seidelmann) */
   a[3] = 5.198469513579922 + tdbtime * (D_2000) + 
                            tdbtime * tdbtime * (-3.340851076525812e-5 + 
                            9.211459941081184e-8 * tdbtime);

   /* "OMEGA": Longitude of the mean ascending node of the lunar orbit on 
	the ecliptic, measured from the mean equinox of date (p.98 Seidelmann) */
   a[4] = 2.1824386243609943 + tdbtime * (OMEGA_MOON_2000) + 
                             tdbtime * tdbtime * (3.614285992671591e-5 + 
                             3.878509448876288e-8 * tdbtime);

   

/* **********************************************************************
   p.115 of the Expl. Supp equations:

		    n
		   --- 
	delta psi =>   (Sa+Sb*T)sin(Ai)
		   ---
		   i=1
	(delta psi = nutation in longitude)

		        n
		       --- 
	delta epsilon =>   (Ca+Cb*T)cos(Ai)
		       ---
		       i=1
	(delta epslon = nutation in obliquity)
	

	where Ai = ai*l + bi*l' + ci*F + di*D + ei*omega


  ** The following code is a variation on the above
********************************************************************** */

/* Calculating Ai's and sin and cos(Ai) (Expl. Supp. p. 115) */
   i = 0;
   for (ii = 0; ii < 10; ii += 2)
   {
      angle = a[nav1[ii]] * (double) (nav1[1+ii]+1);
      c[i] = cos (angle);
      s[i] = sin (angle);
      i += 1;
   }

   i = 5;
   for (ii = 0; ii < 10; ii += 2)
   {
      i1 = nav2[ii];
      i2 = nav2[1+ii];

      c[i] = c[i1] * c[i2] - s[i1] * s[i2];
      s[i] = s[i1] * c[i2] + c[i1] * s[i2];
      i += 1;
   }

   i = 10;
   for (ii = 0; ii < 183; ii += 3)
   {
      iop = nav[ii];
      i1 = nav[1+ii];
      i2 = nav[2+ii];
      switch (iop)
      {
         case 0:
            c[i] = c[i1] * c[i2] - s[i1] * s[i2];
            s[i] = s[i1] * c[i2] + c[i1] * s[i2];
            i += 1;
            break;
         case 1:
            c[i] = c[i1] * c[i2] + s[i1] * s[i2];
            s[i] = s[i1] * c[i2] - c[i1] * s[i2];
            i += 1;
            break;
         case 2:
            cc = c[i1] * c[i2];
            ss1 = s[i1] * s[i2];
            sc = s[i1] * c[i2];
            cs = c[i1] * s[i2];
            c[i] = cc - ss1;
            s[i] = sc + cs;
            i += 1;
            c[i] = cc + ss1;
            s[i] = sc - cs;
            i += 1;
            break;
      }
      if (iop == 3)
         break;
   }

/* calculating "Sa" sum */
   lng = 0.0;
   for (i = 0; i < 106; i++)
      lng += clng[i] * s[llng[i]];

/* calculating "Sb" sum */
   lngx = 0.0;
   for (i = 0; i < 14; i++)
      lngx += clngx[i] * s[llngx[i]];

/* calculating "Ca" sum */
   obl = 0.0;
   for (i = 0; i < 64; i++)
      obl += cobl[i] * c[lobl[i]];

/* calculating "Cb" sum */
   oblx = 0.0;
   for (i = 0; i < 8; i++)
      oblx += coblx[i] * c[loblx[i]];

/* calculating delta psi(nutation in longitude) 
   and delta epsilon (nutation in obliquity) */
   *longnutation = (lng + tdbtime * lngx) / 10000.0;
   *obliqnutation = (obl + tdbtime * oblx) / 10000.0;

   return;
}
