/****************************************************************************
NAME: exx_calcgha

PURPOSE:
Calculate the Greenwich Apparent Sidereal Time at reftime.  Calculate
all the angular parameters needed for transformation of Precession,
Nutation and Earth rotation.

DEVELOPMENT HISTORY:
$Log: exx_calcgha.c,v $
Revision 1.1  2000/11/22 22:20:43  storey
Original version - derived from gxx_calcgha.c


RETURN VALUE:
No value returned.

NOTES:
      Code is from Navel Observatory Vector Astronomy Subroutines (NOVAS)
      C Version 1.0
      June 1996

      U. S. Navel Observatory
      Astronomical Applications Dept.
      3450 Massachusetts Ave., NW
      Washington, DC  20392-5420

ALGORITHM REFERENCES:
	Landsat 7 Program Coordinate System Standard, Martin Marietta, 1993,
	Page 4-16.

	Explanatory Supplement to the Astronomical Almanac, 1992.

	DMA Technical Report, Supplement to Department of Defense World
	Geodetic System 1984 Technical Report, Part 1
****************************************************************************
                        Property of the U.S. Government
                            USGS EROS Data Center
*****************************************************************************/
#include <stdio.h>
#include <math.h>
#include "xxx_const.h"
#include "xxx_Types.h"
#include "xxx_math.h"
#include "exx_geometric.h"

void exx_calcgha(
    const double reftime[],/* I: Reference time		     		*/
    double ut1_utc, 	/* I: UT1-UTC, in seconds, due to variation	*/
		     	/*    of Earth's spin rate		     	*/
    double *zi, 	/* O: Precession angles (radians)	     	*/
    double *theta,	/* O: Precession angles (radians)	     	*/
    double *zeta,	/* O: Precession angles (radians)	     	*/
    double *true_obliquity,/* O: true obliquity of the ecliptic   	*/
    double *mean_obliquity,/* O: Mean ecliptic 				*/
    double *longnut,	/* O: Nutation in logitude (radians)	     	*/
    double *gha)	/* O: Greenwich Apparent Sidereal Time (radians)*/

{
double	fjd,		/* Full Julian date				     */
	dt,		/* Delta time from epoch J2000.0		     */
	tf,		/* Total fractional day (from seconds)		     */
	we,		/* Sidereal Earth rotation rate, degree/day	     */
	h0,		/* Mean sidereal angle at the start of the day	     */
	oblnut,		/* Nutation in obliquity, in degree,		     */
	dh;		/* Sidereal time change due to nutation, in deg	     */
	
int 	year,		/* Year						     */
	mo = 1,		/* First month					     */
	day = 1,	/* First day					     */
	igha;		/* Integer Greenwich hour angle			     */
double  tdbtime;	/* TDB time=(Julian ephem date-EPOCH_2000)/JULIAN_CENTURY */

/*
  Get the year from the reference time
*/
year = (int)reftime[0];

/*
  Get the full julian date of Jan. 1
*/
fjd = xxx_fulljd(year,mo,day,0);

/*
  Add the current day (one is subtracted for day of year)
*/
fjd += (reftime[1] - 1.0);

/*
  Calculate the delta time to epoch J2000.0
*/
dt = fjd - EPOCH_2000;

/*
  Fractional day from Seconds
*/
tf = reftime[2] / SEC_PER_DAY;

/*
  Correct the UTC to UT1 using the Navel Observatory nutation model
*/
tf += (ut1_utc / SEC_PER_DAY);

/* calculate the full julian date and TBD time */
tdbtime = (fjd - EPOCH_2000)/JULIAN_CENTURY;

/*
  Calculate the Nutation angles
*/
*longnut = 0.0;
oblnut = 0.0;
exx_calc_nutang(tdbtime, longnut, &oblnut);
*longnut *= A2R;
oblnut *= A2R;


/* the mean obliquity in radians (p.114 Expl. Supp) 1908 IAU theoryof nutation */
*mean_obliquity = 23.4393022222*D2R - 1.30041666667e-2*D2R * tdbtime 
	- 1.63888888889e-7*D2R * tdbtime * tdbtime 
	- 5.03611111111e-7*D2R * tdbtime * tdbtime * tdbtime;

/* true obliquity in radians (p.114 Expl. Supp) */
*true_obliquity = *mean_obliquity + oblnut;


/*
  Calculate the Precession angles, DMA Document p.A-30 
  (same as Martin Marieta equations - just easier to understand
  where the numbers are coming from (Martin Marieta had the tdb time
  combined with the coefficients of the equations))
*/
*zeta = (2306.2181*A2R * tdbtime + .30188*A2R * tdbtime * tdbtime + 
	.017998*A2R * tdbtime * tdbtime * tdbtime);
*zi = (2306.2181*A2R * tdbtime + 1.09468*A2R* tdbtime * tdbtime + 
	.018203*A2R * tdbtime * tdbtime* tdbtime);
*theta = (2004.3109*A2R * tdbtime - .42665*A2R * tdbtime * tdbtime -
	.041833*A2R * tdbtime * tdbtime * tdbtime);


/*
  Calculate the Greenwich Apparent Sidereal Time
*/
we = 360.98564736629 + 0.58158e-12 * dt;
h0 = 100.460618375 + 0.9856473662862 * dt + 0.29079e-12 * dt * dt;
dh = atan(cos(*true_obliquity) * tan(*longnut));
*gha = h0 + we * tf + dh * DPR;
*gha = *gha / 360.0;
igha = (int)(*gha);
*gha = (*gha - igha) * 360.0;


/*
  make sure angle is 0-360 and convert to radians
*/
if (*gha < 0.0)
    *gha += 360.0;

*gha *= RPD;

}
