/*******************************************************************************
NAME:		exx_centh2det

PURPOSE:
Calculate the geodetic latitude and height of a point above the ellipse,
given the geocentric latitude and the radius  

DEVELOPMENT HISTORY:
$Log: exx_centh2det.c,v $
Revision 1.2  2001/01/03 19:21:38  storey
Added #include xxx_LogStatus.h to declare xxx_LogStatus().

Revision 1.1  2000/11/22 22:20:45  storey
Original version - derived from gxx_centh2det.c


RETURN VALUE:
Type = int 
Value		Description
-----		-----------
SUCCESS		Latitude and height calculated
ERROR		Error calculating satellite height

ALGORITHM REFERENCES:
	ESCOBAL, METHODS OF ORBIT DETERMINATION (page 398)
********************************************************************************
                       Property of the U.S. Government
                           USGS EROS Data Center
*******************************************************************************/
#include "exx_geometric.h"
#include "xxx_LogStatus.h"

#define EPSILON 0.0001		/* tolerance to terminate height calculation */

int exx_centh2det(double latc,		/* I: Geocentric latitude (radians)  */
	 	  double radius,	/* I: Radius of the point (meters)   */
		  const CPF *cpf, 	/* I: Cal parm file constants	     */
	          double *latd,   	/* O: Geodetic latitude (radians)    */
	          double *height) 	/* O: Height of the point (meters)   */
{
double	rsq,		/* Radius of the point squared			     */
	radius1,	/* Temporary radius 				     */
	radius2,	/* Calculated radius of the point		     */
	r2sq,		/* Calculated radius of the point squared	     */
	height2,	/* Height of the satellite			     */
	phis,		/* Latitude of the satellite			     */
	phip,		/* Latitude of the point			     */
	dphip,		/* Delta latitude of the point			     */
	dphis;		/* Delta latitude of the satellite		     */

double	last_height = 0.0; /* Previous height of the satellite		     */

int	i;		/* Looping variable				     */

/*
  Check for invalid radius 
*/
if (radius == 0.0)
    {
    xxx_LogStatus("GPSLIB", __FILE__, __LINE__, "Invalid point radius");
    return(ERROR);
    }

/*
  Save values for calculations
*/
phip = latc;
rsq  = radius * radius;

for (i = 0; i < 20; i++)
{
    /*
      Calculate the radius at the target
    */
    if (exx_earthradius(phip, cpf, &radius2) == ERROR)
    {
	xxx_LogStatus("EXXLIB", __FILE__, __LINE__, 
	    "Error returned from gxx_earthradius");
	return(ERROR);
    }
    r2sq = radius2 * radius2;

    /*
      Calculate the latitude of the satellite
    */
    phis = atan2(tan(phip), (1.0-cpf->Earth_Constants.Eccentricity));
    dphis = phis - phip;

    /*
      Calculate the height of the satellite
    */
    radius1 = rsq - r2sq * sin(dphis) * sin(dphis);
    if (radius1 < 0.0)
    {
	xxx_LogStatus("EXXLIB", __FILE__, __LINE__, 
	    "Error calculating satellite height");
	return(ERROR);
    }
    else
        height2 = sqrt(radius1) - radius2 * cos(dphis);

    /*
      Check the tolerance
    */
    if (fabs(height2 - last_height) <= EPSILON)
	break;

    /*
      Save the current height
    */
    last_height = height2;

    /*
      Calculate the delta latitude correction for the new height
    */
    dphip = asin(height2 / radius * sin(dphis));
    phip = latc - dphip;
}

/*
  Return the new latitude and height
*/
*latd = phis;
*height = height2;

return(SUCCESS);
}
