/****************************************************************************
NAME:                           exx_decdeg

PURPOSE:  Convert angles to total degrees.

DEVELOPMENT HISTORY:
$Header: /llite/cvseo1/eo1/src/global/exxLib/exxGeo/exx_decdeg.c,v 1.2 2001/01/03 19:22:12 storey Exp $
AUTHOR        Change-ID    DATE     Description of Change
------        ---------  --------   ---------------------

RETURN VALUE:   Type = int
Value    Description
-----    -----------
SUCCESS  Successful completion
ERROR    Operation failed

ALGORITHM DESCRIPTION:
        Recieve an angle in seconds, minutes, or DMS
        Convert the angle to total degrees (in place).
        The angle is then checked to be sure it is within the limits of
          its use (LAT, LON, or DEGREES).

ALGORITHM REFERENCES:

******************************************************************************
                        Property of the U.S. Government
                            USGS EROS Data Center
*****************************************************************************/
#include "exx_geometric.h"
#include "xxx_LogStatus.h"

int exx_decdeg (
  double *angle,    /* I/O: Angle in seconds, minutes, or DMS    */
  char *coform,     /* I: Angle units (SEC, MIN, DMS)            */
  char *type)       /* I: Angle usage type (LAT, LON, or DEGREES)*/

{
  float second;     /* Second of DMS angle value                  */
  float upper;      /* Upper bound of the angle value for its use */
  float lower;      /* Lower bound of the angle value for its use */

  int  degree;      /* Degree of DMS angle value                  */
  int  minute;      /* Minute of DMS angle value                  */
  short sign;       /* Sign of the angle                          */
  char ErrMsg[STRLEN];  /* Error Message                          */

  /* Find the upper and lower bound limits for the angle based on its
   * usage type.
   */
  if (strncmp (type,"LAT",3) == 0) 
  { /* LAT */
    upper = 90.0;
    lower = -90.0;
  } /* LAT */
  else if (strncmp (type,"LON",3) == 0)
  { /* LON */
    upper = 180.0;
    lower = -180.0;
  } /* LON */
  else
  { /* DEGREES */
    upper = 360.0;
    lower = 0.0;
  }

  /* Convert the angle to total degrees based on whether the coform is
   * minutes, seconds, or DMS.
   */
  if (strncmp (coform,"MIN",3) == 0)
    *angle = *angle / 60.0;
  else if (strncmp (coform,"SEC",3) == 0)
    *angle = *angle / 3600.0;
  else if (strncmp (coform,"DMS",3) == 0)
  { /* DMS */
    if (*angle < 0)
    {
      sign = -1;
      *angle = *angle * -1;
    }
    else
      sign = 1;

    degree = (int) (*angle / 1000000);
    *angle = *angle - (degree * 1000000);
    minute = (int) (*angle / 1000);
    second = *angle - (minute * 1000);
    *angle = sign * (degree + (minute/60.0) + (second/3600.0));
  } /* DMS */

  /* Check to make sure the angle is within the limits of its use (LAT, LON,
   * or DEGREES)
   */
  if ((*angle > upper) || (*angle < lower))
  {
    strcpy (ErrMsg, "Illegal coordinate value in decdeg.");
    xxx_LogStatus ("EXXLIB","DECDEG", __LINE__, ErrMsg);
    return (ERROR);
  }

  return (SUCCESS);
}
