/****************************************************************************
NAME:                           exx_degdms

PURPOSE:  To convert total degrees, total minutes, or total seconds
          to packed degress, minutes, seconds.

DEVELOPMENT HISTORY:
AUTHOR        Change-ID    DATE     Description of Change
------        ---------  --------   ---------------------

RETURN VALUE:   Type = int
Value    Description
-----    -----------
SUCCESS  Successful completion
ERROR    Operation failed

ALGORITHM DESCRIPTION:
        Receive an angle in seconds, minutes, or degrees
        Convert it to DMS.
        The angle is then checked to be sure it is within the limits
          of its use (LAT, LON, or DEGREES).

ALGORITHM REFERENCES:

******************************************************************************
                        Property of the U.S. Government
                           USGS EROS Data Center
*****************************************************************************/
#include "exx_geometric.h"
#include "xxx_LogStatus.h"

#define MAXMIN 60060.0
#define MAXSEC 60.0
#define MINMIN -60060.0
#define MINSEC -60.0

int exx_degdms (
  double *deg,    /* I: Angle in seconds, minutes, or degrees      */
  double *dms,    /* O: Angle converted to DMS                     */
  char *code,     /* I: Angle units (SEC, MIN, DEG, DMS)           */
  char *check)    /* I: Angle usage type (LAT, LON, or DEGREES)    */ 

{ /* degdms */
  double tsec;    /* Total seconds in the input angle              */
  double maxdms;  /* Upper bound of the angle value for its use    */
  double mindms;  /* Lower bound of the angle value for its use    */
  double tempmin; /* Minutes value of DMS                          */
  double tempsec; /* Seconds value of DMS                          */

  int tdeg;      /* Degrees in the input angle                    */
  int tmin;      /* Minutes in the input angle                    */
  int sign;      /* Sign of the input angle                       */

  char ErrMsg[STRLEN];  /* Error Message                           */

  /* Find the upper and lower bound limits for the angle based on its
   * usage type (LAT, LON, DEGREES).
   */
  if (strcmp (check,"LAT") == 0)
  {
    maxdms = 90000000;
    mindms = -90000000;
  }
  else if (strcmp (check,"LON") == 0)
  {
    maxdms = 180000000;
    mindms = -180000000;
  }
  else
  { /* DEGREES */
    maxdms = 360000000;
    mindms = 0;
  }

  /* Convert the angle to DMS based on whether the coform is minutes,
   * seconds, degrees, or DMS.
   */
  if (strcmp (code,"DMS") != 0)
  {          /* DEG, MIN, or SEC ... If MIN or SEC, then convert to degrees. */
    if (strcmp (code,"MIN") == 0)
      *deg = *deg / 60.0;
    if (strcmp (code,"SEC") == 0)
      *deg = *deg / 3600.0;

    /* Extract the deg, min, and sec portions of the angle.
     */
    exx_find_deg (*deg, &tdeg);
    exx_find_min (*deg, &tmin);
    exx_find_sec (*deg, &tsec);

    /* Find the sign of the angle based on the degrees.
     */
    sign = 1;
    if (tdeg < 0)
      sign = -1;

    /* Calculate DMS from the degree, minutes, seconds values.
     */
    tdeg = abs (tdeg);
    *dms = ((tdeg * 1000000) + (tmin * 1000) + tsec) * sign;

    /*  Check to be sure the coordinate is within the bounds.
     */
    if ((*dms > maxdms) || (*dms < mindms))
    {
      strcpy (ErrMsg, "Invalid coordinate value in degdms.");
      xxx_LogStatus ("EXXLIB", __FILE__, __LINE__, ErrMsg);
      return (ERROR);
    }
  }
  else
  { /* DMS */
    *dms = *deg;

    /*  Check to be sure the coordinate is within its bounds.
     */
    if ((*dms > maxdms) || (*dms < mindms))
    {
      strcpy (ErrMsg, "Invalid coordinate value in degdms.");
      (void) xxx_LogStatus ("EXXLIB", __FILE__, __LINE__, ErrMsg);
      return (ERROR);
    }

    /* Verify that the DMS value is positive for LAT and LON.
     */
    if ((strcmp (check, "LAT") != 0) && (strcmp (check, "LON") != 0))
    {
      if (*dms <= 0)
      {
        strcpy (ErrMsg, "Invalid coordinate value in degdms.");
        (void) xxx_LogStatus ("EXXLIB", __FILE__, __LINE__, ErrMsg);
        return (ERROR);
      }
    }

    /* Parse out the minutes value from DMS and check against MAXMIN.
     */
    tempmin = *dms - (((int)(*dms/1000000)) * 1000000);
    if ((tempmin > MAXMIN) || (tempmin < MINMIN))
    {
      strcpy (ErrMsg, "Invalid coordinate value in degdms.");
      (void) xxx_LogStatus ("EXXLIB", __FILE__, __LINE__, ErrMsg);
      return (ERROR);
    }

    /* Parse out the seconds value from DMS and check against MAXSEC.
     */
    tempsec = *dms - (((int)(*dms/1000)) * 1000);
    if ((tempsec > MAXSEC) || (tempsec < MINSEC))
    {
      strcpy (ErrMsg, "Invalid coordinate value in degdms.");
      (void) xxx_LogStatus ("EXXLIB", __FILE__, __LINE__, ErrMsg);
      return (ERROR);
    }
  }

  return (SUCCESS);
}
