/****************************************************************************
NAME: exx_efc2ic

PURPOSE:
Transform Earth-fixed cartesian coordinates (ECR/Earth-fixed) to
Earth-centered inertial cartesian coordinates (ECI/true-of-date)
at the specified GMT (UTC) time.

DEVELOPMENT HISTORY:
$Log: exx_efc2ic.c,v $
Revision 1.1  2000/11/22 22:20:49  storey
Original version - new routine inverse of exx_ic2efc


RETURN VALUE:
Type = int
Value    Description
-----    -----------
SUCCESS  Successful completion 
ERROR    Operation failed

NOTES:

ALGORITHM REFERENCES:
	Landsat 7 Program Coordinate System Standard, Martin Marietta, 1993,
	Page 4-16.
*****************************************************************************
                        Property of the U.S. Government
                            USGS EROS Data Center
*****************************************************************************/
#include <math.h>
#include "xxx_const.h"
#include "xxx_Types.h"
#include "xxx_LogStatus.h"
#include "xxx_math.h"
#include "exx_geometric.h"

int exx_efc2ic(
    double xp,			/* I: Earth's true pole offset from mean pole, */
				/* in arc second			       */
    double yp,			/* I: Earth's true pole offset from mean pole, */
				/* in arc second			       */
    double ut1_utc,		/* I: Irregular Earth rotation, in time second */
    const VECTOR *craft_pos,	/* I: Satellite position in ECR, input         */
    const VECTOR *craft_vel,	/* I: Satellite velocity in ECR, input         */
    const double ephem_time[3],	/* I: Time of the ephemeris point, 	       */
			        /* in year, day and second, input	       */
    VECTOR *ic_satpos,	/* O: Satellite position in ECI, output        */
    VECTOR *ic_satvel)	/* O: Satellite velocity in ECI, output        */


{
double    gast,			/* Greenwich apparent siderial time, in rad */
          zi, theta, zeta,	/* angles in precession, in radian	    */
          e, ebar, psi;		/* angles in nutation, in radian	    */

VECTOR   ecr_x,			/* ECI X basis vector in ECR system         */
         ecr_y,			/* ECI Y basis vector in ECR system         */
         ecr_z,			/* ECI Z basis vector in ECR system         */
         basis_x,		/* ECI X basis vector 			    */
         basis_y,		/* ECI Y basis vector         		    */
         ecr_vec;		/* vector in ECR sytem			    */
int  status;			/* status of return from function	    */

/* 
  Compute the Greenwich Apparent Sidereal Time, Precession and Nutation 
  angles
*/
exx_calcgha(ephem_time, ut1_utc, &zi, &theta, &zeta, &e, &ebar, &psi, &gast);

/*
  Construct the ECI basis vectors
*/
basis_x.x = 1.0;
basis_x.y = 0.0;
basis_x.z = 0.0;
basis_y.x = 0.0;
basis_y.y = 1.0;
basis_y.z = 0.0;
/*
  Construct the ECR basis vectors
*/
status = exx_ic2efc( xp, yp, ut1_utc, &basis_x, &basis_y, ephem_time, &ecr_x, &ecr_y );
if (status != SUCCESS) 
{
    xxx_LogStatus("EXXLIB", __FILE__, __LINE__,
	"Error returned from exx_ic2efc");
    return(ERROR);
}
/* We can get the third axis from the cross product */
xxx_cross( &ecr_x, &ecr_y, &ecr_z );

/*
  Convert the satellite position vector from ECR to ECI
  using the basis vectors
*/
ecr_vec.x = craft_pos->x;		/* copy from the input vector	   */
ecr_vec.y = craft_pos->y;
ecr_vec.z = craft_pos->z;

ic_satpos->x = xxx_dot( &ecr_vec, &ecr_x );
ic_satpos->y = xxx_dot( &ecr_vec, &ecr_y );
ic_satpos->z = xxx_dot( &ecr_vec, &ecr_z );

/*
  Convert the satellite velocity vector from ECI to ECR, no magnititude 
  change
*/
ecr_vec.x = craft_vel->x;		/* copy vector from input	   */
ecr_vec.y = craft_vel->y;
ecr_vec.z = craft_vel->z;

ic_satvel->x = xxx_dot( &ecr_vec, &ecr_x );
ic_satvel->y = xxx_dot( &ecr_vec, &ecr_y );
ic_satvel->z = xxx_dot( &ecr_vec, &ecr_z );

return(SUCCESS);
}
