/***************************************************************************
NAME: exx_findmjdcoords

PURPOSE:
Finds the x, y, and UT1-UTC coords for a given Modified Julian Date (mjd)
and the CPF's UT1_Time_ Parameters. The function returns true if the mjd 
value was found, otherwise the function returns false.  The x, y, and ut1_utc
values are 0.0 if the mjd was not found, ow their values are those
coords for the specified MJD.

DEVELOPMENT HISTORY:
$Log: exx_findmjdcoords.c,v $
Revision 1.1  2000/11/22 22:20:53  storey
Original version - derived from gxx_findmjdcoords.c


RETURN VALUE:
Type = int
Value    Description
-----    -----------
SUCCESS  Successful completion 
ERROR    Operation failed

NOTES:

ALGORITHM REFERENCE:
*****************************************************************************
                        Property of the U.S. Government
                            USGS EROS Data Center
**************************************************************************/
#include "xxx_Types.h"
#include "xxx_LogStatus.h"
#include "exx_geometric.h"
#include "exx_cpf.h"

int exx_findmjdcoords (
    double mjd,      /* I: mjd value to search for (days)     	*/
    double *x,       /* O: x shift pole wander in are/sec	*/
    double *y,       /* O: y shift pole wander in are/sec	*/
    double *ut1_utc, /* O: UT1-UTC coord time difference in sec */
    const CPF *cpf)  /* I: info from Calibraion Parameter File	*/

{
int i;          /* loop variable                                        */
int  currmjd;   /* current mjd value                                    */
int  found_flag;/* the flag to show we found the date we want
		   if = 0 not found, = 1 found exact match		*/
int  lmjd;	/* the mjd rounded to nearest integer 			*/

/* Set the return values to 0.0 for defaults.
 */
*x = 0.0;
*y = 0.0;
*ut1_utc = 0.0;


/* round the mjd to the nearest integer */
lmjd = (int)(mjd + .5);

found_flag = 0;
i = 0;
currmjd = 0;

/* Search for the desired mjd value.
   (there are values for 180 days (Two 90 day periods))
 */
while (found_flag == 0 && i < 180)
    {
    /* If the desired MJD is found then we are done. 
    */
    currmjd = cpf->UT1_Time_Parameters.UT1_Modified_Julian[i];
    if (currmjd == lmjd)
	found_flag = 1;
    else
        i++;
    }

/* If the mjd is found then read the x, y, and UT1-UTC coords.
 */
if (found_flag == 1)
    {
    *x = cpf->UT1_Time_Parameters.UT1_X[i];
    *y = cpf->UT1_Time_Parameters.UT1_Y[i];
    *ut1_utc = cpf->UT1_Time_Parameters.UT1_UTC[i];
    return (SUCCESS);
    }
else
    { /* The MJD was not found. */
    xxx_LogStatus("EXXLIB",__FILE__,__LINE__,
	"Modified Julian Date: not found in CPF.");
    return (ERROR);
    }
}
