/******************************************************************************
NAME:           exx_gcp_ls

PURPOSE:
Determine the line/sample of the gcps for a specified band resolution.


RETURN VALUE:   Type = int
Value    Description
-----    -----------
SUCCESS  Successful completion
ERROR    Operation failed

NOTES:

ALGORITHM REFERENCES:
See xxx_DDR.h for a description of the ddr structure.

******************************************************************************/

#include "xxx_Types.h"
#include "xxx_LogStatus.h"
#include "xxx_DDR.h"
#include "xxx_const.h"
#include "xxx_structures.h"
#include "xxx_utility.h"
#include "xxx_geometric.h"
#include "exx_gcplib.h"
#include "exx_geometric.h"

int exx_gcp_ls (
  const xxx_DDR_TYPE *ddr,    /* I: DDR for the desired band resolution   */
  int  numgcps,               /* I: Number of GCPs in the GCPLIB          */
  const GCPLIB *in_gcps,      /* I: Input array of GCPLib tiepoints in
                                    row-major format                      */
  GCPLIB *gcps)               /* O: Output array of GCPLib tiepoints in
                                    row-major format                      */
{
  int i, j;             /* Looping variables                              */
  double poly_x[4],     /* Sample-to-projection coefficients for north-up */
         poly_y[4],     /* Line-to-projection coefficients for north-up   */
         geo_parms[COEFCT],
                        /* Geo projection parameters                      */
         proj_parms[COEFCT];
                        /* DDR projection parameters                      */
  char ErrMsg[STRLEN];  /* Error message                                  */
  int  degree,          /* Degree of the polynomials                      */
       out_units,       /* Code for the output projection units.          */
       stat,            /* Function return status                         */
       geo = 0,         /* Geographic projection flag                     */
       four = 4,        /* Constant of four (DEGREES)                     */
       null_zone = NULLZONE,
                        /* Null zone code for projection xforms           */
       spheroid_code,   /* DDR spheroid code                              */
       project_code,    /* DDR projection code                            */
       zone_code;       /* DDR zone code                                  */

int		year, month, day;	/* the year month and day of the */
int		status;				/* the status of the return form a funtion */

  /* Initialize geo_parms.
   */
  for (i = 0; i < COEFCT; i++)
    geo_parms[i] = 0.0;

  /* Convert the ddr projection units to a unit code for gctp.
   */
  if (!strcmp (xxx_strtoupper ((char *)ddr->ProjectionUnits), "RADIANS"))
    out_units = 0;
  else if (!strcmp (xxx_strtoupper ((char *)ddr->ProjectionUnits),
           "FEET"))
    out_units = 1;
  else if (!strcmp (xxx_strtoupper ((char *)ddr->ProjectionUnits),
           "METERS"))
    out_units = 2;
  else if (!strcmp (xxx_strtoupper ((char *)ddr->ProjectionUnits),
           "SECONDS"))
    out_units = 3;
  else if (!strcmp (xxx_strtoupper ((char *)ddr->ProjectionUnits),
           "DEGREES"))
    out_units = 4;
  else if (!strcmp (xxx_strtoupper ((char *)ddr->ProjectionUnits),
           "DMS"))
    out_units = 5;

  /* Set up the projection vars using the DDR information.
   */
  spheroid_code = ddr->SpheroidCode;
  project_code = ddr->ProjectCode;
  zone_code = ddr->ZoneCode;
  for (j = 0; j < COEFCT; j++)
    proj_parms[j] = ddr->ProjectionCoef[j];

  /* Get the polynomials for the xxx_eval call.
   */
  degree = 1;
  stat = xxx_calc_poly (ddr, poly_x, poly_y);
  if (stat == ERROR)
  {
    xxx_LogStatus ("EXXLIB", __FILE__, __LINE__,
      "Error calculating the poly_x and poly_y values.");
    return (ERROR);
  }

  /* Loop through the gcps and calculate the line/sample values.
   */
  for (i = 0; i < numgcps; i++)
  {
    /* Copy the data from the input gcp that we want to keep.
     */
    strcpy (gcps[i].pt_id, in_gcps[i].pt_id);
    strcpy (gcps[i].chip_name, in_gcps[i].chip_name);
    gcps[i].lat = in_gcps[i].lat;
    gcps[i].lon = in_gcps[i].lon;
    gcps[i].elev = in_gcps[i].elev;
    gcps[i].ref_size[0] = in_gcps[i].ref_size[0];
    gcps[i].ref_size[1] = in_gcps[i].ref_size[1];
    strcpy (gcps[i].source, in_gcps[i].source);
    strcpy (gcps[i].date, in_gcps[i].date);

	/* check the date of the GCPLib  */
	sscanf(gcps[i].date,"%ld-%ld-%ld",&month,&day,&year);
	status = xxx_check_year_month_day(year, month, day);
	if (status != SUCCESS)
		{
    	xxx_LogStatus("EXXLIB",__FILE__,__LINE__,"Error in gcp lib date");
    	return(ERROR);
		}

    strcpy (gcps[i].abs_rel_flag, in_gcps[i].abs_rel_flag);

    /* Fill in the pixel size from the DDR.
     */
    gcps[i].pix_size[0] = ddr->ProjectionDistY;  /* Height */
    gcps[i].pix_size[1] = ddr->ProjectionDistX;  /* Width  */

    /* Calculate the proj x/y values from the GCPLib lat/long.  Use the DDR
     * projection information.
     */
    stat = exx_projtran (&geo, &four, &null_zone, geo_parms,
           &spheroid_code, &project_code, &out_units,
           &zone_code, proj_parms,
           &spheroid_code, &gcps[i].lon, &gcps[i].lat,
           &gcps[i].projx, &gcps[i].projy);
    if (stat == ERROR)
    {
      strcpy (ErrMsg, "Error in the projtran call.");
      xxx_LogStatus ("EXXLIB", __FILE__, __LINE__, ErrMsg);
      return (ERROR);
    }

    /* Calculate the output line and sample.  This routine will use
     * the polynomial calculations with both the path-oriented and
     * north-up products.
     */
    xxx_eval (degree, poly_y, gcps[i].projx, gcps[i].projy, &gcps[i].ref_line);
    xxx_eval (degree, poly_x, gcps[i].projx, gcps[i].projy, &gcps[i].ref_samp);
  }

  return (SUCCESS);
}
