/****************************************************************************
NAME: exx_ic2efc

PURPOSE:
Transform inertial cartesian coordinates (ECI/true-of-date) to
Earth-fixed cartesian coordinates (ECR/Earth-fixed) at the specified
GMT (UTC) time.

DEVELOPMENT HISTORY:
$Log: exx_ic2efc.c,v $
Revision 1.1  2000/11/22 22:20:56  storey
Original version - derived from gxx_ic2efc.c


RETURN VALUE:
Type = int
Value    Description
-----    -----------
SUCCESS  Successful completion 
ERROR    Operation failed

NOTES:

ALGORITHM REFERENCES:
	Landsat 7 Program Coordinate System Standard, Martin Marietta, 1993,
	Page 4-16.
*****************************************************************************
                        Property of the U.S. Government
                            USGS EROS Data Center
*****************************************************************************/
#include <math.h>
#include "xxx_const.h"
#include "xxx_Types.h"
#include "xxx_LogStatus.h"
#include "xxx_math.h"
#include "exx_geometric.h"

int exx_ic2efc(
    double xp,			/* I: Earth's true pole offset from mean pole, */
				/* in arc second			       */
    double yp,			/* I: Earth's true pole offset from mean pole, */
				/* in arc second			       */
    double ut1_utc,		/* I: Irregular Earth rotation, in time second */
    const VECTOR *craft_pos,	/* I: Satellite position in ECI, input         */
    const VECTOR *craft_vel,	/* I: Satellite velocity in ECI, input         */
    const double ephem_time[3],	/* I: Time of the ephemeris point, 	       */
			        /* in year, day and second, input	       */
    VECTOR *fe_satpos,	/* O: Satellite position in ECR, output        */
    VECTOR *fe_satvel)	/* O: Satellite velocity in ECR, output        */


{
double    gast,			/* Greenwich apparent siderial time, in rad */
          zi, theta, zeta,	/* angles in precession, in radian	    */
          e, ebar, psi;		/* angles in nutation, in radian	    */

VECTOR   eci_vec,		/* vector in ECI system			    */
                 pre_vec,	/* vector after precession transformation   */
                 nut_vec,	/* vector after nutation transformation     */
                 mid_vec,	/* vector in intermediate step		    */
                 ecr_vec;	/* vector in ECR sytem			    */
int  status;			/* status of return from function	    */

/* 
  Compute the Greenwich Apparent Sidereal Time, Precession and Nutation 
  angles
*/
exx_calcgha(ephem_time, ut1_utc, &zi, &theta, &zeta, &e, &ebar, &psi, &gast);

/*
  Convert the satellite position vector from ECI to ECR
*/
eci_vec.x = craft_pos->x;		/* copy from the input vector	   */
eci_vec.y = craft_pos->y;
eci_vec.z = craft_pos->z;

/* 
  EO1 ACS ephemeris data will be broadcast in J2000.0 system, precession
  and nutation transformation should be added here before the rotation 
  transformation around Earth's spin axis				   
*/

exx_precession(eci_vec, zi, theta, zeta, &pre_vec);
exx_nutation(pre_vec, e, ebar, psi, &nut_vec);
eci_vec.x = nut_vec.x;
eci_vec.y = nut_vec.y;
eci_vec.z = nut_vec.z;

xxx_rotatez(eci_vec, gast, &mid_vec);/* rotate around true pole for gast*/

status = exx_polar_motion(mid_vec, xp, yp, &ecr_vec);/* from true pole to mean pole   */
if (status != SUCCESS) 
    {
    xxx_LogStatus("EXXLIB", __FILE__, __LINE__,
	"Error returned from exx_polar_motion");
    return(ERROR);
    }

fe_satpos->x = ecr_vec.x;		/* copy to the output vector	   */
fe_satpos->y = ecr_vec.y;
fe_satpos->z = ecr_vec.z;

/*
  Convert the satellite velocity vector from ECI to ECR, no magnititude 
  change
*/
eci_vec.x = craft_vel->x;		/* copy vector from input	   */
eci_vec.y = craft_vel->y;
eci_vec.z = craft_vel->z;

/* 
  EO1 ACS ephemeris data will be broadcast in J2000.0 system, precession
  and nutation transformation should be added here before the rotation 
  transformation around Earth's spin axis				   
*/

exx_precession(eci_vec, zi, theta, zeta, &pre_vec);
exx_nutation(pre_vec, e, ebar, psi, &nut_vec);
if (status != SUCCESS) 
    {
    xxx_LogStatus("EXXLIB", __FILE__, __LINE__,
	"Error returned from exx_nutation");
    return(ERROR);
    }
eci_vec.x = nut_vec.x;
eci_vec.y = nut_vec.y;
eci_vec.z = nut_vec.z;

xxx_rotatez(eci_vec, gast, &mid_vec);	/* rotation around true pole   */

status = exx_polar_motion(mid_vec, xp, yp, &ecr_vec);/* from true pole to mean pole	  */
if (status != SUCCESS) 
    {
    xxx_LogStatus("EXXLIB", __FILE__, __LINE__,
	"Error returned from exx_polar_motion");
    return(ERROR);
    }

fe_satvel->x = ecr_vec.x;		/* copy vector to output 	   */
fe_satvel->y = ecr_vec.y;
fe_satvel->z = ecr_vec.z;

return(SUCCESS);
}
