/****************************************************************************
NAME:				xxx_mapedg

PURPOSE:  Calculates the minimum and maximum projection coordinates for
          given upper left and lower right latitude, longitude coordinates.

DEVELOPMENT HISTORY:
AUTHOR        Change-ID    DATE     Description of Change
------        ---------  --------   ---------------------
Gail Schmidt     1.0     05/21/97   Brought over from LAS.
Gail Schmidt     1.1     05/21/97   Changed the c_proj call to be a
                                    exx_projtran call to handle spheroid
                                    codes.

RETURN VALUE:   Type = int
Value    Description
-----    -----------
SUCCESS  Successful completion
ERROR    Operation failed

ALGORITHM DESCRIPTION:
        Initialize variables
        Calculate min/max coordinates along east edge of output area
        Calculate min/max coordinates along west edge of output area
        Calculate min/max coordinates along south edge of output area
        Calculate min/max coordinates along north edge of output area
        Return

ALGORITHM REFERENCES:

*****************************************************************************/
#include "exx_geometric.h"
#include "xxx_LogStatus.h"

int exx_mapedg (
  double *seast,	/* I/O: Upper left longitude coordinate        */
  double *eeast,	/* I/O: Lower right longitude coordinate       */
  double *snorth,	/* I/O: Upper left latitude coordinate         */
  double *enorth,	/* I/O: Lower right latitude coordinate        */
  int  *inunits,	/* I/O: Input space units                      */
  int  *inspheroid,     /* I: Input space spheroid code                */
  int  *proj_code,      /* I: Projection code; output space            */
  int  *zone,	        /* I: Projection zone code; output space       */
  double parout[15],    /* I: Projection parameters; output space      */
  int  *units,	        /* I: Projection units code; output space      */
  int  *spheroid,       /* I: Projection spheroid code; output space   */
  double *pxmin,	/* O: Projection minimum in X                  */
  double *pxmax,	/* O: Projection maximum in X                  */
  double *pymin,	/* O: Projection minimum in Y                  */
  double *pymax)	/* O: Projection maximum in Y                  */
{
  double delta_east;	/* Longitude increment                            */
  double delta_north;	/* Latitude increment                             */
  double inx, iny;	/* Input space coordinates given to proj()        */
  double outx, outy;	/* Output space coordinates given to proj()       */
  double inparm[15];	/* Input projection parameters (constant--all zeros) */
  int  i;		/* Loop counter                                   */
  int  geo = 0;		/* Input space projection--always geographic      */
  int  nozone = 62;	/* Input space zone code                          */
  int  size = 75;	/* Interval of increment to check for min/max     */
  char ErrMsg[STRLEN];  /* Error Message                                  */

  /* If input coordinates are given in DMS format, convert to degrees.
   */
  if (*inunits == 5)
  {
    if (exx_decdeg(seast, "DMS", "LON") != SUCCESS)
    {
      strcpy (ErrMsg, "Error returned from decdeg in mapedg.");
      xxx_LogStatus ("EXXLIB", __FILE__, __LINE__, ErrMsg);
      return (ERROR);
    }
    if (exx_decdeg(eeast, "DMS", "LON") != SUCCESS)
    {
      strcpy (ErrMsg, "Error returned from decdeg in mapedg.");
      xxx_LogStatus ("EXXLIB", __FILE__, __LINE__, ErrMsg);
      return (ERROR);
    }
    if (exx_decdeg(snorth, "DMS", "LAT") != SUCCESS)
    {
      strcpy (ErrMsg, "Error returned from decdeg in mapedg.");
      (void) xxx_LogStatus ("EXXLIB", __FILE__, __LINE__, ErrMsg);
      return (ERROR);
    }
    if (exx_decdeg(enorth, "DMS", "LAT") != SUCCESS)
    {
      strcpy (ErrMsg, "Error returned from decdeg in mapedg.");
      (void) xxx_LogStatus ("EXXLIB", __FILE__, __LINE__, ErrMsg);
      return (ERROR);
    }
    *inunits = 4;
  }

  /* Initialization of variables.
   */
  for (i = 0; i < 15; inparm[i++] = 0.0);
  delta_east = (*eeast - *seast) / (size - 1);
  delta_north = (*enorth - *snorth) / (size - 1);

  *pxmin = 1000000000.0;
  *pymin = 1000000000.0;
  *pxmax = -1000000000.0;
  *pymax = -1000000000.0;

  /* Calculate the minimum and maximum coordinates along the east side of 
   * the output area
   */
  for (i = 0; i < size; i++)
  {
    inx = *eeast;
    iny = *enorth - (delta_north * i);
    if (exx_projtran (&geo, inunits, &nozone, inparm, inspheroid, proj_code,
        units, zone, parout, spheroid, &inx, &iny, &outx, &outy) != SUCCESS)
    {
      strcpy (ErrMsg, "Error returned from projtran in mapedg.");
      (void) xxx_LogStatus ("EXXLIB", __FILE__, __LINE__, ErrMsg);
      return (ERROR);
    }

    if (outx < *pxmin) *pxmin = outx;
    if (outx > *pxmax) *pxmax = outx;
    if (outy < *pymin) *pymin = outy;
    if (outy > *pymax) *pymax = outy;
  }

  /* Calculate the minimum and maximum coordinates along the west side of 
   * the output area
   */
  for (i = 0; i < size; i++)
  {
    inx = *eeast - (delta_east * (size - 1));
    iny = *enorth - (delta_north * i);
    if (exx_projtran (&geo, inunits, &nozone, inparm, inspheroid, proj_code,
        units, zone, parout, spheroid, &inx, &iny, &outx, &outy) != SUCCESS)
    {
      strcpy (ErrMsg, "Error returned from projtran in mapedg.");
      (void) xxx_LogStatus ("EXXLIB", __FILE__, __LINE__, ErrMsg);
      return (ERROR);
    }
     
    if (outx < *pxmin) *pxmin = outx;
    if (outx > *pxmax) *pxmax = outx;
    if (outy < *pymin) *pymin = outy;
    if (outy > *pymax) *pymax = outy;
  }


  /* Calculate the minimum and maximum coordinates along the south side of 
   * the output area
   */
  for (i = 0; i < size; i++)
  {
    inx = *eeast - (delta_east * i);
    iny = *enorth - (delta_north * (size - 1));
    if (exx_projtran (&geo, inunits, &nozone, inparm, inspheroid, proj_code,
        units, zone, parout, spheroid, &inx, &iny, &outx, &outy) != SUCCESS)
    {
      strcpy (ErrMsg, "Error returned from projtran in mapedg.");
      (void) xxx_LogStatus ("EXXLIB", __FILE__, __LINE__, ErrMsg);
      return (ERROR);
    }
     
    if (outx < *pxmin) *pxmin = outx;
    if (outx > *pxmax) *pxmax = outx;
    if (outy < *pymin) *pymin = outy;
    if (outy > *pymax) *pymax = outy;
  }


  /* Calculate the minimum and maximum coordinates along the north side of 
   * the output area
   */
  for (i = 0; i < size; i++)
  {
    inx = *eeast - (delta_east * i);
    iny = *enorth;
    if (exx_projtran (&geo, inunits, &nozone, inparm, inspheroid, proj_code,
        units, zone, parout, spheroid, &inx, &iny, &outx, &outy) != SUCCESS)
    {
      strcpy (ErrMsg, "Error returned from projtran in mapedg.");
      (void) xxx_LogStatus ("EXXLIB", __FILE__, __LINE__, ErrMsg);
      return (ERROR);
    }
     
    if (outx < *pxmin) *pxmin = outx;
    if (outx > *pxmax) *pxmax = outx;
    if (outy < *pymin) *pymin = outy;
    if (outy > *pymax) *pymax = outy;
  }

  return(SUCCESS);
}
