/*******************************************************************************
NAME:           exx_proj_err

PURPOSE:   	To report projection transformation package errors.

DEVELOPMENT HISTORY:
$Header: /llite/cvseo1/eo1/src/global/exxLib/exxGeo/exx_proj_err.c,v 1.2 2001/01/03 19:24:58 storey Exp $
AUTHOR        Change-ID    DATE     Description of Change
------        ---------  --------   ---------------------
Gail Schmidt     1.0     05/21/97   Brought over from LAS.

RETURN VALUE:   Type = int
Value    Description
-----    -----------
SUCCESS  Successful completion
ERROR    Operation failed

ALGORITHM DESCRIPTION:
    Recieve a GCTP error code and print the correct error message.

ALGORITHM REFERENCES:

NOTES:
Errors generated internal to the GCTP are listed below.

  ERROR   ROUTINE     MESSAGE
  -----   ------- ---------------------------
ERR = 0           SUCCESSFUL COMPLETION
ERR = 005         ILLEGAL SOURCE OR TARGET UNIT CODE
ERR = 006         OUTPUT FILENAME NOT SPECIFIED
ERR = 021 PJ02Z0  ILLEGAL ZONE NO 
ERR = 022 PJ02Z0  ERROR OPENING STATE PLANE PARAMETER FILE
ERR = 041 PJ04Z0  EQUAL LATITUDES FOR ST. PARALLELS ON OPPOSITE SIDES OF EQUATOR
ERR = 053 PJ05Z0  TRANSFORMATION CANNOT BE COMPUTED AT THE POLES
ERR = 123 PJ12Z0  POINT PROJECTS INTO CIRCLE OF RADIUS =',F12.2, METERS
ERR = 133 PJ13Z0  POINT MAPS INTO INFINITY
ERR = 153 PJ15Z0  POINT CANNOT BE PROJECTED
ERR = 195 PI19Z0  LATITUDE FAILED TO CONVERGE AFTER',I3,' ITERATIONS
ERR = 214 PJ21Z0  50 ITERATIONS WITHOUT CONV
ERR = 215 PJ21Z0  PROJECTION TRANSFORMATION TRANSFORMATION FAILED
ERR = 233 PJ23Z0  MISSING PROJECTION PARAMETERS
ERR = 234 PJ23Z0  TOO MANY ITERATIONS FOR INVERSE ROBINSON
ERR = 236 PJ23Z0  TOO MANY ITERATIONS IN INVERSE
ERR = 241 PJ23Z0  FAILED TO CONVERGE AFTER MANY ITERATIONS
ERR = 252 PJ23Z0  INPUT DATA ERROR
ERR = 1001        ILLEGAL INPUT SYSTEM CODE
ERR = 1002        ILLEGAL OUTPUT SYSTEM CODE
ERR = 1003        ILLEGAL INPUT UNIT CODE
ERR = 1004        ILLEGAL OUTPUT UNIT CODE
ERR = 1005        INCONSISTANT UNIT AND SYSTEM CODES FOR INPUT
ERR = 1006        INCONSISTANT UNIT AND SYSTEM CODES FOR OUTPUT
ERR = 1007        ILLEGAL INPUT ZONE CODE
ERR = 1008        ILLEGAL OUTPUT ZONE CODE
ERR = 1116 PAKSZ0 ILLEGAL DMS FIELD 
ERR = 1211        SHPEROID CODE RESET TO DEFAULT

******************************************************************************
                        Property of the U.S. Government
                           USGS EROS Data Center
*****************************************************************************/
#include <string.h>
#include "exx_geometric.h"
#include "xxx_LogStatus.h"

int exx_proj_err (
  int err)             /* I: Error returned from the GCTP call    */
{
  char ErrMsg[STRLEN];  /* Error Message                           */

  switch (err)
  {
    case -2:
      sprintf (ErrMsg, "Point lies in break.\n");
      xxx_LogStatus ("EXXLIB", __FILE__, __LINE__, ErrMsg);
      return (ERROR);
    case 0:
      return (SUCCESS);
    case 6:
      sprintf (ErrMsg, "Output file name not specified.\n");
      xxx_LogStatus ("EXXLIB", __FILE__, __LINE__, ErrMsg);
      return (ERROR);
    case 215:
      sprintf (ErrMsg, "Projection transformation failed.\n");
      xxx_LogStatus ("EXXLIB", __FILE__, __LINE__, ErrMsg);
      return (ERROR);
    case 241:
      sprintf (ErrMsg, "Failed to coverge after many iterations.\n");
      xxx_LogStatus ("EXXLIB", __FILE__, __LINE__, ErrMsg);
      return (ERROR);
    case 234:
      sprintf (ErrMsg, "Too many iterations for inverse Robinson.\n");
      xxx_LogStatus ("EXXLIB", __FILE__, __LINE__, ErrMsg);
      return (ERROR);
    case 236:
      sprintf (ErrMsg, "Too many iterations in inverse.\n");
      xxx_LogStatus ("EXXLIB", __FILE__, __LINE__, ErrMsg);
      return (ERROR);
    case 252:
      sprintf (ErrMsg, "Input data error.\n");
      xxx_LogStatus ("EXXLIB", __FILE__, __LINE__, ErrMsg);
      return (ERROR);
    case 1116:
      sprintf (ErrMsg, "Illegal DMS field.\n");
      xxx_LogStatus ("EXXLIB", __FILE__, __LINE__, ErrMsg);
      return (ERROR);
    case 1005:
      sprintf (ErrMsg, "Inconsistant unit and system codes for input.\n");
      xxx_LogStatus ("EXXLIB", __FILE__, __LINE__, ErrMsg);
      return (ERROR);
    case 1001:
      sprintf (ErrMsg, "Illegal input system code.\n");
      xxx_LogStatus ("EXXLIB", __FILE__, __LINE__, ErrMsg);
      return (ERROR);
    case 1003:
      sprintf (ErrMsg, "Illegal input unit code.\n");
      xxx_LogStatus ("EXXLIB", __FILE__, __LINE__, ErrMsg);
      return (ERROR);
    case 1007:
      sprintf (ErrMsg, "Illegal input zone code.\n");
      xxx_LogStatus ("EXXLIB", __FILE__, __LINE__, ErrMsg);
      return (ERROR);
    case 133:
      sprintf (ErrMsg, "Point projects into infinity.\n");
      xxx_LogStatus ("EXXLIB", __FILE__, __LINE__, ErrMsg);
      return (ERROR);
    case 195:
      sprintf (ErrMsg, "Latitude failed to converge after many iterations.\n");
      xxx_LogStatus ("EXXLIB", __FILE__, __LINE__, ErrMsg);
      return (ERROR);
    case 1006:
      sprintf (ErrMsg, "Inconsistant unit and system codes for output.\n");
      xxx_LogStatus ("EXXLIB", __FILE__, __LINE__, ErrMsg);
      return (ERROR);
    case 1002:
      sprintf (ErrMsg, "Illegal output system code.\n");
      xxx_LogStatus ("EXXLIB", __FILE__, __LINE__, ErrMsg);
      return (ERROR);
    case 1004:
      sprintf (ErrMsg, "Illegal output unit code.\n");
      xxx_LogStatus ("EXXLIB", __FILE__, __LINE__, ErrMsg);
      return (ERROR);
    case 1008:
      sprintf (ErrMsg, "Illegal output zone code.\n");
      xxx_LogStatus ("EXXLIB", __FILE__, __LINE__, ErrMsg);
      return (ERROR);
    case 53: 
      sprintf (ErrMsg, "Transformation cannot be computed at the poles.\n");
      xxx_LogStatus ("EXXLIB", __FILE__, __LINE__, ErrMsg);
      return (ERROR);
    case 153:
      sprintf (ErrMsg, "Point cannot be projected.\n");
      xxx_LogStatus ("EXXLIB", __FILE__, __LINE__, ErrMsg);
      return (ERROR);
    case 123:
      sprintf (ErrMsg,
        "Point projects into a circle of unacceptable radius.\n");
      xxx_LogStatus ("EXXLIB", __FILE__, __LINE__, ErrMsg);
      return (ERROR);
    case 214:
      sprintf (ErrMsg, "Fifty iterations performed without conversion.\n");
      xxx_LogStatus ("EXXLIB", __FILE__, __LINE__, ErrMsg);
      return (ERROR);
    case 1211:
      sprintf (ErrMsg, "Spheroid code reset to default.\n");
      xxx_LogStatus ("EXXLIB", __FILE__, __LINE__, ErrMsg);
      return (ERROR);
    case 41:
      sprintf (ErrMsg,
        "Equal latitudes for std parallels on opposite sides of equator.\n");
      xxx_LogStatus ("EXXLIB", __FILE__, __LINE__, ErrMsg);
      return (ERROR);
    case 21:
      sprintf (ErrMsg, "Illegal zone number.\n");
      xxx_LogStatus ("EXXLIB", __FILE__, __LINE__, ErrMsg);
      return (ERROR);
    case 22:
      sprintf (ErrMsg, "Error opening state plane parameter file.\n");
      xxx_LogStatus ("EXXLIB", __FILE__, __LINE__, ErrMsg);
      return (ERROR);
    case 5:
      sprintf (ErrMsg, "Illegal source or target unit code.\n");
      xxx_LogStatus ("EXXLIB", __FILE__, __LINE__, ErrMsg);
      return (ERROR);
    case 233:
      sprintf (ErrMsg, "Missing projection parameters.\n");
      xxx_LogStatus ("EXXLIB", __FILE__, __LINE__, ErrMsg);
      return (ERROR);
  }

  return (SUCCESS);
}
