#include "exx_geometric.h"
#include "xxx_LogStatus.h"

/*******************************************************************************
NAME:			      exx_projtran

PURPOSE:  Converts coordinates from one map projection to another.

DEVELOPMENT HISTORY:
$Log: exx_projtran.c,v $
Revision 1.2  2000/12/03 18:13:59  choate
Changed IASYS to EO1SYS

Revision 1.1  2000/11/22 22:21:01  storey
Original version - derived from gxx_projtran.c

AUTHOR        Change-ID    DATE     Description of Change
------        ---------  --------   ---------------------

RETURN VALUE:   Type = int
Value    Description
-----    -----------
SUCCESS  Successful completion
ERROR    Operation failed

NOTES:

ALGORITHM:
	Combine X and Y input values into an array
   	Unpack array of X and Y output coordinates into separate X & Y
        Call proj_err if an error occured; return ERROR
	Return SUCCESS

ALGORITHM REFERENCES:
1.  Snyder, John P., "Map Projections--A Working Manual", U.S. Geological
    Survey Professional Paper 1395 (Supersedes USGS Bulletin 1532), United
    State Government Printing Office, Washington D.C., 1987. 

2.  "Software Documentation for GCTP General Cartographic Transformation
    Package", U.S. Geological Survey National Mapping Division, May 1982.

3.  Clarie, Charles N, "State Plane Coordinates by Automatic Data Processing",
    U.S. Department of Commerce, Environmental Science Services Admin.,
    Coast and Geodetic Survey, Publication 62-4, 1973.

******************************************************************************
                        Property of the U.S. Government
                           USGS EROS Data Center
*******************************************************************************/

/* Use this for the environment variable */
static char iasys[STRLEN] = {""};

int exx_projtran (
  int *inproj,		/* I: Input projection code                     */
  int *inunit,		/* I: Input projection units code               */
  int *inzone,		/* I: Input projection zone code                */
  double *inparm,	/* I: Array of 15 projection parameters--input  */
  int *inspheroid,	/* I: Input spheroid code                       */
  int *outproj,     /* I: Output projection code                    */
  int *outunit, 	/* I: Output projection units code              */
  int *outzone, 	/* I: Output projection zone code               */
  double *outparm,	/* I: Array of 15 projection parameters--output */
  int *outspheroid,	/* I: Output spheroid code                      */
  double *inx,		/* I: Input X projection coordinate             */
  double *iny,		/* I: Input Y projection coordinate             */
  double *outx,		/* Output X projection coordinate            */
  double *outy)		/* Output Y projection coordinate            */
{
long off = 3;		/* Off flag for GCTP error and print messages */
long status;		/* Status code from call to GCTP */
int out_dms, in_dms;	/* Boolean flags for DMS unit check */
char efile[STRLEN];	/* The file to contain the output error message */
char pfile[STRLEN];	/* The file to contain the parameter message */
char fn27[STRLEN];	/* The file containing the NAD 1927 State Plane
				zone parameters */
char fn83[STRLEN];	/* The file containing the NAD 1983 State Plane
				zone parameters */
char *ptr;		/* Pointer to the environment variable */

double incoor[2];	/* Input coordinates */
double outcoor[2];	/* Output coordinates */
double temp;            /* Temp storage variable */

/* convert the parameters to long for GCTP since we want to use it
   as is without correcting for -64 compiles 
*/
long long_inproj = *inproj;
long long_inzone = *inzone;
long long_inunit = *inunit;
long long_inspheroid = *inspheroid;
long long_outproj = *outproj;
long long_outzone = *outzone;
long long_outunit = *outunit;
long long_outspheroid = *outspheroid;

/* Pack input coordinates into a GCTP compatable array
*/
incoor[0] = *inx;
incoor[1] = *iny;

/* Swap X & Y if the input projection is SOM
*/
if (*inproj == SOM)
    {
    incoor[0] = -(*iny);
    incoor[1] = *inx;
    }

/* Since the GCTP doesn't work with DMS input or output units, convert to
* degrees if DMS units are being used
*/
out_dms = FALSE;
in_dms = FALSE;
if (*inunit == 5)
    {
    if (exx_decdeg (&(incoor[0]), "DMS", "LON") != SUCCESS) 
      {
      xxx_LogStatus ("EXXLIB", __FILE__, __LINE__, 
			"Error returned from decdeg in projtran");
      return (ERROR);
      }
    if (exx_decdeg(&(incoor[1]), "DMS", "LON") != SUCCESS)
      {
      xxx_LogStatus ("EXXLIB", __FILE__, __LINE__, 
				"Error returned from decdeg in projtran");
      return (ERROR);
      }
    in_dms = TRUE;
    *inunit = 4;
    }
if (*outunit == 5)
    {
    out_dms = TRUE;
    *outunit = 4;
    }

/* Place State Plane directory in fn27, fn83
*/
if (strlen (iasys) == 0)
    {
    ptr = (char *) getenv ("EO1SYS");
    if (ptr == NULL)
      {
      xxx_LogStatus ("EXXLIB", __FILE__, __LINE__, 
		"IASYS environmental variable not defined in projtran");
      return (ERROR);
      }
    strcpy(iasys, ptr);
    }

/* Get the state plane file names for the gctp call.
*/
status = sprintf(fn27, "%s/nad27sp", iasys);
if (status < 1)
	{
      xxx_LogStatus ("EXXLIB", __FILE__, __LINE__, "Error in sprintf");
      return (ERROR);
	}
status = sprintf(fn83, "%s/nad83sp", iasys);
if (status < 1)
	{
      xxx_LogStatus ("EXXLIB", __FILE__, __LINE__, "Error in sprintf");
      return (ERROR);
	}


/* Call GCTP.
*/
gctp (incoor,&long_inproj,&long_inzone,inparm,&long_inunit,&long_inspheroid,
      &off,efile,&off,pfile,outcoor,&long_outproj,&long_outzone,outparm,
      &long_outunit,&long_outspheroid,fn27,fn83,&status);

/* Check the status from GCTP.
*/
if (exx_proj_err ((int)status) == ERROR)
    {
    xxx_LogStatus ("EXXLIB", __FILE__, __LINE__,"Error in the GCTP call");
    return (ERROR);
    }

/* Unpack transformed coordinates
*/
*outx = outcoor[0];
*outy = outcoor[1];

/* If either input or output units were originally DMS, adjust values
*/
if (in_dms) *inunit = 5;
if (out_dms)
    {
    *outunit = 5;
    if (exx_degdms(&(outcoor[0]), outx, "DEG", "LON") != SUCCESS) 
      {
      xxx_LogStatus ("EXXLIB", __FILE__, __LINE__, 
				"Error returned from degdms in projtran");
      return (ERROR);
      }
    if (exx_degdms(&(outcoor[1]), outy, "DEG", "LON") != SUCCESS)
      {
      xxx_LogStatus ("EXXLIB", __FILE__, __LINE__, 
				"Error returned from degdms in projtran");
      return (ERROR);
      }

    /* If the output projection is SOM, swap the X & Y coordinates.
     */
    if (*outproj == SOM)
      {
      temp = *outx;
      *outx = *outy;
      *outy = -temp;
      }
    }
else  /* Output is not DMS */
  {
  /* If the output projection is SOM, swap the X & Y coordinates
  */
  if (*outproj == SOM)
    {
    *outx = outcoor[1];
    *outy = -(outcoor[0]);
    }
  }

return (SUCCESS);
}
