/******************************************************************************
NAME:           gxx_calc_poly

PURPOSE:
Calculate the polynomials for the gxx_eval (projection to line/sample)
function.

DEVELOPMENT HISTORY:
$Header: /llite/cvseo1/eo1/src/global/exxLib/exxGeo/gxx_calc_poly.c,v 1.2 2000/11/22 22:14:22 storey Exp $
AUTHOR        Change-ID    DATE     Description of Change
------        ---------  --------   ---------------------
Gail Schmidt     1.0     08/28/97   Original development.

RETURN VALUE:   None

NOTES:

ALGORITHM REFERENCES:

******************************************************************************
                Landsat 7 Image Assessment System (IAS) Software
                        Property of the U.S. Government
                                NASA/GSFC/EDC
******************************************************************************/

#include "xxx_Types.h"
#include "xxx_LogStatus.h"
#include "xxx_DDR.h"
#include "gxx_const.h"
#include "gxx_structures.h"
#include "gxx_geo_math.h"

int gxx_calc_poly (
  const xxx_DDR_TYPE *ddr,    /* I: DDR for the desired band resolution   */
  double poly_x[4],           /* O: Projection-to-sample coefficients     */
  double poly_y[4])           /* O: Projection-to-line coefficients       */
{
  int i,                      /* Loop variable                         */
      stat;                   /* Return status                         */
  double A[12],               /* Matrix used by QRsolve for LSQ fit    */
         v[4];                /* Vector used in above operation        */

  /* Calculate the first order polynomial coefficients which map output
   * line/samples to output projection coordinates.  The 4x3 matrix is
   * populated as shown below, QR-factorized and then Ax = b is solved
   * for two different vectors b; one for samples, the other for lines.
   */
  for (i = 0; i < 4; A[i++] = 1.0);
  A[4]  = ddr->UpperLeft[1];
  A[5]  = ddr->UpperRight[1];  /*     +-                                 -+ */
  A[6]  = ddr->LowerRight[1];  /*     |  1  upper-left  X  upper-left  Y  | */
  A[7]  = ddr->LowerLeft[1];   /* A = |  1  upper-right X  upper-right Y  | */
  A[8]  = ddr->UpperLeft[0];   /*     |  1  lower-right X  lower-right Y  | */
  A[9]  = ddr->UpperRight[0];  /*     |  1  lower-left  X  lower-left  Y  | */
  A[10] = ddr->LowerRight[0];  /*     +-                                 -+ */
  A[11] = ddr->LowerLeft[0];
  stat = gxx_QR(A, 4, 3, v, 0); /* Obtain QR factorization */
  if (stat != SUCCESS)
  {
    xxx_LogStatus ("GPSLIB", __FILE__, __LINE__, "Error in gxx_QR.");
    return (ERROR);
  }

  /* Vector one: Find the coefficients to transform output coordinates
   * in projection space to output samples.  Note that the call
   * to gxx_QRsolve() modifies the values at poly_x for part of the
   * output (this vector becomes the before-mentioned coefficients.)
   */
  poly_x[0] = 0.0;
  poly_x[1] = ddr->NumberSamplesInScene - 1.0;
  poly_x[2] = ddr->NumberSamplesInScene - 1.0;
  poly_x[3] = 0.0;
  stat = gxx_QRsolve(A, 4, 3, v, poly_x, 0);
  if (stat != SUCCESS)
  {
    xxx_LogStatus ("GPSLIB", __FILE__, __LINE__, "Error in gxx_QRsolve.");
    return (ERROR);
  }

  /* Vector two: Find the coefficients to transform output
   * coordinates in projection space to output lines.
   */
  poly_y[0] = 0.0;
  poly_y[1] = 0.0;
  poly_y[2] = ddr->NumberLinesInScene - 1.0;
  poly_y[3] = ddr->NumberLinesInScene - 1.0;
  stat = gxx_QRsolve(A, 4, 3, v, poly_y, 0);
  if (stat != SUCCESS)
  {
    xxx_LogStatus ("GPSLIB", __FILE__, __LINE__, "Error in gxx_QRsolve.");
    return (ERROR);
  }

  return (SUCCESS);
}
