/*******************************************************************************
NAME:		gxx_det2centh

PURPOSE:	
Calculate the geocentric latitude and the radius of a point above the
ellipse, given the geodetic latitude and the height.

DEVELOPMENT HISTORY:
$Log: gxx_det2centh.c,v $
Revision 1.2  2000/11/22 22:14:22  storey
Replaced by exx_transmat.c

Revision 1.1.1.1  1999/12/01 19:01:20  mbs
Import of IAS 3.22

 * 
 *    Rev 1.5   Fri Aug 08 14:43:36 1997   rockvam
 * Walk-through changes (documentation, error checking);
 * changed input pointer arguments to just pass by value.
 * 

RETURN VALUE:
Type = int
Value           Description
-----           -----------
SUCCESS         Latitude and height calculated
ERROR           Error calculating latitude or height

ALGORITHM REFERENCE:
Geometric ATBD, Section 3.1.3, part 8. "ECR to Geodetic"
********************************************************************************
               Landsat 7 Image Assessment System (IAS) Software
                       Property of the U.S. Government
                               NASA/GSFC/EDC
*******************************************************************************/
#include "gxx_geometric.h"

int  gxx_det2centh(double latd, 	/* I: Geodetic latitude (radians)    */
		   double height,	/* I: Height in meters (elevation)   */
		   const CPF *cpf, 	/* I: Cal parm file constants	     */
		   double *latc,   	/* O: Geocentric latitude (radians)  */
		   double *radius) 	/* O: Satellite radius at point (m)  */
{
double	sinlt,		/* Sine of the geodetic latitude		     */
	w,		/* denominator in calculation of radius of curvature */
	g1,		/* N + h {radius of curvature plus geodetic height(m)}*/
	g2,		/* N * (1-e^2) + h   {raduis of curvature * 
			   (1 - eccentr squared) plus geodetic height (m)}   */
	xc,		/* ECR (geocentric) vector x coordinate		     */
	zc;		/* ECR (geocentric) vector z coordinate		     */

/*
  Error checking
*/
if (*radius == 0.0)
    {
    xxx_LogStatus("GPSLIB", __FILE__, __LINE__, "Invalid point radius");
    return(ERROR);
    }

/*
  Calculate the sine of the latitude
*/
sinlt = sin(latd);

w = sqrt(1.0 - cpf->Earth_Constants.Eccentricity * sinlt * sinlt);
if (w == 0.0)
    {
    xxx_LogStatus("GPSLIB", __FILE__, __LINE__, 
        "Error calculating ellipsoid radius of curvature");
    return(ERROR);
    }
g1 = cpf->Earth_Constants.Semi_Major_Axis / w + height;
g2 = (cpf->Earth_Constants.Semi_Major_Axis * 
      (1.0-cpf->Earth_Constants.Eccentricity)) / w + height;

xc = g1 * cos(latd);
zc = g2 * sinlt;

/*
  Calculate the radius of the point
*/
*radius = sqrt(xc * xc + zc * zc);

/*
  Calculate the geocentric latitude of the point
*/
*latc = asin(zc / *radius);

return(SUCCESS);
}
