/******************************************************************************
NAME:           gxx_det_tiepts

PURPOSE:
Determine the tiepoints for a specified band resolution.

DEVELOPMENT HISTORY:
$Header: /llite/cvseo1/eo1/src/global/exxLib/exxGeo/gxx_det_tiepts.c,v 1.2 2000/11/22 22:14:22 storey Exp $
AUTHOR        Change-ID    DATE     Description of Change
------        ---------  --------   ---------------------
Gail Schmidt     1.0     08/25/97   Original development.
Gail Schmidt     1.1     09/22/97   Set the lat, long, elev to 0.0.

RETURN VALUE:   Type = int
Value    Description
-----    -----------
SUCCESS  Successful completion
ERROR    Operation failed

NOTES:
This routine will make sure that the chip windows for the determined tiepoints
don't fall outside the image.  Thus for any given tiepoint, its chip window
(according to the chip_size) will be completely inside the image for the
specified band resolution.

The corner points for the tiepoints are found in the following manner:
  UL(x) = Chip_Size/2               UR(x) = DDR.nsamps - Chip_Size/2
  UL(y) = Chip_Size/2               UR(y) = Chip_Size/2
  LL(x) = Chip_Size/2               LR(x) = DDR.nsamps - Chip_Size/2
  LL(y) = DDR.nlines - Chip_Size/2  LR(y) = DDR.nlines - Chip_Size/2

ALGORITHM REFERENCES:

******************************************************************************
                Landsat 7 Image Assessment System (IAS) Software
                        Property of the U.S. Government
                                NASA/GSFC/EDC
******************************************************************************/

#include "xxx_Types.h"
#include "xxx_LogStatus.h"
#include "xxx_DDR.h"
#include "gxx_const.h"
#include "gxx_structures.h"
#include "gxx_gcplib.h"

int gxx_det_tiepts (
  const xxx_DDR_TYPE *ddr,    /* I: DDR for the desired band resolution   */
  int  nline_pts,             /* I: Number of line tiepoints to create    */
  int  nsamp_pts,             /* I: Number of sample tiepoints to create  */
  struct LNG_LS chip_size,    /* I: Chip size of the tiepoints to create  */
  GCPLIB *gcps)               /* O: Array of GCPLib tiepoints in row-major
                                    format                                */
{
  int i, j,              /* Looping variables                             */
  	  status,			 /* status of return from function				  */
      count;             /* Number of GCPs                                */
  char ErrMsg[STRLEN];   /* String to handle the error messages           */
  double min_x,          /* Minimum x tiepoint value within the image     */
         max_x,          /* Maximum x tiepoint value within the image     */
         min_y,          /* Minimum y tiepoint value within the image     */
         max_y,          /* Maximum y tiepoint value within the image     */
         *x,             /* X values for the GCPs                         */
         *y,             /* Y values for the GCPs                         */
         x_spacing,      /* Line spacing between chips                    */
         y_spacing;      /* Sample spacing between chips                  */

  /* If only one line or sample point then return with an ERROR.
   */
  if (nline_pts <= 1)
  {
    xxx_LogStatus ("GPSLIB", __FILE__, __LINE__,
      "This routine requires more than one line point.");
    return (ERROR);
  }
  if (nsamp_pts <= 1)
  {
    xxx_LogStatus ("GPSLIB", __FILE__, __LINE__,
      "This routine requires more than one sample point.");
    return (ERROR);
  }

  /* Malloc space for the x/y values.
   */
  x = malloc (nsamp_pts * sizeof (double));
  if (x == NULL)
  {
    strcpy (ErrMsg, "Error allocating space.");
    xxx_LogStatus ("GPSLIB", __FILE__, __LINE__, ErrMsg);
    return (ERROR);
  }

  y = malloc (nline_pts * sizeof (double));
  if (y == NULL)
  {
    strcpy (ErrMsg, "Error allocating space.");
    xxx_LogStatus ("GPSLIB", __FILE__, __LINE__, ErrMsg);
    free (x);
    return (ERROR);
  }

  /* Find the min and max corner point values for the chips inside of the
   * image.
   */
  min_x = chip_size.samp / 2.0;
  max_x = ddr->NumberSamplesInScene - chip_size.samp/2.0;
  min_y = chip_size.line / 2.0;
  max_y = ddr->NumberLinesInScene - chip_size.line/2.0;

  /* Determine the line and sample spacings for all other points between
   * the min and max points.
   */
  x_spacing = (max_x - min_x) / (nsamp_pts - 1);
  y_spacing = (max_y - min_y) / (nline_pts - 1);

  /* Fill in the x and y GCP values.
   */
  for (i = 0; i < nsamp_pts-1; i++)
    x[i] = min_x + x_spacing * i;
  for (i = 0; i < nline_pts-1; i++)
    y[i] = min_y + y_spacing * i;
  x[nsamp_pts-1] = max_x;
  y[nline_pts-1] = max_y;

  /* Fill in the complete GCP structures.
   */
  for (i = 0, count = 0; i < nline_pts; i++)
    for (j = 0; j < nsamp_pts; j++, count++)
    {
      status = sprintf (gcps[count].pt_id, "%d", count);
	  if (status < 1)
	  	{
		xxx_LogStatus ("GPSLIB", __FILE__, __LINE__, "Error in sprintf");
		return (ERROR);
		}
      gcps[count].ref_line = y[i];
      gcps[count].ref_samp = x[j];
      gcps[count].pix_size[0] = ddr->ProjectionDistY;  /* Height */
      gcps[count].pix_size[1] = ddr->ProjectionDistX;  /* Width  */
      gcps[count].ref_size[0] = chip_size.line;
      gcps[count].ref_size[1] = chip_size.samp;
      gcps[count].lat = 0.0;
      gcps[count].lon = 0.0;
      gcps[count].elev = 0.0;
    }

  /* Free up the memory.
   */
  free (x);
  free (y);
  
  return (SUCCESS);
}
