/*******************************************************************************
NAME:		gxx_geod2cart

PURPOSE: 	Convert geodetic coordinate (lat, lon, height) into Cartesian
		coordinates (x, y, z).

DEVELOPMENT HISTORY:
$Log: gxx_geod2cart.c,v $
Revision 1.2  2000/11/22 22:14:22  storey
Replaced by exx_transmat.c

Revision 1.1.1.1  1999/12/01 19:01:20  mbs
Import of IAS 3.22

 * 
 *    Rev 1.3   Thu Aug 28 12:44:18 1997   snelson
 * Made code review changes.  Added error checks.
 * General cleanup.

RETURN VALUE:
none

NOTES: 		Input lat & lon are in radians, height, semi-major axis, and 
		output Cartesian position vector are in meters
		flattening is a unitless number.

ALGORITHM REFERENCES:
		Geodesy: The Concept, by Vanicek and Karkivsky, 1986;
		or any other geodesy textbook.

********************************************************************************
                Landsat 7 Image Assessment System (IAS) Software
                        Property of the U.S. Government
                                NASA/GSFC/EDC
*******************************************************************************/
#include "gxx_geometric.h"

void gxx_geod2cart(
        double latitude,	/* I: Lat of geodetic coordinates in radians */
        double longitude,	/* I: Long of geodetic coordinates in radians*/
        double height,		/* I: Height (elevation) of geodetic coord in
					meters */
        double semimajor,	/* I: Reference ellipsoid semi-major axis in
					meters */
        double flattening,	/* I: Flattening of the ellipsoid 
				     (semimajor-semiminor)/semimajor 	*/
	VECTOR *cart)		/* O: Cartesian vector for the coord	*/
{
double	prime_vertical_radius,	/* radius of prime vertical     */
	ecc2,			/* square of eccentricity       */
	coslat, sinlat;		/* cosine and sine of lat       */

/*
 * Calculate the radius of prime vertical of the ellipsoid
 */ 
ecc2 = flattening * (2.0 - flattening);
coslat = cos(latitude);
sinlat = sin(latitude);
/*
   Since sinlat is less than or equal to 1 and ecc2 is less than 1 no
   test is performed for negative square root or division by zero
*/
prime_vertical_radius = semimajor / sqrt(1.0 - ecc2 * sinlat * sinlat);

/*
 * Calculate the cartesian coordinates
 */
cart->x = (prime_vertical_radius + height)*coslat*cos(longitude);
cart->y = (prime_vertical_radius + height)*coslat*sin(longitude);
cart->z = (prime_vertical_radius * (1.0 - flattening) * (1.0 - flattening) + 
	   height) * sinlat;

return;
}
