/****************************************************************************
NAME:                           gxx_mapping  
 
PURPOSE: Calculate mapping coefficients for grid structure.
 
DEVELOPMENT HISTORY:
$LOG$  

 
NOTES: Using the input/output line/samples in a GRID structure these 
    functions calculate the corresponding forward or inverse mapping
    coefficients.  All functions use the QR decomposition to calculate
    the least squares fit of four corners of a grid cell. 

	GET_FORWARD_MAPPING - calculates the forward mapping for a grid cell.

	GET_INVERSE_MAPPING - calculates the forward mapping for a grid cell.

	EXTEND_TO_OUTPUT - Used to calculate scan gap.  Extends a given pixel 
   location from the input space to a pixel location in the output space.

	REVERSE_TO_INPUT - Used to calculate scan gap.  Find the input pixel
   location for a given out pixel location.

	GET_MAPPINGS - Calls both get_forward_mapping and get_inverse_mapping
   to find the forward and reverse mapping coefficients for a grid cell.
 
*****************************************************************************/
#include "gxx_geometric.h"
#include "gxx_geo_math.h"

long qflag = 0;		/* flag for c_qrdecomp indicating matrix is in col */

void gxx_get_forward_mapping  
( 
   gxx_GRID_BAND_TYPE *grid_band_ptr, /* I: */
   double *ol,		/* I: */
   double *os,		/* I: */
   long *il,		/* I: */
   long *is,		/* I: */
   long r,		/* I: */
   long c,		/* I: */
   double *a,		/* O: */
   double *b  		/* O: */
)
{
   double A[16];
   double v[4];
   long ncols;
 
   ncols = grid_band_ptr->ngrid_samps;
 
   /*-----------------------------------------------------------------------
     Pack the a matrix with input lines/samples.
     ----------------------------------------------------------------------*/
   A[0]  = 1.0;
   A[1]  = 1.0;
   A[2]  = 1.0;
   A[3]  = 1.0;
   A[4]  = is[c];
   A[5]  = is[c+1];
   A[6]  = is[c];
   A[7]  = is[c+1];
   A[8]  = il[r];
   A[9]  = il[r];
   A[10] = il[r+1];
   A[11] = il[r+1];
   A[12] = A[4]*A[8];
   A[13] = A[5]*A[9];
   A[14] = A[6]*A[10];
   A[15] = A[7]*A[11];
 
   /*------------------------------------------------------------------------ 
      Pack the vector a with output samples.
      ---------------------------------------------------------------------*/
   a[0]=os[ncols*r+c];
   a[1]=os[ncols*r+c+1];
   a[2]=os[ncols*(r+1)+c];
   a[3]=os[ncols*(r+1)+c+1];
 
   gxx_QR( A, 4, 4, v,qflag );
   gxx_QRsolve( A, 4, 4, v, a, 0 );
 
   /*------------------------------------------------------------------------ 
      Pack the vector b with output lines.
      ---------------------------------------------------------------------*/
   b[0]=ol[ncols*r+c];
   b[1]=ol[ncols*r+c+1];
   b[2]=ol[ncols*(r+1)+c];
   b[3]=ol[ncols*(r+1)+c+1];
 
   gxx_QRsolve( A, 4, 4, v, b, 0 );
 
   return;
}

/***************************************************************************
NAME:			GET_INVERSE_MAPPING

PURPOSE:	Get coefficients for QR decomposition

***************************************************************************/

void gxx_get_inverse_mapping  
(
   gxx_GRID_BAND_TYPE *grid_band_ptr, /* I: */
   double *ol,		/* I: */
   double *os,		/* I: */
   long *il,		/* I: */
   long *is,		/* I: */
   long r,		/* I: */
   long c,		/* I: */
   double *a,		/* O: */
   double *b  		/* O: */ 
)
{
   double A[16];
   double v[4];
   long ncols;
 
   ncols = grid_band_ptr->ngrid_samps;
 
   /*-----------------------------------------------------------------------
     Pack the a matrix with output lines/samples.
     ----------------------------------------------------------------------*/
   A[0]  = 1.0;
   A[1]  = 1.0;
   A[2]  = 1.0;
   A[3]  = 1.0;
   A[4]  = os[ncols*r+c];
   A[5]  = os[ncols*r+c+1];
   A[6]  = os[ncols*(r+1)+c];
   A[7]  = os[ncols*(r+1)+c+1];
   A[8]  = ol[ncols*r+c];
   A[9]  = ol[ncols*r+c+1];
   A[10] = ol[ncols*(r+1)+c];
   A[11] = ol[ncols*(r+1)+c+1];
   A[12] = A[4]*A[8];
   A[13] = A[5]*A[9];
   A[14] = A[6]*A[10];
   A[15] = A[7]*A[11];
 
   /*------------------------------------------------------------------------ 
      Pack the vector a with input samples.
      ---------------------------------------------------------------------*/
   a[0]  = is[c];
   a[1]  = is[c+1];
   a[2]  = is[c];
   a[3]  = is[c+1];
 
   gxx_QR( A, 4, 4, v, qflag );
   gxx_QRsolve( A, 4, 4, v, a, 0 );
 
   /*------------------------------------------------------------------------ 
      Pack the vector b with input lines.
      ---------------------------------------------------------------------*/
   b[0] = il[r];
   b[1] = il[r];
   b[2] = il[r+1];
   b[3] = il[r+1];
 
   gxx_QRsolve( A, 4, 4, v, b, 0 );
 
   return;
}

/***************************************************************************
NAME:				EXTEND_TO_OUTPUT

PURPOSE:   extend_to_output finds the output space position for an 
   	   in_x (sample) and in_y (line) position in the input space. 

***************************************************************************/
void extend_to_output  
(
   gxx_GRID_BAND_TYPE *grid_band_ptr, /* I: */
   double *coeff,	/* O: */
   double *ol,		/* I: */
   double *os,		/* I: */
   long *il,		/* I: */
   long *is,		/* I: */
   long r,		/* I: */
   long c,		/* I: */
   double *out_x,	/* O: */
   double *out_y 	/* O: */
)
{
   long nrows;
   long ncols;
   double in_x;
   double in_y;
   double a[4],b[4];

   nrows = grid_band_ptr->ngrid_lines;
   ncols = grid_band_ptr->ngrid_samps;

   if( r<(nrows-2) )
      in_y = il[r+2];
   else
      in_y = 0.0;
   in_x = is[c];

   gxx_get_forward_mapping(grid_band_ptr, ol, os, il, is, r, c, a, b);

   *out_x = a[0] + a[1]*in_x + a[2]*in_y + a[3]*in_x*in_y;
   *out_y = b[0] + b[1]*in_x + b[2]*in_y + b[3]*in_x*in_y;

   gxx_get_inverse_mapping(grid_band_ptr, ol, os, il, is, r, c, a, b);
 
   *coeff = a[0]; coeff++;
   *coeff = a[1]; coeff++;
   *coeff = a[2]; coeff++;
   *coeff = a[3]; coeff++;
 
   *coeff = b[0]; coeff++;
   *coeff = b[1]; coeff++;
   *coeff = b[2]; coeff++;
   *coeff = b[3]; coeff++;
 
   return;
}

/**************************************************************************
NAME:				REVERSE_TO_INPUT

PURPOSE:	reverse_to_input finds the input space position for an 
		in_x (sample) and in_y (line) position in the output space. 

***************************************************************************/
void reverse_to_input	
(
   gxx_GRID_BAND_TYPE *grid_band_ptr, /* I: */
   double *ol,		/* I: */
   double *os,		/* I: */
   long *il,		/* I: */
   long *is,		/* I: */
   long r,		/* I: */
   long c,		/* I: */
   double in_x,		/* I: */
   double in_y,		/* I: */
   double *out_x,	/* O: */
   double *out_y 	/* O: */
)
{
   double a[COEFS_SIZE],b[COEFS_SIZE];
 
   gxx_get_inverse_mapping(grid_band_ptr, ol, os, il, is, r, c, a, b);

   *out_x = a[0] + a[1]*in_x + a[2]*in_y + a[3]*in_x*in_y;
   *out_y = b[0] + b[1]*in_x + b[2]*in_y + b[3]*in_x*in_y;
   return;
}

/****************************************************************************
NAME:				GET_MAPPINGS

PURPOSE:	get_mappings gets and stores in a grid both the forward
		and inverse mapping coefficients.

***************************************************************************/
void gxx_get_mappings	
( 
   gxx_GRID_BAND_TYPE *grid_band_ptr, /* I: */
   struct COEFFICIENTS *coeff,	/* O: */
   double *ol,			/* I: */
   double *os,			/* I: */
   long *il,			/* I: */
   long *is,			/* I: */
   long r,			/* I: */
   long c 			/* I: */
)
{
   double b[COEFS_SIZE];
   double a[COEFS_SIZE];
 
   gxx_get_inverse_mapping(grid_band_ptr, ol, os, il, is, r, c, a, b);
 
   coeff->a[0] = a[0]; 
   coeff->a[1] = a[1];
   coeff->a[2] = a[2];
   coeff->a[3] = a[3];

   coeff->b[0] = b[0];
   coeff->b[1] = b[1];
   coeff->b[2] = b[2];
   coeff->b[3] = b[3];

   return;
}
 
