/****************************************************************************
NAME:                           gxx_sunangles.c

PURPOSE: Given the year, month, day, second, and the scene center's
         Latitude and Longitude. Calculate the solar elevation and azimuth
	   	angles.

DEVELOPMENT HISTORY:
$Log: gxx_sunangles.c,v $
Revision 1.2  2000/11/22 22:14:22  storey
Replaced by exx_transmat.c

Revision 1.1.1.1  1999/12/01 19:01:20  mbs
Import of IAS 3.22

 * 
 *    Rev 1.4   Wed Aug 19 15:40:46 1998   vanderv
 *  puts 
 * fill values in the angles if the incoming center lat and lon are null.In addition
 * to checking for incoming fill values in the lat/lon fields it changes the way 
 * the azimuth is computed to avoid a divide by zero condition when the sun is 
 * directly overhead (zenith angle of 0 or elevation of 90 degrees).
 * 
 *    Rev 1.3   Thu Mar 05 10:11:54 1998   tbeckman
 * Changed longs to ints to remove warnings from the -64 compile.
 * 
 *    Rev 1.2   Wed Aug 06 13:06:04 1997   tgardner
 * added variable descriptions and substituted
 * defined parameters for two constants used.

RETURN VALUE:
Type = int
Value           Description
-----           -----------
SUCCESS         Successfully calculated sun angles.


****************************************************************************/


#include <math.h>
#include "gxx_algorithm.h"
#include "gxx_const.h"
#include "gxx_geo_math.h"


#define HOURS2DEG	15.0	/* hours to degrees conversion */

int gxx_sunangles
(
   int  year,		/* I: Year (i.e. 1997) */
   int  month, 		/* I: Month in year */
   int  day,   		/* I: Day of Month */
   double seconds,   	/* I: Total seconds into day for scene center    */
   double centerlat, 	/* I: Scene center geodetic latitude (radians)  */
   double centerlon, 	/* I: Scene center longitude (radians 0 -> 2PI) */
   double *solarelev, 	/* O: Solar elevation angle (radians) */
   double *solaraz    	/* O: Solar azimuth angle (radians) */
)

{
   double tjd;		/* full julian day */
   double agha;		/* Greenwich Hour Angle of Aries */
   double sgha;		/* solar Greenwich Hour Angle */
   double ra;		/* right ascension referred to mean equator and
                           equinox of date (hours) */
   double dec;		/* declination referred to mean equator and 
			   equinox of date (radians) */
   double dis;		/* geocentric distance (AU) */
   double sunlon;	/* suns projected longitude */
   double solarzen;	/* solar zenith angle */
   double deltalon;	/* diff between the suns projected longitude and
                           and the longitude at the center of the scene */
   double sunx;		/* solar vector projection on local X axis */
   double suny;		/* solar vector projection on local Y axis */

/*----------------------------------------------------------------
         See if a valid position was input
----------------------------------------------------------------*/

	if ( (centerlat > PIE2) || (centerlat < -PIE2) ||
             (centerlon > PIE2) || (centerlon < -PIE2) )
	{
	  *solaraz = -999.0;
          *solarelev = -999.0;
	  return(SUCCESS);
	}

/*----------------------------------------------------------------
	 Find the Full Julian day for the scene center 
----------------------------------------------------------------*/

	tjd = gxx_fulljd(year, month, day, seconds);

/*----------------------------------------------------------------
 	Find the Sun's spherical position in inertial (TOD) space 
----------------------------------------------------------------*/

	gxx_sphsunpostod (tjd, &ra, &dec, &dis);

/*----------------------------------------------------------------
 	Find the Greenwich Hour Angle (GHA) of Aries  
----------------------------------------------------------------*/

	gxx_ariesgha(tjd, seconds, &agha);

/*----------------------------------------------------------------
 	Find the GHA of the Sun 
----------------------------------------------------------------*/

	sgha = agha - ra * HOURS2DEG * D2R;
	if (sgha >= PIE2) sgha = sgha - PIE2;
	if (sgha < 0.0) sgha = PIE2 + sgha;

/*----------------------------------------------------------------
 	Find the Sun's projected Longitude 
----------------------------------------------------------------*/

	sunlon = PIE2 - sgha;

/*----------------------------------------------------------------
        Calculate results using simple spherical relationships
 	Assume the vector from the Earth's center to the Sun is parallel
        to the vector from the scene's center to the Sun 

 	Solar elevation calculation  
----------------------------------------------------------------*/

	deltalon = sunlon - centerlon;

	solarzen = acos(sin(dec) * sin(centerlat) + 
	     cos(dec) * cos(centerlat) * cos(deltalon));

	*solarelev = HALFPIE - solarzen;

/*----------------------------------------------------------------
 	Solar azimuth calculation 
 	Define North as Zero degrees in azimuth and East as 90 degrees 
----------------------------------------------------------------*/
	sunx = cos(dec) * sin(deltalon);
	suny = sin(dec)*cos(centerlat) - cos(dec)*cos(deltalon)*sin(centerlat);
	if ( fabs( sunx ) > 0.0 || fabs( suny ) > 0.0 )
	  *solaraz = atan2( sunx, suny );
	else
	  *solaraz = 0.0;

	return(SUCCESS);

}
