/******************************************************************************
NAME: exx_get_ali_focal_plane_parameters

PURPOSE:
Read the cpf's focal_plane_parameters params one by one using GSFC's 
xxx_GetODLField, then stuff them into a structure.

DEVELOPMENT HISTORY:
$Log: exx_get_ali_focal_plane_parameters.c,v $
Revision 1.1  2000/11/27 20:33:44  storey
Original version


RETURN VALUE:
Type = int
Value    Description
-----    -----------
  0      Successful completion (SUCCESS)
  1      Operation failed (ERROR)

NOTES:
This function assumes that the CPF has been opened already.

ALGORITHM REFERENCES:

******************************************************************************
                        Property of the U.S. Government
                            USGS EROS Data Center
******************************************************************************/
#include <stdio.h>
#include "xxx_Types.h"  /* include defines ERROR variables (xxx_EDCOdl_type)*/
#include "xxx_LogStatus.h"
#include "xxx_const.h"
#include "exx_cpf.h"


int exx_get_ali_focal_plane_parameters(
    CPF *cpf,	/* O: the structure of CPF info			*/
    OBJDESC *cpf_parms	/* I: params read from CPF, and to be
			      put in the cpf structure (lablib3.h)	*/
    )

{
/* VARS */
int status;			/* status of return from function	*/
int sz;				/* size of a variable			*/
int i;				/* SCA loop variable			*/
int j;				/* Band loop variable			*/

char group_name[STRLEN];	/* SCA parameter group name		*/
char parm_name[STRLEN];		/* SCA parameter name			*/

/* GSFC CSC LIB VARS */
xxx_ValType_TYPE parm_type;	/* the type of param to read from CPF 
				     (xxx_ODL.h)			*/

/* **** FOCAL PLANE PARAMETERS **** */
/* retrieve the number of SCAs */
parm_type = XXX_Int;
sz = sizeof(cpf->Focal_Plane_Parameters.Num_SCA);
status = exx_read_cpf_param(
	&(cpf->Focal_Plane_Parameters.Num_SCA), 
	cpf_parms, parm_type, sz, "ALI_FOCAL_PLANE_PARAMETERS","Num_SCA");
if (status != SUCCESS) 
{
    xxx_LogStatus("EXXLIB", __FILE__, __LINE__,
	"Error returned from exx_read_cpf_param");
    return(ERROR);
}

/* Loop through the SCAs */
for ( i=0; i<cpf->Focal_Plane_Parameters.Num_SCA; i++ )
{

  /* Construct this SCA's parameter group name */
  sprintf(group_name, "ALI_SCA%02d_PARAMETERS", i+1 );

  /* retrieve the number of detectors in each band */
  parm_type = XXX_Int;
  sz = sizeof(cpf->Focal_Plane_Parameters.SCA_Parameters[i].Number_Detectors);
  status = exx_read_cpf_param(
	  (cpf->Focal_Plane_Parameters.SCA_Parameters[i].Number_Detectors), 
	  cpf_parms, parm_type, sz, group_name,"Number_Detectors");
  if (status != SUCCESS) 
  {
    xxx_LogStatus("EXXLIB", __FILE__, __LINE__,
	"Error returned from exx_read_cpf_param");
    return(ERROR);
  }

  /* Loop through the bands to retreive the Along-track LOS coefficients */
  for ( j=0; j<NUM_ALI_BANDS; j++ )
  {
    /* Construct this band's parameter names */
    sprintf(parm_name, "Along_LOS_Legendre_b%d", j );

    /* retrieve the Legendre parameters for this band */
    parm_type = XXX_Float;
    sz = sizeof(cpf->Focal_Plane_Parameters.SCA_Parameters[i].Along_LOS_Legendre[j]);
    status = exx_read_cpf_param(
	    (cpf->Focal_Plane_Parameters.SCA_Parameters[i].Along_LOS_Legendre[j]), 
	    cpf_parms, parm_type, sz, group_name, parm_name);
    if (status != SUCCESS) 
    {
      xxx_LogStatus("EXXLIB", __FILE__, __LINE__,
  	  "Error returned from exx_read_cpf_param");
      return(ERROR);
    }
  }

  /* Loop through the bands to retreive the Across-track LOS coefficients */
  for ( j=0; j<NUM_ALI_BANDS; j++ )
  {
    /* Construct this band's parameter names */
    sprintf(parm_name, "Across_LOS_Legendre_b%d", j );

    /* retrieve the Legendre parameters for this band */
    parm_type = XXX_Float;
    sz = sizeof(cpf->Focal_Plane_Parameters.SCA_Parameters[i].Across_LOS_Legendre[j]);
    status = exx_read_cpf_param(
	    (cpf->Focal_Plane_Parameters.SCA_Parameters[i].Across_LOS_Legendre[j]), 
	    cpf_parms, parm_type, sz, group_name, parm_name);
    if (status != SUCCESS) 
    {
      xxx_LogStatus("EXXLIB", __FILE__, __LINE__,
  	  "Error returned from exx_read_cpf_param");
      return(ERROR);
    }
  }

  /* retrieve the detector offset parameters for this band */
  parm_type = XXX_Float;
  sz = sizeof(cpf->Focal_Plane_Parameters.SCA_Parameters[i].Along_Detector_Offset_PAN);
  status = exx_read_cpf_param(
	 (cpf->Focal_Plane_Parameters.SCA_Parameters[i].Along_Detector_Offset_PAN), 
	 cpf_parms, parm_type, sz, group_name, "Along_Detector_Offset_PAN");
  if (status != SUCCESS) 
  {
    xxx_LogStatus("EXXLIB", __FILE__, __LINE__,
  	"Error returned from exx_read_cpf_param");
    return(ERROR);
  }
  /* Loop through the bands to retrieve the Along-track detector offsets */
  for ( j=1; j<NUM_ALI_BANDS; j++ )
  {
    /* Construct this band's parameter names */
    sprintf(parm_name, "Along_Detector_Offset_MS_b%d", j );

    /* retrieve the detector offset parameters for this band */
    parm_type = XXX_Float;
    sz = sizeof(cpf->Focal_Plane_Parameters.SCA_Parameters[i].Along_Detector_Offset_MS[j-1]);
    status = exx_read_cpf_param(
	    (cpf->Focal_Plane_Parameters.SCA_Parameters[i].Along_Detector_Offset_MS[j-1]), 
	    cpf_parms, parm_type, sz, group_name, parm_name);
    if (status != SUCCESS) 
    {
      xxx_LogStatus("EXXLIB", __FILE__, __LINE__,
  	  "Error returned from exx_read_cpf_param");
      return(ERROR);
    }
  }

  /* retrieve the detector offset parameters for this band */
  parm_type = XXX_Float;
  sz = sizeof(cpf->Focal_Plane_Parameters.SCA_Parameters[i].Across_Detector_Offset_PAN);
  status = exx_read_cpf_param(
	 (cpf->Focal_Plane_Parameters.SCA_Parameters[i].Across_Detector_Offset_PAN), 
	 cpf_parms, parm_type, sz, group_name, "Across_Detector_Offset_PAN");
  if (status != SUCCESS) 
  {
    xxx_LogStatus("EXXLIB", __FILE__, __LINE__,
  	"Error returned from exx_read_cpf_param");
    return(ERROR);
  }
  /* Loop through the bands to retrieve the Across-track detector offsets */
  for ( j=1; j<NUM_ALI_BANDS; j++ )
  {
    /* Construct this band's parameter names */
    sprintf(parm_name, "Across_Detector_Offset_MS_b%d", j );

    /* retrieve the detector offset parameters for this band */
    parm_type = XXX_Float;
    sz = sizeof(cpf->Focal_Plane_Parameters.SCA_Parameters[i].Across_Detector_Offset_MS[j-1]);
    status = exx_read_cpf_param(
	    (cpf->Focal_Plane_Parameters.SCA_Parameters[i].Across_Detector_Offset_MS[j-1]), 
	    cpf_parms, parm_type, sz, group_name, parm_name);
    if (status != SUCCESS) 
    {
      xxx_LogStatus("EXXLIB", __FILE__, __LINE__,
  	  "Error returned from exx_read_cpf_param");
      return(ERROR);
    }
  }

} /* SCA loop */

/* retrieve the nominal odd/even detector offsets */
parm_type = XXX_Float;
sz = sizeof(cpf->Focal_Plane_Parameters.Odd_Even_Offsets.Nom_Detector_Shift);
status = exx_read_cpf_param(
	 (cpf->Focal_Plane_Parameters.Odd_Even_Offsets.Nom_Detector_Shift), 
	 cpf_parms, parm_type, sz, "ALI_ODD_EVEN_OFFSETS","Nom_Detector_Shift");
if (status != SUCCESS) 
{
  xxx_LogStatus("EXXLIB", __FILE__, __LINE__,
    "Error returned from exx_read_cpf_param");
  return(ERROR);
}

/* retrieve the nominal even detector offsets */
parm_type = XXX_Float;
sz = sizeof(cpf->Focal_Plane_Parameters.Odd_Even_Offsets.Even_Detector_Shift);
status = exx_read_cpf_param(
	 (cpf->Focal_Plane_Parameters.Odd_Even_Offsets.Even_Detector_Shift), 
	 cpf_parms, parm_type, sz, "ALI_ODD_EVEN_OFFSETS","Even_Detector_Shift");
if (status != SUCCESS) 
{
  xxx_LogStatus("EXXLIB", __FILE__, __LINE__,
    "Error returned from exx_read_cpf_param");
  return(ERROR);
}

/* retrieve the nominal odd detector offsets */
parm_type = XXX_Float;
sz = sizeof(cpf->Focal_Plane_Parameters.Odd_Even_Offsets.Odd_Detector_Shift);
status = exx_read_cpf_param(
	 (cpf->Focal_Plane_Parameters.Odd_Even_Offsets.Odd_Detector_Shift), 
	 cpf_parms, parm_type, sz, "ALI_ODD_EVEN_OFFSETS","Odd_Detector_Shift");
if (status != SUCCESS) 
{
  xxx_LogStatus("EXXLIB", __FILE__, __LINE__,
    "Error returned from exx_read_cpf_param");
  return(ERROR);
}

return(SUCCESS);
}
