/******************************************************************************
NAME: exx_get_hyp_focal_plane_parameters

PURPOSE:
Read the cpf's focal_plane_parameters params one by one using GSFC's 
xxx_GetODLField, then stuff them into a structure.

DEVELOPMENT HISTORY:
Original version


RETURN VALUE:
Type = int
Value    Description
-----    -----------
  0      Successful completion (SUCCESS)
  1      Operation failed (ERROR)

NOTES:
This function assumes that the CPF has been opened already.

ALGORITHM REFERENCES:

******************************************************************************
                        Property of the U.S. Government
                            USGS EROS Data Center
******************************************************************************/
#include <stdio.h>
#include "xxx_Types.h"  /* include defines ERROR variables (xxx_EDCOdl_type)*/
#include "xxx_LogStatus.h"
#include "xxx_const.h"
#include "exx_cpf.h"


int exx_get_hyp_focal_plane_parameters(
    CPF *cpf,	/* O: the structure of CPF info			*/
    OBJDESC *cpf_parms	/* I: params read from CPF, and to be
			      put in the cpf structure (lablib3.h)	*/
    )

{
/* VARS */
int status;			/* status of return from function	*/
int sz;				/* size of a variable			*/
int i;				/* SCA loop variable			*/
int j;				/* Band loop variable			*/

char group_name[STRLEN];	/* SCA parameter group name		*/
char parm_name[STRLEN];		/* SCA parameter name			*/

/* GSFC CSC LIB VARS */
xxx_ValType_TYPE parm_type;	/* the type of param to read from CPF 
				     (xxx_ODL.h)			*/

/* **** FOCAL PLANE PARAMETERS **** */
/* retrieve the number of Detectors */
parm_type = XXX_Int;
sz = sizeof(cpf->Hyp_Focal_Plane_Parameters.Number_Detectors);
status = exx_read_cpf_param(
	&(cpf->Hyp_Focal_Plane_Parameters.Number_Detectors), 
	cpf_parms, parm_type, sz, "HYP_FOCAL_PLANE_PARAMETERS","Number_Detectors");
if (status != SUCCESS) 
{
    xxx_LogStatus("EXXLIB", __FILE__, __LINE__,
	"Error returned from exx_read_cpf_param");
    return(ERROR);
}

/* Loop through the bands to retreive the Along-track LOS coefficients */
for ( j=1; j<=NUM_HYP_BANDS; j++ )
{
  /* Construct this band's parameter names */
  sprintf(parm_name, "Along_LOS_Legendre_b%d", j );

  /* retrieve the Legendre parameters for this band */
  parm_type = XXX_Float;
  sz = sizeof(cpf->Hyp_Focal_Plane_Parameters.Along_LOS_Legendre[j-1]);
  status = exx_read_cpf_param(
          (cpf->Hyp_Focal_Plane_Parameters.Along_LOS_Legendre[j-1]), 
          cpf_parms, parm_type, sz, "HYP_FOCAL_PLANE_PARAMETERS", parm_name);
  if (status != SUCCESS) 
  {
    xxx_LogStatus("EXXLIB", __FILE__, __LINE__, "Error returned from exx_read_cpf_param");
    return(ERROR);
  }
}

/* Loop through the bands to retreive the Along-track LOS coefficients */
for ( j=1; j<=NUM_HYP_BANDS; j++ )
{
  /* Construct this band's parameter names */
  sprintf(parm_name, "Across_LOS_Legendre_b%d", j );

  /* retrieve the Legendre parameters for this band */
  parm_type = XXX_Float;
  sz = sizeof(cpf->Hyp_Focal_Plane_Parameters.Across_LOS_Legendre[j-1]);
  status = exx_read_cpf_param(
          (cpf->Hyp_Focal_Plane_Parameters.Across_LOS_Legendre[j-1]), 
          cpf_parms, parm_type, sz, "HYP_FOCAL_PLANE_PARAMETERS", parm_name);
  if (status != SUCCESS) 
  {
    xxx_LogStatus("EXXLIB", __FILE__, __LINE__, "Error returned from exx_read_cpf_param");
    return(ERROR);
  }
}

/* Loop through the bands to retrieve the Along-track detector offsets */
for ( j=1; j<=NUM_HYP_BANDS; j++ )
{
  /* Construct this band's parameter names */
  sprintf(parm_name, "Along_Detector_Offset_b%d", j );

  /* retrieve the detector offset parameters for this band */
  parm_type = XXX_Float;
  sz = sizeof(cpf->Hyp_Focal_Plane_Parameters.Along_Detector_Offset[j-1]);
  status = exx_read_cpf_param(
      (cpf->Hyp_Focal_Plane_Parameters.Along_Detector_Offset[j-1]), 
      cpf_parms, parm_type, sz, "HYP_FOCAL_PLANE_PARAMETERS", parm_name);
  if (status != SUCCESS) 
  {
      xxx_LogStatus("EXXLIB", __FILE__, __LINE__,
  	  "Error returned from exx_read_cpf_param");
      return(ERROR);
  }
}

/* Loop through the bands to retrieve the Across-track detector offsets */
for ( j=1; j<=NUM_HYP_BANDS; j++ )
{
  /* Construct this band's parameter names */
  sprintf(parm_name, "Across_Detector_Offset_b%d", j );

  /* retrieve the detector offset parameters for this band */
  parm_type = XXX_Float;
  sz = sizeof(cpf->Hyp_Focal_Plane_Parameters.Across_Detector_Offset[j-1]);
  status = exx_read_cpf_param(
      (cpf->Hyp_Focal_Plane_Parameters.Across_Detector_Offset[j-1]),     
      cpf_parms, parm_type, sz, "HYP_FOCAL_PLANE_PARAMETERS", parm_name);
  if (status != SUCCESS)  
  {
      xxx_LogStatus("EXXLIB", __FILE__, __LINE__,
          "Error returned from exx_read_cpf_param");
      return(ERROR);
  }
}

return(SUCCESS);
}
