/******************************************************************************
NAME: exx_read_cpf

PURPOSE:
Read the cpf's params one by one using xxx_GetODLField, then
stuff them into a structure.

DEVELOPMENT HISTORY:
Original revision of read cpf routines.
$Log: exx_read_cpf.c,v $
Revision 1.2  2001/07/09 12:45:21  choate
Added Hyperion

Revision 1.1  2000/11/27 20:33:52  storey
Original version


RETURN VALUE:
Type = CPF *
Value     Description
-----     -----------
NULL      Operation failed 
otherwise Successful completion, the pointer to the CPF read is returned

NOTES:
The CPF pointer that is returned should be free'ed by calling the
exx_free_cpf routine when it is no longer needed.

ALGORITHM REFERENCES:

******************************************************************************
                        Property of the U.S. Government
                            USGS EROS Data Center
******************************************************************************/
#include <stdio.h>
#include "xxx_Types.h"  /* defines ERROR variables (xxx_EDCOdl_type)*/
#include "xxx_LogStatus.h"
#include "xxx_const.h"  /* defined STRLEN */
#include "exx_cpf.h"



CPF *exx_read_cpf(
    const char* cpf_pathname)  /* I: name of cpf file */
{
/* VARS */
int  status;			/* status of return from function	*/

/* HDF/ODL VARS */
OBJDESC *cpf_parms;		/* params read from CPF, and to be
			           put in the cpf structure (lablib3.h)	*/

/* GSFC CSC LIB VARS */
xxx_EDCOdl_TYPE type_of_odl_file;/* the type of ODL file (xxx_Types.h)	*/

char error_msg[STRLEN];

char *filename; /* filename to use for reading the CPF */
CPF *cpf;       /* pointer to CPF read from file */

/* allocate memory for the CPF structure */
cpf = malloc(sizeof(CPF));
if (cpf == NULL)
{
    xxx_LogStatus("EXXLIB",__FILE__,__LINE__,"Error allocating memory for CPF");
    return(NULL);
}

/* zero the cpf structure so any pointers it contains are NULL'ed */
memset(cpf,0,sizeof(CPF));

/* ******** */
/* OPEN CPF */
/* ******** */

/* use the cpf name specified */
/* set the file name to use to the specified name */
filename = (char *)cpf_pathname;

/* tell xxx_OpenODL not to look for the CPF name */
type_of_odl_file = xxx_NoEDCMeta;

/* open the HDF/ODL CPF file */
cpf_parms = xxx_OpenODL(filename, type_of_odl_file, error_msg);
if(cpf_parms == NULL)
{
   xxx_LogStatus("EXXLIB",__FILE__,__LINE__,error_msg);
   xxx_LogStatus("EXXLIB",__FILE__,__LINE__,"Error opening CPF");
   free(cpf);
   return(NULL);
}

/* ******** */
/* READ CPF */
/* ******** */

status = exx_get_file_attributes(cpf,cpf_parms);
    if (status != SUCCESS)
    {
	xxx_LogStatus("EXXLIB", __FILE__, __LINE__, 
	    "Error retrieving cpf information from exx_get_file_attributes");
        xxx_CloseODL(cpf_parms, error_msg);
        exx_free_cpf(cpf);
        return(NULL);
    }

status = exx_get_earth_constants(cpf,cpf_parms);
    if (status != SUCCESS)
    {
	xxx_LogStatus("EXXLIB", __FILE__, __LINE__, 
	    "Error retrieving cpf infomation from exx_get_earth_constants");
        xxx_CloseODL(cpf_parms, error_msg);
        exx_free_cpf(cpf);
        return(NULL);
    }

status = exx_get_orbit_parameters(cpf,cpf_parms);
    if (status != SUCCESS)
    {
	xxx_LogStatus("EXXLIB", __FILE__, __LINE__, 
	    "Error retrieving cpf infomation from exx_get_orbit_parameters");
        xxx_CloseODL(cpf_parms, error_msg);
        exx_free_cpf(cpf);
        return(NULL);
    }

status = exx_get_ali_parameters(cpf,cpf_parms);
    if (status != SUCCESS)
    {
	xxx_LogStatus("EXXLIB", __FILE__, __LINE__, 
	    "Error retrieving cpf infomation from exx_get_ali_parameters");
        xxx_CloseODL(cpf_parms, error_msg);
        exx_free_cpf(cpf);
        return(NULL);
    }

status = exx_get_ali_focal_plane_parameters(cpf,cpf_parms);
    if (status != SUCCESS)
    {
	xxx_LogStatus("EXXLIB", __FILE__, __LINE__, 
  	 "Error retrieving cpf infomation from exx_get_ali_focal_plane_parameters");
        xxx_CloseODL(cpf_parms, error_msg);
        exx_free_cpf(cpf);
        return(NULL);
    }

status = exx_get_attitude_parameters(cpf,cpf_parms);
    if (status != SUCCESS)
    {
	xxx_LogStatus("EXXLIB", __FILE__, __LINE__, 
	    "Error retrieving cpf infomation from exx_get_attitude_parameters");
        xxx_CloseODL(cpf_parms, error_msg);
        exx_free_cpf(cpf);
        return(NULL);
    }

status = exx_get_UT1_time_parameters(cpf,cpf_parms);
    if (status != SUCCESS)
    {
	xxx_LogStatus("EXXLIB", __FILE__, __LINE__, 
	    "Error retrieving cpf infomation from exx_get_UT1_time_parameters");
        xxx_CloseODL(cpf_parms, error_msg);
        exx_free_cpf(cpf);
        return(NULL);
    }

status = exx_get_ali_scaling_parameters(cpf,cpf_parms);
    if (status != SUCCESS)
    {
	xxx_LogStatus("EXXLIB", __FILE__, __LINE__, 
	    "Error retrieving cpf infomation from exx_get_ali_scaling_parameters");
        xxx_CloseODL(cpf_parms, error_msg);
        exx_free_cpf(cpf);
        return(NULL);
    }

status = exx_get_ali_mtf_compensation(cpf,cpf_parms);
    if (status != SUCCESS)
    {
	xxx_LogStatus("EXXLIB", __FILE__, __LINE__, 
	    "Error retrieving cpf infomation from exx_get_ali_mtf_compensation");
        xxx_CloseODL(cpf_parms, error_msg);
        exx_free_cpf(cpf);
        return(NULL);
    }

status = exx_get_hyp_focal_plane_parameters(cpf,cpf_parms);
    if (status != SUCCESS)
    {
        xxx_LogStatus("EXXLIB", __FILE__, __LINE__,
            "Error retrieving cpf infomation from exx_get_hyp_focal_plane_parameters");
        xxx_CloseODL(cpf_parms, error_msg);
        exx_free_cpf(cpf);
        return(NULL);
    }

status = exx_get_hyp_mtf_compensation(cpf,cpf_parms);
    if (status != SUCCESS)
    {
        xxx_LogStatus("EXXLIB", __FILE__, __LINE__,
            "Error retrieving cpf infomation from exx_get_hyp_mtf_compensation");
        xxx_CloseODL(cpf_parms, error_msg);
        exx_free_cpf(cpf);
        return(NULL);
    }

status = exx_get_hyp_scaling_parameters(cpf,cpf_parms);
    if (status != SUCCESS)
    {
        xxx_LogStatus("EXXLIB", __FILE__, __LINE__,
            "Error retrieving cpf infomation from exx_get_hyp_scaling_parameters");
        xxx_CloseODL(cpf_parms, error_msg);
        exx_free_cpf(cpf);
        return(NULL);
    }

status = exx_get_hyp_scan_parameters(cpf,cpf_parms);
    if (status != SUCCESS)
    {
        xxx_LogStatus("EXXLIB", __FILE__, __LINE__,
            "Error retrieving cpf infomation from exx_get_hyp_scan_parameters");
        xxx_CloseODL(cpf_parms, error_msg);
        exx_free_cpf(cpf);
        return(NULL);
    }

/* ********* */
/* CLOSE CPF */
/* ********* */
status = xxx_CloseODL(cpf_parms, error_msg);
    if(status == ERROR)
	{
	xxx_LogStatus("EXXLIB",__FILE__,__LINE__,error_msg);
	xxx_LogStatus("EXXLIB",__FILE__,__LINE__,"Error Closing CPF");
	printf("READ_CPF: Error Closing CPF");
        exx_free_cpf(cpf);
	return(NULL);
	}

return(cpf);
}
