/******************************************************************************
NAME:           hxx_attitude

PURPOSE:
Find the line of sight vector from the spacecraft to a point on the ground
by transforming the line of sight vector in sensor coordinates to perturbed
spacecraft coordinates

DEVELOPMENT HISTORY:

RETURN VALUE:
Type = int
Value    Description
-----    -----------
SUCCESS  Successful completion 
ERROR    Operation failed

NOTES:

ALGORITHM REFERENCES:
******************************************************************************
                        Property of the U.S. Government
                            USGS EROS Data Center
******************************************************************************/
#include "xxx_LogStatus.h"
#include "xxx_math.h"
#include "hxx_hypmodel.h"

#define	PRINT	0

int hxx_attitude(
    	const VECTOR *satpos,	/*I: Satellite position (meters) */
    	const VECTOR *satvel,	/*I: Satellite velocity (m/s)*/
    	const VECTOR *sensor_los,/*I: LOS to target vector (sensor coord)*/
    	double 	roll,		/*I: Corrected roll (radians) */
    	double 	pitch,		/*I: Corrected pitch (radians) */
    	double 	yaw,		/*I: Corrected yaw (radians) */
	const CPF *cpf,		/*I: Cal Parm File constants */
        VECTOR *plos,           /*O: perturbed LOS (OCS-orbital coord) */
    	VECTOR *newlos)		/*O: New line of sight (ECEF coord) */
{
VECTOR 	vvec;     		/* velocity vector (unperturbed ECEF) */
VECTOR 	xvec;     		/* vector in direction of geocentric nadir*/
VECTOR 	yvec;     		/* vector in direction of angular momentum*/
VECTOR 	zvec;     		/* vector in direction of circular velocity*/
				/* (ECEF coords)	*/
VECTOR 	nav_los;	 	/* line of sight vector in navigation */
				/* reference coords */

double	sensor2arc[3][3];	/* sensor to attitude control reference */
				/* transformation matrix */
double	magx,			/* magnitude of x vector */
	magy,			/* magnitude of y vector */
	magz,			/* magnitude of z vector */
	cr,			/* cosine of roll */
	sr,			/* sine of roll	*/
	cp,			/* cosine of pitch */
	sp,			/* sine of pitch */
	cy,			/* cosine of yaw */
	sy,			/* sine of yaw */
	perturb[3][3],		/* perturbation of the los */
	transc2ecef[3][3]; 	/* transfer of sc to ecef */
int  status;			/* status of return from function */

/*
 * Take the inverse of the CPF Attitude_To_HYP_Matrix (Attitude_Parameters, 
 * 9 values) to get a matrix to transform from sensor to attitude control 
 * reference
 */
status = xxx_invmatrix(cpf->Attitude_Parameters.Attitude_To_HYP_Matrix, sensor2arc);
if (status != SUCCESS)
{
    xxx_LogStatus("HXXLIB",__FILE__,__LINE__,"Error inverting matrix");
    return(ERROR);
}

/*
 * Find unperturbed spacecraft coordinates in terms of the ecef coords
 */
zvec.x = -satpos->x;
zvec.y = -satpos->y;
zvec.z = -satpos->z;

vvec.x = satvel->x;
vvec.y = satvel->y;
vvec.z = satvel->z;

/*
 * Find vector in the direction of the angular momentum
 */
xxx_cross(&zvec,&vvec,&yvec);

/*
 * Find vector in the direction of the velocity
 */
xxx_cross(&yvec,&zvec,&xvec);

/*
 * Calclulate the magnitude of each vector
 */
magx = xxx_norm(&xvec);
magy = xxx_norm(&yvec);
magz = xxx_norm(&zvec);

/*
 * Transform los from sensor to spacecraft coordinates (navigation reference)
 */
nav_los.x = sensor_los->x * sensor2arc[0][0] +
	    sensor_los->y * sensor2arc[0][1] + 
	    sensor_los->z * sensor2arc[0][2];

nav_los.y = sensor_los->x * sensor2arc[1][0] +
	    sensor_los->y * sensor2arc[1][1] + 
	    sensor_los->z * sensor2arc[1][2];

nav_los.z = sensor_los->x * sensor2arc[2][0] +
	    sensor_los->y * sensor2arc[2][1] + 
	    sensor_los->z * sensor2arc[2][2];

if( PRINT ) printf("   NAV LOS %f %f %f\n",nav_los.x,nav_los.y,nav_los.z);

/*
  Perturbation Matrix
                           -     -    -     -    -     -    
                          |       |  |       |  |       |  
    perturbation matrix = |  yaw  |  | pitch |  | roll  | 
                          |       |  |       |  |       |
                           -     -    -     -    -     - 

  Perturb LOS
       -   -     -     -    -      -    -         -    -      -
      | ECEF|   | orbit |  |        |  |           |  | Sensor |
      | LOS | = |  to   |  |perturb |  | TM to ARC |  |   LOS  |
      |     |   | ECEF  |  |        |  |           |  |        |
       -   -     -     -    -      -    -         -    -      - 


 */
cr = cos(roll);
sr = sin(roll);
cp = cos(pitch);
sp = sin(pitch);
cy = cos(yaw);
sy = sin(yaw);

perturb[0][0] = cp * cy;
perturb[0][1] = cy * sr * sp + cr * sy;
perturb[0][2] = sr * sy - sp * cr * cy;

perturb[1][0] = -sy * cp;
perturb[1][1] = cy * cr - sy * sr * sp;
perturb[1][2] = sp * sy * cr + cy * sr;

perturb[2][0] = sp;
perturb[2][1] = -sr * cp;
perturb[2][2] = cp * cr;

/*
 * Calculate the perturbed los vector (OCI Coords)
 */
plos->x = nav_los.x * perturb[0][0] + nav_los.y * perturb[0][1] + 
	  nav_los.z * perturb[0][2];

plos->y = nav_los.x * perturb[1][0] + nav_los.y * perturb[1][1] + 
	  nav_los.z * perturb[1][2];

plos->z = nav_los.x * perturb[2][0] + nav_los.y * perturb[2][1] + 
	  nav_los.z * perturb[2][2];

/*
 * Transform los to ECEF
 */
if (magx == 0.0 || magy == 0.0 || magz == 0.0)
{
    xxx_LogStatus("HXXLIB",__FILE__,__LINE__,"Can not divide by zero");
    return(ERROR);
}
transc2ecef[0][0] = xvec.x / magx;
transc2ecef[0][1] = yvec.x / magy;
transc2ecef[0][2] = zvec.x / magz;

transc2ecef[1][0] = xvec.y / magx;
transc2ecef[1][1] = yvec.y / magy;
transc2ecef[1][2] = zvec.y / magz;

transc2ecef[2][0] = xvec.z / magx;
transc2ecef[2][1] = yvec.z / magy;
transc2ecef[2][2] = zvec.z / magz;

/*
 * Calculate the new line of sight vector
 */
newlos->x = plos->x * transc2ecef[0][0] + plos->y * transc2ecef[0][1] + 
	    plos->z * transc2ecef[0][2];

newlos->y = plos->x * transc2ecef[1][0] + plos->y * transc2ecef[1][1] + 
	    plos->z * transc2ecef[1][2];

newlos->z = plos->x * transc2ecef[2][0] + plos->y * transc2ecef[2][1] + 
	    plos->z * transc2ecef[2][2];

return(SUCCESS);
}
