/******************************************************************************
NAME:           hxx_calc_position

PURPOSE:
Finds the satellite position, velocity, and time using the forward model.

RETURN VALUE:
Type = int
Value    	Description
-----    	-----------
ERROR		Error finding attitude correction
SUCCESS		Successfully found position

NOTES:

ALGORITHM REFERENCES:
******************************************************************************
                        Property of the U.S. Government
                            USGS EROS Data Center
******************************************************************************/
#include "hxx_hypmodel.h"
#include "hxx_init_model.h"
#include "xxx_LogStatus.h"

int hxx_calc_position(
	double line,		/*I: Line of interest   (target)	*/
	double sample,		/*I: Sample of interest (target)	*/
	HBandNumber band,	/*I: Band the line/sample are in.	*/
	const CPF *cpf,		/*I: Cal parm file constants		*/
	HYPMODEL *hmodel,	/*I: Hyperion model structure information	*/
	VECTOR *satpos,		/*O: satellite position (meters) in Earth
				     Centered Inertial (ECI) coordinates*/
	VECTOR *satvel,		/*O: sat velocity (meters/millisec) in ECI*/
	double *Time,		/*O: time of the target image point	*/
	int  *year,		/*O: year of image point		*/
	int  *day)		/*O: day of image point			*/
{
double	delta_time,		/* time from ephemeris/ads to first scan*/
	ref_delta,		/* reference delta time from start of 
				   image to the ephemeris/ads time of 
				   interest				*/
	img_time[3];		/* Image point year, doy, sod		*/

/*
 * Initalize the time.  Time will be referenced from first line/sample in
 * the image to the target line/sample.
 */
*Time = 0.0;

/*
 * Find the time from start of image
 */
hxx_findtime(line, sample, band, hmodel, cpf, Time);

/*
 * Calculate the delta time (seconds) from the start of the ephemeris
 * information to the scan line of interest
 */
delta_time = exx_gpstime( hmodel->dce_time ) - exx_gpstime( hmodel->ephem_time );
ref_delta  = *Time + delta_time;

/*
 * Move the satellite to the ephemeris time
 */
exx_movesat(hmodel->nephem, &hmodel->ecr_satpos[0][0], &hmodel->ecr_satpos[1][0], 
            &hmodel->ecr_satpos[2][0], &hmodel->ecr_satvel[0][0], &hmodel->ecr_satvel[1][0],
            &hmodel->ecr_satvel[2][0],  ref_delta, satpos, satvel);

/*
 * Load the time values
 */
ref_delta = *Time + exx_gpstime( hmodel->dce_time );
exx_gpstoyds( ref_delta, img_time );
*year = (int)(img_time[0]+0.5);
*day = (int)(img_time[1]+0.5);
*Time = img_time[2];

return(SUCCESS);
}
