/****************************************************************************
NAME: hxx_convert_ephem

PURPOSE:
Convert the ephemeris points information (position and velocity) from
Earth Centered Inertial (ECI:true_of_date) system to the Earth
Centered Earth Fixed (EF) system, and output the coefficients of
polynomial fit to the position (x, y, z) and velocity (vx, vy, vz) in
the EF system. The velocity is simply the "inertial" velocity vector
rotated to the EF system, there is no change in magnitude of the
vector. The time variable of the polynomial is referenced to the
Reference Ephemeris Time. Polynomial fit to ephemeris in ECI system
is also output, for the later use of correcting gyro measurements.


DEVELOPMENT HISTORY:


RETURN VALUE:
Type = int
Value    Description
-----    -----------
SUCCESS  Successful completion 
ERROR    Operation failed

NOTES:

ALGORITHM REFERENCES:

*****************************************************************************
                        Property of the U.S. Government
                            USGS EROS Data Center
*****************************************************************************/
#include <stdio.h>
#include <math.h>
#include <malloc.h>
#include "xxx_Types.h"
#include "xxx_LogStatus.h"
#include "hxx_hypmodel.h"
#include "hxx_init_model.h"
#include "exx_geometric.h"
#include "xxx_const.h"
#include "xxx_math.h"

int hxx_convert_ephem(
    struct EPHEM ephem[],/* IO:  Ephemeris information			*/
    double  xp,		/* I: True pole postion in the mean pole coords system,
		           x-axis                            			*/
    double yp, 		/* I: True pole position in the mean pole coords
                           system, y-axis                            		*/
    double ut1_utc, 	/* I: Earth rotation angle difference ut1 - utc 	*/
    HYPMODEL *hmodel)   /* IO: Hperion model					*/

{
VECTOR fe_satpos[NPCD];  /* Fixed earth satellite position coordinates */
VECTOR fe_satvel[NPCD];  /* Fixed earth satellite velocity coordinates */
VECTOR ic_satpos[NPCD];  /* Fixed earth satellite position coordinates */
VECTOR ic_satvel[NPCD];  /* Fixed earth satellite velocity coordinates */
VECTOR Wx;             /* Unit vector in along-track direction       */ 
VECTOR Wy;             /* Unit vector in cross-track direction       */
VECTOR Wz;             /* Unit vector in negative radial direction   */
double dx_ob;                  /* Orbit position correction for x direction  */
double dy_ob;                  /* Orbit position correction for y direction  */
double dz_ob;                  /* Orbit position correction for z direction  */
double dx;                   /* Orbit position correction in Earth fixed sys */
double dy;                   /* Orbit position correction in Earth fixed sys */
double dz;                   /* Orbit position correction in Earth fixed sys */
double dvx;                  /* Orbit velocity correction in Earth fixed sys */
double dvy;                  /* Orbit velocity correction in Earth fixed sys */
double dvz;                  /* Orbit velocity correction in Earth fixed sys */
double dvx_ob;                 /* Orbit velocity correction for x direction  */
double dvy_ob;                 /* Orbit velocity correction for y direction  */
double dvz_ob;                 /* Orbit velocity correction for z direction  */
double normval;
double ephem_time;	       /* ephemeris time */
VECTOR *craft_pos;
VECTOR *craft_vel;

int e;			 /* Looping variable				     */
int status;		 /* status from function return	*/

/*
  Check to determine if enough ephemeris information is known
*/
if (hmodel->nephem < LAGRANGE_PTS)
{
    xxx_LogStatus("HXXLIB",__FILE__, __LINE__,
	"Not enough ephemeris points to process");
    return(ERROR);
}

/*
  Moved rescaling to kilometers per second after position update
  and used new variable to fix cyclical rescaling of velocity
  corrections (was dividing by 1000 each time through the loop).
*/
dvx_ob = hmodel->p_vx/1000.0;
dvy_ob = hmodel->p_vy/1000.0;
dvz_ob = hmodel->p_vz/1000.0;

/*
  Set the ephemeris start time based on the first point
*/
hmodel->ephem_time[0] = ephem[0].time[0];
hmodel->ephem_time[1] = ephem[0].time[1];
hmodel->ephem_time[2] = ephem[0].time[2];

/*
  Process the ephemeris information
*/
for (e = 0; e < hmodel->nephem; e++)
{
    /*
      Convert the satellite position and velocity vectors in ECI (true_of_date)
      cartesian coordinates to ECEF (Earth-fixed) coordinates (if ACSPTS) or
      from ECEF to ECI (if GPSPTS)
    */
    craft_pos = &ephem[e].craft_pos;
    craft_vel = &ephem[e].craft_vel;
    /*
      Examination of live data shows that the GPS data are also ECI.
      This may just be because the GPS isn't on and the housekeeping
      data is copied from the ACS fields. That's why I'm keeping the
      ECR to ECI logic in here even though the "else" will never be
      invoked.
    */
    if ( hmodel->ephem_type == ACSPTS ||
         hmodel->ephem_type == GPSPTS )
    {
	ic_satpos[e].x = craft_pos->x;
	ic_satpos[e].y = craft_pos->y;
	ic_satpos[e].z = craft_pos->z;
        ic_satvel[e].x = craft_vel->x;
        ic_satvel[e].y = craft_vel->y;
        ic_satvel[e].z = craft_vel->z;
        status = exx_ic2efc(xp, yp, ut1_utc, craft_pos, craft_vel,
                            ephem[e].time, &fe_satpos[e], &fe_satvel[e]);
    }
    else
    {
	fe_satpos[e].x = craft_pos->x;
	fe_satpos[e].y = craft_pos->y;
	fe_satpos[e].z = craft_pos->z;
        fe_satvel[e].x = craft_vel->x;
        fe_satvel[e].y = craft_vel->y;
        fe_satvel[e].z = craft_vel->z;
        status = exx_efc2ic(xp, yp, ut1_utc, craft_pos, craft_vel,
                            ephem[e].time, &ic_satpos[e], &ic_satvel[e]);
    }
    if (status != SUCCESS) 
    {
	xxx_LogStatus("GPSLIB", __FILE__, __LINE__,
	    "Error returned from gxx_ic2efc/gxx_efc2ic");
	return(ERROR);
    }

    /*
      Calculate the orbit position correction in Orbit System.
      Note:  hmodel->p_ref_time is relative to the DCE start time
    */
    ephem_time = exx_gpstime( ephem[e].time ) - exx_gpstime( hmodel->dce_time );

    dx_ob = hmodel->p_x + hmodel->p_vx * (ephem_time - hmodel->p_ref_time);
    dy_ob = hmodel->p_y + hmodel->p_vy * (ephem_time - hmodel->p_ref_time);
    dz_ob = hmodel->p_z + hmodel->p_vz * (ephem_time - hmodel->p_ref_time);

    /*
      Apply the precision correction to the ECR coordinates
    */
    /*
      Calculate the unit vector for Wz in the negative radial dir.
      Wz = -X / X where X = {x, y, z}
    */
    normval = xxx_norm(&fe_satpos[e]);
    if (normval == 0.0)
    {
	xxx_LogStatus("HXXLIB", __FILE__, __LINE__, "Cannot divide by zero");
	return(ERROR);
    }
    Wz.x = - fe_satpos[e].x / normval;
    Wz.y = - fe_satpos[e].y / normval;
    Wz.z = - fe_satpos[e].z / normval;

    /*
      Calculate the unit vector for Wy in the cross-track direction.
      Wy = Wz cross V / |Wz cross V| where V = {vx, vy, vz}
    */

    xxx_cross (&Wz, &fe_satvel[e], &Wy);
    normval = xxx_norm(&Wy);
    if (normval == 0.0)
    {
	xxx_LogStatus("HXXLIB", __FILE__, __LINE__, "Cannot divide by zero");
	return(ERROR);
    }
    Wy.x = Wy.x / normval;
    Wy.y = Wy.y / normval;
    Wy.z = Wy.z / normval;

    /* 
      Calculate the unit vector for Wx in the along-track direction.
      Wx = Wy cross Wz
    */
    xxx_cross (&Wy, &Wz, &Wx);

    /*
      Calculate the orbit position correction in the Earth Fixed system.
    */
    dx = dx_ob * Wx.x + dy_ob * Wy.x + dz_ob * Wz.x;
    dy = dx_ob * Wx.y + dy_ob * Wy.y + dz_ob * Wz.y;
    dz = dx_ob * Wx.z + dy_ob * Wy.z + dz_ob * Wz.z;

    /*
      Calculate the orbit velocity correction in the Earth Fixed system.
    */
    dvx = dvx_ob * Wx.x + dvy_ob * Wy.x + dvz_ob * Wz.x;
    dvy = dvx_ob * Wx.y + dvy_ob * Wy.y + dvz_ob * Wz.y;
    dvz = dvx_ob * Wx.z + dvy_ob * Wy.z + dvz_ob * Wz.z;

    /*
      Load the ephemeris points in EF
    */
    hmodel->ecr_satpos[0][e] = fe_satpos[e].x + dx;     /* Coords x in EF		*/
    hmodel->ecr_satpos[1][e] = fe_satpos[e].y + dy;     /* Coords y in EF		*/
    hmodel->ecr_satpos[2][e] = fe_satpos[e].z + dz;     /* Coords z in EF		*/
    hmodel->ecr_satvel[0][e] = fe_satvel[e].x + dvx;	/* Velocity x in EF		*/
    hmodel->ecr_satvel[1][e] = fe_satvel[e].y + dvy;	/* Velocity y in EF		*/
    hmodel->ecr_satvel[2][e] = fe_satvel[e].z + dvz;	/* Velocity z in EF		*/
    
    /*
      Apply the precision correction to the ECI coordinates
    */
    /*
      Calculate the unit vector for Wz in the negative radial dir.
      Wz = -X / X where X = {x, y, z}
    */
    normval = xxx_norm(&ic_satpos[e]);
    if (normval == 0.0)
    {
	xxx_LogStatus("HXXLIB", __FILE__, __LINE__, "Cannot divide by zero");
	return(ERROR);
    }
    Wz.x = - ic_satpos[e].x / normval;
    Wz.y = - ic_satpos[e].y / normval;
    Wz.z = - ic_satpos[e].z / normval;

    /*
      Calculate the unit vector for Wy in the cross-track direction.
      Wy = Wz cross V / |Wz cross V| where V = {vx, vy, vz}
    */

    xxx_cross (&Wz, &ic_satvel[e], &Wy);
    normval = xxx_norm(&Wy);
    if (normval == 0.0)
    {
	xxx_LogStatus("HXXLIB", __FILE__, __LINE__, "Cannot divide by zero");
	return(ERROR);
    }
    Wy.x = Wy.x / normval;
    Wy.y = Wy.y / normval;
    Wy.z = Wy.z / normval;

    /* 
      Calculate the unit vector for Wx in the along-track direction.
      Wx = Wy cross Wz
    */
    xxx_cross (&Wy, &Wz, &Wx);

    /*
      Calculate the orbit position correction in the ECI system.
    */
    dx = dx_ob * Wx.x + dy_ob * Wy.x + dz_ob * Wz.x;
    dy = dx_ob * Wx.y + dy_ob * Wy.y + dz_ob * Wz.y;
    dz = dx_ob * Wx.z + dy_ob * Wy.z + dz_ob * Wz.z;

    /*
      Calculate the orbit velocity correction in the ECI system.
    */
    dvx = dvx_ob * Wx.x + dvy_ob * Wy.x + dvz_ob * Wz.x;
    dvy = dvx_ob * Wx.y + dvy_ob * Wy.y + dvz_ob * Wz.y;
    dvz = dvx_ob * Wx.z + dvy_ob * Wy.z + dvz_ob * Wz.z;

    /*
      Load the ephemeris points in ECI
    */
    hmodel->eci_satpos[0][e] = ic_satpos[e].x + dx;     /* Coords x in ECI	*/
    hmodel->eci_satpos[1][e] = ic_satpos[e].y + dy;     /* Coords y in ECI	*/
    hmodel->eci_satpos[2][e] = ic_satpos[e].z + dz;     /* Coords z in ECI	*/
    hmodel->eci_satvel[0][e] = ic_satvel[e].x + dvx;	/* Velocity x in ECI	*/
    hmodel->eci_satvel[1][e] = ic_satvel[e].y + dvy;	/* Velocity y in ECI	*/
    hmodel->eci_satvel[2][e] = ic_satvel[e].z + dvz;	/* Velocity z in ECI	*/
}

return(SUCCESS);
}
