/******************************************************************************
NAME:           hxx_findatt

PURPOSE:
Find the precise roll, pitch and yaw at this time 

DEVELOPMENT HISTORY:
$Log: hxx_findatt.c,v $
Revision 1.1  2001/07/09 12:06:46  choate
Initial check in



RETURN VALUE:
None.

NOTES:
  This routine takes the initial spacecraft roll, pitch, and yaw, and adjusts
  them with 2 things: (1) the interpolated adjustment for this time and 
		      (2) the precision attitude adjustment.

ALGORITHM REFERENCES:
******************************************************************************
                        Property of the U.S. Government
                            USGS EROS Data Center
******************************************************************************/
#include <stdio.h>
#include <math.h>
#include "xxx_Types.h"
#include "xxx_LogStatus.h"
#include "hxx_hypmodel.h"
#include "hxx_init_model.h"

int hxx_findatt(
	HYPMODEL *hmodel,	/*I: ALI model structure 		*/
	double time,		/*I: Actual time (sec from dce_time) 	*/
	double *roll,		/*O: Output roll(rads)			*/
	double *pitch,		/*O: Output pitch(rads)			*/
	double *yaw)		/*O: Output yaw(rads)			*/
{
double	interp,			/* interpolation value for RPY	*/
	dtime,			/* seconds from acs_time 	*/
	sampling_rate,		/* rpy sampling rate		*/
	t_roll,			/* interpolated roll value	*/
	t_pitch,		/* interpolated pitch value	*/
	t_yaw;			/* interpolated yaw value	*/

double	cr,			/* cosine of roll		*/
	sr,			/* sine of roll			*/
	cp,			/* cosine of pitch		*/
	sp,			/* sine of pitch		*/
	cy,			/* cosine of yaw		*/
	sy,			/* sine of yaw			*/
	a[3][3],		/* precision rotation matrix	*/
	b[3][3],		/* relative rotation matrix	*/
	c[3][3];		/* net rotation matrix		*/

int 	index1,			/* index one into the RPY array	*/
	index2;			/* index two into the RPY array	*/

int	count;			/* error message length		*/
char	msg[STRLEN];		/* error message		*/

double dce_time;
double acs_time;

/*
  Find the time relative to attitude data start
*/
dtime = time + exx_gpstime(hmodel->dce_time) - exx_gpstime(hmodel->acs_time);
sampling_rate = IRU_TIME;

/*
  Find the values of roll, pitch, and yaw before and after input time
*/
index1 = (int)floor(dtime / (sampling_rate));
index2 = index1 + 1;

if ((index1 < 0) || (index2 > (hmodel->nrpy-1)))
{
    count = sprintf(msg,"RPY correction not found for time offset %f",dtime);
    if (count < 1)
    {
	/* if the sprintf had an error */	
    	xxx_LogStatus("HXXLIB", __FILE__, __LINE__,"Error in sprintf");
	return(ERROR);
    }
    xxx_LogStatus("AXXLIB", __FILE__, __LINE__,msg);
    t_roll = 0.0;
    t_pitch = 0.0;
    t_yaw = 0.0;
}
else
{
    /*
      Linear interpolate between the points
    */
    interp  = fmod(dtime, sampling_rate)/sampling_rate;
    t_roll  = hmodel->rpy[index1][0]
            + (hmodel->rpy[index2][0] - hmodel->rpy[index1][0]) * interp;
    t_pitch = hmodel->rpy[index1][1]
            + (hmodel->rpy[index2][1] - hmodel->rpy[index1][1]) * interp;
    t_yaw   = hmodel->rpy[index1][2]
            + (hmodel->rpy[index2][2] - hmodel->rpy[index1][2]) * interp;
}

/*
  Add the values at this time to the initial spacecraft values
*/
*roll  = hmodel->sc_roll  + t_roll;
*pitch = hmodel->sc_pitch + t_pitch;
*yaw   = hmodel->sc_yaw   + t_yaw;

/*
  Also adjust the roll, pitch, and yaw values for precision correction
  Note:  hmodel->p_ref_time is relative to DCE start time
*/
dtime = time - hmodel->p_ref_time;
t_roll  = hmodel->p_roll  + hmodel->p_rrate * dtime;
t_pitch = hmodel->p_pitch + hmodel->p_prate * dtime;
t_yaw   = hmodel->p_yaw   + hmodel->p_yrate * dtime;

/* Build the precision rotation matrix */
cr = cos(t_roll);
sr = sin(t_roll);
cp = cos(t_pitch);
sp = sin(t_pitch);
cy = cos(t_yaw);
sy = sin(t_yaw);
a[0][0] = cp*cy;    a[0][1] = cy*sr*sp + cr*sy;     a[0][2] = sr*sy - sp*cr*cy;
a[1][0] = -sy*cp;   a[1][1] = cy*cr - sy*sr*sp;     a[1][2] = sp*sy*cr + cy*sr;
a[2][0] = sp;       a[2][1] = -sr*cp;               a[2][2] = cp*cr;

/* Build the original rotation matrix */
cr = cos(*roll);
sr = sin(*roll);
cp = cos(*pitch);
sp = sin(*pitch);
cy = cos(*yaw);
sy = sin(*yaw);
b[0][0] = cp*cy;    b[0][1] = cy*sr*sp + cr*sy;     b[0][2] = sr*sy - sp*cr*cy;
b[1][0] = -sy*cp;   b[1][1] = cy*cr - sy*sr*sp;     b[1][2] = sp*sy*cr + cy*sr;
b[2][0] = sp;       b[2][1] = -sr*cp;               b[2][2] = cp*cr;

/* Multiply to get net rotation matrix */
c[0][0] = a[0][0]*b[0][0] + a[0][1]*b[1][0] + a[0][2]*b[2][0];
c[0][1] = a[0][0]*b[0][1] + a[0][1]*b[1][1] + a[0][2]*b[2][1];
c[0][2] = a[0][0]*b[0][2] + a[0][1]*b[1][2] + a[0][2]*b[2][2];
c[1][0] = a[1][0]*b[0][0] + a[1][1]*b[1][0] + a[1][2]*b[2][0];
c[1][1] = a[1][0]*b[0][1] + a[1][1]*b[1][1] + a[1][2]*b[2][1];
c[1][2] = a[1][0]*b[0][2] + a[1][1]*b[1][2] + a[1][2]*b[2][2];
c[2][0] = a[2][0]*b[0][0] + a[2][1]*b[1][0] + a[2][2]*b[2][0];
c[2][1] = a[2][0]*b[0][1] + a[2][1]*b[1][1] + a[2][2]*b[2][1];
c[2][2] = a[2][0]*b[0][2] + a[2][1]*b[1][2] + a[2][2]*b[2][2];

/* Back out net roll, pitch, yaw */
*roll  = atan(-c[2][1]/c[2][2]);
*pitch = asin(c[2][0]);
*yaw   = atan(-c[1][0]/c[0][0]);

return(SUCCESS);
}
