/******************************************************************************
NAME:           hxx_findtarpos

PURPOSE:
Find the position where the line of sight vector intersects the Earth's
surface.

RETURN VALUE:
Type = int
Value		Description
-----		-----------
SUCCESS		Calculated LOS and its intersection with earth
ERROR		Error calculating radius of the earth

NOTES:

ALGORITHM REFERENCES:
******************************************************************************
                        Property of the U.S. Government
                           USGS EROS Data Center
******************************************************************************/
#include "hxx_hypmodel.h"
#include "xxx_math.h"
#include "exx_geometric.h"
#include "xxx_LogStatus.h"

#define NUM_ITERATIONS 6	/* number of iterations for converging 
				   LOS intersection 			    */
#define CONV_TOLERANCE 0.1	/* convergence tolerance		    */

int hxx_findtarpos(
	const VECTOR *satvec,	/*I: Satellite location vector		    */
	const VECTOR *losv,	/*I: Line of sight to target vector	    */
	double tarelev,         /*I: Elevation of target above the ellipsoid*/
	const CPF *cpf,		/*I: CPF constants			    */
	double *tarlat,        	/*O: Target latitude			    */
	double *tarlong,        /*O: Target longitude			    */
	double *tarrad)         /*O: Radius of the target		    */
{
double  sat_los_angle,	/* angle between satellite and LOS vectors           */
        sat_lat,       	/* latitude of the satellite                         */
        sat_long,       /* longitude of the satellite                        */
        earth_radius,   /* radius of the earth                               */
        sat_radius,     /* radius of the satellite                           */
        maglos,         /* magnitude of the line of sight vector             */
	old_maglos,	/* temp to save the magnitude			     */
        psi,            /* earth centeral angle                              */
        lat,           	/* temporary variable to hold a latitude             */
	tmp;		/* temporary	 			     	     */

int 	i;		/* looping variable				     */

VECTOR tarvec;		/* target vector			    	     */

/*
 * Find the angle between the satellite location and the line of sight
 */
xxx_anglebetwn(satvec, losv, &sat_los_angle);
sat_los_angle = PIE - sat_los_angle;

/*
 * Convert the satellite cartesian location to spherical coordinates
 */
exx_cart2sph((VECTOR *)satvec, &sat_lat, &sat_long, &sat_radius);
lat = sat_lat;

/*
 * Initialize the old magnitude variable
 */
old_maglos = 0.0;

/*
 *  Loop until convergence is found where the LOS intersects the Earth
 */
for (i = 0; i < NUM_ITERATIONS; i++)
{
   /*
    * Find the radius of the earth for the specified latitude
    */
    if (exx_earthradius(lat, cpf, &earth_radius) == ERROR)
    {
	xxx_LogStatus("HXXLIB", __FILE__, __LINE__, 
	    "Error returned from exx_earthradius"); 
	return(ERROR);
    }
    earth_radius += tarelev;

   /*
    * Calculate the earth central angle
    */
    tmp = sin(sat_los_angle) * sat_radius / earth_radius;
    if (tmp < -1.0)
	tmp = -1.0;
    else if (tmp > 1.0)
	tmp = 1.0;

    psi = asin(tmp) - sat_los_angle;

   /*
    * Calculate the magnitude of the line of sight vector using the
    * law of cosines
    */
    maglos = sqrt(sat_radius * sat_radius + earth_radius * earth_radius -
                  2.0 * sat_radius * earth_radius * cos(psi));

   /*
    * Set the target location
    */
    tarvec.x = satvec->x + maglos * losv->x;
    tarvec.y = satvec->y + maglos * losv->y;
    tarvec.z = satvec->z + maglos * losv->z;

   /*
    * Convert the target cartesian location to spherical coordinates
    */
    exx_cart2sph(&tarvec, tarlat, tarlong, tarrad);
    lat = *tarlat;

   /*
    * When the magnitudes are less than the tolerance, convergence is
    * obtained so break out of loop
    */
    if (fabs(old_maglos - maglos) <= CONV_TOLERANCE)
	break;

    old_maglos = maglos;
}

return(SUCCESS);
}
