/******************************************************************************
NAME:           hxx_forward_hyp

PURPOSE:
For a given line, sample and band, propagate the forward hyperion satellite 
model to determine a latitude and longitude for the specified point.

DEVELOPMENT HISTORY:

RETURN VALUE:
Type = int
Value    	Description
-----    	-----------
ERROR		error propagating image location 
SUCCESS		image location successfully propagated forward

NOTES:
The axx_getlos function does the majority of the forward model processing.

ALGORITHM REFERENCES:
******************************************************************************
                        Property of the U.S. Government
                            USGS EROS Data Center
******************************************************************************/
#include "xxx_LogStatus.h"
#include "hxx_hypmodel.h"
#include "exx_geometric.h"

int hxx_forward_hyp(
	double line,		/*I: Input line	number			     */
	double sample,		/*I: Input sample number		     */
	HBandNumber band,	/*I: Input band number			     */
	double tarelev,		/*I: Target elevation (at line/sample)	     */
	const CPF *cpf,		/*I: Cal parm file constants                 */
	HYPMODEL *hmodel,	/*I/O: Hyperion model information	     */
	double *tarlatd,	/*O: Target latitude			     */
	double *tarlong)	/*O: Target longitude			     */
{
double	tarlatc,		/* geocentric latitude of the target	     */
        tarrc,			/* radius of the target			     */
        ht;			/* height of the target in geodetic coords   */
VECTOR satpos,                  /* satellite position vector                 */
       los;                     /* line of sight in OCI coordinates          */

/*
 * Call getlos which does the main calculations for forward_hyp.
 */
if  (hxx_getlos(line, sample, band, tarelev, cpf, hmodel, &tarlatc, tarlong, &tarrc, 
                &satpos, &los) != SUCCESS)
{ 
    xxx_LogStatus("HXXLIB", __FILE__, __LINE__,
	"Error calculating line of sight intersection with the Earth");
    return(ERROR);
}

/*
 * Calculate the geodetic latitude and height of the target
 */
if (exx_centh2det(tarlatc, tarrc, cpf, tarlatd, &ht) != SUCCESS)
{ 
    xxx_LogStatus("HXXLIB", __FILE__, __LINE__,
	"Error calculating geodetic latitude and height");
    return(ERROR);
}

return(SUCCESS);
}
