/******************************************************************************
NAME:           hxx_getlos

PURPOSE:
Find the position were the line of sight vector intersects the Earth's surface

RETURN VALUE:
Type = int
Value    	Description
-----    	-----------
ERROR		Error finding attitude correction
SUCCESS		Successfully found position

NOTES:

ALGORITHM REFERENCES:
******************************************************************************
                        Property of the U.S. Government
                            USGS EROS Data Center
******************************************************************************/
#include "hxx_hypmodel.h"
#include "hxx_init_model.h"
#include "xxx_LogStatus.h"

int hxx_getlos(
	double line,		/*I: Line of interest   (target)	*/
	double sample,		/*I: Sample of interest (target)	*/
	HBandNumber band,	/*I: Band the line/sample are in.	*/
	double tarelev,		/*I: Elevation of target above the 
				     ellipsoid				*/
	const CPF *cpf,		/*I: Cal parm file constants		*/
	HYPMODEL *hmodel,	/*I: Hyperion model structure information*/
	double *tarlatc,	/*O: Target geocentric latitude (radians)*/
	double *tarlong,	/*O: Target longitude (radians (radians)*/
	double *tarrc,		/*O: Radius of the earth at the target	*/
        VECTOR *satpos,         /*O: satellite position (meters) in Earth
                                     Centered Inertial (ECI) coordinates */
        VECTOR *pertlos)        /*O: perturbed LOS (OCS-orbital coord)  */
{
int 	status;			/* status of return from function	*/

double 	Time,			/* time of the target from image start	*/
	delta_time,		/* time from ephem_time to image start	*/
	ref_delta,		/* reference delta time from start of 
				   ephemeris data to the image time	*/
	roll,			/* roll correction at time of interest	*/
	pitch,			/* pitch correction at time of interest	*/
	yaw;			/* yaw correction at time of interest	*/

VECTOR  losvec,			/* line of sight to target vector	*/
        newlos,                 /* new line of sight (LOS) vector in
                                   perturbed spacecraft coordinates */
	satvel;			/* sat velocity (meters/millisec) in ECI*/

double satlatd;
double satlatc;
double satlong;
double satrad;
double sath;

/*
 * Initalize the time.  Time will be referenced from first line/sample in
 * the image to the target line/sample.
 */
Time = 0.0;

/*
 * Find the current detector and time from start of image
 */
status = hxx_findtime(line, sample, band, hmodel, cpf, &Time);
if (status != SUCCESS)
{
    xxx_LogStatus("HXXLIB", __FILE__, __LINE__, "Error returned from hxx_findtime");
    return(ERROR);
}

/*
 * Find the Hyperion look vector
 */
status = hxx_findlos(band, sample, hmodel, cpf, &losvec);
if (status != SUCCESS)
{
    xxx_LogStatus("HXXLIB", __FILE__, __LINE__, 
        "Error returned from hxx_findlos");
    return(ERROR);
}

/*
 * Find the roll, pitch, and yaw correction for the above time
 */
status = hxx_findatt(hmodel, Time, &roll, &pitch, &yaw);
if (status != SUCCESS)
{
    xxx_LogStatus("HXXLIB",__FILE__, __LINE__, 
        "Error returned from hxx_findatt");
    return(ERROR);
}

/*
 * Calculate the delta time (seconds) from the start of the ephemeris
 * information to the scan line of interest
 */
delta_time = exx_gpstime( hmodel->dce_time ) - exx_gpstime( hmodel->ephem_time );
ref_delta  = Time + delta_time;

/*
 * Move the satellite to the ephemeris time
 */
exx_movesat(hmodel->nephem, &hmodel->ecr_satpos[0][0], &hmodel->ecr_satpos[1][0], 
            &hmodel->ecr_satpos[2][0], &hmodel->ecr_satvel[0][0], &hmodel->ecr_satvel[1][0], 
            &hmodel->ecr_satvel[2][0], ref_delta, satpos, &satvel);

/*
 * Find the new line of sight vector from the perturbed spacecraft coordinates
 */
status = hxx_attitude(satpos, &satvel, &losvec, roll, pitch, yaw, cpf, pertlos, &newlos);
if (status != SUCCESS)
{
    xxx_LogStatus("HXXLIB",__FILE__, __LINE__, 
        "Error returned from hxx_attitude");
    return(ERROR);
}

/*
 * Find the position were the line of sight vector intersects the Earth's
 * surface
 */
status = hxx_findtarpos(satpos, &newlos, tarelev, cpf, tarlatc, tarlong,tarrc);
if (status != SUCCESS)
{
    xxx_LogStatus("HXXLIB",__FILE__, __LINE__, 
        "Error returned from hxx_findtarpos");
    return(ERROR);
}

return(SUCCESS);
}
