/****************************************************************************
FUNCTION:                      hxx_grid_cell_poly 
 
PURPOSE:  Calculate a "rough" mapping of output to input lines/samples.
 
RETURN VALUE:
Type = int
Value    Description
-----    -----------
SUCCESS  Successful completion
ERROR    Operation failed


NOTES:
	The coefficients returned from grid_cell_poly are
    used as a rough estimate of an inverse model.  This poynomial is
    used to initially locate the correct grid cell to be used in the
*******************************************************************************/
#include <math.h>
#include "xxx_Types.h"
#include "xxx_LogStatus.h"
#include "xxx_math.h"
#include "xxx_structures.h"
#include "hxx_grid.h"

int hxx_grid_cell_poly
( 
   hxx_GRID_BAND_TYPE *grid_band_ptr,   /* I/O: Grid associated with search image*/
   int nzplanes
)
{
   double ii,mm;        /* Counters                                     */
   double line,samp;    /* Line/sample                                  */
   int  *ilines;        /* Input lines                                  */
   int  *isamps;        /* Input samples                                */
   double *olines;      /* Output lines                                 */
   double *osamps;      /* Output samples                               */
   double *b;           /* Array for least squares                      */
   double *v;           /* Array for least squares                      */
   double *A;           /* Array for least squares                      */
   double x,y;          /* Line/sample variables                        */
   int  i,n,m;          /* Counters                                     */
   int  numpts;         /* Number of points used in least squares       */
   int  cnt;            /* Counter for least squares                    */
   int  ncoeff;         /* Number of coefficients used in least squares */
   int  ncols;          /* Number of columns                            */
   int  nrows;          /* Number of rows                               */
   int  pdegree;        /* Degree of the rough transformatioanal coeffs.*/
   int  qflag=0;	/* flag for c_qrdecomp do not order A by columns*/
   long zoffset;
   int zplane;

   pdegree = grid_band_ptr->degree;
   ncoeff = (pdegree+1)*(pdegree+1);
   nrows = grid_band_ptr->ngrid_lines;
   ncols = grid_band_ptr->ngrid_samps;
   numpts = ncols * nrows;
 
/* NOTE!! This will need to change if original grid cell is not passed in */

   for(zplane=0 ; zplane<nzplanes; zplane++ )
   {
      zoffset = zplane * numpts;

      osamps = &grid_band_ptr->out_samps[zoffset];
      olines = &grid_band_ptr->out_lines[zoffset];
      ilines = grid_band_ptr->in_lines;
      isamps = grid_band_ptr->in_samps;
 
      /*---------------------------------------------------------
        Allocate buffer space for least squares routine.
       ---------------------------------------------------------*/
      A = (double *)malloc( sizeof(double) * numpts * ncoeff );
      if( A==NULL )
      {
         xxx_LogStatus ("AXXLIB", __FILE__, __LINE__, "Error allocating memory");
         return (ERROR);
      }
      b = (double *)malloc( sizeof(double) * numpts );
      if( b==NULL )
      {
         xxx_LogStatus ("AXXLIB", __FILE__, __LINE__, "Error allocating memory");
         free(A);
         return (ERROR);
      }
      v = (double *)malloc( sizeof(double) * numpts );
      if( v==NULL )
      {
         xxx_LogStatus ("AXXLIB", __FILE__, __LINE__, "Error allocating memory");
         free(A);
         free(b);
         return (ERROR);
      }
    
      /*--------------------------------------------------------------------
        Fill least squares routine according to degree of polynomial.  This
        mapping is from output image lines/samples (all lines/samples in
        grid output space) to input image lines/samples (all lines/samples in
        grid input space).
       --------------------------------------------------------------------*/
      cnt = 0;
      for( i=0 ; i<=pdegree ; i++ )
      {
         ii = i;
         for( m=0 ; m<=pdegree ; m++ )
         {
            for( n=0 ; n<numpts ; n++ )
            {
               mm = m;
               samp = osamps[n];
               line = olines[n];
               x = pow(samp,mm);
               y = pow(line,ii);
               A[cnt] = x*y;
               cnt++;
            }
         }
      }
    
      /*--------------------------------------------------------------------
        Perform least squares routine.  First the coefficients for output
        lines to input lines are found and stored.  Next the
        coefficients for output samples to input samples are found and
        stored.
       --------------------------------------------------------------------*/
   
      xxx_QR( A, numpts, ncoeff, v, qflag );
    
      for( i=0, m=0 ; m<nrows ; m++ )
         for( n=0 ; n<ncols ; n++, i++ )
            b[i] = ilines[m];
      xxx_QRsolve( A, numpts, ncoeff, v, b, 0 );
      for( n=0 ; n<ncoeff ; n++ )
         grid_band_ptr->poly_lines[n+zplane*ncoeff] = b[n];
    
      for( i=0, m=0 ; m<nrows ; m++ )
         for( n=0 ; n<ncols ; n++, i++ )
            b[i] = isamps[n];
      xxx_QRsolve( A, numpts, ncoeff, v, b, 0 );
      for( n=0 ; n<ncoeff ; n++ )
         grid_band_ptr->poly_samps[n+zplane*ncoeff] = b[n];
   }
 
   free( A );
   free( b );
   free( v );
   return (SUCCESS);
}
