/****************************************************************************
NAME: hxx_init_iru

PURPOSE:
Remove the effects due to orbital motion

RETURN VALUE:
Type = int
Value    Description
-----    -----------
SUCCESS  Successful completion 
ERROR    Operation failed

NOTES:
This routine assumes that the ephemeris arrays in the ALI model structure
have already been populated.

ALGORITHM REFERENCES:
******************************************************************************
                        Property of the U.S. Government
                            USGS EROS Data Center
*****************************************************************************/
#include <math.h>
#include "xxx_Types.h"
#include "xxx_LogStatus.h"
#include "xxx_math.h"
#include "exx_geometric.h"
#include "hxx_hypmodel.h"
#include "hxx_init_model.h"

int hxx_init_iru(
    double *acs_time,
    double *ephem_time,
    int nephem,
    double *satpos_x,
    double *satpos_y,
    double *satpos_z,
    double *satvel_x,
    double *satvel_y,
    double *satvel_z,
    int niru,		/* I: Number of IRU points 			 */
    double *irutime,	/* I: IRU data point times (from acs_time)	 */
    VECTOR *iru		/* I/O: IRU data				 */
    )
{
double	eci2ob0[3][3],	/* Transf. matrix from ECI to OB at gyro ref. t */
	obt2eci[3][3],	/* Transf. matrix from OB to ECI at gyro_time   */
	eci2obt[3][3],	/* Transf. matrix from ECI to OB at gyro_time   */
	obt2ob0[3][3],	/* Transf. matrix from OB time to OB ref. time  */
	droll,		/* delta Roll due to sat. motion, in radian     */
	dpitch,		/* delta Pitch due to sat. motion, in radian    */
	dyaw;		/* delta Yaw due to sat. motion, in radian      */
	
VECTOR  satpos,		/* Satellite position at interpolated time	*/
	satvel;		/* Satellite velocity at interpolated time	*/

int 	i, j, k, l, nn;	/* Looping variables				*/
int 	status;		/* status returned from function		*/
double	etime;		/* Time from ephemeris start 			*/

double temp;

/*
**  Calculate the satellite position and velocity at the first IRU data
**  point, and compose the transformation matrix from ECI system to the
**  Orbit system at this point
*/
etime = irutime[0] + exx_gpstime( acs_time )
                   - exx_gpstime( ephem_time );
exx_movesat( nephem, satpos_x, satpos_y, satpos_z, satvel_x,
             satvel_y, satvel_z, etime, &satpos, &satvel );

status = exx_transmat(&satpos, &satvel, eci2ob0);
if (status != SUCCESS)
{
    xxx_LogStatus("EXXLIB", __FILE__, __LINE__,
	"Error returned from exx_transmat");
    return(ERROR);
}
/*
**  Process all the IRU data points
*/

droll = 0.0;
dpitch = 0.0;
dyaw = 0.0;

for (i = 1; i < niru; i++)
{

   /*
    *  Calculate the time from the first IRU data point (gyro reference
    *  time) to this gyro data point (in seconds) and time from ephemeris 
    *  reference time to this gyro data point (in seconds).
    */
    etime = irutime[i] + exx_gpstime( acs_time )
                       - exx_gpstime( ephem_time );

    exx_movesat(nephem, satpos_x, satpos_y, satpos_z, satvel_x, satvel_y, satvel_z, 
                etime, &satpos, &satvel );

    /*
     *  Compose the transfromation matrix from the Orbit system at the time 
     *  to ECI, for the calculation of transformation from OB this time to
     *  OB at gyro reference time.
     */
    status = exx_transmat(&satpos, &satvel, eci2obt); /* from ECI to OB at time*/
    if (status != SUCCESS)
    {
        xxx_LogStatus("EXXLIB", __FILE__, __LINE__,
	        "Error returned from exx_transmat");
	return(ERROR);
    }
    
    status = xxx_invmatrix(eci2obt,obt2eci);
    if (status != SUCCESS)
    {
        xxx_LogStatus("EXXLIB", __FILE__, __LINE__, 
            "Error inverting matrix: eci2obt");
        return(ERROR);
    }
    
    
    /*
     *  Calculate the transformation from OB at the gyro_time to the OB     
     *  at the gyro reference time, for the calculation of attitude change.
     */
    for(j=0; j<3; j++)
    {
        for(k=0; k<3; k++)
	{
            obt2ob0[j][k] = 0.0;
            for(l=0; l<3; l++)
	        obt2ob0[j][k] += (eci2ob0[j][l] * obt2eci[l][k]);
        }
     }

     /*
      *  Copy the current eci2obt array into eci2ob0 for use with
      *  the next point.
      */
    for(j=0; j<3; j++)
      for(k=0; k<3; k++)
         eci2ob0[j][k] = eci2obt[j][k];
    
     /*  
      *  Calclulate the attitude change by using the unit axes of OB system 
      *  at time in the OB system at gyro reference time.
      */
     droll = -atan2(obt2ob0[2][1], obt2ob0[2][2])/(irutime[i]-irutime[i-1]);
     dpitch = asin(obt2ob0[2][0])/(irutime[i]-irutime[i-1]);
     dyaw = -atan2(obt2ob0[1][0], obt2ob0[0][0])/(irutime[i]-irutime[i-1]);
    
     /*
      *  Remove the attitude change due to the satellite motion during the
      *  time from the previous IRU time to the current IRU time.
      *
      *  Also swap the sign on the IRU data to be body to orbit.
      */
     iru[i].x = -iru[i].x - droll;
     iru[i].y = -iru[i].y - dpitch;
     iru[i].z = -iru[i].z - dyaw; 

     /*
      *  If this is point 1, go back and correct the initial point
      */
     if ( i == 1 )
     {
         iru[0].x = -iru[0].x - droll;
         iru[0].y = -iru[0].y - dpitch;
         iru[0].z = -iru[0].z - dyaw; 

#ifdef DEBUG
         printf("Corrected IRU %03d: %lf %lf %lf %lf\n",
                 i-1, irutime[0], iru[0].x, iru[0].y, iru[0].z );
#endif
     }

#ifdef DEBUG
     printf("Corrected IRU %03d: %lf %lf %lf %lf\n",
             i, irutime[i], iru[i].x, iru[i].y, iru[i].z );
#endif

}/* end for i */

return(SUCCESS);
}
