/* ********************************************************************
NAME: hxx_out_of_pcd

PURPOSE:
fill the model structure with the data from the Hyperion PCD

RETURN VALUE:
Type = int
Value    Description
-----    -----------
SUCCESS  Successful completion 
ERROR    Operation failed

NOTES:

ALGORITHM REFERENCES:
******************************************************************************
		        Property of the U.S. Government
                             USGS EROS Data Center
******************************************************************************/
#include <stdio.h>
#include <string.h>
#include "xxx_Types.h"  	/* defines the ERROR variables */
#include "xxx_HYPPCD.h"		/* defines the PCD structure */
#include "xxx_LogStatus.h"	/* prototype for xxx_LogStatus.c */
#include "xxx_const.h"  	/* defines several constants used */
#include "hxx_hypmodel.h"	/* model structure definition */
#include "hxx_init_model.h"
#include "xxx_math.h"	/* prototype definitions */

#define		EPS		2.0

int hxx_out_of_pcd(
    int nlines,
    HYPPCD *pcd,		/* I: Payload Correction Data from the satellite */
    HYPMODEL *hmodel)		/* IO: The Hyperion model information */
{
int 	ii, jj, kk, qq;     	/* Loop variables                            */
double  dtime;			/* time offset				     */
double  gpstime;		/* GPS reference time                        */
double  timecode;		/* Time code corresponding to gpstime        */
double  timecode_rate;		/* Spacecraft clock rate (seconds per second)*/
char    Message[STRLEN];	/* Output message text		    	     */
int step_buffer=25;		/* Amount to step before/after imagery time  */
double curr_start;
int curr_index;
double swir_time_code;		/* SWIR Time code in GPS		     */
double vnir_time_code;		/* VNIR Time code in GPS		     */

/****************************
  Setting the satellite number
  **************************** */
hmodel->satnum = 1;

/* Search for image start time among DCEs.  

   There could be some slight timing issues associated with the IMAGE_MODE HDF 
   mneumonic (these are currently helping to define image collects) and the start 
   of the imagery.  A small buffer is used before and after DCE time.  This could 
   produce some problems later on in life.

   The COVERPOS mnuemonic starts and stops well after imaging. There is no need to
   do any buffering before or after for DCEs that were determined by the COVERPOS. */ 
if( pcd->HYP_Group->yimage_mode.count != 0 )
   step_buffer = 25;
else
   step_buffer = 0;

pcd->active_DCE = -1;
gpstime = exx_gpstime( hmodel->dce_time );
for (ii=0; ii<pcd->num_DCE; ii++)
{
  if ( gpstime > (pcd->DCE_List[ii].start_time-EPS) && gpstime < (pcd->DCE_List[ii].stop_time+EPS) )
  {
    pcd->active_DCE = ii;
  }
}

if( pcd->active_DCE == -1 )
{
   sprintf(Message,"Could not find DCE");
   xxx_LogStatus("HXXLIB", __FILE__, __LINE__,Message);
   return(ERROR);
}

sprintf(Message,"Selected DCE %d",pcd->active_DCE);
xxx_LogStatus("HXXLIB", __FILE__, __LINE__,Message);

ii = 0;
jj = pcd->DCE_List[pcd->active_DCE].gps_start_index - ii;
while( jj>1 && ii<step_buffer )
{
   if( (pcd->GPS_Group->acsutctime.value.d[jj]-pcd->GPS_Group->acsutctime.value.d[jj-1]) > EPS )
      break;
   ii++;
   jj = pcd->DCE_List[pcd->active_DCE].gps_start_index - ii;
}
pcd->DCE_List[pcd->active_DCE].gps_start_index = jj;
pcd->DCE_List[pcd->active_DCE].start_time = pcd->GPS_Group->acsutctime.value.d[jj];
pcd->DCE_List[pcd->active_DCE].acs_start_tc = pcd->GPS_Group->acsutctime.time[jj];
ii = 0;
jj = pcd->DCE_List[pcd->active_DCE].gps_stop_index + ii;
while( jj<(pcd->GPS_Group->acsutctime.count-1) && ii<step_buffer )
{      
   if( (pcd->GPS_Group->acsutctime.value.d[jj]-pcd->GPS_Group->acsutctime.value.d[jj-1]) > EPS )
      break;
   ii++;
   jj = pcd->DCE_List[pcd->active_DCE].gps_stop_index + ii;
}
pcd->DCE_List[pcd->active_DCE].gps_stop_index = jj;
pcd->DCE_List[pcd->active_DCE].stop_time = pcd->GPS_Group->acsutctime.value.d[jj];
pcd->DCE_List[pcd->active_DCE].acs_stop_tc = pcd->GPS_Group->acsutctime.time[jj];

printf("Starting index for imagery %d\n",pcd->DCE_List[pcd->active_DCE].gps_start_index);
printf("DCE LIST Start time %f\n",pcd->DCE_List[pcd->active_DCE].start_time);
printf("ACS Start %f\n",pcd->DCE_List[pcd->active_DCE].acs_start_tc);

printf("Stoping index for imagery %d\n",pcd->DCE_List[pcd->active_DCE].gps_stop_index);
printf("DCE LIST Stop time %f\n",pcd->DCE_List[pcd->active_DCE].stop_time);
printf("ACS Stop %f\n",pcd->DCE_List[pcd->active_DCE].acs_stop_tc);

/* Find the GPS UTCF record that immediately follows this time code */
curr_index = 0;
curr_start = pcd->DCE_List[pcd->active_DCE].acs_start_tc;
while ( pcd->GPS_Group->utcftime.time[curr_index] < curr_start &&
            curr_index < pcd->GPS_Group->utcftime.count ) curr_index++;
pcd->DCE_List[pcd->active_DCE].utcf_index = curr_index;

/* Convert SWIR/VNIR time codes from MET to GPS epoch */
vnir_time_code = hmodel->vnir_time_code + pcd->GPS_Group->utcftime.value.d[curr_index];
swir_time_code = hmodel->swir_time_code + pcd->GPS_Group->utcftime.value.d[curr_index];
hmodel->vnir_time_code = vnir_time_code;
hmodel->swir_time_code = swir_time_code;
printf("Delta VNIR/SWIR time codes (GPS) %f\n",vnir_time_code-swir_time_code);

/* Convert the actual DCE time to year, doy, sod */
exx_gpstoyds( vnir_time_code, hmodel->dce_time );

sprintf(Message,"Refined DCE Time:  %04d %03d %lf",
        (int)hmodel->dce_time[0], (int)hmodel->dce_time[1], hmodel->dce_time[2]);
xxx_LogStatus("HXXLIB", __FILE__, __LINE__,Message);

sprintf(Message,"DCE Duration:  %lf",
        pcd->DCE_List[pcd->active_DCE].stop_time-pcd->DCE_List[pcd->active_DCE].start_time);
xxx_LogStatus("HXXLIB", __FILE__, __LINE__,Message);

/* find the ACS time */
/* The ACS time reference is in the GPS telemetry. It gives a sequence
   of GPS times (time relative to the GPS epoch) that are time stamped
   with the spacecraft time codes. This provides a basis for building
   the relationship between the spacecraft clock (time codes) and
   GPS time. What we're going to do here is figure out what this
   (presumably linear) relationship is for this DCE and initialize
   the exx_init_timecode routine so that we can subsequently use the
   exx_timecodetogps and exx_gpstotimecode routines to convert back
   and forth (e.g., when mucking with the ephemeris and attitude data). */

kk = pcd->DCE_List[pcd->active_DCE].gps_start_index;
qq = pcd->DCE_List[pcd->active_DCE].gps_stop_index;
if ( qq <= kk )
{
   qq = kk + 1;
   if ( qq >= pcd->GPS_Group->acsutctime.count )
   {
      kk--;
      qq--;
   }
}
gpstime = pcd->GPS_Group->acsutctime.value.d[kk];
timecode = pcd->GPS_Group->acsutctime.time[kk];

/* IMPROVEMENT_OPPORTUNITY
   We'll do the rate computation from the two end points for now.
   Could also do a regression with all the points. */

timecode_rate = (pcd->GPS_Group->acsutctime.value.d[qq] - gpstime)
              / (pcd->GPS_Group->acsutctime.time[qq] - timecode);

exx_init_timecode( gpstime, timecode, timecode_rate );

/* Now we'll use this timing information to find the start time for the
   quaternion data */

timecode = pcd->DCE_List[pcd->active_DCE].acs_start_tc;
kk = xxx_find_index( timecode, pcd->ACS_Group->quat1 );
timecode = pcd->ACS_Group->quat1.time[kk];
gpstime = exx_timecodetogps( timecode );

/* Finally, we'll load the acs_time array */
exx_gpstoyds( gpstime, hmodel->acs_time );

return(SUCCESS);
}
