/******************************************************************************
NAME: hxx_process_hyp_model

PURPOSE:
Load the line-of-sight Legendre polynomial coefficients, convert the
ephemeris data to both ECR and ECI, merge attitude quaternions and IRU
data, generate roll, pitch, and yaw correction information and calculate
the initial spacecraft roll, pitch and yaw. Fill the model structure with
this information.

DEVELOPMENT HISTORY:


RETURN VALUE:
Type = int
Value    Description
-----    -----------
SUCCESS  Successful completion
ERROR    Operation failed

NOTES:
The -of-sight focal plane parameters are read from the calibration parameter 
file and stored in the hyperion model structure for future calls to the model.


ALGORITHM REFERENCES:

******************************************************************************
		        Property of the U.S. Government
                            USGS EROS Data Center
******************************************************************************/
#include <string.h>
#include "xxx_Types.h"  /* This include defines the ERROR vairaibles	     */
#include "xxx_HYPPCD.h"
#include "xxx_LogStatus.h"
#include "xxx_const.h"  /* This include defines several constants used       */
#include "xxx_math.h"
#include "hxx_hypmodel.h"

int hxx_process_hyp_model(
    struct EPHEM *ephem,/* I: Ephemeris points */
    double  xp,         /* I: True pole postion in the mean pole coords system,
                           x-axis */
    double yp,          /* I: True pole position in the mean pole coords
                           system, y-axis */
    double ut1_utc,     /* I: Earth rotation angle difference ut1 - utc */
    CPF *cpf,		/* I: calibration parameter file data	     */
    HYPPCD *pcd,	/* I: Payload Correction Data from the satellite     */
    HYPMODEL *hmodel)	/* IO: The ALI model information	     */
{
int 	status;		/* Status of function return		     */

/*
  Load the SCA line-of-sight vector coefficients
*/
status = hxx_los_coeffs(hmodel, cpf);
if (status != SUCCESS) 
    {
    xxx_LogStatus("HXXLIB", __FILE__, __LINE__, "Error returned from hxx_los_coeffs");
    return(ERROR);
    }

/*
  Convert the ephemeris information to Earth Centered Earth Fixed system 
  and store both ECEF and Earth Centered Inertial system coordinate
  frames in the hmodel structure.
  Here ephem time is the time of the first ephemeris point
*/
status = hxx_convert_ephem(ephem, xp, yp, ut1_utc, hmodel);
if (status != SUCCESS) 
    {
    xxx_LogStatus("AXXLIB", __FILE__, __LINE__, "Error returned from axx_convert_ephem");
    return(ERROR);
    }

/*
    Align the attitude information and generate the roll, pitch, and yaw
    correction information

   IRU and quaternion data (in PCD), try to sync and
   output = r, p, and y (hmodel->rpy)

   Also calculates the initial space craft roll, pitch and yaw
   (at acs_time - before img data)
*/
status = hxx_align_attitude( pcd, hmodel, cpf );
if (status != SUCCESS) 
    {
    xxx_LogStatus("AXXLIB", __FILE__, __LINE__, "Error returned from hxx_align_attitude");
    return(ERROR);
    }

return(SUCCESS);
}
