/******************************************************************************
FUNCTION:	hxx_run_hypmodel

PURPOSE:	Get information needed to process model data and run model.
                Call hxx_out_of_pcd to get timing information. This includes
                using the dce_time parameter in the Hyperion model (initially set
                based on the image time) to determine which PCD DCE we are in,
                updating the dce_time to match the values in the PCD structure,
                and selecting the ephemeris data source (ACS or GPS).

DEVELOPMENT HISTORY:

RETURN VALUE:
Type = int
Value    Description
-----    -----------
ERROR    Error processing the model data
SUCCESS  Model data was updated  

NOTES:

It is assumed that memory for the model structure has been allocated in
the calling program.

There are two sources of ephemeris information in the PCD - from the ACS
records (predicted uplink data) and from the GPS records (on-board GPS data).
This routine will select one based on the setting of the hmodel->ephem_type
parameter which is set in the calling routine based on a user-entered parameter.


ALGORITHM REFERENCES:
	See hxx_hypmodel.h for description of hmodel structure.	
*******************************************************************************/
#include "xxx_LogStatus.h"
#include "xxx_HYPPCD.h"
#include "exx_geometric.h"
#include "hxx_hypmodel.h"

double get_hyp_frame_rate(CPF *cpf)
{
   return(cpf->Hyp_Scan_Parameters.Frame_Rate);
}

double get_hyp_integration_time(CPF *cpf)
{
   return(cpf->Hyp_Scan_Parameters.Integration_Time);
}

int hxx_run_hypmodel
(
   char *hk_dir,			/* I: Housekeeping data directory */
   char *hk_root,			/* I: Housekeeping data file root name */
   char *cpf_name,			/* I: Path of the cpf (optional) */
   int nlines,
   HYPMODEL *hmodel,		        /* I/O: Hyperion model structure */
   CPF **cpf				/* O: CPF structure */
)
{
   HYPPCD pcd;				/* ALI PCD data structure */
   struct EPHEM ephem[NPCD];		/* ephemeris data from the PCD */
   double xp;				/* x shift pole wander in arc sec. */ 
   double yp;				/* y shift pole wander in arc sec. */ 
   double ut1_utc;			/* UT1 - UTC time difference in seconds */
   double mjd;				/* modified Julian date */
   char	ErrMsg[STRLEN];			/* error message used by xxx_LogError */

/*----------------------------------------------------------------------
  Read information needed to process model data. First get the PCD 
  information.
----------------------------------------------------------------------*/

   if ((xxx_read_HYPPCD (&pcd, hk_dir, hk_root )) == FAIL)
   {
      sprintf(ErrMsg,"Error loading Hyperion PCD from housekeeping file");
      xxx_LogStatus ("HXXLIB", __FILE__, __LINE__, ErrMsg);
      return (ERROR);
   }

   if (xxx_find_DCEs_hyp(&pcd) == FAIL)
   {
      sprintf(ErrMsg,"Error finding Hyperion PCD from housekeeping file");
      xxx_LogStatus ("HXXLIB", __FILE__, __LINE__, ErrMsg);
      return (ERROR);
   }

/*----------------------------------------------------------------------
   Get initial timing data from the HYPPCD structure.
   This includes using the hmodel->dce_time values initially populated
   from the image start time to determine which DCE we're in, and
   updating the dce_time array with more accurate data.
----------------------------------------------------------------------*/
   if (hxx_out_of_pcd( nlines, &pcd, hmodel ) != SUCCESS) 
   {
      xxx_LogStatus("HXXLIB", __FILE__, __LINE__,
                    "Error returned from hxx_out_of_pcd");
      xxx_free_HYPPCD( &pcd );
      return(ERROR);
   }

/*----------------------------------------------------------------------
   Get the CPF information.
----------------------------------------------------------------------*/
   *cpf = exx_read_cpf (cpf_name);
   if (*cpf == NULL)
   {
      xxx_LogStatus ("HXXLIB", __FILE__, __LINE__, 
                     "Error reading CPF in exx_read_cpf");
      xxx_free_HYPPCD( &pcd );
      return (ERROR);
   }

   hmodel->frame_rate = get_hyp_frame_rate(*cpf);

   hmodel->integration_time = get_hyp_integration_time(*cpf);

/*----------------------------------------------------------------------
   Make sure our data is in the effective time period for the CPF
----------------------------------------------------------------------*/
if (exx_check_date(hmodel->dce_time, *cpf) != SUCCESS)
    {
    xxx_LogStatus ("HXXLIB", __FILE__, __LINE__, 
                   "Error returned from exx_check_date.");
    xxx_free_HYPPCD( &pcd );
    return (ERROR);
    }

/*----------------------------------------------------------------------
   Select the valid ephemeris points.
----------------------------------------------------------------------*/
   if (hxx_valid_ephem (&pcd, hmodel->ephem_type, *cpf, &hmodel->nephem, ephem) == ERROR)
   {
      xxx_LogStatus ("HXXLIB", __FILE__, __LINE__, 
                     "Error obtaining valid ephemeris points exx_valid_ephem");
      xxx_free_HYPPCD( &pcd );
      return (ERROR);
   }

/*----------------------------------------------------------------------
   Get x and y shift pole wander and UT1-UTC time difference
----------------------------------------------------------------------*/
   if (exx_getmjdcoords (ephem->time, &mjd, &xp, &yp, &ut1_utc, *cpf) == ERROR)
   {
      xxx_LogStatus ("HXXLIB", __FILE__, __LINE__, 
                     "Error returned from exx_getmjdcoords");
      xxx_free_HYPPCD( &pcd );
      return (ERROR);
   }
  
/*----------------------------------------------------------------------
   Generate updated model information.
----------------------------------------------------------------------*/
   if (hxx_process_hyp_model (ephem, xp, yp, ut1_utc, *cpf, &pcd, hmodel) == ERROR)
   {
      xxx_LogStatus ("HXXLIB", __FILE__, __LINE__, 
                     "Error updating the ALI model");
      xxx_free_ALIPCD( &pcd );
      return (ERROR);
   }

/*---------------------
   free the pcd
   ---------------------*/
   xxx_free_HYPPCD( &pcd );

   return(SUCCESS); 
}
