/******************************************************************************
FUNCTION:	hxx_update_hmodel

PURPOSE:	Uses precision data to update the Hyperion model data
		and write to a separate model output file.

RETURN VALUE:
Type = int
Value    Description
-----    -----------
ERROR    Error updating the model file
SUCCESS  Model file was updated  

NOTES:

ALGORITHM REFERENCES:
	See hxx_hypmodel.h for description of hmodel structure.	
*******************************************************************************/
#include <stdio.h>
#include <string.h>
#include "hdf.h"
#include "xxx_LogStatus.h"
#include "xxx_hyp_level1r.h"
#include "hxx_hypmodel.h"
#include "hxx_forward_model.h"

int hxx_update_hmodel
(
   char *hypmodel_name,		/* I: name of original Hyperion model file        */
   char *prec_hypmodel_name,	/* I: name of precision corrected Hyperion model file  */
   char *l1r_name,
   char *hk_dir,		/* I: name of housekeeping data directory         */
   char *hk_root,		/* I: housekeeping data file root name            */
   char *cpf_name,   		/* I: Path of the cpf (optional) 	          */
   double *reftime,   		/* I: precision reference time (year, doy, sod)   */
   VECTOR satpos,    		/* I: precision satellite position                */
   VECTOR satvel,    		/* I: precision satellite velocity                */
   VECTOR rpyangle,  		/* I: precision roll, pitch, yaw angle            */
   VECTOR rpyrate,   		/* I: precision roll, pitch, yaw rate             */
   int nlines,			/* I: number of lines in l1 imagery		  */
   double *refatt		/* O: roll, pitch, and yaw at reference time      */
)

{
   HYPMODEL *hmodel; 			/* points to Hyperion model structure */
   char	ErrMsg[STRLEN];			/* error message used by xxx_LogError */
   CPF *cpf=NULL;			/* CPF structure */
   HYP_L1R_IMAGE_ATTRIBUTES att;        /* Hyperion level 1R image attributes */
   double hxx_get_time_code(int32 *time_code, int32 *sync_time);
   
/*----------------------------------------------------------------------
  Allocate space for the model structure.
----------------------------------------------------------------------*/

   if ((hmodel = malloc(sizeof(HYPMODEL)))== NULL)
   {
      xxx_LogStatus ("HXXLIB", __FILE__,  __LINE__, 
                     "Error - cannot allocate model structure.");
      return (ERROR);
   }

/*----------------------------------------------------------------------
  Open the original model file and fill the model structure.
----------------------------------------------------------------------*/

   if (hxx_get_hypmodel (hypmodel_name, hmodel) == FAIL)
   {
      xxx_LogStatus ("HXXLIB", __FILE__, __LINE__, 
                     "Error returned from hxx_get_hypmodel.");
      free(hmodel);
      return (ERROR);
   }
   /*
      Load the Level 0R data for into a data structure
   */
   if ( xxx_read_hyp_l1r_image_attributes( l1r_name, &att ) != SUCCESS )
   {
       xxx_LogStatus("HXXLIB", __FILE__, __LINE__, 
                     "Error returned from xxx_read_hyp_l1r_image_attributes.");
       return (ERROR);
   }
   att.vnir_time_code[0] = 0;
   att.swir_time_code[0] = 0;
   /* This currently does not appear to work */
   hmodel->vnir_time_code = hxx_get_time_code(&att.vnir_time_code[8],&att.vnir_sync_time[3]);
   hmodel->swir_time_code = hxx_get_time_code(&att.swir_time_code[8],&att.swir_sync_time[3]);
   printf("Time codes VNIR/SWIR %f %f\n",hmodel->vnir_time_code,hmodel->swir_time_code);

/*----------------------------------------------------------------------
  Update the precision values in the model structure.        
----------------------------------------------------------------------*/

   hmodel->p_roll = rpyangle.x;
   hmodel->p_rrate = rpyrate.x;
   hmodel->p_pitch = rpyangle.y;
   hmodel->p_prate = rpyrate.y;
   hmodel->p_yaw = rpyangle.z;
   hmodel->p_yrate = rpyrate.z;
   hmodel->p_x = satpos.x;
   hmodel->p_vx = satvel.x;
   hmodel->p_y = satpos.y;
   hmodel->p_vy = satvel.y;
   hmodel->p_z = satpos.z;
   hmodel->p_vz = satvel.z;
   hmodel->p_ref_time = (reftime[2] - hmodel->dce_time[2])
                      + (reftime[1] - hmodel->dce_time[1])*SEC_PER_DAY;

/*----------------------------------------------------------------------
   Find the spacecraft attitude at the reference time
----------------------------------------------------------------------*/
   if (hxx_findatt(hmodel, hmodel->p_ref_time, 
                   &refatt[0], &refatt[1], &refatt[2]) == ERROR)
   {
      xxx_LogStatus ("HXXLIB", __FILE__, __LINE__, 
                     "Error finding reference attitude in hxx_findatt");
      free(hmodel);
      return (ERROR);
   }

/*----------------------------------------------------------------------
   Set this to let run_alimodel know that it is precision updated.
----------------------------------------------------------------------*/

   hmodel->correction_type = PRECISION;

/*----------------------------------------------------------------------
   Process the model data using the precision information.
----------------------------------------------------------------------*/
   
   if (hxx_run_hypmodel(hk_dir, hk_root, cpf_name, nlines, hmodel, &cpf) == ERROR)
   {
      xxx_LogStatus ("HXXLIB", __FILE__, __LINE__, 
                     "Error processing model data in hxx_run_hypmodel");
      free(hmodel);
      return (ERROR);
   }

/*----------------------------------------------------------------------
   Write out the Hyperion model with the modified information to an HDF file.
----------------------------------------------------------------------*/

   if (hxx_put_hypmodel(prec_hypmodel_name, hmodel) == ERROR)
   {
      sprintf (ErrMsg, "Error writing new Hyperion model file %s", prec_hypmodel_name);
      xxx_LogStatus ("HXXLIB", __FILE__, __LINE__, ErrMsg);
      free(hmodel);
      exx_free_cpf(cpf);
      return (ERROR);
   }

   free(hmodel);

   return(SUCCESS); 
}
