/******************************************************************************
NAME:		hxx_create_1g_band

PURPOSE:	Create a SDS for the specified 1G band.

RETURN VALUE: 
Type = int
Value           Description
-----           -----------
SUCCESS		Successfully created band SDS
ERROR    	Error occurred; band SDS not created

NOTES:		The 1G image is a VGroup containing a SDS all imagery.  The 
		data for each SDS are stored in an external element.

		The HDFIO structure on input must have the hdf_dir_name
		field filled in.
******************************************************************************/
#include <stdio.h>
#include <unistd.h>
#include "xxx_Types.h"
#include "xxx_LogStatus.h"
#include "hxx_hdf_io.h"
#include "hxx_utility.h"

int hxx_create_1g_band(
	int nb,
	int nl,
	int ns,
	DataType dtype,         /* I-Data type of the band      */
	HYPHDFIO *hdfid)	/* I/O-SDS id for the band added to structure */
{
intn	hdf_status;		/* HDF function return status	*/
int32	ref,			/* vgroup element reference	*/
	tag,			/* vgroup element tag		*/
	data_type,		/* data type for 1G images	*/
	rank=3,			/* 2-d SDS (all bands)		*/
	dim_sizes[MAX_VAR_DIMS];/* dimension sizes (nl, ns)	*/

int16 	fill_i16=0;		/* integer fill value		*/
uint16 	fill_ui16=0;		/* integer fill value		*/
int32	fill_i32=0;		/* long fill value		*/
float32 fill_f32=0.0;		/* float fill value		*/
uint8 	fill_ui8=0;		/* byte fill value		*/
void 	*fill_val;		/* pointer to fill value	*/

char 	SDname[MAX_NC_NAME];	/* SDS name			*/
char	msg[STRLEN];		/* Error message string		*/
char	ext_filename[STRLEN];	/* External file name		*/

/* 
 * Set up SDS name using name constructed from hdf dir file name and band
 * This will also be the filename of the external element
 */
hxx_create_band_name(hdfid->hdf_dir_name, "SDS", SDname);
strcpy(hdfid->sds_name,SDname);

dim_sizes[0] = nb;
dim_sizes[1] = nl;
dim_sizes[2] = ns;
if (dtype == I2) 
    {
    data_type = DFNT_INT16;
    fill_val = (void *)&fill_i16;
    }
else if (dtype == UI2) 
    {
    data_type = DFNT_UINT16;
    fill_val = (void *)&fill_ui16;
    }
else if (dtype == I4) 
    {
    data_type = DFNT_INT32;
    fill_val = (void *)&fill_i32;
    }
else if (dtype == R4) 
    {
    data_type = DFNT_FLOAT32;
    fill_val = (void *)&fill_f32;
    }
else if (dtype == BYTE)
    {
    data_type = DFNT_UINT8;
    fill_val = (void *)&fill_ui8;
    }
else
    {
    xxx_LogStatus("HXXLIB",__FILE__,__LINE__,"Invalid image data type");
    return(ERROR);
    }

hdfid->sds_id = SDcreate(hdfid->sd_id, SDname, data_type, rank, dim_sizes);
if (hdfid->sds_id == FAIL)
    {
    sprintf(msg,"Error creating SDS %s", SDname);
    xxx_LogStatus("HXXLIB",__FILE__,__LINE__,msg);
    return(ERROR);
    }

/*
 * Set the fill value
 */
hdf_status = SDsetfillvalue(hdfid->sds_id, fill_val);
if (hdf_status == FAIL)
    {
    HEprint(stderr,0);
    xxx_LogStatus("HXXLIB",__FILE__,__LINE__,"Error setting fill value");
    }

/*
 * Set the SDS into an external file
 * Use the SDS name as the exernal file name
 */
hxx_create_band_name(hdfid->hdf_dir_name, "FILE", ext_filename);

/* remove the file in case it already exists */
unlink(ext_filename);

hdf_status=SDsetexternalfile(hdfid->sds_id, ext_filename, 0);
if (hdf_status == FAIL)
    {
    sprintf(msg,"Error creating external SDS %s", SDname);
    xxx_LogStatus("HXXLIB",__FILE__,__LINE__,msg);
    return(ERROR);
    }

/*
 * Add the SDS to the group
 */
ref = SDidtoref(hdfid->sds_id);
tag = DFTAG_SD;
hdf_status = Vaddtagref(hdfid->group_id, tag, ref);
if (hdf_status == FAIL)
    {
    sprintf(msg,"Error creating SDS %s", SDname);
    xxx_LogStatus("HXXLIB",__FILE__,__LINE__,msg);
    return(ERROR);
    }

return(SUCCESS);
}
