#include "hxx_grid.h"

/******************************************************************************
NAME:           axx_free_grid

PURPOSE:
Free all the pointers in the grid.

RETURN VALUE:   Type = int
Value    Description
-----    -----------
SUCCESS  Successful completion
ERROR    Operation failed

NOTES:
This routine will loop through the specified bands, freeing the pointers
within the grid.  If no bands were specified, then all the bands in the grid
will be freed.  In this case, the bands array will be filled in with the
bands in the grid and nbands will be set to the correct value.  If bands 6L
and 6H are both present, then their pointers within the grid will have the
same address.  Thus we only need to free the pointer for one of the bands.
This routine will choose band 6L.

ALGORITHM REFERENCES:
See hxx_grid.h for a description of the grid structure.

******************************************************************************/

int hxx_free_grid (
  HBandNumber *bands,        /* I/O: Array of bands to free from the grid */
  int *nbands,               /* I/O: Number of bands in the bands array   */
  hxx_GRID_TYPE *grid)       /* I/O: Grid structure to be freed           */
{
  int  i,                    /* Loop variables                           */
       currref;              /* Current reference band value             */

  /* If no bands are specified then free all the bands in the grid.
   */
  if (*nbands == 0)
  {
    *nbands = grid->nbands;
    memcpy (bands, grid->bands, *nbands * sizeof (HBandNumber));
  }

  /* Loop through the bands freeing the correct reference bands.
   */
  for (i = 0; i < *nbands; i++)
  {
    /* Get the reference value of the current band.
     */
    currref = bands[i];

    if (grid->gridbands[currref].in_lines != NULL)
    {
      free (grid->gridbands[currref].in_lines);
      grid->gridbands[currref].in_lines = NULL;
    }
    if (grid->gridbands[currref].in_samps != NULL)
    {
      free (grid->gridbands[currref].in_samps);
      grid->gridbands[currref].in_samps = NULL;
    }
    if (grid->gridbands[currref].out_lines != NULL)
    {
      free (grid->gridbands[currref].out_lines);
      grid->gridbands[currref].out_lines = NULL;
    }
    if (grid->gridbands[currref].out_samps != NULL)
    {
      free (grid->gridbands[currref].out_samps);
      grid->gridbands[currref].out_samps = NULL;
    }
    if (grid->gridbands[currref].poly_lines != NULL)
    {
      free (grid->gridbands[currref].poly_lines);
      grid->gridbands[currref].poly_lines = NULL;
    }
    if (grid->gridbands[currref].poly_samps != NULL)
    {
      free (grid->gridbands[currref].poly_samps);
      grid->gridbands[currref].poly_samps = NULL;
    }
    if (grid->gridbands[currref].sattoproj != NULL)
    {
      free (grid->gridbands[currref].sattoproj);
      grid->gridbands[currref].sattoproj = NULL;
    }
    if (grid->gridbands[currref].projtosat != NULL)
    {
      free (grid->gridbands[currref].projtosat);
      grid->gridbands[currref].projtosat = NULL;
    }
  } /* Loop on bands */

  return (SUCCESS);
}
