#ifndef _XXX_H_GRID_H_
#define _XXX_H_GRID_H_

#include <stdlib.h>
#include "mfhdf.h"
#include "proj.h"
#include "xxx_Types.h"
#include "xxx_LogStatus.h"
#include "hxx_hdf_io.h"

/* The GRID structure contains a geometric mapping grid for mapping values
   from an input image to an output image.  A grid will have many grid cells,
   depending on the grid_cell_size and the number of scans in the image.
   
   The mapping grids are in input space coordinates (line, sample).  Each grid
   contains 'ngrid_lines' rows and 'ngrid_samps' columns.  The field 'in_lines'
   is a list of the line numbers that make up the horizontal rows of the grid
   and 'in_samps' is a list of the sample numbers that make up the vertical
   columns of the grid.  The field 'out_lines' is a list of the line numbers
   that make up the horizontal row values for each grid point.  The field
   'out_samps' is a list of the sample values for each grid point.  There are
   (ngrid_lines * ngrid_samps) grid points in each grid, and
   (ngrid_lines/2 * (ngrid_samps-1)) grid cells in each grid.  This structure
   also contains information needed to fill the output image's DDR and
   miscellaneous mapping grid parameters and statistics.

   EXAMPLE:
      An output image with 10 lines and 13 samps.  In the diagram the grid
      points are marked with the letters A through P.  Assume that each grid
      point has an x and y value associated with it.

      ngrid_lines = 4
      ngrid_samps = 4
      in_lines    = 1 16 17 32
      in_samps    = 1 100 200 300
      out_lines   = Ay By Cy Dy Ey Fy Gy Hy Iy Jy Ky Ly My Ny Oy Py
      out_samps   = Ax Bx Cx Dx Ex Fx Gx Hx Ix Jx Kx Lx Mx Nx Ox Px
      
      A---B---C---D
      |   |   |   |
      |   |   |   |
      E---F---G---H
      |   |   |   |
      |   |   |   |
      I---J---K---L
      |   |   |   |
      |   |   |   |
      M---N---O---P

   When writing the grid structure to a file, the constant values must be
   written first.  These values are ones that are not arrays or pointers and
   their size is always known.  Among these constant parameters is the number
   of bands to be processed.  Now the array structures that depend on the
   number of bands (and no other information) are written to the file.  Among
   these are the bands, number of lines and samples in the input image and in
   the grid.  Finally the elements that rely on the number of lines and samples
   in the image and in the grid are written to the file.  Among these are the
   grid lines and samples in input and output space and the projection
   coefficients.  Don't forget to free_grid when you are done with the grid
   structure.

   Elements to be written to the file (in correct order.)
   1.  gridname, nbands, dtype, correction_type, frame_type, satnum, units,
       code, datum, spheroid, zone, projprms, corners, center_pt,
       proj_center_pt, rotate_angle, orient_angle.
   2.  bands, grid_cell_size, pixsize, lines, samps, ngrid_lines, ngrid_samps,
       degree.
   3.  in_lines, in_samps, out_lines, out_samps, sattoproj, projtosat,
       poly_lines, poly_samps.

   When reading the grid structure from a file, the values must be read in the
   same order as they were output.  After all the constants are read, there
   will be "nbands" number of elements in all of the arrays to be read next.
   After all the arrays are read, the values in lines, samps, ngrid_lines, and
   ngrid_samps are used to malloc the appropriate amount of space for the
   pointers in the grid.  After the appropriate space is malloced for each of
   the pointers, their data can be read from the file.  These pointers will
   contain information for each band.

   Elements to be read from the file (in correct order.)
   1.  gridname, nbands, dtype, correction_type, frame_type, satnum, units,
       code, datum, spheroid, zone, projprms, corners, center_pt,
       proj_center_pt, rotate_angle, orient_angle.
   2.  bands, grid_cell_size, pixsize, lines, samps, ngrid_lines, ngrid_samps,
       degree. 
   3.  malloc space for the pointers
   4.  in_lines, in_samps, out_lines, out_samps, sattoproj, projtosat,
       poly_lines, poly_samps.

==============================================================================*/
#include "xxx_grid.h"
#include "xxx_const.h"
#include "xxx_structures.h"

typedef struct 
{
/* ==== Grid coefficients and other defining info. ====                      */
  HBandNumber bandno;      /* Enumerated band number */

  struct LNG_LS grid_cell_size;
                          /* Number of lines and samples in each grid cell.
                             The standard values for this will be 80 samples
                             for ms bands and 160 samples for the pan band.
                             The number of lines per grid cell will simply be
                             the number of lines per scan.                   */

  double pixsize;         /* Projection distance per pixel in meters.
                             Note: This will be the same for x and y
                             dimensions.                                     */

  double zspacing;        /* Elevation difference (in meters) between grid
                             elevation planes                                */

  int  lines;             /* Number of lines in the output image             */

  int  samps;             /* Number of samples in the output image           */

  int  ngrid_lines;       /* Number of grid lines through the input image.
                             Note: There are ngrid_lines-1 grid cell rows in
                             each grid.                                      */

  int  ngrid_samps;       /* Number of grid columns through the input image.
                             Note: There are ngrid_samps-1 grid cell columns
                             in each grid.                                   */

  int  nzplanes;          /* Number of elevation planes in the 3D grid       */

  int  zeroplane;         /* Index of the zero elevation plane in the grid   */

  int  *in_lines;         /* Pointer to the locations of the horizontal
                             coordinate (line number) on each grid row, going
                             from top to bottom.  The input grid cells are of
                             consistent size and are rectangular, perpendicular
                             to the line-sample axis of the input image.
                             Therefore, only intersections need be given.
                             There are ngrid_lines elements in this element. */

  int  *in_samps;         /* Pointer to the locations of the vertical
                             coordinate (pixel number) on each grid column,
                             going from left to right.  Again, only
                             intersections need be given.  There are
                             ngrid_samps in this element.                    */

  double *out_lines;      /* Pointer to the lines stored in the grid, in
                             output space.  These values are stored in row
                             major format for each grid point.  There
                             are ngrid_lines*ngrid_samps*nzplanes elements
                             in this array.                                  */

  double *out_samps;      /* Pointer to the samples stored in the grid, in
                             output space.  These values are stored in row
                             major format for each grid point.  There
                             are ngrid_lines*ngrid_samps*nzplanes elements
                             in this array.                                  */

  struct COEFFICIENTS *sattoproj;
                          /* Pointers to transformation coefficients for
                             satellite to projection space (forward).
                             There are ngrid_lines*ngrid_samps*nzplanes
                             grid cells per pointer in sattoproj.            */

  struct COEFFICIENTS *projtosat;
                          /* Pointers to transformation coefficients for
                             projection to satellite space (inverse).
                             There are ngrid_lines*ngrid_samps*nzplanes
                             grid cells per pointer in sattoproj.            */

  int  degree;            /* Degree of the rough transformation polynomial.
                             This specifies that there are
                             (degree+1)^2 * nzplanes data elements in the
                             poly_lines and poly_samps ptrs.                 */

  double *poly_lines;     /* Reverse rough transformation coefficients for
                             the lines.  These are for the grid_cell_poly
                             routine.  There are (degree+1)^2 * nzplanes
                             data elements in this pointer.                  */

  double *poly_samps;     /* Reverse rough transformation coefficients for
                             the samples.  These are for the grid_cell_poly
                             routine.  There are (degree+1)^2 * nzplanes
                             data elements in this pointer.                  */
} hxx_GRID_BAND_TYPE;

typedef struct
{
/* ==== General grid info. ====                                              */
  char gridname[GRIDNAME_SIZE];
                          /* Host file name of the grid file.                */

  int  nbands;            /* Number of bands stored in bands.                */

  HBandNumber bands[HYP_NBANDS];
                          /* Array of the band numbers to be processed.        
                             The integer value of this enumerated type will
                             be used to access the grid values in their
                             band-referenced position.
                             Note: These band numbers can be input in any
                             order.  They don't have to be in increasing
                             order.  (i.e. b5, b2, b8)                       */

  int  dtype;             /* The data type of all the values to be stored in
                             the grid.  Valid values are
                               BYTE   = unsigned char
                               WORD   = int (16 bit)
                               LONG   = long (32 bit)
                               REAL   = float
                               DOUBLE = double
                               CHAR   = char                                */

  CorrectionType correction_type;
                          /* This flag specifies if the grid is a systematic,
                             precision, or terrain corrected grid.
                               SYSTEMATIC - Systematic.
                               PRECISION - Precision corrected.
                               TERRAIN   - Terrain corrected.                */

  int  satnum;            /* Number of satellite that acquired the data      */

  FrameType frame_type;   /* Type of framing process to be used.  Valid
                             values are
                               GEOBOX - User specifies UL and LR lat/long
                                 coordinates.
                               PROJBOX - User specifies LR output projection
                                 coordinates.  The user also specifies
                                 another point in output projection space,
                                 along with its coordinating line/sample in
                                 image space.
                               UL_SIZE - User specifies UL output projection
                                 coordinate and the number of lines and
                                 samples in output space.
                               MINBOX - Minbox framing.
                               PATH_ORIENTED - Path-oriented framing.        */

/* ==== Projection parameters and info. ====                                 */
  char units[UNITS_SIZE]; /* Projection units.  This value corresponds to the
                             proj_units field in the DDR.                    */

  int  code;              /* Projection code for the output space image's DDR.
                             Values for this field are defined in the "proj.h"
                             include file.                                   */

  char datum[DATUM_SIZE]; /* Projection datum code.  It has the same valid
                             range and defaults as the "datum_code" field in
                             the DDR.                                        */

  int  spheroid;          /* Projection spheroid code                        */

  int  zone;              /* Projection zone code for UTM or State Plane
                             projections.  It has the same valid ranges and
                             defaults as the "zone_code" field in the DDR.   */

  double projprms[PROJPRMS_SIZE];
                          /* Array of 15 projection coefficients as required
                             by the projection transformation package.  Refer
                             to the projection package documentation for a
                             description of each field for a given projection.
                             This field corresponds to the "proj_coef" field
                             in the DDR.                                     */

/* ==== Grid corners. ====                                                   */
  struct CORNERS corners; /* Projection coordinates of the resulting output
                             image's four corners.  This field corresponds to
                             the DDR's "upleft", "loleft", "upright", and
                             "loright" fields.                               */

/* ==== Path-oriented info. ====                                             */
  struct DBL_LAT_LONG center_pt;
                          /* Lat/long of the WRS scene center point.         */

  DBL_XY proj_center_pt; /* Proj x/y of the WRS scene center point.   */

  double rotate_angle;    /* Rotation angle of the path-oriented product.
                             (degrees)                                       */

  double orient_angle;    /* Frame orientation angle in map coords. (degrees)*/

/* ==== Individual band information. ====                                    */
  hxx_GRID_BAND_TYPE gridbands[HYP_NBANDS];
                          /* Array of pointers to the band specific grid
                             information.                                    */
} hxx_GRID_TYPE;

/* Prototypes for grid_io.
 */
int hxx_put_grid (
  char grid_filename[STRLEN],/* I: HDF output file name                     */
  hxx_GRID_TYPE *grid);      /* I: Grid structure to be output              */

int hxx_write_grid (
  char grid_filename[STRLEN],/* I: HDF output file name                     */
  hxx_GRID_TYPE *grid);      /* I: Grid structure to be output              */

int hxx_get_grid (
  char grid_filename[STRLEN],/* I: HDF input file name                    */
  HBandNumber *bands,         /* I/O: Array of bands to free from the grid */
  int *nbands,               /* I/O: Number of bands in the bands array   */
  hxx_GRID_TYPE *grid);      /* O: Grid structure to be output            */

int hxx_read_grid (
  char grid_filename[STRLEN],/* I: HDF input file name                    */
  HBandNumber *bands,         /* I/O: Array of bands to free from the grid */
  int *nbands,               /* I/O: Number of bands in the bands array   */
  hxx_GRID_TYPE *grid);      /* O: Grid structure to be output            */

void hxx_SOM_grid_calc(
  hxx_GRID_TYPE *grid);		 /* I/O: grid structure to have coords flipped */

int hxx_get_grid_headers (
  char grid_filename[STRLEN],/* I: HDF input file name                    */
  hxx_GRID_TYPE *grid);      /* O: Grid structure to be output            */

int hxx_get_grid_pointers (
  char grid_filename[STRLEN],/* I: HDF input file name                    */
  HBandNumber *bands,         /* I/O: Array of bands to free from the grid */
  int *nbands,               /* I/O: Number of bands in the bands array   */
  hxx_GRID_TYPE *grid);      /* O: Grid structure to be output            */

int hxx_malloc_grid (
  HBandNumber *bands,         /* I/O: Array of bands to free from the grid */
  int *nbands,               /* I/O: Number of bands in the bands array   */
  hxx_GRID_TYPE *grid);      /* I: Grid structure to be malloced          */

int hxx_free_grid (
  HBandNumber *bands,         /* I/O: Array of bands to free from the grid */
  int *nbands,               /* I/O: Number of bands in the bands array   */
  hxx_GRID_TYPE *grid);      /* I: Grid structure to be freed             */

int hxx_grid_cell_poly( 
   hxx_GRID_BAND_TYPE *grid_band_ptr,   /* I/O: Grid associated with search image*/
   int nzplanes);

int hxx_ols2ils( 
    const hxx_GRID_BAND_TYPE *grid_band_ptr, /* I: pointer to grid band */
    int  *row_ptr,     /* O: pointer to input space grid row */
    int  *col_ptr,     /* O: pointer to input space grid column */
    long input_nl,
    long input_ns,
    double oline,      /* I: output space line to map to input space */
    double osamp,      /* I: output space sample to map to input space */
    double *iline_ptr, /* O: pointer to input space line */
    double *isamp_ptr, /* O: pointer to input space sample */
    int  iplane);      /* I: elevation plane */

int hxx_all_ols2ils
(
    const hxx_GRID_BAND_TYPE *grid_band_ptr, /* I: pointer to grid band */
    int  *row_ptr,     /* O: pointer to input space grid row */
    int  *col_ptr,     /* O: pointer to input space grid column */
    long input_nl,
    long input_ns,
    double oline,      /* I: output space line to map to input space */
    double osamp,      /* I: output space sample to map to input space */
    double *iline_ptr, /* O: pointer to input space line */
    double *isamp_ptr, /* O: pointer to input space sample */
    int  iplane        /* I: elevation plane */
);

int hxx_3d_ols2ils
(
    const hxx_GRID_BAND_TYPE *grid_band_ptr, /* I: pointer to grid band */
    long input_nl,      /* I: number of lines in input image */
    long input_ns,      /* I: number of samples in input image */
    double oline,       /* I: output space line to map to input space */
    double osamp,       /* I: output space sample to map to input space */
    double *iline_ptr,  /* O: pointer to input space line */
    double *isamp_ptr,  /* O: pointer to input space sample */
    double elev         /* I: point elevation */
);

int hxx_findgridcell( 
   const hxx_GRID_BAND_TYPE *grid_band_ptr, /* I: Grid band to be searched */
   int *row,            /* I/O: Row of grid, input as first guess */
   int *col,            /* I/O: Column of grid, input as first guess */
   double oline,        /* I: Output line */
   double osamp,        /* I: Output sample */
   int iplane);         /* I: Elevation plane */

#endif
