#include <stdio.h>
#include <unistd.h>
#include <sys/types.h>
#include <stddef.h>
#include "xxx_const.h"
#include "hxx_grid.h"

/******************************************************************************
NAME:           hxx_malloc_grid

PURPOSE:
Malloc space for all the *pointers* in the grid.  The actual grid structure
should already exist.

RETURN VALUE:   Type = int
Value    Description
-----    -----------
SUCCESS  Successful completion
ERROR    Operation failed

NOTES:
This routine will loop through the specified bands, mallocing space for the
pointers within the grid.  If no bands were specified, then all the bands in
the grid will be malloced.  In this case, the bands array will be filled in
with the bands in the grid and nbands will be set to the correct value.  If
bands 6L and 6H are both present, then their pointers within the grid will
have the same address.  Thus we only need to malloc space for the pointer for
one of the bands.  This routine will choose band 6L.

ALGORITHM REFERENCES:
See hxx_grid.h for a description of the grid structure.
******************************************************************************/

int hxx_malloc_grid (
  HBandNumber *bands,        /* I/O: Array of bands to free from the grid */
  int *nbands,               /* I/O: Number of bands in the bands array   */
  hxx_GRID_TYPE *grid)       /* I/O: Grid structure to be malloced       */
{
   int  i, j, n;             /* Loop variables                           */
   int  currref;             /* Current reference band value             */
   char ErrMsg[STRLEN];      /* Error message                            */

  /* If no bands are specified then malloc all the bands in the grid.
   */
   if (*nbands == 0)
   {
      *nbands = grid->nbands;
      memcpy(bands, grid->bands, *nbands * sizeof (HBandNumber));
   }

  /* Loop through the bands mallocing the correct reference bands.
   */
   for (i = 0; i < *nbands; i++)
   {
      currref = bands[i];
     /* Get the reference value of the current band.  */
   
      grid->gridbands[currref].in_lines = 
            calloc (grid->gridbands[currref].ngrid_lines, sizeof (int ));
      if (grid->gridbands[currref].in_lines == NULL)
      {  
         /* Free previously malloced pointers. */
         strcpy (ErrMsg, "Error mallocing space for the pointers.");
         xxx_LogStatus ("HXXLIB", __FILE__, __LINE__, ErrMsg);
         for (j = 0; j < i; j++)
         {   
            currref = bands[j];
            free (grid->gridbands[currref].in_lines);
            free (grid->gridbands[currref].in_samps);
            free (grid->gridbands[currref].out_lines);
            free (grid->gridbands[currref].out_samps);
            free (grid->gridbands[currref].poly_lines);
            free (grid->gridbands[currref].poly_samps);
            free (grid->gridbands[currref].sattoproj);
            free (grid->gridbands[currref].projtosat);
         }
         return (ERROR);
      }

      grid->gridbands[currref].in_samps = 
            calloc (grid->gridbands[currref].ngrid_samps, sizeof (int ));
      if (grid->gridbands[currref].in_samps == NULL)
      { 
         /* Free previously malloced pointers. */
         strcpy (ErrMsg, "Error mallocing space for the pointers.");
         xxx_LogStatus ("HXXLIB", __FILE__, __LINE__, ErrMsg);
         free (grid->gridbands[currref].in_lines);
         for (j = 0; j < i; j++)
         { 
            currref = bands[j];
            free (grid->gridbands[currref].in_lines);
            free (grid->gridbands[currref].in_samps);
            free (grid->gridbands[currref].out_lines);
            free (grid->gridbands[currref].out_samps);
            free (grid->gridbands[currref].poly_lines);
            free (grid->gridbands[currref].poly_samps);
            free (grid->gridbands[currref].sattoproj);
            free (grid->gridbands[currref].projtosat);
         }
         return (ERROR);
      }

      grid->gridbands[currref].out_lines = malloc (grid->gridbands[currref].nzplanes *
                grid->gridbands[currref].ngrid_lines *
                grid->gridbands[currref].ngrid_samps * sizeof (double));
      if (grid->gridbands[currref].out_lines == NULL)
      { 
         /* Free previously malloced pointers. */
         strcpy (ErrMsg, "Error mallocing space for the pointers.");
         xxx_LogStatus ("HXXLIB", __FILE__, __LINE__, ErrMsg);
         free (grid->gridbands[currref].in_lines);
         free (grid->gridbands[currref].in_samps);
         for (j = 0; j < i; j++)
         { 
            currref = bands[j];
            free (grid->gridbands[currref].in_lines);
            free (grid->gridbands[currref].in_samps);
            free (grid->gridbands[currref].out_lines);
            free (grid->gridbands[currref].out_samps);
            free (grid->gridbands[currref].poly_lines);
            free (grid->gridbands[currref].poly_samps);
            free (grid->gridbands[currref].sattoproj);
            free (grid->gridbands[currref].projtosat);
         }
         return (ERROR);
      }

      grid->gridbands[currref].out_samps = malloc (grid->gridbands[currref].nzplanes *
                grid->gridbands[currref].ngrid_lines *
                grid->gridbands[currref].ngrid_samps * sizeof (double));
      if (grid->gridbands[currref].out_samps == NULL)
      { 
         /* Free previously malloced pointers. */
         strcpy (ErrMsg, "Error mallocing space for the pointers.");
         xxx_LogStatus ("HXXLIB", __FILE__, __LINE__, ErrMsg);
         free (grid->gridbands[currref].in_lines);
         free (grid->gridbands[currref].in_samps);
         free (grid->gridbands[currref].out_lines);
         for (j = 0; j < i; j++)
         { 
            currref = bands[j];
            free (grid->gridbands[currref].in_lines);
            free (grid->gridbands[currref].in_samps);
            free (grid->gridbands[currref].out_lines);
            free (grid->gridbands[currref].out_samps);
            free (grid->gridbands[currref].poly_lines);
            free (grid->gridbands[currref].poly_samps);
            free (grid->gridbands[currref].sattoproj);
            free (grid->gridbands[currref].projtosat);
         }
         return (ERROR);
      }

      grid->gridbands[currref].poly_lines = malloc (grid->gridbands[currref].nzplanes * 
                (grid->gridbands[currref].degree+1) *
                (grid->gridbands[currref].degree+1) * sizeof (double));
      if (grid->gridbands[currref].poly_lines == NULL)
      { 
         /* Free previously malloced pointers. */
         strcpy (ErrMsg, "Error mallocing space for the pointers.");
         xxx_LogStatus ("HXXLIB", __FILE__, __LINE__, ErrMsg);
         free (grid->gridbands[currref].in_lines);
         free (grid->gridbands[currref].in_samps);
         free (grid->gridbands[currref].out_lines);
         free (grid->gridbands[currref].out_samps);
         for (j = 0; j < i; j++)
         {
            currref = bands[j];
            free (grid->gridbands[currref].in_lines);
            free (grid->gridbands[currref].in_samps);
            free (grid->gridbands[currref].out_lines);
            free (grid->gridbands[currref].out_samps);
            free (grid->gridbands[currref].poly_lines);
            free (grid->gridbands[currref].poly_samps);
            free (grid->gridbands[currref].sattoproj);
            free (grid->gridbands[currref].projtosat);
         }
         return (ERROR);
      }

      grid->gridbands[currref].poly_samps = malloc (grid->gridbands[currref].nzplanes *
               (grid->gridbands[currref].degree+1) *
               (grid->gridbands[currref].degree+1) * sizeof (double));
      if (grid->gridbands[currref].poly_samps == NULL)
      { 
         /* Free previously malloced pointers. */
         strcpy (ErrMsg, "Error mallocing space for the pointers.");
         xxx_LogStatus ("HXXLIB", __FILE__, __LINE__, ErrMsg);
         free (grid->gridbands[currref].in_lines);
         free (grid->gridbands[currref].in_samps);
         free (grid->gridbands[currref].out_lines);
         free (grid->gridbands[currref].out_samps);
         free (grid->gridbands[currref].poly_lines);
         for (j = 0; j < i; j++)
         { 
            currref = bands[j];
            free (grid->gridbands[currref].in_lines);
            free (grid->gridbands[currref].in_samps);
            free (grid->gridbands[currref].out_lines);
            free (grid->gridbands[currref].out_samps);
            free (grid->gridbands[currref].poly_lines);
            free (grid->gridbands[currref].poly_samps);
            free (grid->gridbands[currref].sattoproj);
            free (grid->gridbands[currref].projtosat);
         }
         return (ERROR);
      }

      grid->gridbands[currref].sattoproj = malloc (grid->gridbands[currref].nzplanes *
                (grid->gridbands[currref].ngrid_lines-1) *
                (grid->gridbands[currref].ngrid_samps-1) * sizeof (struct COEFFICIENTS));
      if (grid->gridbands[currref].sattoproj == NULL)
      { 
         /* Free previously malloced pointers. */
         strcpy (ErrMsg, "Error mallocing space for pointers.");
         xxx_LogStatus ("HXXLIB", __FILE__, __LINE__, ErrMsg);
         free (grid->gridbands[currref].in_lines);
         free (grid->gridbands[currref].in_samps);
         free (grid->gridbands[currref].out_lines);
         free (grid->gridbands[currref].out_samps);
         free (grid->gridbands[currref].poly_lines);
         free (grid->gridbands[currref].poly_samps);
         for (j = 0; j < i; j++)
         { 
            currref = bands[j];
            free (grid->gridbands[currref].in_lines);
            free (grid->gridbands[currref].in_samps);
            free (grid->gridbands[currref].out_lines);
            free (grid->gridbands[currref].out_samps);
            free (grid->gridbands[currref].poly_lines);
            free (grid->gridbands[currref].poly_samps);
            free (grid->gridbands[currref].sattoproj);
            free (grid->gridbands[currref].projtosat);
         }
         return (ERROR);
      }

      grid->gridbands[currref].projtosat = malloc (grid->gridbands[currref].nzplanes *
                (grid->gridbands[currref].ngrid_lines-1) *
                (grid->gridbands[currref].ngrid_samps-1) * sizeof (struct COEFFICIENTS));
      if (grid->gridbands[currref].projtosat == NULL)
      { 
         /* Free previously malloced pointers. */
         strcpy (ErrMsg, "Error mallocing space for pointers.");
         xxx_LogStatus ("HXXLIB", __FILE__, __LINE__, ErrMsg);
         free (grid->gridbands[currref].in_lines);
         free (grid->gridbands[currref].in_samps);
         free (grid->gridbands[currref].out_lines);
         free (grid->gridbands[currref].out_samps);
         free (grid->gridbands[currref].poly_lines);
         free (grid->gridbands[currref].poly_samps);
         free (grid->gridbands[currref].sattoproj);
         for (j = 0; j < i; j++)
         { 
            currref = bands[j];
            free (grid->gridbands[currref].in_lines);
            free (grid->gridbands[currref].in_samps);
            free (grid->gridbands[currref].out_lines);
            free (grid->gridbands[currref].out_samps);
            free (grid->gridbands[currref].poly_lines);
            free (grid->gridbands[currref].poly_samps);
            free (grid->gridbands[currref].sattoproj);
            free (grid->gridbands[currref].projtosat);
         }
         return (ERROR);
      }
   } /* Bands loop */

   return (SUCCESS);
}
