/******************************************************************************
NAME:		hxx_open_1g_image

PURPOSE:	Open the specified 1G image.

RETURN VALUE: 
Type = HYPHDFIO *
Value           Description
-----           -----------
hdfid		HDF structure used to access the image
NULL     	Error occurred; image not open

NOTES:		The 1G naming convention is as follows:
		The HDF directory file name is specified by the user,
		  convention is root_HDF.xxx
******************************************************************************/
#include <stdio.h>
#include <stdlib.h>		/* for malloc */
#include "xxx_Types.h"		/* ERROR/SUCCESS */
#include "xxx_LogStatus.h"	/* for LogStatus error reporting */
#include "hxx_hdf_io.h"		/* HDF i/o defines and prototypes */
#include "xxx_utility.h"	/* for xxx_splitname */

void hxx_cleanup_1g_image(HYPHDFIO *hdfid, char *msg);

HYPHDFIO *hxx_open_1g_image (
  int *nb,
  int *nl,
  int *ns,
  DataType *dtype,
  const char *image_filename,
  AccessMode access_mode)
{
HYPHDFIO 	*hdfid=NULL;		/* HDF I/O data			*/

intn	hdf_access_mode;	/* HDF access mode code		*/
intn	hdf_status;		/* HDF function return status	*/
int32	group_ref,		/* HDF group reference number	*/
	data_type,		/* data type for 1G images	*/
	rank=3,			/* 3-d SDS (all bands)		*/
	dim_sizes[MAX_VAR_DIMS],/* dimension sizes (nl, ns)	*/
	nattrs;			/* number of attributes		*/

int	return_info,		/* flag to return info to the user */
	status,			/* return of status from function */
	sds;			/* sds counter			*/

char	group_access_mode[2];	/* access mode code for Vgroup	*/
char	str_mode[8];		/* string of file access mode	*/
char	msg[STRLEN];		/* error message string		*/
char    dir[STRLEN];		/* directory of the 1G image file*/
char    root[STRLEN];		/* root name of the 1G image file*/
char    ext[STRLEN];		/* extension of the 1G image file*/
char    dirspec[STRLEN];	/* search directories for external files*/

/*
 * Create the structure to track HDF info
 * The structure contains an array of band SDS id's which are indexed by
 * the enumerated band numbers
 */
hdfid = malloc(sizeof(HYPHDFIO));
if (hdfid == NULL)
    {
    hxx_cleanup_1g_image(hdfid, "Error allocating memory");
    return(NULL);
    }

/*
 * Initialize the sds id's
 * Get the root filename for the hdf_dir_name
 */
xxx_splitname(image_filename, dir, root, ext);
status = sprintf(hdfid->hdf_dir_name,"%s.%s",root,ext);
if (status < 1)
   {
   xxx_LogStatus("HXXLIB",__FILE__,__LINE__,"Error in sprintf");
   return(NULL);
   }
hdfid->file_id = -1;
hdfid->group_id = -1;
hdfid->sd_id = -1;
hdfid->sds_id = -1;
strcpy(hdfid->sds_name,"");

/*
 * Do different types of opens for different access modes
 */
if ((access_mode == READ) || (access_mode == UPDATE))
    {
   /*
    * Initialize info returned to the user 
    */
    return_info = FALSE;
    if (*nb == 0)
	{
	return_info = TRUE;
        *nl = 0;
        *ns = 0;
	}

   /*
    * Open the HDF file
    */
    if (access_mode == READ)
	{
        hdf_access_mode = DFACC_READ;
        strcpy(group_access_mode,"r");
        strcpy(str_mode,"READ");
	}
    else /* UPDATE */
	{
        hdf_access_mode = DFACC_RDWR;
        strcpy(group_access_mode,"w");
        strcpy(str_mode,"UPDATE");
	}
    hdfid->file_id = Hopen(image_filename, hdf_access_mode, 0);
    if (hdfid->file_id == FAIL)
        {
        sprintf(msg,"Error opening file %s for requested mode %s", image_filename,str_mode);
	hxx_cleanup_1g_image(hdfid, msg);
    	return(NULL);
        }

   /*
    * Set up the search path for external elements  (HXsetdir) and 
    * path for creating new external elements (HXsetcreatedir)
    */
    xxx_splitname(image_filename, dir, root, ext);
    if (dir[0] != 0)
	{
	/* directory specified */
        if (access_mode == UPDATE)
	    HXsetcreatedir(dir);
        status = sprintf(dirspec,".|%s",dir);
        if (status < 1)
           {
           xxx_LogStatus("HXXLIB",__FILE__,__LINE__,"Error in sprintf");
           return(NULL);
           }
	}
    else
	{
	/* no directory specified; just use CWD  */
        if (access_mode == UPDATE)
	    HXsetcreatedir(".");
        status = sprintf(dirspec,".");
        if (status < 1)
           {
           xxx_LogStatus("HXXLIB",__FILE__,__LINE__,"Error in sprintf");
           return(NULL);
           }
	}
    /* Save the external file directory so the read function can 
       search it using HXsetdir */
    strcpy(hdfid->extdir, dirspec);

   /*
    * Initialize the V interface
    */
    hdf_status = Vstart(hdfid->file_id);
    if (hdf_status == FAIL)
        {
        sprintf(msg,"Error initiating HDF for file %s",image_filename);
	hxx_cleanup_1g_image(hdfid, msg);
	return(NULL);
        }

   /*
    * Locate & attach to the 1G image group
    */
    group_ref = Vfindclass(hdfid->file_id, IMAGE_CLASS);
    if (group_ref != 0)
	{
	hdfid->group_id = Vattach(hdfid->file_id, group_ref, group_access_mode);
	if (hdfid->group_id == FAIL)
	    {
	    hxx_cleanup_1g_image(hdfid,"Error accessing image group");
	    return(NULL);
	    }
	}
    else
	{
       /* 
	* If update mode, create the group and set the class if group not found
	* If read mode, non-existent group is an error
	*/
	if (access_mode == UPDATE)
	    {
    	    group_ref = -1;
    	    hdfid->group_id = Vattach(hdfid->file_id, group_ref, group_access_mode);
    	    if (hdfid->group_id == FAIL)
	        {
	        hxx_cleanup_1g_image(hdfid, "Error creating image group");
	        return(NULL);
	        }

    	    hdf_status = Vsetclass(hdfid->group_id, IMAGE_CLASS);
    	    if (hdf_status == FAIL)
		{
		hxx_cleanup_1g_image(hdfid, "Error creating image group");
		return(NULL);
		}
	    }
	else
	    {
	    hxx_cleanup_1g_image(hdfid, "Error accessing image group");
	    return(NULL);
	    }
        }

   /*
    * Get the SDS interface id
    */
    hdfid->sd_id = SDstart(image_filename, hdf_access_mode);
    if (hdfid->sd_id == FAIL)
	{
	hxx_cleanup_1g_image(hdfid, "Error initializing HDF");
	return(NULL);
	}

   /*
    * Now open the bands 
    */
    if (return_info)  
	{
        hdfid->sds_id = hxx_open_1g_band(hdfid);
        if (hdfid->sds_id != FAIL)
            {
            hdf_status = SDgetinfo(hdfid->sds_id, hdfid->sds_name, &rank, dim_sizes, 
                                   &data_type, &nattrs);
            if (hdf_status == FAIL)
                {
                sprintf(msg,"Error getting %s info",hdfid->sds_name);
                hxx_cleanup_1g_image(hdfid, msg);
                return(NULL);
                }

            *nb = dim_sizes[0];
            *nl = dim_sizes[1];
            *ns = dim_sizes[2];

            if (data_type == DFNT_UINT8) *dtype = BYTE;
            else if (data_type == DFNT_INT16) *dtype = I2;
            else if (data_type == DFNT_UINT16) *dtype = UI2;
            else if (data_type == DFNT_INT32) *dtype = I4;
            else if (data_type == DFNT_UINT32) *dtype = UI4;
            else if (data_type == DFNT_FLOAT32) *dtype = R4;
            else if (data_type == DFNT_FLOAT64) *dtype = R8;
            else *dtype = type_error;

            } /* found a valid band */
	} /* if returning whatever is there */
    else
	{
        hdfid->sds_id = hxx_open_1g_band(hdfid);
        if (hdfid->sds_id == FAIL)
           {
          /*
           * On update if the band isn't there, create it
           * On read it is an error if the band isn't there
           */
           if (access_mode == READ)
              {
              sprintf(msg,"Error locating %s", hdfid->sds_name);
              hxx_cleanup_1g_image(hdfid, msg);
              return(NULL);
              }
           else
	      {
	      /* 
	       * Create the band if it's not there 
               */
              if (hxx_create_1g_band(*nb, *nl, *ns, *dtype, hdfid) != SUCCESS)
                 {
                 sprintf(msg,"Error creating SDS %s",hdfid->sds_name);
                 hxx_cleanup_1g_image(hdfid, msg);
                 return(NULL);
                 }

              } /* adding band SDS on UPDATE */
           } /* band not found */
         else
           {
           hdf_status = SDgetinfo(hdfid->sds_id, hdfid->sds_name, &rank, dim_sizes, 
                                  &data_type, &nattrs);
           if (hdf_status == FAIL)
              {
              sprintf(msg,"Error getting %s info",hdfid->sds_name);
              hxx_cleanup_1g_image(hdfid, msg);
              return(NULL);
              }

           *nb = dim_sizes[0];
	   *nl = dim_sizes[1];
	   *ns = dim_sizes[2];

	   if (data_type == DFNT_UINT8) *dtype = BYTE;
           else if (data_type == DFNT_INT16) *dtype = I2;
           else if (data_type == DFNT_UINT16) *dtype = UI2;
           else if (data_type == DFNT_INT32) *dtype = I4;
           else if (data_type == DFNT_UINT32) *dtype = UI4;
           else if (data_type == DFNT_FLOAT32) *dtype = R4;
           else if (data_type == DFNT_FLOAT64) *dtype = R8;
           else *dtype = type_error;

           } /* band found */
       } /* opening user-specified bands */
    } /* read/update */
else if (access_mode == WRITE)
    {
   /*
    * Open the HDF file
    */
    hdf_access_mode = DFACC_CREATE;
    strcpy(str_mode,"WRITE");
    hdfid->file_id = Hopen(image_filename, hdf_access_mode, 0);
    if (hdfid->file_id == FAIL)
        {
        sprintf(msg,"Error opening file %s for requested mode %s", image_filename,str_mode);
	hxx_cleanup_1g_image(hdfid, msg);
	return(NULL);
        }

   /*
    * Set up the external directory path
    * and set up a band file name without directory so it will
    * be created in HXsetcreatedir
    */
    xxx_splitname(image_filename, dir, root, ext);
    if (dir[0] != 0)
       HXsetcreatedir(dir);
    else
       HXsetcreatedir(".");

   /*
    * Initialize the V interface
    */
    hdf_status = Vstart(hdfid->file_id);
    if (hdf_status == FAIL)
        {
        sprintf(msg,"Error initiating HDF for file %s",image_filename);
	hxx_cleanup_1g_image(hdfid, msg);
	return(NULL);
        }

   /*
    * Create the Vgroup containing the bands
    */
    group_ref = -1;
    hdfid->group_id = Vattach(hdfid->file_id, group_ref, "w");
    if (hdfid->group_id == FAIL)
	{
	hxx_cleanup_1g_image(hdfid, "Error creating image group");
	return(NULL);
	}

   /* 
    * Set the group class name
    */
    hdf_status = Vsetclass(hdfid->group_id, IMAGE_CLASS);
    if (hdf_status == FAIL)
	{
	hxx_cleanup_1g_image(hdfid, "Error creating image group");
	return(NULL);
	}

   /*
    * Get the SDS interface id
    * Change the access to read/write because the file is now created
    */
    hdf_access_mode = DFACC_RDWR;
    hdfid->sd_id = SDstart(image_filename, hdf_access_mode);
    if (hdfid->sd_id == FAIL)
	{
	hxx_cleanup_1g_image(hdfid, "Error initializing HDF");
	return(NULL);
	}

   /*
    * Create a SDS for all bands
    */
     if (hxx_create_1g_band(*nb, *nl, *ns, *dtype, hdfid) != SUCCESS)
        {
        sprintf(msg,"Error creating SDS %s", hdfid->sds_name);
        hxx_cleanup_1g_image(hdfid, msg);
        return(NULL);
        }

    } /* write */
else
    {
    xxx_LogStatus("HXXLIB",__FILE__,__LINE__,"Invalid image access mode");
    return(NULL);
    }

return(hdfid);
}


/******************************************************************************
NAME:		hxx_cleanup_1g_image

PURPOSE:	close the 1g image, shutting down all HDF accesses and
		interfaces

******************************************************************************/

void hxx_cleanup_1g_image(HYPHDFIO *hdfid, char *msg)
{

if (hdfid)
    {
    if (hdfid->sds_id != -1)
        SDendaccess(hdfid->sds_id);

    if (hdfid->sd_id != -1) SDend(hdfid->sd_id);
    if (hdfid->group_id != -1) Vdetach(hdfid->group_id);
    if (hdfid->file_id != -1) 
        {
        Vend(hdfid->file_id);
        Hclose(hdfid->file_id);
        }

    free(hdfid);
    }

xxx_LogStatus("HXXLIB",__FILE__,__LINE__,msg);
}
