/******************************************************************************
*                              PACOR II SOFTWARE
*                      Property of the U.S. Government
*                              NASA/GSFC/Code 560
*******************************************************************************
*
*    UNIT NAME:
*	xxx_FileLock.c
*
*    PURPOSE:
*	To lock/unlock files.
*
*    INVOCATION METHOD:
*	Status = xxx_FileLock(fd, command, type, offset, whence, length)
*
*
*    RETURNED VALUE:
*
*	TYPE = int
*
*	VALUE 		DESCRIPTION
*       ------------	----------------------------
*	>= 0		Successful
*	-1		On failure, fcntl() returns -1 and sets errno to indicate the error.
*
*
*    ARGUMENT LIST:
*
*       NAME 		TYPE		USE	DESCRIPTION
*       ------------	---------	---	----------------------------
*	fd		int		I	File descriptor
*	command		int		I	fcntl commands:
*						  F_SETLK or F_SETLKW
*	type		short		I	Type of lock desired:  
*						  F_RDLCK, FWRLCK, or F_UNLOCK
*	offset		off_t		I	Byte offset, relative to l_whence
*	whence		short		I	SEEK_SET, SEEK_CUR, SEEK_END
*	length		off_t		I	Number of bytes (0 means to EOF)
*
*
*    EXTERNAL FILE/RECORD REFERENCES:
*
*       NAME		USE	DESCRIPTION
*       ----		---	-------------------------------------
*	xxx_FileLock	I	Defines prototype and lock macros
*
* 
*    EXTERNAL VARIABLES:
*
*       SOURCE	NAME	TYPE	USE	DESCRIPTION
*       ------	----	----	---	--------------------
*	none
*
*
*    EXTERNAL UNIT/MACRO/PROCEDURE/FUNCTION REFERENCES:
*
*       NAME		PURPOSE
*       ------------	----------------------------------------------
*	none
*
*
*    ABNORMAL TERMINATION CONDITIONS, ERROR MESSAGES:
*	Lock request failed.
*
*
*    ASSUMPTIONS, CONSTRAINTS, RESTRICTIONS:
*	none
*
*
*    CHANGE HISTORY:
*
*				  BUILD/
*    	AUTHOR	      CHANGE-ID	  RELEASE      DATE     DESCRIPTION OF CHANGE
*    	-----------   ---------	  --------   --------   -----------------------
*	J. Kerich     N/A         R1B2       10/15/93   Original Implementation
*	J. kerich                 R1.0       03/24/97   Ported to IAS
*
*
*    NOTES:
*	1) On a no-wait error, check for EACCES or EAGAIN to see if file is locked 
*	   already.
*
*	2) The read-lock will prevent another process from write-locking the file.
*	   This does not stop another process from reading from the file.
*
*	3) There is a posibility of deadlocking if you lock a file and then
*	   try to lock another file in the wait mode that is already lock.  So be
*	   careful.
*
******************************************************************************/
/******************************************************************************
UNIT xxx_FileLock PDL
-------------------------------------------------------------------------------

1	Build file lock request
	Call fcntl() to do request
	RETURN status for request

-------------------------------------------------------------------------------
END UNIT xxx_FileLock PDL
******************************************************************************/

/* System headers */
#include <fcntl.h>

/* Local headers */
#include <xxx_Types.h>

/* External functions */
#include <xxx_FileLock.h>

/* Main */

int xxx_FileLock(int fd, int command, short type, off_t offset, short whence, off_t length)
{
	struct flock lock;

	lock.l_type = type;	/* F_RDLCK, FWRLCK, F_UNLOCK */
	lock.l_start = offset;	/* byte offset, relative to l_whence */
	lock.l_whence = whence;	/* SEEK_SET, SEEK_CUR, SEEK_END */
	lock.l_len = length;	/* number of bytes (0 means to EOF */

	return ( fcntl(fd, command, &lock) );
}	
