/******************************************************************************
                Landsat 7 Image Assessment System (IAS) Software
                        Property of the U.S. Government
                            NASA / GSFC / Code 510
*******************************************************************************

UNIT NAME:  xxx_GetDirFiles.c

PURPOSE:  Returns a directory listing of the target directory.

INVOCATION METHOD:

        pointer = xxx_GetDirFiles(p_Count, p_PathName, p_Key, p_ErrorMessage)

RETURN VALUE:
Type = xxx_DirList_TYPE *

Value          Description
-------------- ----------------------------------------------------------------
positive       Pointer to the xxx_DirList_TYPE
NULL           Failure

ARGUMENT LIST:
Name             Type               Use      Description
---------------- ------------------ ---      ---------------------------------------
p_Count           int *             O        Number of rows in p_PartionList
p_PathName        char *            I        Directory path name
p_Key             char *            I        List options like "*"
p_ErrorMessage    char *            O        Error message

EXTERNAL FILE/RECORD/DBTABLE REFERENCES:
Name                      Use Description
------------------------- --- -------------------------------------------------
<Temp file>               I/O Temp file in /usr/tmp

EXTERNAL VARIABLES:
Source          Name                Type            Use Description
--------------- ------------------- --------------- --- -----------------------
xxx_Types.h     FALSE                int            I   Defined parameter

EXTERNAL UNIT/MACRO/PROCEDURE/FUNCTION REFERENCES:
Name                  Description
--------------------- -------------------------------------------------
xxx_CloseUnmap        Unmap and close a file
xxx_GetTempName       Generation temp name
xxx_OpenMap           Open and map a file
xxx_RecursiveDeletion Delete the file/directory recursively
XXX_RIGHT_SHIFT       Get system() status

ABNORMAL TERMINATION CONDITIONS, ERROR MESSAGES:
I/O errors
memory errors

ASSUMPTIONS, CONSTRAINTS, RESTRICTIONS:
Temp file will be deleted by cron

DEVELOPMENT HISTORY:
Author            Change-ID   Release Date       Description of Change
----------------- ----------- ------- ---------- ------------------------------
J. kerich         N/A         R1.0    04/29/97   Original Implementation

NOTES:
Key option can be for example:   "*", ".?*", or "L7*"

Unit xxx_GetDirFiles PDL:

        Call xxx_GetTempName() to generate file name
1	IF error THEN
	        Build error message in p_ErrorMessage
	        RETURN NULL
2	ENDIF

	Build list command using p_PathName and p_Key
	Call system() to execute the command
3	IF I/O error THEN
	        Build error message in p_ErrorMessage
		ABORT to ERROR_PROC
4	ENDIF

        Call xxx_OpenMap() to open temp file
5	IF I/O error THEN
	        Build error message in p_ErrorMessage
		ABORT to ERROR_PROC
6	ENDIF

7	DOWHILE more lines to read in
	        Resize the listing memory for new line
8		IF memory error THEN
	                Build error message in p_ErrorMessage
	                ABORT to ERROR_PROC
9		ENDIF
		Save line in list
10	ENDDO

	Call xxx_CloseUnmap() to close temp file

	Call xxx_RecursiveDeletion() to delete the temp file
11	IF error deleting THEN
        	RETURN NULL
12	ENDIF

	RETURN Directory List

ERROR_PROC:
	Call xxx_RecursiveDeletion() to delete the temp file
        RETURN NULL

End Unit xxx_GetDirFiles
******************************************************************************/

/* System headers */
#include <fcntl.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>

/* Local headers */
#include <xxx_Types.h>

/* External functions */
#include <xdb_Defines.h>
#include <xxx_OpenMap.h>
#include <xxx_CloseUnmap.h>
#include <xxx_GetTempName.h>
#include <xxx_GetDirFiles.h>
#include <xxx_RecursiveDeletion.h>

xxx_DirList_TYPE *xxx_GetDirFiles(int *p_Count, char *p_PathName, char *p_Key, char *p_ErrorMessage)
{
	char    Command[400];
	int     status;
	int     FileId;                 /* file descriptor for CPF file */
	long    FileLength;             /* size of CPF file */

	char    *p_TempName;
	xxx_DirList_TYPE *p_Listing = NULL;
	xxx_DirList_TYPE *p_TempPtr;
	char    ErrorMessage[XDB_ORAERRMSGLEN];
	char    line[MAXPATHLEN];
	char    *p_Ptr;
	char cwd[MAXPATHLEN+1];  /* lpgslite: temp debug */

	*p_Count = 0;

	if ((p_TempName = xxx_GetTempName("/usr/tmp", "lsTmp")) == NULL)
	{
		(void)strcpy(p_ErrorMessage, "Can't generate temporary path name\n");
		return(NULL);
	}

	if (p_PathName[strlen(p_PathName)-1] != '/')
	{
		(void)sprintf(Command, "ls %s/%s 1>%s 2>/dev/null", p_PathName, p_Key, p_TempName);
	}
	else
	{
		(void)sprintf(Command, "ls %s%s 1>%s 2>/dev/null", p_PathName, p_Key, p_TempName);
	}

	if ((XXX_RIGHT_SHIFT(status = system(Command))) != 0)
	{
	  getcwd(cwd,MAXPATHLEN);  /* lpgslite: temp debug */
		(void)sprintf(p_ErrorMessage, "%s: bad command, status: %d, quitting... (cwd=%s)\n", Command, status, cwd);	
	goto ERROR_PROC;
	}

	if (xxx_OpenMap(p_TempName, &FileId, O_RDONLY, (void *)NULL, -1, 
	    &FileLength, (long *)NULL, FALSE, FALSE, p_ErrorMessage))
	{
		goto ERROR_PROC;
	}

	p_Ptr = line;
	while(read(FileId, p_Ptr, 1) == 1)
	{
		if (*p_Ptr == '\0' || *p_Ptr == '\n')
		{
			*p_Ptr = '\0';

			(*p_Count)++;
			if ((p_TempPtr = (xxx_DirList_TYPE *)realloc(p_Listing, (*p_Count)*sizeof(xxx_DirList_TYPE))) == NULL)
			{
				(void)strcpy(p_ErrorMessage, "Can't realloc memeory");
				free(p_Listing);
				goto ERROR_PROC;
			}
			p_Listing = p_TempPtr;

			(void)strcpy(p_Listing[(*p_Count)-1].FilePathName, line);

			p_Ptr = line;
		}
		else
		{
		        p_Ptr++;
		}
	}

	(void)xxx_CloseUnmap(FileId, (void *)NULL, FileLength, FALSE, FALSE, p_ErrorMessage);

	if (xxx_RecursiveDeletion(p_TempName, p_ErrorMessage))
	{
		free(p_Listing);
		goto ERROR_PROC2;
        }

	free(p_TempName);

	return (p_Listing);


ERROR_PROC:
	(void)xxx_RecursiveDeletion(p_TempName, ErrorMessage);
ERROR_PROC2:
	free(p_TempName);
	*p_Count = 0;
	return (NULL);
}
