/******************************************************************************
                Landsat 7 Image Assessment System (IAS) Software
                        Property of the U.S. Government
                            NASA / GSFC / Code 510
*******************************************************************************

UNIT NAME: xxx_OpenODL.c

PURPOSE: Open and parse the ODL file.

INVOCATION METHOD:
    Pointer = xxx_OpenODL(p_ODLFile, ODLType, p_ErrorMessage)

RETURN VALUE:
    Type = OBJDESC *

Value           Description
--------------- ----------------------------------------------------------------
positive        Pointer to ODL tree 
NULL            Failure

ARGUMENT LIST:
Name            Type            Use Description
--------------- --------------  --- ---------------------------------------
p_ODLFile       char *           I  If a normal odl file is to be opened, the path is 
                                    to the ODL file otherwise the path is to the 
				    L0R directory file.
ODLType         xxx_EDCOdl_TYPE  I  Indicates if a ODL file to read or 
                                    a normal odl file (ie.xxx_NoEDCMeta). ..
p_ErrorMessage  char *           O  Error message

EXTERNAL FILE/RECORD/DBTABLE REFERENCES:
Name                    Use         Description
----------------------- ---  -------------------------------------------------
none

EXTERNAL VARIABLES:
Source            Name                  Type            Use Description
---------------   -------------------   --------------- --- --------------------
OdlPrintMessage   ODLErrorMessage       char *          I   ODL error message
xxx_Errno.c       xxx_errno             long            O   Error number 

EXTERNAL UNIT/MACRO/PROCEDURE/FUNCTION REFERENCES:
Name                    Description
----------------------- -------------------------------------------------
ODLParseLabelFile       Read in and parse ODL file
xxx_FindDirectory       Find the directory for the file path
xxx_GetDirFiles         List the directory
XXX_ARRAY_SIZE          Return array size
xxx_GetTempName         Generate temp name
xxx_RecursiveDeletion   Delete the file/directory recursively
OdlFreeTree             To free the ODL tree

ABNORMAL TERMINATION CONDITIONS, ERROR MESSAGES:
ODL error
Can't find product meta data file

ASSUMPTIONS, CONSTRAINTS, RESTRICTIONS:
none

DEVELOPMENT HISTORY:
Author            Change-ID   Release Date       Description of Change
----------------- ----------- ------- ---------- -----------------------------
T. Raheja         N/A         R1.0    03/97      Original Implementation
j. kerich         N/A         R1.0    05/01/97   Add EDC Meta data logic
j. kerich         IAS970011   R2.0    11/10/97   Change L7*_CPF* to L7CPF*
j. kerich         IASI970057  R2.0    12/05/97   Delete ODL error file if ODL file 
						 was successfully opened
j. kerich         IASI980424  R3.1    08/11/98   Add Errno logic 
j. kerich         IASI990538  R3.2    11/23/98   Add L1 file logic and ODL_NOEXPAND instead of NULL

NOTES:
The MTA files are shared by L1R and L0R products.  If the user requested a LPS MTA 1 or 2 file, the code first sees
if the L0R version is there (Only the eas readMetaL1 dumper reads L1R products).  If a match is not found, the L1R version is tried.  
To make sure the the L0R naming convertion is not confused with the L1R naming convertion, the L0R pattern match makes sure 
the 10 character in the file name is NOT an '_' character as it is in the L1R naming convertion, but a 0->9 number.

Unit xxx_OpenODL PDL:

1       IF ODLType is not xxx_NoEDCMeta THEN
	        Call xxx_FindDirectory() to find the product meta data directory
		Find the product metadata extention
		Call xxx_GetDirFiles() to get the product meta data path
2	        IF product meta data file was not found THEN
			Build error message in p_ErrorMessage
			Set xxx_errno to XXX_E_FILE_NOT_FOUND
		        RETURN NULL
3		ENDIF
4	ELSE
	        Use p_ODLFile as ODL path
	ENDIF

        Call xxx_GetTempName() to generate ODL error file name
5       IF error THEN
	        Build error message in p_ErrorMessage
		Set xxx_errno to XXX_E_NO_MEMORY
	        RETURN NULL
6       ENDIF

	Call OdlParseLabelFile() to open and parse the ODL file
7       IF error THEN
                Build error message in p_ErrorMessage
		Set xxx_errno to XXX_E_ODL_SYNTAX
                RETURN NULL
8       ENDIF
      
9	IF warning is set in ODLErrorMessage THEN 
                Build error message in p_ErrorMessage
		Call OdlFreeTree() to free the tree
		Set xxx_errno to XXX_E_ODL_SYNTAX
                RETURN NULL
10	ENDIF

        Call xxx_RecursiveDeletion() to delete the ODL error file

        RETURN ODL pointer

End Unit xxx_OpenODL
******************************************************************************/

/* System Headers */
#include <stdio.h>
#include <string.h>
#include <sys/param.h>

/* Local Headers */
#include <lablib3.h>                 /* prototypes for the ODL functions */
#include <xxx_Errno.h>
#include <xxx_GetDirFiles.h>
#include <xxx_Imagery.h>
#include <xxx_ODL.h>                 /* prototype for the xxx ODL functions */
#include <xxx_Types.h>               
#include <xxx_GetTempName.h>
#include <xxx_RecursiveDeletion.h>

extern char ODLErrorMessage[];       /* External Variables */

OBJDESC *xxx_OpenODL (char *p_ODLFile, xxx_EDCOdl_TYPE ODLType, char *p_ErrorMessage)
{
	OBJDESC *p_lp= NULL;
	char ODLPathName[MAXPATHLEN];
	char ODLErrorFile[MAXPATHLEN];
	char *p_TempName;
	int Count;
	int i;
	xxx_DirList_TYPE *p_DirectoryList;

	typedef struct xxx_MetaIndex_TYPE {
		xxx_EDCOdl_TYPE  Want;
		char ProductMeta[24];
	} xxx_MetaIndex_TYPE;

	xxx_MetaIndex_TYPE ProductMetaData[] = {
		{xxx_EDCMeta,  "L7*MTP*"},
		{xxx_LPSMeta1, "L7????1??[0-9]*_MTA.*"},	/* Make sure the 10 character is not '_' as in L1R product */
		{xxx_LPSMeta1, "L71*_MTA.L1R"},
		{xxx_LPSMeta2, "L7????2??[0-9]*_MTA.*"},	/* Make sure the 10 character is not '_' as in L1R product */
		{xxx_LPSMeta2, "L72*_MTA.L1R"},
		{xxx_CPFMeta,  "L7CPF*"},
		{xxx_LPGSMeta, "L7*MTL*"},
	};

	ODLErrorMessage[0] = '\0';

	if (ODLType != xxx_NoEDCMeta)
	{
		xxx_FindDirectory(ODLPathName, p_ODLFile);
		for(i = 0;  i < XXX_ARRAY_SIZE(ProductMetaData) && ProductMetaData[i].Want != ODLType;  i++);
		p_DirectoryList = xxx_GetDirFiles(&Count, ODLPathName, ProductMetaData[i].ProductMeta,  p_ErrorMessage);

		/* Check for other product type */
		if (p_DirectoryList == NULL && (ODLType == xxx_LPSMeta1 || ODLType == xxx_LPSMeta2))
		{
			i++;
			p_DirectoryList = xxx_GetDirFiles(&Count, ODLPathName, ProductMetaData[i].ProductMeta,  p_ErrorMessage);
		}

		if (p_DirectoryList == NULL)
		{
			xxx_errno = XXX_E_FILE_NOT_FOUND;
			return NULL;
		}
		(void)strcpy(ODLPathName, p_DirectoryList[0].FilePathName);
		free (p_DirectoryList);
	}
	else
	{
		(void)strcpy(ODLPathName, p_ODLFile);
	}

	/* Build ODL error file name */
	if ( (p_TempName =  xxx_GetTempName("/usr/tmp", "ODLEr")) == NULL)
	{
		(void)strcpy(p_ErrorMessage, "Can't generate temporary path name\n");
		xxx_errno = XXX_E_NO_MEMORY;
		return(NULL);
	}
	(void)sprintf(ODLErrorFile,"%s.dat", p_TempName);
	free(p_TempName);

	/*   open and parse ODL file - error messages to ODLPathName */
	if ((p_lp = OdlParseLabelFile(ODLPathName, ODLErrorFile, ODL_NOEXPAND, FALSE)) == NULL)
	{
		(void)sprintf(p_ErrorMessage, "ODL Error on File: %s:  %s", ODLPathName, ODLErrorMessage);
		xxx_errno = XXX_E_ODL_SYNTAX;
		return NULL;
	}

	/* Check of ODL syntax errors */
	if (ODLErrorMessage[0] != '\0')
	{
		(void)sprintf(p_ErrorMessage,"%s: check out %s", ODLErrorMessage, ODLErrorFile);
		OdlFreeTree(p_lp);
		xxx_errno = XXX_E_ODL_SYNTAX;
		return NULL;
	}

	/* Delete the ODL error log file */
        (void)xxx_RecursiveDeletion(ODLErrorFile, p_ErrorMessage);

	return p_lp;
}
