/******************************************************************************
UNIT NAME:      xxx_read_ali_l0r_cal.c 

PURPOSE: Reads a Level 0R ALI (from EO-1 instrument) calibration data set.  All
	 data is put in the structure xxx_ali_level0r.h.  It is up to the caller of
	 this routine to access what data is needed by their routine and then
	 to free up the memory associated with the structure xxx_ali_level0r.h.        

*******************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <limits.h>
#include <unistd.h>
#include <sys/types.h>
#include <stddef.h>
#include "xxx_Types.h"
#include "xxx_LogStatus.h"
#include "xxx_ali_level0r.h"

int xxx_read_ali_l0r_cal(
int32 sd_id,                    /* I: SD identifier		*/
int32 sds_index,                /* I: SDS index		*/
L0R_CAL *cal)                   /* O: Structure containing L1R data	*/
{
   char emsg[STRLEN];		/* Error message buffer			*/
   char progname[STRLEN];	/* Program name				*/
   int32 sds_id;                /* SDS file ID                          */
   int32 start[3];		/* Starting dims of SDS "slab"		*/
   int32 edges[3];		/* Edge dims of SDS "slab"		*/
   int32 stride[3];		/* Increment dims of SDS "slab"		*/
   int32 size;			/* Size needed for array allocation	*/ 
   int32 attr_index;		/* Index of attributes			*/

   sprintf(progname,"READ_L0R_CAL");

   /*************************
    * Read calibration data *
    *************************/
   memset( cal, 0, sizeof(L0R_CAL) );

   /* Select SDS index for calibration data
      ------------------------------------- */
   sds_id = SDselect(sd_id, sds_index);

   /* Get SDS info
      ------------ */
   if( SDgetinfo(sds_id,
                 (char *)cal->name,
		 &cal->number_of_dims, 
                 cal->dim_sizes,
		 &cal->HDF_type, 
                 &cal->number_of_attributes )
       == FAIL )
   {
      sprintf(emsg,"Could not get SDS info");
      xxx_LogStatus(progname,__FILE__,__LINE__,emsg);
      return(ERROR);
   }

   /* Set up dimensions of SDS "slab".
      -------------------------------*/
   start[0]  = 0; /* Band start is 0-relative */
   start[1]  = 0;
   start[2]  = 0;

   stride[0] = 1;
   stride[1] = 1;
   stride[2] = 1;

   edges[0]  = cal->dim_sizes[0];    
   edges[1]  = cal->dim_sizes[1];
   edges[2]  = cal->dim_sizes[2];

   size = edges[0] * edges[1] * edges[2];
   cal->data = (int16 *)malloc( sizeof(int16) * size );
   if( (cal->data) == NULL )
   {
      sprintf(emsg,"Error allocating data buffer.");
      xxx_LogStatus(progname,__FILE__,__LINE__,emsg);
      return(ERROR);
   }

   /* Read CAL data into buffer
   -------------------------------*/
   if( SDreaddata(sds_id, start, stride, edges, (VOIDP)cal->data) == FAIL )
   {
      sprintf(emsg,"Could not read SDS data");
      xxx_LogStatus(progname,__FILE__,__LINE__,emsg);
      return(ERROR);
   }

   cal->attributes.valid_map[0] = INVALID;
   if ( (attr_index = SDfindattr( sds_id, "ALI Sensor" )) != FAIL )
   {
     if( SDreadattr(sds_id, attr_index, cal->attributes.ali_sensor) == FAIL )
     {
       sprintf(emsg,"Could not read attribute.");
       xxx_LogStatus(progname,__FILE__,__LINE__,emsg);
       return(ERROR);
     }
     cal->attributes.valid_map[0] = VALID;
   }
   cal->attributes.valid_map[1] = INVALID;
   if ( (attr_index = SDfindattr( sds_id, "Dataset Type" )) != FAIL )
   {
     if( SDreadattr(sds_id, attr_index, cal->attributes.dataset_type) == FAIL )
     {
       sprintf(emsg,"Could not read attribute.");
       xxx_LogStatus(progname,__FILE__,__LINE__,emsg);
       return(ERROR);
     }
     cal->attributes.valid_map[1] = VALID;
   }
   cal->attributes.valid_map[2] = INVALID;
   if ( (attr_index = SDfindattr( sds_id, "Integration Time" )) != FAIL )
   {
     if( SDreadattr(sds_id, attr_index, &cal->attributes.integration_time) == FAIL )
     {
       sprintf(emsg,"Could not read attribute.");
       xxx_LogStatus(progname,__FILE__,__LINE__,emsg);
       return(ERROR);
     }
     cal->attributes.valid_map[2] = VALID;
   }
   cal->attributes.valid_map[3] = INVALID;
   if ( (attr_index = SDfindattr( sds_id, "Frame Rate" )) != FAIL )
   {
     if( SDreadattr(sds_id, attr_index, &cal->attributes.frame_rate) == FAIL )
     {
       sprintf(emsg,"Could not read attribute.");
       xxx_LogStatus(progname,__FILE__,__LINE__,emsg);
       return(ERROR);
     }
     cal->attributes.valid_map[3] = VALID;
   }
   cal->attributes.valid_map[4] = INVALID;
   if ( (attr_index = SDfindattr( sds_id, "Data Gate Open Time" )) != FAIL )
   {
     if( SDreadattr(sds_id, attr_index, &cal->attributes.data_gate_open_time) == FAIL )
     {
       sprintf(emsg,"Could not read attribute.");
       xxx_LogStatus(progname,__FILE__,__LINE__,emsg);
       return(ERROR);
     }
     cal->attributes.valid_map[4] = VALID;
   }
   cal->attributes.valid_map[5] = INVALID;
   if ( (attr_index = SDfindattr( sds_id, "Data Gate Close Time" )) != FAIL )
   {
     if( SDreadattr(sds_id, attr_index, &cal->attributes.data_gate_close_time) == FAIL )
     {
       sprintf(emsg,"Could not read attribute.");
       xxx_LogStatus(progname,__FILE__,__LINE__,emsg);
       return(ERROR);
     }
     cal->attributes.valid_map[5] = VALID;
   }
   cal->attributes.valid_map[6] = INVALID;
   if ( (attr_index = SDfindattr( sds_id, "Number of cross track pixels" )) != FAIL )
   {
     if( SDreadattr(sds_id, attr_index, &cal->attributes.number_cross_track_pixels) == FAIL )
     {
       sprintf(emsg,"Could not read attribute.");
       xxx_LogStatus(progname,__FILE__,__LINE__,emsg);
       return(ERROR);
     }
     cal->attributes.valid_map[6] = VALID;
   }
   cal->attributes.valid_map[7] = INVALID;
   if ( (attr_index = SDfindattr( sds_id, "Number of along track pixels" )) != FAIL )
   {
     if( SDreadattr(sds_id, attr_index, &cal->attributes.number_along_track_pixels) == FAIL )
     {
       sprintf(emsg,"Could not read attribute.");
       xxx_LogStatus(progname,__FILE__,__LINE__,emsg);
       return(ERROR);
     }
     cal->attributes.valid_map[7] = VALID;
   }
   cal->attributes.valid_map[8] = INVALID;
   if ( (attr_index = SDfindattr( sds_id, "Number of bands" )) != FAIL )
   {
     if( SDreadattr(sds_id, attr_index, &cal->attributes.number_of_bands) == FAIL )
     {
       sprintf(emsg,"Could not read attribute.");
       xxx_LogStatus(progname,__FILE__,__LINE__,emsg);
       return(ERROR);
     }
     cal->attributes.valid_map[8] = VALID;
   }
   cal->attributes.valid_map[9] = INVALID;
   if ( (attr_index = SDfindattr( sds_id, "Percent Missing Data" )) != FAIL )
   {
     if( SDreadattr(sds_id, attr_index, &cal->attributes.percent_missing_data) == FAIL )
     {
       sprintf(emsg,"Could not read attribute.");
       xxx_LogStatus(progname,__FILE__,__LINE__,emsg);
       return(ERROR);
     }
     cal->attributes.valid_map[9] = VALID;
   }

   if( SDendaccess(sds_id) == FAIL )
   {
      sprintf(emsg,"Ending SDS access error.");
      xxx_LogStatus(progname,__FILE__,__LINE__,emsg);
      return(ERROR);
   }


   return(SUCCESS);
}
