/******************************************************************************
UNIT NAME:      xxx_read_ali_l1r_image.c 

PURPOSE: Reads a Level 1R ALI (from EO-1 instrument) image data set.  All
	 data is put in the structure xxx_ali_level1r.h.  It is up to the caller of
	 this routine to access what data is needed by their routine and then
	 to free up the memory associated with the structure xxx_ali_level1r.h.        

*******************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <limits.h>
#include <unistd.h>
#include <sys/types.h>
#include <stddef.h>
#include "xxx_Types.h"
#include "xxx_LogStatus.h"
#include "xxx_ali_level1r.h"

int xxx_read_ali_l1r_image(
int32 sd_id,                   /* I: Name of Level 1R file		*/
int32 sds_index,                /* Index of current SDS			*/
L1R_IMAGE *image)               /* O: Structure containing L1R data	*/
{
   char emsg[STRLEN];		/* Error message buffer			*/
   char progname[STRLEN];	/* Program name				*/
   int32 sds_id;                /* SDS file ID                          */
   int32 start[3];		/* Starting dims of SDS "slab"		*/
   int32 edges[3];		/* Edge dims of SDS "slab"		*/
   int32 stride[3];		/* Increment dims of SDS "slab"		*/
   int32 size;			/* Size needed for array allocation	*/ 
   int32 attr_index;		/* Index of attributes			*/
   int16 fill;			/* Image fill value			*/
   int32 i;			/* Band loop index			*/
   int32 j;			/* Line loop index			*/

   sprintf(progname,"READ_L1R_IMAGE");

   /*************************
    * Read image data *
    *************************/
   memset( image, 0, sizeof(L1R_IMAGE) );

   /* Select SDS index for image data
      ------------------------------------- */
   sds_id = SDselect(sd_id, sds_index);

   /* Get SDS info
      ------------ */
   if( SDgetinfo(sds_id,
                 (char *)image->name,
		 &image->number_of_dims, 
                 image->dim_sizes,
		 &image->HDF_type, 
                 &image->number_of_attributes )
       == FAIL )
   {
      sprintf(emsg,"Could not get SDS info");
      xxx_LogStatus(progname,__FILE__,__LINE__,emsg);
      return(ERROR);
   }

   /* Set up dimensions of SDS "slab".
      -------------------------------*/
   start[0]  = 0; /* Band start is 0-relative */
   start[1]  = 0;
   start[2]  = 0;

   stride[0] = 1;
   stride[1] = 1;
   stride[2] = 1;

   edges[0]  = image->dim_sizes[0];    
   edges[1]  = image->dim_sizes[1];
   edges[2]  = image->dim_sizes[2];

   size = edges[0] * edges[1] * edges[2];
   image->data = (int16 *)malloc( sizeof(int16) * size );
   if( (image->data) == NULL )
   {
      sprintf(emsg,"Error allocating data buffer.");
      xxx_LogStatus(progname,__FILE__,__LINE__,emsg);
      return(ERROR);
   }

   /* Read data into buffer
   ----------------------*/
   if( SDreaddata(sds_id, start, stride, edges, (VOIDP)image->data) == FAIL )
   {
      sprintf(emsg,"Could not read SDS data");
      xxx_LogStatus(progname,__FILE__,__LINE__,emsg);
      return(ERROR);
   }

   image->attributes.valid_map[0] = INVALID;
   if ( (attr_index = SDfindattr( sds_id, "Dataset Type")) != FAIL )
   {
     if( SDreadattr(sds_id, attr_index, image->attributes.dataset_type) == FAIL )
     {
       sprintf(emsg,"Could not read attribute.");
       xxx_LogStatus(progname,__FILE__,__LINE__,emsg);
       return(ERROR);
     }
     image->attributes.valid_map[0] = VALID;
   }
   image->attributes.valid_map[1] = INVALID;
   if ( (attr_index = SDfindattr( sds_id, "ALI Sensor")) != FAIL )
   {
     if( SDreadattr(sds_id, attr_index, image->attributes.ali_sensor) == FAIL )
     {
       sprintf(emsg,"Could not read attribute.");
       xxx_LogStatus(progname,__FILE__,__LINE__,emsg);
       return(ERROR);
     }
     image->attributes.valid_map[1] = VALID;
   }
   image->attributes.valid_map[2] = INVALID;
   if ( (attr_index = SDfindattr( sds_id, "Data Start Time")) != FAIL )
   {
     if( SDreadattr(sds_id, attr_index, image->attributes.data_start_time) == FAIL )
     {
       sprintf(emsg,"Could not read attribute.");
       xxx_LogStatus(progname,__FILE__,__LINE__,emsg);
       return(ERROR);
     }
     image->attributes.valid_map[2] = VALID;
   }
   image->attributes.valid_map[3] = INVALID;
   if ( (attr_index = SDfindattr( sds_id, "Data Duration")) != FAIL )
   {
     if( SDreadattr(sds_id, attr_index, &image->attributes.data_duration) == FAIL )
     {
       sprintf(emsg,"Could not read attribute.");
       xxx_LogStatus(progname,__FILE__,__LINE__,emsg);
       return(ERROR);
     }
     image->attributes.valid_map[3] = VALID;
   }
   image->attributes.valid_map[4] = INVALID;
   if ( (attr_index = SDfindattr( sds_id, "Integration Time")) != FAIL )
   {
     if( SDreadattr(sds_id, attr_index, &image->attributes.integration_time) == FAIL )
     {
       sprintf(emsg,"Could not read attribute.");
       xxx_LogStatus(progname,__FILE__,__LINE__,emsg);
       return(ERROR);
     }
     image->attributes.valid_map[4] = VALID;
   }
   image->attributes.valid_map[5] = INVALID;
   if ( (attr_index = SDfindattr( sds_id, "Frame Rate")) != FAIL )
   {
     if( SDreadattr(sds_id, attr_index, &image->attributes.frame_rate) == FAIL )
     {
       sprintf(emsg,"Could not read attribute.");
       xxx_LogStatus(progname,__FILE__,__LINE__,emsg);
       return(ERROR);
     }
     image->attributes.valid_map[5] = VALID;
   }
   image->attributes.valid_map[6] = INVALID;
   if ( (attr_index = SDfindattr( sds_id, "Data Gate Open Time")) != FAIL )
   {
     if( SDreadattr(sds_id, attr_index, &image->attributes.data_gate_open_time) == FAIL )
     {
       sprintf(emsg,"Could not read attribute.");
       xxx_LogStatus(progname,__FILE__,__LINE__,emsg);
       return(ERROR);
     }
     image->attributes.valid_map[6] = VALID;
   }
   image->attributes.valid_map[7] = INVALID;
   if ( (attr_index = SDfindattr( sds_id, "Data Gate Close Time")) != FAIL )
   {
     if( SDreadattr(sds_id, attr_index, &image->attributes.data_gate_close_time) == FAIL )
     {
       sprintf(emsg,"Could not read attribute.");
       xxx_LogStatus(progname,__FILE__,__LINE__,emsg);
       return(ERROR);
     }
     image->attributes.valid_map[7] = VALID;
   }
   image->attributes.valid_map[8] = INVALID;
   if ( (attr_index = SDfindattr( sds_id, "Bytes prior to SYNC")) != FAIL )
   {
     if( SDreadattr(sds_id, attr_index, &image->attributes.bytes_prior_sync) == FAIL )
     {
       sprintf(emsg,"Could not read attribute.");
       xxx_LogStatus(progname,__FILE__,__LINE__,emsg);
       return(ERROR);
     }
     image->attributes.valid_map[8] = VALID;
   }
   image->attributes.valid_map[9] = INVALID;
   if ( (attr_index = SDfindattr( sds_id, "Number of cross track pixels")) != FAIL )
   {
     if( SDreadattr(sds_id, attr_index, &image->attributes.number_cross_track_pixels) == FAIL )
     {
       sprintf(emsg,"Could not read attribute.");
       xxx_LogStatus(progname,__FILE__,__LINE__,emsg);
       return(ERROR);
     }
     image->attributes.valid_map[9] = VALID;
   }
   image->attributes.valid_map[10] = INVALID;
   if ( (attr_index = SDfindattr( sds_id, "Number of along track pixels")) != FAIL )
   {
     if( SDreadattr(sds_id, attr_index, &image->attributes.number_along_track_pixels) == FAIL )
     {
       sprintf(emsg,"Could not read attribute.");
       xxx_LogStatus(progname,__FILE__,__LINE__,emsg);
       return(ERROR);
     }
     image->attributes.valid_map[10] = VALID;
   }
   image->attributes.valid_map[11] = INVALID;
   if ( (attr_index = SDfindattr( sds_id, "Number of bands")) != FAIL )
   {
     if( SDreadattr(sds_id, attr_index, &image->attributes.number_of_bands) == FAIL )
     {
       sprintf(emsg,"Could not read attribute.");
       xxx_LogStatus(progname,__FILE__,__LINE__,emsg);
       return(ERROR);
     }
     image->attributes.valid_map[11] = VALID;
   }
   image->attributes.valid_map[12] = INVALID;
   if ( (attr_index = SDfindattr( sds_id, "Percent Missing Data")) != FAIL )
   {
     if( SDreadattr(sds_id, attr_index, &image->attributes.percent_missing_data) == FAIL )
     {
       sprintf(emsg,"Could not read attribute.");
       xxx_LogStatus(progname,__FILE__,__LINE__,emsg);
       return(ERROR);
     }
     image->attributes.valid_map[12] = VALID;
   }
   image->attributes.valid_map[13] = INVALID;
   if ( (attr_index = SDfindattr( sds_id, "Frame shift PAN")) != FAIL )
   {
     if( SDreadattr(sds_id, attr_index, &image->attributes.frame_shift[0]) == FAIL )
     {
       sprintf(emsg,"Could not read attribute.");
       xxx_LogStatus(progname,__FILE__,__LINE__,emsg);
       return(ERROR);
     }
     sprintf(emsg,"Frame shift Pan = %d", image->attributes.frame_shift[0]);
     xxx_LogStatus(progname,__FILE__,__LINE__,emsg);
     image->attributes.valid_map[13] = VALID;
   }
   image->attributes.valid_map[14] = INVALID;
   if ( (attr_index = SDfindattr( sds_id, "Frame shift 1p")) != FAIL )
   {
     if( SDreadattr(sds_id, attr_index, &image->attributes.frame_shift[1]) == FAIL )
     {
       sprintf(emsg,"Could not read attribute.");
       xxx_LogStatus(progname,__FILE__,__LINE__,emsg);
       return(ERROR);
     }
     image->attributes.valid_map[14] = VALID;
   }
   image->attributes.valid_map[15] = INVALID;
   if ( (attr_index = SDfindattr( sds_id, "Frame shift 1")) != FAIL )
   {
     if( SDreadattr(sds_id, attr_index, &image->attributes.frame_shift[2]) == FAIL )
     {
       sprintf(emsg,"Could not read attribute.");
       xxx_LogStatus(progname,__FILE__,__LINE__,emsg);
       return(ERROR);
     }
     image->attributes.valid_map[15] = VALID;
   }
   image->attributes.valid_map[16] = INVALID;
   if ( (attr_index = SDfindattr( sds_id, "Frame shift 2")) != FAIL )
   {
     if( SDreadattr(sds_id, attr_index, &image->attributes.frame_shift[3]) == FAIL )
     {
       sprintf(emsg,"Could not read attribute.");
       xxx_LogStatus(progname,__FILE__,__LINE__,emsg);
       return(ERROR);
     }
     image->attributes.valid_map[16] = VALID;
   }
   image->attributes.valid_map[17] = INVALID;
   if ( (attr_index = SDfindattr( sds_id, "Frame shift 3")) != FAIL )
   {
     if( SDreadattr(sds_id, attr_index, &image->attributes.frame_shift[4]) == FAIL )
     {
       sprintf(emsg,"Could not read attribute.");
       xxx_LogStatus(progname,__FILE__,__LINE__,emsg);
       return(ERROR);
     }
     image->attributes.valid_map[17] = VALID;
   }
   image->attributes.valid_map[18] = INVALID;
   if ( (attr_index = SDfindattr( sds_id, "Frame shift 4")) != FAIL )
   {
     if( SDreadattr(sds_id, attr_index, &image->attributes.frame_shift[5]) == FAIL )
     {
       sprintf(emsg,"Could not read attribute.");
       xxx_LogStatus(progname,__FILE__,__LINE__,emsg);
       return(ERROR);
     }
     image->attributes.valid_map[18] = VALID;
   }
   image->attributes.valid_map[19] = INVALID;
   if ( (attr_index = SDfindattr( sds_id, "Frame shift 4p")) != FAIL )
   {
     if( SDreadattr(sds_id, attr_index, &image->attributes.frame_shift[6]) == FAIL )
     {
       sprintf(emsg,"Could not read attribute.");
       xxx_LogStatus(progname,__FILE__,__LINE__,emsg);
       return(ERROR);
     }
     image->attributes.valid_map[19] = VALID;
   }
   image->attributes.valid_map[20] = INVALID;
   if ( (attr_index = SDfindattr( sds_id, "Frame shift 5p")) != FAIL )
   {
     if( SDreadattr(sds_id, attr_index, &image->attributes.frame_shift[7]) == FAIL )
     {
       sprintf(emsg,"Could not read attribute.");
       xxx_LogStatus(progname,__FILE__,__LINE__,emsg);
       return(ERROR);
     }
     image->attributes.valid_map[20] = VALID;
   }
   image->attributes.valid_map[21] = INVALID;
   if ( (attr_index = SDfindattr( sds_id, "Frame shift 5")) != FAIL )
   {
     if( SDreadattr(sds_id, attr_index, &image->attributes.frame_shift[8]) == FAIL )
     {
       sprintf(emsg,"Could not read attribute.");
       xxx_LogStatus(progname,__FILE__,__LINE__,emsg);
       return(ERROR);
     }
     image->attributes.valid_map[21] = VALID;
   }
   image->attributes.valid_map[22] = INVALID;
   if ( (attr_index = SDfindattr( sds_id, "Frame shift 7")) != FAIL )
   {
     if( SDreadattr(sds_id, attr_index, &image->attributes.frame_shift[9]) == FAIL )
     {
       sprintf(emsg,"Could not read attribute.");
       xxx_LogStatus(progname,__FILE__,__LINE__,emsg);
       return(ERROR);
     }
     image->attributes.valid_map[22] = VALID;
   }

   /* Remove fill lines if necessary - PAN band */
   if ( (image->attributes.valid_map[13] == VALID) &&
        (image->attributes.frame_shift[0] > 0) )
   {
     fill = image->data[0];
     memcpy((image->data),
            (image->data +
              image->attributes.frame_shift[0]*image->dim_sizes[2]),
            sizeof(int16) *
              image->dim_sizes[2] *
              (image->dim_sizes[1]-image->attributes.frame_shift[0]));
     j=image->dim_sizes[1] - image->attributes.frame_shift[0] - 1;
     while ( (j > 0) &&
             (image->data[j*image->dim_sizes[2]] == fill) &&
             (image->data[j*image->dim_sizes[2]+1] == fill) ) j--;
     image->dim_sizes[1] = j+1;
   }
   /* Remove fill lines if necessary - Multispectral bands */
   size = image->dim_sizes[1];
   for ( i=0; i<image->dim_sizes[0]; i++ )
   {
     if ( (image->attributes.valid_map[14+i] == VALID) &&
          (image->attributes.frame_shift[1+i] > 0) )
     {
       fill = image->data[i*image->dim_sizes[1]*image->dim_sizes[2]];
       memcpy((image->data +
                i*image->dim_sizes[1]*image->dim_sizes[2]),
              (image->data +
                i*image->dim_sizes[1]*image->dim_sizes[2] +
                image->attributes.frame_shift[1+i]*image->dim_sizes[2]),
              sizeof(int16) *
                image->dim_sizes[2] *
                (image->dim_sizes[1]-image->attributes.frame_shift[1+i]));
       j=image->dim_sizes[1] - image->attributes.frame_shift[i+1] - 1;
       while ( (j > 0) &&
               (image->data[i*image->dim_sizes[1]*image->dim_sizes[2]+j*image->dim_sizes[2]] == fill) &&
               (image->data[i*image->dim_sizes[1]*image->dim_sizes[2]+j*image->dim_sizes[2]+1] == fill) ) j--;
       size = j+1;
     }
   }
   for ( i=1; i<image->dim_sizes[0]; i++ )
   {
     if ( size < image->dim_sizes[1] )
     {
       memcpy((image->data+i*size*image->dim_sizes[2]),
              (image->data+i*image->dim_sizes[1]*image->dim_sizes[2]),
              sizeof(int16)*image->dim_sizes[2]*size);
     }
   }
   image->dim_sizes[1] = size;

   if( SDendaccess(sds_id) == FAIL )
   {
      sprintf(emsg,"Ending SDS access error.");
      xxx_LogStatus(progname,__FILE__,__LINE__,emsg);
      return(ERROR);
   }

   return(SUCCESS);
}
