/***************************************************************************
UNIT NAME:      xxx_read_ali_level0r.c 

PURPOSE: Reads a Level 0R ALI (from EO-1 instrument) data set.  All data is 
         put in the structure xxx_ali_level0r.h.  It is up to the caller of this
         routine to access what data is needed by their routine and then to
         free up the memory associated with the structure xxx_ali_level0r.h.        
               
***************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <limits.h>
#include <unistd.h>
#include <sys/types.h>
#include <stddef.h>
#include "xxx_Types.h"
#include "xxx_LogStatus.h"
#include "xxx_ali_level0r.h"

int xxx_read_ali_level0r(
char l0r_name[STRLEN],		/* I: Name of Level 0R file		*/
LEVEL0R *level0r)	        /* O: Structure containing L0R data	*/
{
   char emsg[STRLEN];		/* Error message buffer			*/
   char progname[STRLEN];	/* Program name				*/
   int32 sd_id;                 /* SD file ID                           */
   int32 sds_index;		/* Index of current SDS			*/
   int32 n_datasets;            /* Number of data sets in HDF file      */
   int32 n_file_attrs;          /* Number of file attributes            */
   int32 band;                  /* Current band                         */
   int32 sds_id;		/* SDS ID				*/
   int32 type_code;		/* SDS type code 1-8			*/
   int32 attr_index;		/* SDS attribute index			*/
   char sensor[5];		/* L0 SDS Sensor attribute value	*/
   char dataset_type[13];	/* L0 SDS Dataset Type attribute value	*/

   sprintf(progname,"READ_L0R");

   /* Get SD file ID
   ----------------*/
   if ((sd_id = SDstart(l0r_name, DFACC_RDONLY)) == FAIL)
   {
      sprintf(emsg,"Could not open SDS file");
      xxx_LogStatus(progname,__FILE__,__LINE__,emsg);
      return(ERROR);
   }

   /* Get SD file info
   ------------------*/
   if( SDfileinfo(sd_id, &n_datasets, &n_file_attrs) == FAIL )
   {
      sprintf(emsg,"Could not retrieve SDS file info.");
      xxx_LogStatus(progname,__FILE__,__LINE__,emsg);
      return(ERROR);
   }
   /* Check to see if correct number of attributes.
   Give two choices to handle both old and new format.
   --------------------------------------------------*/
   if( (n_file_attrs != N_FILE_ATTRS) && (n_file_attrs != N_OLD_FILE_ATTRS) )
   {
      sprintf(emsg,"Incorrect number of file attributes in L0R");
      xxx_LogStatus(progname,__FILE__,__LINE__,emsg);
      return(ERROR);
   }
   /* Check to see if correct number of data sets.
   ----------------------------------------------*/
   if( n_datasets != N_DATASETS )
   {
      sprintf(emsg,"Incorrect number of datasets in L0R");
      xxx_LogStatus(progname,__FILE__,__LINE__,emsg);
     /* return(ERROR);*/
   }

   /********************************************************************
    * BEGIN READING L0R PRODUCT IN THIS ORDER
    *
    * 1) Header -- Once
    * 2,3) Precal -- Multi then Pan
    * 4,5) Image -- Multi then Pan
    * 6,7) Dark -- Multi then Pan
    * 8,9) Lamp -- Multi then Pan
    *
    */

   /* Read header from L0R file
    * 1) Header
   ------------------------------*/
   if( xxx_read_ali_l0r_header(sd_id, &level0r->header )
       == FAIL )
   {
      sprintf(emsg,"Could not read header.");
      xxx_LogStatus(progname,__FILE__,__LINE__,emsg);
      return(ERROR);
   }

   /* The data is stored under indexed SDS's
      -------------------------------------- */
   for (sds_index=0; sds_index<n_datasets; sds_index++)
   {
     /* Read the SDS ALI Sensor and Dataset Type attributes
        --------------------------------------------------- */
     sds_id = SDselect(sd_id, sds_index);
     if ( (attr_index = SDfindattr( sds_id, "ALI Sensor" )) != FAIL )
     {
       if( SDreadattr(sds_id, attr_index, sensor) == FAIL )
       {
         sprintf(emsg,"Could not read ALI Sensor attribute.");
         xxx_LogStatus(progname,__FILE__,__LINE__,emsg);
         return(ERROR);
       }
     }
     if ( (attr_index = SDfindattr( sds_id, "Dataset Type" )) != FAIL )
     {
       if( SDreadattr(sds_id, attr_index, dataset_type) == FAIL )
       {
         sprintf(emsg,"Could not read Dataset Type attribute.");
         xxx_LogStatus(progname,__FILE__,__LINE__,emsg);
         return(ERROR);
       }
     }
     if( SDendaccess(sds_id) == FAIL )
     {
        sprintf(emsg,"Ending SDS access error.");
        xxx_LogStatus(progname,__FILE__,__LINE__,emsg);
        return(ERROR);
     }

     /* Determine the data set type code based on the attribute values.
        --------------------------------------------------------------- */
     if ( strncmp( sensor, "MS", 2 ) == 0 ) type_code = 1;
     else type_code = 2;
     if ( strncmp( dataset_type, "LAMP", 4 ) == 0 ) type_code += 6;
     else if ( strncmp( dataset_type, "DARK", 4 ) == 0 ) type_code += 4;
     else if ( strncmp( dataset_type, "LEVEL0", 6 ) == 0 ) type_code += 2;
     else if ( strncmp( dataset_type, "PRECAL", 6 ) == 0 ) type_code += 0;

     /* Switch to the appropriate read logic based on dataset type code.
        ---------------------------------------------------------------- */
     switch(type_code)
     {
     case 1:			/* MS PRECAL */
       /* Read multispectral precal data from L0R file
        * Case 1) Multi-precal
       ---------------------------------*/
       if( xxx_read_ali_l0r_cal(sd_id, sds_index, &level0r->precal[0])
           == FAIL )
       {
          sprintf(emsg,"Could not read header.");
          xxx_LogStatus(progname,__FILE__,__LINE__,emsg);
          return(ERROR);
       }

       /* Setup each of the band's precal data pointer 
          -------------------------------------------- */
       for(band=1;band<PAN_BAND;band++)
        {
          memcpy(&level0r->precal[band], &level0r->precal[0], sizeof(L0R_CAL));
          level0r->precal[band].data += band * level0r->precal[0].dim_sizes[N_FRAMES_IDX]
                                              * level0r->precal[0].dim_sizes[N_DETS_IDX];
        }
        break;

     case 2:			/* PN PRECAL */
       /* Read panchromatic precal data from L0R file
        * Case 2) Pan-precal
       ---------------------------------*/
       if( xxx_read_ali_l0r_cal(sd_id, sds_index, &level0r->precal[PAN_BAND] )
           == FAIL )
       {
          sprintf(emsg,"Could not read header.");
          xxx_LogStatus(progname,__FILE__,__LINE__,emsg);
          return(ERROR);
       }
       break;

     case 3:			/* MS LEVEL0 */
       /* Read multispectral image data from L0R file
        * Case 3) Multi-image
       ---------------------------------*/
       if( xxx_read_ali_l0r_image(sd_id, sds_index, &level0r->image[0] )
           == FAIL )
       {
          sprintf(emsg,"Could not read header.");
          xxx_LogStatus(progname,__FILE__,__LINE__,emsg);
          return(ERROR);
       }

       /* Setup each of the band's image data pointer 
          -------------------------------------------- */
       for(band=1;band<PAN_BAND;band++)
        {
          memcpy(&level0r->image[band], &level0r->image[0], sizeof(L0R_IMAGE));
          level0r->image[band].data += band * level0r->image[0].dim_sizes[N_FRAMES_IDX]
                                              * level0r->image[0].dim_sizes[N_DETS_IDX];
        }
        break;

     case 4:			/* PN LEVEL0 */
       /* Read panchromatic image data from L0R file
        * Case 4) Pan-image
       ---------------------------------*/
       if( xxx_read_ali_l0r_image(sd_id, sds_index, &level0r->image[PAN_BAND] )
           == FAIL )
       {
          sprintf(emsg,"Could not read header.");
          xxx_LogStatus(progname,__FILE__,__LINE__,emsg);
          return(ERROR);
       }
       break;

     case 5:			/* MS DARK */
       /* Read multispectral dark data from L0R file
        * Case 5) Multi-dark
       ---------------------------------*/
       if( xxx_read_ali_l0r_cal(sd_id, sds_index, &level0r->dark[0] )
           == FAIL )
       {
          sprintf(emsg,"Could not read header.");
          xxx_LogStatus(progname,__FILE__,__LINE__,emsg);
          return(ERROR);
       }

       /* Setup each of the band's dark data pointer 
          -------------------------------------------- */
       for(band=1;band<PAN_BAND;band++)
        {
          memcpy(&level0r->dark[band], &level0r->dark[0], sizeof(L0R_CAL));
          level0r->dark[band].data += band * level0r->dark[0].dim_sizes[N_FRAMES_IDX]
                                              * level0r->dark[0].dim_sizes[N_DETS_IDX];
        }
        break;

     case 6:			/* PN DARK */
       /* Read panchromatic dark data from L0R file
        * Case 6) Pan-dark
       ---------------------------------*/
       if( xxx_read_ali_l0r_cal(sd_id, sds_index, &level0r->dark[PAN_BAND] )
           == FAIL )
       {
          sprintf(emsg,"Could not read header.");
          xxx_LogStatus(progname,__FILE__,__LINE__,emsg);
          return(ERROR);
       }
       break;

     case 7:			/* MS LAMP */
       /* Read multispectral lamp data from L0R file
        * Case 7) Multi-lamp
       ---------------------------------*/
       if( xxx_read_ali_l0r_cal(sd_id, sds_index, &level0r->lamp[0] )
           == FAIL )
       {
          sprintf(emsg,"Could not read header.");
          xxx_LogStatus(progname,__FILE__,__LINE__,emsg);
          return(ERROR);
       }

       /* Setup each of the band's lamp data pointer 
          -------------------------------------------- */
       for(band=1;band<PAN_BAND;band++)
        {
          memcpy(&level0r->lamp[band], &level0r->lamp[0], sizeof(L0R_CAL));
          level0r->lamp[band].data += band * level0r->lamp[0].dim_sizes[N_FRAMES_IDX]
                                              * level0r->lamp[0].dim_sizes[N_DETS_IDX];
        }
        break;

     case 8:			/* PN LAMP */
       /* Read panchromatic lamp data from L0R file
        * Case 8) Pan-lamp
       ---------------------------------*/
       if( xxx_read_ali_l0r_cal(sd_id, sds_index, &level0r->lamp[PAN_BAND] )
           == FAIL )
       {
          sprintf(emsg,"Could not read header.");
          xxx_LogStatus(progname,__FILE__,__LINE__,emsg);
          return(ERROR);
       }
       break;

     default:
       sprintf(emsg,"Invalid SDS type.");
       xxx_LogStatus(progname,__FILE__,__LINE__,emsg);
       return(ERROR);
       break;
     }			/* end switch */
   }		/* end SDS for loop */

   /*
    *
    * END READING L0R PRODUCT
    ********************************************************************/

   return(SUCCESS);
}
