/******************************************************************************
UNIT NAME:      read_level1r.c 

PURPOSE: Reads a Level 1R ALI (from EO-1 instrument) data set.  All data is 
         put in the structure xxx_ali_level1r.h.  It is up to the caller of this
         routine to access what data is needed by their routine and then to free
         the memory associated with the structure xxx_ali_level1r.h.        
               
*******************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <limits.h>
#include <unistd.h>
#include <sys/types.h>
#include <stddef.h>
#include "xxx_Types.h"
#include "xxx_LogStatus.h"
#include "xxx_const.h"
#include "xxx_ali_level1r.h"

int xxx_read_ali_level1r(
char *l1r_name,		/* I: Name of Level 1R file		*/
LEVEL1R *level1r)	/* O: Structure containing L1R data	*/
{
   char emsg[STRLEN];		/* Error message buffer			*/
   char progname[STRLEN];	/* Program name				*/
   int32 sd_id;                 /* SD file ID                           */
   int32 sds_id;                /* SDS file ID                          */
   int32 n_datasets;            /* Number of data sets in HDF file      */
   int32 n_file_attrs;          /* Number of file attributes            */
   int32 dim_sizes[MAX_SDS_DIM];/* Dimensions of data                   */
   int32 file_attr_index;       /* Attribute index                      */
   int32 sds_index;		/* Index of current SDS			*/
   int32 start[3];		/* Starting dims of SDS "slab"		*/
   int32 edges[3];		/* Edge dims of SDS "slab"		*/
   int32 stride[3];		/* Increment dims of SDS "slab"		*/
   int32 size;			/* Size needed for array allocation	*/ 
   int32 attr_index;		/* Index of attributes			*/
   int32 band;			/* Band index				*/

   strcpy(progname,"READ_LEVEL1R");

   /* Get SD file ID
   ----------------*/
   if ((sd_id = SDstart(l1r_name, DFACC_RDONLY)) == FAIL)
   {
      strcpy(emsg,"Could not open SDS file\n");
      xxx_LogStatus(progname,__FILE__,__LINE__, emsg);
      return(ERROR);
   }

   /* Get SD file info
   ------------------*/
   if( SDfileinfo(sd_id, &n_datasets, &n_file_attrs) == FAIL )
   {
      strcpy(emsg,"Could not retrieve SDS file info.\n");
      xxx_LogStatus(progname,__FILE__,__LINE__, emsg);
      return(ERROR);
   }
   /* Check to see if correct number of attributes.
   Give three choices to support both old and new formats.
   ------------------------------------------------------*/
   if( (n_file_attrs != 10) && (n_file_attrs != 14) && (n_file_attrs != 16) )
   {
      strcpy(emsg,"Incorrect number of file attributes in L1R\n");
      xxx_LogStatus(progname,__FILE__,__LINE__, emsg);
      return(ERROR);
   }
   /* Check to see if correct number of data sets.
   ----------------------------------------------*/
   if( n_datasets != 6 )
   {
      strcpy(emsg,"Incorrect number of datasets in L1R\n");
      xxx_LogStatus(progname,__FILE__,__LINE__, emsg);
      return(ERROR);
   }

   /* Read file attributes of SD.
   ------------------------------*/
   if ( xxx_read_ali_l1r_header( sd_id, &level1r->header ) != SUCCESS )
   {
      strcpy(emsg,"Could not Level 1R file header attributes.\n");
      xxx_LogStatus(progname,__FILE__,__LINE__, emsg);
      return(ERROR);
   }

   /*********************************
    * Read image data multispectral *
    *********************************/

   /* Select SDS index for multispectral.
   ------------------------------------*/
   sds_index=0;
   if ( xxx_read_ali_l1r_image( sd_id, sds_index, &level1r->image[0] ) != SUCCESS )
   {
      strcpy(emsg,"Could not get multispectral image SDS\n");
      xxx_LogStatus(progname,__FILE__,__LINE__, emsg);
      return(ERROR);
   }

   /* Populate other band structures.
   --------------------------------*/
   for( band=1; band<PAN_BAND; band++ )
   {
      memcpy( &level1r->image[band], &level1r->image[0], sizeof(L1R_IMAGE) );
      level1r->image[band].data += band * level1r->image[0].dim_sizes[1]
                                        * level1r->image[0].dim_sizes[2];
   }

   /*****************************
    * Read Offset multispectral * 
    *****************************/
   sds_index++;
   if ( xxx_read_ali_l1r_cal( sd_id, sds_index, &level1r->offset[0] ) != SUCCESS )
   {
      strcpy(emsg,"Could not get multispectral offset SDS\n");
      xxx_LogStatus(progname,__FILE__,__LINE__, emsg);
      return(ERROR);
   }

   /* Populate other band structures.
   --------------------------------*/
   for( band=1; band<PAN_BAND; band++ )
   {
      memcpy( &level1r->offset[band], &level1r->offset[0], sizeof(L1R_CAL) );
      level1r->offset[band].data += band * level1r->offset[0].dim_sizes[1]
                                         * level1r->offset[0].dim_sizes[2];
   }

   /*******************************
    * Read response multispectral * 
    *******************************/
   sds_index++;
   if ( xxx_read_ali_l1r_cal( sd_id, sds_index, &level1r->response[0] ) != SUCCESS )
   {
      strcpy(emsg,"Could not get multispectral response SDS\n");
      xxx_LogStatus(progname,__FILE__,__LINE__, emsg);
      return(ERROR);
   }

   /* Populate other band structures.
   --------------------------------*/
   for( band=1; band<PAN_BAND; band++ )
   {
      memcpy( &level1r->response[band], &level1r->response[0], sizeof(L1R_CAL) );
      level1r->response[band].data += band * level1r->response[0].dim_sizes[1]
                                           * level1r->response[0].dim_sizes[2];
   }

   /***********************
    * Read image data pan * 
    ***********************/
   sds_index++;
   if ( xxx_read_ali_l1r_image( sd_id, sds_index, &level1r->image[PAN_BAND] ) != SUCCESS )
   {
      strcpy(emsg,"Could not get panchromatic image SDS\n");
      xxx_LogStatus(progname,__FILE__,__LINE__, emsg);
      return(ERROR);
   }

   /*******************
    * Read Offset pan * 
    *******************/
   sds_index++;
   if ( xxx_read_ali_l1r_cal( sd_id, sds_index, &level1r->offset[PAN_BAND] ) != SUCCESS )
   {
      strcpy(emsg,"Could not get panchromatic offset SDS\n");
      xxx_LogStatus(progname,__FILE__,__LINE__, emsg);
      return(ERROR);
   }

   /*********************
    * Read response pan * 
    *********************/
   sds_index++;
   if ( xxx_read_ali_l1r_cal( sd_id, sds_index, &level1r->response[PAN_BAND] ) != SUCCESS )
   {
      strcpy(emsg,"Could not get panchromatic response SDS\n");
      xxx_LogStatus(progname,__FILE__,__LINE__, emsg);
      return(ERROR);
   }

   /* Close the SDS file
   --------------------*/
   if ( SDend( sd_id ) == FAIL )
   {
      strcpy(emsg,"Closing SD file ID failed!\n ");
      xxx_LogStatus(progname,__FILE__,__LINE__, emsg);
      return(ERROR);
   }

   return(SUCCESS);
}
