/******************************************************************************
UNIT NAME:      xxx_read_hyp_l1r_image.c 

PURPOSE: Reads a Level 1R Hyperion (from EO-1 instrument) image data set.  All
	 data is put in the structure xxx_hyp_level0r.h.  It is up to the caller of
	 this routine to access what data is needed by their routine and then
	 to free up the memory associated with the structure xxx_hyp_level0r.h.        

*******************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <limits.h>
#include <unistd.h>
#include <sys/types.h>
#include <stddef.h>
#include <hdf.h>
#include <mfhdf.h>
#include "xxx_Types.h"
#include "xxx_const.h"
#include "xxx_LogStatus.h"

void *xxx_read_hyp_l1r_band(
char *l1r_name,		/* I: File name to read 	*/
int band,		/* I: Band of data to read	*/
int *nl,		/* O: Number of lines in data	*/
int *ns,		/* O: Number of samples in data	*/
DataType *type)        	/* O: Data type			*/
{
   char emsg[STRLEN];		/* Error message buffer			*/
   char progname[STRLEN];	/* Program name				*/
   int32 sds_id;                /* SDS file ID                          */
   int32 start[3];		/* Starting dims of SDS "slab"		*/
   int32 edges[3];		/* Edge dims of SDS "slab"		*/
   int32 stride[3];		/* Increment dims of SDS "slab"		*/
   int32 sd_id;                 /* Name of Level 1R file		*/
   int32 sds_index;             /* Index of current SDS			*/
   int32 HDF_type;		/* SDS HSF data type			*/
   int32 number_of_attributes;	/* Number of attributes in SDS		*/
   int32 number_of_dims;	/* Number of dimensions in SDS		*/
   int32 dim_sizes[3];          /* Dimensions of SDS                    */
   char8 sds_name[9];           /* SDS name                             */
   short *data;			/* Buffer holding data to return	*/
   int32 n_datasets;            /* Number of data sets in HDF file      */
   int32 n_file_attrs;          /* Number of file attributes            */

   sprintf(progname,"READ_HYP_L1R_BAND");

   /* Get SD file ID
   ----------------*/
   if ((sd_id = SDstart(l1r_name, DFACC_RDONLY)) == FAIL)
   {
      sprintf(emsg,"Could not open SDS file");
      xxx_LogStatus(progname,__FILE__,__LINE__,emsg);
      return(NULL);
   }

   if( SDfileinfo(sd_id, &n_datasets, &n_file_attrs) == FAIL )
   {
      strcpy(emsg,"Could not retrieve SDS file info.\n");
      xxx_LogStatus(progname,__FILE__,__LINE__,emsg);
      return(NULL);
   }
   sprintf(emsg,"Number of SDS %d.\n",n_datasets);
   xxx_LogStatus(progname,__FILE__,__LINE__,emsg);

   if( n_datasets != 1 && n_datasets != 5 )
   {
      sprintf(emsg,"Incorrect number of SDS %d.\n",n_datasets);
      xxx_LogStatus(progname,__FILE__,__LINE__,emsg);
      return(NULL);
   }

   /* Select SDS index for image data.  If there is more than one data set then 
      it is assumed that the first is imagery. 
      ------------------------------------- */
   sds_index = 0;
   sds_id = SDselect(sd_id, sds_index);

   /* Get SDS info
      ------------ */
   if( SDgetinfo(sds_id, sds_name, &number_of_dims, dim_sizes, &HDF_type, 
                 &number_of_attributes ) == FAIL )
   {
      sprintf(emsg,"Could not get SDS info");
      xxx_LogStatus(progname,__FILE__,__LINE__,emsg);
      return(NULL);
   }
   /* Note: Data is BIL so dimensions are
        dim_sizes[0] = nl
        dim_sizes[1] = nb
        dim_sizes[2] = ns
      ----------------------------------*/
   sprintf(emsg,"SDS name %s.\n",sds_name);
   xxx_LogStatus(progname,__FILE__,__LINE__,emsg);

   /* Store return variables 
      ---------------------*/
   *nl = dim_sizes[0];
   *ns = dim_sizes[2];
   if( HDF_type == 20 )
      *type = BYTE; 
   else if( HDF_type == 22 )
      *type = I2;
   else if( HDF_type == 23 )
      *type = UI2;
   else if( HDF_type == 24 )
      *type = I4;
   else if( HDF_type == 25 )
      *type = UI4;
   else if( HDF_type == 5 )
      *type = R4;
   else
   {
      sprintf(emsg,"Invalid data type");
      xxx_LogStatus(progname,__FILE__,__LINE__,emsg);
      return(NULL);
   }

   /* Allocate space for the image data
      ---------------------------------*/
   data = malloc(dim_sizes[0] * dim_sizes[2] * sizeof(short));
   if (data == NULL)
   {
      sprintf(emsg,"Error allocating memory");
      xxx_LogStatus(progname,__FILE__,__LINE__,emsg);
      return(NULL);
   }

   /* Set up dimensions of SDS "slab".
      -------------------------------*/
   
   start[0]  = 0; 
   start[1]  = band; /* 0 - relative */
   start[2]  = 0;

   stride[0] = 1;
   stride[1] = 1;
   stride[2] = 1;

   edges[0]  = dim_sizes[0];    
   edges[1]  = 1;
   edges[2]  = dim_sizes[2];

   /* Read image data into buffer
   -------------------------------*/
   if( SDreaddata(sds_id, start, stride, edges, (VOIDP)data) == FAIL )
   {
      sprintf(emsg,"Could not read SDS data");
      xxx_LogStatus(progname,__FILE__,__LINE__,emsg);
      return(NULL);
   }

   /* Close SDS
      --------*/
   if( SDendaccess(sds_id) == FAIL )
   {
      sprintf(emsg,"Could not end access to SDS");
      xxx_LogStatus(progname,__FILE__,__LINE__,emsg);
      return(NULL);
   }

   /* Close SD
      --------*/
   if( SDend(sd_id) == FAIL )
   {
      sprintf(emsg,"Could not end access to SDS");
      xxx_LogStatus(progname,__FILE__,__LINE__,emsg);
      return(NULL);
   }

   return(data);
}
