/******************************************************************************
UNIT NAME:      xxx_read_hyp_l1r_image_attributes.c 

PURPOSE: Reads a Level 1R ALI (from EO-1 instrument) image data set.  All
	 data is put in the structure xxx_ali_level1r.h.  It is up to the caller of
	 this routine to access what data is needed by their routine and then
	 to free up the memory associated with the structure xxx_ali_level1r.h.        

*******************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <limits.h>
#include <unistd.h>
#include <hdf.h>
#include <mfhdf.h>
#include <sys/types.h>
#include <stddef.h>
#include "xxx_Types.h"
#include "xxx_LogStatus.h"
#include "xxx_hyp_level1r.h"

int xxx_read_hyp_l1r_image_attributes(
char *l1r_name,         	 /* I: File name to read         	*/
HYP_L1R_IMAGE_ATTRIBUTES *image) /* O: Structure containing L1R data	*/
{
   char emsg[STRLEN];		/* Error message buffer			*/
   char progname[STRLEN];	/* Program name				*/
   int32 sds_id;                /* SDS file ID                          */
   int32 start[3];		/* Starting dims of SDS "slab"		*/
   int32 edges[3];		/* Edge dims of SDS "slab"		*/
   int32 stride[3];		/* Increment dims of SDS "slab"		*/
   int32 size;			/* Size needed for array allocation	*/ 
   int32 attr_index;		/* Index of attributes			*/
   int32 sd_id;                 /* Name of Level 1R file		*/
   int32 sds_index;             /* Index of current SDS			*/
   int32 n_datasets;
   int32 n_file_attrs;
   int32 data_type;		/* Data type for attribute		*/
   int32 n_values;		/* Number of attributes		 	*/
   char attr_name[STRLEN];      /* SDS Attribute name                   */

   sprintf(progname,"READ_HYP_L1R_IMAGE_ATTRIBUTES");

   /* Get SD file ID
   ----------------*/
   if ((sd_id = SDstart(l1r_name, DFACC_RDONLY)) == FAIL)
   {
      sprintf(emsg,"Could not open SDS file");
      xxx_LogStatus(progname,__FILE__,__LINE__,emsg);
      return(ERROR);
   }

   /* Get SD file info
   ------------------*/
   if( SDfileinfo(sd_id, &n_datasets, &n_file_attrs) == FAIL )
   {
      sprintf(emsg,"Could not retrieve SDS file info.");
      xxx_LogStatus(progname,__FILE__,__LINE__,emsg);
      return(ERROR);
   }

   if( n_datasets != 1 )
   {
      strcpy(emsg,"Incorrect number of SDS.\n");
      xxx_LogStatus(progname,__FILE__,__LINE__,emsg);
      return(NULL);
   }

   sds_index = 0;
   sds_id = SDselect(sd_id, sds_index);

   image->valid_map[0] = INVALID;
   if ( (attr_index = SDfindattr( sds_id, "L0 File Generated By")) != FAIL )
   {
     if( SDreadattr(sds_id, attr_index, image->generated_by) == FAIL )
     {
       sprintf(emsg,"Could not read attribute.");
       xxx_LogStatus(progname,__FILE__,__LINE__,emsg);
       return(ERROR);
     }
     image->valid_map[0] = VALID;
   }
   else
   {
     if ( (attr_index = SDfindattr( sd_id, "L0 File Generated By")) != FAIL )
     {
       if( SDreadattr(sd_id, attr_index, image->generated_by) == FAIL )
       {
         sprintf(emsg,"Could not read attribute.");
         xxx_LogStatus(progname,__FILE__,__LINE__,emsg);
         return(ERROR);
       }
       image->valid_map[0] = VALID;
     }
   }
   image->valid_map[1] = INVALID;
   if ( (attr_index = SDfindattr( sds_id, "File Byte Order")) != FAIL )
   {
     if( SDreadattr(sds_id, attr_index, image->file_byte_order) == FAIL )
     {
       sprintf(emsg,"Could not read attribute.");
       xxx_LogStatus(progname,__FILE__,__LINE__,emsg);
       return(ERROR);
     }
     image->valid_map[1] = VALID;
   }
   else
   {
     if ( (attr_index = SDfindattr( sd_id, "File Byte Order")) != FAIL )
     {
       if( SDreadattr(sd_id, attr_index, image->file_byte_order) == FAIL )
       {
         sprintf(emsg,"Could not read attribute.");
         xxx_LogStatus(progname,__FILE__,__LINE__,emsg);
         return(ERROR);
       }
       image->valid_map[1] = VALID;
     }
   }
   image->valid_map[2] = INVALID;
   if ( (attr_index = SDfindattr( sds_id, "Time of L0 File Generation (UTC)")) != FAIL )
   {
     if( SDreadattr(sds_id, attr_index, image->time_of_file_generation) == FAIL )
     {
       sprintf(emsg,"Could not read attribute.");
       xxx_LogStatus(progname,__FILE__,__LINE__,emsg);
       return(ERROR);
     }
     image->valid_map[2] = VALID;
   }
   else
   {
     if ( (attr_index = SDfindattr( sd_id, "Time of L0 File Generation (UTC)")) != FAIL )
     {
       if( SDreadattr(sd_id, attr_index, image->time_of_file_generation) == FAIL )
       {
         sprintf(emsg,"Could not read attribute.");
         xxx_LogStatus(progname,__FILE__,__LINE__,emsg);
         return(ERROR);
       }
       image->valid_map[2] = VALID;
     }
   }
   image->valid_map[3] = INVALID;
   if ( (attr_index = SDfindattr( sds_id, "Frame Number Range")) != FAIL )
   {
     if( SDreadattr(sds_id, attr_index, &image->frame_range) == FAIL )
     {
       sprintf(emsg,"Could not read attribute.");
       xxx_LogStatus(progname,__FILE__,__LINE__,emsg);
       return(ERROR);
     }
     image->valid_map[3] = VALID;
   }
   else
   {
     if ( (attr_index = SDfindattr( sd_id, "Frame Number Range")) != FAIL )
     {
       if( SDreadattr(sd_id, attr_index, &image->frame_range) == FAIL )
       {
         sprintf(emsg,"Could not read attribute.");
         xxx_LogStatus(progname,__FILE__,__LINE__,emsg);
         return(ERROR);
       }
       image->valid_map[3] = VALID;
     }
   }
   image->valid_map[4] = INVALID;
   if ( (attr_index = SDfindattr( sds_id, "Number of Cross Track Pixels")) != FAIL )
   {
     if( SDreadattr(sds_id, attr_index, &image->number_cross_track_pixels) 
         == FAIL )
     {
       sprintf(emsg,"Could not read attribute.");
       xxx_LogStatus(progname,__FILE__,__LINE__,emsg);
       return(ERROR);
     }
     image->valid_map[4] = VALID;
   }
   else
   {
     if ( (attr_index = SDfindattr( sd_id, "Number of Cross Track Pixels")) != FAIL )
     {
       if( SDreadattr(sd_id, attr_index, &image->number_cross_track_pixels)
           == FAIL )
       {
         sprintf(emsg,"Could not read attribute.");
         xxx_LogStatus(progname,__FILE__,__LINE__,emsg);
         return(ERROR);
       }
       image->valid_map[4] = VALID;
     }
   }
   image->valid_map[5] = INVALID;
   if ( (attr_index = SDfindattr( sds_id, "Number of Along Track Pixels")) != FAIL )
   {
     if( SDreadattr(sds_id, attr_index, &image->number_along_track_pixels) 
         == FAIL )
     {
       sprintf(emsg,"Could not read attribute.");
       xxx_LogStatus(progname,__FILE__,__LINE__,emsg);
       return(ERROR);
     }
     image->valid_map[5] = VALID;
   }
   else
   {
     if ( (attr_index = SDfindattr( sd_id, "Number of Along Track Pixels")) != FAIL )
     {
       if( SDreadattr(sd_id, attr_index, &image->number_along_track_pixels)
           == FAIL )
       {
         sprintf(emsg,"Could not read attribute.");
         xxx_LogStatus(progname,__FILE__,__LINE__,emsg);
         return(ERROR);
       }
       image->valid_map[5] = VALID;
     }
   }
   image->valid_map[6] = INVALID;
   if ( (attr_index = SDfindattr( sds_id, "Number of Bands")) != FAIL )
   {
     if( SDreadattr(sds_id, attr_index, &image->number_of_bands) == FAIL )
     {
       sprintf(emsg,"Could not read attribute.");
       xxx_LogStatus(progname,__FILE__,__LINE__,emsg);
       return(ERROR);
     }
     image->valid_map[6] = VALID;
   }
   else
   {
     if ( (attr_index = SDfindattr( sd_id, "Number of Bands")) != FAIL )
     {
       if( SDreadattr(sd_id, attr_index, &image->number_of_bands) == FAIL )
       {
         sprintf(emsg,"Could not read attribute.");
         xxx_LogStatus(progname,__FILE__,__LINE__,emsg);
         return(ERROR);
       }
       image->valid_map[6] = VALID;
     }
   }
   image->valid_map[7] = INVALID;
   if ( (attr_index = SDfindattr( sds_id, "Interleave Format")) != FAIL )
   {
     if( SDreadattr(sds_id, attr_index, &image->interleave_format) == FAIL )
     {
       sprintf(emsg,"Could not read attribute.");
       xxx_LogStatus(progname,__FILE__,__LINE__,emsg);
       return(ERROR);
     }
     image->valid_map[7] = VALID;
   }
   else
   {
     if ( (attr_index = SDfindattr( sd_id, "Interleave Format")) != FAIL )
     {
       if( SDreadattr(sd_id, attr_index, &image->interleave_format) == FAIL )
       {
         sprintf(emsg,"Could not read attribute.");
         xxx_LogStatus(progname,__FILE__,__LINE__,emsg);
         return(ERROR);
       }
       image->valid_map[7] = VALID;
     }
   }
   image->valid_map[8] = INVALID;
   if ( (attr_index = SDfindattr( sds_id, "Number of Pixels with Missing Data Flag")) 
         == FAIL )
   {
     if( SDreadattr(sds_id, attr_index, &image->pixels_with_missing_data_flag) == FAIL )
     {
       sprintf(emsg,"Could not read attribute Number of Pixels with Missing Data Flag.");
       xxx_LogStatus(progname,__FILE__,__LINE__,emsg);
       /* return(ERROR); */
     }
     else
        image->valid_map[8] = VALID;
   }
   image->valid_map[9] = INVALID;
   if ( (attr_index = SDfindattr( sds_id, "Scene_Id")) != FAIL )
   {
     if( SDreadattr(sds_id, attr_index, &image->scene_id) == FAIL )
     {
       sprintf(emsg,"Could not read attribute.");
       xxx_LogStatus(progname,__FILE__,__LINE__,emsg);
       return(ERROR);
     }
     image->valid_map[9] = VALID;
   }
   else
   {
     if ( (attr_index = SDfindattr( sd_id, "Scene_Id")) != FAIL )
     {
       if( SDreadattr(sd_id, attr_index, &image->scene_id) == FAIL )
       {
         sprintf(emsg,"Could not read attribute.");
         xxx_LogStatus(progname,__FILE__,__LINE__,emsg);
         return(ERROR);
       }
       image->valid_map[9] = VALID;
     }
   }
   image->valid_map[10] = INVALID;
   if ( (attr_index = SDfindattr( sds_id, "Target_Description")) != FAIL )
   {
     if( SDreadattr(sds_id, attr_index, &image->target_description) ==FAIL )
     {
       sprintf(emsg,"Could not read attribute.");
       xxx_LogStatus(progname,__FILE__,__LINE__,emsg);
       return(ERROR);
     }
     image->valid_map[10] = VALID;
   }
   else
   {
     if ( (attr_index = SDfindattr( sd_id, "Target_Description")) != FAIL )
     {
       if( SDreadattr(sd_id, attr_index, &image->target_description) ==FAIL )
       {
         sprintf(emsg,"Could not read attribute.");
         xxx_LogStatus(progname,__FILE__,__LINE__,emsg);
         return(ERROR);
       }
       image->valid_map[10] = VALID;
     }
   }
   image->valid_map[11] = INVALID;
   if ( (attr_index = SDfindattr( sds_id, "time_MET")) != FAIL )
   {
     if( SDreadattr(sds_id, attr_index, &image->time_met) == FAIL )
     {
       sprintf(emsg,"Could not read attribute.");
       xxx_LogStatus(progname,__FILE__,__LINE__,emsg);
       return(ERROR);
     }
     image->valid_map[11] = VALID;
   }
   else
   {
     if ( (attr_index = SDfindattr( sd_id, "time_MET")) != FAIL )
     {
       if( SDreadattr(sd_id, attr_index, &image->time_met) == FAIL )
       {
         sprintf(emsg,"Could not read attribute.");
         xxx_LogStatus(progname,__FILE__,__LINE__,emsg);
         return(ERROR);
       }
       image->valid_map[11] = VALID;
     }
   }
   image->valid_map[12] = INVALID;
   if ( (attr_index = SDfindattr( sds_id, "time_UTC")) != FAIL )
   {
     if( SDreadattr(sds_id, attr_index, &image->time_utc) == FAIL )
     {
       sprintf(emsg,"Could not read attribute.");
       xxx_LogStatus(progname,__FILE__,__LINE__,emsg);
       return(ERROR);
     }
     image->valid_map[12] = VALID;
   }
   else
   {
     if ( (attr_index = SDfindattr( sd_id, "time_UTC")) != FAIL )
     {
       if( SDreadattr(sd_id, attr_index, &image->time_utc) == FAIL )
       {
         sprintf(emsg,"Could not read attribute.");
         xxx_LogStatus(progname,__FILE__,__LINE__,emsg);
         return(ERROR);
       }
       image->valid_map[12] = VALID;
     }
   }
   image->valid_map[13] = INVALID;
   if ( (attr_index = SDfindattr( sds_id, "MET_UTC_adjust")) != FAIL )
   {
     if( SDreadattr(sds_id, attr_index, &image->met_utc_adjust) == FAIL )
     {
       sprintf(emsg,"Could not read attribute.");
       xxx_LogStatus(progname,__FILE__,__LINE__,emsg);
       return(ERROR);
     }
     image->valid_map[13] = VALID;
   }
   else
   {
     if ( (attr_index = SDfindattr( sd_id, "MET_UTC_adjust")) != FAIL )
     {
       if( SDreadattr(sd_id, attr_index, &image->met_utc_adjust) == FAIL )
       {
         sprintf(emsg,"Could not read attribute.");
         xxx_LogStatus(progname,__FILE__,__LINE__,emsg);
         return(ERROR);
       }
       image->valid_map[13] = VALID;
     }
   }
   image->valid_map[14] = INVALID;
   if ( (attr_index = SDfindattr( sds_id, "VNIR Sync Time [beginFrame,endFrame,syncTime]")) 
         != FAIL )
   {
     if( SDattrinfo(sds_id, attr_index, attr_name, &data_type, &n_values) == FAIL )
     {
       sprintf(emsg,"Could not read attribute info.");
       xxx_LogStatus(progname,__FILE__,__LINE__,emsg);
       return(ERROR);
     }
     image->vnir_sync_time = (int32 *)malloc( sizeof(int32) * n_values );
     if( image->vnir_sync_time == NULL )
     {
       sprintf(emsg,"Could not allocate vnir sync time.");
       xxx_LogStatus(progname,__FILE__,__LINE__,emsg);
       return(ERROR);
     }
     if( SDreadattr(sds_id, attr_index, image->vnir_sync_time) == FAIL )
     {
       sprintf(emsg,"Could not read attribute.");
       xxx_LogStatus(progname,__FILE__,__LINE__,emsg);
       return(ERROR);
     }
     image->valid_map[14] = VALID;
   }
   image->valid_map[15] = INVALID;
   if ( (attr_index = SDfindattr( sds_id, 
        "VNIR Time Code [beginFrame,endFrame,TC3,TC4,TC5,TC6,TC7,TC8]")) != FAIL )
   {
     if( SDattrinfo(sds_id, attr_index, attr_name, &data_type, &n_values) == FAIL )
     {
       sprintf(emsg,"Could not read attribute info.");
       xxx_LogStatus(progname,__FILE__,__LINE__,emsg);
       return(ERROR);
     }
     image->vnir_time_code = (int32 *)malloc( sizeof(int32) * n_values );
     if( image->vnir_time_code == NULL )
     {
       sprintf(emsg,"Could not allocate vnir sync time.");
       xxx_LogStatus(progname,__FILE__,__LINE__,emsg);
       return(ERROR);
     }
     if( SDreadattr(sds_id, attr_index, image->vnir_time_code) == FAIL )
     {
       sprintf(emsg,"Could not read attribute.");
       xxx_LogStatus(progname,__FILE__,__LINE__,emsg);
       return(ERROR);
     }
     image->valid_map[15] = VALID;
   }
   image->valid_map[16] = INVALID;
   if ( (attr_index = SDfindattr( sds_id, "VNIR Channel Offset [A,B,C,D]")) 
        != FAIL )
   {
     if( SDreadattr(sds_id, attr_index, &image->vnir_channel_offset) == FAIL )
     {
       sprintf(emsg,"Could not read attribute.");
       xxx_LogStatus(progname,__FILE__,__LINE__,emsg);
       return(ERROR);
     }
     image->valid_map[16] = VALID;
   }
   image->valid_map[17] = INVALID;
   if ( (attr_index = SDfindattr( sds_id, "SWIR Sync Time [beginFrame,endFrame,syncTime]")) 
        != FAIL )
   {
     if( SDattrinfo(sds_id, attr_index, attr_name, &data_type, &n_values) == FAIL )
     {
       sprintf(emsg,"Could not read attribute info.");
       xxx_LogStatus(progname,__FILE__,__LINE__,emsg);
       return(ERROR);
     }
     image->swir_sync_time = (int32 *)malloc( sizeof(int32) * n_values );
     if( image->swir_sync_time == NULL )
     {
       sprintf(emsg,"Could not allocate vnir sync time.");
       xxx_LogStatus(progname,__FILE__,__LINE__,emsg);
       return(ERROR);
     }
     if( SDreadattr(sds_id, attr_index, image->swir_sync_time) == FAIL )
     {
       sprintf(emsg,"Could not read attribute.");
       xxx_LogStatus(progname,__FILE__,__LINE__,emsg);
       return(ERROR);
     }
     image->valid_map[17] = VALID;
   }
   image->valid_map[18] = INVALID;
   if ( (attr_index = SDfindattr( sds_id, 
        "SWIR Time Code [beginFrame,endFrame,TC3,TC4,TC5,TC6,TC7,TC8]")) != FAIL )
   {
     if( SDattrinfo(sds_id, attr_index, attr_name, &data_type, &n_values) == FAIL )
     {
       sprintf(emsg,"Could not read attribute info.");
       xxx_LogStatus(progname,__FILE__,__LINE__,emsg);
       return(ERROR);
     }
     image->swir_time_code = (int32 *)malloc( sizeof(int32) * n_values );
     if( image->swir_time_code == NULL )
     {
       sprintf(emsg,"Could not allocate vnir sync time.");
       xxx_LogStatus(progname,__FILE__,__LINE__,emsg);
       return(ERROR);
     }
     if( SDreadattr(sds_id, attr_index, image->swir_time_code) == FAIL )
     {
       sprintf(emsg,"Could not read attribute.");
       xxx_LogStatus(progname,__FILE__,__LINE__,emsg);
       return(ERROR);
     }
     image->valid_map[18] = VALID;
   }
   image->valid_map[19] = INVALID;
   if ( (attr_index = SDfindattr( sds_id, "SWIR Channel Offset [A,B,C,D]")) 
        != FAIL )
   {
     if( SDreadattr(sds_id, attr_index, &image->swir_channel_offset) == FAIL )
     {
       sprintf(emsg,"Could not read attribute.");
       xxx_LogStatus(progname,__FILE__,__LINE__,emsg);
       return(ERROR);
     }
     image->valid_map[19] = VALID;
   }
   image->valid_map[20] = INVALID;
   if ( (attr_index = SDfindattr( sds_id, "SWIR Gain Setting [A,B,C,D]"))
        != FAIL )
   {
     if( SDreadattr(sds_id, attr_index, &image->swir_gain_setting) == FAIL )
     {
       sprintf(emsg,"Could not read attribute.");
       xxx_LogStatus(progname,__FILE__,__LINE__,emsg);
       return(ERROR);
     }
     image->valid_map[20] = VALID;
   }
   image->valid_map[21] = INVALID;
   if ( (attr_index = SDfindattr( sds_id, "SWIR Integration Time Setting"))
        != FAIL )
   {
     if( SDreadattr(sds_id, attr_index, &image->swir_gain_setting) == FAIL )
     {
       sprintf(emsg,"Could not read attribute.");
       xxx_LogStatus(progname,__FILE__,__LINE__,emsg);
       return(ERROR);
     }
     image->valid_map[21] = VALID;
   }

   if( SDendaccess(sds_id) == FAIL )
   {
      sprintf(emsg,"Ending SDS access error.");
      xxx_LogStatus(progname,__FILE__,__LINE__,emsg);
      return(ERROR);
   }

   if( SDend(sd_id) == FAIL )
   {
      sprintf(emsg,"Ending SD error.");
      xxx_LogStatus(progname,__FILE__,__LINE__,emsg);
      return(ERROR);
   }

   return(SUCCESS);
}
