/******************************************************************************
UNIT NAME:      xxx_write_ali_l0r_cal.c 

PURPOSE: Writes a Level 0R ALI (from EO-1 instrument) calibration data set.  All
	 data is in the structure xxx_ali_level0r.h.  It is up to the caller of
	 this routine to free up the memory associated with the L0R_CAL structure.        

*******************************************************************************/
#include <stdio.h>
#include "xxx_Types.h"
#include "xxx_LogStatus.h"
#include "xxx_ali_level0r.h"

int xxx_write_ali_l0r_cal(
int32 sd_id,                    /* I: SD identifier			*/
L0R_CAL cal)                    /* I: Structure containing L0R data	*/
{
   char emsg[STRLEN];		/* Error message buffer			*/
   char progname[STRLEN];	/* Program name				*/
   int32 sds_id;                /* SDS file ID                          */
   int32 *dim_sizes;            /* Dimensions of data                   */
   int32 start[3];		/* Starting dims of SDS "slab"		*/
   int32 edges[3];		/* Edge dims of SDS "slab"		*/
   int32 stride[3];		/* Increment dims of SDS "slab"		*/
   int32 size;			/* Size needed for array allocation	*/ 
   int32 attr_index;		/* Index of attributes			*/

   sprintf(progname,"WRITE_L0R_CAL");

   /**************************
    * Write calibration data *
    **************************/

   sds_id = SDcreate(sd_id, (char *)cal.name, cal.HDF_type, 
                     cal.number_of_dims, cal.dim_sizes );

   /* Set up dimensions of SDS "slab".
   ----------------------------------*/
   start[0]  = 0;
   start[1]  = 0;
   start[2]  = 0;

   stride[0] = 1;
   stride[1] = 1;
   stride[2] = 1;

   edges[0]  = cal.dim_sizes[0];    
   edges[1]  = cal.dim_sizes[1];
   edges[2]  = cal.dim_sizes[2];

   /* Write data into buffer
   -----------------------*/
   if( SDwritedata(sds_id, start, stride, edges, (VOIDP)cal.data) == FAIL )
   {
      strcpy(emsg,"Could not write SDS data\n");
      xxx_LogStatus(progname,__FILE__,__LINE__, emsg);
      return(ERROR);
   }

   if ( cal.attributes.valid_map[0] == VALID )
   {
      if( SDsetattr(sds_id, "ALI Sensor", DFNT_CHAR8, 4, 
                    cal.attributes.ali_sensor) == FAIL )
      {
         strcpy(emsg,"Could not write attribute.\n");
         xxx_LogStatus(progname,__FILE__,__LINE__, emsg);
         return(ERROR);
      }
   }
   if ( cal.attributes.valid_map[1] == VALID )
   {
      if( SDsetattr(sds_id, "Dataset Type", DFNT_CHAR8, 12, 
                    cal.attributes.dataset_type) == FAIL )
      {
         strcpy(emsg,"Could not write attribute.\n");
         xxx_LogStatus(progname,__FILE__,__LINE__, emsg);
         return(ERROR);
      }
   }
   if ( cal.attributes.valid_map[2] == VALID )
   {
      if( SDsetattr(sds_id, "Integration Time", DFNT_FLOAT32, 1,
                    &cal.attributes.integration_time) == FAIL )
      {
         strcpy(emsg,"Could not write attribute.\n");
         xxx_LogStatus(progname,__FILE__,__LINE__, emsg);
         return(ERROR);
      }
   }
   if ( cal.attributes.valid_map[3] == VALID )
   {
      if( SDsetattr(sds_id, "Frame Rate", DFNT_FLOAT32, 1, 
                    &cal.attributes.frame_rate) == FAIL )
      {
         strcpy(emsg,"Could not write attribute.\n");
         xxx_LogStatus(progname,__FILE__,__LINE__, emsg);
         return(ERROR);
      }
   }
   if ( cal.attributes.valid_map[4] == VALID )
   {
      if( SDsetattr(sds_id, "Data Gate Open Time", DFNT_CHAR8, 20, 
                    cal.attributes.data_gate_open_time) == FAIL )
      {
         strcpy(emsg,"Could not write attribute.\n");
         xxx_LogStatus(progname,__FILE__,__LINE__, emsg);
         return(ERROR);
      }
   }
   if ( cal.attributes.valid_map[5] == VALID )
   {
      if( SDsetattr(sds_id, "Data Gate Close Time", DFNT_CHAR8, 20, 
                    cal.attributes.data_gate_close_time) == FAIL )
      {
         strcpy(emsg,"Could not write attribute.\n");
         xxx_LogStatus(progname,__FILE__,__LINE__, emsg);
         return(ERROR);
      }
   }
   if ( cal.attributes.valid_map[6] == VALID )
   {
      if( SDsetattr(sds_id, "Number of cross track pixels", DFNT_INT16, 1, 
                    &cal.attributes.number_cross_track_pixels) == FAIL )
      {
         strcpy(emsg,"Could not write attribute.\n");
         xxx_LogStatus(progname,__FILE__,__LINE__, emsg);
         return(ERROR);
      }
   }
   if ( cal.attributes.valid_map[7] == VALID )
   {
      if( SDsetattr(sds_id, "Number of along track pixels", DFNT_INT32, 1, 
                    &cal.attributes.number_along_track_pixels) == FAIL )
      {
         strcpy(emsg,"Could not write attribute.\n");
         xxx_LogStatus(progname,__FILE__,__LINE__, emsg);
         return(ERROR);
      }
   }
   if ( cal.attributes.valid_map[8] == VALID )
   {
      if( SDsetattr(sds_id, "Number of bands", DFNT_INT16, 1, 
                    &cal.attributes.number_of_bands) == FAIL )
      {
         strcpy(emsg,"Could not write attribute.\n");
         xxx_LogStatus(progname,__FILE__,__LINE__, emsg);
         return(ERROR);
      }
   }
   if ( cal.attributes.valid_map[9] == VALID )
   {
      if( SDsetattr(sds_id, "Percent Missing Data", DFNT_INT16, 1, 
                    &cal.attributes.percent_missing_data) == FAIL )
      {
         strcpy(emsg,"Could not write attribute.\n");
         xxx_LogStatus(progname,__FILE__,__LINE__, emsg);
         return(ERROR);
      }
   }

   if( SDendaccess(sds_id) == FAIL )
   {
      strcpy(emsg,"Ending SDS access error.\n");
      xxx_LogStatus(progname,__FILE__,__LINE__, emsg);
      return(ERROR);
   }

   return(SUCCESS);
}
