/******************************************************************************
UNIT NAME:      xxx_write_ali_l0r_image.c 

PURPOSE: Writes a Level 0R ALI (from EO-1 instrument) image data set.  All
	 data is in the structure xxx_ali_level0r.h.  It is up to the caller of
	 this routine to free the memory associated with the L0R_IMAGE structure.

*******************************************************************************/
#include <stdio.h>
#include "xxx_Types.h"
#include "xxx_LogStatus.h"
#include "xxx_ali_level0r.h"

int xxx_write_ali_l0r_image(
int32 sd_id,                   /* I: Name of Level 0R file		*/
L0R_IMAGE image)               /* I: Structure containing L0R data	*/
{
   char emsg[STRLEN];		/* Error message buffer			*/
   char progname[STRLEN];	/* Program name				*/
   int32 sds_id;                /* SDS file ID                          */
   int32 *dim_sizes;            /* Dimensions of data                   */
   int32 start[3];		/* Starting dims of SDS "slab"		*/
   int32 edges[3];		/* Edge dims of SDS "slab"		*/
   int32 stride[3];		/* Increment dims of SDS "slab"		*/
   int32 size;			/* Size needed for array allocation	*/ 
   int32 attr_index;		/* Index of attributes			*/

   sprintf(progname,"WRITE_L0R_IMAGE");

   /*************************
    * Read image data *
    *************************/

   sds_id = SDcreate(sd_id, (char *)image.name, image.HDF_type, 
                     image.number_of_dims, image.dim_sizes );

   /* Set up dimensions of SDS "slab".
   ----------------------------------*/
   start[0]  = 0; 
   start[1]  = 0;
   start[2]  = 0;

   stride[0] = 1;
   stride[1] = 1;
   stride[2] = 1;

   edges[0]  = image.dim_sizes[0];  
   edges[1]  = image.dim_sizes[1];
   edges[2]  = image.dim_sizes[2];

   /* Write data into buffer
   -----------------------*/
   if( SDwritedata(sds_id, start, stride, edges, (VOIDP)image.data) == FAIL )
   {
      strcpy(emsg,"Could not write SDS data\n");
      xxx_LogStatus(progname,__FILE__,__LINE__, emsg);
      return(ERROR);
   }

   if ( image.attributes.valid_map[0] == VALID )
   {
      if( SDsetattr(sds_id, "ALI Sensor", DFNT_CHAR8, 4, 
                    image.attributes.ali_sensor) == FAIL )
      {
         strcpy(emsg,"Could not write attribute.\n");
         xxx_LogStatus(progname,__FILE__,__LINE__, emsg);
         return(ERROR);
      }
   }
   if ( image.attributes.valid_map[1] == VALID )
   {
      if( SDsetattr(sds_id, "Dataset Type", DFNT_CHAR8, 12, 
                    image.attributes.dataset_type) == FAIL )
      {
         strcpy(emsg,"Could not write attribute.\n");
         xxx_LogStatus(progname,__FILE__,__LINE__, emsg);
         return(ERROR);
      }
   }
   if ( image.attributes.valid_map[2] == VALID )
   {
      if( SDsetattr(sds_id, "Data Start Time", DFNT_CHAR8, 14, 
                    image.attributes.data_start_time) == FAIL )
      {
         strcpy(emsg,"Could not write attribute.\n");
         xxx_LogStatus(progname,__FILE__,__LINE__, emsg);
         return(ERROR);
      }
   }
   if ( image.attributes.valid_map[3] == VALID )
   {
      if( SDsetattr(sds_id, "Data Duration", DFNT_FLOAT32, 1, 
                    &image.attributes.data_duration) == FAIL )
      {
         strcpy(emsg,"Could not write attribute.\n");
         xxx_LogStatus(progname,__FILE__,__LINE__, emsg);
         return(ERROR);
      }
   }
   if ( image.attributes.valid_map[4] == VALID )
   {
      if( SDsetattr(sds_id, "Integration Time", DFNT_FLOAT32, 1, 
                    &image.attributes.integration_time) == FAIL )
      {
         strcpy(emsg,"Could not write attribute.\n");
         xxx_LogStatus(progname,__FILE__,__LINE__, emsg);
         return(ERROR);
      }
   }
   if ( image.attributes.valid_map[5] == VALID )
   {
      if( SDsetattr(sds_id, "Frame Rate", DFNT_FLOAT32, 1, 
                    &image.attributes.frame_rate) == FAIL )
      {
         strcpy(emsg,"Could not write attribute.\n");
         xxx_LogStatus(progname,__FILE__,__LINE__, emsg);
         return(ERROR);
      }
   }
   if ( image.attributes.valid_map[6] == VALID )
   {
      if( SDsetattr(sds_id, "Data Gate Open Time", DFNT_CHAR8, 20, 
                    image.attributes.data_gate_open_time) == FAIL )
      {
         strcpy(emsg,"Could not write attribute.\n");
         xxx_LogStatus(progname,__FILE__,__LINE__, emsg);
         return(ERROR);
      }
   }
   if ( image.attributes.valid_map[7] == VALID )
   {
      if( SDsetattr(sds_id, "Data Gate Close Time", DFNT_CHAR8, 20, 
                    image.attributes.data_gate_close_time) == FAIL )
      {
         strcpy(emsg,"Could not write attribute.\n");
         xxx_LogStatus(progname,__FILE__,__LINE__, emsg);
         return(ERROR);
      }
   }
   if ( image.attributes.valid_map[8] == VALID )
   {
      if( SDsetattr(sds_id, "Bytes prior to SYNC", DFNT_INT32, 1, 
                    &image.attributes.bytes_prior_sync) == FAIL )
      {
         strcpy(emsg,"Could not write attribute.\n");
         xxx_LogStatus(progname,__FILE__,__LINE__, emsg);
         return(ERROR);
      }
   }
   if ( image.attributes.valid_map[9] == VALID )
   {
      if( SDsetattr(sds_id, "Number of cross track pixels", DFNT_INT16, 1, 
                    &image.attributes.number_cross_track_pixels) == FAIL )
      {
         strcpy(emsg,"Could not write attribute.\n");
         xxx_LogStatus(progname,__FILE__,__LINE__, emsg);
         return(ERROR);
      }
   }
   if ( image.attributes.valid_map[10] == VALID )
   {
      if( SDsetattr(sds_id, "Number of along track pixels", DFNT_INT32, 1, 
                    &image.attributes.number_along_track_pixels) == FAIL )
      {
         strcpy(emsg,"Could not write attribute.\n");
         xxx_LogStatus(progname,__FILE__,__LINE__, emsg);
         return(ERROR);
      }
   }
   if ( image.attributes.valid_map[11] == VALID )
   {
      if( SDsetattr(sds_id, "Number of bands", DFNT_INT16, 1, 
                    &image.attributes.number_of_bands) == FAIL )
      {
         strcpy(emsg,"Could not write attribute.\n");
         xxx_LogStatus(progname,__FILE__,__LINE__, emsg);
         return(ERROR);
      }
   }
   if ( image.attributes.valid_map[12] == VALID )
   {
      if( SDsetattr(sds_id, "Percent Missing Data", DFNT_INT16, 1, 
                    &image.attributes.percent_missing_data) == FAIL )
      {
         strcpy(emsg,"Could not write attribute.\n");
         xxx_LogStatus(progname,__FILE__,__LINE__, emsg);
         return(ERROR);
      }
   }

   if( SDendaccess(sds_id) == FAIL )
   {
      strcpy(emsg,"Ending SDS access error.\n");
      xxx_LogStatus(progname,__FILE__,__LINE__, emsg);
      return(ERROR);
   }

   return(SUCCESS);
}
