#ifndef _XXX_PARALLEL_CORR_H_
#define _XXX_PARALLEL_CORR_H_

#ifdef XXX_THREAD
#include <semaphore.h> /* semaphore prototypes */
#include <pthread.h>   /* pthread library */
#endif
#include "xxx_const.h" /* CorrFitMethod definition */

/*
**
** MACROS
**
*/

#define MAX_CORR_THREADS 6
/* defines the maximum number of correlation threads to allow */

/*
**
** TYPE DEFINITIONS
**
*/

/* NOTE: This structures should NEVER be accessed by any routines outside
   the xxx_parallel_corr module.  They are included here to allow the
   XXX_PARALLEL_CORRELATOR_TYPE pointer to be available to the calling
   routines only for the purpose of calling the parallel correlator 
   routines.. */
   
typedef struct
{
    int valid;  /* set to non-zero if the correlation was valid */
    float fit_offset[2]; /* array to hold the horizontal and vertical 
                            correlation fit offset */
    float strength;      /* strength of correlation */
    float est_err[3]; /* Est horiz error, vert error, and h-v cross term in
                         best-fit offsets (3 values) */
    float diag_disp;  /* Actual diagonal displacement from nominal tiepoint
                         location to location found by correlation */
} XXX_CORRELATION_RESULT_TYPE;
/* this type stores the results of a single correlation. */

typedef struct
{
    int chip_index;          /* index to store results at in results array */
    float *search_image_ptr; /* search image */
    float *ref_image_ptr;    /* reference image */
    int search_size[2];      /* size of search image (samp,line) */
    int ref_size[2];         /* size of reference image (samp,line) */
    float min_corr;          /* minimum acceptable correlation strength */
    CorrFitMethod fit_method;/* fit method to use */
    float max_disp;          /* maximum allowed diagnol displacement */
    float nominal_offset[2]; /* nominal offset of UL corner of ref to search
                                (samp,line) */
    int abs_corr_coeff_flag; /* flag to use the absolute of the correlation
                                coefficients */
} CORRELATE_DATA_TYPE;

#ifdef XXX_THREAD
typedef struct
{
    volatile int insert_index; /* index of next queue insert */
    volatile int remove_index; /* index of next queue remove */
    int queue_size;            /* size of queue */
    sem_t semaphore;           /* semaphore for inserting data into queue */
    pthread_mutex_t remove_mutex; /*mutex to lock for removing data from queue*/
    CORRELATE_DATA_TYPE *data; /* pointer to the correlate queue data */
     
} CORRELATE_QUEUE_TYPE;

typedef struct
{
    float *ref_chip_ptr;    /* pointer to reference chip buffer */
    float *search_chip_ptr; /* pointer to search chip buffer */
    
} CHIP_BUFFER_DATA_TYPE;

typedef struct 
{
    volatile int insert_index; /* index of next queue insert */
    volatile int remove_index; /* index of next queue remove */
    int queue_size;            /* size of queue */
    sem_t semaphore;           /* semaphore for inserting data into queue */
    pthread_mutex_t insert_mutex; /*mutex to lock for inserting data into queue*/
    CHIP_BUFFER_DATA_TYPE *data; /* pointer to the CHIP queue data */
    float* buffers;            /* pointer to the memory allocated as the 
                                  free chip buffers in one hunk */
    
} FREE_CHIP_BUFFER_QUEUE_TYPE;
/* defines the data stored in the free chip buffer queue.  Note that only one 
   thread should ever be removing data from the queue.  If the design ever 
   changes to have multiple threads removing data from the queue, another mutex
   should be created for removing data. */
#endif

typedef struct 
{
    /* flags to check for each correlation */
    int check_edge_flag;
    int check_mult_peak_flag;
    int check_low_peak_flag;
    int check_max_disp_flag;
 
    /* maximum reference and search chip dimensions */
    int max_ref_chip_lines;
    int max_ref_chip_samples;
    int max_search_chip_lines;
    int max_search_chip_samples;
#ifdef XXX_THREAD
    int threads;   /* number of correlation threads created */
   
    FREE_CHIP_BUFFER_QUEUE_TYPE chip_buffer_queue; /* free chip buffers queue*/

    CORRELATE_QUEUE_TYPE correlate_queue; /* correlate queue */
    
    pthread_t thread_ids[MAX_CORR_THREADS]; /* correlation thread ids */

    int error_flag; /* flag to indicate an error was encountered in a 
                       correlation thread.  All other correlation threads
                       should terminate as should the xxx_get_corr_chip_buffers
                       routine. */
#else
    /* buffers for when not multithreaded */
    float *ref_chip_ptr;    /* pointer to reference chip buffer */
    float *search_chip_ptr; /* pointer to search chip buffer */
#endif

    XXX_CORRELATION_RESULT_TYPE *results_ptr; /* pointer to the correlation
                                                 results storage */
    
} XXX_PARALLEL_CORRELATOR_TYPE;
/* structure that defines a parallel correlator */
   
/*
**
** PROTOTYPES
**
*/

XXX_PARALLEL_CORRELATOR_TYPE *xxx_init_parallel_correlator
(
    int check_edge_flag,  /* I: set to non-zero to check the corr edge flag */
    int check_mult_peak_flag, /* I: set to non-zero to check multiple peak flag*/
    int check_low_peak_flag, /* I: set to non-zero to check low peak flag */
    int check_max_disp_flag, /* I: set to non-zero to check max disp flag */
    int max_ref_chip_lines,  /* I: max lines in reference chip */
    int max_ref_chip_samples, /* I: max samples across reference chip */
    int max_search_chip_lines, /* I: max lines in search chip */
    int max_search_chip_samples, /* I: max samples across search chip */
    XXX_CORRELATION_RESULT_TYPE *results_ptr /* I: pointer to results array */
);

int xxx_get_corr_chip_buffers
(
    XXX_PARALLEL_CORRELATOR_TYPE *correlator_ptr, /* I: parallel correlator */
    float **ref_buffer_ptr, /* O: pointer to reference chip buffer */
    float **search_buffer_ptr /* O: pointer to search chip buffer */
);

int xxx_submit_chip_to_corr
(
    XXX_PARALLEL_CORRELATOR_TYPE *correlator_ptr, /* I: parallel correlator */
    int chip_index,          /* I: index to store results at in results array */
    float *search_image_ptr, /* I: search image */
    float *ref_image_ptr,    /* I: reference image */
    int search_size[2],      /* I: size of search image (samp,line) */
    int ref_size[2],         /* I: size of reference image (samp,line) */
    float min_corr,          /* I: minimum acceptable correlation strength */
    CorrFitMethod fit_method,/* I: fit method to use */
    float max_disp,          /* I: maximum allowed diagnol displacement */
    float nominal_offset[2], /* I: nominal offset of UL corner of ref to search
                                   (samp,line) */
    int abs_corr_coeff_flag  /* I: flag to use the absolute of the correlation
                                   coefficients */    
);

int xxx_close_parallel_correlator
(
    XXX_PARALLEL_CORRELATOR_TYPE *correlator_ptr /* I: parallel correlator */
);

#endif
