/******************************************************************************
NAME:		gxx_update_gcplib_corr

PURPOSE:
Updates the GCP library file.  The number of attempts and the number of
successful correlates are incremented for the appropriate GCPs.

DEVELOPMENT HISTORY:
$Log: gxx_update_gcplib_corr.c,v $
Revision 1.1.1.1  2000/10/19 14:37:35  morfitt
ram:  Inital EO1 processing system.

Revision 1.1.1.1  1999/12/01 19:01:20  mbs
Import of IAS 3.22


RETURN VALUE:
type=int
Value		Description
-----		-----------
SUCCESS		Success
ERROR		Failure

******************************************************************************
		Landsat 7 Image Assessment System (IAS) Software
			Property of the U.S. Government
				NASA/GSFC/EDC
******************************************************************************/
#include "gxx_utility.h"

int gxx_update_gcplib_corr(
    struct GCPDATA gcpdata[],	/* I: ground control point data structure */
    char *gcplib_file_name,	/* I: Name of the GCPLIB file */
    long numgcp)		/* I: number of ground control points */
{
long adjust;		/* value to adjust the pointer */
long count;		/* counter of number of points updated */
long curpt;		/* gcpdata looping variable */
long position;		/* pointer position of the current record */
long pos_next;		/* pointer position of the next record */
long zero = 0;		/* zero */
char msg[STRLEN];	/* message for errors */
char buffer[300];	/* string read in from file */
GCPLIB gcp_lib;		/* gcp library structure */
FILE *ptr;		/* pointer to the gcplib file */

/* Open the GCP lib file file
  --------------------------*/
if ((ptr=fopen(gcplib_file_name,"r+")) == NULL)
    {
    xxx_LogStatus("GPSLIB",__FILE__,__LINE__,"Error getting gcp library file");
    return(ERROR);
    }

/* Read the file until end of file or BEGIN flag to bypass header
  -------------------------------------------------------------*/
for(;;)
    {
    if (fgets(buffer, 300, ptr) == NULL)
        {
	strcpy(msg,"No valid data found in GCP library file");
	xxx_LogStatus("GPSLIB",__FILE__,__LINE__,msg);
	fclose(ptr);
        return(ERROR);
        }
    if (strncmp(buffer, "BEGIN", 5) == 0)
        break;
    }

/* Get the chip information from file
  ---------------------------------*/
count=0;

for(;;)
    {
    if (fscanf(
              ptr,"%s%s%lf%lf%lf%lf%lf%lf%lf%lf%lf%lf%lf%s%s%s\n",
		gcp_lib.pt_id,gcp_lib.chip_name,
		&gcp_lib.ref_line, &gcp_lib.ref_samp,
		&gcp_lib.lat,&gcp_lib.lon,
		&gcp_lib.projy, &gcp_lib.projx,
		&gcp_lib.elev, &gcp_lib.pix_size[0],
		&gcp_lib.pix_size[1],&gcp_lib.ref_size[0],
		&gcp_lib.ref_size[1],gcp_lib.source,
		gcp_lib.date,gcp_lib.abs_rel_flag) != CHIPREC)
	break;
    position = ftell(ptr);
    if (fscanf(ptr,"%d%d%d\n", &gcp_lib.attempts,
		&gcp_lib.good_corr, &gcp_lib.good_prec) != CHIPOUT)
	break;

    /* see if this point is in the list of gcpdata points
     ---------------------------------------------------*/
    for (curpt = 0; curpt < numgcp; curpt++)
        {

        /* test the GCP id
         -----------------*/
        if (strcmp(gcp_lib.pt_id,gcpdata[curpt].pt_id) == 0)
	    {
	    /* To ensure that the next chip is not written over, determine the
	       number of digits in the attempt value, subtract the number of 
	       digits less than four to the pointer.  This gives room for the 
	       number of digits to grow.
	     ----------------------------------------------------------------*/
	    if (gcp_lib.attempts < 10)
	        adjust = 3;
	    else if (gcp_lib.attempts < 100)
	        adjust = 2;
	    else if (gcp_lib.attempts < 1000)
	        adjust = 1;
	    else
	        adjust = 0;
	    position = position - adjust;

	    /* Increment counters
	      ------------------*/
	    (gcp_lib.attempts)++;
	    if (gcpdata[curpt].accept_flg)
	        (gcp_lib.good_corr)++;


	    /* Replace data in file with updates
	      ---------------------------------*/
            pos_next = ftell(ptr);
	    if (fseek(ptr,position,SEEK_SET) != zero)
	       {
	       xxx_LogStatus("GPSLIB",__FILE__,__LINE__,
	           "Error seeking gcp library file");
	       fclose(ptr);
	       return(ERROR);
	       }
	    if (fprintf(ptr,"%4d %4d %4d\n",gcp_lib.attempts,
			gcp_lib.good_corr,gcp_lib.good_prec) < zero)
	        {
	        xxx_LogStatus("GPSLIB",__FILE__,__LINE__,
		    "Error writing to gcp library file");
	        fclose(ptr);
	        return(ERROR);
	        }
	    /* seek to the next position, this may not be necessary but may
	       be a safety feature of sorts
	      -----------------------------------------------------------*/
	    count++;
	    if (fseek(ptr,pos_next,SEEK_SET) != zero)
	       {
	       xxx_LogStatus("GPSLIB",__FILE__,__LINE__,
	           "Error seeking gcp library file");
	       fclose(ptr);
	       return(ERROR);
	       }

	    /* when the point is found, stop checking for it
	      ---------------------------------------------*/
	    break;
	    }
	}
    }
fclose(ptr);

/* Test that all points were updated
  ---------------------------------*/
if (count != numgcp)
    {
    sprintf(msg,
        "Not all points found in gcp library file: found %d, expected %d",
	count,numgcp);
    xxx_LogStatus("GPSLIB",__FILE__,__LINE__,msg);
    return(ERROR);
    }

return(SUCCESS);
}
