/******************************************************************************
NAME:		gxx_update_gcplib_prec

PURPOSE:
Updates the GCP library file.  The number of successful precision correlations
is incremented for the appropriate GCPs.

DEVELOPMENT HISTORY:
$Log: gxx_update_gcplib_prec.c,v $
Revision 1.1.1.1  2000/10/19 14:37:35  morfitt
ram:  Inital EO1 processing system.

Revision 1.1.1.1  1999/12/01 19:01:20  mbs
Import of IAS 3.22


RETURN VALUE:
type=int
Value		Description
-----		-----------
SUCCESS		Success
ERROR		Failure

******************************************************************************
		Landsat 7 Image Assessment System (IAS) Software
			Property of the U.S. Government
				NASA/GSFC/EDC
******************************************************************************/
#include "gxx_utility.h"

int gxx_update_gcplib_prec(
    GCP_RESIDUAL gcp_res[],	/* I: GCP precision residual information */
    char *gcplib_file_name,	/* I: Name of the GCPLIB file */
    long numgcp)		/* I: number of ground control points */
{
long adjust;		/* value to adjust the pointer */
long count;		/* counter of updated points */
long curpt;		/* looping variable for gcp residuals */
long position;		/* pointer position of the current record */
long pos_next;		/* pointer position of the next record */
long zero = 0;		/* zero */
char buffer[300];	/* string read in from file */
struct GCPLIB gcp_lib;	/* gcp library structure */
FILE *ptr;		/* pointer to the gcplib file */

/* Open the GCP lib file file
  --------------------------*/
if ((ptr=fopen(gcplib_file_name,"r+")) == NULL)
    {
    xxx_LogStatus("GPSLIB",__FILE__,__LINE__,
        "Error opening gcp library file");
    return(ERROR);
    }

/* Read the file until end of file or BEGIN flag to bypass header
  -------------------------------------------------------------*/
for(;;)
    {
    if (fgets(buffer, 300, ptr) == NULL)
        {
  	xxx_LogStatus("GPSLIB",__FILE__,__LINE__,
            "No valid data found in gcp library");
	fclose(ptr);
        return(ERROR);
        }
    if (strncmp(buffer, "BEGIN", 5) == 0)
        break;
    }

/* Get the chip information from file
  ---------------------------------*/
count=0;

for(;;)
    {
    if (fscanf(ptr,"%s%s%lf%lf%lf%lf%lf%lf%lf%lf%lf%lf%lf%s%s%s\n",
		gcp_lib.pt_id,gcp_lib.chip_name, &gcp_lib.ref_line,
		&gcp_lib.ref_samp, &gcp_lib.lat,&gcp_lib.lon, &gcp_lib.projy,
		&gcp_lib.projx, &gcp_lib.elev, &gcp_lib.pix_size[0],
		&gcp_lib.pix_size[1],&gcp_lib.ref_size[0],&gcp_lib.ref_size[1],
		gcp_lib.source, gcp_lib.date,gcp_lib.abs_rel_flag) != CHIPREC)
	break;
    position = ftell(ptr);
    if (fscanf(ptr,"%d%d%d\n", &gcp_lib.attempts,
		&gcp_lib.good_corr, &gcp_lib.good_prec) != CHIPOUT)
	break;

    /* look for the GCP in the list of gcp_res
     ----------------------------------------*/
    for (curpt = 0; curpt < numgcp; curpt++)
        {
        /* test the GCP id
         -----------------*/
        if (strcmp(gcp_lib.pt_id,gcp_res[curpt].pt_id) == 0)
	    {
	    /* To ensure that the next chip is not written over, determine the
	       number of digits in the attempt value, subtract the number of 
	       digits less than four to the pointer.  This gives room for the 
	       number of digits to grow.
	     -----------------------------------------------------------------*/
	    if (gcp_lib.attempts < 10)
	        adjust = 3;
	    else if (gcp_lib.attempts < 100)
	        adjust = 2;
	    else if (gcp_lib.attempts < 1000)
	        adjust = 1;
	    else
	        adjust = 0;
	    position = position - adjust;

	    /* Increment counter if not an outlier
	      -----------------------------------*/
	    if (gcp_res[curpt].outlier_flag == 0)
	        (gcp_lib.good_prec)++;


	    /* Replace data in file with updates
	      ---------------------------------*/
            pos_next = ftell(ptr);
	    if (fseek(ptr,position,SEEK_SET) != zero)
	        {
    	        xxx_LogStatus("GPSLIB",__FILE__,__LINE__,
                    "Error seeking gcp library file");
	        fclose(ptr);
	        return(ERROR);
	        }
	    if (fprintf(ptr,"%4d %4d %4d\n",gcp_lib.attempts,
			gcp_lib.good_corr,gcp_lib.good_prec) < zero)
	        {
    	        xxx_LogStatus("GPSLIB",__FILE__,__LINE__,
                    "Error writing to gcp library file");
	        fclose(ptr);
	        return(ERROR);
	        }

	    /* seek to the next position, this may not be necessary but may
	       be a safety feature of sorts (also increment the count of
	       updated points)
	      -----------------------------------------------------------*/
	    count++;
	    if (fseek(ptr,pos_next,SEEK_SET) != zero)
	       {
    	       xxx_LogStatus("GPSLIB",__FILE__,__LINE__,
                   "Error seeking gcp library file");
	       fclose(ptr);
	       return(ERROR);
	       }

	    /* Once found, quit looking
	     -------------------------*/
	    break;
	    }
	}
    }
fclose(ptr);

/* Test that all points were updated
  ---------------------------------*/
if (count != numgcp)
    {
    xxx_LogStatus("GPSLIB",__FILE__,__LINE__,
        "Not all points found in gcp library file");
    return(ERROR);
    }

return(SUCCESS);
}
