#include <stdlib.h>
#include "mfhdf.h"
#include "xxx_EDCOdl.h"
#include "xxx_Types.h"     /* SUCCESS/ERROR definitions */
#include "xxx_LogStatus.h" /* xxx_LogStatus prototype */
#include "proj.h"		   /* SOM definition */
#include "xxx_geom_grid.h"

struct ELEMENT_ATTRIBS
{
  char name[STRLEN]; /* Name of the field                               */
  int  dfnt;         /* DFNT type                                       */
  int  size;         /* Size (number of bytes) of an individual element */
  int  order;        /* Order/number of the elements                    */
};

/* Structure for the xxx_GEOM_GRID_TYPE data.
 */
static struct ELEMENT_ATTRIBS grid_type_elements[] =
{
  {"gridname", DFNT_CHAR, sizeof (char), GRIDNAME_SIZE},
  {"gridtype", DFNT_INT32, sizeof(int32),  1},
  {"nbands", DFNT_INT32, sizeof (int32), 1},
  {"bandlist", DFNT_INT32, sizeof (int32), MAX_GEOM_GRID_BANDS},
  {"dtype", DFNT_INT32, sizeof (int32), 1},
  {"correction_type", DFNT_INT32, sizeof (int32), 1},
  {"frame_type", DFNT_INT32, sizeof (int32), 1},
  {"units", DFNT_CHAR, sizeof (char), UNITS_SIZE},
  {"code", DFNT_INT32, sizeof (int32), 1},
  {"datum", DFNT_CHAR, sizeof (char), DATUM_SIZE},
  {"spheroid", DFNT_INT32, sizeof (int32), 1},
  {"zone", DFNT_INT32, sizeof (int32), 1},
  {"projprms", DFNT_DOUBLE, sizeof (double), PROJPRMS_SIZE},
  {"cornerX", DFNT_DOUBLE, sizeof (double), CORNERS_SIZE},
  {"cornerY", DFNT_DOUBLE, sizeof (double), CORNERS_SIZE},
  {"center_pt", DFNT_DOUBLE, sizeof (double), XY_SIZE},
  {"proj_center_pt", DFNT_DOUBLE, sizeof (double), XY_SIZE},
  {"rotate_angle", DFNT_DOUBLE, sizeof (double), 1},
  {"orient_angle", DFNT_DOUBLE, sizeof (double), 1}
};

/* Total number of ints, chars, and doubles in the grid_type_elements
 * structure.
 */
#define NUM_GRID_TYPE_INTS   (8 + MAX_GEOM_GRID_BANDS)
#define NUM_GRID_TYPE_CHARS  (GRIDNAME_SIZE + UNITS_SIZE + DATUM_SIZE)
#define NUM_GRID_TYPE_DBLS   (PROJPRMS_SIZE + 2*CORNERS_SIZE + 2*XY_SIZE + 2)

/* Structure for constants in the xxx_GEOM_GRID_BAND_TYPE data.  This structure
 * will store all bands together.
 */
static struct ELEMENT_ATTRIBS constant_elements[] =
{
  {"bandno", DFNT_INT32, sizeof (int32), 1},
  {"gridtype", DFNT_INT32, sizeof(int32), 1},
  {"grid_cell_sizeX", DFNT_INT32, sizeof (int32), 1},
  {"grid_cell_sizeY", DFNT_INT32, sizeof (int32), 1},
  {"pixsizeX", DFNT_DOUBLE, sizeof (double), 1},
  {"pixsizeY", DFNT_DOUBLE, sizeof (double), 1},
  {"output_lines", DFNT_INT32, sizeof (int32), 1},
  {"output_samps", DFNT_INT32, sizeof (int32), 1},
  {"num_in_lines", DFNT_INT32, sizeof (int32), 1},
  {"num_in_samps", DFNT_INT32, sizeof (int32), 1},
  {"num_out_lines", DFNT_INT32, sizeof (int32), 1},
  {"num_out_samps", DFNT_INT32, sizeof (int32), 1},
  {"forward_present", DFNT_INT32, sizeof (int32), 1},
  {"reverse_present", DFNT_INT32, sizeof (int32), 1},
  {"grid_rows", DFNT_INT32, sizeof (int32), 1},
  {"grid_cols", DFNT_INT32, sizeof (int32), 1},
  {"degree", DFNT_INT32, sizeof (int32), 1}
};

/* Total number of ints, chars, and doubles in the constant_elements structure.
 */
#define NUM_CONSTANT_INTS 15
#define NUM_CONSTANT_DBLS 2

/* Structure for the in_lines data elements.
 */
static struct ELEMENT_ATTRIBS in_lines_elements[] =
{
  {"in_lines", DFNT_DOUBLE, sizeof (double), 0}
};

/* Structure for the in_samps data elements.
 */
static struct ELEMENT_ATTRIBS in_samps_elements[] =
{
  {"in_samps", DFNT_DOUBLE, sizeof (double), 0}
};

/* Structures for the out_lines and out_samps data elements.
 */
static struct ELEMENT_ATTRIBS out_lines_elements[] =
{
  {"out_lines", DFNT_DOUBLE, sizeof (double), 0}
};

static struct ELEMENT_ATTRIBS out_samps_elements[] =
{
  {"out_samps", DFNT_DOUBLE, sizeof (double), 0}
};

/* Structure for the poly_lines and poly_samps data elements.
 */
static struct ELEMENT_ATTRIBS poly_ls_elements[] =
{
  {"poly_lines", DFNT_DOUBLE, sizeof (double), 0},
  {"poly_samps", DFNT_DOUBLE, sizeof (double), 0}
};

/* Structure for the forward and reverse coefficients data elements.
 */
static struct ELEMENT_ATTRIBS forward_coeffs_elements[] =
{
  {"forward_coeffs", DFNT_DOUBLE, sizeof (double), 0}
};
static struct ELEMENT_ATTRIBS reverse_coeffs_elements[] =
{
  {"reverse_coeffs", DFNT_DOUBLE, sizeof (double), 0}
};

#define NGRIDTYPE (sizeof(grid_type_elements) / sizeof(struct ELEMENT_ATTRIBS))
#define NCONSTANT (sizeof(constant_elements) / sizeof(struct ELEMENT_ATTRIBS))

/* Define the number of SDSs, one for in_lines, in_samps, poly_ls
 * and out_lines, out_samps,  forward_coeffs, and reverse_coeffs.
 */
#define NUM_SDS 7

/******************************************************************************
NAME:           xxx_put_geom_grid

PURPOSE:
Write the geometric grid to an HDF file.

RETURN VALUE:   
Type = int
Value    Description
-----    -----------
SUCCESS  Successful completion
ERROR    Operation failed

NOTES:   This function is a bridge routine to the actual xxx_write_geom_grid
         support.  This function only flips SOM corner coorindates
         before they are sent to be written.  This allows a consistant
         user interface, yet allows the algorithms using SOM coodinates
         not to change.

ALGORITHM REFERENCES:
See xxx_geom_grid.h for a description of the geometric grid structure.

******************************************************************************/

int xxx_put_geom_grid 
(
  char grid_filename[STRLEN],/* I: HDF output file name */
  xxx_GEOM_GRID_TYPE *grid   /* I: Grid structure to be output */
)
{
double tmp;		     /* Temporary variable for flipping corner points*/

/*
  Flip the corner points for the SOM projection ONLY
*/
if (grid->code == SOM)
    {
    tmp = -grid->corners.upleft.y;
    grid->corners.upleft.y = grid->corners.upleft.x;
    grid->corners.upleft.x = tmp;

    tmp = -grid->corners.loleft.y;
    grid->corners.loleft.y = grid->corners.loleft.x;
    grid->corners.loleft.x = tmp;

    tmp = -grid->corners.upright.y;
    grid->corners.upright.y = grid->corners.upright.x;
    grid->corners.upright.x = tmp;

    tmp = -grid->corners.loright.y;
    grid->corners.loright.y = grid->corners.loright.x;
    grid->corners.loright.x = tmp;
    }

/*
  Write the grid structure to the HDF file
*/
return(xxx_write_geom_grid(grid_filename,grid));
}

/******************************************************************************
NAME:           xxx_write_geom_grid

PURPOSE:
Write the geometric grid to an HDF file.

RETURN VALUE:   
Type = int
Value    Description
-----    -----------
SUCCESS  Successful completion
ERROR    Operation failed

NOTES:
When writing the geometric grid, the entire grid structure will be written.  The
main xxx_GEOM_GRID_TYPE structure will be written first, to a vdata named 
xxx_GEOM_GRID_TYPE.  These data elements are all of known size.

The constant values for the xxx_GEOM_GRID_BAND_TYPE structures for each band will
all be packed into another vdata named Constants.  This vdata will be for the
items that are not dynamically allocated.

To write the dynamically allocated arrays in the xxx_GEOM_GRID_BAND_TYPE 
structures for each band, the routine will loop through all the grid->nbands 
valid bands in the bandlist data element of xxx_GEOM_GRID_TYPE.  Each array 
in xxx_GEOM_GRID_BAND_TYPE will be written to its own set of SDSs.  Each 
xxx_GEOM_GRID_BAND_TYPE will require NUM_SDS SDSs to store its data.  The 
dynamically allocated arrays are sized by the other members of the 
xxx_GEOM_GRID_BAND_TYPE (which have already been written).  The first
SDS will be for the in_lines, which contains num_in_lines data values.  The
second SDS will be for the in_samps, which contains num_in_samps data values.
The third SDS will be for the out_lines, which contains num_out_lines data
values.  The fourth SDS will be for the out_samps, which contains num_out_samps
data values.  The fifth SDS will be for the poly_lines and poly_samps data, 
which both contain (degree+1)^2 data elements.  The sixth and seventh SDS will 
be for the forward and reverse coefficients, both of which contain gird_rows * 
grid_cols data elements.  

Each of these SDSs will have their name be in_lines, in_samps, out_lines,
out_samps, poly_ls, forward_coeffs, and reverse_coeffs concatinated with the 
band number.  Thus the in_lines pointer data for band 1 will be stored in an 
SDS with the name of in_lines1.

This routine will not free the grid or any of the pointers within the grid
after it is written.  The user is responsible for freeing the grid by calling
free_grid after returning from this routine.

ALGORITHM REFERENCES:
See xxx_geom_grid.h for a description of the geometric grid structure.

******************************************************************************/

int xxx_write_geom_grid 
(
  char grid_filename[STRLEN],/* I: HDF output file name */
  xxx_GEOM_GRID_TYPE *grid   /* I: Grid structure to be output */
)
{
    int  i, j, l,             /* Loop variables                      */
         currsize,            /* Current size of the data element    */
         currband,            /* Current band in the bands array     */
         ref,                 /* Current reference band value        */
         status,              /* status of return from function      */
         order,               /* Order of the current array          */
         num_ints,            /* Number of ints to malloc for        */
         num_doubles,         /* Number of doubles to malloc for     */
         num_chars;           /* Number of characters to malloc for  */
  int32  rank,                /* Rank of the dimsizes array.         */
         dimsizes[4],         /* Array of the dimension sizes.       */
         start[] = {0, 0, 0, 0}, /* Array of 0 starting positions.   */
         file_id,             /* DDR HDF file pointer                */
         sd_id,               /* DDR HDF file pointer for sds        */
         grid_type_vdata,     /* Vdata pointer (grid_type)           */
         constant_vdata,      /* Vdata pointer (constants)           */
         in_lines_sds,        /* SDS id pointer (in_lines)           */
         in_samps_sds,        /* SDS id pointer (in_samps)           */
         out_lines_sds,       /* SDS id pointer (out_lines)          */
         out_samps_sds,       /* SDS id pointer (out_samps)          */
         poly_ls_sds,         /* SDS id pointer (poly_lines/samps)   */
         forward_coeffs_sds,  /* SDS id pointer (forward coeffs)     */
         reverse_coeffs_sds,  /* SDS id pointer (reverse coeffs)     */
         i32,                 /* Holder for integers                 */
        *iarr,                /* Holder for integer arrays           */
         istat32;             /* Return values                       */
  intn   istatn;              /* Return values                       */
 uint8  *databuf,             /* Data pointer                        */
        *pntr;                /* Current position in data pointer    */
  char   str[STRLEN],         /* Holder for character strings        */
         name[STRLEN],        /* Name for the SDS                    */
         fields[NGRIDTYPE * XXX_ODL_FIELD_SIZE],
                             /* Packed string of field names of the
                                model.  The xxx_GEOM_GRID_TYPE structure
                                is the largest so give enough space
                                for it.                             */
  ErrMsg[STRLEN];            /* Error message                       */
  double d,                  /* Holder for doubles                  */
         darr[PROJPRMS_SIZE];/* Holder for double arrays (largest size) */
  double *double_ptr;        /* pointer to databuf, aliased to a double */

  /***********************************************************
   * Open the HDF output file.
   */
  file_id = Hopen (grid_filename, DFACC_CREATE, 0);
  if (file_id == FAIL)
  {
    sprintf (ErrMsg, "Error opening new HDF grid file %s.", grid_filename);
    xxx_LogStatus ("XXXLIB", __FILE__, __LINE__, ErrMsg);
    return (ERROR);
  }
  Vstart (file_id);

  /***********************************************************
   * Assign a vdata id (grid_type) for write-only access.
   */
  grid_type_vdata = VSattach (file_id, -1, "w");
  if (grid_type_vdata == FAIL)
  {
    xxx_LogStatus ("XXXLIB", __FILE__, __LINE__, 
                   "Error creating grid_type Vdata");
    Vend (file_id);
    Hclose (file_id);
    return (ERROR);
  }
  VSsetname (grid_type_vdata, "xxx_GEOM_GRID_TYPE");
  istatn = VSsetinterlace (grid_type_vdata, FULL_INTERLACE);
  if (istatn == FAIL)
  {
    xxx_LogStatus ("XXXLIB", __FILE__, __LINE__, 
                   "Error setting the HDF interface");
    VSdetach (grid_type_vdata);
    Vend (file_id);
    Hclose (file_id);
    return (ERROR);
  }

  /* Define the fields for the grid (grid_type).
   */
  VSfdefine (grid_type_vdata, grid_type_elements[0].name,
             (int32)grid_type_elements[0].dfnt, 
             (int32)grid_type_elements[0].order);
  strcpy (fields, grid_type_elements[0].name);
  for (i = 1; i < NGRIDTYPE; i++)
  {
    VSfdefine (grid_type_vdata, grid_type_elements[i].name,
               (int32)grid_type_elements[i].dfnt, 
               (int32)grid_type_elements[i].order);
    strcat (fields, ",");
    strcat (fields, grid_type_elements[i].name);
  }
  istatn = VSsetfields(grid_type_vdata, fields);
  if (istatn == FAIL)
  {
    xxx_LogStatus ("XXXLIB", __FILE__, __LINE__, 
                   "Error setting the grid_type field names");
    VSdetach (grid_type_vdata);
    Vend (file_id);
    Hclose (file_id);
    return (ERROR);
  }

  /* Malloc space for the integer array for the bands.
   */
  iarr = malloc (MAX_GEOM_GRID_BANDS * sizeof(int32));
  if (iarr == NULL)
  {
    strcpy (ErrMsg, "Error allocating memory.");
    xxx_LogStatus ("XXXLIB", __FILE__, __LINE__, ErrMsg);
    VSdetach (grid_type_vdata);
    Vend (file_id);
    Hclose (file_id);
    return (ERROR);
  }

  /* Set up the data buffer and its pointer for the grid_types.
   */
  num_ints = NUM_GRID_TYPE_INTS;
  num_chars = NUM_GRID_TYPE_CHARS;
  num_doubles = NUM_GRID_TYPE_DBLS;
  databuf = malloc (num_ints*sizeof(int32) + num_doubles*sizeof(double) +
                    num_chars*sizeof(char));
  if (databuf == NULL)
  {
    strcpy (ErrMsg, "Error allocating data buffer memory.");
    xxx_LogStatus ("XXXLIB", __FILE__, __LINE__, ErrMsg);
    VSdetach (grid_type_vdata);
    Vend (file_id);
    Hclose (file_id);
    free (iarr);
    return (ERROR);
  }
  pntr = databuf;
  j = 0;

  /* Write grid_type data to buffers.
   */
  currsize = grid_type_elements[j].order * grid_type_elements[j].size;
  strcpy (str, grid->gridname);
  memcpy (pntr, (void *)str, currsize);
  j++;
  pntr += currsize;

  currsize = grid_type_elements[j].order * grid_type_elements[j].size;
  i32 = (int32)grid->gridtype;
  memcpy (pntr, (void *)&i32, currsize);
  j++;
  pntr += currsize;
 
  currsize = grid_type_elements[j].order * grid_type_elements[j].size;
  i32 = grid->nbands;
  memcpy (pntr, (void *)&i32, currsize);
  j++;
  pntr += currsize;
  
  currsize = grid_type_elements[j].order * grid_type_elements[j].size;
  for (i = 0; i < MAX_GEOM_GRID_BANDS; i++)
    iarr[i] = grid->bandlist[i];
  memcpy (pntr, (void *)iarr, currsize);
  j++;
  pntr += currsize;

  currsize = grid_type_elements[j].order * grid_type_elements[j].size;
  i32 = grid->dtype;
  memcpy (pntr, (void *)&i32, currsize);
  j++;
  pntr += currsize;
  
  currsize = grid_type_elements[j].order * grid_type_elements[j].size;
  i32 = grid->correction_type;
  memcpy (pntr, (void *)&i32, currsize);
  j++;
  pntr += currsize;
  
  currsize = grid_type_elements[j].order * grid_type_elements[j].size;
  i32 = grid->frame_type;
  memcpy (pntr, (void *)&i32, currsize);
  j++;
  pntr += currsize;
  
  currsize = grid_type_elements[j].order * grid_type_elements[j].size;
  strcpy (str, grid->units);
  memcpy (pntr, (void *)str, currsize);
  j++;
  pntr += currsize;

  currsize = grid_type_elements[j].order * grid_type_elements[j].size;
  i32 = grid->code;
  memcpy (pntr, (void *)&i32, currsize);
  j++;
  pntr += currsize;
  
  currsize = grid_type_elements[j].order * grid_type_elements[j].size;
  strcpy (str, grid->datum);
  memcpy (pntr, (void *)str, currsize);
  j++;
  pntr += currsize;

  currsize = grid_type_elements[j].order * grid_type_elements[j].size;
  i32 = grid->spheroid;
  memcpy (pntr, (void *)&i32, currsize);
  j++;
  pntr += currsize;
  
  currsize = grid_type_elements[j].order * grid_type_elements[j].size;
  i32 = grid->zone;
  memcpy (pntr, (void *)&i32, currsize);
  j++;
  pntr += currsize;
  
  currsize = grid_type_elements[j].order * grid_type_elements[j].size;
  for (i = 0; i < PROJPRMS_SIZE; i++)
    darr[i] = grid->projprms[i];
  memcpy (pntr, (void *)darr, currsize);
  j++;
  pntr += currsize;

  currsize = grid_type_elements[j].order * grid_type_elements[j].size;
  darr[0] = grid->corners.upleft.x;
  darr[1] = grid->corners.upright.x;
  darr[2] = grid->corners.loleft.x;
  darr[3] = grid->corners.loright.x;
  memcpy (pntr, (void *)darr, currsize);
  j++;
  pntr += currsize;

  currsize = grid_type_elements[j].order * grid_type_elements[j].size;
  darr[0] = grid->corners.upleft.y;
  darr[1] = grid->corners.upright.y;
  darr[2] = grid->corners.loleft.y;
  darr[3] = grid->corners.loright.y;
  memcpy (pntr, (void *)darr, currsize);
  j++;
  pntr += currsize;

  currsize = grid_type_elements[j].order * grid_type_elements[j].size;
  darr[0] = grid->center_pt.lat;
  darr[1] = grid->center_pt.lng;
  memcpy (pntr, (void *)darr, currsize);
  j++;
  pntr += currsize;

  currsize = grid_type_elements[j].order * grid_type_elements[j].size;
  darr[0] = grid->proj_center_pt.x;
  darr[1] = grid->proj_center_pt.y;
  memcpy (pntr, (void *)darr, currsize);
  j++;
  pntr += currsize;

  currsize = grid_type_elements[j].order * grid_type_elements[j].size;
  d = grid->rotate_angle;
  memcpy (pntr, (void *)&d, currsize);
  j++;
  pntr += currsize;

  currsize = grid_type_elements[j].order * grid_type_elements[j].size;
  d = grid->orient_angle;
  memcpy (pntr, (void *)&d, currsize);
  j++;
  pntr += currsize;

  /* Write buffer to grid_type Vdata.  Note there is only one record for
   * this Vdata.
   */
  istat32 = VSwrite (grid_type_vdata, databuf, 1, FULL_INTERLACE);
  if (istat32 == FAIL)
  {
    xxx_LogStatus ("XXXLIB", __FILE__, __LINE__, 
                   "Error writing the grid structure to the HDF file");
    free (databuf);
    free (iarr);
    VSdetach (grid_type_vdata);
    Vend (file_id);
    Hclose (file_id);
    return (ERROR);
  }

  /* Detach the Vdata.
   */
  VSdetach (grid_type_vdata);
  free (databuf);
  free (iarr);

  /******************************************************
   * Assign a vdata id (constants) for write-only access.
   */
  constant_vdata = VSattach (file_id, -1, "w");
  if (constant_vdata == FAIL)
  {
    xxx_LogStatus ("XXXLIB", __FILE__, __LINE__, 
                   "Error creating constants Vdata");
    Vend (file_id);
    Hclose (file_id);
    return (ERROR);
  }
  VSsetname (constant_vdata, "Constants");
  istatn = VSsetinterlace (constant_vdata, FULL_INTERLACE);
  if (istatn == FAIL)
  {
    xxx_LogStatus ("XXXLIB", __FILE__, __LINE__, "Error setting the interface");
    VSdetach (constant_vdata);
    Vend (file_id);
    Hclose (file_id);
    return (ERROR);
  }

  /* Define the fields for the grid (constants).
   */
  VSfdefine (constant_vdata, constant_elements[0].name,
             (int32)constant_elements[0].dfnt, 
             (int32)constant_elements[0].order);
  strcpy (fields, constant_elements[0].name);
  for (i = 1; i < NCONSTANT; i++)
  {
    VSfdefine (constant_vdata, constant_elements[i].name,
               (int32)constant_elements[i].dfnt, 
               (int32)constant_elements[i].order);
    strcat (fields, ",");
    strcat (fields, constant_elements[i].name);
  }
  istatn = VSsetfields(constant_vdata, fields);
  if (istatn == FAIL)
  {
    xxx_LogStatus ("XXXLIB", __FILE__, __LINE__, 
                   "Error setting the constants field names");
    VSdetach (constant_vdata);
    Vend (file_id);
    Hclose (file_id);
    return (ERROR);
  }

  /* Set up the data buffer and its pointer for the constants.
   */
  num_ints = NUM_CONSTANT_INTS;
  num_doubles = NUM_CONSTANT_DBLS;
  databuf = malloc ((num_ints*sizeof(int32) +
                     num_doubles*sizeof(double)) * grid->nbands);
  if (databuf == NULL)
  {
    xxx_LogStatus ("XXXLIB", __FILE__, __LINE__, 
                   "Error allocating data buffer memory.");
    VSdetach (constant_vdata);
    Vend (file_id);
    Hclose (file_id);
    return (ERROR);
  }
  pntr = databuf;

  for (i = 0, j = 0; i < grid->nbands; i++, j = 0)
  {
    /* Get the reference value of the current band.
     */
    ref = grid->bandlist[i];

    /* Write constants data to buffers.
     */
    currsize = constant_elements[j].order * constant_elements[j].size;
    i32 = grid->gridbands[ref].band_number;
    memcpy (pntr, (void *)&i32, currsize);
    j++;
    pntr += currsize;
  
    currsize = constant_elements[j].order * constant_elements[j].size;
    i32 = grid->gridbands[ref].gridtype;
    memcpy (pntr, (void *)&i32, currsize);
    j++;
    pntr += currsize;
    
     currsize = constant_elements[j].order * constant_elements[j].size;
    i32 = grid->gridbands[ref].cell_lines;
    memcpy (pntr, (void *)&i32, currsize);
    j++;
    pntr += currsize;
  
    currsize = constant_elements[j].order * constant_elements[j].size;
    i32 = grid->gridbands[ref].cell_samps;
    memcpy (pntr, (void *)&i32, currsize);
    j++;
    pntr += currsize;
  
    currsize = constant_elements[j].order * constant_elements[j].size;
    d = grid->gridbands[ref].pixsize_line;
    memcpy (pntr, (void *)&d, currsize);
    j++;
    pntr += currsize;
    
    currsize = constant_elements[j].order * constant_elements[j].size;
    d = grid->gridbands[ref].pixsize_samp;
    memcpy (pntr, (void *)&d, currsize);
    j++;
    pntr += currsize;
    
    currsize = constant_elements[j].order * constant_elements[j].size;
    i32 = grid->gridbands[ref].output_image_lines;
    memcpy (pntr, (void *)&i32, currsize);
    j++;
    pntr += currsize;
  
    currsize = constant_elements[j].order * constant_elements[j].size;
    i32 = grid->gridbands[ref].output_image_samps;
    memcpy (pntr, (void *)&i32, currsize);
    j++;
    pntr += currsize;
  
    currsize = constant_elements[j].order * constant_elements[j].size;
    i32 = grid->gridbands[ref].num_in_lines;
    memcpy (pntr, (void *)&i32, currsize);
    j++;
    pntr += currsize;
  
    currsize = constant_elements[j].order * constant_elements[j].size;
    i32 = grid->gridbands[ref].num_in_samps;
    memcpy (pntr, (void *)&i32, currsize);
    j++;
    pntr += currsize;
  
    currsize = constant_elements[j].order * constant_elements[j].size;
    i32 = grid->gridbands[ref].num_out_lines;
    memcpy (pntr, (void *)&i32, currsize);
    j++;
    pntr += currsize;
  
    currsize = constant_elements[j].order * constant_elements[j].size;
    i32 = grid->gridbands[ref].num_out_samps;
    memcpy (pntr, (void *)&i32, currsize);
    j++;
    pntr += currsize;
    
    currsize = constant_elements[j].order * constant_elements[j].size;
    i32 = grid->gridbands[ref].forward_present;
    memcpy (pntr, (void *)&i32, currsize);
    j++;
    pntr += currsize;
  
    currsize = constant_elements[j].order * constant_elements[j].size;
    i32 = grid->gridbands[ref].reverse_present;
    memcpy (pntr, (void *)&i32, currsize);
    j++;
    pntr += currsize;
    
    currsize = constant_elements[j].order * constant_elements[j].size;
    i32 = grid->gridbands[ref].grid_rows;
    memcpy (pntr, (void *)&i32, currsize);
    j++;
    pntr += currsize;
  
    currsize = constant_elements[j].order * constant_elements[j].size;
    i32 = grid->gridbands[ref].grid_cols;
    memcpy (pntr, (void *)&i32, currsize);
    j++;
    pntr += currsize;

    currsize = constant_elements[j].order * constant_elements[j].size;
    i32 = grid->gridbands[ref].degree;
    memcpy (pntr, (void *)&i32, currsize);
    j++;
    pntr += currsize;
  }

  /* Write buffer to constants Vdata.  There are nbands of these records.
   */
  istat32 = VSwrite (constant_vdata, databuf, (int32)grid->nbands,
                     FULL_INTERLACE);
  if (istat32 == FAIL)
  {
    xxx_LogStatus ("XXXLIB", __FILE__, __LINE__, 
                   "Error writing the grid structure to the HDF file");
    free (databuf);
    VSdetach (constant_vdata);
    Vend (file_id);
    Hclose (file_id);
    return (ERROR);
  }

  /* Detach the Vdata.
   */
  VSdetach (constant_vdata);
  free (databuf);

  /* Close the file.
   */
  Vend (file_id);
  Hclose (file_id);

  /***********************************************************
   * Open the file for the SDS interface.
   */
  sd_id = SDstart (grid_filename, DFACC_RDWR);
  if (sd_id == FAIL)
  {
    sprintf (ErrMsg, "Error opening grid file %s for SDS.", grid_filename);
    xxx_LogStatus ("XXXLIB", __FILE__, __LINE__, ErrMsg);
    return (ERROR);
  }

  /**********************************************************************
   * Loop through the bands in the grid and place their pointer values in
   * separate SDSs for each band.
   */
  for (currband = 0; currband < grid->nbands; currband++)
  {
    /* Find the reference position.
     */
    ref = grid->bandlist[currband];

    /***********************************************************
     * Assign an sds (in_lines) for write-only access.
     */
    /* Create the SDS array.  The array will have 1 dimension.  It will be
     * for the size of in_lines (size = num_in_lines).
     */
    rank = 1;
    dimsizes[0] = (int32)grid->gridbands[ref].num_in_lines;
    status = sprintf (name, "In_lines%d", ref);
	if (status < 1)
		{
    	xxx_LogStatus ("XXXLIB", __FILE__, __LINE__, "Error in sprintf");
    	return (ERROR);
		}
    in_lines_sds = SDcreate (sd_id, name, (int32)in_lines_elements[0].dfnt, 
                             rank, dimsizes);
    if (in_lines_sds == FAIL)
    {
      xxx_LogStatus ("XXXLIB", __FILE__, __LINE__, 
                     "Error creating in_lines_sds");
      SDendaccess (in_lines_sds);
      SDend (sd_id);
      return (ERROR);
    }

    /* Set up the data buffer and its pointer for the in_lines.
     */
    databuf = malloc (grid->gridbands[ref].num_in_lines * sizeof(double));
    if (databuf == NULL)
    {
      xxx_LogStatus ("XXXLIB", __FILE__, __LINE__, 
                     "Error allocating data buffer memory.");
      SDendaccess (in_lines_sds);
      SDend (sd_id);
      return (ERROR);
    }
    double_ptr = (double*)databuf;

    /* Write the in_lines data to buffers, converting types if needed */
    for (j = 0; j < grid->gridbands[ref].num_in_lines; j++)
      double_ptr[j] = grid->gridbands[ref].in_lines[j];

    /* Write buffer to in_lines SDS.
     */
    istatn = SDwritedata (in_lines_sds, start, NULL, dimsizes, (VOIDP)databuf);
    if (istatn == FAIL)
    {
      xxx_LogStatus ("XXXLIB", __FILE__, __LINE__, 
                     "Error writing the in_lines to the HDF file");
      free (databuf);
      SDendaccess (in_lines_sds);
      SDend (sd_id);
      return (ERROR);
    }

    /* Terminate access to the SDS.
     */
    SDendaccess (in_lines_sds);
    free (databuf);

    /***********************************************************
     * Assign an sds (in_samps) for write-only access.
     */
    /* Create the SDS array.  The array will have 1 dimension.  It will be
     * for the size of in_samps (size = num_in_samps).
     */
    rank = 1;
    dimsizes[0] = (int32)grid->gridbands[ref].num_in_samps;
    status = sprintf (name, "In_samps%d", ref);
	if (status < 1)
		{
    	xxx_LogStatus ("XXXLIB", __FILE__, __LINE__, "Error in sprintf");
    	return (ERROR);
		}
    in_samps_sds = SDcreate (sd_id, name, (int32)in_samps_elements[0].dfnt, 
                             rank, dimsizes);
    if (in_samps_sds == FAIL)
    {
      xxx_LogStatus ("XXXLIB", __FILE__, __LINE__, 
                     "Error creating in_samps_sds");
      SDendaccess (in_samps_sds);
      SDend (sd_id);
      return (ERROR);
    }

    /* Set up the data buffer and its pointer for the in_samps.
     */
    databuf = malloc (grid->gridbands[ref].num_in_samps * sizeof(double));
    if (databuf == NULL)
    {
      xxx_LogStatus ("XXXLIB", __FILE__, __LINE__, 
                     "Error allocating data buffer memory");
      SDendaccess (in_samps_sds);
      SDend (sd_id);
      return (ERROR);
    }
    double_ptr = (double*)databuf;

    /* Write the in_samps data, converting type if needed */
    for (j = 0; j < grid->gridbands[ref].num_in_samps; j++)
      double_ptr[j] = grid->gridbands[ref].in_samps[j];

    /* Write buffer to in_samps SDS.
     */
    istatn = SDwritedata (in_samps_sds, start, NULL, dimsizes, (VOIDP)databuf);
    if (istatn == FAIL)
    {
      xxx_LogStatus ("XXXLIB", __FILE__, __LINE__, 
                     "Error writing the in_samps to the HDF file");
      free (databuf);
      SDendaccess (in_samps_sds);
      SDend (sd_id);
      return (ERROR);
    }

    /* Terminate access to the SDS.
     */
    SDendaccess (in_samps_sds);
    free (databuf);

    /***********************************************************
     * Assign an sds (out_lines) for write-only access.
     */
    /* Create the SDS array.  The array will have 1 dimension.  It will be
     * the size of out_lines (size = num_out_lines).
     */
    rank = 1;
    dimsizes[0] = (int32)grid->gridbands[ref].num_out_lines;
    status = sprintf (name, "Out_lines%d", ref);
	if (status < 1)
		{
    	xxx_LogStatus ("XXXLIB", __FILE__, __LINE__, "Error in sprintf");
    	return (ERROR);
		}
    out_lines_sds = SDcreate (sd_id, name, (int32)out_lines_elements[0].dfnt, 
                              rank, dimsizes);
    if (out_lines_sds == FAIL)
    {
      xxx_LogStatus ("XXXLIB", __FILE__, __LINE__, 
                     "Error creating out_lines_sds");
      SDendaccess (out_lines_sds);
      SDend (sd_id);
      return (ERROR);
    }

    /* Set up the data buffer and its pointer for the out_lines.
     */
    databuf = malloc (grid->gridbands[ref].num_out_lines * sizeof(double));
    if (databuf == NULL)
    {
      xxx_LogStatus ("XXXLIB", __FILE__, __LINE__, 
                     "Error allocating data buffer memory");
      SDendaccess (out_lines_sds);
      SDend (sd_id);
      return (ERROR);
    }
    double_ptr = (double*)databuf;

    /* Write the out_lines data to buffers.
     */
    for (j = 0; j < grid->gridbands[ref].num_out_lines; j++)
      double_ptr[j] = grid->gridbands[ref].out_lines[j];

    /* Write buffer to out_lines SDS.
     */
    istatn = SDwritedata (out_lines_sds, start, NULL, dimsizes, (VOIDP)databuf);
    if (istatn == FAIL)
    {
      strcpy (ErrMsg, "Error writing the grid structure to the HDF file.");
      xxx_LogStatus ("XXXLIB", __FILE__, __LINE__, ErrMsg);
      free (databuf);
      SDendaccess (out_lines_sds);
      SDend (sd_id);
      return (ERROR);
    }

    /* Terminate access to the SDS.
     */
    SDendaccess (out_lines_sds);
    free (databuf);

    /***********************************************************
     * Assign an sds (out_samps) for write-only access.
     */
    /* Create the SDS array.  The array will have 1 dimension.  It will be
     * the size of out_samps (size = num_out_samps).
     */
    rank = 1;
    dimsizes[0] = (int32)grid->gridbands[ref].num_out_samps;
    status = sprintf (name, "Out_samps%d", ref);
	if (status < 1)
		{
    	xxx_LogStatus ("XXXLIB", __FILE__, __LINE__, "Error in sprintf");
    	return (ERROR);
		}
    out_samps_sds = SDcreate (sd_id, name, (int32)out_samps_elements[0].dfnt, 
                              rank, dimsizes);
    if (out_lines_sds == FAIL)
    {
      xxx_LogStatus ("XXXLIB", __FILE__, __LINE__, 
                     "Error creating out_samps_sds");
      SDendaccess (out_samps_sds);
      SDend (sd_id);
      return (ERROR);
    }

    /* Set up the data buffer and its pointer for the out_samps.
     */
    databuf = malloc (grid->gridbands[ref].num_out_samps * sizeof(double));
    if (databuf == NULL)
    {
      xxx_LogStatus ("XXXLIB", __FILE__, __LINE__, 
                     "Error allocating data buffer memory");
      SDendaccess (out_samps_sds);
      SDend (sd_id);
      return (ERROR);
    }
    double_ptr = (double*)databuf;

    /* Write the out_samps data to buffers.
     */
    for (j = 0; j < grid->gridbands[ref].num_out_samps; j++)
      double_ptr[j] = grid->gridbands[ref].out_samps[j];

    /* Write buffer to out_samps SDS.
     */
    istatn = SDwritedata (out_samps_sds, start, NULL, dimsizes, (VOIDP)databuf);
    if (istatn == FAIL)
    {
      strcpy (ErrMsg, "Error writing the grid structure to the HDF file.");
      xxx_LogStatus ("XXXLIB", __FILE__, __LINE__, ErrMsg);
      free (databuf);
      SDendaccess (out_samps_sds);
      SDend (sd_id);
      return (ERROR);
    }

    /* Terminate access to the SDS.
     */
    SDendaccess (out_samps_sds);
    free (databuf);
    
    /***********************************************************
     * Assign an sds (poly_lines/samps) for write-only access.
     */
    /* Create the SDS array.  The array will have 2 dimensions.  One will be
     * for the poly_lines and poly_samps themselves and the other will be
     * the size of poly_lines/poly_samps (size = (degree+1) * (degree+1)).
     */
    rank = 2;
    dimsizes[0] = 2;
    dimsizes[1] = (int32)(grid->gridbands[ref].degree+1) *
                  (int32)(grid->gridbands[ref].degree+1);
    status = sprintf (name, "Poly_ls%d", ref);
	if (status < 1)
		{
    	xxx_LogStatus ("XXXLIB", __FILE__, __LINE__, "Error in sprintf");
    	return (ERROR);
		}
    poly_ls_sds = SDcreate (sd_id, name, (int32)poly_ls_elements[0].dfnt, 
                            rank, dimsizes);
    if (poly_ls_sds == FAIL)
    {
      strcpy (ErrMsg, "Error creating poly_ls_sds.");
      xxx_LogStatus ("XXXLIB", __FILE__, __LINE__, ErrMsg);
      SDendaccess (poly_ls_sds);
      SDend (sd_id);
      return (ERROR);
    }

    /* Set up the data buffer and its pointer for the poly_lines/samps.
     */
    databuf = malloc (2 * (grid->gridbands[ref].degree+1) *
                     (grid->gridbands[ref].degree+1) * sizeof(double));
    if (databuf == NULL)
    {
      strcpy (ErrMsg, "Error allocating data buffer memory.");
      xxx_LogStatus ("XXXLIB", __FILE__, __LINE__, ErrMsg);
      SDendaccess (poly_ls_sds);
      SDend (sd_id);
      return (ERROR);
    }
    double_ptr = (double*)databuf;

    /* Write the poly_lines/samps data to buffers.
     */
    order = (grid->gridbands[ref].degree + 1) *
            (grid->gridbands[ref].degree + 1);
    for (j = 0; j < order; j++)
      double_ptr[j] = grid->gridbands[ref].poly_lines[j];

    double_ptr = &double_ptr[order];

    for (j = 0; j < order; j++)
      double_ptr[j] = grid->gridbands[ref].poly_samps[j];

    /* Write buffer to poly_lines/samps SDS.
     */
    istatn = SDwritedata (poly_ls_sds, start, NULL, dimsizes, (VOIDP)databuf);
    if (istatn == FAIL)
    {
      strcpy (ErrMsg, "Error writing the grid structure to the HDF file.");
      xxx_LogStatus ("XXXLIB", __FILE__, __LINE__, ErrMsg);
      free (databuf);
      SDendaccess (poly_ls_sds);
      SDend (sd_id);
      return (ERROR);
    }

    /* Terminate access to the SDS.
     */
    SDendaccess (poly_ls_sds);
    free (databuf);

    /***********************************************************
     * Assign an sds (forward_coeffs) for write-only access if the 
     * forward coefficients are present in the grid
     */
    if (grid->gridbands[ref].forward_present)
    {
        /* Create the SDS array.  The array has 3 dimensions.  One for the
         * COEFS_SIZE.  One for the XY_SIZE.  One for the size of the 
         * coefficient array (size = grid->gridbands[ref].grid_rows *
         *  grid->gridbands[ref].grid_cols).
         */
        rank = 3;
        dimsizes[0] = COEFS_SIZE;
        dimsizes[1] = XY_SIZE;
        dimsizes[2] = (int32)grid->gridbands[ref].grid_rows *
                      (int32)grid->gridbands[ref].grid_cols;
        status = sprintf (name, "forward_coeffs%d", ref);
		if (status < 1)
			{
    		xxx_LogStatus ("XXXLIB", __FILE__, __LINE__, "Error in sprintf");
    		return (ERROR);
			}
        forward_coeffs_sds = SDcreate (sd_id, name, 
                                       (int32)forward_coeffs_elements[0].dfnt, 
                                       rank, dimsizes);
        if (forward_coeffs_sds == FAIL)
        {
          xxx_LogStatus ("XXXLIB", __FILE__, __LINE__, 
                         "Error creating the forward_coeffs_sds");
          SDendaccess (forward_coeffs_sds);
          SDend (sd_id);
          return (ERROR);
        }

        /* Set up the data buffer and its pointer for the sattoproj/projtosat.
         */
        databuf = malloc (grid->gridbands[ref].grid_rows *
                          grid->gridbands[ref].grid_cols *
                          COEFS_SIZE * XY_SIZE * sizeof(double));
        if (databuf == NULL)
        {
          strcpy (ErrMsg, "Error allocating data buffer memory.");
          xxx_LogStatus ("XXXLIB", __FILE__, __LINE__, ErrMsg);
          SDendaccess (forward_coeffs_sds);
          SDend (sd_id);
          return (ERROR);
        }
        double_ptr = (double*)databuf;

        /* Write the forward_coeffs data to buffers.
         */
        order = grid->gridbands[ref].grid_rows *
                grid->gridbands[ref].grid_cols;
        for (j = 0; j < COEFS_SIZE; j++)
        {
            /* forward a coeffs
             */
            for (l = 0; l < order; l++)
              double_ptr[l] = grid->gridbands[ref].forward_coeffs[l].a[j];

            double_ptr = &double_ptr[order];

            /* forward b coefs
             */
            for (l = 0; l < order; l++)
              double_ptr[l] = grid->gridbands[ref].forward_coeffs[l].b[j];

            double_ptr = &double_ptr[order];
        }

        /* Write buffer to sattoproj/projtosat SDS.
         */
        istatn = SDwritedata (forward_coeffs_sds, start, NULL, dimsizes, 
                             (VOIDP)databuf);
        if (istatn == FAIL)
        {
          xxx_LogStatus ("XXXLIB", __FILE__, __LINE__, 
                         "Error writing the grid structure to the HDF file");
          free (databuf);
          SDendaccess (forward_coeffs_sds);
          SDend (sd_id);
          return (ERROR);
        }

        /* Terminate access to the SDS.
         */
        SDendaccess (forward_coeffs_sds);
        free (databuf);
    }

    /***********************************************************
     * Assign an sds (reverse_coeffs) for write-only access if the 
     * reverse coefficients are present in the grid
     */
    if (grid->gridbands[ref].reverse_present)
    {
        /* Create the SDS array.  The array has 3 dimensions.  One for the
         * COEFS_SIZE.  One for the XY_SIZE.  One for the size of the 
         * coefficient array (size = grid->gridbands[ref].grid_rows *
         *  grid->gridbands[ref].grid_cols).
         */
        rank = 3;
        dimsizes[0] = COEFS_SIZE;
        dimsizes[1] = XY_SIZE;
        dimsizes[2] = (int32)grid->gridbands[ref].grid_rows *
                      (int32)grid->gridbands[ref].grid_cols;
        status = sprintf (name, "reverse_coeffs%d", ref);
		if (status < 1)
			{
    		xxx_LogStatus ("XXXLIB", __FILE__, __LINE__, "Error in sprintf");
    		return (ERROR);
			}
        reverse_coeffs_sds = SDcreate (sd_id, name, 
                                       (int32)reverse_coeffs_elements[0].dfnt, 
                                       rank, dimsizes);
        if (reverse_coeffs_sds == FAIL)
        {
          xxx_LogStatus ("XXXLIB", __FILE__, __LINE__, 
                         "Error creating the reverse_coeffs_sds");
          SDendaccess (reverse_coeffs_sds);
          SDend (sd_id);
          return (ERROR);
        }

        /* Set up the data buffer and its pointer for the sattoproj/projtosat.
         */
        databuf = malloc (grid->gridbands[ref].grid_rows *
                          grid->gridbands[ref].grid_cols *
                          COEFS_SIZE * XY_SIZE * sizeof(double));
        if (databuf == NULL)
        {
          strcpy (ErrMsg, "Error allocating data buffer memory.");
          xxx_LogStatus ("XXXLIB", __FILE__, __LINE__, ErrMsg);
          SDendaccess (reverse_coeffs_sds);
          SDend (sd_id);
          return (ERROR);
        }
        double_ptr = (double*)databuf;

        /* Write the reverse_coeffs data to buffers.
         */
        order = grid->gridbands[ref].grid_rows *
                grid->gridbands[ref].grid_cols;
        for (j = 0; j < COEFS_SIZE; j++)
        {
            /* reverse a coeffs
             */
            for (l = 0; l < order; l++)
              double_ptr[l] = grid->gridbands[ref].reverse_coeffs[l].a[j];

            double_ptr = &double_ptr[order];

            /* reverse b coefs
             */
            for (l = 0; l < order; l++)
              double_ptr[l] = grid->gridbands[ref].reverse_coeffs[l].b[j];

            double_ptr = &double_ptr[order];
        }

        /* Write buffer to sattoproj/projtosat SDS.
         */
        istatn = SDwritedata (reverse_coeffs_sds, start, NULL, dimsizes, 
                             (VOIDP)databuf);
        if (istatn == FAIL)
        {
          xxx_LogStatus ("XXXLIB", __FILE__, __LINE__, 
                         "Error writing the grid structure to the HDF file");
          free (databuf);
          SDendaccess (reverse_coeffs_sds);
          SDend (sd_id);
          return (ERROR);
        }

        /* Terminate access to the SDS.
         */
        SDendaccess (reverse_coeffs_sds);
        free (databuf);
    }  
  } /* Loop on the bands */

  /***********************************************************
   * Close the file.
   */
  SDend (sd_id);

  return (SUCCESS);
}

/******************************************************************************
NAME:           xxx_get_geom_grid

PURPOSE:
Read the geometric grid from an HDF file.

RETURN VALUE:   
Type = int
Value    Description
-----    -----------
SUCCESS  Successful completion
ERROR    Operation failed

NOTES:	 This function is a bridge routine to the actual xxx_read_geom_grid
         support.  This function only flips SOM corner coorindates
         after they are read.  This allows a consistant user interface,
         yet allows the algorithms using SOM coodinates not to change.


ALGORITHM REFERENCES:
See xxx_geom_grid.h for a description of the geometric grid structure.

******************************************************************************/

int xxx_get_geom_grid 
(
  char grid_filename[STRLEN],/* I: HDF input file name                    */
  int *bands,                /* I/O: Array of bands to free from the grid */
  int *nbands,               /* I/O: Number of bands in the bands array   */
  xxx_GEOM_GRID_TYPE *grid   /* O: Grid structure to be output            */
)
{
double tmp;		     /* Temporary variable for flipping corner points*/

if (xxx_read_geom_grid (grid_filename, bands, nbands, grid) != SUCCESS)
    return(ERROR);

/* flip the coords if the projection is SOM */
xxx_SOM_geomgrid_calc(grid);

/*
  Write the grid structure to the HDF file
*/
return(SUCCESS);
}

/******************************************************************************
NAME:           xxx_read_geom_grid

PURPOSE:
Read the geometric grid from an HDF file.

RETURN VALUE:   
Type = int
Value    Description
-----    -----------
SUCCESS  Successful completion
ERROR    Operation failed

NOTES:
This routine will read the geometric grid from the HDF file for the specified 
band numbers.  The xxx_GEOM_GRID_TYPE elements are read first, so that we know 
the number of bands in the grid (stored in nbands.)  Next the constant
xxx_GEOM_GRID_BAND_TYPE data elements are read for the entire grid.  These data
elements specify the number of elements in the xxx_GEOM_GRID_BAND_TYPE 
dynamically allocated data elements.  The memory is then allocated for the
dynamically allocated arrays based on what was read.  After the memory is
allocated for the arrays, their data is read from the file.

The user may specify one band, several bands, or no bands.  If no bands are
specified, then all the bands are read from the HDF file.  In this case, the
bandlist array will be filled in with the bands in the grid and nbands will be
set to the correct value.  This routine will check the grid to verify that
the specified bands are in the HDF file.  If a band does not exist, then this
routine will exit with an error.

See the xxx_write_geom_grid function for a description of the HDF file layout.

ALGORITHM REFERENCES:
See xxx_geom_grid.h for a description of the geometric grid structure.

******************************************************************************/

int xxx_read_geom_grid 
(
  char grid_filename[STRLEN],/* I: HDF input file name                    */
  int *bands,                /* I/O: Array of bands to free from the grid */
  int *nbands,               /* I/O: Number of bands in the bands array   */
  xxx_GEOM_GRID_TYPE *grid   /* O: Grid structure to be output            */
)
{
  /* Get the xxx_GEOM_GRID_TYPE data and the constant data for the
   * xxx_GEOM_GRID_BAND_TYPEs.
   */ 
  if (xxx_get_geom_grid_headers (grid_filename, grid) == ERROR)
  {
    xxx_LogStatus ("XXXLIB", __FILE__, __LINE__, 
                   "Error reading the grid headers");
    return (ERROR);
  }

  /* Get the dynamically allocated data for the specified bands.
   */
  if (xxx_get_geom_grid_data (grid_filename, bands, nbands, grid) == ERROR)
  {
    xxx_LogStatus ("XXXLIB", __FILE__, __LINE__, 
                   "Error reading the grid data");
    return (ERROR);
  }

  return (SUCCESS);
}

/******************************************************************************
NAME:           xxx_get_geom_grid_headers

PURPOSE:
Read the xxx_GEOM_GRID_TYPE and xxx_GEOM_GRID_BAND_TYPE constants from an HDF 
file.

RETURN VALUE:   
Type = int
Value    Description
-----    -----------
SUCCESS  Successful completion
ERROR    Operation failed

NOTES:
This routine will read the xxx_GEOM_GRID_TYPE and xxx_GEOM_GRID_BAND_TYPE 
constants for the entire grid.  The xxx_GEOM_GRID_TYPE elements are read first, 
so that we know the number of bands in the grid (stored in nbands.)  Next the 
constant xxx_GEOM_GRID_BAND_TYPE data elements are read for the entire grid.  
These data elements specify the number of elements in the xxx_GEOM_GRID_BAND_TYPE
dynamically allocated arrays.

The xxx_GEOM_GRID_TYPE data is stored in a vdata named xxx_GEOM_GRID_TYPE.  The 
constant data in the xxx_GEOM_GRID_BAND_TYPE structure is stored in a vdata 
named Constants.  This vdata contains the constant data for all of the specified
bands.

ALGORITHM REFERENCES:
See xxx_geom_grid.h for a description of the geometric grid structure.

******************************************************************************/

int xxx_get_geom_grid_headers 
(
  char grid_filename[STRLEN],/* I: HDF input file name                    */
  xxx_GEOM_GRID_TYPE *grid   /* O: Grid structure to be output            */
)
{
     int  i, j,                 /* Loop variables                      */
          currsize,             /* Current size of the data element    */
          ref,                  /* Current reference band value        */
          num_ints,             /* Number of ints to malloc for        */
          num_doubles,          /* Number of doubles to malloc for     */
          num_chars;            /* Number of characters to malloc for  */
    intn  istatn;
   int32  file_id,              /* DDR HDF file pointer                */
          ref_num,              /* Reference number of the vdata       */
          grid_type_vdata,      /* Vdata pointer (grid_type)           */
          constant_vdata,       /* Vdata pointer (constants)           */
          i32,                  /* Holder for integers                 */
         *iarr,                 /* Holder for integer arrays           */
          istat32;              /* Return values                       */
   uint8 *databuf,              /* Data pointer                        */
         *pntr;                 /* Current position in data pointer    */
    char  str[STRLEN],          /* Holder for character strings        */
          fields[NGRIDTYPE * XXX_ODL_FIELD_SIZE],
                                /* Packed string of field names of the
                                   model.  The xxx_GEOM_GRID_TYPE structure
                                   is the largest so give enough space
                                   for it.                             */
        ErrMsg[STRLEN];       /* Error message                       */
  double d,                  /* Holder for doubles                  */
         darr[PROJPRMS_SIZE];/* Holder for double arrays (largest size) */

  /***********************************************************
   * Open the HDF output file.
   */
  file_id = Hopen (grid_filename, DFACC_RDONLY, 0);
  if (file_id == FAIL)
  {
    sprintf (ErrMsg, "Error opening HDF grid file %s.", grid_filename);
    xxx_LogStatus ("XXXLIB", __FILE__, __LINE__, ErrMsg);
    return (ERROR);
  }
  Vstart (file_id);

  /****************************************************
   * Find the xxx_GEOM_GRID_TYPE Vdata.
   */
  ref_num = VSfind (file_id, "xxx_GEOM_GRID_TYPE");
  if (ref_num == 0)
  {
    strcpy (ErrMsg,"Error finding xxx_GEOM_GRID_TYPE vdata.");
    xxx_LogStatus ("XXXLIB", __FILE__, __LINE__, ErrMsg);
    Vend (file_id);
    Hclose (file_id);
    return (ERROR);
  }

  /* Assign a vdata id for read-only access.
   */
  grid_type_vdata = VSattach (file_id, ref_num, "r");
  if (grid_type_vdata == FAIL)
  {
    strcpy (ErrMsg,"Error reading xxx_GEOM_GRID_TYPE vdata.");
    xxx_LogStatus ("XXXLIB", __FILE__, __LINE__, ErrMsg);
    Vend (file_id);
    Hclose (file_id);
    return (ERROR);
  }

  /* Set the order of the fields for the grid_type.  This is the same order we
   * wrote them.
   */
  istatn = VSQueryfields (grid_type_vdata, fields);
  if (istatn == FAIL)
  {
    strcpy (ErrMsg, "Error inquiring about the xxx_GEOM_GRID_TYPE vdata.");
    xxx_LogStatus ("XXXLIB", __FILE__, __LINE__, ErrMsg);
    VSdetach (grid_type_vdata);
    Vend (file_id);
    Hclose (file_id);
    return (ERROR);
  }
  istatn = VSsetfields(grid_type_vdata, fields);
  if (istatn == FAIL)
  {
    strcpy (ErrMsg, "Error setting the field names.");
    xxx_LogStatus ("XXXLIB", __FILE__, __LINE__, ErrMsg);
    VSdetach (grid_type_vdata);
    Vend (file_id);
    Hclose (file_id);
    return (ERROR);
  }

  /* Set up the data buffer and its pointer.
   */
  num_ints = NUM_GRID_TYPE_INTS;
  num_chars = NUM_GRID_TYPE_CHARS;
  num_doubles = NUM_GRID_TYPE_DBLS;
  databuf = malloc (num_ints*sizeof(int32) + num_doubles*sizeof(double) +
                    num_chars*sizeof(char));
  if (databuf == NULL)
  {
    strcpy (ErrMsg, "Error allocating data buffer memory.");
    xxx_LogStatus ("XXXLIB", __FILE__, __LINE__, ErrMsg);
    VSdetach (grid_type_vdata);
    Vend (file_id);
    Hclose (file_id);
    return (ERROR);
  }
  pntr = databuf;

  /* Malloc space for the integer array for the bands.
   */
  iarr = malloc (MAX_GEOM_GRID_BANDS * sizeof(int32));
  if (iarr == NULL)
  {
    strcpy (ErrMsg, "Error allocating memory.");
    xxx_LogStatus ("XXXLIB", __FILE__, __LINE__, ErrMsg);
    VSdetach (grid_type_vdata);
    Vend (file_id);
    Hclose (file_id);
    return (ERROR);
  }

  /* Read the data.  There is only one record in this Vdata.
   */
  istat32 = VSread (grid_type_vdata, databuf, 1, FULL_INTERLACE);
  if (istat32 == FAIL)
  {
    strcpy (ErrMsg, "Error reading the xxx_GEOM_GRID_BAND vdata.");
    xxx_LogStatus ("XXXLIB", __FILE__, __LINE__, ErrMsg);
    free (databuf);
    free (iarr);
    VSdetach (grid_type_vdata);
    Vend (file_id);
    Hclose (file_id);
    return (ERROR);
  }
  j = 0;

  /* Unpack the data elements from the buffer into the grid structure.
   */
  currsize = grid_type_elements[j].order * grid_type_elements[j].size;
  memcpy (str, pntr, currsize);
  strcpy (grid->gridname, str);
  j++;
  pntr += currsize;

  currsize = grid_type_elements[j].order * grid_type_elements[j].size;
  memcpy (&i32, pntr, currsize);
  grid->gridtype = (GEOM_GRID_TYPE)i32;
  j++;
  pntr += currsize;

  currsize = grid_type_elements[j].order * grid_type_elements[j].size;
  memcpy (&i32, pntr, currsize);
  grid->nbands = i32;
  j++;
  pntr += currsize;

  currsize = grid_type_elements[j].order * grid_type_elements[j].size;
  memcpy (iarr, pntr, currsize);
  for (i = 0; i < MAX_GEOM_GRID_BANDS; i++)
    grid->bandlist[i] = iarr[i];
  j++;
  pntr += currsize;

  currsize = grid_type_elements[j].order * grid_type_elements[j].size;
  memcpy (&i32, pntr, currsize);
  grid->dtype = i32;
  j++;
  pntr += currsize;

  currsize = grid_type_elements[j].order * grid_type_elements[j].size;
  memcpy (&i32, pntr, currsize);
  grid->correction_type = (CorrectionType) i32;
  j++;
  pntr += currsize;

  currsize = grid_type_elements[j].order * grid_type_elements[j].size;
  memcpy (&i32, pntr, currsize);
  grid->frame_type = (FRAME_TYPE) i32;
  j++;
  pntr += currsize;

  currsize = grid_type_elements[j].order * grid_type_elements[j].size;
  memcpy (str, pntr, currsize);
  strcpy (grid->units, str);
  j++;
  pntr += currsize;

  currsize = grid_type_elements[j].order * grid_type_elements[j].size;
  memcpy (&i32, pntr, currsize);
  grid->code = i32;
  j++;
  pntr += currsize;

  currsize = grid_type_elements[j].order * grid_type_elements[j].size;
  memcpy (str, pntr, currsize);
  strcpy (grid->datum, str);
  j++;
  pntr += currsize;

  currsize = grid_type_elements[j].order * grid_type_elements[j].size;
  memcpy (&i32, pntr, currsize);
  grid->spheroid = i32;
  j++;
  pntr += currsize;

  currsize = grid_type_elements[j].order * grid_type_elements[j].size;
  memcpy (&i32, pntr, currsize);
  grid->zone = i32;
  j++;
  pntr += currsize;

  currsize = grid_type_elements[j].order * grid_type_elements[j].size;
  memcpy (darr, pntr, currsize);
  for (i = 0; i < PROJPRMS_SIZE; i++)
    grid->projprms[i] = darr[i];
  j++;
  pntr += currsize;

  currsize = grid_type_elements[j].order * grid_type_elements[j].size;
  memcpy (darr, pntr, currsize);
  grid->corners.upleft.x = darr[0];
  grid->corners.upright.x = darr[1];
  grid->corners.loleft.x = darr[2];
  grid->corners.loright.x = darr[3];
  j++;
  pntr += currsize;

  currsize = grid_type_elements[j].order * grid_type_elements[j].size;
  memcpy (darr, pntr, currsize);
  grid->corners.upleft.y = darr[0];
  grid->corners.upright.y = darr[1];
  grid->corners.loleft.y = darr[2];
  grid->corners.loright.y = darr[3];
  j++;
  pntr += currsize;

  currsize = grid_type_elements[j].order * grid_type_elements[j].size;
  memcpy (darr, pntr, currsize);
  grid->center_pt.lat = darr[0];
  grid->center_pt.lng = darr[1];
  j++;
  pntr += currsize;

  currsize = grid_type_elements[j].order * grid_type_elements[j].size;
  memcpy (darr, pntr, currsize);
  grid->proj_center_pt.x = darr[0];
  grid->proj_center_pt.y = darr[1];
  j++;
  pntr += currsize;

  currsize = grid_type_elements[j].order * grid_type_elements[j].size;
  memcpy (&d, pntr, currsize);
  grid->rotate_angle = d;
  j++;
  pntr += currsize;

  currsize = grid_type_elements[j].order * grid_type_elements[j].size;
  memcpy (&d, pntr, currsize);
  grid->orient_angle = d;
  j++;
  pntr += currsize;

  /* Close the Vdata.
   */
  VSdetach (grid_type_vdata);
  free (iarr);
  free (databuf);

  /****************************************************
   * Find the xxx_GEOM_GRID_BAND_TYPE constants Vdata.
   */
  ref_num = VSfind (file_id, "Constants");
  if (ref_num == 0)
  {
    strcpy (ErrMsg,"Error finding xxx_GEOM_GRID_BAND_TYPE constants vdata.");
    xxx_LogStatus ("XXXLIB", __FILE__, __LINE__, ErrMsg);
    Vend (file_id);
    Hclose (file_id);
    return (ERROR);
  }

  /* Assign a vdata id for read-only access.
   */
  constant_vdata = VSattach (file_id, ref_num, "r");
  if (constant_vdata == FAIL)
  {
    strcpy (ErrMsg,"Error reading xxx_GEOM_GRID_BAND_TYPE constants vdata.");
    xxx_LogStatus ("XXXLIB", __FILE__, __LINE__, ErrMsg);
    Vend (file_id);
    Hclose (file_id);
    return (ERROR);
  }

  /* Set the order of the fields for the Constants.  This is the same
   * order we wrote them.
   */
  istatn = VSQueryfields (constant_vdata, fields);
  if (istatn == FAIL)
  {
    strcpy (ErrMsg, "Error inquiring about the constants vdata.");
    xxx_LogStatus ("XXXLIB", __FILE__, __LINE__, ErrMsg);
    VSdetach (constant_vdata);
    Vend (file_id);
    Hclose (file_id);
    return (ERROR);
  }
  istatn = VSsetfields(constant_vdata, fields);
  if (istatn == FAIL)
  {
    strcpy (ErrMsg, "Error setting the field names.");
    xxx_LogStatus ("XXXLIB", __FILE__, __LINE__, ErrMsg);
    VSdetach (constant_vdata);
    Vend (file_id);
    Hclose (file_id);
    return (ERROR);
  }

  /* Set up the data buffer and its pointer for the constants.
   */
  num_ints = NUM_CONSTANT_INTS;
  num_doubles = NUM_CONSTANT_DBLS;
  databuf = malloc ((num_ints*sizeof(int32) +
                     num_doubles*sizeof(double)) * grid->nbands);
  if (databuf == NULL)
  {
    strcpy (ErrMsg, "Error allocating data buffer memory.");
    xxx_LogStatus ("XXXLIB", __FILE__, __LINE__, ErrMsg);
    VSdetach (constant_vdata);
    Vend (file_id);
    Hclose (file_id);
    return (ERROR);
  }
  pntr = databuf;

  /* Read the data.  There are grid->nbands records in this Vdata.
   */
  istat32 = VSread (constant_vdata, databuf, grid->nbands, FULL_INTERLACE);
  if (istat32 == FAIL)
  {
    strcpy (ErrMsg, "Error reading the constant vdata.");
    xxx_LogStatus ("XXXLIB", __FILE__, __LINE__, ErrMsg);
    free (databuf);
    VSdetach (constant_vdata);
    Vend (file_id);
    Hclose (file_id);
    return (ERROR);
  }

  /* Unpack the data elements from the buffer into the grid structure.
   */
  for (i = 0, j = 0; i < grid->nbands; i++, j = 0)
  {
    /* Get the reference value of the current band.
     */
    ref = grid->bandlist[i];

    currsize = constant_elements[j].order * constant_elements[j].size;
    memcpy (&i32, pntr, currsize);
    grid->gridbands[ref].band_number = i32;
    j++;
    pntr += currsize;

    currsize = constant_elements[j].order * constant_elements[j].size;
    memcpy (&i32, pntr, currsize);
    grid->gridbands[ref].gridtype = (GEOM_GRID_TYPE)i32;
    j++;
    pntr += currsize;

    currsize = constant_elements[j].order * constant_elements[j].size;
    memcpy (&i32, pntr, currsize);
    grid->gridbands[ref].cell_lines = i32;
    j++;
    pntr += currsize;

    currsize = constant_elements[j].order * constant_elements[j].size;
    memcpy (&i32, pntr, currsize);
    grid->gridbands[ref].cell_samps = i32;
    j++;
    pntr += currsize;

    currsize = constant_elements[j].order * constant_elements[j].size;
    memcpy (&d, pntr, currsize);
    grid->gridbands[ref].pixsize_line = d;
    j++;
    pntr += currsize;

    currsize = constant_elements[j].order * constant_elements[j].size;
    memcpy (&d, pntr, currsize);
    grid->gridbands[ref].pixsize_samp = d;
    j++;
    pntr += currsize;

    currsize = constant_elements[j].order * constant_elements[j].size;
    memcpy (&i32, pntr, currsize);
    grid->gridbands[ref].output_image_lines = i32;
    j++;
    pntr += currsize;

    currsize = constant_elements[j].order * constant_elements[j].size;
    memcpy (&i32, pntr, currsize);
    grid->gridbands[ref].output_image_samps = i32;
    j++;
    pntr += currsize;

    currsize = constant_elements[j].order * constant_elements[j].size;
    memcpy (&i32, pntr, currsize);
    grid->gridbands[ref].num_in_lines = i32;
    j++;
    pntr += currsize;

    currsize = constant_elements[j].order * constant_elements[j].size;
    memcpy (&i32, pntr, currsize);
    grid->gridbands[ref].num_in_samps = i32;
    j++;
    pntr += currsize;

    currsize = constant_elements[j].order * constant_elements[j].size;
    memcpy (&i32, pntr, currsize);
    grid->gridbands[ref].num_out_lines = i32;
    j++;
    pntr += currsize;

    currsize = constant_elements[j].order * constant_elements[j].size;
    memcpy (&i32, pntr, currsize);
    grid->gridbands[ref].num_out_samps = i32;
    j++;
    pntr += currsize;

    currsize = constant_elements[j].order * constant_elements[j].size;
    memcpy (&i32, pntr, currsize);
    grid->gridbands[ref].forward_present = i32;
    j++;
    pntr += currsize;

    currsize = constant_elements[j].order * constant_elements[j].size;
    memcpy (&i32, pntr, currsize);
    grid->gridbands[ref].reverse_present = i32;
    j++;
    pntr += currsize;

    currsize = constant_elements[j].order * constant_elements[j].size;
    memcpy (&i32, pntr, currsize);
    grid->gridbands[ref].grid_rows = i32;
    j++;
    pntr += currsize;

    currsize = constant_elements[j].order * constant_elements[j].size;
    memcpy (&i32, pntr, currsize);
    grid->gridbands[ref].grid_cols = i32;
    j++;
    pntr += currsize;

    currsize = constant_elements[j].order * constant_elements[j].size;
    memcpy (&i32, pntr, currsize);
    grid->gridbands[ref].degree = i32;
    j++;
    pntr += currsize;
  }

  /* Close the Vdata.
   */
  VSdetach (constant_vdata);
  free (databuf);

  /* Close the file.
   */
  Vend (file_id);
  Hclose (file_id);

  /* Set the pointers for all bands in the grid to NULL.
   */
  for (i = 0; i < MAX_GEOM_GRID_BANDS; i++)
  {
    /* Set the pointers to NULL.
     */
    grid->gridbands[i].in_lines = NULL;
    grid->gridbands[i].in_samps = NULL;
    grid->gridbands[i].out_lines = NULL;
    grid->gridbands[i].out_samps = NULL;
    grid->gridbands[i].poly_lines = NULL;
    grid->gridbands[i].poly_samps = NULL;
    grid->gridbands[i].forward_coeffs = NULL;
    grid->gridbands[i].reverse_coeffs = NULL;
  }

  return (SUCCESS);
}


/******************************************************************************
NAME:           xxx_get_geom_grid_data

PURPOSE:
Read the dynamically allocated grid data from an HDF file for the specified 
bands.

RETURN VALUE:   
Type = int
Value    Description
-----    -----------
SUCCESS  Successful completion
ERROR    Operation failed

NOTES:
This routine will read the dynamically allocated grid data from the HDF file 
for the specified band numbers.  xxx_get_geom_grid_headers should already have 
been called so that the appropriate header and constant information is available
for the size of the dynamically allocated memory.

The user may specify one band, several bands, or no bands.  If no bands are
specified, then all the bands are read from the HDF file.  In this case, the
bandlist array will be filled in with the bands in the grid and nbands will be
set to the correct value.  This routine will check the grid to verify that
the specified bands are in the HDF file.  If a band does not exist, then this
routine will exit with an error.

It is important to note that this routine will allocate the memory needed for 
the grid data.

ALGORITHM REFERENCES:
See xxx_geom_grid.h for a description of the geometric grid structure.

******************************************************************************/

int xxx_get_geom_grid_data 
(
  char grid_filename[STRLEN],/* I: HDF input file name                    */
  int *bands,                /* I/O: Array of bands to free from the grid */
  int *nbands,               /* I/O: Number of bands in the bands array   */
  xxx_GEOM_GRID_TYPE *grid   /* O: Grid structure to be output            */
)
{
     int  i, j, l,              /* Loop variables                           */
          currband,             /* Current band in the bands array          */
          found,                /* Boolean to specify if the band was found */
          ref,                  /* Current reference band value             */
          status,               /* status of return from function           */
          order,                /* Order of the current array               */
          istatnt;              /* Return values                            */
   int32  rank,                 /* Rank of the dimsizes array.              */
          dim_sizes[4],         /* Array of the dimension sizes.            */
          start[] = {0, 0, 0, 0, 0}, /* Array of 0 starting positions.      */
          num_type,             /* Data type for the data in the array      */
          attrib,               /* Number of attributes in the data set     */
          sd_id,                /* DDR HDF file pointer for sds             */
          sds_index,            /* SDS index pointer                        */
          in_lines_sds,         /* SDS id pointer (in_lines)                */
          in_samps_sds,         /* SDS id pointer (in_samps)                */
          out_lines_sds,        /* SDS id pointer (out_lines)               */
          out_samps_sds,        /* SDS id pointer (out_samps)               */
          poly_ls_sds,          /* SDS id pointer (poly_lines/samps)        */
          forward_coeffs_sds,   /* SDS id pointer (forward_coeffs)          */
          reverse_coeffs_sds;   /* SDS id pointer (reverse_coeffs)          */
   int32  istat32;              /* Return values                            */
    intn  istatn;               /* Return values                            */
   uint8 *databuf;              /* Data pointer                             */
    char  sds_name[STRLEN],     /* Name of the sds                          */
          name[STRLEN],         /* SDS name                                 */
          ErrMsg[STRLEN];       /* Error message                            */
  double *double_ptr;           /* alias the databuf to a double buffer     */

  /* If no bands are specified then read all the bands in the grid.         */
  if (*nbands == 0)
  {
    *nbands = grid->nbands;
    memcpy (bands, grid->bandlist, *nbands * sizeof (int));
  }
  else
  {
    /* Verify the specified bands exist in the grid.
     */
    for (i = 0; i < *nbands; i++)
    {
      for (j = 0, found = FALSE; j < grid->nbands; j++)
        if (bands[i] == grid->bandlist[j])
        {
          found = TRUE;
          break;
        }
      
      /* If the band was not found then exit with an error.
       */
      if (!found)
      {
        sprintf (ErrMsg, "Band %d is not available in the HDF file.",
                 bands[i]);
        xxx_LogStatus ("XXXLIB", __FILE__, __LINE__, ErrMsg);
        return (ERROR);
      }
    }
  }

  /***********************************************************
   * Malloc space for the grid.
   */
  istatnt = xxx_malloc_geom_grid (bands, nbands, grid);
  if (istatnt == ERROR)
  {
    strcpy (ErrMsg,"Error allocating memory for the grid data.");
    xxx_LogStatus ("XXXLIB", __FILE__, __LINE__, ErrMsg);
    return (ERROR);
  }

  /***********************************************************
   * Open the file for the SDS interface.
   */
  sd_id = SDstart (grid_filename, DFACC_RDONLY);
  if (sd_id == FAIL)
  {
    sprintf (ErrMsg, "Error opening grid file %s for SDS.", grid_filename);
    xxx_LogStatus ("XXXLIB", __FILE__, __LINE__, ErrMsg);
    return (ERROR);
  }

  /**********************************************************************
   * Loop through the bands and read the pointer values in the SDSs for
   * each band.
   */
  for (currband = 0; currband < *nbands; currband++)
  {
    /* Find the reference position.
     */
    ref = bands[currband];

    /****************************************************
     * Find the in_lines SDS.
     */
    status = sprintf (name, "In_lines%d", ref);
	if (status < 1)
		{
    	xxx_LogStatus ("XXXLIB", __FILE__, __LINE__, "Error in sprintf");
    	return (ERROR);
		}
    sds_index = SDnametoindex (sd_id, name);
    if (sds_index == FAIL)
    {
      sprintf (ErrMsg, "Error finding the in_lines SDS for band %d.",ref);
      xxx_LogStatus ("XXXLIB", __FILE__, __LINE__, ErrMsg);
      SDend (sd_id);
      return (ERROR);
    }
    in_lines_sds = SDselect (sd_id, sds_index);

    /* Get the rank and dimension sizes.
     */
    istatn = SDgetinfo (in_lines_sds, sds_name, &rank, dim_sizes, &num_type,
                        &attrib);
    if (istatn == FAIL)
    {
      strcpy (ErrMsg, "Error getting SDS info.");
      xxx_LogStatus ("XXXLIB", __FILE__, __LINE__, ErrMsg);
      SDendaccess (in_lines_sds);
      SDend (sd_id);
      return (ERROR);
    }

    /* Set up the data buffer and its pointer for the in_lines.
     */
    databuf = malloc (grid->gridbands[ref].num_in_lines * sizeof(double));
    if (databuf == NULL)
    {
      strcpy (ErrMsg, "Error allocating data buffer memory.");
      xxx_LogStatus ("XXXLIB", __FILE__, __LINE__, ErrMsg);
      SDendaccess (in_lines_sds);
      SDend (sd_id);
      return (ERROR);
    }
    double_ptr = (double*)databuf;

    /* Read the in_lines.  This is the same order we wrote them.
     */
    istatn = SDreaddata (in_lines_sds, start, NULL, dim_sizes, (VOIDP)databuf);
    if (istatn == FAIL)
    {
      strcpy (ErrMsg, "Error reading the vdata.");
      xxx_LogStatus ("XXXLIB", __FILE__, __LINE__, ErrMsg);
      free (databuf);
      SDendaccess (in_lines_sds);
      SDend (sd_id);
      return (ERROR);
    }

    for (j = 0; j < grid->gridbands[ref].num_in_lines; j++)
      grid->gridbands[ref].in_lines[j] = double_ptr[j];

    /* Close the SDS.
     */
    SDendaccess (in_lines_sds);
    free (databuf);

    /****************************************************
     * Find the in_samps SDS.
     */
    status = sprintf (name, "In_samps%d", ref);
	if (status < 1)
		{
    	xxx_LogStatus ("XXXLIB", __FILE__, __LINE__, "Error in sprintf");
    	return (ERROR);
		}
    sds_index = SDnametoindex (sd_id, name);
    if (sds_index == FAIL)
    {
      sprintf (ErrMsg, "Error finding the in_samps SDS for band %d.", ref);
      xxx_LogStatus ("XXXLIB", __FILE__, __LINE__, ErrMsg);
      SDend (sd_id);
      return (ERROR);
    }
    in_samps_sds = SDselect (sd_id, sds_index);

    /* Get the rank and dimension sizes.
     */
    istatn = SDgetinfo (in_samps_sds, sds_name, &rank, dim_sizes, &num_type,
                        &attrib);
    if (istatn == FAIL)
    {
      strcpy (ErrMsg, "Error getting SDS info.");
      xxx_LogStatus ("XXXLIB", __FILE__, __LINE__, ErrMsg);
      SDendaccess (in_samps_sds);
      SDend (sd_id);
      return (ERROR);
    }

    /* Set up the data buffer and its pointer for the in_samps.
     */
    databuf = malloc (grid->gridbands[ref].num_in_samps * sizeof(double));
    if (databuf == NULL)
    {
      strcpy (ErrMsg, "Error allocating data buffer memory.");
      xxx_LogStatus ("XXXLIB", __FILE__, __LINE__, ErrMsg);
      SDendaccess (in_samps_sds);
      SDend (sd_id);
      return (ERROR);
    }
    double_ptr = (double*)databuf;

    /* Read the in_samps.  This is the same order we wrote them.
     */
    istatn = SDreaddata (in_samps_sds, start, NULL, dim_sizes, (VOIDP)databuf);
    if (istatn == FAIL)
    {
      strcpy (ErrMsg, "Error reading the vdata.");
      xxx_LogStatus ("XXXLIB", __FILE__, __LINE__, ErrMsg);
      free (databuf);
      SDendaccess (in_samps_sds);
      SDend (sd_id);
      return (ERROR);
    }

    /* Unpack the data elements from the buffer into the grid structure.
     */
    for (j = 0; j < grid->gridbands[ref].num_in_samps; j++)
      grid->gridbands[ref].in_samps[j] = double_ptr[j];

    /* Close the SDS.
     */
    SDendaccess (in_samps_sds);
    free (databuf);

    /****************************************************
     * Find the out_lines SDS.
     */
    status = sprintf (name, "Out_lines%d", ref);
	if (status < 1)
		{
    	xxx_LogStatus ("XXXLIB", __FILE__, __LINE__, "Error in sprintf");
    	return (ERROR);
		}
    sds_index = SDnametoindex (sd_id, name);
    if (sds_index == FAIL)
    {
      sprintf (ErrMsg, "Error finding the out_lines SDS for band %d.", ref);
      xxx_LogStatus ("XXXLIB", __FILE__, __LINE__, ErrMsg);
      SDend (sd_id);
      return (ERROR);
    }
    out_lines_sds = SDselect (sd_id, sds_index);

    /* Get the rank and dimension sizes.
     */
    istatn = SDgetinfo (out_lines_sds, sds_name, &rank, dim_sizes, &num_type,
                       &attrib);
    if (istatn == FAIL)
    {
      strcpy (ErrMsg, "Error getting out_lines SDS info.");
      xxx_LogStatus ("XXXLIB", __FILE__, __LINE__, ErrMsg);
      SDendaccess (out_lines_sds);
      SDend (sd_id);
      return (ERROR);
    }

    /* Set up the data buffer and its pointer for the out_lines.
     */
    databuf = malloc (grid->gridbands[ref].num_out_lines * sizeof(double));
    if (databuf == NULL)
    {
      strcpy (ErrMsg, "Error allocating data buffer memory.");
      xxx_LogStatus ("XXXLIB", __FILE__, __LINE__, ErrMsg);
      SDendaccess (out_lines_sds);
      SDend (sd_id);
      return (ERROR);
    }
    double_ptr = (double*)databuf;

    /* Read the out_lines
     */
    istatn = SDreaddata (out_lines_sds, start, NULL, dim_sizes, (VOIDP)databuf);
    if (istatn == FAIL)
    {
      strcpy (ErrMsg, "Error reading the out_lines vdata.");
      xxx_LogStatus ("XXXLIB", __FILE__, __LINE__, ErrMsg);
      free (databuf);
      SDendaccess (out_lines_sds);
      SDend (sd_id);
      return (ERROR);
    }

    for (j = 0; j < grid->gridbands[ref].num_out_lines; j++)
      grid->gridbands[ref].out_lines[j] = double_ptr[j];

    /* Close the SDS.
     */
    SDendaccess (out_lines_sds);
    free (databuf);

    /****************************************************
     * Find the out_samps SDS.
     */
    status = sprintf (name, "Out_samps%d", ref);
	if (status < 1)
		{
    	xxx_LogStatus ("XXXLIB", __FILE__, __LINE__, "Error in sprintf");
    	return (ERROR);
		}
    sds_index = SDnametoindex (sd_id, name);
    if (sds_index == FAIL)
    {
      sprintf (ErrMsg, "Error finding the out_samps SDS for band %d.", ref);
      xxx_LogStatus ("XXXLIB", __FILE__, __LINE__, ErrMsg);
      SDend (sd_id);
      return (ERROR);
    }
    out_samps_sds = SDselect (sd_id, sds_index);

    /* Get the rank and dimension sizes.
     */
    istatn = SDgetinfo (out_samps_sds, sds_name, &rank, dim_sizes, &num_type,
                       &attrib);
    if (istatn == FAIL)
    {
      strcpy (ErrMsg, "Error getting out_samps SDS info.");
      xxx_LogStatus ("XXXLIB", __FILE__, __LINE__, ErrMsg);
      SDendaccess (out_samps_sds);
      SDend (sd_id);
      return (ERROR);
    }

    /* Set up the data buffer and its pointer for the out_samps.
     */
    databuf = malloc (grid->gridbands[ref].num_out_samps * sizeof(double));
    if (databuf == NULL)
    {
      strcpy (ErrMsg, "Error allocating data buffer memory.");
      xxx_LogStatus ("XXXLIB", __FILE__, __LINE__, ErrMsg);
      SDendaccess (out_samps_sds);
      SDend (sd_id);
      return (ERROR);
    }
    double_ptr = (double*)databuf;

    /* Read the out_samps
     */
    istatn = SDreaddata (out_samps_sds, start, NULL, dim_sizes, (VOIDP)databuf);
    if (istatn == FAIL)
    {
      strcpy (ErrMsg, "Error reading the out_samps vdata.");
      xxx_LogStatus ("XXXLIB", __FILE__, __LINE__, ErrMsg);
      free (databuf);
      SDendaccess (out_samps_sds);
      SDend (sd_id);
      return (ERROR);
    }

    for (j = 0; j < grid->gridbands[ref].num_out_samps; j++)
      grid->gridbands[ref].out_samps[j] = double_ptr[j];

    /* Close the SDS.
     */
    SDendaccess (out_samps_sds);
    free (databuf);
    
    /****************************************************
     * Find the poly_lines/samps SDS.
     */
    status = sprintf (name, "Poly_ls%d", ref);
	if (status < 1)
		{
    	xxx_LogStatus ("XXXLIB", __FILE__, __LINE__, "Error in sprintf");
    	return (ERROR);
		}
    sds_index = SDnametoindex (sd_id, name);
    if (sds_index == FAIL)
    {
      sprintf (ErrMsg, "Error finding the poly_lines/samps SDS for band %d.",
               ref);
      xxx_LogStatus ("XXXLIB", __FILE__, __LINE__, ErrMsg);
      SDend (sd_id);
      return (ERROR);
    }
    poly_ls_sds = SDselect (sd_id, sds_index);

    /* Get the rank and dimension sizes.
     */
    istatn = SDgetinfo (poly_ls_sds, sds_name, &rank, dim_sizes, &num_type,
                        &attrib);
    if (istatn == FAIL)
    {
      strcpy (ErrMsg, "Error getting poly_ls SDS info.");
      xxx_LogStatus ("XXXLIB", __FILE__, __LINE__, ErrMsg);
      SDendaccess (poly_ls_sds);
      SDend (sd_id);
      return (ERROR);
    }

    /* Set up the data buffer and its pointer for the poly_lines/samps.
     */
    databuf = malloc (2 * (grid->gridbands[ref].degree+1) *
                     (grid->gridbands[ref].degree+1) * sizeof(double));
    if (databuf == NULL)
    {
      strcpy (ErrMsg, "Error allocating data buffer memory.");
      xxx_LogStatus ("XXXLIB", __FILE__, __LINE__, ErrMsg);
      SDendaccess (poly_ls_sds);
      SDend (sd_id);
      return (ERROR);
    }
    double_ptr = (double*)databuf;

    /* Read the poly_lines/samps.  This is the same order we wrote them.
     */
    istatn = SDreaddata (poly_ls_sds, start, NULL, dim_sizes, (VOIDP)databuf);
    if (istatn == FAIL)
    {
      strcpy (ErrMsg, "Error reading the poly_ls vdata.");
      xxx_LogStatus ("XXXLIB", __FILE__, __LINE__, ErrMsg);
      free (databuf);
      SDendaccess (poly_ls_sds);
      SDend (sd_id);
      return (ERROR);
    }

    /* Unpack the data elements from the buffer into the grid structure.
     */
    order = (grid->gridbands[ref].degree + 1) *
            (grid->gridbands[ref].degree + 1);
    for (j = 0; j < order; j++)
      grid->gridbands[ref].poly_lines[j] = double_ptr[j];

    double_ptr = &double_ptr[order];

    for (j = 0; j < order; j++)
      grid->gridbands[ref].poly_samps[j] = double_ptr[j];

    /* Close the Vdata.
     */
    SDendaccess (poly_ls_sds);
    free (databuf);

    /***********************************************************
     * Find the forward_coeffs SDS if it is present in the grid
     */
    if (grid->gridbands[ref].forward_present)
    {
        status = sprintf (name, "forward_coeffs%d", ref);
		if (status < 1)
			{
    		xxx_LogStatus ("XXXLIB", __FILE__, __LINE__, "Error in sprintf");
    		return (ERROR);
			}
        sds_index = SDnametoindex (sd_id, name);
        if (sds_index == FAIL)
        {
          sprintf (ErrMsg, "Error finding the forward_coeffs SDS for band %d.",
                   ref);
          xxx_LogStatus ("XXXLIB", __FILE__, __LINE__, ErrMsg);
          SDend (sd_id);
          return (ERROR);
        }
        forward_coeffs_sds = SDselect (sd_id, sds_index);

        /* Get the rank and dimension sizes.
         */
        istatn = SDgetinfo (forward_coeffs_sds, sds_name, &rank, dim_sizes, 
                            &num_type, &attrib);
        if (istatn == FAIL)
        {
          strcpy (ErrMsg, "Error getting forward_coeffs SDS info.");
          xxx_LogStatus ("XXXLIB", __FILE__, __LINE__, ErrMsg);
          SDendaccess (forward_coeffs_sds);
          SDend (sd_id);
          return (ERROR);
        }

        /* Set up the data buffer and its pointer for the forward_coeffs.
         */
        databuf = malloc (grid->gridbands[ref].grid_rows * 
                          grid->gridbands[ref].grid_cols *
                          COEFS_SIZE*XY_SIZE * sizeof(double));
        if (databuf == NULL)
        {
          strcpy (ErrMsg, "Error allocating data buffer memory.");
          xxx_LogStatus ("XXXLIB", __FILE__, __LINE__, ErrMsg);
          SDendaccess (forward_coeffs_sds);
          SDend (sd_id);
          return (ERROR);
        }
        double_ptr = (double*)databuf;

        /* Read the forward coeffs
         */
        istatn = SDreaddata (forward_coeffs_sds, start, NULL, dim_sizes, 
                             (VOIDP)databuf);
        if (istatn == FAIL)
        {
          strcpy (ErrMsg, "Error reading the forward_coeffs vdata.");
          xxx_LogStatus ("XXXLIB", __FILE__, __LINE__, ErrMsg);
          free (databuf);
          SDendaccess (forward_coeffs_sds);
          SDend (sd_id);
          return (ERROR);
        }

        /* Unpack the data elements from the buffer into the grid structure.
         */
        for (j = 0; j < COEFS_SIZE; j++)
        {
            /* forward a coefs
             */
            order = grid->gridbands[ref].grid_rows *
                    grid->gridbands[ref].grid_cols;
            for (l = 0; l < order; l++)
              grid->gridbands[ref].forward_coeffs[l].a[j] = double_ptr[l];

            double_ptr = &double_ptr[order];

            /* forward b coefs
             */
            for (l = 0; l < order; l++)
              grid->gridbands[ref].forward_coeffs[l].b[j] = double_ptr[l];

            double_ptr = &double_ptr[order];
        }

        /* Close the SDS.
         */
        SDendaccess (forward_coeffs_sds);
        free (databuf);
    }

    /***********************************************************
     * Find the reverse_coeffs SDS if it is present in the grid
     */
    if (grid->gridbands[ref].reverse_present)
    {
        status = sprintf (name, "reverse_coeffs%d", ref);
		if (status < 1)
			{
    		xxx_LogStatus ("XXXLIB", __FILE__, __LINE__, "Error in sprintf");
    		return (ERROR);
			}
        sds_index = SDnametoindex (sd_id, name);
        if (sds_index == FAIL)
        {
          sprintf (ErrMsg, "Error finding the reverse_coeffs SDS for band %d.",
                   ref);
          xxx_LogStatus ("XXXLIB", __FILE__, __LINE__, ErrMsg);
          SDend (sd_id);
          return (ERROR);
        }
        reverse_coeffs_sds = SDselect (sd_id, sds_index);

        /* Get the rank and dimension sizes.
         */
        istatn = SDgetinfo (reverse_coeffs_sds, sds_name, &rank, dim_sizes, 
                           &num_type, &attrib);
        if (istatn == FAIL)
        {
          strcpy (ErrMsg, "Error getting reverse_coeffs SDS info.");
          xxx_LogStatus ("XXXLIB", __FILE__, __LINE__, ErrMsg);
          SDendaccess (reverse_coeffs_sds);
          SDend (sd_id);
          return (ERROR);
        }

        /* Set up the data buffer and its pointer for the reverse_coeffs.
         */
        databuf = malloc (grid->gridbands[ref].grid_rows * 
                          grid->gridbands[ref].grid_cols *
                          COEFS_SIZE*XY_SIZE * sizeof(double));
        if (databuf == NULL)
        {
          strcpy (ErrMsg, "Error allocating data buffer memory.");
          xxx_LogStatus ("XXXLIB", __FILE__, __LINE__, ErrMsg);
          SDendaccess (reverse_coeffs_sds);
          SDend (sd_id);
          return (ERROR);
        }
        double_ptr = (double*)databuf;

        /* Read the reverse coeffs
         */
        istatn = SDreaddata (reverse_coeffs_sds, start, NULL, dim_sizes, 
                            (VOIDP)databuf);
        if (istatn == FAIL)
        {
          strcpy (ErrMsg, "Error reading the reverse_coeffs vdata.");
          xxx_LogStatus ("XXXLIB", __FILE__, __LINE__, ErrMsg);
          free (databuf);
          SDendaccess (reverse_coeffs_sds);
          SDend (sd_id);
          return (ERROR);
        }

        /* Unpack the data elements from the buffer into the grid structure.
         */
        for (j = 0; j < COEFS_SIZE; j++)
        {
            /* reverse a coefs
             */
            order = grid->gridbands[ref].grid_rows *
                    grid->gridbands[ref].grid_cols;
            for (l = 0; l < order; l++)
              grid->gridbands[ref].reverse_coeffs[l].a[j] = double_ptr[l];

            double_ptr = &double_ptr[order];

            /* reverse b coefs
             */
            for (l = 0; l < order; l++)
              grid->gridbands[ref].reverse_coeffs[l].b[j] = double_ptr[l];

            double_ptr = &double_ptr[order];
        }

        /* Close the SDS.
         */
        SDendaccess (reverse_coeffs_sds);
        free (databuf);
    }  
  } /* Loop over the bands. */

  /***********************************************************
   * Close the file.
   */
  SDend (sd_id);

  return (SUCCESS);
}

/******************************************************************************
NAME:           xxx_SOM_geomgrid_calc

PURPOSE:
flip the grid coords if the data is SOM, so we can perfom calculations with
the coordinates

DEVELOPMENT HISTORY:
AUTHOR        Change-ID    DATE     Description of Change
------        ---------  --------   ---------------------
Amy Vander Vrste    1.0  08/28/97   function so SOM logic in one place

RETURN VALUE:   Type = int
Value    Description
-----    -----------
SUCCESS  Successful completion
ERROR    Operation failed

NOTES:   This function was written so that we can have the SOM logic in one place
		 and xxx_get_grid and tmresample can call it.  This function only flips 
		 SOM corner coorindates after they are read.  This allows a consistant i
		 user interface, yet allows the algorithms using SOM coodinates not to 
		 change.

ALGORITHM REFERENCES:
See xxx_grid.h for a description of the grid structure.

******************************************************************************/
void xxx_SOM_geomgrid_calc (
  xxx_GEOM_GRID_TYPE *grid)       /* I/O: Grid structure to be output            */
{
double tmp;		     /* Temporary variable for flipping corner points*/

/*
  Flip the corner points for the SOM projection ONLY
*/
if (grid->code == SOM)
    {
	tmp = -grid->corners.upleft.x;
    grid->corners.upleft.x = grid->corners.upleft.y;
    grid->corners.upleft.y = tmp;

    tmp = -grid->corners.loleft.x;
    grid->corners.loleft.x = grid->corners.loleft.y;
    grid->corners.loleft.y = tmp;

    tmp = -grid->corners.upright.x;
    grid->corners.upright.x = grid->corners.upright.y;
    grid->corners.upright.y = tmp;

    tmp = -grid->corners.loright.x;
    grid->corners.loright.x = grid->corners.loright.y;
    grid->corners.loright.y = tmp;
    }
}
