/******************************************************************************
NAME:           xxx_get_dem

PURPOSE:
The xxx_get_dem routine gets a digital elevation model and verifies it is
legal to use for the source image.

RETURN VALUE:
Type = int
Value           Description
-----           -----------
SUCCESS         DEM successfully read
ERROR           Error; DEM not read

NOTES:
- The memory for the DEM is allocated by this routine, but it is the 
  responsibility of the user to free the memory.
- The image corners, line in image, and samples per image line refer to the 
  image being used with the DEM, not the DEM itself.  These parameters are used
  to verify the image and the DEM are compatible.

ALGORITHM REFERENCES:
none

******************************************************************************/
#include <limits.h>    /* min & max short values for calculating dem min/max */
#include "xxx_DDR.h"   /* DDR definition */
#include "exx_geometric.h" /* exx_ReadDDR.c prototype */
#include "xxx_utility.h"   /* xxx_create_DDR_name prototype */
#include "xxx_hdf_io.h" /* axx_get_elev prototype */
#include "xxx_image.h"

int xxx_get_dem
(
    char *DEM_name,              /* INPUT: name of DEM to read */
    struct CORNERS *image_corners_ptr, /* INPUT: corners of image */
    int  lines_in_image,         /* INPUT: lines in image */
    int  samples_per_image_line, /* INPUT: samples in an image line */
    double image_pixsize,        /* INPUT: input image pixel size */
    IMAGE *dem_ptr               /* OUTPUT: data read from the DEM file */
)
{
    char DDR_name[STRLEN];  /* DDR name for the DEM */
    xxx_DDRControl_TYPE *DDR_control_ptr; /* pointer to data read from DDR */
    char msg[STRLEN];        /* buffer for messages */
    short min, max, temp;    /* variable for searching for min/max elevation */
    int  index;              /* loop counter */
    short *dem_buf_ptr;      /* pointer to DEM buffer */
    int  dem_size;           /* number of elements in the DEM buffer */
    double max_corner_diff;  /* maximum allowed difference in the DEM and
                                image corner points */
    int  scaled_lines_in_image; /* number of lines in the image after the
                                   lines_in_image is scaled by the ratio of
                                   the image pixel size to DEM pixel size */
    int  scaled_samps_in_image; /* samples across the image after the
                                   samples_per_image_line is scaled by the ratio
                                   of the image pixel size to DEM pixel size */
    int  dem_nl,             /* number of lines in DEM as read from the HDF */
         dem_ns;             /* number of samps in DEM as read from the HDF */
    DataType dem_datatype;   /* datatype of DEM as read from the HDF */

    /* create the DDR name */
    xxx_create_DDR_name(DDR_name, DEM_name);
    
    /* read the DDR to obtain the pixel size and image size */
    DDR_control_ptr = xxx_ReadDDR(DDR_name, msg);
    if (DDR_control_ptr == NULL)
    {
        xxx_LogStatus("GPS",__FILE__,__LINE__, msg);
        sprintf(msg,"Error reading DEM file: %s",DEM_name);
        xxx_LogStatus("GPS",__FILE__,__LINE__,msg);
        return (ERROR);
    }
    
    /* copy the pixel size from the DDR */
    dem_ptr->pixel_size_x = DDR_control_ptr->p_DDR->ProjectionDistX;
    dem_ptr->pixel_size_y = DDR_control_ptr->p_DDR->ProjectionDistY;

    /* get the DEM size from the DDR */
    dem_ptr->ns = DDR_control_ptr->p_DDR->NumberSamplesInScene;
    dem_ptr->nl = DDR_control_ptr->p_DDR->NumberLinesInScene;
    
    /* set the maximum difference between the DEM and image corner points
       to one-half of the pixsize */
    max_corner_diff = dem_ptr->pixel_size_x * 0.5;
    
    /* verify the DEM is valid */
    /* make sure the corners of the image are within 0.5 pixels */
    if (
      (fabs(image_corners_ptr->upleft.y - DDR_control_ptr->p_DDR->UpperLeft[0])
          > max_corner_diff) ||
      (fabs(image_corners_ptr->upleft.x - DDR_control_ptr->p_DDR->UpperLeft[1])
          > max_corner_diff) ||
      (fabs(image_corners_ptr->upright.y - DDR_control_ptr->p_DDR->UpperRight[0])
          > max_corner_diff) ||
      (fabs(image_corners_ptr->upright.x - DDR_control_ptr->p_DDR->UpperRight[1])
          > max_corner_diff) ||
      (fabs(image_corners_ptr->loleft.y - DDR_control_ptr->p_DDR->LowerLeft[0])
          > max_corner_diff) ||
      (fabs(image_corners_ptr->loleft.x - DDR_control_ptr->p_DDR->LowerLeft[1])
          > max_corner_diff) ||
      (fabs(image_corners_ptr->loright.y - DDR_control_ptr->p_DDR->LowerRight[0])
          > max_corner_diff) ||
      (fabs(image_corners_ptr->loright.x - DDR_control_ptr->p_DDR->LowerRight[1])
          > max_corner_diff)
       )
    {
        xxx_LogStatus("GPS",__FILE__,__LINE__,"Error: DEM and image corners do not match");
        XXX_FREE_DDR(DDR_control_ptr);
        return (ERROR);
    }

    dem_ptr->ul_x = DDR_control_ptr->p_DDR->UpperLeft[1];
    dem_ptr->ul_y = DDR_control_ptr->p_DDR->UpperLeft[0];
   
    /* scale the number of lines and samples in the input image by the 
       ratio of the image pixel size to the DEM pixel size.  The 1.5 is 
       added instead of just 1 to round the value */
    scaled_lines_in_image = (int)((lines_in_image - 1) * image_pixsize / dem_ptr->pixel_size_x + 1.5);
    scaled_samps_in_image = (int)((samples_per_image_line - 1) * image_pixsize / dem_ptr->pixel_size_x + 1.5);

    /* make sure the number of lines and sample agree between the DEM and the image */
    if ((dem_ptr->ns != scaled_samps_in_image ) || 
        (dem_ptr->nl != scaled_lines_in_image))
    {
        xxx_LogStatus("GPS",__FILE__,__LINE__,"Error: DEM and image sizes do not match");
        XXX_FREE_DDR(DDR_control_ptr);
        return (ERROR);
    }

    /* verify data type is short */
    if (DDR_control_ptr->p_DDR->DataType != DFNT_INT16)
    {
        xxx_LogStatus("GPS",__FILE__,__LINE__,"Error: DEM does not contain short integer data");
        XXX_FREE_DDR(DDR_control_ptr);
        return (ERROR);        
    }
    
    /* read the DEM data */
    dem_ptr->data = xxx_get_elev(DEM_name, &dem_nl, &dem_ns, &dem_datatype);
    if (dem_ptr->data == NULL)
    {
        sprintf(msg, "Error reading DEM data from %s", DEM_name);
        xxx_LogStatus("GPS",__FILE__,__LINE__, msg);
        free(dem_ptr->data);
        XXX_FREE_DDR(DDR_control_ptr);
        return (ERROR);        
    }

    /* verify the number of lines, number of samples, and data type read from
       the DEM HDF file agree with the DEM DDR */
    if ((dem_nl != dem_ptr->nl) || (dem_ns != dem_ptr->ns) ||
        (dem_datatype != I2))
    {
        sprintf(msg, "Error reading DEM data from %s - DEM file does not match DDR", DEM_name);
        xxx_LogStatus("GPS",__FILE__,__LINE__, msg);
        free(dem_ptr->data);
        XXX_FREE_DDR(DDR_control_ptr);
        return (ERROR);
    }
    
    /* free the DDR */
    XXX_FREE_DDR(DDR_control_ptr);
    
    return (SUCCESS);
}
