/******************************************************************************
NAME:		xxx_get_elev

PURPOSE:	Read the specified elevation image

RETURN VALUE: 
Type = void *
Value           Description
-----           -----------
pointer to dem image data
NULL     	Error reading elevation image

NOTES:		The elevation image is assumed to be in the same format as a 
		regular 1G image.
		This function uses the first band found in the DEM image.  It 
		allocates memory for the image data and returns a pointer to 
		the data.  It also passes back the size (nl, ns) and data type 
		of the data that was read.
		The caller is resposible for freeing this memory.

ALGORITHM REFERENCES:
******************************************************************************/
#include <stdio.h>
#include <unistd.h>
#include "xxx_Types.h"
#include "xxx_LogStatus.h"
#include "xxx_hdf_io.h"

void *xxx_get_elev (
	const char *dem_name,	/* I-name of the dem image file	*/
	int *dem_nl,		/* O-nl in dem image		*/
	int *dem_ns,		/* O-ns in dem image		*/
	DataType *dem_dtype)	/* O-data type of dem image	*/
{
HDFIO 	*dem_id;		/* HDF id for the dem image 	*/
void 	*dem_buf;		/* buffer of image data		*/
int	status;			/* function return status	*/
char	msg[STRLEN];		/* error message string		*/
int	nbands=0,		/* # of bands returned by open 	*/
	nl[NBANDS],		/* nl of all bands in dem image	*/
	ns[NBANDS],		/* ns of all bands in dem image	*/
	size;			/* size of data (1 pixel)	*/
DataType dtype[NBANDS];		/* data type of all bands in dem*/
BandNumber bandno[NBANDS];	/* band #'s in dem image	*/

/*
 * Open the dem image
 */
dem_id = xxx_open_1g_image(dem_name,READ,&nbands,bandno,dtype,nl,ns);
if (dem_id == NULL)
    {
    sprintf(msg,"Error accessing dem image %s",dem_name);
    xxx_LogStatus("XXXLIB",__FILE__,__LINE__,msg);
    return(NULL);
    }
*dem_nl = nl[0];
*dem_ns = ns[0];
*dem_dtype = dtype[0];
switch (dtype[0])
    {
    case BYTE:
	size = sizeof(uint8);
	break;
    case I2:
	size = sizeof(int16);
	break;
    case UI2:
	size = sizeof(uint16);
	break;
    case I4:
	size = sizeof(int32);
	break;
    case UI4:
	size = sizeof(uint32);
	break;
    case R4:
	size = sizeof(float32);
	break;
    case R8:
	size = sizeof(float64);
	break;
    default:
        {
        sprintf(msg,"Dem image %s contains invalid data type",dem_name);
        xxx_LogStatus("XXXLIB", __FILE__, __LINE__, msg);
        xxx_close_1g_image(dem_id);
        return(NULL);
        }
    }

/*
 * Allocate space for the image data
 */
dem_buf = malloc(*dem_nl * *dem_ns * size);
if (dem_buf == NULL)
    {
    xxx_LogStatus("XXXLIB", __FILE__, __LINE__,
    	"Error allocating memory for dem data");
    xxx_close_1g_image(dem_id);
    return(NULL);
    }

/*
 * Read the full dem into the buffer
 */
status = xxx_read_1g_image(dem_id, bandno[0]/*band*/, 0/*sl*/, 0/*ss*/,
        *dem_nl/*nl*/, *dem_ns/*ns*/, 0/*no subsampling*/, dem_buf);
if (status == ERROR)
    {
    free(dem_buf);
    sprintf(msg,"Error reading dem image %s",dem_name);
    xxx_LogStatus("XXXLIB",__FILE__,__LINE__,msg);
    xxx_close_1g_image(dem_id);
    return(NULL);
    }

/*
 * Close the dem
 */
status = xxx_close_1g_image(dem_id);
if (status == ERROR)
    {
    free(dem_buf);
    sprintf(msg,"Error closing image dem %s",dem_name);
    xxx_LogStatus("XXXLIB",__FILE__,__LINE__,msg);
    return(NULL);
    }

return(dem_buf);
}
