/******************************************************************************
NAME:		xxx_get_gcp

PURPOSE:	Get all records from the GCP data file and convert the 
		lat/long/height coordinate to Cartesian coordinates.

DEVELOPMENT HISTORY:
$Log: xxx_get_gcp.c,v $
Revision 1.1  2000/12/07 23:05:17  storey
Original version.


RETURN VALUE: 
Type = int
Value           Description
-----           -----------
SUCCESS     	Successfully got all GCP data 
ERROR     	Error occurred; no valid GCP data returned

NOTES:		This function allocates memory for all the gcp's read from the
		file.  The calling function is responsible for freeing this
		memory.

ALGORITHM REFERENCES:
*******************************************************************************
		        Property of the U.S. Government
                            USGS EROS Data Center
******************************************************************************/
#include <stdlib.h>
#include <string.h>
#include "xxx_utility.h"

int xxx_get_gcp (
    const char *gcp_data_filename,	/* I-GCP data file name		*/
    const CPF *cpf,			/* I-Cal parm file constants	*/
    GCPDATA **gcp_data,			/* O-GCP data records		*/
    int  *numgcp)			/* O-Number of GCPs read	*/
{
char	msg[STRLEN];		/* error message			*/
GCPDATA gcp_tmp;		/* temporary GCP data record		*/
FILE 	*gcp_fptr;		/* GCP data file pointer		*/
int	status;			/* function return status		*/
int 	curgcp;			/* current gcp structure array element	*/


/*
 * Initialize the number of GCPs read and the array of structures
 */
*numgcp = 0;
curgcp = 0;
*gcp_data = NULL;

/*
 * Open the file
 */
gcp_fptr = fopen(gcp_data_filename, "r");
if (gcp_fptr == NULL)
    {
    xxx_LogStatus("XXXLIB",__FILE__,__LINE__,"Error opening gcp");
    return(ERROR);
    }

/*
 * Read until an error occurs
 */
status = xxx_read_gcp(gcp_fptr, cpf, &gcp_tmp);
while (status != ERROR)
    {
    /*
      Check if outlier
    */
    if (gcp_tmp.accept_flg == TRUE)
	{
   	/*
   	 * Reallocate memory for the growing number of structures
    	*/
    	(*numgcp)++;
    	*gcp_data = realloc(*gcp_data, 
	*numgcp * sizeof(GCPDATA));
    	if (*gcp_data == NULL)
	    {
	    sprintf(msg,"Error allocating memory for GCP %ld",*numgcp);
    	    xxx_LogStatus("XXXLIB",__FILE__,__LINE__,msg);
    	    return(ERROR);
    	    }

   	/*
   	 * Copy the data that was just read into the array
    	*/
    	strcpy((*gcp_data)[curgcp].pt_id, gcp_tmp.pt_id);
    	(*gcp_data)[curgcp].ref_line = gcp_tmp.ref_line;
    	(*gcp_data)[curgcp].ref_samp = gcp_tmp.ref_samp;
    	(*gcp_data)[curgcp].lat = gcp_tmp.lat;
    	(*gcp_data)[curgcp].lon = gcp_tmp.lon;
    	(*gcp_data)[curgcp].elev = gcp_tmp.elev;
    	(*gcp_data)[curgcp].gcppos.x = gcp_tmp.gcppos.x;
    	(*gcp_data)[curgcp].gcppos.y = gcp_tmp.gcppos.y;
    	(*gcp_data)[curgcp].gcppos.z = gcp_tmp.gcppos.z;
    	(*gcp_data)[curgcp].pred_srch_line = gcp_tmp.pred_srch_line;
    	(*gcp_data)[curgcp].pred_srch_samp = gcp_tmp.pred_srch_samp;
    	(*gcp_data)[curgcp].fit_offset[0] = gcp_tmp.fit_offset[0];
    	(*gcp_data)[curgcp].fit_offset[1] = gcp_tmp.fit_offset[1];
    	(*gcp_data)[curgcp].accept_flg = gcp_tmp.accept_flg;
    	strcpy((*gcp_data)[curgcp].ref_name,gcp_tmp.ref_name);
    	(*gcp_data)[curgcp].ref_band = gcp_tmp.ref_band;
    	strcpy((*gcp_data)[curgcp].srch_name,gcp_tmp.srch_name);
    	(*gcp_data)[curgcp].srch_band = gcp_tmp.srch_band;

    	curgcp++;

  	 /*
    	* Get the next record
    	*/
	}
    status = xxx_read_gcp(gcp_fptr, cpf, &gcp_tmp);
    }

/*
 * Close the file
 */
status = fclose(gcp_fptr);
if (status != 0)
    {
    xxx_LogStatus("XXXLIB",__FILE__,__LINE__,"Error getting gcp");
    return(ERROR);
    }

return(SUCCESS);
}
