/******************************************************************************
NAME:		xxx_get_gcplib

PURPOSE:
Read the GCPLIB file.  Put the information of the Ground Control Points
in the GCPLIB structure.

RETURN VALUE:
type=int
Value		Description
-----		-----------
SUCCESS		Successfully retrieved the GCP information
EREROR		Failure to get the GCP information

NOTES
-----
It is assumed that if there are entries for the chip source they will be
entered in the first elements of the array.  No further elements of the
array will be checked after an empty string is found.  This subroutine is
expecting an empty string in the array element after the valid chip sources
are found.

******************************************************************************/
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <ctype.h>
#include "xxx_utility.h"

int xxx_get_gcplib(
    const char *gcplib_file_name,/* I: Name of the GCPLIB file */
    char *abs_or_rel,		/* I: Flag for absolute or relative source */
    int  min_percent_succ,	/* I: Flag for min percent acceptable */
    const int  *begin_date,	/* I: Flag for beginning date 
						[0] = month
						[1] = year */
    const int  *end_date,	/* I: Flag for ending date
						[0] = month
						[1] = year */
    char season[4][7],		/* I: Flag for season of chip */
    char chip_source[10][9],	/* I: Flag for source of chip */
    int  row_wrs,		/* I: wrs row */
    GCPLIB **gcp_lib,		/* O: Structure of chip information */
    int  *numgcp)		/* O: number of ground control points */
{
int  count;		/* counter */
FILE *ptr;		/* pointer to the gcplib file */
char buffer[GCPLIB_RECLEN];/* string read in from file */
double percent_succ;	/* percent of successful correlations for this point */
int  day;		/* Satellite day of the month */
int  month;		/* Satellite month */
int  year;		/* Satellite year */
int status;		/* status of return from function */
int  i;			/* loop control variable */
int  seasonflg = FALSE;	/* Flag indicating if seasons are specified */
int  sourceflg = FALSE;	/* Flag indicating if sources are specified */
int  src_found;		/* Temporary flag in source search */
int  numsource = 0;	/* number of chip source entered */
int  qtr[4];		/* Specific quarters */
int  use_point = TRUE;	/* Flag indicating if point should be used */
GCPLIB tmp_lib;		/* temporary gcp library info */

/* initialize the structure to NULL
  -------------------------------*/
*gcp_lib = NULL;

/* Open the GCP library file
  --------------------------*/
if ((ptr=fopen(gcplib_file_name,"r")) == NULL)
    {
    xxx_LogStatus("XXXLIB",__FILE__,__LINE__,"Error getting gcp library file");
    return(ERROR);
    }

/* Get season input.  The season is dependent on the row therefore a test
   of the row is included to determine southern or northern hemisphere.
  -----------------------------------------------------------------------*/
for (i=0;i<4;i++)
    qtr[i] = FALSE;

for (i=0;i<4;i++)
    if (strcmp(season[i], "") != 0)
    	{
	seasonflg = TRUE;
    	if (strcmp(xxx_strtoupper(season[i]),"SUMMER") == 0)
	    {
	    if ((row_wrs >= 61) && (row_wrs <= 184))
		qtr[0] = TRUE;
	    else
	    	qtr[2] = TRUE;
	    }
    	else if (strcmp(xxx_strtoupper(season[i]),"SPRING") == 0)
	    {
	    if ((row_wrs >= 61) && (row_wrs <= 184))
		qtr[3] = TRUE;
	    else
	    	qtr[1] = TRUE;
	    }
    	else if (strcmp(xxx_strtoupper(season[i]),"FALL") == 0)
	    {
	    if ((row_wrs >= 61) && (row_wrs <= 184))
		qtr[1] = TRUE;
	    else
	    	qtr[3] = TRUE;
	    }
    	else if (strcmp(xxx_strtoupper(season[i]),"WINTER") == 0)
	    {
	    if ((row_wrs >= 61) && (row_wrs <= 184))
		qtr[2] = TRUE;
	    else
	    	qtr[0] = TRUE;
	    }
	else
	    {
   	    xxx_LogStatus("XXXLIB",__FILE__,__LINE__,"Invalid season entered");
	    fclose(ptr);
	    return(ERROR);
	    }
    	}

/* Check the source parameter to see if a check is necessary and get the
  number of sources.
---------------------------------------------------------------------------*/
if (strcmp(chip_source[0], "") != 0)
    {
    sourceflg = TRUE;
    while ((strcmp(chip_source[numsource], "") != 0) && (numsource < 10))
	{
	numsource++;
	}
    }
/* Read the file until end of file or BEGIN flag to bypass header
-----------------------------------------------------------------*/
for(;;)
    {
    if (fgets(buffer, 300, ptr) == NULL) 
	{
   	xxx_LogStatus("XXXLIB",__FILE__,__LINE__, "Unexpected end of gcplib file");
	fclose(ptr);
	return(ERROR);
	}
    if (strncmp(buffer, "BEGIN", 5) == 0)
	break;
    }

/* Get the chip information from file
  ---------------------------------*/
count=0;
for(;;)
    {
    if (fscanf(
	      ptr,"%s%s%lf%lf%lf%lf%lf%lf%lf%lf%lf%lf%lf%s%s%s\n",
		tmp_lib.pt_id,tmp_lib.chip_name, &tmp_lib.ref_line,
		&tmp_lib.ref_samp, &tmp_lib.lat,&tmp_lib.lon,
		&tmp_lib.projy, &tmp_lib.projx, &tmp_lib.elev,
		&tmp_lib.pix_size[0], &tmp_lib.pix_size[1],
		&tmp_lib.ref_size[0], &tmp_lib.ref_size[1],tmp_lib.source,
		tmp_lib.date,tmp_lib.abs_rel_flag) != CHIPREC)
	break;
    if (fscanf(ptr,"%d%d%d\n",
		&tmp_lib.attempts,&tmp_lib.good_corr,
		&tmp_lib.good_prec) != CHIPOUT)
	break;

	/* check the date of the GCPLib  */
	sscanf(tmp_lib.date,"%ld-%ld-%ld",&month,&day,&year);
	status = xxx_check_year_month_day(year, month, day);
	if (status != SUCCESS)
		{
    	        xxx_LogStatus("GPSLIB",__FILE__,__LINE__,"Error in gcp lib date");
    	        return(ERROR);
		}

    /* Test minimum percent successes
    -------------------------------*/
    if ((min_percent_succ != 0) && (tmp_lib.attempts != 0))
	{
	percent_succ = (double)tmp_lib.good_corr / (double)tmp_lib.attempts * 100.0;
	if (percent_succ < min_percent_succ)
	    use_point = FALSE; 
	}
    /* Test beginning and ending year and season
      -----------------------------------------*/
    if ((begin_date[0] != 0) || (end_date[0] != 0) || (begin_date[1] != 0) || 
        (end_date[1] != 0)||(seasonflg))
	{
	if (month < begin_date[0])
	    use_point = FALSE; 
	else if ((month > end_date[0]) && (end_date[0] != 0))
	    use_point = FALSE; 
	else if (year < begin_date[1])
	    use_point = FALSE; 
	else if ((year > end_date[1]) && (end_date[1] != 0))
	    use_point = FALSE;

        /* Test season
         ------------*/
	else if (seasonflg)
	    {
	    if ((!qtr[0]) && (month <= 3))
	        use_point = FALSE;
	    else if ((!qtr[1]) && (month >= 4) && (month <= 6))
		use_point = FALSE;
	    else if ((!qtr[2]) && (month >= 7) && (month <= 9))
		use_point = FALSE;
	    else if ((!qtr[3]) && (month >= 10))
	        use_point = FALSE;
	    }
	}
    /* Test the source
      ---------------*/
    if (sourceflg)
	{
	src_found = FALSE;
        for(i=0;i<numsource;i++)
	    {
	    if (strcmp(xxx_strtoupper(chip_source[i]), xxx_strtoupper(tmp_lib.source))==0)
		{
		src_found = TRUE;
	        break;
		}
	    }
	if(!src_found)
	    use_point = FALSE;
	}

    /* Test the absolute or relative accuracy of the point
      ---------------------------------------------------*/
   if (strcmp(abs_or_rel, "") != 0)
	{
	if (strcmp(xxx_strtoupper(abs_or_rel), xxx_strtoupper(tmp_lib.abs_rel_flag)) != 0)
	    use_point = FALSE;
	}

    /* Allocate memory for the structure.  Reallocate until done reading
    -------------------------------------------------------------------*/

    *gcp_lib = realloc(*gcp_lib,(count+1)*sizeof(GCPLIB));
    if (*gcp_lib == NULL)
	{
	fclose(ptr);
	xxx_LogStatus("XXXLIB",__FILE__,__LINE__,"Error allocating memory");
	return(ERROR);
	}

    /* Copy to final library structure
    ---------------------------------*/

    memcpy((void *)&((*gcp_lib)[count]),(void *)&tmp_lib,sizeof(tmp_lib));

    /* Increment counter
    --------------------*/ 
    if (use_point == TRUE)
	count += 1;
    use_point = TRUE;
    }

*numgcp = count;

fclose(ptr);
return(SUCCESS);
}

/******************************************************************************
NAME:           str2upper

PURPOSE:        
str2upper converts all the lower case characters in a string to upper
case.

RETURN VALUE:
Type = char *
Value    Description
-----    -----------
pointer  Returns a pointer to the string so it can immediately be used as
         a parameter to another function (such as strcmp)

NOTES:
The original string is converted in place.

******************************************************************************/

char *str2upper (char * string_ptr)
{
    char *c_ptr = string_ptr;

    while (*c_ptr != '\0')
    {
        *c_ptr = toupper(*c_ptr);
        c_ptr++;
    }

    return (string_ptr);
}
