/******************************************************************************
NAME:           xxx_get_residual

PURPOSE:        Read the residual along and across scan components
		for each GCP from the residual file.

DEVELOPMENT HISTORY:
$Log: xxx_get_residual.c,v $
Revision 1.1  2000/12/07 23:04:45  storey
Original version.


RETURN VALUE:
Type = int
Value		Description
-----		-----------
SUCCESS		Success
ERROR		Failure

ALGORITHM REFERENCES:
*******************************************************************************
			Property of the U.S. Government
		              USGS EROS Data Center
******************************************************************************/
#include <stdio.h>
#include <string.h>
#include "xxx_utility.h"
#include "exx_gcp_residual.h"
#include "axx_hdf_io.h"

int xxx_get_residual(
        const char *resfile,    	/* I-name of precision residual file  */
        int  iteration_number,          /* I-which iteration from file to get */
        GCP_RESIDUAL **gcp_res,  	/* O-GCP residual info                */
        int  *num_res)                  /* O-number of residual structs read  */
{
FILE *res_fptr;		/* file pointer for residual file */
int  i;           	/* looping variable */
int  file_num;		/* iteration number found in the file */
int user_band;		/* User band from residual file */
BandNumber bandno;		/* Band number converted from user band */
int bandflg = FALSE;	/* Flag indicating that the band was found in the file*/
char keyword[GCP_RECLEN];/* key word when searching file */
char buf1[GCP_RECLEN];	/* buffer for the first word when getting the band */
char buf2[GCP_RECLEN];	/* buffer for the second word when getting the band */
char buffer[GCP_RECLEN];/* buffer for data */
char errmsg[STRLEN];	/* error message */
GCP_RESIDUAL tmp_res; 	/* temporary GCP residual info	*/
int  param_num;		/* The number of parameters per line to read */

/*
 * Open the file
 */
if ((res_fptr = fopen(resfile, "rt")) == NULL)
    {
    sprintf(errmsg,"Error opening residual file %s",resfile);
    xxx_LogStatus("XXXLIB",__FILE__,__LINE__,errmsg);
    return(ERROR);
    }

/*
 * Find the beginning of the specific group
 * Check if specific number or final iteration was specified
 */
for(;;)
    {
    if (fgets(buffer, GCP_RECLEN, res_fptr) == NULL)
	{
	*num_res = 0;
	fclose(res_fptr);
    	xxx_LogStatus("XXXLIB",__FILE__,__LINE__,"Residual data not found");
	return(ERROR);
	}
    /*
     Get the first word and see if it is Band
    */
    sscanf(buffer,"%s",keyword);
    if ((strcmp(keyword, "Band") == 0) && !bandflg)
	{
        sscanf(buffer,"%s%s%d",buf1,buf2,&user_band);
        /*
 	  Convert the band to enumerated type
	*/
	bandno = axx_parse_user_band(user_band);
	if (bandno == berror)
	    {
	    *num_res = 0;
	    fclose(res_fptr);
	    xxx_LogStatus("XXXLIB",__FILE__,__LINE__,
			"Invalid band number from the residual file");
	    return(ERROR);
	    }
	bandflg = TRUE;
	}
    else if (!strcmp(keyword, "Iteration") || !strcmp (keyword, "Final"))
        {
        sscanf(buffer,"%s %ld",keyword,&file_num);
        if (((strcmp(keyword,"Iteration") == 0) &&
             (file_num == iteration_number)) ||
    	     ((strcmp(keyword,"Final") == 0) && (iteration_number < 0)))
    	    {
    	    break;
    	    }
        }
    }
if (!bandflg)
    {
    *num_res = 0;
    fclose(res_fptr);
    xxx_LogStatus("XXXLIB",__FILE__,__LINE__,"Band number not found");
    return(ERROR);
    }

/*
 * Set up to start allocating memory
 */
*gcp_res = NULL;

/*
 * Loop over all GCPs to write out the residual information.  Break out of the
   for loop when no data is found.
 */
param_num = 13;
i=0;
for(;;)
    {
    /*
     if all parameters are not read then break out of loop
    */
    if (fscanf(res_fptr,"%s%lf%lf%lf%lf%lf%lf%lf%lf%lf%lf%lf%d\n", 
	    tmp_res.pt_id, &tmp_res.pred_line, &tmp_res.pred_samp, 
	    &tmp_res.seconds,
	    &tmp_res.lat, &tmp_res.lon, &tmp_res.height, &tmp_res.delta,
	    &tmp_res.vb, &tmp_res.va, &tmp_res.residual_y,
	    &tmp_res.residual_x, &tmp_res.outlier_flag) != param_num)
        {
        break;
        }
    else if (!strcmp (tmp_res.pt_id, "Iteration") ||
             !strcmp (tmp_res.pt_id, "Final"))
	{
	break;
	}
    else
	{
	*gcp_res= (GCP_RESIDUAL *)realloc(*gcp_res, (i+1)*sizeof(GCP_RESIDUAL));
	if (*gcp_res == NULL)
	    {
	    *num_res = 0;
	    fclose(res_fptr);
	    sprintf(errmsg,"Error allocating memory for %d residuals",i+1);
	    xxx_LogStatus("XXXLIB",__FILE__,__LINE__,errmsg);
	    return(ERROR);
	    }
	/*
	  assign the band number into the structure
	*/
	tmp_res.bandno = bandno;

	/* 
	  Copy the structure temporary structure to the actual structure
 	*/
	memcpy((void *)&((*gcp_res)[i]),(void *)&tmp_res,sizeof(tmp_res));
	}

    i++;
    }
*num_res = i;
if (fclose(res_fptr) != 0)
    {
    xxx_LogStatus("XXXLIB",__FILE__,__LINE__,"Error closing residual file");
    return(ERROR);
    }
return(SUCCESS);
}
