/******************************************************************************
NAME:           xxx_malloc_geom_grid

PURPOSE:
Allocates the dynamic memory for a geometric grid.  The geometric grid structure
must already exist and have the non-allocated members filled in so the amount
of memory can be determined.

RETURN VALUE:   
Type = int
Value    Description
-----    -----------
SUCCESS  Successful completion
ERROR    Operation failed

NOTES:
This routine will loop through the specified bands, allocating the dynamic 
memory for the grid.  If no bands were specified, then memory for all the bands 
in the grid will be allocated.  In this case, the bands array will be filled in
with the bands in the grid and nbands will be set to the correct value.  

ALGORITHM REFERENCES:
See xxx_geom_grid.h for a description of the geometric grid structure.

******************************************************************************/

#include <stdlib.h>          /* malloc/free prototypes */
#include <string.h>          /* memcpy prototype */
#include "xxx_Types.h"       /* ERROR/SUCCESS definitions */
#include "xxx_LogStatus.h"   /* xxx_LogStatus prototype */
#include "xxx_geom_grid.h"   /* xxx_GEOM_GRID_TYPE definition */


int xxx_malloc_geom_grid 
(
  int *bands,                /* I/O: Array of bands to free from the grid */
  int *nbands,               /* I/O: Number of bands in the bands array */
  xxx_GEOM_GRID_TYPE *grid   /* I/O: Grid structure allocate memory for */
)
{
    int band;                  /* band loop counter */
    int count;                 /* loop counter */
    int band_index;            /* index to current band */
    int status = SUCCESS;      /* status of this routine */

    /* If no bands are specified then malloc all the bands in the grid */
    if (*nbands == 0)
    {
        *nbands = grid->nbands;
        memcpy (bands, grid->bandlist, *nbands * sizeof(int));
    }

    /* Loop through the bands mallocing the correct reference bands */
    for (band = 0; band < *nbands; band++)
    {
        /* Get the reference value of the current band
        */
        band_index = bands[band];

        /* initialize all the pointers to the dynamic memory to NULL in
           case there is a failure so free can be safely called for all the
           pointers in this band */
        grid->gridbands[band_index].in_lines = NULL;
        grid->gridbands[band_index].in_samps = NULL;
        grid->gridbands[band_index].out_lines = NULL;
        grid->gridbands[band_index].out_samps = NULL;
        grid->gridbands[band_index].poly_lines = NULL;
        grid->gridbands[band_index].poly_samps = NULL;
        grid->gridbands[band_index].forward_coeffs = NULL;
        grid->gridbands[band_index].reverse_coeffs = NULL;

        grid->gridbands[band_index].in_lines =
        malloc (grid->gridbands[band_index].num_in_lines * sizeof (double));
        if (grid->gridbands[band_index].in_lines == NULL)
        {
          status = ERROR;
          break;
        }

        grid->gridbands[band_index].in_samps =
        malloc (grid->gridbands[band_index].num_in_samps * sizeof (double));
        if (grid->gridbands[band_index].in_samps == NULL)
        {
          status = ERROR;
          break;
        }

        grid->gridbands[band_index].out_lines =
        malloc (grid->gridbands[band_index].num_out_lines * sizeof (double));
        if (grid->gridbands[band_index].out_lines == NULL)
        {
          status = ERROR;
          break;
        }

        grid->gridbands[band_index].out_samps =
        malloc (grid->gridbands[band_index].num_out_samps * sizeof (double));
        if (grid->gridbands[band_index].out_samps == NULL)
        {
          status = ERROR;
          break;
        }

        grid->gridbands[band_index].poly_lines =
        malloc ((grid->gridbands[band_index].degree+1) *
        (grid->gridbands[band_index].degree+1) * sizeof (double));
        if (grid->gridbands[band_index].poly_lines == NULL)
        {
          status = ERROR;
          break;
        }

        grid->gridbands[band_index].poly_samps =
        malloc ((grid->gridbands[band_index].degree+1) *
        (grid->gridbands[band_index].degree+1) * sizeof (double));
        if (grid->gridbands[band_index].poly_samps == NULL)
        {
          status = ERROR;
          break;
        }

        if(grid->gridbands[band_index].forward_present)
        {
            grid->gridbands[band_index].forward_coeffs =
            malloc (grid->gridbands[band_index].grid_rows *
                    grid->gridbands[band_index].grid_cols *
                    sizeof(COEFFICIENTS));
            if (grid->gridbands[band_index].forward_coeffs == NULL)
            {
              status = ERROR;
              break;
            }
        }

        if (grid->gridbands[band_index].reverse_present)
        {
            grid->gridbands[band_index].reverse_coeffs =
            malloc (grid->gridbands[band_index].grid_rows *
                    grid->gridbands[band_index].grid_cols *
                    sizeof(COEFFICIENTS));
            if (grid->gridbands[band_index].reverse_coeffs == NULL)
            {
              status = ERROR;
              break;
            }
        }
    }

    /* check if an error occurred during the allocation */
    if (status != SUCCESS)
    {
        xxx_LogStatus ("GPSLIB", __FILE__, __LINE__, 
                       "Error allocating memory for grid bands");
        for (count = 0; count <= band; count++)
        { 
            /* Free previously malloced pointers and set the pointers
               to NULL. */
            band_index = bands[count];
            free (grid->gridbands[band_index].in_lines);
            free (grid->gridbands[band_index].in_samps);
            free (grid->gridbands[band_index].out_lines);
            free (grid->gridbands[band_index].out_samps);
            free (grid->gridbands[band_index].poly_lines);
            free (grid->gridbands[band_index].poly_samps);
            free (grid->gridbands[band_index].forward_coeffs);
            free (grid->gridbands[band_index].reverse_coeffs);
            grid->gridbands[band_index].in_lines = NULL;
            grid->gridbands[band_index].in_samps = NULL;
            grid->gridbands[band_index].out_lines = NULL;
            grid->gridbands[band_index].out_samps = NULL;
            grid->gridbands[band_index].poly_lines = NULL;
            grid->gridbands[band_index].poly_samps = NULL;
            grid->gridbands[band_index].forward_coeffs = NULL;
            grid->gridbands[band_index].reverse_coeffs = NULL;
        }
    }

    return (status);
}
