/******************************************************************************
NAME:           xxx_output_header

PURPOSE:
Outputs the standard IAS header for the report file.

DEVELOPMENT HISTORY:
$Header: /llite/cvseo1/eo1/src/global/xxxLib/xxxIntIO/xxx_output_header.c,v 1.2 2000/12/22 13:43:14 storey Exp $
$Log: xxx_output_header.c,v $
Revision 1.2  2000/12/22 13:43:14  storey
Fixed path/row logic for null reference L0R input and changed header title
to include EO-1 ALI instead of Landsat 7.

Revision 1.1  2000/12/07 23:03:54  storey
Original version.

AUTHOR        Change-ID    DATE     Description of Change
------        ---------  --------   ---------------------
Gail Schmidt     1.0     12/04/97   Original development.
Gail Schmidt     1.1     12/09/97   Removed the srch_L0R and conf_level parms.
Gail Schmidt     1.2     12/16/97   Path and row are now read from the DDR.

RETURN VALUE:   Type = int
Value    Description
-----    -----------
SUCCESS  Successful completion
ERROR    Operation failed

NOTES:
This routine outputs the date, time, application name, workorder ID, WRS path,
WRS row, software version, and L0R reference image name to the specified file.

The IAS_REL environment variable is used to determine the current software
version.  The default version is "", if IAS_REL is not defined.

ALGORITHM REFERENCES:

******************************************************************************
                        Property of the U.S. Government
                            USGS EROS Data Center
******************************************************************************/

#include <sys/time.h>
#include <time.h>
#include <stdlib.h>
#include <stdio.h>
#include "xxx_DDR.h"
#include "xxx_Types.h"
#include "xxx_LogStatus.h"
#include "xxx_geometric.h"
#include "xxx_utility.h"

int xxx_output_header (
  FILE *fileptr,           /* I: Pointer to the output file            */
  const char *app_name,    /* I: String for the application name       */
  const char *ref_L0R)     /* I: L0R HDF reference filename            */
{
  char datestr[STRLEN],    /* String for the date           */
       timestr[STRLEN],    /* String for the time           */
       pathstr[4],         /* String for the path (1-233)   */
       rowstr[4],          /* String for the row (1-248)    */
       *sw_version,        /* Software version              */
       ddr_filename[STRLEN],/* DDR filename                 */
    /* lpgslite: fixed mbs 1/11/1999 - using an un-alloced pointer is never a good idea */
    woid[STRLEN+1],              /* String for the work order ID  */
    *woid_ptr,
       ErrMsg[STRLEN];     /* Error message                 */
  int path,                /* WRS path                      */
      row;                 /* WRS row                       */
  int ddrnameflg = FALSE;  /* Flag indicating if ddr name has been specified */
  int status;			   /* Status of return from function */
  struct tm *timeptr;      /* Struct for time variable      */
  time_t t;                /* Struct for time variables     */
  xxx_DDRControl_TYPE *DDRcontrol;
                           /* DDR control structure from xxx_ReadDDR.
                              Space for this is malloced in xxx_ReadDDR*/

  /* Check to see if the image name is filled in,  if not, do not look
     for the path and row
   */
  if (strcmp(ref_L0R,"") != 0)
  {
    ddrnameflg = TRUE;
    /* Read in the full DDR HDF structure.
     */
    xxx_create_DDR_name (ddr_filename, ref_L0R);
    DDRcontrol = xxx_ReadDDR (ddr_filename, ErrMsg);
    if (DDRcontrol == NULL)
    {
      xxx_LogStatus ("XXXLIB", __FILE__, __LINE__, ErrMsg);
      return (ERROR);
    }

    /* Use the first band in the DDR to access the path/row.
     */
    path = DDRcontrol->p_DDR[0].WRSPath;
    row = (int) DDRcontrol->p_DDR[0].WRSRows;  /* row is a float */
  }

  /* Get the current date and put it in the correct structured format.
   * Store date in the following manner:  Wed. Dec. 10, 1997
   */
  t = time (NULL);
  timeptr = localtime (&t);
  if (!strftime (datestr, STRLEN, "%a. %b. %e, %Y", timeptr))
  {
    xxx_LogStatus ("XXXLIB", __FILE__, __LINE__, "Error extracting the date.");
    return (ERROR);
  }

  /* Get the current time and put it in the correct structured format.
   * Store time in the following manner:  17:24
   */
  if (!strftime (timestr, STRLEN, "%R", timeptr))
  {
    xxx_LogStatus ("XXXLIB", __FILE__, __LINE__, "Error extracting the time.");
    return (ERROR);
  }

  /* Output the date, time, and application name.
   */
  if (fprintf (fileptr,
    "%s             EO-1 ALI                  Time: %s\n"
    "                         EROS Data Center SVT\n"
    "                             %s Report\n"
    "----------------------------------------------------------------------"
    "------\n", datestr, timestr, app_name) < 0)
  {
    xxx_LogStatus ("XXXLIB", __FILE__, __LINE__,
      "Error writing to the output file.");
    return (ERROR);
  }

  /* Get the software version environment variable.
   */
  sw_version = getenv ("IAS_REL");
  if (sw_version == NULL)
    sw_version = "";

  /* Get the work-order ID environment variable.
   */
  /* lpgslite: null string crash fix */
  woid_ptr = getenv ("PCS_WOID");
  if (woid_ptr == NULL)
    strcpy(woid,"");
  else
    strncpy(woid,woid_ptr,STRLEN);  

  /* Convert the path and row to strings with leading zeros if they do
   * not contain 3 characters.  If no path/row, path and row contain 3 blanks.
   */
  if (ddrnameflg == FALSE)
	{
    strcpy(pathstr, "   ");
	}
  else if (path < 10)
  	{
    status = sprintf (pathstr, "00%d", path);
	if (status < 1)
	  	{
		xxx_LogStatus ("XXXLIB", __FILE__, __LINE__, "Error in sprintf");
		return (ERROR);
		}
	}
  else if (path < 100)
  	{
    status = sprintf (pathstr, "0%d", path);
	if (status < 1)
	  	{
		xxx_LogStatus ("XXXLIB", __FILE__, __LINE__, "Error in sprintf");
		return (ERROR);
		}
	}
  else
  	{
    status = sprintf (pathstr, "%d", path);
	if (status < 1)
	  	{
		xxx_LogStatus ("XXXLIB", __FILE__, __LINE__, "Error in sprintf");
		return (ERROR);
		}
	}

  if (ddrnameflg == FALSE)
  	{
    strcpy(rowstr, "   ");
	}
  else if (row < 10)
  	{
    status = sprintf (rowstr, "00%d", row);
	if (status < 1)
	  	{
		xxx_LogStatus ("XXXLIB", __FILE__, __LINE__, "Error in sprintf");
		return (ERROR);
		}
	}
  else if (row < 100)
  	{
    status = sprintf (rowstr, "0%d", row);
	if (status < 1)
	  	{
		xxx_LogStatus ("XXXLIB", __FILE__, __LINE__, "Error in sprintf");
		return (ERROR);
		}
	}
  else
  	{
    status = sprintf (rowstr, "%d", row);
	if (status < 1)
	  	{
		xxx_LogStatus ("XXXLIB", __FILE__, __LINE__, "Error in sprintf");
		return (ERROR);
		}
	}

  /* Output the work order ID, path, row, and software version.
   */
  if (!strcmp (woid, ""))     /* use 7 spaces for better appearance */
    strcpy (woid, "       ");
  if (fprintf (fileptr,
    "WOID: %s            Path/Row: %s / %s          S/W Version: %s\n\n",
    woid, pathstr, rowstr, sw_version) < 0)
  {
    xxx_LogStatus ("XXXLIB", __FILE__, __LINE__,
      "Error writing to the output file.");
    return (ERROR);
  }

  /* Output the Reference Image.
   */
  if (fprintf (fileptr, "L0R Reference Image: %s\n", ref_L0R) < 0)
  {
    xxx_LogStatus ("XXXLIB", __FILE__, __LINE__,
      "Error writing to the output file.");
    return (ERROR);
  }
  fprintf (fileptr, "\n\n");

  return (SUCCESS);
}
