/******************************************************************************
NAME:           xxx_parse_command_line

PURPOSE:
This routine is a general purpose command line optional parameter parser.

RETURN VALUE:
Type = int
Value           Description
-----           -----------
ERROR           Error parsing parameters
SUCCESS         Successfully parsed parameters

NOTES:
- If there are required parameters on the command line, they must appear
  before the optional parameters and the argc and argv parameters passed in
  must be adjusted to skip the required parameters.
- Currently, this routine only recognizes integer and flag parameters.
- Set the default value of the parameters before calling this routine.
- If some parameter names are substrings of other parameter names, place
  the longer parameter names in the list first.

ALGORITHM REFERENCES:
none

******************************************************************************/

#include <string.h>        /* string functions */
#include "xxx_Types.h"     /* SUCCESS/ERROR definitions */
#include "xxx_utility.h"   /* prototype and typedefs for this module */

int xxx_parse_command_line
(
    int argc,                      /* I: number of parameters passed in */
    char *argv[],                  /* I: array of parameters passed in */
    int params,                    /* I: number of entries in param_list */
    xxx_CL_PARAMETER_DEF_TYPE *param_list /* I/O: list of parameters to parse */
)
{
    int current_param;    /* current parameter index */
    int len;              /* string length */
    int count;            /* loop counter */
    int temp;             /* temporary variable */
    int status = SUCCESS; /* status to return from this routine */
    int matched;          /* flag to indicate a match was found */
    
    /* if there are parameters search for matching entries in the 
       param_list */
    if (argc > 0)
    {
        /* start at the first command line parameter */
        current_param = 1;
        
        /* search all the command line parameters for a match in the parameter
           list or until an error is found */
        while ((current_param < argc) && (status == SUCCESS))
        {
            matched = 0;
           
            /* loop through the parameter list passed in */
            for (count = 0; (count < params) && (!matched); count++)
            {
                /* check for a match in parameters for the length of the
                   name in the list in case a number is attached to it */
                len = strlen(param_list[count].name);
                if(strncmp(param_list[count].name,argv[current_param],len)==0)
                {
                    /* parameter matches, handle based on type */
                    if (param_list[count].type == xxx_CL_FLAG)
                    {
                        /* make sure the parameter matches fully since it is
                           a flag parameter.  But don't flag an error if it
                           doesn't since it may match something else. */
                        if (strcmp(param_list[count].name,
                                   argv[current_param])==0)
                        {
                            /* flag parameter matched exactly, set the flag */
                            *param_list[count].param = 1;
                            matched = 1;
                        }
                    }
                    else if (param_list[count].type == xxx_CL_INTEGER)
                    {
                        /* integer parameter */
                        if (strlen(argv[current_param]) > len)
                        {
                            /* parameter and data are joined with no space.  
                               parse the data to an integer */
                            if (sscanf(&argv[current_param][len],"%d",&temp)==1)
                            {
                                *param_list[count].param = temp;
                                matched = 1;
                            }
                            else
                            {
                                status = ERROR;
                                matched = 1;
                            }
                        }
                        else
                        {
                            /* space between the parameter name and value so
                               go to the next parameter */
                            current_param++;
                            if (current_param < argc)
                            {
                                /* parse the parameter value */
                                if (sscanf(argv[current_param],"%d",&temp) == 1)
                                {
                                    *param_list[count].param = temp;
                                    matched = 1;
                                }
                                else
                                {
                                    status = ERROR;
                                    matched = 1;
                                }
                            }
                            else
                            {
                                /* there is no next parameter, so there is an
                                   error */
                                status = ERROR;
                                matched = 1;
                            }
                        }
                    }
                    else
                    {
                        /* error in the parameter type */
                        status = ERROR;
                        matched = 1;
                    }
                }

            }

            /* if no match was found for the command line parameter, it
               is an error */
            if (!matched)
                status = ERROR;

            /* go to the next parameter */
            current_param++;
        }
    }

    return (status);

}

