/******************************************************************************
NAME:           xxx_put_solution

PURPOSE:        Write out the orbit and attitude correction parameters
		estimates to the solution file.

DEVELOPMENT HISTORY:
$Log: xxx_put_solution.c,v $
Revision 1.2  2001/05/21 17:45:53  storey
Added logic to include the reference roll, pitch, and yaw to the precision
solution file output. Used in ALI alignment calibration enhancement.

Revision 1.1  2000/12/07 23:03:28  storey
Original version.


VERSION	DATE	AUTHER		MODIFICATIONS
-------	----	------		-------------
 1.0	05/96	D. Kuang	Original Development

RETURN VALUE:
Type = int
Value           Description
-----           -----------
SUCCESS		Successfully written to the solution file
ERROR     	Failure to write to file

ALGORITHM REFERENCES:


*******************************************************************************
			Property of the U.S. Government
			      USGS EROS Data Center
******************************************************************************/
#include <math.h>
#include "xxx_utility.h"

int xxx_put_solution(
  const char *ref_L0R,     /* I: L0R HDF reference filename            */
  const char solution_file_name[STRLEN],/* I: Name of the solution file */
  const GCP_SOLUTION *gcp_sol)	 	/* I: pointer to correction information
							structure */
{
  int i, j;          	/* looping variables */
  double s1, s2; 	/* temp value for sigma	*/
  double s3, s4;	/* temp value for sigma	*/
  double s5, s6; 	/* temp value for sigma	*/
  FILE *sol_fptr;	/* pointer to the sfile (solution file) */

/*
** Open file
*/
if((sol_fptr=fopen(solution_file_name, "w")) == NULL)
    {
    xxx_LogStatus("XXXLIB",__FILE__,__LINE__,"Error opening residual file");
    return(ERROR);
    }

/*
** Write the standard output header
*/
if (xxx_output_header (sol_fptr, "SOLUTION", ref_L0R) != SUCCESS)
    {
    xxx_LogStatus("XXXLIB", __FILE__, __LINE__,
                  "Error writing the standard header for this report.");
    return(ERROR);
    }

/*
** Write reference time to the solution file
*/
fprintf(sol_fptr, "Reference Time -->\n");
fprintf(sol_fptr,"  Year: %6.0f\n  Day: %6.0f\n  Second: %12.6e\n\n",
  gcp_sol->p_year,gcp_sol->p_day, gcp_sol->p_second);

/*
** Write out the attitude correction parameters for the reference time
*/
fprintf(sol_fptr,
  "Attitude Correction Parameters -->\n");
fprintf(sol_fptr,
  "  Angles (in microradians):\n"
  "    Roll  - %10.3f\n    Pitch - %10.3f\n    Yaw   - %10.3f\n"
  "  Rates (in radians/second):\n"
  "    Roll  - %10.3f\n    Pitch - %10.3f\n    Yaw   - %10.3f\n",
  (gcp_sol->p_rpyangle).x,	/* roll angle, in mico radian	*/
  (gcp_sol->p_rpyangle).y,	/* pitch angle, in micro radian	*/
  (gcp_sol->p_rpyangle).z,	/* yaw angle, in micro radian 	*/
  (gcp_sol->p_rpyrate).x,	/* roll rate, in m radian/sec	*/
  (gcp_sol->p_rpyrate).y,	/* pitch rate, in m radian/sec 	*/
  (gcp_sol->p_rpyrate).z);	/* yaw rate, in m radian/sec	*/

s1 = (gcp_sol->p_cov_along)[0][0];
s2 = (gcp_sol->p_cov_across)[1][1];
s3 = (gcp_sol->p_cov_across)[2][2];
s4 = (gcp_sol->p_cov_along)[3][3];
s5 = (gcp_sol->p_cov_across)[4][4];
s6 = (gcp_sol->p_cov_across)[5][5];
fprintf(sol_fptr,
  "  Angle Sigmas:\n"
  "    Roll  - %10.3f\n    Pitch - %10.3f\n    Yaw   - %10.3f\n"
  "  Rate Sigmas:\n"
  "    Roll  - %10.3f\n    Pitch - %10.3f\n    Yaw   - %10.3f\n\n",
  sqrt(s1),sqrt(s2),sqrt(s3),	/* sigma of roll, pitch, yaw	*/
  sqrt(s4),sqrt(s5),sqrt(s6));	/* sigma for attitude rate	*/

/*
** Write out the orbit correction parameters for the reference time
*/
fprintf(sol_fptr,
  "Orbit Correction Parameters -->\n");
fprintf(sol_fptr,
  "  Position (in meters):\n"
  "    x - %10.3f\n    y - %10.3f\n    z - %10.3f\n"
  "  Velocity (in meters/second):\n"
  "    x - %10.3f\n    y - %10.3f\n    z - %10.3f\n",
  (gcp_sol->p_satpos).x,  /* orbit systme x position, in meter	*/
  (gcp_sol->p_satpos).y,  /* orbit system y position, in meters	*/
  (gcp_sol->p_satpos).z,  /* orbit system z position, in meters	*/
  (gcp_sol->p_satvel).x,  /* orbit system x velocity, in m/sec	*/
  (gcp_sol->p_satvel).y,  /* orbit system y velocity, in m/sec	*/
  (gcp_sol->p_satvel).z); /* orbit system z velocity, in m/sec	*/

s1 = (gcp_sol->p_cov_across)[0][0];
s2 = (gcp_sol->p_cov_along)[1][1];
s3 = (gcp_sol->p_cov_along)[2][2];
s4 = (gcp_sol->p_cov_across)[3][3];
s5 = (gcp_sol->p_cov_along)[4][4];
s6 = (gcp_sol->p_cov_along)[5][5];
fprintf(sol_fptr,
  "  Position Sigmas:\n"
  "    dx - %10.3f\n    dy - %10.3f\n    dz - %10.3f\n"
  "  Velocity Sigmas:\n"
  "    dx - %10.3f\n    dy - %10.3f\n    dz - %10.3f\n\n",
  sqrt(s1),sqrt(s2),sqrt(s3),	/* sigma of dx, dy and dz   	*/
  sqrt(s4),sqrt(s5),sqrt(s6));	/* sigma for dx, dy and dz rate	*/
 
/*
** Write out the covariance matrix for along-scan parameters
*/
fprintf(sol_fptr, "Covariance Matrix for Along-Scan Parameters -->\n");
for(i=0; i<6; i++)
    {
    for(j=0; j<6; j++)
	fprintf(sol_fptr, "%12.4e", (gcp_sol->p_cov_along)[i][j]);
    fprintf(sol_fptr, "\n");
    }
fprintf(sol_fptr, "\n");

/*
** Write out the covariance matrix for cross-scan parameters
*/
fprintf(sol_fptr, "Covariance Matrix for Across-Scan Parameters -->\n");
for(i=0; i<6; i++)
    {
    for(j=0; j<6; j++)
	fprintf(sol_fptr, "%12.4e", (gcp_sol->p_cov_across)[i][j]);
    fprintf(sol_fptr, "\n");
    }
fprintf(sol_fptr, "\n");

/*
** Write out the roll, pitch, and yaw for the reference time
*/
fprintf(sol_fptr,
  "Reference Attitude Parameters -->\n");
fprintf(sol_fptr,
  "  Angles (in microradians):\n"
  "    Roll  - %10.3f\n    Pitch - %10.3f\n    Yaw   - %10.3f\n",
  (gcp_sol->p_refroll),		/* roll angle, in mico radian	*/
  (gcp_sol->p_refpitch),	/* pitch angle, in micro radian	*/
  (gcp_sol->p_refyaw));		/* yaw angle, in micro radian 	*/
fprintf(sol_fptr, "\n");

/*
** Close the file
*/
if (fclose(sol_fptr) != 0)
    {
    xxx_LogStatus("XXXLIB",__FILE__,__LINE__,"Error closing solution file");
    return(ERROR);
    }

return(SUCCESS);
}
