/******************************************************************************
NAME:		xxx_read_gcp

PURPOSE:	Read one record from the GCP data file

DEVELOPMENT HISTORY:
$Log: xxx_read_gcp.c,v $
Revision 1.2  2001/07/09 14:04:34  choate
Removed BandNumber check

Revision 1.1  2000/12/07 23:05:36  storey
Original version.


RETURN VALUE: 
Type = int
Value           Description
-----           -----------
SUCCESS     	Successfully got GCP data record
ERROR     	Error reading or at end of file

NOTES:		The file pointer is left at the end of the current record
		so sequential calls of xxx_read_gcp will sequentially read all
		the records in the file.

		The GCP data file is assumed to be an ASCII file containing
		three lines per GCP data record, with lines separated by 
		newlines.  

		***Each line must be not more than STRLEN characters long.

		The first line contains:
	pt_id ref_line ref_samp lat long elev pred_srch_line pred_srch_samp  $
	delta_y(line) delta_x(sample) accept/reject_flag
		The second line contains the reference image name
		The third line contains the search image name

NOTES:
When the end of the file is reached, the status returned is ERROR. This
really isn't an error.  No message is sent to LogStatus.  Possibly this
should be redesigned to be more logical.

ALGORITHM REFERENCES:
*******************************************************************************
		        Property of the U.S. Government
                            USGS EROS Data Center
******************************************************************************/
#include "xxx_Types.h"
#include "xxx_LogStatus.h"
#include "xxx_utility.h"
#include "exx_geometric.h"
#include "axx_hdf_io.h"

int xxx_read_gcp (
	FILE *gcp_fptr,			/* I-GCP data file pointer	*/
	const CPF *cpf,			/* I-cal parm file constants	*/
	GCPDATA *gcp_data)		/* O-One GCP data record	*/
{
char 	dataline[STRLEN];		/* line from GCP data file	*/
int	status;				/* function return status	*/
double	lon_rad;			/* longitude in radians		*/
double	lat_rad;			/* latitude in radians		*/
int 	srch_tmp;			/* temporary search band	*/

/*
 * Get the first line's information 
 */
if (fgets(dataline,STRLEN,gcp_fptr) == NULL)
    {
    /*
      Finished reading the file,  An error is returned to indicate the end
      of the file has been reached and no more records could be read.  No
      message is displayed because this condition is not truly an error
    */
    return(ERROR);
    }

status = sscanf(dataline,
    "%s %lf %lf %lf %lf %lf %lf %lf %lf %lf %ld %ld %ld",
     gcp_data->pt_id,		/* point id		*/
    &gcp_data->ref_line,	/* reference line 	*/
    &gcp_data->ref_samp,	/* reference sample	*/
    &gcp_data->lat,		/* true latitude of pt	*/
    &gcp_data->lon,		/* true longitude of pt	*/
    &gcp_data->elev,		/* elevation of pt	*/
    &gcp_data->pred_srch_line,	/* predicted search line*/
    &gcp_data->pred_srch_samp,	/* predicted search samp*/
    &gcp_data->fit_offset[0],	/* delta y (line)	*/
    &gcp_data->fit_offset[1],	/* delta x (sample)	*/
    &gcp_data->accept_flg,	/* accept(0)/reject(1) flag */
    &gcp_data->ref_band,	/* band number in ref img */
    &srch_tmp);			/* band number in srch img */
/*
  Get the enumerated band number from the user band number for the search
  band.  The file contains the user band number for easy reading.  This
  structure contains the enumberated band number for processing.
*/
gcp_data->srch_band = srch_tmp;

if (status != 13) /* expected to get 13 values */
    {
    xxx_LogStatus("XXXLIB",__FILE__,__LINE__,"Error reading GCP data");
    return(ERROR);
    }

/*
 * Get the reference image name from the 2nd line
 */
if (fgets(dataline,STRLEN,gcp_fptr) == NULL)
    {
    xxx_LogStatus("XXXLIB",__FILE__,__LINE__,"Error reading GCP data");
    return(ERROR);
    }
status = sscanf(dataline,"%s",gcp_data->ref_name); /* expected to get 1 value */
if (status != 1)
    {
    xxx_LogStatus("XXXLIB",__FILE__,__LINE__,"Error reading GCP data");
    return(ERROR);
    }

/*
 * Get the search image name from the 3rd line
 */
if (fgets(dataline,STRLEN,gcp_fptr) == NULL)
    {
    xxx_LogStatus("XXXLIB",__FILE__,__LINE__,"Error reading GCP data");
    return(ERROR);
    }
status = sscanf(dataline,"%s",gcp_data->srch_name); /* expected to get 1 value*/
if (status != 1)
    {
    xxx_LogStatus("XXXLIB",__FILE__,__LINE__,"Error reading GCP data");
    return(ERROR);
    }

/*
 * Calculate the Cartesian coordinate for the point
 */
lon_rad = gcp_data->lon * D2R;
lat_rad = gcp_data->lat * D2R;
exx_geod2cart(lat_rad,lon_rad,gcp_data->elev,
    cpf->Earth_Constants.Semi_Major_Axis,cpf->Earth_Constants.Ellipticity,
    &gcp_data->gcppos);

return(SUCCESS);
}
