/*******************************************************************************
NAME:	xxx_read_odl_file

PURPOSE:
General purpose routine to read a list of items from an ODL file.

RETURN VALUE:
Type=int
Value           Description
-----           -----------
SUCCESS		All required items successfully read from the ODL file
ERROR	        error locating or reading ODL file

*******************************************************************************/

#include <stdio.h>          /* sprintf prototype */
#include "xxx_LogStatus.h"  /* xxx_LogStatus prototype */
#include "xxx_const.h"      /* STRLEN definition */
#include "xxx_utility.h"    /* xxx_read_odl_file prototype and typedefs */

int xxx_read_odl_file
(
    const char *odl_file_name,   /* INPUT : ODL file name to read */
    int find_cpf,                /* INPUT : set this flag to non-zero to indicate
                                    the file name is a 0R filename and the CPF
                                    should be read from it */
    xxx_READ_ODL_LIST_TYPE *odl_list_ptr, 
                                 /* I/O : pointer to list of items to read
                                    from the ODL file */
    int list_length              /* INPUT : number of items in the list */
)
{
    int     status;	       /* function return status */
    char    msg[STRLEN];       /* error message string */
    OBJDESC *object;	       /* parsed ODL object */
    int     list_index;        /* current list index */
    xxx_READ_ODL_LIST_TYPE *list_ptr;/* pointer to current item in the list */
    int     count;             /* count of items read from ODL field */

    /* Open the ODL file, searching for the CPF filename in an HDF file
       if directed to by the read_cpf flag.  The odl_file_name is cast
       to a (char *) since the xxx_OpenODL does not properly declare the 
       filename as a const. */
    if (find_cpf)
        object = xxx_OpenODL((char *)odl_file_name, xxx_CPFMeta, msg);
    else
        object = xxx_OpenODL((char *)odl_file_name, xxx_NoEDCMeta, msg);
    if (object == NULL)
    {
        xxx_LogStatus("XXXLIB",__FILE__,__LINE__,msg);
        return(ERROR);
    }

    /* loop through the list of parameters to read */
    for (list_index = 0; list_index < list_length; list_index++)
    {
        /* get a pointer to the current item */
        list_ptr = &odl_list_ptr[list_index];

        /* read the current parameter from the ODL file */
        status =  xxx_GetODLField(list_ptr->param_ptr, list_ptr->param_size,
                                  list_ptr->value_type, object, 
                                  list_ptr->group_name, list_ptr->param_name,
                                  msg, &count);
        if ((status != SUCCESS) || (count < list_ptr->min_count))
        {
            /* an error occurred reading the ODL file or not enough items
               were read.  If it was a required parameter, generate
               an error message. */
            if (list_ptr->required == xxx_ODL_PARAM_REQUIRED)
            {
                /* make sure it was really an error so msg is valid */
                if (status != SUCCESS)
                {
                    /* error reading required item, log an error */
                    xxx_LogStatus("XXXLIB",__FILE__,__LINE__,msg);
                }

                /* the ODL supplied error messages are next to useless for
                   debugging the ODL file so provide some extra information */
                sprintf(msg,
                     "Error reading ODL parameter \"%s\" from group \"%s\"\n",
                     list_ptr->param_name, list_ptr->group_name);
                xxx_LogStatus("XXXLIB",__FILE__,__LINE__,msg);
                return (ERROR);
            }
        }
    }

    /* Close the file */
    status = xxx_CloseODL(object, msg);
    if (status != SUCCESS)
    {
        xxx_LogStatus("XXXLIB",__FILE__,__LINE__,msg);
        return(ERROR);
    }

    return(SUCCESS);
}
