/******************************************************************************
NAME:		exx_update_gcplib

PURPOSE:
Updates the GCP library file with either correlation results or precision 
results.  For correlation results, the number of attempts and the number of
successful correlates are incremented for the appropriate GCPs.  For
precision updates, the number of successful uses in the precision process is
incremented.

RETURN VALUE:
type=int
Value		Description
-----		-----------
SUCCESS		Success
ERROR		Failure

NOTES:
To update the correlation statistics, pass in the array of GCPDATA records and 
set gcp_res to NULL.  To update the precision statistic, pass in the array of 
GCP_RESIDUAL records and set gcpdata to NULL.  If both are specified,
both sets of statistics will be updated but the number of GCP records must be 
the same for each.
This function writes a new GCP library file and then replaces the old one.
******************************************************************************/
#include <stdio.h>	 /* for rename() */
#include <sys/types.h>	 /* for getpid() */
#include <unistd.h>	 /* for getpid() and unlink() */
#include "exx_gcplib.h"  /* gcp library data structure */
#include "exx_gcpdata.h" /* gcp data structure */
#include "exx_gcp_residual.h" /* gcp residual data structure */
#include "xxx_utility.h" /* function prototype */

int xxx_update_gcplib(
    const GCPDATA *gcpdata,	  /* I: ground control point data structure */
    const GCP_RESIDUAL *gcp_res,  /* I: GCP precision residual data structure*/
    const char *gcplib_file_name, /* I: Name of the GCPLIB file		    */
    int  numgcp)		  /* I: number of GCP records		    */
{
int  data_count;	/* counter of number of data points updated */
int  res_count;		/* counter of number of residual points updated */
int  curpt;		/* gcpdata looping variable */
int  status;		/* status of return from function */
char msg[STRLEN];	/* message for errors */
char buffer[GCPLIB_RECLEN];/* string read in from file */
char new_gcplibfile[STRLEN]; /* name of new GCP library file	*/
GCPLIB gcp_lib;		/* gcp library structure */
FILE *iptr;		/* pointer to the old gcplib file */
FILE *optr;		/* pointer to the new gcplib file */
int	 year, month,day;	/* the year month and day of the GCPLIB point */

/* See if we have to update anything
  ----------------------------------*/
if ((gcpdata == NULL) && (gcp_res == NULL)) /* none */
    return(SUCCESS);

/* Open the old GCP lib file 
  --------------------------*/
if ((iptr = fopen(gcplib_file_name,"r")) == NULL)
    {
    xxx_LogStatus("XXXLIB",__FILE__,__LINE__,"Error opening gcp library file");
    return(ERROR);
    }

/* Set up the (temporary) new lib file
  ------------------------------------*/
status = sprintf(new_gcplibfile, "%s_%ld", gcplib_file_name, getpid());
if (status < 1)
  	{
	xxx_LogStatus ("XXXLIB", __FILE__, __LINE__, "Error in sprintf");
	return (ERROR);
	}
if ((optr = fopen(new_gcplibfile,"w")) == NULL)
    {
    xxx_LogStatus("XXXLIB",__FILE__,__LINE__, "Error opening temporary gcp library file");
    return(ERROR);
    }

/* Read the file until BEGIN flag to bypass header or end of file
   and copy the header info into the new file
  -------------------------------------------------------------*/
while(1)
    {
    if (fgets(buffer, GCPLIB_RECLEN, iptr) == NULL)
        {
	xxx_LogStatus("XXXLIB",__FILE__,__LINE__, "No valid data found in GCP library file");
	fclose(iptr);
	fclose(optr);
	unlink(new_gcplibfile);
        return(ERROR);
        }
    /* write the buffer back out to the new file */
    if (fwrite(buffer, strlen(buffer), 1, optr) != 1)
        {
	xxx_LogStatus("XXXLIB",__FILE__,__LINE__, "Error writing new GCP library file");
	fclose(iptr);
	fclose(optr);
	unlink(new_gcplibfile);
        return(ERROR);
        }
    if (strncmp(buffer, "BEGIN", 5) == 0)
        break;
    }

/* Get the chip information from the file
  ---------------------------------------*/
data_count=0;
res_count=0;

for(;;)
    {
    if (fscanf(
              iptr,"%s%s%lf%lf%lf%lf%lf%lf%lf%lf%lf%lf%lf%s%s%s\n",
		gcp_lib.pt_id,gcp_lib.chip_name,
		&gcp_lib.ref_line, &gcp_lib.ref_samp,
		&gcp_lib.lat,&gcp_lib.lon,
		&gcp_lib.projy, &gcp_lib.projx,
		&gcp_lib.elev, &gcp_lib.pix_size[0],
		&gcp_lib.pix_size[1],&gcp_lib.ref_size[0],
		&gcp_lib.ref_size[1],gcp_lib.source,
		gcp_lib.date,gcp_lib.abs_rel_flag) != CHIPREC)
	break;

    if (fscanf(iptr,"%d%d%d\n", &gcp_lib.attempts,
		&gcp_lib.good_corr, &gcp_lib.good_prec) != CHIPOUT)
	break;


	/* check the date of the GCPLib  */
	sscanf(gcp_lib.date,"%ld-%ld-%ld",&month,&day,&year);
	status = xxx_check_year_month_day(year, month, day);
	if (status != SUCCESS)
		{
    	        xxx_LogStatus("GPSLIB",__FILE__,__LINE__,"Error in gcp lib date");
    	        return(ERROR);
		}

    /* Rewrite point info to new library file
      ---------------------------------------*/
    if (fprintf(optr,
	"%s %s %lf %lf %lf %lf %lf %lf %lf %lf %lf %lf %lf %s %s %s\n",
	gcp_lib.pt_id, gcp_lib.chip_name,
	gcp_lib.ref_line, gcp_lib.ref_samp,
	gcp_lib.lat, gcp_lib.lon,
	gcp_lib.projy, gcp_lib.projx, gcp_lib.elev, 
	gcp_lib.pix_size[0], gcp_lib.pix_size[1], 
	gcp_lib.ref_size[0], gcp_lib.ref_size[1], 
	gcp_lib.source, gcp_lib.date, gcp_lib.abs_rel_flag) < 0)
	{
	xxx_LogStatus("XXXLIB",__FILE__,__LINE__, "Error writing to new gcp library file");
	fclose(iptr);
	fclose(optr);
	unlink(new_gcplibfile);
	return(ERROR);
	}

    if (gcpdata) /*------------- UPDATE CORRELATION STATS --------------------*/
        {
        /* see if this point is in the list of gcpdata points
         ---------------------------------------------------*/
        for (curpt = 0; curpt < numgcp; curpt++)
            {
            /* test the GCP id
             -----------------*/
            if (strcmp(gcp_lib.pt_id,gcpdata[curpt].pt_id) == 0)
	        {
	        /* Increment counters
	          ------------------*/
		data_count++;
	        (gcp_lib.attempts)++;
	        if (gcpdata[curpt].accept_flg)
	            (gcp_lib.good_corr)++;


	        /* when the point is found, stop checking for it
	          ---------------------------------------------*/
	        break;
	        }
	    }
	} /* UPDATE CORRELATION STATS */

    if (gcp_res) /*------------- UPDATE PRECISION STATS ----------------------*/
	{
        /* see if this point is in the list of gcp residual points
         ---------------------------------------------------------*/
        for (curpt = 0; curpt < numgcp; curpt++)
            {
            /* test the GCP id
             -----------------*/
            if (strcmp(gcp_lib.pt_id,gcp_res[curpt].pt_id) == 0)
	        {
            	/* Increment counter if not an outlier
              	   -----------------------------------*/
		res_count++;
                if (gcp_res[curpt].outlier_flag == 0)
                    (gcp_lib.good_prec)++;

	        /* when the point is found, stop checking for it
	          ---------------------------------------------*/
	        break;
	        }
	    }
	} /* UPDATE PRECISION STATS */

    /* Write updated stats to new library file
    ----------------------------------------*/
    if (fprintf(optr,"%4ld %4ld %4ld\n",gcp_lib.attempts,
	gcp_lib.good_corr,gcp_lib.good_prec) < 0)
	{
	xxx_LogStatus("XXXLIB",__FILE__,__LINE__, "Error writing to new gcp library file");
	fclose(iptr);
	fclose(optr);
	unlink(new_gcplibfile);
	return(ERROR);
	}

    }
fclose(iptr);
fclose(optr);

/* Test that all points were updated
  ---------------------------------*/
if (((gcpdata) && (data_count != numgcp)) || 
    ((gcp_res) && (res_count != numgcp)))
    {
    unlink(new_gcplibfile);
    if ((gcpdata) && (data_count != numgcp))
		{
        status = sprintf(msg, "Not all points' correlation stats updated in gcp "
			"library file: found %ld, expected %ld", data_count, numgcp);
		if (status < 1)
  			{
			xxx_LogStatus ("XXXLIB", __FILE__, __LINE__, "Error in sprintf");
			return (ERROR);
			}
		}
    else
        sprintf(msg, "Not all points' precision stats updated in gcp library file: found %ld, expected %ld", res_count, numgcp);
    xxx_LogStatus("XXXLIB", __FILE__, __LINE__, msg);
    xxx_LogStatus("XXXLIB", __FILE__, __LINE__, "GCP library file not updated");
    return(ERROR);
    }
if ((gcp_res) && (res_count != numgcp))
    {
    unlink(new_gcplibfile);
    sprintf(msg, "Not all points' correlation stats updated in gcp library file: found %ld, expected %ld", data_count, numgcp);
    xxx_LogStatus("XXXLIB", __FILE__, __LINE__, msg);
    xxx_LogStatus("XXXLIB", __FILE__, __LINE__, "GCP library file not updated");
    return(ERROR);
    }

/* Move the new file to the old file
  ----------------------------------*/
if (rename(new_gcplibfile, gcplib_file_name) == -1)
    {
    unlink(new_gcplibfile);
    xxx_LogStatus("XXXLIB", __FILE__, __LINE__, "GCP library file not updated");
    return(ERROR);
    }

return(SUCCESS);
}
