/******************************************************************************
  GPS utility include file
   this file includes:
     structures for the system table
     ephemeris timing constants
     function prototypes for the GPS utility library
*******************************************************************************/
#ifndef _GXX_UTILITY_H_
#define _GXX_UTILITY_H_

#include <stdio.h>
#include <string.h>
#include <math.h>
#include "xxx_Types.h"
#include "xxx_LogError.h"
#include "xxx_LogStatus.h"
#include "xxx_DDR.h"  	
#include "xxx_ODL.h"	
#include "xxx_const.h"
#include "xxx_systbl.h"
#include "exx_cpf.h"
#include "exx_gcpdata.h"
#include "exx_gcplib.h"
#include "exx_gcp_residual.h"
#include "axx_grid.h"
#include "xxx_structures.h"

/*
**
** MACRO DEFINITIONS 
**
*/

#define SYSTEM_TABLE_NAME "system.table"	/* name of GPS system table */
#define GCPLIB_RECLEN 300       /* max length of a GCP Library file record */
#define GCP_RECLEN 250		/* max length of a GCP data file record */


/*
**
** TYPE DEFINITIONS
**
*/
typedef enum
{
    xxx_ODL_PARAM_OPTIONAL,
    xxx_ODL_PARAM_REQUIRED
} xxx_ODL_PARAM_REQUIRED_TYPE;
/* enumerated type for defining whether an item in a gxx_READ_ODL_LIST_TYPE is
   required or optional. */
   
typedef struct
{
    char *group_name;
    /* group name of parameter */

    char *param_name;
    /* name of parameter */
    
    void *param_ptr;
    /* location to return data read from ODL file for this parameter */
    
    int param_size;
    /* number of bytes at param_ptr */

    xxx_ValType_TYPE value_type;
    /* type of parameter */

    int min_count;
    /* minimum count of items to read. 0 = no minimum. */

    xxx_ODL_PARAM_REQUIRED_TYPE required;
    /* flag to indicate the parameter is either optional or required */

} xxx_READ_ODL_LIST_TYPE;
/* type definition for the list of parameters to read from an ODL file for
   a call to xxx_read_odl_file. */

typedef enum
{
    xxx_CL_FLAG,
    xxx_CL_INTEGER
} xxx_CL_PARAMETER_TYPE;
/* enumerated type for defining the data types that can be parsed by the
   xxx_parse_command_line function. */

typedef struct
{
    char *name;
    xxx_CL_PARAMETER_TYPE type;
    int *param;
} xxx_CL_PARAMETER_DEF_TYPE;
/* type definition for the list of command line parameters to parse from the
   command line by the gxx_parse_command_line_function. */
   
/*
**
** FUNCTION PROTOTYPES
**
*/

void xxx_createname (const char *, const char *, const char *, char *);
void xxx_find_underscore (const char *, char *, char *);
void xxx_create_DDR_name
(
    char *DDR_name,        /* O: pointer to formed DDR name */
    const char *HDF_name   /* I: HDF name to form DDR name from */
);
void xxx_splitname (const char *, char *, char *, char *);
int xxx_nearest_power2(int ns);
char *xxx_strtoupper 
(
    char *string_ptr  /* I/O: pointer to string to convert */
);
char *xxx_strtolower 
(
    char *string_ptr  /* I/O: pointer to string to convert */
);
int xxx_read_odl_file
(
    const char *odl_file_name,   /* I: ODL file name to read */
    int find_cpf,                /* I: set this flag to non-zero to indicate
                                    the file name is a 0R filename and the CPF
                                    should be read from it */
    xxx_READ_ODL_LIST_TYPE *odl_list_ptr, 
                                 /* I/O : pointer to list of items to read
                                    from the ODL file */
    int list_length              /* I: number of items in the list */
);
int xxx_parse_command_line
(
    int argc,                      /* I: number of parameters passed in */
    char *argv[],                  /* I: array of parameters passed in */
    int params,                    /* I: number of entries in param_list */
    xxx_CL_PARAMETER_DEF_TYPE *param_list /* I/O: list of parameters to parse */
);

/* system table functions */
int xxx_get_system_table(GPS_SYSTBL *systbl);
int xxx_open_system_table(char *table_name);
int xxx_close_system_table(void);
int xxx_read_system_table(const OBJDESC *odl_tree, GPS_SYSTBL *systbl);

int xxx_calc_poly (
  const xxx_DDR_TYPE *ddr,    /* I: DDR for the desired band resolution   */
  double poly_x[4],           /* O: Sample-to-projection coefficients     */
  double poly_y[4]);          /* O: Line-to-projection coefficients       */

/* routine to calculate transform polynomial coefficients */
int xxx_calc_transform_poly 
(
  const double source_proj_x[4], /* I: source projection x corner coordinates */
  const double source_proj_y[4], /* I: source projection y corner coordinates */
  const double target_proj_x[4], /* I: target projection x corner coordinates */
  const double target_proj_y[4], /* I: target projection y corner coordinates */
  int include_xy_term,           /* I: include x*y term if not zero */
  double poly_x[4],              /* O: calculated x polynomial coefficients */
  double poly_y[4]               /* O: calculated y polynomial coefficients */
);

/* precision solution file I/O routines */
int xxx_put_solution(
  const char *ref_L0R,     		/* I: L0R HDF reference filename */
  const char solution_file_name[STRLEN],/* I: Name of the solution file */
  const GCP_SOLUTION *gcp_sol           /* I: ptr to correction info struct */
);

int xxx_output_header (
  FILE *fileptr,           /* I: Pointer to the output file            */
  const char *app_name,    /* I: String for the application name       */
  const char *ref_L0R      /* I: L0R HDF reference filename            */
);

int xxx_get_residual(
        const char *resfile,            /* I-name of precision residual file  */
        int  iteration_number,          /* I-which iteration from file to get */
        GCP_RESIDUAL **gcp_res,         /* O-GCP residual info                */
        int  *num_res                   /* O-number of residual structs read  */
);

int xxx_write_residual(
  FILE *res_fptr,       /* I: pointer to the rfile (residual file); */
  int  num_gcp,         /* I: the number of GCPs used in the solution; */
  int  iter_num,        /* I: The iteration number, -1 is final iteration */
  const char *ref_L0R,     /* I: L0R HDF reference filename            */
  const struct GCP_RESIDUAL *gcp_res /* I: array of GCP information structures*/
);

int xxx_write_gcp (
        FILE *gcp_fptr,                 /* I: GCP data file pointer     */
        const GCPDATA *gcp_data         /* I: One GCP data record       */
);

int xxx_get_gcp (
    const char *gcp_data_filename,      /* I-GCP data file name         */
    const CPF *cpf,                     /* I-Cal parm file constants    */
    GCPDATA **gcp_data,                 /* O-GCP data records           */
    int  *numgcp                        /* O-Number of GCPs read        */
);

int xxx_read_gcp (
        FILE *gcp_fptr,                 /* I-GCP data file pointer      */
        const CPF *cpf,                 /* I-cal parm file constants    */
        GCPDATA *gcp_data               /* O-One GCP data record        */
);

int xxx_extract_window(
        int  image_nl,          /* I-# of lines in the 1G image         */
        int  image_ns,          /* I-# of samples in the 1G image       */
        BandNumber band,        /* I-band number to read from           */
        DataType dtype,         /* I-data type of the band              */
        double pred_line,       /* I-center line of window to extract   */
        double pred_samp,       /* I-center sample of window to extract */
        int  win_size[2],       /* I-size of window to extract          */
        HDFIO *img_ptr,         /* I-image pointer                      */
        float *img_buf);        /* O-Buffer read from 1G image          */

int xxx_extract_chip (
        const char *chip_name,  /* I-name of the image chip file*/
        const int  chip_size[2],/* I-expected size of image chip*/
        void *chip_buf);        /* O-buffer of chip image data  */

int xxx_get_gcplib(
    const char *gcplib_file_name,/* I: Name of the GCPLIB file */
    char *abs_or_rel,           /* I: Flag for absolute or relative source */
    int  min_percent_succ,      /* I: Flag for min percent acceptable */
    const int  *begin_date,     /* I: Flag for beginning date
                                                [0] = month
                                                [1] = year */
    const int  *end_date,       /* I: Flag for ending date
                                                [0] = month
                                                [1] = year */
    char season[4][7],          /* I: Flag for season of chip */
    char chip_source[10][9],    /* I: Flag for source of chip */
    int  row_wrs,               /* I: wrs row */
    GCPLIB **gcp_lib,           /* O: Structure of chip information */
    int  *numgcp);              /* O: number of ground control points */

int xxx_put_gcp (
        const char *gcp_data_filename,  /* I-Name of GCP data file to add to */
        const GCPDATA gcp_data[],       /* I-GCP data structures             */
        int  num_gcp);                  /* I-Number of GCP's                 */

int xxx_update_gcplib(
    const GCPDATA *gcpdata,       /* I: ground control point data structure */
    const GCP_RESIDUAL *gcp_res,  /* I: GCP precision residual data structure*/
    const char *gcplib_file_name, /* I: Name of the GCPLIB file             */
    int  numgcp);                 /* I: number of GCP records               */

int xxx_check_year_month_day
(
  int year,     	/* I: year (YYYY) */
  int month,    	/* I: month (MM) */
  int day               /* I: day (DD) */
);

#endif
