/******************************************************************************
NAME:           xxx_write_residual

PURPOSE:        Write out the residual for along and across scan components
		for each GCP to the residual file;

DEVELOPMENT HISTORY:
Before release into the IAS code the prototype history is as follows:
 1.0	05/96	D. Kuang	Original Development
 1.2    01/97   J. Storey       Outlier rejection logic added.
$Log: xxx_write_residual.c,v $
Revision 1.2  2001/07/09 12:40:14  choate
Removed BandNumber check

Revision 1.1  2000/12/07 23:04:23  storey
Original version.


RETURN VALUE:
Type = long
Value		Description
-----		-----------
SUCCESS		Successfully wrote to the residual file
ERROR		Did not complete writing to residaul file

ALGORITHM REFERENCES:

*******************************************************************************
			Property of the U.S. Government
		             USGS EROS Data Center
******************************************************************************/
#include <stdio.h>
#include "exx_gcp_residual.h"
#include "xxx_utility.h"

int xxx_write_residual(
  FILE *res_fptr,	/* I: pointer to the rfile (residual file); */
  int  num_gcp,		/* I: the number of GCPs used in the solution; */
  int  iter_num,	/* I: The iteration number, -1 is final iteration */
  const char *ref_L0R,     /* I: L0R HDF reference filename            */
  const struct GCP_RESIDUAL *gcp_res)/* I: array of GCP information structures*/
{
int   i;           	/* looping variables            */
int band_tmp;		/* temporary user band number	*/

/*
  Output the standard header if on the first iteration.  Also output the
  labels and units of the residual information.
*/
if (iter_num == 0)
    {
    if (xxx_output_header (res_fptr, "RESIDUAL", ref_L0R) != SUCCESS)
        {
        xxx_LogStatus("XXXLIB", __FILE__, __LINE__,
                      "Error writing the standard header for this report.");
        return(ERROR);
        }
    /*
      Enter the band number of the first point, it should be the same for all
      the points
    */

    fprintf(res_fptr,"Band Number:  %d\n\n",band_tmp);
    fprintf(res_fptr,
      "Point_ID    Predicted Predicted      Time      Latitude  Longitude    "
      "Height       Scan      Across  Along  Residual Residual Outlier\n"
      "               Line     Sample                                        "
      "             Angle      Scan    Scan     In y     in x    Flag\n"
      "                                                                      "
      "                      Residual Residual   dir      dir   (0=bad\n"
      "                                     (sec)       (deg)     (deg)     "
     "(meters)      (deg)    (meters) (meters) (meters) (meters) 1=OK)\n");
    }

/*
  Write the first line with iteration number
*/
if (iter_num >= 0)
    fprintf(res_fptr, "Iteration %ld\n", iter_num);
else
    fprintf(res_fptr, "Final Iteration\n");

/*
  Loop over all GCPs to write out the residual information
*/
for(i=0; i<num_gcp; i++)
    {
    /*
      Format changed to include outlier flag. Shortened residual fields
      by one character to keep to 80 bytes total. Changed by JCS 1/97.
      Added point id, therefore exceeded the 80 byte total SRN 5/97.
    */
   if(fprintf(res_fptr,
      "%-11s %9.3f %9.3f %12.6f %10.6f %11.6f %9.3f %12.6f %9.3f %9.3f %9.3f %9.3f %1ld\n",
	    gcp_res[i].pt_id,		/* point id			*/
	    gcp_res[i].pred_line,	/* predicted line */
	    gcp_res[i].pred_samp,	/* predicted samp */
            gcp_res[i].seconds,		/* time, in seconds		*/
	    gcp_res[i].lat,		/* latitude, in degree		*/
	    gcp_res[i].lon,		/* longitude, in degree		*/
	    gcp_res[i].height,		/* height, in meters		*/
	    gcp_res[i].delta,		/* scan angle, in degree	*/
	    gcp_res[i].vb,		/* along-scan residual, meters 	*/
	    gcp_res[i].va,		/* cross-scan residual, meters 	*/
	    gcp_res[i].residual_y,	/* Residual in x direction, meters */
	    gcp_res[i].residual_x,	/* Residual in y direction, meters */
            gcp_res[i].outlier_flag)	/* outlier flag:  1=OK, 0=bad	*/
								< 0)
	{
	xxx_LogStatus("XXXLIB",__FILE__,__LINE__,
					"Error printing to residual file");
	return(ERROR);
	}
    }
return(SUCCESS);
}
