/******************************************************************************
*                              PACOR II SOFTWARE
*                      Property of the U.S. Government
*                              NASA/GSFC/Code 560
*******************************************************************************
*
*    UNIT NAME:
*	xxx_CloseUnmap.c
*
*
*    PURPOSE:
*	To close a file and optionally unmemory map the file.
*
*
*    INVOCATION METHOD:
*	status = xxx_CloseUnmap(fd, p_pa, file_len, mapflag, lockflag, p_ErrorMessage)
*
*
*    RETURNED VALUE:
*
*       TYPE = int
*
*       VALUE           DESCRIPTION
*       ------------    ----------------------------
*	XXX_UNMAP_ERR	Indicates unmap error
*	XXX_CLOSE_ERR	Indicates file close error
*       SUCCESS		Terminated successfully
*
*
*    ARGUMENT LIST:
*
*       NAME            TYPE		USE	DESCRIPTION
*       ------------    ------------	---	----------------------------------
*	fd		int		I	File descriptor
*	p_pa		void *		I	Memory map address
*	file_len	long		I	File length
*	mapflag		boolean		I	Mapping indicator
*	lockflag	boolean		I	locking indicator
*	p_ErrorMessage	char *		O	Error Message
*
*
*    EXTERNAL FILE/RECORD REFERENCES:
*
*       NAME			USE     DESCRIPTION
*       ----			---     -----------------------------------------------
*	string.h		I	Contains UNIX error information
*	unistd.h		I	Defines close and SEEK_SET
*	sys/mman.h		I	Header file needed for memory mapping support
*	xxx_CloseUnmap.h	I	Defines function prototypes and contants
*	xxx_FileLock.h		I	Defines prototype and macros for lock and 
*					  unlock files
*	xxx_Types.h		I	Header file needed for PDP type definitions
*	Unknown	file		I	Any file that may be passed into the routine
*
* 
*    EXTERNAL VARIABLES:
*
*       SOURCE			NAME		TYPE  USE DESCRIPTION
*       ------			----		----  --- -------------------------------
*	xxx_CloseUnmap.h	XXX_UNMAP_ERR	int   I	  Indicates unmap error
*	xxx_CloseUnmap.h	XXX_CLOSE_ERR	int   I   Indicates file close error
*	xxx_Types.h		SUCCESS		int   I   Indicates successful completion
*
*
*    EXTERNAL UNIT/MACRO/PROCEDURE/FUNCTION REFERENCES:
*
*       NAME            PURPOSE
*       ------------    -------------------------------------------------------
*	xxx_FileLock	Locks and unlock files
*	XXX_UNLOCK	Macro to call xxx_FileLock to unlock file
*
*
*    ABNORMAL TERMINATION CONDITIONS, ERROR MESSAGES:
*	Can't unmap file
*	Can't close file
*
*
*    ASSUMPTIONS, CONSTRAINTS, RESTRICTIONS:
*	The file is open and/or mapped.
*
*
*    NON-PORTABLE CODE:
*
*	DESCRIPTION	REASON FOR USE 		SUGGESTED PORTING PROCEDURES
*	-----------	---------------		---------------------------------------
*	munmap		Speed up I/O to file
*
*
*    CHANGE HISTORY:
*
*                                  BUILD/
*       AUTHOR          CHANGE-ID  RELEASE  DATE     DESCRIPTION OF CHANGE
*       -----------     ---------  -------- -------- -----------------------
*       J. Kerich	PAC2940631 R2.2     12/28/94 Original Implementation
*	J. kerich                  R1.0     03/24/97 Ported to IAS
*
*
*    NOTES:
*	Unit was originally pxx_close_unmap.c
*
******************************************************************************/
/******************************************************************************
UNIT xxx_CloseUnmap PDL
-------------------------------------------------------------------------------

	Set status to SUCCESS

1	IF mapflag is TRUE THEN
		Call munmap() to unmap the file using p_pa and file_len

2		IF unable to unmap the file THEN 
			Build error message in p_ErrorMessage
			Set status to XXX_UNMAP_ERR
3		ENDIF

4 	ENDIF

5	IF lockflag is TRUE THEN
		Call XXX_UNLOCK() to unlock the file
6	ENDIF

	Close the file pointed to by fd
7	IF close failed THEN
		Build error message in p_ErrorMessage
		Set status to XXX_CLOSE_ERR
8	ENDIF

	RETURN status

-------------------------------------------------------------------------------
END UNIT xxx_CloseUnmap PDL
******************************************************************************/
/* Feature test switches */
#undef _POSIX_SOURCE

/* System headers */
#include <string.h>
#include <unistd.h>
#include <sys/mman.h>

/* Local headers */
#include <xxx_FileLock.h>
#include <xxx_Types.h>

/* External Functions */
#include <xxx_CloseUnmap.h>

/* File Scope Variables */
extern int errno;

int xxx_CloseUnmap(int fd, void *p_pa, long file_len, boolean mapflag, boolean lockflag, char *p_ErrorMessage)
{
	int status = SUCCESS;	/* return status */

/* memory map the file if requested. Return error if one occured */

	if (mapflag)
	{
		if(munmap(p_pa, file_len) == -1)
		{
			(void) strcpy(p_ErrorMessage, strerror(errno));
			status = XXX_UNMAP_ERR;
		}
	}

	if (lockflag)
	{
		(void)XXX_UNLOCK(fd, 0, SEEK_SET, 0);
	}

/* close file and return error if one occured */

	if(close(fd) < 0)
	{
		(void) strcpy(p_ErrorMessage, strerror(errno));
		status = XXX_CLOSE_ERR;
	}

	return (status);
}
