/******************************************************************************
                Landsat 7 Image Assessment System (IAS) Software
                        Property of the U.S. Government
                            NASA / GSFC / Code 510
*******************************************************************************

UNIT NAME:  xxx_LogError.c

PURPOSE:  Write error notification to the IAS software log.  This function is 
meant to be called by Process Control (PCS) and Data Management (DMS) 
subsystems and NOT Radiometry (RPS) and Geometry (GPS) subsystems.

INVOCATION METHOD:
	Status = xxx_LogError(p_UnitName, LineNumber, p_ErrorMessage)

RETURN VALUE:
Type = boolean

Value          	Description
-------------- 	----------------------------------------------------------------
SUCCESS		Wrote error message to log
ERROR		Failed to write error message to log

ARGUMENT LIST:
Name            Type               	Use 	Description
--------------- ------------------ 	--- 	---------------------------------------
p_UnitName	char *			I	Unit name
LineNumber	int			I	Line number
p_ErrorMessage	char *			O	Error message

EXTERNAL FILE/RECORD/DBTABLE REFERENCES:
Name                    Use     Description
----------------------  ---     -------------------------------------------------
IAS-log                 I/O     Local IAS software log file

EXTERNAL VARIABLES:
Source          Name                	Type            Use 	Description
--------------- ------------------- 	--------------- --- 	-----------------------
xxx_LogError	xxx_LogErrorPath	char * 		I	Path to log error file
xxx_LogError	xxx_TaskName		char * 		I	Task name
xxx_Types.h     SUCCESS                 int		I       Defined parameter
xxx_Types.h     ERROR                   int    		I       Defined parameter


EXTERNAL UNIT/MACRO/PROCEDURE/FUNCTION REFERENCES:
Name                          Description
----------------------------- -------------------------------------------------
xxx_FileLock		      Locks and unlocks files
xxx_GetTime		      Creat time stamp for local IAS software log
XXX_UNLOCK		      Macro to call xxx_FileLock to unlock file
XXX_WRITE_LOCK		      Macro to call xxx_FileLock to lock file in nowait mode

ABNORMAL TERMINATION CONDITIONS, ERROR MESSAGES:
none

ASSUMPTIONS, CONSTRAINTS, RESTRICTIONS:
none

DEVELOPMENT HISTORY:
Author            Change-ID   Release Date       Description of Change
----------------- ----------- ------- ---------- ------------------------------
J. kerich                     R1.0    01/31/97   Original Implementation

NOTES:
The task must set the xxx_TaskName string to the task executable, e.g. "DID".

The local IAS software log file is created in the local directory and is
IAS-log.

If any open, malloc, or write error occurs, the unit will return immediately 
with a ERROR status.

Unit xxx_LogError PDL:

1	IF the local IAS software log file does not exit THEN
		Create the local IAS software log file
		Write the local IAS software log banner
2	ELSE
		Open the local IAS software log file in append mode
	ENDIF

3	DOWHILE not locked
		Call XXX_WRITE_LOCK() to lock the local IAS software log file in no-wait mode
4		IF file is not locked THEN
			Close the local IAS software log file
			Sleep for a second
			Open the local IAS software log file in append mode			
5		ENDIF
6	ENDDO

	Call xxx_GetTime() to get the time stamp
	Write the time stamp, xxx_TaskName, getpid(), P_UnitName, LineNumber, 
	  p_ErrorMessage into the local IAS software log

	Call XXX_UNLOCK() to unlock the local IAS software log 
	Close the local IAS software log

	RETURN SUCCESS

End Unit xxx_LogError
******************************************************************************/

/* System headers */
#include <errno.h>
#include <fcntl.h>
#include <malloc.h>
#include <netdb.h>
#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include <sys/param.h>
#include <sys/stat.h>

/* Local headers */
#include <xxx_Types.h>

/* Macros */
#define  NAMESIZE 80
#define  LOG_NAME  "IAS-Log"

#define Header1 "                                              IAS local software log\n\n"
#define Header2 "                   Task Process id                  Unit Name                         Line Number  Error Message\n"
#define Header3 "                   ---- ---------- -------------------------------------------------- -----------  -------------\n"
#define Dummy   "1997 192 34:34:34  PSC   1234567   12345678911234567892123456789312345678941234567895    123456    " 

/* Global variables */
char xxx_LogErrorPath[MAXPATHLEN];
char xxx_TaskName[4];

/* External functions */
#include <xxx_FileLock.h>
#include <xxx_GetTime.h>
#include <xxx_LogError.h>

/* Main */
int xxx_LogError(char *p_UnitName, int LineNumber, char *p_ErrorMessage)

{
  /* lpgslite: log to console instead of files, mbs */
#ifdef LPGSLITE
  /* remove extra \n */
  /* char *p;
     while (p = strchr(p_ErrorMessage,'\n')) *p=0; */
  xxx_LogStatus("---", p_UnitName, LineNumber, p_ErrorMessage);
  return 0;
#else
	int fdLog;
	char LocalLogFilename[NAMESIZE];
	char TimeStamp[20];

	struct stat info;
	char * Buffer;
	long	status;
	char Cantmalic[150];
	int Not_Lock;

	(void)strcpy(LocalLogFilename, xxx_LogErrorPath);
	(void)strcat(LocalLogFilename, LOG_NAME);

	if (stat(LocalLogFilename, &info) == -1)
	{
		if ((fdLog = open(LocalLogFilename, O_CREAT | O_WRONLY,
			 S_IRUSR | S_IWUSR | S_IRGRP | S_IWGRP | S_IROTH | S_IWOTH  )) == -1)
		{
			return ERROR;
		}

		if ((Buffer = (char *)malloc(strlen(Header1)+strlen(Header2)+strlen(Header3)+1)) == (char *)NULL)
		{
			(void) xxx_GetTime(TimeStamp, sizeof(TimeStamp), "%Y %j %H:%M:%S  ");
			(void)sprintf(Cantmalic, "%17s%3s   %-7ld   %-50s    %6d    %s\n",
			TimeStamp, xxx_TaskName, getpid(), __FILE__, __LINE__, "Can't malloc memory for write");
			while ((status = write(fdLog, Cantmalic, strlen(Cantmalic))) == -1 && errno == EINTR);
			(void)close(fdLog);
			return ERROR;
		}

		(void)strcpy(Buffer, Header1);
		(void)strcat(Buffer, Header2);
		(void)strcat(Buffer, Header3);

		while ((status = write(fdLog, Buffer, strlen(Buffer))) == -1 && errno == EINTR);

		free(Buffer);
		if (status < 0) 
		{
			(void)close(fdLog);
			return ERROR;
		}
	}
	else
	{
		if ((fdLog = open(LocalLogFilename, O_WRONLY | O_APPEND)) == -1)
		{
			return ERROR;
		}
	}

	Not_Lock = SUCCESS;
	while (Not_Lock)
	{	
		if ((Not_Lock = XXX_WRITE_LOCK(fdLog, 0, SEEK_SET, 0)))
		{
			(void)close(fdLog);
			(void) sleep(1);
			if ((fdLog = open(LocalLogFilename, O_WRONLY | O_APPEND)) == -1)
				return ERROR;
		}
	}

	if (p_ErrorMessage != (char *)NULL)
	{
		/* the extra 100 bytes are just in case of an overflow in one of the other fields */
		if ((Buffer = (char *)malloc(strlen(Dummy) + strlen(p_ErrorMessage)+2 + 100)) == (char *)NULL)
		{
			(void) xxx_GetTime(TimeStamp, sizeof(TimeStamp),"%Y %j %H:%M:%S  ");
			(void)sprintf(Cantmalic, "%19s%3s   %-7ld   %-50s    %6d    %s\n",
				TimeStamp, xxx_TaskName, getpid(), __FILE__, __LINE__, "Can't malloc memory for write");
			while ((status = write(fdLog, Cantmalic, strlen(Cantmalic))) == -1 && errno == EINTR);
			(void)XXX_UNLOCK(fdLog, 0, SEEK_SET, 0);
			(void)close(fdLog);
			return ERROR;
		}

		(void) xxx_GetTime(TimeStamp, sizeof(TimeStamp),"%Y %j %H:%M:%S  ");
		(void)sprintf(Buffer, "%19s%3s   %-7ld   %-50s    %6d    %s\n",
			TimeStamp, xxx_TaskName, getpid(), p_UnitName, LineNumber, p_ErrorMessage);

		while ((status = write(fdLog, Buffer, strlen(Buffer))) == -1 && errno == EINTR);

		free(Buffer);
		if (status < 0)
		{
			(void)XXX_UNLOCK(fdLog, 0, SEEK_SET, 0);
			(void)close(fdLog);
			return ERROR;
		}
	}

	(void)XXX_UNLOCK(fdLog, 0, SEEK_SET, 0);
	(void)close(fdLog);

return SUCCESS;
#endif
}
