/******************************************************************************
                Landsat 7 Image Assessment System (IAS) Software
                        Property of the U.S. Government
                            NASA / GSFC / Code 510
*******************************************************************************

UNIT NAME:  xxx_ReadDDR.c

PURPOSE:  Read in HDF DDR file and store it in xxx_DDRControl_TYPE.

INVOCATION METHOD:
	Pointer = xxx_ReadDDR(p_PathName, p_ErrorMessage);

RETURN VALUE:
Type = xxx_DDRControl_TYPE *

Value           Description
--------------  ----------------------------------------------------------------
Positive	Pointer to xxx_DDRControl_TYPE
NULL		Error Reading in DDRs

ARGUMENT LIST:
Name             Type           Use	Description
---------------- -------------- ---	---------------------------------------
p_PathName	char *		I	Pathname to DDR file
p_ErrorMessage	char *		O	Error message

EXTERNAL FILE/RECORD/DBTABLE REFERENCES:
Name				Use	Description
-------------------------	---	-------------------------------------------------
DDR file                        I       HDF file

EXTERNAL VARIABLES:
Source		Name				Type		Use	Description
---------------	----------------		---------------	---	-----------------------
herr.h          error_top                       int32           I       Top to HDF error stack

EXTERNAL UNIT/MACRO/PROCEDURE/FUNCTION REFERENCES:
Name            Description
--------------- -------------------------------------------------
Hclose		Close HDF file
Hopen		Open HDF file
HEstring	Returns error message for the error number
HEvalue         Returns the current error number
Vend		Close the vdata interface
Vstart		Initialize the vdata interface
VSattach	Establish access to a specified vdata
VSdetach	Detach from vdata
VSgetid		Get vdata reference id
VSread		Reads from vdata
VSinquire	Get information on vdata
VSsetfields	Pick fields to read

ABNORMAL TERMINATION CONDITIONS, ERROR MESSAGES:
HDF errors
memory errors

ASSUMPTIONS, CONSTRAINTS, RESTRICTIONS:
none

DEVELOPMENT HISTORY:
Author                  Change-ID       Release Date            Description of Change
-----------------       -----------     ------- --------        ------------------------------
J. kerich               N/A             R1.0    03/19/97        Original Implementation
J. Kerich	        IASI970070      R2.0    10/28/97	Add path to open error message
J. kerich		IASI980293     	R3.0	02/12/98	Resize fields NumberLinesInScene,  
 		 		 		 		NumberSamplesInScene, MasterLine, and
 		 		 		 		MasterSample
NOTES:
none

Unit xxx_ReadDDR PDL:

        Allocate memory for xxx_DDRControl_TYPE structure array
1       IF error THEN
                Build error message in p_ErrorMessage
                RETURN NULL
2       ENDIF

	Call Hopen() to open DDR
3       IF error opening file THEN
                Call HEstring(HEvalue(error_top)) to build error message in p_ErrorMessage
                ABORT to ERROR_PROC 
4       ENDIF

        Call Vstart() to initialize the vdata interface

	Call VSgetid() to get first vdata reference id
5       IF error getting reference id THEN
                Call HEstring(HEvalue(error_top)) to build error message in p_ErrorMessage
                ABORT to ERROR_PROC 
6       ENDIF

	Call VSattach() to attach to the vdata
7       IF error attaching to vdata THEN
                Call HEstring(HEvalue(error_top)) to build error message in p_ErrorMessage
                ABORT to ERROR_PROC 
8       ENDIF

        Call VSgetclass() to get the vdata class

	Call VSinquire() to get information on vdata
9       IF error getting vdata information THEN
                Call HEstring(HEvalue(error_top)) to build error message in p_ErrorMessage
                ABORT to ERROR_PROC 
10      ENDIF

	Call VSsetfields() to pick all the fields
11      IF error setting fields to read THEN
                Call HEstring(HEvalue(error_top)) to build error message in p_ErrorMessage
                ABORT to ERROR_PROC 
12      ENDIF

	Allocate memory to read out all the rows in the DDR files
13      IF error allocating memory THEN
                Build error message in p_ErrorMessage
                ABORT to ERROR_PROC 
14      ENDIF

	Allocate memory for xxx_DDR_TYPE structure array
15      IF error allocating memory THEN
                Build error message in p_ErrorMessage
                ABORT to ERROR_PROC 
16      ENDIF

	Call VSread() to read in all the data
17      IF error reading vdata THEN
                Call HEstring(HEvalue(error_top)) to build error message in p_ErrorMessage
                ABORT to ERROR_PROC 
18      ENDIF

	Set the DDR entry count in xxx_DDRControl_TYPE
19	DOFOR all entries
		Break out the field from the DDR buffer and copy them into 
		  the xxx_DDR_TYPE structure array
20	ENDDO

        Call VSdetach() to detach from vdata
	Call Vend() to close the vdata interface

	Call Hclose() to close DDR file
21      IF error closing HDF file THEN
                Call HEstring(HEvalue(error_top)) to build error message in p_ErrorMessage
                ABORT to ERROR_PROC 
22      ENDIF

	Free DDR buffer memory

	RETURN xxx_DDRControl_TYPE structure

ERROR_PROC:
	Call VSdetach() to detach from vdata
	Call Vend() to close the vdata interface
	Call Hclose() to close DDR file
	Free memory

	RETURN NULL



End Unit xxx_ReadDDR
******************************************************************************/

/* System headers */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

/* Local headers */
#include <hdf.h>
#include <xxx_DDR.h>

extern int32 error_top;

xxx_DDRControl_TYPE *xxx_ReadDDR(char *p_PathName, char *p_ErrorMessage)
{
	xxx_DDRControl_TYPE *p_DDRControl = NULL;
	const char *p_HEstring;
	int	k;

	int32	file_id;
	int32	vdata_id = 0;
	int32	vdata_ref = -1;
	int32	field_index;
	char	fields[XXX_DDR_FIELD_SIZE];
	uint8	*p_databuff = NULL;
	uint8	*p_Ptr;

	if ((p_DDRControl = calloc(1, sizeof(xxx_DDRControl_TYPE))) == NULL)
	{
		(void)strcpy(p_ErrorMessage, "Can't calloc xxx_DDRControl_TYPE structure");
		return (NULL);
	}

	if ((file_id = Hopen(p_PathName, DFACC_RDONLY, 0)) == FAIL)
	{
		(void)sprintf(p_ErrorMessage, "xxx_ReadDDR: Can't open DDR file %s (%s): ", 
                              p_PathName, getenv("PWD"));
		/* (void)sprintf(p_ErrorMessage, "Can't open DDR file %s: ", p_PathName); */
		p_HEstring = HEstring((hdf_err_code_t)HEvalue(error_top));
		(void)strcat(p_ErrorMessage, p_HEstring);
		goto ERROR_PROC;
	}

	Vstart(file_id);

	vdata_ref = -1;
	if ((vdata_ref = VSgetid(file_id, vdata_ref)) == FAIL)
	{
		(void)strcpy(p_ErrorMessage, "Can't find vdata in DDR file: ");
		p_HEstring = HEstring((hdf_err_code_t)HEvalue(error_top));
		(void)strcat(p_ErrorMessage, p_HEstring);
		goto ERROR_PROC;
	}

	if ((vdata_id = VSattach(file_id, vdata_ref, "r")) == FAIL)
	{
		(void)strcpy(p_ErrorMessage, "Error attaching to DDR file: ");
		p_HEstring = HEstring((hdf_err_code_t)HEvalue(error_top));
		(void)strcat(p_ErrorMessage, p_HEstring);
		goto ERROR_PROC;
	}

	VSgetclass(vdata_id, p_DDRControl->vdataClass);

	if (VSinquire(vdata_id, &p_DDRControl->Count, &p_DDRControl->Interlace, fields, 
	    &p_DDRControl->vdataSize, p_DDRControl->vdataName) == FAIL)
	{
		(void)strcpy(p_ErrorMessage, "Error inquiring DDR file: ");
		p_HEstring = HEstring((hdf_err_code_t)HEvalue(error_top));
		(void)strcat(p_ErrorMessage, p_HEstring);
		goto ERROR_PROC;
	}

	if (VSsetfields(vdata_id, fields) == FAIL)
	{
		(void)strcpy(p_ErrorMessage, "Error setting DDR fields: ");
		p_HEstring = HEstring((hdf_err_code_t)HEvalue(error_top));
		(void)strcat(p_ErrorMessage, p_HEstring);
		goto ERROR_PROC;
	}

	if ((p_databuff = (uint8 *)calloc(p_DDRControl->Count, p_DDRControl->vdataSize)) == NULL)
	{
		(void)strcpy(p_ErrorMessage, "Can't calloc unpadded databuff");
		goto ERROR_PROC;
	}

	if ((p_DDRControl->p_DDR = (xxx_DDR_TYPE *)calloc(p_DDRControl->Count, sizeof(xxx_DDR_TYPE))) == NULL)
	{
		(void)strcpy(p_ErrorMessage, "Can't calloc xxx_DDR_TYPE structure");
		goto ERROR_PROC;
	}

	if (VSread(vdata_id, p_databuff, p_DDRControl->Count, p_DDRControl->Interlace) == FAIL)
	{
		(void)sprintf(p_ErrorMessage, "Can't read DDR file (%s): ", getenv("PWD"));
		/* (void)strcpy(p_ErrorMessage, "Can't read DDR file: ");*/
		p_HEstring = HEstring((hdf_err_code_t)HEvalue(error_top));
		(void)strcat(p_ErrorMessage, p_HEstring);
		goto ERROR_PROC;
	}

	for (k=0; k<p_DDRControl->Count; k++)
	{
		p_Ptr = p_databuff+(k*p_DDRControl->vdataSize);

		(void)memcpy(&p_DDRControl->p_DDR[k].BandNumber, p_Ptr, sizeof(int8));
		p_Ptr += sizeof(int8);
		(void)memcpy(&p_DDRControl->p_DDR[k].NumberScansInScene, p_Ptr, sizeof(int16));
		p_Ptr += sizeof(int16);
                if ( VFfieldtype( vdata_id, 2 ) == DFNT_INT16 )
                {
		  (void)memcpy(&p_DDRControl->p_DDR[k].NumberdetectorsPerScan, p_Ptr, sizeof(int16));
		  p_Ptr += sizeof(int16);
                }
                else
                {
		  (void)memcpy(&p_DDRControl->p_DDR[k].NumberdetectorsPerScan, p_Ptr, sizeof(int8));
		  p_Ptr += sizeof(int8);
                }
		(void)memcpy(&p_DDRControl->p_DDR[k].NumberLinesInScene, p_Ptr, sizeof(int32));
		p_Ptr += sizeof(int32);
		(void)memcpy(&p_DDRControl->p_DDR[k].NumberSamplesInScene, p_Ptr, sizeof(int32));
		p_Ptr += sizeof(int32);
		(void)memcpy(&p_DDRControl->p_DDR[k].DataType, p_Ptr, sizeof(int16));
		p_Ptr += sizeof(int16);
		(void)memcpy(&p_DDRControl->p_DDR[k].MaxPixelValue, p_Ptr, sizeof(float64));
		p_Ptr += sizeof(float64);
		(void)memcpy(&p_DDRControl->p_DDR[k].MinPixelValue, p_Ptr, sizeof(float64));
		p_Ptr += sizeof(float64);
		(void)memcpy(&p_DDRControl->p_DDR[k].MaxRadianceValue, p_Ptr, sizeof(float64));
		p_Ptr += sizeof(float64);
		(void)memcpy(&p_DDRControl->p_DDR[k].MinRadianceValue, p_Ptr, sizeof(float64));
		p_Ptr += sizeof(float64);
		(void)memcpy(&p_DDRControl->p_DDR[k].MasterLine, p_Ptr, sizeof(int32));
		p_Ptr += sizeof(int32);
		(void)memcpy(&p_DDRControl->p_DDR[k].MasterSample, p_Ptr, sizeof(int32));
		p_Ptr += sizeof(int32);
		(void)memcpy(&p_DDRControl->p_DDR[k].ProjectCode, p_Ptr, sizeof(int16));
		p_Ptr += sizeof(int16);
		(void)memcpy(&p_DDRControl->p_DDR[k].ZoneCode, p_Ptr, sizeof(int16));
		p_Ptr += sizeof(int16);
		(void)memcpy(p_DDRControl->p_DDR[k].DatumCode, p_Ptr, sizeof(char8)*XXX_DATUM_CODE_SIZE);
		p_DDRControl->p_DDR[k].DatumCode[XXX_PROJECTION_UNITS_SIZE] = '\0';
		p_Ptr += sizeof(char8)*XXX_DATUM_CODE_SIZE;
		(void)memcpy(&p_DDRControl->p_DDR[k].SpheroidCode, p_Ptr, sizeof(int32));
		p_Ptr += sizeof(int32);
		(void)memcpy(p_DDRControl->p_DDR[k].ProjectionUnits, p_Ptr, 
                             sizeof(char8)*XXX_PROJECTION_UNITS_SIZE);
		p_DDRControl->p_DDR[k].ProjectionUnits[XXX_PROJECTION_UNITS_SIZE] = '\0';
		p_Ptr += sizeof(char8)*XXX_PROJECTION_UNITS_SIZE;
		(void)memcpy(&p_DDRControl->p_DDR[k].ProjectionCoef, p_Ptr, 
                             sizeof(float64)*XXX_PROJECTION_COEF_SIZE);
		p_Ptr += sizeof(float64)*XXX_PROJECTION_COEF_SIZE;
		(void)memcpy(&p_DDRControl->p_DDR[k].UpperLeft, p_Ptr, sizeof(float64)*XXX_POINT_SIZE);
		p_Ptr += sizeof(float64)*XXX_POINT_SIZE;
		(void)memcpy(&p_DDRControl->p_DDR[k].LowerLeft, p_Ptr, sizeof(float64)*XXX_POINT_SIZE);
		p_Ptr += sizeof(float64)*XXX_POINT_SIZE;
		(void)memcpy(&p_DDRControl->p_DDR[k].UpperRight, p_Ptr, sizeof(float64)*XXX_POINT_SIZE);
		p_Ptr += sizeof(float64)*XXX_POINT_SIZE;
		(void)memcpy(&p_DDRControl->p_DDR[k].LowerRight, p_Ptr, sizeof(float64)*XXX_POINT_SIZE);
		p_Ptr += sizeof(float64)*XXX_POINT_SIZE;
		(void)memcpy(&p_DDRControl->p_DDR[k].ProjectionDistY, p_Ptr, sizeof(float64));
		p_Ptr += sizeof(float64);
		(void)memcpy(&p_DDRControl->p_DDR[k].ProjectionDistX, p_Ptr, sizeof(float64));
		p_Ptr += sizeof(float64);
		(void)memcpy(&p_DDRControl->p_DDR[k].LineSubsampleInc, p_Ptr, sizeof(float64));
		p_Ptr += sizeof(float64);
		(void)memcpy(&p_DDRControl->p_DDR[k].SampleSubsampleInc, p_Ptr, sizeof(float64));
		p_Ptr += sizeof(float64);
		(void)memcpy(p_DDRControl->p_DDR[k].SpaceCraftSource, p_Ptr, sizeof(char8)*XXX_SOURCE_SIZE);
		p_DDRControl->p_DDR[k].SpaceCraftSource[XXX_SOURCE_SIZE] = '\0';
		p_Ptr += sizeof(char8)*XXX_SOURCE_SIZE;
		(void)memcpy(p_DDRControl->p_DDR[k].InstrumentSource, p_Ptr, 
                             sizeof(char8)*XXX_INSTRUMENT_SIZE);
		p_DDRControl->p_DDR[k].InstrumentSource[XXX_INSTRUMENT_SIZE] = '\0';
		p_Ptr += sizeof(char8)*XXX_INSTRUMENT_SIZE;
		(void)memcpy(&p_DDRControl->p_DDR[k].WRSPath, p_Ptr, sizeof(uint8));
		p_Ptr += sizeof(uint8);
		(void)memcpy(&p_DDRControl->p_DDR[k].WRSRows, p_Ptr, sizeof(float64));
		p_Ptr += sizeof(float64);
		(void)memcpy(p_DDRControl->p_DDR[k].CaptureDirection, p_Ptr, sizeof(char8)*XXX_DIRECTION_SIZE);
		p_DDRControl->p_DDR[k].CaptureDirection[XXX_DIRECTION_SIZE] = '\0';
		p_Ptr += sizeof(char8)*XXX_DIRECTION_SIZE;
		(void)memcpy(p_DDRControl->p_DDR[k].CaptureDate, p_Ptr, sizeof(char8)*XXX_CAPTURE_DATE);
		p_DDRControl->p_DDR[k].CaptureDate[XXX_CAPTURE_DATE] = '\0';
		p_Ptr += sizeof(char8)*XXX_CAPTURE_DATE;
		(void)memcpy(p_DDRControl->p_DDR[k].CaptureTime, p_Ptr, sizeof(char8)*XXX_CAPTURE_TIME);
		p_DDRControl->p_DDR[k].CaptureTime[XXX_CAPTURE_TIME] = '\0';
		p_Ptr += sizeof(char8)*XXX_CAPTURE_TIME;
		(void)memcpy(&p_DDRControl->p_DDR[k].FieldValidityFlags, p_Ptr, sizeof(int8)*XXX_VALID_FLAGS);
		p_Ptr += sizeof(int8)*XXX_VALID_FLAGS;
		(void)memcpy(&p_DDRControl->p_DDR[k].CorrectionType, p_Ptr, sizeof(int8));
		p_Ptr += sizeof(int8);
	}

	VSdetach(vdata_id);
	Vend(file_id);
	if (Hclose(file_id) == FAIL)
	{
		(void)strcpy(p_ErrorMessage, "Can't close DDR file: ");
		p_HEstring = HEstring((hdf_err_code_t)HEvalue(error_top));
		(void)strcat(p_ErrorMessage, p_HEstring);
		goto ERROR_PROC;
	}
	file_id = 0;
	free(p_databuff);
	p_databuff = NULL;

	return p_DDRControl;

ERROR_PROC:
	VSdetach(vdata_id);
	Vend(file_id);
	(void)Hclose(file_id);

	free(p_databuff);
	free(p_DDRControl->p_DDR);
	free (p_DDRControl);

	return (NULL);
}
