/******************************************************************************
                Landsat 7 Image Assessment System (IAS) Software
                        Property of the U.S. Government
                            NASA / GSFC / Code 510
*******************************************************************************

UNIT NAME:  xxx_WriteDDR.c

PURPOSE:  Create a HDF DDR file and write the contains of xxx_DDRControl_TYPE.
	  into it.

INVOCATION METHOD:
	status = xxx_WriteDDR(p_PathName, p_DDRControl, p_ErrorMessage);

RETURN VALUE:
Type = int

Value           Description
--------------  ----------------------------------------------------------------
SUCCESS	        DDR file created
ERROR		Error creating/write the DDRs

ARGUMENT LIST:
Name             Type          	        Use	Description
---------------- --------------	 	---	---------------------------------------
p_PathName	char *	 	      	I	Pathname to DDR file
p_DDRControl    xxx_DDRControl_TYPE *   I	DDR information
p_ErrorMessage	char *		       	O	Error message
	 	
EXTERNAL FILE/RECORD/DBTABLE REFERENCES:
Name				Use	Description
-------------------------	---	-------------------------------------------------
DDR file                        I/O     HDF DDR file

EXTERNAL VARIABLES:
Source		Name				Type		Use	Description
---------------	----------------		---------------	---	-----------------------
hdf.h           FAIL                	 	int             I       HDF failure status
herr.h          error_top                       int32           I       Top to HDF error stack
xxx_DDR.h	XXX_DDR_FIELD_SIZE		int		I	Defined string size	
xxx_DDR.h	XXX_DATUM_CODE_SIZE		int		I	Projection units size	
xxx_DDR.h	XXX_PROJECTION_UNITS_SIZE	int		I	Projection units size	
xxx_DDR.h	XXX_PROJECTION_COEF_SIZE       	int		I	GCTP projection parameters  size
xxx_DDR.h	XXX_POINT_SIZE	                int		I	x,y point size	
xxx_DDR.h	XXX_SOURCE_SIZE			int		I       Spacecraft source size
xxx_DDR.h	XXX_INSTRUMENT_SIZE		int		I	Instrument source size	
xxx_DDR.h	XXX_DIRECTION_SIZE		int		I	Capture direction size
xxx_DDR.h	XXX_CAPTURE_DATE_SIZE		int		I	Capture date size
xxx_DDR.h	XXX_CAPTURE_TIME_SIZE		int		I	Capture time size
xxx_DDR.h	XXX_VALID_FLAGS_SIZE		int		I	Field validity flags size


EXTERNAL UNIT/MACRO/PROCEDURE/FUNCTION REFERENCES:
Name                       Description
---------------            -------------------------------------------------
Hclose	        	   Close HDF file
Hopen	        	   Open HDF file
HEstring        	   Returns error message for the error number
HEvalue                    Returns the current error number
Vend	        	   Close the vdata interface
Vstart	            	   Initialize the vdata interface
VSattach              	   Establish access to a specified vdata
VSdetach                   Detach from vdata
VSfdefine                  Define the  vdata field
VSwrite	                   Write vdata to file
VSsetfields        	   Pick fields to read
xxx_GetDataTypeSize        Get the data types byte size

ABNORMAL TERMINATION CONDITIONS, ERROR MESSAGES:
HDF errors
memory errors

ASSUMPTIONS, CONSTRAINTS, RESTRICTIONS:
none

DEVELOPMENT HISTORY:
Author                  Change-ID       Release Date            Description of Change
-----------------       -----------     ------- --------        ------------------------------
J. kerich               N/A             R1.0    03/19/97        Original Implementation
J. Kerich	        IASI970070      R2.0    10/28/97	Add path to open error message
J. kerich		IASI980293     	R3.0	02/12/98	Resize fields NumberLinesInScene,  
 		 		 		 		NumberSamplesInScene, MasterLine, and
 		 		 		 		MasterSample

NOTES:
none

Unit xxx_WriteDDR PDL:

        Call Hopen() to open p_PathName
1       IF error opening file THEN
                Call HEstring(HEvalue(error_top)) to build error message in p_ErrorMessage
                RETURN ERROR 
2       ENDIF

        Call Vstart() to initialize the vdata interface

	Call VSattach() to attach to the vdata
3       IF error attaching to vdata THEN
                Call HEstring(HEvalue(error_top)) to build error message in p_ErrorMessage
                ABORT to ERROR_PROC 
4       ENDIF

5	DOFOR all DDR fields
	        Call VSfdefine() to define a field
6               IF error defining field  THEN
                        Call HEstring(HEvalue(error_top)) to build error message in p_ErrorMessage
                        ABORT to ERROR_PROC 
7               ENDIF
8       ENDDO

	Allocate memory to write out all the rows in the DDR files
9       IF error allocating memory THEN
                Build error message in p_ErrorMessage
                ABORT to ERROR_PROC 
10      ENDIF

11	DOFOR all entries in p_DDRControl
		Break out the field from the DDR buffer and copy them into 
		  the xxx_DDR_TYPE structure array
12	ENDDO

	Call VSsetfields() to set what fields to write to
13      IF error setting fields  THEN
                Call HEstring(HEvalue(error_top)) to build error message in p_ErrorMessage
                ABORT to ERROR_PROC 
14      ENDIF

	Call VSwrite() to write all the data
15      IF error writing vdata THEN
                Call HEstring(HEvalue(error_top)) to build error message in p_ErrorMessage
                ABORT to ERROR_PROC 
16      ENDIF

	Free buffer memory

        Call VSdetach() to detach from vdata
	Call Vend() to close the vdata interface

	Call Hclose() to close DDR file
17      IF error closing HDF file THEN
                Call HEstring(HEvalue(error_top)) to build error message in p_ErrorMessage
                ABORT to ERROR_PROC 
18      ENDIF

	RETURN SUCCESS

ERROR_PROC:
	Call VSdetach() to detach from vdata
	Call Vend() to close the vdata interface
	Call Hclose() to close DDR file
	Free memory

	RETURN ERROR

End Unit xxx_WriteDDR
******************************************************************************/

/* System headers */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

/* Local headers */
#include <hdf.h>
#include <xxx_DDR.h>
#include <xxx_Types.h>
#include <xxx_Imagery.h>

extern int32 error_top;

int xxx_WriteDDR(char *p_PathName, xxx_DDRControl_TYPE *p_DDRControl, char *p_ErrorMessage)
{
	int   i;
	const char *p_HEstring;

	int32 file_id;
	int32 vdata_id;

	char FieldName[XXX_DDR_FIELD_SIZE];
	uint8 *p_buffer = NULL;
	uint8 *p_Ptr;
	long BufferLine = 0;

	xxx_DDRInfo_TYPE FieldInfo[] = {
		{"BandNumber", DFNT_INT8, 1},
		{"NumberScansInScene", DFNT_INT16, 1},
		{"NumberdetectorsPerScan", DFNT_INT16, 1},
		{"NumberLinesInScene", DFNT_INT32, 1},
		{"NumberSamplesInScene", DFNT_INT32, 1},
		{"DataType", DFNT_INT16, 1},
		{"MaxPixelValue", DFNT_FLOAT64, 1},
		{"MinPixelValue", DFNT_FLOAT64, 1},
		{"MaxRadianceValue", DFNT_FLOAT64, 1},
		{"MinRadianceValue", DFNT_FLOAT64, 1},
		{"MasterLine", DFNT_INT32, 1},
		{"MasterSample", DFNT_INT32, 1},
		{"ProjectCode", DFNT_INT16, 1},
		{"ZoneCode", DFNT_INT16, 1},
		{"DatumCode", DFNT_CHAR8, XXX_DATUM_CODE_SIZE},
		{"SpheroidCode", DFNT_INT32, 1},
		{"ProjectionUnits", DFNT_CHAR8, XXX_PROJECTION_UNITS_SIZE},
		{"ProjectionCoef", DFNT_FLOAT64, XXX_PROJECTION_COEF_SIZE},
		{"UpperLeft", DFNT_FLOAT64, XXX_POINT_SIZE},
		{"LowerLeft", DFNT_FLOAT64, XXX_POINT_SIZE},
		{"UpperRight", DFNT_FLOAT64, XXX_POINT_SIZE},
		{"LowerRight", DFNT_FLOAT64, XXX_POINT_SIZE},
		{"ProjectionDistY", DFNT_FLOAT64, 1},
		{"ProjectionDistX", DFNT_FLOAT64, 1},
		{"LineSubsampleInc", DFNT_FLOAT64, 1},
		{"SampleSubsampleInc", DFNT_FLOAT64, 1},
		{"SpaceCraftSource", DFNT_CHAR8, XXX_SOURCE_SIZE},
		{"InstrumentSource", DFNT_CHAR8, XXX_INSTRUMENT_SIZE},
		{"WRSPath", DFNT_UINT8, 1},
		{"WRSRows", DFNT_FLOAT64, 1},
		{"CaptureDirection", DFNT_CHAR8, XXX_DIRECTION_SIZE},
		{"CaptureDate", DFNT_CHAR8, XXX_CAPTURE_DATE},
		{"CaptureTime", DFNT_CHAR8, XXX_CAPTURE_TIME},
		{"FieldValidityFlags", DFNT_INT8, XXX_VALID_FLAGS},
		{"CorrectionType", DFNT_INT8, 1},
	};

	if ((file_id = Hopen(p_PathName, DFACC_CREATE, 0)) == FAIL)
	{
		(void)sprintf(p_ErrorMessage, "xxx_WriteDDR: Can't open DDR file %s (%s): ", p_PathName, getenv("PWD"));
		p_HEstring = HEstring((hdf_err_code_t)HEvalue(error_top));
		(void)strcat(p_ErrorMessage, p_HEstring);
		return(ERROR);
	}

	Vstart(file_id);

	if ((vdata_id = VSattach(file_id, -1, "w")) == FAIL)
	{
		(void)strcpy(p_ErrorMessage, "Error attaching to DDR file: ");
		p_HEstring = HEstring((hdf_err_code_t)HEvalue(error_top));
		(void)strcat(p_ErrorMessage, p_HEstring);
		goto ERROR_PROC;
	}

	(void)strcpy(FieldName, "");
	for (i=0; i < XXX_ARRAY_SIZE(FieldInfo); i++)
	{
		if (VSfdefine(vdata_id, FieldInfo[i].FieldName, FieldInfo[i].Datatype, FieldInfo[i].Order) == FAIL)
		{
			(void)strcpy(p_ErrorMessage, "Error defining  DDR fields: ");
			p_HEstring = HEstring((hdf_err_code_t)HEvalue(error_top));
			(void)strcat(p_ErrorMessage, p_HEstring);
			goto ERROR_PROC;
		}
		(void)strcat(FieldName, FieldInfo[i].FieldName);
		BufferLine += FieldInfo[i].Order * xxx_GetDataTypeSize(FieldInfo[i].Datatype);
		(void)strcat(FieldName, ",");
	}
	FieldName[strlen(FieldName)-1] = '\0';

	if ((p_buffer = calloc(p_DDRControl->Count, BufferLine)) == NULL)
	{
                (void)strcpy(p_ErrorMessage, "Can't calloc DDR buffer");
		goto ERROR_PROC;
	}

	p_Ptr = p_buffer;
	for (i=0; i < p_DDRControl->Count; i++)
	{
		(void)memcpy(p_Ptr, &p_DDRControl->p_DDR[i].BandNumber, sizeof(int8));
		p_Ptr += sizeof(int8);
		(void)memcpy(p_Ptr, &p_DDRControl->p_DDR[i].NumberScansInScene, sizeof(int16));
		p_Ptr += sizeof(int16);
		(void)memcpy(p_Ptr, &p_DDRControl->p_DDR[i].NumberdetectorsPerScan, sizeof(int16));
		p_Ptr += sizeof(int16);
		(void)memcpy(p_Ptr, &p_DDRControl->p_DDR[i].NumberLinesInScene, sizeof(int32));
		p_Ptr += sizeof(int32);
		(void)memcpy(p_Ptr, &p_DDRControl->p_DDR[i].NumberSamplesInScene, sizeof(int32));
		p_Ptr += sizeof(int32);
		(void)memcpy(p_Ptr, &p_DDRControl->p_DDR[i].DataType, sizeof(int16));
		p_Ptr += sizeof(int16);
		(void)memcpy(p_Ptr, &p_DDRControl->p_DDR[i].MaxPixelValue, sizeof(float64));
		p_Ptr += sizeof(float64);
		(void)memcpy(p_Ptr, &p_DDRControl->p_DDR[i].MinPixelValue, sizeof(float64));
		p_Ptr += sizeof(float64);
		(void)memcpy(p_Ptr, &p_DDRControl->p_DDR[i].MaxRadianceValue, sizeof(float64));
		p_Ptr += sizeof(float64);
		(void)memcpy(p_Ptr, &p_DDRControl->p_DDR[i].MinRadianceValue, sizeof(float64));
		p_Ptr += sizeof(float64);
		(void)memcpy(p_Ptr, &p_DDRControl->p_DDR[i].MasterLine, sizeof(int32));
		p_Ptr += sizeof(int32);
		(void)memcpy(p_Ptr, &p_DDRControl->p_DDR[i].MasterSample, sizeof(int32));
		p_Ptr += sizeof(int32);
		(void)memcpy(p_Ptr, &p_DDRControl->p_DDR[i].ProjectCode, sizeof(int16));
		p_Ptr += sizeof(int16);
		(void)memcpy(p_Ptr, &p_DDRControl->p_DDR[i].ZoneCode, sizeof(int16));
		p_Ptr += sizeof(int16);
		(void)strcpy((char *)p_Ptr, (char *)p_DDRControl->p_DDR[i].DatumCode);
		p_Ptr += sizeof(char8)*XXX_DATUM_CODE_SIZE;
		(void)memcpy(p_Ptr, &p_DDRControl->p_DDR[i].SpheroidCode, sizeof(int32));
		p_Ptr += sizeof(int32);
		(void)strcpy((char *)p_Ptr, (char *)p_DDRControl->p_DDR[i].ProjectionUnits);
		p_Ptr += sizeof(char8)*XXX_PROJECTION_UNITS_SIZE;
		(void)memcpy(p_Ptr, &p_DDRControl->p_DDR[i].ProjectionCoef, sizeof(float64)*XXX_PROJECTION_COEF_SIZE);
		p_Ptr += sizeof(float64)*XXX_PROJECTION_COEF_SIZE;
		(void)memcpy(p_Ptr, &p_DDRControl->p_DDR[i].UpperLeft, sizeof(float64)*XXX_POINT_SIZE);
		p_Ptr += sizeof(float64)*XXX_POINT_SIZE;
		(void)memcpy(p_Ptr, &p_DDRControl->p_DDR[i].LowerLeft, sizeof(float64)*XXX_POINT_SIZE);
		p_Ptr += sizeof(float64)*XXX_POINT_SIZE;
		(void)memcpy(p_Ptr, &p_DDRControl->p_DDR[i].UpperRight, sizeof(float64)*XXX_POINT_SIZE);
		p_Ptr += sizeof(float64)*XXX_POINT_SIZE;
		(void)memcpy(p_Ptr, &p_DDRControl->p_DDR[i].LowerRight, sizeof(float64)*XXX_POINT_SIZE);
		p_Ptr += sizeof(float64)*XXX_POINT_SIZE;
		(void)memcpy(p_Ptr, &p_DDRControl->p_DDR[i].ProjectionDistY, sizeof(float64));
		p_Ptr += sizeof(float64);
		(void)memcpy(p_Ptr, &p_DDRControl->p_DDR[i].ProjectionDistX, sizeof(float64));
		p_Ptr += sizeof(float64);
		(void)memcpy(p_Ptr, &p_DDRControl->p_DDR[i].LineSubsampleInc, sizeof(float64));
		p_Ptr += sizeof(float64);
		(void)memcpy(p_Ptr, &p_DDRControl->p_DDR[i].SampleSubsampleInc, sizeof(float64));
		p_Ptr += sizeof(float64);
		(void)strcpy((char *)p_Ptr, (char *)p_DDRControl->p_DDR[i].SpaceCraftSource);
		p_Ptr += sizeof(char8)*XXX_SOURCE_SIZE;
		(void)strcpy((char *)p_Ptr, (char *)p_DDRControl->p_DDR[i].InstrumentSource);
		p_Ptr += sizeof(char8)*XXX_INSTRUMENT_SIZE;
		(void)memcpy(p_Ptr, &p_DDRControl->p_DDR[i].WRSPath, sizeof(uint8));
		p_Ptr += sizeof(uint8);
		(void)memcpy(p_Ptr, &p_DDRControl->p_DDR[i].WRSRows, sizeof(float64));
		p_Ptr += sizeof(float64);
		(void)strcpy((char *)p_Ptr, (char *)p_DDRControl->p_DDR[i].CaptureDirection);
		p_Ptr += sizeof(char8)*XXX_DIRECTION_SIZE;
		(void)strcpy((char *)p_Ptr, (char *)p_DDRControl->p_DDR[i].CaptureDate);
		p_Ptr += sizeof(char8)*XXX_CAPTURE_DATE;
		(void)strcpy((char *)p_Ptr,(char *) p_DDRControl->p_DDR[i].CaptureTime);
		p_Ptr += sizeof(char8)*XXX_CAPTURE_TIME;
		(void)memcpy(p_Ptr, &p_DDRControl->p_DDR[i].FieldValidityFlags, sizeof(int8)*XXX_VALID_FLAGS);
		p_Ptr += sizeof(int8)*XXX_VALID_FLAGS;
		(void)memcpy(p_Ptr, &p_DDRControl->p_DDR[i].CorrectionType, sizeof(int8));
		p_Ptr += sizeof(int8);
	}

	if (VSsetfields(vdata_id, FieldName) == FAIL)
	{
		(void)strcpy(p_ErrorMessage, "Error setting DDR fields: ");
		p_HEstring = HEstring((hdf_err_code_t)HEvalue(error_top));
		(void)strcat(p_ErrorMessage, p_HEstring);
		goto ERROR_PROC;
	}

	if (VSwrite(vdata_id, p_buffer, p_DDRControl->Count, FULL_INTERLACE) == FAIL)
	{
		(void)sprintf(p_ErrorMessage, "Can't write DDR file (%s): ", getenv("PWD"));
		/* (void)strcpy(p_ErrorMessage, "Can't write DDR file: "); */
		p_HEstring = HEstring((hdf_err_code_t)HEvalue(error_top));
		(void)strcat(p_ErrorMessage, p_HEstring);
		goto ERROR_PROC;
	}
	free(p_buffer);

	VSsetname(vdata_id, p_DDRControl->vdataName);
	VSsetclass(vdata_id, p_DDRControl->vdataClass);
	VSdetach(vdata_id);

	Vend(file_id);

	if (Hclose(file_id) == FAIL)
	{
		(void)strcpy(p_ErrorMessage, "Can't close DDR file: ");
		p_HEstring = HEstring((hdf_err_code_t)HEvalue(error_top));
		(void)strcat(p_ErrorMessage, p_HEstring);
		return (ERROR);
	}

	return (SUCCESS);

ERROR_PROC:
	free(p_buffer);

	VSdetach(vdata_id);
	Vend(file_id);
	(void)Hclose(file_id);

	return (ERROR);
}
