/*******************************************************************************
NAME:		gxx_*_complex

PURPOSE:
Perform arithmetic on complex numbers

DEVELOPMENT HISTORY:
$Log: gxx_complex.c,v $
Revision 1.1.1.1  2000/10/19 14:37:35  morfitt
ram:  Inital EO1 processing system.

Revision 1.1.1.1  1999/12/01 19:01:20  mbs
Import of IAS 3.22

 * 
 *    Rev 1.7   Fri Jul 10 13:42:46 1998   vanderv
 * took out the consts
 * that caused 
 * warnings when compiling
 * 
 *    Rev 1.6   Fri Sep 26 07:11:06 1997   ias
 * No change
 * 
 *    Rev 1.5   Mon Aug 11 10:59:58 1997   ias
 * Updated prologs to be standard and added
 * prologs to routines within the same file.
 * Make some routine void since they were not
 * returning anything but SUCCESS
 * 
 *    Rev 1.4   Fri Aug 08 16:45:30 1997   ias
 * add const to input parameters
gxx_complex.c,v $
Revision 1.3  1997/05/22 19:57:05  rockvam
Changes to build gxxMath using gxxInclude files.


RETURN VALUE:
type = int (SUCESS, ERROR, or none)


ALGORITHM REFERENCES:

        "Numerical Recipes, the Art of Scientific Computing"
	by Press, Flannery, Teukolsky, and Vetterling
        Cambridge University Press, Second Edition

	"Calculus", Second Edition, Appendix III
	by Dennis D. Berkey
********************************************************************************
               Landsat 7 Image Assessment System (IAS) Software
                       Property of the U.S. Government
                               NASA/GSFC/EDC
*******************************************************************************/
#include "xxx_LogStatus.h"
#include "gxx_geo_math.h"

/******************************************************************************
NAME:		gxx_scalar_complex

PURPOSE:	Multiply the real and imaginary parts of the complex number
		by the scalar
******************************************************************************/
void gxx_scalar_complex(double scalar,    /* I: Number to multiply by        */
		  const COMPLEX *cpx_in,  /* I: Input complex number	     */ 
			COMPLEX *cpx_out) /* O: Result of multiplication     */
{
/*
  Multiply the real and imaginary parts of the complex number by the scalar
*/
cpx_out->re = scalar * cpx_in->re;
cpx_out->im = scalar * cpx_in->im;
}


/******************************************************************************
NAME:		gxx_add_complex

PURPOSE:	Add the real and imaginary parts of the complex numbers
******************************************************************************/
void gxx_add_complex(const COMPLEX *cpx_in1, /* I: First complex number	     */
	             const COMPLEX *cpx_in2, /* I: Second complex number     */
		           COMPLEX *cpx_out) /* O: Sum of cpx_in1 and cpx_in2 */
{
/*
  Add the real and imaginary parts of the complex numbers
*/
cpx_out->re = cpx_in1->re + cpx_in2->re;
cpx_out->im = cpx_in1->im + cpx_in2->im;
}


/******************************************************************************
NAME:		gxx_sub_complex

PURPOSE:	Subtract the real and imaginary parts of the complex numbers
******************************************************************************/
void gxx_sub_complex(const COMPLEX *cpx_in1, /* I: First complex number	     */
		     const COMPLEX *cpx_in2, /* I: Second complex number     */
		           COMPLEX *cpx_out) /* O: Result of subtraction     */
{
/*
  Subtract the real and imaginary parts of the complex numbers
*/
cpx_out->re = cpx_in1->re - cpx_in2->re;
cpx_out->im = cpx_in1->im - cpx_in2->im;
}


/******************************************************************************
NAME:		gxx_conj_complex

PURPOSE:	Do the conjugate of the complex number
******************************************************************************/
void gxx_conj_complex(const COMPLEX *cpx_in,  /* I: Input complex number     */
		            COMPLEX *cpx_out) /* O: Result from the conjugate*/
{
/*
  Do the conjugate of the complex number
*/
cpx_out->re =  cpx_in->re;
cpx_out->im = -cpx_in->im;
}


/******************************************************************************
NAME:		gxx_abs_complex

PURPOSE:	Get the absolute values of the real and imaginary parts
******************************************************************************/
void gxx_abs_complex(const COMPLEX *cpx_in, /* I: Input complex number	     */
		           double *value)   /* O: Resulting absolute value   */
{
double	re,			/* Absolute value of real part		     */
	im,			/* Absolute value of imaginary part	     */
	temp;			/* Temporary value for calculations	     */

/*
  Get the absolute values of the real and imaginary parts
*/
re = fabs(cpx_in->re);
im = fabs(cpx_in->im);

/*
  Determine the absolute value of the complex
*/
if (re == 0.0)
    *value = im;
else if (im == 0.0)
    *value = re;
else if (re > im)
    {
    temp = im / re;
    *value = re * sqrt(1.0 + temp * temp);
    }
else 
    {
    temp = re / im;
    *value = im * sqrt(1.0 + temp * temp);
    }
}


/******************************************************************************
NAME:		gxx_multiply_complex

PURPOSE:	Multiply the real and imaginary parts of the complex numbers
******************************************************************************/
void gxx_multiply_complex(const COMPLEX *cpx_in1, /* I: Input complex #1     */
			  const COMPLEX *cpx_in2, /* I: Input complex #2     */
			        COMPLEX *cpx_out) /* O: Product of #1 and #2 */
{
/*
  Multiply the real and imaginary parts of the complex numbers
*/
cpx_out->re = cpx_in1->re * cpx_in2->re - cpx_in1->im * cpx_in2->im;
cpx_out->im = cpx_in1->re * cpx_in2->im + cpx_in2->re * cpx_in1->im;
}


/******************************************************************************
NAME:		gxx_divide_complex

PURPOSE:	Divide the real and imaginary parts of the complex numbers

RETURN VALUE:
type = int (SUCESS, ERROR)
******************************************************************************/
int gxx_divide_complex(const COMPLEX *cpx_in1, /* I: Input complex #1	     */
		       const COMPLEX *cpx_in2, /* I: Input complex #2	     */
			     COMPLEX *cpx_out) /* O: Quotient of #1 and #2   */
{
double		temp,		/* temporary calculation		     */
		denom;		/* Denominator for the complex number	     */

/*
  Divide the real and imaginary parts of the complex numbers
*/
if (fabs(cpx_in2->re) >= fabs(cpx_in2->im))
    {
    /*
      Trap the divide by zero error
    */
    if (cpx_in2->re == 0)
	{
	xxx_LogStatus("GPSLIB",__FILE__,__LINE__,"Divide by zero error");
	return(ERROR);
	}

    temp = cpx_in2->im / cpx_in2->re;
    denom = cpx_in2->re + temp * cpx_in2->im;

    /*
      Trap the divide by zero error
    */
    if (denom == 0)
	{
	xxx_LogStatus("GPSLIB",__FILE__,__LINE__,"Divide by zero error");
	return(ERROR);
	}

    cpx_out->re = (cpx_in1->re + temp * cpx_in1->im) / denom;
    cpx_out->im = (cpx_in1->im - temp * cpx_in1->re) / denom;
    }
else
    {

    /*
      Trap the divide by zero error
    */
    if (cpx_in2->re == 0)
	{
	xxx_LogStatus("GPSLIB",__FILE__,__LINE__,"Divide by zero error");
	return(ERROR);
	}

    temp = cpx_in2->re / cpx_in2->im;
    denom = cpx_in2->re * temp + cpx_in2->im;

    /*
      Trap the divide by zero error
    */
    if (denom == 0)
	{
	xxx_LogStatus("GPSLIB",__FILE__,__LINE__,"Divide by zero error");
	return(ERROR);
	}

    cpx_out->re = (cpx_in1->re * temp + cpx_in1->im) / denom;
    cpx_out->im = (cpx_in1->im * temp - cpx_in1->re) / denom;
    }

return(SUCCESS);
}

/******************************************************************************
NAME:		gxx_sqrt_complex

PURPOSE:	Find the square root of a complex number

RETURN VALUE:
type = int (SUCCESS, ERROR)
******************************************************************************/
int gxx_sqrt_complex(const COMPLEX *cpx_in,  /* I: Input complex number     */
		            COMPLEX *cpx_out) /* O: Square root of input cpx */
{
double	re,			/* Absolute value of real input part	     */
	im,			/* Absolute value of imaginary input part    */
	temp1,			/* Temp varible for calculations	     */
	temp2;			/* Temp varible for calculations	     */

if ((cpx_in->re == 0.0) && (cpx_in->im == 0.0))
    {
    /*
      Nothing to do -- return
    */
    cpx_out->re = 0.0;
    cpx_out->im = 0.0;
    return(SUCCESS);
    }
else
    {
    re = fabs(cpx_in->re);
    im = fabs(cpx_in->im);
    if (re >= im)
	{
	/*
	  Trap the divide by zero error
	*/
	if (re == 0.0)
	    {
	    xxx_LogStatus("GPSLIB",__FILE__,__LINE__,"Divide by zero error");
	    return(ERROR);
	    }

	temp1 = im / re;
	temp2 = sqrt(re) * sqrt(0.5 * (1.0 + sqrt(1.0 + temp1 * temp1)));
	}
    else
	{
	/*
	  Trap the divide by zero error
	*/
	if (im == 0.0)
	    {
	    xxx_LogStatus("GPSLIB",__FILE__,__LINE__,"Divide by zero error");
	    return(ERROR);
	    }

	temp1 = re / im;
	temp2 = sqrt(im) * sqrt(0.5 * (re + sqrt(1.0 + temp1 * temp1)));
	}
    }

/*
  Trap the divide by zero error
*/
if (temp2 == 0.0)
    {
    xxx_LogStatus("GPSLIB",__FILE__,__LINE__,"Divide by zero error");
    return(ERROR);
    }

if (cpx_in->re >= 0.0)
    {
    cpx_out->re = temp2;
    cpx_out->im = cpx_in->im / (2.0 * temp2);
    }
else
    {
    if (cpx_in->im < 0.0)
	cpx_out->im = -temp2;
    else
	cpx_out->im = temp2;

    cpx_out->re = cpx_in->im / (2.0 * cpx_out->im);
    }

return(SUCCESS);
}
