/*****************************************************************************
NAME: 		gxx_corr_coeff

PURPOSE:
	Calculates the correlation coefficient between two data groups.


DEVELOPMENT HISTORY:
$Log: gxx_corr_coeff.c,v $
Revision 1.1.1.1  2000/10/19 14:37:35  morfitt
ram:  Inital EO1 processing system.

Revision 1.1.1.1  1999/12/01 19:01:20  mbs
Import of IAS 3.22

 * 
 *    Rev 1.7   Fri Mar 06 07:59:40 1998   tbeckman
 * Changed longs to ints to remove warnings from the -64 compile.
 * 
 *    Rev 1.6   Wed Sep 17 15:05:02 1997   tgardner
 * removed algorithm reference
 * modified check of nsamps to <= 0
 * now subtract one from nsamps in calculations
 * 
 *    Rev 1.5   Mon Sep 08 15:20:22 1997   tgardner
 * corrected prologue
 * 
 *    Rev 1.4   Tue Aug 26 09:43:04 1997   tgardner
 * added $Log: gxx_corr_coeff.c,v $
 * added Revision 1.1.1.1  2000/10/19 14:37:35  morfitt
 * added ram:  Inital EO1 processing system.
 * added
 * added Revision 1.1.1.1  1999/12/01 19:01:20  mbs
 * added Import of IAS 3.22
 * added to prologue and changed comments

RETURN VALUE:
Type = int
Value    Description
-----    -----------
SUCCESS  correlation coefficient was calculated.
ERROR    Sample size or standard deviation was zero.


ALGORITHM REFERENCES:
******************************************************************************
                Landsat 7 Image Assessment System (IAS) Software
                        Property of the U.S. Government
                                NASA/GSFC/EDC
*******************************************************************************/
#include "xxx_LogStatus.h"	/* xxx_LogStatus prototype */
#include "xxx_Types.h"		/* ERROR/SUCCESS defines */
#include "gxx_geo_math.h"	/* gxx_corr_coeff prototype */


int gxx_corr_coeff
(
    const double *x_sample,     /* I: pointer to the x data */
    const double *y_sample,     /* I: pointer to the y data */
    int  nsamps,               	/* I: number of samples in x and in y */
    double x_mean,             	/* I: mean of the x data */
    double y_mean,             	/* I: mean of the y data */
    double x_std_dev,          	/* I: standard deviation of the x data */
    double y_std_dev,          	/* I: standard deviation of the y data */
    double *corr_coeff 	   		/* O: correlation coefficient between 
                                      the x and y data */
)
{
    int  i;	       			/* loop variable */
    double xy_dev_sum=0.0;	/* product of x and y deviation from means */
    double covariance;		/* covariance between x and y */

/*---------------------------------------------------------------------
    Calculate the covaraince and then the correlation coefficient 
    between the two data groups.  Return ERROR if calculations will
    result in divide by zero or value is invalid. 
---------------------------------------------------------------------*/
    if (nsamps <= 0 || x_std_dev <= 0.0 || y_std_dev <= 0.0)
    {
        xxx_LogStatus("GPSLIB", __FILE__, __LINE__, 
                      "Sample size and/or std. deviation is invalid.");
		*corr_coeff = 0.0;
        return(ERROR);
    }

    for (i = 0; i < nsamps; i++)
        xy_dev_sum += (x_sample[i] - x_mean) * (y_sample[i] - y_mean);

    covariance = xy_dev_sum / (nsamps-1);

    *corr_coeff = covariance / (x_std_dev * y_std_dev);

    return(SUCCESS);
}
