/******************************************************************************
NAME: deg2meters

PURPOSE:
Convert a difference of distance in degrees to meters

DEVELOPMENT HISTORY:
AUTHOR	      Change-ID    DATE	    Description of Change
------	      ---------  --------   ---------------------
A.VanderVorste	1.0 	  04/97     Release 1.0 development.
                                    Algorithm from Chuck Wivell.
$Log: gxx_deg2meters.c,v $
Revision 1.1.1.1  2000/10/19 14:37:35  morfitt
ram:  Inital EO1 processing system.

Revision 1.1.1.1  1999/12/01 19:01:20  mbs
Import of IAS 3.22

 * 
 *    Rev 1.3   Wed Mar 04 09:43:44 1998   tbeckman
 * Removed an unused variable to eliminate a warning message.
 * 
 *    Rev 1.2   Wed Aug 20 10:25:40 1997   tgardner
 * corrected prologue, added I/O designation to
 * function arguments, and added divide by zero
 * checks.

RETURN VALUE:
Type = int
Value	Description
-----   -----------
SUCCESS	Sucessful completion
ERROR	Operation failed


NOTES:
LS7 will not cross the poles because: 1. the inclination angle is not
	large/high enough   2. the field of view is not wide enough
	to view the poles. 

ALGORITHM REFERENCES:
Spherical and Practical Astronomy as Applied to Geodesy, by Ivan I. Mueller,
        Fredirick Ungar Publishing Co., New York, 1969.

******************************************************************************
		Landsat 7 Image Assessment System (IAS) Software
		        Property of the U.S. Government
                                NASA/GSFC/EDC
******************************************************************************/
#include <math.h>
#include "xxx_Types.h"
#include "xxx_LogStatus.h"
#include "gxx_algorithm.h"
#include "gxx_const.h"


int gxx_deg2meters(
	double smajor,	/* I: semi-major axis				*/
	double sminor,  /* I: semi-minor axis				*/
	double blat,	/* I: beginning latitude in degrees		*/
	double blon,	/* I: beginning longitude in degrees		*/
	double elat,	/* I: ending latitude in degrees		*/
	double elon,	/* I: ending longitude in degrees		*/
	double *dx,	/* O: x difference in meters			*/
	double *dy)	/* O: y difference in meters			*/

{
double	a = 0.0,		/* semi-major axis 				*/
	asq = 0.0,		/* semi-major axis squared			*/
	b = 0.0,		/* semi-minor axis				*/
	bsq = 0.0,		/* semi-minor axis squared			*/
	denom = 0.0,		/* common calulation in rlat and rlon		*/
	diff_lat = 0.0,		/* difference in latitide			*/
	diff_lon = 0.0,		/* differenct in longitude			*/
	e = 0.0,		/* eccentricity of an ellipse			*/
	phi = 0.0,		/* geodetic latitude				*/
	rlat = 0.0,		/* curvature of the earth in meters
					(in latitude direction)			*/
	rblat = 0.0,		/* beginning latitude in radians		*/
	rlat_denom = 0.0,	/* denominator of the calculation for rlat	*/
	rlon = 0.0,		/* curvature of the earth in meters
					(in longitude direction)		*/
	rlon_denom = 0.0,	/* denominator of the calculation for rlon	*/
	sin_phi = 0.0;		/* sin of the angle phi				*/



/*
 Calculating the lat and long diffs
*/
gxx_latlondiffs(blat, blon, elat, elon, &diff_lat, &diff_lon);

/*
 Convert the lats and lons in degrees to radians
*/
rblat = blat * D2R; 
diff_lat = diff_lat * D2R;
diff_lon = diff_lon * D2R;

/*
 Calculating the ellipsoid information
*/
a = smajor;
b = sminor;
asq = a*a;
bsq = b*b;
e = sqrt((MAX(asq,bsq) - MIN(asq,bsq))/(asq));

/*
 Phi is the geodetic latitude (the one on a map - perpendicular  to the ellipsoid)
 We can assume that since a scene is small in comparison to the globe,
 then blat and elat are not far apart and we can just pick one to represent phi.
 I will pick blat.
*/
phi = blat * D2R; 

/*
 Calculating setup information
*/
sin_phi = sin(phi);
denom = 1-(e*e*sin_phi*sin_phi);

/*
 find the curvature of the earth in the longitude direction
*/
if (denom >= 0.0)
	rlon_denom = sqrt(denom);
else
    {
    xxx_LogStatus("GPSLIB", __FILE__, __LINE__,
	"Error: taking sqrt of a negative number");
    return(ERROR);
    }
if (rlon_denom == 0.0)
    {
    xxx_LogStatus("GPSLIB", __FILE__, __LINE__,
	"Error: next operation will result in divide by zero");
    return(ERROR);
    }
rlon = a/rlon_denom;

/*
 find the curvature of the earth in the latitude direction
*/
rlat_denom = pow((denom),3./2.);
if (rlat_denom == 0.0)
    {
    xxx_LogStatus("GPSLIB", __FILE__, __LINE__,
	"Error: next operation will result in divide by zero");
    return(ERROR);
    }
rlat = (a * (1-e*e))/rlat_denom; 
/*
 find the distance in meters for the difference in latitude
*/
*dy = rlat * diff_lat;

/*
 find the distance in meters for the difference in longitude
*/
*dx = rlon * diff_lon * cos(rblat);

return(SUCCESS);
}
