/******************************************************************************
NAME: 		xxx_earth2orbit

PURPOSE:	Compose the transformation matrix from the Earth-fixed 
		cartesian system to the orbit-oriented cartesian system.

PROGRAM HISTORY:
$Log: xxx_earth2orbit.c,v $
Revision 1.1  2000/11/22 14:21:56  storey
Original Version
Conversion of gxx_earth2orbit.c

VERSION	AUTHOR		DATE	REASON
-------	------		----	------
 1.0	D. Kuang	11/95	Original Development

RETURN VALUE:
Type = int
Value		Description
-----		-----------
SUCCESS		Successfully composed the transformation matrix
ERROR		No transformation was composed

ALGORITHM REFERENCES:

		C. Wivell, ATTITUDE in ELIPS;
		G. Rosborough, et al., IEEE 32.3, 1994; 
		Wz, unit vector in negative radial direction, pointing to the
		Earth center, if the satellite attitude is defined by star
		sensor like Landsat, then;
		Wz = - X / |X|;
		(if Wz to be defined as the negative ellipsoid normal
		direction by using horizon sensor, like AVHRR, then
		call cart2geod(X, ae, flat, &lat, &lon, &height)
		Wz = { -cos(lat)*cos(lon), -cos(lat)*sin(lon), -sin(lat) })
		Wy, unit vector in negative normal (- angular momentum);
		Wy = Wz cross V / |Wz cross V|;
		Wx, unit vector in transverse direction;
		Wx = Wy cross Wz;
		transf_matrix(1,1) = { Wx(1), Wx(2), Wx(3) };
		transf_matrix(2,1) = { Wy(1), Wy(2), Wy(3) };
		transf_matrix(3,1) = { Wz(1), Wz(2), Wz(3) }.

******************************************************************************/
#include <stdio.h>
#include "xxx_LogStatus.h"
#include "xxx_math.h"

int xxx_earth2orbit(
    VECTOR satpos, 	/* I: satellite position vector in Earth-fixed
				system, in meters */
    VECTOR satvel,	/* I: inertial satellite velocity vector in
				Earth-fixed system, in m/sec */
    double transf_matrix[3][3])	/* O: 3 X 3 transformation matrix, from the
				Earth-fixed system to the orbit-oriented
				cartesian system. */

{
  double magx = 0.0;		/* magnitude of vector	  	  */
  VECTOR Wz, Wy, Wx;		/* unit vector of the orbit system*/

/*
  Calculate Unit Z vector
*/
magx = sqrt(xxx_dot(&satpos, &satpos) );
/*
  Test for divide by zero error
*/
if (magx == 0.0)
    {
    xxx_LogStatus("XXXLIB", __FILE__, __LINE__,"Attempting to divide by zero");
    return(ERROR);
    }
Wz.x = -satpos.x /magx;
Wz.y = -satpos.y /magx;
Wz.z = -satpos.z /magx;

/*
  Calculate Unit Y vector
*/
xxx_cross(&Wz, &satvel, &Wy);
magx = sqrt(xxx_dot(&Wy, &Wy) );
/*
  Test for divide by zero error
*/
if (magx == 0.0)
    {
    xxx_LogStatus("XXXLIB", __FILE__, __LINE__,"Attempting to divide by zero");
    return(ERROR);
    }
Wy.x  = Wy.x /magx;
Wy.y  = Wy.y /magx;
Wy.z  = Wy.z /magx;
/*
  Calculate Unit X vector
*/

xxx_cross(&Wy, &Wz, &Wx);

/*
  Pack the unit vectors into the transf_matrix matrix
*/
transf_matrix[0][0] = Wx.x;
transf_matrix[1][0] = Wy.x;
transf_matrix[2][0] = Wz.x;
 
transf_matrix[0][1] = Wx.y;
transf_matrix[1][1] = Wy.y;
transf_matrix[2][1] = Wz.y;
 
transf_matrix[0][2] = Wx.z;
transf_matrix[1][2] = Wy.z;
transf_matrix[2][2] = Wz.z;

return(SUCCESS);
}
