/*******************************************************************************
NAME:	 	xxx_fft	

PURPOSE:	Replaces the "data" array by its discrete Fourier transform,
		if ISIGN = 1.  If ISIGN = -1, this routine performs the
		inverse discrete Fourier transform, multiplied by N.  "data"
		is a complex array of length N, with N an integer power of 2.

DEVELOPMENT HISTORY:
$Log: xxx_fft.c,v $
Revision 1.1  2000/11/22 21:21:39  storey
Original version - conversion from gxx_fft.c



RETURN VALUE:
Type = int
Value	     Description
-----        -----------
SUCCESS      Successful transformation
ERROR        Failure

ALGORITHM REFERENCES:

	"Numerical Recipes, the Art of Scientific
	Computing", by Press, Flannery, Teukolsky, and Vetterling,
	Cambridge University Press, 1986.

	In this code, sines and cosines are computed beforehand and stored
	in a static array.  Other optimizations are also present.
********************************************************************************
                       Property of the U.S. Government
                          USGS EROS Data Center
*******************************************************************************/
#include "xxx_LogStatus.h"
#include "xxx_math.h"

#define SWAP(a,b) tempr=(a);(a)=(b);(b)=tempr

int xxx_fft(COMPLEX *dat, /* I/O: points to input data which is replaced
                                  by its discrete Fourier transform or
                                  inverse discrete fourier transform */
	    int  n,	 /* I: length of the data array */
	    int  isign)	 /* I: type of operation (1=FT, -1=inverse FT) */
{
static double *wr=NULL, *wi=NULL; /* pointers to sin/cos table */
static int  wstore = 0;		/* stores the previous data length */
static int  isign_store = 0;	/* stores the previous operation flag */

int  mmax,m,j,istep,i,half_n,n_1,winc;
double *djre,*djim,*dire,*diim;
double *wrptr, *wiptr;
double wr_old, wi_old;
double tempr,tempi;
double wtemp,wpr,wpi,theta;
COMPLEX *data;

/* Setup sin/cos table if first time in (or if isign or n has changed)
  -------------------------------------------------------------------*/
if ((n != wstore) || (isign != isign_store))
    {
    if (wstore != 0)
	{
	free(wr);
	free(wi);
	}

    wstore = n;
    isign_store = isign;

    if (n == 0)			 /* Null transform */
	return(SUCCESS);

    mmax = n >> 1;
    wr = malloc(mmax * sizeof(double));
    wi = malloc(mmax * sizeof(double));
    if (!wr || !wi)
    {
        xxx_LogStatus("XXXLIB", __FILE__, __LINE__, 
                       "Error allocating memory for sin/cos table");
        free(wr);
        free(wi);
        return(ERROR);
    }

    wrptr = wr;
    wiptr = wi;

    /* Initialize for the trigonometric recurrence
       -------------------------------------------*/
   theta = PIE / (isign * mmax);
   wtemp = sin(0.5 * theta);
   wpr = -2.0 * wtemp * wtemp;
   wpi = sin(theta);

   *wrptr = 1.0;
   *wiptr = 0.0;
   for (m = 1; m < mmax; m++)
	{
	wr_old = *wrptr++;
	wi_old = *wiptr++;
	*wrptr = wr_old * wpr-wi_old * wpi+wr_old;
	*wiptr = wi_old * wpr+wr_old * wpi+wi_old;
	}
    }

data = dat;			/* Set pointer to input data */

/* Perform bit reversal
  --------------------*/
half_n = n >> 1;
n_1 = n - 1;
for (j = 0, i = 1; i < n_1; i++) 
    {
    m = half_n;
    while (m <= j)
	{
	j -= m;
	m >>= 1;
	}

    j += m;
    if (i > j)
	{
	SWAP(data[j].re,data[i].re);
	SWAP(data[j].im,data[i].im);
	}
    }

/* The Danielson-Lanczos section of the routine begins here.  The outter
   loop is executed log2NN times
  -----------------------------*/
winc = n >> 1;
mmax = 1;
while (n > mmax) 
    {
    istep = mmax<<1;
    wrptr = wr;			 /* Reset ptrs to top of W (twiddle factors) */
    wiptr = wi;

    for (m = 0; m < mmax; m++) 
	{
	/* The Danielson-Lanczos formula...
	   --------------------------------*/
	for (i = m; i < n; i += istep) 
	    {
	    j = i + mmax;
	    djre = &(data[j].re);
	    dire = &(data[i].re);
	    djim = &(data[j].im);
	    diim = &(data[i].im);
	    tempr = *wrptr * *djre - *wiptr * *djim;
	    tempi = *wrptr * *djim + *wiptr * *djre;
	    *djre = *dire - tempr;
	    *djim = *diim - tempi;
	    *dire += tempr;
	    *diim += tempi;
	    }

	wrptr += winc;
	wiptr += winc;
	}

    mmax = istep;
    winc >>= 1;
    }

return(SUCCESS);
}
