/****************************************************************************
FILE:		xxx_fft2d

PURPOSE:
Calculate a 2-d discrete Fourier Transform.  This file contains the following
functions:
	xxx_fft2d
	fftnd

*****************************************************************************/

/****************************************************************************
NAME:		xxx_fft2d

PURPOSE:
Calculate a 2-d discrete Fourier Transform

RETURN VALUE:
No return value

NOTES:
   FFT2D replaces DATA by its 2 dimensional discrete Fourier Transform,
   if ISIGN = 1.  Each dimension must be a power of 2.  DATA is an array of 
   type float, twice the length of the product of these lengths in which the 
   data are stored as in a multidimensional complex array:  real and imaginary 
   parts of each element are in consecutive locations, and the rightmost index 
   of the array increases most rapidly as one proceeds along DATA.  This is 
   equivalent to storing the array by rows (the C-way; FORTRAN stores by 
   columns!).  IF ISIGN = -1, DATA is replaced by its inverse transform times 
   the product of the lengths of all dimensions.

ALGORITHM REFERENCES:
   This routine was derrived from a multi-dimensional FFT in "Numerical 
   	Recipes in C", by Press, FLannery, Teukolsky, and Vetterling, 
	Cambridge University Press, 1988.

*****************************************************************************/
#include "xxx_math.h"
#include "xxx_correlation.h"

void xxx_fft2d(
  double *data,	/* I/O: array */
  int  nrows,	/* I: number of rows */
  int  ncols,	/* I: number of columns */
  int  isign)	/* I: Flag where 1 = 2 dimensional descrete Fourier Transform
				-1 = inverse transform times the product of
					the lengths of all dimensions */
{
int  dim[3];	/* dimensions of array, (1) Rows and (2) columns of array */

data--;		/* back up the pointer because it comes in as a 0-relative 
		   array, but the fft routines want to work with an array that 
		   starts at 1 */

if ((nrows == 0) || (ncols == 0)) return;
dim[1]=nrows;
dim[2]=ncols;
fftnd(data,dim,2,isign);	/* Call generic n-d FFT */
return;
}

/****************************************************************************
NAME:				FFTND

PURPOSE:  Generic N-d FFT function

RETURN VALUE:
No return value

NOTES:

ALGORITHM REFERENCES:
   This routine was derrived from a multi-dimensional FFT in "Numerical 
   	Recipes in C", Section 12.4, by Press, FLannery, Teukolsky, and 
	Vetterling, Cambridge University Press, 1988.
******************************************************************************/
void fftnd(
  double data[],	/* I/O: array */
  int  nn[],	/* I: dimensions of array, (1) Rows and (2) columns of array */
  int ndim,	/* I: array dimension */
  int isign)	/* I: Flag where 1 = 2 dimensional descrete Fourier Transform
				-1 = inverse transform times the product of
					the lengths of all dimensions */
{
int i1,i2,i3,i2rev,i3rev,ip1,ip2,ip3,ifp1,ifp2;	/* looping indices 	*/
int ibit,idim,k1,k2,n,nprev,nrem,ntot;	/* looping indices		*/
float tempi,tempr;			/* temporary real & imag values	*/
double theta,wi,wpi,wpr,wr,wtemp;	/* Double for trig recurrences */

ntot=1;
/* Calc no of complex vals
  -----------------------*/
for (idim=1;idim<=ndim;idim++)
    ntot *= nn[idim];
nprev=1;
for (idim=ndim;idim>=1;idim--) 
    {
    n=nn[idim];
    nrem=ntot/(n*nprev);
    ip1=nprev << 1;
    ip2=ip1*n;
    ip3=ip2*nrem;
    i2rev=1;
    /* Perform bit reversal
     --------------------*/
    for (i2=1;i2<=ip2;i2+=ip1)
	{
      	if (i2 < i2rev) 
            {
   	    for (i1=i2;i1<=i2+ip1-2;i1+=2) 
            	{
   	    	for (i3=i1;i3<=ip3;i3+=ip2) 
                    {
   	            i3rev=i2rev+i3-i2;
   	            SWAP(data[i3],data[i3rev]);
   	            SWAP(data[i3+1],data[i3rev+1]);
   	            }
   	 	}
   	    }
      	ibit=ip2 >> 1;
      	while (ibit >= ip1 && i2rev > ibit) 
            {
            i2rev -= ibit;
            ibit >>= 1;
            }
      	i2rev += ibit;
      	}
    /* Danielson-Lanczos section
      -------------------------*/
    ifp1=ip1;
    while (ifp1 < ip2) 
   	{
      	ifp2=ifp1 << 1;
	/* Init. trig recurrence
	  ---------------------*/
      	theta = isign * PIE2/(ifp2/ip1);
        wtemp = sin(0.5 * theta);
      	wpr = -2.0*wtemp*wtemp;
      	wpi=sin(theta);
      	wr=1.0;
	wi=0.0;
      	for (i3 = 1; i3 <= ifp1; i3 += ip1) 
            {
   	    for (i1 = i3; i1 <= i3 + ip1 - 2; i1 += 2) 
            	{
   	   	for (i2 = i1; i2 <= ip3; i2 += ifp2) 
               	    {
   	       	    k1=i2;
   	       	    k2=k1+ifp1;
   	       	    tempr = wr * data[k2] - wi * data[k2+1];
   	       	    tempi = wr * data[k2+1] + wi * data[k2];
   	       	    data[k2] = data[k1] - tempr;
   	       	    data[k2+1] = data[k1+1] - tempi;
   	       	    data[k1] += tempr;
   	       	    data[k1+1] += tempi;
   	       	    }
   	    	}
	    /* Trig recurrence
	     ----------------*/
	    wr += (wtemp = wr) * wpr - wi * wpi;
   	    wi += wi * wpr + wtemp * wpi;
   	    }
      	ifp1 = ifp2;
      	}
    nprev *= n;
    }
return;
}
#undef SWAP
