/****************************************************************************
FILE:           xxx_fitreg

PURPOSE:
Fits a quadratic surface to the neighborhood of the correlation peak and from it
determine the best-fit registration offsets and their estimated errors.  This 
file contains the following functions:
        xxx_fitreg
	sums
	invert
	lu_fact
	back_solve
	init_pivot
	find_pivot
	esterr

*****************************************************************************/

/*******************************************************************************
NAME:	xxx_fitreg

PURPOSE:  Fits a quadratic surface to the neighborhood of the correlation peak 
	  and from it determine the best-fit registration offsets and their 
          estimated errors

DEVELOPMENT HISTORY:
  1.0	 8/92	D. Steinwand 
  1.1	 8/96	M. Suarez	Modified 'wghts' & 'z' to be of dim 9 rather
				than 25
  1.2    4/97	S. Nelson	Updated to standards.  Added return status
				to invert and all functions that call invert

RETURN VALUE:
Type = int
Value           Description
-----           -----------
ERROR		Error fitting (couldn't invert matrix)
SUCCESS		Success

NOTES:

ALGORITHM REFERENCES:
1.0  LAS 4.0 GREYCORR & EDGECORR by R. White 6/83

*****************************************************************************/
#include "xxx_math.h"
#include "xxx_correlation.h"

#define GMAX(A, B)      ((A) > (B) ? (A) : (B)) /* assign maximum of a and b */
#define TINY 1.0e-20;

int xxx_fitreg(
  float *cpval,		/* I: 3 by 3 array of xcorr vals, in units of standard 
                              dev above background, centered on corr peak   */
  CorrFitMethod fit_method, /* I: Method of surface fit: 
  				FIT_ELLIP_PARA - Elliptical paraboloid
                                FIT_ELLIP_GAUSS - Elliptical Gaussian
                                FIT_RECIP_PARA - Reciprocal Paraboloid      */
  float *pkoffs,	/* O: Best-fit horiz and vertical offsets of correlation
                              peak relative to center of 3 by 3 array 	    */
  float *est_err)	/* O: Estimated horizontal error [0], vertical error [1]
  			      and h-v cross term [2] in best-fit offsets    */
{
int i, j;		/* Loop counters 				    */
float B[MAX_DIM][MAX_DIM];/* Matrix of sums of polynomial terms		    */
float A[MAX_DIM][MAX_DIM];/* Inverse of Matrix B			    */
double coeffs[MAX_DIM];	/* Coefs of best-fit quadratic surface 		    */
float vector[MAX_DIM];	/* Right side Col vector to solve for coefs 	    */
float wghts[9];		/* Weight assigned to each value in 3x3 area of peak*/
float z[9];		/* Xcorr coeffs to which 3x3 surface is to be fit   */
double denom;		/* Denominator of equation for best-fit offsets     */

/* Compute elements of matrix and column vector
  ---------------------------------------------*/
sums(cpval, fit_method, z, wghts, B, vector);

/* Invert matrix to get best-fit peak offsets
  -------------------------------------------*/
if (invert(B, A, MAX_DIM) == ERROR)
    {
    xxx_LogStatus("XXXLIB", __FILE__, __LINE__, 
    	"Error inverting matrix of polynomial terms");
    return(ERROR);
    }
for(i = 0; i < MAX_DIM; i++) 
    {
    for(coeffs[i] = 0.0, j = 0; j < MAX_DIM; j++)
      	coeffs[i] += A[i][j] * vector[j];
    }
denom = 4.0 * coeffs[3] * coeffs[5] - coeffs[4] * coeffs[4];
if (denom == 0.0)
    {
    pkoffs[0] = 0.0;
    pkoffs[1] = 0.0;
    est_err[0] = 0.0;
    est_err[1] = 0.0;
    est_err[2] = 0.0;
    }
else
    {
    pkoffs[0] = (coeffs[2] * coeffs[4] - 2.0 * coeffs[1] * coeffs[5]) / denom;
    pkoffs[1] = (coeffs[1] * coeffs[4] - 2.0 * coeffs[2] * coeffs[3]) / denom;

    /* Estimate errors in best-fit offsets
      ------------------------------------*/
    esterr(z, wghts, A, coeffs, pkoffs, est_err);
    }

return(SUCCESS);
}

/****************************************************************************
NAME:				SUMS

PURPOSE:  Compute the sums of terms for the matrix elements and column vector 
          used for determining the best-fit surface to the cross-correlation 
          values in the neighborhood of the peak

DEVELOPMENT HISTORY:
  1.0	 8/92	D. Steinwand 
  1.1	 8/96	M. Suarez	modified to use only 3x3 rather than 5x5

RETURN VALUE:
No return value

NOTES:

ALGORITHM REFERENCES:
*****************************************************************************/
void sums(
  float *cpval,		/* I: 3 by 3 array of xcorr values, in std dev 	   */
  CorrFitMethod fit_method, /* I: Method of fitting surface		   */
  float *z,		/* O: Function of xcorr to which quadratic is fit  */
  float *wghts,		/* O: Weights to be assigned to points in 3 by 3 array*/
  float b[MAX_DIM][MAX_DIM],/* O: Matrix of sums of polynomial terms 	   */
  float *vector)	/* O: Col vector obtained by summing products of poly 
                           terms and an approp. function of xcorr value    */
{
int i,ic,ir,j;		/* Loop counters 				   */
int ivalpt;		/* Index of xcorr vals and weights in peak area    */
float term[6];		/* Terms in approximating quadratic polynomial     */
float val;		/* Value of one element in CPVAL array 		   */
 
/* Initialize arrays and constants
  --------------------------------*/
for(i = 0; i < MAX_DIM; i++)
    {
    for(j = 0; j < MAX_DIM; j++)
	b[i][j] = 0.0;
    vector[i] = 0.0;
    }
 
/* Compute function of correlation coefficient and assign weight for each 
   location in neighborhood of peak
  ----------------------------------*/
term[0] = 1.0;
for(ivalpt = 0,ir = 0; ir < 3; ir++)
    for(ic = 0; ic < 3; ic++, ivalpt++)
      	{
      	val = GMAX(cpval[ivalpt], 1.0);
      	if (fit_method == FIT_ELLIP_PARA)
	    {
	    z[ivalpt] = val;
	    wghts[ivalpt] = 1.0;
	    }
      	else if (fit_method == FIT_ELLIP_GAUSS)
	    {
	    z[ivalpt] = -(log(val));
	    wghts[ivalpt] = val * val;
	    }
      	else /* fit_method == FIT_RECIP_PARA */
	    {
	    z[ivalpt] = 1.0/val;
	    wghts[ivalpt] = val * val * val * val;
	    }
 
	/* Compute matrix and vector elements in linear equations for best-fit 
	  -------------------------------------------------------------------*/
      	term[1] = (float)(ic - 1);  /*MJS 8/22/96 changed from ic-2 to ic-1*/
      	term[2] = (float)(ir - 1);  /*MJS 8/22/96 changed from ir-2 to ir-1*/
      	term[3] = term[1] * term[1];
      	term[4] = term[1] * term[2];
      	term[5] = term[2] * term[2];
      	for(i = 0; i < MAX_DIM; i++)
            {
            vector[i] += wghts[ivalpt] * term[i] * z[ivalpt];
            for(j=0; j<MAX_DIM; j++)
		b[i][j] += wghts[ivalpt] * term[i] * term[j];
            }
      	}
return;
}

/****************************************************************************

NAME:				INVERT			

PURPOSE:  Driver program for Gaussian Elimination (LU factorization) using 
          scaled column pivoting.  All arrays are zero relative--the first 
          element of matrix A is A[0][0].  Solves Ax = b where A is an n x n 
          matrix and x and b are n dimensional vectors.

DEVELOPMENT HISTORY:
  1.0	 8/92	D. Steinwand 

RETURN VALUE:
Type = int
Value           Description
-----           -----------
ERROR		Matrix not invertible
SUCCESS		Matrix inverted

NOTES:

ALGORITHM REFERENCES:
*****************************************************************************/
int invert(
  float (*matrix)[MAX_DIM],     /* I: Matrix A 			*/
  float (*outA)[MAX_DIM],      	/* O: Matrix A 			*/
  int n)			/* I: Dimension of matrix 	*/
{
int ipvt[MAX_DIM];		/* Array of pivots 		*/
float b[MAX_DIM];		/* b & resulting x vectors 	*/
int i,j;			/* Loop counter 		*/
int status;			/* Return status flag 		*/

/* Factor matrix 
  -------------*/
status = lu_fact(n, matrix, ipvt);
if (status == ERROR)
    {
    xxx_LogStatus("EXXLIB", __FILE__, __LINE__, "No solution--matrix not invertible");
    return(ERROR);
    }

/* If factored sucessfully, solve
  ------------------------------*/
for(i=0;i<n;i++)
    {
    for(j=0; j<n; j++)
	b[j]=0.0;
    b[i]=1.0;
    back_solve(n, matrix, ipvt, b);
    for(j=0; j<n; j++)
	outA[j][i]=b[j];
    }
return(SUCCESS);
}

/****************************************************************************
NAME:				LU_FACT

PURPOSE:  Routine to perfrom the LU factorization of a matrix.  Upon input,
	  "matrix" contains matrix A and upon output "matrix" contains A's
	  LU factorization.  The pivoting method used is scaled column
	  pivoting.

DEVELOPMENT HISTORY:
  1.0	 8/92	D. Steinwand 

RETURN VALUE:
Type = int
Value           Description
-----           -----------
ERROR		LU factorization not calculated
SUCCESS		Success

NOTES:

ALGORITHM REFERENCES:
******************************************************************************/
int lu_fact(
  int n,			/* I: Dimension of matrix 		*/
  float (*matrix)[MAX_DIM],	/* I/O: Matrix A 			*/
  int *ipvt)			/* O: Record of pivots performed 	*/
{
float s[MAX_DIM];	/* Vector containing abs() of max val for each row */
float factor;		/* Factor used in zeroing entries (stored) 	*/
int n_1;    		/* Parameter n minus 1 				*/
int i, j, k;		/* Loop counters 				*/
float temp;		/* Temporary variable for row swaping 		*/

if ((n <= 1) && (matrix[0][0] == 0.0))
    {
    xxx_LogStatus("XXXLIB", __FILE__, __LINE__, 
    	"Cannot calculate LU factorization of matrix");
    return(ERROR);
    }
n_1 = n - 1;

/* Initialize scaled column pivoting (calculate the s vector).
  -----------------------------------------------------------*/
if (init_pivot(n, matrix, s) == ERROR)
   {
   xxx_LogStatus("XXXLIB", __FILE__, __LINE__, "Cannot set up matrix pivot");
   return(ERROR);
   }

/* Factor the matrix
  -----------------*/
for (k = 0; k < n_1; k++)
   {
   ipvt[k] = find_pivot(n, matrix, k, s);

   /* Perform row interchange, if necessary
     -------------------------------------*/
   if (ipvt[k] != k)
      for (i = k; i < n; i++)
	{
	temp = matrix[k][i];
	matrix[k][i] = matrix[ipvt[k]][i];
	matrix[ipvt[k]][i] = temp;
	}
   if (matrix[k][k] == 0.0)
   	{
    	xxx_LogStatus("XXXLIB", __FILE__, __LINE__, 
    	    "Cannot calculate LU factorization of matrix");
	return(ERROR);
	}

   /* Factor the row
     --------------*/
   for (i = k + 1; i < n; i++)
      {
      factor = -matrix[i][k] / matrix[k][k];
      matrix[i][k] = factor;
      for (j = k + 1; j < n; j++)
	 matrix[i][j] += factor * matrix[k][j];
      }
   }
if (matrix[n_1][n_1] == 0.0)
   {
   xxx_LogStatus("XXXLIB", __FILE__, __LINE__, 
      "Cannot calculate LU factorization of matrix");
   return(ERROR);
   }

return(SUCCESS);
}

/******************************************************************************
NAME:					BACK_SOLVE

PURPOSE:  Routine to solve the LU factorized matrix A, given a vector b.
	  Upon output, vector b is really vector x.

DEVELOPMENT HISTORY:
  1.0	 8/92	D. Steinwand 

RETURN VALUE:
No return value

NOTES:

ALGORITHM REFERENCES:
******************************************************************************/

void back_solve(
  int  n,			/* I: dimension of matrix 		*/
  float (*matrix)[MAX_DIM],	/* I: LU factorization of matrix A 	*/
  int  *ipvt,			/* I: Record of pivots 			*/
  float *b)			/* I/O: b (in) & x (out) vector 	*/
{
int  k, j, i;			/* Loop counters 			*/
int  n_1;			/* Dimension of matrix - 1 		*/
int  kp1;			/* Var to reduce computations 		*/
float temp;			/* Temporary variable for interchanges 	*/

n_1 = n - 1;

/* Align vector b so that it corresponds to the row interchanges made during
   the factorization process.
  --------------------------*/
for (k = 0; k < n_1; k++)
    {
    if (k != ipvt[k])
	{
	temp = b[k];
	b[k] = b[ipvt[k]];
	b[ipvt[k]] = temp;
	}

     /* Solve Ly = b
       ------------*/
    for (i = k + 1; i < n; i++)
	b[i] = b[i] + matrix[i][k] * b[k];
    }

/* Backsolve--Solve Ux = y
  -----------------------*/
b[n_1] = b[n_1] / matrix[n_1][n_1];
for (k = n_1; k--;)
    {
    kp1 = k + 1;
    for (j = 0; j < kp1; j++)
	b[j] = b[j] - matrix[j][kp1] * b[kp1];
    b[k] = b[k] / matrix[k][k];
    }
return;

}

/******************************************************************************
NAME:					INIT_PIVOT

PURPOSE:  Scaled column pivoting.  The first routine (init_pivot) sets up the 
   	  s array.  The second routine (find_pivot) finds the pivot element and
   	  returns it to the caller.  These routines are isolated so that
	  different pivoting methods can be used with little re-coding
	  (although the s array may be unused).

DEVELOPMENT HISTORY:
  1.0	 8/92	D. Steinwand 

RETURN VALUE:
Type = int
Value           Description
-----           -----------
ERROR		Can't pivot
SUCCESS		s array set up

NOTES:

ALGORITHM REFERENCES:
******************************************************************************/
int init_pivot(
  int  n,			/* I: dimension of matrix */
  float (*matrix)[MAX_DIM],	/* I: matrix A 		*/
  float *s)			/* O: S array 		*/
{
int  i,j;		/* Loop counters */
float test;		/* Temporary variable for maximum test */

/* Find the value of the abs(largest) element in a row.
  ----------------------------------------------------*/
for (i = 0; i < n; i++)
    {
    for (s[i] = 0.0, j = 0; j < n; j++)
	{
	test = fabs(matrix[i][j]);
	if (test > s[i])
	    s[i] = test;
	}

    /* If s[i] is zero, all elements in the row are zero--something this routine
       won`t deal with...
      -----------------*/
    if (s[i] == 0.0)
	{
	xxx_LogStatus("XXXLIB", __FILE__, __LINE__, 
	    "Matrix row contains all zeroes");
	return(ERROR);
	}
   }
return(SUCCESS);
}
/******************************************************************************
NAME				FIND_PIVOT

DEVELOPMENT HISTORY:
  1.0	 8/92	D. Steinwand 

RETURN VALUE:
Type = int
Value           Description
-----           -----------
max_row		row with the maximum value

NOTES:

ALGORITHM REFERENCES:
******************************************************************************/
int find_pivot(
  int  n,			/* I: dimension of matrix 		*/
  float (*matrix)[MAX_DIM], 	/* I/O: matrix A 			*/
  int  k,			/* I: Current row (zero-relative) 	*/
  float *s)			/* I: S array 				*/
{
float max;		/* Maximum value found 				*/
int  j;			/* Loop counter 				*/
float test;		/* Temporary value used in max value test 	*/
int  max_row;		/* Row containing the maximum value 		*/

/* Find the row (zero relative) which have the largest value after being
   scaled with the corresponding element of the s array.
  -----------------------------------------------------*/
max = 0.0;
for (j = k; j < n; j++)
    {
    test = matrix[j][k] / s[j];
    test = fabs(test);
    if (test > max)
	{
	max = test;
	max_row = j;
	}
    }

return(max_row);
}

/****************************************************************************
NAME:				ESTERR

PURPOSE:  Estimate the probable errors in the best-fit location of the 
          correlation peak

DEVELOPMENT HISTORY:
  1.0	 8/92	D. Steinwand 

RETURN VALUE:
No return value

NOTES:

ALGORITHM REFERENCES:
*****************************************************************************/
void esterr(
  float *z,		/* I: Function of xcorr val to which quadratic is fit */
  float *wghts,		/* I: Weights for each point in 3x3 array of values  */
  float bnvrs[MAX_DIM][MAX_DIM],/* I: Inverse of matrix B		     */
  double *coeffs,	/* I: Coefs of best-fit quad surface to values of Z  */
  float *pkoffs,	/* I: Best-fit offset of peak to center of 3x3 array */
  float *est_err)	/* O: Est horiz & vert error & h-v cross term 	     */
{
int  i,	j, x, y;	/* Loop counters 				     */
int  ivalpt;		/* Index to values det from measured corr coeffs     */
float du[MAX_DIM];	/* Partial derivatives of horiz best-fit peak offset 
			   with respect to coefs of polynomial surface 	     */
float dv[MAX_DIM];	/* Partial derivatives of vertical offset 	     */
float c;		/* Constant for computing weights or errors 	     */
float denom;		/* Denominator used for partial derivatives 	     */
float f;		/* Value of function of corr coefficient computed 
                           using best-fit coefficients 			     */
float usum;		/* Accum. for sum of terms to determine est errors in 
			   horizontal offset 				     */
float var;		/* Residual variance of fit			     */
float vsum;		/* Sum of terms to estimate vertical offset error    */
float xsum;		/* Accum. for sum of cross terms to est errors when 
                           search chip was rotated 			     */

/* Compute residual variance for elements of peak array
  -----------------------------------------------------*/
ivalpt = 0;
var = 0.0;
for(y= -1; y <= 1; y++)
   for(x= -1; x <= 1; x++)
   {
   f = coeffs[0] + coeffs[1] * (float)x + coeffs[2] * (float)y + 
       coeffs[3] * (float)x * (float)x + coeffs[4] * (float)x * (float)y + 
       coeffs[5] * (float)y * (float)y;
   var += wghts[ivalpt] * (f - z[ivalpt]) * (f - z[ivalpt]);
   ivalpt++;
   }

/* Compute constant to use with weights
  -------------------------------------*/
c = var/19.0;

/* Compute partial derivatives of peak offsets with respect to poly coefs
  i.e., pkoffs[0] = (c2 * c4 - 2.0 * c1 * c5) / (4.0 * c3 * c5 - c4 * c4)
  so dp0/dc1 = -2.0 * c5 / (4.0 * c3 * c5 - c4 * c4)
        pkoffs[1] = (c1 * c4 - 2.0 * c2 * c3) / (4.0 * c3 * c5 - c4 * c4)
  so dp1/dc1 = c4 / (4.0 * c3 * c5 - c4 * c4)
  ----------------------------------------------------------------------*/
denom = 4.0 * coeffs[3] * coeffs[5] - coeffs[4] * coeffs[4];
du[0] = 0.0;
du[1] = -2.0 * coeffs[5]/denom;
du[2] = coeffs[4]/denom;
du[3] = -4.0 * coeffs[5] * pkoffs[0]/denom;
du[4] = (coeffs[2] + 2.0 * coeffs[4] * pkoffs[0])/denom;
du[5] = (-2.0 * coeffs[1] - 4.0 * coeffs[3] * pkoffs[0]) / denom;

dv[0] = 0.0;
dv[1] = du[2];
dv[2] = -2.0 * coeffs[3] / denom;
dv[3] = (-2.0*coeffs[2] - 4.0 * coeffs[5] * pkoffs[1])/denom;
dv[4] = (coeffs[1] + 2.0 * coeffs[4] * pkoffs[1])/denom;
dv[5] = -4.0 * coeffs[3] * pkoffs[1]/denom;

/* Compute estimated errors in offsets
  ------------------------------------*/
usum = vsum = xsum = 0.0;
for(i=0;i<MAX_DIM;i++)
    for(j=0;j<MAX_DIM;j++)
      	{
      	usum += du[i] * du[j] * bnvrs[i][j];
      	vsum += dv[i] * dv[j] * bnvrs[i][j];
      	xsum += du[i] * dv[j] * bnvrs[i][j];
      	}
est_err[0] = sqrt(fabs(c*usum));
est_err[1] = sqrt(fabs(c*vsum));
est_err[2] = c * xsum;

return;
}
