/****************************************************************************
NAME:				XXX_GREY_CROSS

PURPOSE:  Compute the unnormalized (raw) sum of pixel-by-pixel cross products 
	  between the reference and search images for every combination of 
          horizontal and vertical offsets of the reference relative to the 
          search image

RETURN VALUE:
Type = int
Value           Description
-----           -----------
ERROR		Error allocating memory for cross product calculation
SUCCESS		Calculated cross products

NOTES:

ALGORITHM REFERENCES:
1.  LAS 4.0 GREYCORR by R. White & G. Neal 8/83
*****************************************************************************/
#include "xxx_math.h"
#include "xxx_correlation.h"

int xxx_grey_cross(
  float *images, 	/* I: Search subimage */
  float *imager, 	/* I: Reference subimage */
  int *srch_size, 	/* I: Actual size of search subimage:  samps/lines */
  int *ref_size, 	/* I: Actual size of reference subimage: samps/lines */
  int ncol, 		/* I: # of columns in cross-product sums array(unormc)*/
  int nrow,	    	/* I: # of rows in cross-product sums array (unormc) */
  float *unormc) 	/* O: Array of unnormalized (raw) counts of edge 
                      		coincidences for each alignment of reference
				image relative to search image */
{
float *membfr;		/* Buffer in which reference subimage is properly 
                       aligned and zero padded 			  */
float *membfs;		/* Buffer for aligning and padding search subimage*/
int imgptr;		    /* Pointer into subimage arrays 		  */
int line;	    	/* Loop index:  current buffer line 		  */
int lncont; 		/* Pointer to start of next segment of line 	  */
int lnstrt;		    /* Pointer to first segment of line 		  */
int memptr;	    	/* Loop index:  pointer into AP transfer buffers  */
int ndxout; 		/* Pointer into array for correlation output 	  */
int nsnew[2];		/* Size of search window			  */
int nrnew[2];		/* Size of reference window			  */
int memdim[2];      /* Power-of-2 dimensions for AP arrays 		  */
int i,j;            /* Loop counters 				  */
float denom;		/* Denominator to calc unnormalized xcorr values  */
float tempim;		/* Temporary imaginary value when replacin cser	  */
COMPLEX *cser;    	/* Complex of search subimage 			  */
COMPLEX *cref;    	/* Complex of reference subimage 		  */

nsnew[0] = srch_size[0];
nsnew[1] = srch_size[1];
nrnew[0] = ref_size[0];
nrnew[1] = ref_size[1];

/* Zero extend search image to next higher power of 2
   Minimum window size is 64x64
  ----------------------------------------------------*/
for(i=0;i<2;i++)
    {
    memdim[i]=64;
    for(;;)
   	{
      	if (nsnew[i] <= memdim[i])
	    break;
      	memdim[i] *= 2;
      	}
    }

/* Fix for incorrect handling of non-square arrays
  ------------------------------------------------*/
memdim[0] = memdim[0] > memdim[1] ? memdim[0] : memdim[1];
memdim[1] = memdim[0];

membfr = (float *)malloc(memdim[0] * memdim[1] * sizeof(float));
if (membfr==NULL)
    {
    xxx_LogStatus("XXXLIB", __FILE__, __LINE__, "Error allocating memory");
    return(ERROR);
    }
membfs = (float *)malloc(memdim[0] * memdim[1] * sizeof(float));
if (membfs==NULL)
    {
    free(membfr);
    xxx_LogStatus("XXXLIB", __FILE__, __LINE__, "Error allocating memory");
    return(ERROR);
    }
cser = (COMPLEX *)malloc(memdim[0] * memdim[1] * sizeof(COMPLEX));
if (cser==NULL)
    {
    free(membfr);
    free(membfs);
    xxx_LogStatus("XXXLIB", __FILE__, __LINE__, "Error allocating memory");
    return(ERROR);
    }
cref = (COMPLEX *)malloc(memdim[0] * memdim[1] * sizeof(COMPLEX));
if (cref==NULL)
    {
    free(membfr);
    free(membfs);
    free(cser);
    xxx_LogStatus("XXXLIB", __FILE__, __LINE__, "Error allocating memory");
    return(ERROR);
    }

for(lnstrt = 0, imgptr = 0, line=0; line<memdim[1]; line++)
    {
    if (line < nsnew[1])
    	{
    	lncont = lnstrt + nsnew[0];
    	for(memptr=lnstrt; memptr<lncont; imgptr++, memptr++)
            membfs[memptr]=images[imgptr];
    	}
    else
	lncont = lnstrt;
    lnstrt += memdim[0];
    if (lncont<lnstrt)
    	for(memptr = lncont; memptr < lnstrt; memptr++)
	    membfs[memptr] = 0.0;
    }

/* Make complex 2-d of search data
   Note that the data are put into cser in column-major order since the 
   fft routine wants the data that way (see xxx_fft2d).
  ---------------------------------------------------------------------*/
for(memptr=0, i=0; i<memdim[1]; i++)
    for(j = 0; j < memdim[0]; j++, memptr++)
    	{
       	cser[j*memdim[1]+i].re = membfs[memptr];
       	cser[j*memdim[1]+i].im = 0.0;
       	}

/* Now zero extend reference image
  --------------------------------*/
for(lnstrt=0, imgptr=0, line=0; line<memdim[1]; line++)
    {
    if (line < nrnew[1]) 
    	{
    	lncont = lnstrt + nrnew[0];
    	for(memptr = lnstrt; memptr < lncont; imgptr++, memptr++)
	    membfr[memptr] = imager[imgptr];
    	}
    else
	lncont = lnstrt;
    lnstrt += memdim[0];
    if (lncont<lnstrt) 
      	for(memptr = lncont; memptr < lnstrt; memptr++)
	    membfr[memptr] = 0.0;
    }

/* Make complex 2-d of reference data
   Note that the data are put into cser in column-major order since the 
   fft routine wants the data that way (see xxx_fft2d).
  ---------------------------------------------------------------------*/
for(memptr=0, i=0; i<memdim[1]; i++)
    for(j=0; j<memdim[0]; j++, memptr++)
   	{
      	cref[j*memdim[1]+i].re = membfr[memptr];
      	cref[j*memdim[1]+i].im = 0.0;
      	}

/* Take fft of search and reference data
  --------------------------------------*/
xxx_fft2d((double *)cser,memdim[1],memdim[0],1);
xxx_fft2d((double *)cref,memdim[1],memdim[0],1);

/* Make point by multiplication of search subimage ft with conjugate of 
   reference subimage ft
  ----------------------*/
for(i=0; i<memdim[0]*memdim[1]; i++)
    {
    tempim = cref[i].re*cser[i].im-cref[i].im*cser[i].re;
    cser[i].re = cref[i].re*cser[i].re+cref[i].im*cser[i].im;
    cser[i].im = tempim;
    }

/* Take inverse fft of cser
  ------------------------*/
xxx_fft2d((double *)cser,memdim[1],memdim[0],-1);

/* Extract part of correlation array which is valid
  ------------------------------------------------*/
denom = (float)(memdim[0]*memdim[1]);
ndxout = 0;
for(i = 0; i < nrow; i++)
    for(j = 0; j < ncol; j++, ndxout++)
	unormc[ndxout] = cser[j*memdim[1]+i].re / denom;

free(membfr);
free(membfs);
free(cser);
free(cref);

return(SUCCESS);
}
