/******************************************************************************
PURPOSE:
The kalman filter process

DEVELOPMENT HISTORY:
$Log: xxx_kalman.c,v $
Revision 1.1  2000/11/22 21:26:12  storey
Original version - conversion from gxx_kalman.c


ROUTINES:
xxx_predict_state
xxx_kalman_gain
xxx_predict_error_covar
xxx_filter_state
xxx_filter_error_covar

NOTES:
  This file contains the matrix operations needed for Kalman 
  filtering.  For the notes that preceed each function the 
  following definitions are used.

  [X]  -> State matrix
  [S]  -> State transition matrix
  [P]  -> Error covaraince matrix  
  [Q]  -> Covariance matrix of state vector noise
  [R]  -> Covariance matrix of state vector noise
  [H]  -> Matrix relating state matrix to measurement vector
  [K]  -> Kalman gain matrix
  [Z]  -> Measurement matrix
  [I]  -> Identity matrix
  [_]  -> Predicted 
  [ ]' -> Transpose of matrix
  [ ]^ -> Inverse of matrix

ALGORITHM REFERENCES:
Introduction to Random Signal Analysis and Kalman Filtering, 
by Robert Grover Brown (p. 195)

******************************************************************************
		Landsat 7 Image Assessment System (IAS) Software
		        Property of the U.S. Government
                                NASA/GSFC/EDC
******************************************************************************/
#include <stdio.h>
#include <math.h>
#include <malloc.h>
#include "xxx_LogStatus.h"
#include "xxx_math.h"

/******************************************************************************
NAME: PREDICT_STATE

PURPOSE:
  Predict the state for k+1 given the state at k and the
  corresponding state transition matrix.  

   [X]k+1 = [S][X]k

RETURN VALUE:
Type = int
Value    Description
-----    -----------
SUCCESS  Successful completion
ERROR    Operation failed

NOTES:
******************************************************************************/

int xxx_predict_state(
    double *S, 	/* I: State transition matrix */
    double *Xk,	/* I: State at K */
    double *Xk1,/* O: State at K+1 */
    int m	/* I: size of matrix */
)
{
int status;

status = xxx_matrix_multiply( S, Xk, Xk1, m, m, m, 1 );
if (status != SUCCESS)
    {
    xxx_LogStatus("XXXLIB", __FILE__, __LINE__, 
        "Error returned from xxx_matrix_multiply");
    return(ERROR);
    }
return(SUCCESS);
}
/******************************************************************************
NAME: KALMAN_GAIN

PURPOSE:
  Calculate Kalman Gain matrix.
  
  [K] = [P_][H]'([H][P_][H]'+[R])^

RETURN VALUE:
Type = int
Value    Description
-----    -----------
SUCCESS  Successful completion
ERROR    Operation failed

NOTES:
******************************************************************************/

int xxx_kalman_gain(
    double *Pn,	/* I: Predicted error covariance matrix */
    double *H,	/* I: size of state matrix */
    double *R,	/* I: Covariance of measured noise matrix */
    double *K,	/* O: Kalman gain matrix */
    int m,	/* I: size in m direction */
    int n	/* I: size in n direction */
)
{
   double *t1, *t2, *t3, *Ht, *inv;
   int status;

   t1  = malloc( m * n * sizeof(double) );
   if (t1 == NULL)
 	{
	xxx_LogStatus("XXXLIB", __FILE__, __LINE__, 
            "Error allocating memory");
        return(ERROR);
	}
   t2  = malloc( n * n * sizeof(double) );
   if (t2 == NULL)
 	{
	xxx_LogStatus("XXXLIB", __FILE__, __LINE__, 
            "Error allocating memory");
	free(t1);
        return(ERROR);
	}
   inv = malloc( n * n * sizeof(double) );
   if (inv == NULL)
 	{
	xxx_LogStatus("XXXLIB", __FILE__, __LINE__, 
            "Error allocating memory");
	free(t1);
	free(t2);
        return(ERROR);
	}
   t3  = malloc( n * n * sizeof(double) );
   if (t3 == NULL)
 	{
	xxx_LogStatus("XXXLIB", __FILE__, __LINE__, 
            "Error allocating memory");
	free(t1);
	free(t2);
	free(inv);
        return(ERROR);
	}
   Ht  = malloc( m * n * sizeof(double) );
   if (Ht == NULL)
 	{
	xxx_LogStatus("XXXLIB", __FILE__, __LINE__, 
            "Error allocating memory");
	free(t1);
	free(t2);
	free(t3);
	free(inv);
        return(ERROR);
	}


   xxx_matrix_transpose( H, Ht, n, m );
   status = xxx_matrix_multiply( Pn, Ht, t1, m, m, m, n );
   if (status != SUCCESS)
    {
    xxx_LogStatus("XXXLIB", __FILE__, __LINE__, 
        "Error returned from xxx_matrix_multiply");
    free(t1);
    free(t2);
    free(t3);
    free(inv);
    free(Ht);
    return(ERROR);
    }
   status = xxx_matrix_multiply( H, t1, t2, n, m, m, n );
   if (status != SUCCESS)
    {
    xxx_LogStatus("XXXLIB", __FILE__, __LINE__, 
        "Error returned from xxx_matrix_multiply");
    free(t1);
    free(t2);
    free(t3);
    free(inv);
    free(Ht);
    return(ERROR);
    }
   xxx_matrix_add( t2, R, t3, n, n );
   status = xxx_matrix_inverse( t3, inv, n );
   if (status != SUCCESS)
    {
    xxx_LogStatus("XXXLIB", __FILE__, __LINE__, 
        "Error returned from xxx_matrix_inverse");
    free(t1);
    free(t2);
    free(t3);
    free(inv);
    free(Ht);
    return(ERROR);
    }
   status = xxx_matrix_multiply( t1, inv, K, m, n, n, n );
   if (status != SUCCESS)
    {
    xxx_LogStatus("XXXLIB", __FILE__, __LINE__, 
        "Error returned from xxx_matrix_multiply");
    free(t1);
    free(t2);
    free(t3);
    free(inv);
    free(Ht);
    return(ERROR);
    }

   free(t1);
   free(t2);
   free(t3);
   free(Ht);
   free(inv);
   return(SUCCESS);
}
/******************************************************************************
NAME: PREDICT_ERROR_COVAR

PURPOSE:
  Calculate the predicted state error covariance matrix.
  
  [P_] = [S][P][S]' + [Q]

RETURN VALUE:
Type = int
Value    Description
-----    -----------
SUCCESS  Successful completion
ERROR    Operation failed

NOTES:
******************************************************************************/
  
int xxx_predict_error_covar(
double *S,	/* I: State transition matrix */
double *Pn,	/* I: Filtered error covariance matrix */
double *Pn1,	/* O: Predicted error covariance matrix at k+1 */
double *Q,	/* I: Process noise matrix */
int m		/* I: size in m direction */
)
{
   double *t1, *t2, *St;
   int status; 

   t1 = malloc( m * m * sizeof(double) );
   if (t1 == NULL)
 	{
	xxx_LogStatus("XXXLIB", __FILE__, __LINE__, 
            "Error allocating memory");
        return(ERROR);
	}
   t2 = malloc( m * m * sizeof(double) );
   if (t2 == NULL)
 	{
	xxx_LogStatus("XXXLIB", __FILE__, __LINE__, 
            "Error allocating memory");
	free(t1);
        return(ERROR);
	}
   St = malloc( m * m * sizeof(double) );
   if (St == NULL)
 	{
	xxx_LogStatus("XXXLIB", __FILE__, __LINE__, 
            "Error allocating memory");
	free(t1);
	free(t2);
        return(ERROR);
	}

   xxx_matrix_transpose( S, St, m, m );
   status = xxx_matrix_multiply( S, Pn, t1, m, m, m, m );
   if (status != SUCCESS)
    {
    xxx_LogStatus("XXXLIB", __FILE__, __LINE__, 
        "Error returned from xxx_matrix_multiply");
	free(t1);
	free(t2);
	free(St);
    return(ERROR);
    }
   status = xxx_matrix_multiply( t1, St, t2, m, m, m, m );
   if (status != SUCCESS)
    {
    xxx_LogStatus("XXXLIB", __FILE__, __LINE__, 
        "Error returned from xxx_matrix_multiply");
	free(t1);
	free(t2);
	free(St);
    return(ERROR);
    }
   xxx_matrix_add( t2, Q, Pn1, m, m );

   free(t1);
   free(t2);
   free(St);
   return(SUCCESS);
}
/******************************************************************************
NAME: FILTER_STATE

PURPOSE:
Update estimate of state given new measruement.

  [X] = [X_] + [K]([Z] - [H][X_])

RETURN VALUE:
Type = int
Value    Description
-----    -----------
SUCCESS  Successful completion
ERROR    Operation failed

NOTES:
******************************************************************************/
int xxx_filter_state(
    double *Xk,	/* I: State matrix at time K */
    double *Xk1,/* O: State matrix at time K+1 */
    double *K,	/* I: Kalman gain matrix */
    double *z,	/* I: measure ment of gain matrix */
    double *H,	/* I: size of state matrix */
    int m,	/* I: size in m direction */
    int n	/* I: size in n direction */
)
{
   double *t1, *t2, *t3;
   int status;

   t1 = malloc( n * sizeof(double) );
   if (t1 == NULL)
 	{
	xxx_LogStatus("XXXLIB", __FILE__, __LINE__, 
            "Error allocating memory");
        return(ERROR);
	}
   t2 = malloc( n * sizeof(double) );
   if (t2 == NULL)
 	{
	xxx_LogStatus("XXXLIB", __FILE__, __LINE__, 
            "Error allocating memory");
	free(t1);
        return(ERROR);
	}
   t3 = malloc( m * sizeof(double) );
   if (t3 == NULL)
 	{
	xxx_LogStatus("XXXLIB", __FILE__, __LINE__, 
            "Error allocating memory");
	free(t1);
	free(t2);
        return(ERROR);
	}


   status = xxx_matrix_multiply( H, Xk, t1, n, m, m, 1 );
   if (status != SUCCESS)
    {
    xxx_LogStatus("XXXLIB", __FILE__, __LINE__, 
        "Error returned from xxx_matrix_multiply");
	free(t1);
	free(t2);
	free(t3);
    return(ERROR);
    }
   xxx_matrix_subtract( z, t1, t2, n, 1);
   status = xxx_matrix_multiply( K, t2, t3, m, n, n, 1 );
   if (status != SUCCESS)
    {
    xxx_LogStatus("XXXLIB", __FILE__, __LINE__, 
        "Error returned from xxx_matrix_multiply");
	free(t1);
	free(t2);
	free(t3);
    return(ERROR);
    }
   xxx_matrix_add( Xk, t3, Xk1, m, 1);

   free(t1);
   free(t2);
   free(t3);
   return(SUCCESS);
}
/******************************************************************************
NAME: FILTER_ERROR_COVAR

PURPOSE:
  Update error covariance matrix given update estimate
  of state.

  [P] = ([I] - [K][H])[P_]

RETURN VALUE:
Type = int
Value    Description
-----    -----------
SUCCESS  Successful completion
ERROR    Operation failed

NOTES:
******************************************************************************/
int xxx_filter_error_covar(
    double *K,	/* I: Kalman gain matrix */
    double *H,	/* I: size of state matrix */
    double *Pn,	/* I: Filtered error covariance matrix */
    double *Pn1,/* O: predicted error covariance matrix */
    int m,	/* I: size in m direction */
    int n	/* I: size in n direction */
)
{
   double *t1, *t2;
   int status;

   t1 = malloc( m * m * sizeof(double) );
   if (t1 == NULL)
 	{
	xxx_LogStatus("XXXLIB", __FILE__, __LINE__, 
            "Error allocating memory");
        return(ERROR);
	}
   t2 = malloc( m * m * sizeof(double) );
   if (t2 == NULL)
 	{
	xxx_LogStatus("XXXLIB", __FILE__, __LINE__, 
            "Error allocating memory");
	free(t1);
        return(ERROR);
	}


   status = xxx_matrix_multiply( K, H, t1, m, n, n, m );
   if (status != SUCCESS)
    {
    xxx_LogStatus("XXXLIB", __FILE__, __LINE__, 
        "Error returned from xxx_matrix_multiply");
	free(t1);
	free(t2);
    return(ERROR);
    }
   status = xxx_matrix_multiply( t1, Pn, t2, m, m, m, m );
   if (status != SUCCESS)
    {
    xxx_LogStatus("XXXLIB", __FILE__, __LINE__, 
        "Error returned from xxx_matrix_multiply");
	free(t1);
	free(t2);
    return(ERROR);
    }
   xxx_matrix_subtract( Pn, t2, Pn1, m, m );

   free(t1);
   free(t2);
   return(SUCCESS);
}
