/* ***************************************************************************
PURPOSE:
matrix routines

ROUTINES:
xxx_matrix_transpose
xxx_matrix_multiply
xxx_matrix_inverse
xxx_matrix_add
xxx_matrix_subtract
xxx_ludcmp
xxx_lubksb

DEVELOPMENT HISTORY:
$Log: xxx_matrix.c,v $
Revision 1.1  2000/11/22 21:26:14  storey
Original version - conversion from gxx_matrix.c


RETURN VALUE:
NONE
Type = void

ALGORITHM REFERENCES:
******************************************************************************
                        Property of the U.S. Government
                           USGS EROS Data Center
*******************************************************************************/

/* ***************************************************************************
NAME: TRANSPOSE

PURPOSE:
transposes a matrix

RETURN VALUE:
NONE
Type = void

ALGORITHM REFERENCES:
******************************************************************************
                        Property of the U.S. Government
                            USGS EROS Data Center
*******************************************************************************/
#include <stdio.h>
#include <math.h>
#include <malloc.h>
#include "xxx_Types.h"
#include "xxx_LogStatus.h"
#include "xxx_math.h"

#define TINY	1.0e-20	/* to avoid dividing by zero */

void xxx_matrix_transpose( 
	const double *a, 	/* I: matrix to be transposed */
	double *b, 		/* O: transpose of matrix */
	const int row, 	/* I: number of rows in matrix */
	const int col )	/* I: number of columns in matrix */
{
int i,j; /* loop variables */
 
/* transpose the matrix */
for( i=0 ; i<row ; i++ )
    for( j=0 ; j<col ; j++ )
        b[j*row+i]=a[i*col+j];
}


/* ***************************************************************************
NAME: MATRIX MULTIPLY

PURPOSE:
multipiles two matrices

RETURN VALUE:
Type = int
Value    Description
-----    -----------
SUCCESS  Successful completion 
ERROR    Operation failed

ALGORITHM REFERENCES: None
******************************************************************************
                        Property of the U.S. Government
                            USGS EROS Data Center
*******************************************************************************/
int xxx_matrix_multiply( 
	const double *a,	/* I: ptr to matrix */
	const double *b, 	/* I: ptr to matrix */
	double *m, 		/* O: ptr to output matrix */
	const int arow, 	/* I: number of rows in a */
	const int acol, 	/* I: number of cols in a */
	const int brow, 	/* I: number of rows in b */
	const int bcol) 	/* I: number of cols in b */
{

int j, rr, cc;		/* loop vars */
double sum;		/* sum of matrix elements */


/* see if the matrices are compatible to multiply */
if (acol != brow)
    {
    xxx_LogStatus("GPSLIB", __FILE__, __LINE__, 
	"Matrices are incompatible, unable to multiply");
    return(ERROR); 
    }

/* the matrix multiplication */
for( rr=0 ; rr<arow ; rr++ )
    {
    for( cc=0 ; cc<bcol ; cc++ )
        {
        sum = 0.0;
        for( j=0 ; j<acol ; j++ )
            sum = sum + a[rr*acol+j]*b[j*bcol+cc];
        m[rr*bcol+cc] = sum;
        }
    }

return(SUCCESS);
}

/* ***************************************************************************
NAME: MATRIX INVERSE

PURPOSE:
inverts a matrix

RETURN VALUE:
Type = int
Value    Description
-----    -----------
SUCCESS  Successful completion 
ERROR    Operation failed

ALGORITHM REFERENCES: None
******************************************************************************
                        Property of the U.S. Government
                            USGS EROS Data Center
*******************************************************************************/
int xxx_matrix_inverse(
    const double *a,	/* I: input array */
    double *y,  	/* O: ouput array */
    const int n 	/* I: dimension of a (n y n) */
)
{
   double d, 	/* flag from ludcmp */
	*col,   /* array of columns */
	*mat;   /* temporary matrix */
   int i,j,*indx, status;

   col=(double *)malloc( n*sizeof(double) );
   if( col==NULL )
   {
    xxx_LogStatus("GPSLIB", __FILE__, __LINE__, 
	"Error allocating array");
    return(ERROR); 
   }
   indx=malloc( n*sizeof(int) );
   if( indx==NULL )
   {
    xxx_LogStatus("GPSLIB", __FILE__, __LINE__, 
	"Error allocating array");
    free(col);
    return(ERROR);
   }
   mat=(double *)malloc( n*n*sizeof(double) );
   if( indx==NULL )
   {
    xxx_LogStatus("GPSLIB", __FILE__, __LINE__, 
	"Error allocating array");
    free(col);
    free(indx);
    return(ERROR);
   }

   /* making a copy of the input array */
   for( i=0 ; i<n*n ; i++ )
      mat[i] = a[i];
 
   status = xxx_ludcmp( mat,n,indx,&d );
   if (status != SUCCESS)
	{
        xxx_LogStatus("GPSLIB", __FILE__, __LINE__, 
	    "Error returned from xxx_ludcmp");
        return(ERROR); 
	}
   for( j=0 ;j<n ; j++ )
   {
      for( i=0 ; i<n ; i++ )
         col[i]=0.0;
      col[j]=1.0;
      xxx_lubksb( mat,n,indx,col );
      for( i=0 ; i<n ; i++ )
      {
         y[i*n+j]=col[i];
      }
   }

   free(col);
   free(indx);
   free(mat);
   return(SUCCESS);
} 

/* ***************************************************************************
NAME: MATRIX ADD

PURPOSE:
adds two matrices

RETURN VALUE:
Type = none

ALGORITHM REFERENCES: None
******************************************************************************
                        Property of the U.S. Government
                            USGS EROS Data Center
*******************************************************************************/
void xxx_matrix_add( 
    const double *a,	/* I: input matrix 1 */
    const double *b,  	/* I: input matrix 2 */
    double *m,  	/* O: output matrix */
    const int row, 	/* I: number of rows in matricies */
    const int col	/* I: number of cols in matricies */
)
{
 
   int rr, cc; /* loop vars */
 
   for( rr=0 ; rr<row ; rr++ )
      for( cc=0 ; cc<col ; cc++ )
        m[rr*col+cc] = a[rr*col+cc] + b[rr*col+cc];
 
   return;
}

/* ***************************************************************************
NAME: MATRIX SUBTRACT

PURPOSE:
subtracts two matrices

RETURN VALUE:
Type = void

ALGORITHM REFERENCES: None
******************************************************************************
                        Property of the U.S. Government
                            USGS EROS Data Center
*******************************************************************************/
void xxx_matrix_subtract(
    const double *a, 	/* I: input matrix 1 */
    const double *b, 	/* I: input matrix 2 */
    double *m, 		/* O: output matrix */
    const int row,  	/* I: number of rows in matrix */
    const int col   	/* I: number of cols in matrix */
)
{
 
   int rr, cc;
 
   for( rr=0 ; rr<row ; rr++ )
      for( cc=0 ; cc<col ; cc++ )
         m[rr*col+cc] = a[rr*col+cc] - b[rr*col+cc];
 
   return;
}

/* ***************************************************************************
NAME: LUDCMP

PURPOSE:
performs lu decomposition on a matrix

RETURN VALUE:
Type = int
Value    Description
-----    -----------
SUCCESS  Successful completion 
ERROR    Operation failed

ALGORITHM REFERENCES: None
******************************************************************************
                        Property of the U.S. Government
                            USGS EROS Data Center
*******************************************************************************/
int xxx_ludcmp(
    double *a, 		/* IO: input matrix, output LU decomp matrix */
    const int n,    	/* I: size of matrix */
    int *indx,		/* O: index */
    double *d  		/* O: flag */
)
{
   int i,imax,j,k;
   double big,dum,sum,temp;
   double *vv;

   vv = (double *)malloc( sizeof(double) * n );

   *d = 1.0;
   for( i=0 ; i<n ; i++ )
   {
      big = 0.0;
      for( j=0 ; j<n ; j++ )
         if( (temp=fabs(a[i*n+j])) > big )    
            big = temp;
     if( big==0.0 )
	{
        xxx_LogStatus("GPSLIB", __FILE__, __LINE__, "Singular matrix");
        return(ERROR); 
        }
     vv[i] = 1.0/big;
   }
   for( j=0 ; j<n ; j++ )
   {
      for( i=0 ; i<j ; i++ )
      {
         sum=a[i*n+j];
         for( k=0 ; k<i ; k++ )
            sum-=a[i*n+k]*a[k*n+j];
         a[i*n+j]=sum;
      }
      big=0.0;
      for( i=j ; i<n ; i++ )
      {
          sum=a[i*n+j];
         for( k=0 ; k<j ; k++ )
            sum-=a[i*n+k]*a[k*n+j];
         a[i*n+j]=sum;
         if( (dum=vv[i]*fabs(sum) ) >= big )
         {
            big=dum;
            imax=i;
         }
      }
      if( j!=imax )
      {
         for( k=0 ; k<n ; k++ )
         {
            dum=a[imax*n+k];
            a[imax*n+k]=a[j*n+k];
            a[j*n+k]=dum; 
         }
         *d = - (*d);
         vv[imax]=vv[j];
      }
      indx[j]=imax;
      if( a[j*n+j]==0.0 )
         a[j*n+j]=TINY;
      if( j!=(n-1) )
      {
         dum=1.0/a[j*n+j];
         for( i=j+1 ; i<n ; i++ )
            a[i*n+j] *= dum;
      }
   }
   free(vv);
   return(SUCCESS);
}
/* ***************************************************************************
NAME: 

PURPOSE:
performs back substitution on a lu matrix

RETURN VALUE:
Type = none

ALGORITHM REFERENCES: None
******************************************************************************
                        Property of the U.S. Government
                            USGS EROS Data Center
*******************************************************************************/
void xxx_lubksb(
    double *a, 	/* IO: lu decomp matrix */
    const int n,    	/* I: size of matrix */
    const int *indx,	/* I: flag */
    double b[] 		/* O: solution vector */
)
{
   int i,ii=0,ip,j;
   double sum;

   for( i=1 ; i<=n ; i++ )
   {
      ip=indx[i-1];
      sum=b[ip];
      b[ip]=b[i-1];
      if( ii )
         for( j=(ii-1) ; j<i-1 ; j++ )
            sum-=a[(i-1)*n+j]*b[j];
      else if( sum ) 
         ii=i;
      b[i-1]=sum;
   }
   for( i=(n-1) ; i>=0 ; i-- )
   {
      sum=b[i];
      for( j=i+1 ; j<n ; j++ )
         sum-=a[i*n+j]*b[j];
      if (a[i*n+i] == 0.0)
	  a[i*n+i] = TINY;	
      b[i]=sum/a[i*n+i];
   }
   return;
}
