/******************************************************************************
NAME:           xxx_parallel_corr

PURPOSE:        
The xxx_parallel_corr module implements a parallel correlation object.  Using 
the Posix threading interface, up to MAX_CORR_THREADS (or the number of 
processors available - whichever is less) are created to perform correlation.
The main thread that creates the parallel correlator is then responsible for
"feeding" the parallel correlator chips to correlate.

ROUTINES:
    xxx_init_parallel_correlator
    xxx_get_corr_chip_buffers
    xxx_submit_chip_to_corr
    xxx_close_parallel_correlator


NOTES:
- To use a parallel correlator, the application must first initialize it
  using the xxx_init_parallel_correlator function.  Each of the other
  routines require the pointer to the structure returned by the init routine
  as a parameter.
- The application must load the chips to correlate into the buffers returned by 
  xxx_get_corr_chip_buffers.  There are a limited number of chip buffers so if
  there are none available, the main application thread is put to sleep until
  a set of chip buffers is made available by the correlating threads.
- The xxx_submit_chip_to_corr places the chips into a queue.  The correlation
  threads remove the chips from the queue and perform the correlation.  The
  results of the correlation are not immediately available to the application 
  since xxx_submit_chip_to_corr returns before the correlation is complete.
- Before any of the correlation results are used, the application must call
  xxx_close_parallel_correlator to make sure all the chips have been correlated
  and to destroy the correlation threads.
- This module needs to be compiled with XXX_THREAD defined to enable 
  multithreading.  If XXX_THREAD is not defined, the module is compiled to 
  not use posix threading (i.e. does not use multiprocessing).  XXX_THREAD
  is used to compile the desired version.
- When small chips are being correlated it is very likely that the job will
  become I/O bound instead of CPU bound.
- It is assumed that a single thread will be getting chip buffers (i.e. calling
  xxx_get_corr_chip_buffers) and submitting chips to correlate (i.e. calling
  xxx_submit_chip_to_corr).  If multiple threads are going to be doing this,
  a new mutexs will be needed to serialize removing chips from the chip queue
  and adding chips to the correlate queue.

ALGORITHM REFERENCES:
None

******************************************************************************
                Landsat 7 Image Assessment System (IAS) Software
                        Property of the U.S. Government
                                NASA/GSFC/EDC
******************************************************************************/

/*
**
** INCLUDE FILES 
**
*/

#ifdef XXX_THREAD
#include <unistd.h>   /* sysconf prototype */
#endif
#include <stdlib.h>   /* malloc/free prototypes */
#include <stdio.h>    /* sprintf prototype */
#include "xxx_Types.h" /* SUCCESS/ERROR definitions */
#include "xxx_LogStatus.h" /* xxx_LogStatus prototype */
#include "xxx_parallel_corr.h" /* prototypes for this module */
#include "xxx_correlation.h" /* prototypes for this module */
#include "xxx_math.h" /* xxx_grey_corr prototype */


/*
**
** LOCAL PROTOTYPES
**
*/

#ifdef XXX_THREAD
static void *xxx_correlate_thread(void *params_ptr);
/* prototype for the correlation thread function */
#endif

/******************************************************************************
FUNCTION NAME:	xxx_init_parallel_correlator

PURPOSE:        
xxx_init_parallel_correlator initializes an instance of the parallel correlator.
All the multiprocessing resources are created and the memory for the chip buffers
and queue structures is allocated.

RETURNS:
A pointer to the parallel correlator state structure is returned.  If an error
occurs, an error message is generated and NULL is returned.

NOTES:
- A successfully created parallel correlator must be destroyed using the 
  xxx_close_parallel_correlator routine.
- The calling application must allocate and free the memory for the 
  result_ptr.

ALGORITHM REFERENCES:
none

******************************************************************************/

XXX_PARALLEL_CORRELATOR_TYPE *xxx_init_parallel_correlator
(
    int check_edge_flag,  /* I: set to non-zero to check the corr edge flag */
    int check_mult_peak_flag, /* I: set to non-zero to check multiple peak flag*/
    int check_low_peak_flag, /* I: set to non-zero to check low peak flag */
    int check_max_disp_flag, /* I: set to non-zero to check max disp flag */
    int max_ref_chip_lines,  /* I: max lines in reference chip */
    int max_ref_chip_samples, /* I: max samples across reference chip */
    int max_search_chip_lines, /* I: max lines in search chip */
    int max_search_chip_samples, /* I: max samples across search chip */
    XXX_CORRELATION_RESULT_TYPE *results_ptr /* I: pointer to results array */
)
{
    XXX_PARALLEL_CORRELATOR_TYPE *correlator_ptr; /* pointer to the parallel 
                                    correlator data structure */
#ifdef XXX_THREAD
    long processors; /* number of processors on the machine */
    int counter;     /* loop counter */
    int count;     	 /* # of chars returned from sprintf */
    char msg[STRLEN]; /* message buffer */
#endif
    int ref_chip_size;   /* size of a reference chip (lines * samples) */
    int search_chip_size; /* size of a search chip (lines * samples) */
    
    /* allocate memory for the parallel correlator structure */
    correlator_ptr = malloc(sizeof(XXX_PARALLEL_CORRELATOR_TYPE));
    if (correlator_ptr == NULL)
    {
        /* couldn't allocate memory for the parallel correlator */
        xxx_LogStatus("XXXLIB",__FILE__,__LINE__,
                      "Error allocating memory for the parallel correlator");
        return (NULL);
    }

    /* save the flags that need to be checked for each correlation point */
    correlator_ptr->check_edge_flag = check_edge_flag;
    correlator_ptr->check_mult_peak_flag = check_mult_peak_flag;
    correlator_ptr->check_low_peak_flag = check_low_peak_flag;
    correlator_ptr->check_max_disp_flag = check_max_disp_flag;

    /* save the pointer to the results array */
    correlator_ptr->results_ptr = results_ptr;
    
    /* save the maximum sizes of the search and reference chips */
    correlator_ptr->max_ref_chip_lines = max_ref_chip_lines;
    correlator_ptr->max_ref_chip_samples = max_ref_chip_samples;
    correlator_ptr->max_search_chip_lines = max_search_chip_lines;
    correlator_ptr->max_search_chip_samples = max_search_chip_samples;

    /* calculate the size of the reference and search chips */
    ref_chip_size = max_ref_chip_lines * max_ref_chip_samples;
    search_chip_size = max_search_chip_lines * max_search_chip_samples;
    
#ifdef XXX_THREAD
    /* clear the error flag */
    correlator_ptr->error_flag = 0;
    
    /* determine the number of processors available on the machine */
    processors = sysconf(_SC_NPROC_ONLN);
   
    /* create one less correlation thread than there are processors on the
       machine (but at least one) to allow other processes some time.  Also,
       limit the maximum to MAX_CORR_THREADS. */
    if (processors < 1)
    {
        processors = 1;
    }
    else if (processors > MAX_CORR_THREADS)
    {
        processors = MAX_CORR_THREADS;
    }
    correlator_ptr->threads = (int)processors;
    
    /* initialize the free chip buffer queue */
    correlator_ptr->chip_buffer_queue.insert_index = 0;
    correlator_ptr->chip_buffer_queue.remove_index = 0;
    
    /* allocate memory for twice as many chip buffers as there are threads to 
       allow the main thread to keep some queued up */
    correlator_ptr->chip_buffer_queue.queue_size = 2 * correlator_ptr->threads;

    /* allocate the free chip buffer queue */
    correlator_ptr->chip_buffer_queue.data = malloc(correlator_ptr->
            chip_buffer_queue.queue_size * sizeof(CHIP_BUFFER_DATA_TYPE));
    if (correlator_ptr->chip_buffer_queue.data == NULL)
    {
        xxx_LogStatus("XXXLIB",__FILE__,__LINE__,
                      "Error allocating memory for the free CHIP queue");
        free (correlator_ptr);
        return(NULL);
    }
    
    /* allocate the free chip buffers */
    correlator_ptr->chip_buffer_queue.buffers = 
        malloc(correlator_ptr->chip_buffer_queue.queue_size *
               (ref_chip_size + search_chip_size) * sizeof(float));
    if (correlator_ptr->chip_buffer_queue.buffers == NULL)
    {
        xxx_LogStatus("XXXLIB",__FILE__,__LINE__,
                      "Error allocating memory for the chip buffers");
        free (correlator_ptr->chip_buffer_queue.data);
        free (correlator_ptr);
        return(NULL);
    }
    
    /* split the hunk of memory allocated for chip buffers into individual
       buffers */
    for (counter = 0; counter < correlator_ptr->chip_buffer_queue.queue_size;
         counter++)
    {
        correlator_ptr->chip_buffer_queue.data[counter].ref_chip_ptr =
            &correlator_ptr->chip_buffer_queue.buffers[counter * 
                         (ref_chip_size + search_chip_size)];
        correlator_ptr->chip_buffer_queue.data[counter].search_chip_ptr =
            &correlator_ptr->chip_buffer_queue.data[counter].ref_chip_ptr[
                ref_chip_size];
            
    }
    
    /* initialize the correlate queue */
    correlator_ptr->correlate_queue.insert_index = 0;
    correlator_ptr->correlate_queue.remove_index = 0;

    /* size the queue a little larger than really needed so no special checks
       are needed to prevent queue overflow */
    correlator_ptr->correlate_queue.queue_size = 
            correlator_ptr->chip_buffer_queue.queue_size + 2;
            
    /* allocate memory for the correlation queue data */
    correlator_ptr->correlate_queue.data = malloc (sizeof(CORRELATE_DATA_TYPE) *
                correlator_ptr->correlate_queue.queue_size);
    if (correlator_ptr->correlate_queue.data == NULL)
    {
        xxx_LogStatus("XXXLIB",__FILE__,__LINE__,
                      "Error allocating memory for the correlate queue");
        free (correlator_ptr->chip_buffer_queue.buffers);
        free (correlator_ptr->chip_buffer_queue.data);
        free (correlator_ptr);
        return(NULL);     
    }

    /* create the free chip queue semaphore initialized to the entries in the
       queue since it is filled already */
    if (sem_init(&correlator_ptr->chip_buffer_queue.semaphore,0,
                 correlator_ptr->chip_buffer_queue.queue_size) != 0)
    {
        xxx_LogStatus("XXXLIB",__FILE__,__LINE__,
                      "Error initializing free chip buffer queue semaphore");
        free (correlator_ptr->correlate_queue.data);
        free (correlator_ptr->chip_buffer_queue.buffers);
        free (correlator_ptr->chip_buffer_queue.data);
        free (correlator_ptr);
        return(NULL);   
    }

    /* create the correlation queue semaphore initialized to zero since there
       is no data entered into it yet */
    if (sem_init(&correlator_ptr->correlate_queue.semaphore, 0, 0) != 0)
    {
        xxx_LogStatus("XXXLIB",__FILE__,__LINE__,
                      "Error initializing correlator queue semaphore");
        sem_destroy(&correlator_ptr->chip_buffer_queue.semaphore);
        free (correlator_ptr->correlate_queue.data);
        free (correlator_ptr->chip_buffer_queue.buffers);
        free (correlator_ptr->chip_buffer_queue.data);
        free (correlator_ptr);
        return(NULL);         
    }

    /* create the free chip queue mutex */
    if (pthread_mutex_init(&correlator_ptr->chip_buffer_queue.insert_mutex,NULL)
         != 0)
    {
        xxx_LogStatus("XXXLIB",__FILE__,__LINE__,
                      "Error initializing free chip buffer queue mutex");
        sem_destroy(&correlator_ptr->correlate_queue.semaphore);
        sem_destroy(&correlator_ptr->chip_buffer_queue.semaphore);
        free (correlator_ptr->correlate_queue.data);
        free (correlator_ptr->chip_buffer_queue.buffers);
        free (correlator_ptr->chip_buffer_queue.data);
        free (correlator_ptr);
        return(NULL);    
    }

    /* create the correlation queue mutex */
    if (pthread_mutex_init(&correlator_ptr->correlate_queue.remove_mutex,NULL)
         != 0)
    {
        xxx_LogStatus("XXXLIB",__FILE__,__LINE__,
                      "Error initializing correlator queue mutex");
        pthread_mutex_destroy(&correlator_ptr->correlate_queue.remove_mutex);
        sem_destroy(&correlator_ptr->correlate_queue.semaphore);
        sem_destroy(&correlator_ptr->chip_buffer_queue.semaphore);
        free (correlator_ptr->correlate_queue.data);
        free (correlator_ptr->chip_buffer_queue.buffers);
        free (correlator_ptr->chip_buffer_queue.data);
        free (correlator_ptr);
        return(NULL);        
    }


    /* create the correlation threads */
    for (counter = 0; counter < correlator_ptr->threads; counter++)
    {
        if (pthread_create(&correlator_ptr->thread_ids[counter],NULL,
                           xxx_correlate_thread, correlator_ptr) != 0)
        {
            if (counter == 0)
            {
                /* no correlation threads created so return an error */
                xxx_LogStatus("XXXLIB",__FILE__,__LINE__,
                              "Error creating correlation threads");
                pthread_mutex_destroy(
                        &correlator_ptr->correlate_queue.remove_mutex);
                pthread_mutex_destroy(
                        &correlator_ptr->correlate_queue.remove_mutex);
                sem_destroy(&correlator_ptr->correlate_queue.semaphore);
                sem_destroy(&correlator_ptr->chip_buffer_queue.semaphore);
                free (correlator_ptr->correlate_queue.data);
                free (correlator_ptr->chip_buffer_queue.buffers);
                free (correlator_ptr->chip_buffer_queue.data);
                free (correlator_ptr);
                return(NULL);   
            }

            /* at least one correlation thread was created so issue a 
               warning and continue with execution */
            count = sprintf(msg,"Warning: a thread creation error occurred - only "
                        "%d correlation threads created",counter);
			if (count < 1)
				{
            	/* error in sprintf */
            	xxx_LogStatus("XXXLIB",__FILE__,__LINE__,"Error in sprintf");
                free (correlator_ptr->correlate_queue.data);
                free (correlator_ptr->chip_buffer_queue.buffers);
                free (correlator_ptr->chip_buffer_queue.data);
                free (correlator_ptr);
                return(NULL);   
				}
                        
            /* error creating thread, kill the threads that were created */
            xxx_LogStatus("XXXLIB",__FILE__,__LINE__,msg);

            /* save the number of threads created */
            correlator_ptr->threads = counter;
        }
    }

#else
    /* allocate the chip buffers for single threaded execution */
    correlator_ptr->ref_chip_ptr = malloc(ref_chip_size * sizeof(float));
    if (correlator_ptr->ref_chip_ptr == NULL)
    {
        xxx_LogStatus("XXXLIB",__FILE__,__LINE__,
                      "Error allocating memory for the reference chip buffer");
        free (correlator_ptr);
        return(NULL);
    }
    
    correlator_ptr->search_chip_ptr = malloc(search_chip_size * sizeof(float));
    if (correlator_ptr->search_chip_ptr == NULL)
    {
        xxx_LogStatus("XXXLIB",__FILE__,__LINE__,
                      "Error allocating memory for the search chip buffer");
        free (correlator_ptr->ref_chip_ptr);
        free (correlator_ptr);
        return(NULL);
    }
   
#endif
    /* return the correlator pointer */
    return (correlator_ptr);
}

/******************************************************************************
FUNCTION NAME:	xxx_get_corr_chip_buffers

PURPOSE:        
xxx_get_corr_chip_buffers returns buffers for the search and reference chips 
that will be submitted to the parallel correlator.

RETURNS:
Pointers to the reference chip buffer and search chip buffer are returned. 
A SUCCESS/ERROR flag is also returned.

NOTES:
- The buffers returned are meant to be submitted to the parallel correlator.
  Getting buffers from this routine and not submitting them to the parallel
  correlator will quickly exhaust all the buffers available.
- The buffers will be free'd when the parallel correlator is closed.
- When compiled in single threaded mode, the same set of buffers are used
  for every pair of chips.

ALGORITHM REFERENCES:
none

******************************************************************************/

int xxx_get_corr_chip_buffers
(
    XXX_PARALLEL_CORRELATOR_TYPE *correlator_ptr, /* I: parallel correlator */
    float **ref_buffer_ptr, /* O: pointer to reference chip buffer */
    float **search_buffer_ptr /* O: pointer to search chip buffer */
)
{
#ifdef XXX_THREAD
    int local_remove_index;  /* a local copy of the free buffer remove index */

    /* wait for buffers to be available in the queue */
    if (sem_wait(&correlator_ptr->chip_buffer_queue.semaphore) != 0)
    {
        /* error waiting for free buffer semaphore */
        xxx_LogStatus("XXXLIB",__FILE__,__LINE__,
                      "Error encountered waiting for the free buffer semaphore");
        return (ERROR);
    }

    /* if an error has occurred in any of the correlation threads, abort this
       routine since the application should be shutting down. */
    if (correlator_ptr->error_flag)
    {
        xxx_LogStatus("XXXLIB",__FILE__,__LINE__,
                      "Error reported by a correlation thread");
        return (ERROR);
    }

    /* get a local copy of the remove index */
    local_remove_index = correlator_ptr->chip_buffer_queue.remove_index;
    
    /* get the buffer pointers to return */
    *ref_buffer_ptr = correlator_ptr->chip_buffer_queue.
                        data[local_remove_index].ref_chip_ptr;
    *search_buffer_ptr = correlator_ptr->chip_buffer_queue.
                            data[local_remove_index].search_chip_ptr;

    /* adjust the remove index */
    local_remove_index++;
    if (local_remove_index >= correlator_ptr->chip_buffer_queue.queue_size)
        local_remove_index = 0;
    correlator_ptr->chip_buffer_queue.remove_index = local_remove_index;
#else
    /* compiled for single threading so use the constant buffers */
    *ref_buffer_ptr = correlator_ptr->ref_chip_ptr;
    *search_buffer_ptr = correlator_ptr->search_chip_ptr;
#endif

    return (SUCCESS);
}

/******************************************************************************
FUNCTION NAME:	xxx_submit_chip_to_corr

PURPOSE:        
xxx_submit_chip_to_corr places chips to correlate into the correlation queue.
The chips will be removed from the queue by the correlation threads and 
correlated in the order they are put in.

RETURNS:
A SUCCESS/ERROR flag is returned.

NOTES:
- There is a section of test code that may be uncommented to give an 
  indication of how the application is keeping up with the parallel
  correlator.
- When compiled for single threading, this routine actually calls the 
  correlation routine directly.

ALGORITHM REFERENCES:
none

******************************************************************************/

int xxx_submit_chip_to_corr
(
    XXX_PARALLEL_CORRELATOR_TYPE *correlator_ptr, /* I: parallel correlator */
    int chip_index,          /* I: index to store results at in results array */
    float *search_image_ptr, /* I: search image */
    float *ref_image_ptr,    /* I: reference image */
    int search_size[2],      /* I: size of search image (samp,line) */
    int ref_size[2],         /* I: size of reference image (samp,line) */
    float min_corr,          /* I: minimum acceptable correlation strength */
    CorrFitMethod fit_method,/* I: fit method to use */
    float max_disp,          /* I: maximum allowed diagnol displacement */
    float nominal_offset[2], /* I: nominal offset of UL corner of ref to search
                                   (samp,line) */
    int abs_corr_coeff_flag  /* I: flag to use the absolute of the correlation
                                   coefficients */    
)
{
#ifdef XXX_THREAD
    int local_insert_index; /* local copy of the insert index */
    CORRELATE_DATA_TYPE *q_ptr; /* pointer to current queue location */

    /*-------------------------------------------------------------------*/
    /* UNCOMMENT this code to provide information on how many chips are
       queued up for correlation.  If the number printed out is 1 or
       below, the application is probably not feeding chips to the
       correlator to keep it busy all the time. */
    /*local_insert_index = correlator_ptr->correlate_queue.insert_index -
                         correlator_ptr->correlate_queue.remove_index;
    if (local_insert_index < 0)
        local_insert_index += correlator_ptr->correlate_queue.queue_size;
    printf("%d items in correlation queue\n",local_insert_index);*/
    /*--------------------------------------------------------------------*/
    /* note that there will always be free space in this queue since there 
       are less free buffers than entries in this queue */

    /* get the insert index */
    local_insert_index = correlator_ptr->correlate_queue.insert_index;
    
    /* get a pointer to the queue entry */
    q_ptr = &correlator_ptr->correlate_queue.data[local_insert_index];

    /* insert the data into the queue */
    q_ptr->chip_index = chip_index;
    q_ptr->search_image_ptr = search_image_ptr;
    q_ptr->ref_image_ptr = ref_image_ptr;
    q_ptr->search_size[0] = search_size[0];
    q_ptr->search_size[1] = search_size[1];
    q_ptr->ref_size[0] = ref_size[0];
    q_ptr->ref_size[1] = ref_size[1];
    q_ptr->min_corr = min_corr;
    q_ptr->fit_method = fit_method;
    q_ptr->max_disp = max_disp;
    q_ptr->nominal_offset[0] = nominal_offset[0];
    q_ptr->nominal_offset[1] = nominal_offset[1];
    q_ptr->abs_corr_coeff_flag = abs_corr_coeff_flag;

    /* update the insert index */
    local_insert_index++;
    if (local_insert_index >= correlator_ptr->correlate_queue.queue_size)
        local_insert_index = 0;
    correlator_ptr->correlate_queue.insert_index = local_insert_index;

    /* post to the semaphore to tell the correlation threads that another
       point is ready to correlate */
    if (sem_post(&correlator_ptr->correlate_queue.semaphore) != 0)
    {
        /* error posting to semaphore */
        xxx_LogStatus("XXXLIB",__FILE__,__LINE__,
                      "Error posting to correlation queue semaphore");
        return (ERROR);
    }

    return (SUCCESS);
#else
    XXX_CORRELATION_RESULT_TYPE *result_ptr;  /* pointer to current results */
    int edge_flag;     /* edge error flag from the correlation routine */
    int mult_peak_flag;/* multiple peak error flag from the correlation routine*/
    int low_peak_flag; /* low peak error flag from the correlation routine */
    int max_disp_flag; /* max displacement exceeded error flag from corr */
     
    /* cache the pointer to the results */
    result_ptr = &correlator_ptr->results_ptr[chip_index];
    
    /* perform the correlation */
    if (xxx_grey_corr(search_image_ptr, ref_image_ptr,
        search_size, ref_size, min_corr,
        fit_method, max_disp, nominal_offset,
        &result_ptr->strength, result_ptr->fit_offset, 
        result_ptr->est_err, &result_ptr->diag_disp,
        &mult_peak_flag, &edge_flag, &low_peak_flag, 
        &max_disp_flag, abs_corr_coeff_flag) == ERROR)
    {   
        /* error correlating the point */
        xxx_LogStatus("XXXLIB",__FILE__,__LINE__,
                      "Error correlating a data point");
        return (ERROR);
    }


    /* Check the return flags to see if this correlation succeeded */
    if (((edge_flag==ERROR) && correlator_ptr->check_edge_flag) ||
        ((mult_peak_flag==ERROR) && correlator_ptr->check_mult_peak_flag) || 
        ((low_peak_flag==ERROR) && correlator_ptr->check_low_peak_flag) || 
        ((max_disp_flag==ERROR) && correlator_ptr->check_max_disp_flag))
    {
        /* CORRELATION FAILURE */
        result_ptr->valid = 0;
        result_ptr->fit_offset[0] = 0.0;
        result_ptr->fit_offset[1] = 0.0;
    }
    else
    {
        /* CORRELATION SUCCESS */
        result_ptr->valid = 1;
    }

    return (SUCCESS);
#endif
}

/******************************************************************************
FUNCTION NAME:	xxx_correlate_thread

PURPOSE:        
xxx_correlate_thread is the routine that removes chips from the correlation
queue and calls the correlation routine when compiled for multithreading mode.
The chip buffers are returned to the buffer queue when the correlation is 
completed on each set of chips.
There can be many copies of this routine running at the same time.

RETURNS:
A SUCCESS/ERROR flag is returned, cast to an int since pthread routines must
return a void pointer.

NOTES:
- This routine is not included when compiled for single threading.


ALGORITHM REFERENCES:
none

******************************************************************************/

#ifdef XXX_THREAD

static void *xxx_correlate_thread(void *params_ptr)
{
    XXX_PARALLEL_CORRELATOR_TYPE *correlator_ptr=
                        (XXX_PARALLEL_CORRELATOR_TYPE*)params_ptr;
    
    int edge_flag;     /* edge error flag from the correlation routine */
    int mult_peak_flag;/* multiple peak error flag from the correlation routine*/
    int low_peak_flag; /* low peak error flag from the correlation routine */
    int max_disp_flag; /* max displacement exceeded error flag from corr */

    /* flags to indicate whether each of the error flags returned by the 
       correlation routine should be checked */
    int check_edge_flag = correlator_ptr->check_edge_flag;
    int check_mult_peak_flag = correlator_ptr->check_mult_peak_flag;
    int check_low_peak_flag = correlator_ptr->check_low_peak_flag;
    int check_max_disp_flag = correlator_ptr->check_max_disp_flag;

    CORRELATE_QUEUE_TYPE *correlate_queue_ptr = &correlator_ptr->correlate_queue;
    /* local pointer to the correlation queue */

    CORRELATE_DATA_TYPE corr_data;  /* current correlation parameters */
    int local_remove_index;/* local copy of the correlation queue remove index*/
    
    XXX_CORRELATION_RESULT_TYPE *result_ptr;  /* pointer to current results */
   
    FREE_CHIP_BUFFER_QUEUE_TYPE *buffer_queue_ptr = 
           &correlator_ptr->chip_buffer_queue; /* local pointer to buffer queue*/
    int local_insert_index; /* local copy of the buffer queue insert index */
    
    /* loop through the thread's code until cancelled */
    while(1)
    {
        /* wait for a point to be ready to correlate */
        if (sem_wait(&correlate_queue_ptr->semaphore) != 0)
        {
            xxx_LogStatus("XXXLIB",__FILE__,__LINE__,
                  "Error waiting on correlation queue data available semaphore");
            /* set the error flag and post the buffer queue semaphore to 
               release the main thread if it is waiting for buffers */
            correlator_ptr->error_flag = 1;
            sem_post(&buffer_queue_ptr->semaphore);
            return ((void*)ERROR);
        }

        /* if the queue is empty or an error occurred in one of the other 
           threads, it is the signal to terminate the thread */
        if ((correlate_queue_ptr->remove_index == 
                        correlate_queue_ptr->insert_index) ||
                        correlator_ptr->error_flag)
                 
            break;
  
        /* get the data from the queue */
        /*-----------------------------*/
        
        /* lock the queue to remove data from it */
        if (pthread_mutex_lock(&correlate_queue_ptr->remove_mutex) != 0)
        {
            /* error locking the queue */
            xxx_LogStatus("XXXLIB",__FILE__,__LINE__,
                  "Error locking the correlation queue remove mutex");
            /* set the error flag and post the buffer queue semaphore to 
               release the main thread if it is waiting for buffers */
            correlator_ptr->error_flag = 1;
            sem_post(&buffer_queue_ptr->semaphore);
            return ((void*)ERROR);
        }

        /* get the data from the queue */
        local_remove_index = correlate_queue_ptr->remove_index;
        corr_data = correlate_queue_ptr->data[local_remove_index];

        /* update the remove index */
        local_remove_index++;
        if (local_remove_index >= correlate_queue_ptr->queue_size)
            local_remove_index = 0;
        correlate_queue_ptr->remove_index = local_remove_index;

        /* release the correlate queue */
        if (pthread_mutex_unlock(&correlate_queue_ptr->remove_mutex) != 0)
        {
            /* error unlocking the queue */
            xxx_LogStatus("XXXLIB",__FILE__,__LINE__,
                  "Error unlocking the correlation queue remove mutex");
            /* set the error flag and post the buffer queue semaphore to 
               release the main thread if it is waiting for buffers */
            correlator_ptr->error_flag = 1;
            sem_post(&buffer_queue_ptr->semaphore);
            return ((void*)ERROR);
        }
            
        /* correlate */
        /*-----------*/

        /* cache the pointer to the results */
        result_ptr = &correlator_ptr->results_ptr[corr_data.chip_index];
        
        /* perform the correlation */
        if (xxx_grey_corr(corr_data.search_image_ptr, corr_data.ref_image_ptr,
            corr_data.search_size, corr_data.ref_size, corr_data.min_corr,
            corr_data.fit_method, corr_data.max_disp, corr_data.nominal_offset,
            &result_ptr->strength, result_ptr->fit_offset, 
            result_ptr->est_err, &result_ptr->diag_disp,
            &mult_peak_flag, &edge_flag, &low_peak_flag, 
            &max_disp_flag, corr_data.abs_corr_coeff_flag) == ERROR)
        {   
            /* error correlating the point */
            xxx_LogStatus("XXXLIB",__FILE__,__LINE__,
                          "Error correlating a data point");
            /* set the error flag and post the buffer queue semaphore to 
               release the main thread if it is waiting for buffers */
            correlator_ptr->error_flag = 1;
            sem_post(&buffer_queue_ptr->semaphore);
            return ((void*)ERROR);
        }


        /* Check the return flags to see if this correlation succeeded */
        if (((edge_flag==ERROR) && check_edge_flag) ||
            ((mult_peak_flag==ERROR) && check_mult_peak_flag) || 
            ((low_peak_flag==ERROR) && check_low_peak_flag) || 
            ((max_disp_flag==ERROR) && check_max_disp_flag))
        {
            /* CORRELATION FAILURE */
            result_ptr->valid = 0;
            result_ptr->fit_offset[0] = 0.0;
            result_ptr->fit_offset[1] = 0.0;
        }
        else
        {
            /* CORRELATION SUCCESS */
            result_ptr->valid = 1;
        }

        /* return the chip buffers to the free buffer queue */
        /*--------------------------------------------------*/

        /* lock the free buffer queue */
        if (pthread_mutex_lock(&buffer_queue_ptr->insert_mutex) != 0)
        {
            /* error locking the buffer mutex */
            xxx_LogStatus("XXXLIB",__FILE__,__LINE__,
                          "Error locking the free chip buffer queue");
            /* set the error flag and post the buffer queue semaphore to 
               release the main thread if it is waiting for buffers */
            correlator_ptr->error_flag = 1;
            sem_post(&buffer_queue_ptr->semaphore);
            return ((void*)ERROR);
        }
        
        /* insert the free buffers */
        local_insert_index = buffer_queue_ptr->insert_index;
        buffer_queue_ptr->data[local_insert_index].ref_chip_ptr = 
                    corr_data.ref_image_ptr;
        buffer_queue_ptr->data[local_insert_index].search_chip_ptr =
                    corr_data.search_image_ptr;

        /* adjust the insert index */
        local_insert_index++;
        if (local_insert_index >= buffer_queue_ptr->queue_size)
            local_insert_index = 0;
        buffer_queue_ptr->insert_index = local_insert_index;

        /* unlock the free buffer queue */
        if (pthread_mutex_unlock(&buffer_queue_ptr->insert_mutex) != 0)
        {
            /* error unlocking the buffer mutex */
            xxx_LogStatus("XXXLIB",__FILE__,__LINE__,
                          "Error unlocking the free chip buffer queue");
            /* set the error flag and post the buffer queue semaphore to 
               release the main thread if it is waiting for buffers */
            correlator_ptr->error_flag = 1;
            sem_post(&buffer_queue_ptr->semaphore);
            return ((void*)ERROR);        
        }
        
        /* post to the buffer semaphore to notify a buffer is available */
        if (sem_post(&buffer_queue_ptr->semaphore) != 0)
        {
            /* error posting to the buffer semaphore */
            xxx_LogStatus("XXXLIB",__FILE__,__LINE__,
                          "Error posting to the free chip buffer semaphore");
            /* set the error flag and post the buffer queue semaphore to 
               release the main thread if it is waiting for buffers */
            correlator_ptr->error_flag = 1;
            sem_post(&buffer_queue_ptr->semaphore);
            return ((void*)ERROR);                    
        }

    }

    return ((void*)SUCCESS);
}
#endif

/******************************************************************************
FUNCTION NAME:	xxx_close_parallel_correlator

PURPOSE:        
xxx_close_parallel_correlator is the routine that needs to be called after all
the chips have been submitted to the correlator.  This routine will wait until
all the threads have completed, then destroy this instance of the parallel
correlator.

RETURNS:
A SUCCESS/ERROR flag is returned.

NOTES:
- The results of the correlation are not valid until this routine returns.

ALGORITHM REFERENCES:
none

******************************************************************************/

int xxx_close_parallel_correlator
(
    XXX_PARALLEL_CORRELATOR_TYPE *correlator_ptr /* I: parallel correlator */
)
{
#ifdef XXX_THREAD
    int counter; /* looping variable */
    void *thread_return; /* return value of threads */
    int error = SUCCESS; /* combined error from threads */
    
    /* post to the correlator semaphore once for each thread that was created */
    for (counter = 0; counter < correlator_ptr->threads; counter++)
    {
        if (sem_post(&correlator_ptr->correlate_queue.semaphore) != 0)
        {
            /* error posting to the semaphore */
            xxx_LogStatus("XXXLIB",__FILE__,__LINE__,
                          "Error posting to the correlate queue semaphore");
            return (ERROR);
        }
    }
    
    /* wait for the threads to join */
    for (counter = 0; counter < correlator_ptr->threads; counter++)
    {
        /* join with one of the threads */
        if (pthread_join(correlator_ptr->thread_ids[counter], &thread_return)!=0)
        {
            /* error joining with the thread */
            xxx_LogStatus("XXXLIB",__FILE__,__LINE__,
                          "Error joining with a thread");
            return (ERROR);
        }

        /* combine this thread's return value with the other threads 
           return values */
        error |= (int)thread_return;
    }
    
    /* check if any of the threads errored out */
    if (error != SUCCESS)
    {
        xxx_LogStatus("XXXLIB",__FILE__,__LINE__,
                      "Error reported from a correlation thread");
    }
    
    /* destroy the multiprocessing resources */
    if (pthread_mutex_destroy(&correlator_ptr->correlate_queue.remove_mutex)!=0)
    {
        xxx_LogStatus("XXXLIB",__FILE__,__LINE__,
                      "Error destroying the correlate queue mutex");
        error = ERROR;
    }
    
    if (pthread_mutex_destroy(&correlator_ptr->correlate_queue.remove_mutex)!=0)
    {
        xxx_LogStatus("XXXLIB",__FILE__,__LINE__,
                      "Error destroying the chip buffer queue mutex");
        error = ERROR;
    }
    
    if (sem_destroy(&correlator_ptr->correlate_queue.semaphore) != 0)
    {
        xxx_LogStatus("XXXLIB",__FILE__,__LINE__,
                      "Error destroying the correlate queue semaphore");
        error = ERROR;
    }
    
    if (sem_destroy(&correlator_ptr->chip_buffer_queue.semaphore) != 0)
    {
        xxx_LogStatus("XXXLIB",__FILE__,__LINE__,
                      "Error destroying the chip buffer queue semaphore");
        error = ERROR;
    }
    
    /* free the memory allocated */
    free (correlator_ptr->correlate_queue.data);
    free (correlator_ptr->chip_buffer_queue.buffers);
    free (correlator_ptr->chip_buffer_queue.data);
    free (correlator_ptr);    
    
    /* return error value */
    return (error);
#else
    
    /* free the memory allocated for single threaded compiles */
    free (correlator_ptr->ref_chip_ptr);
    free (correlator_ptr->search_chip_ptr);
    free (correlator_ptr);

    return (SUCCESS);
#endif
}
