#include "aligrid.h"

/******************************************************************************
NAME:           aligrid

PURPOSE:
To generate the ALI mapping grid defining a transformation from the image's
satellite perspective to a user specified output projection.

DEVELOPMENT HISTORY:

RETURN VALUE:   Type = int
Value    Description
-----    -----------
SUCCESS  Successful completion
FAILURE  Operation failed

NOTES:
  Generates an ALI mapping grid that defines a transformation from
  the image's satellite perspective to a user specified output
  projection.  This routine will read the 1R image and ALIModel file.
  Then the output framing coordinates are determined from the user's
  input.  The frame may be map-oriented or path-oriented.  Once the
  frame is determined in output space, the input space is gridded.
  Then the grid in input space is mapped to the output space.
  Transformation coefficients to transform a grid cell from input to
  output space are determined, as well as coefficients to transform
  a grid cell from output to input space.  These coefficients are
  stored in the grid for later use in resampling and band-to-band
  calibration.  The rough mapping polynomial coefficients are also
  calculated and stored in the grid for later use by resampling and
  scan mirror calibration.  The grid structure is then written to an
  output file.

  The grid references the upper-left corner to be (0,0).

  For all framing, the optimal band to use is band 4.  If band 4 is
  not in the list of bands to process for the grid, then the next
  optimal band is used.  The following is the order for which the
  bands will be used for these frame types.
  {4, 3, 7, 2, 5, 1, 8, 6}
  TMGRID will search the list for the most optimal band that was
  specified.  This band information will then be used for minbox
  and path-oriented framing options.

ALGORITHM REFERENCES:
See axx_grid.h for a description of the grid structure.
See axx_alimodel.h for a description of the ETM+ model structure.
See exx_cpf.h for a description of the CPF structure.

******************************************************************************/

main (int argc, char *argv[])
{
  /* Variable declarations.
   */
  axx_GRID_TYPE grid;            /* Grid structure                        */
  ALIMODEL alimodel;             /* ETM+ model structure                  */
  CPF *cpf;                      /* CPF constants                         */
  char ErrMsg[STRLEN],           /* Error message                         */
       param_filename[STRLEN],   /* Parameter file name                   */
       scriptname[STRLEN],       /* Script name                           */
       filename_0R[STRLEN],      /* 0R ETM+ image filename                */
       filename_1R[STRLEN],      /* 1R ETM+ image filename                */
       cal_parm_file[STRLEN],    /* Calibration parameter filename        */
       model_filename[STRLEN];   /* ETM+ model filename                   */

  struct DBL_XY frame_coords[2]; /* User-entered frame coordinates        */
  struct DBL_LS lscoords;        /* User-entered l/s coordinates          */
  int  i,                        /* Loop variable                         */
       band,                     /* Band counter                          */
       sca,                      /* SCA counter                           */
       ref,			 /* Reference band number		  */
       computed_zone,            /* Computed zone value for the scene     */
       tmp_zone,                 /* Zone for a single SCA                 */
       coord_code,               /* User coordinates code                 */
       path,                     /* WRS path                              */
       currref,                  /* Current band reference position       */
       num_0r_lines[NBANDS],  /* Number of lines in the 0R image       		*/
       num_0r_samps[NBANDS];  /* Number of samples in the 0R image     		*/
  double row,                    /* WRS row                               	*/
         poly_x[2][4],           /* X polynomial coefficients (4 coefs for
                                    each band resolution in the order of 10m, 30m) */
         poly_y[2][4];           /* Y polynomial coefficients (4 coefs for
                                    each band resolution in the order of 10m, 30m) */
  BandNumber opt_band,           /* Optimal band of all the specified
                                    bands for tmgrid processing           	*/
             bands[NBANDS];      /* Array of bands for grid I/O           	*/
  int nbands,                    /* Number of bands in the bands array    	*/
      zone_valid;                /* Flag to specify if the UTM zone is valid 	*/
  int status;			 /* status of return from function 		*/
  int num;			 /* Number of z planes 				*/
  int zeroplane;		 /* Zero plane 					*/ 
  float zmax;			 /* Maximum elevation 				*/
  float zmin;			 /* Minimum elevation 				*/
  float zincrement;		 /* Elevation increment 			*/

  /* Print the starting message.
   */
  xxx_LogStatus ("ALIGRID", __FILE__, __LINE__, "Initiate ...");

  /* Check the arguments and get the parameter file name from the arguments.
   */
  if (argc != 2)
  {
    strcpy (ErrMsg, "Usage: aligrid ODLfile");
    xxx_LogStatus ("ALIGRID", __FILE__, __LINE__, ErrMsg);
    exit (EXIT_FAILURE);
  }
  else
    strcpy (param_filename, argv[1]);

  /* Set number of SCAs to maximum value.
   */
  grid.nscas = NSCA;

  /* Get the grid parameters.
   */
  if (get_parms (param_filename, scriptname, filename_0R, filename_1R,
      cal_parm_file, model_filename, frame_coords, &coord_code, &lscoords,
      &path, &row, &zincrement, &zmax, &zmin, &grid) != SUCCESS)
  {
    strcpy (ErrMsg, "Error reading the parameters from the ODL file.");
    xxx_LogStatus ("ALIGRID", __FILE__, __LINE__, ErrMsg);
    exit (EXIT_FAILURE);
  }

  /* Get the ALI model.
   */
  if (axx_get_alimodel (model_filename, &alimodel) != SUCCESS)
  {
    strcpy (ErrMsg, "Error reading the ETM+ model.");
    xxx_LogStatus ("ALIGRID", __FILE__, __LINE__, ErrMsg);
    exit (EXIT_FAILURE);
  }

  /* Set number of SCAs to actual value from model.
   */
  grid.nscas = alimodel.nsca;

  /* Set elevation parameters for now.
   */
  sprintf(ErrMsg, "User entered elevation range %8.2f %8.2f\n",zmin,zmax);
  xxx_LogStatus ("ALIGRID", __FILE__, __LINE__, ErrMsg);

  /* Fix number of planes, and change zmin and zmax, so that
     a zero plane exists.
   */
  if( zmin<0.0 && zmax<0.0 )
  {
     zmax = 0.0;
  }
  else if( zmax>0.0 && zmin>0.0 )
  {
     zmin = 0.0;
  }
  num = (int)(ceil(zmax/zincrement)+ceil((-zmin)/zincrement)) + 1;
  zeroplane = (ceil)((-zmin / zincrement));
  zmin = -zeroplane * zincrement;
  zmax =  (num - 1 - zeroplane) * zincrement;
  sprintf(ErrMsg,
     "New min/max elevation %8.2f %8.2f number of planes %d zero plane %d\n",
     zmin,zmax,num,zeroplane); 
  xxx_LogStatus ("ALIGRID", __FILE__, __LINE__, ErrMsg);

  /* Store elevation parameters in grid.
   */
  for( band=0 ; band<grid.nbands ; band++ )
  {
    ref = grid.bands[band];
    for( sca=0 ; sca<grid.nscas ; sca++ )
    {
       grid.gridbands[sca][ref].nzplanes = num;
       grid.gridbands[sca][ref].zspacing = zincrement;
       grid.gridbands[sca][ref].zeroplane = zeroplane;
    }
  }

  /* Open the 0R or 1R imagery for each band and read the number of lines
   * and samples in the image.
   */
  if (read_num_ls (filename_1R, grid.bands, grid.nbands, grid.nscas, num_0r_lines,
      num_0r_samps) != SUCCESS)
  {
    strcpy (ErrMsg, "Error reading the number of lines and samples from the imagery.");
    xxx_LogStatus ("TMGRID", __FILE__, __LINE__, ErrMsg);
    exit (EXIT_FAILURE);
  }

  /* Determine the number of grid lines and samples.
   */
  if (det_num_grid_ls (num_0r_lines, num_0r_samps, &grid) != SUCCESS)
  {
    strcpy (ErrMsg, "Error determining the number of lines and samples in the grid.");
    xxx_LogStatus ("ALIGRID", __FILE__, __LINE__, ErrMsg);
    exit (EXIT_FAILURE);
  }

  /* Set the grid correction type and grid satellite number.
   */
  grid.correction_type = alimodel.correction_type;
  grid.satnum = alimodel.satnum;

  /* Read the CPF constants.
   */
  cpf = exx_read_cpf (cal_parm_file);
  if (cpf == NULL)
  {
    strcpy (ErrMsg, "Error reading the CPF constants.");
    xxx_LogStatus ("ALIGRID", __FILE__, __LINE__, ErrMsg);
    exit (EXIT_FAILURE);
  }

  /* If the proj_code is UTM, then verify the correct zone was given.  If
   * the user-entered zone is 0, then use the calculated zone and continue.
   */
  if (grid.code == 1)
  {
    /* Use the information from the first band provided.
     */
    currref = grid.bands[0];
    /* Probably don't need to loop on SCAs.
     */
    for(sca = 0; sca < grid.nscas; sca++ )
    {
      if (validate_utm_zone (num_0r_lines[currref], num_0r_samps[currref],
          (BandNumber) currref, sca, &alimodel, cpf, grid.zone, &tmp_zone, 
          &zone_valid) != SUCCESS)
      {
        strcpy (ErrMsg, "Error validating the UTM zone.");
        xxx_LogStatus ("ALIGRID", __FILE__, __LINE__, ErrMsg);
        exit (EXIT_FAILURE);
      }
      if( sca == 0 )
         computed_zone = tmp_zone;
      else
      {
        if( tmp_zone != computed_zone )
        {
          strcpy (ErrMsg, "SCAs fall in differnt UTM zones.");
          xxx_LogStatus ("ALIGRID", __FILE__, __LINE__, ErrMsg);
          strcpy (ErrMsg, "Using SCA 0 for zone.");
          xxx_LogStatus ("ALIGRID", __FILE__, __LINE__, ErrMsg);
        }
      }
    }
    
    /* If the zone is zero then use the calculated zone.
     */
    if (grid.zone == 0)
    {
      status = sprintf (ErrMsg, "Automatically computing UTM zone.  Computed zone is %ld.",
                        computed_zone);
      if (status < 1)
      {
        xxx_LogStatus("ALIGRID", __FILE__, __LINE__, "Error in sprintf");
        return(ERROR);
      }
      xxx_LogStatus ("ALIGRID", __FILE__, __LINE__, ErrMsg);
      grid.zone = computed_zone;
    }

    /* OW, if the zone is not valid then exit the application.
     */
    else if (!zone_valid)
    {
      sprintf (ErrMsg, "User-entered zone is %ld.  Computed zone is %ld.", grid.zone, computed_zone);
      xxx_LogStatus ("ALIGRID", __FILE__, __LINE__, ErrMsg);
      strcpy (ErrMsg, "The zone provided does not correspond to the image scene center longitude.");
      xxx_LogStatus ("ALIGRID", __FILE__, __LINE__, ErrMsg);
      exit (EXIT_FAILURE);
    }
  }

  /* Malloc space for the grid pointers.  Use nbands = 0 to malloc space
   * for the entire grid.
   */
  nbands = 0;
  if (axx_malloc_grid (bands, &nbands, &grid) != SUCCESS)
  {
    strcpy (ErrMsg, "Error mallocing space for the grid.");
    xxx_LogStatus ("ALIGRID", __FILE__, __LINE__, ErrMsg);
    exx_free_cpf (cpf);
    exit (EXIT_FAILURE);
  }

  /* Determine the frame in output space.
   */
  switch (grid.frame_type)
  {
    case GEOBOX:
    case PROJBOX:
    case UL_SIZE:
      /* Determine the "most optimal" band.  Loop through the bands looking
       * for the highest "most optimal" band.  Break out when found.
       */
      if (det_opt_band (grid.bands, grid.nbands, &opt_band) != SUCCESS)
      {
        strcpy (ErrMsg, "Optimal band for MINBOX framing was not found.");
        xxx_LogStatus ("ALIGRID", __FILE__, __LINE__, ErrMsg);
        exx_free_cpf (cpf);
        exit (EXIT_FAILURE);
      }

      if (north_up (&grid, opt_band, frame_coords, lscoords, coord_code) != SUCCESS)
      {
        strcpy (ErrMsg, "Error determining the north-up frame.");
        xxx_LogStatus ("ALIGRID", __FILE__, __LINE__, ErrMsg);
        exx_free_cpf (cpf);
        exit (EXIT_FAILURE);
      }
      break;

    case MINBOX:
      /* Determine the "most optimal" band.  Loop through the bands looking
       * for the highest "most optimal" band.  Break out when found.
       */
      if (det_opt_band (grid.bands, grid.nbands, &opt_band) != SUCCESS)
      {
        strcpy (ErrMsg, "Optimal band for MINBOX framing was not found.");
        xxx_LogStatus ("ALIGRID", __FILE__, __LINE__, ErrMsg);
        exx_free_cpf (cpf);
        exit (EXIT_FAILURE);
      }

      if (minbox (&alimodel, opt_band, num_0r_lines[opt_band], num_0r_samps[opt_band], 
                  cpf, &grid) != SUCCESS)
      {
        strcpy (ErrMsg, "Error determining the minbox frame.");
        xxx_LogStatus ("ALIGRID", __FILE__, __LINE__, ErrMsg);
        exx_free_cpf (cpf);
        exit (EXIT_FAILURE);
      }
      break;

    case MAXBOX:
      if (maxbox (&alimodel, num_0r_lines, num_0r_samps, cpf, &grid) != SUCCESS)
      {
        strcpy (ErrMsg, "Error determining the maxbox frame.");
        xxx_LogStatus ("ALIGRID", __FILE__, __LINE__, ErrMsg);
        exx_free_cpf (cpf);
        exit (EXIT_FAILURE);
      }
      break;

    case PATH_ORIENTED:
      if (path_oriented (&grid, cpf, path, row, poly_x, poly_y) != SUCCESS)
      {
        strcpy (ErrMsg, "Error determining the path_oriented frame.");
        xxx_LogStatus ("ALIGRID", __FILE__, __LINE__, ErrMsg);
        exx_free_cpf (cpf);
        exit (EXIT_FAILURE);
      }
      break;

   case PATH_MINBOX:
      /* Determine the "most optimal" band.  Loop through the bands looking
       * for the highest "most optimal" band.  Break out when found.
       */
      if (det_opt_band (grid.bands, grid.nbands, &opt_band) != SUCCESS)
      {
        strcpy (ErrMsg, "Optimal band for MINBOX framing was not found.");
        xxx_LogStatus ("ALIGRID", __FILE__, __LINE__, ErrMsg);
        exx_free_cpf (cpf);
        exit (EXIT_FAILURE);
      }
      if (path_maxmin_box(&alimodel, opt_band, num_0r_lines, num_0r_samps,
                          cpf, &grid, path, row, poly_x, poly_y) != SUCCESS)
      {
        strcpy (ErrMsg, "Error determining the max/min box frame.");
        xxx_LogStatus ("ALIGRID", __FILE__, __LINE__, ErrMsg);
        exx_free_cpf (cpf);
        exit (EXIT_FAILURE);
      }
      break;

   case PATH_MAXBOX:
      if (path_maxmin_box(&alimodel, opt_band, num_0r_lines, num_0r_samps,
                          cpf, &grid, path, row,poly_x,poly_y) != SUCCESS)
      {
        strcpy (ErrMsg, "Error determining the max/min box frame.");
        xxx_LogStatus ("ALIGRID", __FILE__, __LINE__, ErrMsg);
        exx_free_cpf (cpf);
        exit (EXIT_FAILURE);
      }
      break;

    default:
    {
      strcpy (ErrMsg, "Incorrect framing type was specified.");
      xxx_LogStatus ("ALIGRID", __FILE__, __LINE__, ErrMsg);
      exx_free_cpf (cpf);
      exit (EXIT_FAILURE);
    }
  }

  /* Determine the grid lines in input space.
   */
  if (det_grid_ls (num_0r_lines, num_0r_samps, &grid) != SUCCESS)
  {
    strcpy (ErrMsg, "Error determining the grid lines in input space.");
    xxx_LogStatus ("ALIGRID", __FILE__, __LINE__, ErrMsg);
    exx_free_cpf (cpf);
    exit (EXIT_FAILURE);
  }

  /* Make the grid in output space.
   */
  if (make_grid (&alimodel, cpf, poly_x, poly_y, &grid) != SUCCESS)
  {
    strcpy (ErrMsg, "Error making the grid in output space.");
    xxx_LogStatus ("ALIGRID", __FILE__, __LINE__, ErrMsg);
    exx_free_cpf (cpf);
    exit (EXIT_FAILURE);
  }

  /* Get the mapping coefficients for the grid.
   */
  if (calc_map_coefs (&grid) != SUCCESS)
  {
    strcpy (ErrMsg, "Error calculating the mapping coefficients.");
    xxx_LogStatus ("ALIGRID", __FILE__, __LINE__, ErrMsg);
    exx_free_cpf (cpf);
    exit (EXIT_FAILURE);
  }

  /* Set up the grid_cell_poly coefficients.
   */
  if (calc_rough_map_coefs (&grid) != SUCCESS)
  {
    strcpy (ErrMsg, "Error calculating the rough mapping coefficients.");
    xxx_LogStatus ("ALIGRID", __FILE__, __LINE__, ErrMsg);
    exx_free_cpf (cpf);
    exit (EXIT_FAILURE);
  }

  /* Compute the even/odd detector offset values
   */
  if (calc_det_offsets (&alimodel, cpf, poly_x, poly_y, &grid) != SUCCESS)
  {
    strcpy (ErrMsg, "Error calculating detector offset values.");
    xxx_LogStatus ("ALIGRID", __FILE__, __LINE__, ErrMsg);
    exx_free_cpf (cpf);
    exit (EXIT_FAILURE);
  }

  /* Write the grid structure out to a file.
   */
  if (axx_put_grid (grid.gridname, &grid) != SUCCESS)
  {
    strcpy (ErrMsg, "Error writing the grid to the specified file.");
    xxx_LogStatus ("ALIGRID", __FILE__, __LINE__, ErrMsg);
    exx_free_cpf (cpf);
    exit (EXIT_FAILURE);
  }

  /* Free the CPF structure.
   */
  exx_free_cpf (cpf);

  /* Free the grid.  Use nbands = 0 to free the entire grid.
   */
  nbands = 0;
  if (axx_free_grid (bands, &nbands, &grid) != SUCCESS)
  {
    strcpy (ErrMsg, "Error freeing the grid structure.");
    xxx_LogStatus ("ALIGRID", __FILE__, __LINE__, ErrMsg);
    exit (EXIT_FAILURE);
  }

  /* Print the completion message.
   */
  xxx_LogStatus ("ALIGRID", __FILE__, __LINE__, "Successful completion.");

  return (EXIT_SUCCESS);
} /* done */
