/******************************************************************************
NAME:           angle_to_map

PURPOSE:
This function will convert the WRS rotation angle (from geodetic north) to
a frame orientation angle in map coordinates.


RETURN VALUE:
Type = int (SUCCESS or ERROR)

ALGORITHM REFERENCES:
******************************************************************************/
#include "aligrid.h"

int angle_to_map(
    axx_GRID_TYPE *grid,    /* I/O: Grid structure to examine/modify	*/
    int  units)		    /* I: Integral units for GCTP		*/
{
    int  zero = 0,		/* Constant of zero			*/
	 null_zone = NULLZONE,	/* Null zone for projection xforms	*/
	 status,		/* Return status from library functions */
	 deg_units = 4,		/* Units code for degrees		*/
	 i;			/* Loop counter				*/

    double geo_parms[COEFCT],	/* Geographic projection coefficients	*/
	   azimuth,		/* Grid azimuth of geodetic north	*/
	   microradian,		/* 0.2 seconds to add to WRS latitude	*/
	   sclat, sclong,	/* WRS scene center in lat/long		*/
           x1, y1,		/* Scene center in projection coords	*/
	   x2, y2;		/* Projection center after adding 1	*/
				/* microradian to the SC latitude.	*/
    char ErrMsg[STRLEN];        /* Error message                        */

    /* Zero out the geographic projection coefficients, and give initial
       values to other variables needed for processing.
    --------------------------------------------------------------------*/
    sclat = grid->center_pt.lat;
    sclong = grid->center_pt.lng;
    microradian = (1.0 / 3600.0) * 0.2;
    for (i = 0; i < COEFCT; geo_parms[i++] = 0.0);

    /* Retrieve the projection coordinates of scene center. 
    -------------------------------------------------------*/
    x1 = grid->proj_center_pt.x;
    y1 = grid->proj_center_pt.y;

    /* Add one microradian to the WRS SC latitude and find the projection
       coordinates once again, storing the results in x2 and y2.
    ---------------------------------------------------------------------*/
    sclat += microradian;
    status = exx_projtran(&zero, &deg_units, &null_zone, geo_parms,
                    &(grid->spheroid), &(grid->code), &units,
                    &(grid->zone), grid->projprms, &(grid->spheroid),
                    &sclong, &sclat, &x2, &y2);
    if (status == ERROR)
    {
      strcpy (ErrMsg, "Error in call to projtran.");
      xxx_LogStatus ("ALIGRID", __FILE__, __LINE__, ErrMsg);
      return (ERROR);
    }

    /* Now we have two slightly-differing scene centers in projection
       coordinates.  Compute the azimuth of the line that they form as
       the arctangent of (x2 - x1) / (y2 - y1).  This will represent the
       grid azimuth of geodetic north at the WRS scene center.
    --------------------------------------------------------------------*/
    azimuth = atan ((x2 - x1) / (y2 - y1));
    azimuth *= R2D; 	             /* Convert from radians to degrees */

    /* Finally, we add this angle to the WRS rotation angle to get the
       grid heading.  A standard framed scene puts the satellite direction
       of flight at the bottom of scene, so the scene orientation is the
       grid heading either + or - 180 degrees.  If:

       the grid heading is < 0, subtract 180 degrees.
       the grid heading is > 0, add 180 degrees.
    ----------------------------------------------------------------------*/
    grid->orient_angle = grid->rotate_angle + azimuth;
    if (grid->orient_angle < 0)
	grid->orient_angle -= 180.0;
    else if (grid->orient_angle >= 0)
	grid->orient_angle += 180.0;

    return SUCCESS;
}
