/******************************************************************************
NAME:		calc_center_and_rotation_angle

PURPOSE:
Given a path, row, and various entries from the CPF, this function will 
calculate the scene center in lat/long and the rotation angle for a nominal
WRS scene.  Earth_Constants and Orbit_Parameters hold the required CPF data.


RETURN VALUE:
Type = int (SUCCESS or ERROR)

ALGORITHM REFERENCES:
******************************************************************************/
#include "aligrid.h"

int calc_center_and_rotation_angle(
    int path,		 /* I: WRS path number				*/
    double row,		 /* I: WRS row number (may be fractional)	*/
    CPF *cpf,		 /* I: Calibration parameters			*/
    axx_GRID_TYPE *grid) /* I/O: Grid structure				*/
{
    double inclination;		/* Inclination angle in radians		*/
    double long_path1_row60;	/* Longitude of path 1, row 60 		*/
    double earth_spin_rate;	/* Earth solar rotation rate		*/
    double sc_ang_rate;		/* Angular velocity of spacecraft	*/
    double central_angle;	/* Central travel angle for this row	*/
    double wrs_gclat;		/* WRS scene center geocentric latitude	*/
    double long_origin;		/* Longitude of row 60 for given path	*/
    double delta_long;		/* Longitude offset due to central ang. */
    double wrs_long;		/* WRS scene center longitude		*/
    double wrs_lat;		/* WRS scene center latitude		*/
    double heading_angle;	/* Scene center heading from north	*/

    /* Convert input angles to radians.
    -----------------------------------*/
    inclination = D2R * cpf->Orbit_Parameters.Inclination_Angle;
    long_path1_row60 = D2R * cpf->Orbit_Parameters.Long_Path1_Row60;

    /* Compute the Earth's angular rotation rate.  We use the solar
       rotation rate rather than the sidereal rate as called for in the
       General Electric Landsat D PIR which describes the Worldwide
       Reference System.  This is to account for the orbital precession
       which is designed to make the orbit sun-synchronous.
    -------------------------------------------------------------------*/
    earth_spin_rate = 2.0 * PIE / (double) SEC_PER_DAY;

    /* Compute the spacecraft's angular rotation rate.
    --------------------------------------------------*/
    sc_ang_rate = 2.0 * PIE * (double) cpf->Orbit_Parameters.WRS_Cycle_Orbits /
		  (double) (cpf->Orbit_Parameters.WRS_Cycle_Days * SEC_PER_DAY);

    /* Compute the central travel angle for this row.
    -------------------------------------------------*/
    central_angle = 2.0 * PIE * (row -
		    (double) cpf->Orbit_Parameters.Descending_Node_Row) /
		    (double) cpf->Orbit_Parameters.Scenes_Per_Orbit;

    /* Compute the WRS geocentric latitude.
    ---------------------------------------*/
    wrs_gclat = asin(-sin(central_angle) * sin(inclination));

    /* Compute the longitude of row 60 for this path.
    -------------------------------------------------*/
    long_origin = long_path1_row60 - 2.0 * PIE * (double) (path - 1) /
		  (double) cpf->Orbit_Parameters.WRS_Cycle_Orbits;

    /* Compute the WRS longitude.
    -----------------------------*/
    delta_long = atan2(tan(wrs_gclat) / tan(inclination),
		       cos(central_angle) / cos(wrs_gclat));
    wrs_long = long_origin - delta_long - central_angle * earth_spin_rate /
	       sc_ang_rate;

    /* Ensure the longitude is in the range +/- pi.
    -----------------------------------------------*/
    while (wrs_long > PIE)
	wrs_long -= 2.0 * PIE;
    while (wrs_long < -PIE)
	wrs_long += 2.0 * PIE;

    /* Compute the scene heading angle.
    -----------------------------------*/
    heading_angle = atan2(cos(inclination) / cos(wrs_gclat),
			  -cos(delta_long) * sin (inclination));

    /* Convert the WRS geocentric latitude to geodetic latitude.
    ------------------------------------------------------------*/
    wrs_lat = atan(tan(wrs_gclat) * 
		   (cpf->Earth_Constants.Semi_Major_Axis /
		   cpf->Earth_Constants.Semi_Minor_Axis) *
		   (cpf->Earth_Constants.Semi_Major_Axis /
		   cpf->Earth_Constants.Semi_Minor_Axis));

    /* Convert all computed angles from radians to degrees.
    -------------------------------------------------------*/
    wrs_lat *= R2D;
    wrs_long *= R2D;
    heading_angle *= R2D;

    /* Round WRS scene center lat/long to nearest whole arc minute.
    ---------------------------------------------------------------*/
    wrs_lat = floor(wrs_lat * 60.0 + 0.5) / 60.0;
    wrs_long = floor(wrs_long * 60.0 + 0.5) / 60.0;

    /* Give our results to the caller.
    ----------------------------------*/
    grid->center_pt.lat = wrs_lat;
    grid->center_pt.lng = wrs_long;
    grid->rotate_angle = heading_angle + 180;

    return SUCCESS;
}
