#include "aligrid.h"

/******************************************************************************
NAME:           calc_det_offsets

PURPOSE:
Computes the even/odd detector offset values.  Stores linear mapping
coefficients associated with detector offsets into grid.


RETURN VALUE:   Type = int
Value    Description
-----    -----------
SUCCESS  Successful completion
ERROR    Operation failed

NOTES:

ALGORITHM REFERENCES:
See axx_grid.h for a description of the grid structure.
See axx_alimodel.h for a description of the alimodel structure.
See exx_cpf.h for a description of the CPF structure.

******************************************************************************/

int calc_det_offsets (
  ALIMODEL *alimodel,      /* I: ALI model structure                        */
  CPF *cpf,                /* I: CPF constants                              */
  double poly_x[2][4],     /* I: Array of x polynomial coefficients
                                 (in the order of 10m, 30m)                 */
  double poly_y[2][4],     /* I: Array of y polynomial coefficients
                                 (in the order of 10m, 30m)                 */
  axx_GRID_TYPE *grid)     /* I/O: Grid structure                           */
{
  int  ref,             /* corresponding reference position for the band
                           numbers                                          */
       stat,            /* Return status                                    */
       i, j;            /* Loop variables                                   */
  int  sca,		/* Counter for SCAs				    */
       zplane,          /* Counter for elevation planes                     */
       odd_samp,	/* Closest odd detector sample number               */
       even_samp,	/* Closest even detector sample number              */
       curr_band,       /* Current band number                              */
       row_ptr,		/* Grid row index number                            */
       col_ptr,         /* Grid column index number                         */
       out_units,       /* Projection units code                            */
       geo = 0,         /* Geographic projection flag                       */
       zero = 0,        /* Zero                                             */
       null_zone = NULLZONE,
                        /* UTM projection code                              */
       blank = TRUE;    /* Flag to specify if the output image is blank.
                           If one input pixel maps to an output pixel, then
                           the image is not blank.                          */
  long input_nl,	/* Number of lines in the input image               */
       input_ns;        /* Number of samples in the input image             */
  double elev = 0.0,    /* Elevation at the current point                   */
         in_lat,        /* Latitude in input space                          */
         in_long,       /* Longitude in input space                         */
         proj_x,        /* Projection X                                     */
         proj_y,        /* Projection Y                                     */
         line,          /* Line in output space                             */
         samp,          /* Sample in output space                           */
         iline,		/* Line in input space                              */
         isamp,         /* Sample in input space                            */
         geo_parms[COEFCT];
                        /* Geo projection parameters                        */
  int polyindex[10] = {0, 1, 1, 1, 1, 1, 1, 1, 1, 1};
                        /* Poly x/y indices - bands 1-7 are 30m (1),
                           pan is 10m (0) */
  char ErrMsg[STRLEN];  /* Error message                                    */
  DetType dettype = ACTUAL;

  double *delta_line_even; /* Linear offset coefficients to be stored in grid */
  double *delta_samp_even; /* Linear offset coefficients to be stored in grid */
  double *delta_line_odd;  /* Linear offset coefficients to be stored in grid */
  double *delta_samp_odd;  /* Linear offset coefficients to be stored in grid */
  double *zline_even;  /* Line/even det Offset value per elevation plane */
  double *zsamp_even;  /* Sample/even det Offset value per elevation plane */
  double *zline_odd;   /* Line/odd det Offset value per elevation plane */
  double *zsamp_odd;   /* Sample/odd det Offset value per elevation plane */
  double *z;           /* Elevation values */
  double sum_x;        /* Sum of X values for LSQ solution */
  double sum_x_sqr;    /* Sum of X squared values for LSQ solution */
  double sum_y;        /* Sum of Y values for LSQ solution */
  double sum_xy;       /* Sum of X*Y values for LSQ solution */
  int sz;

  /* Initialize geo_parms.
   */
  for (i = 0; i < COEFCT; i++)
    geo_parms[i] = 0.0;

  /* Convert the grid projection units to a unit code for gctp.
   */
  if (!strcmp (grid->units, "RADIANS"))
    out_units = 0;
  else if (!strcmp (grid->units, "FEET"))
    out_units = 1;
  else if (!strcmp (grid->units, "METERS"))
    out_units = 2;
  else if (!strcmp (grid->units, "SECONDS"))
    out_units = 3;
  else if (!strcmp (grid->units, "DEGREES"))
    out_units = 4;
  else if (!strcmp (grid->units, "DMS"))
    out_units = 5;

  /* Loop through bands 
   */
  for (curr_band = 0; curr_band < grid->nbands; curr_band++)
  {
    ref = grid->bands[curr_band];

    /* Loop through SCAs
     */
    for( sca=0 ; sca<grid->nscas; sca++ )
    {
      z = malloc( sizeof(double) * grid->gridbands[sca][ref].nzplanes );
      if (z == NULL)
      {
        strcpy (ErrMsg, "Error allocating z array.");
        xxx_LogStatus ("ALIGRID", __FILE__, __LINE__, ErrMsg);
        return (ERROR);
      }
      for( zplane=0 ; zplane<grid->gridbands[sca][ref].nzplanes ; zplane++ )
      {
        elev = ( zplane - grid->gridbands[sca][ref].zeroplane ) * grid->gridbands[sca][ref].zspacing;
        z[zplane] = elev;
      }

      delta_line_even = grid->gridbands[sca][ref].delta_line_even;
      delta_samp_even = grid->gridbands[sca][ref].delta_samp_even;
      delta_line_odd  = grid->gridbands[sca][ref].delta_line_odd;
      delta_samp_odd  = grid->gridbands[sca][ref].delta_samp_odd;

      zline_even = malloc( sizeof(double) * zplane );
      if (zline_even == NULL)
      {
         strcpy (ErrMsg, "Error allocating deltas.");
         xxx_LogStatus ("ALIGRID", __FILE__, __LINE__, ErrMsg);
         return (ERROR);
      }
      zsamp_even = malloc( sizeof(double) * zplane );
      if (zsamp_even == NULL)
      {
         strcpy (ErrMsg, "Error allocating deltas.");
         xxx_LogStatus ("ALIGRID", __FILE__, __LINE__, ErrMsg);
         return (ERROR);
      }
      zline_odd  = malloc( sizeof(double) * zplane );
      if (zline_odd == NULL)
      {
         strcpy (ErrMsg, "Error allocating deltas.");
         xxx_LogStatus ("ALIGRID", __FILE__, __LINE__, ErrMsg);
         return (ERROR);
      }
      zsamp_odd  = malloc( sizeof(double) * zplane );
      if (zsamp_odd == NULL)
      {
         strcpy (ErrMsg, "Error allocating deltas.");
         xxx_LogStatus ("ALIGRID", __FILE__, __LINE__, ErrMsg);
         return (ERROR);
      }

      /* Loop through all of the grid points to get the latitude/longitude,
       * latitude/longitude in output space, and finally the line/sample in
       * output space. Use the ACTUAL detector to do this.
       */
      for (i = 0; i < grid->gridbands[sca][ref].ngrid_lines; i++)
      {
        for (j = 0; j < grid->gridbands[sca][ref].ngrid_samps; j++)
        {
          for( zplane=0 ; zplane<grid->gridbands[sca][ref].nzplanes ; zplane++ )
          {
            elev = ( zplane - grid->gridbands[sca][ref].zeroplane ) * grid->gridbands[sca][ref].zspacing;

            input_nl = grid->gridbands[sca][ref].in_lines[grid->gridbands[sca][ref].ngrid_lines-1];
            input_ns = grid->gridbands[sca][ref].in_samps[grid->gridbands[sca][ref].ngrid_samps-1];
  
            /* Find the closest even and odd detector/sample numbers
             */
            even_samp = 2 * (int)(grid->gridbands[sca][ref].in_samps[j]/2);
            odd_samp = even_samp + 1;
  
            /* Calculate the latitude and longitude of the even detector.
             */
            stat = axx_forward_ali((double)grid->gridbands[sca][ref].in_lines[i], 
                       (double)even_samp, grid->gridbands[sca][ref].bandno, 
                       sca, elev, cpf, alimodel, dettype, &in_lat, &in_long);
            if (stat == ERROR)
            {
              strcpy (ErrMsg, "Error in the forward_ali call.");
              xxx_LogStatus ("ALIGRID", __FILE__, __LINE__, ErrMsg);
              return (ERROR);
            }
      
            /* Calculate the location of the latitude and longitude in output
             * space.
             */
            stat = exx_projtran (&geo, &zero, &null_zone, geo_parms,
                       &grid->spheroid, &grid->code, &out_units,
                       &grid->zone, grid->projprms, &grid->spheroid,
                       &in_long, &in_lat, &proj_x, &proj_y);
            if (stat == ERROR)
            {
              strcpy (ErrMsg, "Error in the projtran call.");
              xxx_LogStatus ("ALIGRID", __FILE__, __LINE__, ErrMsg);
              return (ERROR);
            }
    
            /* Calculate the output line and sample.
             */
            if (grid->frame_type == PATH_ORIENTED || grid->frame_type == PATH_MINBOX || 
                grid->frame_type == PATH_MAXBOX)
            { /* Path-oriented */
              xxx_eval(grid->gridbands[sca][ref].degree, poly_y[polyindex[ref]],
                       proj_x, proj_y, &line);
              xxx_eval(grid->gridbands[sca][ref].degree, poly_x[polyindex[ref]],
                       proj_x, proj_y, &samp);
            }
            else
            { /* North-up */
              line = (grid->corners.upleft.y - proj_y)
                   / grid->gridbands[sca][ref].pixsize;
              samp = (proj_x - grid->corners.upleft.x)
                   / grid->gridbands[sca][ref].pixsize;
            }
            if ( axx_all_ols2ils( &grid->gridbands[sca][ref], &row_ptr, &col_ptr,
                               input_nl, input_ns, line, samp, &iline, &isamp,
                               zplane ) == ERROR )
            {
              strcpy (ErrMsg, "Error mapping the detector back to input space.");
              xxx_LogStatus ("ALIGRID", __FILE__, __LINE__, ErrMsg);
              return(ERROR);
            }
            else
            {
              zline_even[zplane] = (double)grid->gridbands[sca][ref].in_lines[i] - iline;
              zsamp_even[zplane] = (double)even_samp - isamp;
            }
  
            /* Calculate the latitude and longitude of the odd detector.
             */
            stat = axx_forward_ali((double)grid->gridbands[sca][ref].in_lines[i], 
                       (double)odd_samp, grid->gridbands[sca][ref].bandno, 
                       sca, elev, cpf, alimodel, dettype, &in_lat, &in_long);
            if (stat == ERROR)
            {
              strcpy (ErrMsg, "Error in the forward_ali call.");
              xxx_LogStatus ("ALIGRID", __FILE__, __LINE__, ErrMsg);
              return (ERROR);
            }
      
            /* Calculate the location of the latitude and longitude in output
             * space.
             */
            stat = exx_projtran (&geo, &zero, &null_zone, geo_parms,
                       &grid->spheroid, &grid->code, &out_units,
                       &grid->zone, grid->projprms, &grid->spheroid,
                       &in_long, &in_lat, &proj_x, &proj_y);
            if (stat == ERROR)
            {
              strcpy (ErrMsg, "Error in the projtran call.");
              xxx_LogStatus ("ALIGRID", __FILE__, __LINE__, ErrMsg);
              return (ERROR);
            }
    
            /* Calculate the output line and sample.
             */
            if (grid->frame_type == PATH_ORIENTED || grid->frame_type == PATH_MINBOX || 
                grid->frame_type == PATH_MAXBOX)
            { /* Path-oriented */
              xxx_eval(grid->gridbands[sca][ref].degree, poly_y[polyindex[ref]],
                       proj_x, proj_y, &line);
              xxx_eval(grid->gridbands[sca][ref].degree, poly_x[polyindex[ref]],
                       proj_x, proj_y, &samp);
            }
            else
            { /* North-up */
              line = (grid->corners.upleft.y - proj_y)
                   / grid->gridbands[sca][ref].pixsize;
              samp = (proj_x - grid->corners.upleft.x)
                   / grid->gridbands[sca][ref].pixsize;
            }
            if ( axx_all_ols2ils( &grid->gridbands[sca][ref], &row_ptr, &col_ptr,
                               input_nl, input_ns, line, samp, &iline, &isamp,
                               zplane ) == ERROR )
            {
              strcpy (ErrMsg, "Error mapping the detector back to input space.");
              xxx_LogStatus ("ALIGRID", __FILE__, __LINE__, ErrMsg);
              return(ERROR);
            }
            else
            {
              zline_odd[zplane] = (double)grid->gridbands[sca][ref].in_lines[i] - iline;
              zsamp_odd[zplane] = (double)odd_samp - isamp;
            }
          } /* zplanes */

          /* Solve for linear mapping coefficients associated with detector delays. Store
             values into appropriate grid array */

          /* Initialize variables */
          sum_x     = 0.0;
          sum_y     = 0.0;
          sum_x_sqr = 0.0;
          sum_xy    = 0.0;
          /* Find values for LSQ solution */
          for( zplane=0 ; zplane<grid->gridbands[sca][ref].nzplanes ; zplane++ )
          {
             sum_x = sum_x + z[zplane];
             sum_y = sum_y + zline_even[zplane];
             sum_x_sqr = sum_x_sqr + z[zplane] * z[zplane];
             sum_xy = sum_xy + z[zplane] * zline_even[zplane];
          }
          /* Solve for line direction even detector mapping coefficients */
          *delta_line_even = ( sum_x_sqr * sum_y - sum_x * sum_xy ) / 
               ( (double)grid->gridbands[sca][ref].nzplanes * sum_x_sqr - sum_x * sum_x );
          delta_line_even++;
          *delta_line_even = ( (double)grid->gridbands[sca][ref].nzplanes * sum_xy - sum_x * sum_y ) / 
               ( (double)grid->gridbands[sca][ref].nzplanes * sum_x_sqr - sum_x * sum_x );
          delta_line_even++;
    
          /* Initialize variables */
          sum_x     = 0.0;
          sum_y     = 0.0;
          sum_x_sqr = 0.0;
          sum_xy    = 0.0;
          /* Find values for LSQ solution */
          for( zplane=0 ; zplane<grid->gridbands[sca][ref].nzplanes ; zplane++ )
          {
             sum_x = sum_x + z[zplane];
             sum_y = sum_y + zsamp_even[zplane];
             sum_x_sqr = sum_x_sqr + z[zplane] * z[zplane];
             sum_xy = sum_xy + z[zplane] * zsamp_even[zplane];
          }
          /* Solve for sample direction even detector mapping coefficients */
          *delta_samp_even = ( sum_x_sqr * sum_y - sum_x * sum_xy ) /
               ( (double)grid->gridbands[sca][ref].nzplanes * sum_x_sqr - sum_x * sum_x );
          delta_samp_even++;
          *delta_samp_even = ( (double)grid->gridbands[sca][ref].nzplanes * sum_xy - sum_x * sum_y ) /
               ( (double)grid->gridbands[sca][ref].nzplanes * sum_x_sqr - sum_x * sum_x );
          delta_samp_even++;
    
          /* Initialize variables */
          sum_x     = 0.0;
          sum_y     = 0.0;
          sum_x_sqr = 0.0;
          sum_xy    = 0.0;
          /* Find values for LSQ solution */
          for( zplane=0 ; zplane<grid->gridbands[sca][ref].nzplanes ; zplane++ )
          {
             sum_x = sum_x + z[zplane];
             sum_y = sum_y + zline_odd[zplane];
             sum_x_sqr = sum_x_sqr + z[zplane] * z[zplane];
             sum_xy = sum_xy + z[zplane] * zline_odd[zplane];
          }
          /* Solve for line direction odd detector mapping coefficients */
          *delta_line_odd = ( sum_x_sqr * sum_y - sum_x * sum_xy ) /
               ( (double)grid->gridbands[sca][ref].nzplanes * sum_x_sqr - sum_x * sum_x );
          delta_line_odd++;
          *delta_line_odd = ( (double)grid->gridbands[sca][ref].nzplanes * sum_xy - sum_x * sum_y ) /
               ( (double)grid->gridbands[sca][ref].nzplanes * sum_x_sqr - sum_x * sum_x );
          delta_line_odd++;
    
          /* Initialize variables */
          sum_x     = 0.0;
          sum_y     = 0.0;
          sum_x_sqr = 0.0;
          sum_xy    = 0.0;
          /* Find values for LSQ solution */
          for( zplane=0 ; zplane<grid->gridbands[sca][ref].nzplanes ; zplane++ )
          {
             sum_x = sum_x + z[zplane];
             sum_y = sum_y + zsamp_odd[zplane];
             sum_x_sqr = sum_x_sqr + z[zplane] * z[zplane];
             sum_xy = sum_xy + z[zplane] * zsamp_odd[zplane];
          }
          /* Solve for sample direction odd detector mapping coefficients */
          *delta_samp_odd = ( sum_x_sqr * sum_y - sum_x * sum_xy ) /
               ( (double)grid->gridbands[sca][ref].nzplanes * sum_x_sqr - sum_x * sum_x );
          delta_samp_odd++;
          *delta_samp_odd = ( (double)grid->gridbands[sca][ref].nzplanes * sum_xy - sum_x * sum_y ) /
               ( (double)grid->gridbands[sca][ref].nzplanes * sum_x_sqr - sum_x * sum_x );
          delta_samp_odd++;
        } /* ngrid_samps */
      } /* ngrid_lines */
      free(zline_even);
      free(zsamp_even);
      free(zline_odd);
      free(zsamp_odd);
      free(z);
    }
  }

  return (SUCCESS);
}
