/******************************************************************************
NAME:           calc_map_coefs

PURPOSE:
Calculate the forward and inverse mapping coefficients for the grid.


RETURN VALUE:   Type = int
Value    Description
-----    -----------
SUCCESS  Successful completion
ERROR    Operation failed

NOTES:
This routine will find the forward and inverse mapping coefficients for the
grid.  The child routines, calc_forward_mappings and calc_inverse_mappings,
will be called for each band.  The forward and inverse mapping coefficients
will be stored in the grid structure for the appropriate band.

ALGORITHM REFERENCES:
See axx_grid.h for a description of the grid structure.

******************************************************************************/
#include <stdio.h>
#include <unistd.h>
#include <sys/types.h>
#include <stddef.h>
#include "xxx_Types.h"
#include "xxx_LogStatus.h"
#include "xxx_const.h"
#include "axx_grid.h"

int calc_map_coefs (
  axx_GRID_TYPE *grid)  /* I/O: Grid structure                              */
{
  int  ref,             /* corresponding reference position for the band
                           numbers                                          */
       rows,            /* Number of rows in the grid                       */
       cols,            /* Number of cols in the grid                       */
       stat,            /* Return status                                    */
       sca,		/* SCA number					    */
       i;               /* Loop variable                                    */
  char ErrMsg[STRLEN];  /* Error message                                    */

  /* Loop through the bands in the grid.
   */
  for (i = 0; i < grid->nbands; i++)
  {
    ref = grid->bands[i];

    /* Find the number of rows and columns in the grid.
     */
    for( sca=0 ; sca<NSCA ; sca++ )
    {
      rows = grid->gridbands[sca][ref].ngrid_lines-1;
      cols = grid->gridbands[sca][ref].ngrid_samps-1;

      /* Calculate the forward mappings for the current band.
       */
      stat = exx_calc_forward_mappings (grid->gridbands[sca][ref].nzplanes,
             grid->gridbands[sca][ref].in_lines, grid->gridbands[sca][ref].in_samps, 
             grid->gridbands[sca][ref].out_lines, grid->gridbands[sca][ref].out_samps, rows, cols,
             grid->gridbands[sca][ref].sattoproj);
      if (stat != SUCCESS)
      {
        strcpy (ErrMsg, "Error in the calc_forward_mappings routine.");
        xxx_LogStatus ("ALIRID", __FILE__, __LINE__, ErrMsg);
        return (ERROR);
      }

      /* Calculate the inverse mappings for the current band.
       */
      stat = exx_calc_inverse_mappings (grid->gridbands[sca][ref].nzplanes, 
             grid->gridbands[sca][ref].in_lines, grid->gridbands[sca][ref].in_samps, 
             grid->gridbands[sca][ref].out_lines, grid->gridbands[sca][ref].out_samps, rows, cols,
             grid->gridbands[sca][ref].projtosat);
      if (stat != SUCCESS)
      {
        strcpy (ErrMsg, "Error in the calc_inverse_mappings routine.");
        xxx_LogStatus ("ALIGRID", __FILE__, __LINE__, ErrMsg);
        return (ERROR);
      }
    }
  }

  return(SUCCESS);
}
