#include "aligrid.h"

/******************************************************************************
NAME:		calc_path_oriented_frame

PURPOSE:
Given the center point and rotation angle, this function will calculate the
image corner coordinates in an SOM or UTM product.  It also calculates the
first-order polynomial coefficients which map output line/sample coords to
their corresponding output projection coordinates.


RETURN VALUE:
Type = int (SUCCESS or ERROR)

NOTES:
This routine will determine the frame in output space for the path-oriented
product.  The frame is calculated for each band, but the frame must be the
same for every band.  If not, then the application will error out.

ALGORITHM REFERENCES:
******************************************************************************/

int calc_path_oriented_frame(
    axx_GRID_TYPE *grid,	/* I/O: Grid structure			 */
    int  units,			/* I: Projection units			 */
    double poly_x[2][4],        /* O: Coefficients which map output lines  */
    double poly_y[2][4])        /*    & samples to projection coordinates  */
                                /*    in the order of 15m, 30m, 60m        */
{
    double geo_parms[COEFCT],	/* Geographic projection coefficients	 */
           line_center,		/* Center line in output space		 */
	   samp_center,		/* Center sample in output space	 */
	   ul_dx, ul_dy,	/* Upper-left delta meters from center	 */
	   ur_dx, ur_dy,	/* Upper-right delta meters from center  */
	   ll_dx, ll_dy,	/* Lower-left delta meteres from center  */
	   lr_dx, lr_dy,	/* Lower-right delta meteres from center */
	   x, y,		/* Scene center in projection coords	 */
	   x1, y1,		/* Auxiliary point projection coords	 */
	   lat1, lng1,		/* Auxiliary point latitude longitude	 */
	   ul_xp, ul_yp,	/* Upper-left projection coordinates	 */
	   ur_xp, ur_yp,	/* Upper-right projection coordinates	 */
	   ll_xp, ll_yp,	/* Lower-left projection coordinates	 */
	   lr_xp, lr_yp,	/* Lower-right projection coordinates	 */
	   ul_xp_old, ul_yp_old,/* Old upper-left projection coordinates */
	   ur_xp_old, ur_yp_old,/* Old upper-right projection coordinates*/
	   ll_xp_old, ll_yp_old,/* Old lower-left projection coordinates */
	   lr_xp_old, lr_yp_old,/* Old lower-right projection coordinates*/
	   rot_angle,		/* Rotation angle			 */
	   map_angle,		/* Map projection angle true north	 */
	   sin_angle,		/* Sine of the rotation angle		 */
	   cos_angle,		/* Cosine of the rotation angle		 */
           epsilon,             /* Epsilon value to add to the center pt */
	   A[12],		/* Matrix used by QRsolve for LSQ fit	 */
	   v[4];		/* Vector used in above operation	 */
    double px[4],               /* Poly_x used for calculations          */
           py[4];               /* Poly_y used for calculations          */

    int  i, sca, currband,      /* Loop counter				 */
         polyindex,             /* Index into the polynomials for the
                                   correct band resolution               */
         ref,                   /* Reference pos for the current band    */
	 nl, ns,		/* Number of output lines/samples	 */
	 status,		/* Status returned from library funcs	 */
	 zero = 0,		/* Constant of zero			 */
         done[2] = {FALSE, FALSE},  /* 10m, 30m */
                                /* Flag to specify the polys have been
                                   calculated for the current band res   */
         process,               /* Flag to specify the current band
                                   should be processed                   */
	 null_zone = NULLZONE;	/* Null zone code for projection xforms	 */

    int  deg_units = 4;		/* Unit code for degrees		 */
    char ErrMsg[STRLEN];        /* Error message                         */

    /* Loop through all the bands.
    ------------------------------*/
    for (currband = 0; currband < grid->nbands; currband++)
    {
      /* Get the reference position and the number of lines and samples
         in the current band resolution.
      -----------------------------------------------------------------*/
      ref = grid->bands[currband];
      for(sca = 0; sca < grid->nscas; sca++)
      {
        nl = grid->gridbands[sca][ref].lines;
        ns = grid->gridbands[sca][ref].samps;

        /* Initialize geo_parms and find the center point.
        --------------------------------------------------*/
        for (i = 0; i < COEFCT; geo_parms[i++] = 0.0);
        line_center = (double) (nl - 1.0) / 2.0;
        samp_center = (double) (ns - 1.0) / 2.0;
  
        /* Call projtran to find the projection coordinates of scene center.
        --------------------------------------------------------------------*/
        status = exx_projtran(&zero, &deg_units, &null_zone, geo_parms,
		      &(grid->spheroid), &(grid->code), &units,
		      &(grid->zone), grid->projprms, &(grid->spheroid),
		      &(grid->center_pt.lng), &(grid->center_pt.lat), &x, &y);
        if (status == ERROR)
        {
          strcpy (ErrMsg, "Error in call to projtran.");
          xxx_LogStatus ("TMGRID", __FILE__, __LINE__, ErrMsg);
          return (ERROR);
        }
  
        /* Construct a point just north of the scene center for use in
           computing the angle between true north and grid north.
        --------------------------------------------------------------------*/
        epsilon = 0.00001;         /* add to center to get point just north */
        lat1 = grid->center_pt.lat + epsilon;
        lng1 = grid->center_pt.lng;
        status = exx_projtran(&zero, &deg_units, &null_zone, geo_parms,
                      &(grid->spheroid), &(grid->code), &units,
                      &(grid->zone), grid->projprms, &(grid->spheroid),
                      &lng1, &lat1, &x1, &y1 );
        if (status == ERROR)
        {
          strcpy (ErrMsg, "Error in call to projtran.");
          xxx_LogStatus ("TMGRID", __FILE__, __LINE__, ErrMsg);
          return (ERROR);
        }
  
        /* Compute the grid rotation angle.
        -----------------------------------*/
        map_angle = atan2( x1-x, y1-y );
   
        /* Precompute the sine and cosine of the rotation angle.
        --------------------------------------------------------*/
        rot_angle = (grid->rotate_angle) * D2R + map_angle;
        sin_angle = sin (rot_angle);
        cos_angle = cos (rot_angle);
  
        /* Calculate the upper left coordinates.
        ----------------------------------------*/
        ul_dx = -samp_center * grid->gridbands[sca][ref].pixsize;
        ul_dy = line_center * grid->gridbands[sca][ref].pixsize;
        ul_xp =  ul_dx * cos_angle + ul_dy * sin_angle + x;
        ul_yp = -ul_dx * sin_angle + ul_dy * cos_angle + y;
  
        /* Calculate the upper right coordinates.
        -----------------------------------------*/
        ur_dx = ((ns-1.0) - samp_center) * grid->gridbands[sca][ref].pixsize;
        ur_dy = line_center * grid->gridbands[sca][ref].pixsize;
        ur_xp =  ur_dx * cos_angle + ur_dy * sin_angle + x;
        ur_yp = -ur_dx * sin_angle + ur_dy * cos_angle + y;
  
        /* Calculate the lower right coordinates.
        -----------------------------------------*/
        lr_dx = ((ns-1.0) - samp_center) * grid->gridbands[sca][ref].pixsize;
        lr_dy = (line_center - (nl-1.0)) * grid->gridbands[sca][ref].pixsize;
        lr_xp =  lr_dx * cos_angle + lr_dy * sin_angle + x;
        lr_yp = -lr_dx * sin_angle + lr_dy * cos_angle + y;
  
        /* Calculate the lower left coordinates.
        ----------------------------------------*/
        ll_dx = -samp_center * grid->gridbands[sca][ref].pixsize;
        ll_dy = (line_center - (nl-1.0)) * grid->gridbands[sca][ref].pixsize;
        ll_xp =  ll_dx * cos_angle + ll_dy * sin_angle + x;
        ll_yp = -ll_dx * sin_angle + ll_dy * cos_angle + y;
  
        /* Check these corner points with the last band's corner point.
        ---------------------------------------------------------------*/
        if (currband == 0 && grid->nbands != 1)
        { /* If only 1 band then don't keep track of each band corner.  If
           * the first band then set the old vals = current vals.
           */
          ul_xp_old = ul_xp;
          ul_yp_old = ul_yp;
          ur_xp_old = ur_xp;
          ur_yp_old = ur_yp;
          ll_xp_old = ll_xp;
          ll_yp_old = ll_yp;
          lr_xp_old = lr_xp;
          lr_yp_old = lr_yp;
        }
        else if ((ul_xp != ul_xp_old || ul_yp != ul_yp_old ||
                 ur_xp != ur_xp_old || ur_yp != ur_yp_old ||
                 ll_xp != ll_xp_old || ll_yp != ll_yp_old ||
                 lr_xp != lr_xp_old || lr_yp != lr_yp_old) &&
                 grid->nbands != 1)
        {
          sprintf (ErrMsg,
            "The corner points for bands %d and %d do not match.\n",
            axx_get_user_band (grid->bands[currband]),
            axx_get_user_band (grid->bands[0]));
          xxx_LogStatus ("TMGRID", __FILE__, __LINE__, ErrMsg);
          return (ERROR);
        }
  
        /* Calculate the polynomials for all available band resolutions.
           Check to see if the current band's resolution needs to be processed.
           If so, then set the px and py pointers to the appropriate band res
           position.
        ----------------------------------------------------------------------*/
        process = FALSE;
        if ((grid->bands[currband] == pan) && (!done[0]))
        {
          process = TRUE;
          polyindex = 0;   /* Index into poly_x for 10m */
          done[0] = TRUE;
        }
        else 
        {
          process = TRUE;
          polyindex = 1;   /* Index into poly_x for 30m */
          done[1] = TRUE;
        }
  
        if (process)
        {
  
          /* Now that we have calculated the corners in projection coordinates,
             calculate the first order polynomial coefficients which map output
             line/samples to output projection coordinates.  The 4x3 matrix is
             populated as shown below, QR-factorized and then Ax = b is solved
             for two different vectors b; one for samples, the other for lines.
          ---------------------------------------------------------------------*/
          for (i = 0; i < 4; A[i++] = 1.0);
          A[4]  = ul_xp;
          A[5]  = ur_xp;	/*     +-                                 -+ */
          A[6]  = lr_xp;	/*     |  1  upper-left  X  upper-left  Y  | */
          A[7]  = ll_xp;	/* A = |  1  upper-right X  upper-right Y  | */
          A[8]  = ul_yp;	/*     |  1  lower-right X  lower-right Y  | */
          A[9]  = ur_yp;	/*     |  1  lower-left  X  lower-left  Y  | */
          A[10] = lr_yp;	/*     +-                                 -+ */
          A[11] = ll_yp;
          xxx_QR(A, 4, 3, v, 0); /* Obtain QR factorization */
  
          /* Vector one: Find the coefficients to transform output samples
             to output coordinates in projection space.  Note that the call
             to xxx_QRsolve() modifies the values at poly_x for part of the
             output (this vector becomes the before-mentioned coefficients.)
          ------------------------------------------------------------------*/
          px[0] = 0.0;
          px[1] = ns - 1.0;
          px[2] = ns - 1.0;
          px[3] = 0.0;
          xxx_QRsolve(A, 4, 3, v, px, 0);
  
          /* Vector two: Find the coefficients to transform output lines
             to output coordinates in projection space.
          --------------------------------------------------------------*/
          py[0] = 0.0;
          py[1] = 0.0;
          py[2] = nl - 1.0;
          py[3] = nl - 1.0;
          xxx_QRsolve(A, 4, 3, v, py, 0);
  
          /* Copy the poly's into their respective band resolution.
          ---------------------------------------------------------*/
          for (i = 0; i < 4; i++)
          {
            poly_x[polyindex][i] = px[i];
            poly_y[polyindex][i] = py[i];
          }
        }
      }
    }

    /* Store the corners in the grid structure.
    -------------------------------------------*/
    grid->corners.upleft.x  = ul_xp;
    grid->corners.upleft.y  = ul_yp;
    grid->corners.upright.x = ur_xp;
    grid->corners.upright.y = ur_yp;
    grid->corners.loleft.x  = ll_xp;
    grid->corners.loleft.y  = ll_yp;
    grid->corners.loright.x = lr_xp;
    grid->corners.loright.y = lr_yp;
    grid->proj_center_pt.x = x;
    grid->proj_center_pt.y = y;

    return SUCCESS;
}
