/******************************************************************************
NAME: calc_path_maxbox_oriented_frame

PURPOSE: This routine calculates the path-oriented frame for maxbox.

AUTHOR        Change-ID    DATE     Description of Change
------        ---------  --------   ---------------------
T.Ratliff     1.0         01/01     Original development

RETURN VALUE:   Type = int
Value    Description
-----    -----------
SUCCESS  Successful completion
ERROR    Operation failed

NOTES:
none

ALGORITHM REFERENCES:
This algorithm was created by combining the logic in the in both
path_oriented.c and minbox.c.  Almost all the code is from these
two modules.
******************************************************************************/
#include "aligrid.h"

int calc_path_oriented_maxbox_frame (
  ALIMODEL *amodel,            /* I: ALI model structure                 */
  int  *num_0r_lines, /* I: Number of lines in the 0R image for each band */
  int  *num_0r_samps, /* I: Number of samps in the 0R image for each band */
  CPF *cpf,                  /* I: CPF constants                        */
  axx_GRID_TYPE *grid,       /* I/O: Grid structure                     */
  int  units,                /* I: Projection units                     */
  double poly_x[2][4],       /* O: Coefficients which map output lines  */
  double poly_y[2][4])       /*    & samples to projection coordinates  */
                             /*    in the order of 10m,30m */
{
  int  i,               /* Loop control                                  */
       nl,              /* Number of lines in the scene                  */
       ns,              /* Number of samples in the scene                */
       out_units,       /* Code for the output projection units.         */
       stat,            /* Function return status                        */
       geo = 0,         /* Geographic projection flag                    */
       zero = 0,        /* Constant of zero                              */
       null_zone = NULLZONE, /* Null zone code for projection xforms     */
       ref,             /* Reference of the first band number.           */
       zplane,          /* Counter for elevation planes                  */
       sca,		/* SCA counter					 */
       currref;         /* Current reference for looping through the bands.*/
  double pxmax,         /* Maximum X projection coordinate               */
         pxmin,         /* Minimum X projection coordinate               */
         pymax,         /* Maximum Y projection coordinate               */
         pymin;         /* Minimum Y projection coordinate               */
  double elev = 0.0;    /* Elevation at the current point                */
  char ErrMsg[STRLEN];  /* Error message                                 */
  struct DBL_LS ul_ls,  /* Upper left line/sample for the corners        */
                ur_ls,  /* Upper right line/sample for the corners       */
                ll_ls,  /* Lower left line/sample for the corners        */
                lr_ls;  /* Lower right line/sample for the corners       */
  struct DBL_LAT_LONG
                ul_ltlg,  /* Upper left lat/long for the corners         */
                ur_ltlg,  /* Upper right lat/long for the corners        */
                ll_ltlg,  /* Lower left lat/long for the corners         */
                lr_ltlg;  /* Lower right lat/long for the corners        */
  struct DBL_XY ul_xy,    /* Upper left x/y for the corners              */
                ur_xy,    /* Upper right x/y for the corners             */
                ll_xy,    /* Lower left x/y for the corners              */
                lr_xy;    /* Lower right x/y for the corners             */
  double geo_parms[COEFCT],   /* Geographic projection coefficients    */
           line_center,         /* Center line in output space           */
           samp_center,         /* Center sample in output space         */
           ul_dx, ul_dy,        /* Upper-left delta meters from center   */
           ur_dx, ur_dy,        /* Upper-right delta meters from center  */
           ll_dx, ll_dy,        /* Lower-left delta meteres from center  */
           lr_dx, lr_dy,        /* Lower-right delta meteres from center */
           x, y,                /* Scene center in projection coords     */
           x1, y1,              /* Auxiliary point projection coords     */
           lat1, lng1,          /* Auxiliary point latitude longitude    */
           ul_xp, ul_yp,        /* Upper-left projection coordinates     */
           ur_xp, ur_yp,        /* Upper-right projection coordinates    */
           ll_xp, ll_yp,        /* Lower-left projection coordinates     */
           lr_xp, lr_yp,        /* Lower-right projection coordinates    */
           ul_xp_old, ul_yp_old,/* Old upper-left projection coordinates */
           ur_xp_old, ur_yp_old,/* Old upper-right projection coordinates*/
           ll_xp_old, ll_yp_old,/* Old lower-left projection coordinates */
           lr_xp_old, lr_yp_old,/* Old lower-right projection coordinates*/
           rot_angle,           /* Rotation angle                        */
           map_angle,           /* Map projection angle true north       */
           sin_angle,           /* Sine of the rotation angle            */
           cos_angle,           /* Cosine of the rotation angle          */
           epsilon,             /* Epsilon value to add to the center pt */
           A[12],               /* Matrix used by QRsolve for LSQ fit    */
           v[4];                /* Vector used in above operation        */
    double px[4],               /* Poly_x used for calculations          */
           py[4];               /* Poly_y used for calculations          */
    int  currband,              /* Loop counter                          */
         polyindex,             /* Index into the polynomials for the
                                   correct band resolution               */
         status;                /* Status returned from library funcs    */
    int  deg_units = 4;         /* Unit code for degrees                 */
    int	 out_nl,		/* Number of lines for optimal band */
	 out_ns;		/* Number of sample for optimal band */
    double dx,			/* Sample/X change due to rotation	 */
         dy,			/* Line/Y change due to rotation	 */
         xp,			/* New sample/X due to rotation		 */
         yp;			/* New line/Y due to rotation		 */
    double line, samp;
    int polyindex_array[10] = {0, 1, 1, 1, 1, 1, 1, 1, 1, 1};
    double nom_center_lng,	/* Center Longitude of optimal band	 */
         nom_center_lat,	/* Center Latitude of optimal band	 */
         nom_proj_x,		/* Center X coord of optimal band	 */
         nom_proj_y; 		/* Center Y coord of optimal band	 */
    int pad_nl,			/* Padding of lines due to WRS/center offset */
        pad_ns;			/* Padding of samples due to WRS/center offset */
    double left_samp,		/* Left sample location for center line of otpmal band */
        right_samp;		/* Right sample location for center line of otpmal band */
    double pad_x,
        pad_y,
        pixsize;		/* Pixel size to be used for padding */
    int current_band;		/* Current band being checked */
    BandNumber bandno;		/* Enumerated band number */
    DetType dettype = NOMINAL;  /* Type of detector type to use */
    int process;		/* Flag indicating to find band polynomails */
    int done[2] = {FALSE, FALSE};/* Flag indicating band polys have been calc. */


  /* Initialize geo_parms
  -----------------------*/
  for (i = 0; i < COEFCT; i++)
    geo_parms[i] = 0.0;

  /* Convert the grid projection units to a unit code for gctp
  ------------------------------------------------------------*/
  if (!strcmp (grid->units, "RADIANS"))
    out_units = 0;
  else if (!strcmp (grid->units, "FEET"))
    out_units = 1;
  else if (!strcmp (grid->units, "METERS"))
    out_units = 2;
  else if (!strcmp (grid->units, "SECONDS"))
    out_units = 3;
  else if (!strcmp (grid->units, "DEGREES"))
    out_units = 4;
  else if (!strcmp (grid->units, "DMS"))
    out_units = 5;

  /* Call projtran to find the projection coordinates of WRS scene center
  -----------------------------------------------------------------------*/
  status = exx_projtran(&zero, &deg_units, &null_zone, geo_parms,
                &(grid->spheroid), &(grid->code), &out_units,
                &(grid->zone), grid->projprms, &(grid->spheroid),
                &(grid->center_pt.lng), &(grid->center_pt.lat), &x, &y);
  if (status == ERROR)
  {
    strcpy (ErrMsg, "Error in call to projtran.");
    xxx_LogStatus ("ALIGRID", __FILE__, __LINE__, ErrMsg);
    return (ERROR);
  }

  /* Construct a point just north of the scene center for use in
     computing the angle between true north and grid north.
  --------------------------------------------------------------------*/
  epsilon = 0.00001;         /* add to center to get point just north */
  lat1 = grid->center_pt.lat + epsilon;
  lng1 = grid->center_pt.lng;
  status = exx_projtran(&zero, &deg_units, &null_zone, geo_parms,
                &(grid->spheroid), &(grid->code), &units,
                &(grid->zone), grid->projprms, &(grid->spheroid),
                &lng1, &lat1, &x1, &y1 );
  if (status == ERROR)
  {
    strcpy (ErrMsg, "Error in call to projtran.");
    xxx_LogStatus ("ALIGRID", __FILE__, __LINE__, ErrMsg);
    return (ERROR);
  }

  /* Compute the grid rotation angle.
  -----------------------------------*/
  map_angle = atan2( x1-x, y1-y );

  /* Precompute the sine and cosine of the rotation angle.
  --------------------------------------------------------*/
  rot_angle = (grid->rotate_angle) * D2R + map_angle;
  sin_angle = sin (rot_angle);
  cos_angle = cos (rot_angle);

  for( sca=0 ; sca<grid->nscas ; sca++ )
  {
     for( current_band=0 ; current_band<grid->nbands ; current_band++ )
     {
        /* Choose biggest pixesize to buffer imagery */
        if( current_band==0 )
        {
           pixsize = grid->gridbands[sca][current_band].pixsize;
           ref = current_band;
        }
        else if( pixsize < grid->gridbands[sca][current_band].pixsize )
        {
           pixsize = grid->gridbands[sca][current_band].pixsize;
           ref = current_band;
        }
   
        for( zplane=0 ; zplane<grid->gridbands[sca][ref].nzplanes ; zplane++ )
        {
           elev = ( zplane - grid->gridbands[sca][current_band].zeroplane ) * 
                    grid->gridbands[sca][current_band].zspacing;
  
          /* Find the SLO offsets for the first and last lines in the scene.  These
           * will determine the corner points in line/sample.
           */
          nl = num_0r_lines[current_band];
          ns = num_0r_samps[current_band];
        
          samp_center = ( ns - 1.0 ) / 2.0;
          line_center = ( nl - 1.0 ) / 2.0;
        
          stat = axx_forward_ali(line_center, samp_center, grid->gridbands[sca][current_band].bandno, 
                                 sca, elev, cpf, amodel, dettype, &nom_center_lat, &nom_center_lng);
          if (stat == ERROR)
          {
            strcpy (ErrMsg, "Error in the forward_etm call.");
            xxx_LogStatus ("ALIGRID", __FILE__, __LINE__, ErrMsg);
            return (ERROR);
          }
        
          stat = exx_projtran (&geo, &zero, &null_zone, geo_parms,
                 &grid->spheroid, &grid->code, &out_units, &grid->zone,
                 grid->projprms, &grid->spheroid, &nom_center_lng, &nom_center_lat,
                 &nom_proj_x, &nom_proj_y);
          if (stat == ERROR)
          {
            strcpy (ErrMsg, "Error in the projtran call.");
            xxx_LogStatus ("ALIGRID", __FILE__, __LINE__, ErrMsg);
            return (ERROR);
          }
        
          /* Find the sample values for the corner points.
           ---------------------------------------------*/
          ul_ls.samp = 0;
          ll_ls.samp = 0;
          ur_ls.samp = ns - 1;
          lr_ls.samp = ns - 1;
        
          /* Find the top and bottom line values for the corner points.
           ---------------------------------------------------------*/
          ul_ls.line = ur_ls.line = 0;
          ll_ls.line = lr_ls.line = nl - 1;
          
          /* Find corner points Lat/Long projection values.
          ------------------------------------------------*/
          stat = axx_forward_ali(ul_ls.line, ul_ls.samp, grid->gridbands[sca][current_band].bandno, 
                                 sca, elev, cpf, amodel, dettype, &ul_ltlg.lat, &ul_ltlg.lng);
          if (stat == ERROR)
          {
            strcpy (ErrMsg, "Error in the forward_etm call.");
            xxx_LogStatus ("ALIGRID", __FILE__, __LINE__, ErrMsg);
            return (ERROR);
          }
        
          stat = axx_forward_ali(ur_ls.line, ur_ls.samp, grid->gridbands[sca][current_band].bandno, 
                                 sca, elev, cpf, amodel, dettype, &ur_ltlg.lat, &ur_ltlg.lng);
          if (stat == ERROR)
          {
            strcpy (ErrMsg, "Error in the forward_etm call.");
            xxx_LogStatus ("ALIGRID", __FILE__, __LINE__, ErrMsg);
            return (ERROR);
          }
        
          stat = axx_forward_ali(ll_ls.line, ll_ls.samp, grid->gridbands[sca][current_band].bandno, 
                                 sca, elev, cpf, amodel, dettype, &ll_ltlg.lat, &ll_ltlg.lng);
          if (stat == ERROR)
          {
            strcpy (ErrMsg, "Error in the forward_etm call.");
            xxx_LogStatus ("ALIGRID", __FILE__, __LINE__, ErrMsg);
            return (ERROR);
          }
        
          stat = axx_forward_ali(lr_ls.line, lr_ls.samp, grid->gridbands[sca][current_band].bandno, 
                                 sca, elev, cpf, amodel, dettype, &lr_ltlg.lat, &lr_ltlg.lng);
          if (stat == ERROR)
          {
            strcpy (ErrMsg, "Error in the forward_etm call.");
            xxx_LogStatus ("ALIGRID", __FILE__, __LINE__, ErrMsg);
            return (ERROR);
          }
        
          /* Calculate the location of the latitude and longitude in output
           * space.  (Units coming out of forward_etm are radians = 0.)
           --------------------------------------------------------------*/
          stat = exx_projtran (&geo, &zero, &null_zone, geo_parms,
                 &grid->spheroid, &grid->code, &out_units, &grid->zone,
                 grid->projprms, &grid->spheroid, &ul_ltlg.lng, &ul_ltlg.lat,
                 &ul_xy.x, &ul_xy.y);
          if (stat == ERROR)
          {
            strcpy (ErrMsg, "Error in the projtran call.");
            xxx_LogStatus ("ALIGRID", __FILE__, __LINE__, ErrMsg);
            return (ERROR);
          }
        
          stat = exx_projtran (&geo, &zero, &null_zone, geo_parms,
                 &grid->spheroid, &grid->code, &out_units, &grid->zone,
                 grid->projprms, &grid->spheroid, &ur_ltlg.lng, &ur_ltlg.lat,
                 &ur_xy.x, &ur_xy.y);
          if (stat == ERROR)
          {
            strcpy (ErrMsg, "Error in the projtran call.");
            xxx_LogStatus ("ALIGRID", __FILE__, __LINE__, ErrMsg);
            return (ERROR);
          }
        
          stat = exx_projtran (&geo, &zero, &null_zone, geo_parms,
                 &grid->spheroid, &grid->code, &out_units, &grid->zone,
                 grid->projprms, &grid->spheroid, &ll_ltlg.lng, &ll_ltlg.lat,
                 &ll_xy.x, &ll_xy.y);
          if (stat == ERROR)
          {
            strcpy (ErrMsg, "Error in the projtran call.");
            xxx_LogStatus ("ALIGRID", __FILE__, __LINE__, ErrMsg);
            return (ERROR);
          }
        
          stat = exx_projtran (&geo, &zero, &null_zone, geo_parms,
                 &grid->spheroid, &grid->code, &out_units, &grid->zone,
                 grid->projprms, &grid->spheroid, &lr_ltlg.lng, &lr_ltlg.lat,
                 &lr_xy.x, &lr_xy.y);
          if (stat == ERROR)
          {
            strcpy (ErrMsg, "Error in the projtran call.");
            xxx_LogStatus ("ALIGRID", __FILE__, __LINE__, ErrMsg);
            return (ERROR);
          }
        
          /* Find the minimum and maximum x and y corners.  The corners
          need to be rotated to a space that is orthogonal to the
             path oriented X/Y axis.
          -----------------------------------------------------------*/
          dx =  (ul_xy.x - x);
          dy =  (ul_xy.y - y);
          xp =  dx * cos_angle - dy * sin_angle + x;
          yp =  dx * sin_angle + dy * cos_angle + y;
          if( current_band==0 && sca==0 && zplane==0 )
          {
             pxmin = xp;
             pymin = yp;
             pxmax = xp;
             pymax = yp;
          }
          else
          {
             if( xp > pxmax )
                pxmax = xp;
             if( xp < pxmin )
                pxmin = xp;
             if( yp > pymax )
                pymax = yp;
             if( yp < pymin )
                pymin = yp;
          }
        
          dx =  (ur_xy.x - x);
          dy =  (ur_xy.y - y);
          xp =  dx * cos_angle - dy * sin_angle + x;
          yp =  dx * sin_angle + dy * cos_angle + y;
          if( xp > pxmax )
            pxmax = xp;
          if( xp < pxmin )
            pxmin = xp;
          if( yp > pymax )
            pymax = yp;
          if( yp < pymin )
            pymin = yp;
        
          dx =  (lr_xy.x - x);
          dy =  (lr_xy.y - y);
          xp =  dx * cos_angle - dy * sin_angle + x;
          yp =  dx * sin_angle + dy * cos_angle + y;
          if( xp > pxmax )
            pxmax = xp;
          if( xp < pxmin )
            pxmin = xp;
          if( yp > pymax )
            pymax = yp;
          if( yp < pymin )
            pymin = yp;
        
          dx =  (ll_xy.x - x);
          dy =  (ll_xy.y - y);
          xp =  dx * cos_angle - dy * sin_angle + x;
          yp =  dx * sin_angle + dy * cos_angle + y;
          if( xp > pxmax )
            pxmax = xp;
          if( xp < pxmin )
            pxmin = xp;
          if( yp > pymax )
            pymax = yp;
          if( yp < pymin )
            pymin = yp;
     
          /* Pad according to nominal scene center to WRS center.
           ----------------------------------------------------*/
          dx =  (nom_proj_x - x);
          dy =  (nom_proj_y - y);
          xp =  dx * cos_angle - dy * sin_angle;
          yp =  dx * sin_angle + dy * cos_angle;
          if( current_band==0 )
          {
             pad_y = yp;
             pad_x = xp;
          }
          else 
          {
             if( xp > pad_x )
                pad_x = xp;
             if( yp > pad_y )
                pad_y = yp;
          }
        } /* Z plane loop */
     } /* Band loop */
  } /* SCA loop */ 

  /* Pad corners to PADVAL * pixel size.
   ------------------------------------*/
  stat = pad_corners (pixsize, &pxmax, &pxmin, &pymax, &pymin);
  if (stat == ERROR)
  {
    strcpy (ErrMsg, "Error in the pad_corners call.");
    xxx_LogStatus ("ALIGRID", __FILE__, __LINE__, ErrMsg);
    return (ERROR);
  }

  /* Calculate output lines and samples according to new max/min.
   --------------------------------------------------------------*/
  out_nl = (int) (((pymax - pymin) / pixsize) + 1.5);
  out_ns = (int) (((pxmax - pxmin) / pixsize) + 1.5);

  /* Pad according to nominal scene center to WRS center. 
   ----------------------------------------------------*/
  pad_nl = (int) ((fabs(pad_y) / pixsize) + 1.5);
  pad_ns = (int) ((fabs(pad_x) / pixsize) + 1.5);
  out_nl = out_nl + 2 * pad_nl;
  out_ns = out_ns + 2 * pad_ns;

  /* Make sure output lines and samples are an odd number 
   ----------------------------------------------------*/
  if( out_nl%2==0 )
     out_nl = out_nl + 1;
  if( out_ns%2==0 )
     out_ns = out_ns + 1;

  /* Store NL and NS in grid structure.
   -----------------------------------*/
  for( sca=0 ; sca<grid->nscas ; sca++ )
  {
     for (i = 0; i < grid->nbands; i++)
     {
       currref = grid->bands[i];
       bandno = grid->gridbands[sca][currref].bandno;
       grid->gridbands[sca][currref].lines = (int) ((grid->gridbands[sca][ref].pixsize / 
             grid->gridbands[sca][bandno].pixsize) * (out_nl - 1)) + 1;
       grid->gridbands[sca][currref].samps = (int) ((grid->gridbands[sca][ref].pixsize / 
             grid->gridbands[sca][bandno].pixsize) * (out_ns - 1)) + 1;
     }
  }

  sin_angle = sin (rot_angle);
  cos_angle = cos (rot_angle);

  for( sca=0 ; sca<grid->nscas ; sca++ )
  {
    /* Loop through all the bands.
    ------------------------------*/
    for (currband = 0; currband < grid->nbands; currband++)
    {
      /* Get the reference position and the number of lines and samples
         in the current band resolution.
      -----------------------------------------------------------------*/
      ref = grid->bands[currband];
      nl = grid->gridbands[sca][ref].lines;
      ns = grid->gridbands[sca][ref].samps;
  
      /* Initialize geo_parms and find the center point.
      --------------------------------------------------*/
      for (i = 0; i < COEFCT; geo_parms[i++] = 0.0);
      line_center = (double) (nl - 1.0) / 2.0;
      samp_center = (double) (ns - 1.0) / 2.0;
  
      /* Calculate the upper left coordinates.
      ----------------------------------------*/
      ul_dx = -samp_center * grid->gridbands[sca][ref].pixsize;
      ul_dy = line_center * grid->gridbands[sca][ref].pixsize;
      ul_xp =  ul_dx * cos_angle + ul_dy * sin_angle + x;
      ul_yp = -ul_dx * sin_angle + ul_dy * cos_angle + y;
  
      /* Calculate the upper right coordinates.
      -----------------------------------------*/
      ur_dx = ((ns-1.0) - samp_center) * grid->gridbands[sca][ref].pixsize;
      ur_dy = line_center * grid->gridbands[sca][ref].pixsize;
      ur_xp =  ur_dx * cos_angle + ur_dy * sin_angle + x;
      ur_yp = -ur_dx * sin_angle + ur_dy * cos_angle + y;
  
      /* Calculate the lower right coordinates.
      -----------------------------------------*/
      lr_dx = ((ns-1.0) - samp_center) * grid->gridbands[sca][ref].pixsize;
      lr_dy = (line_center - (nl-1.0)) * grid->gridbands[sca][ref].pixsize;
      lr_xp =  lr_dx * cos_angle + lr_dy * sin_angle + x;
      lr_yp = -lr_dx * sin_angle + lr_dy * cos_angle + y;
  
      /* Calculate the lower left coordinates.
      ----------------------------------------*/
      ll_dx = -samp_center * grid->gridbands[sca][ref].pixsize;
      ll_dy = (line_center - (nl-1.0)) * grid->gridbands[sca][ref].pixsize;
      ll_xp =  ll_dx * cos_angle + ll_dy * sin_angle + x;
      ll_yp = -ll_dx * sin_angle + ll_dy * cos_angle + y;
  
      /* Check these corner points with the last band's corner point.
      ---------------------------------------------------------------*/
      if (currband == 0 && grid->nbands != 1)
      { /* If only 1 band then don't keep track of each band corner.  If
         * the first band then set the old vals = current vals.
         */
        ul_xp_old = ul_xp;
        ul_yp_old = ul_yp;
        ur_xp_old = ur_xp;
        ur_yp_old = ur_yp;
        ll_xp_old = ll_xp;
        ll_yp_old = ll_yp;
        lr_xp_old = lr_xp;
        lr_yp_old = lr_yp;
      }
      else if ((ul_xp != ul_xp_old || ul_yp != ul_yp_old ||
               ur_xp != ur_xp_old || ur_yp != ur_yp_old ||
               ll_xp != ll_xp_old || ll_yp != ll_yp_old ||
               lr_xp != lr_xp_old || lr_yp != lr_yp_old) &&
               grid->nbands != 1)
      {
        sprintf (ErrMsg,
          "The corner points for bands %d and %d do not match.\n",
          axx_get_user_band (grid->bands[currband]),
          axx_get_user_band (grid->bands[0]));
        xxx_LogStatus ("ALIGRID", __FILE__, __LINE__, ErrMsg);
        return (ERROR);
      }
  
      /* Calculate the polynomials for all available band resolutions.
          Check to see if the current band's resolution needs to be processed.
          If so, then set the px and py pointers to the appropriate band res
          position.
      ----------------------------------------------------------------------*/
      process = FALSE;
      if ((grid->bands[currband] == pan) && (!done[0]))
      {
         process = TRUE;
         polyindex = 0;   /* Index into poly_x for 15m */
         done[0] = TRUE;
      }
      else
      {
         process = TRUE;
         polyindex = 1;   /* Index into poly_x for 30m */
         done[1] = TRUE;
      }
  
      /* Now that we have calculated the corners in projection coordinates,
         calculate the first order polynomial coefficients which map output
         line/samples to output projection coordinates.  The 4x3 matrix is
         populated as shown below, QR-factorized and then Ax = b is solved
         for two different vectors b; one for samples, the other for lines.
      ---------------------------------------------------------------------*/
  
      if( process )
      {
         for (i = 0; i < 4; A[i++] = 1.0);
         A[4]  = ul_xp;
         A[5]  = ur_xp;  /*     +-                                 -+ */
         A[6]  = lr_xp;  /*     |  1  upper-left  X  upper-left  Y  | */
         A[7]  = ll_xp;  /* A = |  1  upper-right X  upper-right Y  | */
         A[8]  = ul_yp;  /*     |  1  lower-right X  lower-right Y  | */
         A[9]  = ur_yp;  /*     |  1  lower-left  X  lower-left  Y  | */
         A[10] = lr_yp;  /*     +-                                 -+ */
         A[11] = ll_yp;
         xxx_QR(A, 4, 3, v, 0); /* Obtain QR factorization */
     
         /* Vector one: Find the coefficients to transform output samples
            to output coordinates in projection space.  Note that the call
            to xxx_QRsolve() modifies the values at poly_x for part of the
            output (this vector becomes the before-mentioned coefficients.)
         ------------------------------------------------------------------*/
         px[0] = 0.0;
         px[1] = ns - 1.0;
         px[2] = ns - 1.0;
         px[3] = 0.0;
         xxx_QRsolve(A, 4, 3, v, px, 0);
     
         /* Vector two: Find the coefficients to transform output lines
            to output coordinates in projection space.
         --------------------------------------------------------------*/
         py[0] = 0.0;
         py[1] = 0.0;
         py[2] = nl - 1.0;
         py[3] = nl - 1.0;
         xxx_QRsolve(A, 4, 3, v, py, 0);
     
         /* Copy the poly's into their respective band resolution.
         ---------------------------------------------------------*/
         for (i = 0; i < 4; i++)
         {
           poly_x[polyindex][i] = px[i];
           poly_y[polyindex][i] = py[i];
         }
         xxx_eval(grid->gridbands[sca][ref].degree, poly_y[polyindex_array[ref]], ul_xp, ul_yp, &line);
         xxx_eval(grid->gridbands[sca][ref].degree, poly_x[polyindex_array[ref]], ul_xp, ul_yp, &samp);
       }
    } /* Band counter */
  } /* SCA counter */

  /* Store the corners in the grid structure.
  -------------------------------------------*/
  grid->corners.upleft.x  = ul_xp;
  grid->corners.upleft.y  = ul_yp;
  grid->corners.upright.x = ur_xp;
  grid->corners.upright.y = ur_yp;
  grid->corners.loleft.x  = ll_xp;
  grid->corners.loleft.y  = ll_yp;
  grid->corners.loright.x = lr_xp;
  grid->corners.loright.y = lr_yp;
  grid->proj_center_pt.x = x;
  grid->proj_center_pt.y = y;

  return (SUCCESS);
}
