#include "aligrid.h"

/******************************************************************************
NAME:           det_opt_band

PURPOSE:
To determine the optimal band in the specified list of bands.

RETURN VALUE:   Type = int
Value    Description
-----    -----------
SUCCESS  Successful completion
ERROR    Operation failed

NOTES:
  For path-oriented and minbox framing, the optimal band to use is
  band 4.  If band 4 is not in the list of bands to process for the
  grid, then the next optimal band is used.  The following is the
  order for which the bands will be used for these frame types.
  {4, 3, 7, 2, 5, 1, 8, 6}
  This routine will search the list for the most optimal band that
  was specified.  This band information will then be used for minbox
  and path-oriented framing options.

ALGORITHM REFERENCES:

******************************************************************************/
int det_opt_band (
  const BandNumber *bands,
                        /* I: Array of bands to search for the optimal band */
  int nbands,           /* I: Number of bands in the list                   */
  BandNumber *opt_band) /* O: Optimal band in the list of bands             */
{
  int i,                        /* Loop variable                         */
      curropt,                  /* Loop var for the current optimal band */
      found;                    /* Boolean to specify found or not found */
  BandNumber optimal[MAX_BANDS] = {b4, b4p, b3, b7, b2, b5, b5p, b1, b1p};
                                /* Array of "optimal" band numbers in order
                                   of best to worst for path-oriented and
                                   minbox framing.                       */

  /* Initializations.
   */
  found = FALSE;
  curropt = 0;

  /* Loop until found.
   */
  while (!found)
  {
    /* Look for the current optimal band in the grid.bands list.
     */
    for (i = 0; i < nbands; i++)
      if (bands[i] == optimal[curropt])
      {
        *opt_band = bands[i];
        found = TRUE;
        break;
      }

    /* Go to the next optimal.  Make sure we are not outside of the
     * optimal bands array bounds.
     */
    if (!found && curropt++ >= MAX_BANDS)
    {
      xxx_LogStatus ("ALIGRID", __FILE__, __LINE__, "Optimal band was not found.");
      return (ERROR);
    }
  }

  return (SUCCESS);
}
