#include "aligrid.h"

/******************************************************************************
NAME:           get_parms

PURPOSE:
Get the grid parameters from the input parameter file and the system table.

RETURN VALUE:   Type = int
Value    Description
-----    -----------
SUCCESS  Successful completion
ERROR    Operation failed

NOTES:
This routine will open the input parameter file, read the grid parameters,
close the parameter file, and return the parameters and the grid.
Pixel sizes < 20.0 refer to 10m band resolution.  Pixel sizes >= 20.0 and
<= 40.0 refer to 30m band resolution.  
resolution.

ALGORITHM REFERENCES:
See axx_grid.h for a description of the grid structure.

******************************************************************************/

int get_parms (
  char param_filename[STRLEN],  /* I: Parameter file name                   */
  char scriptname[STRLEN], 	/* O: Script name                           */
  char file_name_0R[STRLEN],    /* O: ETM+ file name for the 0R image       */
  char file_name_1R[STRLEN],    /* O: ETM+ file name for the 1R image       */
  char cal_parm_file[STRLEN],   /* O: Calibration parameter filename.       */
  char model_file_name[STRLEN], /* O: ETM+ model file name                  */
  struct DBL_XY frame_coords[2],/* O: User-input frame coordinates to define
                                      the frame of the output area          */
  int  *coord_code,             /* O: Code for coordinate units             */
  struct DBL_LS *lscoords,      /* O: Image coordinates of the first coord
                                      pair entered in frame_coords when proj
                                      params are being specified directly   */
  int  *path,                   /* O: WRS path number of the image          */
  double *row,                  /* O: WRS (fractional) row number           */
  float *zincrement,
  float *zmax,
  float *zmin,
  axx_GRID_TYPE *grid)          /* O: Grid structure to be filled           */
{
  OBJDESC *in_parms;   /* Pointer to the structure containing all the ODL
                          parameters.                                       */
  GPS_SYSTBL systbl;   /* System table structure.                           */
  char ErrorMessage[STRLEN],
                       /* Error message returned from the called routine.   */
       units[12],      /* Units                                             */
       dir[STRLEN],    /* Directory portion of the filename.                */
       root[STRLEN],   /* Root portion of the filename.                     */
       ext[STRLEN];    /* Extension portion of the filename.                */
  float frm_coords[4], /* Y/X coords of the frame points (2 sets).          */
        ls_coords[2],  /* Line/sample coords of the point.                  */
        pixsize[2],    /* Pixel size (30m, 60m, 15m)                        */
        wrs_row;       /* Row                                               */
  int wrs_row_int;     /* Row (integer)                                     */
  int det_los;         /* Detector LOS option                               */
  double parms[PROJPRMS_SIZE];
                       /* Projection parameters.                            */
  int  stat,           /* Status return value                               */
       val_line,       /* Line value of the first specified line            */
       val_samp;       /* Sample value of the first specified sample        */
  int i,               /* Loop variable                                     */
      count,           /* Number of values returned from GetODLField        */
      sca,             /* SCA counter                                       */
      found[NBANDS],   /* Flag to specify this band has already been found. */
      ref,             /* Reference position within the grid structure      */
      res[2],          /* Flag to specify if the first line or sample was
                          of this resolution                                */
      nl[2],           /* Num of lines of the band resolutions
                          (30m, 10m)                                   */
      ns[2],           /* Num of samps of the band resolutions
                          (30m, 60m, 15m)                                   */
      bands[NBANDS];   /* Array of band numbers to process                  */ 
  BandNumber bands_arr[NBANDS] = {pan, b1p, b1, b2, b3, b4, b4p, b5p, b5, b7};
                       /* Array of all the bands                            */
  BandNumber bandno;
  const float PIXSIZE[] = {30.0, 10.0}; /* Default pixel sizes             */
  const int  LINES[] = {WRS_SCENE_SIZE_LINES, 3*WRS_SCENE_SIZE_LINES - 1};
                       /* Default NLINES.  The relationship for the NLINES
                          is ??? */
  const int  SAMPS[] = {WRS_SCENE_SIZE_SAMPS, 3*WRS_SCENE_SIZE_SAMPS - 1};
                       /* Default NSAMPS.  The relationship for the NSAMPS
                          is ??? */

  /* Open the ODL parameter file.  (The openODL call should NOT determine
   * the input ODL filename.)
   */
  in_parms = xxx_OpenODL (param_filename, (xxx_EDCOdl_TYPE)xxx_NoEDCMeta, ErrorMessage);
  if (in_parms == NULL)
  {
    xxx_LogStatus ("ALIGRID", __FILE__, __LINE__, ErrorMessage);
    return (ERROR);
  }

  /* Get the scriptname.
   */
  if (xxx_GetODLField (scriptname, STRLEN * sizeof (char), XXX_String, in_parms, NULL, 
                       "SCRIPT_NAME", ErrorMessage, &count) != SUCCESS)
  {
    xxx_LogStatus ("ALIGRID", __FILE__, __LINE__, ErrorMessage);
    strcpy (ErrorMessage, "Error in the scriptname in the parameter file.");
    xxx_LogStatus ("ALIGRID", __FILE__, __LINE__, ErrorMessage);
    xxx_CloseODL (in_parms, ErrorMessage);
    return (ERROR);
  }

  /* Get the 0R ALI level1r file name.
   */
  if (xxx_GetODLField (file_name_0R, STRLEN * sizeof (char), XXX_String, in_parms, NULL, 
                       "L0R_HDFNAME", ErrorMessage, &count) != SUCCESS)
  {
    xxx_LogStatus ("ALIGRID", __FILE__, __LINE__, ErrorMessage);
    strcpy (ErrorMessage, "Error in the 0R filename in the parameter file.");
    xxx_LogStatus ("ALIGRID", __FILE__, __LINE__, ErrorMessage);
    xxx_CloseODL (in_parms, ErrorMessage);
    return (ERROR);
  }

  /* Get the cal_parm_file name.  This is an optional parameter.
   */
  xxx_GetODLField (cal_parm_file, STRLEN * sizeof (char), XXX_String, in_parms, NULL, 
                  "CAL_PARM_FILE", ErrorMessage, &count);

  /* Get the L1R ALI file name.
   */
  if (xxx_GetODLField (file_name_1R, STRLEN * sizeof (char), XXX_String, in_parms, scriptname, 
                       "L1R_IMAGE", ErrorMessage, &count) != SUCCESS)
  {
    xxx_LogStatus ("ALIGRID", __FILE__, __LINE__, ErrorMessage);
    strcpy (ErrorMessage, "Error in the L1R filename in the parameter file.");
    xxx_LogStatus ("ALIGRID", __FILE__, __LINE__, ErrorMessage);
    xxx_CloseODL (in_parms, ErrorMessage);
    return (ERROR);
  }

  /* Get the grid file name.  If no grid file name is given then use the
   * L1R_image name with a .grd extension.  If no extension is given then
   * use the .grd extension.
   */
  xxx_GetODLField (grid->gridname, GRIDNAME_SIZE * sizeof (char),
                   XXX_String, in_parms, scriptname, "GRID_FILE_NAME",
                   ErrorMessage, &count);
  if (!strcmp (grid->gridname, ""))
  { /* If no grid filename was given. */
    xxx_splitname (file_name_1R, dir, root, ext); /* dir not valid */
    xxx_createname (dir, root, "grd", grid->gridname);
  }
  else
  { /* Check for the extension.  If no ext, then use .grd. */
    xxx_splitname (grid->gridname, dir, root, ext); /* dir not valid */
    if (!strcmp (ext, ""))
      xxx_createname (dir, root, "grd", grid->gridname);
  }

  /* Get the model file name.  If no model file name is given then use the
   * L1R_image name with a .ali extension.  If no extension is given then
   * use the .ali extension.
   */
  xxx_GetODLField (model_file_name, STRLEN * sizeof (char), XXX_String,
                   in_parms, scriptname, "ALIMODEL_FILE_NAME", ErrorMessage,
                   &count);
  if (!strcmp (model_file_name, ""))
  { /* If no model filename was given. */
    xxx_splitname (file_name_1R, dir, root, ext); /* dir not valid */
    xxx_createname (dir, root, "ali", model_file_name);
  }
  else
  { /* Check for the extension.  If no ext, then use .ali. */
    xxx_splitname (model_file_name, dir, root, ext); /* dir not valid */
    if (!strcmp (ext, ""))
      xxx_createname (dir, root, "ali", model_file_name);
  }

  /* Get the frame_type.
   */
  if (xxx_GetODLField (&grid->frame_type, sizeof (int), XXX_Int, in_parms, scriptname, 
                       "FRAME_TYPE", ErrorMessage, &count) != SUCCESS)
  {
    xxx_LogStatus ("ALIGRID", __FILE__, __LINE__, ErrorMessage);
    strcpy (ErrorMessage, "Invalid frame type was given in the parameter file.");
    xxx_LogStatus ("ALIGRID", __FILE__, __LINE__, ErrorMessage);
    xxx_CloseODL (in_parms, ErrorMessage);
    return (ERROR);
  }

  /* Get the proj_code.
   * Valid values are <= MAXPROJ.
   */
  if (xxx_GetODLField (&grid->code, sizeof (int), XXX_Int, in_parms, scriptname, "PROJ_CODE", 
                       ErrorMessage, &count) != SUCCESS)
  {
    xxx_LogStatus ("ALIGRID", __FILE__, __LINE__, ErrorMessage);
    strcpy (ErrorMessage, "Error in the proj_code in the parameter file.");
    xxx_LogStatus ("ALIGRID", __FILE__, __LINE__, ErrorMessage);
    xxx_CloseODL (in_parms, ErrorMessage);
    return (ERROR);
  }
  if (grid->code > MAXPROJ)
  {
    strcpy (ErrorMessage, "Incorrect proj_code in get_parms.");
    xxx_LogStatus ("ALIGRID", __FILE__, __LINE__, ErrorMessage);
    xxx_CloseODL (in_parms, ErrorMessage);
    return (ERROR);
  }

  /* Get the frame coordinates.  They are entered in the parameter file as
   * Y then X for each point, up to two points (and possibly no points.)
   * Read 2 sets for GEOBOX (1) and PROJBOX (2).  Read only 1 set for
   * UL_SIZE (2).
   */
  if (grid->frame_type == GEOBOX || grid->frame_type == PROJBOX)
  {
    if (xxx_GetODLField (frm_coords, 4 * sizeof (float), XXX_Float,
        in_parms, scriptname, "FRAME_COORDS", ErrorMessage, &count) != SUCCESS)
    {
      xxx_LogStatus ("ALIGRID", __FILE__, __LINE__, ErrorMessage);
      strcpy (ErrorMessage, "Error in the frame coords in the parameter file.");
      xxx_LogStatus ("ALIGRID", __FILE__, __LINE__, ErrorMessage);
      xxx_CloseODL (in_parms, ErrorMessage);
      return (ERROR);
    }

    /* Fill the frame_coords structure appropriately.
     */
    if (count == 4)
    {
      if (grid->code == SOM)
	{
	frame_coords[0].y = -frm_coords[1];
	frame_coords[0].x = frm_coords[0];
	frame_coords[1].y = -frm_coords[3];
	frame_coords[1].x = frm_coords[2];
	}
      else
	{
	frame_coords[0].y = frm_coords[0];
	frame_coords[0].x = frm_coords[1];
	frame_coords[1].y = frm_coords[2];
	frame_coords[1].x = frm_coords[3];
	}
    }
    else
    {
      strcpy (ErrorMessage, "Four frame coords are needed for this frame_type.");
      xxx_LogStatus ("ALIGRID", __FILE__, __LINE__, ErrorMessage);
      xxx_CloseODL (in_parms, ErrorMessage);
      return (ERROR);
    }
  }
  else if (grid->frame_type == UL_SIZE)
  {
    if (xxx_GetODLField (frm_coords, 4 * sizeof (float), XXX_Float, in_parms, scriptname, 
                         "FRAME_COORDS", ErrorMessage, &count) != SUCCESS)
    {
      xxx_LogStatus ("ALIGRID", __FILE__, __LINE__, ErrorMessage);
      strcpy (ErrorMessage, "Error in the frame coords in the parameter file.");
      xxx_LogStatus ("ALIGRID", __FILE__, __LINE__, ErrorMessage);
      xxx_CloseODL (in_parms, ErrorMessage);
      return (ERROR);
    }

    /* Fill the frame_coords structure appropriately.
     */
    if (count == 4)
    {
      if (grid->code == SOM)
	{
	frame_coords[0].y = -frm_coords[1];
	frame_coords[0].x = frm_coords[0];
	}
      else
	{
	frame_coords[0].y = frm_coords[0];
	frame_coords[0].x = frm_coords[1];
	}
    }
    else
    {
      strcpy (ErrorMessage, "Two frame coords followed by two 0.0s are needed for this frame_type.");
      xxx_LogStatus ("ALIGRID", __FILE__, __LINE__, ErrorMessage);
      xxx_CloseODL (in_parms, ErrorMessage);
      return (ERROR);
    }
  }

  /* Get the proj_units.  Default is METERS.
   * NOTE: This needs to come before the reading of the coord_units.
   */
  if (xxx_GetODLField (units, 12 * sizeof (char), XXX_String, in_parms, scriptname, "PROJ_UNITS", 
                       ErrorMessage, &count) != SUCCESS)
    strcpy (units, "METERS");

  /* Put the units in the grid.
   */
  if (!strcmp (xxx_strtoupper(units), "RADIANS"))
    strcpy (grid->units, units);
  else if (!strcmp (xxx_strtoupper(units), "FEET"))
    strcpy (grid->units, units);
  else if (!strcmp (xxx_strtoupper(units), "METERS"))
    strcpy (grid->units, units);
  else if (!strcmp (xxx_strtoupper(units), "SECONDS"))
    strcpy (grid->units, units);
  else if (!strcmp (xxx_strtoupper(units), "DEGREES"))
    strcpy (grid->units, units);
  else if (!strcmp (xxx_strtoupper(units), "DMS"))
    strcpy (grid->units, units);
  else
  { /* Invalid proj units were given. */
    strcpy (ErrorMessage, "Invalid proj units were given in the parameter file.");
    xxx_LogStatus ("ALIGRID", __FILE__, __LINE__, ErrorMessage);
    xxx_CloseODL (in_parms, ErrorMessage);
    return (ERROR);
  }

  /* If the proj_code != GEOGRAPHIC (0), then the proj_units must be in
   * METERS or FEET.
   */
  if (grid->code != 0)
    if (strcmp (grid->units, "FEET") && strcmp (grid->units, "METERS"))
    { /* Invalid proj units were given for the proj code. */
      strcpy (ErrorMessage, "Non-Geographic proj_codes require proj_units of FEET or METERS.");
      xxx_LogStatus ("ALIGRID", __FILE__, __LINE__, ErrorMessage);
      xxx_CloseODL (in_parms, ErrorMessage);
      return (ERROR);
    }

  /* Get the coord_units of the user coordinates.  This is for GEOBOX (1),
   * PROJBOX (2), and UL_SIZE (3).
   */
  if (grid->frame_type == GEOBOX || grid->frame_type == PROJBOX ||
      grid->frame_type == UL_SIZE)
  {
    if (xxx_GetODLField (units, 12 * sizeof (char), XXX_String, in_parms, scriptname, "COORD_UNITS", 
                         ErrorMessage, &count) != SUCCESS)
    {
      xxx_LogStatus ("ALIGRID", __FILE__, __LINE__, ErrorMessage);
      strcpy (ErrorMessage, "Error in the coord_units in the parameter file.");
      xxx_LogStatus ("ALIGRID", __FILE__, __LINE__, ErrorMessage);
      xxx_CloseODL (in_parms, ErrorMessage);
      return (ERROR);
    }

    /* If PROJECTION was specified then use the same units as the projection
     * units.
     */
    if (!strcmp (xxx_strtoupper(units), "PROJECTION"))
      strcpy (units, grid->units);

    /* Convert the units to a long.
     */
    if (!strcmp (xxx_strtoupper(units), "RADIANS"))
      *coord_code = 0;
    else if (!strcmp (xxx_strtoupper(units), "FEET"))
      *coord_code = 1;
    else if (!strcmp (xxx_strtoupper(units), "METERS"))
      *coord_code = 2;
    else if (!strcmp (xxx_strtoupper(units), "SECONDS"))
      *coord_code = 3;
    else if (!strcmp (xxx_strtoupper(units), "DEGREES"))
      *coord_code = 4;
    else if (!strcmp (xxx_strtoupper(units), "DMS"))
      *coord_code = 5;
    else
    {
      xxx_LogStatus ("ALIGRID", __FILE__, __LINE__, ErrorMessage);
      strcpy (ErrorMessage, "Invalid coord_units in the parameter file.");
      xxx_LogStatus ("ALIGRID", __FILE__, __LINE__, ErrorMessage);
      xxx_CloseODL (in_parms, ErrorMessage);
      return (ERROR);
    }
  }

  /* Get the lscoords.  They are entered in the parameter file as line
   * then sample.
   * Only read for PROJBOX (2) or UL_SIZE (2).  If not specified then
   * default to (0,0).
   */
  if (grid->frame_type == PROJBOX || grid->frame_type == UL_SIZE)
  {
    if (xxx_GetODLField (ls_coords, 2 * sizeof (float), XXX_Float, in_parms, scriptname, 
                         "LSCOORDS", ErrorMessage, &count) != SUCCESS)
    {
      ls_coords[0] = 0.0;
      ls_coords[1] = 0.0;
    }

    /* Fill the lscoords structure appropriately.
     */
    lscoords->line = ls_coords[0];
    lscoords->samp = ls_coords[1];
  }

  /* Initialize all the found band flags to FALSE.
   */
  for (i = 0; i < NBANDS; i++)
    found[i] = FALSE;

  /* Get the band numbers to process.  If none are specified then fill
   * in all 9 bands.
   */
  if (xxx_GetODLField (bands, NBANDS * sizeof (int), XXX_Int, in_parms, scriptname, 
                      "BAND_LIST", ErrorMessage, &count) != SUCCESS)
  {
    xxx_LogStatus ("ALIGRID", __FILE__, __LINE__, ErrorMessage);
    strcpy (ErrorMessage, "Error in the band numbers in the parameter file.");
    xxx_LogStatus ("ALIGRID", __FILE__, __LINE__, ErrorMessage);
    xxx_CloseODL (in_parms, ErrorMessage);
    return (ERROR);
  }

  /* Fill the band numbers and their reference values into the grid.  Also
   * store the number of bands input.  If the user did not specify any bands
   * then fill the array with all NBANDS bands.
   */
  if (count > 0)
  { /* Fill in user-entered band data. */
    grid->nbands = count;
    count = 0;  /* use count for a counter of the unique bands */
    for (i = 0; i < grid->nbands; i++)
    {
      grid->bands[count] = axx_parse_user_band (bands[i]); 
      if (grid->bands[count] == berror)
      {
        sprintf (ErrorMessage, "Invalid band number (%d) in parameter file.", bands[i]);
        xxx_LogStatus ("ALIGRID", __FILE__, __LINE__, ErrorMessage);
        xxx_CloseODL (in_parms, ErrorMessage);
        return (ERROR);
      }

      /* If the band already exists in the list then erase this band from the
       * array (don't increment count.)  OW set found to TRUE.
       */
      if (found[grid->bands[count]] == FALSE)
      {
        found[grid->bands[count]] = TRUE;
        count++;
      }
    }
    grid->nbands = count;

    /* Fill the rest of the data elements with berror.
     */
    for (i = count; i < NBANDS; i++)
      grid->bands[i] = berror;
  }
  else
  { /* Fill in all NBANDS bands. */
    grid->nbands = NBANDS;
    for (i = 0; i < NBANDS; i++)
      grid->bands[i] = bands_arr[i];
  }

  /* Fill in the enumerated band numbers for bandno in the grid.
   */
  for (i = 0; i < grid->nbands; i++)
  {
    ref = grid->bands[i];
    for(sca = 0; sca < grid->nscas; sca++)
       grid->gridbands[sca][ref].bandno = (BandNumber) ref;
  }

  /* Get the pixel_size.  Read for all frame_types.
   */
  xxx_GetODLField (pixsize, 2*sizeof(float), XXX_Float, in_parms, scriptname, "PIXEL_SIZE", 
                   ErrorMessage, &count);

  /* Make sure the pixel size is >0.  If not, then fill in default pixel
   * sizes for the nonvalid values.  The order of the pixel sizes should
   * be 30m, 10m.
   */
  for (i = 0; i < 2; i++)
    if (pixsize[i] <= 0.0)
      pixsize[i] = PIXSIZE[i];

  /* Fill in the pixsizes in the grid based on the bands in the list.
   */
  for(sca = 0; sca < grid->nscas; sca++)
  {
    grid->gridbands[sca][(int)b1p].pixsize = grid->gridbands[sca][(int)b1].pixsize =
      grid->gridbands[sca][(int)b2].pixsize = grid->gridbands[sca][(int)b3].pixsize =
      grid->gridbands[sca][(int)b4].pixsize = grid->gridbands[sca][(int)b4p].pixsize = 
      grid->gridbands[sca][(int)b5p].pixsize = grid->gridbands[sca][(int)b5].pixsize = 
      grid->gridbands[sca][(int)b7].pixsize = pixsize[0];
    grid->gridbands[sca][(int)pan].pixsize = pixsize[1];
  }

  /* Get the number of lines.
   * Only read for UL_SIZE (2) - required, and PATH_ORIENTED (5) - optional.
   * The order of the NLINES should be 30m, 10m.
   */
  if (grid->frame_type == UL_SIZE || grid->frame_type == PATH_ORIENTED)
  {
    stat = xxx_GetODLField (nl, 2*sizeof (int), XXX_Int, in_parms,
                            scriptname, "NLINES", ErrorMessage, &count);
    if (((stat != SUCCESS) || ((nl[0] == 0) && (nl[1] == 0)))
       && grid->frame_type == UL_SIZE)
    {  /* If the lines were not provided or they were all zero. */
      xxx_LogStatus ("ALIGRID", __FILE__, __LINE__, ErrorMessage);
      strcpy (ErrorMessage, "Error in the number of lines in the parameter file.");
      xxx_LogStatus ("ALIGRID", __FILE__, __LINE__, ErrorMessage);
      xxx_CloseODL (in_parms, ErrorMessage);
      return (ERROR);
    }

    /* If number of lines or samples is not specified and frame_type is
     * path-oriented, use the nominal image size of WRS_SCENE_SIZE_LINES
     * and WRS_SCENE_SIZE_SAMPS.  The relationship for the NLINES are ???
     */
    else if (stat != SUCCESS || (nl[0] == 0 && nl[1] == 0) )
    {
      for (i = 0; i < 1; i++)
      {
        if (pixsize[i] < 20.0)        /* 15m resolution */
          nl[i] = LINES[1];
        else                          /* 60m resolution */
          nl[i] = LINES[0];
      }
    }

    /* Use the available resolutions to determine the value of the
     * unspecified nlines.  If one value is specified but other values are
     * not, then calculate the other values based on the given values.  Use
     * the pixel size of the band to determine its resolution.
     */
    if (nl[0] == 0 || nl[1] == 0 )
    {/* At least one nl was specified for a band resolution. */
      /* Find the first specified number of lines.
       */
      for (i = 0; i < 2; i++)
        if (nl[i] != 0)
          break;
      val_line = nl[i];  /* Use this to determine the other values. */

      /* Determine the resolution of the specified value.
       */
      res[0] = res[1] = FALSE;
      if (pixsize[i] < 20.0)        /* 10m resolution */
        res[1] = TRUE;
      else                          /* 30m resolution */
        res[0] = TRUE;

      for (i = 0; i < 2; i++)
      {
        /* If the line value was not specified, then calculate it from
         * the first specified line value and its band resolution.
         */
        if (nl[i] == 0)
        {
          if (res[1])       /* 10m */
          {
            if (pixsize[i] < 20.0)
              /* Use the 10m value to determine this 15m value.
               */
              nl[i] = val_line;
            else
              /* Use the 15m value to determine this 60m value.
               */
              nl[i] = (val_line + 1) / 3;
          }
          else if (res[0])  /* 30m */
          {
            if (pixsize[i] < 20.0)
              /* Use the 30m value to determine this 15m value.
               */
              nl[i] = 3*val_line - 1;
            else 
              /* Use the 30m value to determine this 30m value.
               */
              nl[i] = val_line;
          }
        }
      }
    }

    /* Fill in the nlines for each band.
     */
    for(sca = 0; sca < grid->nscas; sca++)
    {
      grid->gridbands[sca][(int)b1p].lines = grid->gridbands[sca][(int)b1].lines =
        grid->gridbands[sca][(int)b2].lines = grid->gridbands[sca][(int)b3].lines =
        grid->gridbands[sca][(int)b4].lines = grid->gridbands[sca][(int)b4p].lines =
        grid->gridbands[sca][(int)b5p].lines = grid->gridbands[sca][(int)b5].lines =
        grid->gridbands[sca][(int)b7].lines = nl[0];
      grid->gridbands[sca][(int)pan].lines = nl[1];
    }
  }

  /* Get the number of samples.
   * Only read for UL_SIZE (2) - required, and PATH_ORIENTED (5) - optional.
   * The order of the NSAMPS should be 30m, 10m.
   */
  if (grid->frame_type == UL_SIZE || grid->frame_type == PATH_ORIENTED)
  {
    stat = xxx_GetODLField (ns, 2*sizeof (int), XXX_Int, in_parms,
                            scriptname, "NSAMPS", ErrorMessage, &count);
    if (((stat != SUCCESS) || ((ns[0] == 0) && (ns[1] == 0)))
       && grid->frame_type == UL_SIZE)
    {  /* If the samples were not provided or they were all zero. */
      xxx_LogStatus ("ALIGRID", __FILE__, __LINE__, ErrorMessage);
      strcpy (ErrorMessage, "Error in the number of samples in the parameter file.");
      xxx_LogStatus ("ALIGRID", __FILE__, __LINE__, ErrorMessage);
      xxx_CloseODL (in_parms, ErrorMessage);
      return (ERROR);
    }

    /* If number of lines or samples is not specified and frame_type is
     * path-oriented, use the nominal image size of WRS_SCENE_SIZE_LINES
     * and WRS_SCENE_SIZE_SAMPS.  The relationship for the NSAMPS are
     * (2N-1, N) for (10m, 30m).
     */
    else if (stat != SUCCESS || (ns[0] == 0 && ns[1] == 0 ))
    {
      for (i = 0; i < 2; i++)
      {
        if (pixsize[i] < 20.0)        /* 15m resolution */
          ns[i] = SAMPS[1];
        else                          /* 30m resolution */
          ns[i] = SAMPS[0];
      }
    }

    /* Use the available resolutions to determine the value of the
     * unspecified nsamps.  If one value is specified but other values are
     * not, then calculate the other values based on the given values.  Use
     * the pixel size of the band to determine its resolution.
     */
    if (ns[0] == 0 || ns[1] == 0 )
    {/* At least one ns was specified for a band resolution. */
      /* Find the first specified number of samples.
       */
      for (i = 0; i < 2; i++)
        if (ns[i] != 0)
          break;
      val_samp = ns[i];  /* Use this to determine the other values. */

      /* Determine the resolution of the specified value.
       */
      res[0] = res[1] = FALSE;
      if (pixsize[i] < 20.0)        /* 10m resolution */
        res[1] = TRUE;
      else                          /* 30m resolution */
        res[0] = TRUE;

      for (i = 0; i < 2; i++)
      {
        /* If the sample value was not specified, then calculate it from
         * the first specified sample value and its band resolution.
         */
        if (ns[i] == 0)
        {
          if (res[1])       /* 10m */
          {
            if (pixsize[i] < 20.0)
              /* Use the 15m value to determine this 10m value.
               */
              ns[i] = val_samp;
            else 
              /* Use the 10m value to determine this 30m value.
               */
              ns[i] = (val_samp + 1) / 3;
          }
          else if (res[0])  /* 30m */
          {
            if (pixsize[i] < 20.0)
              /* Use the 30m value to determine this 10m value.
               */
              ns[i] = 3*val_samp - 1;
            else
              /* Use the 30m value to determine this 30m value.
               */
              ns[i] = val_samp;
          }
        }
      }
    }

    /* Fill in the nsamps for each band.
     */
    for(sca = 0; sca < grid->nscas; sca++)
    {
      grid->gridbands[sca][(int)b1p].samps = grid->gridbands[sca][(int)b1].samps =
        grid->gridbands[sca][(int)b2].samps = grid->gridbands[sca][(int)b3].samps =
        grid->gridbands[sca][(int)b4].samps = grid->gridbands[sca][(int)b4p].samps = 
        grid->gridbands[sca][(int)b5p].samps = grid->gridbands[sca][(int)b5].samps = 
        grid->gridbands[sca][(int)b7].samps = ns[0];
      grid->gridbands[sca][(int)pan].samps = ns[1];
    }
  }

  /* Get the utm_zone if the proj_code == 1 (UTM).
   */
  if (grid->code == 1)
    if (xxx_GetODLField (&grid->zone, sizeof (int), XXX_Int, in_parms, scriptname, "UTM_ZONE", 
                         ErrorMessage, &count) != SUCCESS)
    {
      xxx_LogStatus ("ALIGRID", __FILE__, __LINE__, ErrorMessage);
      strcpy (ErrorMessage, "Error in the UTM zone in the parameter file.");
      xxx_LogStatus ("ALIGRID", __FILE__, __LINE__, ErrorMessage);
      xxx_CloseODL (in_parms, ErrorMessage);
      return (ERROR);
    }

  /* Get the path.
   * Only read for PATH_ORIENTED (5), SOM projections.
   */
  if (grid->frame_type == PATH_ORIENTED || grid->code == SOM || 
      grid->frame_type == PATH_MINBOX || grid->frame_type == PATH_MAXBOX)
    if (xxx_GetODLField (path, sizeof (int), XXX_Int, in_parms, NULL, "PATH_WRS", ErrorMessage, 
                         &count) != SUCCESS)
    {
      xxx_LogStatus ("ALIGRID", __FILE__, __LINE__, ErrorMessage);
      strcpy (ErrorMessage, "Error in the WRS path in the parameter file.");
      xxx_LogStatus ("ALIGRID", __FILE__, __LINE__, ErrorMessage);
      xxx_CloseODL (in_parms, ErrorMessage);
      return (ERROR);
    }

  /* Get the row.
   * Only read for PATH_ORIENTED (5).
   */
  if (grid->frame_type == PATH_ORIENTED || 
      grid->frame_type == PATH_MINBOX || grid->frame_type == PATH_MAXBOX)
  {
    if (xxx_GetODLField (&wrs_row_int, sizeof (int), XXX_Int, in_parms, NULL, "ROW_WRS", 
                         ErrorMessage, &count) != SUCCESS)
    {
      xxx_LogStatus ("ALIGRID", __FILE__, __LINE__, ErrorMessage);
      strcpy (ErrorMessage, "Error in the WRS row in the parameter file.");
      xxx_LogStatus ("ALIGRID", __FILE__, __LINE__, ErrorMessage);
      xxx_CloseODL (in_parms, ErrorMessage);
      return (ERROR);
    }

    /* get the fractional part of the row */
    if (xxx_GetODLField (&wrs_row, sizeof (float), XXX_Float, in_parms, NULL, "FRAC_ROW", 
                         ErrorMessage, &count) != SUCCESS)
		wrs_row = 0.0;
    else
	if (wrs_row >= 1.0 || wrs_row < 0.0)
    	{
      		xxx_LogStatus ("ALIGRID", __FILE__, __LINE__, ErrorMessage);
      		strcpy (ErrorMessage, "Error in the fractional part of the WRS row"
				"in the parameter file.");
      		xxx_LogStatus ("ALIGRID", __FILE__, __LINE__, ErrorMessage);
      		xxx_CloseODL (in_parms, ErrorMessage);
      		return (ERROR);
    	}

    /* Put the wrs_row into the row parameter.
     */
    *row = wrs_row_int + wrs_row;

  }

  /* Get the proj_parms if PROJ_CODE != 1 (UTM) and PROJ_CODE != 22 (SOM).
   */
  if (grid->code != UTM && grid->code != SOM)
  {
    if (xxx_GetODLField (parms, PROJPRMS_SIZE * sizeof (double), XXX_Double, in_parms, scriptname, 
                         "PROJ_PARMS", ErrorMessage, &count) != SUCCESS)
    {
      xxx_LogStatus ("ALIGRID", __FILE__, __LINE__, ErrorMessage);
      strcpy (ErrorMessage, "Error in the proj_parms in the parameter file.");
      xxx_LogStatus ("ALIGRID", __FILE__, __LINE__, ErrorMessage);
      xxx_CloseODL (in_parms, ErrorMessage);
      return (ERROR);
    }

    /* Fill the proj_parms into the grid structure.
     */
    for (i = 0; i < PROJPRMS_SIZE; i++)
      grid->projprms[i] = parms[i];
  }

  /* Otherwise fill the proj_parms in ourselves.
   */
  else if (grid->code == UTM)
  {
    for (i = 0; i < COEFCT; i++)
      grid->projprms[i] = 0.0;
  }
  else if (grid->code == SOM)
  {
    for (i = 0; i < COEFCT; i++)
      grid->projprms[i] = 0.0;
    grid->projprms[2] = 7.0;              /* satellite number */
    grid->projprms[3] = (double) *path;
    grid->projprms[12] = 1.0;             /* SOMB             */
    /* 	BEWARE:  I am adding functionality consistent with an altered version
       	of GCTPC.  This version of GCTPC is different from the released version.
	We added a pflag to SOMB and changed the way the flag performs.
	This must be evaluated when a new version of GCTPC is adapted.
	Add the pflag value to the parameter array according to row.  For the
	northern rows, set to start flag (0).  For the southern hemisphere rows
	set the start flag to (1).
    */
    if (*row >= 183)
	grid->projprms[10] = 1;    /* End of path */
    else
	grid->projprms[10] = 0;    /* Start of path */
  }

  if( xxx_GetODLField (zmax, sizeof(float), XXX_Float, in_parms, scriptname, "MAX_ELEV",
                   ErrorMessage, &count) != SUCCESS )
  {
    xxx_LogStatus ("ALIGRID", __FILE__, __LINE__, ErrorMessage);
    strcpy (ErrorMessage, "Error reading maximum elevation.");
    xxx_LogStatus ("ALIGRID", __FILE__, __LINE__, ErrorMessage);
    return (ERROR);
  }

  if( xxx_GetODLField (zmin, sizeof(float), XXX_Float, in_parms, scriptname, "MIN_ELEV",
                   ErrorMessage, &count) != SUCCESS )
  {
    xxx_LogStatus ("ALIGRID", __FILE__, __LINE__, ErrorMessage);
    strcpy (ErrorMessage, "Error reading minimum elevation.");
    xxx_LogStatus ("ALIGRID", __FILE__, __LINE__, ErrorMessage);
    return (ERROR);
  }

  if( *zmax < *zmin )
  {
    xxx_LogStatus ("ALIGRID", __FILE__, __LINE__, ErrorMessage);
    strcpy (ErrorMessage, "Error minimum is greater than maximum elevation.");
    xxx_LogStatus ("ALIGRID", __FILE__, __LINE__, ErrorMessage);
    return (ERROR);
  }

  if (xxx_GetODLField (zincrement, sizeof (float), XXX_Float, in_parms, NULL, "ELEV_INCREMENT",
                       ErrorMessage, &count) != SUCCESS)
  {
     xxx_LogStatus ("ALIGRID", __FILE__, __LINE__, ErrorMessage);
     strcpy (ErrorMessage, "Error reading number of elevation planes.");
     xxx_LogStatus ("ALIGRID", __FILE__, __LINE__, ErrorMessage);
     xxx_CloseODL (in_parms, ErrorMessage);
     return (ERROR);
  }

  /* Done with the ODL file so close it.
   */
  if (xxx_CloseODL (in_parms, ErrorMessage) != SUCCESS)
  {
    xxx_LogStatus ("ALIGRID", __FILE__, __LINE__, ErrorMessage);
    strcpy (ErrorMessage, "Error closing the ODL file.");
    xxx_LogStatus ("ALIGRID", __FILE__, __LINE__, ErrorMessage);
    return (ERROR);
  }

  /* Initialize the dtype to DOUBLE.
   */
  grid->dtype = R8;

  /* Initialize the grid_cell_size.
   */
  for (bandno = b1p; bandno <= b7; bandno++)
  {
    i = (int)bandno;
    for(sca = 0; sca < grid->nscas; sca++)
    {
      grid->gridbands[sca][i].grid_cell_size.line = GRID_CELL_SIZE_LINE_MULTI; /* 1-7 */
      grid->gridbands[sca][i].grid_cell_size.samp = GRID_CELL_SIZE_SAMP_MULTI;
    }
  }

  for(sca = 0; sca < grid->nscas; sca++)
  {
    grid->gridbands[sca][(int)pan].grid_cell_size.line = GRID_CELL_SIZE_LINE_PAN;   /* pan   */
    grid->gridbands[sca][(int)pan].grid_cell_size.samp = GRID_CELL_SIZE_SAMP_PAN;
  }

  /* Initialize the degree for the poly_lines and poly_samps.  They are the
   * same for all bands.
   */
  for (i = 0; i < NBANDS; i++)
    for(sca = 0; sca < grid->nscas; sca++)
      grid->gridbands[sca][i].degree = ROUGH_DEGREE;

  /* Read the system table to get the spheroid and datum values.
   */
  stat = xxx_get_system_table (&systbl);
  if (stat != SUCCESS)
  {
    strcpy (ErrorMessage, "Error reading the system table in get_parms.");
    xxx_LogStatus ("ALIGRID", __FILE__, __LINE__, ErrorMessage);
    return (ERROR);
  }

  /* Fill the spheroid and datum values from the system table into the
   * grid structure.
   */
  grid->spheroid = systbl.spheroid; 
  strcpy (grid->datum, systbl.datum);

  return (SUCCESS);
}
