#include "aligrid.h"

/******************************************************************************
NAME:           make_grid

PURPOSE:
Creates the geometric mapping grid in output space.


RETURN VALUE:   Type = int
Value    Description
-----    -----------
SUCCESS  Successful completion
ERROR    Operation failed

NOTES:
The slo array must be in band-referenced order, so that band 1 is followed by
band 2 is followed by band 3 ... up to band 8 (regardless if there is
information for that SLO or not).

ALGORITHM REFERENCES:
See axx_grid.h for a description of the grid structure.
See axx_alimodel.h for a description of the alimodel structure.
See exx_cpf.h for a description of the CPF structure.

******************************************************************************/

int make_grid (
  ALIMODEL *alimodel,        /* I: ETM+ model structure                       */
  CPF *cpf,                /* I: CPF constants                              */
  double poly_x[2][4],     /* I: Array of x polynomial coefficients (in the order of 10m, 30m) */
  double poly_y[2][4],     /* I: Array of y polynomial coefficients (in the order of 10m, 30m) */
  axx_GRID_TYPE *grid)     /* I/O: Grid structure                           */
{
  int  ref,             /* corresponding reference position for the band
                           numbers                                          */
       stat,            /* Return status                                    */
       i, j;            /* Loop variables                                   */
  int  sca,		/* Counter for SCAs				    */
       zplane,          /* Counter for elevation planes                     */
       curr_pt,         /* Current point                                    */
       curr_band,       /* Current band number                              */
       out_units,       /* Projection units code                            */
       geo = 0,         /* Geographic projection flag                       */
       zero = 0,        /* Zero                                             */
       null_zone = NULLZONE,
                        /* UTM projection code                              */
       blank = TRUE;    /* Flag to specify if the output image is blank.
                           If one input pixel maps to an output pixel, then
                           the image is not blank.                          */
  double elev = 0.0,    /* Elevation at the current point                   */
         in_lat,        /* Latitude in input space                          */
         in_long,       /* Longitude in input space                         */
         proj_x,        /* Projection X                                     */
         proj_y,        /* Projection Y                                     */
         line,          /* Line in output space                             */
         samp,          /* Sample in output space                           */
         geo_parms[COEFCT];
                        /* Geo projection parameters                        */
  int polyindex[10] = {0, 1, 1, 1, 1, 1, 1, 1, 1, 1};
                        /* Poly x/y indices - bands 1-7 are 30m (1), pan is 10m (0) */
  char ErrMsg[STRLEN];  /* Error message                                    */
  DetType dettype = NOMINAL;

  /* Initialize geo_parms.
   */
  for (i = 0; i < COEFCT; i++)
    geo_parms[i] = 0.0;

  /* Convert the grid projection units to a unit code for gctp.
   */
  if (!strcmp (grid->units, "RADIANS"))
    out_units = 0;
  else if (!strcmp (grid->units, "FEET"))
    out_units = 1;
  else if (!strcmp (grid->units, "METERS"))
    out_units = 2;
  else if (!strcmp (grid->units, "SECONDS"))
    out_units = 3;
  else if (!strcmp (grid->units, "DEGREES"))
    out_units = 4;
  else if (!strcmp (grid->units, "DMS"))
    out_units = 5;

  /* Loop through bands 
   */
  for (curr_band = 0; curr_band < grid->nbands; curr_band++)
  {
    ref = grid->bands[curr_band];

    /* Loop through SCAs
     */
    for( sca=0 ; sca<grid->nscas; sca++ )
    {
      /* Loop through elevation planes
       */
      for( zplane=0, curr_pt=0 ; zplane<grid->gridbands[sca][ref].nzplanes ; zplane++ )
      {
    
      /* Calculate elevation for plane
         */
        elev = ( zplane - grid->gridbands[sca][ref].zeroplane ) * grid->gridbands[sca][ref].zspacing; 
        /* Loop through all of the grid points to get the latitude/longitude,
         * latitude/longitude in output space, and finally the line/sample in
         * output space.
         */
        for (i = 0; i < grid->gridbands[sca][ref].ngrid_lines; i++)
        {
          for (j = 0; j < grid->gridbands[sca][ref].ngrid_samps; j++, curr_pt++)
          {
            /* Calculate the latitude and longitude of the current grid point.
             */
            stat = axx_forward_ali ((double)grid->gridbands[sca][ref].in_lines[i], 
                          (double)grid->gridbands[sca][ref].in_samps[j], grid->gridbands[sca][ref].bandno, 
                          sca, elev, cpf, alimodel, dettype, &in_lat, &in_long);
            if (stat == ERROR)
            {
              strcpy (ErrMsg, "Error in the forward_ali call.");
                xxx_LogStatus ("ALIGRID", __FILE__, __LINE__, ErrMsg);
              return (ERROR);
            }
    
            /* Calculate the location of the latitude and longitude in output
             * space.
             */
            stat = exx_projtran (&geo, &zero, &null_zone, geo_parms, &grid->spheroid, &grid->code, 
                          &out_units, &grid->zone, grid->projprms, &grid->spheroid, &in_long, 
                          &in_lat, &proj_x, &proj_y);
            if (stat == ERROR)
            {
              strcpy (ErrMsg, "Error in the projtran call.");
              xxx_LogStatus ("ALIGRID", __FILE__, __LINE__, ErrMsg);
              return (ERROR);
            }
  
            /* Calculate the output line and sample.
             */
            if (grid->frame_type == PATH_ORIENTED || grid->frame_type == PATH_MINBOX ||
                grid->frame_type == PATH_MAXBOX)
            { /* Path-oriented */
              xxx_eval (grid->gridbands[sca][ref].degree, poly_y[polyindex[ref]], proj_x, proj_y, &line);
              xxx_eval (grid->gridbands[sca][ref].degree, poly_x[polyindex[ref]], proj_x, proj_y, &samp);
            }
            else
            { /* North-up */
              line = (grid->corners.upleft.y - proj_y) / grid->gridbands[sca][ref].pixsize;
              samp = (proj_x - grid->corners.upleft.x) / grid->gridbands[sca][ref].pixsize;
            }
            grid->gridbands[sca][ref].out_lines[curr_pt] = line;
            grid->gridbands[sca][ref].out_samps[curr_pt] = samp;
  
            /* If the image is currently blank and the output line/sample fall
             * within the specified output image then flag the image as not
             * blank.
             */
            if (blank && (line > 0 && line < grid->gridbands[sca][ref].lines) &&
                (samp > 0 && samp < grid->gridbands[sca][ref].samps))
              blank = FALSE;
          } 
        }
  
        /* If no grid pixels fell within the output image for this band, then
         * exit with an error.
         */
        if (blank)
        {
          sprintf (ErrMsg, "No pixels for band %d fall within the output image.",
                   axx_get_user_band (grid->bands[curr_band]));
          xxx_LogStatus ("ALIGRID", __FILE__, __LINE__, ErrMsg);
          return (ERROR);
        }
      }
    }
  }

  return (SUCCESS);
}
