#include "aligrid.h"

/******************************************************************************
NAME:           minbox

PURPOSE:
Determines the frame in output space for the minbox north-up product.

RETURN VALUE:   Type = int
Value    Description
-----    -----------
SUCCESS  Successful completion
ERROR    Operation failed

NOTES:
This routine will determine the frame in output space for the MINBOX north-up
product.  The actual frame is determined based on the optimal band's pixel
size, but the frame is the same for every band.

ALGORITHM REFERENCES:
See axx_grid.h for a description of the grid structure.
See axx_alimodel.h for a description of the ALI model structure.
See exx_cpf.h for a description of the CPF structure.

******************************************************************************/

int minbox (
  ALIMODEL *alimodel,          /* I: ETM+ model structure                  */
  BandNumber opt_band,       /* I: "Optimal" band to work on             */
  int  num_0r_lines,         /* I: Number of lines in the 0R image for the optimal band */ 
  int  num_0r_samps,         /* I: Number of samps in the 0R image for the optimal band */
  CPF *cpf,                  /* I: CPF constants                         */
  axx_GRID_TYPE *grid)       /* I/O: Grid structure                      */
{
  int  sca, 		/* Counter for scas 				 */
       zplane,          /* Counter for elevations planes                 */
       i,               /* Loop control                                  */
       nl,              /* Number of lines in the scene                  */
       ns,              /* Number of samples in the scene                */
       out_units,       /* Code for the output projection units.         */
       stat,            /* Function return status                        */
       geo = 0,         /* Geographic projection flag                    */
       zero = 0,        /* Constant of zero                              */
       null_zone = NULLZONE,
                        /* Null zone code for projection xforms          */
       ref,             /* Reference of the first band number.           */
       currref;         /* Current reference for looping through the bands.*/
  double pxmax,         /* Maximum X projection coordinate               */
         pxmin,         /* Minimum X projection coordinate               */
         pymax,         /* Maximum Y projection coordinate               */
         pymin;         /* Minimum Y projection coordinate               */
  double elev = 0.0,    /* Elevation at the current point                */
         geo_parms[COEFCT]; /* Geo projection parameters                     */
  char ErrMsg[STRLEN];  /* Error message                                 */
  struct DBL_LS ul_ls,  /* Upper left line/sample for the corners        */
                ur_ls,  /* Upper right line/sample for the corners       */
                ll_ls,  /* Lower left line/sample for the corners        */
                lr_ls;  /* Lower right line/sample for the corners       */
  struct DBL_LAT_LONG
                ul_ltlg,  /* Upper left lat/long for the corners         */
                ur_ltlg,  /* Upper right lat/long for the corners        */
                ll_ltlg,  /* Lower left lat/long for the corners         */
                lr_ltlg;  /* Lower right lat/long for the corners        */
  struct DBL_XY ul_xy,    /* Upper left x/y for the corners              */
                ur_xy,    /* Upper right x/y for the corners             */
                ll_xy,    /* Lower left x/y for the corners              */
                lr_xy;    /* Lower right x/y for the corners             */
  DetType dettype = NOMINAL;

  /* Initialize geo_parms.
   */
  for (i = 0; i < COEFCT; i++)
    geo_parms[i] = 0.0;

  /* Convert the grid projection units to a unit code for gctp.
   */
  if (!strcmp (grid->units, "RADIANS"))
    out_units = 0;
  else if (!strcmp (grid->units, "FEET"))
    out_units = 1;
  else if (!strcmp (grid->units, "METERS"))
    out_units = 2;
  else if (!strcmp (grid->units, "SECONDS"))
    out_units = 3;
  else if (!strcmp (grid->units, "DEGREES"))
    out_units = 4;
  else if (!strcmp (grid->units, "DMS"))
    out_units = 5;

  /* Determine the reference band number.  Implement this on the optimal
   * band.  This will be the band for which the frame is calculated, but
   * the frame will be the same for all bands.
   */
  ref = opt_band;

  /* Find the SLO offsets for the first and last lines in the scene.  These
   * will determine the corner points in line/sample.
   */
  nl = num_0r_lines;
  ns = num_0r_samps;

  /* Find the sample values for the corner points.
   */
  ul_ls.samp = 0.0;
  ll_ls.samp = 0.0;
  ur_ls.samp = ns - 1;
  lr_ls.samp = ns - 1;

  /* Find the top and bottom line values for the corner points.
   */
  ul_ls.line = ur_ls.line = 0;
  ll_ls.line = lr_ls.line = nl - 1;

  for(sca = 0; sca < grid->nscas; sca++)
  {
    for( zplane=0 ; zplane<grid->gridbands[sca][ref].nzplanes ; zplane++ )
    {
      elev = ( zplane - grid->gridbands[sca][ref].zeroplane ) * grid->gridbands[sca][ref].zspacing;

      stat = axx_forward_ali (ul_ls.line, ul_ls.samp, opt_band, sca, elev, cpf, alimodel, 
                              dettype, &ul_ltlg.lat, &ul_ltlg.lng);
      if (stat == ERROR)
      {
        strcpy (ErrMsg, "Error in the forward_ali call.");
        xxx_LogStatus ("ALIGRID", __FILE__, __LINE__, ErrMsg);
        return (ERROR);
      }

      stat = axx_forward_ali (ur_ls.line, ur_ls.samp, opt_band, sca, elev, cpf, alimodel, 
                              dettype, &ur_ltlg.lat, &ur_ltlg.lng);
      if (stat == ERROR)
      {
        strcpy (ErrMsg, "Error in the forward_ali call.");
        xxx_LogStatus ("ALIGRID", __FILE__, __LINE__, ErrMsg);
        return (ERROR);
      }
  
      stat = axx_forward_ali (ll_ls.line, ll_ls.samp, opt_band, sca, elev, cpf, alimodel, 
                              dettype, &ll_ltlg.lat, &ll_ltlg.lng);
      if (stat == ERROR)
      {
        strcpy (ErrMsg, "Error in the forward_ali call.");
        xxx_LogStatus ("ALIGRID", __FILE__, __LINE__, ErrMsg);
        return (ERROR);
      }
    
      stat = axx_forward_ali (lr_ls.line, lr_ls.samp, opt_band, sca, elev, cpf, alimodel, 
                              dettype, &lr_ltlg.lat, &lr_ltlg.lng);
      if (stat == ERROR)
      {
        strcpy (ErrMsg, "Error in the forward_ali call.");
        xxx_LogStatus ("ALIGRID", __FILE__, __LINE__, ErrMsg);
        return (ERROR);
      }
    
      /* Calculate the location of the latitude and longitude in output
       * space.  (Units coming out of forward_etm are radians = 0.)
       */
      stat = exx_projtran (&geo, &zero, &null_zone, geo_parms,
             &grid->spheroid, &grid->code, &out_units, &grid->zone,
             grid->projprms, &grid->spheroid, &ul_ltlg.lng, &ul_ltlg.lat,
             &ul_xy.x, &ul_xy.y);
      if (stat == ERROR)
      {
        strcpy (ErrMsg, "Error in the projtran call.");
        xxx_LogStatus ("ALIGRID", __FILE__, __LINE__, ErrMsg);
        return (ERROR);
      }
  
      stat = exx_projtran (&geo, &zero, &null_zone, geo_parms,
             &grid->spheroid, &grid->code, &out_units, &grid->zone,
             grid->projprms, &grid->spheroid, &ur_ltlg.lng, &ur_ltlg.lat,
             &ur_xy.x, &ur_xy.y);
      if (stat == ERROR)
      {
        strcpy (ErrMsg, "Error in the projtran call.");
        xxx_LogStatus ("ALIGRID", __FILE__, __LINE__, ErrMsg);
        return (ERROR);
      }
    
      stat = exx_projtran (&geo, &zero, &null_zone, geo_parms,
             &grid->spheroid, &grid->code, &out_units, &grid->zone,
             grid->projprms, &grid->spheroid, &ll_ltlg.lng, &ll_ltlg.lat,
             &ll_xy.x, &ll_xy.y);
      if (stat == ERROR)
      {
        strcpy (ErrMsg, "Error in the projtran call.");
        xxx_LogStatus ("ALIGRID", __FILE__, __LINE__, ErrMsg);
        return (ERROR);
      }
    
      stat = exx_projtran (&geo, &zero, &null_zone, geo_parms,
             &grid->spheroid, &grid->code, &out_units, &grid->zone,
             grid->projprms, &grid->spheroid, &lr_ltlg.lng, &lr_ltlg.lat,
             &lr_xy.x, &lr_xy.y);
      if (stat == ERROR)
      {
        strcpy (ErrMsg, "Error in the projtran call.");
        xxx_LogStatus ("ALIGRID", __FILE__, __LINE__, ErrMsg);
        return (ERROR);
      }
    
      /* Find the minimum and maximum x and y corners.
       */
       if( sca==0 && zplane==0 )
          pxmin = ul_xy.x;
       else if( ul_xy.x < pxmin )
          pxmin = ul_xy.x;
       if( ll_xy.x < pxmin )
          pxmin = ll_xy.x;
       if (ur_xy.x < pxmin)
          pxmin = ur_xy.x;
       if (lr_xy.x < pxmin)
          pxmin = lr_xy.x;
  
       if( sca==0 && zplane==0 )
          pxmax = ul_xy.x;
       else if( ul_xy.x > pxmax )
          pxmax = ul_xy.x;
       if( ll_xy.x > pxmax )
         pxmax = ll_xy.x;
       if (ur_xy.x > pxmax)
         pxmax = ur_xy.x;
       if (lr_xy.x > pxmax)
         pxmax = lr_xy.x;
  
       if( sca==0 && zplane==0 )
         pymin = ul_xy.y;
       else if( ul_xy.y < pymin )
         pymin = ul_xy.y;
       if( ll_xy.y < pymin )
         pymin = ll_xy.y;
       if (ur_xy.y < pymin)
         pymin = ur_xy.y;
       if (lr_xy.y < pymin)
         pymin = lr_xy.y;

       if( sca==0 && zplane==0 )
         pymax = ul_xy.y;
       else if ( ul_xy.y > pymax )
         pymax = ul_xy.y;
       if( ll_xy.y > pymax )
         pymax = ll_xy.y;
       if (ur_xy.y > pymax)
         pymax = ur_xy.y;
       if (lr_xy.y > pymax)
         pymax = lr_xy.y;
  
       /* printf("%f %f %f %f\n",pxmin,pxmax,pymin,pymax); */

    } /* Loop on zplanes */
  } /* Loop on SCAs */

  /* Pad corners to PADVAL * pixel size.
   */
  /* I'm going to assume that there is at least one SCA for reference band
     and that the pixel size is the same for every SCA with a band.
   */
  stat = pad_corners (grid->gridbands[0][ref].pixsize, &pxmax, &pxmin, &pymax, &pymin);
  if (stat == ERROR)
  {
    strcpy (ErrMsg, "Error in the pad_corners call.");
    xxx_LogStatus ("ALIGRID", __FILE__, __LINE__, ErrMsg);
    return (ERROR);
  }
  printf("%f %f %f %f\n",pxmin,pxmax,pymin,pymax);

  /* Find the number of lines and samples in output space.
   * This must be implemented for each band.
   * NOTE: This equation adds 1 to the (x-y)/pixsize, but to round we need
   * to add another .5 since the (int) truncates to an integer value.
   */
  for (i = 0; i < grid->nbands; i++)
  {
    currref = grid->bands[i];
    for(sca = 0; sca < grid->nscas; sca++ )
    {
      grid->gridbands[sca][currref].lines = 
            (int) (((pymax - pymin) / grid->gridbands[sca][currref].pixsize) + 1.5);
      grid->gridbands[sca][currref].samps = 
            (int) (((pxmax - pxmin) / grid->gridbands[sca][currref].pixsize) + 1.5);
      /*********************************************************************************************
      printf("NL NS %d %d\n",grid->gridbands[sca][currref].lines,grid->gridbands[sca][currref].samps);
      *********************************************************************************************/
    }
  }

  /* Pack the corner projection coordinates in the frame structure
   */
  grid->corners.upleft.x = grid->corners.loleft.x = pxmin;
  grid->corners.upright.x = grid->corners.loright.x = pxmax;
  grid->corners.loleft.y = grid->corners.loright.y = pymin;
  grid->corners.upleft.y = grid->corners.upright.y = pymax;

  /* Verify the image is not greater than 2 GBs, for any band.  Assume the
   * output is in BYTES.  Thus we only need to check that the number of lines
   * times the number of samples is not greater than 2 GB.
   */
  for (i = 0; i < grid->nbands; i++)
  {
    currref = grid->bands[i];
    for(sca = 0; sca < grid->nscas; sca++ )
    {
      if (grid->gridbands[sca][currref].lines * grid->gridbands[sca][currref].samps > GB2)
      {
        sprintf (ErrMsg, "Band %d is greater than 2GB.", axx_get_user_band (grid->bands[i]));
        xxx_LogStatus ("ALIGRID", __FILE__, __LINE__, ErrMsg);
        return (ERROR);
      }
    }
  }

  return (SUCCESS);
}
