#include "aligrid.h"

/******************************************************************************
NAME:           north_up

PURPOSE:
Determines the frame in output space for the north-up product.

RETURN VALUE:   Type = int
Value    Description
-----    -----------
SUCCESS  Successful completion
ERROR    Operation failed

NOTES:
This routine will determine the frame in output space for the GEOBOX, PROJBOX,
and UL_SIZE north-up products.  The actual frame is determined based on the
optimal band's pixel size, but the frame is the same for every band.

ALGORITHM REFERENCES:
See axx_grid.h for a description of the grid structure.

******************************************************************************/

int north_up (
  axx_GRID_TYPE *grid,          /* I/O: Grid structure                      */
  BandNumber opt_band,          /* I: "Optimal" band to work on             */
  struct DBL_XY frame_coords[2],/* I: User-input frame coordinates to define
                                      the frame of the output area (upper
                                      left followed by lower right corner)  */
  struct DBL_LS lscoords,       /* I: Image coordinates of the first coord
                                      pair entered in frame_coords when proj
                                      params are being specified directly   */
  int  coord_code)              /* I: Coordinate units code                 */
{
  double pxmax,         /* Maximum X projection coordinate                   */
         pxmin,         /* Minimum X projection coordinate                   */
         pymax,         /* Maximum Y projection coordinate                   */
         pymin;         /* Minimum Y projection coordinate                   */
  int  i,               /* Loop control                                      */
       sca,		/* Counter for SCAs				     */
       out_units,       /* Code for the output projection units.             */
       status;          /* Function return status                            */
  int  ref,             /* Reference of the optimal band number.             */
       currref;         /* Current reference for looping through the bands.  */
  char ErrMsg[STRLEN];  /* Error message                                     */

  /* Determine the reference band number.  Implement this on the optimal
   * band.  This will be the band for which the frame is calculated, but the
   * frame will be the same for all bands.
   */
  ref = opt_band;

  /* Determine the frame type
   */
  if (grid->frame_type == GEOBOX)
  { /* GEOBOX */
    /* Convert the grid projection units to a unit code for gctp.
     */
    if (!strcmp (grid->units, "RADIANS"))
      out_units = 0;
    else if (!strcmp (grid->units, "FEET"))
      out_units = 1;
    else if (!strcmp (grid->units, "METERS"))
      out_units = 2;
    else if (!strcmp (grid->units, "SECONDS"))
      out_units = 3;
    else if (!strcmp (grid->units, "DEGREES"))
      out_units = 4;
    else if (!strcmp (grid->units, "DMS"))
      out_units = 5;

    /* Determine output projection space minimums and maximums.  The input
     * and output spheroid code will be the same.
     */
    status = exx_mapedg(&frame_coords[0].x, &frame_coords[1].x,
      &frame_coords[0].y, &frame_coords[1].y, &coord_code,
      &grid->spheroid, &grid->code, &grid->zone, grid->projprms, &out_units,
      &grid->spheroid, &pxmin, &pxmax, &pymin, &pymax);
    if (status != SUCCESS) 
    {
      strcpy (ErrMsg, "Error in the mapedg call in north_up.");
      xxx_LogStatus ("ALIGRID", __FILE__, __LINE__, ErrMsg);
      return (ERROR);
    }

    /* Pad corners to PADVAL * pixel size.
     */
    /* I'm going to assume that there is at least one SCA for reference band
       and that the pixel size is the same for every SCA with a band.
     */
    status = pad_corners (grid->gridbands[0][ref].pixsize, &pxmax, &pxmin, &pymax, &pymin);
    if (status == ERROR)
    {
      strcpy (ErrMsg, "Error in the pad_corners call.");
      xxx_LogStatus ("ALIGRID", __FILE__, __LINE__, ErrMsg);
      return (ERROR);
    }

    /* Find the number of lines and samples in output space.
     * This must be implemented for each band.
     * NOTE: This equation adds 1 to the (x-y)/pixsize, but to round we need
     * to add another .5 since the (int) truncates to an integer value.
     */
    for (i = 0; i < grid->nbands; i++)
    {
      currref = grid->bands[i];
      for(sca = 0; sca < grid->nscas; sca++)
      {
        grid->gridbands[sca][currref].lines 
              = (int) (((pymax - pymin) / grid->gridbands[sca][currref].pixsize) + 1.5);
        grid->gridbands[sca][currref].samps 
              = (int) (((pxmax - pxmin) / grid->gridbands[sca][currref].pixsize) + 1.5);
      }
    }
  }
  else if (grid->frame_type == PROJBOX)
  { /* PROJBOX */
    /* If output frame was given in projection coordinates, for two points
     */
    /* I'm going to assume that there is at least one SCA for reference band
       and that the pixel size is the same for every SCA with a band.
     */
    pymax = frame_coords[0].y + (lscoords.line * grid->gridbands[0][ref].pixsize);
    pxmin = frame_coords[0].x - (lscoords.samp * grid->gridbands[0][ref].pixsize);
    pymin = frame_coords[1].y;
    pxmax = frame_coords[1].x;

    /* Find the number of lines and samples in output space.
     * This must be implemented for each band.
     * NOTE: This equation adds 1 to the (x-y)/pixsize, but to round we need
     * to add another .5 since the (int) truncates to an integer value.
     */
    for (i = 0; i < grid->nbands; i++)
    {
      currref = grid->bands[i];
      for(sca = 0; sca < grid->nscas; sca++)
      {
        grid->gridbands[sca][currref].lines 
              = (int) (((pymax - pymin) / grid->gridbands[sca][currref].pixsize) + 1.5);
        grid->gridbands[sca][currref].samps 
              = (int) (((pxmax - pxmin) / grid->gridbands[sca][currref].pixsize) + 1.5);
      }
    }
  }
  else if (grid->frame_type == UL_SIZE)
  { /* UL_SIZE */
    /* If output frame was given in projection coordinates, 1 pt & nlines/samps
     */
    /* I'm going to assume that there is at least one SCA for reference band
       and that the pixel size is the same for every SCA with a band.
     */
    pymax = frame_coords[0].y + (lscoords.line * grid->gridbands[0][ref].pixsize);
    pxmin = frame_coords[0].x - (lscoords.samp * grid->gridbands[0][ref].pixsize);
    pymin = pymax - ((grid->gridbands[0][ref].lines - 1) * grid->gridbands[0][ref].pixsize);
    pxmax = pxmin + ((grid->gridbands[0][ref].samps - 1) * grid->gridbands[0][ref].pixsize);
  }

  /* Pack the corner projection coordinates in the frame structure
   */
  grid->corners.upleft.x = grid->corners.loleft.x = pxmin;
  grid->corners.upright.x = grid->corners.loright.x = pxmax;
  grid->corners.upleft.y = grid->corners.upright.y = pymax;
  grid->corners.loleft.y = grid->corners.loright.y = pymin;

  /* Verify the image is not greater than 2 GBs, for any band.  Assume the
   * output is in BYTES.  Thus we only need to check that the number of lines
   * times the number of samples is not greater than 2 GB.
   */
  for (i = 0; i < grid->nbands; i++)
  {
    currref = grid->bands[i];
    for(sca = 0; sca < grid->nscas; sca++)
    {
      if (grid->gridbands[sca][currref].lines * grid->gridbands[sca][currref].samps > GB2)
      {
        sprintf (ErrMsg, "Band %d is greater than 2GB.", axx_get_user_band (grid->bands[i]));
        xxx_LogStatus ("ALIGRID", __FILE__, __LINE__, ErrMsg);
        return (ERROR);
      }
    }
  }

  return (SUCCESS);
}
