#include "aligrid.h"

/******************************************************************************
NAME:           pad_corners

PURPOSE:
Pads the input corners by a defined factor of the pixel size.

RETURN VALUE:   Type = int
Value    Description
-----    -----------
SUCCESS  Successful completion
ERROR    Operation failed

NOTES:
The x/y min and max values are input for the corner locations.  These values
are padded by PADVAL * the pixel size.  The newly padded x/y min and max
values are returned, replacing the original values.

ALGORITHM REFERENCES:

******************************************************************************/

int pad_corners (
  double pixsize,  /* I: Pixel size for the coordinates             */
  double *pxmax,   /* I/O: Maximum X projection coordinate          */
  double *pxmin,   /* I/O: Minimum X projection coordinate          */
  double *pymax,   /* I/O: Maximum Y projection coordinate          */
  double *pymin)   /* I/O: Minimum Y projection coordinate          */

{
  int  ixmin,         /* Pixel size increment to be added to pxmin.    */
       ixmax,         /* Pixel size increment to be added to pxmax.    */
       iymin,         /* Pixel size increment to be added to pymin.    */
       iymax;         /* Pixel size increment to be added to pymax.    */

  /* Pixel size of 0.0 will result in a divide by zero.
   */
  if (pixsize == 0.0)
  {
    xxx_LogStatus ("ALIGRID", __FILE__, __LINE__, "Pixel size is zero.");
    return (ERROR);
  }

  /* Pad corners to PADVAL * pixel size.
   */
  if (*pxmin / (PADVAL * pixsize) != 0.0)
  {
    ixmin = (int) floor (*pxmin / (PADVAL * pixsize));
    *pxmin = (double) ixmin * PADVAL * pixsize;
  }
  if (*pxmax / (PADVAL * pixsize) != 0.0)
  {
    ixmax = (int)
      ceil (*pxmax / (PADVAL * pixsize));
    *pxmax = (double) ixmax * PADVAL * pixsize;
  }
  if (*pymin / (PADVAL * pixsize) != 0.0)
  {
    iymin = (int) floor (*pymin / (PADVAL * pixsize));
    *pymin = (double) iymin * PADVAL * pixsize;
  }
  if (*pymax / (PADVAL * pixsize) != 0.0)
  {
    iymax = (int)
      ceil (*pymax / (PADVAL * pixsize));
    *pymax = (double) iymax * PADVAL * pixsize;
  }

  return (SUCCESS);
}
