/******************************************************************************
NAME: path_maxmin_box

PURPOSE: This routine will provide a path-oriented product whose frame is
either the optimal band(MINBOX) or large enough to contain all bands(MAXBOX).

AUTHOR        Change-ID    DATE     Description of Change
------        ---------  --------   ---------------------
T.Ratliff     1.0         01/01     Original development

RETURN VALUE:   Type = int
Value    Description
-----    -----------
SUCCESS  Successful completion
ERROR    Operation failed

NOTES:
none

ALGORITHM REFERENCES:
This algorithm was created by combining the logic in the in both
path_oriented.c and minbox.c.  Almost all the code is from these
two modules.

******************************************************************************/
#include "aligrid.h"

int path_maxmin_box (
  ALIMODEL *amodel,         /* I: ALI model structure */
  BandNumber opt_band,      /* I: Optimal band number, used with MINBOX */
  int *num_0r_lines, /* I: Number of lines in the 0R image for the optimal band */
  int *num_0r_samps, /* I: Number of samps in the 0R image for the optimal band */
  CPF *cpf,		    /* I: CPF to determine WRS lat/long and angle */
  axx_GRID_TYPE *grid,      /* I/O: Grid structure to use/modify */
  int path,		    /* I: WRS path of product to provide */
  double row,		    /* I: WRS row of product (may be fractional) */
  double poly_x[2][4],      /* O: Coefficients which map output lines/samples */
  double poly_y[2][4]       /*    samples to projection coordinates in the
                                  order of 10m, 30m) */
  )
{
  int units;               /* Integral representation of units */
  int stat;                /* Return status */
  int i;                   /* Loop variable */
  int currref;             /* Current reference position */
  char ErrMsg[STRLEN];     /* Error message */
  int sca;		   /* Counter for SCAs */

  /* Convert the units string in the grid structure to a
     numerical representation for GCTP.
  ------------------------------------------------------*/
  if (!strcmp(grid->units,"RADIANS"))
    units = 0;
  else if (!strcmp(grid->units,"FEET"))
    units = 1;
  else if (!strcmp(grid->units,"METERS"))
    units = 2;
  else if (!strcmp(grid->units,"SECONDS"))
    units = 3;
  else if (!strcmp(grid->units,"DEGREES"))
    units = 4;
  else if (!strcmp(grid->units,"DMS"))
    units = 5;

  /* Fill in the grid structure with the framing information for the
     path-oriented product described by the path/row parameters and
     the projection information in the grid structure.
  ------------------------------------------------------------------*/
  stat = calc_center_and_rotation_angle (path, row, cpf, grid);
  if (stat == ERROR)
  {
    xxx_LogStatus ("ALIGRID", __FILE__,  __LINE__,
      "Error calculating the center point and rotation angle.");
    return ERROR;
  }

  if (grid->frame_type == PATH_MINBOX)
  {
    stat = calc_path_oriented_minbox_frame(amodel, opt_band, num_0r_lines[opt_band], 
                                           num_0r_samps[opt_band], cpf, grid, units, poly_x, poly_y);
    if (stat == ERROR)
    {
      xxx_LogStatus("ALIGRID",__FILE__,__LINE__, "Error calculating the path-oriented frame.");
      return ERROR;
    }
  }
  else
  {
    stat = calc_path_oriented_maxbox_frame(amodel, num_0r_lines, num_0r_samps, cpf, grid, units, 
                                           poly_x, poly_y);
    if (stat == ERROR)
    {
      xxx_LogStatus ("ALIGRID", __FILE__,  __LINE__, "Error calculating the path-oriented frame.");
      return ERROR;
    }
  }

  /* Verify the image is not greater than 2 GBs, for any band.  Assume the
     output is in BYTES.  Thus we only need to check that the number of
     lines times the number of samples is not greater than 2 GB.
  ------------------------------------------------------------------------*/
  for (sca = 0; sca < grid->nscas; sca++)
    for (i = 0; i < grid->nbands; i++)
    {
      currref = grid->bands[i];
      if (grid->gridbands[sca][currref].lines * grid->gridbands[sca][currref].samps > GB2)
      {
        sprintf(ErrMsg,"Band %d is greater than 2GB.", axx_get_user_band (grid->bands[i]));
        xxx_LogStatus("ALIGRID", __FILE__, __LINE__, ErrMsg);
        return ERROR;
      }
    }

  stat = angle_to_map(grid, units);
  if (stat == ERROR)
  {
    xxx_LogStatus("ALIGRID", __FILE__,  __LINE__, "Error calculating the angle for mapping.");
    return ERROR;
  }

  return SUCCESS;
}
