/******************************************************************************
NAME:           path_oriented

PURPOSE:
This routine will provide a path-oriented product that is framed to a nominal
WRS scene.  The user specifies only the projection, pixel size, and the path
and row of the scene.  

RETURN VALUE:
Type = int (SUCCESS or ERROR)

ALGORITHM REFERENCES:
Landsat 7 Image Assessment System Geometric Processing Subsystem Detailed
Design Specification, algorithms on pages 57-62.
******************************************************************************/
#include "aligrid.h"

int path_oriented (
    axx_GRID_TYPE *grid, /* I/O: Grid structure to use/modify		*/
    CPF *cpf,		 /* I: CPF to determine WRS lat/long and angle	*/
    int path,		 /* I: WRS path of product to provide		*/
    double row,		 /* I: WRS row of product (may be fractional)	*/
    double poly_x[2][4], /* O: Coefficients which map output lines/	*/
    double poly_y[2][4]) /*    samples to projection coordinates in the */
                         /*    order of 15m, 30m, 60m)                  */
{
    int  units,		 /* Integral representation of units		*/
         sca,            /* SCA counter                                 */
         stat,           /* Return status                               */
         i,              /* Loop variable                               */
         currref;        /* Current reference position                  */
    char ErrMsg[STRLEN]; /* Error message                               */

    /* Convert the units string in the grid structure to a
       numerical representation for GCTP.
    ------------------------------------------------------*/
    if (!strcmp (grid->units, "RADIANS"))
      units = 0;
    else if (!strcmp (grid->units, "FEET"))
      units = 1;
    else if (!strcmp (grid->units, "METERS"))
      units = 2;
    else if (!strcmp (grid->units, "SECONDS"))
      units = 3;
    else if (!strcmp (grid->units, "DEGREES"))
      units = 4;
    else if (!strcmp (grid->units, "DMS"))
      units = 5;

    /* Verify the image is not greater than 2 GBs, for any band.  Assume the
     * output is in BYTES.  Thus we only need to check that the number of
     * lines times the number of samples is not greater than 2 GB.
    ------------------------------------------------------------------------*/
    for (i = 0; i < grid->nbands; i++)
    {
      currref = grid->bands[i];
      for(sca = 0; sca < grid->nscas; sca++)
      {
        if (grid->gridbands[sca][currref].lines * grid->gridbands[sca][currref].samps > GB2)
        {
          sprintf (ErrMsg, "Band %d is greater than 2GB.", axx_get_user_band (grid->bands[i]));
          xxx_LogStatus ("ALIGRID", __FILE__, __LINE__, ErrMsg);
          return (ERROR);
        }
      }
    }

    /* Fill in the grid structure with the framing information for the
       path-oriented product described by the path/row parameters and
       the projection information in the grid structure.
    ------------------------------------------------------------------*/
    stat = calc_center_and_rotation_angle (path, row, cpf, grid);
    if (stat == ERROR)
    {
      xxx_LogStatus ("ALIGRID", __FILE__,  __LINE__,
        "Error calculating the center point and rotation angle.");
      return (ERROR);
    }

    stat = calc_path_oriented_frame (grid, units, poly_x, poly_y);
    if (stat == ERROR)
    {
      xxx_LogStatus ("ALIGRID", __FILE__,  __LINE__, "Error calculating the path-oriented frame.");
      return (ERROR);
    }

    stat = angle_to_map (grid, units);
    if (stat == ERROR)
    {
      xxx_LogStatus ("ALIGRID", __FILE__,  __LINE__, "Error calculating the angle for mapping.");
      return (ERROR);
    }

    return SUCCESS;
}
