/******************************************************************************
NAME:           read_num_ls

PURPOSE:
The read_num_ls routine opens the 0r or 1R HDF file for the indicated band
numbers, and returns the number of lines and samples for each band.

RETURN VALUE:   Type = int
Value    Description
-----    -----------
SUCCESS  Successful completion
ERROR    Operation failed

NOTES:
The number of lines and samples will be returned in their respective arrays,
in band-referenced order.  This is similar to the manner in which the grid
is stored.  Thus the nlines and nsamps arrays MUST be of size NBANDS.

ALGORITHM REFERENCES:
none

******************************************************************************/
#include "axx_l1r_image.h"
#include "aligrid.h"


int read_num_ls 
(
    char *HDF_name,      /* I: HDF file name of image to open (0R or 1R)      */
    const BandNumber *band, /* I: Bands of image to open                      */
    int  nbands,         /* I: Number of bands to process                     */
    int  nsca,
    int  nlines[NBANDS], /* O: Num of lines in image in band-referenced order */
    int  nsamps[NBANDS]  /* O: Num of samples in image in band-reference order*/
)
{
    BandNumber band_number;  /* Integer band number          */
    char l1r_name[STRLEN];   /* L1R HDF name                 */
    char emsg[STRLEN];       /* Error message buffer         */
    char progname[STRLEN];   /* Program name                 */
    char ext[4];             /* Extenstion for L1R           */
    int sca;                 /* Counter for SCAs             */
    L1R_SCA_BAND *input_image = NULL; /* input image file pointer */

    strcpy(progname,"GET_IMAGE_DATA");

    /* Loop on SCAs
       ---------------*/
    for( sca=0 ; sca<nsca ; sca++ )
    {
        strcpy(l1r_name,HDF_name);
        sprintf(ext,".M%dR",(sca+1));
        strcat(l1r_name,ext);

        for (band_number = pan; band_number <= b7; band_number++)
        {
            input_image = axx_open_l1r_sca_band(l1r_name, band_number);
            if (!input_image)
            {
                sprintf(emsg, "Error opening %s", l1r_name);
                xxx_LogStatus(progname, __FILE__, __LINE__, emsg);
                return ERROR;
            }

            if (sca == 0)
            {
                nsamps[band_number] = input_image->ns;
                nlines[band_number] = input_image->nl;
            }
            else
            {
                if ((nsamps[band_number] != input_image->ns)
                    || (nlines[band_number] != input_image->nl))
                {
                    strcpy(emsg,"SCA bands have different number "
                           "of lines/samples.\n");
                    xxx_LogStatus(progname,__FILE__,__LINE__, emsg);
                    return ERROR;
                }
            }

            axx_close_l1r_sca_band(input_image);
            input_image = NULL;
        }
    }

    return(SUCCESS);
}
