#include "aligrid.h"

/******************************************************************************
NAME:           validate_utm_zone

PURPOSE:
This routine will validate the UTM zone that the user entered as a parameter.
The scene center longitude will be used for this verification.

RETURN VALUE:   Type = int
Value    Description
-----    -----------
SUCCESS  Successful completion
ERROR    Operation failed

NOTES:

ALGORITHM REFERENCES:

******************************************************************************/

int validate_utm_zone (
  int  nlines,         /* I: Number of lines in image                    */
  int  nsamps,         /* I: Number of samples in image                  */
  BandNumber band,     /* I: Band number for the current band            */
  int sca,             /* I: SCA number                                  */
  ALIMODEL *alimodel,  /* I: ETM+ model structure                        */
  CPF *cpf,            /* I: CPF constants                               */
  int  zone,           /* I: Zone value to validate                      */
  int  *computed_zone, /* O: Computed zone value for the scene           */
  int *valid)          /* O: Flag to specify if the UTM zone was valid   */
{
  DBL_LS center_ls;      /* Center point line/sample          */
  DBL_LAT_LONG center_lat_long;
                         /* Center point lat/long             */
  double elev = 0.0;     /* Elevation at the current point    */
  DetType dettype = NOMINAL;
char ErrMsg[STRLEN];           /* Error message                         */

  /* Determine the center line/sample value.
   */
  center_ls.line = nlines / 2.0;
  center_ls.samp = nsamps / 2.0;

  /* Determine the lat/long for the center point.
   */
  if (axx_forward_ali (center_ls.line, center_ls.samp, band, sca, elev, cpf, alimodel, 
                       dettype, &center_lat_long.lat, &center_lat_long.lng) != SUCCESS)
  {
    xxx_LogStatus ("ALIGRID", __FILE__, __LINE__, "Error in the forward_ali call.");
    *valid = FALSE;
    return (ERROR);
  }
sprintf (ErrMsg, "Center pont lat/lon is %f/%f", center_lat_long.lat, center_lat_long.lng);
xxx_LogStatus ("ALIGRID", __FILE__, __LINE__, ErrMsg);

  /* Convert the latitude and longitude from Radian to Degrees.
   */
  center_lat_long.lat *= R2D;
  center_lat_long.lng *= R2D;
sprintf (ErrMsg, "Center pont lat/lon is %f/%f", center_lat_long.lat, center_lat_long.lng);
xxx_LogStatus ("ALIGRID", __FILE__, __LINE__, ErrMsg);

  /* Compute the scene zone based on the scene center longitude.  Verify
   * that the zone falls between 1 and 60.
   */
  *computed_zone = (int) ((center_lat_long.lng + 180.0) / 6.0) + 1;
  if (*computed_zone < 1)
    *computed_zone += 60;
  if (*computed_zone > 60)
    zone -= 60;

  /* Compare the input zone to the computed zone.  Input zones that fall
   * within one zone from the computed zone will be valid.  Use the abs
   * value of the input zone to account for southern hemisphere zones.
   * If the user-entered zone is 1 and the computed zone is 60 (or vice
   * versa), then this should be a valid user-entered zone since they are
   * only one zone apart.
   */
  if ((abs (zone) == 1 && *computed_zone == 60) ||
      (abs (zone) == 60 && *computed_zone == 1))
    *valid = TRUE;
  else if (abs (abs (zone) - *computed_zone) > 1)
    *valid = FALSE;  
  else
    *valid = TRUE;

  if (center_lat_long.lat < 0.0)
     *computed_zone = -(*computed_zone);

  return (SUCCESS);
}
