/* ***************************************************************************
NAME: alinit

PURPOSE:
Initialize the ALI model using the Calibration Parameter File, ALI image
date and EO1 housekeeping data, then dump the model out to a file.

DEVELOPMENT HISTORY:
  1.0    J. Storey      11/00   Original development

RETURN VALUE:
Type = int
Value    Description
-----    -----------
SUCCESS  Successful completion 
ERROR    Operation failed

ALGORITHM REFERENCES:
******************************************************************************
                        Property of the U.S. Government
                            USGS EROS Data Center
*******************************************************************************/
#include <stdlib.h>
#include <stdio.h>
#include <sys/types.h>
#include <strings.h>
#include <time.h>
#include "xxx_LogStatus.h"
#include "xxx_ali_level1r.h"
#include "axx_alimodel.h"
#include "axx_init_model.h"
#include "alinit.h"

int main ( int argc, char *argv[])
{
/* vars */
int  status;		    /* status of return from function	 	*/
CPF *cpf;		    /* CPF structure				*/
char err_msg[STRLEN];	    /* message containing the error		*/

/* getpar's vars */
char parm_file[STRLEN];		/* the name of the input parameter file   */
char cpf_name[STRLEN];		/* name of the cpf file                   */
char hk_data_dir[STRLEN];	/* Housekeeping data file directory name  */
char hk_file_root[STRLEN];	/* Housekeeping root file name (no ext)	  */
char img_data_dir[STRLEN];	/* ALI image data file directory name  	  */
char img_file_root[STRLEN];	/* ALI image root file name (no ext)	  */
char img_path[STRLEN];		/* ALI image file path                    */
char ephemtype[4];		/* Ephemeris type selection "ACS" or "GPS"*/
char model_file[STRLEN];        /* name of the output HDF model file	  */

/* structure for the Level 1R data */
L1R_IMAGE_ATTRIBUTES   attributes;	/* ALI level 1R data structure            */
L1R_HEADER header;

/* vars for initializing the model */
ALIMODEL  amodel;	  	/* ALI model structure                    */

/**********************************************************************/
/* make the std out line buffered */
setvbuf(stdout, NULL, _IOLBF, 0);

/* checking the number of arguments */
if (argc != 2)
{
    strcpy(err_msg,"Usage: alinit Parameter_File");
    xxx_LogStatus(PROG_NAME, __FILE__, __LINE__, err_msg);
    exit (EXIT_FAILURE);
}

/* telling the user we've started the prog */
xxx_LogStatus(PROG_NAME, __FILE__, __LINE__, "Initiated.");

/* getting the Parameter File */
strcpy(parm_file, (const char *)argv[1]);

/* Get user's parameters */
status = getpar(parm_file, cpf_name, hk_data_dir, hk_file_root,
    img_data_dir, img_file_root, ephemtype, model_file );
if (status != SUCCESS)
{
    sprintf(err_msg,"Error reading parameters from %s", parm_file);
    xxx_LogStatus(PROG_NAME, __FILE__, __LINE__, err_msg);
    exit (EXIT_FAILURE);
}

/* Initialize the model */
axx_zero_amodel(&amodel);

/*
  Setting the correction type in the model to systematic
*/
amodel.correction_type = SYSTEMATIC;

/*
  Set the ephemeris selection type
*/
if ( strncmp( ephemtype, "GPS", 3 ) == 0 )
  amodel.ephem_type = GPSPTS;
else
  amodel.ephem_type = ACSPTS;

/*
   Construct the file path for the Level 1R image from SCA #1
*/
strcpy( img_path, img_data_dir );
strcat( img_path, "/" );
strcat( img_path, img_file_root );
strcat( img_path, ".M1R" );

#ifdef DEBUG
printf("Opening image file %s\n", img_path );
#endif

/*
   Load the Level 1R data for SCA #1 into a data structure
*/
if ( get_image_info( img_path, "PN1", &attributes, &header ) != SUCCESS )
{
    strcpy(err_msg,"Error returned from axx_read_ali_image_attr_l1r.");
    xxx_LogStatus(PROG_NAME, __FILE__, __LINE__, err_msg);
    exit (EXIT_FAILURE);
}

#ifdef DEBUG
if ( attributes.valid_map[6] == VALID )
{
  printf("Image start time %s\n", attributes.data_gate_open_time);
}
else
{
  printf("Image start time %s\n", attributes.data_start_time);
}
#endif

/*
  Decode the time from YYYYDDDHHMMSS text format to year, doy, sod array
*/
if ( attributes.valid_map[6] == VALID )
{
  get_date_time( attributes.data_gate_open_time, amodel.dce_time );
}
else
{
  get_date_time( attributes.data_start_time, amodel.dce_time );
}

/*
  Load the L0R software version number
*/
amodel.l0r_sw_version[0] = header.software_version[0];
amodel.l0r_sw_version[1] = header.software_version[1];
amodel.l0r_sw_version[2] = header.software_version[2];
/*
  See if the "new" L1R attribute Scene_Id is present to determine
  if this is version 1.1.2 or higher Level 1R data.
*/
if ( (strcmp( (char *)header.scene_id, "None" ) != 0) &&
     (amodel.l0r_sw_version[0] <= 1) &&
     (amodel.l0r_sw_version[1] <= 1) &&
     (amodel.l0r_sw_version[2] <= 1) )
{
  amodel.l0r_sw_version[0] = 1;
  amodel.l0r_sw_version[1] = 1;
  amodel.l0r_sw_version[2] = 2;
}

/*
   Get values from housekeeping data and CPF.
   Get ephemeris data, and run model.
*/
if (axx_run_alimodel(hk_data_dir, hk_file_root, cpf_name, &amodel, &cpf) != SUCCESS)
{
    strcpy(err_msg,"Error returned from axx_run_amodel.");
    xxx_LogStatus(PROG_NAME, __FILE__, __LINE__, err_msg);
    exit (EXIT_FAILURE);
}

#ifdef DEBUG
/* Dump the model */
(void)print_amodel( amodel );
#endif

if (axx_put_alimodel (model_file, &amodel) != SUCCESS)
{
    exx_free_cpf(cpf);
    sprintf(err_msg,"Error writing the model to file %s", model_file);
    xxx_LogStatus(PROG_NAME, __FILE__, __LINE__, err_msg);
    exit (EXIT_FAILURE);
}

/* free the cpf */
exx_free_cpf(cpf);

/* Telling the User we're done with tminit */
xxx_LogStatus(PROG_NAME, __FILE__, __LINE__, "Successful Completion.");

return (EXIT_SUCCESS);
}
