/******************************************************************************
UNIT NAME:      xxx_read_ali_image_attr_l1r.c 

PURPOSE: Reads a Level 1R ALI (from EO-1 instrument) image data set.  All
	 data is put in the structure xxx_ali_level1r.h.  It is up to the caller of
	 this routine to access what data is needed by their routine and then
	 to free up the memory associated with the structure xxx_ali_level1r.h.        

*******************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <limits.h>
#include <unistd.h>
#include <sys/types.h>
#include <stddef.h>
#include "xxx_Types.h"
#include "xxx_LogStatus.h"
#include "xxx_ali_level1r.h"

int get_image_info(
char *file_name,			/*I: filename containing imagery */
char *sensor_name,			/*I: Sensor (or band) name */
L1R_IMAGE_ATTRIBUTES *attributes,	/*O: Structure containing L1R attributes */
L1R_HEADER *header)			/*O: Structure containing L1R header info */
{
   char emsg[STRLEN];		/* Error message buffer			*/
   char progname[STRLEN];	/* Program name				*/
   char name1[STRLEN];
   char name2[STRLEN];
   int32 sds_id;                /* SDS file ID                          */
   int32 start[3];		/* Starting dims of SDS "slab"		*/
   int32 edges[3];		/* Edge dims of SDS "slab"		*/
   int32 stride[3];		/* Increment dims of SDS "slab"		*/
   int32 size;			/* Size needed for array allocation	*/ 
   int32 attr_index;		/* Index of attributes			*/
   int32 dim_sizes[3];
   int32 HDF_type;
   int32 number_of_attributes;
   int32 number_of_dims;
   int32 sds_index;             /* Index of current SDS			*/
   int32 sd_id;                 /* ID of current SD			*/
   int32 n_datasets;            /* Number of data sets in HDF file      */
   int32 n_file_attrs;          /* Number of file attributes            */
   int16 found;

   sprintf(progname,"READ_L1R_IMAGE_ATTRIBUTES");

   memset( attributes, 0, sizeof(L1R_IMAGE_ATTRIBUTES) );

   /* Get SD file ID
   ----------------*/
   if ((sd_id = SDstart(file_name, DFACC_RDONLY)) == FAIL)
   {
      strcpy(emsg,"Could not open SDS file\n");
      xxx_LogStatus(progname,__FILE__,__LINE__, emsg);
      return(ERROR);
   }
   /* Get SD file info
   ------------------*/
   if( SDfileinfo(sd_id, &n_datasets, &n_file_attrs) == FAIL )
   {
      strcpy(emsg,"Could not retrieve SDS file info.\n");
      xxx_LogStatus(progname,__FILE__,__LINE__, emsg);
      return(ERROR);
   }
   /* Check to see if correct number of data sets.
   ----------------------------------------------*/
   if( n_datasets != 6 )
   {
      strcpy(emsg,"Incorrect number of datasets in L1R\n");
      xxx_LogStatus(progname,__FILE__,__LINE__, emsg);
      return(ERROR);
   }

   /* Read file attributes of SD.
   ------------------------------*/
   if ( xxx_read_ali_l1r_header( sd_id, header ) != SUCCESS )
   {
      strcpy(emsg,"Could not Level 1R file header attributes.\n");
      xxx_LogStatus(progname,__FILE__,__LINE__, emsg);
      return(ERROR);
   }

   sds_index = 0;
   found = 0;
   for( sds_index=0 ; sds_index<n_datasets ; sds_index++ )
   {
      /* Select SDS index for data
         ------------------------------------- */
      sds_id = SDselect(sd_id, sds_index);

      /* Get SDS info
         ------------ */
      if( SDgetinfo(sds_id, name1, &number_of_dims, dim_sizes, &HDF_type,
                    &number_of_attributes ) == FAIL )
      {
         strcpy(emsg,"Could get SDS info.\n");
         xxx_LogStatus(progname,__FILE__,__LINE__, emsg);
         return(ERROR);
      }

      attributes->valid_map[1] = INVALID;
      if ( (attr_index = SDfindattr( sds_id, "ALI Sensor")) != FAIL )
      {
        if( SDreadattr(sds_id, attr_index, name1) == FAIL )
        {
          sprintf(emsg,"Could not read attribute.");
          xxx_LogStatus(progname,__FILE__,__LINE__,emsg);
          return(ERROR);
        }
      }
      attributes->valid_map[0] = INVALID;
      if ( (attr_index = SDfindattr( sds_id, "Dataset Type")) != FAIL )
      {
        if( SDreadattr(sds_id, attr_index, name2) == FAIL )
        {
          sprintf(emsg,"Could not read attribute.");
          xxx_LogStatus(progname,__FILE__,__LINE__,emsg);
          return(ERROR);
        }
      }
      if( (strncmp(sensor_name,name1,3) == 0) && (strncmp("Level 1R",name2,6) ==0) )
      { 
        sds_index = n_datasets;
        strcpy(attributes->ali_sensor,name1);
        attributes->valid_map[1] = VALID;
        strcpy(attributes->dataset_type,name2);
        attributes->valid_map[0] = VALID;
        if ( (attr_index = SDfindattr( sds_id, "ALI Sensor")) != FAIL )
        {
          if( SDreadattr(sds_id, attr_index, attributes->ali_sensor) == FAIL )
          {
            sprintf(emsg,"Could not read attribute.");
            xxx_LogStatus(progname,__FILE__,__LINE__,emsg);
            return(ERROR);
          }
          attributes->valid_map[1] = VALID;
        }
        attributes->valid_map[2] = INVALID;
        if ( (attr_index = SDfindattr( sds_id, "Data Start Time")) != FAIL )
        {
          if( SDreadattr(sds_id, attr_index, attributes->data_start_time) == FAIL )
          {
            sprintf(emsg,"Could not read attribute.");
            xxx_LogStatus(progname,__FILE__,__LINE__,emsg);
            return(ERROR);
          }
          attributes->valid_map[2] = VALID;
        }
        attributes->valid_map[3] = INVALID;
        if ( (attr_index = SDfindattr( sds_id, "Data Duration")) != FAIL )
        {
          if( SDreadattr(sds_id, attr_index, &attributes->data_duration) == FAIL )
          {
            sprintf(emsg,"Could not read attribute.");
            xxx_LogStatus(progname,__FILE__,__LINE__,emsg);
            return(ERROR);
          }
          attributes->valid_map[3] = VALID;
        }
        attributes->valid_map[4] = INVALID;
        if ( (attr_index = SDfindattr( sds_id, "Integration Time")) != FAIL )
        {
          if( SDreadattr(sds_id, attr_index, &attributes->integration_time) == FAIL )
          {
            sprintf(emsg,"Could not read attribute.");
            xxx_LogStatus(progname,__FILE__,__LINE__,emsg);
            return(ERROR);
          }
          attributes->valid_map[4] = VALID;
        }
        attributes->valid_map[5] = INVALID;
        if ( (attr_index = SDfindattr( sds_id, "Frame Rate")) != FAIL )
        {
          if( SDreadattr(sds_id, attr_index, &attributes->frame_rate) == FAIL )
          {
            sprintf(emsg,"Could not read attribute.");
            xxx_LogStatus(progname,__FILE__,__LINE__,emsg);
            return(ERROR);
          }
          attributes->valid_map[5] = VALID;
        }
        attributes->valid_map[6] = INVALID;
        if ( (attr_index = SDfindattr( sds_id, "Data Gate Open Time")) != FAIL )
        {
          if( SDreadattr(sds_id, attr_index, &attributes->data_gate_open_time) == FAIL )
          {
            sprintf(emsg,"Could not read attribute.");
            xxx_LogStatus(progname,__FILE__,__LINE__,emsg);
            return(ERROR);
          }
          attributes->valid_map[6] = VALID;
        }
        attributes->valid_map[7] = INVALID;
        if ( (attr_index = SDfindattr( sds_id, "Data Gate Close Time")) != FAIL )
        {
          if( SDreadattr(sds_id, attr_index, &attributes->data_gate_close_time) == FAIL )
          {
            sprintf(emsg,"Could not read attribute.");
            xxx_LogStatus(progname,__FILE__,__LINE__,emsg);
            return(ERROR);
          }
          attributes->valid_map[7] = VALID;
        }
        attributes->valid_map[8] = INVALID;
        if ( (attr_index = SDfindattr( sds_id, "Bytes prior to SYNC")) != FAIL )
        {
          if( SDreadattr(sds_id, attr_index, &attributes->bytes_prior_sync) == FAIL )
          {
            sprintf(emsg,"Could not read attribute.");
            xxx_LogStatus(progname,__FILE__,__LINE__,emsg);
            return(ERROR);
          }
          attributes->valid_map[8] = VALID;
        }
        attributes->valid_map[9] = INVALID;
        if ( (attr_index = SDfindattr( sds_id, "Number of cross track pixels")) != FAIL )
        {
          if( SDreadattr(sds_id, attr_index, &attributes->number_cross_track_pixels) == FAIL )
          {
            sprintf(emsg,"Could not read attribute.");
            xxx_LogStatus(progname,__FILE__,__LINE__,emsg);
            return(ERROR);
          }
          attributes->valid_map[9] = VALID;
        }
        attributes->valid_map[10] = INVALID;
        if ( (attr_index = SDfindattr( sds_id, "Number of along track pixels")) != FAIL )
        {
          if( SDreadattr(sds_id, attr_index, &attributes->number_along_track_pixels) == FAIL )
          {
            sprintf(emsg,"Could not read attribute.");
            xxx_LogStatus(progname,__FILE__,__LINE__,emsg);
            return(ERROR);
          }
          attributes->valid_map[10] = VALID;
        }
        attributes->valid_map[11] = INVALID;
        if ( (attr_index = SDfindattr( sds_id, "Number of bands")) != FAIL )
        {
          if( SDreadattr(sds_id, attr_index, &attributes->number_of_bands) == FAIL )
          {
            sprintf(emsg,"Could not read attribute.");
            xxx_LogStatus(progname,__FILE__,__LINE__,emsg);
            return(ERROR);
          }
          attributes->valid_map[11] = VALID;
        }
        attributes->valid_map[12] = INVALID;
        if ( (attr_index = SDfindattr( sds_id, "Percent Missing Data")) != FAIL )
        {
          if( SDreadattr(sds_id, attr_index, &attributes->percent_missing_data) == FAIL )
          {
            sprintf(emsg,"Could not read attribute.");
            xxx_LogStatus(progname,__FILE__,__LINE__,emsg);
            return(ERROR);
          }
          attributes->valid_map[12] = VALID;
        }
        found = 1;
      }
   }
   
   if( SDendaccess(sds_id) == FAIL )
   {
      sprintf(emsg,"Ending SDS access error.");
      xxx_LogStatus(progname,__FILE__,__LINE__,emsg);
      return(ERROR);
   }

   /* Close the SDS file
   --------------------*/
   if ( SDend( sd_id ) == FAIL )
   {
      strcpy(emsg,"Closing SD file ID failed!\n ");
      xxx_LogStatus(progname,__FILE__,__LINE__, emsg);
      return(ERROR);
   }

   if( found )
      return(SUCCESS);
   else
      return(ERROR);
}
