/*****************************************************************************
NAME: GETPAR

PURPOSE:
Get the input parameters from the Parameter_File
(Odl Format).

DEVELOPMENT HISTORY:
  1.0    J. Storey      11/00   Original development
$Log: getpar.c,v $
Revision 1.1  2000/11/28 16:24:15  storey
Original version - ODL parameter input routine for alinit application


RETURN VALUE:
Type = int
Value    Description
-----    -----------
SUCCESS  Successful completion 
ERROR    Operation failed

NOTES: 

ALGORITHM REFERENCES:
******************************************************************************
                        Property of the U.S. Government
                            USGS EROS Data Center
*******************************************************************************/
#include <stddef.h>
#include <stdio.h>
#include <string.h>
#include "xxx_ODL.h"
#include "xxx_Types.h"
#include "xxx_LogStatus.h"
#include "lablib3.h"
#include "xxx_const.h"

int getpar(
    char parm_file[STRLEN],	/* I: the name of the Parm File 	*/
    char cpf_name[STRLEN],	/* O: name of the cpf files		*/
    char hk_data_dir[STRLEN],	/* O: housekeeping data directory 	*/
    char hk_file_root[STRLEN],	/* O: housekeeping data file name root 	*/
    char img_data_dir[STRLEN],	/* O: ALI image data directory 		*/
    char img_file_root[STRLEN],	/* O: ALI image data file name root 	*/
    char ephemtype[4],		/* O: "GPS" or "ACS" ephemeris switch	*/
    char model_file[STRLEN])	/* O: name of output HDF file		*/

{
/* Vars */
char err_msg[STRLEN];		/* error message */
int array_count;		/* number of elements in array */
int status;			/* status of return from function */

/* GSFC Vars */
xxx_ValType_TYPE parm_type;	/* type parameter read from Parm File */

/* ODL vars */
OBJDESC *in_parms;
xxx_EDCOdl_TYPE odl_type;

/* ******************************/
/* **** OPEN the Parm File **** */
/* ******************************/
odl_type = xxx_NoEDCMeta;	/* don't determine input ODL file name */
in_parms = xxx_OpenODL(parm_file,odl_type,err_msg);
if (in_parms == NULL)
{
    /* print xxx_ funct's errmsg */
    xxx_LogStatus("ALINIT",__FILE__,__LINE__,err_msg);

    sprintf(err_msg, "Error opening ODL file: %s",parm_file);
    xxx_LogStatus("ALINIT",__FILE__,__LINE__,err_msg);
    xxx_CloseODL(in_parms, err_msg);
    return(ERROR);
}

/* ******************************/
/* **** READ the Parm File **** */
/* ******************************/
/* name of the Calibration parameter file */
parm_type = XXX_String;
status = xxx_GetODLField(cpf_name,STRLEN, parm_type,
    in_parms, "ALINIT", "CAL_PARM_FILE", err_msg, &array_count);
if (status != SUCCESS || strcmp(cpf_name,"") == 0) 
{
    xxx_LogStatus("ALINIT", __FILE__, __LINE__,
    "Could not retrieve input CPF filename");
    xxx_CloseODL(in_parms, err_msg);
    return(ERROR);
}

/* name of the Housekeeping data directory */
parm_type = XXX_String;
status = xxx_GetODLField(hk_data_dir,STRLEN, parm_type,
    in_parms, "ALINIT", "HK_DATA_DIR", err_msg, &array_count);
if (status != SUCCESS || strcmp(hk_data_dir,"") == 0) 
{
    xxx_LogStatus("ALINIT", __FILE__, __LINE__,
    "Could not retrieve input housekeeping data directory name");
    xxx_CloseODL(in_parms, err_msg);
    return(ERROR);
}

/* name of the Housekeeping data file root name */
parm_type = XXX_String;
status = xxx_GetODLField(hk_file_root,STRLEN, parm_type,
    in_parms, "ALINIT", "HK_FILE_ROOT", err_msg, &array_count);
if (status != SUCCESS || strcmp(hk_file_root,"") == 0) 
{
    xxx_LogStatus("ALINIT", __FILE__, __LINE__,
    "Could not retrieve input housekeeping file root name");
    xxx_CloseODL(in_parms, err_msg);
    return(ERROR);
}

/* name of the ALI image data directory */
parm_type = XXX_String;
status = xxx_GetODLField(img_data_dir,STRLEN, parm_type,
    in_parms, "ALINIT", "IMG_DATA_DIR", err_msg, &array_count);
if (status != SUCCESS || strcmp(img_data_dir,"") == 0) 
{
    xxx_LogStatus("ALINIT", __FILE__, __LINE__,
    "Could not retrieve input ALI image data directory name");
    xxx_CloseODL(in_parms, err_msg);
    return(ERROR);
}

/* name of the ALI image data file root name */
parm_type = XXX_String;
status = xxx_GetODLField(img_file_root,STRLEN, parm_type,
    in_parms, "ALINIT", "IMG_FILE_ROOT", err_msg, &array_count);
if (status != SUCCESS || strcmp(img_file_root,"") == 0) 
{
    xxx_LogStatus("ALINIT", __FILE__, __LINE__,
    "Could not retrieve input ALI image file root name");
    xxx_CloseODL(in_parms, err_msg);
    return(ERROR);
}

/* Ephemeris type selection switch */
parm_type = XXX_String;
status = xxx_GetODLField(ephemtype,STRLEN, parm_type,
    in_parms, "ALINIT", "EPHEM_TYPE", err_msg, &array_count);
if (status != SUCCESS || strcmp(ephemtype,"") == 0) 
{
    xxx_LogStatus("ALINIT", __FILE__, __LINE__,
    "Could not retrieve ephemeris type, will use ACS");
    strcpy(ephemtype,"ACS");
}

/* name of output HDF model file */
parm_type = XXX_String;
status = xxx_GetODLField(model_file,STRLEN, parm_type,
    in_parms, "ALINIT", "AMODEL_FILE_NAME", err_msg, &array_count);
if (status != SUCCESS || strcmp(model_file,"") == 0) 
{
    xxx_LogStatus("ALINIT", __FILE__, __LINE__,
    "Could not retrieve output model filename, will use: amodel.stm");
    strcpy(model_file,"amodel.stm");
}

/* ****************************************** */
/* **** CLOSE the IAS Geometry Parm File **** */
/* ****************************************** */
status = xxx_CloseODL(in_parms, err_msg);
if (status != SUCCESS) 
{
    /* print xxx_ funct's errmsg */
    xxx_LogStatus("ALINIT",__FILE__,__LINE__,err_msg);

    xxx_LogStatus("ALINIT", __FILE__, __LINE__, 
	"Error closing ODL file");
    return(ERROR);
}

return(SUCCESS);
}
