#include <stdlib.h>
#include <math.h>
#include "xxx_Types.h"     /* SUCCESS/ERROR definitions */
#include "xxx_LogStatus.h" /* xxx_LogStatus prototype */
#include "xxx_proj.h"	   /* NULLZONE */
#include "axx_grid.h"

int calc_path_row(
    int *path,		/* I: WRS path number                          */
    int *row,		/* I: WRS row number (may be fractional)       */
    const axx_GRID_TYPE *grid	/* I: Grid structure                         */
)
{
   int i;			/* Counter				*/
   double twopie = 2.0 * PIE;
   double inclination;		/* Inclination angle in radians         */
   double long_path1_row60;	/* Longitude of path 1, row 60          */
   double earth_spin_rate;	/* Earth solar rotation rate            */
   double sc_ang_rate;		/* Angular velocity of spacecraft       */
   double central_angle;	/* Central travel angle for this row    */
   double wrs_gclat;		/* WRS scene center geocentric latitude */
   double long_origin;		/* Longitude of row 60 for given path   */
   double delta_long;		/* Longitude offset due to central ang. */
   double wrs_long;		/* WRS scene center longitude           */
   double wrs_lat;		/* WRS scene center latitude            */
   double geo_parms[COEFCT];	/* Geo projection parmeters		*/
   double proj_x;		/* Scene center projection X		*/
   double proj_y;		/* Scene center projection Y		*/
   char emsg[STRLEN];		/* Error messaging			*/
   int in_units;		/* Projection units code 		*/
   int geo = 0;			/* Geographic projection flag 		*/
   int zero = 0;		/* Zero 				*/
   int null_zone = NULLZONE;	/* NULL projection zone			*/
   int stat;			/* return status			*/
   double semi_major;		/* Semi major axis of spheroid		*/
   double semi_minor;		/* Semi minor axis of spheroid		*/
   double orbits_per_cycle;	/* Orbits per WRS cycle (233)    	*/
   double days_per_cycle;	/* Days per WRS cycle (16)     		*/
   double scenes_per_orbit;	/* Scene per orbit of spacecraft	*/
   double path1_row60;		/* Longitude of path 1 row 60		*/
   double descending_node_row;
   double tmp;

   if( get_path_row_parameters( &semi_major, &semi_minor, &inclination, &orbits_per_cycle,
                               &days_per_cycle, &path1_row60, &scenes_per_orbit,
                               &descending_node_row) != SUCCESS ) 
   {
      strcpy (emsg, "Error getting CPF parameters call.");
      xxx_LogStatus ("CALC_PATH_ROW", __FILE__, __LINE__, emsg);
      return (ERROR);
   }

  /* Initialize geo_parms.
   ----------------------*/
   for (i = 0; i < COEFCT; i++)
     geo_parms[i] = 0.0;

   if (!strcmp (grid->units, "RADIANS"))
     in_units = 0;
  else if (!strcmp (grid->units, "FEET"))
     in_units = 1;
   else if (!strcmp (grid->units, "METERS"))
     in_units = 2;
   else if (!strcmp (grid->units, "SECONDS"))
     in_units = 3;
   else if (!strcmp (grid->units, "DEGREES"))
     in_units = 4;
   else if (!strcmp (grid->units, "DMS"))
     in_units = 5;

   proj_x = (grid->corners.upright.x + grid->corners.upleft.x ) / 2.0;
   proj_y = (grid->corners.upleft.y + grid->corners.loleft.y ) / 2.0;
   stat = exx_projtran (&grid->code, &in_units, &grid->zone, grid->projprms, &grid->spheroid,
                        &geo, &zero, &null_zone, geo_parms, &grid->spheroid,
                        &proj_x, &proj_y, &wrs_long, &wrs_lat);
   if (stat == ERROR)
   {
      strcpy (emsg, "Error in the projtran call.");
      xxx_LogStatus ("CALC_PATH_ROW", __FILE__, __LINE__, emsg);
      return (ERROR);
   }

   /* Convert input angles to radians.
   -----------------------------------*/
   inclination = D2R * inclination;
   long_path1_row60 = D2R * path1_row60;

   earth_spin_rate = twopie / (double) SEC_PER_DAY;
   sc_ang_rate = (twopie * orbits_per_cycle) / (days_per_cycle * SEC_PER_DAY);

   while( wrs_long > PIE )
      wrs_long = wrs_long - 2.0 * PIE;
   while( wrs_long < -PIE )
      wrs_long = wrs_long + 2.0 * PIE;

   /* Convert the WRS geodetic latitude to geocentric latitude.
   ------------------------------------------------------------*/
   wrs_gclat = atan(tan(wrs_lat) * (semi_minor / semi_major) * (semi_minor / semi_major));

   central_angle = asin(-1.0*(sin(wrs_gclat) / sin(inclination)));
   
   /* Calculate floating point row.
   -------------------------------*/
   tmp = ( scenes_per_orbit * central_angle ) / twopie + descending_node_row; 

   /* Round to nearest row.
   ----------------------*/
   *row = (int)floor(tmp + 0.5);

   delta_long = atan2(tan(wrs_gclat) / tan(inclination), cos(central_angle) / cos(wrs_gclat));

   long_origin = wrs_long + delta_long + central_angle * ( earth_spin_rate / sc_ang_rate );

   /* Calculate floating point path.
   -------------------------------*/
   tmp = (( -long_origin + long_path1_row60 ) / twopie) * orbits_per_cycle + 1;

   /* See if the row has wrapped out of range.
   -----------------------------------------*/
   if ( *row < 1 )
   {
      tmp -= 16.0;
      *row += scenes_per_orbit;
   }
   if ( *row > scenes_per_orbit )
   {
      tmp += 16.0;
      *row -= scenes_per_orbit;
   }

   /* Round to nearest path.
   ----------------------*/
   *path = (int)floor(tmp + 0.5);

   /* Make sure the path is in range.
   --------------------------------*/
   if ( *path < 1 ) *path += orbits_per_cycle;
   if ( *path > orbits_per_cycle ) *path -= orbits_per_cycle;

   return(SUCCESS);
}
