/******************************************************************************
NAME:           generate_ddr

PURPOSE:

ROUTINES:
    generate_ddr
    get_WRS_path_row

******************************************************************************/

#include "xxx_DDR.h"       /* DDR definition */
#include "xxx_LogStatus.h" /* xxx_LogStatus prototype */
#include "xxx_utility.h"   /* axx_create_DDR_name prototype */
#include "generate_ddr.h"  /* prototypes for this module */
#include "aliresample.h"   /* PROGRAM_NAME definition */
#include "get_resample_parameters.h" /* resampler control parameters */
#include "debug.h"         /* ASSERT macro */
#include "calc_path_row.h"

/*
**
** LOCAL VARIABLES
**
*/

static int WRS_path, WRS_row;

/******************************************************************************
NAME:           generate_ddr

PURPOSE:

RETURN VALUE:
Type = int
Value           Description
-----           -----------
ERROR           Error; image not opened
SUCCESS         Successfully determined the scan direction

NOTES:
none

ALGORITHM REFERENCES:
none

******************************************************************************/

int generate_ddr
(
    const axx_GRID_TYPE *grid_ptr, 	/* I: pointer to grid used to create output image */
    int bands_to_output, 		/* I: number of bands of information to write to the 1g DDR */
    int sca_number,			/* I: SCA number */
    const BandNumber *band_list_ptr 	/* I: pointer to list of bands to output */
)
{
    xxx_DDRControl_TYPE target_DDR_control; 	/* target image DDR control structure */
    xxx_DDR_TYPE *DDR_ptr;   			/* pointer to current band's DDR */
    char error_msg[STRLEN];       /* error message for xxx libraries */
    char target_DDR_name[STRLEN]; /* target image DDR name */
    int count;                    /* loop counter */
    BandNumber band;              /* band number */
    int index;                    /* temporary index */
    float min_radiance;           /* minimum radiance value for a band */
    float max_radiance;           /* maximum radiance value for a band */

    /* create the target image DDR name */
    xxx_create_DDR_name(target_DDR_name, get_1G_filename());

    /* save the WRS path and row from the DDR, truncating the row to an
       integer since it is a float in the DDR structure */
    /* get_WRS_path_row(&WRS_path,&WRS_row); */
    if( calc_path_row(&WRS_path,&WRS_row,grid_ptr) != SUCCESS )
    {
	xxx_LogStatus(PROGRAM_NAME,__FILE__, __LINE__, "Error calculating path/row.");
	return (ERROR);
    }
    printf("Calculated PATH ROW %d %d\n",WRS_path,WRS_row);

    /* allocate memory for the DDR information of the bands generated */
    target_DDR_control.p_DDR = malloc(sizeof(xxx_DDR_TYPE) * bands_to_output);
    if (target_DDR_control.p_DDR == NULL)
    {
	xxx_LogStatus(PROGRAM_NAME,__FILE__, __LINE__, "Error allocating memory for 1g DDR.");
	return (ERROR);
    }

    /* Set the DDR VData name and class */
    (void)strcpy(target_DDR_control.vdataName, "Level 1G DDR");
    (void)strcpy(target_DDR_control.vdataClass, "IAS_DDR");

    /* set the count of bands in the target DDR */
    target_DDR_control.Count = bands_to_output;
   
    for (count = 0; count < bands_to_output; count++)
    {
        band = band_list_ptr[count];
        
        DDR_ptr = &target_DDR_control.p_DDR[count];
  
        DDR_ptr->BandNumber = axx_get_user_band(band);
        DDR_ptr->NumberScansInScene = 2295;
        DDR_ptr->NumberdetectorsPerScan = 320;

	/* set the output window information in the DDR */
        DDR_ptr->NumberLinesInScene = grid_ptr->gridbands[sca_number][band].lines;
	DDR_ptr->NumberSamplesInScene = grid_ptr->gridbands[sca_number][band].samps;
        /* Hard coding these values for now. */
	DDR_ptr->LineSubsampleInc = 1;
        DDR_ptr->SampleSubsampleInc = 1;
	DDR_ptr->MasterLine = 1;
	DDR_ptr->MasterSample = 1;

        strcpy(DDR_ptr->SpaceCraftSource,"EO1");
        strcpy(DDR_ptr->InstrumentSource,"ALI");

        DDR_ptr->WRSPath = WRS_path;
        DDR_ptr->WRSRows = WRS_row;

        strcpy(DDR_ptr->CaptureDirection,XXX_DIRECTION_DESCEND);
        strcpy(DDR_ptr->CaptureDate,"00/00/00");
        strcpy(DDR_ptr->CaptureTime,"00:00:00");
        memset( DDR_ptr->FieldValidityFlags, 0, XXX_VALID_FLAGS );

	/* set the data type in the DDR */
	switch (get_output_data_type())
	{
            case BYTE_OUTPUT :
                DDR_ptr->DataType = DFNT_UINT8;
	        break;
		
	    case INT16_OUTPUT :
	        DDR_ptr->DataType = DFNT_INT16;
	        break;
		
	    case INT32_OUTPUT :
                DDR_ptr->DataType = DFNT_INT32;
	        break;
		
	    default :
	        /* verify the output type is FLOAT, in case someone adds one */
		ASSERT (get_output_data_type() == FLOAT_OUTPUT);

                DDR_ptr->DataType = DFNT_FLOAT32;
		break;
	}

	/* set the dynamic range of the output pixels */
	DDR_ptr->MaxPixelValue = get_output_range_maximum();
	DDR_ptr->MinPixelValue = get_output_range_minimum();

	/* set the projection coordinates */

	/* copy the projection window to the DDR */
	DDR_ptr->UpperLeft[1] = grid_ptr->corners.upleft.x;
	DDR_ptr->UpperLeft[0] = grid_ptr->corners.upleft.y;
	DDR_ptr->LowerLeft[1] = grid_ptr->corners.loleft.x;
	DDR_ptr->LowerLeft[0] = grid_ptr->corners.loleft.y;
	DDR_ptr->UpperRight[1] = grid_ptr->corners.upright.x;
	DDR_ptr->UpperRight[0] = grid_ptr->corners.upright.y;
	DDR_ptr->LowerRight[1] = grid_ptr->corners.loright.x;
	DDR_ptr->LowerRight[0] = grid_ptr->corners.loright.y;

	/* set the projected pixel size */
	DDR_ptr->ProjectionDistY = grid_ptr->gridbands[sca_number][band].pixsize;
	DDR_ptr->ProjectionDistX = grid_ptr->gridbands[sca_number][band].pixsize;

	/* Set the correction type */
	if (do_terrain_correction())
	{
	    DDR_ptr->CorrectionType = xxx_Terrain;
	}
	else if (grid_ptr->correction_type == PRECISION)
	{
            /* precision correction */
	    DDR_ptr->CorrectionType = xxx_Precision;
	}
	else
	{
            /* assume systematic correction */
	    DDR_ptr->CorrectionType = xxx_Systematic;
	}

        /* copy other information from the grid */
        DDR_ptr->ProjectCode = (int16)grid_ptr->code;
        DDR_ptr->ZoneCode = (int16)grid_ptr->zone;
        DDR_ptr->SpheroidCode = grid_ptr->spheroid;
 
        /* verify sizes are compatible */
        ASSERT(sizeof(DDR_ptr->DatumCode) >= sizeof(grid_ptr->datum));
        strcpy((char*)DDR_ptr->DatumCode, grid_ptr->datum);

        ASSERT(sizeof(DDR_ptr->ProjectionUnits) >= sizeof(grid_ptr->units));
        strcpy((char*)DDR_ptr->ProjectionUnits, grid_ptr->units);
        
        ASSERT(sizeof(DDR_ptr->ProjectionCoef) >= sizeof(grid_ptr->projprms));
        for (index = 0; index < XXX_PROJECTION_COEF_SIZE; index++)
            DDR_ptr->ProjectionCoef[index] = grid_ptr->projprms[index];

        /* get the min/max radiance values used for the band */
        if (get_band_min_max_radiance(band, &min_radiance, &max_radiance) != SUCCESS)
        {
            sprintf(error_msg, "Error: unable to determine radiance values"
                    " for band %d", axx_get_user_band(band));
            xxx_LogStatus(PROGRAM_NAME,__FILE__, __LINE__, error_msg);
            free(target_DDR_control.p_DDR);
            return (ERROR);
        }
        
        /* set the min/max radiance values in the DDR */
        DDR_ptr->MinRadianceValue = min_radiance;
        DDR_ptr->MaxRadianceValue = max_radiance;
        
    }

    /* write the target DDR to the 1g image */
    if (xxx_WriteDDR(target_DDR_name, &target_DDR_control, error_msg) == ERROR)
    {
        xxx_LogStatus(PROGRAM_NAME,__FILE__, __LINE__, error_msg);
	free(target_DDR_control.p_DDR);
	return (ERROR);
    }
    
    /* free the memory - note that the target DDR does not 
       use XXX_FREE_DDR since it is allocated on the stack */
    free(target_DDR_control.p_DDR);

    /* return that the DDR was successfully written */
    return (SUCCESS);
}

/******************************************************************************
NAME:           get_WRS_path_row

PURPOSE:

RETURN VALUE:
Type = void
Value           Description
-----           -----------
WRS_path_ptr    pointer to the return location for the WRS path
WRS_row_ptr     pointer to the return location for the WRS row

******************************************************************************/

void get_WRS_path_row
(
    int *WRS_path_ptr,   /* O: pointer to return location for WRS_path */
    int *WRS_row_ptr     /* O: pointer to return location for WRS_row */
)
{
    printf("WRS is hard coded to 39 36\n");
    *WRS_path_ptr = 39; /* WRS_path; */
    *WRS_row_ptr = 36;  /* WRS_row; */
}
