/******************************************************************************
NAME:           get_resample_cpf

PURPOSE:
This module reads the cpf parameters required by the resampler.

ROUTINES:
    read_resample_cpf
    get_forward_MTF_weights_along
    get_reverse_MTF_weights_along
    get_forward_MTF_weights_across
    get_reverse_MTF_weights_across
    get_detector_offsets
    get_cpf
    cleanup_resample_cpf

******************************************************************************/

#include <stdio.h>
#include <string.h>
#include "xxx_LogStatus.h"            /* xxx_LogStatus prototype */
#include "xxx_const.h"                /* BandNumber enumerated type */
#include "get_resample_cpf.h"         /* prototypes for this module */
#include "aliresample.h"              /* PROGRAM_NAME definition */
#include "get_resample_parameters.h"  /* get_0R_filename prototype */
#include "debug.h"                    /* ASSERT macro */
#include "mtf.h"                      /* MTF_PARAMETERS_TYPE */

/*
**
** VARIABLE DECLARATIONS
**
*/

static DETECTOR_OFFSETS_TYPE detector_offsets[NUM_ALI_BANDS];
/* structure to store the detector offsets for all the bands */

static MTF_PARAMETERS_TYPE mtf_weights_along[NUM_ALI_BANDS];
static MTF_PARAMETERS_TYPE mtf_weights_across[NUM_ALI_BANDS];
/* MTF resampling kernal weights parameters */

static CPF *cpf_ptr = NULL;
/* pointer to the calibration parameter file returned by the gxx_read_cpf routine */

int read_resample_cpf()
{
   cpf_ptr = exx_read_cpf(get_CPF_filename());

   if( cpf_ptr == NULL )
      return(ERROR);
   return(SUCCESS);
}

/******************************************************************************
NAME:           read_resample_cpf

PURPOSE:
This function reads the CPF file and extracts the information used by the
resampler into a format that is more useable by the resampler.  All the 
necessary error checking is done on the parameters read (except checking to 
see if file names are legal/exist).

RETURN VALUE:
Type = int
Value    Description
-----    -----------
SUCCESS  The parameters were successfully read
ERROR    An error occurred reading the parameters

NOTES:
The cleanup_resample_cpf should be called when the cpf data is no longer needed
to allow the memory used by the cpf to be freed.

******************************************************************************/

int read_resample_cpf_mtf()
{
   ASSERT(cpf_ptr != NULL);

   if (cpf_ptr != NULL)
   {
   /* verify the size of the structures are compatible (the destination
      one is equal or greater size).  Only check one of each type
    *******************************************************************/
      ASSERT(sizeof(mtf_weights_along) >=
             sizeof(cpf_ptr->MTF_Compensation.PAN_weights_along));
      ASSERT(sizeof(mtf_weights_across[(int)pan]) >=
             sizeof(cpf_ptr->MTF_Compensation.PAN_weights_across));

      /* get the MTF weights */
      memcpy(&mtf_weights_along[(int)pan],cpf_ptr->MTF_Compensation.PAN_weights_along,
             sizeof(mtf_weights_along[(int)pan]));
      memcpy(&mtf_weights_across[(int)pan],cpf_ptr->MTF_Compensation.PAN_weights_across,
             sizeof(mtf_weights_across[(int)pan]));
      memcpy(&mtf_weights_along[(int)b1p],cpf_ptr->MTF_Compensation.MS1P_weights_along,
             sizeof(mtf_weights_along[(int)b1p]));
      memcpy(&mtf_weights_across[(int)b1p],cpf_ptr->MTF_Compensation.MS1P_weights_across,
             sizeof(mtf_weights_across[(int)b1p]));
      memcpy(&mtf_weights_along[(int)b1],cpf_ptr->MTF_Compensation.MS1_weights_along,
             sizeof(mtf_weights_along[(int)b1]));
      memcpy(&mtf_weights_across[(int)b1],cpf_ptr->MTF_Compensation.MS1_weights_across,
             sizeof(mtf_weights_across[(int)b1]));
      memcpy(&mtf_weights_along[(int)b2],cpf_ptr->MTF_Compensation.MS2_weights_along,
             sizeof(mtf_weights_along[(int)b2]));
      memcpy(&mtf_weights_across[(int)b2],cpf_ptr->MTF_Compensation.MS2_weights_across,
             sizeof(mtf_weights_across[(int)b2]));
      memcpy(&mtf_weights_along[(int)b3],cpf_ptr->MTF_Compensation.MS3_weights_along,
             sizeof(mtf_weights_along[(int)b3]));
      memcpy(&mtf_weights_across[(int)b3],cpf_ptr->MTF_Compensation.MS3_weights_across,
             sizeof(mtf_weights_across[(int)b3]));
      memcpy(&mtf_weights_along[(int)b4],cpf_ptr->MTF_Compensation.MS4_weights_along,
             sizeof(mtf_weights_along[(int)b4]));
      memcpy(&mtf_weights_across[(int)b4],cpf_ptr->MTF_Compensation.MS4_weights_across,
             sizeof(mtf_weights_across[(int)b4]));
      memcpy(&mtf_weights_along[(int)b4p],cpf_ptr->MTF_Compensation.MS4P_weights_along,
             sizeof(mtf_weights_along[(int)b4p]));
      memcpy(&mtf_weights_across[(int)b4p],cpf_ptr->MTF_Compensation.MS4P_weights_across,
             sizeof(mtf_weights_across[(int)b4p]));
      memcpy(&mtf_weights_along[(int)b5],cpf_ptr->MTF_Compensation.MS5_weights_along,
             sizeof(mtf_weights_along[(int)b5]));
      memcpy(&mtf_weights_across[(int)b5],cpf_ptr->MTF_Compensation.MS5_weights_across,
             sizeof(mtf_weights_across[(int)b5]));
      memcpy(&mtf_weights_along[(int)b7],cpf_ptr->MTF_Compensation.MS7_weights_along,
             sizeof(mtf_weights_along[(int)b7]));
      memcpy(&mtf_weights_across[(int)b7],cpf_ptr->MTF_Compensation.MS7_weights_across,
             sizeof(mtf_weights_across[(int)b7]));
    }
    else
    {
      xxx_LogStatus(PROGRAM_NAME,__FILE__,__LINE__,"Error Reading MTFs from CPF");
      return (ERROR);
    }

    return(SUCCESS);
}

int read_resample_cpf_offsets(
int sca)
{
  BandNumber band;
  int current_band;

  ASSERT(cpf_ptr != NULL);

  if( cpf_ptr != NULL )
  {
    /* CPF was read successfully, extract the resampler data */

    /* verify the size of the structures are compatible (the destination
       one is equal or greater size).  Only check one of each type
     *******************************************************************/
     ASSERT(sizeof(detector_offsets[(int)pan].along) >=
            sizeof(cpf_ptr->Focal_Plane_Parameters.SCA_Parameters[sca].Along_Detector_Offset_PAN));
     ASSERT(sizeof(detector_offsets[(int)pan].across) >=
            sizeof(cpf_ptr->Focal_Plane_Parameters.SCA_Parameters[sca].Across_Detector_Offset_PAN));
     ASSERT(sizeof(mtf_weights_along) >= 
            sizeof(cpf_ptr->MTF_Compensation.PAN_weights_along));
     ASSERT(sizeof(mtf_weights_across[(int)pan]) >= 
            sizeof(cpf_ptr->MTF_Compensation.PAN_weights_across));

     /* get the detector delays */
     memcpy(&detector_offsets[(int)pan].along, 
       cpf_ptr->Focal_Plane_Parameters.SCA_Parameters[sca].Along_Detector_Offset_PAN,
       sizeof(cpf_ptr->Focal_Plane_Parameters.SCA_Parameters[sca].Along_Detector_Offset_PAN));
     memcpy(&detector_offsets[(int)pan].across, 
       cpf_ptr->Focal_Plane_Parameters.SCA_Parameters[sca].Across_Detector_Offset_PAN,
       sizeof(cpf_ptr->Focal_Plane_Parameters.SCA_Parameters[sca].Across_Detector_Offset_PAN));
     for( band=b1p ; band<b7 ; band++ )
     { 
       current_band = (int)band - 1;
       memcpy(&detector_offsets[(int)band].along, 
         cpf_ptr->Focal_Plane_Parameters.SCA_Parameters[sca].Along_Detector_Offset_MS[current_band],
         sizeof(cpf_ptr->Focal_Plane_Parameters.SCA_Parameters[sca].Along_Detector_Offset_MS[current_band]));
       memcpy(&detector_offsets[(int)band].across, 
         cpf_ptr->Focal_Plane_Parameters.SCA_Parameters[sca].Across_Detector_Offset_MS[current_band],
         sizeof(cpf_ptr->Focal_Plane_Parameters.SCA_Parameters[sca].Across_Detector_Offset_MS[current_band]));
     }
  }
  else
  {
    xxx_LogStatus(PROGRAM_NAME,__FILE__,__LINE__,"Error Reading Offsets from CPF file");
    return (ERROR);
  }

  return (SUCCESS);
}

/******************************************************************************
NAME:           various accessor functions

PURPOSE:        
These functions provide access to the parameters read. 

RETURN VALUE:
Varies for the function.

NOTES:
The accessor function method of providing parameters was selected since
passing all the parameters up and down a calling sequence is a 
maintenance nightmare.

ALGORITHM REFERENCES:
none

******************************************************************************/

const MTF_PARAMETERS_TYPE *get_forward_MTF_weights_along
(
    BandNumber band /* INPUT : band number of weights to retrieve */
)
{
    /* verify band number is legal */
    ASSERT((band >= pan) && (band < NBANDS));

    /* verify the CPF has been read */
    ASSERT(cpf_ptr != NULL);
    
    return (&mtf_weights_along[band]);
}

const MTF_PARAMETERS_TYPE *get_reverse_MTF_weights_along
(
    BandNumber band /* INPUT : band number of weights to retrieve */
)
{
    /* verify band number is legal */
    ASSERT((band >= pan) && (band < NBANDS));

    /* verify the CPF has been read */
    ASSERT(cpf_ptr != NULL);
    
    return (&mtf_weights_along[band]);
}

const MTF_PARAMETERS_TYPE *get_forward_MTF_weights_across
(
    BandNumber band /* INPUT : band number of weights to retrieve */
)
{
    /* verify band number is legal */
    ASSERT((band >= pan) && (band < NBANDS));

    /* verify the CPF has been read */
    ASSERT(cpf_ptr != NULL);
    
    return (&mtf_weights_across[band]);
}

const MTF_PARAMETERS_TYPE *get_reverse_MTF_weights_across
(
    BandNumber band /* INPUT : band number of weights to retrieve */
)
{
    /* verify band number is legal */
    ASSERT((band >= pan) && (band < NBANDS));

    /* verify the CPF has been read */
    ASSERT(cpf_ptr != NULL);
    
    return (&mtf_weights_across[band]);
}

const DETECTOR_OFFSETS_TYPE *get_detector_offsets
(
    BandNumber band /* INPUT : band number of detector offsets to retrieve */
)
{
    /* verify band number is legal */
    ASSERT((band >= pan) && (band < NBANDS));

    /* verify the CPF has been read */
    ASSERT(cpf_ptr != NULL);
    
    return (&detector_offsets[band]);
}

CPF *get_cpf(void)
{
    /* verify the CPF has been read */
    ASSERT(cpf_ptr != NULL);
    
    return (cpf_ptr);
}

int get_path_row_parameters(
  double *semi_major,
  double *semi_minor,
  double *inclination,
  double *orbits_per_cycle,
  double *days_per_cycle,
  double *path1_row60,
  double *scenes_per_orbit,
  double *descending_node_row)
{
   if( cpf_ptr == NULL ) 
      return(ERROR);

   *semi_major = cpf_ptr->Earth_Constants.Semi_Major_Axis;
   *semi_minor = cpf_ptr->Earth_Constants.Semi_Minor_Axis;
   *inclination = cpf_ptr->Orbit_Parameters.Inclination_Angle;
   *orbits_per_cycle = (double)cpf_ptr->Orbit_Parameters.WRS_Cycle_Orbits;
   *days_per_cycle = (double)cpf_ptr->Orbit_Parameters.WRS_Cycle_Days;
   *path1_row60 = cpf_ptr->Orbit_Parameters.Long_Path1_Row60; 
   *scenes_per_orbit = (double)cpf_ptr->Orbit_Parameters.Scenes_Per_Orbit;
   *descending_node_row = (double)cpf_ptr->Orbit_Parameters.Descending_Node_Row;

   return(SUCCESS);
}

void cleanup_resample_cpf(void)
{
    /* verify the CPF has been read */
    ASSERT(cpf_ptr != NULL);
    
    /* free the CPF structure */
    exx_free_cpf(cpf_ptr);
    cpf_ptr = NULL;
}
