/******************************************************************************
NAME:           resample_image

PURPOSE:
Perform geometric resampling on an image using cubic convolution 
or nearest neighbor resampling. 

NOTES:
none

******************************************************************************/
 
#include <stdlib.h>
#include <math.h>
#include "xxx_Types.h"     /* SUCCESS/ERROR definitions */
#include "xxx_LogStatus.h" /* xxx_LogStatus prototype */
#include "axx_grid.h"      /* axx_GRID_BAND_TYPE definition */
#include "xxx_image.h"
#include "resample_image.h"/* prototypes for this module */
#include "aliresample.h"   /* PROGRAM_NAME definition */
#include "debug.h"         /* ASSERT macro */
#include "kernal.h"        /* Get_Resample_Weight_Table_Ptr prototype */

/* local versions of the minimum and maximum output range, adjusted for
   the currently selected output type. */
static double min_range;
static double max_range;

/* local copy of the output data type */
static OUTPUT_DATA_TYPE output_data_type;

/******************************************************************************
NAME:           setup_resamp_output

PURPOSE:        
setup_resamp_output configures the resampf1p module to generate the 
output image with the requested output data type and dynamic range.

RETURN VALUE:
Nothing is returned by this routine

NOTES:
- This routine uses the parameters read from the ODL parameter file to 
  create locally cached copies in the most efficient format for this module.
- This routine does not use the min/max values defined for the types in
  limits.h since those are machine specific.  The min/max values here are
  data dependent and cannot change for different machines.

ALGORITHM REFERENCES:
none

******************************************************************************/

void setup_resamp_output(void)
{
    output_data_type = get_output_data_type();

    min_range = get_output_range_minimum();
    max_range = get_output_range_maximum();

    if (output_data_type == BYTE_OUTPUT)
    {
        /* byte output so adjust the min and max accordingly */
        if (min_range < 0.0)
	    min_range = 0.0;
        else
            min_range = ceil(min_range);
        if (max_range > 255.0)
	    max_range = 255.0;
	else
	    max_range = floor(max_range);
    }
    else if (output_data_type == INT16_OUTPUT)
    {
        /* 2 byte integer output so adjust the min and max accordingly */
        if (min_range < -32768.0)
	    min_range = -32768.0;
        else
            min_range = ceil(min_range);
        if (max_range > 32767.0)
	    max_range = 32767.0;
	else
	    max_range = floor(max_range);
    }
    else if (output_data_type == INT32_OUTPUT)
    {
        /* byte output so adjust the min and max accordingly */
        if (min_range < -2147483648.0) 
	    min_range = -2147483648.0;
        else
            min_range = ceil(min_range);
        if (max_range > 2147483647.0)
	    max_range = 2147483647.0;
	else
	    max_range = floor(max_range);
    }
    else
    {
        /* must be float output, if it's not an error should have been 
	   obtained parsing the input, so just assert here during 
	   development.  no range manipulations are needed for the float
	   case */
        ASSERT (output_data_type == FLOAT_OUTPUT);
    }

}


/******************************************************************************
NAME:           convert_data

PURPOSE:                
convert_data converts the intermediate resampling results (currently stored as
doubles) and converts them to the requested output data type.

RETURN VALUE:
The type converted data is returned in the outbuf.

NOTES:
This routine was created to prevent the code having to be copied to the different
specialized resampling routines.  This incurs a small calling penalty, but it is
small when compared to the actual function.

ALGORITHM REFERENCES:
none

******************************************************************************/

static void convert_data
(
    const double *input_ptr,/* I: data to convert to the output type */
    void *outbuf,           /* O: output data buffer */
    int nl,                 /* I: Number of lines in ouput */
    int ns                  /* I: Number of samples in output */
)
{
    int l;  /* counter */

    /* convert the intermediate resampling results to the requested 
       output type */
    if (output_data_type == BYTE_OUTPUT)
    {
        unsigned char *char_ptr = outbuf;

       /* convert the data, using unsigned rounding */
       for (l = 0; l < (nl*ns) ; l++)
           *char_ptr++ = (unsigned char)(*input_ptr++ + 0.5);
    }
    else if (output_data_type == FLOAT_OUTPUT)
    {
        float *float_ptr = outbuf;

        for (l = 0; l < (nl*ns); l++)
            *float_ptr++ = *input_ptr++;
    }
    else if (output_data_type == INT16_OUTPUT)
    {
        int16 *int16_ptr = outbuf;

        for (l = 0; l < (nl*ns); l++)
        {
            /* convert the data, using signed rounding */
            if (*input_ptr >= 0)
	        *int16_ptr++ = (int16)(*input_ptr++ + 0.5);
	    else
	        *int16_ptr++ = (int16)(*input_ptr++ - 0.5);
        }
    }
    else
    {
        int32 *int32_ptr = outbuf;

        ASSERT(output_data_type == INT32_OUTPUT);

        for (l = 0; l < (nl*ns); l++)
        {
            /* convert the data, using signed rounding */
            if (*input_ptr >= 0.0)
                *int32_ptr++ = (int32)(*input_ptr++ + 0.5);
            else
                *int32_ptr++ = (int32)(*input_ptr++ - 0.5);
        }
    }
}

void *get_output_image_buffer
(
    int ns 
)
{
    if (output_data_type == BYTE_OUTPUT)
    {
      unsigned char *char_ptr;
      char_ptr = (unsigned char *)malloc( sizeof(unsigned char) * ns );
      if( char_ptr == NULL )
        return(NULL);
      else
        return(char_ptr);
    }
    else if (output_data_type == FLOAT_OUTPUT)
    {
      float *float_ptr;
      float_ptr = (float *)malloc( sizeof(float) * ns );
      if( float_ptr == NULL )
        return(NULL);
      else
        return(float_ptr);
    }
    else if (output_data_type == INT16_OUTPUT)
    {
      int16 *int16_ptr;
      int16_ptr = (int16 *)malloc( sizeof(int16) * ns );
      if( int16_ptr == NULL )
        return(NULL);
      else
        return(int16_ptr);
    }
    else
    {
      int32 *int32_ptr;
      int32_ptr = (int32 *)malloc( sizeof(int32) * ns );
      if( int32_ptr == NULL )
        return(NULL);
      else
        return(int32_ptr);
    }
}

/******************************************************************************
NAME:           resample_image_cc_delays

PURPOSE:
The specific resample routine to perform cubic convolution resampling
applying detector delays.

RETURN VALUE:
The resampled data is returned.

NOTES:
none

******************************************************************************/

int resample_image_cc_delays
(
  HDFIO *output_hdf_io_ptr,
  int sca,
  int nband,
  const axx_GRID_BAND_TYPE *grid_band_ptr,
  IMAGE dem,
  IMAGE input
)
{
  /* create local copies of the parameters */
  BandNumber band       =       input.band;
  void *outbuf;                 /* Output image buffer           */
  const short *data_ptr;        /* Input data pointer                   */
  COEFFICIENTS *coeff_ptr;      /* pointer to reverse coefficients in grid */
  int  ll, ls;                  /* Resampling weight table locations    */
  int  grid_cols;               /* Number of columns in grid            */
  const double *kernal_weight_table_ptr = Get_Resample_Weight_Table_Ptr();
                                /* pointer to kernal weight table */
  double w;               	/* Resampling weight table              */
  double round = 1.0 / 128.0;    /* Rounding error in resampling incr.   */
  double dl, ds;                /* Fractional line/sample offset        */
  double d_iline, d_isamp;      /* Input line/sample                    */
  int    i_iline, i_isamp;      /* integer input line/sample */
  int    oline, osamp;          /* output line/sample */
  double total;                 /* Resampled pixel output               */
  double fill = get_fill_pixel(); /* fill pixel */
  double *resample_ptr;
  double *ptr;
  int line_counter;
  int samp_counter;
  int count;
  int row,col;

  /* resampling kernal information */
  int  kernal_samples;
  int  kernal_lines;
  int  kernal_subpixel_steps;
  int  kernal_table_steps;
  double delta_line;
  double delta_sample;
  int kernal_end_line;
  int kernal_start_line;
  int kernal_end_sample;
  int kernal_start_sample;

  int cell_index;
  int ncols = grid_band_ptr->ngrid_samps - 1; /* columns in grid */
  int nrows = grid_band_ptr->ngrid_lines - 1; /* columns in grid */
  double sum_weights;
  double delta_line_oe_samp;
  double delta_samp_oe_samp;

  double elev;
  double d_oline,d_osamp;
  int status;
  int terrain_flag;

  /* get the kernal information */
  get_kernal_table_info_delays(&kernal_subpixel_steps, &kernal_table_steps, &kernal_end_line, 
                               &kernal_start_line, &kernal_end_sample, &kernal_start_sample, 
                               &kernal_samples, &kernal_lines);

  delta_line = ((double)(kernal_end_line - kernal_start_line)) / kernal_lines;
  delta_sample = ((double)(kernal_end_sample - kernal_start_sample)) / kernal_samples;
  if( delta_line == 0.0 || delta_sample == 0.0 )
  {
      xxx_LogStatus(PROGRAM_NAME,__FILE__,__LINE__, "Error with resampling weights");
      return(ERROR);
  }

  /* verify the kernal size is correct for the loop that
     actually does the resampling, which is hard coded to use
     a 4 line by 4 sample kernal for speed */
  ASSERT((kernal_lines == 4) && (kernal_samples == 4));

  resample_ptr = malloc(sizeof(double) * grid_band_ptr->samps);
  if (resample_ptr == NULL)
  {
      xxx_LogStatus(PROGRAM_NAME,__FILE__,__LINE__, "Error allocating intermediate buffer");
      return(ERROR);
  }

  terrain_flag = do_terrain_correction();

  outbuf = get_output_image_buffer(grid_band_ptr->samps);
  if (outbuf == NULL)
  {
    xxx_LogStatus(PROGRAM_NAME,__FILE__,__LINE__, "Error getting an output buffer");
    return(ERROR);
  }

  /* Loop through output lines/samples */
  for (oline = 0; oline < grid_band_ptr->lines; oline++)
  {
    ptr = resample_ptr;
    for (osamp = 0; osamp < grid_band_ptr->samps; osamp++)
    {
      /* Get elevation for image */
      if( terrain_flag )
      {
         if( xxx_get_elevation(dem, (double)oline, (double)osamp, grid_band_ptr->pixsize,
                               grid_band_ptr->pixsize, &elev) != SUCCESS)
         {
            xxx_LogStatus(PROGRAM_NAME,__FILE__,__LINE__, "Error getting elevation");
            return(ERROR);
         }
         status = axx_3d_ols2ils(grid_band_ptr, input.nl, input.ns,
                                 (double)oline, (double)osamp, &d_iline, &d_isamp,
                                 elev);
         row = (int)(d_iline / grid_band_ptr->grid_cell_size.line);
         col = (int)(d_isamp / grid_band_ptr->grid_cell_size.samp);
      }
      else
      {
	 elev = 0.0;
         status = axx_all_ols2ils(grid_band_ptr, &row, &col, input.nl, input.ns,
                                  (double)oline, (double)osamp, &d_iline, &d_isamp,
                                  grid_band_ptr->zeroplane);
      }

      /* No zplane in deltas */
      cell_index = row * ncols + col;

      if( status==SUCCESS )
      {
        /* if the input line/sample are within the input image bounds */
        if ((d_iline >= 2) && (d_iline <  (input.nl-3)) &&
            (d_isamp >= 2) && (d_isamp <  (input.ns-3)))
        {

          /* convert the input line/sample into whole and fractional line/sample */
          i_iline = (int)d_iline;
          i_isamp = (int)d_isamp;
          dl = d_iline - i_iline;
          ds = d_isamp - i_isamp;

          /* get the resampling kernal index for the fractional line/sample */

          /* Perform resampling. */
          total = 0.0;     /* initialize total to zero */
          sum_weights = 0.0;

          /* loop through the kernal lines for resampling */
          for (line_counter = -1, count=0; line_counter < 3; line_counter++)
          {
            /* loop through the kernal samples for resampling */
            for(samp_counter = -1; samp_counter < 3; samp_counter++, count++ )
            {

               /* No zplane in deltas */
               if( ((int)floor(i_isamp+samp_counter))%2==0 )
               {
                  delta_line_oe_samp = elev * grid_band_ptr->delta_line_even[2 * cell_index + 1] +
                                       grid_band_ptr->delta_line_even[2 * cell_index];
                  delta_samp_oe_samp = elev * grid_band_ptr->delta_samp_even[2 * cell_index + 1] +
                                       grid_band_ptr->delta_samp_even[2 * cell_index];
               }
               else
               {
                  delta_line_oe_samp = elev * grid_band_ptr->delta_line_odd[2 * cell_index + 1] +
                                       grid_band_ptr->delta_line_odd[2 * cell_index];
                  delta_samp_oe_samp = elev * grid_band_ptr->delta_samp_odd[2 * cell_index + 1] +
                                       grid_band_ptr->delta_samp_odd[2 * cell_index];
               }

               ls = (int)((round + (-(ds+delta_samp_oe_samp) + samp_counter - kernal_start_sample)) 
                          / delta_sample);
               ll = (int)((round + (-(dl+delta_line_oe_samp) + line_counter - kernal_start_line)) 
                          / delta_line);

               if( ll>=0 && ll<=kernal_lines && ls>=0 && ls<=kernal_samples )
               {
                  w = kernal_weight_table_ptr[ll * (kernal_samples+1) + ls];
                  sum_weights = sum_weights + w;
                  total += w * input.data[(i_iline + line_counter) * input.ns + i_isamp + samp_counter];
               }
            }
          }
          if( sum_weights != 0.0 ) total = total / sum_weights;

          /* put new resampled value in intermediate buffer, limiting it to
             the output range */
          if (total < min_range)
            *ptr++ = min_range;
          else if (total > max_range)
            *ptr++ = max_range;
          else
            *ptr++ = total;
          count++;
        }
        else
        {
          /* pixel is outside of input image so set the output pixel to fill */
          *ptr++ = fill;
          count++;
        }
      }
      else
      {
        /* pixel is outside of input image so set the output pixel to fill */
        *ptr++ = fill;
        count++;
      }

    } /* end of samp loop */


    /* Convert data from double to output data type */
    convert_data(resample_ptr, outbuf, 1, grid_band_ptr->samps);

    /* Write output line to 1G HDF */
    if (axx_write_1g_image(output_hdf_io_ptr, sca, nband, band, oline, 0, 1,
                           grid_band_ptr->samps, 0, outbuf) != SUCCESS)
    {
      xxx_LogStatus(PROGRAM_NAME,__FILE__,__LINE__, "Error writing output buffer");
      return(ERROR);
    }

  } /* end of line loop */

  free(resample_ptr);
  return(SUCCESS);
}


/******************************************************************************
NAME:           resample_image_cc

PURPOSE:
The specific resample routine to perform cubic convolution resampling.

RETURN VALUE:
The resampled data is returned.

NOTES:
none

******************************************************************************/

int resample_image_cc
(
  HDFIO *output_hdf_io_ptr,        
  int sca,
  int nband,
  const axx_GRID_BAND_TYPE *grid_band_ptr,
  IMAGE dem,		
  IMAGE input		
)
{
  /* create local copies of the parameters */
  BandNumber band 	=       input.band;
  void *outbuf;			/* Output image buffer           */
  const short *data_ptr;	/* Input data pointer			*/
  COEFFICIENTS *coeff_ptr;	/* pointer to reverse coefficients in grid */
  int  ll, ls;			/* Resampling weight table locations    */
  int  grid_cols;		/* Number of columns in grid            */
  const double *kernal_weight_table_ptr = Get_Resample_Weight_Table_Ptr(); 
                        /* pointer to kernal weight table */
  const double *w_ptr;		/* Resampling weight table              */
  double round = 1.0 / 128.0;	/* Rounding error in resampling incr.   */
  double dl, ds;		/* Fractional line/sample offset        */
  double d_iline, d_isamp;	/* Input line/sample                    */
  int    i_iline, i_isamp;	/* integer input line/sample */
  int    oline, osamp;		/* output line/sample */
  double total;			/* Resampled pixel output               */
  double fill = get_fill_pixel(); /* fill pixel */
  double *resample_ptr;
  double *ptr;
  int line_counter;
  int samp_counter;
  int count;
  int row,col;

  /* resampling kernal information */
  int  kernal_samples;
  int  kernal_lines;
  int  kernal_subpixel_steps;
  double doub_kernal_subpixel_steps;
  int  kernal_table_steps;
  int  kernal_size;   /* samples * lines */
 
  double elev;
  double d_oline,d_osamp;
  int status;
  int terrain_flag;

  /* get the kernal information */
  get_kernal_table_info(&kernal_subpixel_steps, &kernal_table_steps,
                        &kernal_samples, &kernal_lines);
  doub_kernal_subpixel_steps = (double)kernal_subpixel_steps;
  kernal_size = kernal_samples * kernal_lines;

  /* verify the kernal size is correct for the loop that 
     actually does the resampling, which is hard coded to use
     a 4 line by 4 sample kernal for speed */
  ASSERT((kernal_lines == 4) && (kernal_samples == 4));

  resample_ptr = malloc(sizeof(double) * grid_band_ptr->samps);
  if (resample_ptr == NULL)
  {
      xxx_LogStatus(PROGRAM_NAME,__FILE__,__LINE__, "Error allocating intermediate buffer");
      return(ERROR);
  }

  terrain_flag = do_terrain_correction();

  outbuf = get_output_image_buffer(grid_band_ptr->samps);
  if (outbuf == NULL)
  {
    xxx_LogStatus(PROGRAM_NAME,__FILE__,__LINE__, "Error getting an output buffer");
    return(ERROR);
  }

  /* Loop through output lines/samples */
  for (oline = 0; oline < grid_band_ptr->lines; oline++)
  {
    ptr = resample_ptr;
    for (osamp = 0; osamp < grid_band_ptr->samps; osamp++)
    {
      /* Get elevation for image */
      if( terrain_flag )
      {
         if( xxx_get_elevation(dem, (double)oline, (double)osamp, grid_band_ptr->pixsize, 
                               grid_band_ptr->pixsize, &elev) != SUCCESS)
         {
            xxx_LogStatus(PROGRAM_NAME,__FILE__,__LINE__, "Error getting elevation");
            return(ERROR);
         }
         status = axx_3d_ols2ils(grid_band_ptr, input.nl, input.ns,
                                 (double)oline, (double)osamp, &d_iline, &d_isamp,
                                 elev);
         row = (int)(d_iline / grid_band_ptr->grid_cell_size.line);
         col = (int)(d_isamp / grid_band_ptr->grid_cell_size.samp);
      }
      else
      {
         status = axx_all_ols2ils(grid_band_ptr, &row, &col, input.nl, input.ns,
                                  (double)oline, (double)osamp, &d_iline, &d_isamp,
                                  grid_band_ptr->zeroplane);
      }

      if( status==SUCCESS )
      {
        /* if the input line/sample are within the input image bounds */
        if ((d_iline >= 1) && (d_iline <  (input.nl-2)) &&
            (d_isamp >= 1) && (d_isamp <  (input.ns-2)))
        {
          /* convert the input line/sample into whole and fractional line/sample */
          i_iline = (int)d_iline;
          i_isamp = (int)d_isamp;
          dl = d_iline - i_iline;
          ds = d_isamp - i_isamp;
        
          /* get the resampling kernal index for the fractional line/sample */
          ll = (int)(doub_kernal_subpixel_steps * (round + dl));
          ls = (int)(doub_kernal_subpixel_steps * (round + ds));

          /* get pointer to weight table location to use */ 
          /*****************************************************************
          w_ptr = &kernal_weight_table_ptr[(ll * kernal_table_steps + ls) * kernal_size];
          *****************************************************************/

          w_ptr = kernal_weight_table_ptr + (ll * kernal_table_steps + ls) * kernal_size;

          /* Perform resampling. */
          total = 0.0;     /* initialize total to zero */

          /* loop through the kernal lines for resampling */
          for (line_counter = -1, count=0; line_counter < 3; line_counter++)
          {
            /* loop through the kernal samples for resampling */
            for(samp_counter = -1; samp_counter < 3; samp_counter++, count++ )
            {
               total += w_ptr[count] 
                     * input.data[(i_iline + line_counter) * input.ns + i_isamp + samp_counter];
            }
          }

          /* put new resampled value in intermediate buffer, limiting it to
             the output range */
          if (total < min_range)
            *ptr++ = min_range;
          else if (total > max_range)
            *ptr++ = max_range;
          else
            *ptr++ = total;
          count++;
        }
        else
        {
          /* pixel is outside of input image so set the output pixel to fill */
          *ptr++ = fill;
          count++;
        }
      }
      else
      {
        /* pixel is outside of input image so set the output pixel to fill */
        *ptr++ = fill;
        count++;
      }
     
    } /* end of samp loop */

    /* Convert data from double to output data type */
    convert_data(resample_ptr, outbuf, 1, grid_band_ptr->samps);

    /* Write output line to 1G HDF */
    if (axx_write_1g_image(output_hdf_io_ptr, sca, nband, band, oline, 0, 1, 
                           grid_band_ptr->samps, 0, outbuf) != SUCCESS)
    {
      xxx_LogStatus(PROGRAM_NAME,__FILE__,__LINE__, "Error writing output buffer");
      return(ERROR);
    }

  } /* end of line loop */

  free(resample_ptr);
  return(SUCCESS);
}


/******************************************************************************
NAME:           resample_image_nn

PURPOSE:
The specific resample routine to perform nearest neighbor resampling.

RETURN VALUE:
The resampled data is returned.

NOTES:
none

ALGORITHM REFERENCES:
none

******************************************************************************/

int resample_image_nn
(
  HDFIO *output_hdf_io_ptr,        
  int sca,
  int nband,
  const axx_GRID_BAND_TYPE *grid_band_ptr,
  IMAGE dem,
  IMAGE input
)
{
  /* create local copies of the parameters */
  BandNumber band 	         = input.band;
  void *outbuf;			/* Output image buffer 				*/
  double *resample_ptr;       
  COEFFICIENTS *coeff_ptr;	/* pointer to reverse coefficients in grid 	*/
  int  grid_cols;		/* Number of columns in grid			*/
  double d_iline, d_isamp;  	/* Input line/sample 				*/
  int    i_iline, i_isamp;	/* integer input line/sample */
  int    grid_row_index;	/* index to start of row in grid coefficients 	*/
  int    grid_cell_index;	/* index to cell in row for grid coefficients 	*/
  int    oline, osamp;		/* output line/sample 				*/
  double total;			/* Resampled pixel output 			*/
  double fill = get_fill_pixel(); /* fill pixel 				*/
  double *ptr;
  int row,col;
  int status;

  double elev;
  double d_oline,d_osamp;
  int terrain_flag;

  /* allocate space for the intermediate buffer */
  resample_ptr = malloc(sizeof(double) * grid_band_ptr->samps);
  if (resample_ptr == NULL)
  {
      xxx_LogStatus(PROGRAM_NAME,__FILE__,__LINE__, "Error allocating intermediate buffer");
      return(ERROR);
  }

  terrain_flag = do_terrain_correction();

  /* cache the number of grid columns */
  grid_cols = grid_band_ptr->ngrid_samps;

  /* Loop through output lines/samples */
  for (oline = 0; oline < grid_band_ptr->lines; oline++)
  {
    ptr = resample_ptr;
    /* loop through the samples in the output image */ 
    for (osamp = 0; osamp < grid_band_ptr->samps; osamp++)
    {
      /* Get elevation for image */
      if( terrain_flag )
      {
         if( xxx_get_elevation(dem, (double)oline, (double)osamp, grid_band_ptr->pixsize,
                            grid_band_ptr->pixsize, &elev) != SUCCESS)
         {
            xxx_LogStatus(PROGRAM_NAME,__FILE__,__LINE__, "Error getting elevation");
            return(ERROR);
         }
         status = axx_3d_ols2ils(grid_band_ptr, input.nl, input.ns,
                                 (double)oline, (double)osamp, &d_iline, &d_isamp,
                                 elev);
         row = (int)(d_iline / grid_band_ptr->grid_cell_size.line);
         col = (int)(d_isamp / grid_band_ptr->grid_cell_size.samp);
      }
      else
      {
         status = axx_all_ols2ils(grid_band_ptr, &row, &col, input.nl, input.ns,
                                  (double)oline, (double)osamp, &d_iline, &d_isamp,
                                  grid_band_ptr->zeroplane);
      }

      if( status==SUCCESS )
      {
        /* add half a pixel to round to nearest neighbor */
        d_isamp += 0.5;
        d_iline += 0.5;
          
        /* if the input line/sample are within the input image bounds */
        if ((d_iline >= 0) && (d_iline <  input.nl) &&
            (d_isamp >= 0) && (d_isamp <  input.ns))
        {
          /* convert the input line/sample into an integer */
          i_iline = (int)d_iline;
          i_isamp = (int)d_isamp;
          
          /* get the nearest input pixel */
          total = input.data[i_iline * input.ns + i_isamp];;
  
          /* put new resampled value in intermediate buffer, limiting it to
             the output range */
          if (total < min_range)
            *ptr++ = min_range;
          else if (total > max_range)
            *ptr++ = max_range;
          else
            *ptr++ = total;
        }
        else
        {
          /* pixel is outside of input image so set the output pixel to fill */
          *ptr++ = fill;
        }
      }
      else
      {
        /* pixel is outside of input image so set the output pixel to fill */
        *ptr++ = fill;
      }
    } /* end of sample loop */

    /* Convert data from double to output data type */
    convert_data(resample_ptr, outbuf, 1, grid_band_ptr->samps);

    /* Write output line to 1G HDF */
    if (axx_write_1g_image(output_hdf_io_ptr, sca, nband, band, oline, 0, 1,
                           grid_band_ptr->samps, 0, outbuf) != SUCCESS)
    {
      xxx_LogStatus(PROGRAM_NAME,__FILE__,__LINE__, "Error allocating intermediate buffer");
      return(ERROR);
    }
  } /* end of line loop */

  free(resample_ptr);
  return(SUCCESS);
}



/******************************************************************************
NAME:           resample_image_bi

PURPOSE:
The specific resample routine to perform bilinear interpolation resampling.

RETURN VALUE:
The resampled data is returned.

NOTES:
none

******************************************************************************/

int resample_image_bi
(
  HDFIO *output_hdf_io_ptr,        
  int sca,
  int nband,
  const axx_GRID_BAND_TYPE *grid_band_ptr,
  IMAGE dem,
  IMAGE input
)
{
  /* create local copies of the parameters */
  BandNumber band 	        = input.band;
  void *outbuf;			/* Output image buffer           	*/
  double *resample_ptr;         
  const short *data_ptr;	/* Input data pointer			*/
  COEFFICIENTS *coeff_ptr;	/* pointer to reverse coeffs in grid 	*/
  int grid_cols;		/* Number of columns in grid            */
  double dl, ds;		/* Fractional line/sample offset        */
  double d_iline, d_isamp;	/* Input line/sample                    */
  int    i_iline, i_isamp;	/* integer input line/sample 		*/
  int    grid_row_index;  /* index to start of row in grid coefficients */
  int    grid_cell_index; /* index to cell in row for grid coefficients */
  int    oline, osamp;		/* output line/sample */
  double total;			/* Resampled pixel output               */
  double fill = get_fill_pixel();/* fill pixel 				*/
  double *ptr;
  int row,col;
  int status;

  /* resampling kernal information */
  int  kernal_samples;
  int  kernal_lines;
  int  kernal_subpixel_steps;
  int  kernal_table_steps;

  double elev;
  double d_oline,d_osamp;
  int terrain_flag;

  /* get the kernal information */
  get_kernal_table_info(&kernal_subpixel_steps, &kernal_table_steps,
                        &kernal_samples, &kernal_lines);

  /* verify the kernal size is correct for bilinear resampling */
  ASSERT((kernal_lines == 2) && (kernal_samples == 2));

  /* allocate space for the intermediate buffer */
  resample_ptr = malloc(sizeof(double) * grid_band_ptr->samps);
  if (resample_ptr == NULL)
  {
      xxx_LogStatus(PROGRAM_NAME,__FILE__,__LINE__, "Error allocating output buffer");
      return(ERROR);
  }

  terrain_flag = do_terrain_correction();

  /* cache the number of grid columns */
  grid_cols = grid_band_ptr->ngrid_samps;

  /* Loop through output lines/samples */
  for (oline = 0; oline < grid_band_ptr->lines; oline++)
  {
    ptr = resample_ptr;
    /* loop through the samples in the output window */ 
    for (osamp = 0; osamp < grid_band_ptr->samps; osamp++)
    {
      /* Get elevation for image */
      if( terrain_flag )
      {
         if( xxx_get_elevation(dem, (double)oline, (double)osamp, grid_band_ptr->pixsize,
                            grid_band_ptr->pixsize, &elev) != SUCCESS)
         {
            xxx_LogStatus(PROGRAM_NAME,__FILE__,__LINE__, "Error getting elevation");
            return(ERROR);
         }
         status = axx_3d_ols2ils(grid_band_ptr, input.nl, input.ns,
                                 (double)oline, (double)osamp, &d_iline, &d_isamp,
                                 elev);
         row = (int)(d_iline / grid_band_ptr->grid_cell_size.line);
         col = (int)(d_isamp / grid_band_ptr->grid_cell_size.samp);
      }
      else
      {
         status = axx_all_ols2ils(grid_band_ptr, &row, &col, input.nl, input.ns,
                                  (double)oline, (double)osamp, &d_iline, &d_isamp,
                                  grid_band_ptr->zeroplane);
      }

      if( status==SUCCESS )
      {
        /* if the input line/sample are within the input image bounds */
        if ((d_iline >= 0) && (d_iline <  input.nl-1) &&
            (d_isamp >= 0) && (d_isamp <  input.ns-1))
        {
          /* convert the input line/sample into whole and fractional line/sample */
          i_iline = (int)d_iline;
          i_isamp = (int)d_isamp;
          dl = d_iline - i_iline;
          ds = d_isamp - i_isamp;
  
          /* get the address of the data at input line/sample */
          data_ptr = &input.data[i_iline * input.ns + i_isamp];
   
          /* Perform resampling. */
          total = (1.0 - ds) * (1.0 - dl) * data_ptr[0] +
                         ds  * (1.0 - dl) * data_ptr[1] +
                  (1.0 - ds) *        dl  * data_ptr[input.ns] +
                         ds  *        dl  * data_ptr[input.ns + 1];
            
  
          /* put new resampled value in intermediate buffer, limiting it to the output range */
          if (total < min_range)
            *ptr++ = min_range;
          else if (total > max_range)
            *ptr++ = max_range;
          else
            *ptr++ = total;
        
        }
        else
        {
          /* pixel is outside of input image so set the output pixel to fill */
          *ptr++ = fill;
        }
      }
      else
      {
        /* pixel is outside of input image so set the output pixel to fill */
        *ptr++ = fill;
      }
    } /* end of sample loop */

    /* Convert data from double to output data type */
    convert_data(resample_ptr, outbuf, 1, grid_band_ptr->samps ); 

    /* Write output line to 1G HDF */
    if (axx_write_1g_image(output_hdf_io_ptr, sca, nband, band, oline, 0, 1,
                           grid_band_ptr->samps, 0, outbuf) != SUCCESS)
    {
      xxx_LogStatus(PROGRAM_NAME,__FILE__,__LINE__, "Error allocating intermediate buffer");
      return(ERROR);
    }
  } /* end of line loop */

  free(resample_ptr);
  return(SUCCESS);
}
