#include "combine_sca.h"

static void convert_data
(
    const float *input_ptr,	/* I: data to convert to the output type */
    void *outbuf,		/* O: output data buffer 		*/
    DataType dtype,		/* Data type for conversion		*/
    int nl,			/* I: Number of lines in ouput 		*/
    int ns			/* I: Number of samples in output 	*/
)
{
    int l;  /* counter */

    /* convert the intermediate resampling results to the requested
       output type */
    if (dtype == BYTE)
    {
        unsigned char *char_ptr = outbuf;

       /* convert the data, using unsigned rounding */
       for (l = 0; l < (nl*ns) ; l++)
           *char_ptr++ = (unsigned char)(*input_ptr++ + 0.5);
    }
    else if (dtype == R4)
    {
        float *float_ptr = outbuf;

        for (l = 0; l < (nl*ns); l++)
            *float_ptr++ = *input_ptr++;
    }
    else if (dtype == I2)
    {
        int16 *int16_ptr = outbuf;

        for (l = 0; l < (nl*ns); l++)
        {
            /* convert the data, using signed rounding */
            if (*input_ptr >= 0)
                *int16_ptr++ = (int16)(*input_ptr++ + 0.5);
            else
                *int16_ptr++ = (int16)(*input_ptr++ - 0.5);
        }
    }
    else
    {
        int32 *int32_ptr = outbuf;

        ASSERT(dtype == I4);

        for (l = 0; l < (nl*ns); l++)
        {
            /* convert the data, using signed rounding */
            if (*input_ptr >= 0.0)
                *int32_ptr++ = (int32)(*input_ptr++ + 0.5);
            else
                *int32_ptr++ = (int32)(*input_ptr++ - 0.5);
        }
    }
}

void *get_output_image_buffer
(
    DataType dtype,	/* Data type of output image	*/
    int ns		/* Number of sample in output image */
)
{
    /* Check data type and allocate appropriate buffer */
    if (dtype == BYTE)
    {
      unsigned char *char_ptr;
      char_ptr = (unsigned char *)malloc( sizeof(unsigned char) * ns );
      if( char_ptr == NULL )
        return(NULL);
      else
        return(char_ptr);
    }
    else if (dtype == R4)
    {
      float *float_ptr;
      float_ptr = (float *)malloc( sizeof(float) * ns );
      if( float_ptr == NULL )
        return(NULL);
      else
        return(float_ptr);
    }
    else if (dtype == I2)
    {
      int16 *int16_ptr;
      int16_ptr = (int16 *)malloc( sizeof(int16) * ns );
      if( int16_ptr == NULL )
        return(NULL);
      else
        return(int16_ptr);
    }
    else
    {
      int32 *int32_ptr;
      int32_ptr = (int32 *)malloc( sizeof(int32) * ns );
      if( int32_ptr == NULL )
        return(NULL);
      else
        return(int32_ptr);
    }
}

int get_image_data(
   HDFIO *input_hdf_io_ptr,	/* input HDF I/O file id 		*/
   int band,			/* Input band number			*/
   int sca,			/* SCA number				*/
   int line,			/* Line number to read			*/
   DataType dtype,		/* Data type of each band of output	*/
   int nlines,			/* Number of lines to read		*/
   int nsamps,			/* Number of samples to read		*/
   int nbands,			/* Number of bands			*/
   BandNumber *bands,		/* Array of bands present		*/	
   float *img_buf)		/* return buffer with data		*/
{ 
  int samp;			/* Counter				*/
  void *sca_buf_gen=NULL;	/* generic image buffer                 */
  uint8 *sca_buf_i8=NULL;	/* buffer depending on the data type of */
  int16 *sca_buf_i16=NULL;	/*   the 1G image: BYTE (uint8),        */
  int32 *sca_buf_i32=NULL;	/*   I2 (int16), I4 (int32), or         */
  float32 *sca_buf_f32=NULL;	/*   R4 (float32)                       */

  if (dtype == BYTE)
  {
    sca_buf_i8  = malloc(nsamps * sizeof(uint8));
    sca_buf_gen = sca_buf_i8;
  }
  else if (dtype == I2)
  {
    sca_buf_i16 = malloc(nsamps * sizeof(int16));
    sca_buf_gen = sca_buf_i16;
  }
  else if (dtype == I4)
  {
    sca_buf_i32 = malloc(nsamps * sizeof(int32));
    sca_buf_gen = sca_buf_i32;
  }
  else if (dtype == R4)
  {
    sca_buf_f32 = malloc(nsamps * sizeof(float32));
    sca_buf_gen = sca_buf_f32;
  }
  if (sca_buf_gen == NULL )
  {
    xxx_LogStatus(PROGRAM_NAME, __FILE__, __LINE__, "Error allocating memory buffer");
    return(ERROR);
  }

  if (axx_read_1g_image(input_hdf_io_ptr, nbands, sca, bands[band], line, 0, nlines, nsamps, 0, 
                        sca_buf_gen) == ERROR)
  {
    xxx_LogStatus(PROGRAM_NAME, __FILE__, __LINE__, "Error reading 1g image");
    return(ERROR);
  }

  if( dtype==BYTE )
     for( samp=0 ; samp<nsamps ; samp++ ) 
        img_buf[samp] = (float)sca_buf_i8[samp];
  else if( dtype==I2 )
     for( samp=0 ; samp<nsamps ; samp++ ) 
        img_buf[samp] = (float)sca_buf_i16[samp];
  else if( dtype==I4 )
     for( samp=0 ; samp<nsamps ; samp++ ) 
        img_buf[samp] = (float)sca_buf_i32[samp];
  else
     for( samp=0 ; samp<nsamps ; samp++ ) 
        img_buf[samp] = sca_buf_f32[samp];

  free(sca_buf_gen);
  return(SUCCESS);
}

int combine(
  HDFIO *input_hdf_io_ptr,     /* output HDF I/O file id */
  HDFIO *output_hdf_io_ptr,      /* input HDF I/O file id */
  DataType dtype,              /* data type of each band of output */
  int image_lines[NSCA][NBANDS],
  int image_samps[NSCA][NBANDS],
  int nbands,
  BandNumber *bands,
  float fill)
{
  int sca;			/* SCA counter			*/
  int line;			/* Line counter			*/
  int samp;			/* Sample counter		*/
  int band;			/* Band counter			*/
  void *outbuf;                 /* Output image buffer 		*/
  float *resample_ptr;		/* Floating point "Combined"	*/
  int nsamps;			/* Number of samles in line	*/
  int count;
  
  float *data1;			/* Image data pointer		*/
  float *data2;			/* Image data pointer		*/
  float *data3;			/* Image data pointer		*/
  float *data4;			/* Image data pointer		*/
  float total;

  nsamps = image_samps[0][0];
  for(band=1 ; band<nbands ; band++)
    if( image_samps[0][band] > nsamps )
       nsamps = image_samps[0][band];

  /* Allocate imagery data pointers */
  data1 = (float *)malloc( sizeof(float) * nsamps );
  data2 = (float *)malloc( sizeof(float) * nsamps );
  data3 = (float *)malloc( sizeof(float) * nsamps );
  data4 = (float *)malloc( sizeof(float) * nsamps );
  if (data1 == NULL || data2 == NULL || data3 == NULL || data4 == NULL )
  {
    xxx_LogStatus(PROGRAM_NAME,__FILE__,__LINE__, "Error allocating data pointers");
    return(ERROR);
  }

  /* Allocate floating combined buffer */
  resample_ptr = (float *)malloc( sizeof(float) * nsamps );
  if (resample_ptr == NULL)
  {
    xxx_LogStatus(PROGRAM_NAME,__FILE__,__LINE__, "Error allocating resample_ptr");
    return(ERROR);
  }

  /* Get output buffer size according to data type */
  outbuf = get_output_image_buffer(dtype,nsamps);
  if (outbuf == NULL)
  {
    xxx_LogStatus(PROGRAM_NAME,__FILE__,__LINE__, "Error getting an output buffer");
    return(ERROR);
  }
 
  /* Loop on all bands */
  for( band=0 ; band<nbands ; band++ )
  {
    /* Loop on all lines in imagery */
    for( line=0 ; line<image_lines[0][band] ; line++ )
    {
      /* Get imagery for SCA 1 */
      if( get_image_data(input_hdf_io_ptr, band, 0, line, dtype, 1, image_samps[0][band],
                         nbands, bands, data1) != SUCCESS )
      {
        printf("SCA 1 Band %d Line %d\n",band,line);
        xxx_LogStatus(PROGRAM_NAME,__FILE__,__LINE__, "Error reading data");
        return(ERROR);
      }
      /* Get imagery for SCA 2 */
      if( get_image_data(input_hdf_io_ptr, band, 1, line, dtype, 1, image_samps[0][band],
                         nbands, bands, data2) != SUCCESS )
      {
        printf("SCA 2 Band %d Line %d\n",band,line);
        xxx_LogStatus(PROGRAM_NAME,__FILE__,__LINE__, "Error reading data");
        return(ERROR);
      }
      /* Get imagery for SCA 3 */
      if( get_image_data(input_hdf_io_ptr, band, 2, line, dtype, 1, image_samps[0][band],
                         nbands, bands, data3) != SUCCESS )
      {
        printf("SCA 3 Band %d Line %d\n",band,line);
        xxx_LogStatus(PROGRAM_NAME,__FILE__,__LINE__, "Error reading data");
        return(ERROR);
      }
      /* Get imagery for SCA 4 */
      if( get_image_data(input_hdf_io_ptr, band, 3, line, dtype, 1, image_samps[0][band],
                         nbands, bands, data4) != SUCCESS )
      {
        printf("SCA 4 Band %d Line %d\n",band,line);
        xxx_LogStatus(PROGRAM_NAME,__FILE__,__LINE__, "Error reading data");
        return(ERROR);
      }

      /* Combine imagery by averaging */
      for( samp=0 ; samp<image_samps[0][band] ; samp++ )
      {
         total = 0.0;
         count = 0;
         if( data1[samp] != 0.0 ) 
         {
            total = total + data1[samp];
            count++;
         }
         if( data2[samp] != 0.0 ) 
         {
            total = total + data2[samp];
            count++;
         }
         if( data3[samp] != 0.0 ) 
         {
            total = total + data3[samp];
            count++;
         }
         if( data4[samp] != 0.0 ) 
         {
            total = total + data4[samp];
            count++;
         }
         if( count != 0 )
            resample_ptr[samp] = total / count;
         else
            resample_ptr[samp] = total;
      }

      /* Convert data from float to output data type */
      convert_data(resample_ptr, outbuf, dtype, 1, image_samps[0][band]);

      /* Write output line to 1G HDF */
      if (axx_write_1g_image(output_hdf_io_ptr, 0, nbands, bands[band], line, 0, 1,
                             image_samps[0][band], 0, (VOIDP)outbuf) != SUCCESS)
      {
        xxx_LogStatus(PROGRAM_NAME,__FILE__,__LINE__, "Error writing output buffer");
        return(ERROR);
      }
    }
  }

  free(resample_ptr);
  free(data1);
  free(data2);
  free(data3);
  free(data4);
  return(SUCCESS);
}
