/******************************************************************************
UNIT NAME:      write_envi_hdr.c 

PURPOSE:        Generate an Envi header file for 1Gs and 1Gp data.
               
INVOCATION METHOD:  status = write_envi_hdr();

RETURN VALUE:
Type = int

Value           Description
--------------  ---------------------------------------------------------------
SUCCESS         Envi header file created
ERROR           Envi header file not created

ARGUMENT LIST:
Name            Type                 Use Description
--------------- -------------------- --- --------------------------------------
None 

EXTERNAL VARIABLES:
Source          Name           Type  Use Description
--------------- -------------- ----- --- --------------------------------------
rxx_DDR.h       rxx_DDR        struct I  Data Description Record

ABNORMAL TERMINATION CONDITIONS, ERROR MESSAGES:
*******************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include "xxx_const.h"			/* GPS constants */
#include "xxx_utility.h"		/* GPS utility lib prototypes */
#include "exx_geometric.h"		/* GPS geometric lib prototypes */
#include "axx_hdf_io.h"	 		/* SDS band tags and band # conv. */
#include "xxx_DDR.h"			/* DDR structure */
#include "xxx_LogStatus.h"		/* xxx_LogStatus prototype */
#include "write_envi_hdr.h"		/* write_envi_hdr prototype */

int write_envi_hdr(
char *image_file_name) 
{
   int  count; 			/* band loop counter */
   int  stat;			/* status return code */
   int  InProjCode;		/* project code from the DDR */
   int InZoneCode;		/* zone code from the DDR */
   int InSpheroidCode;		/* spheroid code from the DDR */
   int DegreesCode = 4;		/* code for output projection units in degrees */
   int  OutProjCode = 0;	/* output projection code */
   int  inunits;		/* input projection units for GCTP */
   double OutProjParms[XXX_PROJECTION_COEF_SIZE]; /* output projection parms. */
   BandNumber bandno;		/* band number */
   char band_name[STRLEN];	/* generated band file name */ 
   char envi_proj[STRLEN]; 	/* projection name in ENVI map_proj.txt file */
   char Message[STRLEN];	/* output messages */
   char header_name[STRLEN];	/* ENVI header file name */	
   char directory[STRLEN]; 	/* directory name */
   char root[STRLEN];      	/* root file name */
   char extension[STRLEN]; 	/* file extension */
   char NShemis[6];		/* N/S hemisphere */
   char DDR_name[STRLEN];	/* name of the 1G DDR file */
   char error_msg[STRLEN]; 	/* error message */
   char correction[] = {'s', 'p', 't'}; /* 1G correction types */
   xxx_DDRControl_TYPE *DDR_control_ptr; /* pointer to the source image DDR control structure */
   xxx_DDR_TYPE *band_ptr;	/* pointer to data for each band */
   struct DBL_LAT_LONG ul,	/* upper left lat/lon corner */
		     ur,	/* upper right lat/lon corner */
		     ll,	/* lower left lat/lon corner */
		     lr;	/* lower right lat/lon corner */
							     
   FILE *fpr;
   static char band_tag[NBANDS][4] = {SDS_PAN, SDS_MS1P, SDS_MS1, 
         SDS_MS2, SDS_MS3, SDS_MS4, SDS_MS4P, SDS_MS5P, SDS_MS5, SDS_MS7};

   /* create the 1G image  DDR name 
    ----------------------------------*/
   xxx_create_DDR_name(DDR_name, image_file_name);
	
   /* read the 1G image DDR 
    -----------------------*/
   DDR_control_ptr = xxx_ReadDDR(DDR_name, error_msg);
   if (DDR_control_ptr == NULL)
   {
       /* error reading required item, log an error */
       xxx_LogStatus("WRITE_ENVI_HDR",__FILE__,__LINE__, error_msg);
       return (ERROR);
   }
	
   /* Generate an envi header file for each band
    --------------------------------------------*/
   for (count = 0; count < DDR_control_ptr->Count; count++)
   {
      band_ptr = &DDR_control_ptr->p_DDR[count];
      bandno = axx_parse_user_band(band_ptr->BandNumber);
      xxx_create_band_name(image_file_name, "FILE", bandno, band_name);
      strcpy(header_name, band_name);
      strcat(header_name, ".hdr");
   
      /* Check project code and calculate lat/lon for corners
         For now only make header files for UTM, all bands will
         be the same projection.
        ------------------------------------------------------*/
      InProjCode = DDR_control_ptr->p_DDR[count].ProjectCode;
      if (InProjCode != UTM_CODE)
      {
         sprintf(Message, "ENVI header files not produced for projection code %d", InProjCode); 
         xxx_LogStatus("WRITE_ENVI_HDR", __FILE__, __LINE__, Message);
         return SUCCESS;
      }

      if ( (fpr = fopen(header_name,"w")) == NULL ) 
      {
         xxx_LogStatus( "WRITE_ENVI_HDR", __FILE__, __LINE__, "Error creating ENVI header file" );
         return ERROR;
      }
 	
      InZoneCode = band_ptr->ZoneCode;
      InSpheroidCode = band_ptr->SpheroidCode;
      stat = exx_get_units((char*)band_ptr->ProjectionUnits, &inunits);
      if (stat != SUCCESS)
      {
         sprintf(Message, "Error returned from exx_get_units"); 
         xxx_LogStatus("WRITE_ENVI_HDR",__FILE__,__LINE__, Message);
         return (ERROR);
      }
      stat = exx_projtran (&InProjCode, &inunits, &InZoneCode,
                           DDR_control_ptr->p_DDR[count].ProjectionCoef, &InSpheroidCode, 
                           &OutProjCode, &DegreesCode, &InZoneCode, OutProjParms, &InSpheroidCode,
                           &(DDR_control_ptr->p_DDR[count].UpperLeft[1]), 
                           &(DDR_control_ptr->p_DDR[count].UpperLeft[0]), &ul.lng, &ul.lat);
      if (stat != SUCCESS)
      {
         sprintf(Message, "Error calculating UL corner in exx_projtran"); 
         xxx_LogStatus("WRITE_ENVI_HDR",__FILE__,__LINE__, Message);
         return (ERROR);
      }
      stat = exx_projtran (&InProjCode, &inunits, &InZoneCode,
                           DDR_control_ptr->p_DDR[count].ProjectionCoef, &InSpheroidCode, 
                           &OutProjCode, &DegreesCode, &InZoneCode, OutProjParms, &InSpheroidCode,
                           &(DDR_control_ptr->p_DDR[count].UpperRight[1]), 
                           &(DDR_control_ptr->p_DDR[count].UpperRight[0]), &ur.lng, &ur.lat);
      if (stat != SUCCESS)
      {
         sprintf(Message, "Error calculating UR corner in exx_projtran"); 
         xxx_LogStatus("WRITE_ENVI_HDR",__FILE__,__LINE__, Message);
         return (ERROR);
      }
      stat = exx_projtran (&InProjCode, &inunits, &InZoneCode,
                           DDR_control_ptr->p_DDR[count].ProjectionCoef, &InSpheroidCode, 
                           &OutProjCode, &DegreesCode, &InZoneCode, OutProjParms, &InSpheroidCode,
                           &(DDR_control_ptr->p_DDR[count].LowerLeft[1]), 
                           &(DDR_control_ptr->p_DDR[count].LowerLeft[0]), &ll.lng, &ll.lat);
      if (stat != SUCCESS)
      {
         sprintf(Message, "Error calculating LL corner in exx_projtran"); 
         xxx_LogStatus("WRITE_ENVI_HDR",__FILE__,__LINE__, Message);
         return (ERROR);
      }
      stat = exx_projtran (&InProjCode, &inunits, &InZoneCode,
                           DDR_control_ptr->p_DDR[count].ProjectionCoef, &InSpheroidCode, 
                           &OutProjCode, &DegreesCode, &InZoneCode, OutProjParms, &InSpheroidCode,
                           &(DDR_control_ptr->p_DDR[count].UpperRight[1]),
                           &(DDR_control_ptr->p_DDR[count].LowerRight[0]), &lr.lng, &lr.lat);
      if (stat != SUCCESS)
      {
         sprintf(Message, "Error calculating LR corner in exx_projtran"); 
         xxx_LogStatus("WRITE_ENVI_HDR",__FILE__,__LINE__, Message);
         return (ERROR);
      }
   	
     if (band_ptr->ZoneCode > 0)
         strcpy(NShemis, "North");
     else
         strcpy(NShemis, "South");
     sprintf(envi_proj, "WGS84_UTM_%c.z%d", NShemis[0], abs(InZoneCode)); 
   
     /* Print ENVI header file contents 
      ---------------------------------*/
      
     fprintf(fpr,"ENVI\n");
     fprintf(fpr,"description = { EO-1 ALI imagery Level 1G%c Band %s }\n",
             correction[band_ptr->CorrectionType - 1], band_tag[(int)bandno]);
     fprintf(fpr,"samples = %d\n", band_ptr->NumberSamplesInScene);   
     fprintf(fpr,"lines = %d\n", band_ptr->NumberLinesInScene);
     fprintf(fpr,"bands = 1\n");
     fprintf(fpr,"header offset = 0\n");
     fprintf(fpr,"file type  = ENVI Standard\n");
     if( band_ptr->DataType == 21 )
        fprintf(fpr,"data type  = 1\n");
     else if( band_ptr->DataType == 22 )
        fprintf(fpr,"data type  = 2\n");
     else if( band_ptr->DataType == 24 )
        fprintf(fpr,"data type  = 3\n");
     else if( band_ptr->DataType == 5 )
        fprintf(fpr,"data type  = 4\n");
     else
     {
        sprintf(Message, "Error in DDR data type"); 
        xxx_LogStatus("WRITE_ENVI_HDR",__FILE__,__LINE__, Message);
        return (ERROR);
     }
     fprintf(fpr,"interleave = bsq\n");
     fprintf(fpr,"byte order = 1\n");
     fprintf(fpr,"map info = {%s, 1, 1, %f, %f, %f, %f, %d, %s, %s, %s}\n",
             "UTM", band_ptr->UpperLeft[1], band_ptr->UpperLeft[0],
             band_ptr->ProjectionDistX, band_ptr->ProjectionDistY,
             band_ptr->ZoneCode, NShemis, "WGS-84", "units=Meters");
     fprintf(fpr,"geo points = {%f,%f,%f,%f,%f,%f,%f,%f,%f,%f,%f,%f,%f,%f,%f,%f}\n",
             band_ptr->UpperLeft[0], band_ptr->UpperLeft[1], ul.lat, ul.lng,
             band_ptr->UpperRight[0], band_ptr->UpperRight[1], ur.lat, ur.lng,
             band_ptr->LowerLeft[0], band_ptr->LowerLeft[1], ll.lat, ll.lng,
             band_ptr->LowerRight[0], band_ptr->LowerRight[1], lr.lat, lr.lng);
     fprintf(fpr,"band names = {%s}\n", band_name);
     fclose(fpr);
   }

   /* Free the memory before returning */
   XXX_FREE_DDR(DDR_control_ptr);

   return SUCCESS;
}

