/******************************************************************************
NAME:           resample

PURPOSE:
The resample module contains the "main" function for the resampler.

NOTES:
- The scans parameter is converted to lines by multiplying by 16.


******************************************************************************/

#include <stdlib.h>        /* exit prototype */
#include <stdio.h>         /* sprintf prototype */
#include "xxx_Types.h"     /* SUCCESS/ERROR definitions */
#include "xxx_LogStatus.h" /* xxx_LogStatus prototype */
#include "xxx_const.h"     /* ALI constants */
#include "xxx_utility.h"   /* GPS_SYSTBL definition */
#include "proj.h"	   /* definition of SOM */
#include "aliresample.h"   /* PROGRAM_NAME definition */
#include "multiproc.h"   
#include "resample_image.h"/* setup_resample_output prototype */
#include "kernal.h"        /* Kernal_Setup prototype */
#include "get_resample_parameters.h"  /* read parameters routine */
#include "generate_ddr.h"  /* generate_ddr prototype */
#include "get_input_image_data.h"
#include "get_resample_cpf.h"
#include "write_envi_hdr.h"
#include "output_image.h"
#include "file_io.h"

/*
**
** MACROS
**
*/
#define USE_MESSAGE "\nUsage: aliresample <ODL_param_file> [-sN -pN -mN -f]\n" \

/******************************************************************************
NAME:           main

PURPOSE:
This is the main function for the resampler.  It controls the process of
resampling a level 1R (or 0R) image into a level 1G image using the 
grid.

NOTES:
none

******************************************************************************/

int main(int argc, char *argv[]) 
{
    int n;
    int	rs;                          /* Return status */
    int num_bands_processed;	     /* Number of bands processed */
    int band;                        /* Band number */
    int temp;                        /* temp variable */
    int status;		             /* status of return from function */
    int output_image_lines[NSCA][NBANDS]; /* lines in each band of output image */
    int output_image_samps[NSCA][NBANDS]; /* samples in each band of output image */
    DataType odtype_translate[NUMBER_OUTPUT_TYPES] = { BYTE, I2, I4, R4 };     
    DataType output_image_dtype[NBANDS]; /* data type of each band of output */
    axx_GRID_TYPE grid;              /* image grid */
    BandNumber current_band;	     /* current band counter	*/
    BandNumber current_band_list[1]; /* current band counter	*/
    BandNumber *band_list_ptr;       /* pointer to list of bands to resample */
    int current_nbands;
    OUTPUT_DATA_TYPE odtype;         /* selected output type */
    char msg[STRLEN];                /* message buffer */
    GPS_SYSTBL system_table;         /* system table */
    int num_grid_io_bands;           /* number of bands to operate on for grid i/o */
    axx_GRID_BAND_TYPE *ref;          /* Reference structure of current band      */
    RESAMPLE_TYPE resampling_type;   /* current resampling type */
    BandNumber grid_io_band_list[NBANDS]; /* list of bands for grid i/o operations */
    int sca;			     /* SCA counter */
    IMAGE input_image;		     /* Input image data */
    IMAGE dem_image;		     /* DEM data */
    char ext[12];		     /* Array for HDF file extension (defining SCAS) */
    char output_image_name[STRLEN];  /* Output image name */
    char input_image_name[STRLEN];   /* Intput image name */
    char grid_filename[STRLEN];      /* Grid file name */
    HDFIO *output_hdf_io_ptr;        /* output HDF I/O file id */
    int procs_to_use;
    RESAMPLE_THREAD_PARAMS params;      /* parameters to pass to resampling threads */
    int num_cpus = GET_NUM_PROCESSORS();   /* number of processors */
    FILE_IO_TYPE file_io;

    /* generate entry message */
    xxx_LogStatus(PROGRAM_NAME,__FILE__,__LINE__, "Initiated");

   /* if there are no arguments, generate a usage message */
   if (argc < 2)
   {
      xxx_LogStatus(PROGRAM_NAME,__FILE__,__LINE__, USE_MESSAGE);
      exit (EXIT_FAILURE);
   }

   /* read the resampler control parameters */
   if (read_resample_parameters(argv[1]) == ERROR)
   {
      sprintf(msg, "Error reading parameters from %s", argv[1]);
      xxx_LogStatus(PROGRAM_NAME,__FILE__,__LINE__, msg);
      exit(EXIT_FAILURE);
   }

   /* get the resampling type to use and the list of bands to 
      perform the resampling on */
   band_list_ptr = get_band_list();
   resampling_type = get_resampling_type();
   odtype = get_output_data_type(); 

   /* read the system table and get the number of processors to use and 
      the number of scans to buffer from it */
   if (xxx_get_system_table(&system_table) != SUCCESS)
   {
      sprintf(msg, "Error reading parameters from the system table");
      xxx_LogStatus(PROGRAM_NAME,__FILE__,__LINE__, msg);
      exit(EXIT_FAILURE);
   }
   procs_to_use = system_table.max_processors_to_use;
   if (procs_to_use >= num_cpus)
       procs_to_use = num_cpus;
  
   if( axx_get_grid_headers( get_grid_filename(), &grid ) != SUCCESS )
   {
      sprintf(msg, "Error reading the grid from %s",get_grid_filename());
      xxx_LogStatus(PROGRAM_NAME,__FILE__,__LINE__, msg);
      exit (EXIT_FAILURE);
   }

   /* verify the grid has data for all the bands requested by the user */
   for (band = 0; band < NBANDS; band++)
   {
      /* check if at the end of the band list */
      if (band_list_ptr[band] == berror)
         break;

      /* verify the requested band is in the grid */
      for (temp = 0; temp < grid.nbands; temp++)
      {
         if (band_list_ptr[band] == grid.bands[temp])
            break;
      }

      /* if the loop exits with temp equal to grid.nbands, the band is
         not present in the grid */
      if (temp == grid.nbands)
      {
         sprintf(msg, "Band %ld is not present in grid", axx_get_user_band(band_list_ptr[band]));
         xxx_LogStatus(PROGRAM_NAME,__FILE__,__LINE__, msg);
         exit(EXIT_FAILURE);
      }
   }
    
   /* flip the grid coords if the projection is SOM */
   axx_SOM_grid_calc(&grid);

   setup_resamp_output();

   for(sca = 0; sca<grid.nscas; sca++ )
   {
      /* assemble the data needed to open the 1g output image file */
      for (band = 0, num_bands_processed = 0; band < NBANDS; band++)
      {
         current_band = band_list_ptr[band];

         /* make sure a band is not attempted if it does not exist */
         if ((current_band >= NBANDS) || (current_band == berror))
            break;

         output_image_lines[sca][band] = grid.gridbands[sca][current_band].lines;
         output_image_samps[sca][band] = grid.gridbands[sca][current_band].samps;
         output_image_dtype[band] = odtype_translate[odtype];

         num_bands_processed++;
      }
   }

   for (band = 0; band <num_bands_processed; band++)
   {
       current_band = band_list_ptr[band];
       for( sca=1 ; sca<grid.nscas ; sca++ )
       {
          if( (grid.gridbands[0][current_band].samps != grid.gridbands[sca][current_band].samps) ||
              (grid.gridbands[0][current_band].lines != grid.gridbands[sca][current_band].lines) )
          {
             sprintf(msg,"Error SCAs have differing lines/samples %s");
             xxx_LogStatus(PROGRAM_NAME,__FILE__,__LINE__,msg);
             return (ERROR);
          }
       }
   }

   if( do_terrain_correction() )
   {
       band = 0;
       current_nbands = 1;
       current_band = band_list_ptr[band]; /* THIS NEEDS TO CHANGE!! */
       current_band_list[band] = current_band;
       /***********************************************************************************
         Need to find a more efficient way to do this.  Grid pointer are also read in
         in resampling loop.
       ***********************************************************************************/
       /* Get pointer information for the band being processed */
       if( axx_get_grid_pointers(get_grid_filename(), current_band_list, &current_nbands,
                                 &grid) != SUCCESS )
       {
          sprintf(msg, "Error reading the grid pointers from %s",get_grid_filename());
          xxx_LogStatus(PROGRAM_NAME,__FILE__,__LINE__, msg);
          exit(EXIT_FAILURE);
       }
       if (xxx_get_dem(get_DEM_filename(), &grid.corners, grid.gridbands[0][current_band].lines,
                       grid.gridbands[0][current_band].samps, grid.gridbands[0][current_band].pixsize,
                       &dem_image) == ERROR)
       {
           sprintf(msg,"Error reading DEM from %s",get_DEM_filename());
           xxx_LogStatus(PROGRAM_NAME,__FILE__,__LINE__,msg);
           return (ERROR);
       }
       /* Free grid */
       if (axx_free_grid (current_band_list, &current_nbands, &grid) != SUCCESS)
       {
          strcpy (msg, "Error freeing the grid.");
          xxx_LogError (PROGRAM_NAME, __LINE__, msg);
          exit (EXIT_FAILURE);
       }
   }

   strcpy(output_image_name,get_1G_filename());
   /* open the 1g output file */
   output_hdf_io_ptr = axx_open_1g_image(output_image_name, WRITE, &num_bands_processed, grid.nscas, 
                                         (BandNumber *)band_list_ptr, output_image_dtype, 
                                         output_image_lines, output_image_samps);
   if (output_hdf_io_ptr == NULL)
   {
      sprintf(msg, "Error opening output image: %s", output_image_name);
      xxx_LogStatus(PROGRAM_NAME,__FILE__,__LINE__, msg);
      exit(EXIT_FAILURE);
   }

   file_io.output_hdf_io_ptr = output_hdf_io_ptr;
   /* create the mutex's needed for file io */
   if (CREATE_MUTEX(&file_io.file_io_mutex) != 0)
   {
       strcpy(msg,"Error creating the file io mutex");
       xxx_LogStatus(PROGRAM_NAME,__FILE__,__LINE__,msg);
       axx_close_1g_image(output_hdf_io_ptr);
       exit (EXIT_FAILURE);
   }

   if( read_resample_cpf() == ERROR )
   {
      strcpy(msg,"Error opening CPF");
      xxx_LogStatus(PROGRAM_NAME,__FILE__,__LINE__, msg);
      axx_close_1g_image(output_hdf_io_ptr);
      exit(EXIT_FAILURE);
   }
   
   if( resampling_type == MTF_RESAMPLING )
   {
      if( read_resample_cpf_mtf() == ERROR )
      {
         axx_close_1g_image(output_hdf_io_ptr);
         strcpy(msg,"Error reading CPF MTFs");
         xxx_LogStatus(PROGRAM_NAME,__FILE__,__LINE__, msg);
         exit(EXIT_FAILURE);
      }
   }

   /* Start looping on SCAs since data is organized by SCA */
   current_nbands = 1;
   for(sca = 0; sca<grid.nscas; sca++ )
   {
      strcpy(input_image_name,get_1R_filename());
      sprintf(ext,".M%dR",(sca+1));
      strcat(input_image_name,ext);

      if (read_resample_cpf_offsets(sca) == ERROR)
      {
         axx_close_1g_image(output_hdf_io_ptr);
         strcpy(msg,"Error reading CPF offsets");
         xxx_LogStatus(PROGRAM_NAME,__FILE__,__LINE__, msg);
         exit(EXIT_FAILURE);
      }

      /* process all the requested bands */
      for (band = 0; band <num_bands_processed; band++)
      {
         /* use all the available processors to process the band */
         rs = SET_NUM_THREADS_TO_FORK(procs_to_use);
         if (rs != 0)
         {
            axx_close_1g_image(output_hdf_io_ptr);
            xxx_LogStatus(PROGRAM_NAME,__FILE__,__LINE__, "Error setting number of processes");
            return(EXIT_FAILURE);
         }

         current_band = band_list_ptr[band];
         current_band_list[0] = current_band;

         /* Get pointer information for the band being processed */
         if( axx_get_grid_pointers(get_grid_filename(), current_band_list, &current_nbands, 
                                &grid) != SUCCESS )
         {
            axx_close_1g_image(output_hdf_io_ptr);
            sprintf(msg, "Error reading the grid pointers from %s",get_grid_filename());
            xxx_LogStatus(PROGRAM_NAME,__FILE__,__LINE__, msg);
            exit(EXIT_FAILURE);
         }

         status = sprintf(msg,"Resampling band %d SCA %d",
                          axx_get_user_band(current_band),(sca+1));
	 if (status < 1)
	 {
            axx_close_1g_image(output_hdf_io_ptr);
	    xxx_LogStatus(PROGRAM_NAME, __FILE__, __LINE__, "Error in sprintf");
	    exit(EXIT_FAILURE);
	 }
         xxx_LogStatus(PROGRAM_NAME,__FILE__,__LINE__, msg);
   
         /* Set up resampling kernal information */
         if (resampling_type != NN_RESAMPLING)
         { 
            if (Kernal_Setup(current_band, resampling_type, get_cubic_convolution_alpha()) == ERROR)
            {
               axx_close_1g_image(output_hdf_io_ptr);
               strcpy(msg,"Error setting up the resampling kernal");
               xxx_LogStatus(PROGRAM_NAME,__FILE__,__LINE__, msg);
               free(input_image.data);
               exit(EXIT_FAILURE);
            }
         }

         /* Get imagery from HDF file and put in image structure */
         if( get_input_image_data(input_image_name, current_band, &input_image) != SUCCESS )
         {
            axx_close_1g_image(output_hdf_io_ptr);
            free(input_image.data);
            strcpy(msg,"Error setting up the resampling kernal");
            xxx_LogStatus(PROGRAM_NAME,__FILE__,__LINE__, msg);
            exit(EXIT_FAILURE);
         }
   
         /* Get reference band pointer */
         ref = &grid.gridbands[sca][current_band];

         /* set the parameters to pass to the resampling threads */
         params.file_io = &file_io;
         params.sca = sca;
         params.num_bands_processed = num_bands_processed;
         params.grid_band_ptr = ref;
         params.dem_image = &dem_image;
         params.input_image = &input_image;

         printf("Processing Band %d SCA %d\n",band,sca);
         /* Perform resampling according to resampling type */
         switch (resampling_type)
         {
            case CC_RESAMPLING:
               if( do_delay_correction() )
                   rs = FORK_THREADS(resample_image_cc_delays, (void *)&params);
               else
                   rs = FORK_THREADS(resample_image_cc, (void *)&params);
                  break;
   
            case NN_RESAMPLING:
               rs = FORK_THREADS(resample_image_nn, (void *)&params);
                  break;
   
            case BI_RESAMPLING:
               rs = FORK_THREADS(resample_image_bi, (void *)&params);
                  break;
         }
         if( rs == ERROR )
         {
            free(input_image.data);
            axx_free_grid (grid_io_band_list, &num_grid_io_bands, &grid);
            axx_close_1g_image(output_hdf_io_ptr);
            xxx_LogStatus(PROGRAM_NAME, __FILE__, __LINE__, "Error in resampling");
            exit(EXIT_FAILURE);
         }
   
         /* free the image data */
         free(input_image.data);

         /* Free grid */
         if (axx_free_grid (current_band_list, &current_nbands, &grid) != SUCCESS)
         {
            strcpy (msg, "Error freeing the grid.");
            xxx_LogError (PROGRAM_NAME, __LINE__, msg);
            exit (EXIT_FAILURE);
         }

         rs = KILL_FORKED_THREADS();
         if (rs == ERROR)
         {
            axx_close_1g_image(output_hdf_io_ptr); 
            xxx_LogStatus(PROGRAM_NAME,__FILE__,__LINE__, "Error killing processes");
            exit(EXIT_FAILURE);
         }

      } /* for (band = 0; band < NBANDS; band++) */

   } /* SCA number */
   
   /* close the output 1g image */
   if (axx_close_1g_image(output_hdf_io_ptr) == ERROR)
   {
      strcpy(msg,"Error closing output image");
      xxx_LogStatus(PROGRAM_NAME,__FILE__,__LINE__, msg);
      exit(EXIT_FAILURE);
   }

   /* Write all ancillary info (proj info, corners, etc.) */
   sca = 0; /* set SCA to first SCA (DDR information will be the same for all) */
   if (generate_ddr(&grid, (int)num_bands_processed, sca, band_list_ptr) == ERROR)
   {
      strcpy(msg,"Error writing output image DDR file");
      xxx_LogStatus(PROGRAM_NAME,__FILE__,__LINE__, msg);
      exit (EXIT_FAILURE);
   }
   
   /* generate an ENVI header file for each band (UTM only)
      continue processing on an error creating header files.
      -------------------------------------------------------*/
   if (write_envi_hdr(grid.nscas) == ERROR)
   {
      strcpy(msg,"Error generating ENVI header files");
      xxx_LogStatus(PROGRAM_NAME,__FILE__,__LINE__, msg);
   }

   /* clean up */
   Cleanup_Kernal();

   /* generate exit message */
   xxx_LogStatus(PROGRAM_NAME,__FILE__,__LINE__, "Successful Completion");

   return (EXIT_SUCCESS);
}
