#include <stdlib.h>        /* exit prototype */
#include <stdio.h>         /* sprintf prototype */
#include "xxx_Types.h"     /* SUCCESS/ERROR definitions */
#include "xxx_LogStatus.h" /* xxx_LogStatus prototype */
#include "axx_grid.h"
#include "xxx_image.h"
#include "xxx_ali_level1r.h"

/******************************************************************************
NAME:           get_input_image_data

PURPOSE:
get_input_image_data reads the appropriate band of imagery into an
image structure.

NOTES:
none

******************************************************************************/

int get_input_image_data(
char *l1r_name,
BandNumber band, 	/* Band of imagery to retrieve */
IMAGE *output)		/* Structure to hold imagery */
{
   short *ptr;		/* Pointer to imagery			*/
   long offset;		/* Offset to imagery in L1R structure	*/
   long bytes;		/* Total number of imagery samples	*/
   long n;		/* Counter				*/
   int band_number;	/* Integer band number			*/
   int found = 0;	/* Flag indicating imagery was found	*/
   char emsg[STRLEN];   /* Error message buffer                 */
   char progname[STRLEN];/* Program name                         */

   int32 number_of_dims;
   int32 dim_sizes[3];		/* Dimensions of SDS			*/
   char8 sds_name[9];		/* SDS name				*/
   int32 HDF_type;
   int32 number_of_attributes;
   int32 sds_index;             /* Index of current SDS                 */
   int32 sd_id;                 /* SD file ID                           */
   int32 sds_id;                /* SDS file ID                          */
   int32 start[3];              /* Starting dims of SDS "slab"          */
   int32 edges[3];              /* Edge dims of SDS "slab"              */
   int32 stride[3];             /* Increment dims of SDS "slab"         */
   int32 size;                  /* Size needed for array allocation     */

   L1R_IMAGE l1r;		/* Level 1R image data structure 	*/

   strcpy(progname,"GET_IMAGE_DATA");

   /* Get SD file ID
   ----------------*/
   if ((sd_id = SDstart(l1r_name, DFACC_RDONLY)) == FAIL)
   {
      strcpy(emsg,"Could not open SDS file\n");
      xxx_LogStatus(progname,__FILE__,__LINE__, emsg);
      return(ERROR);
   }

   /* Search for imagery SDS */
   sds_index = 0;
   while( !found )
   {
      sds_id = SDselect(sd_id, sds_index);

      /* Get SDS info
       ---------------*/
      if( SDgetinfo(sds_id, sds_name, &number_of_dims, dim_sizes, &HDF_type, 
                    &number_of_attributes ) == FAIL )
      {
         strcpy(emsg,"Could not get SDS info\n");
         xxx_LogStatus(progname,__FILE__,__LINE__, emsg);
         return(ERROR);
      }

      /* Close SDS */
      if( SDendaccess(sds_id) == FAIL )
      {
         strcpy(emsg,"Ending SDS access error.\n");
         xxx_LogStatus(progname,__FILE__,__LINE__, emsg);
         return(ERROR);
      }

      /* See if this is an image SDS
       -----------------------------*/
      if( (strncmp(sds_name,"Level 1R",8) == 0) ||
          (strncmp(sds_name,"LEVEL1R",7) == 0) )
      {
         if( band == pan )
         {
            if( dim_sizes[0] == 1 )
            {
               found = 1;
               /* Load L1R image data structure */
               if ( xxx_read_ali_l1r_image( sd_id, sds_index, &l1r ) != SUCCESS )
               {
                  strcpy(emsg,"Could not read 1R image data\n");
                  xxx_LogStatus(progname,__FILE__,__LINE__, emsg);
                  return(ERROR);
               }
               /* Store image size in structure variables */
               output->nl = l1r.dim_sizes[1];
               output->ns = l1r.dim_sizes[2];
               /* Copy imagery buffer to image structure */
               size = l1r.dim_sizes[1]*l1r.dim_sizes[2];
               output->data = (short *)malloc( sizeof(short) * size );
               if( (output->data) == NULL )
               {
                  strcpy(emsg,"Error allocating data buffer.\n");
                  xxx_LogStatus(progname,__FILE__,__LINE__, emsg);
                  return(ERROR);
               }
               memcpy( (void *)output->data, (void *)(l1r.data), (size*sizeof(int16)) );
               free( l1r.data );
            }
         }
         else 
         {
            if( dim_sizes[0] == 9 )
            {
               found = 1;
               /* Determine band number */
               band_number = axx_get_user_band(band);

               /* Load L1R image data structure */
               if ( xxx_read_ali_l1r_image( sd_id, sds_index, &l1r ) != SUCCESS )
               {
                  strcpy(emsg,"Could not read 1R image data\n");
                  xxx_LogStatus(progname,__FILE__,__LINE__, emsg);
                  return(ERROR);
               }
               /* Store image size in structure variables */
               output->nl = l1r.dim_sizes[1];
               output->ns = l1r.dim_sizes[2];
               /* Subtract 2 from offset since pan = 1 and ms1p = 2 */
               start[0]  = band_number - 2; /* Band start is 0-relative */
               start[1]  = 0;
               start[2]  = 0;
               /* Copy imagery buffer to image structure */
               size = l1r.dim_sizes[1]*l1r.dim_sizes[2];
               output->data = (short *)malloc( sizeof(short) * size );
               if( (output->data) == NULL )
               {
                  strcpy(emsg,"Error allocating data buffer.\n");
                  xxx_LogStatus(progname,__FILE__,__LINE__, emsg);
                  return(ERROR);
               }
               memcpy( (void *)output->data, (void *)(l1r.data+start[0]*size), (size*sizeof(int16)) );
               free( l1r.data );
            }
         }
      }
 
      /* Index to next SDS */
      sds_index++;
   }

   /* Close the SD file
   --------------------*/
   if ( SDend( sd_id ) == FAIL )
   {
      strcpy(emsg,"Closing SD file ID failed!\n ");
      xxx_LogStatus(progname,__FILE__,__LINE__, emsg);
      return(ERROR);
   }
   output->band = band;

   return(SUCCESS);
}
