/******************************************************************************
NAME:           get_resample_parameters

PURPOSE:        
This module reads the ODL file input parameters required by the resampler.

ROUTINES:
    read_resample_parameters
    get_1R_filename
    get_band_list
    get_1G_filename
    get_grid_filename
    do_terrain_correction
    get_DEM_filename
    do_delay_correction
    get_output_data_type
    get_output_data_size
    get_resampling_type
    get_output_range_minimum
    get_output_range_maximum
    get_cubic_convolution_alpha
    get_fill_pixel
    get_tmodel_filename
    get_0R_filename
    get_CPF_filename
    get_script_name
    get_L0R_id
    
******************************************************************************/
#include "xxx_LogStatus.h"            /* xxx_LogStatus prototype */
#include "xxx_utility.h"              /* gxx_read_odl_file prototype */
#include "xxx_const.h"                /* STRLEN and BandNumber definitions */
#include "axx_hdf_io.h"               /* gxx_parse_user_band prototype */
#include "get_resample_parameters.h"  /* prototypes for this module */
#include "debug.h"                    /* ASSERT macro definition */


/*
**
** MACRO definitions
**
*/

#define GROUP_NAME script_name 
/* the ODL group name is the script name.  Note that this must appear 
   in the list of ODL parameters before any non-global ODL parameters. */

#define SCRIPT_NAME_LEN 33
/* number of characters in the script name, including the null terminator */

#define L0R_ID_LEN (XXX_PRODUCT_ID_SIZE + 1)
/* number of characters in the Level 0 ID name, including the null terminator */

/*
**
** VARIABLE DECLARATIONS
**
*/

/* NOTE: if there is a default value for the parameters read from the ODL file
         they need to be set when the variable is declared (unless the auto-zero
         initialization sets the correct default value). */

static BandNumber band_list[NBANDS] = 
      {NBANDS, NBANDS, NBANDS, NBANDS, NBANDS, NBANDS, NBANDS, NBANDS, NBANDS, NBANDS};  
/* list of bands to be processed */

static int terrain_flag;
/* flag to indicate terrain correction should be done. non-zero = do terrain 
   correction */

static int delay_flag;
/* flag to indicate detector delays should be accounted for. 
   non-zero = account for detector delays. */

static float min_max_output[2]= {0.0, 255.0};
/* minimum and maximum output values from the resampler */

static float alpha = -0.5;
/* cubic convolution alpha parameter */

static float fill_pixel = 0.0;
/* value to use for fill pixels */

static RESAMPLE_TYPE resampling_type;
/* resampling type to perform */

static OUTPUT_DATA_TYPE output_type;
/* output data type */

static float gain_threshold = 0.0;
/* Fraction of scans in low gain state needed before scaling the scene
   using low gain scale factors. */

/* various file name storage */
static char filename_1R_image[STRLEN];
static char filename_1G_image[STRLEN];
static char grid_filename[STRLEN];
static char dem_filename[STRLEN];
static char tmodel_filename[STRLEN];
static char cal_parm_file[STRLEN];
static char L0R_filename[STRLEN];
static char script_name[SCRIPT_NAME_LEN];
static char L0R_id[L0R_ID_LEN];


/* the following variables should be able to reside within the 
   read_resample_parameters routine as non-static automatic variables,
   but the SGI compiler doesn't seem to be able to deal with it when
   they are used in the READ_ODL_LIST. */
static char odtype[5] = "BYTE";
/* character string storage for the output data type read from the ODL
   file.  defaulted to byte. */

static char resample_opt[4];
/* resampling option read from the ODL file. */


/******************************************************************************
NAME:           read_resample_parameters

PURPOSE:        
This function reads all the parameters from ODL file that the 
resampler requires.  All the necessary error checking is done on the 
parameters read (except checking to see if file names are legal/exist).

RETURN VALUE:
Type = int
Value    Description
-----    -----------
SUCCESS  The parameters were successfully read
ERROR    An error occurred reading the parameters

NOTES:
The group name for the ODL file is the script name.  The script name is one
of the global parameters (i.e. not in a group).  Since only one list is
built to read the ODL parameters, the script name must appear in the list
before any items that use the GROUP_NAME macro.

******************************************************************************/

int read_resample_parameters
(
    const char *ODL_filename /* INPUT: ODL filename that contains resample 
                                       parameters */
)
{
    char msg[STRLEN];         /* storage for ODL error messages */
    BandNumber temp_band_num; /* temporary band number */
    int count;                /* loop counter */
    int band_already_in_list[NBANDS] = {0}; /* flag to indicate a band is 
                                 already in the band list */
    int band_index;           /* index to next band number to insert */

    OBJDESC *in_parms;
    char param_filename[STRLEN];  /* I: Parameter file name                   */
    char scriptname[STRLEN];      /* O: Script name                           */
    static int bands[NBANDS] = {0};
       
    xxx_READ_ODL_LIST_TYPE ODL_param_list[] =
    {
       { NULL, XXX_PARM_CPF, cal_parm_file, sizeof(cal_parm_file), 
	 XXX_String, 0 /* min count */, xxx_ODL_PARAM_OPTIONAL},
       { NULL, XXX_PARM_L0R_HDFNAME, L0R_filename, sizeof(L0R_filename),
	 XXX_String, 0 /* min count */, xxx_ODL_PARAM_REQUIRED},
       { NULL, XXX_PARM_SCRIPT_NAME, script_name, sizeof(script_name),
	 XXX_String, 0 /* min count */, xxx_ODL_PARAM_REQUIRED},
       { NULL, XXX_PARM_L0R_ID, L0R_id, sizeof(L0R_id),
	 XXX_String, 0 /* min count */, xxx_ODL_PARAM_REQUIRED},
       { GROUP_NAME, "L1R_IMAGE", filename_1R_image,
         sizeof(filename_1R_image), XXX_String, 0 /* min count */, 
         xxx_ODL_PARAM_REQUIRED},
       { GROUP_NAME, "BAND_LIST", bands, sizeof(band_list), XXX_Int,
         1 /*min count */, xxx_ODL_PARAM_REQUIRED},
       { GROUP_NAME, "L1G_IMAGE", filename_1G_image,
         sizeof(filename_1G_image), XXX_String, 0 /* min count */,
         xxx_ODL_PARAM_REQUIRED},
       { GROUP_NAME, "INPUT_GRID", grid_filename, sizeof(grid_filename),
         XXX_String, 0 /* min count */, xxx_ODL_PARAM_REQUIRED},
       { GROUP_NAME, "TERRAIN_FLAG", &terrain_flag, sizeof(terrain_flag),
         XXX_Int, 0 /* min count */, xxx_ODL_PARAM_OPTIONAL},
       { GROUP_NAME, "DEM_NAME", dem_filename, sizeof(dem_filename),
         XXX_String, 0 /* min count */, xxx_ODL_PARAM_OPTIONAL},
       { GROUP_NAME, "DELAY_FLAG", &delay_flag, sizeof(delay_flag),
         XXX_Int, 0 /* min count */, xxx_ODL_PARAM_OPTIONAL},
       { GROUP_NAME, "ODTYPE", odtype, sizeof(odtype), 
         XXX_String, 0 /* min count */, xxx_ODL_PARAM_OPTIONAL},
       { GROUP_NAME, "RESAMPLE", resample_opt, sizeof(resample_opt),
         XXX_String,  0 /* min count */, xxx_ODL_PARAM_REQUIRED},
       { GROUP_NAME, "MINMAX_OUTPUT_DN", min_max_output, sizeof(min_max_output),
         XXX_Float, 2 /* min count */, xxx_ODL_PARAM_OPTIONAL},
       { GROUP_NAME, "PCCALPHA", &alpha, sizeof(alpha),
         XXX_Float, 0 /* min count */, xxx_ODL_PARAM_OPTIONAL},
       { GROUP_NAME, "BACKGRND", &fill_pixel, sizeof(fill_pixel),
         XXX_Float, 0 /* min count */, xxx_ODL_PARAM_OPTIONAL},
       { GROUP_NAME, "PRECISION_MODEL", tmodel_filename, 
         sizeof(tmodel_filename), XXX_String, 0 /* min count */, 
         xxx_ODL_PARAM_OPTIONAL},
       { GROUP_NAME, "GAIN_THRESHOLD", &gain_threshold, sizeof(gain_threshold),
         XXX_Float, 0 /* min count */, xxx_ODL_PARAM_OPTIONAL},
    };
    /* table of items to be read from the ODL file */

    /* calculate number of items in the ODL_param_list */
    count = sizeof(ODL_param_list)/sizeof(xxx_READ_ODL_LIST_TYPE);

    /***************************************************************************
    strcpy(param_filename,ODL_filename);
    in_parms = xxx_OpenODL (param_filename, (xxx_EDCOdl_TYPE)xxx_NoEDCMeta, msg);
    if (in_parms == NULL)
    {
      xxx_LogStatus ("RESAMPLE_GET_PARMS", __FILE__, __LINE__, msg);
      return (ERROR);
    }
    
    if (xxx_GetODLField (scriptname, STRLEN * sizeof (char), XXX_String,
                         in_parms, NULL, "SCRIPT_NAME", msg, &count) != SUCCESS)
    {
      xxx_LogStatus ("RESAMPLE_GET_PARMS", __FILE__, __LINE__, msg);
      strcpy (msg, "Error in the scriptname in the parameter file.");
      xxx_LogStatus ("RESAMPLE_GET_PARMS", __FILE__, __LINE__, msg);
      xxx_CloseODL (in_parms, msg);
      return (ERROR);
    }

    if (xxx_GetODLField (band_list, MAX_BANDS * sizeof (int), XXX_Int, in_parms,
                         scriptname, "BAND_LIST", msg, &count) != SUCCESS)
    {
      strcpy (msg, "Error in the band numbers in the parameter file.");
      xxx_LogStatus ("RESAMPLE_GET_PARMS", __FILE__, __LINE__, msg);
      xxx_CloseODL (in_parms, msg);
      return (ERROR);
    }
    xxx_CloseODL (in_parms, msg);
    ***************************************************************************/

    /* read the parameters from the ODL file */
    if (xxx_read_odl_file(ODL_filename, 0/* not cpf file */,  ODL_param_list, 
                          count) == SUCCESS)
    {
        /* verify the parameters that are required only if some flag turns on
           an option */
        if (terrain_flag)
        { 
            /* terrain correction requested, so make sure the DEM name
               was provided */
            if (dem_filename[0] == '\0')
            {
                /* error reading required item, log an error */
                xxx_LogStatus("GET_RESAMPLE_PARAMS",__FILE__,__LINE__,
                             "DEM name not provided in ODL parameter file");
                return (ERROR);
            }
        }

        /* parse the output data type */
        xxx_strtoupper(odtype);
        if (strcmp(odtype, "BYTE") == 0)
            output_type = BYTE_OUTPUT;
        else if (strcmp(odtype, "I*2") == 0)
            output_type = INT16_OUTPUT; 
        else if (strcmp(odtype, "I*4") == 0)
            output_type = INT32_OUTPUT;
        else if (strcmp(odtype, "R*4") == 0)
            output_type = FLOAT_OUTPUT;
        else
        {
            sprintf(msg,"Unknown output type - %s\n",odtype);
            xxx_LogStatus("GET_RESAMPLE_PARAMS",__FILE__,__LINE__, msg);
            return (ERROR);
        }


        /* parse the resampling type */
        xxx_strtoupper(resample_opt); 
        if (strcmp(resample_opt, "CC") == 0)
            resampling_type = CC_RESAMPLING;
        else if (strcmp(resample_opt, "NN") == 0)
            resampling_type = NN_RESAMPLING;
        else if (strcmp(resample_opt, "MTF") == 0)
            resampling_type = MTF_RESAMPLING;
        else if (strcmp(resample_opt, "BI") == 0)
            resampling_type = BI_RESAMPLING;
        else
        {
            sprintf(msg,"Unknown resample option %s",resample_opt);
            xxx_LogStatus("GET_RESAMPLE_PARAMS",__FILE__,__LINE__, msg);
            return (ERROR);
        }

	/* parse the band number from landsat 7 numbers to the 
	   BandNumber enumerated type */
        band_index = 0;
	for (count = 0; count < NBANDS; count++)
	{
	    /* convert the band number entered into the enumerated
	       band number type */
	    temp_band_num = axx_parse_user_band(bands[count]);
	    if ((count == 0) && (temp_band_num == berror))
	    {
		/* no bands numbers entered, so it is an error */
                sprintf(msg,"Illegal band number %d",band_list[count]);
                xxx_LogStatus("GET_RESAMPLE_PARAMS",__FILE__,__LINE__, msg);
                return (ERROR);
	    }

            /* make sure the band isn't already in the list */
            if (temp_band_num != berror)
            {
                if (!band_already_in_list[temp_band_num])
                {
                    /* put band in list and flag that it has been added 
                       to the list */
                    band_list[band_index] = temp_band_num;
                    band_already_in_list[temp_band_num] += 1;
                    band_index++;
                }
                else if (band_already_in_list[temp_band_num] == 1)
                {
                     /* only print the warning message once for each band */
                     band_already_in_list[temp_band_num] += 1;

                     if (sprintf(msg,"Warning: Band %d appears multiple times "
                                 "in the BAND_LIST, only the first one was used",
                                 axx_get_user_band(temp_band_num)) < 0)
                     {
                         /* error formatting the output */
                         xxx_LogStatus("GET_RESAMPLE_PARAMS",__FILE__,__LINE__,
                                 "Error formatting output");
                         return(ERROR);
                     }
                     else
                     {
                         /* log the warning */
                         xxx_LogStatus("GET_RESAMPLE_PARAMS",__FILE__,__LINE__,msg);
                     }
                }
            }
	}

        /* fill in remaining band locations with the band error indication */
        for (count = band_index; count < NBANDS; count++)
        {
            band_list[count] = berror;
        }
    }
    else
    {
        xxx_LogStatus("GET_RESAMPLE_PARAMS",__FILE__,__LINE__,
                      "error reading ODL parameter file");
        return (ERROR);
    }

    return (SUCCESS);
}

/******************************************************************************
NAME:           various accessor functions

PURPOSE:        
These functions provide access to the parameters read. 

RETURN VALUE:
Varies for the function.

NOTES:
The accessor function method of providing parameters was selected since
passing all the parameters up and down a calling sequence is a 
maintenance nightmare.

******************************************************************************/


char *get_1R_filename(void)
{
    return (filename_1R_image);
}

const BandNumber *get_band_list(void)
{
    return (band_list);
}

char *get_1G_filename(void)
{
    return (filename_1G_image);
}

char *get_grid_filename(void)
{
    return (grid_filename);
}

int do_terrain_correction(void)
{
    return (terrain_flag);
}

char *get_DEM_filename(void)
{
    return (dem_filename);
}

int do_delay_correction(void)
{
    return (delay_flag);
}

float get_gain_threshold(void)
{ 
	return (gain_threshold);
}

OUTPUT_DATA_TYPE get_output_data_type(void)
{
    return (output_type);
}

int get_output_data_size(void)
{
    if (output_type == BYTE_OUTPUT)
	return(sizeof(unsigned char));
    else if (output_type == INT16_OUTPUT)
	return(sizeof(int16));
    else if (output_type == INT32_OUTPUT)
	return(sizeof(int32));
    else 
    {
	ASSERT(output_type == FLOAT_OUTPUT);
	return(sizeof(float));
    }
}

RESAMPLE_TYPE get_resampling_type(void)
{
    return (resampling_type);
}

float get_output_range_minimum(void)
{
    return (min_max_output[0]);
}

float get_output_range_maximum(void)
{
    return (min_max_output[1]);
}

float get_cubic_convolution_alpha(void)
{
    return (alpha);
}

float get_fill_pixel(void)
{
   return (fill_pixel);
}

char *get_tmodel_filename(void)
{
    return (tmodel_filename);
}

char *get_0R_filename(void)
{
    return (L0R_filename);
}

char *get_CPF_filename(void)
{
    return (cal_parm_file);
}

char *get_script_name(void)
{
    return (script_name);
}

char *get_L0R_id(void)
{
    return (L0R_id);
}
