/******************************************************************************
NAME:           kernal

PURPOSE:        
The kernal module contains the routines needed to create and manage the 
resampling kernal.  The kernal is the weight table used for spatially 
resampling pixels.

ROUTINES:
    Kernal_Setup
    Cleanup_Kernal
    Create_Resampling_Kernal
    Get_Resample_Weight_Table_Ptr
    get_lines_in_kernal
    get_samples_in_kernal
    num_left_kernal_samples
    num_right_kernal_samples
    num_top_kernal_lines
    num_bottom_kernal_lines
    get_kernal_step_size

NOTES:
- The kernal module will build the resampling weight table for either 
  cubic convolution or mtf resampling.  The kernal is built for the current 
  band being resampled.
- The Kernal_Setup routine also handles the bilinear resampling option by
  setting the kernal size correctly for bilinear resampling.  However, no
  resampling weight table is built for bilinear resampling.


ALGORITHM REFERENCES:
    The algorithm for constructing the kernel from the five
    coefficients is documented in:
       "Cubic Convolution for One-Pass Restoration and Resampling"
       by Stephen E. Reichenbach and Kevin Haake
       Presented at IGARSS '96
    Note: this reference is actually for MTF resampling which is currently
    disabled (although MTF is a generic form of cubic convolution).


******************************************************************************/

#include <stdio.h>         /* i/o routines */
#include <stdlib.h>        /* memory management routines */
#include <math.h>          /* fabs routine */
#include "xxx_Types.h"     /* SUCCESS/ERROR definition */
#include "xxx_LogStatus.h" /* xxx_LogStatus prototype */
#include "xxx_const.h"     /* BandNumber definition */
#include "kernal.h"        /* prototypes for this module */
#include "cubicconv.h"     /* cubic convolution function */
#include "debug.h"         /* ASSERT macro */

/*
**
** MACROS
**
*/

#define SUBPIXEL_STEPS 64
/* defines the number of subpixel steps to include in the kernal weights. */

#define SUBPIXEL_TABLE_ENTRIES (SUBPIXEL_STEPS + 1)
/* defines the number of entries in the kernal weight table for the subpixel
   steps.  This is one more than the number of SUBPIXEL steps since the 
   fractional line number is rounded to the nearest 1/32 subpixel in the
   resampler.  To avoid having to check for rounding up to the next whole 
   line number, the extra subpixel step is included in the table. */

#define LINES_IN_KERNAL 4
/* defines the number of lines in the resampling kernal. This is the normal 
   cubic convolution kernal size. */

#define SAMPLES_IN_KERNAL 4
/* defines the number of samples in the resampling kernal.  This is two more 
   than the normal convolution kernal size to account for the detector 
   delays that may be present in the sample direction. */ 

#define BI_LINES_IN_KERNAL 2
/* defines the number of lines in the resampling kernal for bilinear 
   resampling. */
 
#define BI_SAMPLES_IN_KERNAL 2
/* defines the number of samples in the resampling kernal for bilinear
   resampling. */

#define DISABLE_MTF
/* disable the MTF compiling for this version */

/*
**
** MODULE VARIABLES
**
*/

static double *kernal_weights_ptr;
/* kernal_weights_ptr is a pointer to the table of resampling weights for
   cubic convolution and mtf (general form of cubic convolution) resampling.

   The table is laid out as a 6-dimensional array with the following indices:
    table[line subpixel offset (size SUBPIXEL_TABLE_ENTRIES)]
         [sample subpixel offset (size SUBPIXEL_TABLE_ENTRIES)]
         [kernal line (size LINES_IN_KERNAL)]
         [kernal sample (size SAMPLES_IN_KERNAL)]
   
*/

static int start_kernal_line = -2;
static int end_kernal_line =  2;
static int start_kernal_sample = -2;
static int end_kernal_sample = 2;

static int lines_in_kernal = 1;
/* number of lines in the kernal.  Its default initialization is one line
   (assuming the nearest neighbor kernal).  This is set to LINES_IN_KERNAL
   when Kernal_Setup is called. */

static int samples_in_kernal = 1;
/* number of samples in the kernal.  Its default initialization is one sample
   (assuming the nearest neighbor kernal).  This is set to SAMPLES_IN_KERNAL
   when Kernal_Setup is called. */

static int kernal_size_accessed;
/* the kernal_size_accessed is set to a non-zero value when either of the 
   functions to return a kernal size parameter is called.  This allows
   a debugging check to be put into the Kernal_Setup routine to make
   sure the kernal size isn't accessed before Kernal_Setup is called and
   changes the kernal size.  kernal_size_access bit 0 means the kernal
   size has been accessed, and bit 1 means the kernal has been setup. */

/*
**
** LOCAL PROTOTYPES
**
*/

static int Create_Resampling_Kernal
(
    double *weight_ptr, /* I/O: pointer to weights table */
    BandNumber band,    /* INPUT : current band         */
    RESAMPLE_TYPE resampling_type, /* INPUT : resampling type */
    double alpha        /* INPUT : cc alpha parameter      */
);

static int Create_Resampling_Delay_Kernal 
(
    double *weight_ptr,
    double cc_alpha
);

/******************************************************************************
NAME:           Kernal_Setup

PURPOSE:                
Kernal_Setup initializes the row weights for creating the extended image and
creates the resampling weight table for the current resampling method.

RETURN VALUE:
Either SUCCESS or ERROR are returned, depending on whether the kernal 
setup is successful or not.

NOTES:
Kernal_Setup sets up the resampling weight table for the current band being
processed.


ALGORITHM REFERENCES:
none

******************************************************************************
                Landsat 7 Image Assessment System (IAS) Software
                        Property of the U.S. Government
                                NASA/GSFC/EDC
******************************************************************************/

int Kernal_Setup
(
    BandNumber band,/* INPUT : current band         */
    RESAMPLE_TYPE resampling_type, /* INPUT : resampling type */
    double cc_alpha /* INPUT : cubic convolution alpha parameter */
)
{
    int kernal_table_size; /* number of weights in the kernal weight table */

    /* verify the kernal size hasn't been accessed before the Kernal_Setup
       is done the first time */
    ASSERT(kernal_size_accessed != 1);
    kernal_size_accessed |= 2;  /* set bit 1 */

    if( do_delay_correction() )
    {
       if( resampling_type != CC_RESAMPLING )
       {
          xxx_LogStatus("KERNAL",__FILE__,__LINE__, "Delays only applied with CC");
          return (ERROR);
       }

       /* initialize the lines and samples in the kernal */
       lines_in_kernal = SUBPIXEL_STEPS * LINES_IN_KERNAL;
       samples_in_kernal = SUBPIXEL_STEPS * SAMPLES_IN_KERNAL;

       /* if the kernal_weight_ptr has already been allocated from a previous
          band, free it since the size may change for the current band */
       free(kernal_weights_ptr);

       /* calculate the size of the kernal weight table */
       kernal_table_size = (lines_in_kernal + 1) * (samples_in_kernal + 1);

       kernal_weights_ptr = (double *)malloc(sizeof(double) * kernal_table_size);
       if (kernal_weights_ptr == NULL)
       {
           xxx_LogStatus("KERNAL",__FILE__,__LINE__, "Error allocating kernal weights");
           return (ERROR);
       }

       Create_Resampling_Delay_Kernal (kernal_weights_ptr, cc_alpha);
    }
    else
    {
       /* setup the kernal if MTF or CC resampling type */
       if ((resampling_type == CC_RESAMPLING) || (resampling_type == MTF_RESAMPLING))
       {
           /* initialize the lines and samples in the kernal */
           lines_in_kernal = LINES_IN_KERNAL;
           samples_in_kernal = SAMPLES_IN_KERNAL;
   
           /* if the kernal_weight_ptr has already been allocated from a previous
              band, free it since the size may change for the current band */
           free(kernal_weights_ptr);
   
           /* calculate the size of the kernal weight table */
           kernal_table_size = SUBPIXEL_TABLE_ENTRIES * SUBPIXEL_TABLE_ENTRIES * 
                               LINES_IN_KERNAL * SAMPLES_IN_KERNAL; 
   
           kernal_weights_ptr = (double *)malloc(sizeof(double) * kernal_table_size);
           if (kernal_weights_ptr == NULL)
           {
               xxx_LogStatus("KERNAL",__FILE__,__LINE__, 
                             "Error allocating space for kernal weights");
               return (ERROR);
           }

           Create_Resampling_Kernal (kernal_weights_ptr, band, resampling_type, cc_alpha);
       }
       else if (resampling_type == BI_RESAMPLING)
       {
           /* setup the kernal size for bilinear resampling */
           lines_in_kernal = BI_LINES_IN_KERNAL;
           samples_in_kernal = BI_SAMPLES_IN_KERNAL;
       }
    }
  
    return (SUCCESS);
}

/******************************************************************************
NAME:           Cleanup_Kernal

PURPOSE:                
Cleanup_Kernal cleans up the effects of the Kernal_Setup routine.

RETURN VALUE:
nothing is returned

NOTES:
Kernal_Setup sets up the resampling weight table for the current band being
processed.


ALGORITHM REFERENCES:
none

******************************************************************************
                Landsat 7 Image Assessment System (IAS) Software
                        Property of the U.S. Government
                                NASA/GSFC/EDC
******************************************************************************/

void Cleanup_Kernal(void)
{
    /* free the space used for the resampling weights */
    free (kernal_weights_ptr);
}


/******************************************************************************
NAME:           Create_Resampling_Delay_Kernal

PURPOSE:
Create_Resampling_Kernal builds the weight table for the resampling kernal.
This kernal only applies to detector delays being applied.

RETURN VALUE:
Either SUCCESS or ERROR are returned, depending on whether the kernal
creation is successful or not.

******************************************************************************
                Landsat 7 Image Assessment System (IAS) Software
                        Property of the U.S. Government
                                NASA/GSFC/EDC
******************************************************************************/
static int Create_Resampling_Delay_Kernal(
double *weight_ptr, 
double alpha)
{
   int subpixel_line,
       subpixel_sample;           /* Counters                             */
   double dx, dy;                 /* Sub-pixel spacing for cubic          */
   double *w_ptr;                 /* pointer to current weight table */
   double ccwy;                   /* weight for current y (line) component */
   double ccwx;                   /* weight for current x (line) component */
   double x,y;

   /* set temporary pointer to the weight table */
   w_ptr = weight_ptr;

   /* calculate subpixel line step */
   dy = ((double)(end_kernal_line - start_kernal_line)) 
         / (double)(LINES_IN_KERNAL * SUBPIXEL_STEPS);

   /* calculate subpixel sample step */
   dx = ((double)(end_kernal_sample - start_kernal_sample)) 
         / (double)(SAMPLES_IN_KERNAL * SUBPIXEL_STEPS);

   y = start_kernal_line;

   for (subpixel_line = 0; subpixel_line <= LINES_IN_KERNAL * SUBPIXEL_STEPS; subpixel_line++)
   {
      /* get the weight for the line */
      ccwy = cubic_convolution(alpha, y);

      x = start_kernal_sample;

      for (subpixel_sample = 0; subpixel_sample <= SAMPLES_IN_KERNAL * SUBPIXEL_STEPS; 
           subpixel_sample++)
      {
         /* get the weight for the current pixel */
         ccwx = cubic_convolution(alpha, x);
         *w_ptr++ = ccwy * ccwx;

         x = x + dx;

      } /* subpixel samples */

      y = y + dy;

   } /* subpixel lines */

   return (SUCCESS);
}

/******************************************************************************
NAME:           Create_Resampling_Kernal

PURPOSE:
Create_Resampling_Kernal builds the weight table for the resampling kernal.
The weights are setup properly for mtf or cubic convolution resampling, with
or without detector delays taken into account.

RETURN VALUE:
Either SUCCESS or ERROR are returned, depending on whether the kernal 
creation is successful or not.

NOTES:
The MTF parameters allow for having separate forward and reverse parameters.
This is not currently used, but is included for future growth.

ALGORITHM REFERENCES:
none

******************************************************************************
                Landsat 7 Image Assessment System (IAS) Software
                        Property of the U.S. Government
                                NASA/GSFC/EDC
******************************************************************************/

static int Create_Resampling_Kernal
(
    double *weight_ptr, /* I/O: pointer to weights table */
    BandNumber band,    /* INPUT : current band         */
    RESAMPLE_TYPE resampling_type, /* INPUT : resampling type */
    double alpha        /* INPUT : cc alpha parameter      */
)
{
    int    kernal_line,
           kernal_sample,
           subpixel_line,
           subpixel_sample;        /* Counters                             */
    int    left_samp,
           right_samp,
           top_line,
           bottom_line;            /* size of the kernal, relative to the
                                      center of the kernal */
    double dx, dy;                 /* Sub-pixel spacing for cubic          */
    double x, y;                   /* X and Y location of value to calc    */
    double *w_ptr;                 /* pointer to current weight table */
    double ccwy;                   /* weight for current y (line) component */

#ifndef DISABLE_MTF
    const MTF_PARAMETERS_TYPE *mtf_along_param_ptr 
			    = get_forward_MTF_weights_along(band);
    const MTF_PARAMETERS_TYPE *mtf_across_param_ptr
			    = get_forward_MTF_weights_across(band);
#endif

    /* verify the resampling kernal is only set up for CC since MTF has 
       been disabled for this version of the resampler */
    ASSERT (resampling_type == CC_RESAMPLING);

    /* get the kernal dimensions */
    left_samp = -num_left_kernal_samples();
    right_samp = num_right_kernal_samples();
    top_line = -num_top_kernal_lines();
    bottom_line = num_bottom_kernal_lines();

    /* set temporary pointer to the weight table */
    w_ptr = weight_ptr;

    /* build the kernal for either cubic convolution or mtf */
    if (resampling_type == CC_RESAMPLING)
    { 
      for (subpixel_line = 0; subpixel_line <= SUBPIXEL_STEPS; 
           subpixel_line++)
      {
        /* calculate subpixel line step */
        dy = subpixel_line / (float)(SUBPIXEL_STEPS);

        for (subpixel_sample = 0; subpixel_sample <= SUBPIXEL_STEPS; 
             subpixel_sample++)
        {
          /* calculate subpixel sample step */ 
          dx = subpixel_sample / (float)(SUBPIXEL_STEPS);

          for (kernal_line = top_line; kernal_line <= bottom_line; kernal_line++)
          {
            /* adjust the kernal line for the current subpixel step.  
               The subpixel step is subtracted since since a subpixel
               step down makes the weight at the current line position
               act like it moved up. */
            y = (double)kernal_line - dy;

            /* get the weight for the line */
            ccwy = cubic_convolution(alpha, y);

            for (kernal_sample = left_samp; kernal_sample <= right_samp; 
                 kernal_sample++)
            {
              /* adjust the kernal sample location for the current 
                 subpixel step. */
              x = (double)kernal_sample - dx;

              /* get the weight for the current pixel */
              *w_ptr++ = ccwy * cubic_convolution(alpha, x);

            } /* kernal samples */
          } /* kernal lines */
        } /* subpixel samples */
      } /* subpixel lines */
    } /* CC weights */

#ifndef DISABLE_MTF
    else
    {
      /* MTF weights */
      for (subpixel_line = 0; subpixel_line <= SUBPIXEL_STEPS; 
           subpixel_line++)
      {
        /* calculate subpixel line step */
        dy = subpixel_line / (float)(SUBPIXEL_STEPS);

        for (subpixel_sample = 0; subpixel_sample <= SUBPIXEL_STEPS; 
             subpixel_sample++)
        {
          /* calculate subpixel sample step */ 
          dx = subpixel_sample / (float)(SUBPIXEL_STEPS);

          for (kernal_line = top_line; kernal_line <= bottom_line; kernal_line++)
          {
            /* adjust the kernal line for the current subpixel step.  
               The subpixel step is subtracted since since a subpixel
               step down makes the weight at the current line position
               act like it moved up. */
            y = (double)kernal_line - dy;

            /* get the weight for the line */
            ccwy = mtf_convolution(mtf_across_param_ptr, y);

            for (kernal_sample = left_samp; kernal_sample <= right_samp; 
                 kernal_sample++)
            {
              /* adjust the kernal sample location for the current 
                 subpixel step. */
              x = (double)kernal_sample - dx;

              /* get the weight for the current pixel */
              *w_ptr++ = ccwy * mtf_convolution(mtf_along_param_ptr, x);

            } /* kernal samples */
          } /* kernal lines */
        } /* subpixel samples */
      } /* subpixel lines */
    } /* MTF weights */
#endif /* disable MTF */

    return (SUCCESS);

}

/******************************************************************************
NAME:           Get_Resample_Weight_Table_Ptr

PURPOSE:
Get_Resample_Weight_Table_Ptr provides access to the resampling kernal weight
table.

RETURN VALUE:
The pointer to the resampling weight table is returned.

NOTES:
The pointer to the weight table could be passed back to the calling routine when
it is created, but this is a little more efficient.

ALGORITHM REFERENCES:
none

******************************************************************************
                Landsat 7 Image Assessment System (IAS) Software
                        Property of the U.S. Government
                                NASA/GSFC/EDC
******************************************************************************/

const double *Get_Resample_Weight_Table_Ptr(void)
{
    ASSERT (kernal_weights_ptr != NULL);
    return (kernal_weights_ptr);
}

/******************************************************************************
NAME:           get_lines_in_kernal, get_samples_in_kernal

PURPOSE:
The get_lines_in_kernal and get_samples_in_kernal routines provide access 
to the number of lines and samples in the resampling kernal.

RETURN VALUE:
Type = int
The number of lines/samples in the resampling kernal is returned

NOTES:
none

ALGORITHM REFERENCES:
none

******************************************************************************
                Landsat 7 Image Assessment System (IAS) Software
                        Property of the U.S. Government
                                NASA/GSFC/EDC
******************************************************************************/

int get_lines_in_kernal(void)
{
    /* set the accessed flag to allow debugging check */
    kernal_size_accessed |= 1;

    return (lines_in_kernal);
}

int get_samples_in_kernal(void)
{
    /* set the accessed flag to allow debugging check */
    kernal_size_accessed |= 1;

    return (samples_in_kernal);
}

/* these routines return the number of pixels to the left, right, top, and
   bottom of the kernal center */

int num_left_kernal_samples(void)
{
    return ((samples_in_kernal) - ((samples_in_kernal)/2) - 1);
}

int num_right_kernal_samples(void)
{
    return (samples_in_kernal/2);
}

int num_top_kernal_lines(void)
{
    return ((lines_in_kernal) - ((lines_in_kernal)/2) - 1);
}

int num_bottom_kernal_lines(void)
{
    return (lines_in_kernal/2);
}

double get_kernal_step_size(void)
{
    return (1.0/(float)SUBPIXEL_STEPS);
}

void get_kernal_table_info
(
    int *subpixel_steps, /* O: number of subpixel steps in table */
    int *table_entries,  /* O: number of table entries for a whole pixel
                                step (one more than subpixel_steps */
    int *kernal_width,   /* O: width of kernal (samples across kernal) */
    int *kernal_height   /* O: height of kernal (lines across kernal) */
)
{
    /* verify this routine is only called for cubic convolution or bilinear */
    ASSERT ((samples_in_kernal == SAMPLES_IN_KERNAL) ||
            (samples_in_kernal == BI_SAMPLES_IN_KERNAL));

   *subpixel_steps = SUBPIXEL_STEPS;
   *table_entries = SUBPIXEL_TABLE_ENTRIES;
   *kernal_width = samples_in_kernal;
   *kernal_height = lines_in_kernal;
}

void get_kernal_table_info_delays
(
    int *subpixel_steps, /* O: number of subpixel steps in table */
    int *table_entries,  /* O: number of table entries */
    int *kernal_end_line,
    int *kernal_start_line,
    int *kernal_end_sample,
    int *kernal_start_sample,
    int *kernal_width,   /* O: width of kernal (samples across kernal) */
    int *kernal_height   /* O: height of kernal (lines across kernal) */
)
{
   /* calculate subpixel line step */
   *kernal_end_line = end_kernal_line;
   *kernal_start_line = start_kernal_line;
   *kernal_end_sample = end_kernal_sample; 
   *kernal_start_sample = start_kernal_sample;
   *subpixel_steps = SUBPIXEL_STEPS;
   *table_entries = (samples_in_kernal + 1) * (lines_in_kernal + 1);
   *kernal_width = samples_in_kernal;
   *kernal_height = lines_in_kernal;
}
