/******************************************************************************
NAME:           mtf

PURPOSE:
The mtf module implements the mtf convolution function.

ROUTINES:
    mtf_f0
    mtf_f1
    mtf_f2
    mtf_convolution
    
NOTES:
none

ALGORITHM REFERENCES:
    The algorithm for constructing the kernel from the five
    coefficients is documented in:
       "Cubic Convolution for One-Pass Restoration and Resampling"
       by Stephen E. Reichenbach and Kevin Haake
       Presented at IGARSS '96


******************************************************************************/

#include "mtf.h"   /* prototypes for this module */

/******************************************************************************
NAME:           mtf_f0, mtf_f1, mtf_f2

PURPOSE:        
The mtf_f0, mtf_f1, and mtf2 functions are helper functions for the main
mtf_convolution function.

RETURN VALUE:
The result of the function evaluation is returned as a double.

NOTES:
none

ALGORITHM REFERENCES:
see above

******************************************************************************/

static double mtf_f0
(
    double x    /* I: value to calculate f0 weight for */
)
{
  double t;             /* temporary variable */

  if ( x < -1.0 )
    t = 0.0;
  else if ( x < 0.0 )
    t = -2.0 * x*x*x - 3.0 * x*x + 1.0;
  else if ( x < 1.0 )
    t = 2.0 * x*x*x - 3.0 * x*x + 1.0;
  else
    t = 0.0;

  return t;
}

static double mtf_f1
(
    double x    /* I: value to calculate f1 weight for */
)
{
  double t;             /* temporary variable */

  if ( x < -1.0 )
    t = 0.0;
  else if ( x < 0.0 )
    t = x*x*x + 2.0 * x*x + x;
  else if ( x < 1.0 )
    t = x*x*x - 2.0 * x*x + x;
  else
    t = 0.0;

  return t;
}

static double mtf_f2
(
    double x    /* I: value to calculate f2 weight for */
)
{
  double t;             /* temporary variable */

  if ( x < -1.5 )
    t = 0.0;
  else if ( x < -0.5 )
    t = 2.0 * x*x*x + 6.0 * x*x + 4.5 * x;
  else if ( x < 0.5 )
    t = -4.0 * x*x*x + 3.0 * x;
  else if ( x < 1.5 )
    t = 2.0 * x*x*x - 6.0 * x*x + 4.5 * x;
  else
    t = 0.0;

  return t;
}

/******************************************************************************
NAME:           mtf_convolution

PURPOSE:        
The mtf_convolution function calculates the mtf weight for a point.

RETURN VALUE:
The mtf weight is returned as a double.

NOTES:
none

******************************************************************************/

double mtf_convolution
(
   const MTF_PARAMETERS_TYPE *mtf_param_ptr, /* I: MTF convolution parameters */
   double x  /* I: value to perform MTF convolution on */
)
{
    return (mtf_f0(x) + mtf_param_ptr->am1 * mtf_f1(x+1.0) +
            mtf_param_ptr->a0 * mtf_f1(x) +
            mtf_param_ptr->ap1 * mtf_f1(x-1.0) +
            mtf_param_ptr->bm1 * mtf_f2(-x-0.5) +
            mtf_param_ptr->bp1* mtf_f2(x-0.5));
}

