/******************************************************************************
NAME:           multiproc.c

PURPOSE:        
The purpose of the multiproc module is to isolate the details of using
multiple processors to a single module.  Most of the current implementation
resides in the associated header file as macros provide most of the 
implementation.  This file will allow easy porting to different 
multiprocessing operating systems.

DEVELOPMENT HISTORY:

RETURN VALUE:
none

NOTES:
none

ALGORITHM REFERENCES:
none

******************************************************************************/

#define _MULTIPROC_C_
#include "multiproc.h"

#ifdef USE_POSIX_MULTIPROC

#include <stdio.h>         /* temp for printf */
#include <stdlib.h>        /* malloc prototype */
#include "xxx_LogStatus.h" /* xxx_LogStatus prototype */
#include "xxx_const.h"     /* STRLEN definition */


static volatile int atomic_inc;
/* variable used to emulate the atomic increment interface */

static int num_threads;
/* number of threads to create when pth_fork is called */

static int thread_count;
/* quick hack to track thread numbers... */

pthread_mutex_t critical_sec_mutex;
/* mutex for the generic critical section */

static pthread_mutex_t atomic_inc_mutex;
/* mutex for the atomic increment critical section */

static pthread_t *thread_ids;
/* pointer to array that holds the ids of the threads created */

int pth_setup
(
    int threads    /* I: number of threads to fork */
)
{
    int return_status;  /* return status for routine */
    
    /* initialize critical section mutex */
    return_status = pthread_mutex_init(&critical_sec_mutex, NULL);
    if (return_status != 0)
    {
        xxx_LogStatus("MULTIPROC",__FILE__,__LINE__,
                      "Error creating critical section mutex");
        return (1); /* return non-zero for error */
    }
    
    /* create atomic increment mutex */
    return_status = pthread_mutex_init(&atomic_inc_mutex, NULL);
    if (return_status != 0)
    {
        xxx_LogStatus("MULTIPROC",__FILE__,__LINE__,
                      "Error creating atomic increment");
        pthread_mutex_destroy(&critical_sec_mutex);
        return (1); /* return non-zero for error */
    }
    
    /* save the number of threads to create */
    num_threads = threads;

    /* allocate memory for the array to hold pthread ids */
    thread_ids = malloc(threads * sizeof(pthread_t));
    if (thread_ids == NULL)
    {
        xxx_LogStatus("MULTIPROC",__FILE__,__LINE__,
                      "Error allocating memory for thread ids");
        pthread_mutex_destroy(&critical_sec_mutex);
        pthread_mutex_destroy(&atomic_inc_mutex);
        return (1); /* return non-zero for error */
    }

    /* return success */
    return (0);
    
}

int pth_cleanup(void)
{
    int return_status = 0;  /* return status */
    
    /* destroy the mutex's created, bit-ORing the return status together
       in case one of them fail */
    return_status = pthread_mutex_destroy(&critical_sec_mutex);
    return_status |= pthread_mutex_destroy(&atomic_inc_mutex);
    
    /* free memory allocated */
    free(thread_ids);
    
    /* return success */
    return (return_status);
}

int pth_fork
(
    THREAD_RETURN_TYPE func(void*), /* I: pointer to the function to call */
    void *param_ptr     /* I: pointer to the parameters to pass to the func */
)
{
    int loop;            /* loop counter */
    int return_status;   /* return status of routine */
    int threads_created; /* number of threads actually created */
    void *junk;          /* junk return value from thread */
    char msg[STRLEN];    /* message buffer */
    int status = 0;      /* status to return from this routine */
	int rs;				 /* status returned from function */
    
    /* zero the atomic increment counter since it is expected to start at 
       zero for each fork */
    atomic_inc = 0;
   
    /* reset the thread counter for the get thread number routine */
    thread_count = 0;

    /* create num_threads and save the thread id's for the get_thread_num 
       function */
#ifdef VERBOSE
    printf("Creating %d threads\n",num_threads);
#endif
    for (threads_created = 0; threads_created < num_threads; threads_created++)
    {
        return_status = pthread_create(&thread_ids[threads_created], NULL, func,
                                       param_ptr);
        if (return_status != 0)
        {
            /* error creating a thread, issue a message and stop trying to 
               create threads.  But still wait for any threads created to 
               terminate */
            rs = sprintf(msg,"Error creating thread %d of %d",threads_created + 1, num_threads);
            if (rs < 1)
            {
               xxx_LogStatus("MULTIPROC", __FILE__, __LINE__, "Error in sprintf");
               status = 1;
            }
            xxx_LogStatus("MULTIPROC",__FILE__,__LINE__,msg);
            break;
        }
    }

    /* wait for threads to rejoin */
    for (loop = 0; loop < threads_created; loop++)
    {
#ifdef VERBOSE    
        printf ("Waiting for thread %d to join\n",loop);
#endif
        return_status = pthread_join(thread_ids[loop],&junk);
        if (return_status != 0)
        {
            rs = sprintf(msg,"Error joining thread %d of %d",loop + 1, num_threads);  
            if (rs < 1)
            {
               xxx_LogStatus("MULTIPROC", __FILE__, __LINE__, "Error in sprintf");
               status = 1;
            }
            xxx_LogStatus("MULTIPROC",__FILE__,__LINE__,msg);
            
            /* set an error return status but continue looping to try and
               join with any remaining threads */
            status = 1;
        }
    }

    /* return success/fail indicator */
    return (status);
}

int pth_get_thread_num(void)
{
    int local_thread;
    
    /* get the current threads id and search table for it.  return the index
       it is found at in the table */
    if (pthread_mutex_lock(&critical_sec_mutex) != 0)
    {
        xxx_LogStatus("MULTIPROC",__FILE__,__LINE__,
                      "Error locking the critical section mutex");
        return (-1);
    }

    local_thread = thread_count;

    thread_count++;
    
    if (pthread_mutex_unlock(&critical_sec_mutex) != 0)
    {
        xxx_LogStatus("MULTIPROC",__FILE__,__LINE__,
                      "Error unlocking the critical section mutex");
        return (-1);
    }

    if (local_thread >= num_threads)
    {
        xxx_LogStatus("MULTIPROC",__FILE__,__LINE__,
                      "Error: GET_NUM_THREADS called too many times");
        return (-1);
    }
    
    return (local_thread);
}

int pth_atomic_inc(void)
{
    int local_count;   /* local copy of atomic increment */
    
    /* lock the atomic increment mutex */
    if (pthread_mutex_lock(&atomic_inc_mutex) != 0)
    {
        xxx_LogStatus("MULTIPROC",__FILE__,__LINE__,
                      "Error locking atomic increment mutex");
        return(-1);
    }
    
    /* get a local copy of the atomic counter */
    local_count = atomic_inc;

    /* increment the actual atomic counter */
    atomic_inc++;
    
    /* unlock the atomic increment mutex */
    if (pthread_mutex_unlock(&atomic_inc_mutex) != 0)
    {
        xxx_LogStatus("MULTIPROC",__FILE__,__LINE__,
                      "Error unlocking atomic increment mutex");
        return(-1);
    }
    
    /* return the local copy of the atomic counter before it was incremented */
    return (local_count);
}


#else
#ifndef USE_SGI_MULTIPROC

int global_atomic_inc;
/* variable used to simulate SGI m_next routine when not 
   compiled to use multiple processors. */

int non_multiproc_fork
(
    void func(void *),  /* I: pointer to the function to call */
    void *param_ptr      /* I: pointer to the parameters to pass to the func */
)
{
    /* set the global atomic increment variable to zero to mimic SGI 
       behavior */
    global_atomic_inc = 0;

    /* call the function */
    func(param_ptr);
    
    return (0); /* can't fail, but mimic SGI behavior */
}
#endif /* not USE_MULTIPLE_PROCESSORS */

#endif /* USE_POSIX */

