/******************************************************************************
NAME:           multiproc.h

PURPOSE:        
The purpose of the multiproc module is to isolate the details of using
multiple processors to a single module.  Most of the current implementation
resides in this associated header file as macros provide most of the 
implementation.  This file will allow easy porting to different 
multiprocessing operating systems.  Currently, only SGI and non-multiprocessing
are supported.

DEVELOPMENT HISTORY:

RETURN VALUE:
none

NOTES:
none

ALGORITHM REFERENCES:
none

******************************************************************************/

#ifndef _MULTIPROC_H_
#define _MULTIPROC_H_

/*---------------------------------------------------------------------------*/

#ifdef USE_POSIX_MULTIPROC

#include <unistd.h>    /* sysconf prototype */
#include <pthread.h>   /* pthread library */
#include <semaphore.h> /* semaphore prototypes */
#include <sched.h>     /* pthreading scheduling prototypes */
/*#include <sys/sched.h> /* pthreading scheduling types */

/*
**
** TYPE DEFINITIONS
**
*/

typedef pthread_mutex_t MUTEX_TYPE;  /* mutex type for posix threads */
typedef sem_t SEM_TYPE;              /* semaphore type for posix threads */
typedef void *THREAD_RETURN_TYPE;    /* type returned by a thread */

/*
**
** FUNCTION PROTOTYPES
**
*/
int pth_setup
(
    int threads    /* I: number of threads to fork */
);
int pth_cleanup(void);
int pth_fork
(
    THREAD_RETURN_TYPE func(void*), /* I: pointer to the function to call */
    void *param_ptr     /* I: pointer to the parameters to pass to the func */
);
int pth_get_thread_num(void);
int pth_atomic_inc(void);
void pth_enter_critical_section(void);
void pth_exit_critical_section(void);

/*
**
** MACRO DEFINITIONS
**
*/

#define CREATE_MUTEX(mutex) pthread_mutex_init(mutex,NULL)
#define DESTROY_MUTEX(mutex) pthread_mutex_destroy(mutex)
#define LOCK_MUTEX(mutex) pthread_mutex_lock(mutex)
#define UNLOCK_MUTEX(mutex) pthread_mutex_unlock(mutex)
/* mutex manipulation macros for pthreads.  All of them return zero when 
   successful. */

#define CREATE_SEMAPHORE(sem,initial_value) sem_init(sem,0,(initial_value))
#define DESTROY_SEMAPHORE(sem) sem_destroy(sem)
#define POST_SEMAPHORE(sem) sem_post(sem)
#define WAIT_SEMAPHORE(sem) sem_wait(sem)
/* semaphore manipulation macros for posix.  All of them return zero when
   successful. */

#define RETURN_FROM_THREAD() return (NULL)
/* defines the action to return from a thread. */

#define ENTER_CRITICAL_SECTION() pthread_mutex_lock(&critical_sec_mutex)
/* Macro to enter a critical section of code.  A zero is returned when
   successful. */

#define EXIT_CRITICAL_SECTION() pthread_mutex_unlock(&critical_sec_mutex)
/* Macro to exit a critical section of code.  A zero is returned when
   successful. */

#define SET_NUM_THREADS_TO_FORK(number)  pth_setup(number)
/* Macro to set the number of threads to create when FORK_THREADS
   is used.  A zero is returned when successful.  This SGI call
   actually creates the threads and they are left busy spinning
   until they are killed. */

#define GET_NUM_PROCESSORS() (int)sysconf(_SC_NPROCESSORS_ONLN)
/* Macro to get the number of processors.  This can be used set the
   number of threads to fork. */

#define FORK_THREADS(routine, parameter) pth_fork(routine, parameter)
/* Macro to fork multiple copies of a routine.  The parameter is a void 
   pointer that will be passed to the routine.  The number of copies 
   forked is set by SET_NUM_THREADS_TO_FORK.  A zero is returned when
   successful.  Control is not returned until the threads terminate. */

#define KILL_FORKED_THREADS() pth_cleanup()
/* Macro to kill the forked threads.  A zero is returned when successful. */

#define ATOMIC_INCREMENT() pth_atomic_inc()
/* Macro to atomically increment a counter.  This makes use of an SGI 
   specific function.  The value of the incremented variable is returned.
   If an error occurs, a negative value is returned. */

#define GET_THREAD_NUMBER() pth_get_thread_num()
/* Macro to return the calling thread's id.  A negative number is returned on
   an error. */

#define GET_NUMBER_OF_THREADS() get_number_threads()

#ifndef _MULTIPROC_C_

extern MUTEX_TYPE critical_sec_mutex;

#endif

/*---------------------------------------------------------------------------*/

#else 
#ifdef USE_SGI_MULTIPROC


#include <task.h>
#include <semaphore.h> /* semaphore prototypes */

/*
**
** TYPE DEFINITIONS
**
*/

typedef sem_t MUTEX_TYPE;           /* use semaphores for mutexs on SGI */
typedef sem_t SEM_TYPE;             /* semaphore type for posix threads */
typedef void THREAD_RETURN_TYPE;    /* type returned by a thread */

/*
**
** MACRO DEFINITIONS
**
*/

#define CREATE_MUTEX(mutex) sem_init(mutex,0,1) 
#define DESTROY_MUTEX(mutex) sem_destroy(mutex)
#define LOCK_MUTEX(mutex) sem_wait(mutex)
#define UNLOCK_MUTEX(mutex) sem_post(mutex)
/* For SGI implementation, use posix semaphores since SGI does not have
   mutex's defined but semaphores are useable.  All of them return zero when 
   successful. */

#define CREATE_SEMAPHORE(sem,initial_value) sem_init(sem,0,(initial_value))
#define DESTROY_SEMAPHORE(sem) sem_destroy(sem)
#define POST_SEMAPHORE(sem) sem_post(sem)
#define WAIT_SEMAPHORE(sem) sem_wait(sem)
/* semaphore manipulation macros for posix.  All of them return zero when
   successful. */

#define RETURN_FROM_THREAD() return
/* defines the action to return from a thread. */

#define ENTER_CRITICAL_SECTION() m_lock(), (0)
/* Macro to enter a critical section of code.  The comma operator is used
   to fake a return value of zero to make it work like the pthreads
   implementation. */

#define EXIT_CRITICAL_SECTION() m_unlock(), (0)
/* Macro to exit a critical section of code. The comma operator is used
   to fake a return value of zero to make it work like the pthreads
   implementation. */

#define SET_NUM_THREADS_TO_FORK(number)  m_set_procs(number)
/* Macro to set the number of threads to create when FORK_THREADS
   is used.  A zero is returned when successful.  This SGI call
   actually creates the threads and they are left busy spinning
   until they are killed. */

#define GET_NUM_PROCESSORS() m_get_numprocs()
/* Macro to get the number of processors.  This can be used set the
   number of threads to fork. */

#define FORK_THREADS(routine, parameter) m_fork(routine, parameter)
/* Macro to fork multiple copies of a routine.  The parameter is a void 
   pointer that will be passed to the routine.  The number of copies 
   forked is set by SET_NUM_THREADS_TO_FORK.  A zero is returned when
   successful.  Control is not returned until the threads terminate. */

#define KILL_FORKED_THREADS() m_kill_procs()
/* Macro to kill the forked threads.  A zero is returned when successful. */

#define ATOMIC_INCREMENT() m_next()
/* Macro to atomically increment a counter.  This makes use of an SGI 
   specific function.  The value of the incremented variable is returned. */

#define GET_THREAD_NUMBER() m_get_myid()
/* Macro to return the calling thread's id.  A negative number is returned on
   an error. */

/*---------------------------------------------------------------------------*/

#else

/*
**
** TYPE DEFINITIONS
**
*/

typedef int  MUTEX_TYPE;            /* dummy mutex type for single threading */
typedef int  SEM_TYPE;              /* dummy semaphore type for single thread */
typedef void THREAD_RETURN_TYPE;    /* type returned by a thread */

/*
**
** FUNCTION PROTOTYPES
**
*/

int non_multiproc_fork
(
    void func(void *),  /* I: pointer to the function to call */
    void *param_ptr      /* I: pointer to the parameters to pass to the func */
);

/*
**
** MACRO DEFINITIONS
**
*/

#define CREATE_MUTEX(mutex) (0)
#define DESTROY_MUTEX(mutex) (0)
#define LOCK_MUTEX(mutex) (0)
#define UNLOCK_MUTEX(mutex) (0)
/* dummy mutex manipulation macros for single threading.  All of them return 
   zero when successful. */

#define CREATE_SEMAPHORE(sem,initial_value) (0)
#define DESTROY_SEMAPHORE(sem) (0)
#define POST_SEMAPHORE(sem) (0)
#define WAIT_SEMAPHORE(sem) (0)
/* dummy semaphore manipulation macros for single threading.  All of them 
   return zero when successful. */

#define RETURN_FROM_THREAD() return 
/* defines the action to return from a thread. */

#define ENTER_CRITICAL_SECTION() (0)
/* Macro to enter a critical section of code.  Nothing is returned. */

#define EXIT_CRITICAL_SECTION() (0)
/* Macro to exit a critical section of code. Nothing is returned. */

#define SET_NUM_THREADS_TO_FORK(number) (0) 
/* Macro to set the number of threads to create when FORK_THREADS
   is used.  A zero is returned when successful.  This SGI call
   actually creates the threads and they are left busy spinning
   until they are killed. */

#define GET_NUM_PROCESSORS() (1) 
/* Macro to get the number of processors.  This can be used set the
   number of threads to fork. */

#define FORK_THREADS(routine, parameter) non_multiproc_fork(routine, parameter)
/* Macro to fork multiple copies of a routine.  The parameter is a void 
   pointer that will be passed to the routine.  The number of copies 
   forked is set by SET_NUM_THREADS_TO_FORK.  A zero is returned when
   successful.  Control is not returned until the threads terminate. */

#define KILL_FORKED_THREADS() (0)
/* Macro to kill the forked threads.  A zero is returned when successful. */

#define ATOMIC_INCREMENT() global_atomic_inc++ 
/* Macro to atomically increment a counter.  This makes use of an SGI 
   specific function.  The value of the variable before incrementing is 
   returned. */
   
#define GET_THREAD_NUMBER() (0)
/* Macro to return the calling thread's id.  A negative number is returned on
   an error. */

#ifndef _MULTIPROC_C_

extern int global_atomic_inc;

#endif

#endif /* USE_MULTIPLE_PROCESSORS */
#endif /* USE_POSIX */

/*---------------------------------------------------------------------------*/

#endif /* _MULTIPROC_H_ */

