/******************************************************************************
NAME:           resample_image

PURPOSE:
Perform geometric resampling on an image using cubic convolution 
or nearest neighbor resampling. 

NOTES:
none

******************************************************************************/
#include <stdlib.h>
#include <math.h>
#include "xxx_Types.h"     /* SUCCESS/ERROR definitions */
#include "xxx_LogStatus.h" /* xxx_LogStatus prototype */
#include "aliresample.h"   /* PROGRAM_NAME definition */
#include "resample_image.h"/* prototypes for this module */
#include "debug.h"         /* ASSERT macro */
#include "kernal.h"        /* Get_Resample_Weight_Table_Ptr prototype */
#include "file_io.h"

/* local versions of the minimum and maximum output range, adjusted for
   the currently selected output type. */
static double min_range;
static double max_range;

/* local copy of the output data type */
static OUTPUT_DATA_TYPE output_data_type;

/******************************************************************************
NAME:           setup_resamp_output

PURPOSE:        
setup_resamp_output configures the resampf1p module to generate the 
output image with the requested output data type and dynamic range.

RETURN VALUE:
Nothing is returned by this routine

NOTES:
- This routine uses the parameters read from the ODL parameter file to 
  create locally cached copies in the most efficient format for this module.
- This routine does not use the min/max values defined for the types in
  limits.h since those are machine specific.  The min/max values here are
  data dependent and cannot change for different machines.

ALGORITHM REFERENCES:
none

******************************************************************************/

void setup_resamp_output(void)
{
    output_data_type = get_output_data_type();

    min_range = get_output_range_minimum();
    max_range = get_output_range_maximum();

    if (output_data_type == BYTE_OUTPUT)
    {
        /* byte output so adjust the min and max accordingly */
        if (min_range < 0.0)
	    min_range = 0.0;
        else
            min_range = ceil(min_range);
        if (max_range > 255.0)
	    max_range = 255.0;
	else
	    max_range = floor(max_range);
    }
    else if (output_data_type == INT16_OUTPUT)
    {
        /* 2 byte integer output so adjust the min and max accordingly */
        if (min_range < -32768.0)
	    min_range = -32768.0;
        else
            min_range = ceil(min_range);
        if (max_range > 32767.0)
	    max_range = 32767.0;
	else
	    max_range = floor(max_range);
    }
    else if (output_data_type == INT32_OUTPUT)
    {
        /* byte output so adjust the min and max accordingly */
        if (min_range < -2147483648.0) 
	    min_range = -2147483648.0;
        else
            min_range = ceil(min_range);
        if (max_range > 2147483647.0)
	    max_range = 2147483647.0;
	else
	    max_range = floor(max_range);
    }
    else
    {
        /* must be float output, if it's not an error should have been 
	   obtained parsing the input, so just assert here during 
	   development.  no range manipulations are needed for the float
	   case */
        ASSERT (output_data_type == FLOAT_OUTPUT);
    }

}


/******************************************************************************
NAME:           convert_data

PURPOSE:                
convert_data converts the intermediate resampling results (currently stored as
doubles) and converts them to the requested output data type.

RETURN VALUE:
The type converted data is returned in the outbuf.

NOTES:
This routine was created to prevent the code having to be copied to the different
specialized resampling routines.  This incurs a small calling penalty, but it is
small when compared to the actual function.

ALGORITHM REFERENCES:
none

******************************************************************************/
static void convert_data
(
    const float *ptr, /* I: data to convert to the output type */
    void *outbuf,           /* O: output data buffer */
    int left_sample,        /* I: starting index in ptr buffer */
    int right_sample,       /* I: ending index in ptr buffer */
    int number_rows,
    int row_index           /* I: row index in outbuf */
)
{
    int l,m;  /* counter */

    /* convert the intermediate resampling results to the requested
       output type.  For the integer/byte types, the long cast makes
       the assembly code generated by the sgi compiler much more
       efficient (don't know why) */
    if (output_data_type == BYTE_OUTPUT)
    {
        unsigned char *char_ptr = outbuf;

        /* if buffering output, offset to the correct line and sample */
        char_ptr = &char_ptr[row_index + left_sample];

        /* convert the data, using unsigned rounding */
        for (m = 0; m < number_rows; m++)
           for (l = left_sample; l < right_sample; l++)
               *char_ptr++ = (unsigned char)(*ptr++ + 0.5);
    }
    else if (output_data_type == FLOAT_OUTPUT)
    {
        float *float_ptr = outbuf;

        /* if buffering output, offset to the correct line and sample */
        float_ptr = &float_ptr[row_index + left_sample];

        for (m = 0; m < number_rows; m++)
            for (l = left_sample; l < right_sample; l++)
                *float_ptr++ = *ptr++;
    }
    else if (output_data_type == INT16_OUTPUT)
    {
        int16 *int16_ptr = outbuf;

        /* if buffering output, offset to the correct line and sample */
        int16_ptr = &int16_ptr[row_index + left_sample];

        for (m = 0; m < number_rows; m++)
            for (l = left_sample; l < right_sample; l++)
            {
                /* convert the data, using signed rounding */
                if (*ptr >= 0)
                    *int16_ptr++ = (int16)(*ptr++ + 0.5);
                else
                    *int16_ptr++ = (int16)(*ptr++ - 0.5);
            }
    }
    else
    {
        int32 *int32_ptr = outbuf;

        /* if buffering output, offset to the correct line and sample */
        int32_ptr = &int32_ptr[row_index + left_sample];

        ASSERT(output_data_type == INT32_OUTPUT);

        for (m = 0; m < number_rows; m++)
            for (l = left_sample; l < right_sample; l++)
            {
                /* convert the data, using signed rounding */
                if (*ptr >= 0.0)
                    *int32_ptr++ = (int32)(*ptr++ + 0.5);
                else
                    *int32_ptr++ = (int32)(*ptr++ - 0.5);
            }
    }
}

/******************************************************************************
NAME:           resample_image_cc_delays

PURPOSE:
The specific resample routine to perform cubic convolution resampling
applying detector delays.

RETURN VALUE:
The resampled data is returned.

NOTES:
none

******************************************************************************/

THREAD_RETURN_TYPE resample_image_cc_delays
(
  void * params   /* IN : pointer to input parameter structure.  The
                          parameters are not passed directly since
                          the fork mechanism is limited to passing
                          a maximum of 6 void pointers. */
)
{
  /* create local copies of the parameters */
  RESAMPLE_THREAD_PARAMS *param_ptr = (RESAMPLE_THREAD_PARAMS *)params;
  int sca = param_ptr->sca;
  int nband = param_ptr->num_bands_processed;
  const axx_GRID_BAND_TYPE *grid_band_ptr = param_ptr->grid_band_ptr;
  IMAGE *dem = param_ptr->dem_image;
  IMAGE *input = param_ptr->input_image;
  FILE_IO_TYPE *file_io = param_ptr->file_io;

  BandNumber band;
  const short *data_ptr;        /* Input data pointer                   */
  COEFFICIENTS *coeff_ptr;      /* pointer to reverse coefficients in grid */
  int  ll, ls;                  /* Resampling weight table locations    */
  int  grid_cols;               /* Number of columns in grid            */
  const double *kernal_weight_table_ptr = Get_Resample_Weight_Table_Ptr();
                                /* pointer to kernal weight table */
  double w;               	/* Resampling weight table              */
  double round = 1.0 / 128.0;    /* Rounding error in resampling incr.   */
  double dl, ds;                /* Fractional line/sample offset        */
  double d_iline, d_isamp;      /* Input line/sample                    */
  int    i_iline, i_isamp;      /* integer input line/sample */
  int    oline, osamp;          /* output line/sample */
  double total;                 /* Resampled pixel output               */
  double fill = get_fill_pixel(); /* fill pixel */
  float *resample_ptr;
  int line_counter;
  int samp_counter;
  int count;
  int row,col;

  /* resampling kernal information */
  int  kernal_samples;
  int  kernal_lines;
  int  kernal_subpixel_steps;
  int  kernal_table_steps;
  double delta_line;
  double delta_sample;
  int kernal_end_line;
  int kernal_start_line;
  int kernal_end_sample;
  int kernal_start_sample;

  void *outbuf;

  int cell_index;
  int ncols;
  int nrows;
  double sum_weights;
  double delta_line_oe_samp;
  double delta_samp_oe_samp;

  double elev;
  double d_oline,d_osamp;
  int status;
  int terrain_flag;

  int m;
  int thread_num;       /* number of current thread */
  int row_index;
  int start_row;
  int end_row;
  int multi_proc_index;

  /* get the current thread number and check if an error (negative) */
  thread_num = GET_THREAD_NUMBER();
  if (thread_num < 0)
  {
      xxx_LogStatus(PROGRAM_NAME,__FILE__,__LINE__,
                    "Error obtaining the thread number");
      param_ptr->thread_error = -1;
      RETURN_FROM_THREAD();
  }

  ncols = grid_band_ptr->ngrid_samps - 1; 
  nrows = grid_band_ptr->ngrid_lines - 1; 

  band = input->band;

  /* get the kernal information */
  get_kernal_table_info_delays(&kernal_subpixel_steps, &kernal_table_steps, &kernal_end_line, 
                               &kernal_start_line, &kernal_end_sample, &kernal_start_sample, 
                               &kernal_samples, &kernal_lines);

  delta_line = ((double)(kernal_end_line - kernal_start_line)) / kernal_lines;
  delta_sample = ((double)(kernal_end_sample - kernal_start_sample)) / kernal_samples;
  if( delta_line == 0.0 || delta_sample == 0.0 )
  {
      xxx_LogStatus(PROGRAM_NAME,__FILE__,__LINE__, "Error with resampling weights");
      param_ptr->thread_error = thread_num + 1;
      RETURN_FROM_THREAD();
  }

  /* verify the kernal size is correct for the loop that
     actually does the resampling, which is hard coded to use
     a 4 line by 4 sample kernal for speed */
  ASSERT((kernal_lines == 4) && (kernal_samples == 4));

  outbuf = (void *)get_output_image_buffer(grid_band_ptr->samps);
  if (outbuf == NULL)
  {
     xxx_LogStatus(PROGRAM_NAME,__FILE__,__LINE__, "Error allocating output buffer");
     param_ptr->thread_error = thread_num + 1;
     RETURN_FROM_THREAD();
  }

  resample_ptr = malloc(sizeof(float) * grid_band_ptr->samps );
  if (resample_ptr == NULL)
  {
     xxx_LogStatus(PROGRAM_NAME,__FILE__,__LINE__, "Error allocating intermediate buffer");
     param_ptr->thread_error = thread_num + 1;
     RETURN_FROM_THREAD();
  }

  terrain_flag = do_terrain_correction();

  row_index = grid_band_ptr->lines;
  count = 0;
 
  end_row = 0;
  start_row = 0;
  for (multi_proc_index = ATOMIC_INCREMENT(); multi_proc_index < grid_band_ptr->lines;
       multi_proc_index = ATOMIC_INCREMENT()) /* Was oline loop */
  {
    /* make sure the ATOMIC_INC succeeded (worked when >= 0) */
    if (multi_proc_index < 0)
    {
      xxx_LogStatus(PROGRAM_NAME,__FILE__,__LINE__, "Error allocating intermediate buffer");
      param_ptr->thread_error = thread_num + 1;
      RETURN_FROM_THREAD();
    }

    start_row = multi_proc_index;
    end_row = start_row + 1;

    /* printf("MultiProc %d Start Row %d End Row %d\n",multi_proc_index,start_row,end_row); */

    for (m = start_row; m < end_row; m++)
    {
       oline = m;
       for (osamp = 0; osamp < grid_band_ptr->samps; osamp++)
       {
         /* Get elevation for image */
         if( terrain_flag )
         {
            if( get_elevation(dem, (double)oline, (double)osamp, grid_band_ptr->pixsize,
                              grid_band_ptr->pixsize, &elev) != SUCCESS)
            {
               xxx_LogStatus(PROGRAM_NAME,__FILE__,__LINE__, "Error getting elevation");
               param_ptr->thread_error = thread_num + 1;
               RETURN_FROM_THREAD();
            }
            status = axx_3d_ols2ils(grid_band_ptr, input->nl, input->ns,
                                    (double)oline, (double)osamp, &d_iline, &d_isamp,
                                    elev);
            row = (int)(d_iline / grid_band_ptr->grid_cell_size.line);
            col = (int)(d_isamp / grid_band_ptr->grid_cell_size.samp);
         }
         else
         {
            elev = 0.0;
            status = axx_all_ols2ils(grid_band_ptr, &row, &col, input->nl, input->ns,
                                     (double)oline, (double)osamp, &d_iline, &d_isamp,
                                     grid_band_ptr->zeroplane);
         }
   
         /* No zplane in deltas */
         cell_index = row * ncols + col;
   
         if( status==SUCCESS )
         {
           /* if the input line/sample are within the input image bounds */
           if ((d_iline >= 2) && (d_iline <  (input->nl-3)) &&
               (d_isamp >= 2) && (d_isamp <  (input->ns-3)))
           {

             /* convert the input line/sample into whole and fractional line/sample */
             i_iline = (int)d_iline;
             i_isamp = (int)d_isamp;
             dl = d_iline - i_iline;
             ds = d_isamp - i_isamp;
   
             /* get the resampling kernal index for the fractional line/sample */
   
             /* Perform resampling. */
             total = 0.0;     /* initialize total to zero */
             sum_weights = 0.0;
   
             /* loop through the kernal lines for resampling */
             for (line_counter = -1; line_counter < 3; line_counter++)
             {
               /* loop through the kernal samples for resampling */
               for(samp_counter = -1; samp_counter < 3; samp_counter++ )
               {
   
                  /* No zplane in deltas */
                  if( ((int)floor(i_isamp+samp_counter))%2==0 )
                  {
                     delta_line_oe_samp = elev * grid_band_ptr->delta_line_even[2 * cell_index + 1] +
                                          grid_band_ptr->delta_line_even[2 * cell_index];
                     delta_samp_oe_samp = elev * grid_band_ptr->delta_samp_even[2 * cell_index + 1] +
                                          grid_band_ptr->delta_samp_even[2 * cell_index];
                  }
                  else
                  {
                     delta_line_oe_samp = elev * grid_band_ptr->delta_line_odd[2 * cell_index + 1] +
                                          grid_band_ptr->delta_line_odd[2 * cell_index];
                     delta_samp_oe_samp = elev * grid_band_ptr->delta_samp_odd[2 * cell_index + 1] + 
                                          grid_band_ptr->delta_samp_odd[2 * cell_index];
                  }
   
                  ls = (int)((round + (-(ds+delta_samp_oe_samp) + samp_counter - kernal_start_sample)) 
                             / delta_sample);
                  ll = (int)((round + (-(dl+delta_line_oe_samp) + line_counter - kernal_start_line)) 
                             / delta_line);
   
                  if( ll>=0 && ll<=kernal_lines && ls>=0 && ls<=kernal_samples )
                  {
                     w = kernal_weight_table_ptr[ll * (kernal_samples+1) + ls];
                     sum_weights = sum_weights + w;
                     total += w * input->data[(i_iline + line_counter) * input->ns + i_isamp + samp_counter];
                  }
               }
             }
             if( sum_weights != 0.0 ) total = total / sum_weights;
   
             /* put new resampled value in intermediate buffer, limiting it to
                the output range */
             if (total < min_range)
                resample_ptr[osamp] = min_range;
             else if (total > max_range)
                resample_ptr[osamp] = max_range;
             else
                resample_ptr[osamp] = total;
             count++;
           }
           else
           {
             /* pixel is outside of input image so set the output pixel to fill */
             resample_ptr[osamp] = fill;
             count++;
           }
         }
         else
         {
           /* pixel is outside of input image so set the output pixel to fill */
           resample_ptr[osamp] = fill;
           count++;
         }
   
       } /* end of samp loop */

    } /* end for m = row */

    /* Convert data from double to output data type */
    convert_data(resample_ptr, outbuf, 0, grid_band_ptr->samps, (end_row-start_row), 0);


    /* lock the file io mutex since only one thread at a time can
       write to the file  */
    if (LOCK_MUTEX(&file_io->file_io_mutex) != 0)
    {
       xxx_LogStatus(PROGRAM_NAME,__FILE__,__LINE__, "Error locking the file io mutex");
       param_ptr->thread_error = thread_num + 1;
       RETURN_FROM_THREAD();
    }

    /* Write output line to 1G HDF */
    if (axx_write_1g_image(file_io->output_hdf_io_ptr, sca, 1, band, start_row, 0, 1,
                           grid_band_ptr->samps, 0, outbuf) != SUCCESS)
    {
       xxx_LogStatus(PROGRAM_NAME,__FILE__,__LINE__, "Error writing data");
       param_ptr->thread_error = thread_num + 1;
       RETURN_FROM_THREAD();
    }

    /* unlock file io mutex */
    if (UNLOCK_MUTEX(&file_io->file_io_mutex) != 0)
    {
       xxx_LogStatus(PROGRAM_NAME,__FILE__,__LINE__, "Error unlocking the file io mutex");
       param_ptr->thread_error = thread_num + 1;
       RETURN_FROM_THREAD();
    }

  } /* end of multiprocessing loop */

  if (ENTER_CRITICAL_SECTION() != 0)
  {
      xxx_LogStatus(PROGRAM_NAME,__FILE__,__LINE__, "Error entering critical section");
      param_ptr->thread_error = thread_num + 1;
      RETURN_FROM_THREAD();
  }

  free(resample_ptr);

  if (EXIT_CRITICAL_SECTION() != 0)
  {
      xxx_LogStatus(PROGRAM_NAME,__FILE__,__LINE__, "Error exiting critical section");
      param_ptr->thread_error = thread_num + 1;
      RETURN_FROM_THREAD();
  }

  RETURN_FROM_THREAD();
}


/******************************************************************************
NAME:           resample_image_cc

PURPOSE:
The specific resample routine to perform cubic convolution resampling.

RETURN VALUE:
The resampled data is returned.

NOTES:
none

******************************************************************************/

THREAD_RETURN_TYPE resample_image_cc
(
  void * params   /* IN : pointer to input parameter structure.  The
                          parameters are not passed directly since
                          the fork mechanism is limited to passing
                          a maximum of 6 void pointers. */
)
{
  /* create local copies of the parameters */
  RESAMPLE_THREAD_PARAMS *param_ptr = (RESAMPLE_THREAD_PARAMS *)params;
  int sca = param_ptr->sca;
  int nband = param_ptr->num_bands_processed;
  const axx_GRID_BAND_TYPE *grid_band_ptr = param_ptr->grid_band_ptr;
  IMAGE *dem = param_ptr->dem_image;
  IMAGE *input = param_ptr->input_image;
  FILE_IO_TYPE *file_io = param_ptr->file_io;			

  BandNumber band;
  const short *data_ptr;	/* Input data pointer			*/
  COEFFICIENTS *coeff_ptr;	/* pointer to reverse coefficients in grid */
  int  ll, ls;			/* Resampling weight table locations    */
  int  grid_cols;		/* Number of columns in grid            */
  const double *kernal_weight_table_ptr = Get_Resample_Weight_Table_Ptr(); 
                        /* pointer to kernal weight table */
  const double *w_ptr;		/* Resampling weight table              */
  double round = 1.0 / 128.0;	/* Rounding error in resampling incr.   */
  double dl, ds;		/* Fractional line/sample offset        */
  double d_iline, d_isamp;	/* Input line/sample                    */
  int    i_iline, i_isamp;	/* integer input line/sample */
  int    oline, osamp;		/* output line/sample */
  double total;			/* Resampled pixel output               */
  double fill = get_fill_pixel(); /* fill pixel */
  float *resample_ptr;
  int line_counter;
  int samp_counter;
  int count;
  int row,col;

  void *outbuf;

  /* resampling kernal information */
  int  kernal_samples;
  int  kernal_lines;
  int  kernal_subpixel_steps;
  double doub_kernal_subpixel_steps;
  int  kernal_table_steps;
  int  kernal_size;   /* samples * lines */
 
  double elev;
  double d_oline,d_osamp;
  int status;
  int terrain_flag;

  int m;
  int thread_num;       /* number of current thread */
  int row_index;
  int start_row;
  int end_row;
  int multi_proc_index;

  /* get the current thread number and check if an error (negative) */
  thread_num = GET_THREAD_NUMBER();
  if (thread_num < 0)
  {
      xxx_LogStatus(PROGRAM_NAME,__FILE__,__LINE__,
                    "Error obtaining the thread number");
      param_ptr->thread_error = -1;
      RETURN_FROM_THREAD();
  }

  band = input->band;

  /* get the kernal information */
  get_kernal_table_info(&kernal_subpixel_steps, &kernal_table_steps,
                        &kernal_samples, &kernal_lines);
  doub_kernal_subpixel_steps = (double)kernal_subpixel_steps;
  kernal_size = kernal_samples * kernal_lines;

  /* verify the kernal size is correct for the loop that 
     actually does the resampling, which is hard coded to use
     a 4 line by 4 sample kernal for speed */
  ASSERT((kernal_lines == 4) && (kernal_samples == 4));

  outbuf = (void *)get_output_image_buffer(grid_band_ptr->samps);
  if (outbuf == NULL)
  {
     xxx_LogStatus(PROGRAM_NAME,__FILE__,__LINE__, "Error allocating output buffer");
     param_ptr->thread_error = thread_num + 1;
     RETURN_FROM_THREAD();
  }

  resample_ptr = malloc(sizeof(float) * grid_band_ptr->samps);
  if (resample_ptr == NULL)
  {
      xxx_LogStatus(PROGRAM_NAME,__FILE__,__LINE__, "Error allocating intermediate buffer");
      param_ptr->thread_error = thread_num + 1;
      RETURN_FROM_THREAD();
  }

  terrain_flag = do_terrain_correction();

  row_index = grid_band_ptr->lines;

  end_row = 0;
  start_row = 0;
  for (multi_proc_index = ATOMIC_INCREMENT(); multi_proc_index < grid_band_ptr->lines;
       multi_proc_index = ATOMIC_INCREMENT()) /* Was oline loop */
  {
    /* make sure the ATOMIC_INC succeeded (worked when >= 0) */
    if (multi_proc_index < 0)
    {
      xxx_LogStatus(PROGRAM_NAME,__FILE__,__LINE__, "Error allocating intermediate buffer");
      param_ptr->thread_error = thread_num + 1;
      RETURN_FROM_THREAD();
    }

    start_row = multi_proc_index;
    end_row = start_row + 1;

    /* Loop through output lines/samples */
    for (m = start_row; m < end_row; m++)
    {
      oline = m;
      for (osamp = 0; osamp < grid_band_ptr->samps; osamp++)
      {
        /* Get elevation for image */
        if( terrain_flag )
        {
           if( get_elevation(dem, (double)oline, (double)osamp, grid_band_ptr->pixsize, 
                             grid_band_ptr->pixsize, &elev) != SUCCESS)
           {
              xxx_LogStatus(PROGRAM_NAME,__FILE__,__LINE__, "Error getting elevation");
              param_ptr->thread_error = thread_num + 1;
              RETURN_FROM_THREAD();
           }
           status = axx_3d_ols2ils(grid_band_ptr, input->nl, input->ns,
                                   (double)oline, (double)osamp, &d_iline, &d_isamp,
                                   elev);
           row = (int)(d_iline / grid_band_ptr->grid_cell_size.line);
           col = (int)(d_isamp / grid_band_ptr->grid_cell_size.samp);
        }
        else
        {
           status = axx_all_ols2ils(grid_band_ptr, &row, &col, input->nl, input->ns,
                                    (double)oline, (double)osamp, &d_iline, &d_isamp,
                                    grid_band_ptr->zeroplane);
        }
  
        if( status==SUCCESS )
        {
          /* if the input line/sample are within the input image bounds */
          if ((d_iline >= 1) && (d_iline <  (input->nl-2)) &&
              (d_isamp >= 1) && (d_isamp <  (input->ns-2)))
          {
            /* convert the input line/sample into whole and fractional line/sample */
            i_iline = (int)d_iline;
            i_isamp = (int)d_isamp;
            dl = d_iline - i_iline;
            ds = d_isamp - i_isamp;
          
            /* get the resampling kernal index for the fractional line/sample */
            ll = (int)(doub_kernal_subpixel_steps * (round + dl));
            ls = (int)(doub_kernal_subpixel_steps * (round + ds));
  
            /* get pointer to weight table location to use */ 
            /******************************************************************************
            w_ptr = &kernal_weight_table_ptr[(ll * kernal_table_steps + ls) * kernal_size];
            ******************************************************************************/
  
            w_ptr = kernal_weight_table_ptr + (ll * kernal_table_steps + ls) * kernal_size;
  
            /* Perform resampling. */
            total = 0.0;     /* initialize total to zero */
  
            /* loop through the kernal lines for resampling */
            for (line_counter = -1, count = 0; line_counter < 3; line_counter++)
            {
              /* loop through the kernal samples for resampling */
              for(samp_counter = -1; samp_counter < 3; samp_counter++, count++ )
              {
                 total += w_ptr[count] 
                       * input->data[(i_iline + line_counter) * input->ns + i_isamp + samp_counter];
              }
            }
  
            /* put new resampled value in intermediate buffer, limiting it to
               the output range */
            if (total < min_range)
               resample_ptr[osamp] = min_range;
            else if (total > max_range)
               resample_ptr[osamp] = max_range;
            else
               resample_ptr[osamp] = total;
          }
          else
          {
            /* pixel is outside of input image so set the output pixel to fill */
            resample_ptr[osamp] = fill;
          }
        }
        else
        {
          /* pixel is outside of input image so set the output pixel to fill */
          resample_ptr[osamp] = fill;
        }
       
      } /* end of samp loop */

    } /* end of m loop */

    /* Convert data from double to output data type */
    convert_data(resample_ptr, outbuf, 0, grid_band_ptr->samps, (end_row-start_row), 0);

    /* lock the file io mutex since only one thread at a time can
       write to the file  */
    if (LOCK_MUTEX(&file_io->file_io_mutex) != 0)
    {
       xxx_LogStatus(PROGRAM_NAME,__FILE__,__LINE__, "Error locking the file io mutex");
       param_ptr->thread_error = thread_num + 1;
       RETURN_FROM_THREAD();
    }

    /* Write output line to 1G HDF */
    if (axx_write_1g_image(file_io->output_hdf_io_ptr, sca, 1, band, start_row, 0, 1,
                           grid_band_ptr->samps, 0, outbuf) != SUCCESS)
    {
       xxx_LogStatus(PROGRAM_NAME,__FILE__,__LINE__, "Error writing data");
       param_ptr->thread_error = thread_num + 1;
       RETURN_FROM_THREAD();
    }

    /* unlock file io mutex */
    if (UNLOCK_MUTEX(&file_io->file_io_mutex) != 0)
    {
       xxx_LogStatus(PROGRAM_NAME,__FILE__,__LINE__, "Error unlocking the file io mutex");
       param_ptr->thread_error = thread_num + 1;
       RETURN_FROM_THREAD();
    }

  } /* end of multiprocessing loop */

  if (ENTER_CRITICAL_SECTION() != 0)
  {
      xxx_LogStatus(PROGRAM_NAME,__FILE__,__LINE__, "Error entering critical section");
      param_ptr->thread_error = thread_num + 1;
      RETURN_FROM_THREAD();
  }

  free(resample_ptr);

  if (EXIT_CRITICAL_SECTION() != 0)
  {
      xxx_LogStatus(PROGRAM_NAME,__FILE__,__LINE__, "Error exiting critical section");
      param_ptr->thread_error = thread_num + 1;
      RETURN_FROM_THREAD();
  }

  RETURN_FROM_THREAD();
}


/******************************************************************************
NAME:           resample_image_nn

PURPOSE:
The specific resample routine to perform nearest neighbor resampling.

RETURN VALUE:
The resampled data is returned.

NOTES:
none

ALGORITHM REFERENCES:
none

******************************************************************************/

THREAD_RETURN_TYPE resample_image_nn
(
  void * params   /* IN : pointer to input parameter structure.  The
                          parameters are not passed directly since
                          the fork mechanism is limited to passing
                          a maximum of 6 void pointers. */
)
{
  /* create local copies of the parameters */
  RESAMPLE_THREAD_PARAMS *param_ptr = (RESAMPLE_THREAD_PARAMS *)params;
  int sca = param_ptr->sca;
  int nband = param_ptr->num_bands_processed;
  const axx_GRID_BAND_TYPE *grid_band_ptr = param_ptr->grid_band_ptr;
  IMAGE *dem = param_ptr->dem_image;
  IMAGE *input = param_ptr->input_image;
  FILE_IO_TYPE *file_io = param_ptr->file_io;			

  BandNumber band; 
  float *resample_ptr;       
  COEFFICIENTS *coeff_ptr;	/* pointer to reverse coefficients in grid 	*/
  int  grid_cols;		/* Number of columns in grid			*/
  double d_iline, d_isamp;  	/* Input line/sample 				*/
  int    i_iline, i_isamp;	/* integer input line/sample */
  int    grid_row_index;	/* index to start of row in grid coefficients 	*/
  int    grid_cell_index;	/* index to cell in row for grid coefficients 	*/
  int    oline, osamp;		/* output line/sample 				*/
  double total;			/* Resampled pixel output 			*/
  double fill = get_fill_pixel(); /* fill pixel 				*/
  float *ptr;
  int row,col;
  int status;

  double elev;
  double d_oline,d_osamp;
  int terrain_flag;

  void *outbuf;

  int m;
  int thread_num;       /* number of current thread */
  int row_index;
  int start_row;
  int end_row;
  int multi_proc_index;

  /* get the current thread number and check if an error (negative) */
  thread_num = GET_THREAD_NUMBER();
  if (thread_num < 0)
  {
      xxx_LogStatus(PROGRAM_NAME,__FILE__,__LINE__,
                    "Error obtaining the thread number");
      param_ptr->thread_error = -1;
      RETURN_FROM_THREAD();
  }

  band = input->band;

  outbuf = (void *)get_output_image_buffer(grid_band_ptr->samps);
  if (outbuf == NULL)
  {
     xxx_LogStatus(PROGRAM_NAME,__FILE__,__LINE__, "Error allocating output buffer");
     param_ptr->thread_error = thread_num + 1;
     RETURN_FROM_THREAD();
  }

  /* allocate space for the intermediate buffer */
  resample_ptr = malloc(sizeof(float) * grid_band_ptr->samps);
  if (resample_ptr == NULL)
  {
      xxx_LogStatus(PROGRAM_NAME,__FILE__,__LINE__, "Error allocating intermediate buffer");
      param_ptr->thread_error = -1;
      RETURN_FROM_THREAD();
  }

  terrain_flag = do_terrain_correction();

  /* cache the number of grid columns */
  grid_cols = grid_band_ptr->ngrid_samps;

  row_index = grid_band_ptr->lines;

  end_row = 0;
  start_row = 0;
  for (multi_proc_index = ATOMIC_INCREMENT(); multi_proc_index < grid_band_ptr->lines;
       multi_proc_index = ATOMIC_INCREMENT()) /* Was oline loop */
  {
    /* make sure the ATOMIC_INC succeeded (worked when >= 0) */
    if (multi_proc_index < 0)
    {
      xxx_LogStatus(PROGRAM_NAME,__FILE__,__LINE__, "Error allocating intermediate buffer");
      param_ptr->thread_error = thread_num + 1;
      RETURN_FROM_THREAD();
    }

    start_row = multi_proc_index;
    end_row = start_row + 1;

    /* Loop through output lines/samples */
    for (m = start_row; m < end_row; m++)
    {
      oline = m;
      /* loop through the samples in the output image */ 
      for (osamp = 0; osamp < grid_band_ptr->samps; osamp++)
      {
        /* Get elevation for image */
        if( terrain_flag )
        {
           if( get_elevation(dem, (double)oline, (double)osamp, grid_band_ptr->pixsize,
                             grid_band_ptr->pixsize, &elev) != SUCCESS)
           {
              xxx_LogStatus(PROGRAM_NAME,__FILE__,__LINE__, "Error getting elevation");
              param_ptr->thread_error = thread_num + 1;
              RETURN_FROM_THREAD();
           }
           status = axx_3d_ols2ils(grid_band_ptr, input->nl, input->ns,
                                   (double)oline, (double)osamp, &d_iline, &d_isamp,
                                   elev);
           row = (int)(d_iline / grid_band_ptr->grid_cell_size.line);
           col = (int)(d_isamp / grid_band_ptr->grid_cell_size.samp);
        }
        else
        {
           status = axx_all_ols2ils(grid_band_ptr, &row, &col, input->nl, input->ns,
                                    (double)oline, (double)osamp, &d_iline, &d_isamp,
                                    grid_band_ptr->zeroplane);
        }

        if( status==SUCCESS )
        {
          /* add half a pixel to round to nearest neighbor */
          d_isamp += 0.5;
          d_iline += 0.5;
            
          /* if the input line/sample are within the input image bounds */
          if ((d_iline >= 0) && (d_iline <  input->nl) &&
              (d_isamp >= 0) && (d_isamp <  input->ns))
          {
            /* convert the input line/sample into an integer */
            i_iline = (int)d_iline;
            i_isamp = (int)d_isamp;
            
            /* get the nearest input pixel */
            total = input->data[i_iline * input->ns + i_isamp];
    
            /* put new resampled value in intermediate buffer, limiting it to
               the output range */
            if (total < min_range)
               resample_ptr[osamp] = min_range;
            else if (total > max_range)
               resample_ptr[osamp] = max_range;
            else
               resample_ptr[osamp] = total;
          }
          else
          {
            /* pixel is outside of input image so set the output pixel to fill */
             resample_ptr[osamp] = fill;
          }
        }
        else
        {
          /* pixel is outside of input image so set the output pixel to fill */
           resample_ptr[osamp] = fill;
        }

      } /* end of sample loop */

    } /* end of m loop */

    /* Convert data from double to output data type */
    convert_data(resample_ptr, outbuf, 0, grid_band_ptr->samps, (end_row-start_row), 0);

    /* lock the file io mutex since only one thread at a time can
       write to the file  */
    if (LOCK_MUTEX(&file_io->file_io_mutex) != 0)
    {
       xxx_LogStatus(PROGRAM_NAME,__FILE__,__LINE__, "Error locking the file io mutex");
       param_ptr->thread_error = thread_num + 1;
       RETURN_FROM_THREAD();
    }

    /* Write output line to 1G HDF */
    if (axx_write_1g_image(file_io->output_hdf_io_ptr, sca, 1, band, start_row, 0, 1,
                           grid_band_ptr->samps, 0, outbuf) != SUCCESS)
    {
       xxx_LogStatus(PROGRAM_NAME,__FILE__,__LINE__, "Error writing data");
       param_ptr->thread_error = thread_num + 1;
       RETURN_FROM_THREAD();
    }

    /* unlock file io mutex */
    if (UNLOCK_MUTEX(&file_io->file_io_mutex) != 0)
    {
       xxx_LogStatus(PROGRAM_NAME,__FILE__,__LINE__, "Error unlocking the file io mutex");
       param_ptr->thread_error = thread_num + 1;
       RETURN_FROM_THREAD();
    }

  } /* end of multiprocessing loop */

  if (ENTER_CRITICAL_SECTION() != 0)
  {
      xxx_LogStatus(PROGRAM_NAME,__FILE__,__LINE__, "Error entering critical section");
      param_ptr->thread_error = thread_num + 1;
      RETURN_FROM_THREAD();
  }

  free(resample_ptr);

  if (EXIT_CRITICAL_SECTION() != 0)
  {
      xxx_LogStatus(PROGRAM_NAME,__FILE__,__LINE__, "Error exiting critical section");
      param_ptr->thread_error = thread_num + 1;
      RETURN_FROM_THREAD();
  }

  RETURN_FROM_THREAD();
}



/******************************************************************************
NAME:           resample_image_bi

PURPOSE:
The specific resample routine to perform bilinear interpolation resampling.

RETURN VALUE:
The resampled data is returned.

NOTES:
none

******************************************************************************/

THREAD_RETURN_TYPE resample_image_bi
(
  void * params   /* IN : pointer to input parameter structure.  The
                          parameters are not passed directly since
                          the fork mechanism is limited to passing
                          a maximum of 6 void pointers. */
)
{
  /* create local copies of the parameters */
  RESAMPLE_THREAD_PARAMS *param_ptr = (RESAMPLE_THREAD_PARAMS *)params;
  int sca = param_ptr->sca;
  int nband = param_ptr->num_bands_processed;
  const axx_GRID_BAND_TYPE *grid_band_ptr = param_ptr->grid_band_ptr;
  IMAGE *dem = param_ptr->dem_image;
  IMAGE *input = param_ptr->input_image;
  FILE_IO_TYPE *file_io = param_ptr->file_io;			

  BandNumber band;
  float *resample_ptr;         
  const short *data_ptr;	/* Input data pointer			*/
  COEFFICIENTS *coeff_ptr;	/* pointer to reverse coeffs in grid 	*/
  int grid_cols;		/* Number of columns in grid            */
  double dl, ds;		/* Fractional line/sample offset        */
  double d_iline, d_isamp;	/* Input line/sample                    */
  int    i_iline, i_isamp;	/* integer input line/sample 		*/
  int    grid_row_index;  /* index to start of row in grid coefficients */
  int    grid_cell_index; /* index to cell in row for grid coefficients */
  int    oline, osamp;		/* output line/sample */
  double total;			/* Resampled pixel output               */
  double fill = get_fill_pixel();/* fill pixel 				*/
  int row,col;
  int status;
  
  void *outbuf;

  /* resampling kernal information */
  int  kernal_samples;
  int  kernal_lines;
  int  kernal_subpixel_steps;
  int  kernal_table_steps;

  double elev;
  double d_oline,d_osamp;
  int terrain_flag;

  int m;
  int thread_num;       /* number of current thread */
  int row_index;
  int start_row;
  int end_row;
  int multi_proc_index;

  /* get the current thread number and check if an error (negative) */
  thread_num = GET_THREAD_NUMBER();
  if (thread_num < 0)
  {
      xxx_LogStatus(PROGRAM_NAME,__FILE__,__LINE__,
                    "Error obtaining the thread number");
      param_ptr->thread_error = -1;
      RETURN_FROM_THREAD();
  }

  band = input->band;

  /* get the kernal information */
  get_kernal_table_info(&kernal_subpixel_steps, &kernal_table_steps,
                        &kernal_samples, &kernal_lines);

  /* verify the kernal size is correct for bilinear resampling */
  ASSERT((kernal_lines == 2) && (kernal_samples == 2));

  outbuf = (void *)get_output_image_buffer(grid_band_ptr->samps);
  if (outbuf == NULL)
  {
     xxx_LogStatus(PROGRAM_NAME,__FILE__,__LINE__, "Error allocating output buffer");
     param_ptr->thread_error = thread_num + 1;
     RETURN_FROM_THREAD();
  }

  /* allocate space for the intermediate buffer */
  resample_ptr = malloc(sizeof(float) * grid_band_ptr->samps);
  if (resample_ptr == NULL)
  {
      xxx_LogStatus(PROGRAM_NAME,__FILE__,__LINE__, "Error allocating output buffer");
      param_ptr->thread_error = thread_num + 1;
      RETURN_FROM_THREAD();
  }

  terrain_flag = do_terrain_correction();

  /* cache the number of grid columns */
  grid_cols = grid_band_ptr->ngrid_samps;

  row_index = grid_band_ptr->lines;

  end_row = 0;
  start_row = 0;
  for (multi_proc_index = ATOMIC_INCREMENT(); multi_proc_index < grid_band_ptr->lines;
       multi_proc_index = ATOMIC_INCREMENT()) /* Was oline loop */
  {
    /* make sure the ATOMIC_INC succeeded (worked when >= 0) */
    if (multi_proc_index < 0)
    {
      xxx_LogStatus(PROGRAM_NAME,__FILE__,__LINE__, "Error allocating intermediate buffer");
      param_ptr->thread_error = thread_num + 1;
      RETURN_FROM_THREAD();
    }

    start_row = multi_proc_index;
    end_row = start_row + 1;

    /* Loop through output lines/samples */
    for (m = start_row; m < end_row; m++)
    {
      oline = m;
      /* loop through the samples in the output window */ 
      for (osamp = 0; osamp < grid_band_ptr->samps; osamp++)
      {
        /* Get elevation for image */
        if( terrain_flag )
        {
           if( get_elevation(dem, (double)oline, (double)osamp, grid_band_ptr->pixsize,
                             grid_band_ptr->pixsize, &elev) != SUCCESS)
           {
              xxx_LogStatus(PROGRAM_NAME,__FILE__,__LINE__, "Error getting elevation");
              param_ptr->thread_error = thread_num + 1;
              RETURN_FROM_THREAD();
           }
           status = axx_3d_ols2ils(grid_band_ptr, input->nl, input->ns,
                                   (double)oline, (double)osamp, &d_iline, &d_isamp,
                                   elev);
           row = (int)(d_iline / grid_band_ptr->grid_cell_size.line);
           col = (int)(d_isamp / grid_band_ptr->grid_cell_size.samp);
        }
        else
        {
           status = axx_all_ols2ils(grid_band_ptr, &row, &col, input->nl, input->ns,
                                    (double)oline, (double)osamp, &d_iline, &d_isamp,
                                    grid_band_ptr->zeroplane);
        }
  
        if( status==SUCCESS )
        {
          /* if the input line/sample are within the input image bounds */
          if ((d_iline >= 0) && (d_iline <  input->nl-1) &&
              (d_isamp >= 0) && (d_isamp <  input->ns-1))
          {
            /* convert the input line/sample into whole and fractional line/sample */
            i_iline = (int)d_iline;
            i_isamp = (int)d_isamp;
            dl = d_iline - i_iline;
            ds = d_isamp - i_isamp;
    
            /* get the address of the data at input line/sample */
            data_ptr = &input->data[i_iline * input->ns + i_isamp];
     
            /* Perform resampling. */
            total = (1.0 - ds) * (1.0 - dl) * data_ptr[0] +
                           ds  * (1.0 - dl) * data_ptr[1] +
                    (1.0 - ds) *        dl  * data_ptr[input->ns] +
                           ds  *        dl  * data_ptr[input->ns + 1];
              
    
            /* put new resampled value in intermediate buffer, limiting it to the output range */
            if (total < min_range)
               resample_ptr[osamp] = min_range;
            else if (total > max_range)
               resample_ptr[osamp] = max_range;
            else
               resample_ptr[osamp] = total;
          
          }
          else
          {
            /* pixel is outside of input image so set the output pixel to fill */
             resample_ptr[osamp] = fill;
          }
        }
        else
        {
          /* pixel is outside of input image so set the output pixel to fill */
           resample_ptr[osamp] = fill;
        }

      } /* end of sample loop */

    } /* end of m loop */

    /* Convert data from double to output data type */
    convert_data(resample_ptr, outbuf, 0, grid_band_ptr->samps, (end_row-start_row), 0);

    /* lock the file io mutex since only one thread at a time can
       write to the file  */
    if (LOCK_MUTEX(&file_io->file_io_mutex) != 0)
    {
       xxx_LogStatus(PROGRAM_NAME,__FILE__,__LINE__, "Error locking the file io mutex");
       param_ptr->thread_error = thread_num + 1;
       RETURN_FROM_THREAD();
    }

    /* Write output line to 1G HDF */
    if (axx_write_1g_image(file_io->output_hdf_io_ptr, sca, 1, band, start_row, 0, 1,
                           grid_band_ptr->samps, 0, outbuf) != SUCCESS)
    {
       xxx_LogStatus(PROGRAM_NAME,__FILE__,__LINE__, "Error writing data");
       param_ptr->thread_error = thread_num + 1;
       RETURN_FROM_THREAD();
    }

    /* unlock file io mutex */
    if (UNLOCK_MUTEX(&file_io->file_io_mutex) != 0)
    {
       xxx_LogStatus(PROGRAM_NAME,__FILE__,__LINE__, "Error unlocking the file io mutex");
       param_ptr->thread_error = thread_num + 1;
       RETURN_FROM_THREAD();
    }

  } /* end of multiprocessing loop */ 

  if (ENTER_CRITICAL_SECTION() != 0)
  {
      xxx_LogStatus(PROGRAM_NAME,__FILE__,__LINE__, "Error entering critical section");
      param_ptr->thread_error = thread_num + 1;
      RETURN_FROM_THREAD();
  }

  free(resample_ptr);

  if (EXIT_CRITICAL_SECTION() != 0)
  {
      xxx_LogStatus(PROGRAM_NAME,__FILE__,__LINE__, "Error exiting critical section");
      param_ptr->thread_error = thread_num + 1;
      RETURN_FROM_THREAD();
  }

  RETURN_FROM_THREAD();
}
