/* ali_rps():                Heart of the RPS (Radiometric Processing System)
                             subsystem for ALI data analysis/processing.
			     Sets up all necessary variables, then calls
			     "scripts" listed in the ODL file "rps.odl" that
			     represent various kinds of processing.  As
			     "scripts" are matched, routines are then called
			     that do the actual processing.  */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>

#include "lablib3.h"                  /* ODL header */

#include "xxx_Types.h"                /* SUCCESS & ERROR defs */
#include "xxx_LogStatus.h"            /* xxx_LogStatus prototype */
#include "xxx_const.h"                /* STRLEN and definitions */
#include "xxx_ODL.h"                  /* ODL type */

#include "xxx_ali_level1r.h"          /* level 1R structures */
#include "xxx_ali_level0r.h"          /* level 0R structures */
#include "axx_rps.h"                  /* Generic ALI RPS header */
#include "mysql.h"                    /* SQL type defs and protos */

main(int argc, char *argv[])
{
     char	ODL_filename[STRLEN]; /* WO ODL file */
     char       CPF_filename[STRLEN]; /* CPF ODL file */
     char	msg[STRLEN];          /* Message string */
     char       hk_path[STRLEN];      /* Housekeeping file directory */
     char	scriptname[STRLEN];   /* RPS scriptname from WO ODL file */
     char	ODL_field[STRLEN];    /* ODL field to read from ODL file */
     char	basename[STRLEN];     /* Base filename for L0R and L1R files */
     char	l0r_path[STRLEN];     /* Directory path to L0R directoy */
     char	l0r_filename[STRLEN]; /* Filename of L0R file */
     char	l1r_path[STRLEN];     /* Directory path to L1R directoy */
     char	l1r_filename[STRLEN]; /* Filename of L0R file */
     char	CPF_field[STRLEN];    /* field to read from CPF */
     char       db_name[STRLEN];      /* Name of trending database */
     char       band_string[6];       /* Name of ALI band, ie(MS-1, etc) */
     char       gain_string[6];       /* Name of gain source */
     int	count;                /* Number of elements returned by ODL */
     int        l1r_product;          /* Flag indicating whether Level 1
					 products are created */
     int        gs;                   /* Gain source used to calibrate
				         scenes */
     int	trend_on;             /* Trend flag 1=on, 0=off */
     int	same_scene=0;         /* Flag to identify if same scene */
     int32	sca_index;            /* Index to current SCA */
     int32	n_scas;               /* Number of SCAs */
     int32	scas[MAX_SCAS];       /* Index of SCAs to be processed */
     int32      band;                 /* Current 0-based band number */
     int32	band_index;           /* Index to current band */
     int32	n_bands;              /* Number of bands */
     int32	bands[MAX_BANDS];     /* Index of bands to be processed */
     int32	n_scripts;            /* Number of RPS scripts */
     int32	script_num;           /* Current RPS script */
     int32      num_dets;             /* Total number of detectors */
     int32	scale_factor;         /* Scale factor for Level 1 product */
     int16      skip_length;          /* Number of frames to skip to get to
					 consecutive data (1 for MS bands,
					 3 for pan band)  */
     int	id;                   /* Scene ID for DB */
     CAL_INFO	cal_info[NUM_BAND];   /* Array of offsets for single SCA */
     OBJDESC	*p_in_parms;          /* Structure of ODL parameters. */
     OBJDESC	*p_cpf_parms;         /* Structure of CPF-ODL parameters. */
     LEVEL0R	level0r;              /* Structure of L0R data */
     LEVEL1R	level1r;              /* Structure of L1R data */
     SCENE_ARTIFACT_STATS   ar;       /* Structure of artifact data */

     double det_op_threshold;         /* Threshold value used in detector
					 operability testing */


  if(argc != 2)
   {
   /* Must have one arg, the ODL file, else error out and log */

     sprintf(msg,"Must supply one ODL filename after invocation.");
     xxx_LogStatus("rps",__FILE__,__LINE__,msg);
     printf("\nUSAGE:  rps <odl_file>\n\n");

     exit(ERROR);
   }

  /* Initialize the number_of_dims for all bands/sds's */
  xxx_init_level0r(&level0r);

 
   /* Set first and only arg. to the ODL filename
     ------------------------------------------- */
  sprintf(ODL_filename,"%s", argv[1]);

  /* Open ODL file
     ------------- */
  if((p_in_parms = xxx_OpenODL(ODL_filename,
                   (xxx_EDCOdl_TYPE)xxx_NoEDCMeta,
                   msg)) == NULL)
   {
     xxx_LogStatus("rps", __FILE__, __LINE__,msg);
     sprintf(msg, "Could not open %s.", ODL_filename);
     xxx_LogStatus("rps", __FILE__, __LINE__, msg);
     exit(ERROR);
   }
  sprintf(msg,"Opened %s Successfully...", ODL_filename);
  xxx_LogStatus("rps", __FILE__, __LINE__, msg);


  /* Read L0R directory path from ODL file
     ------------------------------------- */
  sprintf(ODL_field,"L0R_PATH");
  if(xxx_GetODLField(&l0r_path,
                     STRLEN*sizeof(char),
                     XXX_String,
                     p_in_parms,
                     NULL,
                     ODL_field,
                     msg,
                     &count) != SUCCESS)
    {
      xxx_LogStatus("rps", __FILE__, __LINE__, msg);
      sprintf(msg, "Couldn't find %s in %s.", ODL_field, ODL_filename);
      xxx_LogStatus("rps", __FILE__, __LINE__, msg);
      xxx_CloseODL(p_in_parms, msg);
      exit(ERROR);
    }


  /* Read L0R basename from ODL file
     ------------------------------- */
  sprintf(ODL_field,"L0R_FILENAME");
  if(xxx_GetODLField(&basename,
                     STRLEN*sizeof(char),
                     XXX_String,
                     p_in_parms,
                     NULL,
                     ODL_field,
                     msg,
                     &count) != SUCCESS)
   {
     xxx_LogStatus("rps", __FILE__, __LINE__, msg);
     sprintf(msg,"Couldn't find %s in %s.", ODL_field, ODL_filename);
     xxx_LogStatus("rps", __FILE__, __LINE__, msg);
     xxx_CloseODL(p_in_parms, msg);
     exit(ERROR);
   }
  

  /* Read number of SCAs from ODL file
     ------------------------------- */
  sprintf(ODL_field,"NUMBER_OF_SCAS");
  if(xxx_GetODLField(&n_scas,
                     sizeof(int32),
                     XXX_Int,
                     p_in_parms,
                     NULL,
                     ODL_field,
                     msg,
                     &count) != SUCCESS)
    {
      xxx_LogStatus("rps", __FILE__, __LINE__, msg);
      sprintf(msg,"Couldn't find %s in %s.",ODL_field,ODL_filename);
      xxx_LogStatus("rps", __FILE__, __LINE__, msg);
      xxx_CloseODL(p_in_parms, msg);
      exit(ERROR);
    }
   sprintf(msg,"ODL file specifies %d SCA's to be processed.", n_scas);
   xxx_LogStatus("rps", __FILE__, __LINE__, msg);

   /* Read SCAs to be processed
      -------------------------- */
   sprintf(ODL_field,"SCAS");
   if(xxx_GetODLField(scas,
                      sizeof(int32)*MAX_SCAS,
                      XXX_Int,
                      p_in_parms,
                      NULL,
                      ODL_field,
                      msg,
                      &count)  /* NEED TO VERIFY COUNT */
       != SUCCESS)
     {
       xxx_LogStatus("rps", __FILE__, __LINE__, msg);
       sprintf(msg, "Couldn't find %s in %s.", ODL_field, ODL_filename);
       xxx_LogStatus("rps", __FILE__, __LINE__, msg);
       xxx_CloseODL(p_in_parms, msg);
       exit(ERROR);
     }


  /* Read number of bands from ODL file
     ------------------------------- */
  sprintf(ODL_field,"NUMBER_OF_BANDS");
  if(xxx_GetODLField(&n_bands,
                     sizeof(int32),
                     XXX_Int,
                     p_in_parms,
                     NULL,
                     ODL_field,
                     msg,
                     &count) != SUCCESS)
    {
      xxx_LogStatus("rps", __FILE__, __LINE__, msg);
      sprintf(msg,"Couldn't find %s in %s.", ODL_field, ODL_filename);
      xxx_LogStatus("rps", __FILE__, __LINE__, msg);
      xxx_CloseODL(p_in_parms, msg);
      exit(ERROR);
    }
   sprintf(msg,"ODL file specifies %d bands to be processed.", n_bands);
   xxx_LogStatus("rps", __FILE__, __LINE__, msg);


   /* Read bands to be processed
      -------------------------- */
   sprintf(ODL_field,"BANDS");
   if(xxx_GetODLField(bands,
                      sizeof(int32)*MAX_BANDS,
                      XXX_Int,
                      p_in_parms,
                      NULL,
                      ODL_field,
                      msg,
                      &count)  /* NEED TO VERIFY COUNT */
       != SUCCESS)
     {
       xxx_LogStatus("rps", __FILE__, __LINE__, msg);
       sprintf(msg,"Couldn't find %s in %s.", ODL_field, ODL_filename);
       xxx_LogStatus("rps", __FILE__, __LINE__, msg);
       xxx_CloseODL (p_in_parms, msg);
       exit(ERROR);
     }

   
   /* Get path and name of housekeeping data file
      -------------------------------------------------*/
   sprintf(ODL_field, "HOUSEKEEPING_PATH");
   if(xxx_GetODLField(&hk_path,
		      STRLEN*sizeof(char),
		      XXX_String,
		      p_in_parms,
		      NULL,
		      ODL_field,
		      msg,
		      &count) != SUCCESS)
   {
     xxx_LogStatus("rps", __FILE__, __LINE__, msg);
     sprintf(msg,"Couldn't find %s in %s.", ODL_field, ODL_filename);
     xxx_LogStatus("rps", __FILE__, __LINE__, msg);
     xxx_CloseODL(p_in_parms, msg);
     exit(ERROR);
   }



   /* Do we want to generate any Level 1 products?  Find out here.
    ------------------------------------------------------------------*/
   sprintf(ODL_field, "GENERATE_L1R");
   if(xxx_GetODLField(&l1r_product,
		      sizeof(int),
		      XXX_Int,
		      p_in_parms,
		      NULL,
		      ODL_field,
		      msg,
		      &count) != SUCCESS)
     {
       xxx_LogStatus("rps", __FILE__, __LINE__, msg);
       sprintf(msg, "Couldn't find %s in %s.", ODL_field, ODL_filename);
       xxx_LogStatus("rps", __FILE__, __LINE__, msg);
       xxx_CloseODL(p_in_parms, msg);
       exit(ERROR);
     }

   /* If we do want to generate L1R product(s)...
    -------------------------------------------------*/
   if(l1r_product)
     {
       sprintf(msg, "ODL file specifies L1R products should be generated.");
       xxx_LogStatus("rps", __FILE__, __LINE__, msg);

       /* Read L1R directory path from ODL file
	  ------------------------------------- */
       sprintf(ODL_field,"L1R_PATH");
       if(xxx_GetODLField(&l1r_path,
			  STRLEN*sizeof(char),
			  XXX_String,
			  p_in_parms,
			  NULL,
			  ODL_field,
			  msg,
			  &count) != SUCCESS)
	 {
	   xxx_LogStatus ("rps", __FILE__, __LINE__, msg);
	   sprintf(msg,"Couldn't find %s in %s.",ODL_field,ODL_filename);
	   xxx_LogStatus ("rps", __FILE__, __LINE__, msg);
	   xxx_CloseODL (p_in_parms, msg);
	   exit(ERROR);
	 }


       /* Read gain source from ODL file
	  ------------------------------------*/
       sprintf(ODL_field, "GAIN_SOURCE");
       if(xxx_GetODLField(&gain_string,
			  6*sizeof(char),
			  XXX_String,
			  p_in_parms,
			  NULL,
			  ODL_field,
			  msg,
			  &count) != SUCCESS)
	 {
	   xxx_LogStatus("rps", __FILE__, __LINE__, msg);
	   sprintf(msg, "Couldn't find %s in %s.", ODL_field, ODL_filename);
	   xxx_LogStatus("rps", __FILE__, __LINE__, msg);
	   xxx_CloseODL(p_in_parms, msg);
	   exit(ERROR);
	 }

       if(!strcmp(gain_string, "CPF"))
	 {
	   gs = CAL_REL_CPF;
	   sprintf(ODL_field, "CPF_FILENAME");
	   if(xxx_GetODLField(&CPF_filename,
			      STRLEN*sizeof(char),
			      XXX_String,
			      p_in_parms,
			      NULL,
			      ODL_field,
			      msg,
			      &count) != SUCCESS)
	     {
	       xxx_LogStatus("rps", __FILE__, __LINE__, msg);
	       sprintf(msg, "Couldn't open CPF file %s.", CPF_filename);
	       xxx_LogStatus("rps", __FILE__, __LINE__, msg);
	       xxx_CloseODL(p_in_parms, msg);
	       exit(ERROR);
	     }
	 }
       else if(!strcmp(gain_string, "IC"))
	 gs = CAL_REL_IC;
       else if(!strcmp(gain_string, "Hist"))
	 gs = CAL_REL_HIST;
       else if(!strcmp(gain_string, "Solar"))
	 gs = CAL_ABS_SOLAR;
       else if(!strcmp(gain_string, "Lunar"))
	 gs = CAL_ABS_LUNAR;
       else if(!strcmp(gain_string, "Ground"))
	 gs = CAL_ABS_GLOOK;
       else
	 {
	   sprintf(msg, "Invalid gain source name: %s", gain_string);
	   xxx_LogStatus("rps", __FILE__, __LINE__, msg);
	   exit(ERROR);
	 }

       sprintf(msg, "ODL file specifies %s should be L1R gain source.",
	       gain_string);
       xxx_LogStatus("rps", __FILE__, __LINE__, msg);
     }
   else
     {
       sprintf(msg, "ODL file specifies no generation of L1R products.");
       xxx_LogStatus("rps", __FILE__, __LINE__, msg);
     }

  

   /* Read if trending is to be performed
      ----------------------------------- */
   sprintf(ODL_field,"TREND_ON");
   if(xxx_GetODLField(&trend_on,
                      sizeof(int),
                      XXX_Int,
                      p_in_parms,
                      NULL,
                      ODL_field,
                      msg,
                      &count) != SUCCESS)
     {
       xxx_LogStatus("rps", __FILE__, __LINE__, msg);
       sprintf(msg,"Couldn't find %s in %s.", ODL_field, ODL_filename);
       xxx_LogStatus("rps", __FILE__, __LINE__, msg);
       xxx_CloseODL(p_in_parms, msg);
       exit(ERROR);
     }


   /* Get the name of the database here and whether this scene has been
      previously processed, but ONLY IF we're trending.  If we're not
      trending, we don't care about the database stuff.
      ----------------------------------------------------------------------*/
   if(trend_on)
     {
       sprintf(msg, "ODL file specifies trending should be performed.");
       xxx_LogStatus("rps", __FILE__, __LINE__, msg);

       sprintf(ODL_field, "TREND_DATABASE");
       if(xxx_GetODLField(&db_name,
			  STRLEN*sizeof(char),
			  XXX_String,
			  p_in_parms,
			  NULL,
			  ODL_field,
			  msg,
			  &count) != SUCCESS)
         {
	   xxx_LogStatus("rps", __FILE__, __LINE__, msg);
	   sprintf(msg,"Couldn't find %s in %s.", ODL_field, ODL_filename);
	   xxx_LogStatus("rps", __FILE__, __LINE__, msg);
	   xxx_CloseODL(p_in_parms, msg);
	   exit(ERROR);
	 }
   
       sprintf(msg, "ODL file specifies %s to be trending database.",
	       db_name);
       xxx_LogStatus("rps", __FILE__, __LINE__, msg);

       sprintf(ODL_field, "SAME_SCENE");
       if(xxx_GetODLField(&same_scene,
			  sizeof(int),
			  XXX_Int,
			  p_in_parms,
			  NULL,
			  ODL_field,
			  msg,
			  &count) != SUCCESS)
	 {
	   xxx_LogStatus("rps", __FILE__, __LINE__, msg);
	   sprintf(msg, "Couldn't find %s in %s.", ODL_field, ODL_filename);
	   xxx_LogStatus("rps", __FILE__, __LINE__, msg);
	   xxx_CloseODL(p_in_parms, msg);
	   exit(ERROR);
	 }

       if(same_scene)
	 sprintf(msg, "ODL file specifies scene %s previously processed.",
		 basename);
       else
	 sprintf(msg, "ODL file specifies first processing of scene %s.",
		 basename);
       xxx_LogStatus("rps", __FILE__, __LINE__, msg);
     }
   else
     {
       sprintf(msg, "ODL file specifies no trending for scene %s.",
	       basename);
       xxx_LogStatus("rps", __FILE__, __LINE__, msg);
     }


   /* Read in miscellaneous parameters in this section.
**************************************************************************/

   /* Read the SCALE_FACTOR to scale Level 1R product.
      ------------------------------------------------ */
   sprintf(ODL_field, "SCALE_FACTOR");
   if(xxx_GetODLField(&scale_factor,
		      sizeof(int32),
		      XXX_Int,
		      p_in_parms,
		      NULL,
		      ODL_field,
		      msg,
		      &count) != SUCCESS)
     {
       xxx_LogStatus("rps", __FILE__, __LINE__, msg);
       sprintf(msg, "Could not read ODL field %s.", ODL_field);
       xxx_LogStatus("rps", __FILE__, __LINE__, msg);
       xxx_CloseODL(p_in_parms, msg);
       exit(ERROR);
     }
   sprintf(msg, "ODL file specifies a %s of %d.",
	   ODL_field, scale_factor);
   xxx_LogStatus("rps", __FILE__, __LINE__, msg);


   /* Read the threshold value used in the detector operability test.
    ----------------------------------------------------------------*/
   sprintf(ODL_field, "DET_OP_THRESHOLD");
   if(xxx_GetODLField(&det_op_threshold,
		      sizeof(double),
		      XXX_Double,
		      p_in_parms,
		      NULL,
		      ODL_field,
		      msg,
		      &count) != SUCCESS)
     {
       xxx_LogStatus("rps", __FILE__, __LINE__, msg);
       sprintf(msg, "Could not read ODL field %s.", ODL_field);
       xxx_LogStatus("rps", __FILE__, __LINE__, msg);
       xxx_CloseODL(p_in_parms, msg);
       exit(ERROR);
     }
   sprintf(msg, "ODL file specifies detector operability threshold is %3.2lf",
	   det_op_threshold);
   xxx_LogStatus("rps", __FILE__, __LINE__, msg);



   /* Read number of RPS scripts from ODL file
      ---------------------------------------- */
   sprintf(ODL_field,"NUMBER_OF_SCRIPTS");
   if(xxx_GetODLField(&n_scripts,
		      sizeof(int),
		      XXX_Int,
		      p_in_parms,
		      NULL,
		      ODL_field,
		      msg,
		      &count) != SUCCESS)
     {
       xxx_LogStatus ("rps", __FILE__, __LINE__, msg);
       sprintf(msg, "Could not read ODL field %s.", ODL_field);
       xxx_LogStatus ("rps", __FILE__, __LINE__, msg);
       xxx_CloseODL (p_in_parms, msg);
       exit(ERROR);
     }
   sprintf(msg, "ODL file specifies %d RPS scripts to be processed.",
	   n_scripts);
   xxx_LogStatus ("rps", __FILE__, __LINE__, msg);



  /* SCA loop
     -------- */
   for(sca_index = 0; sca_index < n_scas; sca_index++)
    {
   
      /* Set up the L0R filename according to the SCA
	 -------------------------------------------- */
      sprintf(l0r_filename, "%s/%s.M%dZ", l0r_path, basename, scas[sca_index]);

      sprintf(msg,"Reading %s (SCA #%d) as L0R file...", l0r_filename,
	      scas[sca_index]);
      xxx_LogStatus("rps", __FILE__, __LINE__, msg);


      /* Read L0R data from L0R file
        --------------------------- */
      if(xxx_read_ali_level0r(l0r_filename, &level0r) == ERROR)
	{
	  sprintf(msg,"Couldn't read %s.", l0r_filename);
	  xxx_LogStatus("rps", __FILE__, __LINE__, msg);
	  xxx_CloseODL(p_in_parms, msg);
	  exit(ERROR);
	}


      /* Copy Level 0R header to Level 1R header, and populate the header
	 info that doesn't exist in Level 0R.  Do this ONLY IF L1R products
	 are to be generated. 
	 -------------------------------------------------------------------*/
      if(l1r_product)
	  axx_generate_1r(level0r, &level1r);


      /* Set everything up for trending if the ODL file says we're
	 trending.   Print this only once. 
	 ---------------------------------------------------------------*/
      if(trend_on)
	{
	 /* If this is the first time the scene has been processed, add the
	   scene to the database.  Otherwise, get its ID.  Do these steps
	   ONLY on the first SCA that's processed; otherwise, it's
	   redundant, and could possibly cause some errors. */
	  if(!same_scene)
	    {
	      if(sca_index == 0)
		{
		  sprintf(msg, "Adding scene %s to database.", basename);
		  xxx_LogStatus("rps", __FILE__, __LINE__, msg);
		  if(xxx_InsertScene(&level0r, hk_path, db_name, basename,
				     &id) != SUCCESS)
		    sprintf(msg, "Error adding scene to database.");
		  else
		    sprintf(msg, "Scene added to database, ID = %d.",
			   id);
		  xxx_LogStatus("rps", __FILE__, __LINE__, msg);
		}
	    }
	  else
	    {
	      if(sca_index == 0)
		{
	
		  /* Get the scene ID from the database.
		     ----------------------------------------*/
		  sprintf(msg, "Getting ID for scene %s from database.",
			  basename);
		  xxx_LogStatus("rps", __FILE__, __LINE__, msg);
     
		  if(xxx_InsertScene(&level0r, hk_path, db_name, basename,
				     &id)!= SUCCESS)
		    {
		      /* If we get here, it's because rps.odl has the wrong
			 value for same_scene (1 for a scene that hasn't been
			 processed yet).  Try a fix by:

			 1) changing same_scene to 0.
			 2) adding the scene to the database

			 If adding the scene doesn't work, quit--we've got
			 a more serious problem that needs looking into.
			 Otherwise, add the scene and keep going. */

		      sprintf(msg, "rps.odl incorrectly indicates scene was");
		      sprintf(msg, "%s processed before", msg);
		      xxx_LogStatus("rps", __FILE__, __LINE__, msg);
		      sprintf(msg, "Adding scene to database.");
		      xxx_LogStatus("rps", __FILE__, __LINE__, msg);
		      if(same_scene)
			same_scene = 0;
		      if(xxx_InsertScene(&level0r, hk_path, db_name,
					 basename, &id) != SUCCESS)
			{
			  sprintf(msg, "Error adding scene to database,");
			  sprintf(msg, "%s aborting.  Check ODL file for",
				  msg);
			  sprintf(msg, "%s valid parameters.", msg);
			  xxx_LogStatus("rps", __FILE__, __LINE__, msg);
			  exit(ERROR);
			}
		      else
			{
			  sprintf(msg, "Scene added to database, ID = %d.",
				  id);
			  xxx_LogStatus("rps", __FILE__, __LINE__, msg);
			}
		    } 
		  else
		    {
		      sprintf(msg, "ID for scene %s is %d.", basename, id);
		      xxx_LogStatus("rps", __FILE__, __LINE__, msg);
		    }
		}


	      /* Clear the tables of each SCA's data.  Know that when
		 data is deleted and reinserted, the effective display
		 order will be reversed wrt the order when originally
		 inserted.  The code that inserts the data into the database
		 tries to take that into account.  */

	      sprintf(msg, "Deleting data for SCA %d.", scas[sca_index]);
	      xxx_LogStatus("rps", __FILE__, __LINE__, msg);
	      if(xxx_ClearTables(db_name, id, scas[sca_index]) != SUCCESS)
		sprintf(msg, "Data delete failed for SCA %d",
			scas[sca_index]);
	      else
		sprintf(msg, "Data delete for SCA %d complete.",
			scas[sca_index]);
	      xxx_LogStatus("rps", __FILE__, __LINE__, msg);
	    }
	}
     
   


      /* Band loop  ---- NEED TO CHANGE TO USE BAND INDEX ARRAY
        ------------------- */
      for(band_index = 0; band_index < n_bands; band_index++)
	{
	  /* For display purposes, convert the 0-indexed band number into
	     the ALI band name
	     ------------------------------------------------------------*/
	  band = bands[band_index] - 1;
	  axx_MakeALIBandString(band, band_string);


	  /* Determine the skip length based on the current band.  Currently,
	     the constant PAN_BAND is #defined to be 9 (0-based).  I ran into
	     some difficulty assuming that it was #defined as 10 (1-based).
	     This is strictly only necessary for dealing with bias data. */
	 
	  if(band != PAN_BAND)
	    skip_length = 1;
	  else
	    skip_length = 3;

	
	  /* Allocate and initialize all cal_info pointers
	   * --------------------------------------------- */
	  if(axx_allocate_cal_info(&cal_info[band],
				   level0r.image[band].dim_sizes[N_DETS_IDX],
				   band, skip_length) != SUCCESS)
	    {
	      sprintf(msg, "Could not allocate sufficient buffer memory.");
	      xxx_LogStatus("rps", __FILE__, __LINE__, msg);
	      exit(ERROR);
	    }

	  /* Load up the gains from the CPF file here, but ONLY IF
	     we explicitly state in "rps.odl" we want to calibrate with
	     these gains.
	     -----------------------------------------------------*/
	  if((l1r_product) && (gs == CAL_REL_CPF))
	    {

	      /* Open the CPF file
	       --------------------------*/
	      if((p_cpf_parms = xxx_OpenODL(CPF_filename,
					    (xxx_EDCOdl_TYPE)xxx_NoEDCMeta,
					    msg))  == NULL)
		{
		  xxx_LogStatus("rps", __FILE__, __LINE__ , msg);
		  sprintf(msg, "Could not open %s.", CPF_filename);
		  xxx_LogStatus("rps", __FILE__, __LINE__ , msg);
		  exit(ERROR);
		}

	      
	      /* Get the relative gains from the CPF file. 
		 -----------------------------------------------*/
	      sprintf(CPF_field, "BAND_%d_SCA_%d_p1", bands[band_index],
		      scas[sca_index]);
	      xxx_GetODLField(cal_info[band].gain.p1,
			      cal_info[band].ndets*sizeof(float64),
			      XXX_Double,
			      p_cpf_parms,
			      NULL,
			      CPF_field,
			      msg,
			      &count);

	      /* Get the relative biases from the CPF file.
	      --------------------------------------------------*/
	      sprintf(CPF_field, "BAND_%d_SCA_%d_p2", bands[band_index],
		      scas[sca_index]);
	      xxx_GetODLField(cal_info[band].gain.p2,
			      cal_info[band].ndets*sizeof(float64),
			      XXX_Double,
			      p_cpf_parms,
			      NULL,
			      CPF_field,
			      msg,
			      &count);
	    }
	 
	  /* Here's where we begin to loop through all of the RPS processing
	     scripts.  Whenever new functionality is added to RPS, put it
	     within this loop.
	  ------------------------------------------------------------------*/
	  for(script_num = 1; script_num <= n_scripts; script_num++)
	    {
	     
	      /* Read each RPS script from ODL file
		 ---------------------------------------*/
	      sprintf(ODL_field, "RPS_SCRIPT_%d", script_num);
	      if(xxx_GetODLField(scriptname,
				 STRLEN*sizeof (char),
				 XXX_String,
				 p_in_parms,
				 NULL,
				 ODL_field,
				 msg,
				 &count) != SUCCESS)
		{
		  xxx_LogStatus("rps", __FILE__, __LINE__, msg);
		  sprintf(msg, "Could not read ODL field %s.", ODL_field);
		  xxx_LogStatus("rps", __FILE__, __LINE__, msg);
		  xxx_CloseODL(p_in_parms, msg);
		  exit(ERROR);
		}
	      
	      /* Compare each script name in ODL file to what is known */
	      /* Currently "known" scripts are
	       *
	       *R  Calc_Bias      -- Characterize bias level
	       #T  Calc_ICRDG     -- Characterize IC-based relative
	                             detector gain
	       *R  App_Cal        -- Apply calibration coefficients
	       Char_IC        -- Characterize absolute cal. via internal cal.
	       #T  Char_CN        -- Characterize coherent noise
	       Corr_CN        -- Correct coherent noise
	       #T  Calc_Hist      -- Calculate Histogram
	       Char_GLC       -- Characterize absolute cal. via ground meas.
	       Char_Solar     -- Characterize absolute cal. via solar look
	       Char_Lunar     -- Characterize absolute cal. via lunar look
	       
	       Char_Striping  -- Characterize Striping
	       Corr_Striping  -- Correct Striping
	       Opt_Est        -- Optimal estimator of gain/bias
	       Char_SNR       -- Characterize signal-to-noise ratio
	       Char_EPSF      -- Characterize electronic point spread function
	       Corr_EPSF      -- Correct electronic point spread function
	       Char_IN        -- Characterize impulse noise
	       Corr_IN        -- Correct impulse noise
	       Char_DO        -- Characterize detector operability
	       Corr_DO        -- Correct detector operability
	       Char_RSR       -- Characterize relative spectral response
	       #T  Char_Sat       -- Determine detector saturation levels
	      */
   
	  
	      if(!strcmp(scriptname,"Pause"))
		{
		  /******************
		   * "Pause" SCRIPT *
		   ********************************************************/
		  sprintf(msg,"Running %s script.",scriptname);
		  xxx_LogStatus ("rps", __FILE__, __LINE__, msg);
		  /* PAUSE SCRIPT */
		  printf("Press Enter to Continue\n");
		  getc(stdin); /* Pause & wait for enter key */
		  /************************************************************
		   * "Pause" SCRIPT *
		   ******************/
		 
		}
	      else if(!strcmp(scriptname, "Calc_Bias"))
		{
		  /**********************
		  * "Calc_Bias" SCRIPT *
		  **********************************************************/
		  sprintf(msg, "Running %s script on band %s.",
			  scriptname, band_string);
		  xxx_LogStatus ("rps", __FILE__, __LINE__, msg);
		  
		  /* Call calculate bias routine */
		  axx_calc_bias(basename, &level0r, &cal_info[band],
				    scas[sca_index], band,
				    skip_length, trend_on,
				    db_name);
		  
		  sprintf(msg,"%s script complete for band %s.", scriptname,
			  band_string);
		  xxx_LogStatus("rps", __FILE__, __LINE__, msg);
		  /***********************************************************
		   * "Calc_Bias" SCRIPT *
		   **********************/
		  
		}
	      else if(!strcmp(scriptname, "App_Cal"))
		{
		  /************************
		   *  "App_Cal" SCRIPT *
		   ***********************************************************/
		  if(l1r_product)
		    {
		      sprintf(msg, "Running %s script on band %s.", scriptname,
			      band_string);
		      xxx_LogStatus("rps", __FILE__, __LINE__, msg);

		      /* Calibrate the images
		    ----------------------------*/
		      axx_calibrate(&level0r, &cal_info[band], band,
				    skip_length, scale_factor, &level1r);
		 
		      sprintf(msg, "%s script complete for band %s.",
			      scriptname, band_string);
		      xxx_LogStatus("rps", __FILE__, __LINE__, msg);

		    } else {
                      sprintf(msg, "Not running %s script because L1R product not output", scriptname);
                      xxx_LogStatus("rps", __FILE__, __LINE__, msg);
                    }
 
		  /**********************************************************
		  * "App_Cal" SCRIPT *
		  **********************/
		 
		}
	      else if(!strcmp(scriptname, "Calc_ICRDG"))
		{
		  /********************
		   * "Calc_ICRDG" SCRIPT *
		   ***********************************************************/
		  /* Get the IC-based relative detector gain
		     -----------------------------------------------*/
		  sprintf(msg, "Running %s script on band %s.", scriptname,
			  band_string);
		  if(axx_calc_ICRDG(basename, &level0r, &cal_info[band],
				    scas[sca_index], band,
				    skip_length, trend_on,
				    db_name) != SUCCESS)
		      sprintf(msg, "%s script failure for band %s.",
			      scriptname, band_string);
		  else
		    sprintf(msg,"%s script complete for band %s.", scriptname,
			    band_string);
		  xxx_LogStatus("rps", __FILE__, __LINE__, msg);
		  
		  /*********************************************************
		   * "Calc_ICRDG" SCRIPT *
		   *******************/
		 
		}
	      else if(!strcmp(scriptname, "Calc_Hist"))
		{
		  /**********************
		   *  "Calc_Hist" SCRIPT *
		   ***********************************************************/
		  sprintf(msg, "Running %s script on band %s.", scriptname,
			  band_string);
		  xxx_LogStatus("rps", __FILE__, __LINE__, msg);
		  
		  /* Call histogram calculation routine
		     ---------------------------------------*/ 
		  axx_calc_hist(basename, &level0r, &cal_info[band].hist,
				scas[sca_index], band, cal_info[band].ndets,
				IMAGE, trend_on, db_name);
		  
		  sprintf(msg, "%s script complete for band %s.", scriptname,
			  band_string);
		  xxx_LogStatus("rps", __FILE__, __LINE__, msg);
		  /************************************************************
		   * "Calc_Hist" SCRIPT *
		   ********************/
		 
		}

	      /*else if(!strcmp(scriptname, "Char_Fill"))
		{

		This code and the code for Char_CN below have been commented
		out.  These routines are currently in development and should
		appear in a future release.

		***********************
		* "Char_Fill" SCRIPT *
		************************************************************
		* Find locations in pre-cal, image and lamp datasets where
		there may be fill. In the earlier modules we checked for
		it, then ignored it.  Now, we want to know explicitly
		where its located.  However, ignore the fill at the
		edges of the image.  That should be constant for each
		image type.
		---------------------------------------------------------- 
		sprintf(msg, "Checking band %s pre-cal data for fill.",
		band_string);
		xxx_LogStatus("rps", __FILE__, __LINE__, msg);
		axx_char_det_fill(&level0r, basename, scas[sca_index],
		band, PRECAL);
		
		sprintf(msg, "Checking band %s image data for fill.",
		band_string);
		xxx_LogStatus("rps", __FILE__, __LINE__, msg);
		axx_char_det_fill(&level0r, basename, scas[sca_index],
		band, IMAGE);
		
		sprintf(msg, "Checking band %s dark data for fill.",
		bands[band_index]);
		xxx_LogStatus("rps", __FILE__, __LINE__, msg);
		axx_char_det_fill(&level0r, basename, scas[sca_index],
		band, DARK);
		
		sprintf(msg, "Checking band %s lamp data for fill.",
		band_string);
		xxx_LogStatus("rps", __FILE__, __LINE__, msg);
		axx_char_det_fill(&level0r, basename, scas[sca_index],
		band, LAMP);
		
		sprintf(msg, "Band %s fill data checks completed.",
		band_string);
		xxx_LogStatus("rps", __FILE__, __LINE__, msg);
		***********************************************************
		* "Char_Fill" SCRIPT *
		***********************
		}
		else if(!strcmp(scriptname, "Char_CN"))
		{
		  ********************************************************
		  * "Char_CN" SCRIPT
		  **********************************************************
		  sprintf(msg, "Running %s script on band %s.", scriptname,
		  band_string);
		  xxx_LogStatus("rps", __FILE__, __LINE__, msg);
		  
		  Do coherent noise characterization.
		  ---------------------------------
		  if(axx_char_cn(&level0r, band, cal_info[band].ndets,
		  LAMP) != SUCCESS)
		  sprintf(msg, "Error running %s for band %s.", scriptname,
		  band_string);
		  else
		  sprintf(msg, "Script %s completed for band %s.", scriptname,
		  band_string);
		  xxx_LogStatus("rps", __FILE__, __LINE__, msg);
		 ***********************************************************
		 *     "Char_CN" Script
		 ***********************************************************
		 }*/
	      else if(!strcmp(scriptname, "Char_Sat"))
		/***********************
		 * "Char_Sat" SCRIPT *
		 ************************************************************/
		{
		  /*Initialize pointers in ar to NULL
		    (good for diagnostics).
		    ------------------------------------------------*/
		  ar.type = NULL;
		  ar.value = NULL;
		  ar.number = NULL;
		  
		  sprintf(msg, "Running %s script on band %s.", scriptname,
			  band_string);
		  xxx_LogStatus("rps", __FILE__, __LINE__, msg);
		  if(axx_char_det_sat(basename, &level0r,
				      &cal_info[band], scas[sca_index],
				      band, PRECAL, &ar, trend_on, db_name)
		     != SUCCESS)
		    sprintf(msg, "Error running %s for band %s.",
			    scriptname, band_string);
		  else
		    sprintf(msg, "%s script completed for band %s.",
			    scriptname, band_string);
		  xxx_LogStatus("rps", __FILE__, __LINE__, msg);
		  /***********************************************************
		   * "Char_Sat" SCRIPT *
		   ***********************/
		}
	      else if(!strcmp(scriptname, "Det_Op"))
		{
		  /***********************
		   * "Det_Op" SCRIPT *
		   **********************************************************/
		  sprintf(msg, "Running %s script on band %s.", scriptname,
			  band_string);
		  xxx_LogStatus("rps", __FILE__, __LINE__, msg);
		  
		  /* Check for suspect detectors.
		   -----------------------------------*/
/* TAKEN OUT BECAUSE THIS FUNCTION ISN'T CHECKED IN */
/*		  axx_find_suspect_dets(basename, &level0r, scas[sca_index],
					band, DARK, &cal_info[band],
					(float64)det_op_threshold,
					trend_on, db_name);
*/
sprintf(msg, "FAKING axx_find_suspect_dets BECAUSE IT HASN'T BEEN CHECKED IN.");
xxx_LogStatus("rps", __FILE__, __LINE__, msg);

		  sprintf(msg, "%s script completed for band %s.",
			  scriptname, band_string);
		  xxx_LogStatus("rps", __FILE__, __LINE__, msg);
		  /*********************************************************
		   * "Det_Op" SCRIPT *
		   **********************/
		}
	      else
		{
		  /******************************
		   * Default, exit with warning *
		   **********************************************************/
		  sprintf(msg, "Not running %s, don't know how.", scriptname);
		  sprintf(msg, "%s  Check spelling of %s in %s",
			  msg, ODL_field, ODL_filename);
		  xxx_LogStatus("rps", __FILE__, __LINE__, msg);
		  /***********************************************************
		   * Default, exit with warning *
		   *****************************/
		}
	    }
	}


      /* Generate the Level 1R products ONLY IF the ODL file specifies
	 they're to be generated.
      ---------------------------------------------------------------------*/
      if(l1r_product)
	{
	  /* Setup l1r filename
	     --------------------------*/
	  sprintf(l1r_filename, "%s/%s.M%dR", l1r_path, basename,
		  scas[sca_index]);
	  sprintf(msg,"Writing %s (SCA #%d) as L1R file...", l1r_filename, 
		  scas[sca_index]);
	  xxx_LogStatus("rps",__FILE__,__LINE__,msg);

	  /* Write the Level1R data
	     ---------------------------*/
	  xxx_write_ali_level1r(l1r_filename, level1r);

	  /* Clean up the resources required by level1r for the current
	     SCA
	   ------------------------------------------------*/
	  xxx_close_ali_level1r(&level1r);

	}


     /* Clean up memory for current SCA
        ------------------------------- */
      xxx_close_ali_level0r(&level0r);


     /* Free the allocated memory for each band's CAL_INFO structure
	for the current SCA
      ---------------------------------------------*/
      sprintf(msg, "Freeing memory required by CAL_INFO data structures.");
      xxx_LogStatus("rps", __FILE__, __LINE__, msg);
      
      for(band_index = 0; band_index < n_bands; band_index++)
	axx_free_cal_info(&cal_info[bands[band_index]-1]);

     
    } /* SCA loop */

   /* We're done.  Close everything down gracefully.
      ------------------------------------------------*/
   sprintf(msg, "Completed Successfully.");
   xxx_LogStatus("rps", __FILE__, __LINE__, msg);
   xxx_CloseODL (p_in_parms, msg);
   
   exit(SUCCESS);
}

