#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include "xxx_Types.h"
#include "axx_grid.h"
#include "xxx_const.h"
#include "proj.h"
#include "xxx_structures.h"

/******************************************************************************
NAME:           dump_grid

PURPOSE:
To read in the specified HDF grid file and output the grid to the specified
ASCII output file.

USAGE:  dump_grid <grid HDF filename> <ASCII output filename>

RETURN VALUE:   Type = int
Value    Description
-----    -----------
SUCCESS  Successful completion
FAILURE  Operation failed

NOTES:

ALGORITHM REFERENCES:
See axx_grid.h for a description of the grid structure.

******************************************************************************/

int main(int argc, char *argv[])
{
  axx_GRID_TYPE grid;        /* Grid structure                           */
  char grid_filename[STRLEN],/* Grid filename                            */
       output_filename[STRLEN],/* ASCII output filename                  */
       ErrMsg[STRLEN];       /* Error message                            */
  int i, j, k, n;            /* Loop variables                           */
  int refband,               /* Reference position for the current band  */
      sca,		     /* SCA number                               */
      zplane,		     /* Z plane number				 */
      zoffset,		     /* Offset into lookups due to elevation	 */
      nbands;                /* Number of bands in bands array           */
  long count;
  axx_GRID_BAND_TYPE ref;    /* Reference structure of current band      */
  FILE *output;
  BandNumber bands[MAX_BANDS]; /* Array of bands in the grid             */
  int ncoeff;
  char sw[2];
  int flg;

  /* Check the arguments and get the grid filename and output filename from
   * the arguments.
   */
  if (argc != 3 && argc != 4)
  {
    printf("Usage: dump_grid [ switches ] <grid HDF-file> <ASCII output filename>n\n");
    printf("where [ switches ] can consist of:\n");
    printf("\t-m : dump mapping coefficients (large)\n");
    printf("\t   : No switch will not print mapping coefficients\n");
    exit (EXIT_FAILURE);
  }
  else
  {
    if( argc == 4 )
    {
       strcpy(sw,argv[1]);
       if( strcmp(sw,"-m") != 0 )
       {
          sprintf (ErrMsg, "Error opening the output file %s.", output_filename);
          xxx_LogError ("dump_grid", __LINE__, ErrMsg);
          exit (EXIT_FAILURE);
       }
       flg = 1;
       strcpy (grid_filename, argv[2]);
       strcpy (output_filename, argv[3]);
       output = fopen (output_filename, "w");
       if (output == (FILE *) NULL)
       {
          sprintf (ErrMsg, "Error opening the output file %s.", output_filename);
          xxx_LogError ("dump_grid", __LINE__, ErrMsg);
          exit (EXIT_FAILURE);
       }
    }
    else
    {
       flg = 0;
       strcpy (grid_filename, argv[1]);
       strcpy (output_filename, argv[2]);
       output = fopen (output_filename, "w");
       if (output == (FILE *) NULL)
       {
          sprintf (ErrMsg, "Error opening the output file %s.", output_filename);
          xxx_LogError ("dump_grid", __LINE__, ErrMsg);
          exit (EXIT_FAILURE);
       }
    }
  }
  printf ("Grid filename is %s.\n", grid_filename);
  printf ("ASCII output filename is %s.\n", output_filename);

  /* Read in the full grid HDF structure.  Use nbands = 0 to get entire grid.
   */
  nbands = 0;
  if (axx_get_grid (grid_filename, bands, &nbands, &grid) != SUCCESS)
  {
    sprintf (ErrMsg, "Error reading the grid file %s.", grid_filename);
    xxx_LogError ("get_grid", __LINE__, ErrMsg);
    exit (EXIT_FAILURE);
  }

  /* Print out the non-band specific information.
   */
  fprintf (output, "General Grid Information\n");
  fprintf (output, "------------------------\n");

  fprintf (output, "Grid name: %s\n", grid.gridname);

  fprintf (output, "Number of SCAs: %ld\n", grid.nscas);

  fprintf (output, "Number of bands: %ld\n", grid.nbands);
  fprintf (output, "Bands:\n");
  for (i = 0; i < grid.nbands; i++)
    switch (grid.bands[i])
    {
      case pan:
        fprintf (output, "pan "); break;
      case b1p:
        fprintf (output, "b1p "); break;
      case b1:
        fprintf (output, "b1 "); break;
      case b2:
        fprintf (output, "b2 "); break;
      case b3:
        fprintf (output, "b3 "); break;
      case b4:
        fprintf (output, "b4 "); break;
      case b4p:
        fprintf (output, "b4p "); break;
      case b5p:
        fprintf (output, "b5p "); break;
      case b5:
        fprintf (output, "b5 "); break;
      case b7:
        fprintf (output, "b7 "); break;
      default:
        fprintf (output, "Unknown/invalid "); break;
    }
  fprintf (output, "\n");

  fprintf (output, "Data type: ");
  switch (grid.dtype)
  {
    case BYTE: fprintf (output, "BYTE"); break;
    case I2: fprintf (output, "I2"); break;
    case UI2: fprintf (output, "UI2"); break;
    case I4: fprintf (output, "I4"); break;
    case UI4: fprintf (output, "UI4"); break;
    case R4: fprintf (output, "R4"); break;
    case R8: fprintf (output, "R8"); break;
    case NDTYPES: fprintf (output, "Number of different types"); break;
    default: fprintf (output, "Unknown/invalid");
  }
  fprintf (output, "\n");

  fprintf (output, "Correction type: ");
  switch (grid.correction_type)
  {
    case SYSTEMATIC: fprintf (output, "Systematic"); break;
    case PRECISION: fprintf (output, "Precision corrected"); break;
    case TERRAIN: fprintf (output, "Terrain corrected"); break;
    default: fprintf (output, "Unknown/invalid");
  }
  fprintf (output, "\n");

  fprintf (output, "Satellite number: %ld\n", grid.satnum);

  fprintf (output, "Framing method: ");
  switch (grid.frame_type)
  {
    case GEOBOX:
      fprintf (output, "GEOBOX (UL and LR lat/long coordinates)"); break;
    case PROJBOX:
      fprintf (output, "PROJBOX (LR projection coords + line/samp)"); break;
    case UL_SIZE:
      fprintf (output, "UL+SIZE (UL projection coords + output size)"); break;
    case MINBOX:
      fprintf (output, "MINBOX (Minbox framing)"); break;
    case PATH_ORIENTED:
      fprintf (output, "PATH_ORIENTED (Path-oriented framing)"); break;
    default:
      fprintf (output, "Unknown/invalid");
  }
  fprintf (output, "\n\n");

  fprintf (output, "Projection information\n");
  fprintf (output, "----------------------\n");
  fprintf (output, "Projection units: %s\n", grid.units);

  fprintf (output, "Projection code: ");
  switch (grid.code)
  {
    case UTM:
      fprintf (output, "UTM"); break;
    case LAMCC:
      fprintf (output, "LAMCC"); break;
    case PS:
      fprintf (output, "POLAR STEREOGRAPHIC"); break;
    case POLYC:
      fprintf (output, "POLYCONIC"); break;
    case TM:
      fprintf (output, "TRANSVERSE MERCATOR"); break;
    case HOM:
      fprintf (output, "OBLIQUE MERCATOR"); break;
    case SOM:
      fprintf (output, "SOM"); break;
    default:
      fprintf (output, "Unknown/invalid"); break;
  }
  fprintf (output, "\n");

  fprintf (output, "Datum code: %s\n", grid.datum);
  fprintf (output, "Spheroid code: %ld\n", grid.spheroid);
  fprintf (output, "Zone code: %ld\n", grid.zone);

  fprintf (output, "Projection parms:\n");
  for (i = 0; i < COEFCT; i+=5)
    fprintf (output, "  %f  %f  %f  %f  %f\n", grid.projprms[i],
             grid.projprms[i+1], grid.projprms[i+2], grid.projprms[i+3],
             grid.projprms[i+4]);

  fprintf (output, "Projection corners (lat or y, long or x):\n");
  fprintf (output,
    "  UL = (%f, %f)\n", grid.corners.upleft.y, grid.corners.upleft.x);
  fprintf (output,
    "  UR = (%f, %f)\n", grid.corners.upright.y, grid.corners.upright.x);
  fprintf (output,
    "  LL = (%f, %f)\n", grid.corners.loleft.y, grid.corners.loleft.x);
  fprintf (output,
    "  LR = (%f, %f)\n", grid.corners.loright.y, grid.corners.loright.x);
  fprintf (output, "\n");

  fprintf (output, "Path-Oriented Information\n");
  fprintf (output, "-------------------------\n");
  fprintf (output, "WRS scene center lat/long: %f, %f\n", 
    grid.center_pt.lat, grid.center_pt.lng);
  fprintf (output, "Center point (projection): %f, %f\n",
    grid.proj_center_pt.x, grid.proj_center_pt.y);
  fprintf (output, "Rotation angle: %f\n", grid.rotate_angle);
  fprintf (output, "Orientation angle: %f\n", grid.orient_angle);
  fprintf (output, "\n\n\n");

  /* Print out the band-specific information.
   */
  for (i = 0; i < nbands; i++)
  {
    fprintf (output, "*** Grid info for band ");
    switch (bands[i])
    {
      case pan:
        fprintf (output, "pan"); break;
      case b1p:
        fprintf (output, "b1p"); break;
      case b1:
        fprintf (output, "b1"); break;
      case b2:
        fprintf (output, "b2"); break;
      case b3:
        fprintf (output, "b3"); break;
      case b4:
        fprintf (output, "b4"); break;
      case b4p:
        fprintf (output, "b4p"); break;
      case b5p:
        fprintf (output, "b5p"); break;
      case b5:
        fprintf (output, "b5"); break;
      case b7:
        fprintf (output, "b7"); break;
      default:
        fprintf (output, "Unknown/invalid"); break;
    }
    fprintf (output, " ***\n");

    refband = bands[i];
  
    for(sca = 0; sca < NSCA; sca++ )
    {
      ref = grid.gridbands[sca][refband];  /* Get the reference gridbands structure */
      ncoeff = (ref.degree+1) * (ref.degree+1);

      fprintf (output, "Band number: ");
      switch (ref.bandno)
      {
        case pan:
          fprintf (output, "pan"); break;
        case b1p:
          fprintf (output, "b1p"); break;
        case b1:
          fprintf (output, "b1"); break;
        case b2:
          fprintf (output, "b2"); break;
        case b3:
          fprintf (output, "b3"); break;
        case b4:
          fprintf (output, "b4"); break;
        case b4p:
          fprintf (output, "b4p"); break;
        case b5p:
          fprintf (output, "b5p"); break;
        case b5:
          fprintf (output, "b5"); break;
        case b7:
          fprintf (output, "b7"); break;
        default:
          fprintf (output, "Unknown/invalid"); break;
      }
      fprintf (output, "\n");
  
      fprintf(output, "SCA %d\n",sca);
   
      fprintf(output, " Zspacing %f\n",ref.zspacing);
      fprintf(output, " Number of elevation planes present %d\n",ref.nzplanes);
      fprintf(output, " Location of zero elevation %d\n",ref.zeroplane);

      fprintf (output, "Grid cell size: %ld lines x %ld samples\n",
        ref.grid_cell_size.line, ref.grid_cell_size.samp);
      fprintf (output, "Pixel size: %f\n", ref.pixsize);
      fprintf (output, "Number of output lines: %ld\n", ref.lines);
      fprintf (output, "Number of output samples: %ld\n", ref.samps);
      fprintf (output, "Number of grid lines: %ld\n", ref.ngrid_lines);
      fprintf (output, "Number of grid samples: %ld\n", ref.ngrid_samps);
      fprintf (output, "\n");
  
      for( zplane=0 ; zplane<ref.nzplanes ; zplane++ )
      {
        fprintf (output, "Z plane %d\n",zplane);
        fprintf (output, "Grid Cell Points (line, sample)\n");
        fprintf (output, "Input space ---> Output space (deltas line/sample odd/even)\n");
        fprintf(output, "-------------------------------\n");
        zoffset = zplane * ref.ngrid_lines * ref.ngrid_samps;
        for (j = 0, count=0; j < ref.ngrid_lines; j++)
          for (k = 0; k < ref.ngrid_samps; k++, count++)
            fprintf (output, "%4ld %4ld  (%4ld, %4ld) ---> (%f, %f) (%f %f %f %f)\n", 
              j,k,ref.in_lines[j],
              ref.in_samps[k], ref.out_lines[count+zoffset], ref.out_samps[count+zoffset], 
              ref.delta_line_odd[count], ref.delta_samp_odd[count], 
              ref.delta_line_even[count], ref.delta_samp_even[count]);
        fprintf (output, "\n");
    
        fprintf (output, "Degree of rough transformation polynomial: %ld\n\n",
                 ref.degree);
        fprintf (output, "Reverse rough transformation coefficients (lines)\n");
        fprintf (output, "-------------------------------------------------\n");
        for (j = 0; j < (ref.degree+1) * (ref.degree+1); j++)
          fprintf (output, "%f   ", ref.poly_lines[zplane*ncoeff+j]);
        fprintf (output, "\n\n");
    
        fprintf (output, "Reverse rough transformation coefficients (samples)\n");
        fprintf (output, "---------------------------------------------------\n");
        for (j = 0; j < (ref.degree+1) * (ref.degree+1); j++)
          fprintf (output, "%f   ", ref.poly_samps[zplane*ncoeff+j]);
        fprintf (output, "\n\n\n");

        if( flg )
        {
          fprintf (output, "Grid cell mapping coefficients forward,reverse\n");
          fprintf (output, "---------------------------------------------------\n");
          zoffset = zplane * (ref.ngrid_lines-1) * (ref.ngrid_samps-1);
          for (j = 0, count=0; j < (ref.ngrid_lines-1); j++)
            for (k = 0; k < (ref.ngrid_samps-1); k++, count++)
            {
              fprintf(output, "%5ld %4ld %4ld  Line ---> ",count+zoffset,j,k);
              fprintf(output, "(");
              for( n=0 ; n<COEFS_SIZE ; n++ ) fprintf(output, "%4.2e ",ref.sattoproj[count+zoffset].b[n]);
              fprintf(output, ") ");
              fprintf(output, "(");
              for( n=0 ; n<COEFS_SIZE ; n++ ) fprintf(output, "%4.2e ",ref.projtosat[count+zoffset].b[n]);
              fprintf(output, ")\n");
              fprintf(output, "%5ld %4ld %4ld  Sample ---> ",count+zoffset,j,k);
              fprintf(output, "(");
              for( n=0 ; n<COEFS_SIZE ; n++ ) fprintf(output, "%4.2e ",ref.sattoproj[count+zoffset].a[n]);
              fprintf(output, ") ");
              fprintf(output, "(");
              for( n=0 ; n<COEFS_SIZE ; n++ ) fprintf(output, "%4.2e ",ref.projtosat[count+zoffset].a[n]);
              fprintf(output, ")\n");
            }
          fprintf (output, "\n");
        }

      } /* Loop on zplanes */
    } /* Loop on scas */
  } /* Loop on bands */
  
  /* Free the grid.  Use nbands = 0 to free entire grid.
   */
  nbands = 0;
  if (axx_free_grid (bands, &nbands, &grid) != SUCCESS)
  {
    strcpy (ErrMsg, "Error freeing the grid.");
    xxx_LogError ("get_grid", __LINE__, ErrMsg);
    exit (EXIT_FAILURE);
  }

  exit (SUCCESS);
}
