#include <stdio.h>
#include "xxx_Types.h"
#include "xxx_DDR.h"
#include "xxx_LogStatus.h"
#include "xxx_structures.h"
#include "xxx_const.h"

/******************************************************************************
NAME:           dump_ddr

PURPOSE:
To read in the specified HDF DDR file and output the DDR to the specified
ASCII output file.

USAGE:  dump_ddr <DDR HDF filename> <ASCII output filename>

AUTHOR        Change-ID    DATE     Description of Change
------        ---------  --------   ---------------------
Gail Schmidt     1.0     06/25/97   Original development.
Gail Schmidt     1.1     07/08/97   Changed to output usage error to the
                                    screen rather than the error log.
Gail Schmidt     1.2     08/18/97   Added the prints for the radiance values.

RETURN VALUE:   Type = int
Value    Description
-----    -----------
SUCCESS  Successful completion
FAILURE  Operation failed

NOTES:

ALGORITHM REFERENCES:
See xxx_DDR.h for a description of the DDR structure.

******************************************************************************
                Landsat 7 Image Assessment System (IAS) Software
                        Property of the U.S. Government
                                NASA/GSFC/EDC
******************************************************************************/

main (int argc, char *argv[])
{
  char ddr_filename[STRLEN], /* DDR filename                             */
       output_filename[STRLEN],/* ASCII output filename                  */
       ErrMsg[STRLEN];       /* Error message                            */
  xxx_DDRControl_TYPE *DDRcontrol;
                             /* DDR control structure from xxx_ReadDDR.
                                Space for this is malloced in xxx_ReadDDR*/
  xxx_DDR_TYPE ddri;         /* ith DDR structure                        */
  int i, j,                  /* Loop variable                            */
      numddrs;               /* Number of DDRs read from the file        */
  FILE *output;

  /* Check the arguments and get the DDR filename and output filename from
   * the arguments.
   */
  if (argc != 3)
  {
    printf ("Usage: dump_ddr <DDR HDF filename> <ASCII output filename>\n");
    exit (EXIT_FAILURE);
  }
  else
  {
    strcpy (ddr_filename, argv[1]);
    strcpy (output_filename, argv[2]);
    output = fopen (output_filename, "w");
    if (output == (FILE *) NULL)
    {
      sprintf (ErrMsg, "Error opening the output file %s.\n", output_filename);
      xxx_LogStatus ("dump_ddr", __FILE__, __LINE__, ErrMsg);
      exit (EXIT_FAILURE);
    }
  }
  printf ("DDR filename is %s.\n", ddr_filename);
  printf ("ASCII output filename is %s.\n", output_filename);

  /* Read in the full DDR HDF structure.
   */
  DDRcontrol = xxx_ReadDDR (ddr_filename, ErrMsg);
  if (DDRcontrol == NULL)
  {
    xxx_LogStatus ("dump_ddr", __FILE__,  __LINE__, ErrMsg);
    return (ERROR);
  }
  numddrs = DDRcontrol->Count;  /* Get the number of ddrs read. */

  /* Loop through the DDRs and output the information.
   */
  for (i = 0; i < numddrs; i++)
  {
    ddri = DDRcontrol->p_DDR[i];   /* Get the current structure. */

    fprintf (output, "*** Band %d ***\n", ddri.BandNumber);
    fprintf (output, "NumberScansInScene %d.\n", ddri.NumberScansInScene);
    fprintf (output, "NumberdetectorsPerScan %d.\n",
      ddri.NumberdetectorsPerScan);
    fprintf (output, "NumberLinesInScene %d.\n", ddri.NumberLinesInScene);
    fprintf (output, "NumberSamplesInScene %d.\n", ddri.NumberSamplesInScene);
    fprintf (output, "DataType %d.\n", ddri.DataType);
    fprintf (output, "MaxPixelValue %f.\n", ddri.MaxPixelValue);
    fprintf (output, "MinPixelValue %f.\n", ddri.MinPixelValue);
    fprintf (output, "MaxRadianceValue %f.\n", ddri.MaxRadianceValue);
    fprintf (output, "MinRadianceValue %f.\n", ddri.MinRadianceValue);
    fprintf (output, "MasterLine %d.\n", ddri.MasterLine);
    fprintf (output, "MasterSample %d.\n", ddri.MasterSample);
    fprintf (output, "ProjectCode %d.\n", ddri.ProjectCode);
    fprintf (output, "ZoneCode %d.\n", ddri.ZoneCode);
    fprintf (output, "DatumCode %s.\n", ddri.DatumCode);
    fprintf (output, "SpheroidCode %d.\n", ddri.SpheroidCode);
    fprintf (output, "ProjectionUnits %s.\n", ddri.ProjectionUnits);
    fprintf (output, "ProjectionCoef\n");
    for (j = 0; j < XXX_PROJECTION_COEF_SIZE; j++)
      fprintf (output, "    %lf\n", ddri.ProjectionCoef[j]);
    fprintf (output, "UpperLeft (Y, X) (%lf, %lf).\n", ddri.UpperLeft[0],
      ddri.UpperLeft[1]);
    fprintf (output, "LowerLeft (Y, X) (%lf, %lf).\n", ddri.LowerLeft[0],
      ddri.LowerLeft[1]);
    fprintf (output, "UpperRight (Y, X) (%lf, %lf).\n", ddri.UpperRight[0],
      ddri.UpperRight[1]);
    fprintf (output, "LowerRight (Y, X) (%lf, %lf).\n", ddri.LowerRight[0],
      ddri.LowerRight[1]);
    fprintf (output, "ProjectionDistY %lf.\n", ddri.ProjectionDistY);
    fprintf (output, "ProjectionDistX %lf.\n", ddri.ProjectionDistX);
    fprintf (output, "LineSubsampleInc %lf.\n", ddri.LineSubsampleInc);
    fprintf (output, "SampleSubsampleInc %lf.\n", ddri.SampleSubsampleInc);
    fprintf (output, "SpaceCraftSource %s.\n",ddri.SpaceCraftSource);
    fprintf (output, "InstrumentSource %s.\n",ddri.InstrumentSource);
    fprintf (output, "WRSPath %u.\n", ddri.WRSPath);
    fprintf (output, "WRSRows %lf.\n", ddri.WRSRows);
    fprintf (output, "CaptureDirection %s.\n", ddri.CaptureDirection);
    fprintf (output, "CaptureDate %s.\n", ddri.CaptureDate);
    fprintf (output, "CaptureTime %s.\n", ddri.CaptureTime);
    fprintf (output, "FieldValidityFlags\n");
    for (j = 0; j < XXX_VALID_FLAGS; j++)
      fprintf (output, "    %d\n", ddri.FieldValidityFlags[j]);
    switch (ddri.CorrectionType)
    {
      case xxx_Systematic:
        fprintf (output, "CorrectionType %d - Systematic.\n", xxx_Systematic);
        break;
      case xxx_Precision:
        fprintf (output, "CorrectionType %d - Precision.\n", xxx_Precision);
        break;
      case xxx_Terrain:
        fprintf (output, "CorrectionType %d - Terrain.\n", xxx_Terrain);
        break;
    }
    fprintf (output, "\n\n");
  }

  return (EXIT_SUCCESS);
} /* done */

