/*****************************************************************************
NAME: axx_valid_ephem

PURPOSE:
Sort out the valid ephemeris data.

RETURN VALUE:
Type = int
Value    Description
-----    -----------
SUCCESS  Successful completion 
ERROR    Operation failed

NOTES: 

ALGORITHM REFERENCES:
******************************************************************************
                        Property of the U.S. Government
                           USGS EROS Data Center
*******************************************************************************/
#include <math.h>
#include <ctype.h>
#include "xxx_Types.h"
#include "xxx_LogStatus.h"
#include "xxx_math.h"
#include "axx_alimodel.h"	/* EPHEM structure */
#include "exx_init_model.h"

#define ANGMO_TOL       70.0e3  /* angular momentum tolerance
                                   in meters squared per millisecond */
int axx_valid_ephem(
    const ALIPCD *pcd, 		/* I: pcd structure */
    EphemType ephem_type,	/* I: ephem type to use GPSPTS or ACSPTS */
    const CPF *cpf,		/* I: Calibration Parameter File structure */
    int *num_valid,		/* O: the number of valid ephemeris points */
    struct EPHEM *ephem)	/* O: ephemeris struct with valid ephem data */
{
VECTOR craft_pos;		/* Position vector for QC */
VECTOR craft_vel;		/* Velocity vector for QC */
VECTOR momenvec;       		/* Angular momentum vector for qc check */
double momentum;		/* magnitude of angular momentum */
double rawpos[3][NPCD];		/* ephemeris position telemetry data */
double rawvel[3][NPCD];		/* ephemeris velocity telemetry data */
double smpos[3][NPCD];          /* smoothed ephemeris position telemetry data */
double smvel[3][NPCD];          /* smoothed ephemeris velocity telemetry data */
double etime[NPCD];		/* ephemeris time telemetry */
double start_time;		/* ephemeris data start time */
double stop_time;		/* ephemeris data stop time */
int nephem;			/* number of ephemeris points */
int start_index;		/* first telemetry record index to read */
int stop_index;			/* last telemetry record index to read */
int i,j;			/* Loop indices */

double m;

/* are we using ACS or GPS ephemeris ?*/
if ( ephem_type == ACSPTS )	/* ACS (ECI) ephemeris */
{
    /* Issue a message to the log file to document the ephemeris type */
    xxx_LogStatus( "EXXLIB", __FILE__, __LINE__, "Using ACS ephemeris data" );

    /* Get the ephemeris data bounds from the ACS telemetry */
    start_time = pcd->DCE_List[pcd->active_DCE].acs_start_tc;
    stop_time = pcd->DCE_List[pcd->active_DCE].acs_stop_tc;
    start_index = xxx_find_index( start_time, pcd->ACS_Group->sc_posx );
    stop_index = xxx_find_index( stop_time, pcd->ACS_Group->sc_posx );
    /* Get the actual telemetry times */
    start_time = pcd->ACS_Group->sc_posx.time[start_index];
    stop_time = pcd->ACS_Group->sc_posx.time[stop_index];
    /* Convert the times to GPS time */
    start_time = exx_timecodetogps( start_time );
    stop_time = exx_timecodetogps( stop_time );

    /* Make sure we don't overflow the model array */
    if ( stop_index-start_index+1 > NPCD )
    {
      xxx_LogStatus( "EXXLIB", __FILE__, __LINE__,
                     "Too much ephemeris data found. Limiting length." );
      stop_index = start_index + NPCD - 1;
      stop_time = pcd->ACS_Group->sc_posx.time[stop_index];
      stop_time = exx_timecodetogps( stop_time );
    }

    /* Load the local data arrays */
    for ( nephem=0, j=start_index; j<=stop_index; j++ )
    {
        etime[nephem] = exx_timecodetogps( pcd->ACS_Group->sc_posx.time[j] )
                      - start_time;

        /* Store the ephemeris points converting positions from km to m */
        rawpos[0][nephem] = pcd->ACS_Group->sc_posx.value.d[j]*1000.0;
        rawpos[1][nephem] = pcd->ACS_Group->sc_posy.value.d[j]*1000.0;
        rawpos[2][nephem] = pcd->ACS_Group->sc_posz.value.d[j]*1000.0;
        rawvel[0][nephem] = pcd->ACS_Group->sc_velx.value.d[j];
        rawvel[1][nephem] = pcd->ACS_Group->sc_vely.value.d[j];
        rawvel[2][nephem] = pcd->ACS_Group->sc_velz.value.d[j];

        /* Perform angular momentum check */
        craft_pos.x = rawpos[0][nephem];
        craft_pos.y = rawpos[1][nephem];
        craft_pos.z = rawpos[2][nephem];
        craft_vel.x = rawvel[0][nephem];
        craft_vel.y = rawvel[1][nephem];
        craft_vel.z = rawvel[2][nephem];
        xxx_cross( &craft_pos, &craft_vel, &momenvec );
        momentum = xxx_norm( &momenvec );

        /*****************************************************************
        if( j==start_index )
        {
            printf("Angular momentum %f\n",momentum);
            m = xxx_norm( &craft_pos );
            printf("craft_pos = %f %f %f (%f)\n",craft_pos.x,craft_pos.y,craft_pos.z,m);
            printf("            %f %f\n",pcd->ACS_Group->sc_lat.value.d[j],
                   pcd->ACS_Group->sc_lon.value.d[j]);
            m = xxx_norm( &craft_vel );
            printf("craft_vel = %f %f %f (%f)\n",craft_vel.x,craft_vel.y,craft_vel.z,m);
            printf("ERRORS POS = %f %f %f\n",pcd->ACS_Group->imu_xposerr.value.d[j],
                   pcd->ACS_Group->imu_yposerr.value.d[j], pcd->ACS_Group->imu_zposerr.value.d[j]);
            printf("ERRORS VEL = %f %f %f\n",pcd->ACS_Group->imu_xraterr.value.d[j],
                   pcd->ACS_Group->imu_yraterr.value.d[j], pcd->ACS_Group->imu_zraterr.value.d[j]);
        }
        *****************************************************************/

        /* change angular momentum to meters squared per second */
        if (fabs(momentum-cpf->Orbit_Parameters.Angular_Momentum/1000.0)
            <= ANGMO_TOL)
        {
            nephem++;
        }
    }
}
else				/* GPS (ECF) ephemeris */
{
    /* Issue a message to the log file to document the ephemeris type */
    xxx_LogStatus( "EXXLIB", __FILE__, __LINE__, "Using GPS ephemeris data" );

    /* Get the ephemeris data bounds from the GPS telemetry */
    start_index = pcd->DCE_List[pcd->active_DCE].gps_start_index;
    stop_index = pcd->DCE_List[pcd->active_DCE].gps_stop_index;
    /* Get the actual telemetry times */
    start_time = pcd->GPS_Group->gps_posx.time[start_index];
    stop_time = pcd->GPS_Group->gps_posx.time[stop_index];
    /* Convert the times to GPS time */
    start_time = exx_timecodetogps( start_time );
    stop_time = exx_timecodetogps( stop_time );

    /* Make sure we don't overflow the model array */
    if ( stop_index-start_index+1 > NPCD )
    {
      xxx_LogStatus( "EXXLIB", __FILE__, __LINE__,
                     "Too much ephemeris data found. Limiting length." );
      stop_index = start_index + NPCD - 1;
      stop_time = pcd->GPS_Group->gps_posx.time[stop_index];
      stop_time = exx_timecodetogps( stop_time );
    }

    /* Load the local data arrays */
    for ( nephem=0, j=start_index; j<=stop_index; j++ )
    {
        etime[nephem] = exx_timecodetogps( pcd->GPS_Group->gps_posx.time[j] )
                      - start_time;

        /* Store the ephemeris points converting positions from km to m */
        rawpos[0][nephem] = pcd->GPS_Group->gps_posx.value.d[j]*1000.0;
        rawpos[1][nephem] = pcd->GPS_Group->gps_posy.value.d[j]*1000.0;
        rawpos[2][nephem] = pcd->GPS_Group->gps_posz.value.d[j]*1000.0;
        rawvel[0][nephem] = pcd->GPS_Group->gps_velx.value.d[j];
        rawvel[1][nephem] = pcd->GPS_Group->gps_vely.value.d[j];
        rawvel[2][nephem] = pcd->GPS_Group->gps_velz.value.d[j];

        /* Perform angular momentum check */
        craft_pos.x = rawpos[0][nephem];
        craft_pos.y = rawpos[1][nephem];
        craft_pos.z = rawpos[2][nephem];
        craft_vel.x = rawvel[0][nephem];
        craft_vel.y = rawvel[1][nephem];
        craft_vel.z = rawvel[2][nephem];
        xxx_cross( &craft_pos, &craft_vel, &momenvec );
        momentum = xxx_norm( &momenvec );

        /*****************************************************************
        if( j==start_index )
        {
            printf("Angular momentum %f\n",momentum);
            m = xxx_norm( &craft_pos );
            printf("craft_pos = %f %f %f (%f)\n",craft_pos.x,craft_pos.y,craft_pos.z,m);
            printf("craft_vel = %f %f %f (%f)\n",craft_vel.x,craft_vel.y,craft_vel.z,m);
            printf("ERRORS POS = %f %f %f\n",pcd->ACS_Group->imu_xposerr.value.d[j],
                   pcd->ACS_Group->imu_yposerr.value.d[j], pcd->ACS_Group->imu_zposerr.value.d[j]);
            printf("ERRORS VEL = %f %f %f\n",pcd->ACS_Group->imu_xraterr.value.d[j],
                   pcd->ACS_Group->imu_yraterr.value.d[j], pcd->ACS_Group->imu_zraterr.value.d[j]);
        }
        *****************************************************************/

        /* change angular momentum to meters squared per second */
        if (fabs(momentum-cpf->Orbit_Parameters.Angular_Momentum/1000.0)
            <= ANGMO_TOL)
        {
            nephem++;
        }
    }
}

/*
 * Smooth GPS data to remove small random errors present in the data.
 */
if( exx_kalman_smooth_gps( rawpos, rawvel, nephem, etime, cpf, smpos, smvel ) != SUCCESS )
{
  xxx_LogStatus( "EXXLIB", __FILE__, __LINE__, "Error smoothing GPS data." );
  return(ERROR);
}
for( j=0 ; j < nephem; j++ )
{
   rawpos[0][j] = smpos[0][j];
   rawpos[1][j] = smpos[1][j];
   rawpos[2][j] = smpos[2][j];
   rawvel[0][j] = smvel[0][j];
   rawvel[1][j] = smvel[1][j];
   rawvel[2][j] = smvel[2][j];
}

/* See if there was sufficient valid ephemeris */
if ( nephem < LAGRANGE_PTS )
{
  xxx_LogStatus( "EXXLIB", __FILE__, __LINE__,
                 "Insufficient valid ephemeris data found." );
  return(ERROR);
}

/* Load the ephem structures */
*num_valid = (int)floor((stop_time-start_time)/EPHEM_TIME) + 1;
for ( j=0; j<(*num_valid); j++ )
{
    /* Load time fields */
    stop_time = (double)j * EPHEM_TIME;
    exx_gpstoyds( start_time+stop_time, ephem[j].time );

    /* Find Lagrange interpolation start index */
    i = 0;
    while ( (etime[i] <= stop_time) && (i < nephem) ) i++;
    i -= LAGRANGE_PTS/2;
    if ( i<0 ) i=0;
    if ( i>(nephem-LAGRANGE_PTS) ) i = nephem - LAGRANGE_PTS;
    ephem[j].craft_pos.x = xxx_Lagrange( &rawpos[0][i], &etime[i],
                                         LAGRANGE_PTS, stop_time );
    ephem[j].craft_pos.y = xxx_Lagrange( &rawpos[1][i], &etime[i],
                                         LAGRANGE_PTS, stop_time );
    ephem[j].craft_pos.z = xxx_Lagrange( &rawpos[2][i], &etime[i],
                                         LAGRANGE_PTS, stop_time );
    ephem[j].craft_vel.x = xxx_Lagrange( &rawvel[0][i], &etime[i],
                                         LAGRANGE_PTS, stop_time );
    ephem[j].craft_vel.y = xxx_Lagrange( &rawvel[1][i], &etime[i],
                                         LAGRANGE_PTS, stop_time );
    ephem[j].craft_vel.z = xxx_Lagrange( &rawvel[2][i], &etime[i],
                                         LAGRANGE_PTS, stop_time );
}

/* return that the routine was successful */
return (SUCCESS);
}
