#include "foot_print.h"

#define ANGMO_TOL       50.0e3  /* angular momentum tolerance
                                   in meters squared per millisecond */

int exx_foot_print_ephem(
    const ALIPCD *pcd, 		/* I: pcd structure */
    EphemType ephem_type,	/* I: ephem type to use GPSPTS or ACSPTS */
    const CPF *cpf,		/* I: Calibration Parameter File structure */
    int *num_valid,		/* O: the number of valid ephemeris points */
    struct EPHEM *ephem)	/* O: ephemeris struct with valid ephem data */
{
VECTOR craft_pos;		/* Position vector for QC */
VECTOR craft_vel;		/* Velocity vector for QC */
VECTOR momenvec;       		/* Angular momentum vector for qc check */
double momentum;		/* magnitude of angular momentum */
double etime[NPCD];		/* ephemeris time telemetry */
double start_time;		/* ephemeris data start time */
double stop_time;		/* ephemeris data stop time */
int nephem;			/* number of ephemeris points */
int start_index;		/* first telemetry record index to read */
int stop_index;			/* last telemetry record index to read */
int i,j;			/* Loop indices */

*num_valid = 2;

/* are we using ACS or GPS ephemeris ?*/
if ( ephem_type == ACSPTS )	/* ACS (ECI) ephemeris */
{
    /* Issue a message to the log file to document the ephemeris type */
    xxx_LogStatus( "EXXLIB", __FILE__, __LINE__, "Using ACS ephemeris data" );

    /* Get the ephemeris data bounds from the ACS telemetry */
    start_time = pcd->DCE_List[pcd->active_DCE].acs_start_tc;
    stop_time = pcd->DCE_List[pcd->active_DCE].acs_stop_tc;
    start_index = xxx_find_index( start_time, pcd->ACS_Group->sc_posx );
    stop_index = xxx_find_index( stop_time, pcd->ACS_Group->sc_posx );
    /* Get the actual telemetry times */
    start_time = pcd->ACS_Group->sc_posx.time[start_index];
    stop_time = pcd->ACS_Group->sc_posx.time[stop_index];
    /* Convert the times to GPS time */
    start_time = exx_timecodetogps( start_time );
    stop_time = exx_timecodetogps( stop_time );

    /* Make sure we don't overflow the model array */
    if ( stop_index-start_index+1 > NPCD )
    {
      xxx_LogStatus( "EXXLIB", __FILE__, __LINE__,
                     "Too much ephemeris data found. Limiting length." );
      stop_index = start_index + NPCD - 1;
      stop_time = pcd->ACS_Group->sc_posx.time[stop_index];
      stop_time = exx_timecodetogps( stop_time );
    }

    etime[0] = exx_timecodetogps( pcd->ACS_Group->sc_posx.time[start_index] )
               - start_time;

    /* Load time fields */
    exx_gpstoyds( start_time, ephem[0].time );

    /* Store the ephemeris points converting positions from km to m */
    ephem[0].craft_pos.x = pcd->ACS_Group->sc_posx.value.d[start_index]*1000.0;
    ephem[0].craft_pos.y = pcd->ACS_Group->sc_posy.value.d[start_index]*1000.0;
    ephem[0].craft_pos.z = pcd->ACS_Group->sc_posz.value.d[start_index]*1000.0;
    ephem[0].craft_vel.x = pcd->ACS_Group->sc_velx.value.d[start_index];
    ephem[0].craft_vel.y = pcd->ACS_Group->sc_vely.value.d[start_index];
    ephem[0].craft_vel.z = pcd->ACS_Group->sc_velz.value.d[start_index];

    /* Perform angular momentum check */
    craft_pos.x = ephem[0].craft_pos.x;
    craft_pos.y = ephem[0].craft_pos.y;
    craft_pos.z = ephem[0].craft_pos.z;
    craft_vel.x = ephem[0].craft_vel.x;
    craft_vel.y = ephem[0].craft_vel.y;
    craft_vel.z = ephem[0].craft_vel.z;
    xxx_cross( &craft_pos, &craft_vel, &momenvec );
    momentum = xxx_norm( &momenvec );

    /* change angular momentum to meters squared per second */
    if (fabs(momentum-cpf->Orbit_Parameters.Angular_Momentum/1000.0) >= ANGMO_TOL)
    {
        printf("Angular momentum %f\n",momentum);
        printf("POS %f %f %f\n",craft_pos.x,craft_pos.y,craft_pos.z);
        printf("VEL %f %f %f\n",craft_vel.x,craft_vel.y,craft_vel.z);
        xxx_LogStatus( "EXXLIB", __FILE__, __LINE__, "Angular momentum ephemeris error." );
        /* return(ERROR); */
    }

    etime[1] = exx_timecodetogps( pcd->ACS_Group->sc_posx.time[stop_index] )
               - start_time;

    /* Load time fields */
    exx_gpstoyds( stop_time, ephem[1].time );

    /* Store the ephemeris points converting positions from km to m */
    ephem[1].craft_pos.x = pcd->ACS_Group->sc_posx.value.d[stop_index]*1000.0;
    ephem[1].craft_pos.y = pcd->ACS_Group->sc_posy.value.d[stop_index]*1000.0;
    ephem[1].craft_pos.z = pcd->ACS_Group->sc_posz.value.d[stop_index]*1000.0;
    ephem[1].craft_vel.x = pcd->ACS_Group->sc_velx.value.d[stop_index];
    ephem[1].craft_vel.y = pcd->ACS_Group->sc_vely.value.d[stop_index];
    ephem[1].craft_vel.z = pcd->ACS_Group->sc_velz.value.d[stop_index];

    /* Perform angular momentum check */
    craft_pos.x = ephem[1].craft_pos.x;
    craft_pos.y = ephem[1].craft_pos.y;
    craft_pos.z = ephem[1].craft_pos.z;
    craft_vel.x = ephem[1].craft_vel.x;
    craft_vel.y = ephem[1].craft_vel.y;
    craft_vel.z = ephem[1].craft_vel.z;
    xxx_cross( &craft_pos, &craft_vel, &momenvec );
    momentum = xxx_norm( &momenvec );

    /* change angular momentum to meters squared per second */
    if (fabs(momentum-cpf->Orbit_Parameters.Angular_Momentum/1000.0) >= ANGMO_TOL)
    {
        printf("Angular momentum %f\n",momentum);
        printf("POS %f %f %f\n",craft_pos.x,craft_pos.y,craft_pos.z);
        printf("VEL %f %f %f\n",craft_vel.x,craft_vel.y,craft_vel.z);
        xxx_LogStatus( "EXXLIB", __FILE__, __LINE__, "Angular momentum ephemeris error." );
        /* return(ERROR); */
    }
}
else				/* GPS (ECF) ephemeris */
{
    /* Issue a message to the log file to document the ephemeris type */
    xxx_LogStatus( "EXXLIB", __FILE__, __LINE__, "Using GPS ephemeris data" );

    /* Get the ephemeris data bounds from the GPS telemetry */
    start_index = pcd->DCE_List[pcd->active_DCE].gps_start_index;
    stop_index = pcd->DCE_List[pcd->active_DCE].gps_stop_index;
    /* Get the actual telemetry times */
    start_time = pcd->GPS_Group->gps_posx.time[start_index];
    stop_time = pcd->GPS_Group->gps_posx.time[stop_index];
    /* Convert the times to GPS time */
    start_time = exx_timecodetogps( start_time );
    stop_time = exx_timecodetogps( stop_time );

    /* Make sure we don't overflow the model array */
    if ( stop_index-start_index+1 > NPCD )
    {
      xxx_LogStatus( "EXXLIB", __FILE__, __LINE__,
                     "Too much ephemeris data found. Limiting length." );
      stop_index = start_index + NPCD - 1;
      stop_time = pcd->GPS_Group->gps_posx.time[stop_index];
      stop_time = exx_timecodetogps( stop_time );
    }

    etime[0] = exx_timecodetogps( pcd->GPS_Group->gps_posx.time[start_index] )
                  - start_time;

    /* Load time fields */
    exx_gpstoyds( start_time, ephem[0].time );

    /* Store the ephemeris points converting positions from km to m */
    ephem[0].craft_pos.x = pcd->GPS_Group->gps_posx.value.d[start_index]*1000.0;
    ephem[0].craft_pos.y = pcd->GPS_Group->gps_posy.value.d[start_index]*1000.0;
    ephem[0].craft_pos.z = pcd->GPS_Group->gps_posz.value.d[start_index]*1000.0;
    ephem[0].craft_vel.x = pcd->GPS_Group->gps_velx.value.d[start_index];
    ephem[0].craft_vel.y = pcd->GPS_Group->gps_vely.value.d[start_index];
    ephem[0].craft_vel.z = pcd->GPS_Group->gps_velz.value.d[start_index];

    /* Perform angular momentum check */
    craft_pos.x =  ephem[0].craft_pos.x;
    craft_pos.y =  ephem[0].craft_pos.y;
    craft_pos.z =  ephem[0].craft_pos.z;
    craft_vel.x =  ephem[0].craft_vel.x;
    craft_vel.y =  ephem[0].craft_vel.y;
    craft_vel.z =  ephem[0].craft_vel.z;
    xxx_cross( &craft_pos, &craft_vel, &momenvec );
    momentum = xxx_norm( &momenvec );

    /* change angular momentum to meters squared per second */
    if (fabs(momentum-cpf->Orbit_Parameters.Angular_Momentum/1000.0) >= ANGMO_TOL)
    {
        printf("Angular momentum %f\n",momentum);
        printf("POS %f %f %f\n",craft_pos.x,craft_pos.y,craft_pos.z);
        printf("VEL %f %f %f\n",craft_vel.x,craft_vel.y,craft_vel.z);
        xxx_LogStatus( "EXXLIB", __FILE__, __LINE__, "Angular momentum ephemeris error." );
        /* return(ERROR); */
    }

    etime[1] = exx_timecodetogps( pcd->GPS_Group->gps_posx.time[stop_index] )
               - start_time;

    /* Load time fields */
    exx_gpstoyds( stop_time, ephem[1].time );

    /* Store the ephemeris points converting positions from km to m */
    ephem[1].craft_pos.x = pcd->GPS_Group->gps_posx.value.d[stop_index]*1000.0;
    ephem[1].craft_pos.y = pcd->GPS_Group->gps_posy.value.d[stop_index]*1000.0;
    ephem[1].craft_pos.z = pcd->GPS_Group->gps_posz.value.d[stop_index]*1000.0;
    ephem[1].craft_vel.x = pcd->GPS_Group->gps_velx.value.d[stop_index];
    ephem[1].craft_vel.y = pcd->GPS_Group->gps_vely.value.d[stop_index];
    ephem[1].craft_vel.z = pcd->GPS_Group->gps_velz.value.d[stop_index];

    /* Perform angular momentum check */
    craft_pos.x =  ephem[1].craft_pos.x;
    craft_pos.y =  ephem[1].craft_pos.y;
    craft_pos.z =  ephem[1].craft_pos.z;
    craft_vel.x =  ephem[1].craft_vel.x;
    craft_vel.y =  ephem[1].craft_vel.y;
    craft_vel.z =  ephem[1].craft_vel.z;
    xxx_cross( &craft_pos, &craft_vel, &momenvec );
    momentum = xxx_norm( &momenvec );

    /* change angular momentum to meters squared per second */
    if (fabs(momentum-cpf->Orbit_Parameters.Angular_Momentum/1000.0) >= ANGMO_TOL)
    {
        printf("Angular momentum %f\n",momentum);
        printf("POS %f %f %f\n",craft_pos.x,craft_pos.y,craft_pos.z);
        printf("VEL %f %f %f\n",craft_vel.x,craft_vel.y,craft_vel.z);
        xxx_LogStatus( "EXXLIB", __FILE__, __LINE__, "Angular momentum ephemeris error." );
        /* return(ERROR); */
    }
}

/* return that the routine was successful */
return (SUCCESS);
}
