#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include "xxx_Types.h"
#include "xxx_LogStatus.h"
#include "xxx_const.h"
#include "xxx_math.h"
#include "exx_geometric.h"
char *substr(char *src, int start, int len);
void gxx_sphsunpostod ( double jd, double *ra, double *dec, double *dis);
main(int argc, char **argv)
	{
	int year, month, day;
	int doy;	/* day of year */
	double seconds;
	double centerlat, centerlon;
	double solarelev, solaraz;
	int status;
	
	if (argc != 4)
		{
		fprintf(stderr, "Usage: %s time latitude longitued\n", argv[0]);
		exit(1);
		}
	/*
	 *				   012345678901234567890
	 * The first argument is a time in 2000:336:15:42:04.331 format
	 */
	year = atoi(substr(argv[1], 0, 4));
	doy = atoi(substr(argv[1], 5, 3));
	cvtdoy(year, doy, &month, &day);
	seconds = (double)(3600 * atoi(substr(argv[1], 9, 2))) +
		(double)(60 * atoi(substr(argv[1], 12, 2))) +
		atof(substr(argv[1], 15, 6));
	centerlat = atof(argv[2]);
	centerlon = atof(argv[3]);
	if (centerlon < 0)
		centerlon += 360.0;
	if (sunangles(year, month, day, seconds, centerlat * D2R, centerlon * D2R, &solarelev, &solaraz) == SUCCESS)
		printf("%f %f\n", solarelev * R2D, solaraz * R2D);
	else
		exit(1);
	exit(0);
	}
/*
 * substring routine, returns space allocated with malloc
 */
char *substr(char *src, int start, int len)
	{
	char *buf = (char *)malloc(len+1);
	if (buf)
		{
		strncpy(buf, src+start, len);
		buf[len] = '\0';
		}
	return buf;
	}
/*
 * Convert year, day-of-year to year, month, day
 */
int cvtdoy(int year, int doy, int *month, int *day)
	{
	static months[12] = {31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31};
	int m;
	if (((year % 4) != 0) || (((year % 100) == 0) && ((year % 400) != 0)))
		months[1] = 28;
	else
		months[1] = 29;
	for (m=0; m<12; m++)
		if (doy <= months[m])
			{
			*month = m+1;
			*day = doy;
			return 1;
			}
		else
			doy -= months[m];
	return 0;
	}

/****************************************************************************
NAME:                           sunangles.c

PURPOSE: Given the year, month, day, second, and the scene center's
         Latitude and Longitude. Calculate the solar elevation and azimuth
         angles.

DEVELOPMENT HISTORY:

RETURN VALUE:
Type = int
Value           Description
-----           -----------
SUCCESS         Successfully calculated sun angles.


****************************************************************************/

#define HOURS2DEG	15.0	/* hours to degrees conversion */

int sunangles
(
   int  year,		/* I: Year (i.e. 1997) */
   int  month, 		/* I: Month in year */
   int  day,   		/* I: Day of Month */
   double seconds,   	/* I: Total seconds into day for scene center    */
   double centerlat, 	/* I: Scene center geodetic latitude (radians)  */
   double centerlon, 	/* I: Scene center longitude (radians 0 -> 2PI) */
   double *solarelev, 	/* O: Solar elevation angle (radians) */
   double *solaraz    	/* O: Solar azimuth angle (radians) */
)
{
   double tjd;		/* full julian day */
   double agha;		/* Greenwich Hour Angle of Aries */
   double sgha;		/* solar Greenwich Hour Angle */
   double ra;		/* right ascension referred to mean equator and
                           equinox of date (hours) */
   double dec;		/* declination referred to mean equator and 
			   equinox of date (radians) */
   double dis;		/* geocentric distance (AU) */
   double sunlon;	/* suns projected longitude */
   double solarzen;	/* solar zenith angle */
   double deltalon;	/* diff between the suns projected longitude and
                           and the longitude at the center of the scene */
   double sunx;		/* solar vector projection on local X axis */
   double suny;		/* solar vector projection on local Y axis */

/*----------------------------------------------------------------
         See if a valid position was input
----------------------------------------------------------------*/

	if ( (centerlat > PIE2) || (centerlat < -PIE2) ||
             (centerlon > PIE2) || (centerlon < -PIE2) )
	{
	  *solaraz = -999.0;
          *solarelev = -999.0;
	  return(SUCCESS);
	}

/*----------------------------------------------------------------
	 Find the Full Julian day for the scene center 
----------------------------------------------------------------*/

	tjd = xxx_fulljd(year, month, day, seconds);

/*----------------------------------------------------------------
 	Find the Sun's spherical position in inertial (TOD) space 
----------------------------------------------------------------*/

	gxx_sphsunpostod (tjd, &ra, &dec, &dis);

/*----------------------------------------------------------------
 	Find the Greenwich Hour Angle (GHA) of Aries  
----------------------------------------------------------------*/

	exx_ariesgha(tjd, seconds, &agha);

/*----------------------------------------------------------------
 	Find the GHA of the Sun 
----------------------------------------------------------------*/

	sgha = agha - ra * HOURS2DEG * D2R;
	if (sgha >= PIE2) sgha = sgha - PIE2;
	if (sgha < 0.0) sgha = PIE2 + sgha;

/*----------------------------------------------------------------
 	Find the Sun's projected Longitude 
----------------------------------------------------------------*/

	sunlon = PIE2 - sgha;

/*----------------------------------------------------------------
        Calculate results using simple spherical relationships
 	Assume the vector from the Earth's center to the Sun is parallel
        to the vector from the scene's center to the Sun 

 	Solar elevation calculation  
----------------------------------------------------------------*/

	deltalon = sunlon - centerlon;

	solarzen = acos(sin(dec) * sin(centerlat) + 
	     cos(dec) * cos(centerlat) * cos(deltalon));

	*solarelev = HALFPIE - solarzen;

/*----------------------------------------------------------------
 	Solar azimuth calculation 
 	Define North as Zero degrees in azimuth and East as 90 degrees 
----------------------------------------------------------------*/
	sunx = cos(dec) * sin(deltalon);
	suny = sin(dec)*cos(centerlat) - cos(dec)*cos(deltalon)*sin(centerlat);
	if ( fabs( sunx ) > 0.0 || fabs( suny ) > 0.0 )
	  *solaraz = atan2( sunx, suny );
	else
	  *solaraz = 0.0;

	return(SUCCESS);
}

/***************************************************************************
NAME:                           gxx_sphsunpostod.c

PURPOSE:
      To compute equatorial spherical coordinates of Sun referred to
      the mean equator and equinox of date.

DEVELOPMENT HISTORY:
$Log: gxx_sphsunpostod.c,v $
Revision 1.2  2000/11/22 22:14:22  storey
Replaced by exx_transmat.c

Revision 1.1.1.1  1999/12/01 19:01:20  mbs
Import of IAS 3.22

 * 
 *    Rev 1.2   Mon Aug 11 12:56:28 1997   tgardner
 * added references to Navel Observatory s/w
 * package as source of this routine
 * removed return statement

RETURN VALUE: none


 REFERENCES:
      Bretagnon, P. and Simon, J.L. (1986).  Planetary Programs and
         Tables from -4000 to + 2800. (Richmond, VA: Willmann-Bell).

      Code is from Navel Observatory Vector Astronomy Subroutines (NOVAS)
      C Version 1.0
      June 1996

      U. S. Navel Observatory
      Astronomical Applications Dept.
      3450 Massachusetts Ave., NW
      Washington, DC  20392-5420

   NOTES:
      1. Quoted accuracy is 2.0 + 0.03 * T^2 arcsec, where T is
      measured in units of 1000 years from J2000.0.  See reference.

****************************************************************************
                Landsat 7 Image Assessment System (IAS) Software
                        Property of the U.S. Government
                                NASA/GSFC/EDC
*****************************************************************************/
#include <math.h>
/* #include "gxx_algorithm.h" */

void gxx_sphsunpostod 
(
	double jd,		/* I: Julian date on TDT or ET time scale */
	double *ra,		/* O: Right ascension referred to mean equator 
					and equinox of date (hours) */
	double *dec,		/* O: Declination referred to mean equator and
					equinox of date (radians) */
	double *dis		/* O: Geocentric distance (AU) */
)
{
   short int i;

   double sum_lon = 0.0;
   double sum_r = 0.0;
   const double factor = 1.0e-07;
   double u, arg, lon, t, t2, emean, sin_lon;

   struct sun_con
   {
   	double l;
   	double r;
   	double alpha;
   	double nu;
   };

   static const struct sun_con con[50] =
      {{403406.0,      0.0, 4.721964,     1.621043},
       {195207.0, -97597.0, 5.937458, 62830.348067}, 
       {119433.0, -59715.0, 1.115589, 62830.821524}, 
       {112392.0, -56188.0, 5.781616, 62829.634302}, 
       {  3891.0,  -1556.0, 5.5474  , 125660.5691 }, 
       {  2819.0,  -1126.0, 1.5120  , 125660.9845 }, 
       {  1721.0,   -861.0, 4.1897  ,  62832.4766 }, 
       {     0.0,    941.0, 1.163   ,      0.813  }, 
       {   660.0,   -264.0, 5.415   , 125659.310  }, 
       {   350.0,   -163.0, 4.315   ,  57533.850  }, 
       {   334.0,      0.0, 4.553   ,    -33.931  }, 
       {   314.0,    309.0, 5.198   , 777137.715  }, 
       {   268.0,   -158.0, 5.989   ,  78604.191  }, 
       {   242.0,      0.0, 2.911   ,      5.412  }, 
       {   234.0,    -54.0, 1.423   ,  39302.098  }, 
       {   158.0,      0.0, 0.061   ,    -34.861  }, 
       {   132.0,    -93.0, 2.317   , 115067.698  }, 
       {   129.0,    -20.0, 3.193   ,  15774.337  }, 
       {   114.0,      0.0, 2.828   ,   5296.670  }, 
       {    99.0,    -47.0, 0.52    ,  58849.27   }, 
       {    93.0,      0.0, 4.65    ,   5296.11   }, 
       {    86.0,      0.0, 4.35    ,  -3980.70   }, 
       {    78.0,    -33.0, 2.75    ,  52237.69   }, 
       {    72.0,    -32.0, 4.50    ,  55076.47   }, 
       {    68.0,      0.0, 3.23    ,    261.08   }, 
       {    64.0,    -10.0, 1.22    ,  15773.85   }, 
       {    46.0,    -16.0, 0.14    ,  188491.03  }, 
       {    38.0,      0.0, 3.44    ,   -7756.55  }, 
       {    37.0,      0.0, 4.37    ,     264.89  }, 
       {    32.0,    -24.0, 1.14    ,  117906.27  }, 
       {    29.0,    -13.0, 2.84    ,   55075.75  }, 
       {    28.0,      0.0, 5.96    ,   -7961.39  }, 
       {    27.0,     -9.0, 5.09    ,  188489.81  }, 
       {    27.0,      0.0, 1.72    ,    2132.19  }, 
       {    25.0,    -17.0, 2.56    ,  109771.03  }, 
       {    24.0,    -11.0, 1.92    ,   54868.56  }, 
       {    21.0,      0.0, 0.09    ,   25443.93  }, 
       {    21.0,     31.0, 5.98    ,  -55731.43  }, 
       {    20.0,    -10.0, 4.03    ,   60697.74  }, 
       {    18.0,      0.0, 4.27    ,    2132.79  }, 
       {    17.0,    -12.0, 0.79    ,  109771.63  }, 
       {    14.0,      0.0, 4.24    ,   -7752.82  }, 
       {    13.0,     -5.0, 2.01    ,  188491.91  }, 
       {    13.0,      0.0, 2.65    ,     207.81  }, 
       {    13.0,      0.0, 4.98    ,   29424.63  }, 
       {    12.0,      0.0, 0.93    ,      -7.99  }, 
       {    10.0,      0.0, 2.21    ,   46941.14  }, 
       {    10.0,      0.0, 3.59    ,     -68.29  }, 
       {    10.0,      0.0, 1.50    ,   21463.25  }, 
       {    10.0,     -9.0, 2.55    ,  157208.40  }};

/*------------------------------------------------------------------------------
  Define the time unit 'u', measured in units of 10000 Julian years from J2000.0. 
 -----------------------------------------------------------------------------*/ 

   u = (jd - EPOCH_2000) / 3652500.0;
   
/*------------------------------------------------------------------------------
 Compute longitude and distance terms from the series. 
 -----------------------------------------------------------------------------*/ 

   for (i = 0; i < 50; i++)
   {
      arg = con[i].alpha + con[i].nu * u;
      sum_lon += con[i].l * sin (arg);
      sum_r += con[i].r * cos (arg);
   }

/*-----------------------------------------------------------------------------
 Compute longitude and distance referred to mean equinox
   and ecliptic of date. 
 -----------------------------------------------------------------------------*/ 

   lon = 4.9353929 + 62833.1961680 * u + factor * sum_lon;

   lon = fmod (lon, PIE2);
   if (lon < 0.0)
      lon += PIE2;

   *dis = 1.0001026 + factor * sum_r;

/*-----------------------------------------------------------------------------
 Compute mean obliquity of the ecliptic. 
 -----------------------------------------------------------------------------*/ 

   t = u * 100.0;
   t2 = t * t;
   emean = (0.001813 * t2 * t - 0.00059 * t2 - 46.8150 * t + 84381.448) / SECS2RADS;

/*-----------------------------------------------------------------------------
 Compute equatorial spherical coordinates referred to the mean equator 
   and equinox of date. 
 -----------------------------------------------------------------------------*/ 

   sin_lon = sin (lon);
   *ra = atan2 ((cos (emean) * sin_lon), cos (lon));
   *ra = fmod (*ra, PIE2);
   if (*ra < 0.0)
      *ra += PIE2;
   *ra = *ra * R2D / 15.0;

   *dec = asin (sin (emean) * sin_lon);
}
