/*
 *  File:  cp_align.c       
 *
 *  Description:  Routines for doing band-to-band registration and
 *                adding fill data to align SCAs
 *
 *  Subroutines:  
 *
 *  Author:	  Jenifer Brinker Evans
 *
 *  Date	Author		Action
 * -----------------------------------------------------------------------
 * 11/28/00	JBE		Version 1.1 complete
 * 04/05/01     JBE             Error correction.  Made an adjustment to 
 *                                VNIR/SWIR offset on SCAs 2 & 4.  Was ok
 *                                for SCAs 1 & 3.
 *
 */

#include "hdf.h"
#include "mfhdf.h"
#include "cp_includes.h"



extern DATA_SET Dataset[MAX_SENSORS][MAX_DS_TYPES];
extern GAIN_DB DatabaseCoeff[MAX_SENSORS][MAX_COEFF_TYPES];
extern FILE *logfile;

extern char *SensorName[2];
extern char *BandName[2][9];



/* Table of bandcenters.  Numbers are defined by physical properties of
   of focal plane.  Pan only has one band.  Rest are fill 0's*/
/*float BandCenters[2][9]={{3.74, 4.54, 5.34, 
			  6.14, 6.94, 7.74, 
			  8.54, 9.34, 10.14},
			 {2.50,0,0,
			  0,0,0,0,0,0}};
			  */
/*  Modify table of bandcenters to account for two pixel error in shift for
    the SWIR bands.  The SWIR has primary and redundant sensors, so the actual
    sensors used vary.
    */
/*  Valid for early mission values - pre New Years? 
float BandCenters[2][9]={{3.74, 4.54, 5.34, 
			  6.14, 6.94, 7.74, 
			  8.46, 9.26, 10.06},
			 {2.50,0,0,
			  0,0,0,0,0,0}};
			  */

/*  Modify table of bandcenters again to account for 4 pixel error in shift for
    the SWIR bands.  The SWIR has primary and redundant sensors, so the actual
    sensors used vary.
    */
float BandCenters[2][9]={{3.74, 4.54, 5.34, 
			  6.14, 6.94, 7.74, 
			  8.62, 9.42, 10.22},
			 {2.50,0,0,
			  0,0,0,0,0,0}};




/*
 * Prototypes for functions used only within this module.
 */



/***************************************************************************/
/***************************************************************************/
/*
 *  Name:  ComputeFrameShifts 
 * 
 *  Description:  Computes the number of frames each band for each 
 *                SCA should be shifted for band-to-band and SCA-to
 *                SCA alignment.  
 *
 *  Inputs:
 *    int32       sca_num            which SCA is being processed.
 *    float       speed              rate of image past focal plane
 *
 *  Outputs:
 *    int32       frame_shift        matrix of integer values for each band
 *
 *  Returns:      MaxFrameShift
 *
 */

int32 ComputeFrameShifts(int32 frame_shift[MAX_SENSORS][MAX_BANDS], 
			int32 sca_num, float speed)
{

  enum sensor_types Sensor;
  int32 band;
  int32 MaxFrameShift;  
  float shift, max_shift, min_shift, max;
  float ratio, adj_ratio;
  
  if (speed == 0.0)
    {
    /* Set the speed from the frame rate */
    if (Dataset[PAN][LEVEL0].FRAME_RATE_FLT > 0.0)
      speed = 9.04 / Dataset[PAN][LEVEL0].FRAME_RATE_FLT;
    else
      speed = 0.04;	/* just use the default */
    }
  /* Ratio of pixel height to image motion at focal plane. 
     As programmed, PIXELH cancels in next equation, but done this
     way at request of another. */
  if ((sca_num == 1) || (sca_num == 3))
    ratio = PIXELH/speed;
  else /* shift opposite direction for SCAs 2 & 4) */  
    ratio = (-1)*PIXELH/speed;

  max_shift = 0;
  min_shift = 10000;
  
  /* Process science data for each type of sensor.*/
  for (Sensor = MS; Sensor < MAX_SENSORS; ++Sensor)
    {
      /* Adjust speed for PAN's higher resolution */
      if (Sensor == PAN)
	adj_ratio = ratio*3;
      else
	adj_ratio = ratio;
      
    for (band = 0; band < Dataset[Sensor][LEVEL0].BANDS; ++band)
      {
	shift = BandCenters[Sensor][band]*adj_ratio/PIXELH;
	frame_shift[Sensor][band] = ROUND_UP(shift);
	
	if (shift > max_shift) max_shift = shift;
	if (shift < min_shift) min_shift = shift;

      }
    }
  


  if ((sca_num == 1) || (sca_num == 3))
    {
      MaxFrameShift = ABS((2*ROUND_UP(max_shift)) - 1);

      max = MaxFrameShift - ROUND_UP(max_shift);

      /* Due to structure of focal plane, and unusual format,
	 SCAs 1 & 3 need a 4 frame adjustment to the shift 
	 for the SWIR only
	 */
	  
      Sensor = MS;      
      for (band = 6; band <= 8; ++band)
	frame_shift[Sensor][band] = frame_shift[Sensor][band]-4;
      
      

      for (Sensor = MS; Sensor < MAX_SENSORS; ++Sensor)
	for (band = 0; band < Dataset[Sensor][LEVEL0].BANDS; ++band)
	  {
	    frame_shift[Sensor][band] = frame_shift[Sensor][band] + max;
	    printf("Sensor:  %d  Band:  %s  Shift:  %d \n",
		   Sensor, BandName[Sensor][band], frame_shift[Sensor][band]);
	
	  }


    }
  else /* if ((sca_num == 2) || (sca_num == 4)) */
    {
      MaxFrameShift = ABS((2*ROUND_UP(min_shift)) - 1);
      

      max = MaxFrameShift + ROUND_UP(min_shift) -1;
      for (Sensor = MS; Sensor < MAX_SENSORS; ++Sensor)
	for (band = 0; band < Dataset[Sensor][LEVEL0].BANDS; ++band)
	  {
	    frame_shift[Sensor][band] = frame_shift[Sensor][band] + max;
	    printf("Sensor:  %d  Band:  %s  Shift:  %d \n",
		   Sensor, BandName[Sensor][band], frame_shift[Sensor][band]);
	
	  }
    }
 
  return(MaxFrameShift);
  
}



/***************************************************************************/
/***************************************************************************/
/*
 *  Name:  WriteLeadFill
 * 
 *  Description: Writes frames of fill data to output file for alignment.
 *
 *  Inputs:      Sensor        sensor_type MS or PAN
 *               data          short *     data array being written out
 *               frame_shift   int32       number of frames of fill needed
 *               MaxFrameShift int32       maximum frame shift for all bands
 *               edge[3]       int32       defines size of data in dataset
 *               band          int32       band number being written
 *
 *  Outputs:     nothing
 *
 *  Returns:     nothing
 *
 */

void WriteLeadFill(enum sensor_types Sensor, short *data,
		   int32 shift, 
		   int32 MaxFrameShift, int32 edge[3],
		   int32 cross, int32 along, int32 band)

{
  int32 sds_id, status;
  int32 start[3];
  int32 i;  


  

  /* increment starting point as band number is incremented. */
  start[0] = band;
  start[2] = cross;

  /* determine sds_id */
  sds_id = Dataset[Sensor][LEVEL1R].SDS_ID;


  for (i = 0; i < shift; ++i)
    {      
      start[1] = i;


      /* Write a single band of data */
      status = SDwritedata(sds_id, start, NULL, edge, (VOIDP)data);


      if (status == FAIL)
	{
	  fprintf(logfile,"\nERROR:  SDwritedata failed!  %d \n", status);
	  fprintf(logfile,"\n        Writing fill data to Level 1R");
	  fprintf(logfile,"sds_id: %d  start: %d %d %d    edge: %d %d %d/n",
		  sds_id, start[0], start[1], start[2],
	      edge[0], edge[1], edge[2]);	  
	  
	  return;	  
	}      
    }
  
  return;
}


/***************************************************************************/
/***************************************************************************/
/*
 *  Name:  WriteTrailFill
 * 
 *  Description: Writes frames of fill data to output file for alignment.
 *
 *  Inputs:      Sensor        sensor_type MS or PAN
 *               data          short *     data array being written out
 *               frame_shift   int32       number of frames of fill needed
 *               MaxFrameShift int32       maximum frame shift for all bands
 *               edge[3]       int32       defines size of data in dataset
 *               band          int32       band number being written
 *
 *  Outputs:     nothing
 *
 *  Returns:     nothing
 *
 */

void WriteTrailFill(enum sensor_types Sensor, short *data,
		   int32 shift, 
		   int32 MaxFrameShift, int32 edge[3],
		   int32 cross, int32 along, int32 band)

{
  int32 sds_id, status;
  int32 start[3];
  int32 i;  

  

  /* increment starting point as band number is incremented. */
  start[0] = band;
  start[2] = cross;

  /* determine sds_id */
  sds_id = Dataset[Sensor][LEVEL1R].SDS_ID;


  for (i = 0; i < (MaxFrameShift - shift); ++i)
    {      
      start[1] = along + shift + i;

      /* Write a single band of data */
      status = SDwritedata(sds_id, start, NULL, edge, (VOIDP)data);


      if (status == FAIL)
	{
	  fprintf(logfile,"\nERROR:  SDwritedata failed!  %d \n", status);
	  fprintf(logfile,"\n        Writing fill data to Level 1R");
	  fprintf(logfile,"sds_id: %d  start: %d %d %d    edge: %d %d %d/n",
		  sds_id, start[0], start[1], start[2],
	      edge[0], edge[1], edge[2]);	  
	  
	  return;	  
	}      
    }
  
  return;
}

