/*
 *  File: cp_calib.c         
 *
 *  Description:  This file has routines for computing the dark noise
 *                calibration coefficient for a DCE, reading in the
 *                response coefficients, and generating the output
 *                calibrated response.
 *
 *                These routines assume the input file is a Level0 HDF file.
 *                It assumes there is a single dataset for a WIS or GIS file,
 *                and multiple datasets for a MS/PAN dataset.  It assumes
 *                the Level0 data is written as a three dimensional datacube
 *                in BSQ (band sequential) order as defined in the ICD.
 *
 *  Subroutines:  ComputeOffsetCoeff
 *                ComputeResponseCoeff
 *                CalibrateData
 *                ReadStoredCoeff
 *
 *  Author:	  Jenifer Brinker Evans
 *
 *  Date	Author		Action
 * -----------------------------------------------------------------------
 *  3/11/97	JBE		Version 1.1 complete
 *  4/13/98     JBE             Implemented Dataset structure with enumerated
 *                                datatypes.  Modified response coefficient
 *                                reading to work with HDF files with MS and 
 *                                PAN coefficients.
 *  4/23/98     JBE             Assures fill data is not used in computation of
 *                                of average offset data.  If offset correction
 *                                is greater than raw data for a pixel, the
 *                                corrected value is set to 0, i.e., not 
 *                                allowed to be negative.
 *  4/24/98     JBE             Replace print statements with prints to
 *                                an external log file.
 *  3/18/99     JBE             Implement use of integration time.
 *                              Change output data from float to 16-bit
 *                                scaled int to meet ICD.
 *                              Remove temp code that allowed fill data
 *                                to be a simple 0.
 *  3/25/99     JBE             Implemented ReadStoredCoeff routine.
 * 11/15/00     JBE             Added processing for handling leaky pixels.
 *                                Old method deleted.
 *  1/31/01     JBE             Changed call to index2 to index2_flip to
 *                                handle left to right flipping of image.
 *  2/14/01     JBE             Added command line option in main to allow
 *                                choice in using the left to right flipping
 *  4/05/01     JBE             Error correction for processing scenes with
 *                                missing dark data.  Use width of science
 *                                data instead of nonexistent dark to allocate
 *                                space.
 *  6/27/01     JBE             Added conditional to allow for use of PRECAL
 *                                data when DARK is missing.  Also adjusted
 *                                for use of default dark for flipped or not
 *                                flipped data.
 */



#include "hdf.h"
#include "mfhdf.h"
#include "cp_includes.h"

extern DATA_SET Dataset[MAX_SENSORS][MAX_DS_TYPES];
extern GAIN_DB DatabaseCoeff[MAX_SENSORS][MAX_COEFF_TYPES];
extern int32 sd_id, sd_id_db, sd_1r_id;
extern int32 FLIP;
extern FILE *logfile;

int ReadStoredOffsetCoeff(enum sensor_types ,
			  float *, int32, int32);




/***************************************************************************/
/***************************************************************************/
/*
 *  Name:  ComputeOffsetCoeff
 * 
 *  Description:  Takes a two dimensional array, cross track by along track,
 *                and averages all dark data pixels along track for each
 *                cross track element.
 *
 *                Generates a 1-D array (cross track x 1 line) of offset data.
 *                If the dark data does not vary over time, the same
 *                array is used for every along track line.  If the offset
 *                data is timevarying, this routine is called once for every
 *                along track line.
 *
 *  Inputs:       
 *                data         short *        raw focal plane data
 *                dim[2]       int32          dimensions of data array
 *                begin        int32          where dark data begins
 *                end          int32          where dark data ends
 *
 *  Outputs:      offset_data  float *        resulting offset_data array
 *
 *  Returns:      nothing
 *
 */

void ComputeOffsetCoeff(enum sensor_types Sensor, float *offset_data, int32 band, 
			int32 read)
{
  static int First = TRUE;
  int32 FirstFill;  
  int32 i, j, k, total, sum, inc;
  int32 num_pixels, half;
  short *dark_data;
  float *temp_data;
  int32 dim2[2], 
        dark_data_start[3],
        dark_data_edge[3];  
  float offset_value, temp;
  int32 DarkSet, dark_exists;
  


  if (Dataset[Sensor][DARK].EXISTS)
    {      
      DarkSet = DARK;
      dark_exists = TRUE;
    }
  else if (Dataset[Sensor][PRECAL].EXISTS)
    {
      PlanB:
      DarkSet = PRECAL;
      dark_exists = TRUE;
    }
  else
    {      
      dark_exists = FALSE;
      
      if (First)
	{	  
	  First = FALSE;
	}
      
      
      fprintf(logfile,"WARNING: Using Default Dark Data for Sensor %d\n", Sensor);
      printf("WARNING: Using Default Dark Data for Sensor %d\n", Sensor);
      
      if (ReadStoredOffsetCoeff(Sensor, offset_data, band, read) == FAIL)
	GracefulExit();

      /* 
       * The default dark data is aligned for the original data format.
       *  Data recorded after Feb 15 (not FLIP) needs to be flipped to
       *  work correctly.
       */
      if (!FLIP)
	{
	  num_pixels = Dataset[Sensor][LEVEL0].CROSS;
	  half = num_pixels/2;
	  
	  printf("flipping default... \n");
	  
	  for (i = 0; i < half; ++i)
	    {	      
	      temp = offset_data[i];
	      offset_data[i] = offset_data[num_pixels-i-1];
	      offset_data[num_pixels-i-1] = temp;
	    }
	  
	}
      
      
    }
  

  if (dark_exists)
    {
      /* Malloc space for reading single band of data. */
      num_pixels = Dataset[Sensor][DarkSet].ALONG * Dataset[Sensor][DarkSet].CROSS;
      dark_data = (short *)malloc((unsigned) num_pixels*sizeof(short));
      
      /* Flag indicating if Default Data still hasn't been used. */
      /* If it is used, it's necessary to malloc space for temp_data */
      FirstFill = TRUE;
      
  
      dim2[0] = Dataset[Sensor][DarkSet].ALONG;
      dim2[1] = Dataset[Sensor][DarkSet].CROSS;

      /* 
       *  Define the location, pattern, and size of the data set 
       *   Assumes first data set starts at 0,0,0.   
       */
      
      dark_data_start[0] = 0;
      dark_data_start[1] = 0;
      dark_data_start[2] = 0;
      
      /*  
       *  Working with one band of data at a time, so edges are 1 band,
       *   the along track dimension, and the cross track dimension.
       */
      dark_data_edge[0] = 1;
      dark_data_edge[1] = Dataset[Sensor][DarkSet].ALONG;
      dark_data_edge[2] = Dataset[Sensor][DarkSet].CROSS;
      
      ReadRawData(Dataset[Sensor][DarkSet].SDS_ID, dark_data, dark_data_start,
		  dark_data_edge, band);
      
      /*  Set increment value.  If PAN sensor, offset is determined for 
       *  every 3 reads.  For all other sensors, offset is determined
       *  from every read.
       */
      if (Sensor == PAN)
	inc = 3;
      else
	inc = 1;      


      for (i = 0; i < Dataset[Sensor][DarkSet].CROSS; ++i)
	{      
	  total = 0;      
	  sum = 0;
	  
	  /*  Add 12 frame cushion to be sure occasional glitch in PRECAL
	      data due to focal plane start doesn't affect dark data.  Since
	      their are a multiple of 3 frames in PAN, a multiple of 3 is
	      used for the cushion.*/
	  for (j = read+12; j < Dataset[Sensor][DarkSet].ALONG-12; j += inc)

	    /* Check for fill data.  Fill data has fill info in upper 4 bits */
	    if ((dark_data[index2(j,i,dim2)] & 0xf000) == 0)
		{		    
		  total += dark_data[index2(j,i,dim2)];
		  ++sum;
		}
	  
		  
	  if (sum == 0)
	    {
  

	      /* If post image dark existed but was all fill, go back and try
	         the precal. */
	      if (DarkSet == DARK)
	      	goto PlanB;

	      fprintf(logfile,"\nWARNING: No acceptable dark data.  All fill.\n");
	      {      
		if (First)
		  {	  
		    First = FALSE;
		  }
		
		if (FirstFill)
		  {		    
		    fprintf(logfile,"Using Default Dark Data.  Band %d Cross %d\n",
			    band,i);
		    printf("Using Default Dark Data:  Band %d Cross %d\n",band,i);
		    temp_data = (float *)malloc((unsigned) num_pixels*sizeof(float));
		    FirstFill = FALSE;
		  }
		
		if (ReadStoredOffsetCoeff(Sensor, temp_data, band, read) == FAIL)
		  offset_value = 0;
		else
		  offset_value = temp_data[i];	       
	      }
	    }
	  
	  else
	    offset_value = (float)total/sum;      
	  

	  offset_data[i] = offset_value;
	  
	}
      
        
      free((short *)dark_data);
      if (FirstFill == FALSE)
	free((short *)temp_data);
    }
  
  
  
  
  return;
  
}


/***************************************************************************/
/***************************************************************************/
/*
 *  Name:  ComputeResponseCoeff
 * 
 *  Description:  Routine reads in response coefficient data from a
 *                database which is currently manifested as a file.
 *
 *                Each cross track pixel for each band will have a
 *                function associated with it relating the digital
 *                numbers from the focal plane to the irradiance.
 *                This function can be represented by the coefficients
 *                of a polynomial or by a look-up table.
 *
 *                This function is called once for each band.
 *
 *  Inputs:       dim[2]         int32       dimensions of data array
 *                band           int32       band number
 *
 *  Outputs:      response_data  float *     response data array read in
 *
 *  Returns:      nothing
 *
 */

void ComputeResponseCoeff(enum sensor_types Sensor,  enum ds_types DataType,
			  float *response_table, int32 dim_table[2], int32 band)
{
  static int32 database_start[3], database_edge[3];
  static short first = TRUE;
  static enum sensor_types old_sensor;
  int32 status, i;
  short index;
  
   
  
  if (first || (Sensor != old_sensor))
    {      
      /* If first time called for this sensor, check dims compatability */

      if ((DatabaseCoeff[Sensor][RESP_COEFF].BANDS != 
	   Dataset[Sensor][DataType].BANDS) || 
	  (DatabaseCoeff[Sensor][RESP_COEFF].CROSS != 
	   Dataset[Sensor][DataType].CROSS))	
	{
	  fprintf(logfile,"\nERROR:  Dimensions of gain coefficients database don't agree with data.\n");
	  fprintf(logfile,"  Database:  %3d x %3d    Data:  %3d x %3d\n",
		 DatabaseCoeff[Sensor][RESP_COEFF].BANDS, 
		  DatabaseCoeff[Sensor][RESP_COEFF].CROSS,
		  Dataset[Sensor][DataType].BANDS, 
		  Dataset[Sensor][DataType].CROSS);
	  fprintf(logfile,"Exiting program from ComputeResponseCoeff");
	  printf("ERROR: Dimensions of database don't agree with data");
	  fflush(logfile);
	  GracefulExit();	  

	}
            
      database_start[1] = 0;
      database_start[2] = 0;
      database_edge[0] = 1;
      database_edge[1] = DatabaseCoeff[Sensor][RESP_COEFF].CROSS;
      database_edge[2] = DatabaseCoeff[Sensor][RESP_COEFF].LENGTH;

      /* Define now for accessing data later using index2 function.*/
      dim_table[0] = DatabaseCoeff[Sensor][RESP_COEFF].CROSS;
      dim_table[1] = DatabaseCoeff[Sensor][RESP_COEFF].LENGTH;
      
      /* set flags for next time through */
      first = FALSE;
      old_sensor = Sensor;
      
    }
  

  database_start[0] = band;
  
  /* read response data for appropriate band */
  status = SDreaddata(DatabaseCoeff[Sensor][RESP_COEFF].SDS_ID, database_start,
		      NULL, database_edge,(VOIDP)response_table);

  if (status == FAIL)
    {
      printf("\nERROR:  SDreaddata failed!  %d \n", status);
      fprintf(logfile,"\nERROR:  SDreaddata failed!  %d \n", status);
      fprintf(logfile,"sds_id: %d  start: %d %d %d    edge: %d %d %d /n",
	      DatabaseCoeff[Sensor][RESP_COEFF].SDS_ID, database_start[0], 
	      database_start[1], database_start[2],
	      database_edge[0], database_edge[1], database_edge[2]);	  
      fprintf(logfile,"   Unable to read database coefficients.  Exiting.\n");
      printf("   Unable to read database coefficients.  Exiting.\n");
      GracefulExit();
      
      return;	  
    }      
  
  

  return;
}


/***************************************************************************/
/***************************************************************************/
/*
 *  Name:  CalibrateData
 * 
 *  Description:  Routine that subtracts the offset noise from the 
 *                uncalibrated data, then scales the answer by the
 *                response coefficients.
 *
 *  Inputs:       Sensor        short          enumerated sensor type
 *                fpdata        short *        raw focal plane data
 *                dim[2]        int32          dimensions of data array
 *                offset_data   float *        offset data to subtract
 *                response_data float *        response data to multiply by
 *
 *  Outputs:      cal_data      float *        calibrated focal plane data
 *
 *  Returns:      nothing
 *
 */

void CalibrateData(enum sensor_types Sensor, 
		   enum ds_types DataType,
		   short *fpdata, 
		   int32 dim[2], 
		   int32 dim_table[2],
		   int32 along, 
		   float *response_table,
		   float *offset_data, 
		   float *response_data, 
		   short *cal_data,
		   float integ_time,
		   int32 sca_num,
		   int32 band)

{
  int32 cross, n, m;
  int32 cal_value_scaled;  
  short modified_DN;
  float cal_value;
  int32 start, inc;
  


  

  /* For SCA 3, band 3 and SCA 4, band 2 only every other pixel is 
     calibrated with the look up table due to leaky pixels.  The other
     pixels are calibrated later after the whole frame is processed.
     SCA 4 has the odd numbers saved.  SCA 3 has the even numbers saved.
     */
  if (((sca_num == 3) && (band == 3)) || ((sca_num == 4) && (band == 2)))
    {
      /* if command line arg -f, process pre 2/15/01 way...*/
      if (FLIP)
	{
	  inc = 2;
	  if ((sca_num == 3) && (band == 3))
	    start = 1;
	  else 
	    start = 0;
	}
      else /*process new way with left-to-right flip */
	{
	  inc = 2;
	  if ((sca_num == 3) && (band == 3))
	    start = 0;
	  else 
	    start = 1;
	}
    }  
  else
    {
      inc = 1;
      start = 0;
    }
  

  for(cross = start; cross < Dataset[Sensor][DataType].CROSS; cross += inc)
    {			
      n = index2(along,cross,dim);	  
      
      
      /* 
       *  if data is fill data, set calibrated data and response data 
       *   equal to matching fill data, and offset data is already 
       *   equal to 0
       */
      if ((fpdata[n] & 0xf000) != 0)
	{
	  cal_data[cross] = fpdata[n];
	  response_data[cross] = fpdata[n];
	}
      
      else
	{	
	  modified_DN = (short)(fpdata[n] - offset_data[cross]);


	  if (modified_DN < -10)
	    modified_DN = -10;

	  /* if command line -f set, process old way pre 2/15/01 */

	  if (FLIP) 
	    m = index2(cross, (modified_DN +10 ), dim_table);	  
	  else /* process new way with left-to-right flip */
	    m = index2_flip(cross, (modified_DN +10 ), dim_table);	  

	  cal_value = response_table[m];
	  /*  Comment out line making use of integ_time.
              The current set of rad cal coefficients are considered
              valid only for INTEG_TIME = 12, or integ_time =     ms.
              New method generates output in mW/cm^2/sr/u/DN -- no time
	  cal_data[cross] = ROUND((cal_value/integ_time)*SCALE_FACTOR);
	  */

	  /* convert from float to scaled int */
	  cal_value_scaled = ROUND((cal_value)*SCALE_FACTOR);

	  /* Check for case of hot pixels or anomoly that would overflow
	     short int */
	  if (cal_value_scaled >= 32767) cal_value_scaled = 32767;	  
	  cal_data[cross] = (short)cal_value_scaled;
	  

	  /*  Determine per pixel gain.  When using a linear fit, this
              value is the same for all rows for a single pixel, but for
              a polynomial fit, it varies.  This method which allows for
              variability is being retained in case we ever go back.
	  */
	  if (modified_DN == 0)
	    response_data[cross] = 0;
	  else	
	    response_data[cross] = cal_value/((float)modified_DN);
	    
	    
	}
    }
 
	  
  return;
}


/***************************************************************************/
/***************************************************************************/
/*
 *  Name: ReadStoredOffsetCoeff
 * 
 *  Description:  Routine for reading stored offset coefficients from 
 *                the database.  These coefficients are typically 
 *                computed but if the dark data is not available or
 *                the data is of questionable quality, previously
 *                computed and stored data is used.
 *
 *  Inputs:
 *
 *  Outputs:
 *
 *  Returns:
 *
 */

int ReadStoredOffsetCoeff(enum sensor_types Sensor, 
			  float *offset_data, 
			  int32 band, int32 read)

{

  int32 start[3], edge[3];
  int32 status;  
  

  if (Sensor == PAN)
    start[0] = read;   
  else
    start[0] = band;
  

  start[1] = 0;
  start[2] = 0;
  edge[0] = 1;
  edge[1] = Dataset[Sensor][LEVEL0].CROSS;
  edge[2] = 1;



  /* Read a single band of data */
  status = SDreaddata(DatabaseCoeff[Sensor][OFF_COEFF].SDS_ID, start, NULL, edge, 
			      (VOIDP)offset_data);
	  
  if (status == FAIL)
    {      
      fprintf(logfile,"\nWARNING:  SDreaddata failed!  %d \n", status);
      fprintf(logfile,"Reading Default Dark Data!\n");
      fprintf(logfile,"sds_id: %d  start: %d %d   edge: %d %d/n",
	      DatabaseCoeff[Sensor][OFF_COEFF].SDS_ID, start[0], start[1], 
	      edge[0], edge[1]);	  
      return(FAIL);	  
    }      
	  
  return(SUCCEED); 	  
}

  
  

/***************************************************************************/
/***************************************************************************/
/*
 *  Name:  OpenGainDatabase
 * 
 *  Description:  Function opens Database HDF file.
 *
 *  Inputs:  Name of cal_dbase_file
 *
 *  Outputs: Modified fields in DatabaseCoeff structure
 *
 *  Returns: sd_id for cal_dbase_file
 *
 */

int32 OpenGainDatabase(char cal_dbase_file[])
{
  static int FIRST = TRUE;
  
  int32 sds_id;
  int32 num_datasets, num_attr;
  int32 i, index;
  char sensor_name[5];
  char  type_name[50];         

  enum sensor_types Sensor;
  enum coeff_types Coeff;
  
  

  sd_id_db = SDstart(cal_dbase_file, DFACC_RDONLY);
  fprintf(logfile,"Opened %s.  sd_id: %d\n", cal_dbase_file, sd_id_db);
  
  if (SDfileinfo(sd_id_db, &num_datasets, &num_attr) == FAIL)
    {
      fprintf(logfile,"\nERROR: Error reading gain database file info\n");
      printf("\nERROR: Error reading gain database file info\n");
      GracefulExit();  
    }
  else
    {
      printf("Opened gain database: %s\n", cal_dbase_file);
      printf("found %3d datasets.\n", num_datasets);
      
      
      fprintf(logfile,"\n");      
      fprintf(logfile,"Database:  # of datasets   = %2d\n", num_datasets);
      fprintf(logfile,"Database:  # of attributes = %2d\n", num_attr);
    }
  
  for(Sensor = MS; Sensor < MAX_SENSORS; ++Sensor)
    for(Coeff = RESP_COEFF; Coeff < MAX_COEFF_TYPES; ++Coeff)
      DatabaseCoeff[Sensor][Coeff].EXISTS = FALSE;
  

  for (i = 0; i < num_datasets; ++i)
    {
      sds_id = SDselect(sd_id_db, i);
      
      index = SDfindattr(sds_id, "ALI Sensor");
      SDreadattr(sds_id, index, (VOIDP)sensor_name);
      sensor_name[4] = 0;
      
      if (strncmp(sensor_name, "MS", 2) == 0) 
	Sensor = MS;
      else if (strncmp(sensor_name, "PN", 2) == 0)
	Sensor = PAN;
      else /*currently if the sensor is NOT MS or PAN, there's an error */
	{
	  fprintf(logfile,"\nERROR:  Sensor gain data not for  MS or PN: %s\n", 
		  sensor_name);
	  GracefulExit(); 	  
	}
      fprintf(logfile,"\nOpened gain data for sensor %d.  sds_id: %d\n", 
	      Sensor, sds_id);
      

      index = SDfindattr(sds_id, "Dataset Type");
      SDreadattr(sds_id, index, (VOIDP)type_name);
      fprintf(logfile,"Dataset type: %s\n", type_name);
      
      if (strncmp(type_name, "Response", 8) == 0)
	Coeff = RESP_COEFF;
      else if (strncmp(type_name, "Offset", 6) == 0)
	Coeff = OFF_COEFF;
      else if (strncmp(type_name, "IntCal", 6) == 0)
	Coeff = INTCAL;
      else if (strncmp(type_name, "SolarCal", 8) == 0)
	Coeff = SOLCAL;
      else if (strncmp(type_name, "LeakyPix", 8) == 0)
	Coeff = LEAKYPIX;
      else /*if the dataset type is NOT one of the above, there's an error */
	{
	  fprintf(logfile,"ERROR:  Dataset type not valid: %s\n", type_name);
	  GracefulExit();  
	}



      strcpy(DatabaseCoeff[Sensor][Coeff].SENSOR, sensor_name);

      DatabaseCoeff[Sensor][Coeff].EXISTS = TRUE;
      DatabaseCoeff[Sensor][Coeff].SDS_ID = sds_id;


      index=SDfindattr(DatabaseCoeff[Sensor][Coeff].SDS_ID,
		       "Number of cross track pixels");
      SDreadattr(DatabaseCoeff[Sensor][Coeff].SDS_ID, index, 
		 &DatabaseCoeff[Sensor][Coeff].CROSS);
  

      index = SDfindattr(DatabaseCoeff[Sensor][Coeff].SDS_ID, 
			 "Number of bands");
      SDreadattr(DatabaseCoeff[Sensor][Coeff].SDS_ID, index, 
		 &DatabaseCoeff[Sensor][Coeff].BANDS);

      index = SDfindattr(DatabaseCoeff[Sensor][Coeff].SDS_ID,"Dataset length");
      SDreadattr(DatabaseCoeff[Sensor][Coeff].SDS_ID, index, 
		 &DatabaseCoeff[Sensor][Coeff].LENGTH);
    }

  return(sd_id_db);
  
}














  

