/*
 *  File:  cp_data_attr.c
 *
 *  Description:  This file has routines for reading in the dataset
 *		  attributes from the HDF Level 0 file, and writing
 *		  the dataset attributes to the HDF Level 1R file.
 *
 *  Subroutines:  ReadDataAttributes
 *		  WriteDataAttributes
 *
 *  Author:	  Margaret Dunham Gibbs
 *
 *  Date	Author		Action
 * -----------------------------------------------------------------------
 * 02/24/98	MDG		Version 1.1 complete
 * 03/24/98	MDG		Version 1.2 - added ReadDataAttributes
 *				   routine w/error checking.
 * 03/26/98	MDG		V1.2 moved the creation of HDF array
 *				   datasets from "PrepareOutputFile" 
 *				   to "WriteDataAttributes
 * 04/02/98	MDG		V1.3 separated File & Dataset attributes
 * 04/08/98	MDG		V1.4 corrected "name" to "type_name" in
 *					ReadDataAttributes
 *				     changed strcmp's to strncmp's
 *  4/13/98     JBE             Completed conversion to Dataset structure
 *                                 with enumerated datatypes.  Elimated
 *                                 need for sds_id arrays.
 *  4/24/98     JBE             Replace print statements with prints to
 *                                an external log file.
 * 10/03/98     JBE             Small corrections to assure attributes match
 *                                ICD word for word.
 *  3/20/99     MDG             Add in SCALE_FACTOR, and attributes
 *                                "INTEG_TIME" and "FRAME_RATE"
 *  3/26/99	MDG		Logfile fixes
 * 11/07/00     JBE     Reviewed -- fine for delivery -- added comments
 *
 *  2/14/01     JBE             Added conditionals to handle change of
 *                                frame rate and integration time from
 *                                integers to floats on 2/15/01.  Still
 *                                need to be able to process old format.
 *                                To avoid compiler options, set up with two
 *                                forms for these two variables.
 * 8/30/01      JBE             Addition of units attribute to Level dataset
 */


#include "hdf.h"
#include "mfhdf.h"
#include "cp_includes.h"


extern DATA_SET Dataset[MAX_SENSORS][MAX_DS_TYPES];
extern PARAM_IN FileAttr;

extern char *GenerateFillAttName();
extern void ErrMsg();
extern int32 FLIP;
extern FILE *logfile;
extern char *BandName[2][9];

char *DataName[25] = {"LEVEL0", "CAL1", "CAL2", "CAL3", "CAL4", 
		      "PRECAL", "DARK", "LAMP", "MAP", 
		      "LEVEL1R", "RESPONSE", "OFFSET",
		      "CAL1R", "CAL2R", "CAL3R", "CAL4R",
		      "CAL1_RESP", "CAL2_RESP", "CAL3_RESP", "CAL4_RESP",
		      "CAL1_OFF", "CAL2_OFF", "CAL3_OFF", "CAL4_OFF",
		      "MAPOUT"};

char units[20];


/***************************************************************************/
/*
 *  Name:  ReadDataAttributes
 * 
 *  Description: Read dataset attributes from level0 file
 *
 *  Inputs: int32 sds_id
 *
 *  Outputs:
 *
 *  Returns:
 *
 */

void ReadDataAttributes(int32 num_datasets, int32 sd_id)
{
  int32 i;
  int32 sds_id;  
  int32 index;
  int32 attributes;
  int32 rank_fpdata, dim_fpdata[3],fp_type;
  int32 status;
  char  name[50];         
  char  type_name[50];         
  char  sensor_name[50];         
  enum sensor_types Sensor;
  enum ds_types ds_type;
  int32 temp_along, t_type, t_count;
  char  t_name[50];
  
  


  
  /* Initialize Dataset existence field to False.  Later
     if a specific type of dataset is determined to be in
     the file being processed, the existence field flag
     will be set to True. 
  */
  for(Sensor = MS; Sensor < MAX_SENSORS; ++Sensor)
    for(ds_type = LEVEL0; ds_type < MAX_DS_TYPES; ++ds_type)
      Dataset[Sensor][ds_type].EXISTS = FALSE;


  /*  For each dataset in the file being processed ... */
  for (i = 0; i <num_datasets; ++i)
    {
      /*  Determine the sds_id for each dataset.*/
      sds_id = SDselect(sd_id,i);
      
      
      /* Read dataset (sds) attribute information */
      
      /*  Read "Dataset Type" attribute and determine if
          dataset type (ds_type) is LEVEL0, DARK, LAMP, PRECAL
          or PIXELMAP. 
       */
      index = SDfindattr(sds_id, "Dataset Type");
      SDreadattr(sds_id, index, (VOIDP)type_name);
      type_name[12] = 0;
      
      if ((strncmp(type_name, "LEVEL0", 6) == 0) || 
	  (strncmp(type_name, "Level 0", 7) == 0))
	{
	fprintf(logfile, "\nDataset type: Level 0\n");
	ds_type = LEVEL0;
	}
      else if ((strncmp(type_name, "DARK", 4) == 0) || 
		(strncmp(type_name, "Dark", 4) == 0))
	{
	fprintf(logfile, "\nDataset type: Dark\n");
	ds_type = DARK;
	}
      else if ((strncmp(type_name, "LAMP", 4) == 0) || 
	       (strncmp(type_name, "Lamp", 4) == 0))
	{
	fprintf(logfile, "\nDataset type: Lamp\n");
	ds_type = LAMP;
	}
      else if ((strncmp(type_name, "Pixel Map", 9) == 0) ||
	       (strncmp(type_name, "PixelMap", 9)==0))

	{
	fprintf(logfile, "\nDataset type: Pixel Map\n");
	ds_type = MAP;
	}
      else if (strncmp(type_name, "PIXELMAP", 8) == 0)
	{
	fprintf(logfile, "\nDataset type: Pixel Map\n");
	ds_type = MAP;
	}
      else if ((strncmp(type_name, "PRECAL", 6) == 0) ||
	       (strncmp(type_name, "Precal", 6) == 0))
	{
	fprintf(logfile, "\nDataset type: Pre cal data\n");
	ds_type = PRECAL;
	}
      else if ((strncmp(type_name, "CAL1", 4) == 0) ||
	       (strncmp(type_name, "Cal1", 4) == 0))
	{
	fprintf(logfile, "\nDataset type: Lunar cal data\n");
	ds_type = CAL1;
	}
      else if ((strncmp(type_name, "CAL2", 4) == 0) ||
	       (strncmp(type_name, "Cal2", 4) == 0))
	{
	fprintf(logfile, "\nDataset type: Lunar cal data\n");
	ds_type = CAL2;
	}
      else if ((strncmp(type_name, "CAL3", 4) == 0) ||
	       (strncmp(type_name, "Cal3", 4) == 0))
	{
	fprintf(logfile, "\nDataset type: Lunar cal data\n");
	ds_type = CAL3;
	}
      else if ((strncmp(type_name, "HYP", 3) == 0) ||
	       (strncmp(type_name, "Hyp", 3) == 0))
	{
	fprintf(logfile, "\nDataset type: Lunar cal data\n");
	ds_type = CAL4;
	}
      else /*if the dataset type is NOT one of the above, there's an error */
	{
	  fprintf(logfile,"ERROR:  Dataset type not valid: %s\n", type_name);
	  printf("ERROR:  Dataset type not valid: %s\n", type_name);
	  GracefulExit();  
	}

      /*  "ALI Sensor" attribute to determine what 
          Sensor type, MS or PN, the data is for.
       */
      index = SDfindattr(sds_id, "ALI Sensor");
      SDreadattr(sds_id, index, (VOIDP)sensor_name);
      sensor_name[4] = 0;
      fprintf(logfile,"Sensor Type: %s\n", sensor_name);
      
      if (strncmp(sensor_name, "MS", 2) == 0)
	Sensor = MS;
      else if (strncmp(sensor_name, "PN", 2) == 0)
	Sensor = PAN;
      else /*currently if the sensor is NOT MS or PAN, there's an error */
	{
	  fprintf(logfile,"ERROR: Sensor not equal to MS or PN: %s\n",sensor_name);
	  printf("ERROR: Sensor not equal to MS or PN: %s\n",sensor_name);
	  GracefulExit();  
	}
      fprintf(logfile,"Opened data %d %7s for sensor %d %3s.  sds_id: %d\n", 
	      ds_type, type_name, Sensor, sensor_name, sds_id);

      
      /*  Begin filling in general info for the dataset.  Note a two-
          dimensional data structure, Dataset[Sensor][ds_type] is used
          to hold this info.
       */
      strcpy(Dataset[Sensor][ds_type].SENSOR, sensor_name);
      
      strcpy(Dataset[Sensor][ds_type].DS_TYPE, type_name);
      Dataset[Sensor][ds_type].EXISTS = TRUE;
      Dataset[Sensor][ds_type].SDS_ID = sds_id;
      
      index = SDfindattr(sds_id, "Data Start Time");
      SDreadattr(sds_id, index, (VOIDP)Dataset[Sensor][ds_type].START);
      fprintf(logfile,"Data Start time: %s\n", Dataset[Sensor][ds_type].START);
      
      index = SDfindattr(sds_id, "Data Duration");
      SDreadattr(sds_id, index, &Dataset[Sensor][ds_type].DURATION);
      fprintf(logfile,"Data duration: %f\n", Dataset[Sensor][ds_type].DURATION);
      index = SDfindattr(sds_id, "Integration Time");
      if (FLIP)
	{	  
	  SDreadattr(sds_id, index, &Dataset[Sensor][ds_type].INTEG_TIME_INT);
	  fprintf(logfile,"Integration time: %d\n",
		  Dataset[Sensor][ds_type].INTEG_TIME_INT);
	}
      else
	{	  
	  SDreadattr(sds_id, index, &Dataset[Sensor][ds_type].INTEG_TIME_FLT);
	  fprintf(logfile,"Integration time: %f\n",
		  Dataset[Sensor][ds_type].INTEG_TIME_FLT);
	}
      
      index = SDfindattr(sds_id, "Frame Rate");
      if (FLIP)
	{	  
	  SDreadattr(sds_id, index, &Dataset[Sensor][ds_type].FRAME_RATE_INT);
	  fprintf(logfile,"Frame rate: %d\n", 
		  Dataset[Sensor][ds_type].FRAME_RATE_INT);
	}
      else
	{	  
	  SDreadattr(sds_id, index, &Dataset[Sensor][ds_type].FRAME_RATE_FLT);
	  fprintf(logfile,"Frame rate: %f\n", 
		  Dataset[Sensor][ds_type].FRAME_RATE_FLT);
	}

      
      index = SDfindattr(sds_id, "Number of cross track pixels");
      SDreadattr(sds_id, index, &Dataset[Sensor][ds_type].CROSS);
      fprintf(logfile,"Cross: %d\n", Dataset[Sensor][ds_type].CROSS);

      
      index = SDfindattr(sds_id, "Number of along track pixels");
      SDreadattr(sds_id, index, &Dataset[Sensor][ds_type].ALONG);
      fprintf(logfile,"Along: %d \n", Dataset[Sensor][ds_type].ALONG);

  
      index = SDfindattr(sds_id, "Number of bands");
      SDreadattr(sds_id, index, &Dataset[Sensor][ds_type].BANDS);
      fprintf(logfile,"Bands: %d\n", Dataset[Sensor][ds_type].BANDS);

      
      index = SDfindattr(sds_id, "Percent Missing Data");
      SDreadattr(sds_id, index, &Dataset[Sensor][ds_type].PRCNT_MISS);
      fprintf(logfile,"Percent missing data: %d\n", Dataset[Sensor][ds_type].PRCNT_MISS);


      /* 
       *  Each dataset should have a rank of 2 or 3.  
       *
       *  Single band data will have two dimensions.  All other
       *  data will have three dimensions.
       *
       *  If rank = 2, the dimensions should agree with the cross
       *  and along parameters read from the attributes.
       *
       *  If rank = 3, the dimensions should agree with the cross, 
       *  along, and band parameters read from the attributes.
       *
       *  Report disagreements and exit.
       */
      /* Read and print basic info for each dataset */
      status = SDgetinfo(sds_id, name, &rank_fpdata, dim_fpdata, &fp_type, 
			 &attributes);  
      
      if ((rank_fpdata != 3) && (rank_fpdata !=2))
	{
	  fprintf(logfile,"\nERROR:  Rank not equal to 2 or 3: %d\n", rank_fpdata);
	  GracefulExit();  
         }
      /*
      if (rank_fpdata == 3)
	if ((dim_fpdata[0] != Dataset[Sensor][ds_type].BANDS) ||
	    (dim_fpdata[1] != Dataset[Sensor][ds_type].ALONG) ||
	    (dim_fpdata[2] != Dataset[Sensor][ds_type].CROSS))
	  {
	    fprintf(logfile,"\nERROR:  Dimensions don't agree with attributes:\n");
	    printf("\nERROR:  Dimensions don't agree with attributes:\n");
	    fprintf(logfile,"Dimensions:  %4d %4d %4d\n", dim_fpdata[2],
		   dim_fpdata[1], dim_fpdata[0]);
	    fprintf(logfile,"C-A-B Atts:  %4d %4d %4d\n", 
		   Dataset[Sensor][ds_type].CROSS,
		   Dataset[Sensor][ds_type].ALONG, 
		   Dataset[Sensor][ds_type].BANDS);
	    GracefulExit();  
	  }
      
      if ((rank_fpdata == 2) && (ds_type != MAP))
	if ((dim_fpdata[0] != Dataset[Sensor][ds_type].ALONG) ||
	    (dim_fpdata[1] != Dataset[Sensor][ds_type].CROSS)) 
	  {
	    fprintf(logfile,"\nERROR:  Dimensions don't agree with attributes:\n");
	    printf("\nERROR:  Dimensions don't agree with attributes:\n");
	    fprintf(logfile,"Dimensions:  %4d %4d\n", 
	       dim_fpdata[1], dim_fpdata[0]);
	    fprintf(logfile,"C-A-B Atts:  %4d %4d\n", 
		   Dataset[Sensor][ds_type].CROSS,
		   Dataset[Sensor][ds_type].ALONG);
	    GracefulExit();  
	  }
	  */
    }
  return;

}


/***************************************************************************/
/***************************************************************************/
/*
 *  Name:  WriteDataAttributes
 * 
 *  Description:
 *
 *  Inputs: int32 sensor 
 *          int32 ds_type 
 *	    int32 sd_1r_id
 *
 *  Outputs:
 *
 *  Returns:
 *
 */

void WriteDataAttributes(enum sensor_types Sensor, 
			 enum ds_types DataType,
			 int32 sd_1r_id,
			 int32 MaxFrameShift, 
                         int32 frame_shift[MAX_SENSORS][MAX_BANDS])
{
  enum ds_types DataTypeOut,DataRespOut,DataOffOut;  
  int32 i,j, status;
  int32 scale = SCALE_FACTOR;
  
  int32 rank_1rdata = 3;
  int32 dim_1rdata[3], dim_off_data[3];
  int32 rank_map = 2, dim_map[2];
  char s2[20];



/* Determine proper output dataset names */
switch(DataType)
  {
    
  case LEVEL0:
    DataTypeOut = LEVEL1R;
    DataRespOut = RESPONSE;
    DataOffOut  = OFFSET;	      
    break;
    
  case CAL1:
    DataTypeOut = CAL1R;
    DataRespOut = CAL1_RESP;
    DataOffOut =  CAL1_OFF;	      
    break;
    
  case CAL2:
    DataTypeOut = CAL2R;	      
    DataRespOut = CAL2_RESP;
    DataOffOut =  CAL2_OFF;	      
    break;
    
  case CAL3:
    DataTypeOut = CAL3R;	      
    DataRespOut = CAL3_RESP;
    DataOffOut =  CAL3_OFF;	      
    break;
    
  case CAL4:
    DataTypeOut = CAL4R;	      
    DataRespOut = CAL4_RESP;
    DataOffOut =  CAL4_OFF;	      
    break;
    
  default:
    printf("\nError:  Unknown datatype %d\n", DataType);
    fprintf(logfile,"\nError:  Unknown datatype %d\n", DataType);
    break;
  }


/* create the HDF array data set for level 1R data */
/* and set up the Dataset structure */


dim_1rdata[0] = Dataset[Sensor][DataType].BANDS;
dim_1rdata[1] = Dataset[Sensor][DataType].ALONG + MaxFrameShift;
dim_1rdata[2] = Dataset[Sensor][DataType].CROSS;

/* since the offset data is constant frame to frame, only need to save one */
dim_off_data[0] = Dataset[Sensor][DataType].BANDS;
dim_off_data[1] = 1;
dim_off_data[2] = Dataset[Sensor][DataType].CROSS;

Dataset[Sensor][DataTypeOut].SDS_ID = SDcreate(sd_1r_id,DataName[DataTypeOut], 
					   DFNT_INT16, rank_1rdata, 
					   dim_1rdata);


Dataset[Sensor][DataTypeOut].EXISTS = TRUE;
strcpy(Dataset[Sensor][DataTypeOut].SENSOR, Dataset[Sensor][DataType].SENSOR);
strcpy(Dataset[Sensor][DataTypeOut].DS_TYPE, "Level 1R");
strcpy(units, "mW/cm^2/sr/u/DN");
strcpy(Dataset[Sensor][DataTypeOut].START, Dataset[Sensor][DataType].START);
Dataset[Sensor][DataTypeOut].DURATION = Dataset[Sensor][DataType].DURATION;
if (FLIP)
  {    
    Dataset[Sensor][DataTypeOut].INTEG_TIME_INT = 
      Dataset[Sensor][DataType].INTEG_TIME_INT;
    Dataset[Sensor][DataTypeOut].FRAME_RATE_INT = 
      Dataset[Sensor][DataType].FRAME_RATE_INT;
  }
else
  {    
    Dataset[Sensor][DataTypeOut].INTEG_TIME_FLT = 
      Dataset[Sensor][DataType].INTEG_TIME_FLT;
    Dataset[Sensor][DataTypeOut].FRAME_RATE_FLT = 
      Dataset[Sensor][DataType].FRAME_RATE_FLT;
  }

Dataset[Sensor][DataTypeOut].CROSS = Dataset[Sensor][DataType].CROSS;
Dataset[Sensor][DataTypeOut].ALONG = Dataset[Sensor][DataType].ALONG;
Dataset[Sensor][DataTypeOut].BANDS = Dataset[Sensor][DataType].BANDS;
Dataset[Sensor][DataTypeOut].PRCNT_MISS=Dataset[Sensor][DataType].PRCNT_MISS;
fprintf(logfile, "Dataset %12s   created.  sds_id = %d\n", 
	Dataset[Sensor][DataTypeOut].SENSOR, 
	Dataset[Sensor][DataTypeOut].SDS_ID);


/* create the HDF array data set for dark data */
/* and set up the Dataset structure */

Dataset[Sensor][DataOffOut].SDS_ID = SDcreate(sd_1r_id, DataName[DataOffOut], 
					  DFNT_FLOAT32, rank_1rdata, 
					  dim_off_data);

Dataset[Sensor][DataOffOut].EXISTS = TRUE;
strcpy(Dataset[Sensor][DataOffOut].SENSOR, Dataset[Sensor][DataType].SENSOR);
strcpy(Dataset[Sensor][DataOffOut].DS_TYPE, "Offset");
Dataset[Sensor][DataOffOut].CROSS = Dataset[Sensor][DataType].CROSS;
Dataset[Sensor][DataOffOut].ALONG = Dataset[Sensor][DataType].ALONG;
Dataset[Sensor][DataOffOut].BANDS = Dataset[Sensor][DataType].BANDS;
Dataset[Sensor][DataOffOut].SCALE = scale;
fprintf(logfile, "Dataset %12s   created.  sds_id = %d\n", 
	Dataset[Sensor][DataOffOut].SENSOR, 
	Dataset[Sensor][DataOffOut].SDS_ID);

/* create the HDF array data set for gain data */
/* and set up the Dataset structure */

Dataset[Sensor][DataRespOut].SDS_ID = SDcreate(sd_1r_id,DataName[DataRespOut], 
					    DFNT_FLOAT32, rank_1rdata, 
					    dim_1rdata);

Dataset[Sensor][DataRespOut].EXISTS = TRUE;
strcpy(Dataset[Sensor][DataRespOut].SENSOR, Dataset[Sensor][DataType].SENSOR);
strcpy(Dataset[Sensor][DataRespOut].DS_TYPE, "Response");
Dataset[Sensor][DataRespOut].CROSS = Dataset[Sensor][DataType].CROSS;
Dataset[Sensor][DataRespOut].ALONG = Dataset[Sensor][DataType].ALONG;
Dataset[Sensor][DataRespOut].BANDS = Dataset[Sensor][DataType].BANDS;
Dataset[Sensor][DataRespOut].SCALE = scale;
fprintf(logfile, "Dataset %12s   created.  sds_id = %d\n", 
	Dataset[Sensor][DataRespOut].SENSOR, 
	Dataset[Sensor][DataRespOut].SDS_ID);


/* create the HDF array data set for pixel map data */
/* and set up the Dataset structure */
if ((Sensor == MS) && Dataset[Sensor][MAP].EXISTS)
  {
    dim_map[0] = 1;    
    dim_map[1] = Dataset[Sensor][MAP].CROSS;			 

    Dataset[Sensor][MAPOUT].SDS_ID = SDcreate(sd_1r_id, "Pixel Map", 
	    DFNT_INT16, rank_map, dim_map);
    
    Dataset[Sensor][MAPOUT].EXISTS = TRUE;
    strcpy(Dataset[Sensor][MAPOUT].SENSOR, Dataset[Sensor][MAP].SENSOR);
    strcpy(Dataset[Sensor][MAPOUT].DS_TYPE, Dataset[Sensor][MAP].DS_TYPE);
    Dataset[Sensor][MAPOUT].CROSS = Dataset[Sensor][MAP].CROSS;
  }


/* now set the attributes of the level 1R HDF output file using info 
 * read in from the level 0 HDF input file */

/*first write DataType dataset attributes */

if(status = SDsetattr(Dataset[Sensor][DataTypeOut].SDS_ID, 
		      "Dataset Type",DFNT_CHAR8, 12, 
		      (VOIDP)Dataset[Sensor][DataTypeOut].DS_TYPE) == FAIL) 
  ErrMsg("setattr Dataset type");
if(status = SDsetattr(Dataset[Sensor][DataTypeOut].SDS_ID, 
		      "ALI Sensor",DFNT_CHAR8, 3,
		      (VOIDP)Dataset[Sensor][DataTypeOut].SENSOR) == FAIL) 
  ErrMsg("setattr Sensor");
if(status = SDsetattr(Dataset[Sensor][DataTypeOut].SDS_ID, 
		      "Units",DFNT_CHAR8, 25,
		      (VOIDP)units) == FAIL) 
  ErrMsg("setattr units");
if(status = SDsetattr(Dataset[Sensor][DataTypeOut].SDS_ID, 
		      "Data Start Time",DFNT_CHAR8, 14,
		      (VOIDP)Dataset[Sensor][DataTypeOut].START) == FAIL) 
  ErrMsg("setattr data start time");
if(status = SDsetattr(Dataset[Sensor][DataTypeOut].SDS_ID, 
		      "Data Duration",DFNT_FLOAT32, 1,
		      (VOIDP)(&Dataset[Sensor][DataTypeOut].DURATION)) == FAIL) 
  ErrMsg("setattr data duration");

if(FLIP)
  {    
    if(status = SDsetattr(Dataset[Sensor][DataTypeOut].SDS_ID, 
			  "Integration Time",DFNT_INT16, 1,
			  (VOIDP)(&Dataset[Sensor][DataTypeOut].INTEG_TIME_INT)) 
       == FAIL) 
      ErrMsg("setattr # integration time");
  }
else
  {    
    if(status = SDsetattr(Dataset[Sensor][DataTypeOut].SDS_ID, 
			  "Integration Time",DFNT_FLOAT32, 1,
			  (VOIDP)(&Dataset[Sensor][DataTypeOut].INTEG_TIME_FLT)) 
       == FAIL) 
      ErrMsg("setattr # integration time");
  }

if (FLIP)
  {
    if(status = SDsetattr(Dataset[Sensor][DataTypeOut].SDS_ID, 
			  "Frame Rate",DFNT_INT16, 1,
			  (VOIDP)(&Dataset[Sensor][DataTypeOut].FRAME_RATE_INT)) 
       == FAIL) 
      ErrMsg("setattr # frame rate");
  }
else
  {
    if(status = SDsetattr(Dataset[Sensor][DataTypeOut].SDS_ID, 
			  "Frame Rate",DFNT_FLOAT32, 1,
			  (VOIDP)(&Dataset[Sensor][DataTypeOut].FRAME_RATE_FLT)) 
       == FAIL) 
      ErrMsg("setattr # frame rate");
  }

if(status = SDsetattr(Dataset[Sensor][DataTypeOut].SDS_ID, 
		      "Number of cross track pixels",DFNT_INT16, 1,
		      (VOIDP)(&Dataset[Sensor][DataTypeOut].CROSS)) == FAIL) 
  ErrMsg("setattr # cross track");
if(status = SDsetattr(Dataset[Sensor][DataTypeOut].SDS_ID, 
		      "Number of along track pixels",DFNT_INT32, 1,
		      (VOIDP)(&Dataset[Sensor][DataTypeOut].ALONG)) == FAIL) 
  ErrMsg("setattr # along track");
if(status = SDsetattr(Dataset[Sensor][DataTypeOut].SDS_ID, 
		      "Number of bands",DFNT_INT16, 1,
		      (VOIDP)(&Dataset[Sensor][DataTypeOut].BANDS)) == FAIL) 
  ErrMsg("setattr # bands");
if(status = SDsetattr(Dataset[Sensor][DataTypeOut].SDS_ID, 
		      "Percent Missing Data",DFNT_INT16, 1,
		      (VOIDP)(&Dataset[Sensor][DataTypeOut].PRCNT_MISS)) == FAIL) 
  ErrMsg("setattr percent missing data");

for (i =0; i < Dataset[Sensor][DataTypeOut].BANDS; ++i)
  {
    strcpy(s2,"Frame shift ");    
    strcat(s2,BandName[Sensor][i]);    
    
    if(status = SDsetattr(Dataset[Sensor][DataTypeOut].SDS_ID, 
			  s2,DFNT_INT32, 1,
			  (VOIDP)(&frame_shift[Sensor][i])) == FAIL) 
      ErrMsg("setattr frame shift data");
  }


/*2nd write OFFSET dataset attributes */

if(status = SDsetattr(Dataset[Sensor][DataOffOut].SDS_ID, 
		      "Dataset Type",DFNT_CHAR8, 12, 
		      (VOIDP)Dataset[Sensor][DataOffOut].DS_TYPE) == FAIL) 
  ErrMsg("setattr Dataset type");
if(status = SDsetattr(Dataset[Sensor][DataOffOut].SDS_ID, 
		      "ALI Sensor",DFNT_CHAR8, 3,
		      (VOIDP)Dataset[Sensor][DataOffOut].SENSOR) == FAIL) 
  ErrMsg("setattr Sensor");
if(status = SDsetattr(Dataset[Sensor][DataOffOut].SDS_ID, 
		      "Number of cross track pixels",DFNT_INT16, 1,
		      (VOIDP)(&Dataset[Sensor][DataOffOut].CROSS)) == FAIL) 
  ErrMsg("setattr # cross track");
if(status = SDsetattr(Dataset[Sensor][DataOffOut].SDS_ID, 
		      "Number of along track pixels",DFNT_INT32, 1,
		      (VOIDP)(&Dataset[Sensor][DataOffOut].ALONG)) == FAIL) 
     		ErrMsg("setattr # along track");
if(status = SDsetattr(Dataset[Sensor][DataOffOut].SDS_ID, 
		      "Number of bands",DFNT_INT16, 1,
		      (VOIDP)(&Dataset[Sensor][DataOffOut].BANDS)) == FAIL) 
  ErrMsg("setattr # bands");
if(status = SDsetattr(Dataset[Sensor][DataOffOut].SDS_ID, 
		      "Scale Factor",DFNT_INT16, 1,
		      (VOIDP)(&Dataset[Sensor][DataOffOut].SCALE)) == FAIL) 
  ErrMsg("setattr scale");


/*3rd write DataRespOut dataset attributes */

if(status = SDsetattr(Dataset[Sensor][DataRespOut].SDS_ID, 
		      "Dataset Type",DFNT_CHAR8, 12, 
		      (VOIDP)Dataset[Sensor][DataRespOut].DS_TYPE) == FAIL) 
  ErrMsg("setattr Dataset type");
if(status = SDsetattr(Dataset[Sensor][DataRespOut].SDS_ID, 
		      "ALI Sensor",DFNT_CHAR8, 3,
		      (VOIDP)Dataset[Sensor][DataRespOut].SENSOR) == FAIL) 
  ErrMsg("setattr Sensor");
if(status = SDsetattr(Dataset[Sensor][DataRespOut].SDS_ID, 
		      "Number of cross track pixels",DFNT_INT16, 1,
		      (VOIDP)(&Dataset[Sensor][DataRespOut].CROSS)) == FAIL) 
  ErrMsg("setattr # cross track");
if(status = SDsetattr(Dataset[Sensor][DataRespOut].SDS_ID, 
		      "Number of along track pixels",DFNT_INT32, 1,
		      (VOIDP)(&Dataset[Sensor][DataRespOut].ALONG)) == FAIL) 
  ErrMsg("setattr # along track");
if(status = SDsetattr(Dataset[Sensor][DataRespOut].SDS_ID, 
		      "Number of bands",DFNT_INT16, 1,
		      (VOIDP)(&Dataset[Sensor][DataRespOut].BANDS)) == FAIL) 
  ErrMsg("setattr # bands");
if(status = SDsetattr(Dataset[Sensor][DataRespOut].SDS_ID, 
		      "Scale Factor",DFNT_INT16, 1,
		      (VOIDP)(&Dataset[Sensor][DataRespOut].SCALE)) == FAIL) 
  ErrMsg("setattr scale");

fprintf(logfile,"\n");
fprintf(logfile,"Finished writing dataset attributes for %s\n",
       Dataset[Sensor][DataRespOut].SENSOR);


/*4th write PIXEL MAP dataset attributes */
if ((Sensor == MS) && Dataset[Sensor][MAPOUT].EXISTS)
  {
    
    if(status = SDsetattr(Dataset[Sensor][MAPOUT].SDS_ID, 
			  "Dataset Type",DFNT_CHAR8, 12, 
			  (VOIDP)Dataset[Sensor][MAPOUT].DS_TYPE) == FAIL) 
      ErrMsg("setattr Dataset type");
    if(status = SDsetattr(Dataset[Sensor][MAPOUT].SDS_ID, 
			  "ALI Sensor",DFNT_CHAR8, 3,
			  (VOIDP)Dataset[Sensor][MAPOUT].SENSOR) == FAIL) 
      ErrMsg("setattr Sensor");
    if(status = SDsetattr(Dataset[Sensor][MAPOUT].SDS_ID, 
			  "Number of cross track pixels",DFNT_INT16, 1,
			  (VOIDP)(&Dataset[Sensor][MAPOUT].CROSS)) == FAIL) 
      ErrMsg("setattr # cross track");
  }


}



/************************************************************************/
/************************************************************************/

