/*
 *  File:  cp_input.c       
 *
 *  Description:  File with routines for reading in Level 0 data,
 *                reading the file attributes, and checking for
 *                consistency between the attributes and the dataset
 *                parameters.
 *
 *  Subroutines:  ReadAttributes
 *                CheckFile
 *
 *  Author:	  Jenifer Brinker Evans
 *
 *  Date	Author		Action
 * -----------------------------------------------------------------------
 * 02/19/98	JBE		Version 1.1 complete
 * 02/24/98 	MDG		added more attribute reading
 * 03/24/98 	MDG		subtracted out dataset attributes from 
 *					ReadAttributes routine 
 * 				took out dimensions checking from
 *					CheckFile routine
 * 04/02/98	MDG		separated File & Dataset attributes
 *  4/13/98     JBE             Completed conversion to Dataset structure
 *                                with enumerated datatypes.  Checkfile
 *                                no longer being called.  Will be someday...
 *  4/24/98     JBE             Replace print statements with prints to
 *                                an external log file.
 * 10/01/98     JBE             Small corrections to assure attributes match
 *                                ICD word for word.
 *  2/14/01     JBE             Added processing for two more attributes
 *                                to be added on 2/15/01
 */


#include "hdf.h"
#include "mfhdf.h"
#include "cp_includes.h"

extern PARAM_IN FileAttr;
extern DATA_SET Dataset[];
extern char *GenerateAttName();
extern char *GenerateFillAttName();
extern int32 FLIP;
extern FILE *logfile;


/***************************************************************************/
/***************************************************************************/
/*
 *  Name:  ReadFileAttributes
 * 
 *  Description:  Reads file attribute values.  Uses attribute names as
 *                defined in the ICD.  Attribute values are recorded in
 *                global variable FileAttr.
 *
 *  Inputs:       sd_id     int32     file id
 *
 *  Outputs:      modifies FileAttr values.
 *
 *  Returns:      number of datasets in file
 *
 */

int32 ReadFileAttributes(int32 sd_id)
{
  int32 index,i;
  int32 num_datasets, num_attr;
  char att_name[30];
  

   if (SDfileinfo(sd_id, &num_datasets, &num_attr) == FAIL)
    {
      fprintf(logfile,"\nERROR:  Error reading file info\n");
      GracefulExit();  
    }
  else
    {
      fprintf(logfile,"\n# of datasets   = %2d\n", num_datasets);
      fprintf(logfile,"# of attributes = %2d\n", num_attr);
    }


  /* Read attribute information */
  
  index = SDfindattr(sd_id, "Filename");
  SDreadattr(sd_id, index, (VOIDP)FileAttr.NAME);
  fprintf(logfile,"Filename: %s\n", FileAttr.NAME);

  index = SDfindattr(sd_id, "Data Product Level");
  SDreadattr(sd_id, index, &FileAttr.LEVEL);
  fprintf(logfile,"Level: %d\n", FileAttr.LEVEL);

  index = SDfindattr(sd_id, "LZP Software Version");
  SDreadattr(sd_id, index, (VOIDP)FileAttr.LZP_VERSION);
  fprintf(logfile,"Version: %d.%d.%d\n", FileAttr.LZP_VERSION[0],FileAttr.LZP_VERSION[1],
				FileAttr.LZP_VERSION[2]);

  index = SDfindattr(sd_id, "Big (1) or Little (0) Endian");
  SDreadattr(sd_id, index, &FileAttr.ENDIAN);
  fprintf(logfile,"Endian: %d\n", FileAttr.ENDIAN);

  index = SDfindattr(sd_id, "Time of File Generation");
  SDreadattr(sd_id, index, (VOIDP)FileAttr.GEN_TIME);
  fprintf(logfile,"Generation time: %s\n", FileAttr.GEN_TIME);

  index = SDfindattr(sd_id, "Number Related Files");
  SDreadattr(sd_id, index, &FileAttr.NUM_REL);
  fprintf(logfile,"Num Related files: %d\n", FileAttr.NUM_REL);
  
for (i = 0; i < FileAttr.NUM_REL; ++i)
  {
    GenerateAttName(i, att_name); 
    index = SDfindattr(sd_id, att_name);
    SDreadattr(sd_id, index, (VOIDP)FileAttr.RNAME[i].filename);
    fprintf(logfile,"Related file: %s\n", FileAttr.RNAME[i].filename);
  }

index = SDfindattr(sd_id, "Level 0 Processing Notes");
SDreadattr(sd_id, index, (VOIDP)FileAttr.NOTES);
fprintf(logfile,"Level 0 Notes: %s\n", FileAttr.NOTES);

/* These two attributes were added 2/15/01.  If the command line
   argument -f is set, it is an old file so do not try to read
   these attributes. */
if (!FLIP)
  {
    
    index = SDfindattr(sd_id, "Scene_Id");
    SDreadattr(sd_id, index, (VOIDP)FileAttr.SCENE_ID);
    fprintf(logfile,"Scene ID: %s\n", FileAttr.SCENE_ID);
    
    index = SDfindattr(sd_id, "Target_Description");
    SDreadattr(sd_id, index, (VOIDP)FileAttr.TARG_DESC);
    fprintf(logfile,"Target Description: %s\n", FileAttr.TARG_DESC);
  }

return(num_datasets);

}



/***************************************************************************/
/***************************************************************************/

/*
 *  Name:  CheckFile
 * 
 *  Description:  Performs some simple error checking.  Confirms number
 *                of datasets is proper.  Confirms attributes don't
 *                contradict dataset parameters.  Opens each dataset
 *                as part of error checking.  
 *
 *  Inputs:       sd_id       int32      file id
 *                
 *  Outputs:      sds_id[]    int32      array of dataset ids
 *
 *  Returns:      nothing
 *
 */

void CheckFile(int32 sd_id)
{
  char  name[50];         
  int32 num_datasets, num_attr, attributes;
  int32 rank_fpdata, dim_fpdata[3],fp_type;
  int32 status;
  int32 i;
  
  /* 
   * WIS and GIS files should have one dataset.  MS/PAN files
   *  can have more.
   */
  
  if (SDfileinfo(sd_id, &num_datasets, &num_attr) == FAIL)
    {
      fprintf(logfile,"\nERROR:  Error reading file info\n");
      GracefulExit();  
    }
  else
    {
      fprintf(logfile,"# of datasets   = %2d\n", num_datasets);
      fprintf(logfile,"# of attributes = %2d\n", num_attr);
    }
  

  return;
  
  
}





      
/***************************************************************************/
/***************************************************************************/


















