/*
 *  File:  cp_main.c  
 *
 *  Description:  This file is the main routine for the calibration pipeline.
 *                It asks for the Level0 input file then calls the basic
 *                processing routines for each dataset in the input file.
 *
 *  Author:	  Jenifer Brinker Evans
 *
 *  Date	Author		Action
 * -----------------------------------------------------------------------
 *  2/17/98	JBE		Version 1.1 complete
 *  2/24/98	MDG		V1.2 added some IO routines
 *  3/24/98	MDG		V1.3 added ReadDataAttributes routine
 *  4/02/98	MDG		V1.4 separated File & Dataset attributes
 *  4/06/98	MDG		     use cp_consts.h instead of cal_pipe.h
 *  4/06/98	MDG		     use cp_structs.h instead of param.h
 *  4/08/98	MDG		V1.5 fixed "for i,j" loop to check on j
 *					rather than i in "if" statements
 *  4/13/98     JBE             Eliminated need for sds_id arrays.  Finished
 *                                   conversion to Dataset structure.
 *                                   Replace cp*.h with cp_inlcludes.h
 *  4/24/98     JBE             Establish an external log file.
 *  3/24/99     JBE             Replaced cp.log with "input filename".log
 *  3/26/99	MDG		v 2.7 Added date stamp etc to CP_NOTES
 *  3/26/99	MDG		v 2.8 logfile fixes
 *  3/26/99	MDG		v 2.9 add date stamp to log file
 *  1/31/01     JBE             Adding print out indicating version number.
 *  4/05/01     JBE             Added version number to print out for -h
 *  8/30/01     JBE             Incremented version number.  Other changes to 
 *                               program include addition of units data attr.,
 *                               correction of offset output error, mod
 *                               to allow same default dark for old and new
 *                               formats, and SCA4, Band 2 flipped correction.
 */



#include "hdf.h"
#include "mfhdf.h"
#include "cp_includes.h"
#include <sys/types.h>



PARAM_IN FileAttr;
DATA_SET Dataset[MAX_SENSORS][MAX_DS_TYPES];
GAIN_DB DatabaseCoeff[MAX_SENSORS][MAX_COEFF_TYPES];
int32 sd_id, sd_id_db, sd_1r_id;
int32 FLIP = FALSE;
FILE *logfile;



main(int argc, char **argv)
{


int32 num_datasets;
enum sensor_types Sensor;
enum ds_types DataType;
char raw_data_file[100], cal_dbase_file[100], log_file[100];
int i,j,n, sca_num, status;
float speed=0, version;
FILE *temp;

int32 frame_shift[MAX_SENSORS][MAX_BANDS], MaxFrameShift;


char time_string[28];  /* for date stamping the CAL processing notes*/
time_t tloc;
time_t clock;
char extra_cp_notes[500];


void GetCommandLineArgs(int, char **, char *, char *, float *, float);
void GetLogfileName(char *, char *);
void PrepareOutputFile(int32 *, float);

/* Date 8/30/01 delivery */
/* Date 5/14/02 remove selected attributes */
version = 4.61;

GetCommandLineArgs(argc, argv, raw_data_file, cal_dbase_file, &speed, version);
GetLogfileName(raw_data_file, log_file);

/* Speed is assumed to be 0.04 mm for now. Will become command
line option -s */
/* speed = 0.04; */


/* Open a log file for all printed error, warning, and status messages. */
logfile = fopen(log_file,"w");
if( (logfile = fopen(log_file, "w" )) == NULL )
  {
    printf("\nError:  Can't open log file %s\n", log_file);	      
    exit(-1);
  }

/* Print command for version number.  Version number is updated manually. */
printf("Running Version %f of radiometric calibration pipeline\n", version);
fprintf(logfile,"Running Version %f of radiometric calibration pipeline\n", version);


/* Open raw data file */
sd_id = SDstart(raw_data_file, DFACC_RDONLY);
fprintf(logfile,"Opened %s.  sd_id: %d\n", raw_data_file, sd_id);


/* Open database file */
sd_id_db = OpenGainDatabase(cal_dbase_file);

/* Copy date and time to Cal Processing Notes.*/
clock = time(&tloc);
strcpy(time_string, (char *) ctime(&clock));
strcpy(FileAttr.CP_NOTES, "Date:  ");
strcat(FileAttr.CP_NOTES, time_string);
fprintf(logfile, "\nDate:  %s\n", time_string);

/* Copy name of database file to Cal Processing Notes.*/
strcat(FileAttr.CP_NOTES, " Database used:  ");
strcat(FileAttr.CP_NOTES, cal_dbase_file);

/* Ask user for interactive input for Cal Processing Notes. */
/*
strcpy(extra_cp_notes, " ");
printf("Enter any extra Cal Processing notes for Level 1 output data:\n");
gets(extra_cp_notes);
strcat(FileAttr.CP_NOTES, "     ");
strcat(FileAttr.CP_NOTES, extra_cp_notes);
*/

/* Determine number of datasets in Level 0 file */
/* Read file and dataset attributes */
num_datasets = ReadFileAttributes(sd_id);
ReadDataAttributes(num_datasets, sd_id); 


/* Open Level 1R output file.  Write file attributes. */
PrepareOutputFile(&sd_1r_id, version); 

/* Determine SCA number.  Necessary for leaky pixel correction.*/
n = strlen(FileAttr.NAME);
sca_num = FileAttr.NAME[n-2] - '0';

/* Compute frame shifts necessary for image alignment */
MaxFrameShift = ComputeFrameShifts(frame_shift, sca_num, speed);


/* Process science data for each type of sensor.*/
for (Sensor = MS; Sensor < MAX_SENSORS; ++Sensor)
  for (DataType = LEVEL0; DataType <= CAL4; ++DataType)
    if (Dataset[Sensor][DataType].EXISTS)
      {
	/* Write out the dataset attributes to the level1R file */
	WriteDataAttributes(Sensor, DataType, sd_1r_id, 
			    MaxFrameShift,frame_shift); 
	
	ProcessDataset(Sensor, DataType, sca_num, frame_shift, MaxFrameShift);
      }


/* Copy MS Pixel Map */
CopyPixelMap();



/* end access to all SDS datasets */
for (i=0; i < MAX_SENSORS; ++i)
  {
    fprintf(logfile,"\n");    
    for (j=0; j < MAX_COEFF_TYPES; ++j)
      if (DatabaseCoeff[i][j].EXISTS)
	{	
	  SDendaccess(DatabaseCoeff[i][j].SDS_ID);
	  fprintf(logfile,"Closed database %d for sensor: %d.  sds_id: %d\n", j,i, 
		  DatabaseCoeff[i][j].SDS_ID);
	}
    
    
    for (j=0; j < MAX_DS_TYPES; ++j)
      {
	if (Dataset[i][j].EXISTS)
	  {	    
	    SDendaccess(Dataset[i][j].SDS_ID);
	  }
	
      }
  }


/* and close access to the HDF files */
SDend(sd_id);
fprintf(logfile,"Closed sd_id: %d\n", sd_id);

SDend(sd_id_db);
fprintf(logfile,"Closed sd_id_db: %d\n", sd_id_db);

SDend(sd_1r_id);
fprintf(logfile,"Closed sd_1r_id: %d\n", sd_1r_id);

printf("Successful Completion\n");
fprintf(logfile,"\nSuccessful Completion\n");

exit(0);
}





/***************************************************************************/
/***************************************************************************/
/*
 *  Name:  GracefulExit
 * 
 *  Description:  Routine to gracefully exit after an error.  Closes all
 *                scientific datasets (SDS's) and scientfic data files (SD's).
 *                Assures any useful data already computed is not lost.
 *
 *  Inputs:  none
 *
 *  Outputs:  none
 *
 *  Returns:  nothing
 *
 */

void GracefulExit()

{
  int32 i,j;
  
  printf("Gracefully exiting.  Check logfile for details.\n");
  
  fflush(logfile);
  

/* end access to all SDS datasets */
for (i=0; i < MAX_SENSORS; ++i)
  {
    for (j=0; j < MAX_COEFF_TYPES; ++j)
      if (DatabaseCoeff[i][j].EXISTS)
	{	
	  SDendaccess(DatabaseCoeff[i][j].SDS_ID);
	  fprintf(logfile,"Closed database %d for sensor: %d.  sds_id: %d\n",j, i, 
		  DatabaseCoeff[i][j].SDS_ID);
	}
    
    
    for (j=0; j < MAX_DS_TYPES; ++j)
      {
	if (Dataset[i][j].EXISTS)
	  {	    
	    SDendaccess(Dataset[i][j].SDS_ID);
	  }
	
      }
  }
  
  /* and close access to the HDF files */
  SDend(sd_id);
  fprintf(logfile,"Closed sd_id: %d\n", sd_id);
  
  SDend(sd_id_db);
  fprintf(logfile,"Closed sd_id_db: %d\n", sd_id_db);
  
  SDend(sd_1r_id);
  fprintf(logfile,"Closed sd_1r_id: %d\n", sd_1r_id);
  
  fflush(logfile);

  exit(-1);
}






/***************************************************************************/
/***************************************************************************/
/*
 *  Name:  GetCommandLineArgs
 * 
 *  Description:  Reads two input file names from command line.  If there are
 *                more than two file names, it prints an error and usage message.
 *                If there is one file name, it assumes the file is the raw
 *                data Level 0 file, and prompts for a second file name.  If
 *                there are no file names, it prompts for both file names.
 *
 *  Inputs:       argc             int         number of command line arguments
 *                argv             char **     pointers to strings from command line
 *               
 *  Outputs:      file_in_1        char        string with input file name 1
 *                file_in_2        char        string with input file name 2
 *
 *  Returns:      nothing
 *
 */


void GetCommandLineArgs(int argc, char **argv, char file_in_1[50], char file_in_2[50], float * speed, float version)
{
  void print_usage(float);
  int32 i,FirstName,SecondName;
  float temp;
  
 
  /* Command functions begin with a dash.
     File names begin with non-dash characters.
     The first file name read in is assumed to be the input file.
     The second file name is assumed to be the database file.
     */
  
  *speed = 0.04;  
  FirstName = TRUE;
  
  for(i=1; i < argc; ++i)
    {
      if (argv[i][0] == '-')
	{
	  switch(argv[i][1])
	    {
	      
	    case 'f':		/* for older data before flipped left-right */
	      FLIP = TRUE;
	      printf("Using FLIP option for older files.\n");	      
	      break;
	      
	    case 's':		/* override default frame speed for alignment*/
	      sscanf(argv[i+1],"%f",&temp);
	      *speed = temp;
	      ++i;	  
	      printf("Default speed overridden.  New speed: %f\n", *speed);
	      break;
	      
	    case 'h':		/* override default file spec */
	      print_usage(version);
	      break;
	      
	    default:
	      printf("\nError:  unknown option \'%s\'\n", argv[1]);
	      print_usage(version);
	      break;
	    }
	}
      
      else
	if (FirstName)
	  {
	    FirstName = FALSE;
	    SecondName = TRUE;
	    
	    strcpy(file_in_1, argv[i]);
	    printf("\nLevel 0 input data in file <%s>\n\n", file_in_1);
	  }
      else
	if (SecondName)
	  {
	    SecondName = FALSE;
	    
	    strcpy(file_in_2, argv[i]);
	    printf("\nDatabase in file <%s>\n\n", file_in_2);
	  }
      else
	{
	  printf("\nError:  Too many input parameters.\n");
	  print_usage(version);      
	}
     }
	  
  /* If filenames not set yet, then set. */
  if (FirstName) 
    {
      printf("\nFor convenience:\n");      
      printf("Usage:  cp_main <filename1> <filename2>\n");
      printf("        filename1:  name of HDF file with Level 0 data\n");
      printf("        filename2:  name of HDF file with cal database\n");
      printf("Since filenames are not on the command line, they will be prompted for.\n\n");
      
      printf("Enter file name for Level 0 input data:\n");
      scanf("%s", file_in_1);
      printf("Enter file name for cal database response coefficients:\n");
      scanf("%s", file_in_2);
      
      /* Set flags to indicate filenames are set. */
      FirstName = FALSE;      
      SecondName = FALSE;      
    }
  
  if (SecondName)
    {
      printf("File name for database not read.\n");    
      printf("Enter file name for cal database response coefficients:\n");
      scanf("%s", file_in_2);
      
      /* Set flags to indicate filenames are set. */
      SecondName = FALSE;      
    }
  
  
  /* if not command line arguments were entered, get filenames */
  if ((argc == 1) && (FirstName))
    {
      printf("\nFor convenience:\n");      
      printf("Usage:  cp_main <filename1> <filename2>\n");
      printf("        filename1:  name of HDF file with Level 0 data\n");
      printf("        filename2:  name of HDF file with cal database\n");
      printf("Since filenames are not on the command line, they will be prompted for.\n\n");
      
      printf("Enter file name for Level 0 input data:\n");
      scanf("%s", file_in_1);
      printf("Enter file name for cal database response coefficients:\n");
      scanf("%s", file_in_2);
    }
  
  return;
}


	


/***************************************************************************/
/***************************************************************************/
/*
 *  Name: GetLogfileName
 * 
 *  Description:  Takes name of raw data file, removes all references to
 *                other directories, uses just the rootname and suffixes
 *                and generates the name for the log file.
 *
 *  Inputs:  raw_data_file   char *       Name of input file
 *
 *  Outputs: log_file        char *       Name of output log file
 *
 *  Returns: Nothing
 *
 */



void GetLogfileName(char *raw_data_file, char *log_file)
{
  char null_char, slash[5], log_suffix[5];
  int i,j,n,length;
  char long_name[100], temp_name[100];
  
  /* Define a character for the backslash that divides directories, and Null */
  strcpy(slash,"/");
  null_char='\0';
  
  /* Determine length of raw_data_file name and copy to string for modifying*/
  length = strlen(raw_data_file);  
  strcpy(long_name,raw_data_file);

  /* Determine where in string first slash occurs */
  /* If no slash, n will equal length */
  n = strcspn(long_name,slash);

  
  /* While there remain slashes, do the following */
  while (n != length)
      { 
      j = 0;  
      /* Copy all characters after the slash into temp name and add Null */
      for (i = n+1; i<length; ++i)
	{
	  temp_name[j] = long_name[i];
	  ++j;
	}
      temp_name[j] = null_char;

      /* Copy temp name back to working name */
      strncpy(long_name,temp_name,j+1);

      /* Repeat determination of length and location of slash */
      length = j;  
      n = strcspn(long_name,slash);
      }
  
      
  /* Append .log suffix to remaining file name */
  strcpy(log_suffix,".log");
  strcpy(log_file,long_name);
  strncat(log_file, log_suffix,4);
  
}



/***************************************************************************/
/***************************************************************************/
void print_usage(float version)

{
  printf("\nRunning Version %3.1f of radiometric calibration pipeline\n\n", 
	 version);
  printf("Usage:  cp_main <filename1> <filename2>\n");
  printf("        filename1:  name of HDF file with Level 0 data\n");
  printf("        filename2:  name of HDF file with cal database\n");
  printf("If filenames are not entered on command line, they will be prompted for.\n\n");

  printf("Other options:\n");
  printf("     -f:   for older files before left to right flip (before 2/15/01)\n");
  printf("     -s:   ground speed of sensor.  Affects alignment.  Default is 0.04 mm\n");  
  printf("     -h:   help.  Prints this usage message.\n");
  

  exit(-1);	
}








  

