/*
 *  File:  cp_prepare.c
 *
 *  Description:  This file has routines for preparing the output file
 *                of the calibration pipeline.  The routine creates
 *                an HDF data file & writes out the attributes
 *
 *  Subroutines:  PrepareOutputFile
 *                GenerateAttName
 *                GenerateFillAttName
 *                ErrMsg
 *
 *  Author:	  Margaret Dunham Gibbs
 *
 *  Date	Author		Action
 * -----------------------------------------------------------------------
 * 02/23/98	MDG		Version 1.1 complete
 * 03/26/98     MDG             V1.2 moved the creation of HDF array
 *                                 datasets from "PrepareOutputFile"
 *                                 to "WriteDataAttributes
 * 04/02/98	MDG		V1.3 separated file & dataset attributes
 *  4/13/98     JBE             Finished conversion to Dataset structure
 *                                 with enumerated datatypes.
 *  4/24/98     JBE             Replace print statements with prints to
 *                                an external log file.
 * 10/01/98     JBE             Small corrections to assure attributes match
 *                                ICD word for word.
 *  2/14/01     JBE             Added attributes for Scene_Id and Target_Desc
 *                                These only exist after 2/15/01 so they are
 *                                a function of command line argument.
 *  2/14/01     JBE             Changed handling of version number.  Now
 *                                controlled by cp_main.
 *  2/14/01     JBE             Added attribute for output of database name.
 */


#include "hdf.h"
#include "mfhdf.h"
#include "cp_includes.h"

extern PARAM_IN FileAttr;
extern DATA_SET Dataset[];
extern int32 FLIP;
extern FILE *logfile;

void GenerateAttName();
void ErrMsg();

/***************************************************************************/
/***************************************************************************/
/*
 *  Name:  PrepareOutputFile
 * 
 *  Description:
 *
 *  Inputs: 
 *     int32 *sd_1r_id;		ID for level1R HDF output file
 *
 *  Outputs:  
 *
 *  Returns:  
 *
 */

void PrepareOutputFile(int32 *sd_1r_id, float version)
{  
int32 rank_1rdata = 3;
int32 dim_1rdata[3];
int32 rank_pixel = 2;
int32 dim_pixel[2];

short level;

int status;
int i, index, k, length;

char att_name[30];
char outfilename[31];
char dce_type[4];
char blank = ' ';

int index2(), index3();

/*set up version number, etc for sd attributes*/
level = 1;
strcpy(dce_type, "SCI");
FileAttr.CP_VERSION = version;


memset(outfilename, 0, 31);

/* get name of old level 0 HDF input file*/
strncpy(outfilename, FileAttr.NAME, 30);
printf("input file name is %s\n",outfilename);
if (!FLIP)
  {
    /* use the sceneid.M?R as the output file name */
    /* sceneID is modified to include A for ALI */
    char *dot;
    memset(outfilename, 0, 31);
    strcpy(outfilename, "EO1A");
    strncpy(outfilename+4, FileAttr.SCENE_ID+3, 18);
    outfilename[22] = '\0';
    /* get file file name extension from the original file */
    dot = strchr(FileAttr.NAME, '.');
    if (dot && ((strlen(outfilename) + strlen(dot)) <= 30))
      strcat(outfilename, dot);
  }
k = strlen(outfilename);  /*get length of file name*/
if (outfilename[k-1] == 'Z')
  outfilename[k-1] = 'R';	/*change end of file name to reflect level 1R data*/
printf("output file name is %s\n",outfilename);

/* create the level 1R HDF output file */

*sd_1r_id = SDstart (outfilename, DFACC_CREATE);

/* set the attributes of the level 1R HDF output file using info read in from
 * the level 0 HDF input file */

if(status = SDsetattr(*sd_1r_id, "Filename",DFNT_CHAR8, 30,
  (VOIDP)outfilename) == FAIL) ErrMsg("setattr Filename");
if(status = SDsetattr(*sd_1r_id, "Data Product Level",DFNT_INT16, 1,
  (VOIDP)(&level)) == FAIL) ErrMsg("setattr Data Product Level");
if(status = SDsetattr(*sd_1r_id, "LZP Software Version",DFNT_INT16, 3,
  (VOIDP)FileAttr.LZP_VERSION) == FAIL) ErrMsg("setattr LZP Software Version");
if(status = SDsetattr(*sd_1r_id, "Big (1) or Little (0) Endian",DFNT_INT16, 1,
  (VOIDP)(&FileAttr.ENDIAN))== FAIL) ErrMsg("setattr Big or Little Endian");
if(status = SDsetattr(*sd_1r_id, "Time of File Generation",DFNT_CHAR8, 14,
  (VOIDP)FileAttr.GEN_TIME) == FAIL) ErrMsg("setattr time of generation");
if(status = SDsetattr(*sd_1r_id, "Type of DCE",DFNT_CHAR8, 4,
  (VOIDP)dce_type) == FAIL) ErrMsg("setattr type of dce");
#if 0
if(status = SDsetattr(*sd_1r_id, "Number Related Files",DFNT_INT16, 1,
  (VOIDP)&FileAttr.NUM_REL) == FAIL) ErrMsg("setattr number related files");
for (i = 0; i < FileAttr.NUM_REL; ++i)
  {
    GenerateAttName(i, att_name);    
    if(status = SDsetattr(*sd_1r_id, att_name, DFNT_CHAR8, 30, 
			  (VOIDP)FileAttr.RNAME[i].filename) == FAIL) 
      ErrMsg("setattr related filenames");
  }
#endif

if (!FLIP)
  {
    if(status = SDsetattr(*sd_1r_id, "Scene_Id",DFNT_CHAR8, 21,
			  (VOIDP)FileAttr.SCENE_ID) == FAIL) 
      ErrMsg("setattr Scene ID");
#if 0
    if(status = SDsetattr(*sd_1r_id, "Target_Description",DFNT_CHAR8, 65,
			  (VOIDP)FileAttr.TARG_DESC) == FAIL) 
      ErrMsg("setattr target description");
#endif
  }


length = strlen(FileAttr.NOTES);
while ((blank == FileAttr.NOTES[length-1]) && (length > 1))
	length = length - 1;

if(status = SDsetattr(*sd_1r_id, "Level 0 Processing Notes",DFNT_CHAR8, length,
  (VOIDP)FileAttr.NOTES) == FAIL) ErrMsg("setattr Level 0 Notes");
if(status = SDsetattr(*sd_1r_id, "Cal Pipe Software Version",DFNT_FLOAT32, 1,
  (VOIDP)&FileAttr.CP_VERSION)==FAIL) ErrMsg("setattr Cal Pipe Software Version");

length = strlen(FileAttr.CP_NOTES);
if(status = SDsetattr(*sd_1r_id, "Cal Pipe Processing Notes",DFNT_CHAR8,length,
  (VOIDP)FileAttr.CP_NOTES) == FAIL) ErrMsg("setattr CP Notes");


}  /*end of PrepareOutputFile*/
    
    



/************************************************************************/
/************************************************************************/

void ErrMsg(message)
     char message[];

{  
  fprintf(logfile,"ERROR:  Run-time error:\n");  
  fprintf(logfile,"   %s\n", message);
  fprintf(logfile,"exiting...\n");  
  GracefulExit();  
}


/***************************************************************************/
/***************************************************************************/
/*
 *  Name:  GenerateAttName
 * 
 *  Description:  This function takes an integer, i, and generates a
 *                string to be used as an attribute name.  The attribute
 *                name is "Related File i" where i is the integer converted
 *                to the proper characters.
 *
 *  Inputs:       i        int32     integer to be converted
 *
 *  Outputs:      none
 *
 *  Returns:      pointer to character string with attribute name
 *
 */

void GenerateAttName(int32 i, char att_name[30])
     
{
  char number[3];
  int j, k;
  
    if (i < 9)
      {	
	number[0] = '1';
	number[0] += i;
	number[1] = '\0';
      }
    else
      {
	j = i;
	k = 0;	
	while (j >= 9)
	  {
	    j -= 10;
	    ++k;
	  }
	number[0] = '0' + k;
	number[1] = '0' + j;
	number[2] = '\0';
      }

    strcpy(att_name, "Related File ");
    strcat(att_name, number);   

    return;    
}















