/*
 *  File:  cp_process.c
 *
 *  Description:  This file has routines for processing the Level 0 data
 *                input to the calibration pipeline.  The routines subtract
 *                out the dark noise data and multiply by the gain data.
 *
 *  Subroutines:  ProcessDataset
 *                ReadRawData
 *                WriteFloatDataset
 *                WriteShortDataset
 *                Index2
 *                Index2_flip
 *                Index3

 *
 *  Author:	  Jenifer Brinker Evans
 *
 *  Date	Author		Action
 * -----------------------------------------------------------------------
 * 02/19/98	JBE		Version 1.1 complete
 *  4/13/98     JBE             Implements use of Dataset structure with
 *                                enumerated datatypes.
 *  4/24/98     JBE             Replace print statements with prints to
 *                                an external log file.
 * 11/16/00     JBE             Due to memory limitations on long datasets
 *                                modified to allow for reading raw science
 *                                data frame by frame.  Offset data still
 *                                read in mass.
 * 11/28/00     JBE             Added rows of fill to different bands and
 *                                SCAs for band-to-band registration
 *  1/31/01     JBE             Added index2_flip to handle left to right
 *                                flipping of image.  
 *  6/27/01     JBE             Bug fix for writing offset data to file.  
 *                                Need to count from 0, not 1.
 */


#include "hdf.h"
#include "mfhdf.h"
#include "cp_includes.h"



extern DATA_SET Dataset[MAX_SENSORS][MAX_DS_TYPES];
extern GAIN_DB DatabaseCoeff[MAX_SENSORS][MAX_COEFF_TYPES];
extern int32 FLIP;
extern FILE *logfile;


/* Define integration time table.  Given an integration time from 0 to 15, */ 
/* convert to integration time in ms. Different for MS and PAN */

/*  Note:  The current radiometric calibration coefficients only apply to 
    INTEG_TIME = 12.  This table is being left in in case the radiometric
    calibration coefficients are ever returned to their previous quality 
    level.  But given the men behind the change, I doubt it will ever be as
    good again....*/
float INT_TIME_TABLE[2][16]= {{0.81, 1.08, 1.35, 1.62,
                               1.89, 2.16, 2.43, 2.70,
                               3.24, 3.51, 3.78, 4.05,
                               4.32, 4.59, 4.86, 5.13},
                              {0.27, 0.36, 0.45, 0.54,
                               0.63, 0.72, 0.81, 0.90,
                               0.99, 1.08, 1.17, 1.26,
                               1.35, 1.44, 1.53, 1.62}};


char *SensorName[2] = {"MS","PAN"};
char *BandName[2][9] = {{"1p","1","2","3","4","4p","5p","5","7"},
			 {"PAN","err","err","err","err","err","err",
			  "err","err"}};


/***************************************************************************/
/***************************************************************************/
/*
 *  Name:  ProcessDataset
 * 
 *  Description:  Main processing loop for a single dataset.  Processes
 *                a single band of data at a time.
 *
 *  Inputs:       sds_id          int32      dataset id
 *                sds_ir[]        int32      array of dataset ids for output
 *                ds_index        int32      dataset index number to sds_id
 *
 *  Outputs:      nothing
 *
 *  Returns:      nothing
 *
 */

void ProcessDataset(enum sensor_types Sensor, enum ds_types DataType,
		    int32 sca_num, 
		    int32 frame_shift[MAX_SENSORS][MAX_BANDS],
		    int32 MaxFrameShift)

{  
short *fpdata;
short *cal_data, *fill_data;
float *offset_data[3], *response_data, *response_table, *lp_gain_data;
float integ_time;
int32 fpdata_dim[2], dim_table[2];
int32 data_in_start[3], data_in_edge[3], data_out_edge[3];
int32 LeakyPixel;

int32 num_pixels;
int32 status;
int32 cross, along, band;
int32 inc, read, max_read;
int32 i, j, n;

enum ds_types DataTypeOut,DataRespOut,DataOffOut;  

/* List must agree with enumeration in cp_consts.h */
char *sensor_list[] = {"MS","PAN"};


void ReadRawData(int32 , short *, int32 * , int32 *, int32);
void WriteFloatDataset(int32 , float *, int32 * , int32 , int32, int32);
void WriteShortDataset(int32 , short *, int32 * , int32 , int32, int32, int32);



/*  Due to the PAN tri-read, the single PAN band is treated as 3 bands of offset data.*/
if (Sensor == PAN)
  max_read = 3;
else
  max_read = 1;



if (Dataset[Sensor][DataType].EXISTS)
  {
  
    printf("Begin processing sensor %s\n",SensorName[Sensor]);
    fprintf(logfile,"Begin processing sensor %s\n",SensorName[Sensor]);


    /* Determine proper output dataset names */
	  switch(DataType)
	    {
	      
	    case LEVEL0:
	      DataTypeOut = LEVEL1R;
	      DataRespOut = RESPONSE;
	      DataOffOut  = OFFSET;	      
	      break;

	    case CAL1:
	      DataTypeOut = CAL1R;
	      DataRespOut = CAL1_RESP;
	      DataOffOut =  CAL1_OFF;	      
	      break;

	    case CAL2:
	      DataTypeOut = CAL2R;	      
	      DataRespOut = CAL2_RESP;
	      DataOffOut =  CAL2_OFF;	      
	      break;

	    case CAL3:
	      DataTypeOut = CAL3R;	      
	      DataRespOut = CAL3_RESP;
	      DataOffOut =  CAL3_OFF;	      
	      break;

	    case CAL4:
	      DataTypeOut = CAL4R;	      
	      DataRespOut = CAL4_RESP;
	      DataOffOut =  CAL4_OFF;	      
		break;

	    default:
	      printf("\nError:  Unknown datatype %d\n", DataType);
	      fprintf(logfile,"\nError:  Unknown datatype %d\n", DataType);
	      break;
	    }
	  
    
  /* 
   *  Define dimensions for a single band as Along x Cross 
   *  This is necessary to allow use of index2 routine for relating
   *    fpdata[n][m] to a single dimensional array fpdata[p]
   */
  fpdata_dim[0] = Dataset[Sensor][DataType].ALONG;
  fpdata_dim[1] = Dataset[Sensor][DataType].CROSS;
  
  
  /* Malloc space for one line of calibrated data. */
  num_pixels = Dataset[Sensor][DataType].CROSS;
  cal_data = (short *)malloc((unsigned) num_pixels*sizeof(short));

  /* Malloc space for one line of offset data. */
  num_pixels = Dataset[Sensor][DataType].CROSS;
  for (read = 0; read < max_read; ++read)
    offset_data[read] = (float *)malloc((unsigned) num_pixels*sizeof(float));

  /* Malloc space for one line of response data. */
  num_pixels = Dataset[Sensor][DataType].CROSS;
  response_data = (float *)malloc((unsigned) num_pixels*sizeof(float));
    
  /* Malloc space for one line of fill data. */
  num_pixels = Dataset[Sensor][DataType].CROSS;
  fill_data = (short *)malloc((unsigned) num_pixels*sizeof(short));

  /* and fill with fill data of 0x3000 */
  for (i = 0; i < Dataset[Sensor][DataType].CROSS; ++i)
    fill_data[i] = 0x3000;
  
	   
  /* Malloc space for one line of response table (cross x Table length). */
  num_pixels = DatabaseCoeff[Sensor][RESP_COEFF].CROSS * 
    DatabaseCoeff[Sensor][RESP_COEFF].LENGTH;
  response_table = (float *)malloc((unsigned) num_pixels*sizeof(float));

  /* Malloc space for reading single band of data, the raw focal plane data. */
  num_pixels = Dataset[Sensor][DataType].ALONG*Dataset[Sensor][DataType].CROSS;
  fpdata = (short *)malloc((unsigned) num_pixels*sizeof(short));

  

  /* Before FLIP, the integration time is an int.  After change, it's a float*/
  if (FLIP)
    {      
      /* Determine integration time in milliseconds from global variable. */ 
      integ_time = 
	INT_TIME_TABLE[Sensor][Dataset[Sensor][DataType].INTEG_TIME_INT];
    }
  else
    integ_time = Dataset[Sensor][DataType].INTEG_TIME_FLT;
  

  if (ABS(integ_time - 4.32) > 0.01)
    {
      printf("WARNING:  Radiometric calibration coefficients only valid for int_time= 12\n");
      printf("          (4.32ms MS and 1.35ms PAN)  Current value is %f\n",
             integ_time);
      fprintf(logfile,"WARNING:  Radiometric calibration coefficients only valid for int_time= 12\n");
      fprintf(logfile,"          (4.32ms MS and 1.35ms PAN)  Current value is %f,\n",integ_time);
    }
  
  
  
  /* 
   *  Define the location, pattern, and size of the data set 
   *   Assumes first data set starts at 0,0,0.   
   *  And data is read one band at a time.  Hence, edge
   *   is 1, ALONG, CROSS and the starting point is 0, 0, 0
   */
  
  data_in_start[0] = 0;
  data_in_start[1] = 0;
  data_in_start[2] = 0;
  
  data_in_edge[0] = 1;
  data_in_edge[1] = Dataset[Sensor][DataType].ALONG;
  data_in_edge[2] = Dataset[Sensor][DataType].CROSS;
  
  
  for (band = 0; band < Dataset[Sensor][DataType].BANDS; ++band)
    {
      printf("Starting Band %s\n",BandName[Sensor][band]);
      fprintf(logfile,"Starting Band %s\n",BandName[Sensor][band]);
      

      /* Determine if this is a band and SCA with a leaky pixel */
      if ((Sensor == MS) && (((sca_num == 3) && (band == 3)) ||
			     ((sca_num == 4) && (band == 2))))
	{	  
	  LeakyPixel = TRUE;

	  /* If so, malloc space for leaky gain data and read data. */
	  num_pixels = Dataset[Sensor][DataType].CROSS/2;
	  lp_gain_data = (float *)malloc((unsigned)num_pixels*sizeof(float));
	  ReadLeakyGains(Sensor, lp_gain_data);	  
	}
      else
	LeakyPixel = FALSE;
      

      /* Read single band of uncalibrated, raw data */
      ReadRawData(Dataset[Sensor][DataType].SDS_ID, fpdata, 
		  data_in_start, data_in_edge, band);
      



      /* 
       *  Define the location, pattern, and size of the output data set 
       *   The data sets start at cross, along, band.  Cross will
       *   always be 0 since a whole line of data is written at 
       *   one time.  along and band change with each write.   
       *  The data is written one line of one band at a time, for
       *   all the cross track dimension.  Hence, edge
       *   is 1, 1, CROSS
       */

      cross = 0;      
      data_out_edge[0] = 1;      
      data_out_edge[1] = 1;     
      data_out_edge[2] = Dataset[Sensor][DataType].CROSS;

      /* Write fill data to Level 1R file for band-to-band registration 
	 Some of data is written now to beginning of dataset.  Some
	 will be written later to end of dataset.  Amount varies for
	 each band and for each SCA.
	 */
      WriteLeadFill(Sensor, fill_data, frame_shift[Sensor][band], 
		    MaxFrameShift,  
		    data_out_edge, cross, along, band);

      
      /* Process data */
      for (read = 0; read < max_read; ++read)
	ComputeOffsetCoeff(Sensor, offset_data[read], band, read);
      
      ComputeResponseCoeff(Sensor, DataType, response_table, dim_table, band);

      
      for (along = 0; along < Dataset[Sensor][DataType].ALONG; ++along)
	{
	  if (Sensor == PAN)
	      read = along % 3;
	  else
	    read = 0;

	  
	  CalibrateData(Sensor, DataType, fpdata, fpdata_dim, 
			dim_table, along, 
			response_table, offset_data[read], 
			response_data, cal_data, integ_time, 
			sca_num, band);


	  /*  If a leaky pixel exists, calibrate that frame as a whole */
	  if (LeakyPixel)
	    CalibrateLeakyPixels(Sensor, DataType, fpdata, fpdata_dim, 
				 dim_table, along, 
				 lp_gain_data, offset_data[read], 
				 response_data, cal_data, integ_time, 
				 sca_num, band);

      
	  /*write out the calibrated data, offset data, and response data, */
	  /* band by band */
	  

	  WriteShortDataset(Dataset[Sensor][DataTypeOut].SDS_ID, cal_data, 
		       data_out_edge, cross, along, band, 
			    frame_shift[Sensor][band]);
	  WriteFloatDataset(Dataset[Sensor][DataRespOut].SDS_ID,response_data,
		       data_out_edge, cross, along, band);


	}
      
      WriteFloatDataset(Dataset[Sensor][DataOffOut].SDS_ID, offset_data[read],
			data_out_edge, cross, 0, band);
      
      
      
      /* and fill with fill data of 0x3000 */
      for (i = 0; i < Dataset[Sensor][DataType].CROSS; ++i)
	fill_data[i] = 0x3000;
      
      
      WriteTrailFill(Sensor, fill_data, frame_shift[Sensor][band], 
		       MaxFrameShift,  
		       data_out_edge, cross, along, band);
      
      
    } /* end for band, ParamIn.BANDS */
  
  free((short *)fpdata);
  free((short *)cal_data);
  free((short *)fill_data);
  for (read = 0; read < max_read; ++read)
    free((float *)offset_data[read]);
  free((float *)response_data);
  if (LeakyPixel) free((float *)lp_gain_data);
  }
else
  fprintf(logfile,"\nWARNING:  No data for sensor type: %s\n",sensor_list[Sensor]);

return;

}







/***************************************************************************/
/***************************************************************************/
/*
 *  Name:  ReadRawData
 * 
 *  Description:  Reads Level0 raw focal plane data.  Reads one band.
 *
 *  Inputs:       sds_id       int32        dataset id
 *                start[3]     int32        defines where data starts
 *                edge[3]      int32        defines size of data
 *                band         int32        band number being read
 *
 *  Outputs:      data         short *      band of raw focal plane data
 *
 *  Returns:      nothing
 *
 */

void ReadRawData(int32 sds_id, short *data, int32 start[3], int32 edge[3], int32 band)
{
  int status;
  

  /* increment starting point as band number is incremented. */
  start[0] = band;
  
  /* Read a single band of data */
  status = SDreaddata(sds_id, start, NULL, edge, (VOIDP)data);

  if (status == FAIL)
    {
      fprintf(logfile,"\nERROR:  SDreaddata failed!  %d \n", status);
      fprintf(logfile,"sds_id: %d  start: %d %d %d    edge: %d %d %d/n",
	     sds_id, start[0], start[1], start[2],
	     edge[0], edge[1], edge[2]);	  
      return;	  
    }      
  
  return;
}

/***************************************************************************/
/***************************************************************************/
/*
 *  Name:  WriteFloatDataset
 * 
 *  Description: Writes one band of data at a time to an HDF file.
 *
 *  Inputs:      sds_id        int32       dataset id number
 *               data          float *     data array being written out
 *               start[3]      int32       defines start of data in dataset
 *               edge[3]       int32       defines size of data in dataset
 *               band          int32       band number being written
 *
 *  Outputs:     nothing
 *
 *  Returns:     nothing
 *
 */

void WriteFloatDataset(int32 sds_id, float *data, int32 edge[3], 
		  int32 cross, int32 along, int32 band)
{
  int32 status;
  int32 start[3];
  int32 i,temp_id, temp_rank, temp_type, temp_num, temp_dims[5];
  char temp_name[50];


  /* increment starting point as band number is incremented. */
  start[0] = band;
  start[1] = along;
  start[2] = cross;
  
  /* Write a single band of data */
  status = SDwritedata(sds_id, start, NULL, edge, (VOIDP)data);

  if (status == FAIL)
    {
      fprintf(logfile,"\nERROR:  SDwritedata failed!  %d \n", status);
      fprintf(logfile,"sds_id: %d  start: %d %d %d    edge: %d %d %d/n",
	     sds_id, start[0], start[1], start[2],
	     edge[0], edge[1], edge[2]);	  

      return;	  
    }      


  
  return;
}


/***************************************************************************/
/***************************************************************************/
/*
 *  Name:  WriteShortDataset
 * 
 *  Description: Writes one band of data at a time to an HDF file.
 *
 *  Inputs:      sds_id        int32       dataset id number
 *               data          float *     data array being written out
 *               start[3]      int32       defines start of data in dataset
 *               edge[3]       int32       defines size of data in dataset
 *               band          int32       band number being written
 *
 *  Outputs:     nothing
 *
 *  Returns:     nothing
 *
 */

void WriteShortDataset(int32 sds_id, short *data, int32 edge[3], 
		       int32 cross, int32 along, int32 band,
		       int32 shift)
{
  int32 status;
  int32 start[3];
  int32 i;  


  

  /* increment starting point as band number is incremented. */
  start[0] = band;
  start[1] = along + shift;
  start[2] = cross;


  /* Write a single band of data */
  status = SDwritedata(sds_id, start, NULL, edge, (VOIDP)data);


  if (status == FAIL)
    {
      printf("Bingo 2\n");
      
      fprintf(logfile,"\nERROR:  SDwritedata failed!  %d \n", status);
      fprintf(logfile,"sds_id: %d  start: %d %d %d    edge: %d %d %d/n",
	     sds_id, start[0], start[1], start[2],
	     edge[0], edge[1], edge[2]);	  
      
      return;	  
    }      
  
  return;
}


/************************************************************************/
/************************************************************************/
/*  
 *   Takes array indices (i,j) and array dimensions (imax, jmax)
 *   and finds the corresponding index (a) in the single dimension array.
 */


int index2(i,j,dim)
     int i,j;
     ing32 dim[2];
     
{
  int a;
  

  a = (i*dim[1]) + j;
  return a;
}

/************************************************************************/
/************************************************************************/
/*  
 *   Takes array indices (i,j) and array dimensions (imax, jmax)
 *   and finds the corresponding index (a) in the single dimension array
 *   but with the i dimension flipped left to right.
 *
 *   This function was added to handle the fact that the image is flipped
 *   left to right to make it imageable in Envi.
 */


int index2_flip(i,j,dim)
     int i,j;
     int dim[2];
     
{
  int a;
  
  /* To handle the flip, instead of just using the column number, i,
     the maximum column number - i is used.
     */

  a = (((dim[0]-1)-i)*dim[1]) + j;
  return a;
}


/************************************************************************/
/************************************************************************/
/*  
 *   Takes array indices (i,j,k) and array dimensions (imax, jmax, kmax)
 *   and finds the corresponding index (a) in the single dimension array.
 */


int index3(i,j,k,dim)
     int i,j,k;
     int dim[3];
     
{
  int a;
  

  a = (i*dim[1]*dim[2]) + (j*dim[2]) + k;
  return a;
}

/************************************************************************/
/************************************************************************/



