;Routine to create flag mask.
; Pat Scaramuzza
; Release: 0.0b May 23, 2002
;
; Revisions:
;   May 31, 2002  Pat Scaramuzza   Changed processing to one-band-at-a-time, 
;                                  to conserve memory.
;   June 12, 2002 Pat Scaramuzza   Clean-up.  Changed attribute name,
;                                  changed some print output.
;   July 01, 2002 Lawrence Ong     Added variables nframes and offset
;                                  to process partial scenes
;   July 03, 2002 Lawrence Ong     Fix getting L0 data from subsequent
;                                  bands, added hdf getdata offsets 
;                                  to next band.

function hypMask,prefix,satlevel=satlevel,offset=offset,nframes=nframes

    ; Saturation is at a count of 4095 or higher, or whatever the user specifies.
    IF (keyword_set(satlevel) eq 0) THEN satlevel = 4095

    ; Create a list of dead or flat detectors.
    ; If this list gets any larger, we should put it in a file and read it in.

    util = { band: 0, sample: 0, type: 2 }
    deadlist = REPLICATE(util, 49)

    deadlist[0:34].band = indgen(35)+1
    deadlist[0:34].sample = 0

    ; Band is decremented because IDL counts from zero instead of one.
    deadlist[35:48].band = [ 61, 72, 94, 99, 116, 119, 120, 165, 168, 169, 190, 200, 201, 203 ]-1
    ; Sample is decremented twice -- one for starting at zero, and one for the SWIR detector shift.
    deadlist[35:48].sample = [ 93, 95, 93, 92, 138, 240, 240, 148, 256, 23, 113, 8, 8, 115 ]-2
    ; Type = 2 (dead) is the default.  A few detectors are marked Type = 3 (flat).
    deadlist[40:42].type = 3

    ; Open up the L0 file.

    print, 'HypMask:  Opening '+prefix+' ...'
   
    OPEN_HDF,prefix,$
      arrsize,datasets,sds_id,sd_id,dsname,attrname,sds_name

    if not keyword_set(offset) then offset=0
    if not keyword_set(nframes) then nframes=arrsize[2]-offset

    ; Now create a *_mask file.
;    satsize = arrsize
    satsize = [arrsize[0],arrsize[1],long(nframes)]
    satsize[2] = satsize[2] + 1
    sdout_id=hdf_sd_start(prefix+"_mask",/create)
    print, 'Satsize: ', satsize
    sdsout_id = HDF_SD_CREATE(sdout_id, 'Flag Mask', satsize, /BYTE)

    n_satpix = long(0)

    ; Now get the L0 data, one band at a time.

    FOR band = 0, arrsize[1]-1 DO BEGIN
;       HDF_SD_GETDATA, sds_id[0], satdata, count=[ arrsize[0], 1, arrsize[2] ]

        HDF_SD_GETDATA, sds_id[0], satdata, $
          COUNT=[arrsize[0],1,nframes],$
          START=[0,band,offset]

       print, 'HypMask Processing band '+STRTRIM(band,2)

       ; Begin by marking all saturated pixels.
       util = where(satdata ge satlevel)
       satdata = byte(TEMPORARY(satdata)*0)
       IF (util[0] ne -1) THEN BEGIN
          satdata[util] = 1
          n_satpix = n_satpix + 1
          print, n_elements(util), ' saturated pixels ( > '+STRTRIM(satlevel,2)+') in band ', band
       ENDIF
   
       ; And mark all -32767 values.
       util = where(satdata eq long(-32767))
       IF (util[0] ne -1) THEN BEGIN
          satdata[util] = 4
          print, n_elements(util), ' bad values (-32767) in band ', band
       ENDIF
   
       ; At this point the mask matches the L0; it must be modified to match the L1.
       util = bytarr(satsize[0], 1, satsize[2])
       util[*, 0, 0:satsize[2]-2] = satdata
   
       IF (band ge 70) AND (band le 241) THEN BEGIN
          ; Shift half the SWIR detectors down one scan.
          util[128:255, 0, 1:satsize[2]-1] = util[128:255, 0, 0:satsize[2]-2]
   
          ; L1 processing shifts all SWIR Detectors to the left.
          util = shift(TEMPORARY(util), -1)
          util[255, 0, *] = 0
       ENDIF
       satdata = util
   
       ; Add detectors that are dead or flat to the flag mask.
   
       util = where(deadlist.band eq band)
       IF (util[0] ne -1) THEN BEGIN
          satdata[deadlist[util[0]].sample, 0, 0:*] = deadlist[util[0]].type
          ; print, 'Marking detector '+STRTRIM(deadlist[util[0]].sample,2)+' in band '+STRTRIM(band, 2)+' as '+STRTRIM(deadlist[util[0]].type,2)
       ENDIF
   
       ; And put this band's data into the new *_mask file.
       HDF_SD_ADDDATA, sdsout_id, satdata, count=[ satsize[0], 1, satsize[2] ], start=[0, band, 0]

    ENDFOR

    print, 'HypMask:  Number of saturated pixels :  ', n_elements(util)

    ; Close our L0 and *_mask files.

    HDF_SD_ENDACCESS, sds_id
    HDF_SD_END, sd_id

    HDF_SD_ENDACCESS, sdsout_id
    HDF_SD_END,sdout_id

    return, 1
end

