#!/usr/bin/perl -w
# Given a list of EO-1 definitive ephemeris files, read each file,
# determine the time span it covers and create links for each day.
use strict;
use vars qw($opt_v);
use Time::Local;
use Getopt::Std;
getopts("v");

foreach my $DEfile (@ARGV)
{
	unless ($DEfile =~ m/EO1_(\d\d\d\d)(\d\d\d)_def.e/) {
		print STDERR "File name must be EO1_YYYYDDD_def.e\nSkipping $DEfile\n";
		next;
	}
	my ($orig_year, $orig_doy) = ($1, $2);
	unless (open(DE, "<$DEfile")) {
		perror("$DEfile: $!");
		next;
	}
	while (<DE>) {
		last if m/BEGIN Ephemeris/;
	}

	my $NumberOfEphemerisPoints = 0;
	my $pointsFound = 0;
	my $ScenarioEpoch = undef;
	my @timePosVel;
	my $earliest = undef;
	my $latest = undef;
	while (<DE>) {
		next if m/^$/;
		last if m/END Ephemeris/;
		chomp;
		next if m/EphemerisEciTimePosVel/;
		if (m/NumberOfEphemerisPoints\s+(\d+)/) {
			$NumberOfEphemerisPoints = $1;
		} elsif (m/ScenarioEpoch\s*(.*)/) {
			$ScenarioEpoch = getEpoch($1);
		} elsif (@timePosVel = split(/ /, $_)) {
			$pointsFound++;
			my $ts = $ScenarioEpoch + $timePosVel[0];
			if (!defined($earliest) || ($ts < $earliest)) {
				$earliest = $ts;
			}
			if (!defined($latest) || ($ts > $latest)) {
				$latest = $ts;
			}
		}
	}

	if ($pointsFound != $NumberOfEphemerisPoints) {
		print STDERR "Warning $DEfile: Found $pointsFound, expected $NumberOfEphemerisPoints\n";
	}
	for (my $i = $earliest; $i < $latest - 86399; $i += 86400) {
		my @gmtime = gmtime($i);
		my $year = sprintf("%04d", $gmtime[5] + 1900);
		my $doy = sprintf("%03d", $gmtime[7] + 1);
		unless (($year == $orig_year) && ($doy == $orig_doy)) {
			my $newfile = $DEfile;
			$newfile =~ s/EO1_${orig_year}${orig_doy}_def.e/EO1_${year}${doy}_def.e/;
			unless (-f $newfile) {
				print "ln $DEfile $newfile\n" if $opt_v;
				link($DEfile, $newfile) || warn("ln DEfile $newfile: $!");
			}
		}
	}
	close(DE);
}

sub getEpoch {
	unless ($_[0] =~ m/(\d+) (...) (\d+) (\d\d):(\d\d):(\d\d)/) {
		print STDERR "Bad timestamp: $_[0]\n";
		return time();
	}
	my ($day, $month, $year, $hour, $min, $sec) = ($1, $2, $3, $4, $5, $6);
	my %months = ("Jan", 1, "Feb", 2, "Mar", 3, "Apr", 4, "May", 5, "Jun", 6,
		"Jul", 7, "Aug", 8, "Sep", 9, "Oct", 10, "Nov", 11, "Dec", 12);
	return timegm($sec, $min, $hour, $day, $months{$month}-1, $year-1900);
}
