#include <stdio.h>
#include <unistd.h>
#include <dirent.h>
#include <sys/types.h>
#include <sys/stat.h>
#include "hdf.h"
#include "mfhdf.h"

#define MAX_BANDS 3
#define N_SCA 4
/*
 * Generate a browse for an EO-1 ALI scene.  Expects a directory name
 * parameters, it then looks for the 4 detector files of Level 1 data.
 * The output is a PPM on stdout.
 */

int Verbose = 0;	/* output lots of information */
int Debug = 0;
int Fudge_pan[] = {32, 28, 31, 0};	/* between SCA overlap in pixels, pan band */
int Fudge_ms[] = {11, 10, 9, 0};	/* between SCA overlap in pixels, multispectral bands */
char File_suffix = 'R';			/* last char of file names */
int Stitched = 0;			/* read a single HDF file with the SCAs stitched together */
int Flipped = 0;			/* old data with SCAs flipped */
int N_sca = N_SCA;			/* number of SCA files to read */

main(int argc, char **argv)
	{
	int output_fd = 1;
	int option;
	int bands[MAX_BANDS];		/* band combination to use for color composite, zeroes means use pan */
	int32 sca_hdf[N_SCA];	/* HDF sid's of the four SCA files */
	int lines;		/* number of lines in the image */
	int samples;		/* number of samples */
	short *full_image[MAX_BANDS];	/* buffer for 16-bit image bands, with the SCAs pasted together */
	int low_cutoff = 1;	/* low stretch point, in percent */
	int high_cutoff = 3;	/* high stretch point, in 100 - percent */

	bands[0] = bands[1] = bands[2] = 0;
	while ((option = getopt(argc, argv, "vb:drsl:h:f")) != -1)
		{
		switch (option)
			{
			case 'v':
				Verbose = (Verbose << 1) | 1;
				break;
			case 'b':
				parsebands(optarg, bands);
				break;
			case 'd':
				Debug = (Debug << 1) | 1;
				break;
			case 'r':	/* "raw", read a level-0 image */
				File_suffix = 'Z';
				/* No fudge factors, they don't align anyway */
				Fudge_pan[0] = Fudge_pan[1] = Fudge_pan[2] = 0;
				Fudge_ms[0] = Fudge_ms[1] = Fudge_ms[2] = 0;
				break;
			case 's':	/* read a stitched level-1 image */
					/* In the case the argument must be the file */
				Stitched = 1;
				N_sca = 1;
				Fudge_pan[0] = Fudge_pan[1] = Fudge_pan[2] = 0;
				Fudge_ms[0] = Fudge_ms[1] = Fudge_ms[2] = 0;
				break;
			case 'l':
				low_cutoff = atoi(optarg);
				break;
			case 'h':
				high_cutoff = atoi(optarg);
				break;
			case 'f':
				Flipped = 1;
				break;
			}
		}
	if (Verbose)
		{
		if (bands[1] > 0)
			fprintf(stderr, "using bands %d, %d, %d\n", bands[0], bands[1], bands[2]);
		else if (bands[0] > 0)
			fprintf(stderr, "using band %d\n", bands[0]);
		else
			fprintf(stderr, "using pan band\n");
		}
	findfiles(sca_hdf, argv[optind]);
	getsize(sca_hdf[0], &lines, &samples, bands[0] == 0);
	if (Verbose)
		fprintf(stderr, "Input is %d x %d\n", N_sca * samples, lines);
	if (bands[1])
		{
		int band_index;
		int offset;
		int lut[MAX_BANDS][65536];		/* lookup table for 16 -> 8 bit conversion */
		char ppmheader[1024];
		for (band_index = 0; band_index < MAX_BANDS; band_index++) {
			int sca;
			if (!(full_image[band_index] = (short *)malloc(lines * (samples+1) * sizeof(short) * N_sca)))
				{
				perror("Out of memory");
				exit(10);
				}
			if (Verbose & 4)
				fprintf(stderr, "malloc(%d) = 0x%X\n", lines * (samples+1) * sizeof(short) * N_sca, full_image[band_index]);
			offset = 0;
			for (sca = 0; sca < N_sca; sca++)
				{
				readband(sca_hdf[N_sca-1 - sca], bands[band_index], full_image[band_index] + offset);
				offset += samples - Fudge_ms[sca];
				}
			compute_lut(full_image[band_index], lut[band_index], lines, N_sca*samples, offset, low_cutoff, high_cutoff);
			}	
		if (Verbose)
			fprintf(stderr, "Output will be %d x %d\n", offset, lines);
		sprintf(ppmheader, "P6 %d %d %d\n", offset, lines, 255);
		write(output_fd, ppmheader, strlen(ppmheader));
		outputcolor(output_fd,
			full_image[0], full_image[1], full_image[2],
			lut[0], lut[1], lut[2],
			lines, N_sca * samples, offset);
		}
	else
		{
		int sca;
		int offset;
		char pgmheader[1024];
		int lut[65536];		/* lookup table for 16 -> 8 bit conversion */
		if (!(full_image[0] = (short *)malloc(lines * (samples+1) * sizeof(short) * N_sca)))
			{
			perror("Out of memory");
			exit(10);
			}
		if (Verbose & 4)
			fprintf(stderr, "malloc(%d) = 0x%X\n", lines * (samples+1) * sizeof(short) * N_sca, full_image[0]);
		offset = 0;
		for (sca = 0; sca < N_sca; sca++)
			{
			readband(sca_hdf[N_sca-1 - sca], bands[0], full_image[0] + offset);
			offset += samples - (bands[0] ? Fudge_ms[sca] : Fudge_pan[sca]);
			}
		if (Verbose)
			fprintf(stderr, "Output will be %d x %d\n", offset, lines);
		if (Debug & 4)
			{
			int i;
			for (i=0; i<lines; i++)
				write(output_fd, full_image[0] + samples * i, 2 *offset);
			exit(0);
			}
		compute_lut(full_image[0], lut, lines, N_sca*samples, offset, low_cutoff, high_cutoff);
		sprintf(pgmheader, "P5 %d %d %d\n", offset, lines, 255);
		write(output_fd, pgmheader, strlen(pgmheader));
		output(output_fd, full_image[0], lut, lines, N_sca * samples, offset);
				
		}
	exit(0);
	}

output(int output_fd, unsigned short *buf, int *lut, int lines, int samples, int offset)
	{
	int row;
	int pixel;
	char *output_buffer;

	if (Verbose & 4)
		fprintf(stderr, "output called(%d, %X, %d, %d, %d)\n", output_fd, buf, lines, samples, offset);
	if (!(output_buffer = (char *)malloc(offset)))
		{
		perror("Out of memory");
		exit(10);
		}
	for (row = 0; row < lines; row++)
		{
		if (Verbose & 4)
			fprintf(stderr, "write(%d, %X, %d)\n", output_fd, buf, offset);
		for (pixel=0; pixel<offset; pixel++)
			{
			output_buffer[pixel] = lut[buf[pixel]];
			}
		if (write(output_fd, output_buffer, offset) < 0)
			{
			perror("write error");
			exit(9);
			}
		buf += samples;
		}
	free(output_buffer);
	}

outputcolor(int output_fd, short *red, short *green, short *blue,
	int *red_lut, int *green_lut, int *blue_lut,
	int lines, int samples, int offset)
	{
	int row;
	int pixel;
	char *output_buffer;

	if (!(output_buffer = (char *)malloc(3 * offset)))
		{
		perror("Out of memory");
		exit(10);
		}
	for (row = 0; row < lines; row++)
		{
		for (pixel=0; pixel<offset; pixel++)
			{
			output_buffer[3*pixel] = red_lut[red[pixel]];
			output_buffer[3*pixel+1] = green_lut[green[pixel]];
			output_buffer[3*pixel+2] = blue_lut[blue[pixel]];
			}
		if (write(output_fd, output_buffer, 3 * offset) < 0)
			{
			perror("write error");
			exit(9);
			}
		red += samples;
		green += samples;
		blue += samples;
		}
	free(output_buffer);
	}

compute_lut(short *buf, int *lut, int lines, int samples, int offset,
	int lowcutoff, int highcutoff)
	{
	int min = 512;
	int max = 5000;
	int pixel;
	int row;
	int cutoff;
	int accum = 0;

	/*
	 * First generate a histogram of the data values, in the LUT array.
	 */
	for (pixel = 0; pixel < 65536; pixel++)
		lut[pixel] = 0;
	for (row = 0; row < lines; row++)
		{
		for (pixel = 0; pixel < offset; pixel++)
			lut[buf[pixel]+32768]++;
		buf += samples;
		}
	if (Debug)
		{
		FILE *histogram;
		histogram = fopen("histogram", "w");
		if (!histogram)
			{
			perror("histogram");
			exit(1);
			}
		for (pixel = 0; pixel < 65536; pixel++)
			fprintf(histogram, "%d %d\n", pixel, lut[pixel]);
		fclose(histogram);
		}
	/*
	 * Ignore fill.  Stitched images have some zero fill at the right.
	 */
	lut[0xb000] = 0;
	if (Stitched)
		lut[0+32768] = 0;

			
	/*
	 * Now walk through the histogram and compute stretch points.
	 */
	accum = 0;
	cutoff = lines * offset * lowcutoff / 100;
	for (pixel=-32768; pixel<32768; pixel++)
		{
		if (Verbose && (lut[pixel+32768] > 0) && (accum == 0))
			fprintf(stderr, "smallest value: %d, cutoff at %d\n", pixel, cutoff);
		if ((accum += lut[pixel+32768]) > cutoff)
			{
			min = pixel;
			break;
			}
		}
	accum = 0;
	cutoff = lines * offset * highcutoff / 100;
	for (pixel=32767; pixel >= -32768; pixel--)
		{
		if (Verbose && (lut[pixel+32768] > 0) && (accum == 0))
			fprintf(stderr, "largest value: %d, cutoff at %d\n", pixel, cutoff);
		if ((accum += lut[pixel+32768]) > cutoff)
			{
			max = pixel;
			break;
			}
		}
	if (max == min)	/* avoid a zero divide */
		max = min+1;
	if (Verbose)
		fprintf(stderr, "min=%d, max=%d\n", min, max);

	/*
	 * Now go back and fill in the LUT, based on the min and max
	 */
	for (pixel = 0; pixel < 65536; pixel++)
		if (pixel <= min)
			lut[pixel] = 0;
		else if (pixel >= max)
			lut[pixel] = 255;
		else
			lut[pixel] = (pixel-min)*255/(max-min);

	}
readband(int32 sd_fid, int band, short *buf)
	{
	int32 num_datasets, num_attr;
	int dataset_index;
	int i;
	int row;
	int debug_file;
	static int debug_number;
	int32 dataset_id;

	if (Verbose & 4)
		fprintf(stderr, "readband(%d, %d, 0x%X)\n", sd_fid, band, (int)buf);
	if (SDfileinfo(sd_fid, &num_datasets, &num_attr) == FAIL)
		{
		fprintf(stderr,"SDfileinfo failed\n");
		exit(1);
		}
	if (Verbose & 4)
		{
		char attr_name[1024];
		int32 attr_nt;
		int32 attr_count;
		fprintf(stderr, "%d datasets, %d attributes\n", num_datasets, num_attr);

		for (i=0; i<num_attr; i++)
			{
			if (SDattrinfo(sd_fid, i, attr_name, &attr_nt, &attr_count) == FAIL)
				{
				fprintf(stderr, "SDattrinfo failed\n");
				exit(1);
				}
			fprintf(stderr,"\t%d %s %d %d\n", i, attr_name, attr_nt, attr_count);
			}
		}

	/*
	 * 	Find the appropriate data set
	 */
	for (dataset_index=0; dataset_index<num_datasets; dataset_index++)
		{
		int32 index;
		short width;
		int32 height;
		short bands;
		char type_name[50];
		char sensor_name[50];
		int32 dimensions;
		int32 dimsizes[MAX_VAR_DIMS];
		int32 data_type;
		int32 num_attrs;
		char *target_sensor;

		dataset_id = SDselect(sd_fid, dataset_index);
		index = SDfindattr(dataset_id, "Dataset Type");
		SDreadattr(dataset_id, index, (void *)type_name);
		if (Verbose & 2)
			fprintf(stderr,"dataset %d: type %s\n", dataset_index, type_name);
		if (strcmp(type_name, "LEVEL0") && strcmp(type_name, "Level 0") && strcmp(type_name, "Level 1R"))
			continue;
		if ((index = SDfindattr(dataset_id, "ALI Sensor")) == FAIL)
			strcpy(sensor_name, "Unknown");
		else
			SDreadattr(dataset_id, index, (void *)sensor_name);
		if (band == 0)
			target_sensor = "PN";
		else
			target_sensor = "MS";
		if (strncmp(sensor_name, target_sensor, 2) != 0)
			continue;
		sensor_name[3] = '\0';

		if ((index = SDfindattr(dataset_id, "Number of cross track pixels")) == FAIL)
			width = 0;
		else
			SDreadattr(dataset_id, index, &width);
		if ((index = SDfindattr(dataset_id, "Number of along track pixels")) == FAIL)
			height = 0;
		else
			SDreadattr(dataset_id, index, &height);
		if ((index = SDfindattr(dataset_id, "Number of bands")) == FAIL)
			bands = 0;
		else
			SDreadattr(dataset_id, index, &bands);
		
		SDgetinfo(dataset_id, (char *)0, &dimensions, dimsizes, &data_type, &num_attrs);

		if (Verbose & 2)
			fprintf(stderr, "dimsizes={%d, %d, %d}\n", dimsizes[0], dimsizes[1], dimsizes[2]);
		if (Debug & 2)
			{
			char filename[1024];
			sprintf(filename, "sca%d", ++debug_number);
			debug_file = open(filename, O_WRONLY | O_CREAT | O_TRUNC, 0644);
			if (debug_file < 0)
				{
				perror(filename);
				exit(1);
				}
			}
		for (row=0; row<dimsizes[1]; row++)
			{
			int32 start[3];
			int32 stride[3];
			int32 edge[3];

			start[0] = (band == 0 ? 0 : band-1);
			start[1] = row;
			start[2] = 0;
			edge[0] = 1;
			edge[1] = 1;
			edge[2] = dimsizes[2];
			stride[0] = stride[1] = stride[2] = 1;
			if (Verbose & 4)
				fprintf(stderr, "SDreaddata(%d, {%d, %d, %d}, {%d, %d, %d}, {%d, %d, %d}, 0x%X)\n",
					dataset_id, start[0], start[1], start[2],
					stride[0], stride[1], stride[2],
					edge[0], edge[1], edge[2], buf);
			if (SDreaddata(dataset_id, start, stride, edge, buf) == FAIL)
				{
				perror("SDreaddata error");
				exit(1);
				}
			if (Debug & 2)
				write(debug_file, buf, 2 * dimsizes[2]);
			buf += (N_sca * dimsizes[2]);
			}
		if (Debug & 2)
			close(debug_file);
		}
	}
computehistogram(int *histogram, int32 dataset_id, int band, int length, int width, short *input_buffer)
	{
	int row;
	int32 start[3];
	int32 stride[3];
	int32 edge[3];
	int pixel;

	for (row=0; row<length; row++)
		{
		start[0] = band-1;
		start[1] = row;
		start[2] = 0;
		edge[0] = 1;
		edge[1] = 1;
		edge[2] = width;
		stride[0] = stride[1] = stride[2] = 1;
		if (SDreaddata(dataset_id, start, stride, edge, input_buffer) == FAIL)
			{
			perror("SDreaddata error");
			exit(1);
			}
		for (pixel=0; pixel<width; pixel++)
			{
			histogram[input_buffer[pixel]+32768]++;
			}
		}
	}
parsebands(char *options, int *bands)
	{
	int i;
	int n;
	n = sscanf(options, "%d,%d,%d", bands, bands+1, bands+2);
	for (i=0; i<n;i++)
		{
		if ((bands[i] < 1) || (bands[i] > 9))
			{
			fprintf(stderr, "Invalid band number: %d\n", bands[i]);
			exit(2);
			}
		}
	}
findfiles(int *sca_hdf, char *dirname)
	{
	DIR *d;
	struct dirent *dirent;
	int i;
	int errors;
	struct stat s;

	/*
	 *	Initialize the array of file pointers.
	 */
	for (i=0; i<N_sca; i++)
		{
		sca_hdf[i] = -1;
		}
	if (Stitched)
		{
		if ((sca_hdf[0] = SDstart(dirname, DFACC_RDONLY)) == FAIL)
			{
			fprintf(stderr,"SDstart(%s, DFACC_RDONLY) returned %d\n", dirname, sca_hdf[0]);
			exit(1);
			}
		return;
		}

	if (stat(dirname, &s) < 0)
		{
		perror(dirname);
		exit(1);
		}
	if (S_ISDIR(s.st_mode))
		{
		if (!(d = opendir(dirname)))
			{
			perror(dirname);
			exit(1);
			}
		while (dirent = readdir(d))
			{
			int len;	/* length of the name */
			if (dirent->d_name[0] == '.')
				continue;
			len = strlen(dirent->d_name);
			if ((dirent->d_name[len-1] != File_suffix) || (dirent->d_name[len-3] != 'M'))
				continue;
			if (sscanf(dirent->d_name + (len-2), "%d", &i) > 0)
				if ((i > 0) && (i <= N_sca))
					{
					/*
					 * We found a file.  Try to open it, but
					 * first check to make sure we have only
					 * one file per SCA
					 */
					char *pathname;		/* directory/filename */
					if (Flipped)
						i = 5 - i;
					if (sca_hdf[i-1] != -1)
						{
						fprintf(stderr, "Found two files for SCA %d\n", i);
						exit(3);
						}

					pathname = (char *)malloc(strlen(dirname) + strlen(dirent->d_name) + 2);
					if (!pathname)
						{
						perror("Out of memory");
						exit(1);
						}
					strcpy(pathname, dirname);
					strcat(pathname, "/");
					strcat(pathname, dirent->d_name);
					if ((sca_hdf[i-1] = SDstart(pathname, DFACC_RDONLY)) == FAIL)
						{
						fprintf(stderr,"SDstart(%s, DFACC_RDONLY) returned %d\n", pathname, sca_hdf[i-1]);
						exit(1);
						}
					}
			}

		/*
		 *	Make sure we found all four files.
		 */
		errors = 0;
		for (i=0; i<N_sca; i++)
			{
			if (sca_hdf[i] == -1)
				{
				fprintf(stderr, "Missing HDF file for SCA %d\n", i);
				errors++;
				}
			}
		if (errors > 0)
			exit(3);
		}
	else
		{
		/* dirent is a file, look for the other three */
		int len;
		char *pathname;
		len = strlen(dirname);
		pathname = (char *)malloc(len+1);
		if (!pathname)
			{
			perror("out of memory");
			exit(1);
			}
		strncpy(pathname, dirname, len-2);
		for (i=1; i<=N_sca; i++)
			{
			char suffix[3];
			sprintf(suffix, "%dR", i);
			strcpy(pathname + len-2, suffix);
			if ((sca_hdf[i-1] = SDstart(pathname, DFACC_RDONLY)) == FAIL)
				{
				fprintf(stderr,"SDstart(%s, DFACC_RDONLY) returned %d\n", pathname, sca_hdf[i-1]);
				exit(1);
				}
			}
		free(pathname);
		}
	}
/*
 *	Get the lines and samples out of one of the SCA files.
 */
getsize(int32 sd_fid, int *lines, int *samples, int use_pan)
	{
	int32 num_datasets, num_attr;
	int dataset_index;
	int i;
	int32 dataset_id;

	if (SDfileinfo(sd_fid, &num_datasets, &num_attr) == FAIL)
		{
		fprintf(stderr,"SDfileinfo failed\n");
		exit(1);
		}
	if (Verbose & 2)
		{
		char attr_name[1024];
		int32 attr_nt;
		int32 attr_count;
		fprintf(stderr, "%d datasets, %d attributes\n", num_datasets, num_attr);

		for (i=0; i<num_attr; i++)
			{
			if (SDattrinfo(sd_fid, i, attr_name, &attr_nt, &attr_count) == FAIL)
				{
				fprintf(stderr, "SDattrinfo failed\n");
				exit(1);
				}
			fprintf(stderr,"\t%d %s %d %d\n", i, attr_name, attr_nt, attr_count);
			}
		}

	for (dataset_index=0; dataset_index<num_datasets; dataset_index++)
		{
		int32 index;
		short width;
		int32 height;
		short bands;
		char type_name[50];
		char sensor_name[50];
		int32 dimensions;
		int32 dimsizes[MAX_VAR_DIMS];
		int32 data_type;
		int32 num_attrs;
		char *target_sensor;

		dataset_id = SDselect(sd_fid, dataset_index);
		index = SDfindattr(dataset_id, "Dataset Type");
		SDreadattr(dataset_id, index, (void *)type_name);
		if (Verbose & 2)
			fprintf(stderr,"dataset %d: type %s\n", dataset_index, type_name);
		if (strcmp(type_name, "LEVEL0") && strcmp(type_name, "Level 0") && strcmp(type_name, "Level 1R"))
			continue;
		if ((index = SDfindattr(dataset_id, "ALI Sensor")) == FAIL)
			strcpy(sensor_name, "Unknown");
		else
			SDreadattr(dataset_id, index, (void *)sensor_name);
		if (use_pan)
			target_sensor = "PN";
		else
			target_sensor = "MS";
		if (strncmp(sensor_name, target_sensor, 2) != 0)
			continue;
		sensor_name[3] = '\0';

		if ((index = SDfindattr(dataset_id, "Number of cross track pixels")) == FAIL)
			width = 0;
		else
			SDreadattr(dataset_id, index, &width);
		if ((index = SDfindattr(dataset_id, "Number of along track pixels")) == FAIL)
			height = 0;
		else
			SDreadattr(dataset_id, index, &height);
		if ((index = SDfindattr(dataset_id, "Number of bands")) == FAIL)
			bands = 0;
		else
			SDreadattr(dataset_id, index, &bands);
		
		SDgetinfo(dataset_id, (char *)0, &dimensions, dimsizes, &data_type, &num_attrs);

		if (Verbose & 2)
			fprintf(stderr,"dataset %d: type %s, sensor %s, %d x %d x %d\n",
				dataset_index, type_name, sensor_name, width, height, bands);
		*lines = dimsizes[1];
		*samples = dimsizes[2];
		return;
		}
	/*
	 *	We didn't find a dataset.
	 */
	fprintf(stderr, "Can't get dimensions\n");
	exit(1);
	}
