#!/usr/bin/perl
# Read a daily report file and check to see if all the scenes have been loaded.
use strict;
BEGIN {
	push(@INC, "/home/eo1/bin");
}
use vars qw($opt_d $opt_v);
use Getopt::Std;
use reportfiles;

getopts("d:v") || die <<END_USAGE;
Usage: $0 [-v] [-d topdir] [days...]
	-v		verbose
	-d topdir	Top level directory, default is /hsm/lta4/eo1 (used to find report files and data files)
Days are in the form 2005/234.
A range of days can be given as 2005/213-243 (can't span years yet)
or 2009/360-2010/002 (to span years).
END_USAGE
$opt_d ||= "/hsm/lta4/eo1";

my $total_ali = 0;
my $total_ali_missing = 0;
my $total_hyp = 0;
my $total_hyp_missing = 0;

# Default to the week of one week ago
if (@ARGV == 0)
	{
	my @t = localtime;
	my @t1= localtime(time() - (((24*3600)*$t[6])+((24*3600)*9)));
	my @t2= localtime(time() - (((24*3600)*$t[6])+((24*3600)*3)));
	my $start_year = $t1[5] + 1900;
	my $start_doy = sprintf("%03d",$t1[7] + 1);
	my $end_year = $t2[5] + 1900;
	my $end_doy = sprintf("%03d",$t2[7] + 1);
	$ARGV[0] = "$start_year/$start_doy-$end_year/$end_doy";
	}

print "For day(s)  $ARGV[0]\n\n";

foreach my $day (@ARGV)
	{
	if ($day =~ m"^(\d\d\d\d)/(\d\d\d)$")
		{
		checkday($1, $2);
		}
	elsif ($day =~ m"^(\d\d\d\d)/(\d\d\d)-(\d\d\d)$")
		{
		my ($start_year, $start_doy, $end_doy) = ($1, $2, $3);
		foreach my $doy ($start_doy .. $end_doy)
			{
			checkday($start_year, $doy);
			}
		}
	elsif ($day =~ m"^(\d\d\d\d)/(\d\d\d)-(\d\d\d\d)/(\d\d\d)$")
		{
		my ($start_year, $start_doy, $end_year, $end_doy) = ($1, $2, $3, $4);
		my $year = $start_year;
		my $doy = $start_doy;
		while (($year < $end_year) || (($year == $end_year) && ($doy <= $end_doy)))
			{
			checkday($year, $doy);
			if (++$doy > daysinyear($year))
				{
				$year++;
				$doy = 1;
				}
			}
		}
	else
		{
		print STDERR "Bad day: $day, skipped\n";
		next;
		}
	}

if ($opt_v)
	{
	print "\nMissing $total_ali_missing of $total_ali ALI scenes\n";
	print "Missing $total_hyp_missing of $total_hyp Hyperion scenes\n";
	}

sub checkday
	{
	my ($year, $doy) = @_;
	# print "Checking $year/$doy\n" if $opt_v;
	my @rpt_files = eo1::find_rpt_files($year, $doy, $opt_d);
	if (@rpt_files < 1)
		{
		print STDERR "No daily report file for $year/$doy, skipped\n";
		return;
		}
	my %rpt_records;
	my @raw_recs;
	eo1::readreportfile($rpt_files[0], \%rpt_records, \@raw_recs);
	find_downlinks(\%rpt_records, \@raw_recs);

	foreach my $sceneid (keys %rpt_records)
		{
		unless ($sceneid =~ m/EO1(\d\d\d)(\d\d\d)(\d\d\d\d)(\d\d\d)(\d)(\d)/)
			{
			print STDERR "Bad sceneid: $sceneid, skipped\n";
			next;
			}
		my ($path, $row, $sceneid_year, $sceneid_doy, $hyp, $ali) = ($1, $2, $3, $4, $5, $6);
		if ($ali)
			{
			my $missing = 1;
			$total_ali++;
			if (opendir(ALI, "$opt_d/$sceneid_year/$sceneid_doy/ali"))
				{
				my $file;
				my $ali_sceneid = $sceneid;
				$ali_sceneid =~ s/EO1/EO1A/;
				while (defined($file = readdir(ALI)))
					{
					next unless $file =~ m/$ali_sceneid/;
					$missing = 0;
					}
				closedir(ALI);
				}
			if ($missing)
				{
				print "Missing ALI      data for $sceneid $rpt_records{$sceneid}->{downlink} $rpt_records{$sceneid}->{'SPECIAL REQUEST'}\n";
				$total_ali_missing++;
				}
			}
		if ($hyp)
			{
			my $missing = 1;
			$total_hyp++;
			if (opendir(HYP, "$opt_d/$sceneid_year/$sceneid_doy/hyp"))
				{
				my $file;
				my $hyp_sceneid = $sceneid;
				$hyp_sceneid =~ s/EO1/EO1H/;
				while (defined($file = readdir(HYP)))
					{
					next unless $file =~ m/$hyp_sceneid/;
					$missing = 0;
					}
				closedir(HYP);
				}
			if ($missing)
				{
				print "Missing Hyperion data for $sceneid $rpt_records{$sceneid}->{downlink} $rpt_records{$sceneid}->{'SPECIAL REQUEST'}\n";
				$total_hyp_missing++;
				}
			}
		}
	}

sub daysinyear
	{
	my $year = shift;
	return 366 if ((($year % 4) == 0) && ((($year % 100) != 0) || ($year % 400 == 0)));
	return 365;
	}

sub find_downlinks
	{
	my ($rpt_records, $rpt_array) = @_;
	my @scenes_in_warp;
	foreach my $rawrec (@$rpt_array)
		{
		if ($rawrec->{recordtype} eq "ScheduledScene")
			{
			push(@scenes_in_warp, $rawrec->{'SCENE ID'});
			}
		elsif (($rawrec->{recordtype} eq "XBand_Baseline_Contact") ||
			($rawrec->{recordtype} eq "non_WOTIS_XBand_Baseline_Cont"))
			{
			foreach (@scenes_in_warp)
				{
				my $downlink_id = $rawrec->{starttime};
				$downlink_id =~ s/(\d\d\d\d):(\d\d\d):(\d\d):(\d\d):(\d\d)/$1$2$3$4/;
				$downlink_id .= "_$rawrec->{STATION}";
				$$rpt_records{$_}->{downlink} = $downlink_id;
				}
			}
		elsif ($rawrec->{recordtype} eq "WARP_Format")
			{
			undef @scenes_in_warp;
			}
		}
	}
