#!/usr/local/bin/perl
# Run "footprint" on everything we have.
# If an argument of the form YYYY/DDD is given, only run
# for that day.
use strict;
use Getopt::Std;
use vars qw($opt_d $opt_v $opt_c $opt_F $opt_n);
getopts("d:vc:Fn") || die <<END_USAGE;
Usage:
	$0 [-d dir] [-v] [-c cpfdir] [-F] [-n] YYYY/DDD...
Where
	-d dir		"dir" is the top-leve directory, default /edclxs37/eo1
	-v		verbose
	-c cpfdir	"cpfdir" is the directory containing the CPFs,
			default is /edclxs37/data1/production/proddata/cpf
	-F		force: normally a day is skipped if the files already
			exist, with -F they are regenerated
	-n		don't generate the ../alicorners.txt and
			../hypcorners.txt files at the end
Zero or moree days can be given, in which case only those days will
be generated.  With no days, a find is run and any day with an ACS
houseskeeping file will be generated.
END_USAGE
$opt_d ||= "/edclxs37/eo1";
$opt_c ||= "/edclxs37/data1/production/proddata/cpf";

my @cpf;	# list of calibration parameter files

opendir(CPFDIR, "$opt_c") || die("$opt_c: $!");
my $file;
while (defined($file = readdir(CPFDIR)))
	{
	next unless $file =~ m/^EO1ACPF(\d\d\d\d)(\d\d)(\d\d)_(\d\d\d\d)(\d\d)(\d\d)/;
	push(@cpf, $file);
	}
closedir(CPFDIR);

if (@ARGV)
	{
	foreach (@ARGV)
		{
		unless (m"(\d\d\d\d)/(\d\d\d)")
			{
			print STDERR "Bad YYYY/DDD: $_, ignored\n";
			next;
			}
		my ($year, $doy) = ($1, $2);
		gencorners($year, $doy);
		}
	}
else
	{
	# Run it on everything.
	open(FIND, "find $opt_d -follow -name '*_acs.hdf' -print|") || die("fork: $!");
	while (<FIND>)
		{
		chomp;
		next unless m"$opt_d.*meta/(\d\d\d\d)_(\d\d\d)_acs.hdf";
		gencorners($1, $2);
		}
	close(FIND) || die("find returned $?");
	}

unless ($opt_n)
	{
	system("find . -name 'alicorners.*.txt' -print | sort | xargs cat > ../alicorners.txt");
	system("find . -name 'hypcorners.*.txt' -print | sort | xargs cat > ../hypcorners.txt");
	}

# Generate corners for a given day.
# This read the YYYY/DDD/meta directory and looks for any *_acs.hdf files.
# It then compares the dates of the acs.hdf and the corresponding ali.hdf
# and hyp.hdf to the ???corners.YYYYDDD.txt files.  If any are newer,
# it runs the footprint program on all of them and merges the
# inputs into a new set of ???corners.YYYYDDD.txt files.
sub gencorners
	{
	my ($year, $day) = @_;
	my $metadir = "$opt_d/$year/$day/meta";
	unless (opendir(META, $metadir))
		{
		print STDERR "Can't open $metadir: $!\n";
		return 0;
		}
	my @acsfiles;
	my $needtorun = $opt_F;
	my $file;
	while (defined($file = readdir(META)))
		{
		if ($file =~ m/.*_acs.hdf$/)
			{
			push(@acsfiles, $file);
			}
		# Check the timestamps, unless we have already determined that we need to run,
		# either because of -F or we already found an out-of-date file.
		next if $needtorun;
		my $acs_housekeepingfile = "$metadir/$file";
		my $ali_housekeepingfile = $acs_housekeepingfile;
		$ali_housekeepingfile =~ s/acs.hdf/ali.hdf/;
		my $hyp_housekeepingfile = $acs_housekeepingfile;
		$hyp_housekeepingfile =~ s/acs.hdf/hyp.hdf/;
		if (!((compare_file_mtime("alicorners.$year$day.txt", $acs_housekeepingfile) > 0) &&
			(compare_file_mtime("hypcorners.$year$day.txt", $acs_housekeepingfile) > 0) &&
			(compare_file_mtime("alicorners.$year$day.txt", $ali_housekeepingfile) > 0) &&
			(compare_file_mtime("hypcorners.$year$day.txt", $hyp_housekeepingfile) > 0)))
			{
				$needtorun = 1;
			}
		}
	closedir(META);
	unless ($needtorun)
		{
		print STDERR "$year/$day already done, skipping it\n" if $opt_v;
		return 0;
		}
	my $cpf = findcpf($year, $day);
	unless ($cpf)
		{
		print STDERR "Can't find calibration parameter file for $year/$day\n";
		exit(1);
		}
	my %alicorners;
	my %hypcorners;
	foreach $file (@acsfiles)
		{
		my $file_root = $file;
		$file_root =~ s/acs.hdf//;
		open(ODL, ">footprint.odl") || die("footprint.odl: $!");
		print ODL <<END_ODL;
OBJECT = FOOT_PRINT
  CAL_PARM_FILE = "$opt_c/$cpf"
  HK_DATA_DIR = "$metadir"
  HK_FILE_ROOT = "${file_root}"
  EPHEM_TYPE = "ACS"
END_OBJECT = FOOT_PRINT
END
END_ODL
		close(ODL) || die("footprint.odl: $!");
		system("/edclxs37/data1/production/software/bin/footprint footprint.odl");
		if ($?)
			{
			print STDERR "footprint returned $? for $year/$day\n";
			# next;
			}
		mergecorners(\%alicorners, "alicorners.txt");
		mergecorners(\%hypcorners, "hypcorners.txt");
		}
	open(ALICORNERS, ">alicorners.$year$day.txt") || warn("alicorners.$year$day.txt: $!");
	foreach my $ts (sort keys %alicorners)
		{
		print ALICORNERS $ts, @{$alicorners{$ts}};
		}
	close(ALICORNERS);
	open(HYPCORNERS, ">hypcorners.$year$day.txt") || warn("hypcorners.$year$day.txt: $!");
	foreach my $ts (sort keys %hypcorners)
		{
		print HYPCORNERS $ts, @{$hypcorners{$ts}};
		}
	close(HYPCORNERS);
	}

sub findcpf
	{
	my $target = yydddtoyymmdd($_[0], $_[1]);
	foreach (@cpf)
		{
		next unless m/^EO1ACPF(\d\d\d\d\d\d\d\d)_(\d\d\d\d\d\d\d\d)/;
		return $_ if (($1 <= $target) && ($target <= $2));
		}
	return undef;
	}

# Merge a ???corners.txt file with an array of corners in the same format
sub mergecorners
	{
	my ($corners, $file) = @_;
	open(C, "<$file") || return;
	my $ts;
	while ($ts = <C>)
		{
		@{$corners->{$ts}} = ();
		push(@{$corners->{$ts}}, <C>);
		push(@{$corners->{$ts}}, <C>);
		push(@{$corners->{$ts}}, <C>);
		push(@{$corners->{$ts}}, <C>);
		}
	close(C);
	}

# Convert year and day of year to year, month, day of month.
sub yydddtoyymmdd
	{
	my ($year, $day) = @_;
	my @days = (31,28,31,30,31,30,31,31,30,31,30,31);
	# Leap year check.
	if ((($year % 4) == 0) && ((($year % 100) != 0) || ($year % 400 == 0)))
		{
		$days[1] = 29;
		}
	my $month;
	for ($month = 0; $day > $days[$month]; $month++)
		{
		$day -= $days[$month];
		}
	return sprintf("%04d%02d%02d", $year, $month+1, $day);
	}
# Compare the mtime of two files, return -1, 0, or 1 if the first is
# (respectively) older, the same, or newer than the second
sub compare_file_mtime
	{
	my ($file1, $file2) = @_;
	my @s1 = stat($file1);
	return -1 unless @s1;
	my @s2 = stat($file2);
	return 1 unless @s2;
	return $s1[9] <=> $s2[9];
	}
