#include <stdio.h>
#include <unistd.h>
#include <dirent.h>
#include "hdf.h"
#include "mfhdf.h"

#define MAX_BANDS 3
#define HYP_BANDS 242
#define HYP_DETECTORS 256
/*
 * Generate a browse for an EO-1 Hyperion scene.  Expects a file name
 * parameters.
 * The output is a PPM on stdout.
 */

int Verbose = 0;	/* output lots of information */
int Debug = 0;
int **read_dark_cal(char *, int, int, int *);
float ** read_gain_file(char *);
void apply_bias(short *image, int *bias, int lines, int samples);
void apply_gain(short *image, float *gain, int lines, int samples);

main(int argc, char **argv)
	{
	int output_fd = 1;
	int option;
	int bands[MAX_BANDS];		/* band combination to use for color composite, zeroes means use pan */
	int32 hdf;		/* HDF sid of the data file */
	int lines;		/* number of lines in the image */
	int samples;		/* number of samples */
	int nbands;		/* number of bands in input image */
	short *full_image[MAX_BANDS];	/* buffer for 16-bit image bands, with the SCAs pasted together */
	int low_cutoff = 1;	/* low stretch point, in percent */
	int high_cutoff = 3;	/* high stretch point, in 100 - percent */
	int band_index;
	int lut[MAX_BANDS][65536];		/* lookup table for 16 -> 8 bit conversion */
	char pnmheader[1024];	/* header for the PGM/PPM output file */
	char *dark_cal_image = (char *)0;	/* name of file containing a dark calibration image */
	int **bias = (int **)0;		/* bias values determined from dark cal */
	char *gain_file = (char *)0;	/* name of gain file */
	float **gain = (float **)0;	/* array of gain values, one per band per detector */

	bands[0] = 40;		/* the default is single, band 40 */
	bands[1] = bands[2] = 0;
	while ((option = getopt(argc, argv, "vb:dl:h:c:g:")) != -1)
		{
		switch (option)
			{
			case 'v':
				Verbose = (Verbose << 1) | 1;
				break;
			case 'b':
				parsebands(optarg, bands);
				break;
			case 'd':
				Debug = (Debug << 1) | 1;
				break;
			case 'l':
				low_cutoff = atoi(optarg);
				break;
			case 'h':
				high_cutoff = atoi(optarg);
				break;
			case 'c':
				dark_cal_image = optarg;
				break;
			case 'g':
				gain_file = optarg;
				break;
			}
		}
	if (Verbose)
		{
		if (bands[1] > 0)
			fprintf(stderr, "using bands %d, %d, %d\n", bands[0], bands[1], bands[2]);
		else
			fprintf(stderr, "using band %d\n", bands[0]);
		}
	if ((hdf = SDstart(argv[optind], DFACC_RDONLY)) == FAIL)
		{
		fprintf(stderr,"SDstart(%s, DFACC_RDONLY) returned %d\n", argv[optind], hdf);
		exit(1);
		}
	getsize(hdf, &lines, &samples, &nbands);
	if (dark_cal_image)
		bias = read_dark_cal(dark_cal_image, samples, nbands, bands);
	if (Verbose)
		fprintf(stderr, "Input is %d x %d x %d\n", samples, lines, nbands);
	if (gain_file)
		gain = read_gain_file(gain_file);
	for (band_index = 0; (band_index < MAX_BANDS) && bands[band_index]; band_index++) {
		if (!(full_image[band_index] = (short *)malloc(lines * (samples+1) * sizeof(short))))
			{
			perror("Out of memory");
			exit(10);
			}
		if (Verbose & 4)
			fprintf(stderr, "malloc(%d) = 0x%X\n", lines * (samples+1) * sizeof(short), full_image[band_index]);
		readband(hdf, bands[band_index], full_image[band_index]);
		if (bias)
			apply_bias(full_image[band_index], bias[band_index], lines, samples);
		if (gain)
			apply_gain(full_image[band_index], gain[bands[band_index]], lines, samples);
		compute_lut(full_image[band_index], lut[band_index], lines, samples,  low_cutoff, high_cutoff);
		}	
	sprintf(pnmheader, "%s %d %d %d\n", (bands[1] ? "P6" : "P5"), samples, lines, 255);
	write(output_fd, pnmheader, strlen(pnmheader));
	if (bands[1])
		outputcolor(output_fd,
			full_image[0], full_image[1], full_image[2],
			lut[0], lut[1], lut[2],
			lines, samples);
	else
		output(output_fd, full_image[0], lut, lines, samples);
				
	exit(0);
	}

output(int output_fd, short *buf, int *lut, int lines, int samples)
	{
	int row;
	int pixel;
	char *output_buffer;

	if (Verbose & 4)
		fprintf(stderr, "output called(%d, %X, %d, %d)\n", output_fd, buf, lines, samples);
	if (!(output_buffer = (char *)malloc(samples)))
		{
		perror("Out of memory");
		exit(10);
		}
	for (row = 0; row < lines; row++)
		{
		if (Verbose & 4)
			fprintf(stderr, "write(%d, %X, %d)\n", output_fd, buf, samples);
		for (pixel=0; pixel<samples; pixel++)
			{
			output_buffer[pixel] = lut[buf[pixel]+32768];
			}
		if (write(output_fd, output_buffer, samples) < 0)
			{
			perror("write error");
			exit(9);
			}
		buf += samples;
		}
	free(output_buffer);
	}

outputcolor(int output_fd, short *red, short *green, short *blue,
	int *red_lut, int *green_lut, int *blue_lut,
	int lines, int samples)
	{
	int row;
	int pixel;
	char *output_buffer;

	if (!(output_buffer = (char *)malloc(3 * samples)))
		{
		perror("Out of memory");
		exit(10);
		}
	for (row = 0; row < lines; row++)
		{
		for (pixel=0; pixel<samples; pixel++)
			{
			output_buffer[3*pixel] = red_lut[red[pixel]+32768];
			output_buffer[3*pixel+1] = green_lut[green[pixel]+32768];
			output_buffer[3*pixel+2] = blue_lut[blue[pixel]+32768];
			}
		if (write(output_fd, output_buffer, 3 * samples) < 0)
			{
			perror("write error");
			exit(9);
			}
		red += samples;
		green += samples;
		blue += samples;
		}
	free(output_buffer);
	}

compute_lut(short *buf, int *lut, int lines, int samples,
	int lowcutoff, int highcutoff)
	{
	int min = 512;
	int max = 5000;
	int pixel;
	int row;
	int cutoff;
	int accum = 0;

	/*
	 * First generate a histogram of the data values, in the LUT array.
	 */
	for (pixel = 0; pixel < 65536; pixel++)
		lut[pixel] = 0;
	for (row = 0; row < lines; row++)
		{
		for (pixel = 0; pixel < samples; pixel++)
			lut[buf[pixel]+32768]++;
		buf += samples;
		}
	if (Debug)
		{
		FILE *histogram;
		histogram = fopen("histogram", "w");
		if (!histogram)
			{
			perror("histogram");
			exit(1);
			}
		for (pixel = 0; pixel < 65536; pixel++)
			fprintf(histogram, "%d %d\n", pixel, lut[pixel]);
		fclose(histogram);
		}
	/*
	 * Ignore fill
	 */
	lut[0] = lut[65535] = 0;
			
	/*
	 * Now walk through the histogram and compute stretch points.
	 */
	accum = 0;
	cutoff = lines * samples * lowcutoff / 100;
	for (pixel=-32768; pixel<32768; pixel++)
		{
		if (Verbose && (lut[pixel+32768] > 0) && (accum == 0))
			fprintf(stderr, "smallest value: %d, cutoff at %d\n", pixel, cutoff);
		if ((accum += lut[pixel+32768]) > cutoff)
			{
			min = pixel;
			break;
			}
		}
	accum = 0;
	cutoff = lines * samples * highcutoff / 100;
	for (pixel=32767; pixel >= -32768; pixel--)
		{
		if (Verbose && (lut[pixel+32768] > 0) && (accum == 0))
			fprintf(stderr, "largest value: %d, cutoff at %d\n", pixel, cutoff);
		if ((accum += lut[pixel+32768]) > cutoff)
			{
			max = pixel;
			break;
			}
		}
	if (max == min)	/* avoid a zero divide */
		max = min+1;
	if (Verbose)
		fprintf(stderr, "min=%d, max=%d\n", min, max);

	/*
	 * Now go back and fill in the LUT, based on the min and max
	 */
	for (pixel=-32768; pixel<32768; pixel++)
		if (pixel <= min)
			lut[pixel+32768] = 0;
		else if (pixel >= max)
			lut[pixel+32768] = 255;
		else
			lut[pixel+32768] = (pixel-min)*255/(max-min);

	}
readband(int32 sd_fid, int band, short *buf)
	{
	int32 num_datasets, num_attr;
	int dataset_index;
	int i;
	int row;
	int debug_file;
	static int debug_number;
	int32 dataset_id;

	if (Verbose & 4)
		fprintf(stderr, "readband(%d, %d, 0x%X)\n", sd_fid, band, (int)buf);
	if (SDfileinfo(sd_fid, &num_datasets, &num_attr) == FAIL)
		{
		fprintf(stderr,"SDfileinfo failed\n");
		exit(1);
		}
	if (Verbose & 4)
		{
		char attr_name[1024];
		int32 attr_nt;
		int32 attr_count;
		fprintf(stderr, "%d datasets, %d attributes\n", num_datasets, num_attr);

		for (i=0; i<num_attr; i++)
			{
			if (SDattrinfo(sd_fid, i, attr_name, &attr_nt, &attr_count) == FAIL)
				{
				fprintf(stderr, "SDattrinfo failed\n");
				exit(1);
				}
			fprintf(stderr,"\t%d %s %d %d\n", i, attr_name, attr_nt, attr_count);
			}
		}

	/*
	 * 	Find the appropriate data set
	 */
	for (dataset_index=0; dataset_index<num_datasets; dataset_index++)
		{
		short width;
		int32 height;
		short bands;
		int32 dimensions;
		int32 dimsizes[MAX_VAR_DIMS];
		int32 data_type;
		int32 num_attrs;

		dataset_id = SDselect(sd_fid, dataset_index);
		SDgetinfo(dataset_id, (char *)0, &dimensions, dimsizes, &data_type, &num_attrs);
		height = dimsizes[0];
		bands = dimsizes[1];
		width = dimsizes[2];

		if (Verbose & 2)
			{
			int i;
			char *sep = "";
			fprintf(stderr, "dimsizes={");
			for (i=0; i<dimensions; i++)
				{
				fprintf(stderr, "%s%d", sep, dimsizes[i]);
				sep = ", ";
				}
			fprintf(stderr, "}\n");
			}
		if (Debug & 2)
			{
			char filename[1024];
			sprintf(filename, "sca%d", ++debug_number);
			debug_file = open(filename, O_WRONLY | O_CREAT | O_TRUNC, 0644);
			if (debug_file < 0)
				{
				perror(filename);
				exit(1);
				}
			}
		for (row=0; row<height; row++)
			{
			int32 start[3];
			int32 stride[3];
			int32 edge[3];
			int readrslt;

			start[0] = row;
			start[1] = band-1;
			start[2] = 0;
			edge[0] = 1;
			edge[1] = 1;
			edge[2] = width;
			stride[0] = stride[1] = stride[2] = 1;
			if (Verbose & 4)
				fprintf(stderr, "SDreaddata(%d, {%d, %d, %d}, {%d, %d, %d}, {%d, %d, %d}, 0x%X)\n",
					dataset_id, start[0], start[1], start[2],
					stride[0], stride[1], stride[2],
					edge[0], edge[1], edge[2], buf);
			if ((readrslt = SDreaddata(dataset_id, start, stride, edge, buf)) == FAIL)
				{
				fprintf(stderr, "SDreaddata returned %d\n", readrslt);
				exit(1);
				}
			if (Debug & 2)
				write(debug_file, buf, 2 * width);
			buf +=  width;
			}
		if (Debug & 2)
			close(debug_file);
		break;
		}
	}
parsebands(char *options, int *bands)
	{
	int i;
	int n;
	if (strcmp(options, "vnir") == 0)
		{
		bands[0] = 40;
		bands[1] = 31;
		bands[2] = 13;
		return;
		}
	if (strcmp(options, "swir") == 0)
		{
		bands[0] = 204;
		bands[1] = 150;
		bands[2] = 93;
		return;
		}
	n = sscanf(options, "%d,%d,%d", bands, bands+1, bands+2);
	for (i=0; i<n;i++)
		{
		if ((bands[i] < 1) || (bands[i] > HYP_BANDS))
			{
			fprintf(stderr, "Invalid band number: %d\n", bands[i]);
			exit(2);
			}
		}
	}
/*
 *	Get the lines and samples out of one of the HDF file.
 */
getsize(int32 sd_fid, int *lines, int *samples, int *bands)
	{
	int32 num_datasets, num_attr;
	int dataset_index;
	int i;

	if (SDfileinfo(sd_fid, &num_datasets, &num_attr) == FAIL)
		{
		fprintf(stderr,"SDfileinfo failed\n");
		exit(1);
		}
	if (Verbose & 2)
		{
		char attr_name[1024];
		int32 attr_nt;
		int32 attr_count;
		fprintf(stderr, "%d datasets, %d attributes\n", num_datasets, num_attr);

		for (i=0; i<num_attr; i++)
			{
			if (SDattrinfo(sd_fid, i, attr_name, &attr_nt, &attr_count) == FAIL)
				{
				fprintf(stderr, "SDattrinfo failed\n");
				exit(1);
				}
			fprintf(stderr,"\t%d %s %d %d\n", i, attr_name, attr_nt, attr_count);
			}
		}

	for (dataset_index=0; dataset_index<num_datasets; dataset_index++)
		{
		int32 dimensions;
		int32 dimsizes[MAX_VAR_DIMS];
		int32 data_type;
		int32 num_attrs;
		int32 dataset_id;

		dataset_id = SDselect(sd_fid, dataset_index);
		SDgetinfo(dataset_id, (char *)0, &dimensions, dimsizes, &data_type, &num_attrs);
		*lines = dimsizes[0];
		*bands = dimsizes[1];
		*samples = dimsizes[2];
		if (Verbose & 2)
			{
			int i;
			char *sep = "";
			fprintf(stderr, "dimsizes={");
			for (i=0; i<dimensions; i++)
				{
				fprintf(stderr, "%s%d", sep, dimsizes[i]);
				sep = ", ";
				}
			fprintf(stderr, "}\n");
			fprintf(stderr,"dataset %d: %d x %d x %d\n",
				dataset_index, *samples, *lines, *bands);
			}
		return;
		}
	/*
	 *	We didn't find a dataset.
	 */
	fprintf(stderr, "Can't get dimensions\n");
	exit(1);
	}

/*
 * Read a dark calibration image.  For the bands given,
 * compute the average values for each pixel.
 * Returns a pointer to an array of up to three buffers.
 */
int **
read_dark_cal(char *dark_cal_image, int image_samples, int image_nbands, int *bands)
	{
	int32 hdf;	/* HDF sid of dark cal image */
	int lines, samples, nbands;	/* number of lines, samples and bands in the dark cal image */
	int nbands_used;	/* number of bands to use in the composite */
	short *dark_image;	/* dark image data for a single band */
	int **buf;	/* buffers to be returned */
	int b,l,s;

	if (Verbose && 2)
		{
		fprintf(stderr, "read_dark_cal(%s, %d, %d, [%d,%d,%d])\n",
			dark_cal_image, image_samples, image_nbands, bands[0], bands[1], bands[2]);
		}
	if ((hdf = SDstart(dark_cal_image, DFACC_RDONLY)) == FAIL)
		{
		fprintf(stderr,"SDstart(%s, DFACC_RDONLY) returned %d\n", dark_cal_image, hdf);
		exit(1);
		}
	getsize(hdf, &lines, &samples, &nbands);
	if ((samples != image_samples) || (nbands != image_nbands))
		{
		fprintf(stderr, "Dark cal has %d samples, %d bands; image as %d samples, %d bands.  The dark cal will be ignored\n",
			samples, nbands, image_samples, image_nbands);
		return (int **)0;
		}
	/*
	 * Allocate space for the returned buffers.
	 */
	for (b=0; (b<MAX_BANDS) && bands[b]; b++)
		; 
	nbands_used = b;
	if (Verbose & 2)
		fprintf(stderr, "nbands_used=%d\n", nbands_used);
	buf = (int **)malloc(nbands_used * sizeof(int *));
	if (!buf)
		{
		perror("Out of memory");
		exit(10);
		}
	if (!(dark_image = (short *)malloc(lines * samples * sizeof(short))))
		{
		perror("Out of memory");
		exit(10);
		}
	for (b=0; b<nbands_used; b++)
		{
		if (!(buf[b] = (int *)malloc(samples * sizeof(int))))
			{
			perror("Out of memory");
			exit(10);
			}
		for (s = 0; s < samples; s++)
			buf[b][s] = 0;
		readband(hdf, bands[b], dark_image);
		for (l = 0; l < lines; l++)
			for (s = 0; s < samples; s++)
				buf[b][s] += dark_image[l*samples+s];
		for (s = 0; s < samples; s++)
			buf[b][s] /= lines;
		}

	free(dark_image);
	if (Verbose & 4)
		{
		for (b=0; b<nbands_used; b++)
			{
			fprintf(stderr, "Dark bias for band %d\n", bands[b]);
			for (s=0; s<samples; s++)
				fprintf(stderr, " %d", buf[b][s]);
			fprintf(stderr, "\n");
			}
		}
	return buf;
	}

void
apply_bias(short *image, int *bias, int lines, int samples)
	{
	int l, s;
	for (l=0; l<lines; l++)
		for (s=0; s<samples; s++)
			image[l*samples + s] -= bias[s];
	}


/*
 * Read a gain file, with a floating-point gain value for each (of 256)
 * detector of each band.
 */
float **
read_gain_file(char *gain_file)
	{
	int gain_fd;
	float **gain;
	int band;

	gain_fd = open(gain_file, O_RDONLY);
	if (gain_fd < 0)
		{
		perror(gain_file);
		exit(1);
		}
	if (!(gain = (float **)malloc(HYP_BANDS * sizeof(float *))))
		{
		perror("Out of memory");
		exit(10);
		}
	for (band = 0; band<HYP_BANDS; band++)
		{
		if (!(gain[band] = malloc(HYP_DETECTORS * sizeof(float))))
			{
			perror("Out of memory");
			exit(10);
			}
		if (read(gain_fd, gain[band], HYP_DETECTORS * sizeof(float)) <= 0)
			{
			fprintf(stderr, "Error reading gain file %s\n", gain_file);
			exit(2);
			}
		}
	return gain;
	}

void
apply_gain(short *image, float *gain, int lines, int samples)
	{
	int l, s;
	for (l=0; l<lines; l++)
		for (s=0; s<samples; s++)
			image[l*samples + s] *= gain[s];
	}
