/*
 * Read a Hyperion housekeeping file and look for image start and stop times.
 * Given an interval, with the -i option in the form
 *	YYYY:DDD:HH:MM:SS.mmm-YYYY:DD:HH:MM:SS.mmm
 * look for and print an image interval that overlaps it.
 */
#include <stdio.h>
#include <math.h>
#include <time.h>
#include <string.h>
#include "hdf.h"
#include "mfhdf.h"

char *fmttime(float64 t);
float64 timeunfmt(char *formatted);
void findinterval(uint16 *image_mode, float64 *image_mode_ts, int32 n,
	float64 interval_start, float64 interval_end, int verbose);
int getinterval(char *str, float64 *interval_start, float64 *interval_end);
void dumpmode(uint16 *image_mode, float64 *image_mode_ts, int32 n, int verbose);
#ifdef sun
time_t timegm (struct tm *tm);
#endif

main(int argc, char **argv)
	{
	int32 hyp_hdf;
	int32 index;
	int32 id;
	char *image_mode_name = "YIMAGE_MODE";	/* name of the HDF attribute */
	char *image_mode_ts_name = "A@YIMAGE_MODE(TIME_MEASURED)";
	int32 rank;
	int32 dimsizes[MAX_VAR_DIMS];
	int32 data_type;
	int32 num_attrs;
	uint16 *image_mode;
	float64 *image_mode_ts;
	int32 start;
	int32 edge;
	int option;
	int verbose = 0;
	int debug = 0;
	int i;
	float64 interval_start = 0.0, interval_end = 0.0;

	while ((option = getopt(argc, argv, "dvi:")) != -1)
		{
		switch (option)
			{
			case 'd':
				debug =1 ;
				break;
			case 'v':
				verbose = 1;
				break;
			case 'i':
				if (!getinterval(optarg, &interval_start, &interval_end))
					{
					fprintf(stderr, "Bad interval option: %s\n", optarg);
					exit(1);
					}
				break;
			default:
				fprintf(stderr, "bad option %c ignored\n", option);
				break;
			}
		}

	for (; optind<argc; optind++)
		{
		if ((hyp_hdf = SDstart(argv[optind], DFACC_RDONLY)) == FAIL)
			{
			fprintf(stderr, "SDstart(%s, DFACC_RDONLY) returned %d\n", argv[optind], hyp_hdf);
			exit(1);
			}
		if ((index = SDnametoindex(hyp_hdf, image_mode_name)) == FAIL)
			{
			fprintf(stderr, "Can't find %s\n", image_mode_name);
			exit(1);
			}
		if ((id = SDselect(hyp_hdf, index)) == FAIL)
			{
			fprintf(stderr, "SDselect failed\n");
			exit(1);
			}
		if (SDgetinfo(id, (char *)0, &rank, dimsizes, &data_type, &num_attrs) == FAIL)
			{
			fprintf(stderr, "SDgetinfo failed\n");
			exit(1);
			}
		if (data_type != DFNT_UINT16)
			{
			fprintf(stderr, "%s is not 16-bit integer data (%d)\n",
				image_mode_name, data_type);
			exit(1);
			}
		start = 0;
		edge = dimsizes[0];
		image_mode = (uint16 *)malloc(dimsizes[0] * sizeof(uint16));
		if (!image_mode)
			{
			fprintf(stderr, "out of memory\n");
			exit(1);
			}
		if (SDreaddata(id, &start, (int32 *)0, &edge, image_mode) == FAIL)
			{
			fprintf(stderr, "failed reading data\n");
			exit(1);
			}

		if ((index = SDnametoindex(hyp_hdf, image_mode_ts_name)) == FAIL)
			{
			fprintf(stderr, "Can't find %s\n", image_mode_ts_name);
			exit(1);
			}
		if ((id = SDselect(hyp_hdf, index)) == FAIL)
			{
			fprintf(stderr, "SDselect failed\n");
			exit(1);
			}
		if (SDgetinfo(id, (char *)0, &rank, dimsizes, &data_type, &num_attrs) == FAIL)
			{
			fprintf(stderr, "SDgetinfo failed\n");
			exit(1);
			}
		if (data_type != DFNT_FLOAT64)
			{
			fprintf(stderr, "%s is not 64-bit floating point data\n", image_mode_ts_name);
			exit(1);
			}
		start = 0;
		edge = dimsizes[0];
		image_mode_ts = (float64 *)malloc(dimsizes[0] * sizeof(float64));
		if (!image_mode_ts)
			{
			fprintf(stderr, "out of memory\n");
			exit(1);
			}
		if (SDreaddata(id, &start, (int32 *)0, &edge, image_mode_ts) == FAIL)
			{
			fprintf(stderr, "failed reading data\n");
			exit(1);
			}
		SDend(hyp_hdf);

		if (verbose || debug)
			printf("%s:\n", argv[optind]);
		if (debug)
			{
			for (i=0; i<dimsizes[0]; i++)
				printf("%d %s\n", image_mode[i], fmttime(image_mode_ts[i]));
			}
		else if (interval_start)
			findinterval(image_mode, image_mode_ts, dimsizes[0],
				interval_start, interval_end, verbose);
		else
			dumpmode(image_mode, image_mode_ts, dimsizes[0], verbose);
		}
	exit(0);
	}

void
findinterval(uint16 *image_mode, float64 *image_mode_ts, int32 n,
	float64 interval_start, float64 interval_end, int verbose)
	{
	int i;
	float64 image_start = 0.0;
	float64 image_end = 0.0;

	if (verbose)
		printf("Looking for range %s to %s\n",
			fmttime(interval_start), fmttime(interval_end));
	for (i=1; i<n; i++)
		{
		if (image_mode[i] > 1)
			if (image_start)
				image_end = image_mode_ts[i];
			else
				image_start = image_mode_ts[i-1];
		else if (image_start)
			{
			if (!image_end)
				image_end = image_start; /* one second image interval */
			/* We have an image interval, check to see if it overlaps */
			if (verbose)
				printf("Checking interval %s - %s for overlap\n",
					fmttime(image_start), fmttime(image_end));
			if ((interval_start <= image_end) && (interval_end >= image_start))
				{
				printf("%s %s\n", fmttime(image_start), fmttime(image_end));
				return;
				}
			else
				image_start = image_end = 0.0;
			}
		}
	}

/*
 * Parse a timestamp range, return 0 if error, 1 for success.
 */
int
getinterval(char *str, float64 *interval_start, float64 *interval_end)
	{
	char *buffer;
	char *p;

	buffer = (char *)malloc(strlen(str) + 1);
	if (!buffer)
		{
		perror("out of memory");
		exit(1);
		}
	strcpy(buffer, str);
	p = strchr(buffer, '-');
	if (!p)
		{
		free(buffer);
		return 0;
		}
	*p = '\0';
	*interval_start = timeunfmt(buffer);
	*interval_end = timeunfmt(p+1);
	free(buffer);
	return 1;
	}

void
dumpmode(uint16 *image_mode, float64 *image_mode_ts, int32 n, int verbose)
	{
	int i;
	uint16 prev_image_mode = image_mode[0];
	float64 prev_ts = image_mode_ts[0];
	for (i=0; i<n; i++)
		{
		if (image_mode[i] != prev_image_mode)
			{
			if (verbose)
				printf("mode %d from %s to %s, duration %.0f\n",
					prev_image_mode,
					fmttime(prev_ts),
					fmttime(image_mode_ts[i-1]),
					image_mode_ts[i-1] - prev_ts);
			else if (prev_image_mode > 1)
				printf("%s %s\n",
					fmttime(prev_ts),
					fmttime(image_mode_ts[i-1]));
			prev_image_mode = image_mode[i];
			prev_ts = image_mode_ts[i];
			}
		}
	}

char *fmttime(float64 t)
	{
	/* static char formatted[1024]; */
	char *formatted;
	time_t seconds;
	int milliseconds;
	struct tm *utc;

	if (!(formatted = (char *)malloc(22)))
		return NULL;
	seconds = floor(t);
	milliseconds = 1000 * (t - seconds);
	utc = gmtime(&seconds);
	/* sprintf(formatted, "%d.%03d\0", seconds, milliseconds); */
	sprintf(formatted, "%04d:%03d:%02d:%02d:%02d.%03d",
		utc->tm_year+1900, utc->tm_yday+1, utc->tm_hour, utc->tm_min,
		utc->tm_sec, milliseconds);
	return formatted;
	}

/*
 * Inverse of fmttime, takes a string formatted YYYY:DDD:HH:MM:SS.mmm and
 * returns the equivalent float (which is roughly a time_t in the integer
 * part and fraction of a second in the fractional part).  Everything is UTC.
 */
float64 timeunfmt(char *formatted)
	{
	struct tm tm;
	int monthlen[12] = {31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31};
	char *buffer = (char *)malloc(strlen(formatted) + 1);
	if (!buffer)
		return 0.0;
	strcpy(buffer, formatted);

	/* Split the fields, blindly assuming that everthing is formatted properly */
	buffer[4] = '\0';
	buffer[8] = '\0';
	buffer[11] = '\0';
	buffer[14] = '\0';
	buffer[17] = '\0';

	tm.tm_year = atoi(buffer);
	tm.tm_yday = atoi(buffer+5);
	tm.tm_hour = atoi(buffer+9);
	tm.tm_min = atoi(buffer+12);
	tm.tm_sec = atoi(buffer+15);

	if (((tm.tm_year % 4) == 0) && (((tm.tm_year % 100) != 0) || (tm.tm_year % 400 == 0)))
		monthlen[1] = 29;
	for (tm.tm_mon=0; tm.tm_yday > monthlen[tm.tm_mon]; tm.tm_mon++)
		tm.tm_yday -= monthlen[tm.tm_mon];
	tm.tm_mday = tm.tm_yday;
	tm.tm_year -= 1900;
	free(buffer);
	return (float64)timegm(&tm) + ((float64)atoi(buffer+18) / pow(10, strlen(buffer+18)));
	}

#ifdef sun
time_t timegm (struct tm *tm)
	{
	time_t ret;
	char *tz;

	tz = getenv("TZ");
	setenv("TZ", "GMT0", 1);
	tzset();
	ret = mktime(tm);
	if (tz)
		setenv("TZ", tz, 1);
	else
		unsetenv("TZ");
	tzset();
	return ret;
	}
#endif
