% Function Name:  atoeul
%
% Purpose - Converts attitude matrix to Euler angles
%
% Invocation Method:   [angles] = atoeul( A, mode)
%
% Argument       I/O  Description
% -------------  ---  -------------------------------------------------
% A(9xn or 3x3n)  i    Array of attitude matrices in either 9xn column
%                      format or 3x3n side-by-side format
%
% mode(1 or 3)    i    Type of Euler rotation
%                      Note:  will accept mode=3 or mode=231 or [2;3;1]
%                             or [2 3 1] for a 2-3-1 Euler sequence
%
%                      1 = 1-2-3 ROTATION;   7 = 1-2-1 ROTATION
%                      2 = 1-3-2 ROTATION;   8 = 1-3-1 ROTATION
%                      3 = 2-3-1 ROTATION;   9 = 2-1-2 ROTATION
%                      4 = 2-1-3 ROTATION;  10 = 2-3-2 ROTATION
%                      5 = 3-1-2 ROTATION;  11 = 3-1-3 ROTATION
%                      6 = 3-2-1 ROTATION;  12 = 3-2-3 ROTATION
%
% angles(3xn)      o   (rad) Array of 1st,2nd,3rd Euler angles = (PHI,THET,PSI)
%                      Note:  for singular cases, the 1st and 3rd Euler
%                             angles are ambiguous; set the 1st angle to 0
%                             and return the net rotation in the 3rd angle.
%
% External References:
% Function Name		Purpose
% -------------		-------
% none
%
% Global References:
% Parameters  Type  I/O  Description
% ----------  ----  ---  ----------------------------------------------
% none 
%

% Development History:
% Name          Date      Description of Change
% ------------  --------  -------------------------------------------
% R. Harman     11/09/96  Created
% J. Sedlak     07/01/98 - modified to handle multiple input attitudes
%                        - accept mode in differing formats
%                        - correct PSI for singular cases for modes 7,10,11
%                        - correct mode 9 for normal case

      function [ angles ] = atoeul( A, mode)

% reshape input matrix into column 9xn format
      [ m_in, n_in] = size( A );
      if m_in == 3,
         n = n_in/3;
         T = reshape( A, 9, n);
      else
         n = n_in;
         T = A;
      end;

% set mode
      if length( mode )==3,
         mode = 100*mode(1) + 10*mode(2) + mode(3);
      end;
      
% set sin and cos row vectors according to mode
      if  mode==1 | mode==123
         ntyp =  1;
         S2   =  T(3,:);
         S3C2 = -T(2,:);
         C3C2 =  T(1,:);
         S1C2 = -T(6,:);
         C1C2 =  T(9,:);
         S3C1 =  T(4,:);
         C3C1 =  T(5,:);
      elseif  mode==2 | mode==132
         ntyp =  1;
         S2   = -T(2,:);
         S3C2 =  T(3,:);
         C3C2 =  T(1,:);
         S1C2 =  T(8,:);
         C1C2 =  T(5,:);
         S3C1 = -T(7,:);
         C3C1 =  T(9,:);
      elseif  mode==3 | mode==231
         ntyp =  1;
         S2   =  T(4,:);
         S3C2 = -T(6,:);
         C3C2 =  T(5,:);
         S1C2 = -T(7,:);
         C1C2 =  T(1,:);
         S3C1 =  T(8,:);
         C3C1 =  T(9,:);
      elseif  mode==4 | mode==213
         ntyp =  1;
         S2   = -T(6,:);
         S3C2 =  T(4,:);
         C3C2 =  T(5,:);
         S1C2 =  T(3,:);
         C1C2 =  T(9,:);
         S3C1 = -T(2,:);
         C3C1 =  T(1,:);
      elseif  mode==5 | mode==312
         ntyp =  1;
         S2   =  T(8,:);
         S3C2 = -T(7,:);
         C3C2 =  T(9,:);
         S1C2 = -T(2,:);
         C1C2 =  T(5,:);
         S3C1 =  T(3,:);
         C3C1 =  T(1,:);
      elseif  mode==6 | mode==321
         ntyp =  1;
         S2   = -T(7,:);
         S3C2 =  T(8,:);
         C3C2 =  T(9,:);
         S1C2 =  T(4,:);
         C1C2 =  T(1,:);
         S3C1 = -T(6,:);
         C3C1 =  T(5,:);
      elseif  mode==7 | mode==121
         ntyp =  2;
         C2   =  T(1,:);
         C1S2 = -T(7,:);
         S1S2 =  T(4,:);
         S3S2 =  T(2,:);
         C3S2 =  T(3,:);
         S1C3 = -T(6,:);
         C1C3 =  T(5,:);
      elseif  mode==8 | mode==131
         ntyp =  2;
         C2   =  T(1,:);
         C1S2 =  T(4,:);
         S1S2 =  T(7,:);
         S3S2 =  T(3,:);
         C3S2 = -T(2,:);
         S1C3 =  T(8,:);
         C1C3 =  T(9,:);
      elseif  mode==9 | mode==212
         ntyp =  2;
         C2   =  T(5,:);
         C1S2 =  T(8,:);
         S1S2 =  T(2,:);
         S3S2 =  T(4,:);
         C3S2 = -T(6,:);
         S1C3 =  T(3,:);
         C1C3 =  T(1,:);
      elseif  mode==10 | mode==232
         ntyp =  2;
         C2   =  T(5,:);
         C1S2 = -T(2,:);
         S1S2 =  T(8,:);
         S3S2 =  T(6,:);
         C3S2 =  T(4,:);
         S1C3 = -T(7,:);
         C1C3 =  T(9,:);
      elseif  mode==11 | mode==313
         ntyp =  2;
         C2   =  T(9,:);
         C1S2 = -T(6,:);
         S1S2 =  T(3,:);
         S3S2 =  T(7,:);
         C3S2 =  T(8,:);
         S1C3 = -T(2,:);
         C1C3 =  T(1,:);
      elseif  mode==12 | mode==323
         ntyp =  2;
         C2   =  T(9,:);
         C1S2 =  T(3,:);
         S1S2 =  T(6,:);
         S3S2 =  T(8,:);
         C3S2 = -T(7,:);
         S1C3 =  T(4,:);
         C1C3 =  T(5,:);
      end;

      zer  = zeros(1,n);
      PHI  = zeros(1,n);
      THET = zeros(1,n);
      PSI  = zeros(1,n);

% IF sequence includes rotations about all 3 axes
      if ntyp==1
% compute 2nd rotation angle
         THET     = asin(S2);
% special case if magnitude of 2nd angle is pi/2 or 3pi/2
         ind      = find( abs(1-abs(S2)) < 1.e-14 );
         PSI(ind) = atan2( S3C1(ind), C3C1(ind));     %net rotation for 1 & 3
         PHI(ind) = zer(ind);                         %zero rotation on 1
% normal case; magnitude of 2nd angle not pi/2 or 3pi/2
         ind      = find( abs(1-abs(S2)) >= 1.e-14 );
         PSI(ind) = atan2( S3C2(ind), C3C2(ind));
         PHI(ind) = atan2( S1C2(ind), C1C2(ind));

% ELSE if sequence includes rotations about only 2 distinct axes
      else
% compute 2nd rotation angle
         THET     = acos(C2);
% special case if magnitude of 2nd angle is 0 or pi
% (S1C3, C1C3 are misnamed (here they yield PSI not PHI); they
%  actually combine 1st & 3rd angles into sum or diff, as needed)
         ind      = find( abs(1-abs(C2)) < 1.e-14 );
         PSI(ind) = atan2( S1C3(ind), C1C3(ind));     %net rotation for 1 & 3
         PHI(ind) = zer(ind);                         %zero rotation on 1
% normal case; magnitude of 2nd angle not 0 or pi
         ind      = find( abs(1-abs(C2)) >= 1.e-14 );
         PHI(ind) = atan2( S1S2(ind), C1S2(ind));
         PSI(ind) = atan2( S3S2(ind), C3S2(ind));
      end;

      angles = [ PHI; THET; PSI ];
