#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <math.h>
#include "hdf.h"
#include "mfhdf.h"

/*
 * Compute orbit inclincation, in degrees, from the position and
 * velocity vectors.
 */
double inclination(double pos[3], double vel[3])
	{
	double h[3];
	cross(h, pos, vel);
	/*
	printf("pos=(%f,%f,%f) vel=(%f,%f,%f) inclination=%02f\n",
		pos[0], pos[1], pos[2],
		vel[0], vel[1], vel[2],
		180.0 + (atan(sqrt(h[0]*h[0] + h[1]*h[1])/h[2]) * 360.0 / (2.0 * M_PI)));
	*/
	return 180.0 + (atan(sqrt(h[0]*h[0] + h[1]*h[1])/h[2]) * 360.0 / (2.0 * M_PI));
	}
/*
 * Compute the cross product of two 3-dimensional vectors
 */
cross(double result[3], double a[3], double b[3])
	{
	result[0] = a[1] * b[2] - a[2] * b[1];
	result[1] = a[2] * b[0] - a[0] * b[2];
	result[2] = a[0] * b[1] - a[1] * b[0];
	}

/*
 * Given the name of an ACS housekeeping HDF file and a range of
 * timestamps, read the position and velocity vectors from
 * the ACS file and calculate the orbital inclination.
 *
 * Exit status:
 *	0 success
 *	1 failed to open then input file
 *	2 out of memory (malloc failed)
 *	3 problems with the HDF
 *	4 no timestamps within tolerance range
 */


int getvals(int32 acs_hdf, char *name, float64 **values_in_range, float64 starttime, float64 endtime);
char *substr(char *src, int start, int len);
float64 gettime(char *src);
/* If we can't find a timestamp within "tolerance" seconds, exit with an error */
float64 tolerance = 16.0;

main(int argc, char **argv)
	{
	int32 acs_hdf;
	float64 starttime;
	float64 endtime;
	float64 *posx;
	float64 *posy;
	float64 *posz;
	int32 nposx;
	int32 nposy;
	int32 nposz;
	float64 *velx;
	float64 *vely;
	float64 *velz;
	int32 nvelx;
	int32 nvely;
	int32 nvelz;
	float64 pos[3];
	float64 vel[3];

	if (argc < 3)
		{
		fprintf(stderr, "Usage: %s acs.hdf ts1 [ts2]\n", argv[0]);
		exit(1);
		}
	/* Expect file name and time spec (in UTC) as parameters */
	if ((acs_hdf = SDstart(argv[1], DFACC_RDONLY)) == FAIL)
		{
		fprintf(stderr, "SDstart(%s, DFACC_RDONLY) returned %d\n", argv[1], acs_hdf);
		exit(1);
		}
	/* Get the timestamp from argv[2] in YYYYDDDHHMMSS.uuu format */
	starttime = gettime(argv[2]);
	if (argc > 3)
		endtime = gettime(argv[3]);
	else
		endtime = starttime;

	nposx = getvals(acs_hdf, "ACSSCPOSXE", &posx, starttime, endtime);
	nposy = getvals(acs_hdf, "ACSSCPOSYE", &posy, starttime, endtime);
	nposz = getvals(acs_hdf, "ACSSCPOSZE", &posz, starttime, endtime);
	if ((nposx != nposy) || (nposx != nposz))
		{
		fprintf(stderr, "Position vectors have wrong number of components: %d %d %d\n", nposx, nposy, nposz);
		exit(5);
		}
	
	nvelx = getvals(acs_hdf, "ACSSCVELXE", &velx, starttime, endtime);
	nvely = getvals(acs_hdf, "ACSSCVELYE", &vely, starttime, endtime);
	nvelz = getvals(acs_hdf, "ACSSCVELZE", &velz, starttime, endtime);
	if ((nvelx != nposy) || (nvelx != nvelz))
		{
		fprintf(stderr, "Velocity vectors have wrong number of components: %d %d %d\n", nvelx, nvely, nvelz);
		exit(5);
		}
	if (nposx != nvelx)
		{
		fprintf(stderr, "Position and velocity vectors have differenct number of components: %d %d\n", nposx, nvelx);
		exit(5);
		}
	
	/*
	 * Compute the average.
	 */
	float64 ave_inclination = 0.0;
	int i;
	for (i=0; i < nposx; i++)
		{
		pos[0] = posx[i];
		pos[1] = posy[i];
		pos[2] = posz[i];
		vel[0] = velx[i];
		vel[1] = vely[i];
		vel[2] = velz[i];
		ave_inclination += inclination(pos, vel);
		}
	ave_inclination = ave_inclination/nposx;
	printf("%0.2f\n", ave_inclination);

	SDend(acs_hdf);
	exit(0);
	}

/*
 * Read an array of values in a given timestamp range.
 * Returns the number of entries.
 */
int
getvals(int32 acs_hdf, char *name, float64 **values_in_range, float64 starttime, float64 endtime)
	{
	int32 index;
	int32 id;
	char *attr_name;
	float64 *ts;
	float64 *values;
	int32 rank;
	int32 dimsizes[MAX_VAR_DIMS];
	int32 data_type;
	int32 num_attrs;
	int32 start, edge;
	float64 mindiff;	/* minimum timestamp difference */
	float64 diff;
	int32 index_start;
	int32 index_end;
	int32 index_closest;

	attr_name = (char *)malloc(strlen(name) + 32);
	if (!attr_name)
		{
		fprintf(stderr, "out of memory");
		exit(2);
		}

	/* First, get the timestamps */
	strcpy(attr_name, "A@");
	strcat(attr_name, name);
	strcat(attr_name, "(TIME_MEASURED)");
	if ((index = SDnametoindex(acs_hdf, attr_name)) == FAIL)
		{
		fprintf(stderr, "Missing attribute %s\n", attr_name);
		exit(3);
		}
	if ((id = SDselect(acs_hdf, index)) == FAIL)
		{
		fprintf(stderr, "SDselect failed\n");
		exit(3);
		}
	if (SDgetinfo(id, (char *)0, &rank, dimsizes, &data_type, &num_attrs) == FAIL)
		{
		fprintf(stderr, "SDgetinfo failed\n");
		exit(3);
		}
	if (data_type != DFNT_FLOAT64)
		{
		fprintf(stderr, "%s is not 64-bit floating point data\n", attr_name);
		exit(3);
		}
	start = 0;
	edge = dimsizes[0];
	ts = (float64 *)malloc(dimsizes[0] * sizeof(float64));
	if (!ts)
		{
		fprintf(stderr, "out of memory");
		exit(2);
		}
	if (SDreaddata(id, &start, (int32 *)0, &edge, ts) == FAIL)
		{
		fprintf(stderr, "failed reading data\n");
		exit(3);
		}

	/* Next get the actual values */
	strcpy(attr_name, "P@");
	strcat(attr_name, name);
	if ((index = SDnametoindex(acs_hdf, attr_name)) == FAIL)
		{
		fprintf(stderr, "Missing attribute %s\n", attr_name);
		exit(3);
		}
	if ((id = SDselect(acs_hdf, index)) == FAIL)
		{
		fprintf(stderr, "SDselect failed\n");
		exit(3);
		}
	if (SDgetinfo(id, (char *)0, &rank, dimsizes, &data_type, &num_attrs) == FAIL)
		{
		fprintf(stderr, "SDgetinfo failed\n");
		exit(3);
		}
	if (data_type != DFNT_FLOAT64)
		{
		fprintf(stderr, "%s is not 64-bit floating point data\n", attr_name);
		exit(3);
		}
	start = 0;
	edge = dimsizes[0];
	values = (float64 *)malloc(dimsizes[0] * sizeof(float64));
	if (!values)
		{
		fprintf(stderr, "out of memory");
		exit(2);
		}
	if (SDreaddata(id, &start, (int32 *)0, &edge, values) == FAIL)
		{
		fprintf(stderr, "failed reading data\n");
		exit(3);
		}

	/*
	 * Now look for data in the desired range, or the
	 * closest single value if nothing is in range.
	 * corresponding value
	 */
	mindiff = 72000.0;
	index_start = -1;
	index_end = -1;
	for (index=0; index < edge; index++)
		{
		if ((ts[index] >= starttime) && (ts[index] <= endtime))
			{
			if (index_start < 0)
				index_start = index;
			index_end = index;
			}
		else
			{
			diff = fabs(ts[index] - ((starttime + endtime)/2));
			if (diff < mindiff)
				{
				mindiff = diff;
				index_closest = index;
				}
			}
		}
	free(attr_name);
	if (index_start < 0)
		{
		if (mindiff > tolerance)
			{
			fprintf(stderr, "No timestamps within %f seconds (mindiff=%f) for %s\n",
				tolerance, mindiff, name);
			exit(4);
			}
		/*
		printf("using %s value %f at %f, index %d, diff %f\n", name,
			values[index_closest], ts[index_closest], index_closest, mindiff);
		*/
		index_start = index_end = index_closest;
		}
	*values_in_range = values + index_start;
	return (index_end - index_start + 1);
	}

/*
 * substring routine, returns space allocated with malloc
 */
char *substr(char *src, int start, int len)
	{
	char *buf = (char *)malloc(len+1);
	if (buf)
		{
		strncpy(buf, src+start, len);
		buf[len] = '\0';
		}
	return buf;
	}
/*
 * Convert YYYYDDDHHMMSS.uuu to seconds since 1970/01/01 UTC
 */
float64 gettime(char *src)
	{
	float64 fraction;
	int monthlen[12] = {31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30 ,31};
	struct tm t;
	int i;

	t.tm_year = atoi(substr(src, 0, 4));
	t.tm_yday = atoi(substr(src, 4, 3));
	t.tm_hour = atoi(substr(src, 7, 2));
	t.tm_min = atoi(substr(src, 9, 2));
	t.tm_sec = atoi(substr(src, 11, 2));
	fraction = atof(substr(src, 13, 4));
	if (((t.tm_year % 4) == 0) && (((t.tm_year % 100) != 0) || ((t.tm_year % 400) == 0)))
		monthlen[1] = 29;
	else
		monthlen[1] = 28;
	t.tm_mon = 0;
	for (i=0; i<12; i++)
		{
		if (t.tm_yday > monthlen[i])
			{
			t.tm_yday -= monthlen[i];
			t.tm_mon++;
			}
		else
			{
			t.tm_mday = t.tm_yday;
			break;
			}
		}
	t.tm_year -= 1900;
	return ((float64)timegm(&t)) + fraction;
	}
