% Function Name: lookangle.m
%
% Purpose: Computes EO-1 ALI/HSI look angles based on parameters available in the HDF file.
%
% Invocation Method:
%  [ALI_lookangle, HSI_lookangle ]  =  lookangle( pos, vel, quat )
%
% Argument       I/O  Description
% -------------  ---  --------------------------------------------------------------------
% pos(3,1)        I   EO-1 GCI Position Vector (ACSSCPOSXE,ACSSCPOSYE,ACSSCPOSZE)  
% vel(3,1)        I   EO-1 GCI Velocity Vector (ACSSCVELXE,ACSSCVELYE,ACSSCVELZE)
% quat(4,1)       I   EO-1 GCI Attitude Quaterion (ACSKFQ1C,ACSKFQ2C,ACSKFQ3C,ACSKFQ4C)
% ALI_lookangle   O   ALI Look Angle (deg)
% HSI_lookangle   O   HSI Look Angle (deg)
%
% Note:  pos,vel, and quat must all occur at the same time !!!
%
% External References:
% Function Name     Purpose
% -------------     -------------------------------------------------
% unitvec           Unitize a vector            
% qtoa              Convert quaterion to attitude matrix
% atoeul            Convert attitude matrix to Euler angles

% Development History:
% Name          Date      Description of Change
% ------------  --------  -------------------------------------------
% S. Shulman    09/19/02  Created
% S. Shulman    11/19/03  Updated ALI/HSI Roll offsets to post J2000/TOD
%                            conversion error values

function [ ALI_lookangle, HSI_lookangle ]  =  lookangle( pos, vel, quat )

% Define nominal ALI/HSI roll offset angles (deg)
ALI_rolloffset = 6.302; %Post TOD correction bias 
HSI_rolloffset = 4.952; %Post TOD correction bias

% Define GCI to OCS attitude matrix
u_nadir = -unitvec(pos);
u_normal = unitvec(cross(pos,vel));
u_nxp = unitvec(cross(u_normal,pos));
A_GCI2OCS  = [u_nxp;-u_normal;u_nadir];

% Convert input attitude quaterion to matrix
A_GCI2BODY = qtoa(quat);

% Compute OCS to Body matrix
A_OCS2BODY = A_GCI2BODY * A_GCI2OCS';

% Convert to 1-2-3 Euler angles
Euler_angles = atoeul(A_OCS2BODY,[1;2;3]);

% Extract Roll angle (deg)
SC_roll = Euler_angles(1) * 180/pi;

% Compute instrument look angles (deg)
ALI_lookangle = SC_roll + ALI_rolloffset
HSI_lookangle = SC_roll + HSI_rolloffset



