/*
 * Read the ACS housekeeping HDF and output the position, velocity and
 * attitude quaternion vectors for a given time.  These can then be used
 * to calculate the sensor look angles.
 *
 * Exit status:
 *	0 success
 *	1 failed to open then input file
 *	2 out of memory (malloc failed)
 *	3 problems with the HDF
 *	4 no timestamps within tolerance range
 */

#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <math.h>
#include "hdf.h"
#include "mfhdf.h"

float64 getval(int32 acs_hdf, char *name, float64 timestamp);
char *substr(char *src, int start, int len);
float64 gettime(char *src);
/* If we can't find a timestamp within "tolerance" seconds, exit with an error */
float64 tolerance = 16.0;

main(int argc, char **argv)
	{
	int32 acs_hdf;
	float64 timestamp;
	time_t seconds;
	struct tm *tm;
	float64 pos[3];
	float64 vel[3];
	float64 quat[4];

	/* Expect file name and time spec (in UTC) as parameters */
	if ((acs_hdf = SDstart(argv[1], DFACC_RDONLY)) == FAIL)
		{
		fprintf(stderr, "SDstart(%s, DFACC_RDONLY) returned %d\n", argv[1], acs_hdf);
		exit(1);
		}
	/* Get the timestamp from argv[2] */
	if (argc > 3)
		{
		float64 starttime;
		float64 endtime;
		/* Start and stop time in YYYYDDDHHMMSS.uuu format, compute midpoint */
		timestamp = (gettime(argv[2]) + gettime(argv[3])) / 2.0;
		}
	else if (argc > 2)
		timestamp = atof(argv[2]);
	else
		timestamp = 1092757535.568306;


	printf("%% position, velocity and attitude at time ");
	seconds = (time_t)floor(timestamp);
	tm = gmtime(&seconds);
	printf("%04d:%03d:%02d:%02d:%02d\n",
		tm->tm_year + 1900, tm->tm_yday+1, tm->tm_hour, tm->tm_min, tm->tm_sec);

	pos[0] = getval(acs_hdf, "ACSSCPOSXE", timestamp);
	pos[1] = getval(acs_hdf, "ACSSCPOSYE", timestamp);
	pos[2] = getval(acs_hdf, "ACSSCPOSZE", timestamp);
	printf("pos = [%f, %f, %f];\n", pos[0], pos[1], pos[2]);
	vel[0] = getval(acs_hdf, "ACSSCVELXE", timestamp);
	vel[1] = getval(acs_hdf, "ACSSCVELYE", timestamp);
	vel[2] = getval(acs_hdf, "ACSSCVELZE", timestamp);
	printf("vel = [%f, %f, %f];\n", vel[0], vel[1], vel[2]);
	quat[0] = getval(acs_hdf, "ACSKFQ1C", timestamp);
	quat[1] = getval(acs_hdf, "ACSKFQ2C", timestamp);
	quat[2] = getval(acs_hdf, "ACSKFQ3C", timestamp);
	quat[3] = getval(acs_hdf, "ACSKFQ4C", timestamp);
	printf("quat = [%f; %f; %f; %f];\n", quat[0], quat[1], quat[2], quat[3]);

	printf("[ali,hsi] = lookangle(pos, vel, quat);\n");

	SDend(acs_hdf);
	exit(0);
	}

float64
getval(int32 acs_hdf, char *name, float64 timestamp)
	{
	int32 index;
	int32 id;
	char *attr_name;
	float64 *ts;
	float64 *values;
	int32 rank;
	int32 dimsizes[MAX_VAR_DIMS];
	int32 data_type;
	int32 num_attrs;
	int32 start, edge;
	float64 mindiff;	/* minimum timestamp difference */
	float64 diff;
	float64 target;
	int32 chosen_index;

	attr_name = (char *)malloc(strlen(name) + 32);
	if (!attr_name)
		{
		fprintf(stderr, "out of memory");
		exit(2);
		}

	/* First, get the timestamps */
	strcpy(attr_name, "A@");
	strcat(attr_name, name);
	strcat(attr_name, "(TIME_MEASURED)");
	if ((index = SDnametoindex(acs_hdf, attr_name)) == FAIL)
		{
		fprintf(stderr, "Missing attribute %s\n", attr_name);
		exit(3);
		}
	if ((id = SDselect(acs_hdf, index)) == FAIL)
		{
		fprintf(stderr, "SDselect failed\n");
		exit(3);
		}
	if (SDgetinfo(id, (char *)0, &rank, dimsizes, &data_type, &num_attrs) == FAIL)
		{
		fprintf(stderr, "SDgetinfo failed\n");
		exit(3);
		}
	if (data_type != DFNT_FLOAT64)
		{
		fprintf(stderr, "%s is not 64-bit floating point data\n", attr_name);
		exit(3);
		}
	start = 0;
	edge = dimsizes[0];
	ts = (float64 *)malloc(dimsizes[0] * sizeof(float64));
	if (!ts)
		{
		fprintf(stderr, "out of memory");
		exit(2);
		}
	if (SDreaddata(id, &start, (int32 *)0, &edge, ts) == FAIL)
		{
		fprintf(stderr, "failed reading data\n");
		exit(3);
		}

	/* Next get the actual values */
	strcpy(attr_name, "P@");
	strcat(attr_name, name);
	if ((index = SDnametoindex(acs_hdf, attr_name)) == FAIL)
		{
		fprintf(stderr, "Missing attribute %s\n", attr_name);
		exit(3);
		}
	if ((id = SDselect(acs_hdf, index)) == FAIL)
		{
		fprintf(stderr, "SDselect failed\n");
		exit(3);
		}
	if (SDgetinfo(id, (char *)0, &rank, dimsizes, &data_type, &num_attrs) == FAIL)
		{
		fprintf(stderr, "SDgetinfo failed\n");
		exit(3);
		}
	if (data_type != DFNT_FLOAT64)
		{
		fprintf(stderr, "%s is not 64-bit floating point data\n", attr_name);
		exit(3);
		}
	start = 0;
	edge = dimsizes[0];
	values = (float64 *)malloc(dimsizes[0] * sizeof(float64));
	if (!values)
		{
		fprintf(stderr, "out of memory");
		exit(2);
		}
	if (SDreaddata(id, &start, (int32 *)0, &edge, values) == FAIL)
		{
		fprintf(stderr, "failed reading data\n");
		exit(3);
		}

	/*
	 * Now look for the closest timestamp, and return the
	 * corresponding value
	 */
	mindiff = 72000.0;
	for (index=0; index < edge; index++)
		{
		diff = fabs(ts[index] - timestamp);
		if (diff < mindiff)
			{
			target = values[index];
			mindiff = diff;
			chosen_index = index;
			}
		}
	/*
	printf("using value %f at %f, index %d, diff %f\n",
		target, ts[chosen_index], chosen_index, mindiff);
	*/
	free(attr_name);
	free(ts);
	free(values);
	if (mindiff > tolerance)
		{
		fprintf(stderr, "No timestamps within %f seconds (mindiff=%f) for %s\n",
			tolerance, mindiff, name);
		exit(4);
		}
	return target;
	}

/*
 * substring routine, returns space allocated with malloc
 */
char *substr(char *src, int start, int len)
	{
	char *buf = (char *)malloc(len+1);
	if (buf)
		{
		strncpy(buf, src+start, len);
		buf[len] = '\0';
		}
	return buf;
	}
/*
 * Convert YYYYDDDHHMMSS.uuu to seconds since 1970/01/01 UTC
 */
float64 gettime(char *src)
	{
	float64 fraction;
	int monthlen[12] = {31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30 ,31};
	struct tm t;
	int i;

	t.tm_year = atoi(substr(src, 0, 4));
	t.tm_yday = atoi(substr(src, 4, 3));
	t.tm_hour = atoi(substr(src, 7, 2));
	t.tm_min = atoi(substr(src, 9, 2));
	t.tm_sec = atoi(substr(src, 11, 2));
	fraction = atof(substr(src, 13, 4));
	if (((t.tm_year % 4) == 0) && (((t.tm_year % 100) != 0) || ((t.tm_year % 400) == 0)))
		monthlen[1] = 29;
	else
		monthlen[1] = 28;
	t.tm_mon = 0;
	for (i=0; i<12; i++)
		{
		if (t.tm_yday > monthlen[i])
			{
			t.tm_yday -= monthlen[i];
			t.tm_mon++;
			}
		else
			{
			t.tm_mday = t.tm_yday;
			break;
			}
		}
	t.tm_year -= 1900;
	return ((float64)timegm(&t)) + fraction;
	}
