#!/usr/bin/perl
# Read through EO-1 inventory metadata input files and generate an email message
# to notify people about what has been ingested recently.

use strict;
use vars qw($opt_d $opt_h $opt_i $opt_t $opt_c $opt_T);
use Getopt::Std;
getopts("d:h:i:c:t:T") || die(<<END_USAGE);
Usage: $0 [-d dir] [-h #] [-i dir] [-t list]
Where
	-d dir	use "dir" as the head of the directory, default is /hsm/lta4/eo1
	-h #	Look for files produced in the last # hours, default is 24.
	-i dir	Use "dir" as the directory with the inventory metadata files,
		default is $opt_d/ingest/tmp
	-t list	Send email to list, see below for default.
	-c list	CC list for email
	-T	test
END_USAGE
$opt_d ||= "/hsm/lta4/eo1";
$opt_h ||= 24;
$opt_i ||= "$opt_d/ingest/tmp";
$opt_t = 'Dan.mandl@gsfc.nasa.gov,Lawrence.Ong.1@gsfc.nasa.gov,Stephen.G.Ungar.1@gsfc.nasa.gov,nick.speciale@gsfc.nasa.gov,mendenhall@ll.mit.edu,dennis.reuter@gsfc.nasa.gov,larry.a.alexander@nasa.gov,stuart.frye@nasa.gov,jesse.allen@nasa.gov,adam.p.voiland@nasa.gov' unless defined($opt_t);
$opt_c = 'katherine.goodale.ctr@usgs.gov,leann.dix.ctr@usgs.gov,michael.neiers.ctr@usgs.gov' unless defined($opt_c);

my $cutofftime = time() - ($opt_h * 3600);
my %scenes;

opendir(TMP, $opt_i) || die("$opt_i: $!");
while (my $file = readdir(TMP))
	{
	# EO1YYYYMMDDHHMMSS
	# EO120051001050510.ali
	next unless $file =~ m/^EO1(\d\d\d\d)(\d\d)(\d\d)(\d\d)(\d\d)(\d\d)\.(ali|hyp)$/;
	my ($year, $month, $day, $hour, $minute, $second, $instrument) =
		($1, $2, $3, $4, $5, $6, $7);
	my @s = stat("$opt_i/$file");
	unless (@s)
		{
		warn("stat $opt_i/$file: $!");
		next;
		}
	if ($s[9] >= $cutofftime)
		{
		readfile("$opt_i/$file", $instrument);
		}
	}
closedir(TMP);

unless (%scenes)
	{
	print "No new EO-1 data in the past $opt_h hours\n";
	exit(0);
	}

my $subject = "GSFC-processed L0 scenes loaded in the last $opt_h hours";
my $mailcmd = "/home/eo1/inventory/mailfiles -t $opt_t -s '$subject'";
$mailcmd .= " -c $opt_c" if $opt_c;
if ($opt_T)
	{
	$mailcmd = "(echo $mailcmd;cat)";
	}
open(MAIL, "|$mailcmd") || die("fork: $!");
print MAIL <<END_MAIL;
$subject


END_MAIL
my %total;
foreach my $sceneid (keys %scenes)
	{
	foreach my $i (qw(ali hyp))
		{
		if ($scenes{$sceneid}->{$i})
			{
			$total{$i}++;
			}
		}
	}
foreach my $i (qw(ali hyp))
	{
	printf(MAIL "Total number of %s scenes processed in the past %d days: %d\n",
		uc($i), $opt_h/24, $total{$i});
	}

foreach my $sceneid (sort byacqdate keys %scenes)
	{
	print MAIL "$sceneid $scenes{$sceneid}->{target}\n";
	foreach my $i (qw(ali hyp))
		{
		if ($scenes{$sceneid}->{$i})
			{
			print MAIL uc $i, " browse http://earthexplorer.usgs.gov/browse/eo-1/$i/",
				$scenes{$sceneid}->{$i}, "\n";
			}
		}
	print MAIL "\n";
	}
close(MAIL) || warn("mailfiles returned $?");

sub readfile
	{
	my ($file, $instrument) = @_;
	my $browse_offset = $instrument eq "ali" ? 27 : 28;
	unless (open(M, "<$file"))
		{
		warn("$file: $!");
		return;
		}
	while (<M>)
		{
		chomp;
		my @fields = split(/\|/, $_);
		my ($sceneid, $target, $browse) = ($fields[0], $fields[8], $fields[$browse_offset]);
		$sceneid =~ s/EO1[AH]/EO1/;
		$scenes{$sceneid}->{target} = $target;
		$scenes{$sceneid}->{$instrument} = $browse;
		}
	close(M);
	}

# # Sort by acquisition date
sub byacqdate
	{
	return ((substr($a, 9, 7) cmp substr($b, 9, 7)) || ($a cmp $b));
	}
