#!/usr/local/bin/perl
# Generate EO-1 ALI or Hyperion metadata
# Input files:
#	$opt_d/meta/00indexEO1???	to get a list of sceneids and files
#	$opt_d/YYYY/DDD/0event_YYYYDDD.(rpt|txt)
#	$opt_d/YYYY/DDD/meta/00meta_${sceneid}_${recstat}_??.DAT
#	(ali|hyp)corners.txt		corner lat/long from Mike C.'s program
#	eo1tapes			table of tape ids and EDC storage locations
#	browse.files			list of browse files
#	exclusions			list of scene IDs that should be excluded for various reasons

# Options:
#	-v	verbose
#	-d dir	parent directory of everything
#	-i ins	instrument: default is "ali"
#	-s c	set the field separator character to "c", default is "|"
#	-A	look for all 00indexEO1??? files to process, otherwise expect a list of tape IDs
#	-h	print header line

use strict;
use vars qw($opt_v $opt_d $opt_i $opt_s $opt_A $opt_h);
use Getopt::Std;
require "gmt.pl";
my %filesize;
my $total_records = 0;
my $missing_corners = 0;
my $missing_browse = 0;

getopts("vd:i:s:Ah");
unless (defined($opt_d))
	{
	foreach (qw(/data1/neiers/eo1 /edclxs37/eo1 .))
		{
		if (-d $_)
			{
			$opt_d = $_;
			last;
			}
		}
	}
$opt_i ||= "ali";
$opt_s ||= "|";

# The corners.txt file has corner points by scene start GMT.
my %corners = readcorners("${opt_i}corners.txt");

my %tapelocations = readtapelocations("eo1tapes");
my %browseurls = generatebrowseurls("browse.files", $opt_i);

my %exclusions = readexclusions("exclusions.$opt_i");

# Look through the metadata directory for report files and extract records.
my %rpt_records;	# Records from the reports files, by sceneid.
my $rec;

open(META, "find $opt_d -follow -name '0event*' -print|") || die("fork failed: $!");
my $reportfile;
while (defined($reportfile = <META>))
	{
	next unless $reportfile =~ m/0event_\d\d\d\d_*\d\d\d\.(rpt|txt)/;
	chomp $reportfile;
	open(RPT, "<$reportfile") || die("$reportfile: $!");
	while (<RPT>)
		{
		if (m/^ADD (\d\d\d\d:\d\d\d:\d\d:\d\d:\d\d).*(\d\d\d\d:\d\d\d:\d\d:\d\d:\d\d).*'([^']*)'/)
			{
			$rec = {};
			$rec->{starttime} = $1;
			$rec->{endtime} = $2;
			$rec->{recordtype} = $3;
			$rec->{reportfile} = $reportfile;
			next;
			}
		next unless defined $rec;
		if (m/^;$/)
			{
			if (defined($rec->{'SCENE ID'}))
				{
				if (defined($rpt_records{$rec->{'SCENE ID'}}))
					{
					# Merge the records
					my $prev_rec = $rpt_records{$rec->{'SCENE ID'}};
					foreach my $f (keys %$rec)
						{
						$prev_rec->{$f} = $rec->{$f};
						}
					print STDERR "Duplicate scene ID: ",
						$rec->{'SCENE ID'}, " in\n\t",
						$rpt_records{$rec->{'SCENE ID'}}->{reportfile},
						"\n\t$reportfile\n" if $opt_v;
					}
				else
					{
					$rpt_records{$rec->{'SCENE ID'}} = $rec;
					}
				}
			undef $rec;
			}
		elsif (m/'([^']*)'\s+=\s+'([^']*)'/)
			{
			$rec->{$1} = $2;
			}
		}
	close(RPT);
	}
close(META) || warn("find returned $?");

print STDERR scalar(keys %rpt_records), " rpt records\n" if $opt_v;

if ($opt_h)
	{
	print join($opt_s, qw(
		entity_id
		orbit_path
		orbit_row
		acquisition_date
		cloud_cover
		quality
		scene_status
		rec_station
		special_request
		scene_start_time
		scene_center_time
		scene_stop_time
		sun_azimuth
		sun_elevation
		data_type
		));
	print $opt_s;
	print("processing_level", $opt_s) unless ($opt_i eq "ali");
	print join($opt_s, qw(
		scene_center_lat
		scene_center_long
		ul_corner_lat
		ul_corner_long
		ur_corner_lat
		ur_corner_long
		ll_corner_lat
		ll_corner_long
		lr_corner_lat
		lr_corner_long
		day_or_night
		browse_availability
		browse_image_file_path
		working_dlt_id
		working_dlt_location
		archive_dlt_id
		archive_dlt_location
		));
	print $opt_s;
	unless ($opt_i eq "ali")
		{
		print join($opt_s, qw(
			working_level1_id
			working_level1_location 
			archive_level1_id
			archive_level1_location
			)), $opt_s;
		}
	print join($opt_s, qw(
		raw_dlt_id
		raw_dlt_location
		cal_file_used
		file_number
		directory_name
		restriction_comments
		look_angle
		inclination
		)), "\n";
	}

# Next, read through the tape index files to find scene IDs
my %scenes;
my $file;
if ($opt_A)
	{
	@ARGV = findindexfiles();
	}
foreach my $tapeid (@ARGV)
	{
	undef %filesize;
	foreach my $file (readindexfile("$opt_d/meta/00index$tapeid"))
		{
		if ($file =~ m"^(ali|hyp)/.*/(EO1\d\d\d\d\d\d\d\d\d\d\d\d\d[01][01][012][NPKLSDAIX][A-Z0-9])/.*(_..._\d\d)(\.M[1234]Z$|\.L0$)")
			{
			my ($instrument, $entity_id) = ($1, $2 . $3);
			$scenes{$entity_id}->{$instrument}->{files}++;
			$scenes{$entity_id}->{$instrument}->{bytes} += $filesize{$file};
			$scenes{$entity_id}->{$instrument}->{tapeid}->{$tapeid} = 1;
			($scenes{$entity_id}->{$instrument}->{directory_name} = $file) =~ s"/[^/]*$"";
			}
		}
	}

foreach my $sceneid (sort keys %scenes)
	{
	next unless defined($scenes{$sceneid}->{$opt_i});
	my $short_sceneid = substr($sceneid,0,21);
	my $rpt_rec = $rpt_records{$short_sceneid};
	unless (defined($rpt_rec))
		{
		print STDERR "No report file for $sceneid\n" if $opt_v;
		$rpt_rec->{'SCENE ID'} = $sceneid;
		}
	print STDERR "scene $sceneid\n" if $opt_v;
	$total_records++;
	if (defined($rpt_rec->{starttime}))
		{
		$scenes{$sceneid}->{$opt_i}->{acquisition_date} = yydddtoyymmmdd(substr($rpt_rec->{starttime}, 0, 4),
			substr($rpt_rec->{starttime}, 5, 3));
		}
	else
		{
		$scenes{$sceneid}->{$opt_i}->{acquisition_date} = yydddtoyymmmdd(substr($sceneid, 9, 4), substr($sceneid, 13,3));
		}
	# Get the receiving station and the real start time from the DAT file.
	unless (finddat($sceneid))
		{
		$scenes{$sceneid}->{$opt_i}->{starttime} = $rpt_rec->{starttime};
		$scenes{$sceneid}->{$opt_i}->{endtime} = $rpt_rec->{endtime};
		$scenes{$sceneid}->{$opt_i}->{special_request} = $rpt_rec->{'SPECIAL REQUEST'};
		}

	# Find the corner point coordinates
	my $time;
	my $timediff = 99999999;	# difference between target time and corner time
	if ($scenes{$sceneid}->{$opt_i}->{starttime})
		{
		foreach (keys %corners)
			{
			if ((gmtdiff($scenes{$sceneid}->{$opt_i}->{starttime}, $_) >= 0) &&
			   (gmtdiff($_, $scenes{$sceneid}->{$opt_i}->{endtime}) >= 0))
				{
				$timediff = 0;
				$time = $_;
				last;
				}
			my $diff = abs(gmtdiff($_, $scenes{$sceneid}->{$opt_i}->{starttime}));
			if ((!defined($timediff)) || ($diff < $timediff))
				{
				$timediff = $diff;
				$time = $_;
				}
			}
		}
	if ($timediff > 3)
		{
		$missing_corners++;
		print STDERR "No corners for $sceneid $scenes{$sceneid}->{$opt_i}->{starttime}, $scenes{$sceneid}->{$opt_i}->{endtime}, closest is $time\n";
		}
	else
		{
		$scenes{$sceneid}->{$opt_i}->{ul_corner_lat} = $corners{$time}->{ULlat};
		$scenes{$sceneid}->{$opt_i}->{ul_corner_long} = $corners{$time}->{ULlon};
		$scenes{$sceneid}->{$opt_i}->{ur_corner_lat} = $corners{$time}->{URlat};
		$scenes{$sceneid}->{$opt_i}->{ur_corner_long} = $corners{$time}->{URlon};
		$scenes{$sceneid}->{$opt_i}->{ll_corner_lat} = $corners{$time}->{LLlat};
		$scenes{$sceneid}->{$opt_i}->{ll_corner_long} = $corners{$time}->{LLlon};
		$scenes{$sceneid}->{$opt_i}->{lr_corner_lat} = $corners{$time}->{LRlat};
		$scenes{$sceneid}->{$opt_i}->{lr_corner_long} = $corners{$time}->{LRlon};
		# If we have coordinates, compute the sun angles at the center point
		my $ctr_lat = ($corners{$time}->{ULlat} +
			$corners{$time}->{URlat} +
			$corners{$time}->{LLlat} +
			$corners{$time}->{LRlat}) /4;
		my $ctr_lon = ($corners{$time}->{ULlon} +
			$corners{$time}->{URlon} +
			$corners{$time}->{LLlon} +
			$corners{$time}->{LRlon}) /4;
		($scenes{$sceneid}->{$opt_i}->{sun_elevation},
		 $scenes{$sceneid}->{$opt_i}->{sun_azimuth}) = split(/\s+/, 
		 	`sunangles $time $ctr_lat, $ctr_lon`);
		$scenes{$sceneid}->{$opt_i}->{day_or_night} = 
			($scenes{$sceneid}->{$opt_i}->{sun_elevation} < 0.0) ? "N" : "D";
		}

	# Add the A or H to the scene ID for ALI or Hyperion
	my $altered_sceneid = $sceneid;
	if ($opt_i eq "ali")
		{
		$altered_sceneid =~ s/^EO1/EO1A/;
		}
	elsif ($opt_i eq "hyp")
		{
		$altered_sceneid =~ s/^EO1/EO1H/;
		}
		
	# Output the fields.
	print $altered_sceneid, $opt_s;
	foreach my $field (
		'PATH',
		'START ROW')
		{
		print $rpt_rec->{$field}, $opt_s;
		}
	foreach my $field (
		'acquisition_date',
		'cloud_cover',
		'quality',
		'scene_status',
		'rec_station',
		'special_request',
		'starttime', 'scene_center_time', 'endtime',
		'sun_azimuth',
		'sun_elevation',
		'data_type',
		)
		{
		print $scenes{$sceneid}->{$opt_i}->{$field}, $opt_s;
		}
	print $opt_s unless $opt_i eq "ali";	# processing_level is TBD
	foreach my $field qw(
		scene_center_lat
		scene_center_long
		ul_corner_lat
		ul_corner_long
		ur_corner_lat
		ur_corner_long
		ll_corner_lat
		ll_corner_long
		lr_corner_lat
		lr_corner_long
		day_or_night
		)
		{
		print $scenes{$sceneid}->{$opt_i}->{$field}, $opt_s;
		}

	if ($browseurls{$sceneid})
		{
		print "Y$opt_s$browseurls{$sceneid}$opt_s";
		}
	elsif ($browseurls{$short_sceneid})
		{
		print "Y$opt_s$browseurls{$short_sceneid}$opt_s";
		}
	else
		{
		print "N$opt_s$opt_s";
		$missing_browse++;
		}

	# Not sure what to do with scenes on multiple tapes.
	# For now just use the latest one.  Use the same tape as working and archive.
	my @tapeids = sort keys %{$scenes{$sceneid}->{$opt_i}->{tapeid}};
	my $tapeid = pop(@tapeids);
	print "$tapeid$opt_s$tapelocations{$tapeid}$opt_s";
	print "$tapeid$opt_s$tapelocations{$tapeid}$opt_s";

	# TBD: working_level1_id working_level1_location archive_level1_id
	# archive_level1_location
	# Omit for ALI.
	print "$opt_s$opt_s$opt_s$opt_s" unless ($opt_i eq "ali");

	# Raw tape is TBD
	print "$opt_s$opt_s";

	# cal_file_used is TBD
	print $opt_s;

	# File number is constant for each instrument.
	if ($opt_i eq "ali")
		{
		print "2";
		}
	elsif ($opt_i eq "hyp")
		{
		print "3";
		}
	print $opt_s;

	print $scenes{$sceneid}->{$opt_i}->{directory_name}, $opt_s;
	print get_restriction_comments($sceneid), $opt_s;
	printf("%0.4f$opt_s",
		compute_lookangle($sceneid, $scenes{$sceneid}->{$opt_i}->{starttime}, $scenes{$sceneid}->{$opt_i}->{endtime}, $opt_i));
	my $inclination = compute_inclination($sceneid, $scenes{$sceneid}->{$opt_i}->{starttime}, $scenes{$sceneid}->{$opt_i}->{endtime});
	if (defined($inclination))
		{
		printf("%0.2f$opt_s", $inclination);
		}
	else
		{
		print $opt_s;
		}

	# Output the record separator:
	print "\n";
	}
print STDERR "No corners for $missing_corners scenes out of $total_records\n"
	if ($missing_corners > 0);
print STDERR "No browse for $missing_browse scenes out of $total_records\n"
	if ($missing_browse > 0);

# Convert year and day of year to year, month, day of month.
sub yydddtoyymmmdd
	{
	my ($year, $day) = @_;
	my @days = (31,28,31,30,31,30,31,31,30,31,30,31);
	my @monthname = qw(Jan Feb Mar Apr May Jun Jul Aug Sep Oct Nov Dec);
	# Leap year check.
	if ((($year % 4) == 0) && ((($year % 100) != 0) || ($year % 400 == 0)))
		{
		$days[1] = 29;
		}
	my $month;
	for ($month = 0; $day > $days[$month]; $month++)
		{
		$day -= $days[$month];
		}
	return sprintf("%02d-%s-%4d", $day, $monthname[$month], $year);
	}

# Read the corners.txt file and build a hash of corner points by time.
sub readcorners
	{
	my %corners;
	my $time;
	open(CORNERS, "<$_[0]") || die("$_[0]: $!");
	while (<CORNERS>)
		{
		chomp;
		if (m/^\d\d\d\d:\d\d\d:\d\d:\d\d:\d\d/)
			{
			$time = $_;
			}
		elsif (m/(UL|UR|LL|LR):\s*Lat\s*([-0-9\.]+)\s*Lon\s*([-0-9\.]+)/)
			{
			$corners{$time}->{"${1}lat"} = $2;
			$corners{$time}->{"${1}lon"} = $3;
			}
		}
	close(CORNERS);
	return %corners;
	}

# Find and read the 00meta_<sceneid>*.DAT file.
# Returns ALI start time, stop time, and ground station ID
sub finddat
	{
	my ($sceneid) = @_;
	my $target_sceneid = $sceneid;
	my ($starttime, $stoptime, $groundstation, $target_name);
	unless ($sceneid =~ m/^EO1\d\d\d\d\d\d(\d\d\d\d)(\d\d\d)/)
		{
		print STDERR "Bad sceneid $sceneid passwd to finddat\n";
		return undef;
		}
	my ($year, $month) = ($1, $2);
	my $file;
	my $found = undef;
	LookForAFile:
	if (!opendir(DATDIR, "$opt_d/$year/$month/meta"))
		{
		warn("$sceneid: $opt_d/$year/$month/meta: $!");
		print STDERR "No DAT file for $sceneid\n";
		return undef;
		}
	while (defined($file = readdir(DATDIR)))
		{
		if ($file =~ m/^00meta_$target_sceneid.*DAT$/)
			{
			($starttime, $stoptime, $groundstation, $target_name) = readdat("$opt_d/$year/$month/meta/$file");
			if (defined($starttime))
				{
				if (defined($found))
					{
					print STDERR "Multiple DAT files for $sceneid: $found & $file\n";
					return undef;
					}
				$found = $file;
				}
			}
		}
	closedir(DATDIR);
	if (!defined($found))
		{
		# Plan B: if we cant find (for example) 00meta_EO10691102000330010NP_AGS_02.DAT
		# look for 00meta_EO10691102000330010NP_AGS*.DAT or
		# 00meta_EO10691102000330010NP*.DAT.
		if (($target_sceneid =~ s/_\d\d$//) || ($target_sceneid =~ s/_[A-Z0-9][A-Z0-9][A-Z0-9]$//))
			{
			goto LookForAFile;
			}
		print STDERR "No DAT file for $sceneid\n";
		return undef;
		}
	if ($found ne "00meta_$sceneid.DAT")
		{
		print STDERR "No DAT file for $sceneid, using $found\n";
		}

	$starttime =~ s/(\d\d\d\d)(\d\d\d)(\d\d)(\d\d)(\d\d)\.(\d\d\d)/$1:$2:$3:$4:$5.$6/;
	$stoptime =~ s/(\d\d\d\d)(\d\d\d)(\d\d)(\d\d)(\d\d)\.(\d\d\d)/$1:$2:$3:$4:$5.$6/;
	$scenes{$sceneid}->{$opt_i}->{starttime} = $starttime if defined($starttime);
	$scenes{$sceneid}->{$opt_i}->{endtime} = $stoptime if defined($stoptime);
	$scenes{$sceneid}->{$opt_i}->{rec_station} = $groundstation;
	$scenes{$sceneid}->{$opt_i}->{special_request} = $target_name;
	return 1;
	}

# Read a .DAT file and return the starttime, stoptime and groundstation.
sub readdat
	{
	my ($starttime, $stoptime, $groundstation, $target_name) = (undef, undef, undef);
	my $instrument = uc $opt_i;

	open(DAT, "<$_[0]") || die("$_[0]: $!");
	my $longest = 0;	# Longest duration file (which should be the image)
	while (<DAT>)
		{
		if ((m/^$instrument\s+\S+\s+([0-9\.]+)\s+([0-9\.]+)\s+([0-9\.]+)/) && ($3 > $longest))
			{
			$starttime = $1;
			$stoptime = $2;
			$longest = $3;
			}
		elsif (m/^$instrument Start Time\s+(\d\d\d\d)-(\d\d\d) (\d\d):{0,1}(\d\d):{0,1}(\d\d)/)
			{
			$starttime = "$1:$2:$3:$4:$5" unless $longest;
			}
		elsif (m/^$instrument Stop Time\s+(\d\d\d\d)-(\d\d\d)[ :](\d\d):{0,1}(\d\d):{0,1}(\d\d)/)
			{
			$stoptime = "$1:$2:$3:$4:$5" unless $longest;
			}
		elsif (m/^Ground Site ID\s*(\S\S\S)/)
			{
			$groundstation = $1;
			}
		elsif (m/Scene Request File\s*(.*)/)
			{
			$target_name = $1;
			}
		}
	close(DAT);
	return($starttime, $stoptime, $groundstation, $target_name);
	}

# Read an index file and build the list of files and sizes.
sub readindexfile
	{
	my ($toc_file) = @_;
	my @filelist;
	open(TOC, "$toc_file") || die("$toc_file: $!");
	my $currentdir = "meta";
	while (<TOC>)
		{
		chomp;
		next if m/^d/;
		if (m/^[-rwx]+\s+\d+\s+\S+\s+\S+\s+(\d+) [A-Z][a-z][a-z] [ \d]\d \d\d:\d\d (.*)/)
			{
			my $file = "$currentdir/$2";
			my $size = $1;
			# The deflate_hyp file is in the hyp directory, but this is not
			# obvious from the index file (at least in the first two tapes).
			$file = "hyp/deflate_hyp" if ($file =~ m/deflate_hyp$/);
			$filesize{$file} = $size;
			push(@filelist, $file);
			}
		elsif (m/(.*):$/)
			{
			$currentdir = $1;
			}
		}
	close(TOC);
	return @filelist;
	}
# Read a simple text file and turn it into a hash.  The file has lines
# like this:
#	EO1616 dt0667
#	EO1623 dt0668
#	EO1630 dt0669
sub readtapelocations
	{
	my ($filename) = @_;
	my %tapelocations;
	open(TAPEL, "<$filename") || die("$filename: $!");
	while (<TAPEL>)
		{
		chomp;
		if (m/^(EO1...) (......)$/)
			{
			$tapelocations{$1} = $2;
			}
		else
			
			{
			print STDERR "Bad $filename entry ignored: $_\n" if $opt_v;
			}
		}
	return %tapelocations;
	}

# Read a list of browse file names (generated by "find") and return a
# hash, indexed by sceneid, containing the browse URL.
sub generatebrowseurls
	{
	my ($browse_file_list, $instrument) = @_;
	my %browseurls;
	my $browsedir = "/edcsns17/browse1";
	my $urlhead = "http://edcsns17.cr.usgs.gov/browse";
	
	open(BROWSE, "<$browse_file_list") || die("$browse_file_list: $!");
	while (<BROWSE>)
		{
		chomp;
		if (m"$browsedir/(eo-1/$instrument/.*)/([^/]+).jpeg")
			{
			$browseurls{$2} = "$urlhead/$1/$2.jpeg";
			}
		}
	close(BROWSE);
	return %browseurls;
	}
# Search through the metadata directory for 00indexEO1??? files, return the list of tape IDs
sub findindexfiles
	{
	my @tapeids;
	opendir(DIR, "$opt_d/meta") || die("$opt_d/meta: $!");
	while (defined($file = readdir(DIR)))
		{
		next unless ($file =~ m/^00index(EO1...)$/);
		push(@tapeids, $1);
		}
	closedir(DIR);
	return @tapeids;
	}

# Compare two dates, in the form "01-Apr-2002", return -1, 0, 1 if the
# first is (respectively) less than, equal to or greater than the second.
sub cmpdate
	{
	my ($a, $b) = @_;
	my ($a_year, $a_month, $a_day);
	my ($b_year, $b_month, $b_day);
	my %monthno = (qw(Jan 01 Feb 02 Mar 03 Apr 04 May 05 Jun 06 Jul 07 Aug 08 Sep 09 Oct 10 Nov 11 Dec 12));
	if ($a =~ m"(\d+)[-/]([A-Z][a-z][a-z])[/-](\d\d\d\d)")
		{
		$a_year = $3;
		$a_day = $1;
		$a_month = $monthno{$2};
		}
	if ($b =~ m"(\d+)[-/]([A-Z][a-z][a-z])[/-](\d\d\d\d)")
		{
		$b_year = $3;
		$b_day = $1;
		$b_month = $monthno{$2};
		}
	($a_year <=> $b_year) || ($a_month <=> $b_month) || ($a_day <=> $b_day);
	}

# Look for restrictions on this scene.
sub get_restriction_comments
	{
	my ($sceneid) = @_;
	my @restrictions;		# to be returned
	my $rec = $scenes{$sceneid}->{$opt_i};

	push(@restrictions, "Calibration") if (($sceneid =~ m/^EO1000000/) ||
		($rec->{special_request} =~ m/90Yaw/));
	push(@restrictions, "Instrument check-out")
		if (cmpdate($rec->{acquisition_date}, "15-Mar-2001") < 0);
	push(@restrictions, "DOD collect")
		if ($rec->{special_request} =~ m/Warfigher|Paules|W\.A\.A\./);
	push(@restrictions, "PPT test scene")
		if (($rec->{acquisition_date} eq "07-Jan-2002") ||
		    ($rec->{acquisition_date} eq "14-Mar-2002") ||
		    ($rec->{acquisition_date} eq "25-Apr-2002"));
	foreach (keys %exclusions)
		{
		if ($sceneid =~ m/$_/)
			{
			push(@restrictions, $exclusions{$_});
			}
		}
	if ($opt_i eq "ali")
		{
		}
	if ($opt_i eq "hyp")
		{
		push(@restrictions, "Israel")
			if ($sceneid =~ m/^EO1174039/);
		}
	return(join(";", @restrictions));
	}


# Read a file with lines like this:
#	EO10920842001329112KP Frame rate anomaly
# extract the scene ID and point to the exclusion reason.  Returns a hash.
sub readexclusions
	{
	my %ex;
	open(EX, "<$_[0]") || die("$_[0]: $!");
	while (<EX>)
		{
		chomp;
		if (m/(\S+)\s+(.*)/)
			{
			$ex{$1} = $2;
			}
		}
	close(EX);
	return %ex;
	}

sub compute_lookangle
	{
	my ($sceneid, $starttime, $stoptime, $instrument) = @_;
	my $acs_filename;
	# return undef unless ($sceneid =~ m/EO1\d\d\d\d\d\d(\d\d\d\d)(\d\d\d)/);
	unless ($starttime =~ m/^(\d\d\d\d):?(\d\d\d)/)
		{
		print STDERR "$0: Starttime $starttime, no lookangles for $sceneid\n";
		return undef;
		}
	$acs_filename = "$opt_d/$1/$2/meta/$1_$2_acs.hdf";
	unless (-f $acs_filename)
		{
		print STDERR "$0: No file $acs_filename, no lookangles for $sceneid\n";
		return undef;
		}
	$starttime =~ s/://g;
	$stoptime =~ s/://g;
	my $output = `cd lookangle;./lookangleparams $acs_filename $starttime $stoptime | octave -q`;
	unless ($? == 0)
		{
		print STDERR "$0: No lookangles for $sceneid\n";
		return undef;
		}
	my $ali;
	my $hyp;
	if ($output =~ m/ALI_lookangle\s*=\s*([-0-9\.e]+)/)
		{
		$ali = $1;
		}
	if ($output =~ m/HSI_lookangle\s*=\s*([-0-9\.e]+)/)
		{
		$hyp = $1;
		}
	if ($instrument eq "ali")
		{
		return $ali;
		}
	else
		{
		return $hyp;
		}
	}
sub compute_inclination
	{
	my ($sceneid, $starttime, $stoptime) = @_;
	my $acs_filename;
	# return undef unless ($sceneid =~ m/EO1\d\d\d\d\d\d(\d\d\d\d)(\d\d\d)/);
	unless ($starttime =~ m/^(\d\d\d\d):?(\d\d\d)/)
		{
		print STDERR "$0: Starttime $starttime, no inclination for $sceneid\n";
		return undef;
		}
	$acs_filename = "$opt_d/$1/$2/meta/$1_$2_acs.hdf";
	unless (-f $acs_filename)
		{
		print STDERR "$0: No file $acs_filename, no inclination for $sceneid\n";
		return undef;
		}
	$starttime =~ s/://g;
	$stoptime =~ s/://g;
	my $output = `cd lookangle;./inclination $acs_filename $starttime $stoptime`;
	unless ($? == 0)
		{
		print STDERR "$0: No inclination for $sceneid\n";
		return undef;
		}
	if ($output =~ m/([-0-9\.e]+)/)
		{
		return $1;
		}
	}
