#!/bin/csh -f

if ( ${#argv} == 4 ) then
    set dontarchive = 0
else if ( ${#argv} == 5 ) then
    set dontarchive = $5
else if ( ${#argv} != 4 ) then
    echo "Usage: tarlist.csh <file> <file size> <current_tar> <logfile> [-nonew]"
    echo "    Passing the last argument will keep $0 from beginning a new"
    echo "    archive.  This is so that the caller can keep individual groups"
    echo "    of files together."
    exit 2
endif

set file        = $1
set filesize    = $2
set recoverfile = $3
set logfile     = $4

# Find out where the recoverfile resides and change to that directory
#--------------------------------------------------------------------
set curdir = `pwd`
set dir    = `echo $recoverfile:h`
set root   = `echo $recoverfile:r`
if ( "$root" != "$dir" ) then
    set recoverfile = $root
    cd $dir
endif

# Maximum size of a cshell integer variable is 2 Gb so tape
# capacity must be in Kb.
# LTO 4 has 800Gb capacity: 90% of 800 Gb = 754974720 Kb
# LTO 3 has 400Gb capacity: 90% of 400 Gb = 377487360 Kb
# LTO 2 has 200Gb capacity: 90% of 200 Gb = 188743680 Kb
# LTO 1 has 100Gb capacity: 90% of 100 Gb =  94371840 Kb
#----------------------------------------------------------
set tapecapacity = 754974720

# There have been instances of more than one process calling this script at or
# near the same time.  This has caused errors in the recover file.  To prevent
# this from occurring again, check for a lock file, if one is present wait
# until it's gone then lock it before continuing.
#-----------------------------------------------------------------------------
#while ( -e ${recoverfile}.lock )
    #sleep 1
#end
#touch ${recoverfile}.lock

# Set up the tarfile and tarsize
#-------------------------------
if ( ! -e $recoverfile ) then
    echo -n `date +'%D'`"   "`date +'%T'` >> $logfile
    echo -n "   Information     " >> $logfile
    echo "  $recoverfile does not exist, beginning a new one" \
        >> $logfile
    set datestr = `date +%Y%m%d%H%M%S`
    set tarfile = "tarfile_$datestr"
    touch $tarfile
    set tarsize_kb = 0
    set tarsize_b  = 0
    echo "$tarfile-$tarsize_kb-$tarsize_b" > $recoverfile
    echo -n `date +'%D'`"   "`date +'%T'` >> $logfile
    echo -n "   TAR BEGIN       " >> $logfile
    echo "  New tarfile $tarfile started" >> $logfile
else
    set tarfile = `head -1 $recoverfile | awk -F'-' '{print $1}'`
    set tarsize_kb = `head -1 $recoverfile | awk -F'-' '{print $2}'`
    set tarsize_b  = `head -1 $recoverfile | awk -F'-' '{print $3}'`
    if ( ! -e $tarfile ) then
        echo -n `date +'%D'`"   "`date +'%T'` >> $logfile
	echo -n "   ERROR           " >> $logfile
	echo "  Sorry $tarfile does not exist" >> $logfile
	exit 10
    endif
endif

# Add the scene to the tarfile.
#------------------------------
echo $file >> $tarfile

# Add the scene size to tarsize
#------------------------------
# Tape capacity is in Kb since it is greater than 2Gb
@ tarsize_kb += $filesize / 1024
@ tarsize_b  += $filesize % 1024
if ( "$tarsize_b" >= 1024 ) then
    @ tarsize_kb += 1
    @ tarsize_b  -= 1024
endif

# See if it's time to tar.
#-------------------------
set exitstatus = 0
if ( $tarsize_kb > $tapecapacity ) then
    if ( "$dontarchive" == "-nonew" ) then
        set exitstatus = 20
    else
	echo -n `date +'%D'`"   "`date +'%T'` >> $logfile
	echo -n "   TAR COMPLETE    " >> $logfile
	echo "  Create tar with $tarfile" >> $logfile
	echo "$tarfile-$tarsize_kb-$tarsize_b" > $recoverfile
	cp -f $recoverfile $tarfile.cur
	set datestr = `date +%Y%m%d%H%M%S`
	set tarsize_kb = 0
	set tarsize_b  = 0
	set tarfile = "tarfile_$datestr"
	touch $tarfile
	echo -n `date +'%D'`"   "`date +'%T'` >> $logfile
	echo -n "   TAR BEGIN       " >> $logfile
	echo "  New tarfile $tarfile started" >> $logfile
	set exitstatus = 30
    endif
endif

# Save the current tarfile and new tarsize
#-----------------------------------------
cp -f --reply=yes $recoverfile $recoverfile.bak
echo "$tarfile-$tarsize_kb-$tarsize_b" > $recoverfile

# Remove the lock file then sleep for five.  This is an attempt to make the
# testing and creation of the lock file a quasi-atomic operation and hopefully
# eliminate race conditions from occurring.
#-----------------------------------------------------------------------------
#rm ${recoverfile}.lock
#sleep 5

cd $curdir
exit $exitstatus

